/*
 *  Software Switch Configuration for the ADSP-CM408F EZ-KIT
 *
 *  Please see the SoftConfig documentation in CCES Help for detailed
 *  information on SoftConfig.
 *
 */

#include <drivers\twi\adi_twi.h>

/* TWI settings */
#define TWI_PRESCALE  (3u)
#define TWI_BITRATE   (100u)
#define TWI_DUTYCYCLE (50u)

#define BUFFER_SIZE (32u)

/* TWI hDevice handle */
static ADI_TWI_HANDLE hDevice;

/* TWI data buffer */
static uint8_t twiBuffer[BUFFER_SIZE];

/* TWI device memory */
static uint8_t deviceMemory[ADI_TWI_MEMORY_SIZE];

/* switch register structure */
typedef struct {
	uint8_t Register;
	uint8_t Value;
} SWITCH_CONFIG;

/* switch parameter structure */
typedef struct {
	uint32_t TWIDevice;
	uint16_t HardwareAddress;
	uint32_t NumConfigSettings;
	SWITCH_CONFIG *ConfigSettings;
} SOFT_SWITCH;

/* prototype */
void ConfigSoftSwitches(void);

/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_7:"Objects shall be defined at block scope")
#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic")
#endif /* _MISRA_RULES */


/************************SoftConfig Information********************************

    ~ means the signal is active low

    Please see the ADSP-CM408F EZ-KIT manual for more information on using
    Software-Controlled Switches(SoftConfig)

********************************************************************************/

/* switch 0 register settings */
static SWITCH_CONFIG SwitchConfig0[] =
{

 /*
          U40 Port A                                  U40 Port B

  7--------------- ~UART0CTS_RST_EN       |       7---------------  EMPTY
  | 6------------- ~UART0CTS_RTS_LPBK     |       | 6------------- ~LED4_GPIO4_EN
  | | 5----------- ~UART0CTS_EN           |       | | 5----------- ~LED3_GPIO3_EN
  | | | 4--------- ~UART0RX_EN            |       | | | 4--------- ~LED2_GPIO2_EN
  | | | | 3------- ~UART0RTS_EN           |       | | | | 3------- ~LED1_GPIO1_EN
  | | | | | 2----- ~SPI0D3_EN             |       | | | | | 2----- ~PUSHBUTTON2_EN
  | | | | | | 1--- ~SPI0D2_EN             |       | | | | | | 1--- ~PUSHBUTTON1_EN
  | | | | | | | 0- ~SPIFLASH_CS_EN        |       | | | | | | | 0- ~CAN0_EN
  | | | | | | | |                         |       | | | | | | | |
  N N N Y Y Y Y Y                         |       X Y Y Y Y Y Y Y     ( Active Y or N )
  1 1 1 0 0 0 0 0                         |       X 0 0 0 0 0 0 0     ( value being set )
*/
  { 0x12u, 0xE0u },                               { 0x13u, 0x00u },

 /*
  * specify inputs/outputs
  */

  { 0x0u, 0x0u },   /* Set IODIRA direction (all output) */
  { 0x1u, 0x0u },   /* Set IODIRB direction (all output) */
};
/* switch 2 register settings */
static SWITCH_CONFIG SwitchConfig1[] =
{

/*
            U41 Port A                                                  U41 Port B

    7--------------- ~WAKE_PUSHBUTTON_EN  |       7---------------  EMPTY
    | 6-------------  RMII_CLK_EN         |       | 6-------------  EMPTY
    | | 5-----------  PHYINT_EN           |       | | 5-----------  EMPTY
    | | | 4--------- ~ETHERNET_EN         |       | | | 4--------- ~ETH_EXP_EN 
    | | | | 3------- ~ETH0_MDIO_BUS_EN    |       | | | | 3------- ~PB_08_09_EXP_EN 
    | | | | | 2----- ~CAN1_EN             |       | | | | | 2----- ~TRACE_EXP_EN 
    | | | | | | 1---  RS485_DE            |       | | | | | | 1--- ~DATA_HIGH_EXP_EN
    | | | | | | | 0- ~RS485_RE            |       | | | | | | | 0- ~DATA_LOW_EXP_EN
    | | | | | | | |                       |       | | | | | | | |
    N N N N N Y N N                       |       X X X N N N N N     ( Active Y or N )
    1 0 0 1 1 0 0 1                       |       X X X 1 1 1 1 1     ( value being set )
*/
  { 0x12u, 0x99u },                               { 0x13u, 0x1Fu },

  /*
   * specify inputs/outputs
   */

  { 0x0u, 0x0u },    /* Set IODIRA direction (all output) */
  { 0x1u, 0x0u },    /* Set IODIRB direction (all output) */
};

/* switch configuration */
static SOFT_SWITCH SoftSwitch[] =
{
  {
    0u,
    0x20u,
    sizeof(SwitchConfig0)/sizeof(SWITCH_CONFIG),
    SwitchConfig0
  },
  {
    0u,
    0x22u,
    sizeof(SwitchConfig1)/sizeof(SWITCH_CONFIG),
    SwitchConfig1
  }
};
     
#if defined(ADI_DEBUG)
#include <stdio.h>
#define CHECK_RESULT(result, message) \
	do { \
		if((result) != ADI_TWI_SUCCESS) \
		{ \
			printf((message)); \
			printf("\n"); \
		} \
	} while (0)  /* do-while-zero needed for Misra Rule 19.4 */
#else
#define CHECK_RESULT(result, message)
#endif
 
void ConfigSoftSwitches(void)
{
	uint32_t switchNum;
	uint32_t configNum;
	uint32_t settings;
	uint32_t switches;

	SOFT_SWITCH *ss;
	SWITCH_CONFIG *configReg;
	ADI_TWI_RESULT result;

	switches = (uint32_t)(sizeof(SoftSwitch)/sizeof(SOFT_SWITCH));
	for (switchNum=0u; switchNum<switches; switchNum++)
	{
		ss = &SoftSwitch[switchNum];

		result = adi_twi_Open(ss->TWIDevice, ADI_TWI_MASTER, 
    		deviceMemory, ADI_TWI_MEMORY_SIZE, &hDevice);
		CHECK_RESULT(result, "adi_twi_Open failed");

		result = adi_twi_SetHardwareAddress(hDevice, ss->HardwareAddress);
		CHECK_RESULT(result, "adi_twi_SetHardwareAddress failed");

		result = adi_twi_SetPrescale(hDevice, TWI_PRESCALE);
		CHECK_RESULT(result, "adi_twi_Prescale failed");

		result = adi_twi_SetBitRate(hDevice, TWI_BITRATE);
		CHECK_RESULT(result, "adi_twi_SetBitRate failed");

		result = adi_twi_SetDutyCycle(hDevice, TWI_DUTYCYCLE);
		CHECK_RESULT(result, "adi_twi_SetDutyCycle failed");
		
		/* switch register settings */
		for (configNum=0u; configNum<ss->NumConfigSettings; configNum++)
		{
			configReg = &ss->ConfigSettings[configNum];

			/* write register value */
			twiBuffer[0] = configReg->Register;
			twiBuffer[1] = configReg->Value;
			result = adi_twi_Write(hDevice, twiBuffer, (uint32_t)2, false);
			CHECK_RESULT(result, "adi_twi_Write failed");
		}

		result = adi_twi_Close(hDevice);
		CHECK_RESULT(result, "adi_twi_Close failed");
	}
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */
 
