/*!
 *****************************************************************************
 * @file:    system.c
 * @brief:   System startup code for ADSP-CM40z
 * @version: $Revision: 22006 $
 * @date:    $Date: 2013-06-25 14:59:11 -0400 (Tue, 25 Jun 2013) $
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2012 Analog Devices, Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Modified versions of the software must be conspicuously marked as such.
  - This software is licensed solely and exclusively for use with processors
    manufactured by or for Analog Devices, Inc.
  - This software may not be combined or merged with other code in any manner
    that would cause the software to become subject to terms and conditions
    which differ from those listed here.
  - Neither the name of Analog Devices, Inc. nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.
  - The use of this software may or may not infringe the patent rights of one
    or more patent holders.  This license does not release you from the
    requirement that you obtain separate licenses from these patent holders
    to use this software.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF CLAIMS OF INTELLECTUAL
PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *                                                            modified by Keil
 *****************************************************************************/

#include <stdint.h>
#include "startup.h"
#include "system.h"

//extern uint32_t __relocated_vector_table[];
extern uint32_t __vector_table[];

/*----------------------------------------------------------------------------
  External Clock Variables
 *---------------------------------------------------------------------------*/
volatile int _Processor_cycles_per_sec = -1;


/* -- Start added from Keil --- because we use a ASM startup file --- */
#ifndef DO_NOT_RELOCATE_IVT
/*
	A relocated IVT is requested.  Provision for IVT relocation
	to RAM during startup.	This allows for dynamic interrupt
	vector patching required by RTOS.  Places the relocated IVT
	at the start of RAM.  Note: the IVT placement is required
	to be next power-of-two	of the vector table size.  So the
	IVT includes 61	programmable interrupts, 15 system exception
	vectors and	the main stack pointer, therefore we need
	(61 + 15 + 1)*4	= 308 bytes, which rounds up to a 512 (0x200)
	address	boundary (which address 0x20000000 satisfies).

	Also note use of the "__no_init" attibute to force the
	relocated IVT into the ".noinit" section.  This prevents
	the CRTL startup sequence from initializing the relocated
	IVT after we have activated it during the reset vector.
	so that the CRTL does not clear it.
*/
#define RELOCATION_ADDRESS       (0x10000000ul)
#define RELOCATION_ALIGNMENT          (0x200ul)
#define SIZEOF_IVT                      (145ul)

/* reserve no-init aligned IVT space at top of RAM */
uint32_t __relocated_vector_table[SIZEOF_IVT] __attribute__((used, 
                                                             zero_init, 
                                                             at(RELOCATION_ADDRESS), 
                                                             aligned (RELOCATION_ALIGNMENT) ));
#endif //DO_NOT_RELOCATE_IVT


void preSystemInit (void) {
  
// FPU enabling must be done before jump to __main!
#if (__FPU_PRESENT == 1) && (__FPU_USED == 1)
    // the FPU is disabled by default so  enable FPU (NEON and VFP)
    // set the System Control Block, Coprocessor Access Control Register bits:
    // CP10 = grant CP10 coprocessor privlidged and user mode access (full access)
    // CP11 = grant CP11 coprocessor privlidged and user mode access (full access)
    // (CP10 and CP11 MUST be the same or "BEHAIVOR IS UNPREDICTABLE")
	SCB->CPACR |= ((3UL << 10*2)|(3UL << 11*2));  /* set CP10 and CP11 for Full Access */
#endif

// following code is copied from original startup.c
   /*
        Note: the BOOT-ROM IVT is still active at this time, which sets
        the fault pin on any non-maskable exceptions.  After 1) the default
        IVT is copied to SRAM (below) and, 2) the interrupt handlers are
        copied (by the CRTL), we change the Interrupt Vector Table Offset
        Register (VTOR) in main as part of SystemInit().  Until this sequence
        is complete, the BOOT-ROM IVT remains active.
    */


    /* reset the main SP to clean up any leftovers from the boot rom. */
// already done in the Reset_Handler

	/* disable oscillator watchdog false positive anomoly */
    /* (avoid driving the fault light on false positives) */
    __disable_irq();
    pADI_SEC0->FCTL &= ~BITM_SEC_FCTL_EN;  //turn off fault out/in enables
    pADI_SEC0->FCTL &= ~(BITM_SEC_FCTL_FIEN | BITM_SEC_FCTL_FOEN);  //turn off fault out/in enables

    pADI_CGU0->OSCWDCTL &= ~BITM_CGU_OSCWDCTL_FAULTEN;
    __ISB();

    if (((pADI_SEC0->FSTAT & BITM_SEC_FSTAT_ACT) >> BITP_SEC_FSTAT_ACT) &
       ((pADI_SEC0->FEND & BITM_SEC_FSID_FEXT) >> BITP_SEC_FSID_FEXT)) {
       pADI_SEC0->FEND = BITM_SEC_FEND_FEXT;
       }

    pADI_SEC0->FCTL |= (BITM_SEC_FCTL_FOEN | BITM_SEC_FCTL_FIEN);
    pADI_SEC0->FCTL |= BITM_SEC_FCTL_EN;
    __enable_irq();

#ifndef DO_NOT_RELOCATE_IVT
    int i;
	uint32_t *pSrc = (uint32_t*)__vector_table;
  uint32_t *pDst = (uint32_t*)__relocated_vector_table;

    /* copy the IVT (avoid use of memcpy here so it does not become locked into flash) */
    for (i = 0; i < SIZEOF_IVT; i++) {
    	*pDst++ = *pSrc++;
    }
#endif //DO_NOT_RELOCATE_IVT

}
/* -- End   added from Keil ------------------ */


/**
 * Initialize the system
 *
 * @param  none
 * @return none
 *
 * @brief  Setup the microcontroller system.
 *         Initialize the System and update the SystemFrequency variable.
 */
void SystemInit (void)
{
    /* Switch the Interrupt Vector Table Offset Register
     * (VTOR) to point to the relocated IVT in SRAM
     */

    __disable_irq();  // do all this in safe way

    // switch from boot ROM IVT to application's IVT
    // set the System Control Block, Vector Table Offset Register
#ifndef DO_NOT_RELOCATE_IVT
    SCB->VTOR = (uint32_t) &__relocated_vector_table;
#else
    SCB->VTOR = (uint32_t) &__vector_table;
#endif

	  // set all three (USGFAULTENA, BUSFAULTENA, and MEMFAULTENA) fault enable bits
    // in the System Control Block, System Handler Control and State Register
    // otherwise these faults are handled as hard faults
	SCB->SHCSR = SCB_SHCSR_USGFAULTENA_Msk |
	             SCB_SHCSR_BUSFAULTENA_Msk |
	             SCB_SHCSR_MEMFAULTENA_Msk ;

// FPU enabling moved to preSystemInit ()

	// flush instruction and data pipelines to insure assertion of new settings
	__ISB();  // MUST OCCURE IMMEDIATELY AFTER UPDATING SCB->CPACR!!!
	__DSB();
  __enable_irq();

}
