/*
** init_spi.c
**
** Initialize SPI programmatically
**
** Copyright (c) 2011-2012 Analog Devices, Inc. All Rights Reserved.
**/

#include "init_spi.h"

/* __ramfunc */ static void spi_configure(ADI_SPI_TypeDef *pSpiRegs);
/* __ramfunc */ static void spi_regs_copy(ADI_SPI_TypeDef *pSrc, ADI_SPI_TypeDef *pDest);
/* __ramfunc */ static uint8_t get_config_reg(ADI_SPI_TypeDef *pSpiRegs);
/* __ramfunc */ static void set_config_reg(ADI_SPI_TypeDef *pSpiRegs, uint8_t cfgRegVal);
/* __ramfunc */ static void spi_send_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t * pData, uint32_t nNumBytes);
/* __ramfunc */ static void spi_receive_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t *pData, uint32_t count);
/* __ramfunc */ static void spi_enter_xip(ADI_SPI_TypeDef * pSpiRegs);
/* __ramfunc */ static void spi_exit_xip(ADI_SPI_TypeDef * pSpiRegs);


// #pragma location = "hw_init"
// __ramfunc INIT_SPI_RESULT init_spi(void)

#pragma arm section code = "hw_init"
INIT_SPI_RESULT init_spi(void)
{
  ADI_SPI_TypeDef *pSPI = pADI_SPI2;
  ADI_SPI_TypeDef spiBackup;

  pSPI->CTL &= ~BITM_SPI_CTL_EN;         /* Disable the SPI before reconfiguring */
  pSPI->CLK = 0;                         /* Increase SPI speed */

  spi_regs_copy(pSPI, &spiBackup);       /* Backup our SPI registers */
  spi_configure(pSPI);                   /* Reconfigure the spi to core write mode */
  spi_exit_xip(pSPI);                /* Just in case we are in XIP mode ensure the flash comes out of it */
  uint8_t spiCfg = get_config_reg(pSPI); /* Read config register */
  spiCfg &= ~0x08;                       /* Enable XIP functionality */
  set_config_reg(pSPI, spiCfg);          /* Write config reg back */
  spi_enter_xip(pSPI);
  spi_regs_copy(&spiBackup, pSPI);       /* Restore our SPI registers */

  /* In order to maintain XIP mode we must the mode bits to the required value and ensure
     CMDSKIP is enabled. To come out of XIP mode change the value of the mode field
   */
  pSPI->MMRDH = (pSPI->MMRDH & (~BITM_SPI_MMRDH_MODE)) | XIP_CONFIRMATION_BIT | BITM_SPI_MMRDH_CMDSKIP;

  pSPI->CTL |= BITM_SPI_CTL_EN;

  return INIT_SPI_RESULT_SUCCESS;
}

// #pragma location = "hw_init"
// __ramfunc static void spi_regs_copy(ADI_SPI_TypeDef *pSrc, ADI_SPI_TypeDef *pDest)

#pragma arm section code = "hw_init"
static void spi_regs_copy(ADI_SPI_TypeDef *pSrc, ADI_SPI_TypeDef *pDest)
{
  pDest->CTL = pSrc->CTL & ~BITM_SPI_CTL_EN; /* Backup but ensure disabled */
  pDest->RXCTL = pSrc->RXCTL;
  pDest->TXCTL = pSrc->TXCTL;
  pDest->CLK = pSrc->CLK;
  pDest->DLY = pSrc->DLY;
  pDest->DLY = pSrc->SLVSEL;
  pDest->DLY = pSrc->RWC;
  pDest->DLY = pSrc->RWCR;
  pDest->DLY = pSrc->TWC;
  pDest->DLY = pSrc->TWCR;
  pDest->DLY = pSrc->IMSK;
  pDest->MMRDH = pSrc->MMRDH;
  pDest->MMTOP = pSrc->MMTOP;
}

// #pragma location = "hw_init"
// __ramfunc static void spi_configure(ADI_SPI_TypeDef *pSpiRegs)

#pragma arm section code = "hw_init"
static void spi_configure(ADI_SPI_TypeDef *pSpiRegs)
{
  /* need to ensure SPI is configured for 1 bit mode of operation */
  pSpiRegs->CTL = 0;   /* Disable the SPI */
  pSpiRegs->CLK = 0x3; /* Slow the clokk down a bit for the program operations */
  pSpiRegs->RXCTL = 0x0;
  pSpiRegs->TXCTL = 0x0;
  pSpiRegs->TWC = 0;
  pSpiRegs->RWC = 0;

  /* Now set us up for single bit mode of operation */
  pSpiRegs->CTL = 0x3;
}

// #pragma location = "hw_init"
// __ramfunc uint8_t get_config_reg(ADI_SPI_TypeDef *pSpiRegs)

#pragma arm section code = "hw_init"
uint8_t get_config_reg(ADI_SPI_TypeDef *pSpiRegs)
{
  uint32_t nSlvSelHigh = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00));
  uint32_t nSlvSelLow = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << ADI_SPI2_SSEL_ID)));
  uint8_t spiData = 0x85;
  uint8_t cfgRegVal = 0;

  pSpiRegs->SLVSEL = nSlvSelLow; /* set slave select low */
  spi_send_bytes(pSpiRegs, &spiData, 1);
  spi_receive_bytes(pSpiRegs, &cfgRegVal, 1);
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */

  return cfgRegVal;
}

// #pragma location = "hw_init"
// __ramfunc void set_config_reg(ADI_SPI_TypeDef *pSpiRegs, uint8_t cfgRegVal)

#pragma arm section code = "hw_init"
void set_config_reg(ADI_SPI_TypeDef *pSpiRegs, uint8_t cfgRegVal)
{
  uint32_t nSlvSelHigh = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00));
  uint32_t nSlvSelLow = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << ADI_SPI2_SSEL_ID)));
  uint8_t spiData[3];// = {0x06, 0x81, cfgRegVal};

  spiData[0] = 0x06;
  spiData[1] = 0x81;
  spiData[2] = cfgRegVal;

  pSpiRegs->SLVSEL = nSlvSelLow; /* set slave select low */
  spi_send_bytes(pSpiRegs, &spiData[0], 1);
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
  pSpiRegs->SLVSEL = nSlvSelLow; /* set slave select low */
  spi_send_bytes(pSpiRegs, &spiData[1], 2);
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
}

// #pragma location = "hw_init"
// __ramfunc void spi_send_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t * pData, uint32_t nNumBytes)

#pragma arm section code = "hw_init"
void spi_send_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t * pData, uint32_t nNumBytes)
{
  uint32_t i;

  if((nNumBytes != 0))
  {
    uint32_t txctl = pSpiRegs->TXCTL;
    pSpiRegs->RXCTL = 0;

    pSpiRegs->TWC = nNumBytes;
    pSpiRegs->TXCTL = BITM_SPI_TXCTL_TEN | BITM_SPI_TXCTL_TTI | BITM_SPI_TXCTL_TWCEN;

    for(i=0; i<nNumBytes; i++)
    {
      pSpiRegs->TFIFO = (pData[i]);
      while((pSpiRegs->STAT & BITM_SPI_STAT_TFS) == ENUM_SPI_STAT_TFIFO_FULL);  /* Wait until there is space in the FIFO */
    }

    while((pSpiRegs->STAT & BITM_SPI_STAT_TF) == 0);  /* Wait until finished */
    pSpiRegs->STAT = BITM_SPI_STAT_TF;
    pSpiRegs->TXCTL = txctl & (~BITM_SPI_TXCTL_TEN);
  }
}

// #pragma location = "hw_init"
// __ramfunc void spi_receive_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t *pData, uint32_t count)

#pragma arm section code = "hw_init"
void spi_receive_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t *pData, uint32_t count)
{
  pSpiRegs->TXCTL = 0;
  pSpiRegs->STAT = BITM_SPI_STAT_RF;
  pSpiRegs->RWC = count;
  pSpiRegs->RXCTL = BITM_SPI_RXCTL_REN | BITM_SPI_RXCTL_RWCEN | BITM_SPI_RXCTL_RTI;

  for(uint32_t i = 0; i<count; i++)
  {
    while(pSpiRegs->STAT & BITM_SPI_STAT_RFE);  /* Wait until data is received in the receive FIFO */
    pData[i] = pSpiRegs->RFIFO & 0xFF;
  }
  while((pSpiRegs->RWC) != 0);  /* Also make sure the read word counter is now zero */

  pSpiRegs->STAT = BITM_SPI_STAT_RF;
  pSpiRegs->RXCTL = 0;

  pSpiRegs->CTL &= ~0x00000001;
  pSpiRegs->CTL |= 0x00000001;
}

// #pragma location = "hw_init"
// __ramfunc void spi_enter_xip(ADI_SPI_TypeDef * pSpiRegs)

#pragma arm section code = "hw_init"
void spi_enter_xip(ADI_SPI_TypeDef * pSpiRegs)
{
  uint32_t nSlvSelHigh = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00));
  uint32_t nSlvSelLow = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << ADI_SPI2_SSEL_ID)));
  uint8_t spiData[5];

  spiData[0] = 0xEB;
  spiData[1] = 0x00;
  spiData[2] = 0x00;
  spiData[3] = 0x00;
  spiData[4] = 0x00;

  pSpiRegs->SLVSEL = nSlvSelLow; /* set slave select low */
  spi_send_bytes(pSpiRegs, &spiData[0], 5);
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
}

// #pragma location = "hw_init"
// __ramfunc void spi_exit_xip(ADI_SPI_TypeDef * pSpiRegs)

#pragma arm section code = "hw_init"
void spi_exit_xip(ADI_SPI_TypeDef * pSpiRegs)
{
  uint32_t nSlvSelHigh = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00));
  uint32_t nSlvSelLow = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << ADI_SPI2_SSEL_ID)));
  uint8_t spiData[5];

  spiData[0] = 0xEB;
  spiData[1] = 0x00;
  spiData[2] = 0x00;
  spiData[3] = 0xFF;
  spiData[4] = 0xFF;

  pSpiRegs->SLVSEL = nSlvSelLow; /* set slave select low */
  spi_send_bytes(pSpiRegs, &spiData[0], 5);
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
}
