/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*
* Use the GPIO service to Toggle LED's when the push buttons are pressed
* on the CM408F EZ-Kit.
*/

#include <services/gpio/adi_gpio.h>
#include <stdio.h>
#include "system/adi_initialize.h"

#define DEBUG_PRINT (1)  /* 0 = don't use printf, 1 = use printf */

#if (DEBUG_PRINT == 0)
/* printf must be removed when booting from flash without the debugger */
#define printf(...)
#endif

/* used for exit timeout */
#define MAXCOUNT (300000000u)

/* memory for two callbacks */
#define GPIO_MEMORY_SIZE (ADI_GPIO_CALLBACK_MEM_SIZE*2u)

static bool bError;
static volatile uint64_t count;

static void CheckResult(ADI_GPIO_RESULT result, char *message);
extern void ConfigSoftSwitches(void);

/*
 * Report error if result code is not success
 */
static void CheckResult(ADI_GPIO_RESULT result, char *message)
{
  if (result != ADI_GPIO_SUCCESS) {
    printf("%s failed\n", message);
    bError = true;
  }
}

/*
 * GPIO event Callback function
 */
static void pinIntCallback(ADI_GPIO_PIN_INTERRUPT ePinInt, uint32_t PinIntData,  void *pCBParam)
{
  switch(ePinInt)
  {
  case ADI_GPIO_PIN_INTERRUPT_3:
    /* push button 2 */
    if (PinIntData & ADI_GPIO_PIN_8)
    {
      /* toggle LED 2 */
      adi_gpio_Toggle(ADI_GPIO_PORT_E, ADI_GPIO_PIN_6);
    }
    break;
  case ADI_GPIO_PIN_INTERRUPT_4:
    /* push button 1 */
    if (PinIntData & ADI_GPIO_PIN_10)
    {
      /* toggle LED 1 */
      adi_gpio_Toggle(ADI_GPIO_PORT_E, ADI_GPIO_PIN_7);
    }
    break;
  default:
    break;
  }
  
  /* reset the exit counter */
  count = 0u;
}

/*
 * main
 */
int main(void)
{
  uint32_t gpioMaxCallbacks;
  ADI_GPIO_RESULT result;
  static uint8_t gpioMemory[GPIO_MEMORY_SIZE];
  
  adi_initComponents();

  /* Software Switch Configuration for the ADSP-CM408F EZ-Kit */
  ConfigSoftSwitches();
  
  /* init the GPIO service */
  result = adi_gpio_Init(
                         (void*)gpioMemory,
                         GPIO_MEMORY_SIZE,
                         &gpioMaxCallbacks);
  CheckResult(result, "adi_gpio_Init");
  
  /*
  * Setup Push Button 1
  */
  
  /* set GPIO input */
  result = adi_gpio_SetDirection(
                                 ADI_GPIO_PORT_F,
                                 ADI_GPIO_PIN_10,
                                 ADI_GPIO_DIRECTION_INPUT);
  CheckResult(result, "adi_gpio_SetDirection");
  
  /* set edge sense mode (PORT F is connected to Pin Interrupt 4)*/
  result = adi_gpio_SetPinIntEdgeSense(
                                       ADI_GPIO_PIN_INTERRUPT_4,
                                       ADI_GPIO_PIN_10,
                                       ADI_GPIO_SENSE_RISING_EDGE);
  CheckResult(result, "adi_gpio_SetPinIntEdgeSense");
  
  /* register pinint 0 callback */
  result = adi_gpio_RegisterCallback(
                                     ADI_GPIO_PIN_INTERRUPT_4,
                                     ADI_GPIO_PIN_10,
                                     pinIntCallback,
                                     (void*)0);
  CheckResult(result, "adi_gpio_RegisterCallback");
  
  /* set pin interrupt mask */
  result = adi_gpio_EnablePinInterruptMask(
                                           ADI_GPIO_PIN_INTERRUPT_4,
                                           ADI_GPIO_PIN_10,
                                           true);
  CheckResult(result, "adi_gpio_EnablePinInterruptMask");
  
  /*
  * Setup Push Button 2
  */
  
  /* set GPIO input */
  result = adi_gpio_SetDirection(
                                 ADI_GPIO_PORT_E,
                                 ADI_GPIO_PIN_8,
                                 ADI_GPIO_DIRECTION_INPUT);
  CheckResult(result, "adi_gpio_SetDirection");
  
  /* set edge sense mode (PORT E is connected to Pin Interrupt 3)*/
  result = adi_gpio_SetPinIntEdgeSense(
                                       ADI_GPIO_PIN_INTERRUPT_3,
                                       ADI_GPIO_PIN_8,
                                       ADI_GPIO_SENSE_RISING_EDGE);
  CheckResult(result, "adi_gpio_SetPinIntEdgeSense");
  
  /* register pinint 3 callback */
  result = adi_gpio_RegisterCallback(
                                     ADI_GPIO_PIN_INTERRUPT_3,
                                     ADI_GPIO_PIN_8,
                                     pinIntCallback,
                                     (void*)0);
  CheckResult(result, "adi_gpio_RegisterCallback");
  
  /* set pin interrupt mask */
  result = adi_gpio_EnablePinInterruptMask(
                                           ADI_GPIO_PIN_INTERRUPT_3,
                                           ADI_GPIO_PIN_8,
                                           true);
  CheckResult(result, "adi_gpio_EnablePinInterruptMask");
  
  /*
  * Setup LED's
  */
  
  /* set GPIO output LED 1 */
  result = adi_gpio_SetDirection(
                                 ADI_GPIO_PORT_E,
                                 ADI_GPIO_PIN_7,
                                 ADI_GPIO_DIRECTION_OUTPUT);
  CheckResult(result, "adi_gpio_SetDirection");
  
  /* set GPIO output LED 2 */
  result = adi_gpio_SetDirection(
                                 ADI_GPIO_PORT_E,
                                 ADI_GPIO_PIN_6,
                                 ADI_GPIO_DIRECTION_OUTPUT);
  CheckResult(result, "adi_gpio_SetDirection");
  
  count = 0u;
  printf("\nPress PB1 or PB2 to toggle LED's on the CM408F EZ-Kit\n\n");
  
  /* wait for push button interrupts - exit the loop after a while */
  while(count < MAXCOUNT)
  {
    count++;
  }
  
  if (bError) {
    printf("Error configuring the push buttons or LED's\n");
  } else {
    printf("All done\n");
  }
  
  return 0;
}
