/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_adcc.h
*
* @brief     ADCC service API header file.
*
*/

/** @defgroup ADCC_Driver ADC Controller APIs.
 */

/** @addtogroup ADCC_Driver ADC Controller (ADCC) Driver
 *  @{
 */

#ifndef _ADI_ADCC_H_
#define _ADI_ADCC_H_

#include <stdbool.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#endif /* _MISRA_RULES */


/*!
 * ADC Controller handle.
 */
typedef void* ADI_ADCC_HANDLE;

/*!
 * ADC Controller timer handle.
 */
typedef void* ADI_ADCC_TMR_HANDLE;

/*! Size of the memory in bytes, required for the operation of ADCC driver. */
#define ADI_ADCC_MEMORY              12u //TODO: Fix this

/*! Size of the memory in bytes, required for the operation each timer within ADCC. */
#define ADI_ADCC_TMR_MEMORY          104u //TODO: Fix this


/*! The maximum number of events supported by ADCC (0-23). */
#define ADI_ADCC_MAX_EVENTS          24u


/*!
 *  Enumeration of all the result codes returned by ADCC service.
 */
typedef enum
{
    /*! The operation completed successfully */
    ADI_ADCC_SUCCESS,

    /*! The operation failed */
    ADI_ADCC_FAILURE,

    /*! The given ADCC handle is invalid */
    ADI_ADCC_INVALID_HANDLE,

    /*! The given pointer is pointing to NULL. */
    ADI_ADCC_NULL_POINTER,

    /*! The given device instance number is outside
        the number of instances supported by the processor. */
    ADI_ADCC_INVALID_DEVICE_NUM,

    /*! The given timer number is outside
        the number of timer instances supported by the ADCC. */
    ADI_ADCC_INVALID_TMR_NUM,

    /*! The given instance is already opened and in use. */
    ADI_ADCC_DEVICE_IN_USE,

    /*! The given timer already opened and in use. */
    ADI_ADCC_TMR_IN_USE,

    /*! Failed to register the interrupt handler with interrupt service. */
    ADI_ADCC_INT_FAILED,

    /*! Semaphore operation failed */
    ADI_ADCC_SEM_FAILED,

    /*! The given operation is not permitted in the current state of the device. */
    ADI_ADCC_INVALID_OPERATION,

    /*! If any of the given parameters are invalid. */
    ADI_ADCC_INVALID_PARAMETER,

    /*! If the given buffer size is less than two bytes and not multiple of two bytes. */
    ADI_ADCC_INVALID_BUF_SIZE,

    /*! If the given frame increment is outside the supported range (0x8000 to 0x7FFFF)
     *  and not multiples of 2 bytes. */
    ADI_ADCC_INVALID_FRAME_INC,

    /*! If the given number of frames is invalid, should be >= 1. */
    ADI_ADCC_INVALID_NUM_FRAMES,

    /*! If the given event number is greater than #ADI_ADCC_MAX_EVENTS. */
    ADI_ADCC_INVALID_EVENT_NUM,

    /*! If tring to submit new buffer when the previous buffer is being processed
     *  or the processed buffer is not yet retrieved by the application.
     */
    ADI_ADCC_BUFFER_FULL,

    /*! Hardware error(s) is detected. The hardware error can be known by calling
     *  the API adi_adcc_GetHWErrorStatus()
     */
    ADI_ADCC_HW_ERROR_DETECTED

}ADI_ADCC_RESULT;


/*!
 * Enumeration of errors that are generated by the ADC controller
 */
typedef enum
{
    /*! Trigger overrun error. A trigger occurred before completing
     *  the previous frame.
     *
     *  The callback argument will have the timer number for which
     *  the trigger overrrun occurred.
     */
    ADI_ADCC_ERR_TRIGGER_OVERRUN,

    /*! Bandwidth error. The time taken to complete the events in a
     *  frame exceeded the count value set using the API
     *  adi_adcc_ConfigBWMon.
     *
     *  The callback argument will have the timer number for which
     *  the bandwidth error occurred.
     *
     */
    ADI_ADCC_ERR_BANDWIDTH_ERROR,

    /*! Memory error. Memory write error detected due to unaligned
     *  memory access.
     *
     *  The callback argument will have the timer number for which
     *  the memory error occurred.
     */
    ADI_ADCC_ERR_MEMORY_ERROR,

    /*! Event collision occurred. Event collision occurs when a event
     *  cannot be issued to ADC for sampling due to a sampling in
     *  progress for a previous event.
     *
     *  The callback argument will have the bitmask of events that caused
     *  the collision error.
     */
    ADI_ADCC_ERR_EVENT_COLLISION,

    /*! Event miss occurred. The event miss occurs for the following reasons
     *
     * -# If the event FIFO is full.
     * -# If two events of the same timer going to the same ADC are expired
     *    at the same time.
     * -# If "simultaneous sampling" was enabled for an event at one ADC
     *    but there is no corresponding "simultaneous sampling" event programmed
     *    for the other ADC interface from the same timer.
     * -# If one of a simultaneous sampling pair (from a TIMER) is missed at
     *    one ADC due to any event miss reasons above, its counterpart at the
     *    other ADC is also missed.
     *
     *  The callback argument will have the bitmask of events that caused
     *  the event miss error.
     */
    ADI_ADCC_ERR_EVENT_MISS

}ADI_ADCC_ERR;

/*!
 * Enumeration of event IDs for the ADCC events.
 */
typedef enum
{
    ADI_ADCC_EVT_0  =   BITM_ADCC_EIMSK_EVT00,      /*!< Event ID 0.  */
    ADI_ADCC_EVT_1  =   BITM_ADCC_EIMSK_EVT01,      /*!< Event ID 1.  */
    ADI_ADCC_EVT_2  =   BITM_ADCC_EIMSK_EVT02,      /*!< Event ID 2.  */
    ADI_ADCC_EVT_3  =   BITM_ADCC_EIMSK_EVT03,      /*!< Event ID 3.  */
    ADI_ADCC_EVT_4  =   BITM_ADCC_EIMSK_EVT04,      /*!< Event ID 4.  */
    ADI_ADCC_EVT_5  =   BITM_ADCC_EIMSK_EVT05,      /*!< Event ID 5.  */
    ADI_ADCC_EVT_6  =   BITM_ADCC_EIMSK_EVT06,      /*!< Event ID 6.  */
    ADI_ADCC_EVT_7  =   BITM_ADCC_EIMSK_EVT07,      /*!< Event ID 7.  */
    ADI_ADCC_EVT_8  =   BITM_ADCC_EIMSK_EVT08,      /*!< Event ID 8.  */
    ADI_ADCC_EVT_9  =   BITM_ADCC_EIMSK_EVT09,      /*!< Event ID 9.  */
    ADI_ADCC_EVT_10 =   BITM_ADCC_EIMSK_EVT10,      /*!< Event ID 10. */
    ADI_ADCC_EVT_11 =   BITM_ADCC_EIMSK_EVT11,      /*!< Event ID 11. */
    ADI_ADCC_EVT_12 =   BITM_ADCC_EIMSK_EVT12,      /*!< Event ID 12. */
    ADI_ADCC_EVT_13 =   BITM_ADCC_EIMSK_EVT13,      /*!< Event ID 13. */
    ADI_ADCC_EVT_14 =   BITM_ADCC_EIMSK_EVT14,      /*!< Event ID 14. */
    ADI_ADCC_EVT_15 =   BITM_ADCC_EIMSK_EVT15,      /*!< Event ID 15. */
    ADI_ADCC_EVT_16 =   BITM_ADCC_EIMSK_EVT16,      /*!< Event ID 16. */
    ADI_ADCC_EVT_17 =   BITM_ADCC_EIMSK_EVT17,      /*!< Event ID 17. */
    ADI_ADCC_EVT_18 =   BITM_ADCC_EIMSK_EVT18,      /*!< Event ID 18. */
    ADI_ADCC_EVT_19 =   BITM_ADCC_EIMSK_EVT19,      /*!< Event ID 19. */
    ADI_ADCC_EVT_20 =   BITM_ADCC_EIMSK_EVT20,      /*!< Event ID 20. */
    ADI_ADCC_EVT_21 =   BITM_ADCC_EIMSK_EVT21,      /*!< Event ID 21. */
    ADI_ADCC_EVT_22 =   BITM_ADCC_EIMSK_EVT22,      /*!< Event ID 22. */
    ADI_ADCC_EVT_23 =   BITM_ADCC_EIMSK_EVT23       /*!< Event ID 23. */

} ADI_ADCC_EVT_ID;


/*!
 * Enumeration of status events returned by the ADCC driver.
 *
 * The events are passed back to the application via the optional registered callback.
 */
typedef enum
{
    /*! one or more events are completed. The completed events are indicated by callback argument.
     *  The callback argument can be logical or of the event Ids defined in the enumeration
     *  #ADI_ADCC_EVT_ID. */
    ADI_ADCC_EVENT_COMPLETED,

    /*! All the events related to ADCC Timer are completed. The argument contains the pointer
     * to the frame data. A frame is set of events for timer. */
    ADI_ADCC_EVENT_FRAME_PROCESSED,

    /*! Application supplied buffer is processed and filled with ADC data. A buffer is a collection
     *  one or more frames. */
    ADI_ADCC_EVENT_BUFFER_PROCESSED

} ADI_ADCC_EVENT;

/*!
 *  Enumeration of timers in ADCC.
 *
 *  ADCC has two timers for generation of events based on the
 *  configured time-offsets from the start of external trigger.
 */
typedef enum
{
    /*! ADCC Timer0. */
    ADI_ADCC_TIMER0,

    /*! ADCC Timer1. */
    ADI_ADCC_TIMER1,

    /*! ADCC Maximum number of timers */
    ADI_ADCC_MAX_TIMERS

} ADI_ADCC_TIMER;

/*!
 *  Enumeration of ADC Interfaces in ADCC.
 */
typedef enum
{
    /*! ADC interface 0. */
    ADI_ADCC_ADCIF0,

    /*! ADC interface 1. */
    ADI_ADCC_ADCIF1,

    /*! Maximum number of ADC Interfaces. */
    ADI_ADCC_ADCIF_MAX

} ADI_ADCC_ADCIF;

/*!
 * Enumeration of different external triggers.
 *
 * Even though ADCC timer is enabled it will not start counting
 * unless an external trigger is received. The external trigger
 * can be configured (using API #adi_adcc_ConfigTimer) to any
 * one of the enumerated values.
 *
 * For the processors which has TRU, application should configure the TRU
 * to route the master trigger to the chosen timer trigger slave.
 */
typedef enum
{

    /*! Trigger 0. */
    ADI_ADCC_TRIG0,

    /*! Trigger 1. */
    ADI_ADCC_TRIG1,

    /*! Trigger 2. */
    ADI_ADCC_TRIG2,

    /*! Trigger 3. */
    ADI_ADCC_TRIG3,

    /*! Trigger 4. */
    ADI_ADCC_TRIG4

} ADI_ADCC_TRIG;


/*!
 *  Structure to hold the event configuration related parameters. This
 *  is used with the API #adi_adcc_ConfigEvent.
 */
typedef struct
{
   /*!
    *   Event number to be configured. This can be any number from
    *   0 to (ADI_ADCC_MAX_EVENTS - 1)
    */
   uint8_t          nEvent;

   /*!
    *   For every event a control word can be sent to the ADC before sampling.
    *   The Control word should be programmed LSB aligned.
    */
    uint16_t        AdcCtlWord;

   /*!
    *   ADC interface on which to execute the event.
    */
   ADI_ADCC_ADCIF   eAdcIF;

   /*!
    *   Timer to which the event to be assigned.
    */
   ADI_ADCC_TIMER   eTimer;

   /*!
    *  Flag which indicates whether to enable simultaneous sampling.
    *  \n
    *   - 'true'  -   Enable simultaneous sampling.
    *   - 'false' -   Disable simultaneous sampling.
    *
    *   When simultaneous sampling is enabled there should be another event programmed
    *   with identical time and assigned to the same timer with a different ADC interface to
    *   execute on.
    */
   bool             bSimSamp;

   /*!
    *   Memory offset in terms of bytes from the base pointer where the ADC data for this event to be written. 
    *   The memory offset should be given in multiples of two bytes. This is not relevant if DMA mode is not used. 
    *   The base pointer is buffer pointer supplied using the APIs #adi_adcc_SubmitBuffer or  
    *   #adi_adcc_SubmitCircBuffer.
    */
   uint16_t         nMemOffset;

   /*!
    *   Time in terms of system clock cycles at which to sample the ADC.
    */
   uint32_t         nEventTime;

} ADI_ADCC_EVENT_CFG;


/*!
 *  Structure which will be used by application to provide the information
 *  regarding which event data to be read. This also contains the place
 *  holder for the event data to be written by the driver.
 */
typedef struct
{
    /*! Event number whose data to be read. It can be any value from 0 to
     *  #ADI_ADCC_MAX_EVENTS */
    uint32_t    nEvent;

    /*!
     *  Place holder for the event data to be written by the driver
     */
    uint32_t    EventData;

} ADI_ADCC_EVT_DATA;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 *Opens ADC Controller Device instance.
 */
ADI_ADCC_RESULT  adi_adcc_OpenDevice (
    uint32_t                    nDeviceNum,
    void const                 *pMemory,
    ADI_ADCC_HANDLE            *phDevice
);

/*
 * Closes ADC Controller Device.
 */
ADI_ADCC_RESULT  adi_adcc_CloseDevice (
   ADI_ADCC_HANDLE const        hDevice
);


/*
 * Opens Timer within the ADC Controller.
 */
ADI_ADCC_RESULT  adi_adcc_OpenTimer (
    ADI_ADCC_HANDLE             hDevice,
    ADI_ADCC_TIMER              eTimer,
    void const                 *pMemory,
    ADI_ADCC_TMR_HANDLE        *phTimer
);

/*
 * Closes Timer within the ADC Controller.
 */
ADI_ADCC_RESULT  adi_adcc_CloseTimer (
    ADI_ADCC_TMR_HANDLE         hTimer
);


/*
 * Register/Unregister an optional callback for ADC Controller.
 */
ADI_ADCC_RESULT  adi_adcc_RegisterDevCallback (
    ADI_ADCC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
);

/*
 * Register or Unregister optional callback for ADCC Timer.
 */
ADI_ADCC_RESULT  adi_adcc_RegisterTmrCallback (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
);

/*
 * Submit the buffer for filling the ADC data.
 */
ADI_ADCC_RESULT  adi_adcc_SubmitBuffer (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    void * const                pBuffer,
    int32_t                     nFrameInc,
    uint32_t                    nNumFrames
);

/*
 * Submit the circular buffer for getting the ADC data.
 */
ADI_ADCC_RESULT  adi_adcc_SubmitCircBuffer (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    void * const                pBuffer,
    int32_t                     nFrameInc,
    uint32_t                    nBufSize
);

/*
 * Gets back the filled buffer.
 */
ADI_ADCC_RESULT  adi_adcc_GetBuffer (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    void ** const               ppBuffer
);

/*
 * Queries if filled buffer is available.
 */
ADI_ADCC_RESULT  adi_adcc_IsBufferAvailable (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    bool *const                 pbAvailable
);

/*
 * Gets the current write pointer.
 */
ADI_ADCC_RESULT  adi_adcc_GetCurrWrPtr (
    ADI_ADCC_TMR_HANDLE         hTimer,
    void **const                ppWrPtr
);

/*
 * Gets the data for the given events.
 */
ADI_ADCC_RESULT  adi_adcc_GetEventData(
    ADI_ADCC_HANDLE             hDevice,
    ADI_ADCC_EVT_DATA *const    pEventData,
    uint32_t                    nEvents
);

/*
 * Enables or disables the ADC Controller.
 */
ADI_ADCC_RESULT  adi_adcc_EnableDevice(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bEnable
);

/*
 * Enables or disables the ADCC Timer.
 */
ADI_ADCC_RESULT  adi_adcc_EnableTimer(
    ADI_ADCC_TMR_HANDLE const   hTimer,
    bool                        bEnable
);

/*
 * Configure ADCC timer related parameters.
 */
ADI_ADCC_RESULT  adi_adcc_ConfigTimer (
    ADI_ADCC_TMR_HANDLE         hTimer,
    ADI_ADCC_TRIG               eTrig,
    bool                        bRiseEdgeTrig,
    bool                        bEnableTrigOut
);

/*
 * Configure Chip Select (CS) signal.
 */
ADI_ADCC_RESULT  adi_adcc_ConfigChipSelect(
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bActiveHigh,
    uint8_t                     tCSCK,
    uint8_t                     tCKCS,
    uint8_t                     tCSCS
);

/*
 * Configure ADCC Clock (ACK).
 */
ADI_ADCC_RESULT  adi_adcc_ConfigADCCClock (
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bRisingEdge,
    uint16_t                    nClkDiv,
    uint8_t                     nNumACK
);

/*
 * Configure data interface.
 */
ADI_ADCC_RESULT  adi_adcc_ConfigDataIF(
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bSwap,
    bool                        bLSBF
);


/*
 * @brief       Configure control interface to the ADC.
 */
ADI_ADCC_RESULT  adi_adcc_ConfigCtrlIF(
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bIdleHigh
);

/*
 * Set the ADC configuration word.
 */
ADI_ADCC_RESULT  adi_adcc_ConfigADC (
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bExtRef
);

/*
 * Enable or disable dual bit data interface.
 */
ADI_ADCC_RESULT  adi_adcc_EnableDualBitDataIF(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bDualBit
);

/*
 * Enable Dual bit control interface.
 */
ADI_ADCC_RESULT  adi_adcc_EnableDualBitCtrlIF(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bDualBit
);

/*
 * Enable or disable DMA mode.
 */
ADI_ADCC_RESULT  adi_adcc_EnableDMAMode(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bEnable
);

/*
 * Configure the event.
 */
ADI_ADCC_RESULT  adi_adcc_ConfigEvent(
    ADI_ADCC_HANDLE             hDevice,
    ADI_ADCC_EVENT_CFG *        pEventCfg,
    uint32_t                    nNumEvents
);

/*
 * Sets event mask.
 */
ADI_ADCC_RESULT  adi_adcc_SetEventMask (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nEventMask
);

/*
 * Sets the event interrupt mask.
 */
ADI_ADCC_RESULT  adi_adcc_SetEventIntMask (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nEventIntMask
);

/*
 * Configure Bandwidth monitoring .
 */
ADI_ADCC_RESULT  adi_adcc_ConfigBWMon(
    ADI_ADCC_TMR_HANDLE         hTimer,
    uint32_t                    nCount
);

/*
 * Sets a given ADCC register.
 */
ADI_ADCC_RESULT  adi_adcc_SetRegister (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t                    nRegValue
);

/*
 * Gets the contents of given ADCC register.
 */
ADI_ADCC_RESULT  adi_adcc_GetRegister (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t * const            pRegValue
);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_ADCC_H_ */

/*@}*/
