/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_crc.h
 * @brief:   Cyclic Redundancy Check (CRC) peripheral definitions and API
 * @version: $Revision: 18384 $
 * @date:    $Date: 2013-04-04 17:17:34 -0400 (Thu, 04 Apr 2013) $
 *
 * @details
 *           This is the primary header file for the Cyclic Redundancy Check (CRC)
 *           peripheral driver, which contains the API declarations,
 *           data and constant definitions used in the APIs
 */

/** @addtogroup CRC_Driver Cyclic Redundancy Check (CRC) peripheral driver
 *  @{
 */

#ifndef __ADI_CRC_H__
#define __ADI_CRC_H__

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/

/* OSAL includes */
#include <adi_osal.h>
/* DMA Manager includes */
#include <services/int/adi_int.h>
/* DMA Manager includes */
#include <services/dma/adi_dma.h>

/*==============  D E F I N E S  ===============*/

#if !defined(_LANGUAGE_ASM)

/** The amount of application supplied memory required to operate a core driven CRC device */
#define ADI_CRC_CORE_MEMORY_SIZE        (52u)
/** The amount of application supplied memory required to operate a DMA driven CRC device */
#define ADI_CRC_DMA_MEMORY_SIZE         (ADI_CRC_CORE_MEMORY_SIZE + ADI_DMA_STREAM_REQ_MEMORY)

/** A device handle used in all API functions to identify the CRC device */
typedef void *ADI_CRC_HANDLE;

/**
 * CRC driver operating modes.
 */
typedef enum
{
    ADI_CRC_MODE_NONE                   = (0u),     /*!< CRC Operation disabled */
    ADI_CRC_MODE_XFER_COMPUTE_COMPARE   = (0x10u),  /*!< DMA/Core Driven Memory Transfer Compute Compare Mode,
                                                         uses CRC input and output channels */
    ADI_CRC_MODE_FILL                   = (0x20u),  /*!< DMA Driven Memory Transfer Data Fill Mode
                                                         uses only CRC output channel */
    ADI_CRC_MODE_SCAN_COMPUTE_COMPARE   = (0x30u),  /*!< DMA/Core Driven Memory Scan Compute Compare Mode
                                                         uses only CRC input channel */
    ADI_CRC_MODE_SCAN_VERIFY            = (0x40u)   /*!< DMA/Core Driven Memory Scan Data Verify Mode
                                                         uses only CRC input channel */
} ADI_CRC_MODE;

/**
 * CRC data mirroring modes.
 */
typedef enum
{
    ADI_CRC_MIRROR_NONE          = (0u),        /*!< No Data mirroring (Default)    - Dout[31:0] = Din[31:0] */
    ADI_CRC_MIRROR_BIT           = (0x10000u),  /*!< Enable Bit Mirror              - Dout[31:0] = Din[24:31],Din[16:23],Din[8:15],Din[0:7]*/
    ADI_CRC_MIRROR_BYTE          = (0x20000u),  /*!< Enable Byte Mirror             - Dout[31:0] = Din[7:0],Din[15:8],Din[23:16],Din[31:24]*/
    ADI_CRC_MIRROR_WORD          = (0x40000u),  /*!< Enable Word Mirror             - Dout[31:0] = Din[15:0], D[31:16] */
    ADI_CRC_MIRROR_BIT_BYTE      = (0x30000u),  /*!< Enable Bit & Byte Mirror       - Dout[31:0] = Din[0:7],Din[8:15],Din[16:23],Din[24:31]*/
    ADI_CRC_MIRROR_BIT_WORD      = (0x50000u),  /*!< Enable Bit & Word Mirror       - Dout[31:0] = Din[8:15],Din[0:7], Din[24:31],Din[16:23]*/
    ADI_CRC_MIRROR_BYTE_WORD     = (0x60000u),  /*!< Enable Byte & Word Mirror      - Dout[31:0] = Din[23:16],Din[31:24], Din[7:0],Din[15:8]*/
    ADI_CRC_MIRROR_BIT_BYTE_WORD = (0x70000u)   /*!< Enable Bit, Byte & Word Mirror - Dout[31:0] = Din[16:23],Din[24:31], Din[0:7],Din[8:15]*/
} ADI_CRC_MIRROR;

/**
 * Enumeration of conditions on which the CRC device can issue an outgoing Trigger
 */
typedef enum
{
    ADI_CRC_TRIGGER_OUT_NONE        = ADI_DMA_TRIGGER_OUT_NONE,         /*!< No trigger (default) */
    ADI_CRC_TRIGGER_OUT_DMA_DONE    = ADI_DMA_TRIGGER_OUT_XCNT_EXPIRY   /*!< Generate trigger when DMA is done. */
} ADI_CRC_TRIGGER_OUT;

/**
 * CRC events
 */
enum
{
    ADI_CRC_EVENT_BUFFER_PROCESSED,     /*!< CRC peripheral has completed processing a buffer
                                             (pArg = Processed buffer address) */
    ADI_CRC_EVENT_DATA_COUNT_EXPIRED,   /*!< CRC data count expired (end of CRC operation)
                                             (pArg = NULL) */
    ADI_CRC_EVENT_COMPARE_ERROR,        /*!< CRC data comparison failed
                                             (pArg = Position of error in the data stream) */
    ADI_CRC_EVENT_DMA_ERROR             /*!< Error identified by CRC Memory DMA stream
                                             (pArg = Error cause field value of DMA status register) */
};

/**
 *  CRC driver return codes
 */
typedef enum
{
    ADI_CRC_SUCCESS = 0,                /*!< 0x00 - Generic success */
    ADI_CRC_FAILURE,                    /*!< 0x01 - Generic failure */
    ADI_CRC_IN_USE,                     /*!< 0x02 - Supplied CRC device number is already open and in use */
    ADI_CRC_NOT_OPEN,                   /*!< 0x03 - CRC device is not open to preform the requested task */
    ADI_CRC_INSUFFICIENT_MEMORY,        /*!< 0x04 - Supplied memory is insufficient to operate the CRC device */
    ADI_CRC_FN_NOT_SUPPORTED,           /*!< 0x05 - Function not supported */
    ADI_CRC_INVALID_FN_CALL,            /*!< 0x05 - Invalid function call for the current CRC mode */
    ADI_CRC_FN_NOT_PERMITTED,           /*!< 0x06 - Function not permitted at current stage */
    ADI_CRC_BAD_HANDLE,                 /*!< 0x07 - Bad CRC device handle */
    ADI_CRC_BAD_DEVICE_NUMBER,          /*!< 0x08 - Bad CRC device number */
    ADI_CRC_INT_REGISTER_FAILED,        /*!< 0x09 - Can't register an interrupt */
    ADI_CRC_INT_UNREGISTER_FAILED,      /*!< 0x0A - Can't unregister an interrupt */
	ADI_CRC_COMPARE_ERROR,              /*!< 0x0B - CRC data comparison failed */
	ADI_CRC_DMA_NOT_OPEN,               /*!< 0x0C - Memory DMA Stream used for CRC operation is not open */
	ADI_CRC_DMA_ERROR,                  /*!< 0x0D - Error with CRC DMA */
	ADI_CRC_DATA_COUNT_INVALID,         /*!< 0x0E - CRC Data count register value is invalid */
	ADI_CRC_CORE_WRITE_NOT_READY,       /*!< 0x0F - CRC FIFO (Input buffer) is not ready to accept data */
	ADI_CRC_CORE_READ_NOT_READY         /*!< 0x10 - CRC FIFO can't be read as the output buffer is empty */
} ADI_CRC_RESULT;

/*=======  P U B L I C   P R O T O T Y P E S  ========*/
/*            (globally-scoped functions)             */

#ifdef __cplusplus
extern "C" {
#endif

/* Opens a CRC device instance */
ADI_CRC_RESULT adi_crc_Open(
	uint32_t                DeviceNum,
	void                    *pMemory,
	uint32_t                MemorySize,
	ADI_CRC_HANDLE          *phDevice);

/* Closes a CRC device instance */
ADI_CRC_RESULT adi_crc_Close(
    ADI_CRC_HANDLE const    hDevice);

/* Stops current CRC in progress and/or resets CRC peripheral */
ADI_CRC_RESULT adi_crc_StopReset(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bStopCrc,
    bool                    bResetCrc);

/* Sets mirroring configuration for CRC registers */
ADI_CRC_RESULT adi_crc_SetRegMirror(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bMirrCompare,
    bool                    bMirrPolynomial,
    bool                    bMirrResult);

/* Sets mirroring configuration for CRC data stream */
ADI_CRC_RESULT adi_crc_SetDataMirror(
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_MIRROR          eDataMirror);

/* Sets CRC auto-clear mode */
ADI_CRC_RESULT adi_crc_SetAutoClear(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bEnable,
    bool                    bAutoClearZero);

/* Enables/Disables DMA driven CRC operation */
ADI_CRC_RESULT adi_crc_EnableDmaMode(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bEnable);

/* Sets CRC operating mode */
ADI_CRC_RESULT adi_crc_SetOperatingMode(
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_MODE            eCrcMode);

/* Sets the 32-bit polynomial for CRC operations */
ADI_CRC_RESULT adi_crc_SetPolynomialVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                PolynomialVal);

/* Sets the data count for the CRC operation that is about to take place */
ADI_CRC_RESULT adi_crc_SetDataCount(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                DataCount,
    uint32_t                ReloadDataCount);

/* Sets the initial seed value for the CRC operation that is about to take place */
ADI_CRC_RESULT adi_crc_SetCrcSeedVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                CrcSeedVal);

/*
 * Sets the expected CRC32 value of a data stream that is about to be processed (OR)
 * Sets the 32-bit value that is to be used to validate a memory region.
 */
ADI_CRC_RESULT adi_crc_SetExpectedVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                ExpectedVal);

/* Sets the 32-bit value that is to be used to fill a memory region */
ADI_CRC_RESULT adi_crc_SetMemFillVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                MemFillVal);

/* Submits data buffer for DMA driven Data Fill (OR) Scan-Verify (OR) Scan-Compute-Compare  CRC opeartion */
ADI_CRC_RESULT adi_crc_DmaMemFillScan(
    ADI_CRC_HANDLE const    hDevice,
    void                    *pCrcBuf,
    uint32_t                Num32bitElements);

/*
 * Submits source and destination data buffers for DMA driven
 * Memory Transfer Compute and Compare CRC opeartion.
 */
ADI_CRC_RESULT adi_crc_DmaMemXfer(
    ADI_CRC_HANDLE const    hDevice,
    void                    *pSrcBuf,
    void                    *pDestBuf,
    uint32_t                Num32bitElements);

/*
 * Writes a 32-bit data to CRC FIFO for Core driven CRC operation.
 * Supported ONLY when CRC is operated in core access mode.
 */
ADI_CRC_RESULT adi_crc_CoreWrite(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                DataToCrcFifo);

/*
 * Reads a 32-bit data from CRC FIFO for Core driven CRC operation.
 * Supported ONLY when CRC is operated in core access mode.
 */
ADI_CRC_RESULT adi_crc_CoreRead(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pDataFromCrcFifo);

/*
 * Peek function to check if the CRC peripheral FIFO is ready for Core Write access.
 * Supported ONLY when CRC is operated in core access mode.
 */
ADI_CRC_RESULT adi_crc_IsCoreWriteReady (
    ADI_CRC_HANDLE const    hDevice,
    bool                    *pbIsCoreWriteReady);

/*
 * Peek function to check if the CRC peripheral FIFO is ready for Core Read access.
 * Supported ONLY when CRC is operated in core access mode.
 */
ADI_CRC_RESULT adi_crc_IsCoreReadReady (
    ADI_CRC_HANDLE const    hDevice,
    bool                    *pbIsCoreReadReady);

/* Gets the current CRC peripheral status */
ADI_CRC_RESULT adi_crc_IsCrcInProgress(
    ADI_CRC_HANDLE const    hDevice,
    bool                    *pbCrcInProgress);

/* Gets the final CRC result computed for a data stream */
ADI_CRC_RESULT adi_crc_GetFinalCrcVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pFinalCrcVal);

/* Gets the current/intermediate CRC result computed for a data stream */
ADI_CRC_RESULT adi_crc_GetCurrentCrcVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pCurrentCrcVal);

/* Gets the position of error when a compare operation fails in data verify mode */
ADI_CRC_RESULT adi_crc_GetCompareFailPos(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pFailPos);

/* Registers or unregisters a callback with the CRC device */
ADI_CRC_RESULT adi_crc_RegisterCallback(
    ADI_CRC_HANDLE const    hDevice,
    ADI_CALLBACK            pfCallback,
    void *const             pCBParam);

/* Configures Trigger input settings for DMA driven CRC input channel */
ADI_CRC_RESULT  adi_crc_ConfigCrcInTriggerIn (
    ADI_CRC_HANDLE const    hDevice,
    bool                    bTriggerWait,
    bool                    bTriggerOvError);

/* Configures Trigger input settings for DMA driven CRC output channel */
ADI_CRC_RESULT  adi_crc_ConfigCrcOutTriggerIn (
    ADI_CRC_HANDLE const    hDevice,
    bool                    bTriggerWait,
    bool                    bTriggerOvError);

/* Configures Trigger output settings for DMA driven CRC input channel */
ADI_CRC_RESULT  adi_crc_ConfigCrcInTriggerOut (
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_TRIGGER_OUT     eCrcTrigger);

/* Configures Trigger output settings for DMA driven CRC output channel */
ADI_CRC_RESULT  adi_crc_ConfigCrcOutTriggerOut (
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_TRIGGER_OUT     eCrcTrigger);

#ifdef __cplusplus
}
#endif

#endif /*!defined(_LANGUAGE_ASM) */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif  /* __ADI_CRC_H__ */

/*****/

/*@}*/

