/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_dacc.h
*
* @brief     DACC service API header file.
*
* @details
*            This is the primary header file DAC Controller (DACC) driver.
*/

/** @defgroup DACC_Driver DAC Controller APIs.
 */

/** @addtogroup DACC_Driver DAC Controller (DACC) Driver
 *  @{
 */

#ifndef _ADI_DACC_H_
#define _ADI_DACC_H_

#include <stdbool.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#endif /* _MISRA_RULES */

/*!
 * DAC Controller handle.
 */
typedef void* ADI_DACC_HANDLE;

/*! Size of the memory in bytes, required for the operation of DACC driver. */
#define ADI_DACC_MEMORY              116u 

/*!
 *  Enumeration of all the result codes returned by DACC service.
 */
typedef enum
{
    /*! The operation completed successfully */
    ADI_DACC_SUCCESS,

    /*! The operation failed */
    ADI_DACC_FAILURE,

    /*! The given DACC handle is invalid */
    ADI_DACC_INVALID_HANDLE,

    /*! The given pointer is pointing to NULL. */
    ADI_DACC_NULL_POINTER,

    /*! The given device instance number is outside
        the number of instances supported by the processor. */
    ADI_DACC_INVALID_DEVICE_NUM,

    /*! The given instance is already opened and in use. */
    ADI_DACC_DEVICE_IN_USE,

    /*! Failed to register the interrupt handler with interrupt service. */
    ADI_DACC_INT_FAILED,

    /*! The given operation is not permitted in the current state of the device. */
    ADI_DACC_INVALID_OPERATION,

    /*! If any of the given parameters are invalid */
    ADI_DACC_INVALID_PARAMETER,

    /*! Semaphore creation failed */
    ADI_DACC_SEM_FAILED,

    /*! DAC operation timeout */
    ADI_DACC_TIMEOUT,
    /*! Hardware error(s) is detected. The hardware error can be known by calling
     *  the API adi_dacc_GetHWErrorStatus()
     */
    ADI_DACC_HW_ERROR_DETECTED

}ADI_DACC_RESULT;


/*!
 * Enumeration of errors that are generated by the DAC controller,
 *
 * The errors are passed back to the application via the optional
 * registered callback. If the callback is not registered application
 * can get the pending errors using the #adi_dacc_GetHWErrorStatus()
 * API.
 *
 */
typedef enum
{
    ADI_DACC_ERR_NONE,
    ADI_DACC_ERR_UNDERFLOW,
    ADI_DACC_ERR_ADDR_ALIGN,
    ADI_DACC_ERR_MEMORY,
    ADI_DACC_ERR_MAX

}ADI_DACC_ERR;

/*!
 * Enumeration of status events returned by the DACC driver.
 *
 * The events are passed back to the application via the optional registered callback.
 */
typedef enum
{
    /*! The given buffer is processed. The argument contains the buffer address */
    ADI_DACC_EVENT_BUFFER_PROCESSED,
    ADI_DACC_EVENT_HWERR

} ADI_DACC_EVENT;



/*!
 *  Enumeration of DAC Interfaces in DACC.
 */
typedef enum
{
    /*! DAC interface 0. */
    ADI_DACC_DACIF0,

    /*! DAC interface 1. */
    ADI_DACC_DACIF1,

    /*! Maximum number of DAC Interfaces. */
    ADI_DACC_DACIF_MAX

} ADI_DACC_DACIF;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/**
 * @brief       Opens DACC Device.
 *
 * @details     DACC device instance should be opened to call any other DACC APIs.
 *              If the device is opened successfully the handle to the device
 *              is returned to the application. The opened instance can be
 *              closed using #adi_dacc_Close API.
 *
 * @param [in]  nDeviceNum      DACC instance number to be opened. The processor
 *                              can have one or more instances of DACC. This
 *                              indicates the instance number of the DACC to be
 *                              opened.
 *
 * @param [in]  pMemory         Memory required for the operation of DACC driver.
 *                              The size of the given memory should be equal to
 *                              #ADI_DACC_MEMORY bytes.
 *
 * @param [out] phDevice        Handle to the instance that is opened.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully opened the DACC device.
 *
 * @sa adi_dacc_Close()
 */
ADI_DACC_RESULT  adi_dacc_Open (
    uint32_t                    nDeviceNum,
    void const                 *pMemory,
    ADI_DACC_HANDLE            *phDevice
);

/**
 * @brief       Closes DACC Device.
 *
 * @details     Once the DACC device is closed none of the APIs will
 *              work with the given handle.
 *
 * @param [in]  hDevice         Handle to the DACC device to be close.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully closed the DACC device.
 *
 * @sa adi_dacc_Open()
 */
ADI_DACC_RESULT  adi_dacc_Close (
   ADI_DACC_HANDLE const        hDevice
);

/**
 * @brief       Register or unregister an optional callback with DACC.
 *
 * @details     When a callback is registered some of the buffer transaction
 *              APIs cannot be used. The list of APIs that cannot be used
 *              are:
 *
 *              adi_dacc_GetBuffer()
 *              adi_dacc_IsBufferAvailable()
 *
 *              When the callback is registered the processed buffer pointer
 *              is returned via the callback in the #ADI_DACC_EVENT_BUFFER_PROCESSED
 *              event. Also all the hardware errors are returned via the
 *              callback. Please see the enumeration ADI_DACC_ERR.
 *
 *              When the callback is not registered the processed buffer pointer
 *              can be accessed using the #adi_dacc_GetBuffer(). When a hardware
 *              error is detected the API #adi_dacc_GetBuffer will immediately
 *              return with an error #ADI_DACC_HW_ERROR_DETECTED. Upon knowing
 *              that there is a hardware error(s), the application can call the
 *              API #adi_dacc_GetHWErrorStatus() to know exact error(s).
 *
 *              The callback can be unregistered by passing the pfCallback parameter
 *              as NULL.
 *
 * @note        This API cannot be called when DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device.
 *
 * @param [in]  pfCallback      Pointer to the callback function, which will be called
 *                              by the driver to notify hardware errors and events.
 *
 * @param [in]  pCBParam        Callback parameter which will be passed back to the
 *                              application when the callback is called.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully registered or unregistered
 *                              the callback.
 *
 * @sa adi_dacc_Open()
 */
ADI_DACC_RESULT  adi_dacc_RegisterDevCallback (
    ADI_DACC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
);


/**
 * @brief       Submits the given buffer for DACC.
 *
 * @details     This is a non-blocking call. Only one buffer can be
 *              submitted at a time. The next buffer can be submitted only when the
 *              previously submitted buffer is processed.
 *
 *              When the callback is note registered the processed buffer can be
 *              obtained back by using using the API #adi_dacc_GetBuffer(). If the
 *              callback is registered the processed buffer is returned as callback
 *              argument for the event #ADI_DACC_EVENT_BUFFER_PROCESSED.
 *
 * @param [in]  hDevice         Handle to the DACC device to which the buffer to be
 *                              submitted.
 *
 * @param [in]  pBuffer         Pointer to the buffer to be submitted.
 *
 * @param [in]  pBufSize        Buffer size in terms of samples.
 *
 * @param [in]  nIncrement      Buffer address increment in terms of bytes which will be
 *                              applied after sending each sample to the DAC. The increment
 *                              is a signed number. So the increment can also be a negative
 *                              number.
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully submitted the buffer.
 *
 * @sa  adi_dacc_GetBuffer()
 * @sa  adi_dacc_IsBufferAvailable()
 * @sa  adi_dacc_SubmitCircBuffer()
 */
ADI_DACC_RESULT  adi_dacc_SubmitBuffer (
    ADI_DACC_HANDLE const       hDevice,
    void * const                pBuffer,
    uint32_t                    pBufSize,
    int32_t                     nIncrement
);

/**
 * @brief       Submit circular buffer for DACC.
 *
 * @details     This API only works in DMA mode. So DMA mode should be set first
 *              (using the API #adi_dacc_EnableDMAMode) before using this API.
 *              It is a is a non-blocking call and returns immediately after
 *              submitting the buffer.
 *
 *              The read pointer to the circular buffer is updated by the device and the
 *              write pointers should be maintained by the application.
 *              The application can get the current read pointer by using the API
 *              #adi_dacc_GetCurrRdPtr()
 *
 * @note        This API cannot be called after enabling the DAC.
 *
 * @param [in]  hDevice         Handle to the DACC device to which the buffer to be
 *                              submitted.
 *
 * @param [in]  pBuffer         Pointer to the circular buffer to be submitted.
 *
 * @param [in]  nBufSize        Buffer size in terms of samples.
 *
 * @param [in]  nIncrement      Buffer address increment in terms of bytes which will be
 *                              applied after sending each sample to the DAC. The increment
 *                              is a signed number. So can be a negative number.
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully submitted the circular buffer.
 *
 * @sa  adi_dacc_SubmitBuffer()
 * @sa  adi_dacc_GetCurrRdPtr()
 */
ADI_DACC_RESULT  adi_dacc_SubmitCircBuffer (
    ADI_DACC_HANDLE const       hDevice,
    void * const                pBuffer,
    uint32_t                    nBufSize,
    int32_t                     nIncrement
);


/**
 * @brief       Gets back the processed buffer.
 *
 * @details     This API blocks until the processed buffer is available. To avoid the
 *              block application can use the API #adi_dacc_IsBufferAvailable() to know
 *              if there is a processed buffer available. Upon knowing that there is
 *              buffer available this API can be used.
 *
 *              This API will return immediately if a buffer is available. It can also
 *              return with a partially processed buffer with an error
 *              #ADI_DACC_HW_ERROR_DETECTED if an hardware error is detected. To know
 *              the specifics of the error application can call the API
 *              #adi_dacc_GetHWErrorStatus().
 *
 * @note        This API cannot be called in circular buffering mode. That is if a
 *              circular buffer is submitted before by using #adi_dacc_SubmitCircBuffer.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice         Handle to the DACC device from which buffer to be
 *                              obtained.
 *
 * @param [out] ppBuffer        Pointer to a location where the processed buffer pointer
 *                              is written.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully obtained the processed buffer.
 *
 * @sa  adi_dacc_SubmitBuffer()
 * @sa  adi_dacc_GetHWErrorStatus()
 * @sa  adi_dacc_IsBufferAvailable()
 */
ADI_DACC_RESULT  adi_dacc_GetBuffer (
    ADI_DACC_HANDLE  const      hDevice,
    void ** const               ppBuffer
);

/**
 * @brief       Gets the current read pointer.
 *
 * @details     This API is valid only in circular buffering mode. The API returns the
 *              address of the buffer from where the device will be reading the samples
 *              next. This information will be useful to compute the amount of room
 *              available to fill the samples.
 *
 *              The write pointer to the circular buffer is not maintained by the driver.
 *              It is application responsibility to maintain the write pointer and also
 *              make sure that there are enough samples in the circular buffer to avoid
 *              the underflow.
 *
 *              When a callback is not registered this API will with an error
 *              #ADI_DACC_HW_ERROR_DETECTED if an hardware error is detected. To know
 *              the specifics of the error(s) application can call the API
 *              #adi_dacc_GetHWErrorStatus().
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice         Handle to the DACC device whose read pointer to be read.
 *
 * @param [out]  ppRdPtr        Pointer to a location where the current read pointer is
 *                              written.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully read back the current read pointer.
 *
 * @sa  adi_dacc_SubmitCircBuffer()
 * @sa  adi_dacc_GetHWErrorStatus()
 */
ADI_DACC_RESULT  adi_dacc_GetCurrRdPtr(
    ADI_DACC_HANDLE  const      hDevice,
    void **                     ppRdPtr
);

/**
 * @brief       Checks if a empty buffer is available.
 *
 * @details     This API can be used to check if an empty buffer is available.
 *              Upon knowing that there is a buffer available the API
 *              #adi_dacc_GetBuffer can be used to get back the empty buffer.
 *
 * @note        This API is not valid in circular buffering mode and when callback is
 *              registered.
 *
 * @param [in]   hDevice        Handle to the DACC device from which the buffer status
 *                              to be known.
 *
 * @param [out]  pbAvailable    Pointer to a location where the buffer availability
 *                              status is written.
 * \n
 *                              - 'true'  -   If buffer is available.
 *                              - 'false' -   If buffer is not available.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully queried the buffer status.
 *
 * @sa  adi_dacc_SubmitBuffer()
 * @sa  adi_dacc_GetBuffer()
 */
ADI_DACC_RESULT  adi_dacc_IsBufferAvailable (
    ADI_DACC_HANDLE const   hDevice,
    bool * const            pbAvailable
);

/**
 * @brief       Enable or disable the DAC data flow.
 *
 * @details     After the buffer DACC is configured and buffer is submitted DACC should
 *              be enabled to start sending the data to DAC.
 *
 * @param [in]  hDevice         Handle to the DACC device which should be enabled or
 *                              disabled.
 *
 * @param [in]  bEnable         Flag which indicates whether to enable or disable the DACC.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully enabled or disabled the DACC.
 */
ADI_DACC_RESULT  adi_dacc_Enable(
    ADI_DACC_HANDLE const       hDevice,
    bool                        bEnable
);


/**
 * @brief       Configures data interface.
 *
 * @details     Configures the parameters related to the data interface. This API
 *              cannot be used after the DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device whose data interface to be
 *                              configured.
 *
 * @param [in]  bLSBFirst       Flag to indicate if the data to be sent out as LSB
 *                              first or MSB first.
 * \n
 *                              - 'true'  -   For LSB first.
 *                              - 'false' -   For MSB first.
 *
 * @param [in]  nDataWidth      The data width is in terms of number of bits.
 *                              A valid value can be 1 to 16. Data for bit widths
 *                              other than 16bit should be LSB aligned.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully configured the data interface.
 */
ADI_DACC_RESULT  adi_dacc_ConfigDataIF (
    ADI_DACC_HANDLE             hDevice,
    bool                        bLSBFirst,
    uint8_t                     nDataWidth
);

/**
 * @brief       Configure DAC clock.
 *
 * @details     Configures the parameters related to the DAC clock. This API
 *              cannot be used after the DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device whose clock to be
 *                              configured.
 *
 * @param [in]  nClkDiv         Clock divide for DAC clock DACCK.
 *                              DACCK frequency = SCLK Frequency/(nClkDiv + 1)
 *
 * @param [in]  bGatedClk       Flag which indicates if clock gating to be enabled
 *                              or disabled. When clock gating is enabled DAC Clock
 *                              is only generated only when data line is driven.
 *                              Otherwise DAC Clock is always driven.
 * \n
 *                              - 'true'  -   Enable clock gating.
 *                              - 'false' -   Disable clock gating.
 *
 * @param [in]  bRisingEdge     Flag to indicate the clock polarity.
 * \n
 *                              - 'true'  -   Frame Sync and Data are driven at falling edge of
 *                                            DAC Clock.
 *                              - 'false' -   Frame Sync and Data are driven at rising edge of
 *                                            DAC clock.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully configured clock.
 */
ADI_DACC_RESULT  adi_dacc_ConfigClock(
    ADI_DACC_HANDLE             hDevice,
    uint16_t                    nClkDiv,
    bool                        bGatedClk,
    bool                        bRisingEdge
);

/**
 * @brief       Set DMA transfer width.
 *
 * @details     By default the DMA transfer width is 16bit. It can be changed
 *              to 32bit DMA to achieve better throughput.
 *
 * @note        This API is only valid in DMA mode. So DMA mode should be enabled
 *              first (using API #adi_dacc_EnableDMAMode) before calling this API.
 *
 * @note        This API cannot be called after the DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device whose DMA width to be set.
 *
 * @param [in]  b32bitMode      Flag to indicate whether to enable 32bit DMA.
 * \n
 *                              - 'true'  -   For 32bit DMA.
 *                              - 'false' -   For 16bit DMA.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the DMA width.
 *
 * @sa  adi_dacc_EnableDMAMode()
 * @sa  adi_dacc_Enable()
 */
ADI_DACC_RESULT  adi_dacc_SetDMAWidth(
    ADI_DACC_HANDLE             hDevice,
    bool                        b32bitMode
);

/**
 * @brief       Configures the frame sync.
 *
 * @details     Configures the frame sync signal related parameters.
 *
 * @note        This API cannot be called after enabling the DACC.
 *
 * @param [in]  hDevice         Handle to the DACC device whose Frame Sync to
 *                              be configured.
 *
 * @param [in]  bActiveHigh     Flag to indicate whether the frame sync signal
 *                              is active high or active low (default).
 * \n
 *                              - 'true'  -   For active high.
 *                              - 'false  -   For active low.
 *
 * @param [in]  nIdlePeriod     Idle period period between two active pulses of
 *                              Frame sync signal. By setting the idle period
 *                              the periodicity of the frame sync signal can be
 *                              varied. The periodicity of the signal is given
 *                              by:
 *
 *                              Periodicity = (nIdlePeriod + 1) + 16.
 *
 *                              The range of nIdlePriod can be 0 to 65535.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully configured the frame sync.
 */
ADI_DACC_RESULT  adi_dacc_ConfigFrameSync(
    ADI_DACC_HANDLE            hDevice,
    bool                        bActiveHigh,
    uint16_t                    nIdlePeriod
);

/**
 * @brief       Enable or disable DMA mode.
 *
 * @details     Enable DMA to get the data from the event data registers to memory.
 *
 * @note        This API cannot be called after enabling the DACC.
 *
 * @param [in]  hDevice         Handle to the DACC device which should be enabled
 *                              or disabled.
 *
 * @param [in]  bEnable         Flag which indicates whether to enable or disable
 *                              the DMA mode.
 * \n
 *                              - 'true'  -   To enable DMA mode.
 *                              - 'false' -   To disable DMA mode.
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully enabled the DMA mode.
 */
ADI_DACC_RESULT  adi_dacc_EnableDMAMode(
    ADI_DACC_HANDLE const       hDevice,
    bool                        bEnable
);


/**
 * @brief       Set the mask for error interrupt.
 *
 * @details     Set the error mask to disable the errors reported by DAC
 *              Controller. The interrupts specified by the mask will be 
 *              masked. By default underflow errors are disabled.
 *
 *              When a callback is registered the errors are reported
 *              via the callback.
 *
 *              When the callback is not registered application the
 *              API #adi_dacc_GetBuffer() will return with an error
 *              #ADI_DACC_HW_ERROR_DETECTED when an error is detected.
 *              Upon knowing that there is an error application can
 *              call the API #adi_dacc_GetHWErrorStatus to know the
 *              actual error(s).
 *
 *
 * @param [in]  hDevice         Handle to the DACC device whose error mask to
 *                              be set.
 *
 * @param [in]  nErrMask        Mask is constructed orring required errors in
 *                              ADI_DACC_ERR.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the error mask.
 */
ADI_DACC_RESULT  adi_dacc_SetErrMask(
    ADI_DACC_HANDLE const       hDevice,
    uint32_t                    nErrMask
);

/**
 * @brief       Clear the mask for error interrupt.
 *
 * @details     Clear the error mask to enable the errors reported by DAC
 *              Controller. By default only underflow errors are disabled.
 *              Rest of the errors are enabled.
 *
 *              When a callback is registered the errors are reported
 *              via the callback.
 *
 *              When the callback is not registered application the
 *              API #adi_dacc_GetBuffer() will return with an error
 *              #ADI_DACC_HW_ERROR_DETECTED when an error is detected.
 *              Upon knowing that there is an error application can
 *              call the API #adi_dacc_GetHWErrorStatus to know the
 *              actual error(s).
 *
 *
 * @param [in]  hDevice         Handle to the DACC device whose error mask to
 *                              be set.
 *
 * @param [in]  nErrMask        Mask is constructed orring required errors in
 *                              ADI_DACC_ERR.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the error mask.
 */
ADI_DACC_RESULT  adi_dacc_ClearErrMask(
    ADI_DACC_HANDLE const       hDevice,
    uint32_t                    nErrMask
);

/**
 * @brief       Queries for the hardware errors.
 *
 * @details     When the DAC controller callback is not registered, the hardware
 *              errors cannot be returned via the callback. So application can
 *              use this API to query for the hardware errors. The errors will
 *              be cleared after reading them.
 *
 *              When a callback is not registered the #adi_dacc_GetBuffer returns
 *              with an error #ADI_DACC_HW_ERROR_DETECTED when a hardware error is
 *              detected. Upon getting this error application can use this API to
 *              query the errors detected.
 *
 * @param [in]  hDevice         Handle to DACC device whose hardware error status
 *                              to be queried.
 *
 * @param [out] pHWErrStatus    Pointer to a location where the hardware error
 *                              status word is written.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully queried the hardware error status.
 *
 * @sa adi_dacc_GetBuffer()
 */
ADI_DACC_RESULT  adi_dacc_GetHWErrorStatus(
    ADI_DACC_HANDLE             hDevice,
    uint32_t *const             pHWErrStatus
);


/**
 * @brief       Sets a given DACC register.
 *
 * @details     Most of the commonly used register fields are accessed via
 *              the APIs. If a register field is not covered by the APIs
 *              application can use this API to set the given register
 *              with the given value.
 *
 * @note        This API cannot be called after enabling the DACC.
 *
 * @param [in]  hDevice         Handle to the DACC device whose register to
 *                              be set.
 *
 * @param [in]  nRegAddr        Address of the register to be set.
 *
 * @param [in]  nRegValue       Value to be set into the given register.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the register.
 */
ADI_DACC_RESULT  adi_dacc_SetRegister (
    ADI_DACC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t                    nRegValue
);


/**
 * @brief       Gets the contents of the given DACC register.
 *
 * @details     Most of the commonly used register fields are accessed via
 *              the APIs. If a register field is not covered by the APIs
 *              application can use this API to get contents of the given
 *              register.
 *
 * @param [in]  hDevice         Handle to the DACC device whose register to
 *                              be read.
 *
 * @param [in]  nRegAddr        Address of the register to be read.
 *
 * @param [out] pRegValue       Pointer to a location where the contents
 *                              given register is written.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully read the register.
 */
ADI_DACC_RESULT  adi_dacc_GetRegister (
    ADI_DACC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t * const            pRegValue
);


/**
 * @brief       Register or unregister an optional erorr callback with DACC.
 *
 * @details     
 *
 * @note        This API cannot be called when DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device.
 *
 * @param [in]  pfCallback      Pointer to the callback function, which will be called
 *                              by the driver to notify hardware errors and events.
 *
 * @param [in]  pCBParam        Callback parameter which will be passed back to the
 *                              application when the callback is called.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully registered or unregistered
 *                              the callback.
 *
 * @sa adi_dacc_Open()
 */
ADI_DACC_RESULT  adi_dacc_RegisterErrCallback (
    ADI_DACC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_DACC_H_ */

/*@}*/
