/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
* @file      adi_flash.h
*
* @brief     FLASH driver API Header file
*
* @details
*            Header File for the ADSP-CM40z flash driver API functions and definitions
*/

/** @addtogroup FLASH_Driver_CM40z FLASH Device Driver
*  @{
*/

#ifndef _ADI_FLASH_H_
#define _ADI_FLASH_H_

#include <drivers/spi/adi_spi.h>

#ifdef __cplusplus 
extern "C" { 
#endif /* __cplusplus */ 


/*==============  D E F I N E S  ===============*/

/* Size of flash device memory */
#define ADI_FLASH_DEV_MEMORY_SIZE (ADI_SPI_INT_MEMORY_SIZE)

/*! FLASH Device handle */
typedef void* ADI_FLASH_HANDLE;


/*! Enumeration for the return codes used by the adi_flash API functions */
typedef enum
{
    ADI_FLASH_SUCCESS = 0,             /*!< The operation succeeded.                    */
    ADI_FLASH_PENDING,                 /*!< A non-blocking operation is in progress.    */
    ADI_FLASH_NO_PENDING,              /*!< No non-blocking operation is in progress.   */
    ADI_FLASH_INVALID_DEV_NUM,         /*!< DeviceNumber is not in the valid range.     */
    ADI_FLASH_ALREADY_OPEN,            /*!< Device has been opened and not yet closed.  */
    ADI_FLASH_NOT_OPEN,                /*!< The device is not open.                     */
    ADI_FLASH_INVALID_ADDRESS,         /*!< The specified address is not valid.         */
    ADI_FLASH_INSUFFICIENT_MEMORY,     /*!< The supplied memory is too small.           */
    ADI_FLASH_INVALID_BLOCK_NUM,       /*!< Block number is out of range.               */
    ADI_FLASH_VERIFY_FAILED,           /*!< Flash contents do not match specifed data.  */
    ADI_FLASH_INVALID_MODE,            /*!< An invalid mode enumerator was supplied.    */
    ADI_FLASH_PERIPHERAL_MODE_ONLY,    /*!< Not allowed in MMAP or MMAP_XIP modes.      */
    ADI_FLASH_FAILURE                  /*!< An unspecified failure occurred.            */
} ADI_FLASH_RESULT;

/*! Enumeration for the operating modes of the flash device */
typedef enum
{
    ADI_FLASH_MODE_MMAP,               /*!< Memory-mapped mode, no execute-in-place.    */
    ADI_FLASH_MODE_MMAP_XIP,           /*!< Memory-mapped mode, with execute-in-place.  */
    ADI_FLASH_MODE_PERIPHERAL          /*!< Peripheral mode, no memory-mapping.         */
} ADI_FLASH_MODE;

/*! Structure to return information about the flash memory */
typedef struct
{
    uint32_t baseAddress;              /*!< Base address of the flash memory.           */
    uint32_t totalSize;                /*!< Total size, in bytes, of the flash memory.  */
    uint32_t blockSize;                /*!< The size of a flash block (subsector).      */
} ADI_FLASH_INFO;

/*************************************
  adi_flash API
*************************************/
__ramfunc
ADI_FLASH_RESULT adi_flash_Open(
    uint32_t          nDeviceNumber,
    void             *pDeviceMemory,
    uint32_t	      nMemorySize,
    ADI_FLASH_HANDLE *phDevice
);

__ramfunc
ADI_FLASH_RESULT adi_flash_Close(
    ADI_FLASH_HANDLE       const hDevice
);

__ramfunc
ADI_FLASH_RESULT adi_flash_EraseAll(
    ADI_FLASH_HANDLE       const hDevice
);

__ramfunc
ADI_FLASH_RESULT adi_flash_EraseBlocks(
    ADI_FLASH_HANDLE       const hDevice,
    uint32_t               const nStartBlock,
    uint32_t               const nEndBlock
);

__ramfunc
ADI_FLASH_RESULT adi_flash_SubmitEraseAll(
    ADI_FLASH_HANDLE       const hDevice
);

__ramfunc
ADI_FLASH_RESULT adi_flash_SubmitEraseBlocks(
    ADI_FLASH_HANDLE       const hDevice,
    uint32_t               const nStartBlock,
    uint32_t               const nEndBlock
);

__ramfunc
ADI_FLASH_RESULT adi_flash_IsOperationComplete(
    ADI_FLASH_HANDLE       const hDevice,
    bool_t *               const pbIsComplete
);

__ramfunc
ADI_FLASH_RESULT adi_flash_Finish(
    ADI_FLASH_HANDLE       const hDevice
);

__ramfunc
ADI_FLASH_RESULT adi_flash_GetBlockNumber(
    ADI_FLASH_HANDLE       const hDevice,
    uint32_t               const nAddress,
    uint32_t *             const pnBlockNumber
);
                                      
__ramfunc
ADI_FLASH_RESULT adi_flash_Write(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t const *  const pData,
    uint32_t         const nBytes
);

__ramfunc
ADI_FLASH_RESULT adi_flash_SubmitWrite(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t const *  const pData,
    uint32_t         const nBytes
);

__ramfunc
ADI_FLASH_RESULT adi_flash_Read(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t *        const pData,
    uint32_t         const nBytes
);

__ramfunc
ADI_FLASH_RESULT adi_flash_Verify(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t const *  const pData,
    uint32_t         const nBytes
);
                                      
__ramfunc
ADI_FLASH_RESULT adi_flash_Sync(
    ADI_FLASH_HANDLE const hDevice
);
 
__ramfunc
ADI_FLASH_RESULT adi_flash_Reset(
    ADI_FLASH_HANDLE const hDevice
);

__ramfunc
ADI_FLASH_RESULT adi_flash_GetMode(
    ADI_FLASH_HANDLE const hDevice,
    ADI_FLASH_MODE * const pMode
);

__ramfunc
ADI_FLASH_RESULT adi_flash_SetMode(
    ADI_FLASH_HANDLE const hDevice,
    ADI_FLASH_MODE   const mode
);

__ramfunc
ADI_FLASH_RESULT adi_flash_GetInfo(
    ADI_FLASH_HANDLE const hDevice,
    ADI_FLASH_INFO * const pInfo
);

                                      
#ifdef __cplusplus 
} 
#endif /* __cplusplus */ 

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_FLASH_H_ */
