/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_hae.h
*
* @brief     HAE driver API header file.
*
*/

/** @defgroup HAE_Driver Harmonic Analysis Engine (HAE) Driver
 */

/** @addtogroup HAE_Driver Harmonic Analysis Engine (HAE) Driver
 *  @{
 */

#ifndef _ADI_HAE_H_
#define _ADI_HAE_H_
                        
#include <stdbool.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#endif /* _MISRA_RULES */

/*! The HAE handle. */
typedef void* ADI_HAE_HANDLE;

/*! Size of the memory in bytes required for interrupt mode operation of HAE driver. */
#define ADI_HAE_INT_MEMORY_SIZE      (372u)

/*! Size of the memory in bytes required for DMA mode operation of HAE driver. */
#define ADI_HAE_DMA_MEMORY_SIZE      (720u)

/*!
 *  Enumeration of all the result codes returned by the HAE driver.
 */
typedef enum
{
    /*! The operation completed successfully */
    ADI_HAE_SUCCESS,

    /*! The operation failed */
    ADI_HAE_FAILURE,

    /*! The given HAE handle is invalid */
    ADI_HAE_INVALID_HANDLE,

    /*! The given pointer is pointing to NULL. */
    ADI_HAE_NULL_POINTER,

    /*! The given device instance number is outside
        the number of instances supported by the processor. */
    ADI_HAE_INVALID_DEVICE_NUM,

    /*! The given instance is already opened and in use. */
    ADI_HAE_DEVICE_IN_USE,

    /*! The devive is not enabled. */
    ADI_HAE_DEVICE_NOT_ENABLED,

    /*! Failed to register the interrupt handler with interrupt service. */
    ADI_HAE_INT_FAILED,

    /*! Semaphore operation failed */
    ADI_HAE_SEM_FAILED,

    /*! The given operation is not permitted in the current state of the device. */
    ADI_HAE_INVALID_OPERATION,

    /*! If any of the given parameters are invalid. */
    ADI_HAE_INVALID_PARAMETER,
    
    /*! Failed to submit the given buffer. */
    ADI_HAE_SUBMIT_FAILED,
    
    /*! DMA related failure occurred. */
    ADI_HAE_DMA_FAILED,
    
    /*! When trying to enable the data flow before submitting the initial
     *  two buffers  */
    ADI_HAE_BUFFERS_NOT_SUBMITTED,

}ADI_HAE_RESULT;

/*!
 * Event enumeration.
 */
typedef enum
{
    ADI_HAE_EVENT_RESULTS_READY,    /*!< HAE results are finished and ready to transmit. */
    ADI_HAE_EVENT_INPUT_SAMPLE      /*!< HAE requests a new input sample on both the I and V channels. */
} ADI_HAE_EVENT;

/*!
 * Line frequency enumeration.
 */
typedef enum
{
    ADI_HAE_LINE_FREQ_50  = 0x0u,   /*!< Select 50 Hz line frequency. */
    ADI_HAE_LINE_FREQ_60  = 0x1u    /*!< Select 60 Hz line frequency. */
} ADI_HAE_LINE_FREQ;

/*!
 * interrupt enumeration.
 */
typedef enum
{
    ADI_HAE_INT_RX,                 /*!< The Rx sample interrupt.  */
    ADI_HAE_INT_TX                  /*!< The Tx results interrupt. */
} ADI_HAE_INT;

/*!
 * Result mode enumeration.
 */
typedef enum
{
    ADI_HAE_RESULT_MODE_SETTLE     = 0x0u,   /*!< Wait till the settle field is satisfied before producing results. */
    ADI_HAE_RESULT_MODE_IMMEDIATE  = 0x1u    /*!< Produce results immediately. */
} ADI_HAE_RESULT_MODE;

/*!
 * Settle time enumeration.
 */
typedef enum
{
    ADI_HAE_SETTLE_TIME_512  = 0x0u,   /*!< Select 512 ms settle time. */
    ADI_HAE_SETTLE_TIME_768  = 0x1u,   /*!< Select 768 ms settle time. */
    ADI_HAE_SETTLE_TIME_1024 = 0x2u,   /*!< Select 1024 ms settle time. */
    ADI_HAE_SETTLE_TIME_1280 = 0x3u    /*!< Select 1280 ms settle time. */
} ADI_HAE_SETTLE_TIME;

/*!
 * Update rate enumeration.
 */
typedef enum
{
    ADI_HAE_UPDATE_RATE_125     = 0x0u,   /*!< Select 125 us update rate. */
    ADI_HAE_UPDATE_RATE_250     = 0x1u,   /*!< Select 250 us update rate. */
    ADI_HAE_UPDATE_RATE_1000    = 0x2u,   /*!< Select 1 ms update rate. */
    ADI_HAE_UPDATE_RATE_16000   = 0x3u,   /*!< Select 16 ms update rate. */
    ADI_HAE_UPDATE_RATE_128000  = 0x4u,   /*!< Select 128 ms update rate. */
    ADI_HAE_UPDATE_RATE_512000  = 0x5u,   /*!< Select 512 ms update rate. */
    ADI_HAE_UPDATE_RATE_1024000 = 0x6u    /*!< Select 1024 ms update rate. */
} ADI_HAE_UPDATE_RATE;

/*!
 * Channel enumeration.
 */
typedef enum
{
    ADI_HAE_CHAN_TX_FUNDAMENTAL     = 0x0001u,   /*!< Enable fundamental result. */
    ADI_HAE_CHAN_TX_HARMONIC_1      = 0x0002u,   /*!< Enable harmonic 1 result. */
    ADI_HAE_CHAN_TX_HARMONIC_2      = 0x0004u,   /*!< Enable harmonic 2 result. */
    ADI_HAE_CHAN_TX_HARMONIC_3      = 0x0008u,   /*!< Enable harmonic 3 result. */
    ADI_HAE_CHAN_TX_HARMONIC_4      = 0x0010u,   /*!< Enable harmonic 4 result. */
    ADI_HAE_CHAN_TX_HARMONIC_5      = 0x0020u,   /*!< Enable harmonic 5 result. */
    ADI_HAE_CHAN_TX_HARMONIC_6      = 0x0040u,   /*!< Enable harmonic 6 result. */
    ADI_HAE_CHAN_TX_HARMONIC_7      = 0x0080u,   /*!< Enable harmonic 7 result. */
    ADI_HAE_CHAN_TX_HARMONIC_8      = 0x0100u,   /*!< Enable harmonic 8 result. */
    ADI_HAE_CHAN_TX_HARMONIC_9      = 0x0200u,   /*!< Enable harmonic 9 result. */
    ADI_HAE_CHAN_TX_HARMONIC_10     = 0x0400u,   /*!< Enable harmonic 10 result. */
    ADI_HAE_CHAN_TX_HARMONIC_11     = 0x0800u,   /*!< Enable harmonic 11 result. */
    ADI_HAE_CHAN_TX_HARMONIC_12     = 0x1000u    /*!< Enable harmonic 12 result. */
} ADI_HAE_CHAN_TX;

/*!
 * Harmonic index.
 */
typedef enum
{
    ADI_HAE_HARMONIC_INDEX_1,                  /*!< Set the harmonic 1 index value. */
    ADI_HAE_HARMONIC_INDEX_2,                  /*!< Set the harmonic 2 index value. */
    ADI_HAE_HARMONIC_INDEX_3,                  /*!< Set the harmonic 3 index value. */
    ADI_HAE_HARMONIC_INDEX_4,                  /*!< Set the harmonic 4 index value. */
    ADI_HAE_HARMONIC_INDEX_5,                  /*!< Set the harmonic 5 index value. */
    ADI_HAE_HARMONIC_INDEX_6,                  /*!< Set the harmonic 6 index value. */
    ADI_HAE_HARMONIC_INDEX_7,                  /*!< Set the harmonic 7 index value. */
    ADI_HAE_HARMONIC_INDEX_8,                  /*!< Set the harmonic 8 index value. */
    ADI_HAE_HARMONIC_INDEX_9,                  /*!< Set the harmonic 9 index value. */
    ADI_HAE_HARMONIC_INDEX_10,                 /*!< Set the harmonic 10 index value. */
    ADI_HAE_HARMONIC_INDEX_11,                 /*!< Set the harmonic 11 index value. */
    ADI_HAE_HARMONIC_INDEX_12,                 /*!< Set the harmonic 12 index value. */
} ADI_HAE_HARMONIC_INDEX;

/*!
 * HAE result structure.
 */
typedef struct 
{
    int32_t  irms;                            /*!< The IRMS result. */
    int32_t  vrms;                            /*!< The VRMS result. */
    int32_t  activePwr;                       /*!< The active power result. */
    int32_t  reactivePwr;                     /*!< The reative power result. */
    int32_t  apparentPwr;                     /*!< The apparent power result. */
    int32_t  powerFactor;                     /*!< The power factor result. */
    int32_t  ihdn;                            /*!< The IHDN result. */
    int32_t  vhdn;                            /*!< The VHDN result. */
} ADI_HAE_RESULT_STRUCT;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Open a HAE device instance.
 */
ADI_HAE_RESULT  adi_hae_Open (
    uint32_t             const  nDeviceNum,
    void                 const *pMemory,
    uint32_t             const  MemorySize,
    ADI_HAE_HANDLE             *phDevice
);

/*
 * Close the given HAE device.
 */
ADI_HAE_RESULT  adi_hae_Close (
   ADI_HAE_HANDLE const        hDevice
);


/*
 * Register/Unregister an application defined status callback function.
 */
ADI_HAE_RESULT  adi_hae_RegisterCallback (
    ADI_HAE_HANDLE     const    hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
);

/*
 * Start or idle the HAE block harmonic calculations.
 */
ADI_HAE_RESULT adi_hae_Run (
    ADI_HAE_HANDLE        const   hDevice,
    bool                  const   bEnable
);

/*
 * Config 0
 */

/*
 * Enable/Disable interrupt events.
 */
ADI_HAE_RESULT  adi_hae_EnableInterrupt (
    ADI_HAE_HANDLE      const   hDevice,
    ADI_HAE_INT         const   eDataInt,
    bool                const   bEnable
);

/*
 * Enables or disables DMA mode.
 */
ADI_HAE_RESULT adi_hae_EnableDMAMode (
    ADI_HAE_HANDLE         const hDevice,
    bool                   const bEnable
);

/*
 * Select the line frequency to analyze.
 */
ADI_HAE_RESULT adi_hae_SelectLineFreq (
    ADI_HAE_HANDLE        const   hDevice,
    ADI_HAE_LINE_FREQ     const   eLineFreq
);

/*
 * Config 1
 */

/*
 * Set the sample clock divider.
 */
ADI_HAE_RESULT adi_hae_ConfigSampleDivider (
    ADI_HAE_HANDLE        const   hDevice,
    uint32_t              const   nSclk
);

/*
 * Config 2
 */

/*
 * Configure the HAE results.
 */
ADI_HAE_RESULT adi_hae_ConfigResults (
    ADI_HAE_HANDLE        const   hDevice,
    ADI_HAE_RESULT_MODE   const   eResultMode,
    ADI_HAE_SETTLE_TIME   const   eSettleTime,
    ADI_HAE_UPDATE_RATE   const   eUpdateRate
);

/*
 * Select the HAE result channels to transfer over DMA to memory.
 */
ADI_HAE_RESULT adi_hae_EnableChannels (
    ADI_HAE_HANDLE        const   hDevice,
    uint32_t              const   channels
);

/*
 * Config 3
 */

/*
 * Provide the voltage and current input samples (interrupt mode).
 */
ADI_HAE_RESULT adi_hae_InputSample (
    ADI_HAE_HANDLE        const   hDevice,
    int32_t               const   current,
    int32_t               const   voltage
);

/*
 * Get the processed input sample waveform.
 */
ADI_HAE_RESULT adi_hae_GetWaveform (
    ADI_HAE_HANDLE        const   hDevice,
    uint32_t                      *pCurrent,
    uint32_t                      *pVoltage
);

/*
 * Config 4
 */

/*
 * Enable input signal processing.
 */
ADI_HAE_RESULT adi_hae_EnableInputProcessing (
    ADI_HAE_HANDLE        const   hDevice,
    bool                  const   bEnableIntegrator,
    bool                  const   bEnableFilter
);

/*
 * Set the fundamental voltage input level.
 */
ADI_HAE_RESULT adi_hae_SetVoltageLevel (
    ADI_HAE_HANDLE        const   hDevice,
    float                 const   voltage
);

/*
 * Set the index of the harmonic to analyze.
 */
ADI_HAE_RESULT adi_hae_HarmonicIndex (
    ADI_HAE_HANDLE             const   hDevice,
    ADI_HAE_HARMONIC_INDEX     const   eIndex,
    uint32_t                   const   value
);

/*
 * Buffer releated API's
 */

/*
 * Submits a buffer to transmit HAE results.
 */
ADI_HAE_RESULT adi_hae_SubmitTxBuffer(
    ADI_HAE_HANDLE         const     hDevice,
    void                            *pBuffer,
    uint32_t               const     nBufSize
);

/*
 * Submits a buffer to receive HAE current (I) input samples.
 */
ADI_HAE_RESULT adi_hae_SubmitRx0Buffer(
    ADI_HAE_HANDLE        const      hDevice,
    void                            *pBuffer,
    uint32_t              const      nBufSize
);

/*
 * Submits a buffer to receive HAE voltage (V) input samples.
 */
ADI_HAE_RESULT adi_hae_SubmitRx1Buffer(
    ADI_HAE_HANDLE        const      hDevice,
    void                            *pBuffer,
    uint32_t              const      nBufSize
);

/*
 * Get the buffer which contains the latest HAE results.
 */
ADI_HAE_RESULT adi_hae_GetTxBuffer (
    ADI_HAE_HANDLE        const  hDevice,
    void                **const  ppBuffer
);

/*
 * Get the buffer which transmits the HAE current (I) input samples.
 */
ADI_HAE_RESULT adi_hae_GetRx0Buffer (
    ADI_HAE_HANDLE        const  hDevice,
    void                **const  ppBuffer
);

/*
 * Get the buffer which transmits the HAE voltage (V) input samples.
 */
ADI_HAE_RESULT adi_hae_GetRx1Buffer (
    ADI_HAE_HANDLE        const  hDevice,
    void                **const  ppBuffer
);

/*
 * Queries to find if a result buffer is available.
 */
ADI_HAE_RESULT adi_hae_IsTxBufferAvailable (
    ADI_HAE_HANDLE          const hDevice,
    bool                   *const pbAvailable
);

/*
 * Queries to find if a input current (I) sample buffer is available.
 */
ADI_HAE_RESULT adi_hae_IsRx0BufferAvailable (
    ADI_HAE_HANDLE          const hDevice,
    bool                   *const pbAvailable
);

/*
 * Queries to find if a input voltage (V) sample buffer is available.
 */
ADI_HAE_RESULT adi_hae_IsRx1BufferAvailable (
    ADI_HAE_HANDLE          const hDevice,
    bool                   *const pbAvailable
);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_HAE_H_ */

/*@}*/
