/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_counter_bf6xx.h
*
* @brief     Counter   driver API Header file
*
* @details
*            Header File for the counter driver API functions and definitions
*/

/** @addtogroup rotary_counter_Driver_BF6xx Rotary Counter Driver
 *  @{
 */


#ifndef _ADI_COUNTER_BFXXXH_
#define _ADI_COUNTER_BFXXXH_
/*=============  I N C L U D E S   =============*/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <adi_types.h>
#include <stddef.h>
#include <sys/platform.h> 
#include <sys/exception.h>
#include <services/int/adi_int.h>
#include <adi_osal.h>
#if defined(__ADI_HAS_SEC__)
#include <services/int/adi_sec.h>
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_1:"Allow for forward declaration of handles")
#endif /* _MISRA_RULES */

/*==============  D E F I N E S  ===============*/
/*! Amount of memory(In bytes) required by the rotary counter  device driver  for managing the
 *  operation. This memory is completely owned by the driver till the  end of the operation.
 */
#define ADI_COUNTER_MEMORY_SIZE 16


/**
 * Handle to the rotary counter device.
 */
typedef void*  ADI_COUNTER_HANDLE;

/**
 *  \enum ADI_COUNTER_EVENT
 * Enumeration used by the driver for  different event. This enumeration can be used to
 * enable the interrupt and will be notified with the same enumeration.
 *
 * */
typedef enum
{
    /*! illegal gray/binary code interrupt   */
    ADI_CNT_EVENT_ILLEGAL_CODE_INT  = 0X001 ,
    /*! Upcount interrupt                    */
    ADI_CNT_EVENT_UPCOUNT_INT       = 0X002 ,
    /*! Downcount interrupt                  */
    ADI_CNT_EVENT_DOWNCOUNT_INT     = 0X004 ,
    /*! CNT_COUNTER == CNT_MIN interrupt     */
    ADI_CNT_EVENT_MINCOUNT_INT      = 0X008 ,
    /*! CNT_COUNTER == CNT_MAX interrupt     */
    ADI_CNT_EVENT_MAXCOUNT_INT      = 0X010 ,
    /*! Bit 31 overflow interrupt            */
    ADI_CNT_EVENT_OVERFLOW31_INT    = 0X020 ,
    /*! Bit 15 overflow interrupt            */
    ADI_CNT_EVENT_OVERFLOW15_INT    = 0X040 ,
    /*! count to zero interrupt              */
    ADI_CNT_EVENT_COUNT_TO_ZERO_INT = 0X080 ,
    /*! CZM pin/push button interrupt        */
    ADI_CNT_EVENT_CZMPIN_INT        = 0X100 ,
    /*! Zero marker error interrupt          */
    ADI_CNT_EVENT_CZM_ERROR_INT     = 0X200 ,
    /*! Counter zeroes by zero marker int    */
    ADI_CNT_EVENT_CZM_COUNTZERO_INT = 0X400
} ADI_COUNTER_EVENT;

/**
 *  \enum ADI_COUNTER_COMMAND
 * Enumeration whcih can be used to indicate the driver to perfrom different command.
 *
 * */
typedef enum
{
    /*! To zero CNT_COUNTER   */
    ADI_CNT_SET_COUNTER_TO_ZERO              = 0X0001,
    /*! To load COUNTER from MINIMUM bourndary value. */
    ADI_CNT_LOAD_COUNTER_FROM_MIN            = 0X0004,
    /*! To load COUNTER from MAXIMUM bourndary value. */
    ADI_CNT_LOAD_COUNTER_FROM_MAX            = 0X0008,
    /*! To load  zero MINIMUM boundary register */
    ADI_CNT_SET_MIN_TO_ZERO                  = 0X0010,
    /*! To capture current COUNT to MIN bourndary value.*/
    ADI_CNT_LOAD_MIN_FROM_COUNTER            = 0X0020,
    /*! To copy former MAXIMUM  as new MINIMUM boundary value*/
    ADI_CNT_LOAD_MIN_FROM_MAX                = 0X0080,
    /*! To load  zero MAXIMUM  boundary register             */
    ADI_CNT_SET_MAX_TO_ZERO                  = 0X0100,
    /*! To capture current COUNT to MAXIMUM bourndary value.*/
    ADI_CNT_LOAD_MAX_FROM_COUNTER            = 0X0200,
    /*! To copy form MINIMUM   as new MAXIMUM boundary value */
    ADI_CNT_LOAD_MAX_FROM_MIN                = 0X0400,
    /*! To enable single Zero-marker-clear CNT_COUNTER  */
    ADI_CNT_ENABLE_ZERO_MARKER_COUNTER_CLEAR = 0X1000
} ADI_COUNTER_COMMAND;


/**
 *  \enum ADI_COUNTER_OPERATIONAL_MODE
 *  Enumeration used by the driver to palce the driver in various operational  modes.
 *
 * */
typedef enum
{
    /*! Quadrature Encoder Mode   */
    ADI_CNT_QUAD_ENC_MODE = 0X000,
    /*! Binary Encoder Mode       */
    ADI_CNT_BIN_ENC_MODE  = 0X100,
    /*! Up/Down Counter Mode      */
    ADI_CNT_UD_CNT_MODE   = 0X200,
    /*! Direction Counter Mode    */
    ADI_CNT_DIR_CNT_MODE  = 0X400,
    /*! Direction Timer Mode      */
    ADI_CNT_DIR_TMR_MODE  = 0X500

} ADI_COUNTER_OPERATIONAL_MODE;

/**
 *  \enum ADI_COUNTER_BOUNDARY_MODE
 *  Enumeration used by the driver to set various boundary register  modes.
 *
 * */
typedef enum
{
    /*! boundary comparator mode                      */
    ADI_CNT_BND_COMP = 0X0000,
    /*! boundary compare and zero CNT_COUNTER mode    */
    ADI_CNT_BND_ZERO = 0X1000,
    /*! boundary capture mode                         */
    ADI_CNT_BND_CAPT = 0X2000,
    /*! boundary auto-extend mode                     */
    ADI_CNT_BND_AEXT = 0X3000
} ADI_COUNTER_BOUNDARY_MODE;

/**
 *  \enum ADI_COUNTER_REGISTER
 * Enumeration which can be used to set various register of the device.
 *
 * */
typedef enum
{
  /*! Counter Config Register */
  ADI_CNT_CONFIG_REG   ,
  /*! Counter interrupt mask(IMASK) Register */
  ADI_CNT_IMASK_REG    ,
  /*! Counter Command Register */
  ADI_CNT_COMMAND_REG  ,
  /*! Counter Debounce Register */
  ADI_CNT_DEBOUNCE_REG ,
  /*! Counter COUNT Register */
  ADI_CNT_COUNTER_REG   ,
  /*! Counter Maximum boundary Register */
  ADI_CNT_MAX_REG      ,
  /*! Counter Minimum Register */
  ADI_CNT_MIN_REG
} ADI_COUNTER_REGISTER;

/**
 *  \enum ADI_COUNTER_RESULT
 *  Enumeration  for counter  return values
 *
 * */

typedef enum
{
    /*! Success.*/
    ADI_COUNTER_SUCCESS,
    /*! Generic Failure to indicate the call to an counter API is unsuccessful. */
    ADI_COUNTER_FAILED,
    /*! The given handle is invalid. */
    ADI_COUNTER_INVALID_HANDLE,
    /*! The specified counter device is in use. */
    ADI_COUNTER_DEVICE_IN_USE,
    /*! Failed to register the counter driver interrupt manager. */
    ADI_COUNTER_INT_REGISTRATION_FAILED,
     /*! The size of the  memory is not enough for the device operation. */
    ADI_COUNTER_INSUFFICIENT_MEMORY,
     /*! Invalid device number. */
    ADI_COUNTER_INVALID_DEVICE_NUM ,
     /*! Specified pointer is invalid. */
    ADI_COUNTER_INVALID_NULL_POINTER,
    /*! Specified register is invalid. */
    ADI_COUNTER_INVALID_REGISTER,
    /*! Specified operation is not allowed. */
    ADI_COUNTER_OPERATION_NOT_ALLOWED,
    /*! Specified Pre-Scale value is out of range. */
    ADI_COUNTER_INVALID_PRESCALE,
    /*! Specified mode is invalid. */
    ADI_COUNTER_INVALID_MODE
} ADI_COUNTER_RESULT;
/**********************************************************
  General APIs for managing the rotary counter.
**********************************************************/
 /*
 * To open a counter device instance.
 */
ADI_COUNTER_RESULT adi_counter_Open(
                   uint32_t             nDeviceNum,
                   void                *pMemory,
                   uint32_t             nMemSize,
                   ADI_CALLBACK         pfCallback,
                   void                *pCBparam,
                   ADI_COUNTER_HANDLE  *phDevice
                   );

 /*
 * To close the specified  counter device instance.
 */

ADI_COUNTER_RESULT adi_counter_Close(
                   ADI_COUNTER_HANDLE  hDevice
                   );
 /*
 * To enable  the specified  counter device instance.
 */

ADI_COUNTER_RESULT adi_counter_Enable (
                   ADI_COUNTER_HANDLE  hDevice,
                   bool  bEnable
                   );
 /*
 * To enable the debouncing filter .
 */

ADI_COUNTER_RESULT adi_counter_EnableDebounce(
                   ADI_COUNTER_HANDLE  hDevice,
                   bool  bEnable
                   );
 /*
 * To set the register(s) of the device.
 */

ADI_COUNTER_RESULT adi_counter_SetControlReg(
                   ADI_COUNTER_HANDLE    hDevice,
                   ADI_COUNTER_REGISTER  eRegister,
                   uint32_t              nValue
                   );

 /*
 * To set the device operation mode.
 */

ADI_COUNTER_RESULT adi_counter_SetOperationMode(
                   ADI_COUNTER_HANDLE            hDevice,
                   ADI_COUNTER_OPERATIONAL_MODE  eOperationMode
                   );

 /*
 * To set the boundary mode for the device.
 */

ADI_COUNTER_RESULT adi_counter_SetBoundaryMode(
                   ADI_COUNTER_HANDLE         hDevice,
                   ADI_COUNTER_BOUNDARY_MODE  eBoundaryMode
                   );

 /*
 * To set the polarity of CDG,CUD,CZM input.
 */

ADI_COUNTER_RESULT adi_counter_SetPolarity(
                   ADI_COUNTER_HANDLE  hDevice,
                   bool                blowCDG,
                   bool                blowCUD,
                   bool                blowCZM
                   );

 /*
 * To set prescale value which determines the debouncing period.
 */

ADI_COUNTER_RESULT adi_counter_SetPreScale(
                   ADI_COUNTER_HANDLE hDevice,
                   uint8_t            nPreScale
                   );

 /*
 * To enable/disable input(s) with out disabling the device.
 */

ADI_COUNTER_RESULT adi_counter_EnableInput(
                   ADI_COUNTER_HANDLE  hDevice,
                   bool                bEnableInput
                   );


 /*
 * To set MINIMUM and MAXIMUM boundary limit.
 */

ADI_COUNTER_RESULT adi_counter_SetBoundary(
                   ADI_COUNTER_HANDLE         hDevice,
                   int32_t                   nMinimum,
                   int32_t                   nMaximum
                   );

 /*
 * To enable/disable different interrupt.
 */

ADI_COUNTER_RESULT adi_counter_EnableInterrupt(
                   ADI_COUNTER_HANDLE hDevice,
                   ADI_COUNTER_EVENT  eEventID,
                   bool               bEnable
                   );

 /*
 * To perform various command.
 */

ADI_COUNTER_RESULT adi_counter_SetCommand(
                   ADI_COUNTER_HANDLE     hDevice,
                   ADI_COUNTER_COMMAND    eCommand
                   );

#ifdef __cplusplus
}
#endif /* __cplusplus */


#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_COUNTER_BFXXXH_ */

/*@}*/
