/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_sinc.h
*
* @brief     SINC driver API header file.
*
*/

/** @defgroup SINC_Driver SINC Filter Driver
 */

/** @addtogroup SINC_Driver SINC Filter Driver
 *  @{
 */

#ifndef _ADI_SINC_H_
#define _ADI_SINC_H_
                        
#include <stdbool.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#endif /* _MISRA_RULES */

/*! The SINC filter handle. */
typedef void* ADI_SINC_HANDLE;

/*! Size of the memory in bytes required for the operation of SINC driver. */
#define ADI_SINC_MEMORY_SIZE      (200u)


/*!
 *  Enumeration of all the result codes returned by the SINC driver.
 */
typedef enum
{
    /*! The operation completed successfully */
    ADI_SINC_SUCCESS,

    /*! The operation failed */
    ADI_SINC_FAILURE,

    /*! The given SINC handle is invalid */
    ADI_SINC_INVALID_HANDLE,

    /*! The given pointer is pointing to NULL. */
    ADI_SINC_NULL_POINTER,

    /*! The given device instance number is outside
        the number of instances supported by the processor. */
    ADI_SINC_INVALID_DEVICE_NUM,

    /*! The given instance is already opened and in use. */
    ADI_SINC_DEVICE_IN_USE,

    /*! Failed to register the interrupt handler with interrupt service. */
    ADI_SINC_INT_FAILED,

    /*! Semaphore operation failed */
    ADI_SINC_SEM_FAILED,

    /*! The given operation is not permitted in the current state of the device. */
    ADI_SINC_INVALID_OPERATION,

    /*! If any of the given parameters are invalid. */
    ADI_SINC_INVALID_PARAMETER

}ADI_SINC_RESULT;

/*!
 * SINC pair enumeration.
 */
typedef enum
{
    ADI_SINC_PAIR_0,         /*!< Select SINC pair 0. */
    ADI_SINC_PAIR_1,         /*!< Select SINC pair 1. */
    ADI_SINC_PAIR_2,         /*!< Select SINC pair 2. */
    ADI_SINC_PAIR_3          /*!< Select SINC pair 3. */
} ADI_SINC_PAIR;

/*!
 * SINC group enumeration.
 */
typedef enum
{
    ADI_SINC_GROUP_0,       /*!< Select SINC group 0. */
    ADI_SINC_GROUP_1        /*!< Select SINC group 1. */
} ADI_SINC_GROUP;

/*!
 * SINC filter enumeration.
 */
typedef enum
{
    ADI_SINC_FILTER_PRIMARY,     /*!< Select the primary SINC filter. */
    ADI_SINC_FILTER_SECONDARY    /*!< Select the secondary SINC filter. */
} ADI_SINC_FILTER;

/*!
 * SINC filter order enumeration.
 */
typedef enum
{
    ADI_SINC_FILTER_THIRD_ORDER  =  0u,   /*!< Set the filter to 3rd order. */
    ADI_SINC_FILTER_FORTH_ORDER  =  1u    /*!< Set the filter to 4th order. */
} ADI_SINC_FILTER_ORDER;

/*!
 * Control interrupt mask enumeration.
 */
typedef enum
{
    ADI_SINC_INT_EFOVF0  =   BITM_SINC_CTL_EFOVF0,      /*!< FIFO Overflow Mask for Group 0.  */
    ADI_SINC_INT_EPCNT0  =   BITM_SINC_CTL_EPCNT0,      /*!< Primary SINC Output Count Mask for Group 0.  */
    ADI_SINC_INT_ESAT0   =   BITM_SINC_CTL_ESAT0,       /*!< Output Saturation Mask for Group 0.  */
    ADI_SINC_INT_ELIM0   =   BITM_SINC_CTL_ELIM0,       /*!< Limit Mask for Group 0.  */
    ADI_SINC_INT_EFOVF1  =   BITM_SINC_CTL_EFOVF1,      /*!< FIFO Overflow Mask for Group 1.  */
    ADI_SINC_INT_EPCNT1  =   BITM_SINC_CTL_EPCNT1,      /*!< Primary SINC Output Count Mask for Group 1.  */
    ADI_SINC_INT_ESAT1   =   BITM_SINC_CTL_ESAT1,       /*!< Output Saturation Mask for Group 1.  */
    ADI_SINC_INT_ELIM1   =   BITM_SINC_CTL_ELIM1        /*!< Limit Mask for Group 1.  */
} ADI_SINC_INT_MASK;

/*!
 * Event enumeration.
 */
typedef enum
{
    ADI_SINC_EVENT_STATUS,    /*!< The SINC status event. */
    ADI_SINC_EVENT_DATA0,     /*!< The SINC data0 event. */
    ADI_SINC_EVENT_DATA1      /*!< The SINC data1 event. */
} ADI_SINC_EVENT;

/*!
 * data interrupt enumeration.
 */
typedef enum
{
    ADI_SINC_DATA_INT_0  = ADI_INT_TRU0_INT0,   /*!< The TRU interrupt 0. */
    ADI_SINC_DATA_INT_1  = ADI_INT_TRU0_INT1,   /*!< The TRU interrupt 1. */
    ADI_SINC_DATA_INT_2  = ADI_INT_TRU0_INT2,   /*!< The TRU interrupt 2. */
    ADI_SINC_DATA_INT_3  = ADI_INT_TRU0_INT3    /*!< The TRU interrupt 3. */
} ADI_SINC_DATA_INT;

/*!
 * Status register enumeration.
 */
typedef enum
{
    ADI_SINC_STATUS_MIN0  = BITM_SINC_STAT_MIN0,       /*!< Min Limit for SD bitstream 0 exceeded */ 
    ADI_SINC_STATUS_MIN1  = BITM_SINC_STAT_MIN1,       /*!< Min Limit for SD bitstream 1 exceeded */
    ADI_SINC_STATUS_MIN2  = BITM_SINC_STAT_MIN2,       /*!< Min Limit for SD bitstream 2 exceeded */
    ADI_SINC_STATUS_MIN3  = BITM_SINC_STAT_MIN3,       /*!< Min Limit for SD bitstream 3 exceeded */
    ADI_SINC_STATUS_PFAB0 = BITM_SINC_STAT_PFAB0,      /*!< Primary SINC Output Error for Group 0 */
    ADI_SINC_STATUS_FOVF0 = BITM_SINC_STAT_FOVF0,      /*!< Output Data FIFO Overflow Status for Group 0 */
    ADI_SINC_STATUS_PCNT0 = BITM_SINC_STAT_PCNT0,      /*!< Primary SINC Output Count Reached for Group 0 */
    ADI_SINC_STATUS_GSAT0 = BITM_SINC_STAT_GSAT0,      /*!< Primary SINC Output Saturation for Group 0 */
    ADI_SINC_STATUS_GLIM0 = BITM_SINC_STAT_GLIM0,      /*!< Limit Exceeded for Group 0 */
    ADI_SINC_STATUS_MAX0  = BITM_SINC_STAT_MAX0,       /*!< Max Limit for SD bitstream 0 exceeded */
    ADI_SINC_STATUS_MAX1  = BITM_SINC_STAT_MAX1,       /*!< Max Limit for SD bitstream 1 exceeded */
    ADI_SINC_STATUS_MAX2  = BITM_SINC_STAT_MAX2,       /*!< Max Limit for SD bitstream 2 exceeded */
    ADI_SINC_STATUS_MAX3  = BITM_SINC_STAT_MAX3,       /*!< Max Limit for SD bitstream 3 exceeded */
    ADI_SINC_STATUS_PSAT0 = BITM_SINC_STAT_PSAT0,      /*!< Primary SINC filter 0 Saturated */
    ADI_SINC_STATUS_PSAT1 = BITM_SINC_STAT_PSAT1,      /*!< Primary SINC filter1 Saturated */
    ADI_SINC_STATUS_PSAT2 = BITM_SINC_STAT_PSAT2,      /*!< Primary SINC filter2 Saturated */
    ADI_SINC_STATUS_PSAT3 = BITM_SINC_STAT_PSAT3,      /*!< Primary SINC filter3 Saturated */
    ADI_SINC_STATUS_PFAB1 = BITM_SINC_STAT_PFAB1,      /*!< Primary SINC Output Error for Group 1 */
    ADI_SINC_STATUS_FOVF1 = BITM_SINC_STAT_FOVF1,      /*!< Output Data FIFO Overflow Status for Group 1 */
    ADI_SINC_STATUS_PCNT1 = BITM_SINC_STAT_PCNT1,      /*!< Primary SINC Output Count Reached for Group 1 */
    ADI_SINC_STATUS_GSAT1 = BITM_SINC_STAT_GSAT1,      /*!< Primary SINC Output Saturation for Group 1 */
    ADI_SINC_STATUS_GLIM1 = BITM_SINC_STAT_GLIM1       /*!< Limit Exceeded for Group 1 */
} ADI_SINC_STATUS;

/*!
 * Modulator clock enable enumeration.
 */
typedef enum
{
    ADI_SINC_MOD_CLK_DISABLED  = 0x0u,   /*!< Disable the modulator clock. */
    ADI_SINC_MOD_CLK_IMMEDIATE = 0x2u,   /*!< Enable, and commenced immediately upon being enabled. */
    ADI_SINC_MOD_CLK_PWM_SYNC  = 0x3u    /*!< Enable, and commenced upon next rising edge of PWMSYNC (PWM synchronizing output clock) */
} ADI_SINC_MOD_CLK;

/*!
 * History status enumeration.
 */
typedef enum
{
	ADI_SINC_HISTORY_REG3_MS = 0x0u,        /*!< The History Reg 3, MS position. */
	ADI_SINC_HISTORY_REG0_LS = 0x1u,        /*!< The History Reg 0, LS position. */
	ADI_SINC_HISTORY_REG0_MS = 0x2u,        /*!< The History Reg 0, MS position. */
	ADI_SINC_HISTORY_REG1_LS = 0x3u,        /*!< The History Reg 1, LS position. */
	ADI_SINC_HISTORY_REG1_MS = 0x4u,        /*!< The History Reg 1, MS position. */
	ADI_SINC_HISTORY_REG2_LS = 0x5u,        /*!< The History Reg 2, LS position. */
	ADI_SINC_HISTORY_REG2_MS = 0x6u,        /*!< The History Reg 2, MS position. */
	ADI_SINC_HISTORY_REG3_LS = 0x7u,        /*!< The History Reg 3, LS position. */
} ADI_SINC_HISTORY_STAT;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Open a SINC device instance.
 */
ADI_SINC_RESULT  adi_sinc_Open (
    uint32_t             const  nDeviceNum,
    void                 const *pMemory,
    uint32_t             const  MemorySize,
    ADI_SINC_HANDLE            *phDevice
);

/*
 * Close the given SINC device.
 */
ADI_SINC_RESULT  adi_sinc_Close (
   ADI_SINC_HANDLE const        hDevice
);


/*
 * Register/Unregister an application defined status callback function.
 */
ADI_SINC_RESULT  adi_sinc_RegisterStatusCallback (
    ADI_SINC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
);

/*
 * Register/Unregister an application defined data callback function.
 */
ADI_SINC_RESULT  adi_sinc_RegisterDataCallback (
    ADI_SINC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
);

/*
 * Enable/Disable the data interrupt event.
 */
ADI_SINC_RESULT  adi_sinc_EnableDataInterrupt (
    ADI_SINC_HANDLE     const   hDevice,
    ADI_SINC_GROUP      const   eGroup,
    ADI_SINC_DATA_INT   const   eDataInt,
    bool                const   bEnable
);

/*
 * Control
 */

/*
 * Assigns the pair to a group and enable/disable the SINC pair.
 */
ADI_SINC_RESULT  adi_sinc_EnablePair(
    ADI_SINC_HANDLE       const     hDevice,
    ADI_SINC_PAIR         const     ePair,
    ADI_SINC_GROUP        const     eGroup,
    bool                  const     bEnable
);

/*
 * Sets the control interrupt mask.
 */
ADI_SINC_RESULT  adi_sinc_SetControlIntMask (
    ADI_SINC_HANDLE const       hDevice,
    uint32_t        const       nControlIntMask
);

/*
 * Modulator clock
 */

/*
 * Sets the modulator clock controls.
 */
ADI_SINC_RESULT  adi_sinc_ConfigModClock (
    ADI_SINC_HANDLE const        hDevice,
    ADI_SINC_GROUP  const        eGroup,
    uint32_t        const        nSysClk,
    uint32_t        const        nModClockFreq,
    uint32_t        const        nClockAdjust,
    bool            const        bShiftRequest
);

/*
 * Enable/Disable the modulator clock.
 */
ADI_SINC_RESULT  adi_sinc_EnableModClock (
    ADI_SINC_HANDLE       const      hDevice,
    ADI_SINC_GROUP        const      eGroup,
    ADI_SINC_MOD_CLK      const      eEnable  
);

/*
 * Rate control
 */

/*
 * Sets the rate control values.
 */
ADI_SINC_RESULT  adi_sinc_SetRateControl (
    ADI_SINC_HANDLE   const     hDevice,
    ADI_SINC_GROUP    const     eGroup,
    ADI_SINC_FILTER   const     eFilter,
    uint16_t          const     nDecimationRate,
    uint16_t          const     nPhaseAdjust
);

/*
 * Level control
 */

/*
 * Sets the filter order.
 */
ADI_SINC_RESULT  adi_sinc_SetFilterOrder (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    ADI_SINC_FILTER_ORDER  const     ePrimaryFilterOrder,
    ADI_SINC_FILTER_ORDER  const     eSecondaryFilterOrder
);

/*
 * Sets the level control values.
 */
ADI_SINC_RESULT  adi_sinc_SetLevelControl (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    uint8_t                const     nLimitWindow,
    uint8_t                const     nLimitCount,
    uint8_t                const     nPrimaryIntRate,
    uint8_t                const     nPrimaryOutScale
);

/*
 * Sets the amplitude limits.
 */
ADI_SINC_RESULT  adi_sinc_SetAmplitudeLimit (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_PAIR          const     ePair,
    uint16_t               const     nMinAmplitude,
    uint16_t               const     nMaxAmplitude
);


/*
 * Sets the group output bias.
 */
ADI_SINC_RESULT  adi_sinc_SetOutputBias (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    uint32_t               const     nBias
);


/*
 * Circular buffer pointer
 */

/*
 * Gets the primary circular buffer pointer.
 */
ADI_SINC_RESULT  adi_sinc_GetResultLocation (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    uint32_t                        *pAddress
);

/*
 * Sets the primary circular buffer used to receive SINC data.
 */
ADI_SINC_RESULT adi_sinc_SetCircBuffer(
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    void                            *pBuffer,
    uint32_t               const     nBufSize
);

/*
 * Submits a buffer to receive SINC data samples.
 */
ADI_SINC_RESULT adi_sinc_SubmitBuffer(
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    void                            *pBuffer,
    uint32_t               const     nBufSize
);

/*
 * Get the buffer which contains the latest SINC data.
 */
ADI_SINC_RESULT adi_sinc_GetBuffer (
    ADI_SINC_HANDLE        const   hDevice,
    ADI_SINC_GROUP         const   eGroup,
    void                         **ppBuffer
);

/*
 * History buffer
 */

/*
 * Gets the secondary history buffer status.
 */
ADI_SINC_RESULT  adi_sinc_GetHistoryStatus (
    ADI_SINC_HANDLE       const       hDevice,
    ADI_SINC_PAIR         const       ePair,
    ADI_SINC_HISTORY_STAT            *ePosition
);

/*
 * Gets the secondary history values.
 */
ADI_SINC_RESULT  adi_sinc_GetHistoryValues (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_PAIR          const     ePair,
    uint16_t               const     size,
    uint16_t                        *pValues
);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_SINC_H_ */

/*@}*/
