/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_sport_bf6xx.h
*
* @brief     SPORT driver API Header file
*
* @details
*            Header File for the SPORT driver API functions and definitions
*/

/** @addtogroup SPORT_Driver_BF6xx SPORT Device Driver
 *  @{
 */


#ifndef _ADI_SPORT_BF6XXH_
#define _ADI_SPORT_BF6XXH_
/*=============  I N C L U D E S   =============*/

#ifdef __cplusplus 
extern "C" { 
#endif /* __cplusplus */ 

#include <adi_types.h>
#include <stddef.h>
#include <sys/platform.h> 
#include <sys/exception.h>
#include <services/int/adi_int.h>
#include <services/dma/adi_dma.h>
#include <adi_osal.h>
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_1:"Allow for forward declaration of handles")
#endif /* _MISRA_RULES */

/*==============  D E F I N E S  ===============*/
#if defined(__ADSPBLACKFIN__)  
/*! Amount of memory(In bytes) required by the SPORT device driver  for managing the operation
 *  in interrupt mode. This memory is completely owned by the driver till the end of
 *  the operation.
 */
#define ADI_SPORT_INT_MEMORY_SIZE 200U
/*! Amount of memory(In bytes) required by the SPORT device driver for managing the operation
 *  in DMA mode. This memory is completely owned by the driver till the end of
 *  the operation.
 */
#define ADI_SPORT_DMA_MEMORY_SIZE 316U
#elif defined(__ICCARM__)
#define ADI_SPORT_INT_MEMORY_SIZE 188u
#define ADI_SPORT_DMA_MEMORY_SIZE 304u
#elif defined (__CC_ARM)                    // KEIL: used same settings as for Blackfin (largest)
#define ADI_SPORT_INT_MEMORY_SIZE 200U
#define ADI_SPORT_DMA_MEMORY_SIZE 316U
#else
#error "Unknown compiler type"
#endif

/**
 * Handle to the SPORT Device 
 */
typedef void*  ADI_SPORT_HANDLE;

/**
 *  Enumeration of different channel of the SPORT 
 *
 */
typedef enum 
{
    /*! First half sport  */
    ADI_HALF_SPORT_A      = 0,
    /*! Second half sport */
    ADI_HALF_SPORT_B      = 1
} ADI_SPORT_CHANNEL;

/**
 *  Enumeration of different modes in which SPORT operates.
 *
 */
typedef enum 
{
    /*! DSP serial mode */
    ADI_SPORT_SERIAL_MODE,
    /*! Multi channel Mode */
    ADI_SPORT_MC_MODE,
    /*! I2S mode */
    ADI_SPORT_I2S_MODE
} ADI_SPORT_MODE;

/**
 * Enumeration of different events generated by SPORT driver 
 * 
 * */
typedef enum
{
    ADI_SPORT_HW_ERR_NONE                     = 0,
    /*! Data underflow in primary channel when SPORT is operating  in  Tx mode  */
    ADI_SPORT_HW_ERR_PRIMARY_CHNL_UNDERFLOW   = 1,
    /*! Data overflow primary channel when SPORT is operating  in  Rx mode  */
    ADI_SPORT_HW_ERR_PRIMARY_CHNL_OVERFLOW    = 2,
    /*! Data underflow in secondary channel when SPORT is operating  in  Tx mode  */
    ADI_SPORT_HW_ERR_SECONDARY_CHNL_UNDERFLOW = 4,
    /*! Data overflow secondary channel  when SPORT is operating  in  Rx mode  */
    ADI_SPORT_HW_ERR_SECONDARY_CHNL_OVERFLOW  = 8,
    /*! Error due to frame sync  */
    ADI_SPORT_HW_ERR_FS                       = 16,
    /*! Error in the DMA associated with the SPORT */
    ADI_SPORT_HW_ERR_DMA                      = 32,
    /*!< Processed the submitted RX buffer  */
    ADI_SPORT_EVENT_RX_BUFFER_PROCESSED       = 64 ,  
    /*!<  Processed the submitted TX  buffer  */
    ADI_SPORT_EVENT_TX_BUFFER_PROCESSED       = 128 
    
} ADI_SPORT_EVENT;

/**
 * SPORT  direction 
 * */
typedef enum 
{
    /*! Sport in Rx mode */    
    ADI_SPORT_DIR_RX   = 1,  
    /*! Sport in Tx mode */    
    ADI_SPORT_DIR_TX   = 2
} ADI_SPORT_DIRECTION;

/**
 * SPORT control registers
 * */
typedef enum 
{
    /*! "Main control register" which contains most of the control fields with which SPORT can be configured. */    
    ADI_SPORT_CONTROL_REGISTER ,
    /*! "Control register2" controls  the  muxing  of clock,FS between the two  half register 
        and enabling the loop back  */    
    ADI_SPORT_CONTROL_REGISTER2 ,  
    /*! Control register for Multi channel operation.  */    
    ADI_SPORT_CONTROL_MC_REGISTER,
    /*! Error Control register. */    
    ADI_SPORT_ERRCTL_REGISTER	    
} ADI_SPORT_CTL_REGISTER;


/** 
 *  SPORT return values  
 *  
 */
typedef enum 
{
    /*! Success*/    
    ADI_SPORT_SUCCESS,
    /*! Generic Failure */    
    ADI_SPORT_FAILED, 
    /*! The given handle is invalid */    
    ADI_SPORT_INVALID_HANDLE, 
    /*! The specified sport channel is in use */       
    ADI_SPORT_DEVICE_IN_USE,      
    /*! Failed to register the SPORT driver interrupt manager */
    ADI_SPORT_INT_REGISTRATION_FAILED,
    /*! The specified channel is invalid */           
    ADI_SPORT_INVALID_CHANNEL_ID,    
    /*! Failed to submit the buffer to the DMA due to non avilability of the free DMA descriptor. */
    ADI_SPORT_BUFFERS_NOT_SUBMITTED,
     /*! Invalid word size */        
    ADI_SPORT_INVALID_WORD_LENGTH,
     /*! Invalid data type */        
    ADI_SPORT_INVALID_DATA_TYPE,
     /*! Invalid frame delay(returns code only in  Multi channel mode). */        
    ADI_SPORT_INVALID_FRAME_DELAY,
    /*!< Invalid window size(returns code only in Multi channel mode)*/
    ADI_SPORT_INVALID_WINDOW_SIZE,
     /*! Invalid window offset(returns code only in Multi channel mode). */        
    ADI_SPORT_INVALID_WINDOW_OFFSET,     
     /*! Specified mode is invalid */                
    ADI_SPORT_INVALID_MODE,
     /*! Invalid device number */        
    ADI_SPORT_INVALID_DEVICE_NUM ,
     /*! Specified pointer is invalid. */        
    ADI_SPORT_INVALID_NULL_POINTER,
     /*! Not permitted in  the mode with  which device is opened. */        
    ADI_SPORT_MODE_NOT_PERMITTED,
     /*! Failed to open/close the DMA channel associated with specified SPORT device. . */
    ADI_SPORT_DMA_FAILED,
     /*! Speicified register is invalid. */            
    ADI_SPORT_INVALID_REGISTER,
    /*! The memory specified is not enough */
    ADI_SPORT_INSUFFICIENT_MEMORY,
    /*! Invalide Direction */
    ADI_SPORT_INVALID_DIR,
    /*! Specified operation is not allowed when data flow is enabled */
    ADI_SPORT_OPERATION_NOT_ALLOWED,
    /*! Invalid channel number for Multi channel operation */
    ADI_SPORT_INVALID_CHANNEL_NUM,
    /*! SPORT device encountered an peripheral error */
    ADI_SPORT_PERIPHERAL_ERROR,
    /*! Invalid slot number(returns code only in Multi channel mode). */
    ADI_SPORT_INVALID_SLOT_NUM,
    /*! Other half sport is not opened */
    ADI_SPORT_OTHER_HALF_SPORT_NOT_OPENED,
    /*! Specified Transfer size is invalid */
    ADI_SPORT_INVALID_TRANSFER_SIZE	    

} ADI_SPORT_RESULT;

/**
 *
 *  data type formatting for normal and multichannel transmissions.
 *  
 */
typedef enum 
{
    /*! Right-justify, zero-fill unused MSBs*/
        ADI_SPORT_DTYPE_ZERO_FILL,
    /*! Right-justify, sign-extend unused MSBs */
        ADI_SPORT_DTYPE_SIGN_FILL,
    /*! Compand using u-law */
        ADI_SPORT_DTYPE_COMPAND_U_LAW, 
    /*! Compand using A-law */
        ADI_SPORT_DTYPE_COMPAND_A_LAW
} ADI_SPORT_DATATYPE;

/*! Enumeration for the SPORT DMA word transfer size */
typedef enum
{
    ADI_SPORT_DMA_TRANSFER_1BYTE   = ADI_DMA_MSIZE_1BYTE,           /*!<  8  bit transfer size  */ 
    ADI_SPORT_DMA_TRANSFER_2BYTES  = ADI_DMA_MSIZE_2BYTES,          /*!<  16  bit transfer size  */    
    ADI_SPORT_DMA_TRANSFER_4BYTES  = ADI_DMA_MSIZE_4BYTES,          /*!<  32  bit transfer size  */    
    ADI_SPORT_DMA_TRANSFER_8BYTES  = ADI_DMA_MSIZE_8BYTES,          /*!<  64  bit transfer size  */   
    ADI_SPORT_DMA_TRANSFER_16BYTES = ADI_DMA_MSIZE_16BYTES,         /*!<  128 bit transfer size  */ 
    ADI_SPORT_DMA_TRANSFER_32BYTES = ADI_DMA_MSIZE_32BYTES          /*!<  256 bit transfer size  */ 
}ADI_SPORT_DMA_TRANSFER_SIZE;

/**
 * Enumeration of conditions on which an SPORT device can issue an outgoing Trigger
 */
typedef enum
{
     /*!< No trigger (default) */
    ADI_SPORT_TRIGGER_OUT_NONE      = ENUM_DMA_CFG_NO_TRIG, 
     /*!< Generate trigger when DMA transfer is completed.*/  
    ADI_SPORT_TRIGGER_OUT_DMA_DONE  = ENUM_DMA_CFG_XCNT_TRIG
} ADI_SPORT_TRIGGER_OUT;

/*
 * To open the device
*/

ADI_SPORT_RESULT adi_sport_Open(
                 uint32_t nDeviceNum,    
                 ADI_SPORT_CHANNEL      eChannel,
                 ADI_SPORT_DIRECTION    eDirection,        
                 ADI_SPORT_MODE         eMode,
                 void                  *pMemory,
                 uint32_t               nMemSize,
                 ADI_SPORT_HANDLE      *phDevice 
                 );
/*
 * To close the device
*/

ADI_SPORT_RESULT adi_sport_Close(
                 ADI_SPORT_HANDLE  hDevice
                 );
/*
 * To enable/disalbe the data flow.
*/

ADI_SPORT_RESULT adi_sport_Enable(
                 ADI_SPORT_HANDLE  hDevice,
                 bool              bEnable
                 );
/*
 * Submit an buffer to the driver.
*/

ADI_SPORT_RESULT adi_sport_SubmitBuffer(
                 ADI_SPORT_HANDLE  hDevice,
                 void             *pBuffer,
                 uint32_t          nBufSize
                 );
/*
 * Get the processed buffer from the driver.
*/

ADI_SPORT_RESULT adi_sport_GetBuffer(
                 ADI_SPORT_HANDLE const   hDevice,
                 void                   **ppBuffer
                 );
/*
 * peek function to know whether an processed buffer is avilable.
*/

ADI_SPORT_RESULT adi_sport_IsBufferAvailable(
                 ADI_SPORT_HANDLE const hDevice,
                 bool     *const pbAvailable
                 );

/*
 * To register the callback function
*/

ADI_SPORT_RESULT adi_sport_RegisterCallback(
                 ADI_SPORT_HANDLE  hDevice,
                 ADI_CALLBACK      pfCallback,
                 void             *pCBparam
                 );
/*
 * To enable/disable the dma mode of operation.
*/

ADI_SPORT_RESULT adi_sport_EnableDMAMode(
                 ADI_SPORT_HANDLE hDevice,
                 bool             bEnableDMAmode
                 );
/*
 * Configure the data
*/

ADI_SPORT_RESULT adi_sport_ConfigData(
                 ADI_SPORT_HANDLE   hDevice,
                 ADI_SPORT_DATATYPE eDataType,
                 uint8_t            nWordLength,
                 bool               bLSBFirst,
                 bool               bEnablePack,
                 bool               bRightJustifiedMode
                 );
/*
 * Configure the Clock 
*/

ADI_SPORT_RESULT adi_sport_ConfigClock(
                 ADI_SPORT_HANDLE  hDevice,
                 uint16_t          nClockRatio,
                 bool              bUseIntlClock,
                 bool              bFallingEdge,
                 bool              bGatedClk
               );
/*
 * Configure the frame sync. 
*/

ADI_SPORT_RESULT adi_sport_ConfigFrameSync(
                 ADI_SPORT_HANDLE  hDevice,
                 uint16_t          nFsDivisor,
                 bool              bFSRequired,
                 bool              bInternalFS,
                 bool              bDataFS,
                 bool              bActiveHighFS,
                 bool              bLateFS,
                 bool              bEdgeSensitiveFS
                 );
/*
 * Mutlichannel configuration function
*/

ADI_SPORT_RESULT adi_sport_ConfigMC(
                 ADI_SPORT_HANDLE  hDevice,
                 uint8_t           nFrameDelay,
                 uint8_t           nNumSlots,
                 uint16_t          nWindowSize,
                 bool              bEnableDMAPack
                 );
/*
 * Select(s) the channels in multi channel mode. 
*/

ADI_SPORT_RESULT adi_sport_SelectChannel(
                 ADI_SPORT_HANDLE  hDevice,
                 uint32_t          nStChnlNo,
                 uint32_t          nEndChnlNo
                 );
/*
 * Select the channel order. 
*/

ADI_SPORT_RESULT adi_sport_SelectChannelOrder(
                 ADI_SPORT_HANDLE  hDevice,
                 bool              bRightChFirst
                 );

/*
 * To get the Hardware error(To be used only in non-blocking mode).
*/
ADI_SPORT_RESULT adi_sport_GetHWErrorStatus (
                 ADI_SPORT_HANDLE const   hDevice,
                 uint32_t                *pHwError,
                 void *                  *pArg
                 );
/*
 * To enable the secondary channel
*/

ADI_SPORT_RESULT adi_sport_EnableSecondary(
                 ADI_SPORT_HANDLE  hDevice,
                 bool              bEnable
                 );

/*
 * To mux the halfsport. This makes the device to use FS and Clock from other halfsport.
*/

ADI_SPORT_RESULT adi_sport_MuxHalfSport(
                 ADI_SPORT_HANDLE   hDevice,
                 bool               bUseOtherFS,
                 bool               bUseOtherClk                           
                 );
/*
 * Enables the loopback between the two halfsport.
*/

ADI_SPORT_RESULT adi_sport_EnableLoopBack(
                 ADI_SPORT_HANDLE  hDevice,
                 bool              bEnable
                 );
/*
 * Set the control word to specified register.
*/
ADI_SPORT_RESULT adi_sport_SetControlReg(
                 ADI_SPORT_HANDLE         hDevice,
                 ADI_SPORT_CTL_REGISTER   eRegister,
                 uint32_t                 nControlWord
                 );
/*
 * Get the control word from the  specified register.
*/

ADI_SPORT_RESULT adi_sport_GetControlReg(
                 ADI_SPORT_HANDLE         hDevice,
                 ADI_SPORT_CTL_REGISTER   eRegister,
                 uint32_t                *pControlWord
                 );
/*
 * Enable the streaming. Useful when the data need to be transmit/receive 
 * continiously 
*/
ADI_SPORT_RESULT adi_sport_StreamingEnable(  
                 ADI_SPORT_HANDLE const hDevice,
                 bool bStreaming
                 );
/*
 * To set the DMA transfer size.
*/
ADI_SPORT_RESULT adi_sport_SetDmaTransferSize(
                 ADI_SPORT_HANDLE    const   hDevice,
                 ADI_SPORT_DMA_TRANSFER_SIZE eTransferSize
                );
/*
 * Configures the trigger input setting for a specified SPORT device.
*/
ADI_SPORT_RESULT adi_sport_ConfigTriggerIn (
                 ADI_SPORT_HANDLE    const   hDevice,
                 bool                   bTriggerWait,
                 bool                   bTriggerOvError
                 );

/*
 * Configures Trigger Output settings for the  DMA associated with  SKPORT device.
*/
ADI_SPORT_RESULT adi_sport_ConfigTriggerOut( 
                 ADI_SPORT_HANDLE    const   hDevice,
                 ADI_SPORT_TRIGGER_OUT eTrigger 
                 );

#ifdef __cplusplus 
} 
#endif /* __cplusplus */ 


#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_SPORT_BF6XXH_ */

/*@}*/
