/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_twi.h
 * @brief:   TWI device driver definitions
 * @version: $Revision: 18490 $
 * @date:    $Date: 2013-04-26 13:37:29 -0400 (Fri, 26 Apr 2013) $
 *
 * @details
 *            This is the primary header file for the TWI driver, which contains the
 *            API declarations, data and constant definitions used in the APIs
 */

#ifndef __ADI_TWI_H__
#define __ADI_TWI_H__

#include <sys/platform.h>
#include <services/int/adi_int.h>

/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"(advisory) Sections of code should not be commented out.")
#endif /* _MISRA_RULES */

/** @addtogroup TWI_Driver TWI Device Driver
 *  @{
 */

#if defined(__ADSPBF538_FAMILY__)
#include "adi_twi_bf538.h"
#elif defined(__ADSPBF548_FAMILY__) || defined(__ADSPBF548M_FAMILY__)
#include "adi_twi_bf54x.h"
#elif defined(__ADSPBF609_FAMILY__) || defined(ADI_ADSP_CM40Z) || defined(__ADSPBF707_FAMILY__)
/* use definitions from platform.h */
#elif defined (__ADSPBLACKFIN__) && !defined(__ADSPBF60x__)
#include "adi_twi_common.h"
#else
#error "Unknown processor family"
#endif

/*! A device handle used in all API functions to identify the TWI device. */
typedef void *ADI_TWI_HANDLE;

/*! The amount of application supplied memory required by the TWI driver. */
#define ADI_TWI_MEMORY_SIZE (230u)

/**
 * Result codes generated by the TWI driver.
 */
typedef enum
{
    ADI_TWI_SUCCESS = 0,           /*!< The API call succeeded. */
    ADI_TWI_FAILURE,               /*!< The API call failed. */
    ADI_TWI_BAD_DEVICE_NUMBER,     /*!< The device number is invalid. */
    ADI_TWI_MODE_NOT_SUPPORTED,    /*!< The operating mode is invalid. */
    ADI_TWI_INSUFFICIENT_MEMORY,   /*!< The application supplied memory size is insufficient. */
    ADI_TWI_BAD_ADDRESS,           /*!< The master or slave address is invalid. */
    ADI_TWI_DEVICE_IN_USE,         /*!< The device is in use. */
    ADI_TWI_BAD_DEVICE_HANDLE,     /*!< The device handle is invalid. */
    ADI_TWI_DEVICE_NOT_OPEN,       /*!< The device is not open. */
    ADI_TWI_BAD_PRESCALE_VALUE,    /*!< The prescale value is invalid. */
    ADI_TWI_BAD_BITRATE,           /*!< The bit rate is invalid. */
    ADI_TWI_BAD_DUTYCYCLE,         /*!< The duty cycle is invalid. */
    ADI_TWI_BAD_DATAWIDTH,         /*!< The data width is invalid. */
    ADI_TWI_BUFFER_NOT_AVAILABLE,  /*!< A descriptor is not available to store the given buffer. */
    ADI_TWI_BUFFER_ERROR,          /*!< An error occurred while processing the buffer. */
    ADI_TWI_BLOCKING_MODE_INVALID, /*!< The blocking mode is invalid for this API. */
    ADI_TWI_SEMAPHORE_FAILED       /*!< The OSAL returned an error. */
} ADI_TWI_RESULT;

/**
 * The TWI driver operating modes.
 */
typedef enum
{
    ADI_TWI_MASTER,                       /*!< Master TWI device. */
    ADI_TWI_MASTER_SCCB,                  /*!< Master TWI device with SCCB compatibility. */
    ADI_TWI_SLAVE                         /*!< Slave TWI device. */
} ADI_TWI_MODE;

/**
 * The TWI field masks of the TWI_MASTER_STAT register.
 */
typedef enum
{
	ADI_TWI_BUSBUSY  = BITM_TWI_MSTRSTAT_BUSBUSY,    /*!< Indicates whether the bus is currently busy or free. */
	ADI_TWI_SCLSEN   = BITM_TWI_MSTRSTAT_SCLSEN,     /*!< Direct sensing of the serial clock line is required. */
	ADI_TWI_SDASEN   = BITM_TWI_MSTRSTAT_SDASEN,     /*!< Direct sensing of the serial data line is required. */
	ADI_TWI_BUFWRERR = BITM_TWI_MSTRSTAT_BUFWRERR,   /*!< The current master transfer was aborted due to a receive buffer write error. */
	ADI_TWI_BUFRDERR = BITM_TWI_MSTRSTAT_BUFRDERR,   /*!< The current master transfer was aborted due to a transmit buffer read error. */
	ADI_TWI_DNAK     = BITM_TWI_MSTRSTAT_DNAK,       /*!< The current master transfer was aborted due to the detection of a NAK during data transmission. */
	ADI_TWI_ANAK     = BITM_TWI_MSTRSTAT_ANAK,       /*!< The current master transfer was aborted due to the detection of a NAK during the address phase of the transfer. */
	ADI_TWI_LOSTARB  = BITM_TWI_MSTRSTAT_LOSTARB,    /*!< The current transfer was aborted due to the loss of arbitration with another master */
	ADI_TWI_MPROG    = BITM_TWI_MSTRSTAT_MPROG       /*!< A master transfer is in progress. */
} ADI_TWI_MASTER_STATUS;

/**
 * The TWI field masks of the TWI_INT_STAT register.
 */
typedef enum
{
	ADI_TWI_ISTAT_SCLI   = BITM_TWI_ISTAT_SCLI,      /*!< The serial clock interrupt. */
	ADI_TWI_ISTAT_SDAI   = BITM_TWI_ISTAT_SDAI,      /*!< The serial data interrupt. */
	ADI_TWI_ISTAT_RXSERV = BITM_TWI_ISTAT_RXSERV,    /*!< The receive FIFO service. */
	ADI_TWI_ISTAT_TXSERV = BITM_TWI_ISTAT_TXSERV,    /*!< The transmit FIFO service. */
	ADI_TWI_ISTAT_MERR   = BITM_TWI_ISTAT_MERR,      /*!< Indicates a master transfer error occurred. */
	ADI_TWI_ISTAT_MCOMP  = BITM_TWI_ISTAT_MCOMP,     /*!< The master transfer is complete. */
	ADI_TWI_ISTAT_SOVF   = BITM_TWI_ISTAT_SOVF,      /*!< A slave overflow occurred. */
	ADI_TWI_ISTAT_SERR   = BITM_TWI_ISTAT_SERR,      /*!< A slave transfer error occurred. */
	ADI_TWI_ISTAT_SCOMP  = BITM_TWI_ISTAT_SCOMP,     /*!< The slave transfer is complete. */
	ADI_TWI_ISTAT_SINIT  = BITM_TWI_ISTAT_SINIT      /*!< A slave transfer is initiated. */
} ADI_TWI_INTERRUPT_STATUS;

/**
 * The rate at which buffer interrupts are to be generated.
 */
typedef enum
{
	ADI_TWI_ONE_BYTE_WIDTH,        /*!< Interrupt when one or two bytes in the FIFO are full. */
	ADI_TWI_TWO_BYTE_WIDTH         /*!< Interrupt when two bytes in the FIFO are full. */
}ADI_TWI_DATA_WIDTH;

/**
 * The TWI interrupt events.
 */
typedef enum
{
	ADI_TWI_EVENT_BUFFER_PROCESSED,        /*!< The buffer completed processing. */
	ADI_TWI_EVENT_XMT_PROCESSED,           /*!< A slave transmit buffer completed processing. */
	ADI_TWI_EVENT_RCV_PROCESSED,           /*!< A slave receive buffer completed processing. */
	ADI_TWI_EVENT_BUFWRERR,                /*!< The current master transfer was aborted due to a receive buffer write error. */
	ADI_TWI_EVENT_BUFRDERR,                /*!< The current master transfer was aborted due to a transmit buffer read error. */
	ADI_TWI_EVENT_DNAK,                    /*!< The current master transfer was aborted due to the detection of a NAK during data transmission. */
	ADI_TWI_EVENT_ANAK,                    /*!< The current master transfer was aborted due to the detection of a NAK during the address phase of the transfer. */
	ADI_TWI_EVENT_LOSTARB,                 /*!< The current transfer was aborted due to the loss of arbitration with another master. */
	ADI_TWI_EVENT_SERR,                    /*!< The current transfer was aborted due to a slave error. */
	ADI_TWI_EVENT_SOVF,                    /*!< The current transfer was aborted due to a slave error. */
	ADI_TWI_EVENT_SCOMP,                   /*!< The transfer is complete and either a stop, or a restart was detected. */
	ADI_TWI_EVENT_GCALL,                   /*!< A slave general call was received. */
	ADI_TWI_EVENT_RCV_NEED,                /*!< A slave receive buffer is needed. */
	ADI_TWI_EVENT_XMT_NEED                 /*!< A slave transmit buffer is needed. */
}ADI_TWI_EVENT;

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Open a TWI device instance.
 */
ADI_TWI_RESULT adi_twi_Open(
	uint32_t               const  DeviceNum,
	ADI_TWI_MODE           const  eMode,
	void                 * const  pMemory,
	uint32_t               const  MemorySize,
	ADI_TWI_HANDLE       * const  phDevice
);

/*
 * Close the given TWI device.
 */
ADI_TWI_RESULT adi_twi_Close(
    ADI_TWI_HANDLE         const hDevice
);

/*
 * Register an application defined callback function.
 */
ADI_TWI_RESULT adi_twi_RegisterCallback(
    ADI_TWI_HANDLE         const hDevice,
    ADI_CALLBACK           pfCallback,
    void                  *pCBParam);

/*
 * Write data to a TWI device (non-blocking).
 */
ADI_TWI_RESULT adi_twi_SubmitTxBuffer(
    ADI_TWI_HANDLE         const hDevice,
    void                        *pBuffer,
    uint32_t               const nBufSize,
    bool                   const bRestart
);

/*
 * Read data from a TWI device (non-blocking).
 */
ADI_TWI_RESULT adi_twi_SubmitRxBuffer(
    ADI_TWI_HANDLE         const hDevice,
    void                        *pBuffer,
    uint32_t               const nBufSize,
    bool                   const bRestart
);

/*
 * Checks if the Rx Buffer is available for processing.
 */
ADI_TWI_RESULT adi_twi_IsRxBufferAvailable(
	    ADI_TWI_HANDLE         const hDevice,
	    bool                   *pbAvailable
);

/*
 * Checks if the Tx Buffer is available for processing.
 */
ADI_TWI_RESULT adi_twi_IsTxBufferAvailable(
	    ADI_TWI_HANDLE         const hDevice,
	    bool                   *pbAvailable
);

/*
 * This will return the Rx buffer if a filled buffer is available,
 * otherwise waits until a buffer is filled.
 */
ADI_TWI_RESULT adi_twi_GetRxBuffer (
                    ADI_TWI_HANDLE   const   hDevice,
                    void                   **ppBuffer
);

/*
 * This will return the Rx buffer if a filled buffer is available,
 * otherwise waits until a buffer is filled.
 */
ADI_TWI_RESULT adi_twi_GetTxBuffer (
                    ADI_TWI_HANDLE   const   hDevice,
                    void                   **ppBuffer
);

/*
 * Enable dataflow.
 */
ADI_TWI_RESULT adi_twi_Enable (
                    ADI_TWI_HANDLE   const   hDevice
);

/*
 * Write data to a TWI device (blocking).
 */
ADI_TWI_RESULT adi_twi_Write(
    ADI_TWI_HANDLE         const hDevice,
    void                        *pBuffer,
    uint32_t               const nBufSize,
    bool                   const bRestart
);

/*
 * Read data from a TWI device (blocking).
 */
ADI_TWI_RESULT adi_twi_Read(
    ADI_TWI_HANDLE         const hDevice,
    void                        *pBuffer,
    uint32_t               const nBufSize,
    bool                   const bRestart
);

/*
 * Set the TWI prescale value.
 */
ADI_TWI_RESULT adi_twi_SetPrescale(
    ADI_TWI_HANDLE         const hDevice,
    uint16_t               const Prescale
);

/*
 * Set the TWI serial clock bit rate.
 */
ADI_TWI_RESULT adi_twi_SetBitRate(
    ADI_TWI_HANDLE         const hDevice,
    uint16_t               const BitRate
);

/*
 * Set the TWI duty cycle.
 */
ADI_TWI_RESULT adi_twi_SetDutyCycle(
    ADI_TWI_HANDLE         const hDevice,
    uint16_t               const DutyCycle
);

/*
 * Set the TWI device address.
 */
ADI_TWI_RESULT adi_twi_SetHardwareAddress(
    ADI_TWI_HANDLE         const hDevice,
    uint16_t               const Address
);

/*
 * Get the value of the interrupt status register.
 */
ADI_TWI_RESULT adi_twi_GetInterruptStatus(
    ADI_TWI_HANDLE         const hDevice,
    uint16_t                     *Value
);

/*
 * Get the value of the master mode status register.
 */
ADI_TWI_RESULT adi_twi_GetMasterStatus(
    ADI_TWI_HANDLE               const hDevice,
    uint16_t                     *Value
);

/*
 * Clear error bits in the master mode status register.
 */
ADI_TWI_RESULT adi_twi_ClearMasterStatus(
    ADI_TWI_HANDLE               const hDevice,
    uint16_t                     const Value
);

/*
 * Determines the rate at which receive buffer interrupts are to be generated.
 */
ADI_TWI_RESULT adi_twi_SetReceiveDataWidth(
    ADI_TWI_HANDLE        const hDevice,
    ADI_TWI_DATA_WIDTH    const eWidth
);

/*
 * Determines the rate at which transmit buffer interrupts are to be generated.
 */
ADI_TWI_RESULT adi_twi_SetTransmitDataWidth(
    ADI_TWI_HANDLE        const hDevice,
    ADI_TWI_DATA_WIDTH    const eWidth
);

/*
 * Enable master serial clock override.
 */
ADI_TWI_RESULT adi_twi_EnableClockOverride(
    ADI_TWI_HANDLE        const hDevice,
    bool                  const bEnable
);

/*
 * Enable master serial data override.
 */
ADI_TWI_RESULT adi_twi_EnableDataOverride(
    ADI_TWI_HANDLE        const hDevice,
    bool                  const bEnable
);

/*
 * Enable slave general call address matching.
 */
ADI_TWI_RESULT adi_twi_EnableGeneralCall(
    ADI_TWI_HANDLE        const hDevice,
    bool                  const bEnable
);

#ifdef __cplusplus
}
#endif

/**@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* __ADI_TWI_H__ */
