/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
* @file      adi_uart.h
*
* @brief     UART driver API header file
*
* @details
*            This is the primary header file for the UART driver, which
*            contains the API declarations, data and constant definitions used
*            in the APIs
*/

#ifndef _ADI_UART_BF60X_H_
#define _ADI_UART_BF60X_H_

#include <sys/platform.h>
#include <services/int/adi_int.h>
#include <services/dma/adi_dma.h>
#include <stdint.h>
#include <stdbool.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

/** @addtogroup UART_Driver_BF6xx UART Device Driver
 *  @{
 */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_1: "Allow structures and unions that are not complete, required for defining the handle")
#endif


/*==============  D E F I N E S  ===============*/

#if defined(__ADSPBLACKFIN__)

/*! Amount of memory(In bytes) required by the UART driver operating in interrupt mode
 *  and unidirection. This memory is completely owned by the driver till the end of the operation.  */
#define ADI_UART_UNIDIR_INT_MEMORY_SIZE     216u

/*! Amount of memory(In bytes) required by the UART driver operating in interrupt mode
 *  and bi-direction. This memory is completely owned by the driver till the end of the operation.  */
#define ADI_UART_BIDIR_INT_MEMORY_SIZE      360u

/*! Amount of memory(In bytes) required by the UART driver for operating in unidirection with DMA.
 *  This memory is completely owned by the driver till the end of the operation. */
#define ADI_UART_UNIDIR_DMA_MEMORY_SIZE     332u

/*! Amount of memory(In bytes) required by the UART driver for operating in bidirection with DMA.
 *  This memory is completely owned by the driver till the end of the operation. */
#define ADI_UART_BIDIR_DMA_MEMORY_SIZE      592u

#elif defined(__ICCARM__)

#define ADI_UART_UNIDIR_INT_MEMORY_SIZE     204u
#define ADI_UART_BIDIR_INT_MEMORY_SIZE      344u
#define ADI_UART_UNIDIR_DMA_MEMORY_SIZE     320u
#define ADI_UART_BIDIR_DMA_MEMORY_SIZE      576u

#elif defined (__CC_ARM)                    // KEIL: used same settings as for Blackfin (largest)
#define ADI_UART_UNIDIR_INT_MEMORY_SIZE     216u
#define ADI_UART_BIDIR_INT_MEMORY_SIZE      360u
#define ADI_UART_UNIDIR_DMA_MEMORY_SIZE     332u
#define ADI_UART_BIDIR_DMA_MEMORY_SIZE      592u

#else
#error "Unknown compiler type"
#endif

/*! UART device handle */
typedef struct _ADI_UART_DEVICE_* ADI_UART_HANDLE;



/*! Enumeration of result codes generated by UART driver.
 *
 *  These return codes may have different meaning with respect to different
 *  API. Please refer the description of each API for the description of the
 *  return codes with respect to that particular API.
 *
 * */
typedef enum
{
    /*! Success */
    ADI_UART_SUCCESS,

    /*! Generic failure. */
    ADI_UART_FAILURE,

    /*! The given device instance number is invalid. */
    ADI_UART_INVALID_DEVICE_NUM,

    /*! The given UART handle is NULL or invalid. */
    ADI_UART_INVALID_HANDLE,

    /*! The given UART device is not initialized */
    ADI_UART_NOT_OPENED,

    /*! The given UART device instance is already opened. */
    ADI_UART_ALREADY_OPENED,

    /*! The given pointer is invalid or NULL. */
    ADI_UART_NULL_POINTER,

    /*! The given parameter is not in the valid range. */
    ADI_UART_INVALID_PARAMETER,

    /*! The given amount memory is insufficient for the given mode of operation. */
    ADI_UART_INSUFFICIENT_MEMORY,

    /*! DMA related failure occurred. */
    ADI_UART_DMA_FAILED,

    /*! Failed to register or unregister an interrupt handler with interrupt service. */
    ADI_UART_INT_FAILED,

    /*! Semaphore related failure occurred */
    ADI_UART_SEM_FAILED,

    /*! Failed to configure the GP timer for Autobaud detection. */
    ADI_UART_TMR_FAILED,

    /*! Failed to get the system clock frequency. */
    ADI_UART_GET_SYSCLK_FAILED,

    /*! Operation not allowed in the current state of the driver. */
    ADI_UART_OPERATION_NOT_ALLOWED,

    /*! Failed to submit the given buffer. */
    ADI_UART_SUBMIT_FAILED,

    /*! When trying to enable the data flow before submitting the initial
     *  two buffers  */
    ADI_UART_BUFFERS_NOT_SUBMITTED,

    /*! When trying to enable the data flow without setting the baud rate */
    ADI_UART_BAUD_RATE_NOT_SET,

    /*! When trying to call the APIs which are related to a particular
     *  direction, but the device in not opened in that direction or
     *  in bidirectional mode.
     */
    ADI_UART_NOT_OPENED_IN_THIS_DIR,

    /*! When a buffer is submitted using a non-bloking adi_uart_SubmitRx/TxBuffer,
     *  when a blocking Read/Write call is in progress or a Blocking adi_uart_Read/Write
     *  call is made when a non-blocking buffer transaction is in progress.
     */
    ADI_UART_INVALID_DATA_TRANSFER_MODE,

    /*! A hardware error was detected. The API adi_uart_GetHWErrorStatus can be used
     *  to know the hardware errors that were detected. If the hardware errors are
     *  not retrieved they will be overwritten by new hardware errors. */
    ADI_UART_HW_ERROR

} ADI_UART_RESULT;


/*! Enumeration of data flow directions, can only be chosen when opening
 *  the driver.
 */
typedef enum
{
    ADI_UART_DIR_TRANSMIT   =   0,                      /*!< Transmit only. */
    ADI_UART_DIR_RECEIVE,                               /*!< Receive only. */
    ADI_UART_DIR_BIDIRECTION                            /*!< Both Transmit and Receive. */

} ADI_UART_DIRECTION;

/*! Enumeration for the UART DMA word transfer size */
typedef enum
{
    ADI_UART_DMA_TRANSFER_1BYTE   = ADI_DMA_MSIZE_1BYTE,           /*!<  8  bit transfer size  */
    ADI_UART_DMA_TRANSFER_2BYTES  = ADI_DMA_MSIZE_2BYTES,          /*!<  16  bit transfer size  */
    ADI_UART_DMA_TRANSFER_4BYTES  = ADI_DMA_MSIZE_4BYTES,          /*!<  32  bit transfer size  */
    ADI_UART_DMA_TRANSFER_8BYTES  = ADI_DMA_MSIZE_8BYTES,          /*!<  64  bit transfer size  */
    ADI_UART_DMA_TRANSFER_16BYTES = ADI_DMA_MSIZE_16BYTES,         /*!<  128 bit transfer size  */
    ADI_UART_DMA_TRANSFER_32BYTES = ADI_DMA_MSIZE_32BYTES          /*!<  256 bit transfer size  */

}ADI_UART_DMA_TRANSFER_SIZE;


/*! Enumeration of modes that UART can operate */
typedef enum
{
    ADI_UART_MODE_UART   =  ENUM_UART_CTL_UART_MODE,    /*!< Regular UART Mode. */
    ADI_UART_MODE_MDB    =  ENUM_UART_CTL_MDB_MODE,     /*!< Multi Drop Bus Mode. */
    ADI_UART_MODE_IRDA   =  ENUM_UART_CTL_IRDA_MODE     /*!< SIR IrDA mode. */

} ADI_UART_MODE;

/*! Enumeration of data width */
typedef enum
{
    ADI_UART_WORDLEN_5BITS = ENUM_UART_CTL_WL5BITS,     /*!< 5 Bits wide. */
    ADI_UART_WORDLEN_6BITS = ENUM_UART_CTL_WL6BITS,     /*!< 6 Bits Wide. */
    ADI_UART_WORDLEN_7BITS = ENUM_UART_CTL_WL7BITS,     /*!< 7 Bits Wide. */
    ADI_UART_WORDLEN_8BITS = ENUM_UART_CTL_WL8BITS      /*!< 8 Bits Wide. */

} ADI_UART_WORDLEN;

/*! Enumeration of parity modes */
typedef enum
{
    ADI_UART_NO_PARITY   =   0,                         /*!< No parity. */
    ADI_UART_ODD_PARITY,                                /*!< Odd parity. */
    ADI_UART_EVEN_PARITY,                               /*!< Even Parity. */
    ADI_UART_EVEN_PARITY_STICKY,                        /*!< Sticky Even parity. */
    ADI_UART_ODD_PARITY_STICKY                          /*!< Sticky Odd parity. */

} ADI_UART_PARITY;

/*! Enumeration of number of stop bits */
typedef enum
{
    ADI_UART_ONE_STOPBIT = 0,                           /*!< One Stop bit. */
    ADI_UART_ONE_AND_HALF_STOPBITS,                     /*!< 0ne and half Stop bits. */
    ADI_UART_TWO_STOPBITS,                              /*!< Two Stop bits. */
    ADI_UART_TWO_AND_HALF_STOPBITS                      /*!< Two and Half Stop bits. */

} ADI_UART_STOPBITS;


/*! @brief Enumeration of watermark levels on Rx FIFO
 *
 *  When water mark level is set an event is notified
 *  after Rx FIFO reaches the water mark level */
typedef enum
{
    ADI_UART_RX_FIFO_WM_4WORDS =  0,                    /*!< Event notified when number of words >= 4. */
    ADI_UART_RX_FIFO_WM_7WORDS                          /*!< Event notified when number of words >= 7. */

}ADI_UART_RX_FIFO_WM;

/*! Enumeration of events generated by UART driver */
typedef enum
{
    ADI_UART_EVENT_OVERRUN_ERROR        =   0x0001,     /*!< Receive overrun. */
    ADI_UART_EVENT_PARITY_ERROR         =   0x0002,     /*!< Detected Rx parity error. */
    ADI_UART_EVENT_FRAMING_ERROR        =   0x0004,     /*!< Detected Rx framing error. */
    ADI_UART_EVENT_BREAK_INTERRUPT      =   0x0008,     /*!< Detected break condition. */
    ADI_UART_EVENT_RX_FIFO_WM           =   0x0010,     /*!< Reached Rx FIFO watermark level. */
    ADI_UART_EVENT_RECIEVED_ADDR        =   0x0020,     /*!< Received Address (valid only in MDB mode). */
    ADI_UART_EVENT_AUTOBAUD_COMPLETE    =   0x0040,     /*!< The initiated autobuad is complete. */
    ADI_UART_EVENT_TX_COMPLETE          =   0x0080,     /*!< Completed sending out the data from
                                                             device buffers.*/

    ADI_UART_EVENT_RX_BUFFER_PROCESSED  =   0x0100,     /*!< The given Rx buffer is processed. */
    ADI_UART_EVENT_TX_BUFFER_PROCESSED  =   0x0200,     /*!< The given Tx buffer is processed. */
    ADI_UART_EVENT_RX_DMA_ERROR         =   0x1000,     /*!< DMA error occurred in Rx channel. */
    ADI_UART_EVENT_TX_DMA_ERROR         =   0x2000      /*!< DMA error occurred in Tx channel. */



}ADI_UART_EVENT;


/*! Enumeration of different Hardware errors */
typedef enum
{
    /*! No hardware errors detected. */
    ADI_UART_HW_ERR_NONE        =   0x0,

    /*! Receive overrun detected.*/
    ADI_UART_HW_ERR_OVERRUN     =   ADI_UART_EVENT_OVERRUN_ERROR,

    /*! Detected Rx parity error. */
    ADI_UART_HW_ERR_PARITY      =   ADI_UART_EVENT_PARITY_ERROR,

    /*! Detected Rx framing error. */
    ADI_UART_HW_ERR_FRAMING     =   ADI_UART_EVENT_FRAMING_ERROR,

    /*! Detected break condition. */
    ADI_UART_HW_ERR_BREAK       =   ADI_UART_EVENT_BREAK_INTERRUPT,

    /*! Rx DMA error occurred. */
    ADI_UART_HW_ERR_RX_DMA      =   ADI_UART_EVENT_RX_DMA_ERROR,

    /*! Tx DMA error occurred. */
    ADI_UART_HW_ERR_TX_DMA      =   ADI_UART_EVENT_TX_DMA_ERROR

}ADI_UART_HW_ERR;


/**
 * Enumeration of conditions on which an UART device can issue an outgoing Trigger
 */
typedef enum
{
    ADI_UART_TRIGGER_OUT_NONE        = ADI_DMA_TRIGGER_OUT_NONE,         /*!< No trigger (default) */
    ADI_UART_TRIGGER_OUT_DMA_DONE    = ADI_DMA_TRIGGER_OUT_XCNT_EXPIRY   /*!< Generate trigger when DMA is done. */
} ADI_UART_TRIGGER_OUT;



#ifdef __cplusplus 
extern "C" { 
#endif /* __cplusplus */ 

ADI_UART_RESULT adi_uart_Open(
    uint32_t                      nDeviceNum,
    ADI_UART_DIRECTION            eDirection,
    void                   *const pMemory,
    uint32_t                      nMemSize,
    ADI_UART_HANDLE        *const phDevice
);

ADI_UART_RESULT adi_uart_Close(
    ADI_UART_HANDLE         const hDevice
);

ADI_UART_RESULT adi_uart_SubmitTxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
);

ADI_UART_RESULT adi_uart_SubmitRxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
);

ADI_UART_RESULT adi_uart_GetTxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                  **const ppBuffer
);

ADI_UART_RESULT adi_uart_GetRxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                  **const ppBuffer
);

ADI_UART_RESULT adi_uart_Read(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
);

ADI_UART_RESULT adi_uart_Write(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
);

ADI_UART_RESULT adi_uart_IsTxBufferAvailable (
    ADI_UART_HANDLE         const hDevice,
    bool                   *const pbAvailable
);

ADI_UART_RESULT adi_uart_IsRxBufferAvailable (
    ADI_UART_HANDLE         const hDevice,
    bool                   *const pbAvailable
);

ADI_UART_RESULT adi_uart_EnableTx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
);

ADI_UART_RESULT adi_uart_EnableRx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
);

ADI_UART_RESULT adi_uart_GetHWErrorStatus(
    ADI_UART_HANDLE         const hDevice,
    uint32_t               *const pHWError
);

ADI_UART_RESULT  adi_uart_ConfigRxTriggerIn (
    ADI_UART_HANDLE const       hDevice,
    bool                        bTriggerWait,
    bool                        bTriggerOvError
);

ADI_UART_RESULT  adi_uart_ConfigTxTriggerIn (
    ADI_UART_HANDLE const       hDevice,
    bool                        bTriggerWait,
    bool                        bTriggerOvError
);

ADI_UART_RESULT  adi_uart_ConfigRxTriggerOut (
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_TRIGGER_OUT        eUartTrigger
);

ADI_UART_RESULT  adi_uart_ConfigTxTriggerOut (
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_TRIGGER_OUT        eUartTrigger
);

ADI_UART_RESULT  adi_uart_SetRxDmaTransferSize(
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_DMA_TRANSFER_SIZE  eTransferSize
);

ADI_UART_RESULT  adi_uart_SetTxDmaTransferSize(
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_DMA_TRANSFER_SIZE  eTransferSize
);

ADI_UART_RESULT adi_uart_RegisterCallback(
    ADI_UART_HANDLE         const hDevice,
    ADI_CALLBACK            const pfCallback,
    void                   *const pCBParam
);

ADI_UART_RESULT adi_uart_SetBaudRate(
    ADI_UART_HANDLE         const hDevice,
    uint32_t                      BaudRate
);

ADI_UART_RESULT adi_uart_ConfigAutobaud(
    ADI_UART_HANDLE         const hDevice,
    uint8_t                       AutoBaudChar
);

ADI_UART_RESULT adi_uart_EnableAutobaud(
   ADI_UART_HANDLE          const hDevice,
   bool                           bEnable
);

ADI_UART_RESULT adi_uart_GetBaudRate(
    ADI_UART_HANDLE         const hDevice,
    uint32_t *              const pnBaudRate
);


ADI_UART_RESULT adi_uart_SetMode(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_MODE                 eMode
);

ADI_UART_RESULT adi_uart_EnableLineStatusEvents(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
);

ADI_UART_RESULT adi_uart_MDBTxAddress(
   ADI_UART_HANDLE         const hDevice,
   uint8_t                       Address
);

ADI_UART_RESULT adi_uart_MDBRxAddress(
   ADI_UART_HANDLE         const hDevice
);

ADI_UART_RESULT adi_uart_MDBDiscardRxData(
   ADI_UART_HANDLE         const hDevice
);

ADI_UART_RESULT adi_uart_EnableLoopBack(
   ADI_UART_HANDLE         const hDevice,
   bool                          bEnable
);

ADI_UART_RESULT adi_uart_SetNumStopBits(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_STOPBITS             eStopBits
);

ADI_UART_RESULT adi_uart_SetWordLen(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_WORDLEN              eWordLen
);

ADI_UART_RESULT adi_uart_SetParity(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_PARITY               eParity
);

ADI_UART_RESULT adi_uart_ForceTxBreak(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
);

ADI_UART_RESULT adi_uart_SetFCPolarity(
    ADI_UART_HANDLE         const hDevice,
    bool                          bActiveHigh
);

ADI_UART_RESULT adi_uart_SetRxPolarity(
    ADI_UART_HANDLE         const hDevice,
    bool                          bActiveHigh
);

ADI_UART_RESULT adi_uart_SetTxPolarity(
    ADI_UART_HANDLE         const hDevice,
    bool                          bActiveHigh
);

ADI_UART_RESULT adi_uart_PauseRx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bPause
);

ADI_UART_RESULT adi_uart_PauseTx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bPause
);

ADI_UART_RESULT adi_uart_EnableTxHWFlowCntl(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
);

ADI_UART_RESULT adi_uart_EnableRxHWFlowCntl(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
);

ADI_UART_RESULT adi_uart_SetRxFifoWM(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_RX_FIFO_WM           eFifoWM
);

ADI_UART_RESULT adi_uart_EnableDMAMode (
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
);

#ifdef __cplusplus 
}
#endif /* __cplusplus */ 

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif /* _ADI_UART_BF60X_H_ */
