/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_mdma_bf60x.h
 * @brief:   Memory DMA Manager definitions and API for ADSP-BF60x processor family
 * @version: $Revision: 18484 $
 * @date:    $Date: 2013-04-26 10:04:51 -0400 (Fri, 26 Apr 2013) $
 *
 * @details
 *           This is the primary header file for the Memory DMA Manager service
 *           for ADSP-BF60x processor family, which contains the API declarations,
 *           data and constant definitions used in the APIs
 */

/** @defgroup DMA_Manager_bf60x DMA Manager
 *  @defgroup MDMA_Manager_bf60x Memory DMA Manager
 *  @ingroup DMA_Manager_bf60x
 */


#ifndef __ADI_MDMA_BF60X_H__
#define __ADI_MDMA_BF60X_H__

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_1:"Allow for forward declaration of handles")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/

/* Include Memory DMA Stream ID definitions for different processors */
#if defined(__ADSPBF609_FAMILY__)
#include <services/dma/adi_mdma_streams_bf60x.h>
#elif defined( ADI_ADSP_CM40Z)
#include <services/dma/adi_mdma_streams_cm40z.h>
#elif defined(__ADSPBF707_FAMILY__)
#include <services/dma/adi_mdma_streams_bf70x.h>
#else
#error "Processor not supported"
#endif

/*==============  D E F I N E S  ===============*/

#if !defined(_LANGUAGE_ASM)

/** @ingroup MDMA_Manager_bf60x
 *  Memory size (in bytes) required to handle a Memory DMA Stream
 */
#define ADI_DMA_STREAM_REQ_MEMORY         (160u)

/** @ingroup PDMA_Manager_bf60x
 *  Peripheral DMA Channel handle
 */
typedef void* ADI_DMA_CHANNEL_HANDLE;
/** @ingroup MDMA_Manager_bf60x
 *  Memory DMA Stream Handle
 */
typedef void* ADI_DMA_STREAM_HANDLE;

/** @ingroup MDMA_Manager_bf60x
 * Enumeration to enable Trigger Input configuration for a
 * Memory DMA channel operated in List or Array mode
 */
#define ADI_DMA_TRIGGER_IN_EN               (ENUM_DMA_CFG_TRGWAIT)

/** @ingroup MDMA_Manager_bf60x
 * Enumeration to enable Trigger overrun error configuration for a
 * Memory DMA channel operated in List or Array mode
 */
#define ADI_DMA_TRIGGER_IN_OVERRUN_EN       (ENUM_DMA_CFG_TOV_EN)

/** @ingroup DMA_Manager_bf60x
 * Enumeration of conditions on which a DMA channel can issue an outgoing Trigger
 */
typedef enum
{
    ADI_DMA_TRIGGER_OUT_NONE        = ENUM_DMA_CFG_NO_TRIG,    /*!< No trigger (default) */
    ADI_DMA_TRIGGER_OUT_XCNT_EXPIRY = ENUM_DMA_CFG_XCNT_TRIG,  /*!< Generate trigger when X Count reaces 0 */
    ADI_DMA_TRIGGER_OUT_YCNT_EXPIRY = ENUM_DMA_CFG_YCNT_TRIG   /*!< Generate trigger when Y Count reaches 0 */
} ADI_DMA_TRIGGER_OUT;

/** @ingroup PDMA_Manager_bf60x
 * Peripheral Transfer word size supported by the DMA channels
 */
typedef enum
{
    ADI_DMA_PSIZE_1BYTE   = ENUM_DMA_CFG_PSIZE01, /*!< 1  byte transfer  (8   bits) */
    ADI_DMA_PSIZE_2BYTES  = ENUM_DMA_CFG_PSIZE02, /*!< 2  bytes transfer (16  bits) */
    ADI_DMA_PSIZE_4BYTES  = ENUM_DMA_CFG_PSIZE04, /*!< 4  bytes transfer (32  bits) */
    ADI_DMP_PSIZE_8BYTES  = ENUM_DMA_CFG_PSIZE08  /*!< 8  bytes transfer (64  bits) */
} ADI_DMA_PSIZE;

/** @ingroup DMA_Manager_bf60x
 * Memory Transfer word size supported by the DMA channels
 */
typedef enum
{
    ADI_DMA_MSIZE_1BYTE   = ENUM_DMA_CFG_MSIZE01, /*!< 1  byte transfer  (8   bits) */
    ADI_DMA_MSIZE_2BYTES  = ENUM_DMA_CFG_MSIZE02, /*!< 2  bytes transfer (16  bits) */
    ADI_DMA_MSIZE_4BYTES  = ENUM_DMA_CFG_MSIZE04, /*!< 4  bytes transfer (32  bits) */
    ADI_DMA_MSIZE_8BYTES  = ENUM_DMA_CFG_MSIZE08, /*!< 8  bytes transfer (64  bits) */
    ADI_DMA_MSIZE_16BYTES = ENUM_DMA_CFG_MSIZE16, /*!< 16 bytes transfer (128 bits) */
    ADI_DMA_MSIZE_32BYTES = ENUM_DMA_CFG_MSIZE32  /*!< 32 bytes transfer (256 bits) */
} ADI_DMA_MSIZE;

/** @ingroup MDMA_Manager_bf60x
 * Structure to handle List Descriptor transfers
 */
#pragma pack(4)
typedef struct __AdiDmaDescListMode
{
    uint32_t                        Reserved;           /*!< Reserved for DMA Manager */
    void                            *pStartAddress;     /*!< Start address of the data buffer to be processed */
    uint32_t                        Config;             /*!< DMA Configuration register - Specify Transfer word size (MSIZE/PSIZE) and Trigger configuration */
    uint32_t                        XCount;             /*!< Inner loop count start value in number of DMA data bus width (MSIZE) transfers */
    int32_t                         XModify;            /*!< Inner loop address increment in bytes */
    uint32_t                        YCount;             /*!< Outer loop count start value in number of rows */
    int32_t                         YModify;            /*!< Outer loop address increment in bytes */
    struct __AdiDmaDescListMode   	*pNext;             /*!< Next list descriptor work unit in the chain */
    bool                            bCallbackWhenDone;  /*!< TRUE to notify application via callback on completion of this work unit */
} ADI_DMA_DESC_LIST_MODE;
#pragma pack()

/** @ingroup MDMA_Manager_bf60x
 * Structure to handle 1D Array Descriptor transfers
 */
#pragma pack(4)
typedef struct
{
    void        *pStartAddress; /*!< Start address of the data buffer to be processed */
    uint32_t    Config;         /*!< DMA Configuration register - Specify Transfer word size (MSIZE/PSIZE) and Trigger configuration */
    uint32_t    XCount;         /*!< Inner loop count start value in number of DMA data bus width (MSIZE) transfers */
    int32_t     XModify;        /*!< Inner loop address increment in bytes */
} ADI_DMA_DESC_ARRAY_1D_MODE;
#pragma pack()

/** @ingroup MDMA_Manager_bf60x
 * Structure to handle 2D Array Descriptor transfers
 */
#pragma pack(4)
typedef struct
{
    void        *pStartAddress; /*!< Start address of the data buffer to be processed */
    uint32_t    Config;         /*!< DMA Configuration register - Specify Transfer word size (MSIZE/PSIZE) and Trigger configuration */
    uint32_t    XCount;         /*!< Inner loop count start value in number of DMA data bus width (MSIZE) transfers */
    int32_t     XModify;        /*!< Inner loop address increment in bytes */
    uint32_t    YCount;         /*!< Outer loop count start value in number of rows */
    int32_t     YModify;        /*!< Outer loop address increment in bytes */
} ADI_DMA_DESC_ARRAY_2D_MODE;
#pragma pack()

/** @ingroup MDMA_Manager_bf60x
 * Structure to handle 2D One-shot memory transfers
 */
typedef struct
{
    void        *pStartAddress; /*!< Start address of the data buffer to be processed */
    uint32_t    XCount;         /*!< X (column) count */
    int32_t     XModify;        /*!< X (column) stride */
    uint32_t    YCount;         /*!< Y (row) count */
    int32_t     YModify;        /*!< Y (row) stride */
} ADI_DMA_2D_MEM_TRANSFER;

/** @ingroup DMA_Manager_bf60x
 * DMA Event codes posted with the callback function
 */
typedef enum
{
    ADI_DMA_EVENT_BUFFER_PROCESSED,         /*!< Processed a whole (One-shot) buffer
                                                 Applies for both Peripheral DMA and Memory DMA
                                                 (pArg = Start address of the processed buffer) */
    ADI_DMA_EVENT_SUB_BUFFER_PROCESSED,     /*!< Processed a Sub-buffer within a Circular buffer
                                                 Applies for Peripheral DMA only
                                                 (pArg = Circular buffer start address to which the sub-buffer belongs) */
    ADI_DMA_EVENT_DESC_LIST_PROCESSED,     	/*!< Processed a Descriptor List
                                                 Applies for Memory DMA only
                                                 (pArg = pointer to Descriptor list processed) */
    ADI_DMA_EVENT_DESC_1D_ARRAY_PROCESSED,  /*!< Processed a 1D Descriptor array
                                                 Applies for Memory DMA only
                                                 (pArg = pointer to 1D Descriptor array processed) */
    ADI_DMA_EVENT_DESC_2D_ARRAY_PROCESSED,  /*!< Processed a 2D Descriptor array
                                                 Applies for Memory DMA only
                                                 (pArg = pointer to 2D Descriptor array processed) */
    ADI_DMA_EVENT_ERROR_INT                 /*!< DMA error interrupt asserted
                                                 Applies for both Peripheral DMA and Memory DMA
                                                 (pArg = Error cause field value of DMA status register) */
}ADI_DMA_EVENT;

/** @ingroup DMA_Manager_bf60x
 * DMA API return codes
 */
typedef enum
{
    ADI_DMA_SUCCESS = 0,                /*!< 0x00 - Generic success */
    ADI_DMA_FAILURE,                    /*!< 0x01 - Generic failure */
    ADI_DMA_IN_USE,                     /*!< 0x02 - Supplied DMA channel/stream ID is already open and in use */
    ADI_DMA_NOT_OPEN,                   /*!< 0x03 - Channel is not open to preform the requested task */
    ADI_DMA_ALREADY_RUNNING,            /*!< 0x04 - Function not permitted since DMA is already running */
    ADI_DMA_FN_NOT_SUPPORTED,           /*!< 0x05 - Function not supported */
    ADI_DMA_FN_NOT_PERMITTED,           /*!< 0x06 - Function not permitted at current stage */
    ADI_DMA_BAD_HANDLE,                 /*!< 0x07 - Bad channel/stream handle */
    ADI_DMA_BAD_CHANNEL_ID,             /*!< 0x08 - No such channel ID */
    ADI_DMA_BAD_STREAM_ID,              /*!< 0x09 - No such memory stream ID */
    ADI_DMA_BAD_MSIZE,                  /*!< 0x0A - Bad memory transfer size */
    ADI_DMA_BAD_PSIZE,                  /*!< 0x0B - Bad peripheral transfer size */
    ADI_DMA_BAD_XCOUNT,                 /*!< 0x0C - An invalid XCount value was supplied */
    ADI_DMA_BAD_YCOUNT,                 /*!< 0x0D - An invalid YCount value was supplied */
    ADI_DMA_BAD_CIRC_BUFFER,           	/*!< 0x0E - An invalid circular buffer configuration was supplied */
    ADI_DMA_BAD_START_ADDR,             /*!< 0x0F - Supplied start address in invalid */
    ADI_DMA_WORK_UNIT_FULL,				/*!< 0x10 - Can't submit buffer as the DMA work units are full */
    ADI_DMA_NO_BUFFER_TO_PROCESS,       /*!< 0x11 - Can't start DMA data transfer as there is no valid buffer to process */
    ADI_DMA_NO_VALID_CALLBACK_FN,		/*!< 0x12 - A valid Callback function is not supplied */
    ADI_DMA_BUFFER_TYPE_MISMATCH,		/*!< 0x13 - Can't submit buffer as the DMA is set for circular data transfer */
    ADI_DMA_ADDR_ALIGNEMNT_ERROR,       /*!< 0x14 - Start address alignment error */
    ADI_DMA_INT_REGISTER_FAILED,        /*!< 0x15 - Can't register an interrupt */
    ADI_DMA_INT_UNREGISTER_FAILED,      /*!< 0x16 - Can't unregister an interrupt */
    ADI_DMA_XFER_TYPE_INVALID,          /*!< 0x17 - Data transfer type of source and/or destination is invalid */
    ADI_DMA_XFER_TYPE_ALREADY_SET       /*!< 0x18 - DMA Transfer type already configured */
} ADI_DMA_RESULT;

/*=======  P U B L I C   P R O T O T Y P E S  ========*/
/*            (globally-scoped functions)             */

#ifdef __cplusplus
extern "C" {
#endif

/* Opens a Memory DMA stream for use */
ADI_DMA_RESULT adi_mdma_Open(
    ADI_DMA_STREAM_ID               eStreamID,
    void                            *pStreamMemory,
    ADI_DMA_STREAM_HANDLE           *phStream,
    ADI_DMA_CHANNEL_HANDLE          *phSrcChannel,
    ADI_DMA_CHANNEL_HANDLE          *phDestChannel,
    ADI_CALLBACK                    pfCallback,
    void                            *pCBParam);

/* Closes a Memory DMA stream */
ADI_DMA_RESULT adi_mdma_Close(
    ADI_DMA_STREAM_HANDLE           hStream,
    bool                            bWaitFlag);

/* Gets the current data copy status of a MDMA stream */
ADI_DMA_RESULT adi_mdma_IsCopyInProgress(
    ADI_DMA_STREAM_HANDLE           hStream,
    bool                            *pbCopyInProgress);

/* Disables/terminates Memory DMA copy that is currently in progress */
ADI_DMA_RESULT adi_mdma_Disable(
    ADI_DMA_STREAM_HANDLE           hStream);

/* Sets the Source or Destination channel of the MDMA stream for
   continuous data transfer from a Memory Mapped location or a
   single data buffer */
ADI_DMA_RESULT adi_mdma_CopyCircular1D(
    ADI_DMA_STREAM_HANDLE           hStream,
    void                            *pAddress,
    uint32_t                        ElementCount,
    ADI_DMA_MSIZE                   eElementWidth,
    bool                            bUseSrcChannel);

/* Copies data between memory in a 1D fashion */
ADI_DMA_RESULT adi_mdma_Copy1D(
    ADI_DMA_STREAM_HANDLE           hStream,
    void                            *pMemDest,
    void                            *pMemSrc,
    ADI_DMA_MSIZE                   eElementWidth,
    uint32_t                        ElementCount);

/* Copies data between memory in a 2D fashion */
ADI_DMA_RESULT adi_mdma_Copy2D(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_MSIZE                   eElementWidth,
    ADI_DMA_2D_MEM_TRANSFER         *pDestDesc,
    ADI_DMA_2D_MEM_TRANSFER         *pSrcDesc);

/* Copies data between memory using Descriptor List */
ADI_DMA_RESULT adi_mdma_CopyQueueList(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_LIST_MODE          *pDestDesc,
    ADI_DMA_DESC_LIST_MODE          *pSrcDesc);

/* Copies data between memory using 1D Descriptor Array */
ADI_DMA_RESULT adi_mdma_Copy1DArray(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pDestDesc,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pSrcDesc,
    uint16_t					    NumEntries);

/* Copies data between memory using 2D Descriptor Array */
ADI_DMA_RESULT adi_mdma_Copy2DArray(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pDestDesc,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pSrcDesc,
    uint16_t					    NumEntries);

/* Configures Trigger Input settings for a DMA channel */
ADI_DMA_RESULT  adi_dma_ConfigTriggerIn (
    ADI_DMA_CHANNEL_HANDLE      hChannel,
    bool                        bTriggerWait,
    bool                        bTriggerOvError);

/* Configures Trigger Output settings for a DMA channel */
ADI_DMA_RESULT  adi_dma_ConfigTriggerOut (
    ADI_DMA_CHANNEL_HANDLE      hChannel,
    ADI_DMA_TRIGGER_OUT         eDmaTrigger);

/* Update DMA Channel callback function */
ADI_DMA_RESULT adi_dma_UpdateCallback(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    ADI_CALLBACK                    pfCallback,
    void *const                     pCBParam);

/* Gets Current Descriptor pointer register value */
ADI_DMA_RESULT adi_dma_GetCurrentDesc(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    void                            **pCurrentDesc);

/* Gets Address Current Register value */
ADI_DMA_RESULT adi_dma_GetCurrentAddr(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    void                            **pCurrentAddr);

/* Gets Current Inner loop count (current X count) Register value */
ADI_DMA_RESULT adi_dma_GetCurrentXCount(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    uint32_t                        *pCurrentXCount);

/* Gets Current Inner loop count (current Y count) Register value */
ADI_DMA_RESULT adi_dma_GetCurrentYCount(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    uint32_t                        *pCurrentYCount);

#ifdef __cplusplus
}
#endif

#endif /*!defined(_LANGUAGE_ASM) */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif  /* __ADI_MDMA_BF60X_H__ */

/*****/

