/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_pdma_bf60x.h
 * @brief:   Peripheral DMA Manager definitions and API for ADSP-BF60x processor family
 * @version: $Revision: 18484 $
 * @date:    $Date: 2013-04-26 10:04:51 -0400 (Fri, 26 Apr 2013) $
 *
 * @details
 *           This is the primary header file for the Peripheral DMA Manager service
 *           for ADSP-BF60x processor family, which contains the API declarations,
 *           data and constant definitions used in the APIs
 */

/** @defgroup PDMA_Manager_bf60x Peripheral DMA Manager
 *  @ingroup DMA_Manager_bf60x
 */

#ifndef __ADI_PDMA_BF60X_H__
#define __ADI_PDMA_BF60X_H__

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_1:"Allow for forward declaration of handles")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/
/* Include Peripheral DMA Channel ID definitions */
#if defined(__ADSPBF609_FAMILY__)
#include <services/dma/adi_pdma_channels_bf60x.h>
#elif defined (ADI_ADSP_CM40Z)
#include <services/dma/adi_pdma_channels_cm40z.h>
#elif defined(__ADSPBF707_FAMILY__)
#include <services/dma/adi_pdma_channels_bf70x.h>
#else
#error "Processor not supported"
#endif
/*==============  D E F I N E S  ===============*/

#if !defined(_LANGUAGE_ASM)

/** @ingroup PDMA_Manager_bf60x
 * Memory size (in bytes) required to handle a Peripheral DMA channel
 */
#define ADI_DMA_CHANNEL_REQ_MEMORY         (116u)

/*=======  P U B L I C   P R O T O T Y P E S  ========*/
/*            (globally-scoped functions)             */

#ifdef __cplusplus
extern "C" {
#endif

/* APIs specific to Peripheral DMA channels */

/* Opens a peripheral DMA channel for use */
ADI_DMA_RESULT adi_pdma_Open(
    ADI_DMA_CHANNEL_ID              eChannelID,
    void *const                     pChannelMemory,
    ADI_DMA_CHANNEL_HANDLE *const   phChannel,
    ADI_CALLBACK                    pfCallback,
    void *const                     pCBParam);

/* Closes a peripheral DMA channel */
ADI_DMA_RESULT adi_pdma_Close(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bWaitFlag);

/* Enables/Disables Peripheral data streaming */
ADI_DMA_RESULT adi_pdma_EnableStreaming(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable);

/* Enables/Disables buffer repetition */
ADI_DMA_RESULT adi_pdma_EnableBufferRepeat(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable);

/* Configure DMA Sync field in DMA Configuration register */
ADI_DMA_RESULT adi_pdma_SetSync(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable);

/* Configures Peripheral interrupt forward settings */
ADI_DMA_RESULT adi_pdma_SetPeripheralIntFwd(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable);

/* Configures Peripheral data request forward settings */
ADI_DMA_RESULT adi_pdma_SetPeripheralReqFwd(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable);

/* Configures Peripheral DMA Memory Transfer Word Size */
ADI_DMA_RESULT adi_pdma_SetMSize(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    ADI_DMA_MSIZE                   eMSize);

/* Configures DMA Peripheral Transfer word size */
ADI_DMA_RESULT adi_pdma_SetPSize(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    ADI_DMA_PSIZE                   ePSize);

/* Enable dataflow (data transfer) of a DMA channel */
ADI_DMA_RESULT adi_pdma_EnableXfer(
    ADI_DMA_CHANNEL_HANDLE const    hChannel);

/* Disable dataflow (data transfer) of a DMA channel */
ADI_DMA_RESULT adi_pdma_DisableXfer(
    ADI_DMA_CHANNEL_HANDLE const    hChannel);

/* Submits a buffer to Peripheral DMA to write to Peripheral FIFO */
ADI_DMA_RESULT adi_pdma_PeripheralWrite(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        XCount,
    int32_t                         XModify,
    uint32_t                        YCount,
    int32_t                         YModify);

/* Submits a buffer to Peripheral DMA to read from Peripheral FIFO */
ADI_DMA_RESULT adi_pdma_PeripheralRead(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        XCount,
    int32_t                         XModify,
    uint32_t                        YCount,
    int32_t                         YModify);

/* Updates Start Address and XCount of a Descriptor work unit that is currently not active */
ADI_DMA_RESULT adi_pdma_UpdateDescriptor(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pNewStartAddr,
    uint32_t                        XCount);

/* Submits a circular buffer to Peripheral DMA to write to Peripheral FIFO */
ADI_DMA_RESULT adi_pdma_PeripheralCircWrite(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        NumSubBuffers,
    uint32_t                        ElementsPerSubBuffer,
    bool                            bBufferDoneCallback,
    bool                            bSubBufferDoneCallback);

/* Submits a circular buffer to Peripheral DMA to read from Peripheral FIFO */
ADI_DMA_RESULT adi_pdma_PeripheralCircRead(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        NumSubBuffers,
    uint32_t                        ElementsPerSubBuffer,
    bool                            bBufferDoneCallback,
    bool                            bSubBufferDoneCallback);

/* Gets the System Interrupt ID (SID) of a Peripheral DMA channel */
ADI_DMA_RESULT adi_pdma_GetSID(
    ADI_DMA_CHANNEL_ID              eChannelID,
    uint32_t *const                 pSID);

#ifdef __cplusplus
}
#endif

#endif /*!defined(_LANGUAGE_ASM) */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif  /* __ADI_PDMA_BF60X_H__ */

/*****/

