/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_sec.h
*
* @brief     SEC driver API header file
*
* @details
*            This is the primary SEC API header file for Application writers
*/

/** @defgroup SEC_Driver System Event Controller (SEC) APIs
 *  @ingroup INTERRUPTS
 */


/** @addtogroup SEC_Driver System Event Controller (SEC) Service
 *  @{
 */
 
#ifndef _ADI_SEC_H
#define _ADI_SEC_H


#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Supress checking for all the other header files")
#endif

#include <stdint.h>
#include <stdbool.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#if defined(__ADI_HAS_SEC__)

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#endif


/*! Fault generated from an external source */
#define ADI_SEC_EXTERNAL_FAULT      (ENUM_SEC_FSID_SRC_EXTFLT)

/*! Enumeration of different error codes returned by SEC driver */
typedef enum
{
  ADI_SEC_SUCCESS = 0,              /*!< Success */

  ADI_SEC_INVALID_GROUPING,         /*!< The given Group ID is invalid */
  ADI_SEC_INVALID_CORE_ID,          /*!< The given Core ID is invalid */
  ADI_SEC_INVALID_SID_ID,           /*!< The given SID ID  is invalid */
  ADI_SEC_INVALID_POINTER,          /*!< The given pointer is invalid */
  ADI_SEC_INVALID_PRIORITY,         /*!< The given priority is invalid */
  ADI_SEC_REGISTRATION_FAILED,      /*!< Failed to register the given handler */
  ADI_SEC_UNREGISTERATION_FAILED    /*!< Unregistration of the handler failed */

} ADI_SEC_RESULT;

/*! Enumeration of interrupt grouping */
typedef enum
{
    ADI_SEC_INT_NO_GRP  = 0,        /*!< Interrupt not grouped */
    ADI_SEC_INT_GRP0    = 1,        /*!< Interrupt group 0 */
    ADI_SEC_INT_GRP1    = 2,        /*!< Interrupt group 1 */
    ADI_SEC_INT_GRP2    = 4,        /*!< Interrupt group 2 */
    ADI_SEC_INT_GRP3    = 8         /*!< Interrupt group 3 */

} ADI_SEC_INT_GRP;

/*! Enumeration of Core IDs */
typedef enum
{
    ADI_SEC_CORE_0      =  0,       /*!< Core ID 0 */
    ADI_SEC_CORE_1                  /*!< Core ID 1 */

} ADI_SEC_CORE_ID;

/*! Maximum interrupt priority level */
#define ADI_SEC_MAX_INT_PRIO        255u

/*! Enumeration of error causes for System Interface */
typedef enum
{
    /*! Indicates that a source signal assertion occured or attempted to raise
     *   while it was pending to be served.
     */
    ADI_SEC_SSI_SRC_OVF_ERR     =   ENUM_SEC_SSTAT_SOVFERR,

    /*! Indicates that an "End" was recieved for a source while it was
     * not active
     */
    ADI_SEC_SSI_END_ERR         =   ENUM_SEC_SSTAT_ENDERR

}ADI_SEC_SSI_ERR_CAUSE;


/*! Enumeration of error causes for Core Interface */
typedef enum
{
    /*! Indicates that a Core Interface recieved an acknowledge when there is
     *   no acitve interrupt (No interrupt is being serviced).
     */
    ADI_SEC_SCI_ACK_ERR     =   ENUM_SEC_CSTAT_ACKERR

}ADI_SEC_SCI_ERR_CAUSE;


/*! Enumeration of error causes for Fault Interface */
typedef enum
{
    /*! Indicates that an "End" was recieved for a source while it was
     * not active or pending
     */
    ADI_SEC_SFI_END_ERR     =   ENUM_SEC_FSTAT_ENDERR

}ADI_SEC_SFI_ERR_CAUSE;



/*! Structure which contains the information regarding the Core Interface
 *  error.
 */
typedef struct  _ADI_SEC_SCI_ERROR_INFO_
{
    /*!< Core ID that caused the error */
    uint32_t                    CoreID;

    /*!< Error cause that triggered the error */
    ADI_SEC_SCI_ERR_CAUSE       eErrCause;

} ADI_SEC_SCI_ERROR_INFO;


/*! Structure which contains the information regarding the System Interface
 *  error.
 */
typedef struct  _ADI_SEC_SSI_ERROR_INFO_
{
    /*!< System Interrupt Source ID that caused the error */
    uint32_t                    SID;

    /*!< Error cause that triggered the error */
    ADI_SEC_SSI_ERR_CAUSE       eErrCause;

} ADI_SEC_SSI_ERROR_INFO;


/*! @brief Enumeration of errors reported by SEC
 *
 * For each of the errors different callback argument is passed which will
 * have additional information regarding the error. The details of the
 * callback argument passed back is documented below at each of the error.
 */
typedef enum
{
    /*!< Error generated by System interface
     *
     *   pArg = pointer to ADI_SEC_SSI_ERROR_INFO struct which contains
     *          information regarding the source interrupt generated the error
     *          and cause of the error.
     */
    ADI_SEC_SSI_ERROR,

    /*!< Error generated by Core interface
     *
     *  pArg = pointer to ADI_SEC_SCI_ERROR_INFO struct which contains
     *         information regarding the core interface that generated the error
     *         and the cause.
     */
    ADI_SEC_SCI_ERROR,

    /*!< Error generated by Fault interface
     *
     * pArg = One of the enumurated error causes from ADI_SEC_SFI_ERROR_CAUSE
     */
    ADI_SEC_SFI_ERROR

} ADI_SEC_ERRORS;


/*! @brief Enumeration of prority levels
 *
 * The total number of available prority levels can be devided into levels and
 * sub levels. priority resolution provides the tie breaker for siumltaneously
 * pending interrupts assigned to the same level.
 *
 */
typedef enum
{
    ADI_SEC_2_PRIORITY_LEVELS    =   0,
    ADI_SEC_4_PRIORITY_LEVELS,
    ADI_SEC_8_PRIORITY_LEVELS,
    ADI_SEC_16_PRIORITY_LEVELS,
    ADI_SEC_32_PRIORITY_LEVELS,
    ADI_SEC_64_PRIORITY_LEVELS,
    ADI_SEC_128_PRIORITY_LEVELS,
    ADI_SEC_256_PRIORITY_LEVELS

}ADI_SEC_PRIORITY_LEVELS;

/*! @brief Enumeration of fault trigger event types.
 *
 *  A fault trigger can be set such that it will be generated when fault state
 *  changes to pending or when it changes to active.
 */
typedef enum
{
    ADI_SEC_TRIG_FAULT_ACTIVE,      /*!< Generate trigger when fault state changes to active. */
    ADI_SEC_TRIG_FAULT_PENDING      /*!< Generate trigger when fault state changes to pending. */

}ADI_SEC_TRIG_EVENT;


#ifdef __cplusplus
extern "C" {
#endif


ADI_SEC_RESULT adi_sec_Init(void);
void adi_sec_Enable(bool bEnable);

/* ADSP-CM40x only implements the SEC fault interface */
#if !defined(ADI_ADSP_CM40Z)

void adi_sec_Reset(void);
ADI_SEC_RESULT adi_sec_Raise(uint32_t SID);
ADI_SEC_RESULT adi_sec_SetPriority(uint32_t SID, uint32_t Priority);
ADI_SEC_RESULT adi_sec_SetSIDGrouping(uint32_t SID, uint32_t grouping);
ADI_SEC_RESULT adi_sec_SetCoreID(uint32_t SID, ADI_SEC_CORE_ID eCoreID);
ADI_SEC_RESULT adi_sec_RegisterErrorHandler(ADI_CALLBACK pfHandler,
                                            void *pCBParam);
ADI_SEC_RESULT adi_sec_EnableInterrupt(uint32_t SID, bool bEnable);
ADI_SEC_RESULT adi_sec_EnableSource(uint32_t SID, bool bEnable);
ADI_SEC_RESULT adi_sec_SetPriorityMask(ADI_SEC_CORE_ID eCoreID, uint32_t Priority);
ADI_SEC_RESULT adi_sec_SetGroupMask(ADI_SEC_CORE_ID eCoreID, uint32_t Groups);
ADI_SEC_RESULT adi_sec_SetUnGroupedMask(ADI_SEC_CORE_ID eCoreID, bool bMask);
ADI_SEC_RESULT adi_sec_SetPriorityLevels(ADI_SEC_CORE_ID eCoreID,
                                         ADI_SEC_PRIORITY_LEVELS eLevels);

ADI_SEC_RESULT adi_sec_EnableNMI(ADI_SEC_CORE_ID eCoreID, bool bEnable );
ADI_SEC_RESULT adi_sec_GetLockWriteError(bool *pError);
ADI_SEC_RESULT adi_sec_GetAddrError(bool *pError);

ADI_SEC_RESULT adi_sec_EnableFault(uint32_t SID, bool bEnable);
ADI_SEC_RESULT adi_sec_EnableError(uint32_t SID, bool bEnable);
ADI_SEC_RESULT adi_sec_EnableEdgeSense(uint32_t SID, bool bEnable);
ADI_SEC_RESULT adi_sec_GetSIDChannelID(uint32_t SID, uint32_t *pChannelID);
ADI_SEC_RESULT adi_sec_ClearPendStatus(uint32_t SID);

ADI_SEC_RESULT adi_sec_EnableSCI(ADI_SEC_CORE_ID eCoreID, bool bEnable );
ADI_SEC_RESULT adi_sec_ResetSCI(ADI_SEC_CORE_ID eCoreID );
ADI_SEC_RESULT adi_sec_WaitForIdle(ADI_SEC_CORE_ID eCoreID );
ADI_SEC_RESULT adi_sec_GetWFIStatus(ADI_SEC_CORE_ID eCoreID, bool *pWFI);
ADI_SEC_RESULT adi_sec_ClearWFIStatus(ADI_SEC_CORE_ID eCoreID );
ADI_SEC_RESULT adi_sec_ClearNMIStatus(ADI_SEC_CORE_ID eCoreID );

/* ADSP-CM40x only implements the SEC fault interface */
#endif /* !defined (ADI_ADSP_CM40Z) */

ADI_SEC_RESULT adi_sec_EnableSFI(bool bEnable);
ADI_SEC_RESULT adi_sec_ResetSFI(void);
ADI_SEC_RESULT adi_sec_EnableFaultOutput(bool bEnable);
ADI_SEC_RESULT adi_sec_EnableTriggerOutput(bool bEnable);
ADI_SEC_RESULT adi_sec_EnableSystemReset(bool bEnable);
ADI_SEC_RESULT adi_sec_EnableFaultInput(bool bEnable);
ADI_SEC_RESULT adi_sec_EnableCOPMode(bool bEnable);
ADI_SEC_RESULT adi_sec_GetFaultID (uint32_t *pFID);
ADI_SEC_RESULT adi_sec_ClearFault(void);
ADI_SEC_RESULT adi_sec_SelectTriggerEvent(ADI_SEC_TRIG_EVENT eTrigEvent);
ADI_SEC_RESULT adi_sec_SetFaultDelay(uint32_t nDelay);
ADI_SEC_RESULT adi_sec_SetSysResetDelay(uint32_t nDelay);
ADI_SEC_RESULT adi_sec_SetCOPPeriod(uint32_t nPeriod);


#ifdef __cplusplus
}
#endif


/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#else
#warning "This processor does not have SEC"
#endif /* __HAS_SEC__ */

#endif   /* #ifndef _ADI_SEC_H */
