/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_pwr_bf60x.h
 * @brief:   Dynamic Power Management service definitions
 * @version: $Revision: 22793 $
 * @date:    $Date: 2013-08-26 12:17:34 -0400 (Mon, 26 Aug 2013) $
 *
 * @details
 *            This is the primary header file for the Dynamic Power Management service,
 *            which contains the API declarations, data and constant definitions used in the APIs
 */

/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"(advisory) Sections of code should not be commented out.")
#endif /* _MISRA_RULES */

/** @addtogroup Power_Service_BF60x Dynamic Power Management Service
 *  @{
 */

#ifndef ADI_PWR_BF60X_H_
#define ADI_PWR_BF60X_H_

#ifndef  __ADI_PWR_H__
#error "This file may not be directly included. The file <services/pwr/adi_pwr.h> should be included instead."
#endif

/* disable misra diagnostics as necessary for the ROM headers */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_2)
#pragma diag(suppress:misra_rule_5_3)
#pragma diag(suppress:misra_rule_5_4)
#pragma diag(suppress:misra_rule_19_4)
#endif /* _MISRA_RULES */

#if defined (__ADSPBF609_FAMILY__)
#define USE_SYSCTRL
#define USE_HIBERNATEB

#include <defBF609_rom.h>
#include <cdefBF609_rom.h>
#endif

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#define MSELMIN    (1u)
#define MSELMAX    (128u)
#define CSELMIN    (1u)
#define CSELMAX    (32u)
#define SYSSELMIN  (1u)
#define SYSSELMAX  (32u)
#define SSELMAX    (8u)
#define DSELMAX    (32u)
#define OSELMAX    (128u)

#define ADI_PWR_PLLMAX   (1020u) /* MHz */

#define ADI_PWR_CTL_LOCK (0x80000000u)

/**
 * Result codes generated by the power service.
 */
typedef enum
{
    ADI_PWR_SUCCESS = 0,                /*!< The API call succeeded. */
    ADI_PWR_FAILURE,                    /*!< The API call failed. */
    ADI_PWR_NOT_INITIALIZED,            /*!< The power service is not initialized. */
    ADI_PWR_INVALID_REGISTER,           /*!< The register is invalid. */
    ADI_PWR_INVALID_VALUE,              /*!< The register value is invalid. */
    ADI_PWR_MODE_UNKNOWN,               /*!< The power mode is unknown. */
    ADI_PWR_INVALID_MODE,               /*!< The power mode is invalid. */
    ADI_PWR_INVALID_MODE_TRANSITION,    /*!< The power mode transition is invalid. */
    ADI_PWR_INVALID_FREQUENCY,          /*!< The frequency is invalid. */
    ADI_PWR_RESULT_PLL_ERROR,           /*!< An error was detected during the PLL configuration. */
    ADI_PWR_CALLBACK_INSTALLED,         /*!< The callback function is already installed. */
    ADI_PWR_CALLBACK_LIMIT_EXCEEDED,    /*!< The number of installed callbacks is exceeded. */
    ADI_PWR_CALLBACK_NOT_INSTALLED      /*!< The callback function is not installed. */
} ADI_PWR_RESULT;

/**
 * The dynamic power management power modes.
 */
typedef enum {
    ADI_PWR_MODE_FULL_ON                = 0x01u,   /*!< The maximum performance mode. */
    ADI_PWR_MODE_ACTIVE                 = 0x02u,   /*!< The PLL is enabled but bypassed. */
    ADI_PWR_MODE_ACTIVE_PLLDISABLED     = 0x03u,   /*!< The PLL is disabled and bypassed. */
    ADI_PWR_MODE_DEEP_SLEEP             = 0x04u,   /*!< Maximizes power savings by disabling the PLL, CCLK, and SCLK. */
    ADI_PWR_MODE_HIBERNATE              = 0x05u    /*!< The lowest possible power dissipation. */
} ADI_PWR_MODE;

/**
 * The power service notification events.
 */
typedef enum {
    ADI_PWR_EVENT_FREQ_PRE_CHANGE,            /*!< The core or system frequency is about to change. */
    ADI_PWR_EVENT_FREQ_POST_CHANGE,           /*!< The core or system frequency has changed. */
    ADI_PWR_EVENT_MODE_PRE_CHANGE,            /*!< The power mode is about to change. */
    ADI_PWR_EVENT_MODE_POST_CHANGE            /*!< The power mode has changed. */
} ADI_PWR_EVENT;

/*
 * DPM registers
 */

/**
 * The DPM_CTL register field bit mask.
 */
typedef enum
{
#if defined (__ADSPBF609_FAMILY__) || defined (ADI_ADSP_CM40Z)
	ADI_PWR_CTL_ENTER_ACTIVE    = BITM_DPM_CTL_PLLBPST,        /* The PLL Bypass Set setting. */
	ADI_PWR_CTL_EXIT_ACTIVE     = BITM_DPM_CTL_PLLBPCL,        /* The PLL Bypass Clear setting. */
	ADI_PWR_CTL_PLL_DISABLE     = BITM_DPM_CTL_PLLDIS,         /* The PLL Disable setting. */
	ADI_PWR_CTL_DEEP_SLEEP      = BITM_DPM_CTL_DEEPSLEEP       /* The Deep Sleep setting. */
#if defined (__ADSPBF609_FAMILY__)
	,
	ADI_PWR_CTL_HIBERNATE       = BITM_DPM_CTL_HIBERNATE       /* The Hibernate setting. */
#endif
#elif defined (__ADSPBF707_FAMILY__)
	ADI_PWR_CTL_DEEP_SLEEP      = BITM_DPM_CTL_DEEPSLEEP,      /* The Deep Sleep setting. */
	ADI_PWR_CTL_HIBERNATE       = BITM_DPM_CTL_HIBERNATE       /* The Hibernate setting. */
#endif   
} ADI_PWR_CTL;

/**
 * The DPM_STAT register field bit mask.
 */
typedef enum
{
#if defined (__ADSPBF609_FAMILY__) || defined (ADI_ADSP_CM40Z)
	ADI_PWR_STAT_CURR_MODE       = BITM_DPM_STAT_CURMODE,      /* The Current Mode status. */
	ADI_PWR_STAT_PREV_MODE       = BITM_DPM_STAT_PRVMODE,      /* The Previous Mode status. */
	ADI_PWR_STAT_CCLK_DIS        = BITM_DPM_STAT_CCLKDIS,      /* The Core Clock(s) Disabled status. */
	ADI_PWR_STAT_DPMHV_READY     = BITM_DPM_STAT_HVBSY,        /* The HV Busy status. */
	ADI_PWR_STAT_ADDR_ERR        = BITM_DPM_STAT_ADDRERR,      /* The Address Error status. */
	ADI_PWR_STAT_LOCK_WR_ERR     = BITM_DPM_STAT_LWERR,        /* The Lock Write Error status. */
	ADI_PWR_STAT_READ_BUSY_ERR   = BITM_DPM_STAT_HVBSYERR,     /* The HV Busy Error status. */
	ADI_PWR_STAT_PLL_CONFIG_ERR  = BITM_DPM_STAT_PLLCFGERR     /* The PLL Configuration Error status. */
#elif defined (__ADSPBF707_FAMILY__)
	ADI_PWR_STAT_CURR_MODE       = BITM_DPM_STAT_CURMODE,      /* The Current Mode status. */
	ADI_PWR_STAT_PREV_MODE       = BITM_DPM_STAT_PRVMODE,      /* The Previous Mode status. */
	ADI_PWR_STAT_DPMHV_READY     = BITM_DPM_STAT_HVBSY,        /* The HV Busy status. */
	ADI_PWR_STAT_ADDR_ERR        = BITM_DPM_STAT_ADDRERR,      /* The Address Error status. */
	ADI_PWR_STAT_LOCK_WR_ERR     = BITM_DPM_STAT_LWERR,        /* The Lock Write Error status. */
	ADI_PWR_STAT_READ_BUSY_ERR   = BITM_DPM_STAT_HVBSYERR      /* The HV Busy Error status. */
#endif
} ADI_PWR_STAT;

/**
 * The wake-up signal field bit mask.
 */
typedef enum
{
	ADI_PWR_WAKEUP0        = 0x0001u,  /*!< Mask for PA15 as Wakeup Enable Source */
	ADI_PWR_WAKEUP1        = 0x0002u,  /*!< Mask for PB15 as Wakeup Enable Source */
	ADI_PWR_WAKEUP2        = 0x0004u,  /*!< Mask for PC15 as Wakeup Enable Source */
	ADI_PWR_WAKEUP3        = 0x0008u,  /*!< Mask for PD6 as Wakeup Enable Source */
	ADI_PWR_WAKEUP4        = 0x0010u,  /*!< Mask for PE12 as Wakeup Enable Source */
	ADI_PWR_WAKEUP5        = 0x0020u,  /*!< Mask for PG4 as Wakeup Enable Source */
	ADI_PWR_WAKEUP6        = 0x0040u,  /*!< Mask for PG13 as Wakeup Enable Source */
	ADI_PWR_WAKEUP7        = 0x0080u,  /*!< Mask for USB as Wakeup Enable Source */
	ADI_PWR_WAKEUP8        = 0x0100u,  /*!< Reserved */
	ADI_PWR_WAKEUP9        = 0x0200u,  /*!< Reserved */
	ADI_PWR_WAKEUP10       = 0x0400u,  /*!< Reserved */
	ADI_PWR_WAKEUP11       = 0x0800u,  /*!< Reserved */
	ADI_PWR_WAKEUP12       = 0x1000u,  /*!< Reserved */
	ADI_PWR_WAKEUP13       = 0x2000u,  /*!< Reserved */
	ADI_PWR_WAKEUP14       = 0x4000u,  /*!< Reserved */
	ADI_PWR_WAKEUP15       = 0x8000u   /*!< Reserved */
} ADI_PWR_WAKE_SIGNAL;

/**
 * The wake-up signal polarity.
 */
typedef enum
{
	ADI_PWR_WAKE_POL_LOW     = 0u,    /*!< The wakeup polarity low setting. */
	ADI_PWR_WAKE_POL_HIGH    = 1u     /*!< The wakeup polarity high setting. */
} ADI_PWR_WAKE_POL;

/**
 * The wake-up signal status field bit mask.
 */
typedef enum
{
	ADI_PWR_WAKE_STAT0        = 0x0001u,    /*!< The wakeup source 0 status. */
	ADI_PWR_WAKE_STAT1        = 0x0002u,    /*!< The wakeup source 1 status. */
	ADI_PWR_WAKE_STAT2        = 0x0004u,    /*!< The wakeup source 2 status. */
	ADI_PWR_WAKE_STAT3        = 0x0008u,    /*!< The wakeup source 3 status. */
	ADI_PWR_WAKE_STAT4        = 0x0010u,    /*!< The wakeup source 4 status. */
	ADI_PWR_WAKE_STAT5        = 0x0020u,    /*!< The wakeup source 5 status. */
	ADI_PWR_WAKE_STAT6        = 0x0040u,    /*!< The wakeup source 6 status. */
	ADI_PWR_WAKE_STAT7        = 0x0080u,    /*!< The wakeup source 7 status. */
	ADI_PWR_WAKE_STAT8        = 0x0100u,    /*!< Reserved */
	ADI_PWR_WAKE_STAT9        = 0x0200u,    /*!< Reserved */
	ADI_PWR_WAKE_STAT10       = 0x0400u,    /*!< Reserved */
	ADI_PWR_WAKE_STAT11       = 0x0800u,    /*!< Reserved */
	ADI_PWR_WAKE_STAT12       = 0x1000u,    /*!< Reserved */
	ADI_PWR_WAKE_STAT13       = 0x2000u,    /*!< Reserved */
	ADI_PWR_WAKE_STAT14       = 0x4000u,    /*!< Reserved */
	ADI_PWR_WAKE_STAT15       = 0x8000u     /*!< Reserved */
} ADI_PWR_WAKE_STAT;

/*
 * CGU0 registers
 */

/**
 * The CGU_CTL register field bit mask.
 */
typedef enum
{
	ADI_PWR_CLK_CTL_DF           = BITM_CGU_CTL_DF,    /* The Divide Frequency setting. */
	ADI_PWR_CLK_CTL_MSEL         = BITM_CGU_CTL_MSEL,  /* The Multiplier Select setting. */
	ADI_PWR_CLK_CTL_WFI          = BITM_CGU_CTL_WFI    /* The Wait For Idle setting. */
} ADI_PWR_CLK_CTL;

/**
 * The CGU_STAT register field bit mask.
 */
typedef enum
{
#if defined (__ADSPBF609_FAMILY__) || defined (ADI_ADSP_CM40Z)
	ADI_PWR_CLK_STAT_PLL_EN          = BITM_CGU_STAT_PLLEN,        /* The PLL Enable setting. */
	ADI_PWR_CLK_STAT_PLL_BYPASS      = BITM_CGU_STAT_PLLBP,        /* The PLL Bypass setting. */
	ADI_PWR_CLK_STAT_PLL_LOCKED      = BITM_CGU_STAT_PLOCK,        /* The PLL Lock setting. */
	ADI_PWR_CLK_STAT_CLK_ALIGN       = BITM_CGU_STAT_CLKSALGN,     /* The Clock Alignment setting. */
	ADI_PWR_CLK_STAT_CCLK0_BUFF_EN   = BITM_CGU_STAT_CCBF0,        /* The CCLK0 Buffer Status setting. */
#if defined (__ADSPBF609_FAMILY__)
	ADI_PWR_CLK_STAT_CCLK1_BUFF_EN   = BITM_CGU_STAT_CCBF1,        /* The CCLK1 Buffer Status setting. */
	ADI_PWR_CLK_STAT_SCLK0_BUFF_EN   = BITM_CGU_STAT_SCBF0,        /* The SCLK0 Buffer Status setting. */
	ADI_PWR_CLK_STAT_SCLK1_BUFF_EN   = BITM_CGU_STAT_SCBF1,        /* The SCLK1 Buffer Status setting. */
	ADI_PWR_CLK_STAT_PLL_LOCK_ERR    = BITM_CGU_STAT_PLOCKERR,     /* The PLL Lock Error setting. */
#endif
	ADI_PWR_CLK_STAT_DCLK_BUFF_EN    = BITM_CGU_STAT_DCBF,         /* The DCLK Buffer Status setting. */
	ADI_PWR_CLK_STAT_OUTCLK_BUFF_EN  = BITM_CGU_STAT_OCBF,         /* The OUTCLK Buffer Status setting. */
	ADI_PWR_CLK_STAT_ADDR_ERR        = BITM_CGU_STAT_ADDRERR,      /* The Address Error setting. */
	ADI_PWR_CLK_STAT_LOCK_WR_ERR     = BITM_CGU_STAT_LWERR,        /* The Lock Write Error setting. */
	ADI_PWR_CLK_STAT_DIV_ERR         = BITM_CGU_STAT_DIVERR,       /* The DIV Error setting. */
	ADI_PWR_CLK_STAT_DF_MSEL_WR_ERR  = BITM_CGU_STAT_WDFMSERR,     /* The Write to DF or MSEL Error setting. */
	ADI_PWR_CLK_STAT_DIV_WR_ERR      = BITM_CGU_STAT_WDIVERR       /* The Write to DIV Error setting. */
#elif defined (__ADSPBF707_FAMILY__)
	ADI_PWR_CLK_STAT_PLL_EN          = BITM_CGU_STAT_PLLEN,        /* The PLL Enable setting. */
	ADI_PWR_CLK_STAT_PLL_BYPASS      = BITM_CGU_STAT_PLLBP,        /* The PLL Bypass setting. */
	ADI_PWR_CLK_STAT_PLL_LOCKED      = BITM_CGU_STAT_PLOCK,        /* The PLL Lock setting. */
	ADI_PWR_CLK_STAT_CLK_ALIGN       = BITM_CGU_STAT_CLKSALGN,     /* The Clock Alignment setting. */
	ADI_PWR_CLK_STAT_ADDR_ERR        = BITM_CGU_STAT_ADDRERR,      /* The Address Error setting. */
	ADI_PWR_CLK_STAT_LOCK_WR_ERR     = BITM_CGU_STAT_LWERR,        /* The Lock Write Error setting. */
	ADI_PWR_CLK_STAT_DF_MSEL_WR_ERR  = BITM_CGU_STAT_WDFMSERR,     /* The Write to DF or MSEL Error setting. */
	ADI_PWR_CLK_STAT_DIV_WR_ERR      = BITM_CGU_STAT_WDIVERR,      /* The Write to DIV Error setting. */
	ADI_PWR_CLK_STAT_PLL_CFG_ERR     = BITM_CGU_STAT_PCFGERR      /* PLL Configuration Error */
#endif
} ADI_PWR_CLK_STAT;

/**
 * The CGU_CLKOUT register field bit mask.
 */
typedef enum
{
	ADI_PWR_CLK_SELECT_CLKOUT        = BITM_CGU_CLKOUTSEL_CLKOUTSEL  /* The CLKOUT Select setting.*/
} ADI_PWR_CLK_SELECT;

/**
 * The CGU_DIV register field bit mask.
 */
typedef enum
{
	ADI_PWR_CLK_DIV_CSEL             = BITM_CGU_DIV_CSEL,             /* The CCLK Divisor setting. */
	ADI_PWR_CLK_DIV_S0SEL            = BITM_CGU_DIV_S0SEL,            /* The SCLK 0 Divisor setting. */
	ADI_PWR_CLK_DIV_SYSSEL           = BITM_CGU_DIV_SYSSEL,           /* The SYSCLK Divisor setting. */
	ADI_PWR_CLK_DIV_S1SEL            = BITM_CGU_DIV_S1SEL,            /* The SCLK 1 Divisor setting. */
	ADI_PWR_CLK_DIV_DSEL             = BITM_CGU_DIV_DSEL,             /* The DCLK Divisor setting. */
	ADI_PWR_CLK_DIV_OSEL             = BITM_CGU_DIV_OSEL,             /* The OUTCLK Divisor setting. */
	ADI_PWR_CLK_DIV_ALGN             = BITM_CGU_DIV_ALGN,             /* The Align setting. */
	ADI_PWR_CLK_DIV_UPDT             = BITM_CGU_DIV_UPDT              /* The Update Clock Divisors setting. */
} ADI_PWR_CLK_DIV;

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Initialize the dynamic power management service.
 */
ADI_PWR_RESULT adi_pwr_Init(
	uint32_t   const fclkin,
	uint32_t   const fcclkMax,
	uint32_t   const fsclkMax,
	uint32_t   const fvcoMin
);

/*
 * Set the core clock (CCLK) and system clock (SCLK) frequencies.
 */
ADI_PWR_RESULT adi_pwr_SetFreq(
	uint32_t   const cclk,
	uint32_t   const sclk
);

/*
 * Calculate the core clock frequency.
 */
ADI_PWR_RESULT adi_pwr_GetCoreFreq(
	uint32_t   *fcclk
);

/*
 * Calculate the system clock frequency.
 */
ADI_PWR_RESULT adi_pwr_GetSystemFreq(
	uint32_t   *fsysclk,
	uint32_t   *fsclk0,
	uint32_t   *fsclk1
);

/*
 * Set the dynamic power management operating mode.
 */
ADI_PWR_RESULT adi_pwr_SetPowerMode(
	ADI_PWR_MODE  const eMode
);

/*
 * Returns dynamic power management operating mode.
 */
ADI_PWR_RESULT adi_pwr_GetPowerMode(
	ADI_PWR_MODE  *eMode
);

/*
 * Set the signal used to wake-up the processor.
 */
ADI_PWR_RESULT adi_pwr_EnableWakeupSignal(
	ADI_PWR_WAKE_SIGNAL  const eWakeup,
	ADI_PWR_WAKE_POL     const ePol,
	bool                 const bEnable
);

/*
 * CGU0 registers
 */

/*
 * Set CGU divide register values.
 */
ADI_PWR_RESULT adi_pwr_SetClkDivideRegister(
    ADI_PWR_CLK_DIV        const eDiv,
    uint32_t               const value
);

/*
 * Get CGU divide register values.
 */
ADI_PWR_RESULT adi_pwr_GetClkDivideRegister(
    ADI_PWR_CLK_DIV       const eDiv,
    uint32_t               *value
);

/*
 * Set CGU control register values.
 */
ADI_PWR_RESULT adi_pwr_SetClkControlRegister(
    ADI_PWR_CLK_CTL        const eCtl,
    uint32_t               const value
);

/*
 * Get CGU control register values.
 */
ADI_PWR_RESULT adi_pwr_GetClkControlRegister(
    ADI_PWR_CLK_CTL        const eCtl,
    uint32_t               *value
);

/*
 * Set CGU clock select register values.
 */
ADI_PWR_RESULT adi_pwr_SetClkOutSelectRegister(
    ADI_PWR_CLK_SELECT     const eSel,
    uint32_t               const value
);

/*
 * Get CGU clock select register values.
 */
ADI_PWR_RESULT adi_pwr_GetClkOutSelectRegister(
    ADI_PWR_CLK_SELECT     const eSel,
    uint32_t               *value
);

/*
 * Get CGU status register values.
 */
ADI_PWR_RESULT adi_pwr_GetClkStatusRegister(
    ADI_PWR_CLK_STAT       const eStat,
    uint32_t               *value
);

/*
 * Calculate the DDR clock frequency.
 */
ADI_PWR_RESULT adi_pwr_GetDDRClkFreq(
	uint32_t   *fdclk
);

/*
 * Calculate the OUT clock frequency.
 */
ADI_PWR_RESULT adi_pwr_GetOutClkFreq(
	uint32_t   *foclk
);


/*
 * DPM registers
 */

/*
 * Set DPM control register values.
 */
ADI_PWR_RESULT adi_pwr_SetControlRegister(
    ADI_PWR_CTL            const eCtl,
    uint32_t               const value
);

/*
 * Get DPM control register values.
 */
ADI_PWR_RESULT adi_pwr_GetControlRegister(
    ADI_PWR_CTL            const eCtl,
    uint32_t               *value
);

/*
 * Get DPM status register values.
 */
ADI_PWR_RESULT adi_pwr_GetStatusRegister(
	ADI_PWR_STAT           const eStat,
    uint16_t               *value
);

/*
 * Install a user defined callback function to receive power service notifications.
 */
ADI_PWR_RESULT adi_pwr_InstallCallback(
    ADI_CALLBACK        pfCallback
);

/*
 * Remove a user defined callback function that receives power service notifications.
 */
ADI_PWR_RESULT adi_pwr_RemoveCallback(
    ADI_CALLBACK        pfCallback
);

#ifdef __cplusplus
}
#endif

#endif /* ADI_PWR_BF60X_H_ */

/**@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */
