/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_tmr_bf6xx.h
*
* @brief     Timer service global API.
*
* @details   This is a header file which contains the API of global
*            API functions of the timer device.
**********************************************************************************/

/** @addtogroup Timer_Service_BF6xx Timer Service
*  @{
*/

/*!
* @file      adi_tmr_bf6xx.h
*
* @brief     API header file for GP timer service.
*
* @details   This is the primary API header file for GP timer service.
*
*/

#ifndef _ADI_TMR_BF6XX_H_
#define _ADI_TMR_BF6XX_H_


#include <stdint.h>
#include <stdbool.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>

/*! Timer handle. */
typedef void*    	ADI_TMR_HANDLE;

/*! Memory required in bytes for the operation of each timer instance. */
#define ADI_TMR_MEMORY          	8u

/*! Enumeration of possible return codes from timer service. */
typedef enum _ADI_TMR_RESULT_
{
    ADI_TMR_SUCCESS  = 0,           /*!< Success. */
    ADI_TMR_FAILURE,                /*!< Generic Failure. */
    ADI_TMR_INVALID_HANDLE,         /*!< If the given timer handle is invalid. */
    ADI_TMR_INVALID_MODE,           /*!< If the given timer mode is invalid. */
    ADI_TMR_IN_USE,                 /*!< If trying to open a timer which is already in use. */
    ADI_TMR_INVALID_POINTER,        /*!< If any of the given pointer parameters are invalid. */
    ADI_TMR_INVALID_TIMERID,        /*!< If the TIMER ID is out of range. */
    ADI_TMR_ALREADY_CLOSED,			/*!< If trying to close the timer which is already closed. */
    ADI_TMR_INSUFFICIENT_MEMORY,    /*!< If the given memory is insufficient to operate the timer instance. */
    ADI_TMR_INIT_FAILED,            /*!< If the timer service initialization failed. */
    ADI_TMR_INT_FAILED,             /*!< If failed to register/unregister interrupt handler with inerrupt dispatcher. */
    ADI_TMR_INVALID_OPERATION,      /*!< If trying to configure the timer when it is running. */
    ADI_TMR_INVALID_PARAMETER       /*!< If one or more parameters are invalid. */

} ADI_TMR_RESULT;


/*! Enumeration of different modes the GP timer can operate. */
typedef enum _ADI_TMR_MODE_
{
    /*! Idle mode. */
    ADI_TMR_MODE_IDLE               = ENUM_TIMER_TMR_CFG_IDLE_MODE,

    /*! Period watchdog mode. */
    ADI_TMR_MODE_PERIOD_WD          = ENUM_TIMER_TMR_CFG_WDPER_MODE,

    /*! Width watchdog mode. */
    ADI_TMR_MODE_WIDTH_WD           = ENUM_TIMER_TMR_CFG_WDWID_MODE,

    /*! Width capture mode at asserting edge. */
    ADI_TMR_MODE_CAPTURE_ASSERT     = ENUM_TIMER_TMR_CFG_WIDCAP0_MODE,

    /*! Width capture mode at de-asserting edge. */
    ADI_TMR_MODE_CAPTURE_DEASSERT   = ENUM_TIMER_TMR_CFG_WIDCAP1_MODE,

    /*! Continuous PWM out mode. */
    ADI_TMR_MODE_CONTINUOUS_PWMOUT  = ENUM_TIMER_TMR_CFG_PWMCONT_MODE,

    /*! Single PWM out mode. */
    ADI_TMR_MODE_SINGLE_PWMOUT      = ENUM_TIMER_TMR_CFG_PWMSING_MODE,

    /*! External clock mode. */
    ADI_TMR_MODE_EXTCLK             = ENUM_TIMER_TMR_CFG_EXTCLK_MODE,

    /*! Pin interrupt mode. */
    ADI_TMR_MODE_PINT               = ENUM_TIMER_TMR_CFG_PININT_MODE

} ADI_TMR_MODE ;


/*! Enumeration of different events generated by timer service */
typedef enum _ADI_TMR_EVENT_
{
    /*!< Counter overflow error. */
    ADI_TMR_EVENT_COUNT_OVF,

    /*!< Period register programming error. */
    ADI_TMR_EVENT_PERIOD_PROG_ERR,

    /*!< Width or delay register programming error. */
    ADI_TMR_EVENT_WDTDLY_PROG_ERR,

    /*!< Timer data interrupt occurred. Ths event can occur for several
     *   reasons based on the timer mode. The following is the list of
     *   reasons.
     *
     *   - Pulse high/low is detected on the input signal. This can
     *     occur when timer is configured in pin interrupt mode
     *     (#ADI_TMR_MODE_PINT).
     *
     *   - When delay is expired. This can occur when timer is
     *     configured in PWM out mode (#ADI_TMR_MODE_SINGLE_PWMOUT or
     *     #ADI_TMR_MODE_CONTINUOUS_PWMOUT) and IRQ mode is set to
     *     interrupt after delay expires (#ADI_TMR_IRQMODE_DELAY).
     *
     *   - When width + delay is expired. This can occur when timer is
     *     configured in PWM out mode (#ADI_TMR_MODE_SINGLE_PWMOUT or
     *     #ADI_TMR_MODE_CONTINUOUS_PWMOUT) and IRQ mode is set to
     *     interrupt after delay expires (#ADI_TMR_IRQMODE_WIDTH_DELAY).
     *
     *   - When period is expired. This can occur when timer is
     *     configured in PWM out mode (#ADI_TMR_MODE_SINGLE_PWMOUT or
     *     #ADI_TMR_MODE_CONTINUOUS_PWMOUT) or external clock mode
     *     (#ADI_TMR_MODE_EXTCLK) and IRQ mode is set to
     *     interrupt after delay expires (#ADI_TMR_IRQMODE_PERIOD).
     *
     *   - When the asserting edge of an input signal falls inside or
     *     outside the window. This can occur when timer is configured
     *     in Windowed watchdog mode (#ADI_TMR_MODE_PERIOD_WD or
     *     #ADI_TMR_MODE_WIDTH_WD) and IRQ mode is set to either
     *     #ADI_TMR_IRQMODE_INSIDE_WINDOW or ADI_TMR_IRQMODE_OUTSIDE_WINDOW.
     */
    ADI_TMR_EVENT_DATA_INT

} ADI_TMR_EVENT;


/*! Enumeration of different IRQ conditions. */
typedef enum _ADI_TMR_IRQ_MODE_
{
    /*! Generate interrupt every active edge. */
    ADI_TMR_IRQMODE_PULSE           = ENUM_TIMER_TMR_CFG_IRQMODE0,

    /*! Generate interrupt after timer counts till TMR_DELAY register value. */
    ADI_TMR_IRQMODE_DELAY           = ENUM_TIMER_TMR_CFG_IRQMODE1,

    /*! Generate interrupt after timer counts till TMR_DELAY + TMR_WIDTH
     * register value. */
    ADI_TMR_IRQMODE_WIDTH_DELAY     = ENUM_TIMER_TMR_CFG_IRQMODE2,

    /*! Generate interrupt if de-asserting edge of timer input falls within
         width/period watchdog window. */
    ADI_TMR_IRQMODE_INSIDE_WINDOW   = ENUM_TIMER_TMR_CFG_IRQMODE2,

    /*! Generate interrupt if de-asserting edge of timer input falls outside
         width/period watchdog window. */
    ADI_TMR_IRQMODE_OUTSIDE_WINDOW  = ENUM_TIMER_TMR_CFG_IRQMODE3,

    /*! Generate interrupt after timer counts till TMR_PERIOD register value. */
    ADI_TMR_IRQMODE_PERIOD          = ENUM_TIMER_TMR_CFG_IRQMODE3

} ADI_TMR_IRQ_MODE;


/*! Enumeration of different clock inputs for the timer. */
typedef enum _ADI_TMR_CLKIN_
{
    /*! use system clock as clock input. */
    ADI_TMR_CLKIN_SYSCLK        = ENUM_TIMER_TMR_CFG_CLKSEL_SCLK,

    /*! use alternative clock input source 0. */
    ADI_TMR_CLKIN_ALTCLK0       = ENUM_TIMER_TMR_CFG_CLKSEL_ALT0,

    /*! Use alternative clock input source 1. */
    ADI_TMR_CLKIN_ALTCLK1       = ENUM_TIMER_TMR_CFG_CLKSEL_ALT1

} ADI_TMR_CLKIN;

/*! Enumeration of timer parameters that can be set by broadcast option. */
typedef enum _ADI_TMR_BCAST_PARAM_
{
    /*! Broadcast timer delay parameter. */
    ADI_TMR_BCAST_DELAY         = BITM_TIMER_TMR_CFG_BDLYEN,

    /*! Broadcast timer width parameter. */
    ADI_TMR_BCAST_WIDTH         = BITM_TIMER_TMR_CFG_BWIDEN,

    /*! Broadcast timer period parameter. */
    ADI_TMR_BCAST_PERIOD        = BITM_TIMER_TMR_CFG_BPEREN

} ADI_TMR_BCAST_PARAM;

#ifdef __cplusplus 
extern "C" { 
#endif /* __cplusplus */ 

ADI_TMR_RESULT  adi_tmr_Open (
    uint32_t                nTimerNum,
    void *            const pMemory,
    uint32_t                nMemSize,
    ADI_CALLBACK            pfCallback,
    void *            const pCBParam,
    ADI_TMR_HANDLE   *const phTimer
);

ADI_TMR_RESULT  adi_tmr_Close (
    ADI_TMR_HANDLE    const hTimer
);

ADI_TMR_RESULT  adi_tmr_SetMode(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_MODE            eTimerMode
);

ADI_TMR_RESULT  adi_tmr_SetWidthWindow(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nMinWidth,
    uint32_t                nMaxWidth
);

ADI_TMR_RESULT  adi_tmr_SetPeriodWindow(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nMinPeriod,
    uint32_t                nMaxPeriod
);

ADI_TMR_RESULT  adi_tmr_SetIRQMode(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_IRQ_MODE        eIRQMode
);

ADI_TMR_RESULT  adi_tmr_SetPulsePolarity(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bActiveHigh
);

ADI_TMR_RESULT  adi_tmr_SetClkInSource(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_CLKIN           eClkIn
);

ADI_TMR_RESULT  adi_tmr_SetInputSource(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bAuxIn
);

ADI_TMR_RESULT  adi_tmr_DisableOutput(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bDisable
);

ADI_TMR_RESULT  adi_tmr_EnableBroadcast(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_BCAST_PARAM     eParam,
    bool                    bEnable
);

ADI_TMR_RESULT  adi_tmr_SetEmuRun(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bRun
);

ADI_TMR_RESULT  adi_tmr_SetPeriod(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nPeriod
);

ADI_TMR_RESULT  adi_tmr_SetWidth(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nWidth
);

ADI_TMR_RESULT  adi_tmr_SetDelay(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nDelay
);

ADI_TMR_RESULT  adi_tmr_BCastPeriod(
    uint32_t                nPeriod
);

ADI_TMR_RESULT  adi_tmr_BCastWidth(
    uint32_t                nWidth
);

ADI_TMR_RESULT  adi_tmr_BCastDelay(
    uint32_t                nDelay
);

ADI_TMR_RESULT  adi_tmr_EnableGracefulStop(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bGraceful
);


ADI_TMR_RESULT  adi_tmr_Enable(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
);

ADI_TMR_RESULT  adi_tmr_GroupEnable(
    uint32_t                TmrGroupMsk,
    bool                    bEnable
);

ADI_TMR_RESULT  adi_tmr_SetTrigSlaveMode(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bRun
);

ADI_TMR_RESULT  adi_tmr_EnableTriggerIn(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
);

ADI_TMR_RESULT  adi_tmr_EnableTriggerOut(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
);


ADI_TMR_RESULT  adi_tmr_EnableDataInt(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
);

ADI_TMR_RESULT  adi_tmr_EnableStatusInt(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
);

ADI_TMR_RESULT  adi_tmr_GetCount(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pCount
);

ADI_TMR_RESULT  adi_tmr_GetDelay(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pDelay
);

ADI_TMR_RESULT  adi_tmr_GetWidth(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pWidth
);

ADI_TMR_RESULT  adi_tmr_GetPeriod(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pPeriod
);

#ifdef __cplusplus 
}
#endif /* __cplusplus */ 


#endif /* _ADI_TMR_BF6XX_H_ */

/*@}*/

