/*!
 *****************************************************************************
 * @file:    system.h
 * @brief:   CMSIS Cortex-M4 Device Peripheral Access Layer Header File
 *           for the ADI ADSP-CM40z Device Series
 * @version: $Revision: 21692 $
 * @date:    $Date: 2013-05-31 14:47:25 -0400 (Fri, 31 May 2013) $
 *-----------------------------------------------------------------------------
 *
 * Copyright (C) 2009 ARM Limited. All rights reserved.
 *
 * ARM Limited (ARM) is supplying this software for use with Cortex-M3
 * processor based microcontrollers.  This file can be freely distributed
 * within development tools that are supporting such ARM based processors.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *****************************************************************************/


#ifndef __SYSTEM_H__
#define __SYSTEM_H__

#include "CM40z_device.h"

#ifdef __cplusplus
 extern "C" {
#endif

/* need bool_t ahead of system.h */
#ifndef BOOL_TYPE
#define BOOL_TYPE /*!< Create a boolean type */

/*!
 * \typedef bool_t
 * boolean type, since use of C99 "bool" is not MISRA compliant
 */
typedef uint8_t  bool_t;

/* simple true/false defines for functional APIs */
#ifndef false
#define false 0u /*!< Create a boolean 'false' value */
#endif
#ifndef true
#define true 1u /*!< Create a boolean 'true' value */
#endif

#endif /* BOOL_TYPE */

/* clock control reset values */
#define CLKCON0_RVAL                (0x0000)
#define CLKCON1_RVAL                (0x1010)
#define CLKCON2_RVAL                (0x0101)
#define CLKCON3_RVAL                (0x0218)
#define CLKCON4_RVAL                (0x000f)
#define CLKCON5_RVAL                (0x0080)

/* HF Mux (CLKCON0) */
#define HFMUX_MSK					(0x0018)
#define HFMUX_INTERNAL_OSC_VAL		(0 << 3)
#define HFMUX_EXTERNAL_XTAL_VAL		(1 << 3)
#define HFMUX_SYSTEM_PLL_VAL		(2 << 3)
#define HFMUX_GPIO_VAL				(3 << 3)

/* Dividers (CLKCON1) */
#define PCLKDIV_MSK					(0x3f00)
#define PCLKDIV_POS					(8)
#define USBCTLDIV_MSK				(0X0080)
#define USBCTLDIV_POS				(7)
#define HCLKDIV_MSK					(0x003f)
#define HCLKDIV_POS					(0)

/* Dividers (CLKCON2) */
#define CTCLKDIV_MSK				(0x003f)
#define CTCLKDIV_POS				(0)

/* System PLL (CLKCON3) */
#define SPLLIE_MSK					(0x0400)
#define SPLLEN_MSM					(0x0200)
#define SPLLDIV2_MSK				(0x0100)
#define SPLLMSEL_MSK				(0x00c0)
#define SPLLMSEL_DIV1_VAL			(0 << 6)
#define SPLLMSEL_DIV2_VAL			(1 << 6)
#define SPLLMSEL_DIV4_VAL			(2 << 6)
#define SPLLNSEL_MSK				(0x003f)
#define SPLLNSEL_POS				(0)

/* USB PLL (CLKCON4) */
#define UPLLIE_MSK					(0x0400)
#define UPLLEN_MSM					(0x0200)
#define UPLLDIV2_MSK				(0x0100)
#define UPLLMSEL_MSK				(0x00c0)
#define UPLLMSEL_DIV1_VAL			(0 << 6)
#define UPLLMSEL_DIV2_VAL			(1 << 6)
#define UPLLMSEL_DIV4_VAL			(2 << 6)
#define UPLLNSEL_MSK				(0x003f)
#define UPLLNSEL_POS				(0)

/* User Clock Gating (CLKCON5) */
#define CTCLK_GATE					(1 << 8)
#define ACLK_GATE					(1 << 7)
#define PCLK_GATE					(1 << 6)
#define I2SCLK_GATE					(1 << 5)
#define UARTCLK_GATE				(1 << 4)
#define I2CCLK_GATE					(1 << 3)
#define SPIHCLK_GATE				(1 << 2)
#define SPI1CLK_GATE				(1 << 1)
#define SPI0CLK_GATE				(1 << 0)


/* clock IDs */
typedef enum {

	/* clocks */
	ADI_SYS_CLOCK_USBPHY,
	ADI_SYS_CLOCK_USBCTL,
	ADI_SYS_CLOCK_PCLK,
	ADI_SYS_CLOCK_I2C,
	ADI_SYS_CLOCK_UART,
	ADI_SYS_CLOCK_I2S,
	ADI_SYS_CLOCK_SPI0,
	ADI_SYS_CLOCK_SPI1,
	ADI_SYS_CLOCK_MCLK,
	ADI_SYS_CLOCK_SPIH,
	ADI_SYS_CLOCK_CORE,
	ADI_SYS_CLOCK_BUS,
	ADI_SYS_CLOCK_FCLK,
	ADI_SYS_CLOCK_NVC_FCLK,
	ADI_SYS_CLOCK_HCLK_CT,
	ADI_SYS_CLOCK_ACLK,
	ADI_SYS_CLOCK_RCLK,
	ADI_SYS_CLOCK_CT,

	/* input mux */
	ADI_SYS_CLOCK_MUX_HF_INTERNAL_OSC,
	ADI_SYS_CLOCK_MUX_HF_EXTERNAL_XTAL,
	ADI_SYS_CLOCK_MUX_HF_SYSTEM_PLL,
	ADI_SYS_CLOCK_MUX_HF_EXTERNAL_GPIO,

} ADI_SYS_CLOCK_ID;


/* clock divider values */
#define ADI_SYS_CLOCK_DIVIDER uint32_t

/*!
    System API function return codes
 */

typedef enum
{
    ADI_SYS_SUCCESS = 0,                                   /*!< No error detected.                      */
    ADI_SYS_ERR_UNKNOWN_ERROR = 0x00001000,                             /*!< Generic unknown error occurred.         */
	ADI_SYS_ERR_INVALID_CLOCK_ID,                          /*!< Invalid ADI_SYS_CLOCK_ID specified.     */
	ADI_SYS_ERR_REQUESTED_CLOCK_TOO_SLOW,                  /*!< Peripheral clock must >= system clock.  */
	ADI_SYS_ERR_CLOCK_DIVIDE_OUT_OF_RANGE,                 /*!< Requested divide value is out of range. */
} ADI_SYS_RESULT_TYPE;



/* Peripheral and system related clock APIs */
extern ADI_SYS_RESULT_TYPE SystemSetClockDivider   (ADI_SYS_CLOCK_ID dev, ADI_SYS_CLOCK_DIVIDER div);
extern uint32_t            SystemGetClockFrequency (ADI_SYS_CLOCK_ID dev);
extern ADI_SYS_RESULT_TYPE SystemEnableClock       (ADI_SYS_CLOCK_ID dev, bool_t bFlag);


/**
 * @brief  Initialize the system
 *
 * @param  none
 * @return none
 *
 * Setup the microcontroller system.
 * Initialize the System and update the SystemCoreClock variable.
 */
extern void SystemInit (void);

/**
 * @brief  Select the system HF clock mux
 *
 * @param  none
 * @return none
 *
 * Setup the microcontroller system HF clock multiplexed.
 * Initialize the System clock multiplexer and update the SystemCoreClock variable.
 */
extern void SetSystemClockMux (ADI_SYS_CLOCK_ID dev);


/**
 * @brief  Update internal SystemCoreClock variable
 *
 * @param  none
 * @return none
 *
 * Updates the internal SystemCoreClock with current core
 * Clock retrieved from cpu registers.
 */
extern void SystemCoreClockUpdate (void);


/**
 * @brief  Sets the system external clock frequency
 *
 * @param  ExtClkFreq   External clock frequency in Hz
 * @return none
 *
 * Sets the clock frequency of the source connected to P0.5 clock input source
 */
extern void SetSystemExtClkFreq (uint32_t ExtClkFreq);


/**
 * @brief  Gets the system external clock frequency
 *
 * @return External Clock frequency
 *
 * Gets the clock frequency of the source connected to P0.5 clock input source
 */
extern uint32_t GetSystemExtClkFreq (void);


#ifdef __cplusplus
}
#endif

#endif /* __SYSTEM_H__ */
