/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_adcc.c
*
* @brief     ADCC service API implementation.
*
* @details
*            This is the primary header file ADC Controller (ADCC) driver.
*
*            Terminology used in the driver:
*
*            - DMA Mode:
*
*              If DMA is used to get the data from the event data registers to
*              the memory.
*
*            - Programmed I/O mode:
*
*              If core reads are used to read the data from event data registers.
*
*            - Callback mode:
*
*              When a callback is registered for the timer to get the events. The
*              callback registration is optional.
*
*            - Non blocking mode:
*
*              When a callback is not not registered for the timer to get the
*              buffer related events.
*
*            - Circular buffer mode:
*
*              If a circular buffer is submitted to read the data from event
*              data registers. The same buffer is reused for filling the data.
*
*            - Linear buffer mode:
*
*              If a linear buffer is submitted to read the data from the event
*              data registers. A new buffer (or same buffer) should be resubmitted
*              once the driver returns back the filled buffer.
*
*/

/** @defgroup ADCC_Driver ADC Controller APIs.
 */

/** @addtogroup ADCC_Driver ADC Controller (ADCC) Driver
 *  @{
 */

#include <stdbool.h>
#include <stddef.h>
#include <string.h>
#include <assert.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>
#include <adi_osal.h>
#include <drivers/adcc/adi_adcc.h>
#include "adi_adcc_def.h"
#include "adi_adcc_data.c"

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#endif /* _MISRA_RULES */

/**
 * @brief       Opens ADC Controller Device instance.
 *
 * @details     ADCC device instance should be opened to call any other APIs.
 *              If the device is opened successfully the handle to the device
 *              is returned to the application. The opened instance can be
 *              closed using #adi_adcc_CloseDevice API.
 *
 * @param [in]  nDeviceNum      ADCC instance number to be opened. The processor
 *                              can have one or more instances of ADCC. This
 *                              indicates the instance number of the ADCC to be
 *                              opened.
 *
 * @param [in]  pMemory         Memory required for the operation of ADCC driver.
 *                              The size of the given memory should be equal to
 *                              #ADI_ADCC_MEMORY bytes.
 *
 * @param [out] phDevice        Handle to the instance that is opened.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully opened the ADCC device.
 *
 *  - #ADI_ADCC_INVALID_DEVICE_NUM  [D] If the given device number is outside the
 *                                      supported number of devices.
 *
 *  - #ADI_ADCC_NULL_POINTER        [D] If one of the given pointers are pointing
 *                                      to NULL.
 *
 *  - #ADI_ADCC_DEVICE_IN_USE       [D] If the given device is already in use.
 *
 *  - #ADI_ADCC_INT_FAILED          [D] Failed to install the error interrupt handler.
 *
 * @sa adi_adcc_CloseDevice()
 */
ADI_ADCC_RESULT  adi_adcc_OpenDevice (
    uint32_t                    nDeviceNum,
    void const                 *pMemory,
    ADI_ADCC_HANDLE            *phDevice
)
{
    /* Pointer to the device data*/
    ADI_ADCC_DEVICE     *pDevice = NULL;

    /* Pointer to the ADCC registers */
    volatile ADI_ADCC_REG        *pADCCRegs;

    uint32_t            i;

#ifdef ADI_DEBUG

    /* Check if the given device number is valid */
    if(nDeviceNum > ADI_ADCC_NUM_DEVICES)
    {
        return ADI_ADCC_INVALID_DEVICE_NUM;
    }

    /* Check if the given pointer are valid */
    if(pMemory == NULL || phDevice == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }

    /* Check if device is already opened */
    if(gADCCDevices[nDeviceNum].eADCCState != ADI_ADCC_STATE_NOT_OPENED)
    {
        return ADI_ADCC_DEVICE_IN_USE;
    }

    /* Assert to validate the memory sizes */
    assert(ADI_ADCC_MEMORY  ==  sizeof(ADI_ADCC_DEV_DATA));
    assert(ADI_ADCC_TMR_MEMORY  ==  sizeof(ADI_ADCC_TMR_DATA));

#endif /* ADI_DEBUG */ 
   
    /* clear the given memory */
    memset((void *)pMemory, 0, sizeof(ADI_ADCC_DEV_DATA));

    pDevice             =   &gADCCDevices[nDeviceNum];
    pADCCRegs           =   pDevice->pADCCRegs;

    /*
     * Set the ADCC registers to default values
     */

    pADCCRegs->CTL          =   0ul;            /* ADCC Control register                */
    pADCCRegs->ERRMSK_SET   =   0xFFFFFFFFul;   /* Error Mask Clear register            */
    pADCCRegs->EIMSK_SET    =   0xFFFFFFFFul;   /* Event interrupt Mask Clear register  */
    pADCCRegs->FIMSK_SET    =   0xFFFFFFFFul;   /* Frame interrupt Mask Clear register  */
    pADCCRegs->EVTEN_CLR    =   0xFFFFFFFFul;   /* Event Enable Clear Register          */
    pADCCRegs->BPTR0        =   0ul;            /* TIMER0 DMA base pointer              */
    pADCCRegs->FRINC0       =   0ul;            /* TIMER0 DMA frame increment           */
    pADCCRegs->CBSIZ0       =   0ul;            /* TIMER0 DMA Circular buffer size      */
    pADCCRegs->TCA0         =   0ul;            /* ADC0 Timing control A register       */
    pADCCRegs->TCB0         =   0ul;            /* ADC0 Timing control B register       */
    pADCCRegs->CFG          =   0ul;            /* ADC Configuration register           */
    pADCCRegs->BPTR1        =   0ul;            /* TIMER0 DMA base pointer              */
    pADCCRegs->FRINC1       =   0ul;            /* TIMER0 DMA frame increment           */
    pADCCRegs->CBSIZ1       =   0ul;            /* TIMER0 DMA Circular buffer size      */
    pADCCRegs->TCA1         =   0ul;            /* ADC0 Timing control A register       */
    pADCCRegs->TCB1         =   0ul;            /* ADC0 Timing control B register       */

    /* Clear event specific configuration registers */
    for(i = 0ul; i < ADI_ADCC_MAX_EVENTS; i++)
    {
        pADCCRegs->EVT_REG[i].EVCTL   =   0ul;
        pADCCRegs->EVT_REG[i].EVT     =   0ul;
    }

    /* Install the error interrupt handler */
    if(adi_int_InstallHandler(pDevice->nErrorIntID,
                              ErrorIntHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
        return ADI_ADCC_INT_FAILED;
    }

    /* Mark the device as in use */
    adi_osal_EnterCriticalRegion();
    pDevice->eADCCState =   ADI_ADCC_STATE_NOT_OPENED;
    adi_osal_ExitCriticalRegion();

    /* Save the given memory pointer */
    pDevice->pADCCData  =   (ADI_ADCC_DEV_DATA *) pMemory;

    /* Return the device handle */
    *phDevice = (ADI_ADCC_HANDLE) pDevice;

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Closes ADC Controller Device.
 *
 * @details     This also closes the timer(s) which are opened by the application.
 *
 * @param [in]  hDevice         Handle to the ADCC device to be closed.
 *
 * @return      Status
 *
 *  - #ADI_ADCC_SUCCESS             If successfully closed the ADCC device.
 *  - #ADI_ADCC_INVALID_HANDLE  [D] If the given device handle is invalid.
 *  - #ADI_ADCC_INT_FAILED      [D] If failed to uninstall the error interrupt
 *                                  handler.
 *
 * @sa adi_adcc_OpenDevice()
 */
ADI_ADCC_RESULT  adi_adcc_CloseDevice (
   ADI_ADCC_HANDLE const        hDevice
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE    *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Mask all the error interrupts */
    pDevice->pADCCRegs->ERRMSK_SET  =   0xFFFFFFFF;

    /* Un-install the error interrupt handler */
    if(adi_int_UninstallHandler(pDevice->nErrorIntID) != ADI_INT_SUCCESS)
    {
        return ADI_ADCC_INT_FAILED;
    }

    /* Mark the device as not in use */
    pDevice->eADCCState = ADI_ADCC_STATE_NOT_OPENED;

    /* Clear the device data pointer */
    pDevice->pADCCData  = NULL;

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Opens Timer within the ADC Controller.
 *
 * @details     Before calling this API ADCC controller should be opened by
 *              calling the API #adi_adcc_OpenDevice. Once the ADCC is opened
 *              this API can be used to open the timer within the ADCC.
 *
 *              The timer instance can be closed using #adi_adcc_CloseTimer.
 *
 * @param [in]  hDevice         Handle to the ADCC whose timer to be opened.
 *
 * @param [in]  eTimer          Timer number to be opened. It can be one of the
 *                              values from the #ADI_ADCC_TIMER enumeration.
 *
 * @param [in]  pMemory         Memory required for the operation of the timer
 *                              instance within the ADCC. The amount of memory
 *                              should be equal to #ADI_ADCC_TMR_MEMORY.
 *
 * @param [out] phTimer         Handle to the timer instance that is opened.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS             If successfully opened the timer instance.
 *  - #ADI_ADCC_INVALID_HANDLE  [D] If the given device handle is invalid.
 *  - #ADI_ADCC_NULL_POINTER    [D] If one of the given pointer(s) is pointing to NULL.
 *  - #ADI_ADCC_TMR_IN_USE      [D] If the given timer is already opened.
 *  - #ADI_ADCC_SEM_FAILED      [D] If the semaphore creation failed.
 *  - #ADI_ADCC_INT_FAILED      [D] If failed to install the timer interrupt handler.
 *
 * @sa adi_adcc_CloseTimer()
 */
ADI_ADCC_RESULT  adi_adcc_OpenTimer (
    ADI_ADCC_HANDLE             hDevice,
    ADI_ADCC_TIMER              eTimer,
    void const                 *pMemory,
    ADI_ADCC_TMR_HANDLE        *phTimer
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE    *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData = (ADI_ADCC_TMR_DATA *)pMemory;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Validate the given pointers */
    if(pMemory == NULL || phTimer == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }

    /* Check if the timer is already opened */
    if(pDevice->TmrDevice[eTimer].pTmrData != NULL)
    {
        return ADI_ADCC_TMR_IN_USE;
    }
#endif /* ADI_DEBUG */

    /* Get pointer to the timer device */
    pTmrDevice  =   &pDevice->TmrDevice[eTimer];

    /* Clear the given memory */
    memset((void *)pMemory, 0, sizeof(ADI_ADCC_TMR_DATA));

    /* Create the timer semaphore */
    if(adi_osal_SemCreateStatic(pTmrData->SemMemory,
                                ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                &pTmrData->hTmrSem,
                                0) != ADI_OSAL_SUCCESS)
    {
        return ADI_ADCC_SEM_FAILED;
    }

    /* acknowledge the pending interrupt at the NVIC level */
    ADI_ACK_PENDINT((IRQn_Type)pTmrDevice->nTmrIntID);

    /* Install the timer interrupt handler */
    if(adi_int_InstallHandler(pTmrDevice->nTmrIntID,
                              TimerIntHandler,
                              pTmrDevice,
                              true) != ADI_INT_SUCCESS)
    {

        /* Destroy the semaphore */
        adi_osal_SemDestroyStatic(pTmrData->hTmrSem);

        return ADI_ADCC_INT_FAILED;
    }

    /* Set the pointer to the ADC Device */
    pTmrData->pADCCDevice   =   (ADI_ADCC_DEVICEP) pDevice;

    /* Mark the device as in use */
    adi_osal_EnterCriticalRegion();
    pTmrDevice->pTmrData = pTmrData;
    adi_osal_ExitCriticalRegion();

    /* Enable frame interrupt */
    pTmrDevice->pADCCRegs->FIMSK_CLR  = (BITM_ADCC_FIMSK_FINT0 << eTimer);
    
    /* Return the handle to the timer device */
    *phTimer    =       pTmrDevice;
    
    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Closes Timer within the ADC Controller .
 *
 * @details     None of the APIs related to the given timer will be accessible
 *              after closing it.
 *
 * @param [in]  hTimer          Handle to the ADCC timer to be closed.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS             If successfully closed the timer instance.
 *  - #ADI_ADCC_SEM_FAILED      [D] If failed to destroy the semaphore.
 *  - #ADI_ADCC_INT_FAILED      [D] If failed to uninstall the timer interrupt handler.
 *
 * @sa adi_adcc_CloseTimer()
 */
ADI_ADCC_RESULT  adi_adcc_CloseTimer (
    ADI_ADCC_TMR_HANDLE         hTimer
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

    /* Destroy the semaphore */
    if(pTmrData->hTmrSem != NULL)
    {
        if(adi_osal_SemDestroyStatic(pTmrData->hTmrSem ) != ADI_OSAL_SUCCESS)
        {
            return ADI_ADCC_SEM_FAILED;
        }
    }

    /* Mask the frame interrupt */
    pTmrDevice->pADCCRegs->FIMSK_SET  =   ((BITM_ADCC_FIMSK_CLR_FINT0) << pTmrDevice->nTimerID);

    /* Uninstall the timer handler */
    if(adi_int_UninstallHandler(pTmrDevice->nTmrIntID) != ADI_INT_SUCCESS)
    {
        return ADI_ADCC_INT_FAILED;
    }

    /* Mark the timer as unused by clearing the timer data pointer */
    adi_osal_EnterCriticalRegion();
    pTmrDevice->pTmrData    =   NULL;
    adi_osal_ExitCriticalRegion();

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Register/Unregister an optional callback for ADC Controller.
 *
 * @details     The callback is mainly used to report the hardware errors.
 *              When a callback is registered the controller
 *              errors (#ADI_ADCC_ERR) are reported back via the callback.
 *
 *              To unregister the callback, call this API with pointer
 *              to the callback function (pfCallback) set to NULL.
 *
 * @note        Callback cannot be registered/unregistered after controller
 *              is enabled.
 *
 * @param [in]  hDevice         Handle to the ADCC for which callback to
 *                              be registered.
 *
 * @param [in]  pfCallback      Pointer to the callback function. Can be
 *                              NULL to unregister the callback.
 *
 * @param [in]  pCBParam        Pointer to the callback parameter which
 *                              will be passed back to the application when
 *                              the callback function is called.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully registered the callback.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given device handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to uninstall the callback when
 *                                      the device is enabled.
 * @sa adi_adcc_EnableDevice()
 * @sa adi_adcc_EnableTimer()
 */
ADI_ADCC_RESULT  adi_adcc_RegisterDevCallback (
    ADI_ADCC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE    *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT     eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if the device is enabled */
    if(pDevice->eADCCState == ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Save the given callback and callback parameter */
    adi_osal_EnterCriticalRegion();
    pDevice->pADCCData->pfCallback  =   pfCallback;
    pDevice->pADCCData->pCBParam    =   pCBParam;
    adi_osal_ExitCriticalRegion();

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Register or Unregister optional callback for ADCC Timer.
 *
 * @details     When the timer callback is registered the events from
 *              the driver are notified to the application via the
 *              registered callback. Some of the data transaction APIs
 *              cannot be used when the callback is registered.
 *
 *              The following buffer transaction APIs cannot be used:
 *
 *              1. #adi_adcc_GetBuffer(). The filled buffer will be
 *                 returned to the application via the callback in the
 *                 #ADI_ADCC_EVENT_BUFFER_PROCESSED event.
 *
 *              2. #adi_adcc_IsBufferAvailable()
 *
 *              3. #adi_adcc_GetCurrWrPtr(). The filled frame pointer
 *                 is returned via the #ADI_ADCC_EVENT_FRAME_COMLETED.
 *
 *
 * @note        Callback cannot be registered/unregistered after controller
 *              is enabled.
 *
 * @param [in]  hTimer          Handle to the timer for which callback to
 *                              be registered.
 *
 * @param [in]  pfCallback      Pointer to the callback function. Can be
 *                              NULL to unregister the callback.
 *
 * @param [in]  pCBParam        Pointer to the callback parameter which
 *                              will be passed back to the application when
 *                              the callback function is called.
 *
 * @return      Status
 *
 *  - #ADI_ADCC_SUCCESS                 If successfully registered the callback.
 *
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given timer handle is invalid.
 *
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to uninstall the callback when
 *                                      the timer is enabled.
 *
 * @sa adi_adcc_GetBuffer()
 * @sa adi_adcc_IsBufferAvailable()
 * @sa adi_adcc_GetCurrWrPtr()
 */
ADI_ADCC_RESULT  adi_adcc_RegisterTmrCallback (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;


#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is enabled */
    if(pTmrDevice->pTmrData->eTmrState == ADI_ADCC_TMR_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

    /* Save the given callback and callback parameter */
    adi_osal_EnterCriticalRegion();
    pTmrData->pfCallback  =   pfCallback;
    pTmrData->pCBParam    =   pCBParam;
    adi_osal_ExitCriticalRegion();

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Submit the buffer for filling the ADC data.
 *
 * @details     This API only works in DMA mode. So DMA mode should be set
 *              first before using this API. The filled buffer can be obtained
 *              by #adi_adcc_GetBuffer(). When the timer callback is registered
 *              the filled buffer is returned via callback in the
 *              #ADI_ADCC_EVENT_BUFFER_PROCESSED event.
 *
 *              As there is no queuing mechanism implemented in the driver,
 *              application can only submit one buffer at a time. The next
 *              buffer can be submitted only after the previous buffer is
 *              returned by the driver after filling.
 *
 * @param [in]  hTimer          Handle to the timer for which the buffer to
 *                              be submitted.
 * @param [in]  pBuffer         Pointer to the buffer where ADC samples
 *                              will be stored.
 * @param [in]  nFrameInc       Increment in bytes after a frame is filled.
 *                              It should be in multiples of two bytes.
 *                              The value can be from -32768 to 32767.
 *
 * @param [in]  nNumFrames      Number of frames that fits into the given buffer.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully submitted the given buffer.
 *
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given timer handle is invalid.
 *
 *  - #ADI_ADCC_BUFFER_FULL         [D] If the previous buffer is being processed or
 *                                      the processed buffer is not retrieved by the
 *                                      application yet.
 *
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to submit a buffer circular buffering
 *                                      mode or when DMA mode is not enabled.
 *
 *  - #ADI_ADCC_INVALID_FRAME_INC   [D] If the frame increment is outside the supported
 *                                      range or not multiples of two bytes. It can be
 *                                      any value from -32768 to +32768.
 *
 *  - #ADI_ADCC_INVALID_NUM_FRAMES  [D] If the given number of frames is invalid it
 *                                      should be >= 1.
 *
 *  - #ADI_ADCC_NULL_POINTER        [D] If the given buffer pointer is pointing to NULL.
 *
 * @sa adi_adcc_GetBuffer()
 * @sa adi_adcc_SubmitCircBuffer()
 * @sa adi_adcc_IsBufferAvailable()
 */
ADI_ADCC_RESULT  adi_adcc_SubmitBuffer (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    void * const                pBuffer,
    int32_t                     nFrameInc,
    uint32_t                    nNumFrames
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

    /* Pointer to the Timer DMA registers */
    volatile ADI_ADCC_TMR_DMA_REG *pTmrDMARegs;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

#ifdef ADI_DEBUG
    
    /* Check if we are in linear buffering mode and DMA mode is enabled */
    if((pTmrData->eIOMode != ADI_ADCC_TMR_IOMODE_LINEAR) || (pTmrData->pADCCDevice->pADCCData->bDMAMode != true))
    {
        return ADI_ADCC_INVALID_OPERATION;
    }

    /* Check if the buffer holder is free */
    if(pTmrData->BufInfo.eBufState != ADI_ADCC_BUF_STATE_FREE)
    {
        return ADI_ADCC_BUFFER_FULL;
    }

    /* Validate the frame increment value */
    if((nFrameInc > 0xFFFFF) || ((nFrameInc & 0x1) != 0))
    {
        return ADI_ADCC_INVALID_FRAME_INC;
    }

    /* Validate the given number of frames */
    if(nNumFrames < 1u)
    {
        return ADI_ADCC_INVALID_NUM_FRAMES;
    }

    /* Check if the given buffer pointer is valid */
    if(pBuffer == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    pTmrDMARegs =   pTmrDevice->pTmrDMARegs;

    /* Save the given buffer information */
    pTmrData->BufInfo.pBuffer       =   pBuffer;
    pTmrData->BufInfo.nBufSize      =   nNumFrames;

    /* Mark the buffer as in use */
    pTmrData->BufInfo.eBufState     =   ADI_ADCC_BUF_STATE_INUSE;

    /* Submit the given buffer to the device */
    pTmrDMARegs->BPTR   =   (uint32_t)pBuffer;
    pTmrDMARegs->FRINC  =   nFrameInc;

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Submit the circular buffer for getting the ADC data.
 *
 * @details     This API only works in DMA mode. So DMA mode should be set
 *              first before using this API. The DMA mode can be set using
 *              the API #adi_adcc_EnableDMAMode().
 *
 *              This API can be called only once after the ADCC is enabled.
 *              The ADCC should be disabled to submit the circular buffer
 *              again.
 *
 *              The filled buffer can be obtained back as follows in
 *              various operating modes:
 *
 *              - Non blocking mode:
 *
 *                  The number of samples available in the circular buffer
 *                  can be known by the difference between read pointer
 *                  (maintained by application) and write pointer
 *                  (Can be obtained by using the API #adi_adcc_GetCurrWrPtr).
 *
 *              - Callback mode:
 *
 *                  The pointer to the filled frame is returned via callback
 *                  in the #ADI_ADCC_EVENT_FRAME_COMLETED event. Every time
 *                  the callback is called an updated pointer to the filled
 *                  frame is returned. If the pointer reaches end of the given
 *                  buffer the pointer is wrapped back to the base of the given
 *                  buffer.
 *
 *                  The #ADI_ADCC_EVENT_BUFFER_PROCESSED event is not
 *                  generated in case of circular buffering as the same buffer
 *                  is reused for filling.
 *
 * @param [in]  hTimer          Handle to the timer for which the buffer to
 *                              be submitted.
 *
 * @param [in]  pBuffer         Pointer to the buffer where ADC samples
 *                              will be stored.
 *
 * @param [in]  nFrameInc       Increment in bytes after a frame is filled.
 *                              It should be in multiples of two bytes. The
 *                              value can be from -32768 to 32767. It should not
 *                              be zero.
 *
 * @param [in]  nBufSize        Circular buffer size in number of samples.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully submitted the given buffer.
 *
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given timer handle is invalid.
 *
 *  - #ADI_ADCC_BUFFER_FULL         [D] If the previous buffer is being processed or
 *                                      the processed buffer is not retrieved by the
 *                                      application yet.
 *
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to submit a buffer when the timer 
 *                                      is enabled or when DMA mode is not enabled.
 *
 *  - #ADI_ADCC_INVALID_FRAME_INC   [D] If the frame increment is zero or outside
 *                                      the supported range (0x8000 to 0x7FFFF) or not multiples
 *                                      of two bytes.
 *
 *  - #ADI_ADCC_INVALID_BUF_SIZE    [D] If the given buffer size less than 2 bytes and
 *                                      not multiple of 2 bytes.
 *
 *  - #ADI_ADCC_NULL_POINTER        [D] If the given buffer pointer is pointing to NULL.
 *
 * @sa adi_adcc_GetCurrWrPtr()
 * @sa adi_adcc_SubmitBuffer()
 * @sa adi_adcc_IsBufferAvailable()
 * @sa adi_adcc_EnableDMAMode()
 */
ADI_ADCC_RESULT  adi_adcc_SubmitCircBuffer (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    void * const                pBuffer,
    int32_t                     nFrameInc,
    uint32_t                    nBufSize
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

    /* Pointer to the Timer DMA registers */
    volatile ADI_ADCC_TMR_DMA_REG *pTmrDMARegs;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

#ifdef ADI_DEBUG

    /* Check if the timer is not enabled and DMA mode is enabled */
    if((pTmrData->eTmrState == ADI_ADCC_TMR_STATE_ENABLED) || (pTmrData->pADCCDevice->pADCCData->bDMAMode != true))
    {
        return ADI_ADCC_INVALID_OPERATION;
    }

    /* Check if the buffer holder is free */
    if(pTmrData->BufInfo.eBufState != ADI_ADCC_BUF_STATE_FREE)
    {
        return ADI_ADCC_BUFFER_FULL;
    }

    /* Validate the given buffer size */
    if((nBufSize < 2u) || ((nBufSize & 0x1u) != 0u))
    {
        return ADI_ADCC_INVALID_BUF_SIZE;
    }

    /* Validate the frame increment value */
    if((nFrameInc > 0xFFFFF) || ((nFrameInc & 0x1) != 0))
    {
        return ADI_ADCC_INVALID_FRAME_INC;
    }
    /* Check if the given buffer pointer is valid */
    if(pBuffer == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    pTmrDMARegs =   pTmrDevice->pTmrDMARegs;

    /* Save the given buffer information */
    pTmrData->BufInfo.pBuffer       =   pBuffer;
    pTmrData->BufInfo.nBufSize      =   nBufSize;

    /* Mark the buffer as in use */
    pTmrData->BufInfo.eBufState     =   ADI_ADCC_BUF_STATE_INUSE;

    /* Submit the given buffer to the device */
    pTmrDMARegs->BPTR   =   (uint32_t) pBuffer;
    pTmrDMARegs->FRINC  =   nFrameInc;
    pTmrDMARegs->CBSIZ  =   nBufSize;

    /* Set the buffering mode as circular buffering mode */
    pTmrData->eIOMode   =   ADI_ADCC_TMR_IOMODE_CIRC;

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Gets back the filled buffer.
 *
 * @details     This is a blocking call to get back the filled buffer
 *              from the driver. To avoid the blocking application can
 *              call the non-blocking  #adi_adcc_IsBufferAvailable() API to
 *              know if filled buffer is available.
 *
 *              The API returns immediately if :
 *
 *                  - Filled buffer available.
 *                  - When hardware error is detected:
 *                      Returns with error code #ADI_ADCC_HW_ERROR_DETECTED.
 *                      The actual hardware error(s) can be known by calling the
 *                      API #adi_adcc_GetHWErrorStatus().
 *
 *              Validity of this API in various operating modes:
 *
 *                  - DMA Mode              : Valid.
 *                  - Linear buffer mode    : Valid.
 *                  - Circular buffer mode  : Not valid.
 *                  - Callback mode         : Not valid.
 *
 *
 * @param [in]  hTimer          Handle to the timer from which the buffer to
 *                              be obtained.

 * @param [out] ppBuffer        Pointer to a location where the filled buffer
 *                              pointer is written.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully obtained the buffer.
 *
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given timer handle is invalid.
 *
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to get a buffer in circular buffering
 *                                      mode or when DMA mode is not enabled or when
 *                                      a timer callback is installed.
 *
 *  - #ADI_ADCC_NULL_POINTER        [D] If the given pointer to filled buffer is pointing to NULL.
 *
 *  - #ADI_ADCC_SEM_FAILED          [D] If a semaphore related failure occurred.
 *
 *
 * @sa adi_adcc_SubmitBuffer()
 * @sa adi_adcc_IsBufferAvailable()
 * @sa adi_adcc_EnableDMAMode()
 * @sa adi_adcc_GetHWErrorStatus()
 */
ADI_ADCC_RESULT  adi_adcc_GetBuffer (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    void ** const               ppBuffer
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

#ifdef ADI_DEBUG
    /* Check if we are in linear buffering mode, DMA mode is enabled
     * callback is not installed */
    if((pTmrData->eIOMode != ADI_ADCC_TMR_IOMODE_LINEAR)    ||
       (pTmrData->pADCCDevice->pADCCData->bDMAMode != true) ||
       (pTmrData->pfCallback != NULL))
    {
        return ADI_ADCC_INVALID_OPERATION;
    }

    /* Check if the given pointer to buffer is valid */
    if(ppBuffer == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    pTmrData    =   pTmrDevice->pTmrData;

    /* Wait until buffer is available */
    if(adi_osal_SemPend(pTmrData->hTmrSem, ADI_OSAL_TIMEOUT_FOREVER) != ADI_OSAL_SUCCESS)
    {
        return ADI_ADCC_SEM_FAILED;
    }

    /* Mark the buffer holder as free */
    pTmrData->BufInfo.eBufState =   ADI_ADCC_BUF_STATE_FREE;

    /* Check if we got the semaphore due to an hardware error */
    if(pTmrData->HwErrStatus != 0u)
    {
        pTmrData->HwErrStatus = 0;
        return ADI_ADCC_HW_ERROR_DETECTED;
    }

    /* Return back the buffer pointer */
    *ppBuffer   =   pTmrData->BufInfo.pBuffer;

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Queries if filled buffer is available.
 *
 * @details     This is a non-blocking call which can be used to check if a
 *              filled buffer is available. This API can be used to avoid the
 *              blocking when the #adi_adcc_GetBuffer API is called.
 *
 *
 *              Validity of this API in various operating modes:
 *
 *                  - DMA Mode              : Valid.
 *                  - Linear buffer mode    : Valid.
 *                  - Circular buffer mode  : Not valid.
 *                  - Callback mode         : Not valid.
 *
 * @param [in]  hTimer          Handle to the timer whose buffer status to be
 *                              queried.
 *
 * @param [out] pbAvailable     Pointer to a location where the buffer status
 *                              is written.
 * \n
 *                              - 'true'  -   if buffer is available.
 *                              - 'false' -   if buffer is not available.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully queried the buffer availability.
 *
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given timer handle is invalid.
 *
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to get the buffer status in circular buffering
 *                                      mode or when DMA mode is not enabled or when a callback
 *                                      is installed.
 *
 *  - #ADI_ADCC_NULL_POINTER        [D] If the given pointer to the availability flag is
 *                                      pointing to NULL.
 *
 * @sa adi_adcc_GetBuffer()
 * @sa adi_adcc_IsBufferAvailable()
 * @sa adi_adcc_EnableDMAMode()
 */
ADI_ADCC_RESULT  adi_adcc_IsBufferAvailable (
    ADI_ADCC_TMR_HANDLE const   hTimer,
    bool *const                 pbAvailable
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

#ifdef ADI_DEBUG
    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    pTmrData = pTmrDevice->pTmrData;

    /* Check if we are in linear buffering mode and DMA mode is enabled */
    if((pTmrData->eIOMode != ADI_ADCC_TMR_IOMODE_LINEAR)    ||
       (pTmrData->pADCCDevice->pADCCData->bDMAMode != true) ||
       (pTmrData->pfCallback != NULL))
    {
        return ADI_ADCC_INVALID_OPERATION;
    }

    /* Check if the given pointer is valid */
    if(pbAvailable == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    *pbAvailable =  (pTmrDevice->pTmrData->BufInfo.eBufState == ADI_ADCC_BUF_STATE_PROCESSED) ? true : false;

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Gets the current write pointer.
 *
 * @details     Write pointer is where the DMA is going to write the data
 *              next. The the write buffer pointer is updated at frame level granularity.
 *              It will not be updated when the data for each event is written.
 *
 *              This API returns back with an error #ADI_ADCC_HW_ERROR_DETECTED
 *              if some hardware error(s) are detected. The actual error(s) can be known
 *              by calling the API #adi_adcc_GetHWErrorStatus().
 *
 *              Validity of this API in various operating modes:
 *
 *                  - DMA Mode              : Valid.
 *                  - Linear buffer mode    : Not Valid.
 *                  - Circular buffer mode  : Valid.
 *                  - Callback mode         : Not valid.
 *
 * @param [in]  hTimer          Handle to the timer whose circular buffer write
 *                              pointer to be read.
 *
 * @param [out]  ppWrPtr        Pointer to a location where the write pointer is
 *                              written.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully read write pointer.
 *
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given timer handle is invalid.
 *
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to get the write pointer in linear buffering
 *                                      mode or when DMA mode is not enabled or when a callback
 *                                      is installed.
 *
 *  - #ADI_ADCC_NULL_POINTER        [D] If the given pointer to write pointer is pointing to NULL.
 *
 * @sa adi_adcc_EnableDMAMode()
 * @sa adi_adcc_SubmitCircBuffer()
 * @sa adi_adcc_GetHWErrorStatus()
 */
ADI_ADCC_RESULT  adi_adcc_GetCurrWrPtr (
    ADI_ADCC_TMR_HANDLE         hTimer,
    void **const                ppWrPtr
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

    /* Current frame counter */
    uint32_t             nCurFrameCnt;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

#ifdef ADI_DEBUG
    /* Check if we are in circular buffering mode, DMA mode is enabled and
     * callback is not installed */
    if((pTmrData->eIOMode != ADI_ADCC_TMR_IOMODE_CIRC)      ||
       (pTmrData->pADCCDevice->pADCCData->bDMAMode != true) ||
       (pTmrData->pfCallback != NULL))
    {
        return ADI_ADCC_INVALID_OPERATION;
    }

    /* Check if the given pointer is valid */
    if(ppWrPtr == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    /* If any hardware errors were detected report error */
    if(pTmrData->HwErrStatus != 0u)
    {
        pTmrData->HwErrStatus = 0u;
        return ADI_ADCC_HW_ERROR_DETECTED;
    }

    /* Get the current frame count */
    nCurFrameCnt = (pTmrDevice->pADCCRegs->TMRSTAT[pTmrDevice->nTimerID].STAT & BITM_ADCC_T0STAT_CURFR);

    /* Compute the current frame pointer based on the current frame count and the frame increment */
    *ppWrPtr = ((uint8_t *)pTmrData->BufInfo.pBuffer + (nCurFrameCnt * pTmrDevice->pTmrDMARegs->FRINC));

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Gets the data for the given events.
 *
 * @details     The sampled ADC data for a given event can be read using
 *              this API. The ADC data can be read after getting a callback
 *              with the #ADI_ADCC_EVENT_COMPLETED event. The event argument
 *              for the #ADI_ADCC_EVENT_COMPLETED indicates the event number
 *              that is completed.
 *            
 *              To get the #ADI_ADCC_EVENT_COMPLETED event the event interrupt
 *              mask should be set (using #adi_adcc_SetEventIntMask).
 *
 * @param [in]  hDevice         Handle to device whose event data to be read.
 *
 * @param [in]  pEvtData        Pointer to the table containing the event numbers
 *                              whose data to be read.
 *
 * @param [in]  nEvents         Number of events in the table.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully read the events' data.
 *
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given timer handle is invalid.
 *
 *  - #ADI_ADCC_NULL_POINTER        [D] If the given pointer to event table is NULL.
 *
 *  - #ADI_ADCC_INVALID_EVENT_NUM   [D] If any of the event numbers in the given table
 *                                      are greater than #ADI_ADCC_MAX_EVENTS.
 * 
 * @sa adi_adcc_SetEventIntMask()
 */
ADI_ADCC_RESULT  adi_adcc_GetEventData(
    ADI_ADCC_HANDLE             hDevice,
    ADI_ADCC_EVT_DATA          *pEvtData,
    uint32_t                    nEvents
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

        /* Pointer to the device data */
    ADI_ADCC_DEVICE    *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Index to the event data table */
    uint32_t             nEventIdx;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

#ifdef ADI_DEBUG
    /* Check if the given pointer is valid */
    if(pEvtData == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }
#endif /* ADI_DEBUG */


    for(nEventIdx = 0; nEventIdx < nEvents; nEventIdx++)
    {
#ifdef ADI_DEBUG
        if(pEvtData[nEventIdx].nEvent > ADI_ADCC_MAX_EVENTS)
        {
            return ADI_ADCC_INVALID_EVENT_NUM;
        }
#endif /* ADI_DEBUG */
        pEvtData[nEventIdx].EventData = pDevice->pADCCRegs->EVDAT[pEvtData[nEventIdx].nEvent];
    }

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Enables or disables the ADC Controller.
 *
 * @details     After the ADC Controller is configured and the buffers are
 *              submitted (in DMA mode), it should be enabled to start the
 *              sampling. Also the timer should be enabled (using the API
 *              #adi_adcc_EnableTimer) before enabling the ADCC.
 *
 *              The ADC is programmed with the parameters provided in the
 *              API #adi_adcc_ConfigADC.
 *
 *              To disable the ADCC, disable the timer first and then disable
 *              the ADCC.
 *
 * @note        Timer0 is automatically enabled when the ADCC is enabled. So
 *              it is not required to enable it explicitly.
 *
 * @param [in]  hDevice         Handle to ADCC device which should be enabled
 *                              or disabled.
 *
 * @param [in]  bEnable         Flag which indicates whether to enable or disable
 *                              the ADC Controller.
 * \n
 *                              - 'true'  - to enable.
 *                              - 'false' - to disable.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS             If successfully enabled or disabled the ADCC.
 *  - #ADI_ADCC_INVALID_HANDLE  [D] If the given handle is invalid.
 *
 * @sa  adi_adcc_EnableTimer()
 */
ADI_ADCC_RESULT  adi_adcc_EnableDevice(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bEnable
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE    *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* [2DO] Should we add checks here to see if atleast one of the timer is enabled ?? */

    if(bEnable == true)
    {   
        /* Enable ADC controller and enable the ADC configuration */
        pDevice->pADCCRegs->CTL |=  (BITM_ADCC_CTL_EN | BITM_ADCC_CTL_ADCFG);
        
        /* Wait until ADC configuration is done */
        while((pDevice->pADCCRegs->CFG & BITM_ADCC_CFG_PND) != 0);
   
        pDevice->eADCCState     =   ADI_ADCC_STATE_ENABLED;
    }
    else
    {
        pDevice->pADCCRegs->CTL &=  (~BITM_ADCC_CTL_EN);

        pDevice->eADCCState     =   ADI_ADCC_STATE_NOT_OPENED;
    }

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Enables or disables the ADCC Timer.
 *
 * @details     Before enabling the ADC Controller this API should be
 *              called to enable the given timer.
 *
 *              The corresponding timer trigger is also enabled/disabled when the 
 *              timer is enabled/disabled.
 *
 * @param [in]  hTimer          Handle to timer to be enabled or disabled.
 *
 * @param [in]  bEnable         Flag which indicates whether to enable or disable
 *                              the timer.
 * \n
 *                              - 'true'  - to enable.
 *                              - 'false' - to disable.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully enabled or disabled the timer.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to enable the timer before submitting
 *                                      buffer and when DMA mode is enabled.
 *
 * @sa  adi_adcc_EnableDevice()
 */
ADI_ADCC_RESULT  adi_adcc_EnableTimer(
    ADI_ADCC_TMR_HANDLE const   hTimer,
    bool                        bEnable
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

#ifdef ADI_DEBUG

    /* If DMA mode is enabled, make sure that a buffer is submitted
     * before enabling the timer
     */
    if((bEnable == true) && (pTmrData->pADCCDevice->pADCCData->bDMAMode == true))
    {
        if(pTmrData->BufInfo.pBuffer == NULL)
        {
            return ADI_ADCC_INVALID_OPERATION;
        }
    }
#endif /* ADI_DEBUG */

    if(bEnable == true)
    {
        /* Enable the timer and its corresponding trigger */
        pTmrDevice->pADCCRegs->CTL  |=  ((BITM_ADCC_CTL_TRGIE0 << pTmrDevice->nTimerID) |
                                         (BITM_ADCC_CTL_TMR1EN & (pTmrDevice->nTimerID << BITP_ADCC_CTL_TMR1EN)));

        /* Change the timer state to enabled */
        pTmrData->eTmrState         =   ADI_ADCC_TMR_STATE_ENABLED;
    }
    else
    {
        /* Disable the timer and its corresponding trigger */
        pTmrDevice->pADCCRegs->CTL  &=  (~((BITM_ADCC_CTL_TRGIE0 << pTmrDevice->nTimerID) |
                                         (BITM_ADCC_CTL_TMR1EN & (pTmrDevice->nTimerID << BITP_ADCC_CTL_TMR1EN))));

        /* Change the timer state to opened */
        pTmrData->eTmrState         =   ADI_ADCC_TMR_STATE_OPENED;

        /* Free the buffer that is in use */
        pTmrData->BufInfo.eBufState = ADI_ADCC_BUF_STATE_FREE;
    }

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Configure ADCC timer related parameters.
 *
 * @details     This API cannot called when the timer is enabled.
 *
 * @param [in]  hTimer          Handle to timer to be configured.
 *
 * @param [in]  eTrig           Trigger to be used to start the timer. It can
 *                              one of the triggers from the enumeration
 *                              ADI_ADCC_TRIG.
 *
 * @param [in]  bRiseEdgeTrig   The trigger polarity can be set to falling edge
 *                              or rising edge.
 * \n
 *                              - 'true'  - Rising edge is considered as trigger.
 *                              - 'false' - Falling edge is considered as trigger.
 *
 * @param [in]  bEnableTrigOut  The timer can be configured to enable the trigger
 *                              output so that it can be used to trigger some
 *                              other parts of the system.
 * \n
 *                              - 'true'  -   to enable the trigger output.
 *                              - 'false' -   to disable the trigger output.
 *
 * @note        It is application responsibility to configure the Trigger Routing
 *              Unit to route the required Trigger Master to the ADCC Trigger Slave.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured the timer.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If trying to configure the timer when it is enabled.
 *
 * @sa  adi_adcc_EnableDevice()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigTimer (
    ADI_ADCC_TMR_HANDLE         hTimer,
    ADI_ADCC_TRIG               eTrig,
    bool                        bRiseEdgeTrig,
    bool                        bEnableTrigOut
)
{
    /* Result code */
    ADI_ADCC_RESULT     eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA   *pTmrData;

    /* Control register contents */
    uint32_t            CtlRegValue;

    uint32_t            Mask;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

#ifdef ADI_DEBUG
    if( pTmrData->eTmrState == ADI_ADCC_TMR_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Create mask based on the timer */
    Mask = ((BITM_ADCC_CTL_TRGOE0   << pTmrDevice->nTimerID) |
            (BITM_ADCC_CTL_TRGSEL0  << pTmrDevice->nTimerID) | 
            (BITM_ADCC_CTL_TRGPOL0  << pTmrDevice->nTimerID));

    /* Get the current value of control register and clear the current timer configuration */
    CtlRegValue = pTmrDevice->pADCCRegs->CTL & (~Mask);

    pTmrDevice->pADCCRegs->CTL  =   ( CtlRegValue                                                       |
                                     (bRiseEdgeTrig << (BITP_ADCC_CTL_TRGPOL0 << pTmrDevice->nTimerID)) | 
                                     (eTrig << (BITP_ADCC_CTL_TRGSEL0 << pTmrDevice->nTimerID))         |
                                     (bEnableTrigOut << (BITP_ADCC_CTL_TRGOE0 << pTmrDevice->nTimerID)));

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Configure Chip Select (CS) signal.
 *
 * @details     Configures the CS signal timing and polarity. Each of the ADC
 *              interface CS signal can be configured differently.
 *
 * @param [in]  hDevice         Handle to ADCC device whose ADC CS signal to
 *                              be configured.
 *
 * @param [in]  eADCIF          ADC interface whose CS signal to be configured.
 *                              It can be one of the value from the enumeration
 *                              ADI_ADCC_ADCIF.
 *
 * @param [in]  bActiveHigh     Polarity of the CS signal.
 * \n
 *                              - 'true'  - Active high CS.
 *                              - 'false' - Active low CS.
 *
 * @param [in]  tCSCK           The minimum delay required from the "going active"
 *                              edge of CS till the first edge of clock (in terms
 *                              of ACK cycles). A value of 0 implies 256 clock cycles.
 *
 * @param [in]  tCKCS           The minimum delay after (TCSK + NCK) clock cycles
 *                              have elapsed in a chip select pulse, for which the
 *                              chip select should be active. A value of 0 implies 0
 *                              clock cycle delay.
 *\n
 *                              Where NCK is the number of clock cycles to be given
 *                              with each CS pulse. It can be configured by calling
 *                              the API #adi_adcc_ConfigADCCClock.
 *
 * @param [in]  tCSCS           The minimum delay required between two active edges
 *                              of CS. A value of 0 implies 256 cycles delay.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured the CS signal.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If configuring the chipselect when ADCC device
 *                                      is enabled.
 *
 *
 * @sa  adi_adcc_ConfigADCCClock()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigChipSelect(
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bActiveHigh,
    uint8_t                     tCSCK,
    uint8_t                     tCKCS,
    uint8_t                     tCSCS
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Construct the register TCB register contents based on the given timing parameters */
    uint32_t                tCB =   (tCSCK | (tCKCS << BITP_ADCC_TCB0_TCKCS) | (tCSCS << BITP_ADCC_TCB0_TCSCS));
#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    if(bActiveHigh == true)
    {
        /* Set the chip select polarity bit */
        pDevice->pADCCRegs->CTL |=  (BITM_ADCC_CTL_CSPOL0 << eADCIF);
    }
    else
    {
        /* clear the chip select polarity bit */
        pDevice->pADCCRegs->CTL &=  (~(BITM_ADCC_CTL_CSPOL0 << eADCIF));
    }

    /* Based on the ADC interface write the timing parameter to the appropriate register */
    if(eADCIF == ADI_ADCC_ADCIF0)
    {
        pDevice->pADCCRegs->TCB0    =   tCB;
    }
    else
    {
        pDevice->pADCCRegs->TCB1    =   tCB;
    }

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Configure ADCC Clock (ACK).
 *
 * @details     Configures the ADC clock ratio to system clock, polarity and number
 *              of clock cycles per CS pulse.
 *
 * @param [in]  hDevice         Handle to ADCC device whose clock to be configured.
 *
 * @param [in]  eADCIF          ADC interface whose ADC clock signal to be configured.
 *                              It can be one of the value from the enumeration
 *                              ADI_ADCC_ADCIF.
 *
 * @param [in]  bRisingEdge     Polarity of the ACK signal.
 * \n
 *                              - 'true'  - Positive edge after CS signal goes active.
 *                              - 'false' - Negative edge after CS signal goes active.
 *
 * @param [in]  nClkDiv         The ratio of ACK frequency to the system clock
 *                              frequency.
 *
 *                              ACK frequency = SCLK frequency /(1 + nClkDiv).
 *
 * @param [in]  nNumACK         The number of ACK cycles to be given per each CS pulse.
 *                              A value of 0 implies 16 cycles.
 *
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured the ADC Clock.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If configuring the ADC clock when ADCC device
 *                                      is enabled.
 *  - #ADI_ADCC_INVALID_PARAMETER   [D] If one of the given parameters are not valid.
 *
 * @sa  adi_adcc_ConfigChipSelect()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigADCCClock (
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bRisingEdge,
    uint16_t                    nClkDiv,
    uint8_t                     nNumACK
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* TCA register value */
    uint32_t                tCA = ((nNumACK << BITP_ADCC_TCA1_NCK) | nClkDiv);

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }

    /* Check if the given number of ADC clock for each CS pulse is valid */
    if(nNumACK >= 16u)
    {
        return ADI_ADCC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if(bRisingEdge == true)
    {
        /* Set the clock polarity bit */
        pDevice->pADCCRegs->CTL |=  (BITM_ADCC_CTL_CKPOL0 << eADCIF);
    }
    else
    {
        /* clear the clock polarity bit */
        pDevice->pADCCRegs->CTL &=  (~(BITM_ADCC_CTL_CKPOL0 << eADCIF));
    }

    /* Based on the ADC interface write the timing parameter to the appropriate register */
    if(eADCIF == ADI_ADCC_ADCIF0)
    {
        pDevice->pADCCRegs->TCA0    =   tCA;
    }
    else
    {
        pDevice->pADCCRegs->TCA1    =   tCA;
    }

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Configure data interface.
 *
 * @details     Configures the data signals swap (when using dual bit data interface)
 *              and the significance of data (LSB first or MSB first). The dual bit
 *              data interface is a ADC controller wide setting and can be enabled
 *              by using the API #adi_adcc_EnableDualBitDataIF().
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose data interface to be
 *                              configured.
 *
 * @param [in]  eADCIF          ADC interface whose data interface to be configured.
 *                              It can be one of the value from the enumeration
 *                              ADI_ADCC_ADCIF.
 *
 * @param [in]  bSwap           Flag to indicate if data lines swap is required.
 *                              This is applicable only when dual bit data interface is
 *                              enabled using the API #adi_adcc_EnableDualBitDataIF.
 * \n
 *                              - 'true'  - Enable data swapping.
 *                              - 'false' - Disable data swapping.
 *
 * @param [in]  bLSBF           Flag to indicate the significance of the data. LSB first
 *                              or MSB first.
 * \n
 *                              - 'true'  -   LSB first.
 *                              - 'false' -   MSB first.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured the data interface.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If configuring the data interface when ADCC device
 *                                      is enabled.
 *
 * @sa  adi_adcc_EnableDualBitDataIF()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigDataIF(
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bSwap,
    bool                        bLSBF
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Control register value */
    uint32_t                CtrlRegValue;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Get the current Control register value and clear the Data Swap and LSB first fields */
    CtrlRegValue    =   ((pDevice->pADCCRegs->CTL) & (~((BITM_ADCC_CTL_DSWP0 | BITM_ADCC_CTL_LSBF0) << eADCIF)));

    /* Set the given Data Swap and LSB first settings */
    pDevice->pADCCRegs->CTL =   (CtrlRegValue |
                                ((bSwap << BITP_ADCC_CTL_DSWP0) | (bLSBF << BITP_ADCC_CTL_LSBF0)) << eADCIF);

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Configure control interface to the ADC.
 *
 * @details     Configures the control interface idling mode. Whether it should
 *              be held high or low during idle period.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose control interface to be
 *                              configured.
 *
 * @param [in]  eADCIF          ADC interface whose control interface to be configured.
 *                              It can be one of the value from the enumeration
 *                              ADI_ADCC_ADCIF.
 *
 * @param [in]  bIdleHigh       Flag to indicate whether to idle the control line high
 *                              or low.
 * \n
 *                              - 'true'  - Enable data swapping.
 *                              - 'false' - Disable data swapping.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured the control interface.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If configuring the control interface when ADCC device
 *                                      is enabled.
 * @sa  adi_adcc_EnableDualBitCtrlIF()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigCtrlIF(
    ADI_ADCC_HANDLE const       hDevice,
    ADI_ADCC_ADCIF              eADCIF,
    bool                        bIdleHigh
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    if(bIdleHigh == true)
    {
        /* Set the idle high bit */
        pDevice->pADCCRegs->CTL |=  (BITM_ADCC_CTL_TIDLE0 << eADCIF);
    }
    else
    {
        /* clear the idle high bit */
        pDevice->pADCCRegs->CTL &=  (~(BITM_ADCC_CTL_TIDLE0 << eADCIF));
    }

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Set the ADC configuration word.
 *
 * @details     The configuration word is sent to the ADC when the ADCC is
 *              enabled. The triggers that arrive during the ADC configuration
 *              are ignored.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose ADC to be configured.
 *
 * @param [in]  bExtRef         Flag to indicate whether to use internal (default)
 *                              or external reference.
 * \n
 *                              - 'true'  - Use external reference.
 *                              - 'false' - Use internal reference.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully set the ADC configuration word.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If configuring the ADC when the device is enabled.
 *
 * @sa  adi_adcc_EnableDevice()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigADC (
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bExtRef
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* ADC Configuration register value */
    uint32_t                CfgRegValue;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Get the current contents of ADC configuration register */
    CfgRegValue =   (pDevice->pADCCRegs->CFG & (~BITM_ADCC_CFG_REFSEL));

    /* Write back the register value with the new settings */
    pDevice->pADCCRegs->CFG |=  (CfgRegValue |
                                (bExtRef << BITP_ADCC_CFG_REFSEL));

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Enable or disable dual bit data interface.
 *
 * @details     Sets the width of the data interface to the ADC. one bit or two
 *              bit interface.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose data width to be
 *                              configured.
 *
 * @param [in]  bDualBit        Flag to indicate whether to single bit (default)
 *                              or dual bit data interface to the ADC.
 * \n
 *                              - 'true'  - Dual bit data interface.
 *                              - 'false' - Single bit data interface.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured data width.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If configuring data interface width when the device is enabled.
 *
 * @sa  adi_adcc_EnableDualBitCtrlIF()
 */
ADI_ADCC_RESULT  adi_adcc_EnableDualBitDataIF(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bDualBit
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    if(bDualBit == true)
    {
        pDevice->pADCCRegs->CTL |=  BITM_ADCC_CTL_DSIZE;
    }
    else
    {
        pDevice->pADCCRegs->CTL &=  (~BITM_ADCC_CTL_DSIZE);
    }

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Enable Dual bit control interface.
 *
 * @details     Sets the width of the control interface to the ADC. one bit or two
 *              bit control interface.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose control width to be
 *                              configured.
 *
 * @param [in]  bDualBit        Flag to indicate whether to single bit (default)
 *                              or dual bit control interface to the ADC.
 * \n
 *                              - 'true'  - Dual bit control interface.
 *                              - 'false' - Single bit control interface.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured control width.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If configuring control interface width when the device is enabled.
 *
 * @sa  adi_adcc_EnableDualBitDataIF()
 */
ADI_ADCC_RESULT  adi_adcc_EnableDualBitCtrlIF(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bDualBit
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    if(bDualBit == true)
    {
        pDevice->pADCCRegs->CTL |=  BITM_ADCC_CTL_CSIZE;
    }
    else
    {
        pDevice->pADCCRegs->CTL &=  (~BITM_ADCC_CTL_CSIZE);
    }

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Enable or disable DMA mode.
 *
 * @details     When DMA mode is enabled, DMA is used to get the data from the
 *              event data registers to memory otherwise the data should be read
 *              by the core. Whenever possible it would be advantageous to the
 *              application to use DMA mode so that core cycles are not wasted
 *              to get the data from event data registers.
 *
 *              By default DMA mode is disabled. Application should explicitly
 *              call this API to enable the DMA mode.
 *
 *              When DMA mode is not used none of the data transaction APIs
 *              can be used. If DMA mode is not used application should use the
 *              #adi_adcc_GetEventData API to read the data from the event data
 *              registers. The list of APIs that cannot be used are:
 *
 *              #adi_adcc_SubmitBuffer()
 *              #adi_adcc_SubmitCircBuffer()
 *              #adi_adcc_GetBuffer()
 *              #adi_adcc_GetCurrWrPtr()
 *              #adi_adcc_IsBufferAvailable()
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose DMA mode to be enabled
 *                              or disabled.
 *
 * @param [in]  bEnable         Flag to indicate whether to enable or disable (default)
 *                              DMA mode.
 * \n
 *                              - 'true'  - Enable DMA mode.
 *                              - 'false' - Disable DMA mode.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully enabled or disabled the DMA mode.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If DMA mode is being enabled/disabled when the device is enabled.
 */
ADI_ADCC_RESULT  adi_adcc_EnableDMAMode(
    ADI_ADCC_HANDLE const       hDevice,
    bool                        bEnable
)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Result code */
    ADI_ADCC_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Check if device is already enabled */
    if(pDevice->eADCCState  ==   ADI_ADCC_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    if(bEnable == true)
    {
        pDevice->pADCCRegs->CTL |=  BITM_ADCC_CTL_DMAEN;
    }
    else
    {
        pDevice->pADCCRegs->CTL &=  (~BITM_ADCC_CTL_DMAEN);
    }

    pDevice->pADCCData->bDMAMode    =   bEnable;

    return ADI_ADCC_SUCCESS;
}

/**
 * @brief       Configure the events.
 *
 * @details     Configures the event related parameters. A set of events can be
 *              configured at once by passing the configuration table for the
 *              events to be configured.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose event(s) to be configured.
 *
 * @param [in]  pEventCfg       Pointer to the event configuration table.
 *
 * @param [in]  nNumEvents      Number of entries in the event configuration table.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured the given events.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_EVENT_NUM   [D] If the given event number in the event configuration
 *                                      table entry is invalid.
 *
 * @sa  adi_adcc_SetEventMask()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigEvent(
    ADI_ADCC_HANDLE             hDevice,
    ADI_ADCC_EVENT_CFG *        pEventCfg,
    uint32_t                    nNumEvents
)
{
    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

    /* Index to the event data table */
    uint32_t                nEventIdx;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */


    for(nEventIdx = 0; nEventIdx < nNumEvents; nEventIdx++)
    {
#ifdef ADI_DEBUG
        /* Validate the given evnet number */
        if(pEventCfg->nEvent > ADI_ADCC_MAX_EVENTS)
        {
            return ADI_ADCC_INVALID_EVENT_NUM;
        }
#endif /* ADI_DEBUG */

        /* Set the givent event configuration */
        pDevice->pADCCRegs->EVT_REG[pEventCfg->nEvent].EVCTL   = ((pEventCfg->AdcCtlWord)                           |
                                                                  (pEventCfg->eAdcIF     << BITP_ADCC_EVCTL_ADCSEL) |
                                                                  (pEventCfg->eTimer     << BITP_ADCC_EVCTL_TMRSEL) |
                                                                  (pEventCfg->bSimSamp   << BITP_ADCC_EVCTL_SIMSAMP)|
                                                                  (pEventCfg->nMemOffset << BITP_ADCC_EVCTL_EVTOFS));
        /* Set the given event time */
        pDevice->pADCCRegs->EVT_REG[pEventCfg->nEvent].EVT     =  pEventCfg->nEventTime;


        /* Set the mask for the timers that are configured */
        pDevice->TmrDevice[pEventCfg->eTimer].pTmrData->TmrMask |=  (1u << pEventCfg->nEvent);

        pEventCfg++;
    }

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Set event mask.
 *
 * @details     Set the mask to enable or disable the events. Each bit in the
 *              mask indicates whether to enable that particular event or
 *              disable it.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose event mask to be set.
 *
 * @param [in]  nEventMask      A bit mask which to enable or disable the events.
 *                              A '1' at a bit position "N" enables the event "N" and
 *                              a '0' disables it.
 *
 * @note    The event configuration that is done using the API #adi_adcc_ConfigEvent
 *          persists even if that particular event is disabled.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully set enabled/disabled the events.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *
 *  @sa adi_adcc_ConfigEvent()
 */
ADI_ADCC_RESULT  adi_adcc_SetEventMask (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nEventMask
)
{
    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Clear the existing events */
    pDevice->pADCCRegs->EVTEN_CLR   =   0xFFFFFFFFu;

    /* Set the new events */
    pDevice->pADCCRegs->EVTEN_SET   =   nEventMask;

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Set event interrupt mask.
 *
 * @details     Set the event interrupt mask to enable or disable the event
 *              interrupt mask. This API is useful when not using DMA to get
 *              the event data. 
 *              
 *              The event interrupt mask can be set to get the event completion
 *              notification #ADI_ADCC_EVENT_COMPLETED via the callback. The
 *              event data can be read back (using adi_adcc_GetEventData) upon 
 *              getting this notification.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose event interrupt mask to be set.
 *
 * @param [in]  nIntEventMask   A bit mask which to enable or disable the event interrupt.
 *                              A '1' at a bit position "N" enables the event "N" 
 *                              interrupt and a '0' disables it.
 *
 * @note        Then event interrupt will generated for only events that are 
 *              enabled (using #adi_adcc_SetEventMask).
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully set enabled/disabled the event interrupts.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *
 *  @sa adi_adcc_SetEventMask()
 *  @sa adi_adcc_GetEventData()
 */
ADI_ADCC_RESULT  adi_adcc_SetEventIntMask (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nEventIntMask
)
{
    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Mask all the event interrupts */
    pDevice->pADCCRegs->EIMSK_SET   =   0xFFFFFFFFu;

    /* Enable the given event interrupts */
    pDevice->pADCCRegs->EIMSK_CLR   =   nEventIntMask;

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Configure Bandwidth monitoring.
 *
 * @details     Bandwidth monitoring feature can be used to verify if all the
 *              events are completed within the intended period.An error will
 *              be signaled if the timers count beyond the set value before
 *              the DMA transfer for all the events is not completed.

 * @note        This feature works only in DMA mode.
 *
 * @note        This API can only be used before ADC controller is enabled.
 *
 * @param [in]  hTimer          Handle to timer whose Bandwidth monitoring
 *                              to be configured.
 *
 * @param [in]  nCount          The maximum expected count for the Timer.
 *                              Setting a value of 0 disables bandwidth monitoring.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS                 If successfully configured bandwidth monitoring.
 *  - #ADI_ADCC_INVALID_HANDLE      [D] If the given handle is invalid.
 *  - #ADI_ADCC_INVALID_OPERATION   [D] If setting the bandwidth monitoring count when
 *                                      the timer is enabled.
 *
 * @sa adi_adcc_EnableDMAMode()
 */
ADI_ADCC_RESULT  adi_adcc_ConfigBWMon(
    ADI_ADCC_TMR_HANDLE         hTimer,
    uint32_t                    nCount
)
{
    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE     *pTmrDevice = (ADI_ADCC_TMR_DEVICE *) hTimer;

    /* Pointer to the Timer data */
    ADI_ADCC_TMR_DATA       *pTmrData;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateTimer(pTmrDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    pTmrData = pTmrDevice->pTmrData;

#ifdef ADI_DEBUG
    if( pTmrData->eTmrState == ADI_ADCC_TMR_STATE_ENABLED)
    {
        return ADI_ADCC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Set the given bandwidth count based on the given timer handle */
    if(pTmrDevice->nTimerID    ==  0u)
    {
        pTmrDevice->pADCCRegs->BWMON0   =   nCount;
    }
    else
    {
        pTmrDevice->pADCCRegs->BWMON1   =   nCount;
    }

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Sets a given ADCC register.
 *
 * @details     Any ADCC register can be set using this API. The driver provides
 *              APIs to set register fields for most common use case. For any
 *              registers that are not covered by other APIs, this API can be
 *              used to set those.
 *
 * @param [in]  hDevice         Handle to ADCC device whose registers to be set.
 *
 * @param [in]  nRegAddr        Address of the register to be set.
 *
 * @param [in]  nRegValue       Register value to be set.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS             If successfully set the given register.
 *  - #ADI_ADCC_INVALID_HANDLE  [D] If the given handle is invalid.
 *
 * @sa adi_adcc_GetRegister()
 */
ADI_ADCC_RESULT  adi_adcc_SetRegister (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t                    nRegValue
)
{
    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Set the given register value */
    *((volatile uint32_t *)nRegAddr) = nRegValue;

    return ADI_ADCC_SUCCESS;
}


/**
 * @brief       Gets the contents of given ADCC register .
 *
 * @details     This API can be used to read any of the ADCC register. This API
 *              can be used to read the event data registers when DMA mode is not
 *              enabled.
 *
 * @param [in]  hDevice         Handle to ADCC device whose registers to be read.
 *
 * @param [in]  nRegAddr        Address of the register to be read.
 *
 * @param [in]  pRegValue       Pointer to a location where the given register
 *                              contents will be written.
 *
 * @return      Status
 *  - #ADI_ADCC_SUCCESS             If successfully read the register contents.
 *  - #ADI_ADCC_INVALID_HANDLE  [D] If the given handle is invalid.
 *  - #ADI_ADCC_NULL_POINTER    [D] If the given pointer to the register value is
 *                                  pointing to NULL.
 * @sa adi_adcc_SetRegister()
 */
ADI_ADCC_RESULT  adi_adcc_GetRegister (
    ADI_ADCC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t * const            pRegValue
)
{
    /* Result code */
    ADI_ADCC_RESULT         eResult;

    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) hDevice;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_ADCC_SUCCESS)
    {
        return eResult;
    }

    /* Validate the given pointer */
    if(pRegValue == NULL)
    {
        return ADI_ADCC_NULL_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Get the register value */
    *pRegValue   =   *((volatile uint32_t *)nRegAddr);

    return ADI_ADCC_SUCCESS;
}


/* Error interrupt handler */
static void ErrorIntHandler(uint32_t iid, void *pCBParam)
{
    /* Pointer to the device data */
    ADI_ADCC_DEVICE         *pDevice = (ADI_ADCC_DEVICE *) pCBParam;

    /* Error event */
    ADI_ADCC_ERR            eError;

    /* Error argument */
    uint32_t                ErrArg = 0u;

    /* Error status register value */
    uint32_t                ErrStat =   pDevice->pADCCRegs->ERRSTAT;

    /* Mask to clear the status register */
    uint32_t                Mask = 0u;

    if((Mask = (ErrStat & BITM_ADCC_ERRSTAT_TRGOV0)) != 0u)
    {
        eError  =   ADI_ADCC_ERR_TRIGGER_OVERRUN;
    }
    else if((Mask = (ErrStat & BITM_ADCC_ERRSTAT_TRGOV1)) != 0u)
    {
        eError  =   ADI_ADCC_ERR_TRIGGER_OVERRUN;

        /* Timer ID for which the overrun occurred */
        ErrArg  =   1;
    }
    else if((Mask = (ErrStat & BITM_ADCC_ERRSTAT_BWERR0)) != 0u)
    {
        eError  =   ADI_ADCC_ERR_BANDWIDTH_ERROR;
    }
    else if((Mask = (ErrStat & BITM_ADCC_ERRMSK_CLR_BWERR1)) != 0u)
    {
        eError  =   ADI_ADCC_ERR_BANDWIDTH_ERROR;

        /* Timer ID for which the bandwidth error occurred */
        ErrArg  =   1;
    }
    else if((Mask = (ErrStat & BITM_ADCC_ERRSTAT_MERR0)) != 0u)
    {
        eError  =   ADI_ADCC_ERR_MEMORY_ERROR;
    }
    else if((Mask = (ErrStat & BITM_ADCC_ERRSTAT_MERR1)) != 0u)
    {
        eError  =   ADI_ADCC_ERR_MEMORY_ERROR;

        /* Timer ID for which the memory error occurred */
        ErrArg  =   1;
    }
    else if((ErrStat & BITM_ADCC_ERRSTAT_ECOL) != 0u)
    {
        eError  =   ADI_ADCC_ERR_EVENT_COLLISION;

        /* Get the events that caused the collision error */
        ErrArg  =   pDevice->pADCCRegs->ECOL;

        /* Clear the events that caused the collision error */
        pDevice->pADCCRegs->ECOL = ErrArg;
    }
    else if((ErrStat & BITM_ADCC_ERRSTAT_EMIS) != 0u)
    {
        eError  =   ADI_ADCC_ERR_EVENT_MISS;

        /* Get the events that caused the event miss error */
        ErrArg  =   pDevice->pADCCRegs->EMISS;

        /* Clear the events that caused the event miss error */
        pDevice->pADCCRegs->EMISS = ErrArg;
    }
    else
    {
        /* ideally we should not reach here */
        return;
    }

    /* Clear status bits which are handled */
    pDevice->pADCCRegs->ERRSTAT =   Mask;

    /* if a callback is registered report the errors */
    if(pDevice->pADCCData->pfCallback != NULL)
    {
        pDevice->pADCCData->pfCallback(pDevice->pADCCData->pCBParam, eError, (void *)ErrArg);
    }

    /* If there is a timer related error and if we are in non-blocking mode post the semaphore */
    if((eError != ADI_ADCC_ERR_EVENT_MISS) && (eError != ADI_ADCC_ERR_EVENT_COLLISION))
    {
        /* Pointer to the timer data */
        ADI_ADCC_TMR_DATA   *pTmrData =   pDevice->TmrDevice[ErrArg].pTmrData;

        if(pTmrData->pfCallback == NULL)
        {
            /* Indicate that there is a hardware error */
            pTmrData->HwErrStatus   =   1u;

            /* Post the semaphore */
            adi_osal_SemPost(pTmrData->hTmrSem);
        }
    }

    return;
}

/* Timer related interrupt handler */
static void TimerIntHandler(uint32_t iid, void *pCBParam)
{

    /* Pointer to the timer device */
    ADI_ADCC_TMR_DEVICE     *pTmrDevice =   (ADI_ADCC_TMR_DEVICE *) pCBParam;

    /* Pointer to the ADCC registers */
    volatile ADI_ADCC_REG   *pADCCRegs  =   pTmrDevice->pADCCRegs;

    /* Pointer to the timer data */
    ADI_ADCC_TMR_DATA       *pTmrData   =   pTmrDevice->pTmrData;

    /* Event interrupt status register value */
    uint32_t EventStat  =   pADCCRegs->EISTAT;

    /* Frame interrupt status register value */
    uint32_t FrameStat  =   pADCCRegs->FISTAT;

    if((FrameStat = (FrameStat & (BITM_ADCC_FISTAT_FINT0 << pTmrDevice->nTimerID))) != 0)
    {
        /* Get the current frame count */
        uint32_t nCurFrameCnt = (pADCCRegs->TMRSTAT[pTmrDevice->nTimerID].STAT & BITM_ADCC_T0STAT_CURFR);

        /* Clear the frame interrupt status */
        pADCCRegs->FISTAT   =   FrameStat;

        /* if a callback is registered call it otherwise post a semaphore */
        if(pTmrData->pfCallback != NULL)
        {
            /* Compute the processed frame pointer based on the current frame count and the frame increment */
            uint8_t *pFrame = ((uint8_t *)pTmrData->BufInfo.pBuffer + ((nCurFrameCnt - 1) * pTmrDevice->pTmrDMARegs->FRINC));

            pTmrData->pfCallback(pTmrData->pCBParam, ADI_ADCC_EVENT_FRAME_PROCESSED, pFrame);
        }
        
        /* Only when a buffer is submitted by application (buffer submission is allowed only in DMA mode) */
        if(pTmrData->BufInfo.eBufState  ==      ADI_ADCC_BUF_STATE_INUSE)
        {
          /* If we are in linear mode and the buffer is processed */
          if((pTmrData->eIOMode    ==  ADI_ADCC_TMR_IOMODE_LINEAR) && (nCurFrameCnt >= pTmrData->BufInfo.nBufSize))
          {
              /* if a callback is registered notify buffer processed event */
              if(pTmrData->pfCallback != NULL)
              {
                  /* Mark the buffer as free */
                  pTmrData->BufInfo.eBufState     =       ADI_ADCC_BUF_STATE_FREE;
                  pTmrData->pfCallback(pTmrData->pCBParam, ADI_ADCC_EVENT_BUFFER_PROCESSED, pTmrData->BufInfo.pBuffer);
              }
              else
              {
                  adi_osal_SemPost(pTmrData->hTmrSem);
              }
          }
       }
    }

    /* If there are any event status bits are set */
    if((EventStat = (EventStat & pTmrData->TmrMask)) != 0u)
    {
        /* Clear the events that are corresponding to this timer */
        pADCCRegs->EISTAT   =   EventStat;

        /* If a callback is registered report the events */
        if(pTmrData->pfCallback != NULL)
        {
            pTmrData->pfCallback(pTmrData->pCBParam, ADI_ADCC_EVENT_COMPLETED, (void *)EventStat);
        }
    }

    return;
}


#ifdef ADI_DEBUG
/* Validate the device */
static ADI_ADCC_RESULT ValidateDevice(ADI_ADCC_DEVICE   *pDevice)
{
    uint32_t i;

    /* Check if the given pointer matches with any of the devices */
    for(i = 0; i < ADI_ADCC_NUM_DEVICES; i++)
    {
        /* If found a match return success */
        if(pDevice == &gADCCDevices[i])
        {
            return ADI_ADCC_SUCCESS;
        }
    }

    /* Could not find the match return failure */
    return ADI_ADCC_INVALID_HANDLE;
}


/* Validate the timer */
static ADI_ADCC_RESULT ValidateTimer(ADI_ADCC_TMR_DEVICE    *pTmrDevice)
{
    uint32_t i, j;

    /* Check if the given pointer matches with any of the devices */
    for(i = 0; i < ADI_ADCC_NUM_DEVICES; i++)
    {
        for(j = 0; j < ADI_ADCC_MAX_TIMERS; j++)
        {
            /* If found a match return success */
            if(pTmrDevice == &gADCCDevices[i].TmrDevice[j])
            {
                return ADI_ADCC_SUCCESS;
            }
        }
    }

    /* Could not find the match return failure */
    return ADI_ADCC_INVALID_HANDLE;
}
#endif /* ADI_DEBUG */



#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*@}*/
