/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_crc.c
 * @brief     CRC Peripheral driver.
 * @version:  $Revision: 18424 $
 * @date:     $Date: 2013-04-12 02:50:36 -0400 (Fri, 12 Apr 2013) $
 *
 * @details
 *            This is the the primary source file for
 *            Cyclic Redundancy Check (CRC) Peripheral Driver
 *
 */

/** @addtogroup CRC_Driver Cyclic Redundancy Check (CRC) peripheral driver
 *  @{

<b>Cyclic Redundancy Check (CRC) peripheral driver</b>

The CRC peripheral is used to perform the Cyclic Redundancy Check (CRC) of the
block of data that is presented to the peripheral. The peripheral provides a
means to periodically verify the integrity of the system memory, the contents
of memory-mapped registers (MMRs), or communication message objects, and it is
based on a CRC32 engine that computes the signature of 32-bit data presented
to the peripheral. The CRC hardware compares the calculated signature of the
operation to a  pre-loaded expected signature and if the two fail to match,
the peripheral generates an error.

CRC use cases include the following features:

    - Memory scan with CRC compute or compare
    - Memory transfer with CRC compute or compare
    - Memory fill with 32-bit data patterns
    - Memory verify

CRC also provides Data mirroring options for endian and reflected polynomial
cases and this can be achieved using adi_crc_SetDataMirror() API.

<b>Memory DMA driver CRC operations</b>

The CRC module can use Memory DMA stream to perform its operations.
The CRC DMA mode can be enabled using adi_crc_EnableDmaMode() API.

<em><b>Memory Transfer Compute Compare Mode</b></em>

The memory transfer can be used to transfer a block of data from one region
of memory to another. Along with data transfer, the CRC engine also performs
a 32-bit CRC operation on the incoming data stream.

The initial seed of CRC computation can be configured using
adi_crc_SetAutoClear() and adi_crc_SetCrcSeedVal() APIs.

Sequence of function calls for Memory Transfer Compute Compare

    - adi_crc_Open() to open CRC device.
    - adi_crc_StopReset() resets the CRC registers.
    - adi_crc_SetPolynomialVal() sets the polynomial value to be used for CRC operation.
    - adi_crc_SetDataMirror() configures data mirroring.
    - adi_crc_EnableDmaMode() enables Memory DMA for CRC.
    - adi_crc_SetOperatingMode() sets CRC operating mode for Memory Transfer Compute Compare.
    - adi_crc_SetDataCount() sets data word and reload count.
    - adi_crc_SetExpectedVal() sets expected CRC value for the incoming data stream. Pass
      expected value parameter as zero if the CRC value is unknown.
    - adi_crc_DmaMemXfer() submits data buffer for Memory Transfer-Compute-Compare.
    - adi_crc_IsCrcInProgress() to poll the current status of CRC operation or
      wait for callback event.
    - adi_crc_GetFinalCrcVal() to get the CRC value of the data stream if its
      CRC value is unknown.

<em><b>Memory Transfer Data Fill</b></em>

This mode can be used to initialize a region of memory with a pre-loaded 32-bit constant value.

Sequence of function calls for Memory Transfer Data Fill

    - adi_crc_Open() to open CRC device.
    - adi_crc_StopReset() resets the CRC registers.
    - adi_crc_SetDataMirror() configures data mirroring.
    - adi_crc_EnableDmaMode() enables Memory DMA for CRC.
    - adi_crc_SetOperatingMode() sets CRC operating mode for Memory Transfer Data Fill.
    - adi_crc_SetDataCount() sets data word and reload count.
    - adi_crc_SetMemFillVal() sets 32-bit value to be used to fill memory region.
    - adi_crc_DmaMemFillScan() submits data buffer to be filled.
    - adi_crc_IsCrcInProgress() to poll the current status of CRC operation or
      wait for callback event.

<em><b>Memory Scan Compute Compare</b></em>

Compute and Compare performs a CRC calculation on data presented to the CRC module and
compares the CRC result with a pre-determined and pre-loaded result. An error is
generated if the results differ.

Sequence of function calls for Memory Scan Compute Compare

    - adi_crc_Open() to open CRC device.
    - adi_crc_StopReset() resets the CRC registers.
    - adi_crc_SetPolynomialVal() sets the polynomial value to be used for CRC operation.
    - adi_crc_SetDataMirror() configures data mirroring.
    - adi_crc_EnableDmaMode() enables Memory DMA for CRC.
    - adi_crc_SetOperatingMode() sets CRC operating mode for Memory Scan Compute Compare.
    - adi_crc_SetDataCount() sets data word and reload count.
    - adi_crc_SetExpectedVal() sets expected CRC value for the incoming data stream. Pass
      expected value parameter as zero if the CRC value is unknown.
    - adi_crc_DmaMemFillScan() submits data buffer for Memory Scan-Compute-Compare.
    - adi_crc_IsCrcInProgress() to poll the current status of CRC operation or
      wait for callback event.
    - adi_crc_GetFinalCrcVal() to get the CRC value of the data stream if its
      CRC value is unknown.

<em><b>Memory Scan Data Verify</b></em>

This mode compares each 32-bit data word presented to the CRC module to a
pre-loaded 32-bit value and generates an error if the data is found to be different.

Sequence of function calls for Memory Scan Data Verify

    - adi_crc_Open() to open CRC device.
    - adi_crc_StopReset() resets the CRC registers.
    - adi_crc_SetDataMirror() configures data mirroring.
    - adi_crc_EnableDmaMode() enables Memory DMA for CRC.
    - adi_crc_SetOperatingMode() sets CRC operating mode for Memory Scan Data Verify.
    - adi_crc_SetDataCount() sets data word and reload count.
    - adi_crc_SetMemFillVal() sets the 32-bit value that is to be used to validate a memory region.
    - adi_crc_DmaMemFillScan() submits data buffer for Memory Scan Data Verify.
    - adi_crc_IsCrcInProgress() to poll the current status of CRC operation or
      wait for callback event.

<b>CRC Trigger Functionality</b>

The trigger can be used to synchronize other processing in the system. A DMA driven CRC module
can be configured to wait for a trigger input before processing a buffer and/or to
generate a trigger output after processing a data buffer. The trigger input to CRC
can be routed from a trigger master. Similarly, the trigger output generated by CRC
can be routed to one or more slave devices. Trigger routing can be done by calling the
Trigger Routing Unit APIs.

Trigger configuration can be set on a per-buffer basis. To configure a buffer with specific
trigger settings, the Trigger API should be called before and after submitting the buffer.

CRC trigger configuration APIs are:

    - adi_crc_ConfigCrcInTriggerIn()
    - adi_crc_ConfigCrcOutTriggerIn()
    - adi_crc_ConfigCrcInTriggerOut()
    - adi_crc_ConfigCrcOutTriggerOut()

The trigger APIs should be selectively called depending up on the CRC operating mode.

The APIs with string 'CrcIn' configures to CRC input (Source) DMA channel and should be used
only with operating modes that uses the CRC input channel, such as Memory Transfer Compute Compare,
Memory Scan Compute Compare and Memory Scan Data Verify.

The APIs with string 'CrcOut' configures to CRC output (Destination) DMA channel and should be used
only with operating modes that uses the CRC output channel, such as Memory Transfer Compute Compare
and Memory Transfer Data Fill.

The following snippet shows how to configure CRC input channel to wait for a trigger input.

@code

// Enable trigger input, disable overrun error
adi_crc_ConfigCrcInTriggerIn(hCrcDev, true, false);

// Submit data buffer(s) to CRC device

// Disable trigger input configuration
adi_crc_ConfigCrcInTriggerIn(hCrcDev, false, false);

@endcode

The following snippet shows how to configure CRC output channel to generate an outgoing trigger.

@code

// Enable trigger output
adi_crc_ConfigCrcInTriggerOut(hCrcDev, ADI_CRC_TRIGGER_OUT_DMA_DONE);

// Submit data buffer(s) to CRC device

// Disable trigger output configuration
adi_crc_ConfigCrcInTriggerOut(hCrcDev, ADI_CRC_TRIGGER_OUT_NONE);

@endcode

 */

/*=============  I N C L U D E S   =============*/

/* Header file with definitions specific to CRC driver implementation */
#include "adi_crc_def.h"

/*=============  C R C    I M P L E M E N T A T I O N    S O U R C E    F I L E S  =============*/

/* Processor specific data for CRC Driver */
/* IF (ADSP-BF606/607/608/609) */
#if defined( __ADSPBF609_FAMILY__)
/* ADSP-BF60x Processor specific data for CRC Driver */
#include "adi_crc_data_bf60x.c"
/* ELSE (ADSP-CM402/CM403/CM407/CM408 processors) */
#elif defined (ADI_ADSP_CM40Z)
/* ADSP-CM40z Processor specific data for CRC Driver */
#include "adi_crc_data_cm40z.c"
/* ELSE */
#else
#error "CRC Driver not available for this processor"
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_8_5:" ")
#pragma diag(suppress:misra_rule_8_7:" ")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#pragma diag(suppress:misra_rule_10_3:" ")
#pragma diag(suppress:misra_rule_11_3:" ")
#pragma diag(suppress:misra_rule_12_2:" ")
#pragma diag(suppress:misra_rule_12_7:" ")
#pragma diag(suppress:misra_rule_13_7:" ")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exit points")
#pragma diag(suppress:misra_rule_16_7:" ")
#endif /* _MISRA_RULES */

/*==============  D E F I N E S  ===============*/

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/

/* Resets CRC registers */
static void ResetCrcRegs (ADI_CRC_INFO const *pCrcInfo);
/* Opens a Memory DMA stream for CRC operation */
static ADI_CRC_RESULT OpenMemDmaStream (ADI_CRC_INFO *pCrcInfo);
/* Closes the Memory DMA stream opened for CRC operation */
static ADI_CRC_RESULT CloseMemDmaStream (ADI_CRC_INFO const *pCrcInfo);
/* Starts Memory DMA driven CRC operation */
static ADI_CRC_RESULT StartMemDmaCrc(
    ADI_CRC_INFO const  *pCrcInfo,
    void                *pSrcBuf,
    void                *pDestBuf,
    uint32_t            Num32bitElements);
/* Ends CRC operation */
static void EndCrcOperation (ADI_CRC_INFO const   *pCrcInfo);
/* Callback from MDMA Manager */
static void MemDmaCallback(void *pCBParam, uint32_t Event, void *pArg);
/* Interrupt service routine for CRC Data Count expired event */
static void CrcDataCountHandler(uint32_t SID, void *pCBParam);

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validates CRC device handle */
static ADI_CRC_RESULT ValidateCrcHandle (ADI_CRC_HANDLE const  hDevice);

#endif /* ADI_DEBUG */

/*=============  C O D E  =============*/

/*=============  L O C A L   F U N C T I O N S =============*/

/*
 * Resets CRC registers.
 *
 * Parameters
 *  - [in]  pCrcInfo    CRC Device instance to work on.
 *
 * Returns: None
 *
 */
static void ResetCrcRegs (ADI_CRC_INFO  const *pCrcInfo)
{
    /* Reset CRC registers to default */
    pCrcInfo->pReg->Control         = 0u;
    pCrcInfo->pReg->DataCount       = 0u;
    pCrcInfo->pReg->FinalResult     = 0u;
    pCrcInfo->pReg->CurrentResult   = 0u;
    pCrcInfo->pReg->ReloadDataCount = 0u;
    pCrcInfo->pReg->DataCompare     = 0u;
    pCrcInfo->pReg->FillValue       = 0u;
    pCrcInfo->pReg->IntEnClear   	= (uint32_t) ADI_CRC_CLEAR_INTERRUPTS;
    pCrcInfo->pReg->Polynomial      = 0u;
    pCrcInfo->pDevice->ControlReg   = 0u;

    /* Clear W1C fields in status register */
    pCrcInfo->pReg->Status = (uint32_t) (ENUM_CRC_STAT_DCNTEXP | ENUM_CRC_STAT_CMPERR);
}

/*
 * Opens a Memory DMA stream for CRC operation.
 *
 * Parameters:
 *  - [in]  pCrcInfo   Pointer to CRC device instance to work on.
 *
 * Return  Status
 *  - ADI_CRC_SUCCESS: Successfully opened CRC device.
 *  - ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *  - ADI_CRC_DMA_ERROR: Failed to open Memory DMA stream for CRC operation.
 *  - ADI_CRC_INSUFFICIENT_MEMORY: Supplied memory is insufficient to operate CRC device in DMA driven mode.
 *
 */
static ADI_CRC_RESULT OpenMemDmaStream (
    ADI_CRC_INFO    *pCrcInfo)
{
    /* DMA Return code */
    ADI_DMA_RESULT          eResult = ADI_DMA_SUCCESS;
    /* DMA Handle */
    ADI_DMA_CHANNEL_HANDLE  hDmaChannel;

    /* IF (Memory not allocated to support DMA mode) */
    if (pCrcInfo->pDevice->pMdmaStream == NULL)
    {
        /* Return failure as memory not available to support DMA mode */
        return (ADI_CRC_INSUFFICIENT_MEMORY);
    }
    /* ELSE (Open MDMA stream) */
    else
    {
        /* Open a Memory DMA Stream */
        eResult = adi_mdma_Open (pCrcInfo->eStreamID,
                                 pCrcInfo->pDevice->pMdmaStream,
                                 &pCrcInfo->pDevice->hMdmaStream,
                                 &pCrcInfo->pDevice->hSrcChannel,
                                 &pCrcInfo->pDevice->hDestChannel,
                                 MemDmaCallback, pCrcInfo);

        /* IF (Failed to open Memory DMA Stream) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return failure */
            return (ADI_CRC_DMA_ERROR);
        }

        /* Update MDMA mode indicating we're in CRC mode */
        ((ADI_CRC_MDMA_STREAM *)(pCrcInfo->pDevice->pMdmaStream))->bIsCrcMode = true;
    }

    /* Return success */
    return (ADI_CRC_SUCCESS);
}

/*
 * Closes the Memory DMA stream opened for CRC operation.
 *
 * Parameters:
 *  - [in]  pCrcInfo   Pointer to CRC device instance to work on.
 *
 * Return  Status
 *  - ADI_CRC_SUCCESS: Successfully closed CRC device.
 *  - ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *  - ADI_CRC_DMA_ERROR: Failed to close Memory DMA stream opened for CRC operation.
 *
 */
static ADI_CRC_RESULT CloseMemDmaStream (
    ADI_CRC_INFO    const *pCrcInfo)
{
    /* DMA Return code */
    ADI_DMA_RESULT          eResult = ADI_DMA_SUCCESS;

    /* IF (we've a valid Memory DMA Stream handle) */
    if (pCrcInfo->pDevice->hMdmaStream != NULL)
    {
        /* Close Memory DMA Stream */
        eResult = adi_mdma_Close (pCrcInfo->pDevice->hMdmaStream, false);

        /* Clear MDMA Stream handle */
        pCrcInfo->pDevice->hMdmaStream = NULL;

        /* IF (Failed to close Memory DMA Stream) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return failure */
            return (ADI_CRC_DMA_ERROR);
        }
    }

    /* Return success */
    return (ADI_CRC_SUCCESS);
}

/*
 * Starts Memory DMA driven CRC operation.
 *
 * Parameters:
 *  - [in]  pCrcInfo            CRC device instance to work on.
 *  - [in]  pSrcBuf             Address of 32-bit aligned source buffer.
 *  - [in]  pDestBuf            Address of 32-bit aligned destination buffer.
 *  - [in]  Num32bitElements    Number of 32-bit elements in data buffer.
 *
 * return  Status
 *  - ADI_CRC_SUCCESS: Successfully set expected CRC result.
 *  - ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
static ADI_CRC_RESULT StartMemDmaCrc(
    ADI_CRC_INFO  const *pCrcInfo,
    void                *pSrcBuf,
    void                *pDestBuf,
    uint32_t            Num32bitElements)
{
    /* DMA return code */
    ADI_DMA_RESULT      eResult;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC operation in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

    /* IF (MDMA Disabled) */
    if (pCrcInfo->pDevice->hMdmaStream == NULL)
    {
        /* Report error as Memory DMA not open */
        return (ADI_CRC_DMA_NOT_OPEN);
    }

    /* IF (CRC Data count register invalid) */
    if (pCrcInfo->pReg->DataCount == 0u)
    {
        /* Report error as data count invalid */
        return (ADI_CRC_DATA_COUNT_INVALID);
    }

#endif /* ADI_DEBUG */

    /* Update CRC operation status */
    pCrcInfo->pDevice->eCrcOpStatus |= ADI_CRC_OP_IN_PROGRESS;

    /* Load CRC Control Register */
    pCrcInfo->pReg->Control = (pCrcInfo->pDevice->ControlReg | (uint32_t)pCrcInfo->pDevice->eCrcMode);

    /* IF (Memory Fill Mode) */
    if (pCrcInfo->pDevice->eCrcMode == ADI_CRC_MODE_FILL)
    {
        /* Submit buffer to Memory DMA */
        eResult = adi_mdma_Copy1D (pCrcInfo->pDevice->hMdmaStream,
                                   pDestBuf, NULL, ADI_DMA_MSIZE_4BYTES, Num32bitElements);
        /* Enable CRC Block */
        pCrcInfo->pReg->Control |= ENUM_CRC_CTL_BLKEN_EN;
    }
    /* ELSE IF (Memory Transfer-Compute-Compare mode) */
    else if (pCrcInfo->pDevice->eCrcMode == ADI_CRC_MODE_XFER_COMPUTE_COMPARE)
    {
    	/* Enable CRC Block */
    	pCrcInfo->pReg->Control |= ENUM_CRC_CTL_BLKEN_EN;
        /* Submit buffer to Memory DMA */
        eResult = adi_mdma_Copy1D (pCrcInfo->pDevice->hMdmaStream,
                                   pDestBuf, pSrcBuf, ADI_DMA_MSIZE_4BYTES, Num32bitElements);
    }
    /* ELSE (Memory Scan-Compute-Compare (OR) Scan-Verify mode) */
    else
    {
        /* Submit buffer to Memory DMA */
        eResult = adi_mdma_Copy1D (pCrcInfo->pDevice->hMdmaStream,
                                   NULL, pSrcBuf, ADI_DMA_MSIZE_4BYTES, Num32bitElements);
        /* Enable CRC Block */
        pCrcInfo->pReg->Control |= ENUM_CRC_CTL_BLKEN_EN;
    }

    /* IF (MDMA submit failed) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        /* Clear CRC control register */
        pCrcInfo->pReg->Control = 0;
        /* Update CRC operational status */
        pCrcInfo->pDevice->eCrcOpStatus = ADI_CRC_OP_IDLE;
        /* Return error */
        return (ADI_CRC_FAILURE);
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/*
 * Callback from MDMA Manager
 *
 * Parameters
 *  - [in]  pCBParam    Callback parameter supplied by application
 *  - [in]  Event       Callback event
 *  - [in]  pArg        Callback argument
 *
 * Returns  None
 *
 */
static void MemDmaCallback(void *pCBParam, uint32_t Event, void *pArg)
{
    /* Get the CRC instance to which this callback belongs */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *) pCBParam;

    /* IF (CRC not idle) */
    if (pCrcInfo->pDevice->eCrcOpStatus > ADI_CRC_OP_IDLE)
    {
    	/*
    	 * We may end up getting more than one buffer processed event for
    	 * a buffer since the MDMA dataflow will be kept enabled until the
    	 * CRC data count expired interrupt is received. This is to make sure
    	 * that the DMA FIFO is completely drained before closing a CRC operation.
    	 */

    	/* IF (Processed one-shot buffer) */
    	if (Event == (uint32_t) ADI_DMA_EVENT_BUFFER_PROCESSED)
    	{
    		/* Save the processed buffer address */
    		pCrcInfo->pDevice->pProcessedBuf = pArg;
    		/* Update CRC operational status */
    		pCrcInfo->pDevice->eCrcOpStatus |= ADI_CRC_OP_DMA_PROCESSED;
    	}

    	/* IF (DMA Error) */
    	if (Event == (uint32_t) ADI_DMA_EVENT_ERROR_INT)
    	{
    		/* Update CRC operational status */
    		pCrcInfo->pDevice->eCrcOpStatus |= ADI_CRC_OP_DMA_ERROR;
    	}
    }

    /* End this CRC operation */
    EndCrcOperation (pCrcInfo);
}

/*
 * Interrupt service routine for CRC Data Count expired event
 *
 * Parameters
 *  - [in]  SID         SID corresponding to this interrupt
 *  - [in]  pCBParam    Callback parameter
 *
 * Returns  None
 *
 */
static void CrcDataCountHandler(uint32_t SID, void *pCBParam)
{
    /* Get the CRC instance to which this interrupt belongs */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *) pCBParam;

    /* Update CRC operational status */
    pCrcInfo->pDevice->eCrcOpStatus |= ADI_CRC_OP_DATA_COUNT_EXPIRED;

    /* IF (CRC Data compare error) */
    if ((pCrcInfo->pReg->Status & BITM_CRC_STAT_CMPERR) == ENUM_CRC_STAT_CMPERR)
    {
    	/* Update CRC operational status */
    	pCrcInfo->pDevice->eCrcOpStatus |= ADI_CRC_OP_CRC_COMPARE_ERROR;
    }

    /* Clear W1C fields in status register */
    pCrcInfo->pReg->Status = (ENUM_CRC_STAT_DCNTEXP | ENUM_CRC_STAT_CMPERR);

    /* End this CRC operation */
    EndCrcOperation (pCrcInfo);
}

/*
 * End this CRC operation
 * Posts buffer processed callback to application while operating in DMA mode
 *
 * Parameters
 *  - [in]  pCrcInfo    CRC info to work on
 *
 * Returns  None
 *
 */
static void EndCrcOperation (ADI_CRC_INFO const   *pCrcInfo)
{
    /* IF (Error or CRC complete) */
    if (pCrcInfo->pDevice->eCrcOpStatus >= ADI_CRC_OP_COMPLETE)
    {
        /* Clear CRC control register */
    	pCrcInfo->pReg->Control = 0;

        /* Disable MDMA dataflow */
        adi_mdma_Disable (pCrcInfo->pDevice->hMdmaStream);

        /* IF (CRC Complete) */
        if (pCrcInfo->pDevice->eCrcOpStatus == ADI_CRC_OP_COMPLETE)
        {
            /* IF (Post callback to application) */
            if (pCrcInfo->pDevice->pfCallback != NULL)
            {
                /* IF (DMA driven CRC) */
                if (pCrcInfo->pDevice->hMdmaStream != NULL)
                {
                    /* Post callback to report CRC buffer processed */
            	    (pCrcInfo->pDevice->pfCallback) (pCrcInfo->pDevice->pCBParam, ADI_CRC_EVENT_BUFFER_PROCESSED, pCrcInfo->pDevice->pProcessedBuf);
                }
                /* ELSE (Core driven CRC) */
                else
                {
                    /* Post callback to report CRC Data Count expired */
        	        (pCrcInfo->pDevice->pfCallback) (pCrcInfo->pDevice->pCBParam, ADI_CRC_EVENT_DATA_COUNT_EXPIRED, NULL);
                }
            }

            /* Reset CRC Result */
            pCrcInfo->pDevice->eCrcResult = ADI_CRC_SUCCESS;
        }
        /* ELSE (Error) */
        else
        {
            /* IF (Data compare error) */
            if ((pCrcInfo->pDevice->eCrcOpStatus & ADI_CRC_OP_CRC_COMPARE_ERROR) == ADI_CRC_OP_CRC_COMPARE_ERROR)
            {
                /* IF (Post callback to application) */
                if (pCrcInfo->pDevice->pfCallback != NULL)
                {
                    /* Post callback to report CRC Data compare error */
        	        (pCrcInfo->pDevice->pfCallback) (pCrcInfo->pDevice->pCBParam, ADI_CRC_EVENT_COMPARE_ERROR, NULL);
        	    }
        	    /* ELSE (Register CRC Data compare error) */
        	    else
        	    {
        	        /* CRC data compare error */
                    pCrcInfo->pDevice->eCrcResult = ADI_CRC_COMPARE_ERROR;
                }
            }
            /* ELSE (DMA Error) */
            else
            {
                /* IF (Post callback to application) */
                if (pCrcInfo->pDevice->pfCallback != NULL)
                {
                    /* Post callback to report CRC DMA error */
        	        (pCrcInfo->pDevice->pfCallback) (pCrcInfo->pDevice->pCBParam, ADI_CRC_EVENT_DMA_ERROR, NULL);
        	    }
        	    /* ELSE (Register CRC DMA error) */
        	    else
        	    {
        	        /* CRC DMA error */
                    pCrcInfo->pDevice->eCrcResult = ADI_CRC_DMA_ERROR;
                }
            }
        }

        /* Reset CRC Operational status */
        pCrcInfo->pDevice->eCrcOpStatus = ADI_CRC_OP_IDLE;
    }
}

/*=============  D E B U G   F U N C T I O N S =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validates CRC device handle */
static ADI_CRC_RESULT ValidateCrcHandle (ADI_CRC_HANDLE const  hDevice)
{
    /* Loop variable */
    uint32_t    i;

    /* FOR (All CRC Device instances) */
    for (i = 0; i < (sizeof (CrcDevInfo) / sizeof (CrcDevInfo[0])); i++)
    {
        /* IF (CRC handle is valid) */
        if (hDevice == &CrcDevInfo[i])
        {
            /* Channel valid, return success */
            return (ADI_CRC_SUCCESS);
        }
    }

    /* Device ID invalid, return failure */
    return (ADI_CRC_FAILURE);
}

#endif /* ADI_DEBUG */

/*=============  P U B L I C   F U N C T I O N S =============*/

/**
 * @brief   Opens a CRC device instance.
 *
 * @param [in]  DeviceNum       Device number to open.
 * @param [in]  pMemory         Pointer to a #ADI_CRC_CORE_MEMORY_SIZE or #ADI_CRC_DMA_MEMORY_SIZE
 *                              sized buffer to manage the device instance.
 * @param [in]  MemorySize      Size of the buffer to which "pMemory" points.
 * @param [out] phDevice        Pointer to a location where CRC device handle to be written.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully opened a CRC device.
 *          - #ADI_CRC_IN_USE: Supplied CRC Device ID is already in use.
 *          - #ADI_CRC_BAD_DEVICE_NUMBER: Supplied CRC Device ID is invalid.
 *          - #ADI_CRC_INSUFFICIENT_MEMORY: Supplied memory is not sufficient to handle a CRC device instance.
 *          - #ADI_CRC_INT_REGISTER_FAILED: Failed to register CRC Data/Error interrupt.
 *
 * @note
 *      For Core driven CRC device, the device memory should be of size #ADI_CRC_CORE_MEMORY_SIZE.
 *      For DMA driven CRC device, the device memory should be of size #ADI_CRC_DMA_MEMORY_SIZE.
 *
 */
ADI_CRC_RESULT adi_crc_Open(
	uint32_t            DeviceNum,
	void                *pMemory,
	uint32_t            MemorySize,
	ADI_CRC_HANDLE      *phDevice)
{
    /* Return Code */
    ADI_CRC_RESULT  eResult = ADI_CRC_SUCCESS;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (This is not a valid device number) */
    if (DeviceNum > (sizeof (CrcDevInfo)/sizeof(CrcDevInfo[0])))
    {
        /* Report failure as bad device number */
        return (ADI_CRC_BAD_DEVICE_NUMBER);
    }

    /* IF (Supplied memory size is insufficient) */
    if (MemorySize < ADI_CRC_CORE_MEMORY_SIZE)
    {
        /* Report failure as insufficient memory */
        return (ADI_CRC_INSUFFICIENT_MEMORY);
    }

    /* Asserts to validate minimum memory size required to manage a Core driven CRC device */
    assert(ADI_CRC_CORE_MEMORY_SIZE >= sizeof(ADI_CRC_DEVICE));
    /* Asserts to validate minimum memory size required to manage a DMA driven CRC device */
    assert(ADI_CRC_DMA_MEMORY_SIZE >= (sizeof(ADI_CRC_DEVICE) + ADI_DMA_STREAM_REQ_MEMORY));

#endif  /* ADI_DEBUG */

    /* Clear the given memory */
    memset(pMemory, 0, MemorySize);

    /* Entering critical region, disable interrupts */
	adi_osal_EnterCriticalRegion();

    /* IF (The device is in use) */
    if (CrcDevInfo[DeviceNum].pDevice != NULL)
    {
        /* CRC Device is in use */
        eResult = ADI_CRC_IN_USE;
    }
    /* ELSE (Device is available for use) */
    else
    {
        /* Save the supplied device memory address */
        CrcDevInfo[DeviceNum].pDevice = (ADI_CRC_DEVICE *) pMemory;
    }

    /* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

	/* IF (Success) */
    if (eResult == ADI_CRC_SUCCESS)
    {
        /* Initialize the device instance */

        /* IF (Supplied memory can support DMA driven CRC operation) */
        if (MemorySize >= ADI_CRC_DMA_MEMORY_SIZE)
        {
            /* Get the memory address allocated for MDMA stream */
            CrcDevInfo[DeviceNum].pDevice->pMdmaStream = (void *)((uint32_t)pMemory + sizeof (ADI_CRC_DEVICE));
        }

        /* Reset CRC registers */
        ResetCrcRegs (&CrcDevInfo[DeviceNum]);

        /* Register the CRC driver Data count expiration interrupt handler with interrupt dispatcher */
        if(adi_int_InstallHandler((uint32_t)CrcDevInfo[DeviceNum].DataCountSID,
                                  CrcDataCountHandler,
                                  &CrcDevInfo[DeviceNum],
                                  true) != ADI_INT_SUCCESS)
        {
            /* Could not register with interrupt handler, free CRC device and return  */
            CrcDevInfo[DeviceNum].pDevice = NULL;
            return (ADI_CRC_INT_REGISTER_FAILED);
        }

        /* Pass a valid handle to this CRC device */
        *phDevice = (ADI_CRC_HANDLE) &CrcDevInfo[DeviceNum];

        /* Enable CRC interrupts */
        CrcDevInfo[DeviceNum].pReg->IntEnSet = ENUM_CRC_INEN_SET_DCNTEXP_SET;
    }

    /* return */
    return (eResult);
}

/**
 * @brief   Closes CRC device instance opened for use.
 *
 * @param [in]  hDevice     Handle to CRC Device instance to close.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully closed CRC device.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_NOT_OPEN: CRC device is not in use.
 *          - #ADI_CRC_INT_UNREGISTER_FAILED: Failed to un-register CRC Data/Error interrupt.
 *
 */
ADI_CRC_RESULT adi_crc_Close(ADI_CRC_HANDLE const    hDevice)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC device is not in use) */
    if (pCrcInfo->pDevice == NULL)
    {
        return (ADI_CRC_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (MDMA Stream handle is valid) */
    if (pCrcInfo->pDevice->hMdmaStream != NULL)
    {
        /* Close MDMA stream */
        adi_mdma_Close (pCrcInfo->pDevice->hMdmaStream, false);
        pCrcInfo->pDevice->hMdmaStream = NULL;
    }

    /* Reset CRC registers */
    ResetCrcRegs (pCrcInfo);

    /* Un-register CRC data interrupt handler */
    if (adi_int_UninstallHandler ((uint32_t)pCrcInfo->DataCountSID) != ADI_INT_SUCCESS)
    {
    	/* Return failure as we failed to un-register the interrupt */
    	return (ADI_CRC_INT_UNREGISTER_FAILED);
    }

    /* Successfully un-registered CRC interrupt handlers */
    pCrcInfo->pDevice = NULL;

    /* Return success */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Stops current CRC in progress and/or resets CRC peripheral.
 *
 * @param [in]  hDevice     Handle to CRC Device instance to work on.
 * @param [in]  bStopCrc    'true' to treminate the current CRC in progress.
 * @param [in]  bResetCrc   'true' to reset CRC registers and CRC driver to default values.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully stopped/reset CRC device.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_StopReset(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bStopCrc,
    bool                    bResetCrc)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* IF (Stop CRC operation) */
    if (bStopCrc)
    {
        /* Disable CRC Block */
        pCrcInfo->pReg->Control &= ~ENUM_CRC_CTL_BLKEN_EN;
    }

    /* IF (Reset CRC device) */
    if (bResetCrc)
    {
        /* Reset CRC Registers */
        ResetCrcRegs (pCrcInfo);
    }

    /* Return success */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets mirroring configuration for CRC registers.
 *
 * @param [in]  hDevice             Handle to CRC Device instance to work on.
 * @param [in]  bMirrCompare        'true' to enable data mirroring for CRC data comparison register.
 * @param [in]  bMirrPolynomial     'true' to enable data mirroring for the CRC polynomial register.
 * @param [in]  bMirrResult         'true' to enable data mirroring for the CRC final result and CRC current result.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully updated CRC configuration.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetRegMirror(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bMirrCompare,
    bool                    bMirrPolynomial,
    bool                    bMirrResult)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Update CRC Control register configuration */

    /* Clear register mirror configuration fields */
    pCrcInfo->pDevice->ControlReg &= ~BITM_CRC_CTL_MIRR_REG;

    /* IF (Enable data mirroring for CRC data comparison register) */
    if (bMirrCompare)
    {
        pCrcInfo->pDevice->ControlReg |= ENUM_CRC_CTL_CMPMIRR_EN;
    }

    /* IF (Enable data mirroring for CRC polynomial register) */
    if (bMirrPolynomial)
    {
        pCrcInfo->pDevice->ControlReg |= ENUM_CRC_CTL_POLYMIRR_EN;
    }

    /* IF (Enable data mirroring for CRC final result and CRC current result register) */
    if (bMirrResult)
    {
        pCrcInfo->pDevice->ControlReg |= ENUM_CRC_CTL_RSLTMIRR_EN;
    }

    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets mirroring configuration for CRC data stream.
 *
 * Parameters:
 *  @param [in]  hDevice         Handle to CRC device instance to work on.
 *  @param [in]  eDataMirror     CRC Data mirror mode.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully updated CRC configuration.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetDataMirror(
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_MIRROR          eDataMirror)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Clear data mirror configuration fields */
    pCrcInfo->pDevice->ControlReg &= ~BITM_CRC_CTL_MIRR_DATA;

    /* Update data mirror configuration */
    pCrcInfo->pDevice->ControlReg |= (uint32_t) (eDataMirror);

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets CRC auto-clear mode.
 *
 * Parameters:
 *  @param [in]  hDevice         Handle to CRC device instance to work on.
 *  @param [in]  bEnable         'true' to enable auto clear operation.
 *  @param [in]  bAutoClearZero  'true' to Auto Clear to Zero, 'false' to Auto Clear to One.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully updated CRC configuration.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetAutoClear(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bEnable,
    bool                    bAutoClearZero)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Update CRC Control register configuration */

    /* Clear auto clear configuration */
    pCrcInfo->pDevice->ControlReg &= ~(ENUM_CRC_CTL_AUTOCLRF_EN | ENUM_CRC_CTL_AUTOCLRZ_EN);

    /* IF (Enable auto clear operation) */
    if (bEnable)
    {
        /* IF (Enable Auto Clear to Zero) */
        if (bAutoClearZero)
        {
            pCrcInfo->pDevice->ControlReg |= ENUM_CRC_CTL_AUTOCLRZ_EN;
        }
        /* ELSE (Enable Auto Clear to One) */
        else
        {
            pCrcInfo->pDevice->ControlReg |= ENUM_CRC_CTL_AUTOCLRF_EN;
        }
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Enables/Disables DMA driven CRC operation.
 *
 * Parameters:
 *  @param [in]  hDevice    Handle to CRC device instance to work on.
 *  @param [in]  bEnable    'true' to enable DMA driven CRC, 'false' for core driven CRC.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully enabled/disabled DMA mode.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *          - #ADI_CRC_DMA_ERROR: Failed to open/close Memory DMA stream.
 *          - #ADI_CRC_INSUFFICIENT_MEMORY: Supplied memory is insufficient to operate CRC device in DMA driven mode.
 *
 */
ADI_CRC_RESULT adi_crc_EnableDmaMode(
    ADI_CRC_HANDLE const    hDevice,
    bool                    bEnable)
{
    /* Return code */
    ADI_CRC_RESULT  eResult = ADI_CRC_SUCCESS;
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* IF (Enable MDMA) */
    if (bEnable)
    {
        /* IF (MDMA already enabled) */
        if (pCrcInfo->pDevice->hMdmaStream != NULL)
        {
            /* Return success */
            eResult = ADI_CRC_SUCCESS;
        }
        /* ELSE (MDMA not enabled yet) */
        else
        {
            /* Open MDMA stream */
            eResult = OpenMemDmaStream (pCrcInfo);
        }
    }
    /* ELSE (Close MDMA) */
    else
    {
        /* Close MDMA stream */
        eResult = CloseMemDmaStream (pCrcInfo);
    }

    /* Return */
    return (eResult);
}

/**
 * @brief   Sets CRC operating mode.
 *
 * Parameters:
 *  @param [in]  hDevice    Handle to CRC device instance to work on.
 *  @param [in]  eCrcMode   CRC Operating mode.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set CRC operating mode.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetOperatingMode(
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_MODE            eCrcMode)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Save Operating mode */
    pCrcInfo->pDevice->eCrcMode = eCrcMode;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets the 32-bit polynomial for CRC operations.
 *
 * Parameters:
 *  @param [in]  hDevice        Handle to CRC device instance to work on.
 *  @param [in]  PolynomialVal  32-bit CRC polynomial to use for CRC operation.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set polynomial value.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetPolynomialVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                PolynomialVal)
{
    /* CRC device to work on */
    ADI_CRC_INFO        *pCrcInfo = (ADI_CRC_INFO *)hDevice;
    /* Loop variable */
    volatile uint32_t   i = 0x1FFFFFu;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Load Polynomial value */
    pCrcInfo->pReg->Polynomial = PolynomialVal;

    /* Wait until the Look-up table entries are calculated */
    while ((pCrcInfo->pReg->Status & ENUM_CRC_STAT_LUTDONE) != ENUM_CRC_STAT_LUTDONE)
    {
        /* Make sure we don't end up in an infinite loop */
        if (i == 0)
        {
            break;
        }
        i--;
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets the data count for the CRC operation that is about to take place.
 *
 * Parameters:
 *  @param [in]  hDevice            Handle to CRC device instance to work on.
 *  @param [in]  DataCount          Data count for the CRC operation. Each count corresponds to a 32-bit Word.
 *  @param [in]  ReloadDataCount    Data count reload value.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set CRC data count.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetDataCount(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                DataCount,
    uint32_t                ReloadDataCount)
{
    /* CRC device to work on */
    ADI_CRC_INFO        *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Update Data Count register */
    pCrcInfo->pReg->DataCount = DataCount;

    /* Update Reload Data Count register */
    pCrcInfo->pReg->ReloadDataCount = ReloadDataCount;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets the initial seed value for the CRC operation that is about to take place.
 *
 * Parameters:
 *  @param [in]  hDevice         Handle to CRC device instance to work on.
 *  @param [in]  CrcSeedVal      Initial seed value for the CRC operation that is about to take place.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set CRC seed value.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetCrcSeedVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                CrcSeedVal)
{
    /* CRC device to work on */
    ADI_CRC_INFO        *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Load the CRC seed value */
    pCrcInfo->pReg->CurrentResult = CrcSeedVal;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets the expected CRC value of a data stream that is about to be processed (OR)
 *          Sets the 32-bit value that is to be used to validate a memory region.
 *
 * @details The parameter (ExpectedVal) passed with this API takes different meaning
 *          depending upon the CRC operating mode.
 *          For memory compare mode (#ADI_CRC_MODE_COMPARE), the 32-bit value passed as
 *          "ExpectedVal" will be used to validate a memory region. Each 32-bit data
 *          presented to the CRC peripheral will be compared with this value.
 *          For Memory scan (#ADI_CRC_MODE_MEM_SCAN) and memory transfer (#ADI_CRC_MODE_MEM_XFER)
 *          modes, the parameter "ExpectedVal" provides the expected CRC32 result.
 *
 * Parameters:
 *  @param [in]  hDevice        Handle to CRC device instance to work on.
 *  @param [in]  ExpectedVal    Expected CRC32 result of a data stream to be processed or
 *                              32-bit value to be used to validate a memory region.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set expected CRC result.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetExpectedVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                ExpectedVal)
{
    /* CRC device to work on */
    ADI_CRC_INFO        *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Load CRC Compare register */
    pCrcInfo->pReg->DataCompare = ExpectedVal;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Sets the 32-bit value that is to be used to fill a memory region.
 *
 * Parameters:
 *  @param [in]  hDevice        Handle to CRC device instance to work on.
 *  @param [in]  MemFillVal     32-bit value to be used to fill a memory region.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set expected CRC result.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_SetMemFillVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                MemFillVal)
{
    /* CRC device to work on */
    ADI_CRC_INFO        *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Load CRC Fill Value register */
    pCrcInfo->pReg->FillValue = MemFillVal;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Submits data buffer for DMA driven Data Fill (OR) Scan-Verify (OR) Scan-Compute-Compare CRC opeartion.
 *           This API is supported only in DMA mode.
 *
 * @details This API can be used to submit data buffer for DMA driven CRC operations such as
 *          Memory Transfer Data Fill (#ADI_CRC_MODE_FILL) or
 *          Memory Scan Data Verify (#ADI_CRC_MODE_SCAN_VERIFY) or
 *          Memory Scan Compute Compare (#ADI_CRC_MODE_SCAN_COMPUTE_COMPARE)
 *
 * Parameters:
 *  @param [in]  hDevice            Handle to CRC device instance to work on.
 *  @param [in]  pCrcBuf            Address of 32-bit aligned CRC data buffer to work on
 *  @param [in]  Num32bitElements   Number of 32-bit elements in data buffer.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully submitted 32-bit aligned data buffer.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_DMA_NOT_OPEN: Memory DMA channel is not opened yet.
 *          - #ADI_CRC_INVALID_FN_CALL: Function call invalid with the current CRC mode.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_DmaMemFillScan(
    ADI_CRC_HANDLE const    hDevice,
    void                    *pCrcBuf,
    uint32_t                Num32bitElements)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

    /* IF (MDMA Disabled) */
    if (pCrcInfo->pDevice->hMdmaStream == NULL)
    {
        /* Report error as Memory DMA not open */
        return (ADI_CRC_DMA_NOT_OPEN);
    }

    /* IF (CRC mode is not configured in Scan-Compute-Compare (OR) Data Fill (OR) Scan-Verify Mode) */
    if ((pCrcInfo->pDevice->eCrcMode == ADI_CRC_MODE_NONE) ||\
        (pCrcInfo->pDevice->eCrcMode == ADI_CRC_MODE_XFER_COMPUTE_COMPARE))
    {
        /* Report error as this function call is invalid for the current CRC mode */
        return (ADI_CRC_INVALID_FN_CALL);
    }

#endif /* ADI_DEBUG */

    /* IF (Memory Fill Mode) */
    if (pCrcInfo->pDevice->eCrcMode == ADI_CRC_MODE_FILL)
    {
        /* Start Memory DMA driven CRC Memory Fill operation */
        return (StartMemDmaCrc (pCrcInfo, NULL, pCrcBuf, Num32bitElements));
    }
    /* ELSE (Memory Scan-Compute-Compare (OR) Scan-Verify mode) */
    else
    {
        /* Start Memory DMA driven CRC Memory Scan-Compute-Compare (OR) Scan-Verify operation */
        return (StartMemDmaCrc (pCrcInfo, pCrcBuf, NULL, Num32bitElements));
    }
}

/**
 * @brief   Submits source and destination data buffers for DMA driven
 *          Memory Transfer Compute and Compare (#ADI_CRC_MODE_XFER_COMPUTE_COMPARE)
 *          CRC opeartion. This API is supported only in DMA mode.
 *
 * Parameters:
 *  @param [in]  hDevice            Handle to CRC device instance to work on.
 *  @param [in]  pSrcBuf            Address of 32-bit aligned source data buffer.
 *  @param [in]  pDestBuf           Address of 32-bit aligned destination data buffer.
 *  @param [in]  Num32bitElements   Number of 32-bit elements in data buffer.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully submitted 32-bit aligned data buffer.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_INVALID_FN_CALL: This function call is invalid for the current CRC mode.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when CRC operation is in progress.
 *
 */
ADI_CRC_RESULT adi_crc_DmaMemXfer(
    ADI_CRC_HANDLE const    hDevice,
    void                    *pSrcBuf,
    void                    *pDestBuf,
    uint32_t                Num32bitElements)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus != ADI_CRC_OP_IDLE)
    {
        /* Report error as this function is not permitted when CRC operation is in progress */
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

    /* IF (MDMA Disabled) */
    if (pCrcInfo->pDevice->hMdmaStream == NULL)
    {
        /* Report error as Memory DMA not open */
        return (ADI_CRC_DMA_NOT_OPEN);
    }

    /* IF (CRC mode is not configured in Memory transfer Mode) */
    if (pCrcInfo->pDevice->eCrcMode != ADI_CRC_MODE_XFER_COMPUTE_COMPARE)
    {
        /* Report error as this function call is invalid for the current CRC mode */
        return (ADI_CRC_INVALID_FN_CALL);
    }

#endif /* ADI_DEBUG */

    /* Start Memory DMA driven CRC Memory transfer */
    return (StartMemDmaCrc (pCrcInfo, pSrcBuf, pDestBuf, Num32bitElements));
}

/**
 * @brief   Writes a 32-bit data to CRC FIFO for Core driven CRC operation.
 *          Supported ONLY when CRC is operated in core access mode with
 *          CRC DMA disabled.
 *
 * Parameters:
 *  @param [in]  hDevice            Handle to CRC device instance to work on.
 *  @param [in]  DataToCrcFifo      32-bit data to be written to CRC FIFO.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: 32-bit data was successfully written to CRC FIFO.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_CORE_WRITE_NOT_READY: CRC FIFO (Input buffer) is not ready to accept data.
 *
 */
ADI_CRC_RESULT adi_crc_CoreWrite(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                DataToCrcFifo)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* IF (Input buffer ready) */
    if ((pCrcInfo->pReg->Status & BITM_CRC_STAT_IBR) == ENUM_CRC_STAT_IBR)
    {
        /* Write data to FIFO */
        pCrcInfo->pReg->DataFifo = DataToCrcFifo;
        /* return success */
        return (ADI_CRC_SUCCESS);
    }
    /* ELSE (Return error as input buffer is not ready to accept data) */
    else
    {
        return (ADI_CRC_CORE_WRITE_NOT_READY);
    }
}

/**
 * @brief   Reads a 32-bit data from CRC FIFO for Core driven CRC operation.
 *          Supported ONLY when CRC is operated in core access mode with
 *          CRC DMA disabled.
 *
 * @details FIFO can be read only when the CRC is operated in
 *          Memory transfer-compute-compare mode (ADI_CRC_MODE_XFER_COMPUTE_COMPARE).
 *
 * Parameters:
 *  @param [in]  hDevice            Handle to CRC device instance to work on.
 *  @param [in]  pDataFromCrcFifo   Pointer to location to save 32-bit data to read from CRC FIFO.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully read 32-bit data from CRC FIFO.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_SUPPORTED: Function not supported for current CRC operating mode.
 *          - #ADI_CRC_CORE_READ_NOT_READY: CRC FIFO can't be read as the output buffer is empty.
 *
 */
ADI_CRC_RESULT adi_crc_CoreRead(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pDataFromCrcFifo)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (Function not supported for this mode) */
    if (pCrcInfo->pDevice->eCrcMode != ADI_CRC_MODE_XFER_COMPUTE_COMPARE)
    {
        return (ADI_CRC_FN_NOT_SUPPORTED);
    }

#endif /* ADI_DEBUG */

    /* IF (Output buffer ready) */
    if ((pCrcInfo->pReg->Status & BITM_CRC_STAT_OBR) == ENUM_CRC_STAT_OBR)
    {
        /* Read data from FIFO */
        *pDataFromCrcFifo = pCrcInfo->pReg->DataFifo;
        /* return success */
        return (ADI_CRC_SUCCESS);
    }
    /* ELSE (Return error as output buffer is empty) */
    else
    {
        return (ADI_CRC_CORE_READ_NOT_READY);
    }
}

/**
 * @brief   Peek function to check if the CRC peripheral FIFO is ready for Core Write access.
 *          Supported ONLY when CRC is operated in core access mode.
 *
 * Parameters:
 *  @param  [in]  hDevice             Handle to CRC device instance to work on
 *  @param  [in]  pbIsCoreWriteReady  Pointer to location to store the current core write accessibility status of CRC.
 *                                    returned 'true' when the CRC FIFO is ready to be written via core access.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set expected CRC result.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_IsCoreWriteReady (
    ADI_CRC_HANDLE const    hDevice,
    bool                    *pbIsCoreWriteReady)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* IF (Input buffer ready) */
    if ((pCrcInfo->pReg->Status & BITM_CRC_STAT_IBR) == ENUM_CRC_STAT_IBR)
    {
        *pbIsCoreWriteReady = true;
    }
    /* ELSE (Input not buffer ready) */
    else
    {
        *pbIsCoreWriteReady = false;
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}


/**
 * @brief   Peek function to check if the CRC peripheral FIFO is ready for Core Read access.
 *          Supported ONLY when CRC is operated in core access mode.
 *
 * Parameters:
 *  @param  [in]  hDevice             Handle to CRC device instance to work on
 *  @param  [in]  pbIsCoreReadReady   Pointer to location to store the current core read accessibility status of CRC.
 *                                    returned 'true' when the CRC FIFO is ready to be read via core access.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set expected CRC result.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_IsCoreReadReady (
    ADI_CRC_HANDLE const    hDevice,
    bool                    *pbIsCoreReadReady)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* IF (Output buffer ready) */
    if ((pCrcInfo->pReg->Status & BITM_CRC_STAT_OBR) == ENUM_CRC_STAT_OBR)
    {
        *pbIsCoreReadReady = true;
    }
    /* ELSE (Output not buffer ready) */
    else
    {
        *pbIsCoreReadReady = false;
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Gets the current CRC peripheral status.
 *
 * Parameters:
 *  @param  [in]  hDevice           Handle to CRC device instance to work on
 *  @param  [in]  pbCrcInProgress   Pointer to location to store the current status of CRC peripheral.
 *                                  'true' when CRC peripheral is in currently performing a CRC operation.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully set expected CRC result.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_IsCrcInProgress(
    ADI_CRC_HANDLE const    hDevice,
    bool                    *pbCrcInProgress)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;
    /* CRC Result */
    ADI_CRC_RESULT  eResult = ADI_CRC_SUCCESS;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* IF (CRC in progress) */
    if (pCrcInfo->pDevice->eCrcOpStatus > ADI_CRC_OP_IDLE)
    {
        /* Update CRC status flag */
        *pbCrcInProgress = true;
    }
    /* ELSE (CRC is idle) */
    else
    {
        /* Update CRC status flag */
        *pbCrcInProgress = false;
    }

    /* Return code */
    eResult = pCrcInfo->pDevice->eCrcResult;
    /* Reset CRC result */
    pCrcInfo->pDevice->eCrcResult = ADI_CRC_SUCCESS;

    /* Return */
    return (eResult);
}

/**
 * @brief   Gets the final CRC result computed for a data stream
 *
 * @details This API gets the final CRC result computed for a data stream and
 *          clears the current and final CRC results register. The CRC Current result
 *          register holds the current or intermediate CRC result.
 *          Whenever a CRC operation is initiated, the CRC peripheral takes the
 *          CRC Current register value as initial seed for CRC computation. This
 *          API clears both results register to start a fresh CRC computation.
 *          Use the adi_crc_GetCurrentCrcVal() API to get an intermediate CRC result
 *          without clearing the results register.
 *
 * Parameters:
 *  @param  [in]  hDevice         Handle to CRC device instance to work on
 *  @param  [out] pFinalCrcVal    Pointer to location where the final CRC result of
 *                                a data stream to be processed will be written.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully read final CRC result.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_GetFinalCrcVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pFinalCrcVal)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Get the final CRC result */
    *pFinalCrcVal = pCrcInfo->pReg->FinalResult;

    /* Clear the result register to start a fresh CRC operation */
    pCrcInfo->pReg->CurrentResult = 0;
    pCrcInfo->pReg->FinalResult = 0;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Gets the current/intermediate CRC result computed for a data stream.
 *
 * Parameters:
 *  @param  [in]  hDevice           Handle to CRC device instance to work on
 *  @param  [out] pCurrentCrcVal    Pointer to location where the intermediate CRC result of
 *                                  a data stream to be processed will be written.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully read current CRC result.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_GetCurrentCrcVal(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pCurrentCrcVal)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Get the current CRC result */
    *pCurrentCrcVal = pCrcInfo->pReg->CurrentResult;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Gets the position of error when a compare operation fails in data verify mode.
 *
 * @details     The API should be called only when the application receives Data Compare error/event
 *              from the driver. This API provides the exact position of error in the data stream
 *              that was submitted by the application for data verification.
 *
 * Parameters:
 *  @param  [in]  hDevice       Handle to CRC device instance to work on
 *  @param  [out] pFailPos      Pointer to location where the position of data compare error will be stored.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully read data compare error position.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_GetCompareFailPos(
    ADI_CRC_HANDLE const    hDevice,
    uint32_t                *pFailPos)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Get the data compare error position */
    *pFailPos = pCrcInfo->pReg->CountCapture;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Registers or unregisters a callback with the CRC device
 *
 * @details     It is not required to register a callback for the operation of the
 *              driver. Data compare or DMA error will be notified via the
 *              adi_crc_IsCrcInProgress() API. But if application requires the
 *              errors/events to be notified immediately it can register a callback
 *              with the driver which will be called to notify errors/events.
 *
 *              When a callback is registered the API adi_crc_IsCrcInProgress()
 *              will not return error.
 *
 * Parameters:
 *  @param  [in]  hDevice         Handle to CRC device instance to work on
 *  @param  [in]  pfCallback      Pointer to application callback function. The callback function
 *                                has the prototype
 *                                void callback(void *pCBParam, uint32_t nEvent, void *pArg)
 *                                To unregister a callback pass the the pointer to the callback
 *                                function as NULL.
 *  @param  [in]  pCBParam        Callback parameter which will be returned back to the
 *                                application when the callback function is called.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully registered callback.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *
 */
ADI_CRC_RESULT adi_crc_RegisterCallback(
    ADI_CRC_HANDLE const    hDevice,
    ADI_CALLBACK            pfCallback,
    void *const             pCBParam)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Update Callback information */
    pCrcInfo->pDevice->pfCallback   = pfCallback;
    pCrcInfo->pDevice->pCBParam     = pCBParam;

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Configures Trigger input settings for DMA driven CRC input channel.
 *
 * @details     This API is supported only in DMA mode. Calling this API in interrupt mode will result in an error.
 *              CRC input (Source) DMA channel is used only in Memory Transfer Compute Compare
 *              (#ADI_CRC_MODE_XFER_COMPUTE_COMPARE), Memory Scan Compute Compare (#ADI_CRC_MODE_SCAN_COMPUTE_COMPARE)
 *              and Memory Scan Data Verify (#ADI_CRC_MODE_SCAN_VERIFY) modes.
 *
 *              The CRC operation can be configured to wait for an external trigger. The trigger to be used
 *              for starting the CRC operation should be configured by application using the Trigger Routing
 *              Unit API adi_tru_TriggerRoute.
 *
 * Parameters:
 *  @param  [in]  hDevice           Handle to CRC device instance to work on
 *  @param  [in]  bTriggerWait      Flag which indicates whether to wait for an incomming trigger to start a CRC operation.
 *
 *                                  'true' - Wait for an incoming trigger.
 *                                  'false'- Ignore incoming trigger and start CRC operation soon after the buffer is submitted.
 *
 *  @param  [in]  bTriggerOvError   Flag which indicates whether to generate an error when a trigger overrun occurs.
 *                                  A trigger overrun occurs when a trigger is recieved before a CRC operation
 *                                  initiated by a previous trigger is still in progress.
 *
 *                                  'true' to enable and report trigger overrun error.
 *                                  'false' to disable trigger overrun error.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully configured trigger settings.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when DMA mode is disabled.
 *          - #ADI_CRC_DMA_ERROR: Failed to configure DMA channel with trigger settings.
 *
 */
ADI_CRC_RESULT  adi_crc_ConfigCrcInTriggerIn (
    ADI_CRC_HANDLE const    hDevice,
    bool                    bTriggerWait,
    bool                    bTriggerOvError)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;
    /* DMA Result */
    ADI_DMA_RESULT  eResult;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (Not in DMA mode) */
    if(pCrcInfo->pDevice->hMdmaStream == NULL)
    {
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Configure the Source DMA channel with trigger input settings */
    eResult = adi_dma_ConfigTriggerIn (pCrcInfo->pDevice->hSrcChannel, bTriggerWait, bTriggerOvError);

    /* IF (DMA Error) */
    if(eResult != ADI_DMA_SUCCESS)
    {
        return (ADI_CRC_DMA_ERROR);
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief   Configures Trigger input settings for DMA driven CRC output channel.
 *
 * @details     This API is supported only in DMA mode. Calling this API in interrupt mode will result in an error.
 *              CRC output (Destination) DMA channel is used only in Memory Transfer Compute Compare
 *              (#ADI_CRC_MODE_XFER_COMPUTE_COMPARE), and Memory Transfer Data Fill (#ADI_CRC_MODE_FILL) modes.
 *
 *              The CRC operation can be configured to wait for an external trigger. The trigger to be used
 *              for starting the CRC operation should be configured by application using the Trigger Routing
 *              Unit API adi_tru_TriggerRoute.
 *
 * Parameters:
 *  @param  [in]  hDevice           Handle to CRC device instance to work on
 *  @param  [in]  bTriggerWait      Flag which indicates whether to wait for an incomming trigger to start a CRC operation.
 *
 *                                  'true' - Wait for an incoming trigger.
 *                                  'false'- Ignore incoming trigger and start CRC operation soon after the buffer is submitted.
 *
 *  @param  [in]  bTriggerOvError   Flag which indicates whether to generate an error when a trigger overrun occurs.
 *                                  A trigger overrun occurs when a trigger is recieved before a CRC operation
 *                                  initiated by a previous trigger is still in progress.
 *
 *                                  'true' to enable and report trigger overrun error.
 *                                  'false' to disable trigger overrun error.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully configured trigger settings.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when DMA mode is disabled.
 *          - #ADI_CRC_DMA_ERROR: Failed to configure DMA channel with trigger settings.
 *
 */
ADI_CRC_RESULT  adi_crc_ConfigCrcOutTriggerIn (
    ADI_CRC_HANDLE const    hDevice,
    bool                    bTriggerWait,
    bool                    bTriggerOvError)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;
    /* DMA Result */
    ADI_DMA_RESULT  eResult;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (Not in DMA mode) */
    if(pCrcInfo->pDevice->hMdmaStream == NULL)
    {
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Configure the Destination DMA channel with trigger input settings */
    eResult = adi_dma_ConfigTriggerIn (pCrcInfo->pDevice->hDestChannel, bTriggerWait, bTriggerOvError);

    /* IF (DMA Error) */
    if(eResult != ADI_DMA_SUCCESS)
    {
        return (ADI_CRC_DMA_ERROR);
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief Configures Trigger output settings for DMA driven CRC input channel.
 *
 * @details     This API is supported only in DMA mode. Calling this API in interrupt mode will result in an error.
 *              CRC input (Source) DMA channel is used only in Memory Transfer Compute Compare
 *              (#ADI_CRC_MODE_XFER_COMPUTE_COMPARE), Memory Scan Compute Compare (#ADI_CRC_MODE_SCAN_COMPUTE_COMPARE)
 *              and Memory Scan Data Verify (#ADI_CRC_MODE_SCAN_VERIFY) modes.
 *
 *              A trigger can be generated after completing a CRC operation. The trigger can be used to
 *              syncronize other processing in the system. The trigger generated by CRC can be routed to
 *              one or more slave devices and the application should route the trigger by calling the
 *              Trigger Routing Unit unit API adi_tru_TriggerRoute.
 *
 * Parameters:
 *  @param  [in]  hChannel          Handle to CRC device instance to work on
 *  @param  [in]  eCrcTrigger       Condition to generate an outgoing trigger.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully configured trigger settings.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when DMA mode is disabled.
 *          - #ADI_CRC_DMA_ERROR: Failed to configure DMA channel with trigger settings.
 */
ADI_CRC_RESULT  adi_crc_ConfigCrcInTriggerOut (
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_TRIGGER_OUT     eCrcTrigger)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;
    /* DMA Result */
    ADI_DMA_RESULT  eResult;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (Not in DMA mode) */
    if(pCrcInfo->pDevice->hMdmaStream == NULL)
    {
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Configure the Source DMA channel with trigger output settings */
    eResult = adi_dma_ConfigTriggerOut (pCrcInfo->pDevice->hSrcChannel, (ADI_DMA_TRIGGER_OUT)eCrcTrigger);

    /* IF (DMA Error) */
    if(eResult != ADI_DMA_SUCCESS)
    {
        return (ADI_CRC_DMA_ERROR);
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

/**
 * @brief Configures Trigger output settings for DMA driven CRC output channel.
 *
 * @details     This API is supported only in DMA mode. Calling this API in interrupt mode will result in an error.
 *              CRC output (Destination) DMA channel is used only in Memory Transfer Compute Compare
 *              (#ADI_CRC_MODE_XFER_COMPUTE_COMPARE), and Memory Transfer Data Fill (#ADI_CRC_MODE_FILL) modes.
 *
 *              A trigger can be generated after completing a CRC operation. The trigger can be used to
 *              syncronize other processing in the system. The trigger generated by CRC can be routed to
 *              one or more slave devices and the application should route the trigger by calling the
 *              Trigger Routing Unit unit API adi_tru_TriggerRoute.
 *
 * Parameters:
 *  @param  [in]  hChannel          Handle to CRC device instance to work on
 *  @param  [in]  eCrcTrigger       Condition to generate an outgoing trigger.
 *
 * @return  Status
 *          - #ADI_CRC_SUCCESS: Successfully configured trigger settings.
 *          - #ADI_CRC_BAD_HANDLE: Supplied CRC handle is invalid.
 *          - #ADI_CRC_FN_NOT_PERMITTED: Function not permitted when DMA mode is disabled.
 *          - #ADI_CRC_DMA_ERROR: Failed to configure DMA channel with trigger settings.
 */
ADI_CRC_RESULT  adi_crc_ConfigCrcOutTriggerOut (
    ADI_CRC_HANDLE const    hDevice,
    ADI_CRC_TRIGGER_OUT     eCrcTrigger)
{
    /* CRC device to work on */
    ADI_CRC_INFO    *pCrcInfo = (ADI_CRC_INFO *)hDevice;
    /* DMA Result */
    ADI_DMA_RESULT  eResult;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (CRC device handle is invalid) */
    if (ValidateCrcHandle (hDevice) != ADI_CRC_SUCCESS)
    {
        return (ADI_CRC_BAD_HANDLE);
    }

    /* IF (Not in DMA mode) */
    if(pCrcInfo->pDevice->hMdmaStream == NULL)
    {
        return (ADI_CRC_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* Configure the Destination DMA channel with trigger output settings */
    eResult = adi_dma_ConfigTriggerOut (pCrcInfo->pDevice->hDestChannel, (ADI_DMA_TRIGGER_OUT) eCrcTrigger);

    /* IF (DMA Error) */
    if(eResult != ADI_DMA_SUCCESS)
    {
        return (ADI_CRC_DMA_ERROR);
    }

    /* Return */
    return (ADI_CRC_SUCCESS);
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*@}*/

