/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_dacc.h
*
* @brief     DACC service API header file.
*
* @details
*            This is the primary header file DAC Controller (DACC) driver.
*/

/** @defgroup DACC_Driver DAC Controller APIs.
 */

/** @addtogroup DACC_Driver DAC Controller (DACC) Driver
 *  @{
 */
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <sys/platform.h>
#include <adi_osal.h>
#include <drivers/dacc/adi_dacc.h>
#include <services/int/adi_int.h>
#include "adi_dacc_def.h"
#include "adi_dacc_data.c"

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#endif /* _MISRA_RULES */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/**
 * @brief       Opens DACC Device.
 *
 * @details     DACC device instance should be opened to call any other APIs.
 *              If the device is opened successfully the handle to the device
 *              is returned to the application. The opened instance can be
 *              closed using #adi_dacc_Close API.
 *
 * @param [in]  nDeviceNum      DACC instance number to be opened. The processor
 *                              can have one or more instances of DACC. This
 *                              indicates the instance number of the DACC to be
 *                              opened.
 *
 * @param [in]  pMemory         Memory required for the operation of DACC driver.
 *                              The size of the given memory should be equal to
 *                              #ADI_DACC_MEMORY bytes.
 *
 * @param [out] phDevice        Handle to the instance that is opened.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully opened the DACC device.
 *
 * @sa adi_dacc_Close()
 */
ADI_DACC_RESULT  adi_dacc_Open (
    uint32_t                   nDeviceNum,
    void const                 *pMemory,
    ADI_DACC_HANDLE            *phDevice
)
{
    /* Pointer to the device data*/
    ADI_DACC_DEVICE     *pDevice = NULL;

#ifdef ADI_DEBUG
     
    /* Check if the given device number is valid */
    if(nDeviceNum > ADI_DACC_NUM_DEVICES) {
        return ADI_DACC_INVALID_DEVICE_NUM;
    }

    /* Check if the given pointer are valid */
    if(pMemory == NULL || phDevice == NULL) {
        return ADI_DACC_NULL_POINTER;
    }

    /* Check if device is already opened */
    if(gDACCDevices[nDeviceNum].eDeviceState != ADI_DACC_STATE_NOT_OPENED) {
        return ADI_DACC_DEVICE_IN_USE;
    }

    /* Assert to validate the memory sizes */
    assert(ADI_DACC_MEMORY  >=  sizeof(ADI_DACC_DEV_DATA));
#endif /* ADI_DEBUG */ 
   
    /* Clear the given memory */
    memset((void*)pMemory, 0, sizeof(ADI_DACC_DEV_DATA));

    /* Set the device pointer */
    pDevice =(ADI_DACC_DEVICE*) &gDACCDevices[nDeviceNum];

    /* Save the given memory pointer */
    pDevice->pDacData  =   (ADI_DACC_DEV_DATA *) pMemory;

    /* Resets registers and hooks up interrupt handlers */
    if(InitializeDAC((ADI_DACC_HANDLE)pDevice,pDevice->eInterfaceNum) != ADI_DACC_SUCCESS) {
        return ADI_DACC_INT_FAILED;
    }

    /* Disable fifo empty interrupts */
    pDevice->pDacRegs->IMSK_SET = (pDevice->eInterfaceNum == ADI_DACC_DACIF0) ? 
                                   BITM_DACC_IMSK_CINT0 : BITM_DACC_IMSK_CINT1;

    /* Install the error interrupt handler - error handler is same for both dacs */
    if(adi_int_InstallHandler(ADI_INT_DACC0_ERR,
                              ErrorInterruptHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS) {
        return ADI_DACC_INT_FAILED;
    }


#if ADI_DACC_CFG_BIO
    /* Create the DAC semaphore */
    if(adi_osal_SemCreateStatic(pDevice->pDacData->SemMemory,
                                ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                &pDevice->pDacData->hDacSem,
                                0) != ADI_OSAL_SUCCESS)
    {
        return ADI_DACC_SEM_FAILED;
    }
#endif

    /* Mark the device as opened */
    adi_osal_EnterCriticalRegion();
    pDevice->eDeviceState =   ADI_DACC_STATE_OPENED;
    adi_osal_ExitCriticalRegion();

  
    /* Mask off underflow errors. In programmed i/o mode underrun is used to 
     * send the next sample.
     */ 
    pDevice->pDacRegs->ERRMSK = (BITM_DACC_ERRMSK_SET_DUVF0 | BITM_DACC_ERRMSK_SET_DUVF1);

    /* Return the device handle */
    *phDevice = (ADI_DACC_HANDLE) pDevice;

    return ADI_DACC_SUCCESS;
}

/**
 * @brief       Closes DACC Device.
 *
 * @details     Once the DACC device is closed none of the APIs will
 *              work with the given handle.
 *
 * @param [in]  hDevice         Handle to the DACC device to be close.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully closed the DACC device.
 *
 * @sa adi_dacc_Open()
 */
ADI_DACC_RESULT  adi_dacc_Close (
   ADI_DACC_HANDLE const        hDevice
)
{
    /* Pointer to the device data */
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

    /* Result code */
    ADI_DACC_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* wait until data is flushed out completely */
    if (pDevice->eInterfaceNum == ADI_DACC_DACIF0) {
        while(pDevice->pDacRegs->STAT & BITM_DACC_STAT_DPND0);
    }
    else {
        while(pDevice->pDacRegs->STAT & BITM_DACC_STAT_DPND1);
    }

    uint32_t  DacInterruptID = (pDevice->eInterfaceNum == ADI_DACC_DACIF0) ? ADI_INT_DACC0_DAC0 : ADI_INT_DACC0_DAC1;
    
    /* un-install dac interrupt handler */
    if(adi_int_UninstallHandler(DacInterruptID)!= ADI_INT_SUCCESS) {
        return ADI_DACC_INT_FAILED;
    }
#if ADI_DACC_CFG_BIO
    /* destroy the semaphore */
    if (pDevice->pDacData->hDacSem) {
        adi_osal_SemDestroyStatic(pDevice->pDacData->hDacSem);
    }
#endif

    /* set the device state to not opened */
    pDevice->eDeviceState = ADI_DACC_STATE_NOT_OPENED;
    
    return (ADI_DACC_SUCCESS);
}

/**
 * @brief       Register or unregister an optional callback with DACC.
 *
 * @details     When a callback is registered some of the buffer transaction
 *              APIs cannot be used. The list of APIs that cannot be used
 *              are:
 *
 *              adi_dacc_GetBuffer()
 *              adi_dacc_IsBufferAvailable()
 *
 *              When the callback is registered the processed buffer pointer
 *              is returned via the callback in the #ADI_DACC_EVENT_BUFFER_PROCESSED
 *              event. Also all the hardware errors are returned via the
 *              callback. Please see the enumeration ADI_DACC_ERR.
 *
 *              When the callback is not registered the processed buffer pointer
 *              can be accessed using the #adi_dacc_GetBuffer(). When a hardware
 *              error is detected the API #adi_dacc_GetBuffer will immediately
 *              return with an error #ADI_DACC_HW_ERROR_DETECTED. Upon knowing
 *              that there is a hardware error(s), the application can call the
 *              API #adi_dacc_GetHWErrorStatus() to know exact error(s).
 *
 *              The callback can be unregistered by passing the pfCallback parameter
 *              as NULL.
 *
 * @note        This API cannot be called when DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device.
 *
 * @param [in]  pfCallback      Pointer to the callback function, which will be called
 *                              by the driver to notify hardware errors and events.
 *
 * @param [in]  pCBParam        Callback parameter which will be passed back to the
 *                              application when the callback is called.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully registered or unregistered
 *                              the callback.
 *
 * @sa adi_dacc_Open()
 */
ADI_DACC_RESULT  adi_dacc_RegisterDevCallback (
    ADI_DACC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;
    ADI_DACC_RESULT    eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  !=   ADI_DACC_STATE_OPENED) {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();
    pDevice->pDacData->pfCallback = pfCallback;
    pDevice->pDacData->pCBParam  = pCBParam;
    adi_osal_ExitCriticalRegion();

    return ADI_DACC_SUCCESS;
}

/**
 * @brief       Register or unregister an optional erorr callback with DACC.
 *
 * @details     
 *
 * @note        This API cannot be called when DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device.
 *
 * @param [in]  pfCallback      Pointer to the callback function, which will be called
 *                              by the driver to notify hardware errors and events.
 *
 * @param [in]  pCBParam        Callback parameter which will be passed back to the
 *                              application when the callback is called.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully registered or unregistered
 *                              the callback.
 *
 * @sa adi_dacc_Open()
 */
ADI_DACC_RESULT  adi_dacc_RegisterErrCallback (
    ADI_DACC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;
    ADI_DACC_RESULT    eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  !=   ADI_DACC_STATE_OPENED) {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();
    pDevice->pDacData->pfErrCallback = pfCallback;
    pDevice->pDacData->pErrCBParam  = pCBParam;
    adi_osal_ExitCriticalRegion();

    return ADI_DACC_SUCCESS;
}

/**
 * @brief       Submits the given buffer for DACC.
 *
 * @details     This is a non-blocking call. Only one buffer can be
 *              submitted at a time. The next buffer can be submitted only when the
 *              previously submitted buffer is processed.
 *
 *              When the callback is not registered the processed buffer can be
 *              obtained back by using using the API #adi_dacc_GetBuffer(). If the
 *              callback is registered the processed buffer is returned as callback
 *              argument for the event #ADI_DACC_EVENT_BUFFER_PROCESSED.
 *
 * @param [in]  hDevice         Handle to the DACC device to which the buffer to be
 *                              submitted.
 *
 * @param [in]  pBuffer         Pointer to the buffer to be submitted.
 *
 * @param [in]  pBufSize        Buffer size in terms of samples.
 *
 * @param [in]  nIncrement      Buffer address increment in terms of bytes which will be
 *                              applied after sending each sample to the DAC. The increment
 *                              is a signed number. So the increment can also be a negative
 *                              number.
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully submitted the buffer.
 *
 * @sa  adi_dacc_GetBuffer()
 * @sa  adi_dacc_IsBufferAvailable()
 * @sa  adi_dacc_SubmitCircBuffer()
 */
ADI_DACC_RESULT  adi_dacc_SubmitBuffer (
    ADI_DACC_HANDLE const       hDevice,
    void * const                pBuffer,
    uint32_t                    nBufSize,
    int32_t                     nIncrement
)
{
     return (ProcessBuffer(hDevice,pBuffer,nBufSize,nIncrement,false));
}

/**
 * @brief       Submit circular buffer for DACC.
 *
 * @details     This API only works in DMA mode. So DMA mode should be set first
 *              (using the API #adi_dacc_EnableDMAMode) before using this API.
 *              It is a is a non-blocking call and returns immediately after
 *              submitting the buffer.
 *
 *              The read pointer to the circular buffer is updated by the device and the
 *              write pointers should be maintained by the application.
 *              The application can get the current read pointer by using the API
 *              #adi_dacc_GetCurrRdPtr()
 *
 * @note        This API cannot be called after enabling the DAC.
 *
 * @param [in]  hDevice         Handle to the DACC device to which the buffer to be
 *                              submitted.
 *
 * @param [in]  pBuffer         Pointer to the circular buffer to be submitted.
 *
 * @param [in]  nBufSize        Buffer size in terms of samples.
 *
 * @param [in]  nIncrement      Buffer address increment in terms of bytes which will be
 *                              applied after sending each sample to the DAC. The increment
 *                              is a signed number. So can be a negative number.
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully submitted the circular buffer.
 *
 * @sa  adi_dacc_SubmitBuffer()
 * @sa  adi_dacc_GetCurrRdPtr()
 */
ADI_DACC_RESULT  adi_dacc_SubmitCircBuffer (
    ADI_DACC_HANDLE const       hDevice,
    void * const                pBuffer,
    uint32_t                    nBufSize,
    int32_t                     nIncrement
)
{
    return (ProcessBuffer(hDevice,pBuffer,nBufSize,nIncrement,true));
}

/**
 * @brief       Gets back the processed buffer.
 *
 * @details     This API blocks until the processed buffer is available. To avoid the
 *              block application can use the API #adi_dacc_IsBufferAvailable() to know
 *              if there is a processed buffer available. Upon knowing that there is
 *              buffer available this API can be used.
 *
 *              This API will return immediately if a buffer is available. It can also
 *              return with a partially processed buffer with an error
 *              #ADI_DACC_HW_ERROR_DETECTED if an hardware error is detected. To know
 *              the specifics of the error application can call the API
 *              #adi_dacc_GetHWErrorStatus().
 *
 * @note        This API cannot be called in circular buffering mode. That is if a
 *              circular buffer is submitted before by using #adi_dacc_SubmitCircBuffer.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice         Handle to the DACC device from which buffer to be
 *                              obtained.
 *
 * @param [out] ppBuffer        Pointer to a location where the processed buffer pointer
 *                              is written.
 *
 * @param [in] msTimeout        Timeout in milli-seconds, a 0 value indicates infinite timeout
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully obtained the processed buffer.
 *  - #ADI_DACC_FAILURE         Failure will be returned in case of timeout or any semaphore errors and 
 *                              when used when callback is supplied (non-blocking i/o) 
 *
 * @note                        If timeout error occurs then applications are required to increase the timeout 
 *                              and re-test their application. 
 *
 * @sa  adi_dacc_SubmitBuffer()
 * @sa  adi_dacc_GetHWErrorStatus()
 * @sa  adi_dacc_IsBufferAvailable()
 */
ADI_DACC_RESULT  adi_dacc_GetBuffer (
    ADI_DACC_HANDLE  const      hDevice,
    void ** const               ppBuffer
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  !=   ADI_DACC_STATE_ENABLED) {
        return ADI_DACC_INVALID_OPERATION;
    }

    /* Check if callback is supplied if so we operate in Non Blocking mode */
    if(pDevice->pDacData->pfCallback  !=   NULL) {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* only applicable in Blocking I/O */
#if ADI_DACC_CFG_BIO

    /* pend on the semaphore */
    if (adi_osal_SemPend (pDevice->pDacData->hDacSem,ADI_OSAL_TIMEOUT_FOREVER) != ADI_OSAL_SUCCESS) {
            return ADI_DACC_SEM_FAILED;
    }

    /* Mark the buffer state to free */
    pDevice->pDacData->BufferInfo.eBufferState = ADI_DACC_BUF_STATE_FREE;

    if (pDevice->pDacData->eErrorStatus != 0u) {
            return ADI_DACC_HW_ERROR_DETECTED;
    }

    /* Set the buffer pointer */
    *ppBuffer = pDevice->pDacData->BufferInfo.pBuffer;

    /* Set the buffer pointer to NULL */
    pDevice->pDacData->BufferInfo.pBuffer = NULL;

    return (ADI_DACC_SUCCESS );

/* NON-BLOCKING mode this call returns an error */
#else 
    *ppBuffer = NULL;
    return (ADI_DACC_FAILURE);
#endif
}

/**
 * @brief       Gets the current read pointer.
 *
 * @details     This API is valid only in circular buffering mode. The API returns the
 *              address of the buffer from where the device will be reading the samples
 *              next. This information will be useful to compute the amount of room
 *              available to fill the samples.
 *
 *              The write pointer to the circular buffer is not maintained by the driver.
 *              It is application responsibility to maintain the write pointer and also
 *              make sure that there are enough samples in the circular buffer to avoid
 *              the underflow.
 *
 *              When a callback is not registered this API will with an error
 *              #ADI_DACC_HW_ERROR_DETECTED if an hardware error is detected. To know
 *              the specifics of the error(s) application can call the API
 *              #adi_dacc_GetHWErrorStatus().
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice         Handle to the DACC device whose read pointer to be read.
 *
 * @param [out]  ppRdPtr        Pointer to a location where the current read pointer is
 *                              written.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully read back the current read pointer.
 *
 * @sa  adi_dacc_SubmitCircBuffer()
 * @sa  adi_dacc_GetHWErrorStatus()
 */
ADI_DACC_RESULT  adi_dacc_GetCurrRdPtr(
    ADI_DACC_HANDLE  const      hDevice,
    void **                     ppRdPtr
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;
    uint32_t           NumBytesProcessed = 0;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT  eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  !=   ADI_DACC_STATE_ENABLED) {
        return ADI_DACC_INVALID_OPERATION;
    }

    /* check if operating in circular buffer mode */
    if(pDevice->pDacData->BufferInfo.bCircular  !=   true) {
        return ADI_DACC_INVALID_OPERATION;
    }

#endif /* ADI_DEBUG */
    
    ADI_DACC_BUFFER_INFO *pBufferInfo = &pDevice->pDacData->BufferInfo;

    if (pDevice->pDacData->BufferInfo.eBufferState != ADI_DACC_BUF_STATE_INUSE) {
        return( ADI_DACC_INVALID_OPERATION);
    }
  
    if (pDevice->pDacData->bDMAMode)
    {
        NumBytesProcessed = pBufferInfo->nBufferSize - 
                                     ((pDevice->eInterfaceNum == ADI_DACC_DACIF0) ? 
                                     pDevice->pDacRegs->CNTCUR0 : pDevice->pDacRegs->CNTCUR1);

        *ppRdPtr = (uint8_t*)pBufferInfo->pBuffer + NumBytesProcessed; 
    }
#if (ADI_DACC_CFG_ALL || ADI_DACC_CFG_PIO)
    else
    {
        NumBytesProcessed = pDevice->pDacData->BufferInfo.nCurBufferIdx - 1;

        *ppRdPtr = (uint8_t*)pBufferInfo->pBuffer + NumBytesProcessed; 
    }
#endif

    return (ADI_DACC_SUCCESS);
}

/**
 * @brief       Checks if a empty buffer is available.
 *
 * @details     This API can be used to check if an empty buffer is available.
 *              Upon knowing that there is a buffer available the API
 *              #adi_dacc_GetBuffer can be used to get back the empty buffer.
 *
 * @note        This API is not valid in circular buffering mode and when callback is
 *              registered.
 *
 * @param [in]   hDevice        Handle to the DACC device from which the buffer status
 *                              to be known.
 *
 * @param [out]  pbAvailable    Pointer to a location where the buffer availability
 *                              status is written.
 * \n
 *                              - 'true'  -   If buffer is available.
 *                              - 'false' -   If buffer is not available.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully queried the buffer status.
 *
 * @sa  adi_dacc_SubmitBuffer()
 * @sa  adi_dacc_GetBuffer()
 */
ADI_DACC_RESULT  adi_dacc_IsBufferAvailable (
    ADI_DACC_HANDLE const   hDevice,
    bool * const            pbAvailable
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if ((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* Set dma mode before device is enabled */
    if (pDevice->eDeviceState  !=   ADI_DACC_STATE_ENABLED) {
        return ADI_DACC_INVALID_OPERATION;
    }

    /* This call can not be used in circular buffer mode */
    if (pDevice->pDacData->BufferInfo.bCircular) {
        return ADI_DACC_INVALID_OPERATION;
    }

    /* if buffer is NULL then its not a vaild buffer */
    if (pDevice->pDacData->BufferInfo.pBuffer == NULL) {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    ADI_DACC_BUFFER_INFO  *pBufferInfo = &pDevice->pDacData->BufferInfo;

    /* Set the buffer available boolean to true if there is a buffer */
    *pbAvailable = (pBufferInfo->eBufferState == ADI_DACC_BUF_STATE_PROCESSED) ? true : false;

    return (ADI_DACC_SUCCESS);
}

/**
 * @brief       Enable or disable the DAC data flow.
 *
 * @details     After the buffer DACC is configured and buffer is submitted DACC should
 *              be enabled to start sending the data to DAC.
 *
 * @param [in]  hDevice         Handle to the DACC device which should be enabled or
 *                              disabled.
 *
 * @param [in]  bEnable         Flag which indicates whether to enable or disable the DACC.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully enabled or disabled the DACC.
 */
ADI_DACC_RESULT  adi_dacc_Enable(
    ADI_DACC_HANDLE const       hDevice,
    bool                        bEnable
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  !=   ADI_DACC_STATE_OPENED) {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Enable dma if operational mode is dma */
    if (pDevice->pDacData->bDMAMode) 
    {
        if (pDevice->eInterfaceNum == ADI_DACC_DACIF0) {
            while(pDevice->pDacRegs->STAT & BITM_DACC_STAT_DPND0);
        }
        else {
            while(pDevice->pDacRegs->STAT & BITM_DACC_STAT_DPND1);
        }
        pDevice->pDacCtrlReg->CTL |= BITM_DACC_CTL0_DMAEN; 
    } 

    /* Enable DAC */
    if (bEnable) {
        pDevice->eDeviceState = ADI_DACC_STATE_ENABLED;
        pDevice->pDacCtrlReg->CTL |=  BITM_DACC_CTL0_EN;
    }
    else {
        pDevice->eDeviceState = ADI_DACC_STATE_OPENED;
        pDevice->pDacCtrlReg->CTL &=  ~BITM_DACC_CTL0_EN;
    }

    return (ADI_DACC_SUCCESS);
}

/**
 * @brief       Configures data interface.
 *
 * @details     Configures the parameters related to the data interface. This API
 *              cannot be used after the DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device whose data interface to be
 *                              configured.
 *
 * @param [in]  bLSBFirst       Flag to indicate if the data to be sent out as LSB
 *                              first or MSB first.
 * \n
 *                              - 'true'  -   For LSB first.
 *                              - 'false' -   For MSB first.
 *
 * @param [in]  nDataWidth      The data width is in terms of number of bits.
 *                              A valid value can be 1 to 16. Data for bit widths
 *                              other than 16bit should be LSB aligned.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully configured the data interface.
 */
ADI_DACC_RESULT  adi_dacc_ConfigDataIF (
    ADI_DACC_HANDLE             hDevice,
    bool                        bLSBFirst,
    uint8_t                     nDataWidth
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS)
    {
        return eResult;
    }

    /* Set dma mode before device is enabled */
    if(pDevice->eDeviceState  ==   ADI_DACC_STATE_ENABLED)
    {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Least significant bit is transferred first */
    if(bLSBFirst) {
        pDevice->pDacCtrlReg->CTL |= BITM_DACC_CTL0_LSBF;
    }
    else {
        pDevice->pDacCtrlReg->CTL &= ~BITM_DACC_CTL0_LSBF;
    }

    /* Set DAC sample data witdh */
    pDevice->pDacCtrlReg->CTL |= ((nDataWidth & 0xf)  << BITP_DACC_CTL0_DLEN);

    return ADI_DACC_SUCCESS;
}

/**
 * @brief       Configure DAC clock.
 *
 * @details     Configures the parameters related to the DAC clock. This API
 *              cannot be used after the DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device whose clock to be
 *                              configured.
 *
 * @param [in]  nClkDiv         Clock divide for DAC clock DACCK.
 *                              DACCK frequency = SCLK Frequency/(nClkDiv + 1)
 *
 * @param [in]  bGatedClk       Flag which indicates if clock gating to be enabled
 *                              or disabled. When clock gating is enabled DAC Clock
 *                              is only generated only when data line is driven.
 *                              Otherwise DAC Clock is always driven.
 * \n
 *                              - 'true'  -   Enable clock gating.
 *                              - 'false' -   Disable clock gating.
 *
 * @param [in]  bRisingEdge     Flag to indicate the clock polarity.
 * \n
 *                              - 'true'  -   Frame Sync and Data are driven at falling edge of
 *                                            DAC Clock.
 *                              - 'false' -   Frame Sync and Data are driven at rising edge of
 *                                            DAC clock.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully configured clock.
 */
ADI_DACC_RESULT  adi_dacc_ConfigClock(
    ADI_DACC_HANDLE             hDevice,
    uint16_t                    nClkDiv,
    bool                        bGatedClk,
    bool                        bRisingEdge
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT  eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS)
    {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  ==   ADI_DACC_STATE_ENABLED)
    {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    if(bGatedClk) {
            pDevice->pDacCtrlReg->CTL |= BITM_DACC_CTL0_GCKEN;
    }
    else {
            pDevice->pDacCtrlReg->CTL &= ~BITM_DACC_CTL0_GCKEN;
    }

    if(bRisingEdge) {
   
            pDevice->pDacCtrlReg->CTL |= BITM_DACC_CTL0_CKPOL;
    }
    else{
            pDevice->pDacCtrlReg->CTL &= ~BITM_DACC_CTL0_CKPOL;
    }

    pDevice->pDacDmaRegs->TC |=  nClkDiv; 
   
    return (ADI_DACC_SUCCESS);
}

/**
 * @brief       Set DMA transfer width.
 *
 * @details     By default the DMA transfer width is 16bit. It can be changed
 *              to 32bit DMA to achieve better throughput.
 *
 * @note        This API is only valid in DMA mode. So DMA mode should be enabled
 *              first (using API #adi_dacc_EnableDMAMode) before calling this API.
 *
 * @note        This API cannot be called after the DACC is enabled.
 *
 * @param [in]  hDevice         Handle to the DACC device whose DMA width to be set.
 *
 * @param [in]  b32bitMode      Flag to indicate whether to enable 32bit DMA.
 * \n
 *                              - 'true'  -   For 32bit DMA.
 *                              - 'false' -   For 16bit DMA.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the DMA width.
 *
 * @sa  adi_dacc_EnableDMAMode()
 * @sa  adi_dacc_Enable()
 */
ADI_DACC_RESULT  adi_dacc_SetDMAWidth(
    ADI_DACC_HANDLE             hDevice,
    bool                        b32bitMode
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS)
    {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  ==   ADI_DACC_STATE_ENABLED)
    {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

     if (b32bitMode) {
         pDevice->pDacCtrlReg->CTL |=  BITM_DACC_CTL0_DMAW;
     }
     else{
         pDevice->pDacCtrlReg->CTL &= ~BITM_DACC_CTL0_DMAW;
     }

     return (ADI_DACC_SUCCESS);
}

/**
 * @brief       Configures the frame sync.
 *
 * @details     Configures the frame sync signal related parameters.
 *
 * @note        This API cannot be called after enabling the DACC.
 *
 * @param [in]  hDevice         Handle to the DACC device whose Frame Sync to
 *                              be configured.
 *
 * @param [in]  bActiveHigh     Flag to indicate whether the frame sync signal
 *                              is active high or active low (default).
 * \n
 *                              - 'true'  -   For active high.
 *                              - 'false  -   For active low.
 *
 * @param [in]  nIdlePeriod     Idle period period between two active pulses of
 *                              Frame sync signal. By setting the idle period
 *                              the periodicity of the frame sync signal can be
 *                              varied. The periodicity of the signal is given
 *                              by:
 *
 *                              Periodicity = (nIdlePeriod + 1) + 16.
 *
 *                              The range of nIdlePriod can be 0 to 65535.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully configured the frame sync.
 */
ADI_DACC_RESULT  adi_dacc_ConfigFrameSync(
    ADI_DACC_HANDLE             hDevice,
    bool                        bActiveHigh,
    uint16_t                    nIdlePeriod
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS)
    {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  ==   ADI_DACC_STATE_ENABLED)
    {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    if (bActiveHigh) {
        pDevice->pDacCtrlReg->CTL |=  BITM_DACC_CTL0_SYNCPOL;
    }
    else {
        pDevice->pDacCtrlReg->CTL &=  ~BITM_DACC_CTL0_SYNCPOL;
    }

    pDevice->pDacDmaRegs->TC |=  (nIdlePeriod << BITP_DACC_TC0_FSIDLE);

    return(ADI_DACC_SUCCESS);
}

/**
 * @brief       Enable or disable DMA mode.
 *
 * @details     Enable DMA to get the data from the event data registers to memory.
 *
 * @note        This API cannot be called after enabling the DACC.
 *
 * @param [in]  hDevice         Handle to the DACC device which should be enabled
 *                              or disabled.
 *
 * @param [in]  bEnable         Flag which indicates whether to enable or disable
 *                              the DMA mode.
 * \n
 *                              - 'true'  -   To enable DMA mode.
 *                              - 'false' -   To disable DMA mode.
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully enabled the DMA mode.
 */
ADI_DACC_RESULT  adi_dacc_EnableDMAMode(
    ADI_DACC_HANDLE const       hDevice,
    bool                        bEnable
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS)
    {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  ==   ADI_DACC_STATE_ENABLED)
    {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    pDevice->pDacData->bDMAMode = bEnable;

    return ADI_DACC_SUCCESS;
}


/**
 * @brief       Set the mask for error interrupt.
 *
 * @details     Set the error mask to disable the errors reported by DAC
 *              Controller. The interrupts specified by the mask will be 
 *              masked. By default underflow errors are disabled.
 *
 *              When a callback is registered the errors are reported
 *              via the callback.
 *
 *              When the callback is not registered application the
 *              API #adi_dacc_GetBuffer() will return with an error
 *              #ADI_DACC_HW_ERROR_DETECTED when an error is detected.
 *              Upon knowing that there is an error application can
 *              call the API #adi_dacc_GetHWErrorStatus to know the
 *              actual error(s).
 *
 *
 * @param [in]  hDevice         Handle to the DACC device whose error mask to
 *                              be set.
 *
 * @param [in]  nErrMask        Mask is constructed orring required errors in
 *                              ADI_DACC_ERR.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the error mask.
 */
ADI_DACC_RESULT  adi_dacc_SetErrMask(
    ADI_DACC_HANDLE const       hDevice,
    uint32_t                    nErrMask
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */

    /* Set the error mask */
    pDevice->pDacRegs->ERRMSK_SET = (nErrMask & 0x000000001Fu);

    return ADI_DACC_SUCCESS; 
}

/**
 * @brief       Clear the mask for error interrupt.
 *
 * @details     Clear the error mask to enable the errors reported by DAC
 *              Controller. By default only underflow errors are disabled.
 *              Rest of the errors are enabled.
 *
 *              When a callback is registered the errors are reported
 *              via the callback.
 *
 *              When the callback is not registered application the
 *              API #adi_dacc_GetBuffer() will return with an error
 *              #ADI_DACC_HW_ERROR_DETECTED when an error is detected.
 *              Upon knowing that there is an error application can
 *              call the API #adi_dacc_GetHWErrorStatus to know the
 *              actual error(s).
 *
 *
 * @param [in]  hDevice         Handle to the DACC device whose error mask to
 *                              be set.
 *
 * @param [in]  nErrMask        Mask is constructed orring required errors in
 *                              ADI_DACC_ERR.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the error mask.
 */
ADI_DACC_RESULT  adi_dacc_ClearErrMask(
    ADI_DACC_HANDLE const       hDevice,
    uint32_t                    nErrMask
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT    eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */

    /* Clear the error mask */
    pDevice->pDacRegs->ERRMSK_CLR = (nErrMask & 0x000000001Fu);

    return ADI_DACC_SUCCESS; 
}

/**
 * @brief       Queries for the hardware errors.
 *
 * @details     When the DAC controller callback is not registered, the hardware
 *              errors cannot be returned via the callback. So application can
 *              use this API to query for the hardware errors. The errors will
 *              be cleared after reading them.
 *
 *              When a callback is not registered the #adi_dacc_GetBuffer returns
 *              with an error #ADI_DACC_HW_ERROR_DETECTED when a hardware error is
 *              detected. Upon getting this error application can use this API to
 *              query the errors detected.
 *
 * @param [in]  hDevice         Handle to DACC device whose hardware error status
 *                              to be queried.
 *
 * @param [out] pHWErrStatus    Pointer to a location where the hardware error
 *                              status word is written.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully queried the hardware error status.
 *
 * @sa adi_dacc_GetBuffer()
 */
ADI_DACC_RESULT  adi_dacc_GetHWErrorStatus(
    ADI_DACC_HANDLE             hDevice,
    uint32_t *const             pHWErrStatus
)
{
    ADI_DACC_DEVICE      *pDevice = (ADI_DACC_DEVICE *) hDevice;

    /* Get the error status */
    *pHWErrStatus =  pDevice->pDacData->eErrorStatus;

    /* Reset error status */
    pDevice->pDacData->eErrorStatus = 0;

    return (ADI_DACC_SUCCESS);
}


/**
 * @brief       Sets a given DACC register.
 *
 * @details     Most of the commonly used register fields are accessed via
 *              the APIs. If a register field is not covered by the APIs
 *              application can use this API to set the given register
 *              with the given value.
 *
 * @note        This API cannot be called after enabling the DACC.
 *
 * @param [in]  hDevice         Handle to the DACC device whose register to
 *                              be set.
 *
 * @param [in]  nRegAddr        Address of the register to be set.
 *
 * @param [in]  nRegValue       Value to be set into the given register.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully set the register.
 */
ADI_DACC_RESULT  adi_dacc_SetRegister (
    ADI_DACC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t                    nRegValue
)
{
     ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    ADI_DACC_RESULT  eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  !=   ADI_DACC_STATE_OPENED) {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* set the requested register contents */
    *((volatile uint32_t *)nRegAddr) = nRegValue;

    return (ADI_DACC_SUCCESS);
}


/**
 * @brief       Gets the contents of the given DACC register.
 *
 * @details     Most of the commonly used register fields are accessed via
 *              the APIs. If a register field is not covered by the APIs
 *              application can use this API to get contents of the given
 *              register.
 *
 * @param [in]  hDevice         Handle to the DACC device whose register to
 *                              be read.
 *
 * @param [in]  nRegAddr        Address of the register to be read.
 *
 * @param [out] pRegValue       Pointer to a location where the contents
 *                              given register is written.
 *
 * @return      Status
 *  - #ADI_DACC_SUCCESS         If successfully read the register.
 */
ADI_DACC_RESULT  adi_dacc_GetRegister (
    ADI_DACC_HANDLE             hDevice,
    uint32_t                    nRegAddr,
    uint32_t * const            pRegValue
)
{
     ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;
#ifdef ADI_DEBUG
      ADI_DACC_RESULT  eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if(pDevice->eDeviceState  !=   ADI_DACC_STATE_OPENED) {
        return ADI_DACC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Return the requested register contents */
    *pRegValue   =   *((volatile uint32_t *)nRegAddr);

    return (ADI_DACC_SUCCESS);
}

#if (ADI_DACC_CFG_ALL || ADI_DACC_CFG_DMA)
/*
 *  DACC DMA interrupt handler. This function can be selectively compiled out if programemd i/o
 *  is used for DAC transmission.
 */
static void DacDMAInterruptHandler(ADI_DACC_DEVICE *pDevice)
{
    uint32_t interruptStauts = pDevice->pDacRegs->ISTAT;
    ADI_DACC_BUFFER_INFO   *pBufferInfo = &pDevice->pDacData->BufferInfo;

    /* Linear buffer mode 
     * Set the buffer state to free so that next buffer is accepted
     * Disable DMA SubmitBuffer will re-enable DMA once new buffer is submitted
     * in circular buffer mode we do not accept any buffers while system is active 
     */
    if (!pBufferInfo->bCircular)
    {
        /* Mark the buffer state to processed */
        pDevice->pDacData->BufferInfo.eBufferState = ADI_DACC_BUF_STATE_PROCESSED;

        /* disable dma, new transaction will enable dma again */
        pDevice->pDacCtrlReg->CTL &= ~BITM_DACC_CTL0_DMAEN;
    }
     
    /* acknowledge the associated dac interrupt */
    switch(pDevice->eInterfaceNum)
    {
        case ADI_DACC_DACIF0:
             if (pDevice->pDacRegs->ISTAT & BITM_DACC_ISTAT_DINT0)
             {
                 pDevice->pDacRegs->ISTAT = BITM_DACC_ISTAT_DINT0;
             }
        break;

       case ADI_DACC_DACIF1:
             if (pDevice->pDacRegs->ISTAT & BITM_DACC_ISTAT_DINT1)
             {
                 pDevice->pDacRegs->ISTAT = BITM_DACC_ISTAT_DINT1;
             }
        break;
    }

    /* inform the application about buffer completion event */
    if (pDevice->pDacData->pfCallback)
    {
         if(!pBufferInfo->bCircular)
            pBufferInfo->eBufferState = ADI_DACC_BUF_STATE_FREE;

         /* call back is defined we throw pass the buffer back to applicaiton */
         pDevice->pDacData->pfCallback(pDevice->pDacData->pCBParam, ADI_DACC_EVENT_BUFFER_PROCESSED,pBufferInfo->pBuffer);
    }
    else
    {
#if ADI_DACC_CFG_BIO
         /* post the semaphore  */
        adi_osal_SemPost(pDevice->pDacData->hDacSem);
#endif
    }
}
#endif /* ADI_DACC_CFG_DMA */

#if (ADI_DACC_CFG_ALL || ADI_DACC_CFG_PIO)
/* write data in to the dac fifo */
static void WriteFifo(ADI_DACC_HANDLE hDevice)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;
    ADI_DACC_BUFFER_INFO *pBufferInfo = &pDevice->pDacData->BufferInfo;

    /* check if we have data to send */
    if ((pBufferInfo->eBufferState == ADI_DACC_BUF_STATE_INUSE) && 
        pBufferInfo->nCurBufferIdx < pBufferInfo->nBufferSize)
    {
        uint32_t interruptStatus =  pDevice->pDacRegs->ISTAT;
        uint32_t errorStatus =pDevice->pDacRegs->ERRSTAT;

        /* space availble in DAC 0 */
        if (pDevice->eInterfaceNum == ADI_DACC_DACIF0) 
        {
            while(interruptStatus & BITM_DACC_ISTAT_CINT0) 
            {
               *pREG_DACC0_DAT0 = *(((uint16_t*)pBufferInfo->pBuffer + pBufferInfo->nCurBufferIdx));
                pBufferInfo->nCurBufferIdx += pBufferInfo->nIncrement;
                pDevice->pDacRegs->ISTAT = BITM_DACC_ISTAT_CINT0;
                interruptStatus = pDevice->pDacRegs->ISTAT;
            }
        }
        else 
        {
            while (interruptStatus & BITM_DACC_ISTAT_CINT1) 
            {
               *pREG_DACC0_DAT1 = *(((uint16_t*)pBufferInfo->pBuffer + pBufferInfo->nCurBufferIdx));
                pBufferInfo->nCurBufferIdx += pBufferInfo->nIncrement;
                pDevice->pDacRegs->ISTAT = BITM_DACC_ISTAT_CINT1;
                interruptStatus = pDevice->pDacRegs->ISTAT;
            }
        }
   }
}

/*
 * DAC programmed i/o handler. When ever there is an underflow interrupt this handler gets called
 * and it schedules the next transfers until fifo is full. In case of circular buffer mode once
 * all data is transferred it will reset the buffer index to 0. In linear or circular buffer mode 
 * once transfer is completed it will callback to the registerd callback with the buffer processed
 * event. In case of blocking mode semaphore will get posted.
 */
static void DacPIOHandler(ADI_DACC_DEVICE *pDevice)
{
    uint32_t interruptStauts = pDevice->pDacRegs->ISTAT;
    ADI_DACC_BUFFER_INFO   *pBufferInfo = &pDevice->pDacData->BufferInfo;

    /* we got fifo empty interrupt so write more data if available */
    if (pBufferInfo->nCurBufferIdx < pBufferInfo->nBufferSize)
    {
        WriteFifo(pDevice);
    }
    else if (pBufferInfo->pBuffer != NULL)
    {
       /* set the buffer state to free in case of linear mode to accept another buffer 
        * in circular buffer mode we do not accept any buffers while system is active 
        */
        if (!pBufferInfo->bCircular)
        {
            /* set the buffer to processed state. Once the buffer is returnd to applicaiton the
             * state will be set to free. In case of nonblocking i/o it will be done once callback
             * is finished. In case of blocking i/o GetBuffer will reset the buffer.
             */
            pBufferInfo->eBufferState = ADI_DACC_BUF_STATE_PROCESSED;

            /* Disable programmed i/o interrupts */
            pDevice->pDacRegs->IMSK_SET = (pDevice->eInterfaceNum == ADI_DACC_DACIF0) ? 
                                           BITM_DACC_IMSK_CINT0 : BITM_DACC_IMSK_CINT1;

        }
        else 
        {
            /* reset the current buffer index */
            pBufferInfo->nCurBufferIdx = 0;

            /* start writing again */
            WriteFifo(pDevice);
        }

        /* inform the application about buffer completion event */
        if (pDevice->pDacData->pfCallback)
        {
           if(!pBufferInfo->bCircular)
            pBufferInfo->eBufferState = ADI_DACC_BUF_STATE_FREE;
            /* call back is defined we throw pass the buffer back to applicaiton */
            pDevice->pDacData->pfCallback(pDevice->pDacData->pCBParam, ADI_DACC_EVENT_BUFFER_PROCESSED,pBufferInfo->pBuffer);
        }
        else
        {
             /* post the semaphore  */
             adi_osal_SemPost(pDevice->pDacData->hDacSem);
        }
    }

}
#endif /* ADI_DACC_CFG_PIO */


/* 
 * Dac interrupt handler - calls appropriate handler depending on the mode of operation
 * for the current transfer.
 */
static void DacInterruptHandler(uint32_t iid, void *pCBParam)
{    
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) pCBParam;

    uint32_t interruptStauts = pDevice->pDacRegs->ISTAT;    

#ifdef ADI_DEBUG
    if (pDevice->pDacData == NULL) {
          ADI_DACC_STAT_INC(nDacUnexpectedInts);
          return;
    }
#endif

    if (pDevice->pDacData->bDMAMode) {
#if (ADI_DACC_CFG_ALL || ADI_DACC_CFG_DMA) 
    DacDMAInterruptHandler(pDevice);
#endif
    }
    else {
#if (ADI_DACC_CFG_ALL || ADI_DACC_CFG_PIO) 
    DacPIOHandler(pDevice);
#endif  /* ADI_DACC_PIO */
    }
}

/* Error handler is shared between two DACs. So we check for both dac's errors and call
 * the callback function.
 */
static void ErrorInterruptHandler(uint32_t iid, void *pCBParam)
{
    ADI_DACC_DEVICE      *pDevice =NULL;

   /* DAC memory errors */ 
    pDevice = &gDACCDevices[ADI_DACC_DACIF0];
    uint32_t  errorStatus = pDevice->pDacRegs->ERRSTAT;

   if (errorStatus & BITM_DACC_ERRSTAT_MER0) {
           ADI_DACC_STAT_INC(DaccStats.nDac0MemoryErr);
           pDevice->pDacData->eErrorStatus = ADI_DACC_ERR_MEMORY;
   }
   else if (errorStatus & BITM_DACC_ERRSTAT_DUVF0){
           ADI_DACC_STAT_INC(DaccStats.nDac0UnderFlowErr);
           pDevice->pDacData->eErrorStatus = ADI_DACC_ERR_UNDERFLOW;
   }
   else if (errorStatus & BITM_DACC_ERRSTAT_AER0) {
           ADI_DACC_STAT_INC(DaccStats.nDac0AlignmentErr);
           pDevice->pDacData->eErrorStatus = ADI_DACC_ERR_ADDR_ALIGN;
   }

   /* Release the semaphore in case a task is waiting on it */
   if( pDevice->pDacData->eErrorStatus)
   {
       if (pDevice->pDacData->pfErrCallback ) {
           pDevice->pDacData->pfErrCallback(pDevice->pDacData->pErrCBParam, ADI_DACC_EVENT_HWERR,(void*)errorStatus);
       }
       else {
           adi_osal_SemPost(pDevice->pDacData->hDacSem);
       }
   }
#if  (ADI_DACC_NUM_DEVICES > 1)
{
    pDevice = &gDACCDevices[ADI_DACC_DACIF1];

   if (errorStatus & BITM_DACC_ERRSTAT_MER1) {
           ADI_DACC_STAT_INC(DaccStats.nDac0MemoryErr);
           pDevice->pDacData->eErrorStatus = ADI_DACC_ERR_MEMORY;
   }
   else if (errorStatus & BITM_DACC_ERRSTAT_DUVF1){
           ADI_DACC_STAT_INC(DaccStats.nDac0UnderFlowErr);
           pDevice->pDacData->eErrorStatus = ADI_DACC_ERR_UNDERFLOW;
   }
   else if (errorStatus & BITM_DACC_ERRSTAT_AER1) {
           ADI_DACC_STAT_INC(DaccStats.nDac0AlignmentErr);
           pDevice->pDacData->eErrorStatus = ADI_DACC_ERR_ADDR_ALIGN;
   }

   /* Release the semaphore in case a task is waiting on it */
   if( pDevice->pDacData->eErrorStatus)
   {
       if (pDevice->pDacData->pfErrCallback ) {
           pDevice->pDacData->pfErrCallback(pDevice->pDacData->pErrCBParam, ADI_DACC_EVENT_HWERR,(void*)errorStatus);
       }
       else {
           adi_osal_SemPost(pDevice->pDacData->hDacSem);
       }
   }
}
#endif

   /* acknowlede the error interrupts */
   pDevice->pDacRegs->ERRSTAT = errorStatus;
}

/* schedules dma transfer */
static void ScheduleDMA(ADI_DACC_HANDLE hDevice)
{
    ADI_DACC_DEVICE      *pDevice = (ADI_DACC_DEVICE *) hDevice;
    ADI_DACC_BUFFER_INFO *pBufferInfo = &pDevice->pDacData->BufferInfo;

#if ADI_DEBUG
    // validate DAC Control register
#endif
    pDevice->pDacDmaRegs->BPTR = (uint32_t) pBufferInfo->pBuffer;
    pDevice->pDacDmaRegs->MOD  = pBufferInfo->nIncrement;
    pDevice->pDacDmaRegs->CNT  = pBufferInfo->nBufferSize;

    /* Enable DMA interrupt */
    pDevice->pDacRegs->IMSK_CLR = (pDevice->eInterfaceNum == ADI_DACC_DACIF0) ? 
                                BITM_DACC_IMSK_DINT0 : BITM_DACC_IMSK_DINT1;

    /* Disable programmed i/o interrupts */
    pDevice->pDacRegs->IMSK_SET = (pDevice->eInterfaceNum == ADI_DACC_DACIF0) ? 
                                   BITM_DACC_IMSK_CINT0 : BITM_DACC_IMSK_CINT1;
}




/* initialize Dac */
static ADI_DACC_RESULT InitializeDAC (
       ADI_DACC_HANDLE             hDevice,
       const ADI_DACC_DACIF        eInterfaceNum 
)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

    uint32_t  regValue =0;

    /* reset control register */
    pDevice->pDacCtrlReg->CTL  = 0u; 
    pDevice->pDacDmaRegs->TC   = 0u; 
    pDevice->pDacDmaRegs->BPTR = 0u; 
    pDevice->pDacDmaRegs->MOD  = 0u; 
    pDevice->pDacDmaRegs->CNT  = 0u; 
    pDevice->pDacRegs->ERRMSK_CLR  = 0x0000001C; 

    /* reset any pending interrupts */
    regValue = pDevice->pDacRegs->ISTAT;

    if(eInterfaceNum == ADI_DACC_DACIF0){
            regValue &= ~(BITM_DACC_ISTAT_CINT1 | BITM_DACC_ISTAT_DINT1); /* clear off IF1 settings and keep IF0 */
    }
    else {
            regValue &= ~(BITM_DACC_ISTAT_CINT0 | BITM_DACC_ISTAT_DINT0);
    }

    /* w1c register */
    pDevice->pDacRegs->ISTAT = regValue;

    /* acknowledge the pending interrupt at the NVIC level */
    ADI_ACK_PENDINT((IRQn_Type)(eInterfaceNum == ADI_DACC_DACIF0 ? ADI_INT_DACC0_DAC0 : ADI_INT_DACC0_DAC1));

    /* reset the buffer information to 0 */   
    memset(&pDevice->pDacData->BufferInfo,0, sizeof(ADI_DACC_BUFFER_INFO));

    /* set the buffer state to free */
    pDevice->pDacData->BufferInfo.eBufferState = ADI_DACC_BUF_STATE_FREE;

    /* install dac interrupt handler */
    if(adi_int_InstallHandler( (eInterfaceNum == ADI_DACC_DACIF0) ? ADI_INT_DACC0_DAC0 : ADI_INT_DACC0_DAC1,
                              DacInterruptHandler ,
                              hDevice,
                              true) != ADI_INT_SUCCESS) 
    {
            return (ADI_DACC_INT_FAILED);
    }
    
     return (ADI_DACC_SUCCESS);
}

/*
 * ProcessBuffer schedules buffer transfer depending on the mode of the transfer
 * it can be DMA driven or programmed i/o based 
 */
static ADI_DACC_RESULT ProcessBuffer(ADI_DACC_HANDLE const hDevice,
                                     void *const pBuffer,
                                     uint32_t    nBufSize, 
                                     int32_t     nIncrement,
                                     bool        bCircular)
{
    ADI_DACC_DEVICE    *pDevice = (ADI_DACC_DEVICE *) hDevice;

#ifdef ADI_DEBUG
    ADI_DACC_RESULT  eResult;
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_DACC_SUCCESS) {
        return eResult;
    }

    /* set dma mode before device is enabled */
    if ((pDevice->eDeviceState  !=   ADI_DACC_STATE_OPENED) && 
        (pDevice->eDeviceState  !=   ADI_DACC_STATE_ENABLED)){
        return ADI_DACC_INVALID_OPERATION;
    }

    if (pBuffer == NULL) {
        return ADI_DACC_NULL_POINTER;
    }
#endif /* ADI_DEBUG */

    ADI_DACC_BUFFER_INFO *pBufferInfo = &pDevice->pDacData->BufferInfo;

    /* if a buffer is already in process we will not accecpt another buffer */
    if (pDevice->pDacData->BufferInfo.eBufferState != ADI_DACC_BUF_STATE_FREE) {
        return( ADI_DACC_DEVICE_IN_USE);
    }

    adi_osal_EnterCriticalRegion();

    /* setup the buffer */
    pBufferInfo->pBuffer       = pBuffer;
    pBufferInfo->nBufferSize   = nBufSize;
    pBufferInfo->nIncrement    = nIncrement;
    pBufferInfo->eBufferState  = ADI_DACC_BUF_STATE_INUSE;
    pBufferInfo->bCircular     = bCircular;

    adi_osal_ExitCriticalRegion();

    /* check if dma mode is enabled */
    if (pDevice->pDacData->bDMAMode)
    {
#if (ADI_DACC_CFG_ALL || ADI_DACC_CFG_DMA)
        if(pBufferInfo->bCircular) 
        {
            /* Enable circular buffering */
            pDevice->pDacCtrlReg->CTL |= (BITM_DACC_CTL0_CBUFEN);
    
           /* Enable Circular Buffer interrupt */
           pDevice->pDacCtrlReg->CTL |= BITM_DACC_CTL0_CINTEN;    
        }

        /* Schedule DMA */
        ScheduleDMA(hDevice);

        /* if device is already enabled, re-enable dma as it would be disabled once previous
         * linear transaction is finished 
         */
        if (pDevice->eDeviceState == ADI_DACC_STATE_ENABLED) {
            pDevice->pDacCtrlReg->CTL |= BITM_DACC_CTL0_DMAEN;
        }
#endif  
    }
    else
    {
#if (ADI_DACC_CFG_ALL || ADI_DACC_CFG_PIO)
        /* Driver operates in programmed i/o mode */
        pDevice->pDacData->BufferInfo.nCurBufferIdx = 0;

        /* Write data to the fifo */
        WriteFifo(pDevice);

        /* Enable FIFO Interrupt for the associated DAC,subsequent writes to the fifo
         * happen from the interrupt.
         */
        pDevice->pDacRegs->IMSK_CLR = (pDevice->eInterfaceNum == ADI_DACC_DACIF0) ? 
                                BITM_DACC_IMSK_CINT0 : BITM_DACC_IMSK_CINT1;
#endif
    }
   
    return (ADI_DACC_SUCCESS);
}

#ifndef ADI_DACC_NBIO
/* Converts the timeout given in milliseconds to OS ticks */
ADI_OSAL_TICKS ConvertMsToTicks(const uint32_t msTimeout)
{
   uint32_t OSTickPeriod =0;

   /* get number of ticks for given timeout in milli second */
    if (msTimeout && (adi_osal_TickPeriodInMicroSec(&OSTickPeriod) == ADI_OSAL_SUCCESS)) {
        return ((1000 * msTimeout)  / OSTickPeriod);
    }
    return (0);
}       
#endif

#ifdef ADI_DEBUG
/* Validate the device */
static ADI_DACC_RESULT ValidateDevice(ADI_DACC_DEVICE    *pDevice)
{
    uint32_t i=0;
 
    /* Check if the given pointer matches with any of the devices */
    for(i = 0; i < (uint32_t)ADI_DACC_NUM_DEVICES; i++)
    {
        /* If found a match return success */
        if(pDevice && (pDevice == (ADI_DACC_DEVICE*)&gDACCDevices[i]))
        {
            return ADI_DACC_SUCCESS;
        }
    }

    /* Could not find the match return failure */
    return ADI_DACC_INVALID_HANDLE;
}
#endif /* ADI_DEBUG */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */



/*@}*/
