/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
#ifndef _ADI_DACC_DEF_H_
#define _ADI_DACC_DEF_H_

#include <drivers/dacc/adi_dacc.h>
#include <adi_osal.h>

/* DAC Controller registers */
typedef struct
{
     uint32_t CTL0;                          /* DAC0 control register */
     uint32_t CTL1;                          /* DAC1 control register */
     uint32_t ERRSTAT;                       /* Error status register */
     uint32_t ERRMSK;                        /* Error Mask  Resgiter */
     uint32_t ERRMSK_SET;                    /* Error Mask Set Register */
     uint32_t ERRMSK_CLR;                    /* Error Mask Clear Register */
     uint32_t ISTAT;                         /* Interrupt status Register */
     uint32_t IMSK;                          /* Interrupt Mask Register */
     uint32_t IMSK_SET;                      /* Interrupt Mask Set Register */
     uint32_t IMSK_CLR;                      /* Interrupt Mask Clear Register */
     uint32_t TC0;                           /* DAC0 Timing Control register */
     uint32_t BPTR0;                         /* DAC0 DMA Base pointer */
     uint32_t MOD0;                          /* DAC0 DMA Modify */
     uint32_t CNT0;                          /* DAC0 DMA Count */
     uint32_t DAT0;                          /* DAC0 data register for core mode */
     uint32_t TC1;                           /* DAC1 Timing Control register */
     uint32_t BPTR1;                         /* DAC1 DMA Base pointer */
     uint32_t MOD1;                          /* DAC1 DMA Modify */
     uint32_t CNT1;                          /* DAC1 DMA Count */
     uint32_t DAT1;                          /* DAC1 data register for core mode */
     uint32_t BCST_CTL;                      /* Broadcast write register for CTL0,CTL1 */
     uint8_t  RESERVED0[172];
     uint32_t CNTCUR0;                       /* DAC0 DMA current count */
     uint32_t CNTCUR1;                       /* DAC1 DMA current count */
     uint32_t STAT;                          /* DAC status register */
     uint32_t REVID;                         /* REVID register for DACC */
     uint32_t CNTR0;                         /* DAC0 counter reads in testmode */
     uint32_t CNTR1;                         /* DAC1 counter reads in testmode */
     uint32_t ERRSTAT_SET;                   /* Set register for ERRSTAT */
     uint32_t ISTAT_SET;                     /* Set register for ISTAT */

} ADI_DACC_REGS;

typedef enum
{
  ADI_DACC_STATE_NOT_OPENED,                 /* Device is not opened */
  ADI_DACC_STATE_OPENED,                     /* Device is opened */
  ADI_DACC_STATE_ENABLED                     /* Device is opened and enabled */
} ADI_DACC_STATE;



/* enumeration of buffer states */
typedef enum
{
    ADI_DACC_BUF_STATE_FREE,                /* No buffer submittted (Buffer taken back by application) */
    ADI_DACC_BUF_STATE_INUSE,               /* Buffer is being processed */
    ADI_DACC_BUF_STATE_PROCESSED            /* Buffer processed and ready to be taken by application */

} ADI_DACC_BUF_STATE;

/* buffer structure */
typedef struct
{
  void                        *pBuffer;                /* Buffer pointer       */
  uint32_t                    nBufferSize;             /* Buffer size in bytes */
  ADI_DACC_BUF_STATE          eBufferState;            /* Buffer state         */
  uint32_t                    nIncrement;              /* Buffer increment     */
  bool                        bCircular;               /* Circular Buffer      */
  uint32_t                    nCurBufferIdx;           /* Current buffer index */

} ADI_DACC_BUFFER_INFO;

/* DACC device data */
typedef struct
{
    /* below callback parameters are only used when callbacks are used */
    ADI_CALLBACK                    pfCallback;                              /* Pointer to the device callback */
    void *                          pCBParam;                                /* Callback parameter */

    bool                            bDMAMode;                                /* Flag which indicates whether DMA mode is enabled */
    ADI_DACC_BUFFER_INFO            BufferInfo;                              /* Buffer information */   
    uint32_t                        eErrorStatus;                            /* Error status         */
    ADI_CALLBACK                    pfErrCallback;                           /* Pointer to the error callback */
    void *                          pErrCBParam;                             /* Error callback parameter */

    /* used only case of Blocking I/O */
    uint8_t                         SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR];   /* Semaphore memory */
    ADI_OSAL_SEM_HANDLE             hDacSem;                                 /* Handle to the semaphore */

} ADI_DACC_DEV_DATA;

/* Statistical collection */
#if ADI_DACC_CFG_STATS
typedef struct
{
    uint32_t nDac0MemoryErr;                 /* DAC 0 memory errors */
    uint32_t nDac1MemoryErr;                 /* DAC 1 memory errors */
    uint32_t nDac0UnderFlowErr;              /* DAC 0 underflow error */
    uint32_t nDac1UnderFlowErr;              /* DAC 1 underflow error */
    uint32_t nDac0AlignmentErr;              /* DAC 1 alignment error */
    uint32_t nDac1AlignmentErr;              /* DAC 1 alignment error */
    uint32_t nDacInterrupts;                 /* Number of DAC interrupts */
    uint32_t nDacDmaInterrupts;              /* Number of DAC DMA interrupts */
    uint32_t nDacBuffersProcessed;           /* Number of DAC buffers processed */
    uint32_t nDacUnexpectedInts;             /* Unexpected DAC interrupts  */

} ADI_DACC_STATS;

#define ADI_DACC_STAT_INC(x) (x++)

#else
#define ADI_DACC_STAT_INC(x)
#endif

/* DAC DMA registers */
typedef struct 
{
     uint32_t TC;                           /* DAC0 Timing Control register */
     uint32_t BPTR;                         /* DAC0 DMA Base pointer */
     uint32_t MOD;                          /* DAC0 DMA Modify */
     uint32_t CNT;                          /* DAC0 DMA Count */
     uint32_t DAT;                          /* DAC0 data register for core mode */
} ADI_DACC_DMA_REGS;

typedef struct
{
  uint32_t CTL;
} ADI_DACC_CTRL_REG;

/* DAC device structure */
typedef struct 
{
     ADI_DACC_STATE                          eDeviceState;     /* DACC device state */
     ADI_DACC_DACIF                          eInterfaceNum;    /* DACC interface number */
     volatile ADI_DACC_CTRL_REG   *const     pDacCtrlReg;      /* DACC control register  */
     volatile ADI_DACC_REGS       *const     pDacRegs;         /* DACC registers   */
     volatile ADI_DACC_DMA_REGS   *const     pDacDmaRegs;      /* DACC registers   */
     ADI_DACC_DEV_DATA *                     pDacData;         /* DACC device data */

} ADI_DACC_DEVICE;

extern ADI_DACC_DEVICE     gDACCDevices[];

/* Number of ADCC Devices in the processor */
#define ADI_DACC_NUM_DEVICES         2 //((sizeof(gDACCDevices)/sizeof(ADI_DACC_DEVICE)))

static void DacInterruptHandler(uint32_t iid, void *pCBParam);
static void ErrorInterruptHandler(uint32_t iid, void *pCBParam);

static ADI_DACC_RESULT InitializeDAC(ADI_DACC_HANDLE hDevice,const ADI_DACC_DACIF eInterfaceNum);
static void WriteFifo(ADI_DACC_HANDLE hDevice);
static void ScheduleDMA(ADI_DACC_HANDLE hDevice);
static ADI_DACC_RESULT ProcessBuffer(ADI_DACC_HANDLE const hDevice,void *const pBuffer,
		                     uint32_t nBufSize, int32_t nIncrement,bool bCircular);

#ifdef ADI_DEBUG
/* Validate the device */
static ADI_DACC_RESULT ValidateDevice(ADI_DACC_DEVICE    *pDevice);
#endif /* ADI_DEBUG */

#endif /* _ADI_DACC_DEF_H_ */
