/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*******************************************************************************
 *
 * @file:    adi_flash.c
 * @brief:   FLASH device driver implementation
 * @version: $Revision: 22701 $
 * @date:    $Date: 2013-08-21 12:13:11 -0400 (Wed, 21 Aug 2013) $
 *
 ******************************************************************************/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

/** @addtogroup FLASH_Driver_CM40z FLASH Device Driver
 *  @{
 *
 * The driver controls the in-package QSPI memory device, which attaches to
 * SPI2 interface and is normally configured for memory-mapped operation.
 *
 * The driver API functions take care of the details of switching the
 * flash memory to "peripheral" mode, sending commands and data to/from the
 * memory, and restoring the previous mode.
 *
 * See the ADSP-CM40z data sheet for details of total flash memory size and
 * base address, and flash block (subsector) size.
 */


/*********************************************************************

Include files

*********************************************************************/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>
#include <adi_osal.h>
#include <stddef.h>
#include <sys/platform.h>
#include <string.h>  // for memcmp()

#include <drivers/flash/adi_flash.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#define XIP_CONFIRMATION_BIT (0u << BITP_SPI_MMRDH_MODE)
#define ADI_SPI2_SSEL_ID 0u
#define SUBSECTOR_SIZE 4096u
#define SUBSECTORS_PER_SECTOR 16u
#define SECTOR_SIZE (SUBSECTORS_PER_SECTOR * SUBSECTOR_SIZE)
#define PAGE_SIZE 256u
#define SLOW_SPI_CLOCK 3u

#define READ_DATA				0x03 /* READ opcode */
#define FAST_READ				0x0b /* FAST_READ opcode */
#define SECTOR_ERASE				0xd8 /* SE opcode */
#define SUBSECTOR_ERASE				0x20 /* SSE opcode */
#define BULK_ERASE				0xc7 /* BE opcode */
#define PAGE_PROGRAM				0x02 /* PP opcode */
#define READ_STATUS_REGISTER		        0x05 /* RDSR opcode */
#define WRITE_STATUS_REGISTER		        0x01 /* WRSR opcode */
#define WRITE_ENABLE				0x06 /* WREN opcode */
#define WRITE_DISABLE				0x04 /* WRDI opcode */
#define JEDEC_ID				0x9f /* RDID opcode */
#define READ_SIGNATURE				0xab /* ? opcode */
#define QUAD_IO_FAST_READ			0xeb /* QIOFR opcode */
#define READ_CONFIG_REGISTER			0x85 /* RDVCR opcode */
#define WRITE_CONFIG_REGISTER			0x81 /* WRVCR opcode */

/* STATUS bits */
#define STATUS_BUSY				0x01
#define STATUS_WEL				0x02

enum State
{
    STATE_IDLE,
    STATE_ERASE_ALL,
    STATE_ERASE_BLOCKS,
    STATE_WRITE
};

struct FlashDevice
{
    /* SPI/flash hardware info */
    ADI_SPI_TypeDef *pSPI;
    uint32_t flashSlaveID;
    
    /* Driver state variables */
    enum State state;
    bool_t devIsOpen;
    bool_t memHasChanged;
    
    /* Flash memory info */
    uint32_t baseAddress;
    uint32_t memSizeBytes;
    
    /* SPI register save data */
    uint32_t CTL;
    uint32_t MMRDH;
    uint32_t CLK;
    uint32_t RXCTL;
    uint32_t TXCTL;
    
    /* State data for non-blocking eraseBlocks */
    uint32_t endBlock;
    uint32_t block;
    
    /* State data for non-blocking write */
    uint32_t bytesRemaining;
    uint8_t const *pSrc;
    uint32_t addr;
};

static struct FlashDevice vFlashDevices[1] =
{
  {
    pADI_SPI2,
    ADI_SPI2_SSEL_ID,
    STATE_IDLE,
    false,
    false,
    0x18000000u,
    2097152
  }
};

typedef void (*ROM_N25Q_RESET_FUNC)(ADI_SPI_TypeDef *, uint32_t);
 

/* Module static functions. These are all declared __ramfunc as they may be called while the
 * flash memory is not in memory-mapped mode, and hence they cannet reside in flash.
 */
__ramfunc static void doErase(struct FlashDevice *pDev, uint32_t * const pBlock, const uint32_t nEndBlock);
__ramfunc static void doWrite(struct FlashDevice *pDev, uint32_t addr, uint32_t bytesToWrite, uint8_t const *pSrc);
__ramfunc static inline void sync_cache(struct FlashDevice *pDev);
__ramfunc static inline bool_t compare_mem(uint8_t const vBuf1[], uint8_t const vBuf2[], uint32_t count);
__ramfunc static inline void assign_address(struct FlashDevice *pDev, uint8_t *buf, uint32_t addr, int *count);
__ramfunc static inline void assign_instruction(struct FlashDevice *pDev, uint8_t *buf, uint8_t insn, int *count);
__ramfunc static void send_bytes(struct FlashDevice *pDev, uint8_t * pData, uint32_t nNumBytes);
__ramfunc static void recv_bytes(struct FlashDevice *pDev, uint8_t *pData, uint32_t count);
__ramfunc static void exit_mm_xip_mode(struct FlashDevice *pDev, uint32_t clk);
__ramfunc static void enter_mm_xip_mode(struct FlashDevice *pDev);
__ramfunc static uint8_t read_reg(struct FlashDevice *pDev, uint32_t opcode);
__ramfunc static void write_reg(struct FlashDevice *pDev, uint32_t opcode, uint8_t cfgRegVal);
__ramfunc static void enable_writes(struct FlashDevice *pDev);
__ramfunc static void wait_while_busy(struct FlashDevice *pDev);
__ramfunc static inline bool_t test_if_busy(struct FlashDevice *pDev);
__ramfunc static inline void selectSlave(struct FlashDevice *pDev);
__ramfunc static inline void deselectSlave(struct FlashDevice *pDev);


/**
 * @brief       Open the given FLASH device instance
 *
 * @param [in]  nDeviceNumber       FLASH device instance number to be opened.
 * @param [in]  pDeviceMemory       Memory required for driver operation, this
 *                                  should be an array of at least
 *                                  ADI_FLASH_DEV_MEMORY_SIZE bytes.
 * @param [in]  nMemorySize         Size of the provided memory in bytes.
 * @param [out] phDevice            Pointer to a location where the handle to the
 *                                  opened device is written.
 *
 * @return      Status
 *
 *  - #ADI_FLASH_SUCCESS                  if successfully opened the given instance.
 *  - #ADI_FLASH_INVALID_DEV_NUM [D]      if device number out of range.
 *  - #ADI_FLASH_ALREADY_OPEN [D]         if device already open.
 *  - #ADI_FLASH_INSUFFICIENT_MEMORY [D]  if the supplied memory size is too small.
 *
 * No other FLASH APIs may be called until the device open function is called.
 *
 * @sa          adi_flash_Close()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_Open(
    uint32_t          nDeviceNumber,
    void             *pDeviceMemory,
    uint32_t	      nMemorySize,
    ADI_FLASH_HANDLE *phDevice
)
{
#if defined(ADI_DEBUG)
    if (nDeviceNumber != 0u)
        return ADI_FLASH_INVALID_DEV_NUM;
    
    if (vFlashDevices[nDeviceNumber].devIsOpen)
        return ADI_FLASH_ALREADY_OPEN;
    
    if (nMemorySize < ADI_FLASH_DEV_MEMORY_SIZE)
        return ADI_FLASH_INSUFFICIENT_MEMORY;
#endif
    
    *phDevice = (ADI_FLASH_HANDLE)&vFlashDevices[nDeviceNumber];
    vFlashDevices[nDeviceNumber].devIsOpen = true; /* mark the device as open */
      
    return ADI_FLASH_SUCCESS;
}


/**
 * @brief       Closes a FLASH device
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be closed.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully closed the device.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *
 * No FLASH APIs (other than adi_flash_Open) may be called after the device
 * close function is called.
 *
 * If the flash memory has been erased and/or written to since the last
 * adi_flash_Open() then the close operation will clear the CPU's memory cache,
 * so that the CPU receives an up-to-date view of memory.
 *
 * @sa          adi_flash_Open()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_Close(
    ADI_FLASH_HANDLE       const hDevice
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;

#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;
    
    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
#endif
   
    sync_cache(pDev);
    
    pDev->devIsOpen = false; /* mark the device as closed */

    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Bulk erase a FLASH device
 *
 * @param [in]  hDevice             Handle to the FLASH device to be bulk-erased.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully erased the device.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *
 * The bulk erase operation sets all of the bytes in the flash memory to 0xFF.
 *
 * Note that a bulk erase may take up to 60 seconds to complete.
 *
 * @sa          adi_flash_EraseBlocks()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_EraseAll(
    ADI_FLASH_HANDLE       const hDevice
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    uint8_t tbuf[4];
    int count;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;
    
    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
#endif
    
    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    count = 0;
    assign_instruction(pDev, tbuf, BULK_ERASE, &count);
    enable_writes(pDev);
    selectSlave(pDev);
    send_bytes(pDev, tbuf, count);
    deselectSlave(pDev);
    
    // Wait for erase completion
    wait_while_busy(pDev);

    enter_mm_xip_mode(pDev);
    
    pDev->memHasChanged = true;
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Erase a FLASH device, starting and ending with the
 *              specified blocks (inclusive).
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be block-erased.
 * @param [in]  nStartBlock         The number of the first block to erase.
 * @param [in]  nEndBlock           The number of the final block to erase.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully erased the specified blocks.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *  - #ADI_FLASH_INVALID_BLOCK_NUM [D]       if nStartBlock or nEndBlock is out of range.
 *
 * An erase operation sets all of the bytes, in each block in the specified range, to 0xFF.
 * The adi_flash_EraseBlocks() uses a combination of "subsector erase" and
 * "sector erase" commands to erase the specified range in the most efficient
 * way available. Note that erase operations may take many seconds to complete.
 *
 * @sa          adi_flash_EraseAll(), adi_flash_GetBlockNumber()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_EraseBlocks(
    ADI_FLASH_HANDLE       const hDevice,
    uint32_t               const nStartBlock,
    uint32_t               const nEndBlock
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    uint32_t block;
    const uint32_t nMaxBlock = (pDev->memSizeBytes / SUBSECTOR_SIZE) - 1u;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
    
    if ((nStartBlock > nMaxBlock) || (nEndBlock > nMaxBlock) || (nEndBlock < nStartBlock))
        return ADI_FLASH_INVALID_BLOCK_NUM;
#endif
    
    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    /* Perform one erase command (sector or subsector) per loop iteration
     * until the whole of the specified region has been erased.
     */
    for (block = nStartBlock;
         block <= nEndBlock;
         )
    {
        doErase(pDev, &block, nEndBlock);
        
        // Wait for erase completion
        wait_while_busy(pDev);
    }
    
    enter_mm_xip_mode(pDev);
    
    pDev->memHasChanged = true;
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Begin a non-blocking bulk erase of the FLASH device
 *
 * @param [in]  hDevice             Handle to the FLASH device to be bulk-erased.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if the erase was successfully started.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *  - #ADI_FLASH_PERIPHERAL_MODE_ONLY [D]    if the device is in MMAP or MMAP_XIP mode.
 *
 * The bulk erase operation sets all of the bytes in the flash memory to 0xFF.
 * A bulk erase may take up to 60 seconds to complete, and this non-blocking
 * erase-all API allows other work to be performed while the erase occurs
 * in the background. However, for this to happen the device must first be in mode
 * ADI_FLASH_MODE_PERIPHERAL, as the memory-mapped view of flash memory cannot
 * be available during an erase operation. Note that all code executed while the
 * flash device is on ADI_FLASH_MODE_PERIPHERAL must be located in RAM.
 *
 * If ADI_FLASH_SUCCESS is returned then the bulk erase has been successfully started,
 * but in order to complete the operation it is necessary to call
 * adi_flash_IsOperationComplete() at intervals until a true indication is returned.
 * adi_flash_IsOperationComplete() never blocks.
 *
 * @sa adi_flash_SubmitEraseBlocks(), adi_flash_SetMode(), adi_flash_IsOperationComplete()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_SubmitEraseAll(
    ADI_FLASH_HANDLE       const hDevice
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    uint8_t tbuf[4];
    int count;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
    
    if (pDev->pSPI->CTL & BITM_SPI_CTL_MMSE)
        return ADI_FLASH_PERIPHERAL_MODE_ONLY;
#endif
    
    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    count = 0;
    assign_instruction(pDev, tbuf, BULK_ERASE, &count);
    enable_writes(pDev);
    selectSlave(pDev);
    send_bytes(pDev, tbuf, count);
    deselectSlave(pDev);
    pDev->state = STATE_ERASE_ALL;
    
    enter_mm_xip_mode(pDev);
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Begin a non-blocking erase of the FLASH device, starting and
 *              ending with the specified blocks (inclusive).
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be block-erased.
 * @param [in]  nStartBlock         The number of the first block to erase.
 * @param [in]  nEndBlock           The number of the final block to erase.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if the erase was successfully started.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *  - #ADI_FLASH_INVALID_BLOCK_NUM [D]       if nStartBlock or nEndBlock is out of range.
 *  - #ADI_FLASH_PERIPHERAL_MODE_ONLY [D]    if the device is in MMAP or MMAP_XIP mode.
 *
 * An erase operation sets all of the bytes, in each block in the specified range, to 0xFF.
 * The adi_flash_EraseBlocks() uses a combination of "subsector erase" and
 * "sector erase" commands to erase the specified range in the most efficient
 * way available.
 *
 * A sector or subsector erase may take many seconds to complete, and a number
 * of these operation may be required to cover the specified range of blocks.
 * This non-blocking erase API allows other work to be performed while
 * the erase occurs in the background. However, for this to happen the device must
 * first be in mode ADI_FLASH_MODE_PERIPHERAL, as the memory-mapped view of flash
 * memory cannot be available during an erase operation. Note that all code executed
 * while the flash device is on ADI_FLASH_MODE_PERIPHERAL must be located in RAM.
 *
 * If ADI_FLASH_SUCCESS is returned then the erase has been successfully started,
 * but in order to complete the operation it is necessary to call
 * adi_flash_IsOperationComplete() at intervals until a true indication is returned.
 * adi_flash_IsOperationComplete() never blocks.
 *
 * @sa adi_flash_SubmitEraseAll(), adi_flash_SetMode(), adi_flash_IsOperationComplete()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_SubmitEraseBlocks(
    ADI_FLASH_HANDLE       const hDevice,
    uint32_t               const nStartBlock,
    uint32_t               const nEndBlock
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    const uint32_t nMaxBlock = (pDev->memSizeBytes / SUBSECTOR_SIZE) - 1u;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
    
    if (pDev->pSPI->CTL & BITM_SPI_CTL_MMSE)
        return ADI_FLASH_PERIPHERAL_MODE_ONLY;

    if ((nStartBlock > nMaxBlock) || (nEndBlock > nMaxBlock) || (nEndBlock < nStartBlock))
        return ADI_FLASH_INVALID_BLOCK_NUM;
#endif
    
    pDev->block      = nStartBlock;
    pDev->endBlock   = nEndBlock;
    pDev->state      = STATE_ERASE_BLOCKS;
    
    /* Start one erase operation (sector or subsector) */
    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    doErase(pDev, &pDev->block, pDev->endBlock);      
    enter_mm_xip_mode(pDev);
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Poll for completion of a non-blocking operation.
 *              
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be polled.
 * @param [out] pbIsComplete        Returns true if operation complete, false otherwise.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if the operation has completed.
 *  - #ADI_FLASH_NO_PENDING                  if there is no non-blocking operation in progress.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *
 * 
 * If there is a non-blocking operation in progress and the operation has not yet
 * completed then this function will return *pbIsComplete == false. If the operation
 * has completed then this function will return pbIsComplete == true. If the non-
 * -blocking operation has failed then an error result may be returned, but this
 * will be specific to the operation, and no such errors are currently defined.
 *
 * Once a non-blocking operation has been initiated, this function must be
 * called at intervals. Until this function has returned a true result
 * the operation is still considered to be in progress and will
 * prevent the execution of any other API functions.
 *
 * @sa adi_flash_SubmitEraseAll(), adi_flash_SubmitEraseBlocks(), adi_flash_SubmitWrite()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_IsOperationComplete(
    ADI_FLASH_HANDLE       const hDevice,
    bool_t *               const pbIsComplete
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    ADI_FLASH_RESULT result = ADI_FLASH_SUCCESS;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;
#endif
    
    switch (pDev->state)
    {
    default:
    case STATE_IDLE:
        result = ADI_FLASH_NO_PENDING;
        break;
    case STATE_ERASE_ALL:
        exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);

        /* Test for erase completion */
        if (test_if_busy(pDev))
        {
            /* The previous erase command is still executing */
            *pbIsComplete = false;
        }
        else
        {
            /* The mass erase command has completed */
            pDev->memHasChanged = true;
            pDev->state = STATE_IDLE;
            *pbIsComplete = true;
        }
        
        enter_mm_xip_mode(pDev);
        break;
    case STATE_ERASE_BLOCKS:
        exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
        
        if (test_if_busy(pDev))
        {
            /* The previous erase command is still executing */
            *pbIsComplete = false;
       }
        else if (pDev->block <= pDev->endBlock)
        {
            /* Start the next erase command (sector or subsector) */
            doErase(pDev, &pDev->block, pDev->endBlock);
            *pbIsComplete = false;
        }
        else
        {
            /* No more blocks to erase */
            pDev->memHasChanged = true;
            pDev->state = STATE_IDLE;
            *pbIsComplete = true;
        }
        
        enter_mm_xip_mode(pDev);
        break;
    case STATE_WRITE:
        exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
        
        if (test_if_busy(pDev))
        {
            /* The previous write command is still executing */
            *pbIsComplete = false;
        }
        else if (pDev->bytesRemaining > 0)
        {
            /* There is still data to be written, so start the next write command.
             * All writes after the first begin on a page boundary.
             */
            uint32_t bytesToWrite = PAGE_SIZE;
            
            if (pDev->bytesRemaining < bytesToWrite)
                bytesToWrite = pDev->bytesRemaining;
            
            pDev->bytesRemaining -= bytesToWrite;
            
            doWrite(pDev, pDev->addr, bytesToWrite, pDev->pSrc);
            pDev->addr += bytesToWrite;
            pDev->pSrc += bytesToWrite;
            doErase(pDev, &pDev->block, pDev->endBlock);
            *pbIsComplete = false;
        }
        else
        {
            /* No more data to be written */
            pDev->memHasChanged = true;
            pDev->state = STATE_IDLE;
            *pbIsComplete = true;
        }
        
        enter_mm_xip_mode(pDev);
        break;
    }
    
    return result;
}

/**
 * @brief       Gets the flash block number containing the specified address.
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 * @param [in]  nAddress            Absolute address contained by block.
 * @param [out] pnBlockNumber       Returns the number of the block containg the address.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully returned the block number.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_INVALID_ADDRESS [D]         if the address is out of range.
 *
 * This function translates an absolute memory address within the flash memory
 * space to a relative block number within the flash memory itself. A block
 * corresponds to a flash memory subsector, which is the smallest unit of
 * memory that can be erased.
 *
 * @sa          adi_flash_EraseBlocks()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_GetBlockNumber(
    ADI_FLASH_HANDLE       const hDevice,
    uint32_t               const nAddress,
    uint32_t *             const pnBlockNumber
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
   
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if ((nAddress < pDev->baseAddress) ||
        ((nAddress - pDev->baseAddress) >= pDev->memSizeBytes))
        return ADI_FLASH_INVALID_ADDRESS;
#endif
    
    *pnBlockNumber = (nAddress - pDev->baseAddress) / SUBSECTOR_SIZE  ;
        
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Write to flash memory at the specified address.
 *
 *
 * @note        Since flash programming can only change bits from 1 to 0,
 *              correct programming requires the destination bytes to be 0xFF
 *              prior to the write. Blocks of bytes can be restored to the
 *              0xFF state using the adi_flash_EraseBlocks() API, and the entire
 *              memory can be restored to 0xFF using adi_flash_EraseAll(). The
 *              contents of the destination bytes is *not* checked by
 *              adi_flash_Write() prior to the write operation.
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 * @param [in]  nAddress            Absolute address of destination in flash.
 * @param [in]  pData               Pointer to the start of the source data,
 *                                  which must not be in flash memory.
 * @param [in]  nBytes              Number of bytes to write.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully wrote to the device.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *  - #ADI_FLASH_INVALID_ADDRESS [D]         if the address is out of range.
 *
 * This function programs data into the flash memory. An arbitrary number of
 * bytes may be programmed with a single call, starting at any byte address,
 * subject to the limits of the flash address space.
 *
 * Because the source data is read while the flash is in peripheral mode,
 * the source data itself must not be located in flash memory, and should be in
 * SRAM instead.
 *
 * @sa          adi_flash_SubmitWrite()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_Write(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t const *  const pData,
    uint32_t         const nBytes
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    uint32_t bytesRemaining = nBytes;
    uint8_t const *pSrc = pData;
    uint32_t addr = nAddress;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
    
    if ((nAddress < pDev->baseAddress) ||
        ((nAddress + nBytes - pDev->baseAddress) > pDev->memSizeBytes))
        return ADI_FLASH_INVALID_ADDRESS;
#endif

    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    while (bytesRemaining > 0u)
    {
        /* The write might not begin on a page boundary */
        uint32_t bytesToWrite = PAGE_SIZE - (addr % PAGE_SIZE);
        
        if (bytesRemaining < bytesToWrite)
            bytesToWrite = bytesRemaining;
        
        // Write a page and wait for completion
        doWrite(pDev, addr, bytesToWrite, pSrc);
        wait_while_busy(pDev);
        
        addr += bytesToWrite;
        pSrc += bytesToWrite;
        bytesRemaining -= bytesToWrite;
    }
    
    enter_mm_xip_mode(pDev);
    
    pDev->memHasChanged = true;
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Begins a non-blocking write to flash memory at the specified address.
 *
 *
 * @note        Since flash programming can only change bits from 1 to 0,
 *              correct programming requires the destination bytes to be 0xFF
 *              prior to the write. Blocks of bytes can be restored to the
 *              0xFF state using the adi_flash_EraseBlocks() API, and the entire
 *              memory can be restored to 0xFF using adi_flash_EraseAll(). The
 *              contents of the destination bytes is *not* checked by
 *              adi_flash_Write() prior to the write operation.
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 * @param [in]  nAddress            Absolute address of destination in flash.
 * @param [in]  pData               Pointer to the start of the source data,
 *                                  which must not be in flash memory.
 * @param [in]  nBytes              Number of bytes to write.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully wrote to the device.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *  - #ADI_FLASH_INVALID_ADDRESS [D]         if the address is out of range.
 *  - #ADI_FLASH_PERIPHERAL_MODE_ONLY [D]    if the device is in MMAP or MMAP_XIP mode.
 *
 * This function programs data into the flash memory. An arbitrary number of
 * bytes may be programmed with a single call, starting at any byte address,
 * subject to the limits of the flash address space.
 *
 * Because the source data is read while the flash is in peripheral mode,
 * the source data itself must not be located in flash memory, and should be in
 * SRAM instead.
 *
 * A page-program may take one or more milliseconds to complete, and a number
 * of these operation may be required to cover the specified range of addresses.
 * This non-blocking write API allows other work to be performed while
 * the writes occur in the background. However, for this to happen the device must
 * first be in mode ADI_FLASH_MODE_PERIPHERAL, as the memory-mapped view of flash
 * memory cannot be available during an erase operation. Note that all code executed
 * while the flash device is on ADI_FLASH_MODE_PERIPHERAL must be located in RAM.
 *
 * If ADI_FLASH_SUCCESS is returned then the write has been successfully started,
 * but in order to complete the operation it is necessary to call
 * adi_flash_IsOperationComplete() at intervals until a true indication is returned.
 * adi_flash_IsOperationComplete() never blocks.
 *
 * @sa          adi_flash_Write(), adi_flash_SetMode(), adi_flash_IsOperationComplete()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_SubmitWrite(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t const *  const pData,
    uint32_t         const nBytes
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
    
    if (pDev->pSPI->CTL & BITM_SPI_CTL_MMSE)
        return ADI_FLASH_PERIPHERAL_MODE_ONLY;

    if ((nAddress < pDev->baseAddress) ||
        ((nAddress + nBytes - pDev->baseAddress) > pDev->memSizeBytes))
        return ADI_FLASH_INVALID_ADDRESS;
#endif

    pDev->bytesRemaining = nBytes;
    pDev->pSrc = pData;
    pDev->addr = nAddress;
    pDev->state = STATE_WRITE;
    
    /* Start one page write operation */
    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    /* The write might not begin on a page boundary */
    uint32_t bytesToWrite = PAGE_SIZE - (pDev->addr % PAGE_SIZE);
    
    if (pDev->bytesRemaining < bytesToWrite)
        bytesToWrite = pDev->bytesRemaining;
    
    pDev->bytesRemaining -= bytesToWrite;
    
    doWrite(pDev, pDev->addr, bytesToWrite, pDev->pSrc);
    pDev->addr += bytesToWrite;
    pDev->pSrc += bytesToWrite;
    
    enter_mm_xip_mode(pDev);
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Read data from the specified address in flash.
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 * @param [in]  nAddress            Absolute address of source in flash.
 * @param [out] pData               Pointer to the start of the destination buffer.
 * @param [in]  nBytes              Number of bytes to read.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully read from the device.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *  - #ADI_FLASH_INVALID_ADDRESS [D]         if the address is out of range.
 *
 * This function reads data from the flash memory device using peripheral mode,
 * rather than memory-mapped mode. An arbitrary number of bytes may be read with
 * a single call, starting at any byte address, subject to the limits of the
 * flash address space.
 *
 * After the operation completes the flash memory is restored to its previous
 * mode of operation, which will typically be memory-mapped mode.
 *
 * @sa          adi_flash_Write(), adi_flash_Verify()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_Read(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t *        const pData,
    uint32_t         const nBytes
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    uint32_t bytesRemaining = nBytes;
    uint8_t  *pDst = pData;
    uint32_t addr = nAddress;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
    
    if ((nAddress < pDev->baseAddress) ||
        ((nAddress + nBytes - pDev->baseAddress) >= pDev->memSizeBytes))
        return ADI_FLASH_INVALID_ADDRESS;
#endif

    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    while (bytesRemaining > 0u)
    {
        uint8_t tbuf[4];
        int count = 0;

        uint32_t bytesToRead = 0xFFFFu;
        
        if (bytesRemaining < bytesToRead)
            bytesToRead = bytesRemaining;
              
        assign_instruction(pDev, tbuf, READ_DATA, &count);
        assign_address(pDev, tbuf, addr, &count);

        selectSlave(pDev);
        send_bytes(pDev, tbuf, count);
        recv_bytes(pDev, pDst, bytesToRead);
        deselectSlave(pDev);
        
        bytesRemaining -= bytesToRead;
        addr += bytesToRead;
        pDst += bytesToRead;
    }

    enter_mm_xip_mode(pDev);

    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Verify data at the specified address in flash.
 *
 *
 * @note        If this function is used to verify data which has just been
 *              written to flash, and returns ADI_FLASH_VERIFY_FAILED, this may
 *              be because one or more of the destination bytes in flash memory
 *              were not 0xFF prior to the write.
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 * @param [in]  nAddress            Absolute address of location in flash.
 * @param [in]  pData               Pointer to the start of the comparison data.
 * @param [in]  nBytes              Number of bytes to compare.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully verified the device.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *  - #ADI_FLASH_VERIFY_FAILED               if flash contents don't match data.
 *  - #ADI_FLASH_INVALID_ADDRESS [D]         if the address is out of range.
 *
 * This function reads data from the flash memory device using peripheral mode,
 * and compares it for equality with the supplied user data. An arbitrary number
 * of bytes may be verified with a single call, starting at any byte address,
 * subject to the limits of the flash address space.
 *
 * Because the comparison data is read while the flash is in peripheral mode,
 * the comparison data itself must not be located in flash memory, and should be in
 * SRAM instead.
 *
 * After the operation completes the flash memory is restored to its previous
 * mode of operation, which will typically be memory-mapped mode.
 *
 * @sa          adi_flash_Write(), adi_flash_Read()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_Verify(
    ADI_FLASH_HANDLE const hDevice,
    uint32_t         const nAddress,
    uint8_t const *  const pData,
    uint32_t         const nBytes
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    uint32_t bytesRemaining = nBytes;
    uint8_t const *pSrc = pData;
    uint32_t addr = nAddress;
    ADI_FLASH_RESULT result = ADI_FLASH_SUCCESS;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
   
    if ((nAddress < pDev->baseAddress) ||
        ((nAddress + nBytes - pDev->baseAddress) >= pDev->memSizeBytes))
        return ADI_FLASH_INVALID_ADDRESS;
#endif

    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    while(bytesRemaining > 0u)
    {
        uint8_t tbuf[4];
        int count = 0;
        uint8_t vCmpBuf[256];
        uint32_t bytesToRead = sizeof(vCmpBuf);
        uint32_t i;
        
        for (i = 0u; i < sizeof(vCmpBuf); ++i)
            vCmpBuf[i] = !pSrc[i];
         
        if (bytesToRead > bytesRemaining)
            bytesToRead = bytesRemaining;
        
        assign_instruction(pDev, tbuf, READ_DATA, &count);
        assign_address(pDev, tbuf, addr, &count);

        selectSlave(pDev);
        send_bytes(pDev, tbuf, count);
        recv_bytes(pDev, vCmpBuf, bytesToRead);
        deselectSlave(pDev);
        
        /* Compare the supplied data to the buffer. We don't use memcmp()
         * because that function may not be resident in RAM.
         */
        if (!compare_mem(vCmpBuf, pSrc, bytesToRead))
        {
            result = ADI_FLASH_VERIFY_FAILED;
            break;
        }
        addr += bytesToRead;
        pSrc += bytesToRead;
        bytesRemaining -= bytesToRead;
    }
    
    enter_mm_xip_mode(pDev);

    return result;
}

/**
 * @brief       Instruct the CPU cache to discard stored flash data.
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully synced the cache.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *
 * The ADSP-CM40z includes a memory cache which stores recently-accessed parts of the
 * flash memory, accelerating subsequent accesses. The cache is read-only and
 * does not detect write or erase operations on the flash memory. If the flash
 * contents are changed it is therefore necesssary to clear (or invalidate)
 * the cache so as to prevent it from returning out-of-date data.
 *
 * Note that adi_flash_Close() will automatically clear the cache if the
 * contents of the flash have been changed since the most recent adi_flash_Open().
 * It is only necessary to call adi_flash_Sync() if it is desired to clear the
 * cache while the flash device is still open. 
 *
 * The cache only affects memory-mapped reads from the flash address space.
 * Data reads using adi_flash_Read() bypass the cache  and always read from the
 * underlying flash memory.
 *
 * @sa          adi_flash_Write(), adi_flash_Erase(), adi_flash_Close()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_Sync(
    ADI_FLASH_HANDLE const hDevice
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;
#endif   

    sync_cache(pDev);
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Force the flash memory into a known state.
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully synced the cache.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *
 * The in-package flash memory has no RESET pin for doing a hardware reset operation.
 * It is therefore possible for a situation where the SPI flash memory is in a
 * different state from that of the SPI controller. This situation will normally
 * be resolved by the boot ROM, but this public API provides a user-callable
 * entry to the ROM's flash reset routine.
 *
 * It should not be necessary to use this API in typical applications.
 *
 * @sa          
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_Reset(
    ADI_FLASH_HANDLE const hDevice
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    ROM_N25Q_RESET_FUNC flashReset = (ROM_N25Q_RESET_FUNC)0x191;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
#endif   

    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    flashReset(pDev->pSPI, 0);
    
    enter_mm_xip_mode(pDev);
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Change the current mode of the flash device.
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 * @param [in]  mode                The device mode to set.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully synced the cache.
 *  - #ADI_FLASH_INVALID_MODE                if an invalid mode enumerator was supplied.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *  - #ADI_FLASH_PENDING [D]                 if a non-blocking operation is in progress.
 *
 * The flash device has three major operating modes:.
 *  - ADI_FLASH_MODE_MMAP, where the flash is mapped into the CPU's address space,
 *    and full SPI flash commands are sent to the memory for each read operation.
 *    This is normally the startup mode after boot.
 *  - ADI_FLASH_MODE_MMAP_XIP, where the flash is mapped into the CPU's address
 *    space, but only addresses are sent to the memory for each read operation.
 *    This mode makes memory-mapped flash reads more efficient, but also makes
 *    adi_flash API functions execute more slowly.
 *  - ADI_FLASH_MODE_PERIPHERAL, where the flash is not mapped into the CPU's
 *    address space, and the flash memory is only accessed using the adi_flash
 *    or adi_spi API functions. Note that selecting this mode will lead to
 *    subsequent execution errors if any attempt is made to access flash memory
 *    via the memory-mapped address space, e.g. for code execution or for
 *    reading of constant data. Therefore all code executed while the
 *    flash device is on ADI_FLASH_MODE_PERIPHERAL must be located in RAM.
 *
 * This function allows the current mode to be changed.
 *
 * @sa          adi_flash_GetMode()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_SetMode(
    ADI_FLASH_HANDLE const hDevice,
    ADI_FLASH_MODE mode
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    ADI_FLASH_RESULT result = ADI_FLASH_SUCCESS;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;

    if (STATE_IDLE != pDev->state)
        return ADI_FLASH_PENDING;
#endif   

    exit_mm_xip_mode(pDev, SLOW_SPI_CLOCK);
    
    switch (mode)
    {
    case ADI_FLASH_MODE_MMAP_XIP:
        pDev->CTL   |= BITM_SPI_CTL_MMSE;
        pDev->MMRDH |= BITM_SPI_MMRDH_CMDSKIP;
        break;
    case ADI_FLASH_MODE_MMAP:
        pDev->CTL   |= BITM_SPI_CTL_MMSE;
        pDev->MMRDH &= ~BITM_SPI_MMRDH_CMDSKIP;
        break;
    case ADI_FLASH_MODE_PERIPHERAL:
        pDev->CTL   &= ~BITM_SPI_CTL_MMSE;
        pDev->MMRDH &= ~BITM_SPI_MMRDH_CMDSKIP;
        break;
    default:
        result = ADI_FLASH_INVALID_MODE;
        break;
    }
    
    enter_mm_xip_mode(pDev);
    
    return result;
}

/**
 * @brief       Query the current mode of the flash device.
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be used.
 * @param [out] pMode               Returns the current mode.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if successfully synced the cache.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *
 * The flash device has three major operating modes:.
 *  - ADI_FLASH_MODE_MMAP, where the flash is mapped into the CPU's address space,
 *    and full SPI flash commands are sent to the memory for each read operation.
 *  - ADI_FLASH_MODE_MMAP_XIP, where the flash is mapped into the CPU's address
 *    space, but only addresses are sent to the memory for each read operation.
 *  - ADI_FLASH_MODE_PERIPHERAL, where the flash is not mapped into the CPU's
 *    address space, and the flash memory is only accessed using the adi_flash
 *    or adi_spi API functions.
 *
 * This function allows the current mode to be queried.
 *
 * @sa          adi_flash_SetMode()
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_GetMode(
    ADI_FLASH_HANDLE const hDevice,
    ADI_FLASH_MODE * const pMode
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;
#endif   

    if (pDev->pSPI->CTL & BITM_SPI_CTL_MMSE)
    {
        if (pDev->pSPI->MMRDH & BITM_SPI_MMRDH_CMDSKIP)
        {
            *pMode = ADI_FLASH_MODE_MMAP_XIP;
        }
        else
        {
            *pMode = ADI_FLASH_MODE_MMAP;
        }
    }
    else
    {
        *pMode = ADI_FLASH_MODE_PERIPHERAL;
    }
    
    return ADI_FLASH_SUCCESS;
}

/**
 * @brief       Query the base memory address of the flash region.
 *
 *
 * @param [in]  hDevice             Handle to the FLASH device to be queried.
 * @param [out] pInfo               Returns the information.
 *
 * @return      Status
 *  - #ADI_FLASH_SUCCESS                     if the value was returned.
 *  - #ADI_FLASH_NOT_OPEN [D]                if the device is not open.
 *
 * This function allows the base address of the flash memory to be queried.
 * Note that the contents of the flash memory will only be accessible at
 * this adddress when the device is in modes ADI_FLASH_MODE_MMAP or
 * ADI_FLASH_MODE_MMAP_XIP. When in mode ADI_FLASH_MODE_PERIPHERAL any access
 * to the flash memory region will fail.
 *
 * @sa          
 */
__ramfunc
ADI_FLASH_RESULT adi_flash_GetInfo(
    ADI_FLASH_HANDLE const hDevice,
    ADI_FLASH_INFO * const pInfo
)
{
    struct FlashDevice *pDev = (struct FlashDevice *)hDevice;
    
#if defined(ADI_DEBUG)
    if (!pDev->devIsOpen)
        return ADI_FLASH_NOT_OPEN;
#endif   

    pInfo->baseAddress = pDev->baseAddress;;
    pInfo->totalSize   = pDev->memSizeBytes;
    pInfo->blockSize   = SUBSECTOR_SIZE;

    return ADI_FLASH_SUCCESS;
}

/* Send a single page-program command to the flash memory */
__ramfunc
static void doWrite(struct FlashDevice *pDev, uint32_t addr, uint32_t bytesToWrite, uint8_t const *pSrc)
{
    uint8_t tbuf[4 + PAGE_SIZE];
    int count = 0;
    
    assign_instruction(pDev, tbuf, PAGE_PROGRAM, &count);
    assign_address(pDev, tbuf, addr, &count);
    addr += bytesToWrite;

    /* Copy the data to the buffer. We don't use memcpy() because
     * that function may not be resident in RAM.
     */
    while (bytesToWrite)
    {
        tbuf[count++] = *pSrc++;
        --bytesToWrite;
    }
    
    /* Send the PAGE_PROGRAM operation to the flash memory.
     */
    enable_writes(pDev);
    selectSlave(pDev);
    send_bytes(pDev, tbuf, count);
    deselectSlave(pDev);
}

/* Send a single erase command (sector or subsector) to the flash memory */
__ramfunc
static void doErase(struct FlashDevice *pDev, uint32_t * const pBlock, const uint32_t nEndBlock)
{
    uint8_t tbuf[4];
    int count = 0;
    uint32_t addr = *pBlock * SUBSECTOR_SIZE;
    
    /* Use full-sector erase where applicable, subsector erase otherwise.
     */
    if ((0 == (*pBlock % SUBSECTORS_PER_SECTOR)) &&    /* if on a sector boundary, and     */
        ((nEndBlock - *pBlock) >= SUBSECTORS_PER_SECTOR)) /* entire sector is within the erase */
    {
        /* We can use full-sector erase, and advance the block count by 16 */
        *pBlock += SUBSECTORS_PER_SECTOR;
        assign_instruction(pDev, tbuf, SECTOR_ERASE, &count);
    }
    else
    {
        /* We have to use subsector erase, and advance the block count by 1 */
        *pBlock += 1;
        assign_instruction(pDev, tbuf, SUBSECTOR_ERASE, &count);
    }

    assign_address(pDev, tbuf, addr, &count);
    enable_writes(pDev);
    selectSlave(pDev);
    send_bytes(pDev, tbuf, count);
    deselectSlave(pDev);
}

/* Clear the CPU cache */
__ramfunc
static inline
void sync_cache(struct FlashDevice *pDev)
{
    /* If the flash memory has been erased and/or written to since the last adi_flash_Open()
     * then the close operation will invalidate the CPU's memory cache, so that the
     * CPU receives an up-to-date view of memory.
     */
    if (pDev->memHasChanged)
    {
        pDev->memHasChanged = false; /* reset the "changed" flag */
        
        /* Clear the CPU's cache, so that it sees the new
         * contents of the flash memory.
         */
        pADI_M4P0->CACHE_CFG |= 0x10; /* set the cache clear bit */
     
        /* Wait for the cache hardware to clear the bit. */
        while (pADI_M4P0->CACHE_CFG & 0x10);
    }
}

/* Compare two regions of memory for equality (partial memcmp() replacement) */
__ramfunc
static inline
bool_t compare_mem(uint8_t const vBuf1[], uint8_t const vBuf2[], uint32_t count)
{
    uint32_t i;
    
    for (i = 0u; i < count; ++i)
    {
        if (vBuf1[i] != vBuf2[i])
            return false;
    }
    
    return true;
}

/* Activate the chip select for the specified SPI slave */
__ramfunc
static inline void selectSlave(struct FlashDevice *pDev)
{
    const uint32_t slaveNum = pDev->flashSlaveID;
    const uint32_t nSlvSelLow = ((BITM_SPI_SLVSEL_SSE1 << slaveNum) | (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << slaveNum)));

    pDev->pSPI->SLVSEL = nSlvSelLow; /* set slave select low */
}

/* Deactivate the chip select for the specified SPI slave */
__ramfunc
static inline void deselectSlave(struct FlashDevice *pDev)
{
    const uint32_t slaveNum = pDev->flashSlaveID;
    const uint32_t nSlvSelHigh = ((BITM_SPI_SLVSEL_SSE1 << slaveNum) | (0xFF00));

    pDev->pSPI->SLVSEL = nSlvSelHigh; /* set slave select high */
    pDev->pSPI->SLVSEL = nSlvSelHigh; /* set slave select high */
}

/* Insert an instruction into the specified command buffer */
__ramfunc
static inline void assign_instruction(struct FlashDevice *pDev, uint8_t *buf, uint8_t insn, int *count)
{
    buf[0] = insn;
    *count = 1;
}

/* Insert an address into the specified command buffer */
__ramfunc
static inline void assign_address(struct FlashDevice *pDev, uint8_t *buf, uint32_t addr, int *count)
{
    buf[*count + 0] = (uint8_t)(addr >> 16);
    buf[*count + 1] = (uint8_t)(addr >> 8);
    buf[*count + 2] = (uint8_t)(addr >> 0);
    *count += 3;
}

/* Send the specified bytes to the SPI device */
__ramfunc
static void send_bytes(struct FlashDevice *pDev, uint8_t * pData, uint32_t nNumBytes)
{
  uint32_t i;

  if(nNumBytes != 0)
  {
    uint32_t txctl = pDev->pSPI->TXCTL;
    pDev->pSPI->RXCTL = 0;

    pDev->pSPI->TWC = nNumBytes;
    pDev->pSPI->TXCTL = BITM_SPI_TXCTL_TEN | BITM_SPI_TXCTL_TTI | BITM_SPI_TXCTL_TWCEN;

    for(i = 0; i < nNumBytes; i++)
    {
      pDev->pSPI->TFIFO = pData[i];
      while((pDev->pSPI->STAT & BITM_SPI_STAT_TFS) == ENUM_SPI_STAT_TFIFO_FULL);  /* Wait until there is space in the FIFO */
    }

    while(0 == (pDev->pSPI->STAT & BITM_SPI_STAT_TF));  /* Wait until finished */
    
    pDev->pSPI->STAT = BITM_SPI_STAT_TF;
    pDev->pSPI->TXCTL = txctl & ~BITM_SPI_TXCTL_TEN;
  }
}

/* Receive the specified number of bytes from the SPI device */
__ramfunc
static void recv_bytes(struct FlashDevice *pDev, uint8_t *pData, uint32_t count)
{
  pDev->pSPI->TXCTL = 0;
  pDev->pSPI->STAT = BITM_SPI_STAT_RF;
  pDev->pSPI->RWC = count;
  pDev->pSPI->RXCTL = BITM_SPI_RXCTL_REN | BITM_SPI_RXCTL_RWCEN | BITM_SPI_RXCTL_RTI;

  for(uint32_t i = 0; i<count; i++)
  {
    while(pDev->pSPI->STAT & BITM_SPI_STAT_RFE);  /* Wait until data is received in the receive FIFO */
    pData[i] = pDev->pSPI->RFIFO & 0xFF;
  }
  while((pDev->pSPI->RWC) != 0);  /* Also make sure the read word counter is now zero */

  pDev->pSPI->STAT = BITM_SPI_STAT_RF;
  pDev->pSPI->RXCTL = 0;

  pDev->pSPI->CTL &= ~0x00000001;
  pDev->pSPI->CTL |= 0x00000001;
}

/* Switch the SPI device to peripheral mode, saving its previous state first */
__ramfunc
static void exit_mm_xip_mode(struct FlashDevice *pDev, uint32_t clk)
{
    // Save copies of the SPI registers that we're going to change
    pDev->CTL   = pDev->pSPI->CTL;
    pDev->CLK   = pDev->pSPI->CLK;
    pDev->RXCTL = pDev->pSPI->RXCTL;
    pDev->TXCTL = pDev->pSPI->TXCTL;
    pDev->MMRDH = pDev->pSPI->MMRDH;

    /* need to ensure SPI is configured for 1 bit mode of operation */
    pDev->pSPI->CTL   = 0u;  /* Disable the SPI */
    pDev->pSPI->CLK   = clk; /* Slow the clock down a bit for the program operations */
    pDev->pSPI->RXCTL = 0u;
    pDev->pSPI->TXCTL = 0u;
    pDev->pSPI->MMRDH = 0u;
    pDev->pSPI->CTL   = BITM_SPI_CTL_MSTR | BITM_SPI_CTL_EN;
 
    /* If the CTL register value of the saved SPI state has the MMSE bit set
     * *and* the MMRDH register value of the saved SPI state has the CMDSKIP bit
     * set then we are currently in XIP mode, so we need to return the flash
     * to normal mode.
     */
    if ((pDev->CTL   & BITM_SPI_CTL_MMSE) &&
        (pDev->MMRDH & BITM_SPI_MMRDH_CMDSKIP))
    {
        uint8_t spiData[5];

        spiData[0] = 0xEB;
        spiData[1] = 0x00;
        spiData[2] = 0x00;
        spiData[3] = 0xFF;
        spiData[4] = 0xFF;

        selectSlave(pDev);
        send_bytes(pDev, spiData, sizeof(spiData));
        deselectSlave(pDev);
    }
}

/* Restore the previous mode of the SPI device, which is assumed to be a
 * memory-mapped mode, either normal or execute-in-place (XIP).
 */
__ramfunc
static void enter_mm_xip_mode(struct FlashDevice *pDev)
{
    /* If the CTL register value of the saved SPI state has the MMSE bit set
     * *and* the MMRDH register value of the saved SPI state has the CMDSKIP bit
     * set then we were previously in XIP mode, so we need to return the flash
     * to XIP mode prior to restoring the SPI state.
     */
    if ((pDev->CTL   & BITM_SPI_CTL_MMSE) &&
        (pDev->MMRDH & BITM_SPI_MMRDH_CMDSKIP))
    {
        /* Clear the XIP disable bit to signal "Ready to enter XIP mode".
         * The function of this bit, when zero, is "To make the data on DQ0 during
         * the first dummy clock NOT Dont Care. I.e. if the XIP bit is zero then
         * the flash memory will test the value of DQ0 during the first dummy clock
         * to determine if it is an XIP (no opcode) operation.
         */
        uint8_t flashVCR = read_reg(pDev, READ_CONFIG_REGISTER); /* Read flash config register */
        flashVCR &= ~0x08;                       /* Enable XIP functionality */
        write_reg(pDev, WRITE_CONFIG_REGISTER, flashVCR);          /* Write config reg back */

        /* Send a read operation to the flash. This is a dummy operation which
         * just provides the opcode that will be used for subsequent XIP reads.
         * The address and the returned data are don't-cares.
         *
         * "The SPI Flash device must be primed with an initial read command,
         * before the CMDSKIP bit is set." - HRM 0.1 p22-23
         */
        uint8_t spiData[5];

        spiData[0] = QUAD_IO_FAST_READ;
        spiData[1] = 0x00;
        spiData[2] = 0x00;
        spiData[3] = 0x00;
        spiData[4] = 0x00;

        selectSlave(pDev);
        send_bytes(pDev, spiData, sizeof(spiData));
        recv_bytes(pDev, spiData, 4);    // may not be required
        deselectSlave(pDev);

        /* In order to maintain XIP mode we must set the mode bits to the required
         * value(LSB == 1), so that a 1 value is driven onto DQ0 during the first
         * dummy clock. Clearing this will cause XIP mode to be exited.
         * We know that the CMDSKIP bit is already set in the saved MMRDH value
         * because we checked it above.
         */
        pDev->pSPI->CTL = 0u;       /* Disable the SPI before reconfiguring */
        pDev->pSPI->MMRDH = (pDev->MMRDH & (~BITM_SPI_MMRDH_MODE)) | XIP_CONFIRMATION_BIT | BITM_SPI_MMRDH_CMDSKIP;
    }
    else
    {
        pDev->pSPI->CTL = 0u;       /* Disable the SPI before reconfiguring */
        pDev->pSPI->MMRDH = pDev->MMRDH;
    }
    
    pDev->pSPI->CLK   = pDev->CLK;
    pDev->pSPI->RXCTL = pDev->RXCTL;
    pDev->pSPI->TXCTL = pDev->TXCTL;
    pDev->pSPI->CTL   = pDev->CTL;    // restore CTL last, enabling SPI
}

/* Read the contents of a specified flash device register, via the SPI bus */
__ramfunc
static uint8_t read_reg(struct FlashDevice *pDev, uint32_t opcode)
{
    uint8_t spiData[1];
    uint8_t cfgRegVal = 0;
    int count = 0;
    
    assign_instruction(pDev, spiData, opcode, &count);
    selectSlave(pDev);
    send_bytes(pDev, spiData, count);
    recv_bytes(pDev, &cfgRegVal, sizeof(cfgRegVal));
    deselectSlave(pDev);

    return cfgRegVal;
}

/* Write a value to a specified flash device register, via the SPI bus */
__ramfunc
static void write_reg(struct FlashDevice *pDev, uint32_t opcode, uint8_t cfgRegVal)
{
    uint8_t spiData[2];
    int count = 0;

    enable_writes(pDev);
    
    assign_instruction(pDev, spiData, opcode, &count);
    assign_instruction(pDev, spiData, cfgRegVal, &count);
    selectSlave(pDev);
    send_bytes(pDev, spiData, count);
    deselectSlave(pDev);
}

/* Send a WRITE_ENABLE command to the flash memory to set the WE flag */
__ramfunc
static void enable_writes(struct FlashDevice *pDev)
{
    uint8_t flashStatus;
    
    do
    {
        uint8_t spiData[1];
        int count = 0;

        assign_instruction(pDev, spiData, WRITE_ENABLE, &count);
        selectSlave(pDev);
        send_bytes(pDev, spiData, count);
        deselectSlave(pDev);

        flashStatus = read_reg(pDev, READ_STATUS_REGISTER);
    } while (0u == (flashStatus & STATUS_WEL));
}

/* Loop reading the flash memory's status register until the BUSY flag is clear */
__ramfunc
static void wait_while_busy(struct FlashDevice *pDev)
{
    uint8_t flashStatus;
    
    do
    {
        flashStatus = read_reg(pDev, READ_STATUS_REGISTER);
    } while (flashStatus & STATUS_BUSY);
}

/* Loop reading the flash memory's status register until the BUSY flag is clear */
__ramfunc
static inline bool_t test_if_busy(struct FlashDevice *pDev)
{
    uint8_t flashStatus = read_reg(pDev, READ_STATUS_REGISTER);
    
    return (0u != (flashStatus & STATUS_BUSY));
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*@}*/
