/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*******************************************************************************
 *
 * @file:    adi_hae.c
 * @brief:   HAE device driver implementation
 * @version: $Revision: 22061 $
 * @date:    $Date: 2013-06-27 15:22:07 -0400 (Thu, 27 Jun 2013) $
 *
 ******************************************************************************/

#include <stdbool.h>
#include <stddef.h>
#include <string.h>
#include <assert.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>
#include <services/dma/adi_dma.h>
#include <adi_osal.h>

#include <drivers/hae/adi_hae.h>
#include "adi_hae_def.h"
#include "adi_hae_data.c"

#ifdef _MISRA_RULES
#pragma diag(push)
#endif /* _MISRA_RULES */
  
/** @addtogroup HAE_Driver Harmonic Analysis Engine (HAE) Driver
 *  @{
 *
 * The Harmonic Analysis Engine (HAE) block receives input samples from two 
 * source signals whose frequencies is between 45 Hz and 65 Hz. The HAE will 
 * then process the input samples and produce output results. The output results 
 * consist of power quality measuremets of the fundamental and up to 12 additional 
 * harmonics.
 */


/*
 * API's
 */

/**
 * @brief       Open a HAE device instance.
 *
 *  If successful the returned ADI_HAE_HANDLE parameter will hold a device handle
 *  to be used as an input to other HAE API functions.  The HAE device must be
 *  opened before using any other HAE API functions.
 *
 *  The amount of memory required by the driver is specified by the ADI_HAE_INT_MEMORY_SIZE
 *  or ADI_HAE_DMA_MEMORY_SIZE definition.
 *
 * @param [in]  nDeviceNum          The HAE device instance number to be opened.
 * @param [in]  pMemory             Application supplied memory space for use by the driver.
 * @param [in]  MemorySize          Application supplied memory size (bytes).
 * @param [out] phDevice            A pointer to a location where the handle to the
 *                                  opened device is written.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully opened the given instance.
 *              - #ADI_HAE_INVALID_DEVICE_NUM  [D] The device number is invalid.
 *              - #ADI_HAE_NULL_POINTER        [D] A NULL pointer parameter error. 
 *              - #ADI_HAE_INVALID_PARAMETER   [D] The application supplied memory size is insufficient.
 *              - #ADI_HAE_DEVICE_IN_USE           The HAE device is already open.
 *              - #ADI_HAE_INT_FAILED              The interrupt install failed.
 *              - #ADI_HAE_SEM_FAILED              The semaphore create failed.
 *
 *
 * @sa          adi_hae_Close()
 */
ADI_HAE_RESULT  adi_hae_Open (
    uint32_t             const  nDeviceNum,
    void                 const *pMemory,
    uint32_t             const  MemorySize,
    ADI_HAE_HANDLE            *phDevice)
{
    /* Pointer to the device data*/
    ADI_HAE_DEVICE *pDevice = NULL;

    /* Pointer to track the memory available for allocation */
    uint8_t *pAvailableMem = (uint8_t*)pMemory;

#ifdef ADI_DEBUG
    /* Check if the given device number is valid */
    if(nDeviceNum > ADI_HAE_NUM_DEVICES)
    {
        return ADI_HAE_INVALID_DEVICE_NUM;
    }

    /* Check if the given pointer are valid */
    if(pMemory == NULL || phDevice == NULL)
    {
        return ADI_HAE_NULL_POINTER;
    }

    /* Check if device is already opened */
    if(gHaeDevices[nDeviceNum].eHaeState != ADI_HAE_STATE_NOT_OPENED)
    {
        return ADI_HAE_DEVICE_IN_USE;
    }

    /* Asserts to validate the memory size macros */
    assert(ADI_HAE_INT_MEMORY_SIZE == sizeof(ADI_HAE_CHANNEL) * 3u);
    assert(ADI_HAE_DMA_MEMORY_SIZE == (ADI_HAE_INT_MEMORY_SIZE + (ADI_DMA_CHANNEL_REQ_MEMORY * 3u)));


  	if ((MemorySize < (uint32_t)ADI_HAE_INT_MEMORY_SIZE) ||
        (MemorySize > (uint32_t)ADI_HAE_INT_MEMORY_SIZE && (MemorySize < (uint32_t)ADI_HAE_DMA_MEMORY_SIZE)))
	{
		return ADI_HAE_INVALID_PARAMETER;
	}
         
#endif /* ADI_DEBUG */ 

    pDevice = &gHaeDevices[nDeviceNum];

    /* clear the given memory */
    memset((void *)pAvailableMem, 0, MemorySize);

    /* assign application memory to the device */
    pDevice->pTxChannel = (void *)pAvailableMem;
    pAvailableMem += sizeof(ADI_HAE_CHANNEL);

    pDevice->pRx0Channel = (void *)pAvailableMem;
    pAvailableMem += sizeof(ADI_HAE_CHANNEL);

    pDevice->pRx1Channel = (void *)pAvailableMem;
    pAvailableMem += sizeof(ADI_HAE_CHANNEL);

    /* If application intends to operate in DMA mode */
    if(MemorySize >= ADI_HAE_DMA_MEMORY_SIZE)
    {
        pDevice->pTxChannel->pDMAChnlMemory = pAvailableMem;
        pAvailableMem += ADI_DMA_CHANNEL_REQ_MEMORY;
        
        pDevice->pRx0Channel->pDMAChnlMemory = pAvailableMem;
        pAvailableMem += ADI_DMA_CHANNEL_REQ_MEMORY;

        pDevice->pRx1Channel->pDMAChnlMemory = pAvailableMem;
        pAvailableMem += ADI_DMA_CHANNEL_REQ_MEMORY;
        
       /* Initialize DMA related parameters */
        pDevice->pTxChannel->eDMAChnlID  = ADI_DMA_CHNL_HAE0_TX;
        pDevice->pRx0Channel->eDMAChnlID = ADI_DMA_CHNL_HAE0_RX_CH0;
        pDevice->pRx1Channel->eDMAChnlID = ADI_DMA_CHNL_HAE0_RX_CH1;
            
        pDevice->pTxChannel->nBytesPerTransfer = 4u;
        pDevice->pRx0Channel->nBytesPerTransfer = 4u;
        pDevice->pRx1Channel->nBytesPerTransfer = 4u;
    }

    /* Create the semaphore */
    if(adi_osal_SemCreateStatic(pDevice->pTxChannel->SemMemory,
                                ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                &pDevice->pTxChannel->hSemaphore, 0u) != ADI_OSAL_SUCCESS)
    {
        return ADI_HAE_SEM_FAILED;
    }

    /* Create the semaphore */
    if(adi_osal_SemCreateStatic(pDevice->pRx0Channel->SemMemory,
                                ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                &pDevice->pRx0Channel->hSemaphore, 0u) != ADI_OSAL_SUCCESS)
    {
        return ADI_HAE_SEM_FAILED;
    }

    /* Create the semaphore */
    if(adi_osal_SemCreateStatic(pDevice->pRx1Channel->SemMemory,
                                ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                &pDevice->pRx1Channel->hSemaphore, 0u) != ADI_OSAL_SUCCESS)
    {
        return ADI_HAE_SEM_FAILED;
    }

    /*
     * Clear config data and enable main clock
     */
    *pDevice->HaeRegs.pCFG0  = 0x0u;                    /* HAE0 Configuration Register 0 */
    *pDevice->HaeRegs.pCFG1  = 0x0u;                    /* HAE0 Configuration Register 1 */
    *pDevice->HaeRegs.pCFG2  = 1u << BITP_HAE_CFG2_EN;  /* HAE0 Configuration Register 2 */
    *pDevice->HaeRegs.pCFG3  = 0x0u;                    /* HAE0 Configuration Register 3 */
    *pDevice->HaeRegs.pRUN   = 0x0u;                    /* HAE0 Run Register */
    *pDevice->HaeRegs.pSTAT  = BITM_HAE_STAT_TXIRQ | BITM_HAE_STAT_RXIRQ;  /* HAE0 Status Register (W1C) */

    /* Install the status interrupt handler */
    if(adi_int_InstallHandler(pDevice->nStatusIntID,
                              StatusIntHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
        return ADI_HAE_INT_FAILED;
    }

    /* Load the interrupt mode (default) function table */
    pDevice->pFnTable = &gIntFnTable;

    /* Mark the device as in use */
    pDevice->eHaeState = ADI_HAE_STATE_OPENED;

    /* set the active buffer pointer */
    pDevice->pTxChannel->pActiveBuf  = &pDevice->pTxChannel->BufferInfo0;
    pDevice->pRx0Channel->pActiveBuf = &pDevice->pRx0Channel->BufferInfo0;
    pDevice->pRx1Channel->pActiveBuf = &pDevice->pRx1Channel->BufferInfo0;

    /* poll ready status */
    while(((*pDevice->HaeRegs.pSTAT & BITM_HAE_STAT_RDY) >> BITP_HAE_STAT_RDY) == 0u);
    
    /* Return the device handle */
    *phDevice = (ADI_HAE_HANDLE) pDevice;

    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Close the given HAE device.
 *
 * The status interrupt is uninstalled and any open DMA channels are closed.
 *
 * @param [in]  hDevice  The handle to the HAE device.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                Successfully closed the given instance.
 *              - #ADI_HAE_INVALID_HANDLE     [D] The device handle is invalid.
 *              - #ADI_HAE_INT_FAILED             The interrupt uninstall failed.
 *              - #ADI_HAE_SEM_FAILED             The semaphore destroy failed.
 *
 * @sa          adi_hae_Open()
 */
ADI_HAE_RESULT  adi_hae_Close (
   ADI_HAE_HANDLE const        hDevice)
{
      /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;

    /* Result code */
    ADI_HAE_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* disable device */
    *pDevice->HaeRegs.pCFG2 = 0x0u;

    /* Un-install the status interrupt handler */
    if(adi_int_UninstallHandler(pDevice->nStatusIntID) != ADI_INT_SUCCESS)
    {
        return ADI_HAE_INT_FAILED;
    }

    if(pDevice->pTxChannel->hSemaphore != NULL)
    {
        if(adi_osal_SemDestroyStatic(pDevice->pTxChannel->hSemaphore) != ADI_OSAL_SUCCESS)
        {
            return ADI_HAE_SEM_FAILED;
        }
    }

    /* Close the DMA Channel if opened */
    if((eResult = pDevice->pFnTable->pfClose(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }

    /* Mark the device as not in use */
    pDevice->eHaeState = ADI_HAE_STATE_NOT_OPENED;

    /* Clear the device data pointer */
    pDevice->pTxChannel = NULL;

    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Register/Unregister an application defined status callback function.
 *
 * An application defined callback function is used to notify the application of
 * HAE status events.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  pfCallback          Application supplied callback function which is
 *                                  called to notify HAE events.  A value of
 *                                  NULL disables application callbacks.
 * @param [in]  pCBParam            Application supplied callback parameter which
 *                                  will be passed back in the callback function.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully set the application defined callback function.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *              - #ADI_HAE_INT_FAILED              Failed to enable the status interrupt.
 *
 * @sa  #ADI_HAE_EVENT
 */
ADI_HAE_RESULT  adi_hae_RegisterCallback (
    ADI_HAE_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam)
{
    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;

    /* Result code */
    ADI_HAE_RESULT     eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }

    /* Check if the device is enabled */
    if(pDevice->eHaeState == ADI_HAE_STATE_ENABLED)
    {
        return ADI_HAE_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Save the given callback and callback parameter */
    adi_osal_EnterCriticalRegion();
    pDevice->pfCallback  =   pfCallback;
    pDevice->pCBParam    =   pCBParam;
    adi_osal_ExitCriticalRegion();

    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Start or idle the HAE block harmonic calculations.
 *
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  bEnable             True to start the harmonic calculations, false to idle.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully start or idle the harmonic calculations.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_Run (
    ADI_HAE_HANDLE        const   hDevice,
    bool                  const   bEnable)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;

    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    mask = BITM_HAE_RUN_RUN;
    
    adi_osal_EnterCriticalRegion();

    reg = *pDevice->HaeRegs.pRUN;

    reg &= (uint32_t)~mask;

    if (bEnable)
    {
        reg |= 1u << BITP_HAE_RUN_RUN;
    }
    
    *pDevice->HaeRegs.pRUN = reg;
    
    adi_osal_ExitCriticalRegion();
    
    eResult = EnableDataFlow(pDevice, pDevice->pTxChannel, bEnable);
    eResult = EnableDataFlow(pDevice, pDevice->pRx0Channel, bEnable);
    eResult = EnableDataFlow(pDevice, pDevice->pRx1Channel, bEnable);

    return ADI_HAE_SUCCESS;  
}

/*
 *  Config 0
 */

/**
 * @brief       Enable/Disable interrupt events.
 *
 * Interrupts are triggered when HAE requests a new input sample on both 
 * the I and V channels or HAE results are finished and ready to transmit.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  eDataInt            The interrupt type to enable/disable.
 * @param [in]  bEnable             True to enable the interrupt, false to disable.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully enabled/disabled the interrupt.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *              - #ADI_HAE_INT_FAILED              Failed to install/uninstall the data interrupt.
 *
 * @sa  #ADI_HAE_INT
 */
ADI_HAE_RESULT  adi_hae_EnableInterrupt (
    ADI_HAE_HANDLE      const   hDevice,
    ADI_HAE_INT         const   eDataInt,
    bool                const   bEnable)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    uint32_t pos  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    
    if (eDataInt > ADI_HAE_INT_TX)
    {
        return ADI_HAE_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();

    reg = *pDevice->HaeRegs.pCFG0;

    if (eDataInt == ADI_HAE_INT_RX)
    {
        mask = BITM_HAE_CFG0_RXIRQEN;
        pos = BITP_HAE_CFG0_RXIRQEN;
    }
    else
    {
        mask = BITM_HAE_CFG0_TXIRQEN;
        pos = BITP_HAE_CFG0_TXIRQEN;        
    }
    
    reg &= (uint32_t)~mask;
    if (bEnable)
    {
        reg |= 1u << pos;
    }
    *pDevice->HaeRegs.pCFG0 = reg;
    
    adi_osal_ExitCriticalRegion();
    
    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Enables or disables DMA mode.
 *
 * By default DMA mode is disabled. Use this API to operate the HAE driver in DMA mode.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  bEnable             true to enable DMA mode, false to disable DMA 
 *                                  and enable interrupt mode.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully enabled/disabled the interrupt.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *              - #ADI_HAE_INT_FAILED              Failed to install/uninstall the data interrupt.
 */
ADI_HAE_RESULT adi_hae_EnableDMAMode (
    ADI_HAE_HANDLE         const hDevice,
    bool                   const bEnable)
{
    /* Pointer to HAE device instance */
    ADI_HAE_DEVICE *pDevice = (ADI_HAE_DEVICE *)hDevice;

    /* Return code */
    ADI_HAE_RESULT eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* IF (we're already in DMA mode) */
    if (pDevice->bDMAMode == bEnable)
    {
    	return ADI_HAE_SUCCESS;
    }

    /* IF DMA is required */
    if(bEnable == true)
    {
        /* clear DMA stat */
        *pDevice->HaeRegs.pSTAT  = BITM_HAE_STAT_TXIRQ | BITM_HAE_STAT_RXIRQ;  /* HAE0 Status Register (W1C) */

        /*
         * Open the DMA channels based on the direction.
         */

        /* Open Transmit DMA channel */
        if(adi_pdma_Open(pDevice->pTxChannel->eDMAChnlID,
                         pDevice->pTxChannel->pDMAChnlMemory,
                         &pDevice->pTxChannel->hDMAChnl,
                         HaeTxDMAHandler,
                         pDevice
                         ) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }

        /* Enable DMA SYNC configuration */
        if (adi_pdma_SetSync (pDevice->pTxChannel->hDMAChnl, true) != ADI_DMA_SUCCESS)
        {
        	return ADI_HAE_DMA_FAILED;
        }

        /* Set the PSize */
        if(adi_pdma_SetPSize(pDevice->pTxChannel->hDMAChnl, ADI_DMA_PSIZE_4BYTES) != ADI_DMA_SUCCESS)
        {
           return ADI_HAE_DMA_FAILED;
        } 
        
        /* Set the MSize */
        if(adi_pdma_SetMSize(pDevice->pTxChannel->hDMAChnl, ADI_DMA_MSIZE_4BYTES) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }

        /* Open Receive 0 DMA channel */
        if(adi_pdma_Open(pDevice->pRx0Channel->eDMAChnlID,
                         pDevice->pRx0Channel->pDMAChnlMemory,
                         &pDevice->pRx0Channel->hDMAChnl,
                         HaeRx0DMAHandler,
                         pDevice
                         ) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }

        /* Enable DMA SYNC configuration */
        if (adi_pdma_SetSync (pDevice->pRx0Channel->hDMAChnl, true) != ADI_DMA_SUCCESS)
        {
        	return ADI_HAE_DMA_FAILED;
        }

        /* Set the PSize */
        if(adi_pdma_SetPSize(pDevice->pRx0Channel->hDMAChnl, ADI_DMA_PSIZE_4BYTES) != ADI_DMA_SUCCESS)
        {
           return ADI_HAE_DMA_FAILED;
        } 
        
        /* Set the MSize */
        if(adi_pdma_SetMSize(pDevice->pRx0Channel->hDMAChnl, ADI_DMA_MSIZE_4BYTES) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }

        /* Set the function pointer table to DMA mode functions */
        pDevice->pFnTable   =   &gDMAFnTable;

        /* Open Receive 1 DMA channel */
        if(adi_pdma_Open(pDevice->pRx1Channel->eDMAChnlID,
                         pDevice->pRx1Channel->pDMAChnlMemory,
                         &pDevice->pRx1Channel->hDMAChnl,
                         HaeRx1DMAHandler,
                         pDevice
                         ) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }

        /* Enable DMA SYNC configuration */
        if (adi_pdma_SetSync (pDevice->pRx1Channel->hDMAChnl, true) != ADI_DMA_SUCCESS)
        {
        	return ADI_HAE_DMA_FAILED;
        }

        /* Set the PSize */
        if(adi_pdma_SetPSize(pDevice->pRx1Channel->hDMAChnl, ADI_DMA_PSIZE_4BYTES) != ADI_DMA_SUCCESS)
        {
           return ADI_HAE_DMA_FAILED;
        } 
        
        /* Set the MSize */
        if(adi_pdma_SetMSize(pDevice->pRx1Channel->hDMAChnl, ADI_DMA_MSIZE_4BYTES) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }

        /* Set the function pointer table to DMA mode functions */
        pDevice->pFnTable   =   &gDMAFnTable;

    }
    /* Interrupt Mode required */
    else
    {
        /*
         * If DMA channels were opened before, close them
         */

        /* Close Tx channel */
        if(pDevice->pTxChannel->hDMAChnl != NULL)
        {
            if(adi_pdma_Close(pDevice->pTxChannel->hDMAChnl, false) != ADI_DMA_SUCCESS)
            {
                return ADI_HAE_DMA_FAILED;
            }
        }

        /* Close Rx0 channel */
        if(pDevice->pRx0Channel->hDMAChnl != NULL)
        {
            if(adi_pdma_Close(pDevice->pRx0Channel->hDMAChnl, false) != ADI_DMA_SUCCESS)
            {
                return ADI_HAE_DMA_FAILED;
            }
        }

        /* Close Rx1 channel */
        if(pDevice->pRx1Channel->hDMAChnl != NULL)
        {
            if(adi_pdma_Close(pDevice->pRx1Channel->hDMAChnl, false) != ADI_DMA_SUCCESS)
            {
                return ADI_HAE_DMA_FAILED;
            }
        }

        /* Set the function pointer table to Interrupt mode functions */
        pDevice->pFnTable   =   &gIntFnTable;
    }

    /* Save the chosen mode */
    pDevice->bDMAMode   =   bEnable;

    return ADI_HAE_SUCCESS;    
}

/**
 * @brief       Select the line frequency to analyze.
 *
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  eLineFreq           The line frequency selection.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully selected the line frequency.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_SelectLineFreq (
    ADI_HAE_HANDLE        const   hDevice,
    ADI_HAE_LINE_FREQ     const   eLineFreq)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    uint32_t nLineFreq  = (uint32_t)eLineFreq;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();

    reg = *pDevice->HaeRegs.pCFG0;

    mask = BITM_HAE_CFG0_LFREQ;
    reg &= (uint32_t)~mask;
    reg |= nLineFreq << BITP_HAE_CFG0_LFREQ;
    *pDevice->HaeRegs.pCFG0 = reg;
    
    adi_osal_ExitCriticalRegion();
    
    return ADI_HAE_SUCCESS;
}

/*
 * Config 1
 */

/**
 * @brief       Set the sample clock divider.
 *
 * This function calculates the sample clock divider to obtain a 8Khz sample rate.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  nSclk               The system clock frequency.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully set the sample clock divider.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_ConfigSampleDivider (
    ADI_HAE_HANDLE        const   hDevice,
    uint32_t              const   nSclk)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;

    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    uint32_t pos  = 0u;
    uint32_t div = 0u;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    
    if (nSclk == 0)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
#endif /* ADI_DEBUG */

    mask = BITM_HAE_CFG1_STARTDIV;
    pos = BITP_HAE_CFG1_STARTDIV;

    /* divide the main clock down to 8Khz */
    div = nSclk / 8000u;
    if (div > (mask << pos))
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
    
    adi_osal_EnterCriticalRegion();

    reg = *pDevice->HaeRegs.pCFG1;

    reg &= (uint32_t)~mask;
    reg |= div << pos;
    *pDevice->HaeRegs.pCFG1 = reg;
    
    adi_osal_ExitCriticalRegion();
    
    return ADI_HAE_SUCCESS;
}


/*
 * Config 2
 */

/**
 * @brief       Configure the HAE results.
 *
 * This function sets the result update rate, settle time and mode.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  eResultMode         The mode determines if the HAE waits to produce results 
 *                                  per the HAE settle field or produces them immediately.
 * @param [in]  eSettleTime         The time to wait before producing results. The time is 
 *                                  based on an 8K sample count. 
 * @param [in]  eUpdateRate         The rate that the HAE updates it's results.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully configured the HAE results.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_ConfigResults (
    ADI_HAE_HANDLE        const   hDevice,
    ADI_HAE_RESULT_MODE   const   eResultMode,
    ADI_HAE_SETTLE_TIME   const   eSettleTime,
    ADI_HAE_UPDATE_RATE   const   eUpdateRate)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;

    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    uint32_t mode  = (uint32_t)eResultMode;
    uint32_t time  = (uint32_t)eSettleTime;
    uint32_t rate  = (uint32_t)eUpdateRate;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    
    if (eResultMode > ADI_HAE_RESULT_MODE_IMMEDIATE)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
    
    if (eSettleTime > ADI_HAE_SETTLE_TIME_1280)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
    
    if (eUpdateRate > ADI_HAE_UPDATE_RATE_1024000)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
#endif /* ADI_DEBUG */

    mask = BITM_HAE_CFG2_MODE | BITM_HAE_CFG2_SETTLE | BITM_HAE_CFG2_UPDATE;
    
    adi_osal_EnterCriticalRegion();

    reg = *pDevice->HaeRegs.pCFG2;

    reg &= (uint32_t)~mask;

    reg |= mode << BITP_HAE_CFG2_MODE;
    reg |= time << BITP_HAE_CFG2_SETTLE;
    reg |= rate << BITP_HAE_CFG2_UPDATE;
    
    *pDevice->HaeRegs.pCFG2 = reg;
    
    adi_osal_ExitCriticalRegion();
    
    return ADI_HAE_SUCCESS;    
}

/**
 * @brief       Select the HAE result channels to transfer over DMA to memory.
 *
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  channels            The result channels to transfer.  Multiple
 *                                  channels are selected by using a logical OR
 *                                  (ADI_HAE_CHAN_TX_FUNDAMENTAL | ADI_HAE_CHAN_TX_HARMONIC_1)
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully selected the HAE result channels.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #ADI_HAE_CHAN_TX
 */
ADI_HAE_RESULT adi_hae_EnableChannels (
    ADI_HAE_HANDLE        const   hDevice,
    uint32_t              const   channels)  
                                  //bEnable
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;

    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    
    if (channels > BITM_HAE_CFG3_CHANEN)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
#endif /* ADI_DEBUG */
    
    /* clear DMA stat */
    *pDevice->HaeRegs.pSTAT  = BITM_HAE_STAT_TXIRQ | BITM_HAE_STAT_RXIRQ;  /* HAE0 Status Register (W1C) */

    mask = BITM_HAE_CFG3_CHANEN;
    
    adi_osal_EnterCriticalRegion();

    reg = *pDevice->HaeRegs.pCFG3;

    reg &= (uint32_t)~mask;

    reg |= channels << BITP_HAE_CFG3_CHANEN;
    
    *pDevice->HaeRegs.pCFG3 = reg;
    
    adi_osal_ExitCriticalRegion();
    
    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Provide the voltage and current input samples (interrupt mode).
 *
 *              When the HAE issues an Rx interrupt the application must provide 
 *              the input samples.
 *              
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  current             The current input (24-bit signed).
 * @param [in]  voltage             The voltage input (24-bit signed).
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully selected the HAE result channels.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_InputSample (
    ADI_HAE_HANDLE        const   hDevice,
    int32_t               const   current,
    int32_t               const   voltage)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    
    if (current > BITM_HAE_ISAMPLE_VALUE)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
    
    if (voltage > BITM_HAE_VSAMPLE_VALUE)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
#endif /* ADI_DEBUG */

    *pDevice->HaeRegs.pISAMPLE = (current & 0x00FFFFFFu) << BITP_HAE_ISAMPLE_VALUE;
    *pDevice->HaeRegs.pVSAMPLE = (voltage & 0x00FFFFFFu) << BITP_HAE_VSAMPLE_VALUE;
    
    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Get the processed input sample waveform.
 *
 *              The waveform represents the processed input sample. There will 
 *              be some amplitude and phase delay, but the waveform should 
 *              follow the sample input.
 *              
 *
 * @param [in]   hDevice             Handle to the HAE device.
 * @param [out]  pCurrent            The current waveform.
 * @param [out]  pVoltage            The voltage waveform.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully got the waveform.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_GetWaveform (
    ADI_HAE_HANDLE        const   hDevice,
    uint32_t                      *pCurrent,
    uint32_t                      *pVoltage)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    *pCurrent = (*pDevice->HaeRegs.pISAMPLE & BITM_HAE_IWAVEFORM_VALUE) >> BITP_HAE_IWAVEFORM_VALUE;
    *pVoltage = (*pDevice->HaeRegs.pVSAMPLE & BITM_HAE_VWAVEFORM_VALUE) >> BITP_HAE_VWAVEFORM_VALUE;
    
    return ADI_HAE_SUCCESS;    
}

/*
 * Config 4
 */

/**
 * @brief       Enable input signal processing.
 *
 *              The voltage and current have the option of being high-pass filtered 
 *              in order to remove DC offsets. For the case when the current is sensed 
 *              with a Rogowski coil, there is an internal digital integrator for compensation.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  bEnableIntegrator   True to enable the digital integrator, false to disable.
 * @param [in]  bEnableFilter       True to enable the high pass filter, false to disable.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully enabled/disabled input signal processing.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_EnableInputProcessing (
    ADI_HAE_HANDLE        const   hDevice,
    bool                  const   bEnableIntegrator,
    bool                  const   bEnableFilter)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;

    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    if (!IsEnabled(pDevice))
    {
        return ADI_HAE_DEVICE_NOT_ENABLED;
    }        
#endif /* ADI_DEBUG */

    mask = BITM_HAE_CFG4_HPFEN | BITM_HAE_CFG4_DIDTEN;
    
    adi_osal_EnterCriticalRegion();

    reg = *pDevice->HaeRegs.pCFG4;

    reg &= (uint32_t)~mask;

    reg |= (bEnableIntegrator << BITP_HAE_CFG4_DIDTEN);
    reg |= (bEnableFilter << BITP_HAE_CFG4_HPFEN);
    
    *pDevice->HaeRegs.pCFG4 = reg;
    
    adi_osal_ExitCriticalRegion();
    
    if (bEnableIntegrator)
    {
        *pDevice->HaeRegs.pDIDT_GAIN = 0x0u;
        *pDevice->HaeRegs.pDIDT_COEF = 0x0ffff000u;
    }
    
    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Set the fundamental voltage input level.
 *
 *              The voltage level is used to internally scale the fixed fundamental
 *              voltage level. This assists the HAE to lock onto the fundamental 
 *              voltage channel.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  voltage             The fundamental voltage input level.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully set the fundamental voltage input level.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_SetVoltageLevel (
    ADI_HAE_HANDLE        const   hDevice,
    float                 const   voltage)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    
    if (voltage == 0u)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
#endif /* ADI_DEBUG */

    /* calculate voltage input level */
    
    /* TODO - avoid floating point math? */
    *pDevice->HaeRegs.pVLEVEL = (uint32_t) (0.64 / voltage * 5033168);
    
    return ADI_HAE_SUCCESS;    
}

/**
 * @brief       Set the index of the harmonic to analyze.
 *
 *              The harmonic results will appear in the results RAM based on the index.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  eIndex              The harmonic index.
 * @param [in]  value               The index value.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully set the harmonic index value.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_HAE_RESULT adi_hae_HarmonicIndex (
    ADI_HAE_HANDLE             const   hDevice,
    ADI_HAE_HARMONIC_INDEX     const   eIndex,
    uint32_t                   const   value)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
    
    uint32_t index  = (uint32_t) eIndex;
    volatile uint32_t *pRegIndex  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
    
    if (eIndex > ADI_HAE_HARMONIC_INDEX_12)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }
    
    if (value > BITM_HAE_H_INDX_VALUE)
    {
        return ADI_HAE_INVALID_PARAMETER;        
    }

#endif /* ADI_DEBUG */
    
    pRegIndex = pDevice->HaeRegs.pH00_INDX;
    pRegIndex += index;
    
    *pRegIndex = value;
    
    return ADI_HAE_SUCCESS;
}

/*
 * Buffer releated API's
 */

/**
 * @brief       Submits a buffer to transmit HAE results.
 *
 *              The results for the fundamental and 12 harmonics are stored in the 
 *              rfile in 13 8-location fields. Therefore the max number to transfer 
 *              is 13x8=104 dwords.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  pBuffer             Pointer to the buffer.
 * @param [in]  nBufferSize         Size of the given buffer in bytes.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully submitted a buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 * @sa adi_hae_GetTxBuffer
 */
ADI_HAE_RESULT adi_hae_SubmitTxBuffer(
    ADI_HAE_HANDLE        const     hDevice,
    void                           *pBuffer,
    uint32_t              const     nBufSize)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;
    
    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    
    eResult = SubmitBuffer(pDevice, pDevice->pTxChannel, pBuffer, nBufSize, true);

    return eResult;
}

/**
 * @brief       Submits a buffer to receive HAE current (I) input samples.
 *
 *              The request will occur at a nominal 8Khz rate, depending on 
 *              the clock divide value. The rx I and V samples typically come from 
 *              a sinc filter source via memory buffer. For accurate harmonic results, 
 *              the input samples should be derived at exactly the same rate as the 
 *              HAE sample loop. Therefore the sinc and HAE should be programmed to 
 *              be timing matched. There is one dword transfer each HAE sample loop.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  pBuffer             Pointer to the buffer.
 * @param [in]  nBufferSize         Size of the given buffer in bytes.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully submitted a buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 * @sa adi_hae_GetRx0Buffer
 * @sa adi_hae_IsRx0BufferAvailable
 */
ADI_HAE_RESULT adi_hae_SubmitRx0Buffer(
    ADI_HAE_HANDLE        const     hDevice,
    void                           *pBuffer,
    uint32_t              const     nBufSize)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;
    
    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    
    eResult = SubmitBuffer(pDevice, pDevice->pRx0Channel, pBuffer, nBufSize, false);

    return eResult;
}

/**
 * @brief       Submits a buffer to receive HAE voltage (V) input samples.
 *
 *              The request will occur at a nominal 8Khz rate, depending on 
 *              the clock divide value. The rx I and V samples typically come from 
 *              a sinc filter source via memory buffer. For accurate harmonic results, 
 *              the input samples should be derived at exactly the same rate as the 
 *              HAE sample loop. Therefore the sinc and HAE should be programmed to 
 *              be timing matched. There is one dword transfer each HAE sample loop.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [in]  pBuffer             Pointer to the buffer.
 * @param [in]  nBufferSize         Size of the given buffer in bytes.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully submitted a buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 * @sa adi_hae_GetRx1Buffer
 * @sa adi_hae_IsRx1BufferAvailable
 */
ADI_HAE_RESULT adi_hae_SubmitRx1Buffer(
    ADI_HAE_HANDLE        const     hDevice,
    void                           *pBuffer,
    uint32_t              const     nBufSize)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;
    
    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    
    eResult = SubmitBuffer(pDevice, pDevice->pRx1Channel, pBuffer, nBufSize, false);

    return eResult;
}

/**
 * @brief       Get the buffer which contains the latest HAE results.
 *
 * @details     This is a blocking API which waits until the result buffer is available
 *              and returns the pointer to the result buffer. This API also returns back
 *              immediately with an error if there was an hardware error detected.
 *
 *              To avoid the blocking a non-blocking adi_hae_IsTxBufferAvailable can be used.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [out] ppBuffer            Pointer to a location where the processed buffer pointer will
 *                                  be written.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully got the result buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 * @sa          adi_hae_IsTxBufferAvailable
 * @sa          adi_hae_SubmitTxBuffer
 */
ADI_HAE_RESULT adi_hae_GetTxBuffer (
    ADI_HAE_HANDLE        const  hDevice,
    void                **const  ppBuffer)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;
    ADI_HAE_BUF_INFO  *pBufInfo;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    
    /* Wait until a processed buffer is available using
     * Semaphore pend. The Semaphore pend will return immediately
     * is there is buffer available. */
    if(adi_osal_SemPend(pDevice->pTxChannel->hSemaphore,
                        ADI_OSAL_TIMEOUT_FOREVER) != ADI_OSAL_SUCCESS)
    {
        return ADI_HAE_SEM_FAILED;
    }

    if (pDevice->bDMAMode == false)
    {
        /* non-DMA copy into application buffer */
        memcpy(pDevice->pTxChannel->pActiveBuf->pBuffer, 
              (void const *)pDevice->HaeRegs.pRESULTS_RAM, 
               pDevice->pTxChannel->pActiveBuf->nBufSize);
    }
    
    pBufInfo = pDevice->pTxChannel->pActiveBuf;

    /* Return the pointer to the processed buffer */
    *ppBuffer = pBufInfo->pBuffer;
    
    return ADI_HAE_SUCCESS;    
}

/**
 * @brief       Get the buffer which transmits the HAE current (I) input samples.
 *
 * @details     This is a blocking API which waits until the result buffer is available
 *              and returns the pointer to the result buffer. This API also returns back
 *              immediately with an error if there was an hardware error detected.
 *
 *              To avoid the blocking a non-blocking adi_hae_IsRx0BufferAvailable can be used.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [out] ppBuffer            Pointer to a location where the processed buffer pointer will
 *                                  be written.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully got the result buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 * @sa          adi_hae_IsRx0BufferAvailable
 * @sa          adi_hae_SubmitRx0Buffer
 */
ADI_HAE_RESULT adi_hae_GetRx0Buffer (
    ADI_HAE_HANDLE        const  hDevice,
    void                **const  ppBuffer)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;
    ADI_HAE_BUF_INFO  *pBufInfo;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    
    /* Wait until a processed buffer is available using
     * Semaphore pend. The Semaphore pend will return immediately
     * is there is buffer available. */
    if(adi_osal_SemPend(pDevice->pRx0Channel->hSemaphore,
                        ADI_OSAL_TIMEOUT_FOREVER) != ADI_OSAL_SUCCESS)
    {
        return ADI_HAE_SEM_FAILED;
    }

    if (pDevice->pRx0Channel->BufferInfo0.bInUse == false)
    {
        pBufInfo = &pDevice->pRx0Channel->BufferInfo0;
    }
    else
    {
        pBufInfo = &pDevice->pRx0Channel->BufferInfo1;
    }
    
    /* Return the pointer to the processed buffer */
    *ppBuffer = pBufInfo->pBuffer;
    
    return ADI_HAE_SUCCESS;    
}

/**
 * @brief       Get the buffer which transmits the HAE voltage (V) input samples.
 *
 * @details     This is a blocking API which waits until the result buffer is available
 *              and returns the pointer to the result buffer. This API also returns back
 *              immediately with an error if there was an hardware error detected.
 *
 *              To avoid the blocking a non-blocking adi_hae_IsRx1BufferAvailable can be used.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [out] ppBuffer            Pointer to a location where the processed buffer pointer will
 *                                  be written.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully got the result buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 * @sa          adi_hae_IsRx1BufferAvailable
 * @sa          adi_hae_SubmitRx1Buffer
 */
ADI_HAE_RESULT adi_hae_GetRx1Buffer (
    ADI_HAE_HANDLE        const  hDevice,
    void                **const  ppBuffer)
{
    /* Result code */
    ADI_HAE_RESULT     eResult;
    ADI_HAE_BUF_INFO  *pBufInfo;

    /* Pointer to the device data */
    ADI_HAE_DEVICE    *pDevice = (ADI_HAE_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    
    /* Wait until a processed buffer is available using
     * Semaphore pend. The Semaphore pend will return immediately
     * is there is buffer available. */
    if(adi_osal_SemPend(pDevice->pRx1Channel->hSemaphore,
                        ADI_OSAL_TIMEOUT_FOREVER) != ADI_OSAL_SUCCESS)
    {
        return ADI_HAE_SEM_FAILED;
    }

    if (pDevice->pRx1Channel->BufferInfo0.bInUse == false)
    {
        pBufInfo = &pDevice->pRx1Channel->BufferInfo0;
    }
    else
    {
        pBufInfo = &pDevice->pRx1Channel->BufferInfo1;
    }
    
    /* Return the pointer to the processed buffer */
    *ppBuffer = pBufInfo->pBuffer;
    
    return ADI_HAE_SUCCESS;    
}

/**
 * @brief       Queries to find if a result buffer is available.
 *
 * @details     This API can be used to avoid the blocking when the
 *              adi_hae_GetTxBuffer is called. The adi_hae_GetTxBuffer can be
 *              called after finding a processed buffer is available.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [out] pbAvailable         Pointer to a boolean variable where the status of buffer
 *                                  availibility is written.
 *                                  'true'  - is written if buffer is available.
 *                                  'false' - is written if buffer is not available.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully queried the result buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 *
 * @sa          adi_hae_GetTxBuffer
 * @sa          adi_hae_SubmitTxBuffer
 */
ADI_HAE_RESULT adi_hae_IsTxBufferAvailable (
    ADI_HAE_HANDLE         const hDevice,
    bool                  *const pbAvailable
)
{
    /* Pointer to HAE device instance */
    ADI_HAE_DEVICE         *pDevice = (ADI_HAE_DEVICE *)hDevice;
    /* Return code */
    ADI_HAE_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }

    /* This API will only work in non-blocking buffer transaction mode */
    if(pDevice->pTxChannel->eBufferMode != ADI_HAE_BUFFER_MODE_NONBLOCKING)
    {
        return  ADI_HAE_INVALID_OPERATION;
    }

    /* Check if the given pointer is invalid */
    if(pbAvailable == NULL)
    {
        return ADI_HAE_NULL_POINTER;
    }
#endif /* ADI_DEBUG */

    if ((pDevice->pTxChannel->BufferInfo0.bInUse == false) ||
        (pDevice->pTxChannel->BufferInfo1.bInUse == false)) {
        *pbAvailable = true;
    } else {
        *pbAvailable = false;
    }

    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Queries to find if a input current (I) sample buffer is available.
 *
 * @details     This API can be used to avoid the blocking when the
 *              adi_hae_GetRx0Buffer is called. The adi_hae_GetRx0Buffer can be
 *              called after finding a processed buffer is available.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [out] pbAvailable         Pointer to a boolean variable where the status of buffer
 *                                  availibility is written.
 *                                  'true'  - is written if buffer is available.
 *                                  'false' - is written if buffer is not available.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully queried the result buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 *
 * @sa          adi_hae_GetRx0Buffer
 * @sa          adi_hae_SubmitRx0Buffer
 */
ADI_HAE_RESULT adi_hae_IsRx0BufferAvailable (
    ADI_HAE_HANDLE         const hDevice,
    bool                  *const pbAvailable
)
{
    /* Pointer to HAE device instance */
    ADI_HAE_DEVICE         *pDevice = (ADI_HAE_DEVICE *)hDevice;
    /* Return code */
    ADI_HAE_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }

    /* This API will only work in non-blocking buffer transaction mode */
    if(pDevice->pRx0Channel->eBufferMode != ADI_HAE_BUFFER_MODE_NONBLOCKING)
    {
        return  ADI_HAE_INVALID_OPERATION;
    }

    /* Check if the given pointer is invalid */
    if(pbAvailable == NULL)
    {
        return ADI_HAE_NULL_POINTER;
    }
#endif /* ADI_DEBUG */

    if ((pDevice->pRx0Channel->BufferInfo0.bInUse == false) ||
        (pDevice->pRx0Channel->BufferInfo1.bInUse == false)) {
        *pbAvailable = true;
    } else {
        *pbAvailable = false;
    }

    return ADI_HAE_SUCCESS;
}

/**
 * @brief       Queries to find if a input voltage (V) sample buffer is available.
 *
 * @details     This API can be used to avoid the blocking when the
 *              adi_hae_GetRx1Buffer is called. The adi_hae_GetRx1Buffer can be
 *              called after finding a processed buffer is available.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the HAE device.
 * @param [out] pbAvailable         Pointer to a boolean variable where the status of buffer
 *                                  availibility is written.
 *                                  'true'  - is written if buffer is available.
 *                                  'false' - is written if buffer is not available.
 *
 * @return      Status
 *              - #ADI_HAE_SUCCESS                 Successfully queried the result buffer.
 *              - #ADI_HAE_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_HAE_INVALID_OPERATION   [D] The device state is enabled.
 *
 *
 * @sa          adi_hae_GetRx1Buffer
 * @sa          adi_hae_SubmitRx1Buffer
 */
ADI_HAE_RESULT adi_hae_IsRx1BufferAvailable (
    ADI_HAE_HANDLE         const hDevice,
    bool                  *const pbAvailable
)
{
    /* Pointer to HAE device instance */
    ADI_HAE_DEVICE         *pDevice = (ADI_HAE_DEVICE *)hDevice;
    /* Return code */
    ADI_HAE_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_HAE_SUCCESS)
    {
        return eResult;
    }

    /* This API will only work in non-blocking buffer transaction mode */
    if(pDevice->pRx1Channel->eBufferMode != ADI_HAE_BUFFER_MODE_NONBLOCKING)
    {
        return  ADI_HAE_INVALID_OPERATION;
    }

    /* Check if the given pointer is invalid */
    if(pbAvailable == NULL)
    {
        return ADI_HAE_NULL_POINTER;
    }
#endif /* ADI_DEBUG */

    if ((pDevice->pRx1Channel->BufferInfo0.bInUse == false) ||
        (pDevice->pRx1Channel->BufferInfo1.bInUse == false)) {
        *pbAvailable = true;
    } else {
        *pbAvailable = false;
    }

    return ADI_HAE_SUCCESS;
}

/*
 * Internal
 */

/*
 * Does the required buffer management and calls the abstracted
 * (DMA and Interrupt mode) functions
 */
static ADI_HAE_RESULT SubmitBuffer(
    ADI_HAE_DEVICE            *pDevice,
    ADI_HAE_CHANNEL           *pChannel,
    void                      *pBuffer,
    uint32_t                   nBufferSize,
    bool                       bRead
)
{
    ADI_HAE_BUF_INFO   *pBufInfo;
    ADI_HAE_RESULT      eResult;

#ifdef ADI_DEBUG
    /* Check if the given buffer pointer is valid */
    if(pBuffer == NULL)
    {
        return ADI_HAE_NULL_POINTER;
    }

    /* Check the given buffer size */
    if(nBufferSize == 0u)
    {
        return ADI_HAE_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (pChannel->BufferInfo0.bInUse == false)
    {
        pBufInfo = &pChannel->BufferInfo0;
        pBufInfo->pBuffer = pBuffer;
        pBufInfo->nBufSize = nBufferSize;
        pBufInfo->bInUse = true;
    }
    else if (pChannel->BufferInfo1.bInUse == false)
    {
        pBufInfo = &pChannel->BufferInfo1;
        pBufInfo->pBuffer = pBuffer;
        pBufInfo->nBufSize = nBufferSize;
        pBufInfo->bInUse = true;
    }
    else
    {
        return ADI_HAE_SUBMIT_FAILED;
    }
        
    /* Submit the buffer */
    if((eResult = pDevice->pFnTable->pfSubmitBuffer(pDevice,
                                                    pChannel,
                                                    pBuffer,
                                                    nBufferSize,
                                                    bRead)) != ADI_HAE_SUCCESS)
    {
        /* If there was a failure mark the buffer as free */
        pBufInfo->bInUse = false;

        return eResult;
    }
    
    return ADI_HAE_SUCCESS;
}

/* ===================  Interrupt mode functions ======================== */

/* Takes care of cleanup required in interrupt mode */
static ADI_HAE_RESULT hae_Int_Close(
   ADI_HAE_DEVICE      *pDevice
)
{
    return ADI_HAE_SUCCESS;
}

/* Submits a buffer for Rx/Tx in interrupt mode */
static ADI_HAE_RESULT hae_Int_SubmitBuffer (
   ADI_HAE_DEVICE      *pDevice,
   ADI_HAE_CHANNEL     *pChannel,
   void                 *pBuffer,
   uint32_t              nBufferSize,
   bool                  bRead
)
{
    return ADI_HAE_SUCCESS;
}


/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_HAE_RESULT hae_Int_Enable  (
   ADI_HAE_DEVICE      *pDevice,
   ADI_HAE_CHANNEL     *pChannel,
   bool                  bEnable
)
{
    return ADI_HAE_SUCCESS;
}

/* ==================   DMA mode functions ============================== */

/* Takes care of cleanup required when closing HAE which was in DMA mode */
static ADI_HAE_RESULT hae_DMA_Close(
    ADI_HAE_DEVICE      *pDevice
)
{
    /* If a Tx DMA channel was opened */
    if(pDevice->pTxChannel->hDMAChnl != NULL)
    {
        if(adi_pdma_Close(pDevice->pTxChannel->hDMAChnl, false) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }
    }

    /* If a Rx0 DMA channel was opened */
    if(pDevice->pRx0Channel->hDMAChnl != NULL)
    {
        if(adi_pdma_Close(pDevice->pRx0Channel->hDMAChnl, false) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }
    }

    /* If a Rx1 DMA channel was opened */
    if(pDevice->pRx1Channel->hDMAChnl != NULL)
    {
        if(adi_pdma_Close(pDevice->pRx1Channel->hDMAChnl, false) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }
    }

    return ADI_HAE_SUCCESS;
}


/* Submits a buffer in DMA mode */
static ADI_HAE_RESULT hae_DMA_SubmitBuffer(
    ADI_HAE_DEVICE      *pDevice,                                             
    ADI_HAE_CHANNEL     *pChannel,
    void                 *pBuffer,
    uint32_t              nBufferSize,
    bool                  bRead
)
{
    int32_t nBytesPerTransfer = pChannel->nBytesPerTransfer;
#ifdef ADI_DEBUG
    if((uint32_t)(nBufferSize & ((uint32_t)nBytesPerTransfer - 1ul)) != 0ul)
    {
        return ADI_HAE_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */
    if(bRead == true)
    {
        if(adi_pdma_PeripheralRead(pChannel->hDMAChnl,
                                   pBuffer,                                   /* Start Address */
                                   nBufferSize/(uint32_t)nBytesPerTransfer,   /* X Count */
                                   nBytesPerTransfer,                         /* X Modify */
                                   0u,                                        /* Y Count */
                                   0) != ADI_DMA_SUCCESS)                     /* Y Modify */
        {
            return ADI_HAE_DMA_FAILED;
        }
    }
    else
    {
        if(adi_pdma_PeripheralWrite(pChannel->hDMAChnl,
                                   pBuffer,                                   /* Start Address */
                                   1u,                                        /* X Count */
                                   0u,                                        /* X Modify */
                                   nBufferSize/(uint32_t)nBytesPerTransfer,   /* Y Count */
                                   nBytesPerTransfer) != ADI_DMA_SUCCESS)     /* Y Modify */
        {
            return ADI_HAE_DMA_FAILED;
        }
    }

    return ADI_HAE_SUCCESS;
}


/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_HAE_RESULT hae_DMA_Enable(
    ADI_HAE_DEVICE      *pDevice,
    ADI_HAE_CHANNEL     *pChannel,
    bool                 bEnable
)
{

    /* If DMA channel to be enabled*/
    if(bEnable == true)
    {
        if(adi_pdma_EnableXfer(pChannel->hDMAChnl) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }
    }
    /* DMA channel to be disabled */
    else
    {
        if(adi_pdma_DisableXfer(pChannel->hDMAChnl) != ADI_DMA_SUCCESS)
        {
            return ADI_HAE_DMA_FAILED;
        }
    }

    return ADI_HAE_SUCCESS;
}

/* Enables/Disables the data flow */
static ADI_HAE_RESULT EnableDataFlow(
    ADI_HAE_DEVICE           *pDevice,
    ADI_HAE_CHANNEL          *pChannel,
    bool                       bEnable
)
{
    /* HAE return code */
    ADI_HAE_RESULT     eResult;

#ifdef ADI_DEBUG
    /* Check if the data flow is already enabled/disabled */
    if( (( bEnable == true ) && (pChannel->eDataFlowState == ADI_HAE_DATAFLOW_ENABLED)) ||
         ((bEnable == false) && (pChannel->eDataFlowState == ADI_HAE_DATAFLOW_DISABLED)))
    {
        /* If it already in the required state do nothing */
        return ADI_HAE_SUCCESS;
    }

    if(bEnable == true)
    {
        /* Check if buffers are submitted for either Ping and Pong descriptors */
        if((pChannel->BufferInfo0.pBuffer == NULL) &&
           (pChannel->BufferInfo1.pBuffer == NULL))
        {
            return ADI_HAE_BUFFERS_NOT_SUBMITTED;
        }
    }
#endif /* ADI_DEBUG */

    /* If enabling the data flow */
    if(bEnable == true)
    {
        /* Enable the data flow */
        if((eResult = pDevice->pFnTable->pfEnable(pDevice, pChannel, bEnable)) != ADI_HAE_SUCCESS)
        {
            return eResult;
        }

        /* Update the dataflow state */
        pChannel->eDataFlowState    =   ADI_HAE_DATAFLOW_ENABLED;
    }
    else
    {
        /* Disable the data flow */
        if((eResult = pDevice->pFnTable->pfEnable(pDevice, pChannel, bEnable)) != ADI_HAE_SUCCESS)
        {
            return eResult;
        }

        /* Update the dataflow sate */
        pChannel->eDataFlowState    =   ADI_HAE_DATAFLOW_DISABLED;
    }

    return ADI_HAE_SUCCESS;
}

static void swapBuffers(ADI_HAE_CHANNEL *pChannel)
{
    pChannel->pActiveBuf->bInUse = false;
    
    /* swap buffers */
    if (pChannel->pActiveBuf == &(pChannel->BufferInfo0))
    {
        pChannel->pActiveBuf = &(pChannel->BufferInfo1);
    }
    else
    {
        pChannel->pActiveBuf = &(pChannel->BufferInfo0);
    }
    
    pChannel->pActiveBuf->bInUse = true;
}

/* Status interrupt handler */
static void StatusIntHandler(uint32_t iid, void *pCBParam)
{   
    ADI_HAE_BUF_INFO *pBufInfo0;
    ADI_HAE_BUF_INFO *pBufInfo1;
    int32_t *pSamples;
    int32_t current;
    int32_t voltage;
    
    /* Pointer to the device data */
    ADI_HAE_DEVICE *pDevice = (ADI_HAE_DEVICE *) pCBParam;
    
    /* status bits */
    uint32_t status = *pDevice->HaeRegs.pSTAT;
        
    if (status & BITM_HAE_STAT_TXIRQ)
    {        
        /* application callback */   
        if(pDevice->pfCallback != NULL)
        {            
            /* non-DMA copy into application buffer */
            memcpy(pDevice->pTxChannel->pActiveBuf->pBuffer, 
                   (void const *)pDevice->HaeRegs.pRESULTS_RAM, 
                   pDevice->pTxChannel->pActiveBuf->nBufSize);

            pDevice->pfCallback(pDevice->pCBParam, 
                ADI_HAE_EVENT_RESULTS_READY, pDevice->pTxChannel->pActiveBuf->pBuffer);
        }
        else
        {
            /* Unblock Tx GetBuffer/Read calls */
            if(pDevice->pTxChannel->hSemaphore != NULL)
            {
                adi_osal_SemPost(pDevice->pTxChannel->hSemaphore);
            }
        }
        
        swapBuffers(pDevice->pTxChannel);

        /* clear status W1C */
        *pDevice->HaeRegs.pSTAT |= BITM_HAE_STAT_TXIRQ;        
    }
    
    if (status & BITM_HAE_STAT_RXIRQ)
    {        
        /* application callback */   
        if(pDevice->pfCallback != NULL)
        {            
            pDevice->pfCallback(pDevice->pCBParam, 
                ADI_HAE_EVENT_INPUT_SAMPLE, NULL);
        }
        else
        {
            pBufInfo0 = pDevice->pRx0Channel->pActiveBuf;
            pSamples = (int32_t*)pBufInfo0->pBuffer;
            pSamples += pBufInfo0->index;
            current = *pSamples;
            pBufInfo0->index++;

            pBufInfo1 = pDevice->pRx1Channel->pActiveBuf;
            pSamples = (int32_t*)pBufInfo1->pBuffer;
            pSamples += pBufInfo1->index;
            voltage = *pSamples;
            pBufInfo1->index++;
            
            adi_hae_InputSample(pDevice, current, voltage);

            if (pBufInfo0->index >= pBufInfo0->nBufSize/4u)  /* 4 bytes per sample */
            {
                /* Unblock Rx GetBuffer/Read calls */
                if(pDevice->pRx0Channel->hSemaphore != NULL)
                {
                   adi_osal_SemPost(pDevice->pRx0Channel->hSemaphore);
                } 
                pBufInfo0->index = 0u;
                
                swapBuffers(pDevice->pRx0Channel);
            }
            
            if (pBufInfo1->index >= pBufInfo1->nBufSize/4u)  /* 4 bytes per sample */
            {
                /* Unblock Rx GetBuffer/Read calls */
                if(pDevice->pRx1Channel->hSemaphore != NULL)
                {
                   adi_osal_SemPost(pDevice->pRx1Channel->hSemaphore);
                } 
                pBufInfo1->index = 0u;
                
                swapBuffers(pDevice->pRx1Channel);
            }
        }
            
       /* clear status W1C */
       *pDevice->HaeRegs.pSTAT |= BITM_HAE_STAT_RXIRQ;        
    }
}

/*
 * HAE Tx DMA interrupt handler
 */
static void HaeTxDMAHandler(void *pCBParam, uint32_t Event, void *pArg)
{
    /* Pointer to the HAE device instance data */
    ADI_HAE_DEVICE *pDevice = (ADI_HAE_DEVICE  *)pCBParam;

    switch(Event)
    {
        /* The given buffer processed */
        case (uint32_t)ADI_DMA_EVENT_BUFFER_PROCESSED:

        /* application callback */   
        if(pDevice->pfCallback != NULL)
        {            
            pDevice->pfCallback(pDevice->pCBParam, 
                ADI_HAE_EVENT_RESULTS_READY, pDevice->pTxChannel->pActiveBuf->pBuffer);
        }
        else
        {
            /* Unblock Tx GetBuffer/Read calls */
            if(pDevice->pTxChannel->hSemaphore != NULL)
            {
                adi_osal_SemPost(pDevice->pTxChannel->hSemaphore);
            }
        }
        
        /* resubmit buffer */
        if (hae_DMA_SubmitBuffer(pDevice, pDevice->pTxChannel, 
                                 pDevice->pTxChannel->pActiveBuf->pBuffer, 
                                 pDevice->pTxChannel->pActiveBuf->nBufSize, true))
        {
            return;
        }

        swapBuffers(pDevice->pTxChannel);
        break;

        default:
            /* We don't care about other events */
            break;
    }

    return;
}

/*
 * HAE Rx0 DMA interrupt handler
 */
static void HaeRx0DMAHandler(void *pCBParam, uint32_t Event, void *pArg)
{
    /* Pointer to the HAE device instance data */
    ADI_HAE_DEVICE *pDevice = (ADI_HAE_DEVICE  *)pCBParam;

    switch(Event)
    {
        /* The given buffer processed */
        case (uint32_t)ADI_DMA_EVENT_BUFFER_PROCESSED:

        /* application callback */   
        if(pDevice->pfCallback != NULL)
        {            
            pDevice->pfCallback(pDevice->pCBParam, 
                ADI_HAE_EVENT_RESULTS_READY, pDevice->pRx0Channel->pActiveBuf->pBuffer);
        }
        else
        {
            /* Unblock Rx GetBuffer/Read calls */
            if(pDevice->pRx0Channel->hSemaphore != NULL)
            {
                adi_osal_SemPost(pDevice->pRx0Channel->hSemaphore);
            }
        }

        swapBuffers(pDevice->pRx0Channel);
        break;

        default:
            /* We don't care about other events */
            break;
    }

    return;
}

/*
 * HAE Rx1 DMA interrupt handler
 */
static void HaeRx1DMAHandler(void *pCBParam, uint32_t Event, void *pArg)
{
    /* Pointer to the HAE device instance data */
    ADI_HAE_DEVICE *pDevice = (ADI_HAE_DEVICE  *)pCBParam;

    switch(Event)
    {
        /* The given buffer processed */
        case (uint32_t)ADI_DMA_EVENT_BUFFER_PROCESSED:

        /* application callback */   
        if(pDevice->pfCallback != NULL)
        {            
            pDevice->pfCallback(pDevice->pCBParam, 
                ADI_HAE_EVENT_RESULTS_READY, pDevice->pRx1Channel->pActiveBuf->pBuffer);
        }
        else
        {
            /* Unblock Rx GetBuffer/Read calls */
            if(pDevice->pRx1Channel->hSemaphore != NULL)
            {
                adi_osal_SemPost(pDevice->pRx1Channel->hSemaphore);
            }
        }
                
        swapBuffers(pDevice->pRx1Channel);
        break;

        default:
            /* We don't care about other events */
            break;
    }

    return;
}

#ifdef ADI_DEBUG
/* Validate the device */
static ADI_HAE_RESULT ValidateDevice(ADI_HAE_DEVICE   *pDevice)
{
    uint32_t i;

    /* Check if the given pointer matches with any of the devices */
    for(i = 0; i < ADI_HAE_NUM_DEVICES; i++)
    {
        /* If found a match return success */
        if(pDevice == &gHaeDevices[i])
        {
            return ADI_HAE_SUCCESS;
        }
    }

    /* Could not find the match return failure */
    return ADI_HAE_INVALID_HANDLE;
}
    
static bool IsEnabled(ADI_HAE_DEVICE   *pDevice)
{
    return  (*pDevice->HaeRegs.pCFG2 & BITM_HAE_CFG2_EN) >> BITP_HAE_CFG2_EN;
}
#endif /* ADI_DEBUG */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*@}*/
