/*
 *****************************************************************************
 * @file:    adi_counter_bf6xx.c
 * @brief:   Rotary counter  Device Driver for BF6XX family  processor
 * @version: $Revision: 22007 $
 * @date:    $Date: 2013-06-25 15:06:10 -0400 (Tue, 25 Jun 2013) $
 *****************************************************************************

 Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

 This software is proprietary and confidential.  By using this software you agree
 to the terms of the associated Analog Devices License Agreement.

 *********************************************************************************/
/*!
 * @file     adi_counter_bf6xx.c
 *
 * @brief     This is the primary source file containing rotary device Driver for BF6XX family processor.
 *
 * @details
 *            Contains the implementation of the APIs for the rotary counter peripheral driver.
 */


/** @addtogroup rotary_counter_Driver_BF6xx Rotary Counter Driver
 *  @{
 */


/*********************************************************************

Include files

*********************************************************************/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif

#include <string.h>
#include <assert.h>
#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#include <drivers/rotary/adi_counter.h>

#if defined(ADI_CODE_IN_ROM)
#include "../rom/adi_rom_def.h"
#else
/* All driver data structure members are initialized at compile time. CONST data is possible */
#define CONST const
#endif


#if defined(__ADSPBF548_FAMILY__) || \
    defined(__ADSPBF527_FAMILY__) || \
    defined(__ADSPBF526_FAMILY__) || \
    defined(__ADSPBF518_FAMILY__) || \
    defined(__ADSPBF506F_FAMILY__)
#include "adi_counter_reg_bf5xx.h"
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"Allow defenition of objects in the source file. MISRA treats this as header file.")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined outside the block eventhough they are used in one function, for ease maintainence.")
#pragma diag(suppress:misra_rule_10_1_a:"Allow The value of an expression of integer type to be implicitly converted to a different underlying type if it is not a conversion to a wider integer type of the same signedness. (\"signed char\" and \"unsigned short\"")
#pragma diag(suppress:misra_rule_13_1:"Allow assignment in if statements. Required for efficient code")
#pragma diag(suppress:misra_rule_13_5:"Allow expressions of for statement that are not concerned with loop control for code efficiency.")
#pragma diag(suppress:misra_rule_13_7:"Allow  Boolean operations whose results are invariant ") /* ADI_DEBUG only */
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_17_4:"Allow pointer arithmetic for MMR address generation")
#endif /* _MISRA_RULES */

#include "adi_counter_data_bfxxx.c"



/**
 * @param [in]  nDeviceNum  Rotary counter device instance to be opened.
 *
 * @param [in]  pMemory     Pointer to a 32 bit aligned buffer of size #ADI_COUNTER_MEMORY_SIZE
 *                          required by the driver for the operation of specified rotary counter
 *                          device.
 *
 * @param [in]  nMemSize    Size of the buffer to which "pMemory" points.
 *
 * @param [in]  pfCallback  Function pointer to Callback function. Registered call back function will be called
 *                          with EVENT and the content of the count register.
 *
 * @param [in]  pCBparam    Call back function parameter.
 *
 * @param [out] phDevice    Pointer to a location where rotary device handle to be written. This
 *                          handle will be used as an identity of device and need to be passed as
 *                          one of the argument to functions which manage the device.
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS                   Successfully initialized the specified counter
 *                                             device.
 *
 *    - #ADI_COUNTER_DEVICE_IN_USE             Specified counter is already in use.
 *
 *    - #ADI_COUNTER_INT_REGISTRATION_FAILED   Failed to register the device with interrupt manager.
 *
 *    - #ADI_COUNTER_INVALID_DEVICE_NUM   [D]  Specified device instance is invalid.
 *
 *    - #ADI_COUNTER_INVALID_NULL_POINTER [D]  Specified pointer is invalid.
 *
 *   @sa  adi_counter_Enable()
 *   @sa  adi_counter_Close()
*/

ADI_COUNTER_RESULT adi_counter_Open(
                                       uint32_t             nDeviceNum,
                                       void                *pMemory,
                                       uint32_t             nMemSize,
                                       ADI_CALLBACK         pfCallback,
                                       void                *pCBparam,
                                       ADI_COUNTER_HANDLE  *phDevice
                                     )

{
    /* Pointer to the counter Device Instance Data */
    ADI_COUNTER_DEVICE   *pDevice = NULL;
#ifdef ADI_DEBUG

    /* Check if the given device number is within the range of number counter
     * devices present in the processor
     */
    if(nDeviceNum >(ADI_NUM_COUNTER_DEVICE_INSTANCES-1U))
    {
        return ADI_COUNTER_INVALID_DEVICE_NUM;
    }
    if(nMemSize < (uint32_t)ADI_COUNTER_MEMORY_SIZE)
    {
         return(ADI_COUNTER_INSUFFICIENT_MEMORY);
    }
    if((pMemory  == NULL)||(phDevice == NULL)||(pfCallback == NULL) )
    {
        return (ADI_COUNTER_INVALID_NULL_POINTER);
    }
      /* Asserts to validate the memory size macros */
    assert(ADI_COUNTER_MEMORY_SIZE == sizeof(ADI_COUNTER_DEVICE));
#endif /* ADI_DEBUG */
    /* reset the buffer state */
    memset(pMemory, 0, (uint32_t)ADI_COUNTER_MEMORY_SIZE);

#if !defined(ADI_CODE_IN_ROM)
    /* Check if the device instance is in use */
    if(counter_device_info[nDeviceNum].eState != ADI_COUNTER_STATE_UNINITIALIZED)
#else
    if(pram_for_rom_Code->counter_device_info[nDeviceNum].eState  != ADI_COUNTER_STATE_UNINITIALIZED)
#endif
    {
        return ADI_COUNTER_DEVICE_IN_USE;
    }



    /* Assign the pointers with the supplied memory. */
    pDevice = (ADI_COUNTER_DEVICE *)pMemory;

#if !defined(ADI_CODE_IN_ROM)
    pDevice->pDeviceInfo   =   &counter_device_info[nDeviceNum];

    pDevice->pCounterRegs   =   counter_device_info[nDeviceNum].pCounterRegs;
#else
    pDevice->pDeviceInfo   =   &pram_for_rom_Code->counter_device_info[nDeviceNum];

    pDevice->pCounterRegs   =   pram_for_rom_Code->counter_device_info[nDeviceNum].pCounterRegs;
#endif


#if defined(__ADSPBF609_FAMILY__) || \
    defined(__ADSPBF707_FAMILY__)
    /* clear any pending interrupt */
    if(adi_sec_ClearPendStatus(pDevice->pDeviceInfo->nStatusSID)!= ADI_SEC_SUCCESS)
    {
        return(ADI_COUNTER_INT_REGISTRATION_FAILED);
    }


    if(adi_int_InstallHandler(pDevice->pDeviceInfo->nStatusSID,
                              StatusHandler,
                              pDevice,
                              true) != ADI_SEC_SUCCESS)
    {
#ifndef ADI_CODE_IN_ROM
    	 pDevice->pDeviceInfo->eState =   ADI_COUNTER_STATE_UNINITIALIZED;

#else
    	 pram_for_rom_Code->counter_device_info[nDeviceNum].eState  =   ADI_COUNTER_STATE_UNINITIALIZED;
#endif

         return ADI_COUNTER_INT_REGISTRATION_FAILED;
    }
#else
    if(adi_int_InstallHandler(pDevice->pDeviceInfo->nStatusSID,
                              StatusHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
         pDevice->pDeviceInfo->eState =   ADI_COUNTER_STATE_UNINITIALIZED;
         return ADI_COUNTER_INT_REGISTRATION_FAILED;
    }
#endif
    pDevice->pCounterRegs->COUNTER_CONFIG = 0x00U;
    pDevice->pCounterRegs->COUNTER_IMASK = 0x00U;
    pDevice->pfCallback = pfCallback;
    pDevice->pCBParam   = pCBparam;
    /* Return the device handle to the application */
    *phDevice   =   pDevice;
    adi_osal_EnterCriticalRegion();
#ifndef ADI_CODE_IN_ROM
    counter_device_info[nDeviceNum].hDevice = pDevice;
    counter_device_info[nDeviceNum].eState =   ADI_COUNTER_STATE_INITIALIZED;
#else
	 pram_for_rom_Code->counter_device_info[nDeviceNum].hDevice = pDevice;
	 pram_for_rom_Code->counter_device_info[nDeviceNum].eState  =   ADI_COUNTER_STATE_INITIALIZED;
#endif
    adi_osal_ExitCriticalRegion();
    return ADI_COUNTER_SUCCESS;
}

 /**
  * @brief       Closes the operation of specified rotary counter device.
  *
  * @details     Closes the operation of specified counter device. Associated DMA will be closed if
  *              the device is operating in DMA mode. Device need to be opened again for any further
  *              use.
  *
  * @param [in]  hDevice  Device handle to counter device, obtained when a counter device is opened
  *                       successfully.
  *
  * @return      Status
  *
  *    - #ADI_COUNTER_SUCCESS             Successfully closed the specified counter device.
  *
  *    - #ADI_COUNTER_INT_REGISTRATION_FAILED   Failed to un-register the device with interrupt manager.
  *
  *    - #ADI_COUNTER_FAILED              Failed to close counter device.
  *
  *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
  *
  * @sa          adi_counter_Open()
  *
  */
ADI_COUNTER_RESULT adi_counter_Close( ADI_COUNTER_HANDLE  hDevice )
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE     *pDevice = (ADI_COUNTER_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT        eResult;

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    /* Unregister the status interrupt handler */
    if(adi_int_UninstallHandler(pDevice->pDeviceInfo->nStatusSID)!= ADI_INT_SUCCESS)
    {
            /* return the error */
            return ADI_COUNTER_INT_REGISTRATION_FAILED;
    }

    adi_osal_EnterCriticalRegion();
    /* clear the STAT register */
    pDevice->pCounterRegs->COUNTER_CONFIG = 0x00U;
    pDevice->pCounterRegs->COUNTER_IMASK = 0x00U;
    /* Free up the device */
    pDevice->pDeviceInfo->eState  =   ADI_COUNTER_STATE_UNINITIALIZED;
    pDevice->pDeviceInfo->hDevice = NULL;
    adi_osal_ExitCriticalRegion();
    return ADI_COUNTER_SUCCESS;
}
/**
 * @brief       This function Enable/Disable the specified counter device.
 *
 * @param [in]  hDevice  Device handle to counter device, obtained when a counter device is opened
 *                       successfully.
 *
 * @param [in]  bEnable  Boolean flag to indicate whether to enable/disable the counter device as
 *                       below.
 * \n
 * \n                    true  : Enable the device.
 * \n
 * \n                    false : Disable the device.
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully enable/disable the specified counter device.
 *
 *    - #ADI_COUNTER_FAILED              Failed to enable/disable the counter device.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 *
 */
ADI_COUNTER_RESULT adi_counter_Enable(ADI_COUNTER_HANDLE  hDevice,bool  bEnable)
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
    /* Return code */
    ADI_COUNTER_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */

    if(bEnable == true)
    {
       /* Enable the device */
       pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(ENUM_CNT_CFG_CNTEN);
       pDevice->pDeviceInfo->eState =   ADI_COUNTER_STATE_ACTIVE;
    }
    else
    {
        /* Disable the device */
        pDevice->pCounterRegs->COUNTER_CONFIG &=(uint16_t)(~(ENUM_CNT_CFG_CNTEN));
        /* Change the device status */
        adi_osal_EnterCriticalRegion();
        pDevice->pDeviceInfo->eState = ADI_COUNTER_STATE_INITIALIZED;
        adi_osal_ExitCriticalRegion();
    }
    /* retturn success */
    return ADI_COUNTER_SUCCESS;
}
/**
 * @brief       Sets the mode of operation.
 *
 * @param [in]  hDevice         Device handle to which control word to be set.
 *
 * @param [in]  eOperationMode  Mode with which the device need to function.
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully set mode for counter device.
 *
 *    - #ADI_COUNTER_FAILED              Failed to set the mode for counter device.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 *    - #ADI_COUNTER_INVALID_MODE   [D]  Specified mode is Invalid.
 *
 */
ADI_COUNTER_RESULT adi_counter_SetOperationMode(
                                             ADI_COUNTER_HANDLE            hDevice,
                                             ADI_COUNTER_OPERATIONAL_MODE  eOperationMode
                                           )

{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
#ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
    if((eOperationMode != ADI_CNT_QUAD_ENC_MODE) &&
       (eOperationMode != ADI_CNT_BIN_ENC_MODE)  &&
       (eOperationMode != ADI_CNT_UD_CNT_MODE)   &&
       (eOperationMode != ADI_CNT_DIR_CNT_MODE)  &&
       (eOperationMode != ADI_CNT_DIR_TMR_MODE)
     )
    {
        return(ADI_COUNTER_INVALID_MODE);
    }
#endif /* ADI_DEBUG */
    pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(eOperationMode);
    return(ADI_COUNTER_SUCCESS);
}
/**
 * @brief       Sets the mode for boundary register..
 *
 * @param [in]  hDevice        Device handle to which control word to be set.
 *
 * @param [in]  eBoundaryMode  Mode for boundary register..
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully set boundary mode for counter device.
 *
 *    - #ADI_COUNTER_FAILED              Failed to set the boundary mode for counter device.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 *    - #ADI_COUNTER_INVALID_MODE   [D]  Specified mode is Invalid.
 *
 */
ADI_COUNTER_RESULT adi_counter_SetBoundaryMode(
                                             ADI_COUNTER_HANDLE         hDevice,
                                             ADI_COUNTER_BOUNDARY_MODE  eBoundaryMode
                                           )
{
/* Pointer to counter device instance */
ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
#ifdef ADI_DEBUG
/* Return code */
ADI_COUNTER_RESULT         eResult;
/* Validate the given handle */
if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
{
    return eResult;
}
if((eBoundaryMode != ADI_CNT_BND_COMP)  &&
   (eBoundaryMode != ADI_CNT_BND_ZERO)  &&
   (eBoundaryMode != ADI_CNT_BND_CAPT)  &&
   (eBoundaryMode != ADI_CNT_BND_AEXT)
 )
{
    return(ADI_COUNTER_INVALID_MODE);
}
#endif /* ADI_DEBUG */
pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(eBoundaryMode);
return(ADI_COUNTER_SUCCESS);
}

/**
 * @brief       Sets the polarity for the input pins(CUD,CDG,CZM).
 *
 * @param [in]  hDevice  Device handle to which control word to be set.
 *
 * @param [in]  blowCDG  Sets the polarity for CDG(Count Down and Gate) Pin. i.e Sets the active
 *                       edge for CDG input.
 * \n
 * \n                    true : Active low/Falling edge enables the counter.
 * \n
 * \n                    false : Active high/Rising edge enables the counter.
 *
 * @param [in]  blowCUD  Sets the polarity for CUD(Count Up and Direction) Pin.i.e Sets the active
 *                       edge for CUD input.
 * \n
 * \n                    true : Active low/Falling edge enables the counter.
 * \n
 * \n                    false : Active high/Rising edge enables the counter.
 *
 * @param [in]  blowCZM  Sets the polarity for CZM(CountZero Marker) Pin.i.e Sets the active edge
 *                       for CZM input.
 * \n
 * \n                    true : Active low/Falling edge as active edge for CZM input.
 * \n
 * \n                    false : Active high/Rising edge as active edge for CZM input
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully set polarity for the input pins.
 *
 *    - #ADI_COUNTER_FAILED              Failed to set the polarity.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 */
ADI_COUNTER_RESULT adi_counter_SetPolarity(
                                            ADI_COUNTER_HANDLE  hDevice,
                                            bool                blowCDG,
                                            bool                blowCUD,
                                            bool                blowCZM
                                       )
{

    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
    #ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
    #endif /* ADI_DEBUG */
    pDevice->pCounterRegs->COUNTER_CONFIG &=(uint16_t)~(ENUM_CNT_CFG_CDGINV_ALO |ENUM_CNT_CFG_CUDINV_ALO |ENUM_CNT_CFG_CZMINV_ALO);
    if(blowCDG == true)
    {
        pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(ENUM_CNT_CFG_CDGINV_ALO);
    }
    if(blowCUD == true)
    {
        pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(ENUM_CNT_CFG_CUDINV_ALO);
    }
    if(blowCZM == true)
    {
        pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(ENUM_CNT_CFG_CZMINV_ALO);
    }

    return(ADI_COUNTER_SUCCESS);

}

/**
 * @brief       Sets the prescale value for debouncing filter.
 *
 * @details     The filtering mechanism is implemented for all input  pins of counter device. When a
 *              transition is detected on a pin, the corresponding counter starts counting up to
 *              the programmed  number of system clock ( SCLK ) cycles. The state of the pin is
 *              latched after specified time (Tf) elapsed. Duration of this period is calculated
 *              as below:
 * \n
 * \n           Tf = 128 * ((2^nPreScale)/system_clock) Seconds.
 * \n
 * \n           Valid range for nPreScale is between 0 to  17.
 * \n
 * \n           Example: nPreScale = 2 and system clock = 125 MHz
 * \n
 * \n           Tf = (128*4)/125000000 = 4.096 Micro seconds.
 *
 * @param [in]  hDevice    Device handle to which control word to be set.
 *
 * @param [in]  nPreScale  Prescale value for input pin noise filtering.
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS               Successfully set polarity for the input pins.
 *
 *    - #ADI_COUNTER_FAILED                Failed to set the polarity.
 *
 *    - #ADI_COUNTER_INVALID_PRESCALE [D]  Failed to set the polarity.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE   [D]  Invalid counter device handle.
 *
 *
 */
ADI_COUNTER_RESULT adi_counter_SetPreScale(
                                            ADI_COUNTER_HANDLE hDevice,
                                            uint8_t            nPreScale
                                          )
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
    #ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
    if(nPreScale > MAX_PRESCALE_VALUE )
    {
        return(ADI_COUNTER_INVALID_PRESCALE);
    }

    #endif /* ADI_DEBUG */
    pDevice->pCounterRegs->COUNTER_DEBOUNCE =nPreScale;
    /* pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(ENUM_CNT_CFG_DEBEN);*/
    return(ADI_COUNTER_SUCCESS);
}
/**
 * @brief       Enale/Disable the input pins without disabling the device itself.
 *
 * @param [in]  hDevice       Device handle to which control word to be set.
 *
 * @param [in]  bEnableInput  Boolean flag to indicate whether to enable/disable CUDand CDG input.
 *                            By default it is enabled when the device is opened, CDG and CUD inputs
 *                            are enabled.
 * \n
 * \n                         true : Enable CUD and CDG input.
 * \n
 * \n                         false : Disable CUD and CDG input.
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully enabled/disabled  the input pins.
 *
 *    - #ADI_COUNTER_FAILED              Failed to enable/disable the input pins.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 */
ADI_COUNTER_RESULT adi_counter_EnableInput(
                                         ADI_COUNTER_HANDLE  hDevice,
                                         bool                bEnableInput
                                      )
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
#ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    pDevice->pCounterRegs->COUNTER_CONFIG &=(uint16_t)~(ENUM_CNT_CFG_INPDIS);
    if(bEnableInput == false)
    {
        pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(ENUM_CNT_CFG_INPDIS);
    }
    return(ADI_COUNTER_SUCCESS);
}
/**
 * @brief       Sets the minimum and maximum boundary values..
 *
 * @param [in]  hDevice   Device handle to which control word to be set.
 *
 * @param [in]  nMinimum  Minimum boundary value.
 *
 * @param [in]  nMaximum  Maximum boundary value.
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully set boundary values for the specified counter device.
 *
 *    - #ADI_COUNTER_FAILED              Failed to set the boundary value.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 */
ADI_COUNTER_RESULT adi_counter_SetBoundary(
                                         ADI_COUNTER_HANDLE         hDevice,
                                         int32_t                   nMinimum,
                                         int32_t                   nMaximum
                                       )
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
    #ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
    #endif /* ADI_DEBUG */
    pDevice->pCounterRegs->COUNTER_MIN =nMinimum;
    pDevice->pCounterRegs->COUNTER_MAX =nMaximum;
    return(ADI_COUNTER_SUCCESS);
}
/**
 * @brief       Enable/disable the interrupt for the specified event.
 *
 * @param [in]  hDevice   Device handle to which control word to be set.
 *
 * @param [in]  eEventID  Event for which interrupt to be enabled/disabled.
 *
 * @param [in]  bEnable   Boolean flag to indicate whether to enable/disable the interrupt for the
 *                        specified event.
 * \n
 * \n                     true : Enable interrupt for specified interrupt ID(Unmask the interrupt for the specified event).
 * \n
 * \n                     false : Disable interrupt for specified interrupt ID(Mask the interrupt for the specified event).
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully enabled/disabled the interrupt for the
 *                                       specified event.
 *
 *    - #ADI_COUNTER_FAILED              Failed enable/disable the interrupt for the specified
 *                                       event.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 */
ADI_COUNTER_RESULT adi_counter_EnableInterrupt(
                                            ADI_COUNTER_HANDLE hDevice,
                                            ADI_COUNTER_EVENT  eEventID,
                                            bool               bEnable
                                          )
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
    uint16_t nEvent = (uint16_t)eEventID;
    #ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
    #endif /* ADI_DEBUG */
    pDevice->pCounterRegs->COUNTER_IMASK &=(~(nEvent));
    if(true== bEnable)
    {
        pDevice->pCounterRegs->COUNTER_IMASK |=(nEvent);
        if((eEventID == ADI_CNT_EVENT_COUNT_TO_ZERO_INT) )
        {
            pDevice->pCounterRegs->COUNTER_CONFIG |= (uint16_t)ENUM_CNT_CFG_ZMZC_EN;
        }
    }
    return(ADI_COUNTER_SUCCESS);
}
/**
 * @brief       Performs various commands supported by the device.
 *
 * @details     Various commands like setting the counter to zero, Copying  the current counter value
 *              Minimum/Maximum  boundary registers etc.
 *
 * @param [in]  hDevice  Device handle to which control word to be set.
 *
 * @param [in]  eCommand Specify the operation to be performed.
 *
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully performed the specified command.
 *
 *    - #ADI_COUNTER_FAILED              Failed perform the specified command
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 */
ADI_COUNTER_RESULT adi_counter_SetCommand(
                                         ADI_COUNTER_HANDLE     hDevice,
                                         ADI_COUNTER_COMMAND    eCommand
                                       )
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
#ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    pDevice->pCounterRegs->COUNTER_COMMAND &=(uint16_t)(eCommand);
    return(ADI_COUNTER_SUCCESS);
}
/**
 * @brief       Sets the control word to specified register of the counter device.
 *
 * @param [in]  hDevice       Device handle to which control word to be set.
 *
 * @param [in]  eRegister     Indicate which control register to be written.
 *
 * @param [in]  nValue        Value to be written to the specified register of the counter device.
 *                            Please note that, although there are functional configuration APIs available,
 *                            this API is intended for expert configuration and will overwrite any other value
 *                            already established in the specified register.Upper 16 bits(16 MSBs) are ignored
 *                            if the width of the specified register is 16 bit wide
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully written  given  value to  the specified register counter of
 *                                       device.
 *
 *    - #ADI_COUNTER_FAILED              Failed to set the control word to the specified device.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 */
ADI_COUNTER_RESULT adi_counter_SetControlReg(
                                          ADI_COUNTER_HANDLE    hDevice,
                                          ADI_COUNTER_REGISTER  eRegister,
                                          uint32_t              nValue
                                        )
{
    /* Pointer to SPORT device instance */
     ADI_COUNTER_DEVICE        *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
     /* Return code */
     ADI_COUNTER_RESULT        eResult = ADI_COUNTER_SUCCESS;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }
   /* Don't allow to change the control register when the Data flow is enabled.
     */
    if(pDevice->pDeviceInfo->eState == ADI_COUNTER_STATE_ACTIVE)
    {
        return ADI_COUNTER_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */
    /* set the control register. */
    switch(eRegister)
    {
        case ADI_CNT_CONFIG_REG:
            pDevice->pCounterRegs->COUNTER_CONFIG =(uint16_t)nValue;
            break;
        case ADI_CNT_IMASK_REG:
            pDevice->pCounterRegs->COUNTER_IMASK = (uint16_t)nValue;
            if((nValue & (uint16_t)ADI_CNT_EVENT_COUNT_TO_ZERO_INT)!=0U )
            {
                pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t) ENUM_CNT_CFG_ZMZC_EN;
            }
            break;
        case ADI_CNT_COMMAND_REG:
            pDevice->pCounterRegs->COUNTER_COMMAND = (uint16_t)nValue;
            break;
        case ADI_CNT_DEBOUNCE_REG:
            pDevice->pCounterRegs->COUNTER_DEBOUNCE =(uint16_t) nValue;
            break;
        case ADI_CNT_COUNTER_REG:
            pDevice->pCounterRegs->COUNTER_COUNTER = nValue;
            break;
        case ADI_CNT_MAX_REG:
            pDevice->pCounterRegs->COUNTER_MAX = nValue;
            break;
        case ADI_CNT_MIN_REG:
            pDevice->pCounterRegs->COUNTER_MIN = nValue;
            break;
        default:
            eResult = ADI_COUNTER_INVALID_REGISTER;
            break;
    }
    return eResult;
}
/*
 * @brief       Callback function for interrupt manager.
 *
 * @param [in]  IID       Event ID.
 *
 * @param [in]  pCBParam  Callback parameter interrupt manager.
 *
 *
 */
static void StatusHandler(uint32_t IID, void *pCBParam  )
{
    /* Pointer to the counter device instance data */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE  *)pCBParam;
    uint16_t nStatus = pDevice->pCounterRegs->COUNTER_STATUS;
    uint16_t i;
    uint32_t nCount;
    nStatus &= pDevice->pCounterRegs->COUNTER_IMASK;
    for ( i = 1U; i <=(uint16_t)(BITM_CNT_IMSK_CZMZ) ; i = i<<1U)
    {
           /* check each of the interrupt pending bits */
             if ((nStatus & i) != 0U)
             {
                 /* clear the status bit(write-1-to-clear) */
                 pDevice->pCounterRegs->COUNTER_STATUS |= i;
                   nCount = pDevice->pCounterRegs->COUNTER_COUNTER;
                   if(pDevice->pfCallback != NULL)
                   {
                     (pDevice->pfCallback)(pDevice->pCBParam,(uint32_t)i,(void *)nCount);
                   }
             }

    }
    return;
}
/**
 * @brief       This function Enable/Disable the debouncing filter.
 *
 * @param [in]  hDevice  Device handle to counter device, obtained when a counter device is opened
 *                       successfully.
 *
 * @param [in]  bEnable  Boolean flag to indicate whether to enable/disable the debounce filter.
 * \n
 * \n                    true  : Enable the debouncing filter.
 * \n
 * \n                    false : Disable the debouncing filter
 *
 * @return      Status
 *
 *    - #ADI_COUNTER_SUCCESS             Successfully enable/disable the debouncing filter device.
 *
 *    - #ADI_COUNTER_FAILED              Failed to enable/disable the debouncing filter.
 *
 *    - #ADI_COUNTER_INVALID_HANDLE [D]  Invalid counter device handle.
 *
 *
 */
ADI_COUNTER_RESULT adi_counter_EnableDebounce(ADI_COUNTER_HANDLE  hDevice,bool  bEnable)
{
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE         *pDevice = (ADI_COUNTER_DEVICE *)hDevice;
    #ifdef ADI_DEBUG
    /* Return code */
    ADI_COUNTER_RESULT         eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_COUNTER_SUCCESS)
    {
        return eResult;
    }

    #endif /* ADI_DEBUG */
     pDevice->pCounterRegs->COUNTER_CONFIG &=(uint16_t)~(ENUM_CNT_CFG_DEBEN);
    if(bEnable == true)
    {
      pDevice->pCounterRegs->COUNTER_CONFIG |=(uint16_t)(ENUM_CNT_CFG_DEBEN);
    }
    return(ADI_COUNTER_SUCCESS);
}
#ifdef ADI_DEBUG
static ADI_COUNTER_RESULT ValidateHandle(
                                           ADI_COUNTER_DEVICE *pInDevice
                                         )
{
    /* Return code */
    ADI_COUNTER_RESULT    nResult = ADI_COUNTER_INVALID_HANDLE;
    /* Pointer to counter device instance */
    ADI_COUNTER_DEVICE_INFO *poDeviceInfo = &counter_device_info[0];
    uint32_t i;
    for(i=0U; i< (ADI_NUM_COUNTER_DEVICE_INSTANCES ); i++)
    {
        if(pInDevice == poDeviceInfo->hDevice)
        {
            return(ADI_COUNTER_SUCCESS);
        }
        poDeviceInfo++;
    }
    return (nResult);
}

#endif /* ADI_DEBUG */

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif
