/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*******************************************************************************
 *
 * @file:    adi_sinc.c
 * @brief:   SINC device driver implementation
 * @version: $Revision: 22295 $
 * @date:    $Date: 2013-07-23 16:00:57 -0400 (Tue, 23 Jul 2013) $
 *
 ******************************************************************************/

#include <stdbool.h>
#include <stddef.h>
#include <string.h>
#include <assert.h>
#include <sys/platform.h>
#include <services/int/adi_int.h>
#include <adi_osal.h>
#include <drivers/sinc/adi_sinc.h>
#include "adi_sinc_def.h"

#define ENUM_SINC_PAIR_DISABLED  (0x0u)      /* Disable pair assignment. */
#define ENUM_SINC_PAIR_GROUP_0   (0x2u)      /* Assign pair to group 0. */
#define ENUM_SINC_PAIR_GROUP_1   (0x3u)      /* Assign pair to group 1. */

#define BYTES_PER_SAMPLE         (2u)

#ifdef _MISRA_RULES
#pragma diag(push)
#endif /* _MISRA_RULES */

/** @addtogroup SINC_Driver SINC Filter Driver
 *  @{
 *  
 *  The SINC module will process four bitstreams using a pair of user configurable
 *  SINC filters for each bitstream.
 *  
 *  The purpose of the primary SINC filter of each pair is to produce the filtered & 
 *  decimated output for the pair.
 *  
 *  The purpose of the secondary SINC filter of each pair is to rapidly detect overload 
 *  situations.
 *  
 * Typical SINC Module Initiation Sequence:
 * -#  Specify Group 0 Parameters
 * -#  If required, Specify Group 1 Parameters 
 * -#  Reset overload amplitude detection limits 0 to FullScale (to avoid trip by startup transient)
 * -#  Specify Modulator Clock frequency, phase & startup synchronization (i.e., perhaps wait for PWM)
 * -#  Enable & assign used SINC filter pairs to either Group 0 or Group 1, and specify interrupt masks
 * -#  Specify overload amplitude detection limits
 * -#  While the filters are enabled, it is permissible to change the following:
 *     - Modulator Clock fine phase adjustments MADJx and MREQx in the CLK register
 *     - SINC Filter coarse phase adjust PADJx and SADJx in the RATEx register
 *     - Secondary limit detection registers LIMITx
 *     - Write-1-clear of any interrupt or status bit in the STAT register
 *     - Any interrupt enables in the CTL register
 * -#  While any filter in the group is enabled, do not change any of the following, or the results will be UNPREDICTABLE:
 *     - SINC filter group assignments in the CTL register (except when disabling all the filters in the group)
 *     - Modulator Clock enables MCENx or frequencies MDIVx in the CLK register
 *     - SINC Filter decimation ratios SDECx or PDECx in the RATEx registers 
 *     - SINC filter order SORDx or PORDx in the LEVELx register
 *     - Primary filter output bias registers BIASx 
 *     - Filter output scaling PSCALEx, interrupt rates PCNTx or limit windows LCNTx, LWINx  in the LEVELx register
 *     - DMA buffer addresses PHEADx and PTAILx
 *
 */

/* the device information array */
ADI_SINC_DEVICE gSincDevices[] = 
{
    {   
        ADI_SINC_STATE_NOT_OPENED,                          /* SINC device state */
        (ADI_SINC_REG *) REG_SINC0_CTL,                     /* Pointer to SINC registers */
        (uint32_t)ADI_INT_SINC0_STAT,                       /* SINC status interrupt ID */
        NULL,                                               /* SINC device data */
     }
};

/*
 * API's
 */

/**
 * @brief       Open a SINC device instance.
 *
 *  If successful the returned ADI_SINC_HANDLE parameter will hold a device handle
 *  to be used as an input to other SINC API functions.  The SINC device must be
 *  opened before using any other SINC API functions.
 *
 *  The amount of memory required by the driver is specified by the ADI_SINC_MEMORY_SIZE
 *  definition.
 *
 * @param [in]  nDeviceNum          The SINC device instance number to be opened.
 * @param [in]  pMemory             Application supplied memory space for use by the driver.
 * @param [in]  MemorySize          Application supplied memory size (bytes).
 * @param [out] phDevice            A pointer to a location where the handle to the
 *                                  opened device is written.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully opened the given instance.
 *              - #ADI_SINC_INVALID_DEVICE_NUM  [D] The device number is invalid.
 *              - #ADI_SINC_NULL_POINTER        [D] A NULL pointer parameter error. 
 *              - #ADI_SINC_INVALID_PARAMETER   [D] The application supplied memory size is insufficient.
 *              - #ADI_SINC_DEVICE_IN_USE           The SINC device is already open.
 *              - #ADI_SINC_INT_FAILED              The interrupt install failed.
 *              - #ADI_SINC_SEM_FAILED              The semaphore create failed.
 *
 *
 * @sa          adi_sinc_Close()
 */
ADI_SINC_RESULT  adi_sinc_Open (
    uint32_t             const  nDeviceNum,
    void                 const *pMemory,
    uint32_t             const  MemorySize,
    ADI_SINC_HANDLE            *phDevice)
{
    /* Pointer to the device data*/
    ADI_SINC_DEVICE *pDevice = NULL;

    /* Pointer to the SINC registers */
    volatile ADI_SINC_REG  *pSincRegs;

#ifdef ADI_DEBUG
    /* Check if the given device number is valid */
    if(nDeviceNum > ADI_SINC_NUM_DEVICES)
    {
        return ADI_SINC_INVALID_DEVICE_NUM;
    }

    /* Check if the given pointer are valid */
    if(pMemory == NULL || phDevice == NULL)
    {
        return ADI_SINC_NULL_POINTER;
    }

    /* Check if device is already opened */
    if(gSincDevices[nDeviceNum].eSincState != ADI_SINC_STATE_NOT_OPENED)
    {
        return ADI_SINC_DEVICE_IN_USE;
    }

    /* Assert to validate the memory sizes */
    assert(ADI_SINC_MEMORY_SIZE  ==  sizeof(ADI_SINC_DEV_DATA));

  	if (MemorySize < (uint32_t)ADI_SINC_MEMORY_SIZE)
	{
		return ADI_SINC_INVALID_PARAMETER;
	}
#endif /* ADI_DEBUG */ 
   
    /* clear the given memory */
    memset((void *)pMemory, 0, ADI_SINC_MEMORY_SIZE);

    pDevice    =   &gSincDevices[nDeviceNum];
    pSincRegs  =   pDevice->pSincRegs;

    /* assign application memory to the device */
    pDevice->pSincData = (ADI_SINC_DEV_DATA*)pMemory;
    
    /* Create the group0 semaphore */
    if(adi_osal_SemCreateStatic(pDevice->pSincData->Group0.SemMemory,
                                ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                &pDevice->pSincData->Group0.hSemaphore, 0u) != ADI_OSAL_SUCCESS)
    {
        return ADI_SINC_SEM_FAILED;
    }

    /* Create the group1 semaphore */
    if(adi_osal_SemCreateStatic(pDevice->pSincData->Group1.SemMemory,
                                ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                &pDevice->pSincData->Group1.hSemaphore, 0u) != ADI_OSAL_SUCCESS)
    {
        return ADI_SINC_SEM_FAILED;
    }

    /*
     * Set the SINC registers to default values
     */

    pSincRegs->CTL          =   0ul;            /* SINC Control register                */
    pSincRegs->STAT         =   0xFFFFFFFFul;   /* SINC0 Status bits W1C */
    pSincRegs->CLK          =   0x10001000ul;   /* SINC0 Modulator Clock Control */
    pSincRegs->RATE0        =   0x00000804ul;   /* SINC0 Rate Control for Group 0 */
    pSincRegs->RATE1        =   0x00000804ul;   /* SINC0 Rate Control for Group 1 */
    pSincRegs->LEVEL0       =   0x04000000ul;   /* SINC0 Level Control for Group 0 */
    pSincRegs->LEVEL1       =   0x04000000ul;   /* SINC0 Level Control for Group 1 */
    pSincRegs->LIMIT0       =   0ul;            /* SINC0 Amplitude Limits for Secondary SINC of Pair 0 */
    pSincRegs->LIMIT1       =   0ul;            /* SINC0 Amplitude Limits for Secondary SINC of Pair 1 */
    pSincRegs->LIMIT2       =   0ul;            /* SINC0 Amplitude Limits for Secondary SINC of Pair 2 */
    pSincRegs->LIMIT3       =   0ul;            /* SINC0 Amplitude Limits for Secondary SINC of Pair 3 */
    pSincRegs->BIAS0        =   0ul;            /* SINC0 Primary SINC Output Bias for Group 0 */
    pSincRegs->BIAS1        =   0ul;            /* SINC0 Primary SINC Output Bias for Group 1 */
    pSincRegs->PHEAD0       =   0ul;            /* SINC0 Primary SINC Circular Buffer Head for Group 0 */
    pSincRegs->PHEAD1       =   0ul;            /* SINC0 Primary SINC Circular Buffer Head for Group 1 */
    pSincRegs->PTAIL0       =   0ul;            /* SINC0 Primary SINC Circular Buffer Tail for Group 0 */
    pSincRegs->PTAIL1       =   0ul;            /* SINC0 Primary SINC Circular Buffer Tail for Group 1 */

    /* Install but don't enable the status interrupt handler */
    if(adi_int_InstallHandler(pDevice->nStatusIntID,
                              StatusIntHandler,
                              pDevice,
                              false) != ADI_INT_SUCCESS)
    {
        return ADI_SINC_INT_FAILED;
    }

    /* Mark the device as in use */
    adi_osal_EnterCriticalRegion();
    pDevice->eSincState =   ADI_SINC_STATE_OPENED;
    adi_osal_ExitCriticalRegion();

    /* Save the given memory pointer */
    pDevice->pSincData  =   (ADI_SINC_DEV_DATA *) pMemory;

    /* set the active buffer pointer */
    pDevice->pSincData->Group0.pActiveBuf = &pDevice->pSincData->Group0.BufferInfo0;
    pDevice->pSincData->Group1.pActiveBuf = &pDevice->pSincData->Group1.BufferInfo0;
    
    /* Return the device handle */
    *phDevice = (ADI_SINC_HANDLE) pDevice;

    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Close the given SINC device.
 *
 * @param [in]  hDevice  The handle to the SINC device.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                Successfully closed the given instance.
 *              - #ADI_SINC_INVALID_HANDLE     [D] The device handle is invalid.
 *              - #ADI_SINC_INT_FAILED             The interrupt uninstall failed.
 *              - #ADI_SINC_SEM_FAILED             The semaphore destroy failed.
 *
 * @sa          adi_sinc_Open()
 */
ADI_SINC_RESULT  adi_sinc_Close (
   ADI_SINC_HANDLE const        hDevice)
{
      /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;

    /* Result code */
    ADI_SINC_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* disable device and mask interrupts */
    pDevice->pSincRegs->CTL = 0ul;

    /* Un-install the status interrupt handler */
    if(adi_int_UninstallHandler(pDevice->nStatusIntID) != ADI_INT_SUCCESS)
    {
        return ADI_SINC_INT_FAILED;
    }

    if(pDevice->pSincData->Group0.hSemaphore != NULL)
    {
        if(adi_osal_SemDestroyStatic(pDevice->pSincData->Group0.hSemaphore) != ADI_OSAL_SUCCESS)
        {
            return ADI_SINC_SEM_FAILED;
        }
    }

    if(pDevice->pSincData->Group1.hSemaphore != NULL)
    {
        if(adi_osal_SemDestroyStatic(pDevice->pSincData->Group1.hSemaphore) != ADI_OSAL_SUCCESS)
        {
            return ADI_SINC_SEM_FAILED;
        }
    }

    /* Mark the device as not in use */
    pDevice->eSincState = ADI_SINC_STATE_NOT_OPENED;

    /* Clear the device data pointer */
    pDevice->pSincData  = NULL;

    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Register/Unregister an application defined status callback function.
 *
 * An application defined callback function is used to notify the application of
 * SINC status events.
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  pfCallback          Application supplied callback function which is
 *                                  called to notify SINC status events.  A value of
 *                                  NULL disables application callbacks.
 * @param [in]  pCBParam            Application supplied callback parameter which
 *                                  will be passed back in the callback function.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the application defined callback function.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_OPERATION   [D] The device state is enabled.
 *              - #ADI_SINC_INT_FAILED              Failed to enable the status interrupt.
 *
 * @sa  #ADI_SINC_EVENT
 */
ADI_SINC_RESULT  adi_sinc_RegisterStatusCallback (
    ADI_SINC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam)
{
    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;

    /* Result code */
    ADI_SINC_RESULT     eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }

    /* Check if the device is enabled */
    if(pDevice->eSincState == ADI_SINC_STATE_ENABLED)
    {
        return ADI_SINC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Save the given callback and callback parameter */
    adi_osal_EnterCriticalRegion();
    pDevice->pSincData->pfStatusCallback  =   pfCallback;
    pDevice->pSincData->pStatusCBParam    =   pCBParam;
    adi_osal_ExitCriticalRegion();

    if(adi_int_EnableInt(pDevice->nStatusIntID, true))
    {
        return ADI_SINC_INT_FAILED;
    }

    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Register/Unregister an application defined data callback function.
 *
 * An application defined callback function is used to notify the application of
 * SINC data events.  If more than one pair is enabled, the data is interleaved.
 * The order of interleave will be from lowest numbered to highest numbered SINC filter.
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  pfCallback          Application supplied callback function which is
 *                                  called to notify SINC data events.  A value of
 *                                  NULL disables application callbacks.
 * @param [in]  pCBParam            Application supplied callback parameter which
 *                                  will be passed back in the callback function.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the application defined callback function.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_OPERATION   [D] The device state is enabled.
 *
 * @sa  #ADI_SINC_EVENT
 */
ADI_SINC_RESULT  adi_sinc_RegisterDataCallback (
    ADI_SINC_HANDLE const       hDevice,
    ADI_CALLBACK                pfCallback,
    void *                      pCBParam)
{
    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;

    /* Result code */
    ADI_SINC_RESULT     eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }

    /* Check if the device is enabled */
    if(pDevice->eSincState == ADI_SINC_STATE_ENABLED)
    {
        return ADI_SINC_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Save the given callback and callback parameter */
    adi_osal_EnterCriticalRegion();
    pDevice->pSincData->pfDataCallback  =   pfCallback;
    pDevice->pSincData->pDataCBParam    =   pCBParam;
    adi_osal_ExitCriticalRegion();

    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Enable/Disable the data interrupt event.
 *
 * Routes the SINC TRU master trigger to a core interrupt when data is available.
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  eGroup              The pair group data interrupt.
 * @param [in]  eDataInt            The TRU core interrupt to use.
 * @param [in]  bEnable             True to enable the interrupt, false to disable.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the application defined callback function.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_OPERATION   [D] The device state is enabled.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *              - #ADI_SINC_INT_FAILED              Failed to install/uninstall the data interrupt.
 *
 * @sa  #ADI_SINC_DATA_INT
 */
ADI_SINC_RESULT  adi_sinc_EnableDataInterrupt (
    ADI_SINC_HANDLE     const   hDevice,
    ADI_SINC_GROUP      const   eGroup,
    ADI_SINC_DATA_INT   const   eDataInt,
    bool                const   bEnable)
{
    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;

    /* Result code */
    ADI_SINC_RESULT         eResult;
    
    uint32_t  trigger;
    ADI_INT_HANDLER_PTR pfHandler;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
    
    if (eDataInt > ADI_SINC_DATA_INT_3)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (bEnable)
    {
        if (eGroup == ADI_SINC_GROUP_0)
        {
            pfHandler = Data0IntHandler;
            trigger = TRGM_SINC0_DATA0;
        }
        else
        {
            pfHandler = Data1IntHandler;
            trigger = TRGM_SINC0_DATA1;
        }            
        
        /* Install the data interrupt handler */
        if(adi_int_InstallHandler(eDataInt,
                              pfHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
        {
            return ADI_SINC_INT_FAILED;
        }

        if (eDataInt == ADI_SINC_DATA_INT_0)
        {
            *pREG_TRU0_SSR12 = trigger;  /* TRU0 IRQ 0 */
        } 
        else if (eDataInt == ADI_SINC_DATA_INT_1)
        {
            *pREG_TRU0_SSR13 = trigger;  /* TRU0 IRQ 1 */
        }
        else if (eDataInt == ADI_SINC_DATA_INT_2)
        {
            *pREG_TRU0_SSR14 = trigger;  /* TRU0 IRQ 2 */
        } 
        else 
        {
            *pREG_TRU0_SSR15 = trigger;  /* TRU0 IRQ 3 */
        }
        
        *pREG_TRU0_GCTL |= 1;
    }
    else
    {
        /* UnInstall the data interrupt handler */
        if(adi_int_UninstallHandler(eDataInt))
        {
            return ADI_SINC_INT_FAILED;
        }

        trigger = 0u;
        
        if (eDataInt == ADI_SINC_DATA_INT_0)
        {
            *pREG_TRU0_SSR12 = trigger;  /* TRU0 IRQ 0 */
        } 
        else if (eDataInt == ADI_SINC_DATA_INT_1)
        {
            *pREG_TRU0_SSR13 = trigger;  /* TRU0 IRQ 1 */
        }
        else if (eDataInt == ADI_SINC_DATA_INT_2)
        {
            *pREG_TRU0_SSR14 = trigger;  /* TRU0 IRQ 2 */
        } 
        else 
        {
            *pREG_TRU0_SSR15 = trigger;  /* TRU0 IRQ 3 */
        }
    }

    return ADI_SINC_SUCCESS;
}

/*
 * Control
 */

/**
 * @brief       Assigns the pair to a group and enable/disable the SINC pair.
 *
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  ePair               The SINC pair to assign and enable.
 * @param [in]  eGroup              The pair group assignment and enable selection.
 * @param [in]  bEnable             True to assign and enable the pair.  False to disable.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully assigned the pair.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_SINC_RESULT  adi_sinc_EnablePair(
    ADI_SINC_HANDLE       const     hDevice,
    ADI_SINC_PAIR         const     ePair,
    ADI_SINC_GROUP        const     eGroup,
    bool                  const     bEnable)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;

    uint32_t mask = 0u;
    uint32_t pos  = 0u;
    uint32_t reg  = 0u;
    uint8_t  value;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }

    if (ePair > ADI_SINC_PAIR_3)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (ePair == ADI_SINC_PAIR_0)
    {
        mask = BITM_SINC_CTL_EN0;
        pos  = BITP_SINC_CTL_EN0;
    }
    else if (ePair == ADI_SINC_PAIR_1)
    {
        mask = BITM_SINC_CTL_EN1;
        pos  = BITP_SINC_CTL_EN1;
    }
    else if (ePair == ADI_SINC_PAIR_2)
    {
        mask = BITM_SINC_CTL_EN2;
        pos  = BITP_SINC_CTL_EN2;
    }
    else
    {
        mask = BITM_SINC_CTL_EN3;
        pos  = BITP_SINC_CTL_EN3;
    }

    if (bEnable == false)
    {
        value = ENUM_SINC_PAIR_DISABLED;
    }
    else if (eGroup == ADI_SINC_GROUP_0)
    {
        value = ENUM_SINC_PAIR_GROUP_0;
    }
    else
    {
        value = ENUM_SINC_PAIR_GROUP_1;
    }

    adi_osal_EnterCriticalRegion();

    reg = pDevice->pSincRegs->CTL;
    
    /* clear the control value */
    reg &= (uint32_t)~mask;
    
    reg |= value << pos;
    
    pDevice->pSincRegs->CTL = reg;
        
    adi_osal_ExitCriticalRegion();
    
    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Sets the control interrupt mask.
 *
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  nControlIntMask     The SINC interrupts to enable.  Multiple
 *                                  interrupt enums can be OR'ed together (i.e.
 *                                  ADI_SINC_INT_EPCNT0|ADI_SINC_INT_ESAT0)
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the control interrupt mask.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *
 * @sa  #ADI_SINC_INT_MASK
 */
ADI_SINC_RESULT  adi_sinc_SetControlIntMask (
    ADI_SINC_HANDLE const       hDevice,
    uint32_t        const       nControlIntMask)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }    
#endif /* ADI_DEBUG */

    mask = ADI_SINC_INT_EFOVF0 | ADI_SINC_INT_EPCNT0 | ADI_SINC_INT_ESAT0
          | ADI_SINC_INT_ELIM0 | ADI_SINC_INT_EFOVF1 | ADI_SINC_INT_EPCNT1
          | ADI_SINC_INT_ESAT1 | ADI_SINC_INT_ELIM1;

    adi_osal_EnterCriticalRegion();

    reg = pDevice->pSincRegs->CTL;
        
    /* clear all interrupts */
    reg &= (uint32_t)~mask;
    
    /* enable given interrupts */
    reg |= nControlIntMask;
        
    pDevice->pSincRegs->CTL = reg;
        
    adi_osal_ExitCriticalRegion();
    
    return ADI_SINC_SUCCESS;
}

/*
 * Modulator clock
 */

/**
 * @brief       Sets the modulator clock controls.
 *
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  eGroup              The SINC group clock control to set.
 * @param [in]  nSysClk             The system clock frequency (Hz)
 * @param [in]  nModClockFreq       The desired modulator clock frequency (Hz)
 * @param [in]  nClockAdjust        The modulator clock phase adjust (ns). The 
 *                                  granularity is based on the modulator clock 
 *                                  frequency (e.g. This equates to 100ns increments 
 *                                  with a 10MHz modulator clock).
 * @param [in]  bShiftRequest       Modulator clock phase shift request/status.
 *                                  When true, the modulator clock period will be 
 *                                  lengthened by the clock phase adjust value. 
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the modulator clock controls.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_EnableModClock()
 */
ADI_SINC_RESULT  adi_sinc_ConfigModClock (
    ADI_SINC_HANDLE const        hDevice,
    ADI_SINC_GROUP  const        eGroup,
    uint32_t        const        nSysClk,
    uint32_t        const        nModClockFreq,
    uint32_t        const        nClockAdjust,
    bool            const        bShiftRequest)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;
    uint32_t mdiv;
    uint32_t madj;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
    
    if (nModClockFreq == 0u)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    /* modulator clock divide */
    mdiv = nSysClk / nModClockFreq;
    
    /* phase adjust */
    madj = 0u;
    if (nClockAdjust > 0u)
    {
        madj = nModClockFreq / (1000000000u / nClockAdjust);
    }
#ifdef ADI_DEBUG
    if (mdiv < 4u || mdiv > 63u)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
    if (madj > 63u)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */
    
    adi_osal_EnterCriticalRegion();

    reg = pDevice->pSincRegs->CLK;
        
    if (eGroup == ADI_SINC_GROUP_0)
    {
        mask = BITM_SINC_CLK_MADJ0 | BITM_SINC_CLK_MREQ0 | BITM_SINC_CLK_MDIV0;
        
        /* clear group settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= madj << BITP_SINC_CLK_MADJ0;
        reg |= bShiftRequest << BITP_SINC_CLK_MREQ0;
        reg |= mdiv << BITP_SINC_CLK_MDIV0;
    } else {
        mask = BITM_SINC_CLK_MADJ1 | BITM_SINC_CLK_MREQ1 | BITM_SINC_CLK_MDIV1;

        /* clear group settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= madj << BITP_SINC_CLK_MADJ1;
        reg |= bShiftRequest << BITP_SINC_CLK_MREQ1;
        reg |= mdiv << BITP_SINC_CLK_MDIV1;
    }

    pDevice->pSincRegs->CLK = reg;
        
    adi_osal_ExitCriticalRegion();
    
        return ADI_SINC_SUCCESS;
}

/**
 * @brief       Enable/Disable the modulator clock.
 *
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  eGroup              The SINC group clock control to enable.
 * @param [in]  eEnable             The enable selection. 
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully enabled the modulator clock.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_ConfigModClock()
 */
ADI_SINC_RESULT  adi_sinc_EnableModClock (
    ADI_SINC_HANDLE       const      hDevice,
    ADI_SINC_GROUP        const      eGroup,
    ADI_SINC_MOD_CLK      const      eEnable)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();

    reg = pDevice->pSincRegs->CLK;
        
    if (eGroup == ADI_SINC_GROUP_0)
    {
        mask = BITM_SINC_CLK_MCEN0;
        
        /* clear enable */
        reg &= (uint32_t)~mask;
    
        /* set enable */
        reg |= eEnable << BITP_SINC_CLK_MCEN0;
        
    } else {
        mask = BITM_SINC_CLK_MCEN1;

        /* clear enable */
        reg &= (uint32_t)~mask;
    
        /* set enable */
        reg |= eEnable << BITP_SINC_CLK_MCEN1;
    }

    pDevice->pSincRegs->CLK = reg;
        
    adi_osal_ExitCriticalRegion();
    
    return ADI_SINC_SUCCESS;
}

/*
 * Rate control
 */

/**
 * @brief       Sets the rate control values.
 *
 * @note        The bias value is automatically calculated and set when 
 *              this API is called.
 *
 *
 * @param [in]  hDevice             Handle to the SINC device.
 * @param [in]  eGroup              The SINC group rate control values to set.
 * @param [in]  eFilter             Primary or secondary filter selection.
 * @param [in]  nDecimationRate     The filter output decimation Rate.
 *                                  Primary filter permitted range is 256 to 4.
 *                                  Secondary filter:
 *                                    3rd order permitted range is 4 to 40
 *                                    4th order permitted range is 4 to 16
 * @param [in]  nPhaseAdjust        Filter output phase adjust.
 *                                  Permitted range is 0 to value of nDecimationRate-1.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the rate control values.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 */
ADI_SINC_RESULT  adi_sinc_SetRateControl (
    ADI_SINC_HANDLE   const     hDevice,
    ADI_SINC_GROUP    const     eGroup,
    ADI_SINC_FILTER   const     eFilter,
    uint16_t          const     nDecimationRate,
    uint16_t          const     nPhaseAdjust)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if (eFilter > ADI_SINC_FILTER_SECONDARY)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
    
    if (nDecimationRate > 0x1FFu)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if (nPhaseAdjust > 0x1FFu)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();
        
    if (eGroup == ADI_SINC_GROUP_0)
    {
        reg = pDevice->pSincRegs->RATE0;

        if (eFilter == ADI_SINC_FILTER_PRIMARY)
        {
            mask = BITM_SINC_RATE0_PDEC | BITM_SINC_RATE0_PADJ;

            /* clear rate settings */
            reg &= (uint32_t)~mask;
    
            /* config */
            reg |= nDecimationRate << BITP_SINC_RATE0_PDEC;
            reg |= nPhaseAdjust << BITP_SINC_RATE0_PADJ;
        } else {
            mask = BITM_SINC_RATE0_SDEC | BITM_SINC_RATE0_SADJ;

            /* clear rate settings */
            reg &= (uint32_t)~mask;
    
            /* config */
            reg |= nDecimationRate << BITP_SINC_RATE0_SDEC;
            reg |= nPhaseAdjust << BITP_SINC_RATE0_SADJ;
        }

        pDevice->pSincRegs->RATE0 = reg;

    } else {
        reg = pDevice->pSincRegs->RATE1;

        if (eFilter == ADI_SINC_FILTER_PRIMARY)
        {
            mask = BITM_SINC_RATE1_PDEC | BITM_SINC_RATE1_PADJ;

            /* clear rate settings */
            reg &= (uint32_t)~mask;
    
            /* config */
            reg |= nDecimationRate << BITP_SINC_RATE1_PDEC;
            reg |= nPhaseAdjust << BITP_SINC_RATE1_PADJ;
        } else {
            mask = BITM_SINC_RATE1_SDEC | BITM_SINC_RATE1_SADJ;

            /* clear rate settings */
            reg &= (uint32_t)~mask;
    
            /* config */
            reg |= nDecimationRate << BITP_SINC_RATE1_SDEC;
            reg |= nPhaseAdjust << BITP_SINC_RATE1_SADJ;
        }

        pDevice->pSincRegs->RATE1 = reg;
    }
        
    adi_osal_ExitCriticalRegion();
    
    eResult = CalcOutputBias (hDevice, eGroup);

    return ADI_SINC_SUCCESS;
}

/*
 * Level control
 */

/**
 * @brief       Sets the filter order.
 *
 * @note        The bias value is automatically calculated and set when 
 *              this API is called.
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  eGroup                The SINC group filter order to set.
 * @param [in]  ePrimaryFilterOrder   The primary filter order for this group.
 * @param [in]  eSecondaryFilterOrder The secondary filter order for this group.
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the filter order.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_SetLevelControl()
 *
 */
ADI_SINC_RESULT  adi_sinc_SetFilterOrder (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    ADI_SINC_FILTER_ORDER  const     ePrimaryFilterOrder,
    ADI_SINC_FILTER_ORDER  const     eSecondaryFilterOrder)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if (ePrimaryFilterOrder > ADI_SINC_FILTER_FORTH_ORDER)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
    
    if (eSecondaryFilterOrder > ADI_SINC_FILTER_FORTH_ORDER)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();
        
    if (eGroup == ADI_SINC_GROUP_0)
    {
        reg = pDevice->pSincRegs->LEVEL0;

        mask = BITM_SINC_LEVEL0_SORD | BITM_SINC_LEVEL0_PORD;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= eSecondaryFilterOrder << BITP_SINC_LEVEL0_SORD;
        reg |= ePrimaryFilterOrder << BITP_SINC_LEVEL0_PORD;

        pDevice->pSincRegs->LEVEL0 = reg;
    } else {
        reg = pDevice->pSincRegs->LEVEL1;

        mask = BITM_SINC_LEVEL1_SORD | BITM_SINC_LEVEL1_PORD;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= eSecondaryFilterOrder << BITP_SINC_LEVEL1_SORD;
        reg |= ePrimaryFilterOrder << BITP_SINC_LEVEL1_PORD;

        pDevice->pSincRegs->LEVEL1 = reg;
    }
        
    adi_osal_ExitCriticalRegion();
    
    eResult = CalcOutputBias (hDevice, eGroup);

    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Sets the level control values.
 *
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  eGroup                The SINC group level control to set.
 * @param [in]  nLimitWindow          The window size for excursion checking.
 *                                    Value from 0 to 7 may be specified.
 * @param [in]  nLimitCount           The excursion limit count. Value range from 
 *                                    0 to 7 may be specified.
 *                                    0 = Require 1 excursion above amplitude limit
 *                                    7 = Require 8 excursions above amplitude limit.
 * @param [in]  nPrimaryIntRate       Primary SINC filter interrupt rate.
 *                                    Value must be one less than the desired modulo.
 *                                    0 = PCNT0 in register STATUS set to 1 after 
 *                                        every primary SINC filter output transferred.
 *                                    255 = PCNT0 in register STATUS set to 1 after 
 *                                        every 256 Primary SINC filter outputs transferred.
 * @param [in]  nPrimaryOutScale      Specifies the scaling applied to the Primary SINC output 
 *                                    prior to DMA transfer to memory. Permitted range is 4 to 32. 
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the level control values.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_SetFilterOrder()
 *
 */
ADI_SINC_RESULT  adi_sinc_SetLevelControl (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    uint8_t                const     nLimitWindow,
    uint8_t                const     nLimitCount,
    uint8_t                const     nPrimaryIntRate,
    uint8_t                const     nPrimaryOutScale)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if (nLimitWindow > 0x7u)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if (nLimitCount > 0x7u)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if (nPrimaryOutScale > 0x3Fu)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();
        
    if (eGroup == ADI_SINC_GROUP_0)
    {
        reg = pDevice->pSincRegs->LEVEL0;

        mask = BITM_SINC_LEVEL0_LWIN | BITM_SINC_LEVEL0_LCNT | 
               BITM_SINC_LEVEL0_PCNT | BITM_SINC_LEVEL0_PSCALE;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= nLimitWindow << BITP_SINC_LEVEL0_LWIN;
        reg |= nLimitCount << BITP_SINC_LEVEL0_LCNT;
        reg |= (nPrimaryIntRate-1u) << BITP_SINC_LEVEL0_PCNT; /* write PCNT-1 to get the correct interrupt rate */
        reg |= nPrimaryOutScale << BITP_SINC_LEVEL0_PSCALE;

        pDevice->pSincRegs->LEVEL0 = reg;
    } else {
        reg = pDevice->pSincRegs->LEVEL1;

        mask = BITM_SINC_LEVEL1_LWIN | BITM_SINC_LEVEL1_LCNT | 
               BITM_SINC_LEVEL1_PCNT | BITM_SINC_LEVEL1_PSCALE;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= nLimitWindow << BITP_SINC_LEVEL1_LWIN;
        reg |= nLimitCount << BITP_SINC_LEVEL1_LCNT;
        reg |= (nPrimaryIntRate-1u) << BITP_SINC_LEVEL1_PCNT; /* write PCNT-1 to get the correct interrupt rate */
        reg |= nPrimaryOutScale << BITP_SINC_LEVEL1_PSCALE;

        pDevice->pSincRegs->LEVEL1 = reg;
    }
        
    adi_osal_ExitCriticalRegion();
    
    return ADI_SINC_SUCCESS;
}

/*
 * Amplitude limits
 */

/**
 * @brief       Sets the amplitude limits.
 *
 * In conjunction with bits LCNT and LWIN in the LEVEL register, these bits 
 * specify the SINC filter output signal conditions required to set an associated 
 * minimum and maximum limit warning bit in STATUS register.
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  ePair                 The SINC pair to set the amplitude limits.
 * @param [in]  nMinAmplitude         The minimum amplitude limit.
 * @param [in]  nMaxAmplitude         The maximum amplitude limit.
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the amplitude limits.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_SetLevelControl()
 *
 */
ADI_SINC_RESULT  adi_sinc_SetAmplitudeLimit (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_PAIR          const     ePair,
    uint16_t               const     nMinAmplitude,
    uint16_t               const     nMaxAmplitude)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t mask  = 0u;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (ePair > ADI_SINC_PAIR_3)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();
        
    if (ePair == ADI_SINC_PAIR_0)
    {
        reg = pDevice->pSincRegs->LIMIT0;

        mask = BITM_SINC_LIMIT0_LMIN | BITM_SINC_LIMIT0_LMAX;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= nMinAmplitude << BITP_SINC_LIMIT0_LMIN;
        reg |= nMaxAmplitude << BITP_SINC_LIMIT0_LMAX;

        pDevice->pSincRegs->LIMIT0 = reg;
    } else if (ePair == ADI_SINC_PAIR_1) {
        reg = pDevice->pSincRegs->LIMIT1;

        mask = BITM_SINC_LIMIT1_LMIN | BITM_SINC_LIMIT1_LMAX;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= nMinAmplitude << BITP_SINC_LIMIT1_LMIN;
        reg |= nMaxAmplitude << BITP_SINC_LIMIT1_LMAX;

        pDevice->pSincRegs->LIMIT1 = reg;
    } else if (ePair == ADI_SINC_PAIR_2) {
        reg = pDevice->pSincRegs->LIMIT2;

        mask = BITM_SINC_LIMIT2_LMIN | BITM_SINC_LIMIT2_LMAX;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= nMinAmplitude << BITP_SINC_LIMIT2_LMIN;
        reg |= nMaxAmplitude << BITP_SINC_LIMIT2_LMAX;

        pDevice->pSincRegs->LIMIT2 = reg;
    } else {
        reg = pDevice->pSincRegs->LIMIT3;

        mask = BITM_SINC_LIMIT3_LMIN | BITM_SINC_LIMIT3_LMAX;

        /* clear rate settings */
        reg &= (uint32_t)~mask;
    
        /* config */
        reg |= nMinAmplitude << BITP_SINC_LIMIT3_LMIN;
        reg |= nMaxAmplitude << BITP_SINC_LIMIT3_LMAX;

        pDevice->pSincRegs->LIMIT3 = reg;
    }
    
    adi_osal_ExitCriticalRegion();
    
    return ADI_SINC_SUCCESS;
}

/*
 * Output bias
 */

/**
 * @brief       Sets the group output bias.
 *
 * This value specifies a bias which will be added to the Primary SINC filter 
 * output prior to saturation and DMA transfer to memory.  For this number to 
 * be represented in two's complement format, this register must be programmed 
 * to a value equal to -(d^o)/2 where d is the decimation rate and o is the order 
 * selected for the Primary SINC filter.
 * 
 * @note        The bias value is automatically calculated and set when the 
 *              adi_sinc_SetFilterOrder() or adi_sinc_SetRateControl() API's are called.
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  eGroup                The SINC group bias to set.
 * @param [in]  nBias                 The primary SINC output bias.
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the output bias.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_SetFilterOrder()
 * @sa  #adi_sinc_SetRateControl()
 *
 */
ADI_SINC_RESULT  adi_sinc_SetOutputBias (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    uint32_t               const     nBias)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (eGroup == ADI_SINC_GROUP_0)
    {
        pDevice->pSincRegs->BIAS0 = nBias;
    } else {
        pDevice->pSincRegs->BIAS1 = nBias;
    }
    
    return ADI_SINC_SUCCESS;
}

/*
 * Circular buffer pointer
 */

/**
 * @brief       Gets the primary circular buffer pointer.
 *
 * The pointer identifies the final location in memory of the most recent set of 
 * Primary SINC filter results.  The location is incremented once all Primary SINC 
 * filter data assigned to the group and associated to a particular timestamp have 
 * been successfully presented to the system fabric.
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  eGroup                The SINC group circular buffer pointer to get.
 * @param [out] pAddress              The primary SINC result location.
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully got the circular buffer pointer.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_SetCircBuffer()
 *
 */
ADI_SINC_RESULT  adi_sinc_GetResultLocation (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    uint32_t                        *pAddress)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
        
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (eGroup == ADI_SINC_GROUP_0)
    {
        *pAddress = pDevice->pSincRegs->PPTR0;
    } 
    else 
    {
        *pAddress = pDevice->pSincRegs->PPTR1;
    }
        
    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Sets the primary circular buffer used to receive SINC data.
 * 
 * Outputs from Primary SINC filter will be transferred to memory commencing at 
 * PHEAD and wrap back to PHEAD after PTAIL is reached. The address must be 16-bit aligned.
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  eGroup                The SINC group circular buffer head address to set.
 * @param [in]  *pBuffer              The buffer to receive the SINC data.
 * @param [in]  nBufSize              The size of the receive buffer (bytes).
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the circular buffer address.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_GetResultLocation()
 *
 */
ADI_SINC_RESULT adi_sinc_SetCircBuffer(
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    void                            *pBuffer,
    uint32_t               const     nBufSize)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t  head = (uint32_t)pBuffer;
    uint32_t  tail = head + 2u * (nBufSize-1); /* 16-bit buffer */
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }

    if ((head % 2u) != 0u)
    {
        /* not 2-byte aligned */
        return ADI_SINC_INVALID_PARAMETER;
    }

    if ((tail % 2u) != 0u)
    {
        /* not 2-byte aligned */
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (eGroup == ADI_SINC_GROUP_0)
    {
        pDevice->pSincRegs->PHEAD0 = head;
        pDevice->pSincRegs->PTAIL0 = tail;
    } else {
        pDevice->pSincRegs->PHEAD1 = head;
        pDevice->pSincRegs->PTAIL1 = tail;
    }
    
    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Submits a buffer to receive SINC data samples.
 *
 * Up to two buffers can be submitted to receive the SINC data samples.  The 
 * data from the SINC circular buffer is copied into the data buffer when a
 * data interrupt is processed.
 *
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  eGroup                The SINC group circular buffer head address to set.
 * @param [in]  *pBuffer              The buffer to receive the SINC data.
 * @param [in]  nBufSize              The size of the receive buffer (bytes).
 *
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully set the circular buffer address.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *              - #ADI_SINC_INVALID_OPERATION       Failed to submit the buffer.  Only two buffers can be submitted.
 *
 * @sa          adi_sinc_SetCircBuffer()
 * @sa          adi_sinc_GetBuffer()
 */
ADI_SINC_RESULT adi_sinc_SubmitBuffer(
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup,
    void                            *pBuffer,
    uint32_t               const     nBufSize)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;
    
    ADI_SINC_GROUP_DATA  *group;
    
    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (eGroup == ADI_SINC_GROUP_0)
    {
        group = &pDevice->pSincData->Group0;
    }
    else
    {
        group = &pDevice->pSincData->Group1;
    }        
    
    if (group->BufferInfo0.pBuffer == NULL)
    {
        group->BufferInfo0.pBuffer = pBuffer;
        group->BufferInfo0.nBufSize = nBufSize;
    }
    else if (group->BufferInfo1.pBuffer == NULL)
    {
        group->BufferInfo1.pBuffer = pBuffer;
        group->BufferInfo1.nBufSize = nBufSize;
    }
    else
    {
        return ADI_SINC_INVALID_OPERATION;
    }
    
    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Get the buffer which contains the latest SINC data.
 * 
 * Waits until a group data interrupt is received then return the SINC data from 
 * the circular buffer.  If more than one pair is enabled, the data is interleaved.
 * The order of interleave will be from lowest numbered to highest numbered SINC filter. 
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  eGroup                The SINC group buffer data to get.
 * @param [out] ppBuffer              A buffer pointer to the latest SINC data.
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully got the SINC data.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *              - #ADI_SINC_SEM_FAILED              A semaphore error occurred.
 *              - #ADI_SINC_INVALID_OPERATION   [D] The driver is in callback mode.
 *
 * @sa  #adi_sinc_SubmitBuffer()
 * @sa  #adi_sinc_EnableDataInterrupt()
 *
 */
ADI_SINC_RESULT adi_sinc_GetBuffer (
    ADI_SINC_HANDLE        const   hDevice,
    ADI_SINC_GROUP         const   eGroup,
    void                         **ppBuffer)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    ADI_SINC_GROUP_DATA *pGroup;
    
    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    /* callback mode */
    if(pDevice->pSincData->pfDataCallback != NULL)
    {
        return ADI_SINC_INVALID_OPERATION;
    }

    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    if (eGroup == ADI_SINC_GROUP_0)
    {
        pGroup = &pDevice->pSincData->Group0;
    }
    else
    {
        pGroup = &pDevice->pSincData->Group1;
    }        
    
    /* wait for the data interrupt */
    if(adi_osal_SemPend(pGroup->hSemaphore,
                    ADI_OSAL_TIMEOUT_FOREVER) != ADI_OSAL_SUCCESS)
    {
        return ADI_SINC_SEM_FAILED;
    }
        
    *ppBuffer = (void*)pGroup->pReadyBuf->pBuffer;
    
    return ADI_SINC_SUCCESS;
}

/*
 * History buffer
 */

/**
 * @brief       Gets the secondary history buffer status.
 * 
 * The secondary history buffer pointer indicates the position in the history 
 * register block corresponding to the most recent secondary data sample for 
 * the secondary SINC filter.
 * 
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  ePair                 The SINC pair to set the amplitude limits.
 * @param [out] ePosition             The pair secondary history buffer pointer.
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully got the secondary history buffer pointer.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_GetHistoryValues()
 *
 */
ADI_SINC_RESULT  adi_sinc_GetHistoryStatus (
    ADI_SINC_HANDLE       const       hDevice,
    ADI_SINC_PAIR         const       ePair,
    ADI_SINC_HISTORY_STAT            *ePosition)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
        
    uint32_t reg;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    if (ePair > ADI_SINC_PAIR_3)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    reg = pDevice->pSincRegs->HIS_STAT;
    
    if (ePair == ADI_SINC_PAIR_0)
    {
        *ePosition = (ADI_SINC_HISTORY_STAT)((reg & BITM_SINC_HIS_STAT_P0HISPTR) >> BITP_SINC_HIS_STAT_P0HISPTR);
    } 
    else if (ePair == ADI_SINC_PAIR_1)
    {
        *ePosition = (ADI_SINC_HISTORY_STAT)((reg & BITM_SINC_HIS_STAT_P1HISPTR) >> BITP_SINC_HIS_STAT_P1HISPTR);
    }
    else if (ePair == ADI_SINC_PAIR_2)
    {
        *ePosition = (ADI_SINC_HISTORY_STAT)((reg & BITM_SINC_HIS_STAT_P2HISPTR) >> BITP_SINC_HIS_STAT_P2HISPTR);
    }
    else 
    {
        *ePosition = (ADI_SINC_HISTORY_STAT)((reg & BITM_SINC_HIS_STAT_P3HISPTR) >> BITP_SINC_HIS_STAT_P3HISPTR);
    }
    
    return ADI_SINC_SUCCESS;
}

/**
 * @brief       Gets the secondary history values.
 * 
 * The history registers provide the most recent samples produced by secondary SINC 
 * filter. These are the values which are compared to limit, count, and window settings 
 * resulting in setting the Max and Min status bits in the STATUS register.
 * 
 * The ADSP-CM40z parts hold up to 8 most recent history values.
 *
 * @param [in]  hDevice               Handle to the SINC device.
 * @param [in]  ePair                 The SINC pair to get the secondary history values.
 * @param [in]  size                  The size of the values buffer.
 * @param [out] pValues               Buffer to hold the most recent history values.
 *                                    
 * 
 * @return      Status
 *              - #ADI_SINC_SUCCESS                 Successfully got the secondary history values.
 *              - #ADI_SINC_INVALID_HANDLE      [D] The device handle is invalid.
 *              - #ADI_SINC_INVALID_PARAMETER   [D] A parameter is invalid.
 *
 * @sa  #adi_sinc_GetHistoryStatus()
 *
 */
ADI_SINC_RESULT  adi_sinc_GetHistoryValues (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_PAIR          const     ePair,
    uint16_t               const     size,
    uint16_t                        *pValues)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
        
    uint32_t reg0;
    uint32_t reg1;
    uint32_t reg2;
    uint32_t reg3;
    uint32_t masklo;
    uint32_t maskhi;
    uint32_t bitlo;
    uint32_t bithi;
    
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }

    if (ePair > ADI_SINC_PAIR_3)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
    
    if (size < 8u)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }        
#endif /* ADI_DEBUG */

    if (ePair == ADI_SINC_PAIR_0)
    {
        reg0 = pDevice->pSincRegs->P0SEC_HIST0;
        reg1 = pDevice->pSincRegs->P0SEC_HIST1;
        reg2 = pDevice->pSincRegs->P0SEC_HIST2;
        reg3 = pDevice->pSincRegs->P0SEC_HIST3;
        masklo = BITM_SINC_P0SEC_HIST_LO;
        maskhi = BITM_SINC_P0SEC_HIST_HI;
        bitlo = BITP_SINC_P0SEC_HIST_LO;
        bithi = BITP_SINC_P0SEC_HIST_HI;
    } else if (ePair == ADI_SINC_PAIR_1) {
        reg0 = pDevice->pSincRegs->P1SEC_HIST0;
        reg1 = pDevice->pSincRegs->P1SEC_HIST1;
        reg2 = pDevice->pSincRegs->P1SEC_HIST2;
        reg3 = pDevice->pSincRegs->P1SEC_HIST3;
        masklo = BITM_SINC_P1SEC_HIST_LO;
        maskhi = BITM_SINC_P1SEC_HIST_HI;
        bitlo = BITP_SINC_P1SEC_HIST_LO;
        bithi = BITP_SINC_P1SEC_HIST_HI;
    } else if (ePair == ADI_SINC_PAIR_2) {
        reg0 = pDevice->pSincRegs->P2SEC_HIST0;
        reg1 = pDevice->pSincRegs->P2SEC_HIST1;
        reg2 = pDevice->pSincRegs->P2SEC_HIST2;
        reg3 = pDevice->pSincRegs->P2SEC_HIST3;
        masklo = BITM_SINC_P2SEC_HIST_LO;
        maskhi = BITM_SINC_P2SEC_HIST_HI;
        bitlo = BITP_SINC_P2SEC_HIST_LO;
        bithi = BITP_SINC_P2SEC_HIST_HI;
    } else {
        reg0 = pDevice->pSincRegs->P3SEC_HIST0;
        reg1 = pDevice->pSincRegs->P3SEC_HIST1;
        reg2 = pDevice->pSincRegs->P3SEC_HIST2;
        reg3 = pDevice->pSincRegs->P3SEC_HIST3;
        masklo = BITM_SINC_P3SEC_HIST_LO;
        maskhi = BITM_SINC_P3SEC_HIST_HI;
        bitlo = BITP_SINC_P3SEC_HIST_LO;
        bithi = BITP_SINC_P3SEC_HIST_HI;
    }
    
    *pValues++ = (reg0 & masklo) >> bitlo;
    *pValues++ = (reg0 & maskhi) >> bithi;
    *pValues++ = (reg1 & masklo) >> bitlo;
    *pValues++ = (reg1 & maskhi) >> bithi;
    *pValues++ = (reg2 & masklo) >> bitlo;
    *pValues++ = (reg2 & maskhi) >> bithi;
    *pValues++ = (reg3 & masklo) >> bitlo;
    *pValues++ = (reg3 & maskhi) >> bithi;
    
    return ADI_SINC_SUCCESS;
}

/*
 * Internal functions
 */

/* Status interrupt handler */
static void StatusIntHandler(uint32_t iid, void *pCBParam)
{
    /* Pointer to the device data */
    ADI_SINC_DEVICE         *pDevice = (ADI_SINC_DEVICE *) pCBParam;
    
    /* status bits */
    uint32_t status = pDevice->pSincRegs->STAT;
    
    /* clear status W1C */
    pDevice->pSincRegs->STAT = status;
 
    /* application callback */   
    if(pDevice->pSincData->pfStatusCallback != NULL)
    {
        pDevice->pSincData->pfStatusCallback(pDevice->pSincData->pStatusCBParam, 
            ADI_SINC_EVENT_STATUS, (void*)status);
    }
}

/*
 * Copy data from the device circular buffer to the application supplied data buffer.
 */
static void OnDataReady(
    ADI_SINC_GROUP_DATA          *pGroup,
    uint16_t               const *currentPtr,
    uint32_t               const  headPtr,
    uint32_t               const  tailPtr,
    uint8_t                const  numPairs,
    uint32_t               const  pcnt)
{
    uint32_t numBytes;
    uint32_t nCurToTail;
    uint8_t *dataPtr;
    
    if ((pGroup->pActiveBuf == NULL) || 
        (pGroup->pActiveBuf->pBuffer) == NULL)
    {
        return;
    }
    
    /* Determine the number of data samples to copy.  For example, if Group 0 
     * was assigned three SINC filters and PCNT was set to 5, then 
     * the number of samples would be 3x5 = 15 data samples. 
     */
    numBytes = ((pcnt+1u) * numPairs) * BYTES_PER_SAMPLE;
        
    /* since the SINC device uses a circular buffer we may need two copy operations */
    nCurToTail = tailPtr - (uint32_t)currentPtr;
    
    if (numBytes <= nCurToTail)
    {
        /* can use one copy */
        dataPtr = pGroup->pActiveBuf->pBuffer;
        memcpy(dataPtr, (void*)currentPtr, numBytes);
    }
    else
    {
        /* copy from current to tail */
        dataPtr = pGroup->pActiveBuf->pBuffer;
        memcpy(dataPtr, (void*)currentPtr, nCurToTail);
        
        /* copy remaining from head */
        dataPtr += nCurToTail;
        memcpy(dataPtr, (void*)headPtr, numBytes-nCurToTail);
    }

    /* active is now ready */    
    pGroup->pReadyBuf = pGroup->pActiveBuf;

    /* change active to other buffer */
    if (pGroup->pActiveBuf == &pGroup->BufferInfo0)
    {
        pGroup->pActiveBuf = &pGroup->BufferInfo1;
    }
    else
    {
        pGroup->pActiveBuf = &pGroup->BufferInfo0;
    }        
}


/* Data0 interrupt handler */
static void Data0IntHandler(uint32_t iid, void *pCBParam)
{
    uint32_t ctl;
    uint8_t numPairs;
    uint32_t pcnt;
    uint32_t phead;
    uint32_t ptail;
    uint16_t *pCurr;
    
    /* Pointer to the device data */
    ADI_SINC_DEVICE  *pDevice = (ADI_SINC_DEVICE *) pCBParam;

    /* determine how many pairs are enabled for this group */
    numPairs = 0u;
    ctl = pDevice->pSincRegs->CTL;
    if (((ctl & BITM_SINC_CTL_EN0) >> BITP_SINC_CTL_EN0) == ENUM_SINC_PAIR_GROUP_0)
    {
        numPairs++;
    }
    if (((ctl & BITM_SINC_CTL_EN1) >> BITP_SINC_CTL_EN1) == ENUM_SINC_PAIR_GROUP_0)
    {
        numPairs++;
    }
    if (((ctl & BITM_SINC_CTL_EN2) >> BITP_SINC_CTL_EN2) == ENUM_SINC_PAIR_GROUP_0)
    {
        numPairs++;
    }
    if (((ctl & BITM_SINC_CTL_EN3) >> BITP_SINC_CTL_EN3) == ENUM_SINC_PAIR_GROUP_0)
    {
        numPairs++;
    }
    
    if (numPairs == 0u)
    {
        /* no pairs enabled */
        return;
    }

    /* copy data to the active buffer */
    pcnt = (pDevice->pSincRegs->LEVEL0 & BITM_SINC_LEVEL0_PCNT) >> BITP_SINC_LEVEL0_PCNT;
        
    phead = pDevice->pSincRegs->PHEAD0;
    ptail = pDevice->pSincRegs->PTAIL0;

    /* the current pointer is at the end of the valid data, so find the beginning */
    pCurr = (uint16_t*)pDevice->pSincRegs->PPTR0;
    pCurr -= (numPairs-1u);    /* align to last pair0 result */
    pCurr -= (pcnt*numPairs);  /* go to first pair0 result */
    if (pCurr < (uint16_t*)phead)
    {
       /* wrap around */
       pCurr = (uint16_t*)ptail;
       pCurr -= ((uint16_t*)phead - pCurr);
    }

    OnDataReady(&pDevice->pSincData->Group0, pCurr, 
                phead, ptail, numPairs, pcnt);

    /* application callback */
    if(pDevice->pSincData->pfDataCallback != NULL)
    {
        pDevice->pSincData->pfDataCallback(pDevice->pSincData->pDataCBParam, 
            ADI_SINC_EVENT_DATA0, (void*)pDevice->pSincData->Group0.pReadyBuf->pBuffer);
    }
    else
    {
        /* Post the semaphore when a callback is not registered */
        adi_osal_SemPost(pDevice->pSincData->Group0.hSemaphore);
    }
    
    /* clear status W1C */
    pDevice->pSincRegs->STAT |= (1u << BITP_SINC_STAT_PCNT0);
}

/* Data1 interrupt handler */
static void Data1IntHandler(uint32_t iid, void *pCBParam)
{
    uint32_t ctl;
    uint8_t numPairs;
    uint32_t pcnt;
    uint32_t phead;
    uint32_t ptail;
    uint16_t *pCurr;

    /* Pointer to the device data */
    ADI_SINC_DEVICE  *pDevice = (ADI_SINC_DEVICE *) pCBParam;
     
    /* determine how many pairs are enabled for this group */
    numPairs = 0u;
    ctl = pDevice->pSincRegs->CTL;
    if (((ctl & BITM_SINC_CTL_EN0) >> BITP_SINC_CTL_EN0) == ENUM_SINC_PAIR_GROUP_1)
    {
        numPairs++;
    }
    if (((ctl & BITM_SINC_CTL_EN1) >> BITP_SINC_CTL_EN1) == ENUM_SINC_PAIR_GROUP_1)
    {
        numPairs++;
    }
    if (((ctl & BITM_SINC_CTL_EN2) >> BITP_SINC_CTL_EN2) == ENUM_SINC_PAIR_GROUP_1)
    {
        numPairs++;
    }
    if (((ctl & BITM_SINC_CTL_EN3) >> BITP_SINC_CTL_EN3) == ENUM_SINC_PAIR_GROUP_1)
    {
        numPairs++;
    }

    if (numPairs == 0u)
    {
        /* no pairs enabled */
        return;
    }

    /* copy data to the active buffer */
    pcnt = (pDevice->pSincRegs->LEVEL1 & BITM_SINC_LEVEL1_PCNT) >> BITP_SINC_LEVEL1_PCNT;

    phead = pDevice->pSincRegs->PHEAD1;
    ptail = pDevice->pSincRegs->PTAIL1;

    /* the current pointer is at the end of the valid data, so find the beginning */
    pCurr = (uint16_t*)pDevice->pSincRegs->PPTR1;
    pCurr -= (numPairs-1u);    /* align to last pair0 result */
    pCurr -= (pcnt*numPairs);  /* go to first pair0 result */
    if (pCurr < (uint16_t*)phead)
    {
        /* wrap around */
        pCurr = (uint16_t*)ptail - ((uint16_t*)phead - pCurr);
    }

    OnDataReady(&pDevice->pSincData->Group1, pCurr, 
                phead, ptail, numPairs, pcnt);
                
    /* application callback */   
    if(pDevice->pSincData->pfDataCallback != NULL)
    {
        pDevice->pSincData->pfDataCallback(pDevice->pSincData->pDataCBParam, 
            ADI_SINC_EVENT_DATA1, (void*)pDevice->pSincData->Group1.pReadyBuf->pBuffer);
    }
    else
    {
        /* Post the semaphore when a callback is not registered */
        adi_osal_SemPost(pDevice->pSincData->Group1.hSemaphore);
    }
    
    /* clear status W1C */
    pDevice->pSincRegs->STAT |= (1u << BITP_SINC_STAT_PCNT1);
}

/*
 * Calc the primary output bias value.
 *
 * For this number to be represented in two's complement format, this register 
 * must be programmed to a value equal to -(d^o)/2 where d is the decimation rate 
 * and o is the order selected for the Primary SINC filter.
 */
static ADI_SINC_RESULT  CalcOutputBias (
    ADI_SINC_HANDLE        const     hDevice,
    ADI_SINC_GROUP         const     eGroup)
{
    /* Result code */
    ADI_SINC_RESULT     eResult;

    /* Pointer to the device data */
    ADI_SINC_DEVICE    *pDevice = (ADI_SINC_DEVICE *) hDevice;
    
    uint32_t reg  = 0u;
    uint32_t d;
    uint32_t o;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateDevice(pDevice)) != ADI_SINC_SUCCESS)
    {
        return eResult;
    }
    
    if (eGroup > ADI_SINC_GROUP_1)
    {
        return ADI_SINC_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();
        
    if (eGroup == ADI_SINC_GROUP_0)
    {
        d = (pDevice->pSincRegs->RATE0 & BITM_SINC_RATE0_PDEC) >> BITP_SINC_RATE0_PDEC;
        o = (pDevice->pSincRegs->LEVEL0 & BITM_SINC_LEVEL0_PORD) >> BITP_SINC_LEVEL0_PORD;
        
        if (o == 0)
        {
            /* 3rd order */
            reg = d*d*d/2u;
        } else {
            /* 4th order */
            reg = d*d*d*d/2u;
        }            
        reg = (uint32_t)(~reg + 1u);  /* 2's complement */

        pDevice->pSincRegs->BIAS0 = reg;
    } else {
        d = (pDevice->pSincRegs->RATE1 & BITM_SINC_RATE1_PDEC) >> BITP_SINC_RATE1_PDEC;
        o = (pDevice->pSincRegs->LEVEL1 & BITM_SINC_LEVEL1_PORD) >> BITP_SINC_LEVEL1_PORD;

        if (o == 0)
        {
            /* 3rd order */
            reg = d*d*d/2u;
        } else {
            /* 4th order */
            reg = d*d*d*d/2u;
        }            
        reg = (uint32_t)(~reg + 1u);  /* 2's complement */

        pDevice->pSincRegs->BIAS1 = reg;
    }
        
    adi_osal_ExitCriticalRegion();
    
    return ADI_SINC_SUCCESS;
}

#ifdef ADI_DEBUG
/* Validate the device */
static ADI_SINC_RESULT ValidateDevice(ADI_SINC_DEVICE   *pDevice)
{
    uint32_t i;

    /* Check if the given pointer matches with any of the devices */
    for(i = 0; i < ADI_SINC_NUM_DEVICES; i++)
    {
        /* If found a match return success */
        if(pDevice == &gSincDevices[i])
        {
            return ADI_SINC_SUCCESS;
        }
    }

    /* Could not find the match return failure */
    return ADI_SINC_INVALID_HANDLE;
}
#endif /* ADI_DEBUG */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*@}*/
