/* $Id: adi_spi_def_bf6xx.h 22007 2013-06-25 19:06:10Z skafka $ */
/*********************************************************************************

 Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

 This software is proprietary and confidential.  By using this software you agree
 to the terms of the associated Analog Devices License Agreement.

 *********************************************************************************/
/*!
 * @file      adi_spi_def_bf6xx.h
 *
 * @brief     SPI driver data definitions
 *
 * @details
 *            This is a source file which contains the non-exposed definitions of
 *            the adi_spi_bf6x.c driver.
 */

/** @addtogroup SPI_Driver_BF6xx SPI Device Driver
 *  @{
 */

#ifndef __ADI_SPI_DEF_BF6XX_H__
#define __ADI_SPI_DEF_BF6XX_H__


#include <adi_types.h>
#include <stddef.h>
#include <string.h>
#include <sys/platform.h>
#include <sys/exception.h>
#include <sys/16000030.h>
#include <adi_osal.h>
#include <drivers/spi/adi_spi.h>

/* Prototype for SPI submit transceiver buffer functions */
typedef ADI_SPI_RESULT (*ADI_SPI_SUBMIT_BUFFERS_FPTR)(ADI_SPI_HANDLE const hDevice, ADI_SPI_TRANSCEIVER *pTransceiver);
/* Prototype for SPI close functions */
typedef ADI_SPI_RESULT (*ADI_SPI_CLOSE_FPTR)(ADI_SPI_HANDLE const hDevice);

/*! Enumeration for the SPI operating mode */
typedef enum
{
    ADI_SPI_MODE_NONE,			/*!< default, no mode of operation                							*/
    ADI_SPI_MODE_NONBLOCKING,	/*!< non-blocking mode, use ad_spi_SubmitBuffer, no callbacks       		*/
    ADI_SPI_MODE_BLOCKING,		/*!< blocking mode, use ad_spi_ReadWritebuffer, no callbacks    			*/
    ADI_SPI_MODE_CALLBACK 		/*!< callback mode, use ad_spi_SubmitBuffer, with callbacks         		*/
} ADI_SPI_OPERATING_MODE;

/* local structure to the transfer of SPI transceiver data */
typedef struct _ADI_SPI_TRANSFER_
{
	uint8_t *pTxData; 									/* pointer to tx buffer                             */
	uint8_t *pRxData; 									/* pointer to rx buffer                             */
	uint32_t TxBytesToEmpty;	 						/* number of bytes left before tx buffer is empty   */
	uint32_t RxBytesToFill; 							/* number of bytes left before rx buffer is full    */
} ADI_SPI_TRANSFER;

/* SPI function table */
typedef struct
{
	ADI_SPI_SUBMIT_BUFFERS_FPTR pfSpiSubmitBuffer; 	/* Spi submit transceiver buffers function pointer	*/
	ADI_SPI_CLOSE_FPTR pfSpiClose; 					/* Spi close function pointer						*/
} ADI_SPI_FN_TABLE;

/* SPI driver memory parameters */
typedef struct
{
	ADI_SPI_TRANSCEIVER *pActiveTransceiver; 			/* pointer to active transceiver structure          */
	ADI_SPI_TRANSCEIVER *pPrimaryTransceiver; 			/* pointer to primary transceiver structure         */
	ADI_SPI_TRANSCEIVER *pSecondaryTransceiver;			/* pointer to secondary transceiver structure       */
	bool bPrimaryProcessed;								/* flag, primary transceiver is processed			*/
	bool bSecondaryProcessed;							/* flag, secondary transceiver is processed    		*/
	ADI_SPI_TRANSFER Transfer; 							/* instance for the transfer of the transceiver     */
	ADI_SPI_FN_TABLE FuncTable; 						/* Function table pointer							*/
	ADI_CALLBACK pfCallback; 							/* Application supplied callback function           */
	void* pCBParam; 									/* Callback parm passed back in callbacks           */
	ADI_DMA_CHANNEL_HANDLE hTxDMAChannel; 				/* DMA Tx Channel Handle	     					*/
	ADI_DMA_CHANNEL_HANDLE hRxDMAChannel; 				/* DMA Rx Channel Handle	     					*/
	ADI_OSAL_SEM_HANDLE hSemHandle; 					/* Semaphore handle for interrupt blocking mode		*/
	ADI_SPI_OPERATING_MODE SpiMode;						/* SPI operating mode								*/
	uint8_t SpiTransferSize; 							/* SPI transfer size                                */
    uint8_t BytesPerDMATransfer;                        /* Bytes per DMA transfer                           */
	ADI_SPI_SSENABLE SpiSlaveSelect;                    /* Slave select enumeration                         */
	bool SpiSWSlaveSelectEnable;	                    /* Flag for software slave select control           */
	bool bLSBFirstEnable; 			                    /* Flag for the tx/rx least significant byte first  */
	bool bDmaMode; 					                    /* Flag for DMA mode                                */
	bool bDmaSupported;				                    /* Flag for DMA supported, is there enough memory   */
	bool bManualSlaveSelect;				            /* Flag for manual slave select control   			*/
	ADI_SPI_TRANSCEIVER_MODE eMode;				        /* Flag for manual slave select control   			*/
	uint32_t SpiStatusEvent;				            /* Spi status event                                 */
	uint32_t TransferFinishMask;						/* Mask needed to read RF and TF bits of status		*/
	uint32_t *pDummyRxPrologue;						    /* Dummy rx prologue needed for DMA transfers       */
	uint32_t *pDummyTx;						            /* Dummy tx buffer needed for DMA transfers         */
	uint8_t SpiSemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR];   /* Semaphore Memory  								*/
	uint8_t *pSpiDMAMemory;                             /* Pointer to SPI Tx and RX DMA memory, if used     */
} ADI_SPI_DEV_DATA;

/* SPI Register structure */
typedef volatile struct _ADI_SPI_REGS_
{
    /*uint32_t   SPI_REVID;*/                                       /* SPI read only revision register              */
    uint32_t   SPI_CONTROL;                                     /* SPI control register                         */
    uint32_t   SPI_RXCONTROL;                                   /* SPI receive channel control register         */
    uint32_t   SPI_TXCONTROL;                                   /* SPI transmit channel control register        */
    uint32_t   SPI_CLOCK;                                       /* SPI clock rate register                      */
    uint32_t   SPI_DELAY;                                       /* SPI delay between frames register            */
    uint32_t   SPI_SSEL;                                        /* SPI slave select register                    */
    uint32_t   SPI_RWC;                                         /* SPI receive word count register              */
    uint32_t   SPI_RWCR;                                        /* SPI receive word count reload register       */
    uint32_t   SPI_TWC;                                         /* SPI transmit word count register             */
    uint32_t   SPI_TWCR;                                        /* SPI transmit word count reload register      */
    uint32_t   SPI_RESERVED0;                                   /* Reserved0                                    */
    uint32_t   SPI_EMASK;                                       /* SPI set interrupt mask register              */
    uint32_t   SPI_EMASKCL;                                     /* SPI clear interrupt mask register, W1C       */
    uint32_t   SPI_EMASKST;                                     /* SPI set interrupt mask register, W1C         */
    uint32_t   SPI_RESERVED1;                                   /* Reserved1                                    */
    uint32_t   SPI_STATUS;                                      /* SPI status register                          */
    uint32_t   SPI_ELAT;                                        /* SPI interrupt latch register, read only      */
    uint32_t   SPI_ELATCL;                                      /* SPI interrupt latch register, W1C            */
    uint32_t   SPI_RESERVED2;                                   /* Reserved2                                    */
    uint32_t   SPI_RFIFO;                                       /* SPI transmit fifo register                   */
    uint32_t   SPI_RESERVED3;                                  /* Reserved3                                    */
    uint32_t   SPI_TFIFO;                                       /* SPI transmit fifo register                   */
}ADI_SPI_REGS;

typedef struct
{
	uint32_t bTx;
	uint32_t Stat;
	uint32_t Cycles;
	uint32_t Data[4];
} SPI_TEST;

/* SPI Device Structure */
typedef struct _ADI_SPI_
{
	CONST uint32_t                  SpiStatusSID;       /* SPI status interrupt ID                          */
	CONST uint32_t                  SpiTransmitterSID;  /* SPI tx data SID                     				*/
	CONST uint32_t                  SpiReceiverSID;     /* SPI rx data SID                     				*/
	ADI_DMA_CHANNEL_ID CONST        eDMATxChannelID;    /* SPI tx DMA ID                                    */
	ADI_DMA_CHANNEL_ID CONST        eDMARxChannelID;    /* SPI rx DNA ID                                    */
	ADI_SPI_REGS * CONST            pRegs;              /* Base Address of Registers                        */
	ADI_SPI_DEV_DATA               *pDevData;           /* SPI driver data                                  */
} ADI_SPI;

#endif
