/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_sport_bf60x.c
*
* @brief     This is the primary source file for the sport Driver for  ADSP-BF609  family processor.
*
*            Contains the implementation of the APIs for the SPORT peripheral
*            driver. 
*
*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

/** @addtogroup SPORT_Driver_BF6xx SPORT Device Driver (ADSP-BF6xx)
 *  @{
 */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


/*********************************************************************

Include files

*********************************************************************/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif

#include <drivers/sport/adi_sport.h>
#include <string.h>
#include <assert.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif




#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined outside the block eventhough they are used in one function, for ease maintainence.")
#pragma diag(suppress:misra_rule_13_1:"Allow assignment in if statements. Required for efficient code")
#pragma diag(suppress:misra_rule_13_5:"Allow expressions of for statement that are not concerned with loop control for code efficiency.")
#pragma diag(suppress:misra_rule_13_7:"Boolean operations whose results are invariant shall not be permitted.")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")    
#pragma diag(suppress:misra_rule_17_4:"Allow pointer arithmetic for MMR address generation")
#pragma diag(suppress:misra_rule_19_7:"Allow macros to be used for readability")

#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct.")
#pragma diag(suppress:misra_rule_11_5:"A cast should not be performed that removes any const or volatile qualification from the type addressed by a pointer.")

#endif /* _MISRA_RULES */

#define ENUM_SPORT_CTL2_LOOPBACK_EN  0X00000004

#include "adi_sport_reg.h"

#ifndef ADI_CODE_IN_ROM
#define SPORT_eSTATE(deviceNum)    sport_device_info[(deviceNum)][eChannel].eState
#define CONST const
#else
#include "../rom/adi_rom_def.h"
#define SPORT_eSTATE(deviceNum)    pram_for_rom_Code->sport_device_info[(deviceNum)][eChannel].eState
#endif

#include "adi_sport_data_bf6xx.c"

/* For anomaly workaround */
#include <sys/16000030.h>




#ifdef ADI_DEBUG
static ADI_SPORT_RESULT  ValidateHandle(ADI_SPORT_DEVICE *pInDevice);
#endif /* ADI_DEBUG */

/**********************************************************
  General SPORT APIs
**********************************************************/
/**
 * @brief       Initialization function for the SPORT device. 
 *
 * @details     Initialize the specified SPORT. By default, Device is configured with the following 
 *              settings. This function must be called before operating any SPORT device.
 *
 * @param [in]  nDeviceNum  SPORT Device instance to be opened. 
 *
 * @param [in]  eChannel    Channel ID of the SPORT device( A or B) 
 *
 * @param [in]  eDirection  Direction of the SPORT operation. (i.e Rx or Tx) 
 *
 * @param [in]  pMemory     Pointer to a 32 bit aligned buffer of size ADI_SPORT_INT_MEMORY_SIZE 
 *                          required by the driver for the operation of specified SPORT device. 
 *                          Buffer Size should be "ADI_SPORT_DMA_MEMORY_SIZE" bytes if the device is 
 *                          expected to operate in DMA mode. 
 *
 * @param [in]  eMode       Mode of operation. Indicates in which mode the device is expected to be 
 *                          operated. 
 *
 * @param [in]  nMemSize    Size of the buffer to which "pMemory" points. DMA mode can be enabled 
 *                          only if *pMemory points to the buffer whose size is sufficient to 
 *                          operate in the DMA mode. This parameter ("nMemSize") indicate whether 
 *                          the device is expected to operate in DMA mode or in interrupt mode. 
 *
 * @param [out] phDevice    Pointer to a location where SPORT device handle to be written. This 
 *                          handle will be used as an identity of device and need to be passed as 
 *                          one of the argument to functions which manage the device. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                      Successfully initialized the specified SPORT device. 
 *
 *  - #ADI_SPORT_DEVICE_IN_USE                Specified SPORT is already initialized. 
 *
 *  - #ADI_SPORT_FAILED                       Failed initialize a semaphore for managing device. 
 *
 *  - #ADI_SPORT_INT_REGISTRATION_FAILED      Failed to register the specified sport device with 
 *                                            interrupt manager. 
 *
 *  - #ADI_SPORT_INVALID_CHANNEL_ID      [D]  Invalid channel ID. 
 *
 *  - #ADI_SPORT_INVALID_MODE            [D]  Invalid SPORT mode. 
 *
 *  - #ADI_SPORT_INVALID_DIR             [D]  Invalid direction. 
 *
 *  - #ADI_SPORT_INVALID_DEVICE_NUM      [D]  Specified device instance is invalid(valid range 0 - 
 *                                            2). 
 *
 *  - #ADI_SPORT_INVALID_NULL_POINTER    [D]  Specified pointer is invalid(either callback function 
 *                                            or device handle). 
 *
 * @sa          adi_sport_Close() 
 *
 */

ADI_SPORT_RESULT adi_sport_Open(
                                 uint32_t             nDeviceNum,
                                 ADI_SPORT_CHANNEL    eChannel,
                                 ADI_SPORT_DIRECTION  eDirection,
                                 ADI_SPORT_MODE       eMode,
                                 void                *pMemory,
                                 uint32_t             nMemSize,
                                 ADI_SPORT_HANDLE    *phDevice
                                  )
{
    /* Pointer to the SPORT Device Instance Data */
    ADI_SPORT_DEVICE         *pDevice;
    uint32_t                 Config;
    uint8_t *pDevMemory =  pMemory;
#ifdef ADI_DEBUG

    /* Check if the given device number is within the range of number SPORT
     * devices present in the processor
     */
    if(nDeviceNum >= ADI_SPORT_NUM_INSTANCES)
    {
        return ADI_SPORT_INVALID_DEVICE_NUM;
    }

    /* Check if the callback function pointer and  pointer to device handle is valid */
    if((phDevice == NULL))
    {
        return ADI_SPORT_INVALID_NULL_POINTER;
    }

    if( (eDirection != ADI_SPORT_DIR_RX) && (eDirection != ADI_SPORT_DIR_TX))
    {
        return ADI_SPORT_INVALID_DIR;
    }

    if(eMode > ADI_SPORT_I2S_MODE)
    {
        return ADI_SPORT_INVALID_MODE;
    }

    if(eChannel > ADI_HALF_SPORT_B)
    {
        return ADI_SPORT_INVALID_CHANNEL_ID;
    }
    /* Asserts to validate the memory size macros */
    assert(ADI_SPORT_INT_MEMORY_SIZE == sizeof(ADI_SPORT_DEVICE));
    assert(ADI_SPORT_DMA_MEMORY_SIZE == (ADI_SPORT_INT_MEMORY_SIZE + ADI_DMA_CHANNEL_REQ_MEMORY));

#endif /* ADI_DEBUG */
    memset(pMemory, 0, nMemSize);        
    /* Assign the pointers with the supplied memory. */
    pDevice =(ADI_SPORT_DEVICE *)pDevMemory;
    /* Increment pointer by size of the device structure */
    pDevMemory += (sizeof(ADI_SPORT_DEVICE));

    /* Check if the device instance is in use */
    if(SPORT_eSTATE(nDeviceNum) != ADI_SPORT_STATE_UNINITIALIZED)
    {
        return ADI_SPORT_DEVICE_IN_USE;
    }
    /* Change the state of the specified device */
    adi_osal_EnterCriticalRegion();
    SPORT_eSTATE(nDeviceNum) =   ADI_SPORT_STATE_INITIALIZED;
#if !defined(ADI_CODE_IN_ROM)
    sport_device_info[nDeviceNum][eChannel].hDevice = pDevice;
#endif

    adi_osal_ExitCriticalRegion();
    /* Initialize the  pointers which provides the device information. */
#if !defined(ADI_CODE_IN_ROM)
    pDevice->pSportInfo   =   &sport_device_info[nDeviceNum][eChannel];
    /* Initialize the base address of the device */
    pDevice->pSPORTRegs   =   sport_device_info[nDeviceNum][eChannel].pSportRegs;
#else
    pDevice->pSportInfo   =   &pram_for_rom_Code->sport_device_info[nDeviceNum][eChannel];
    /* Initialize the base address of the device */
    pDevice->pSPORTRegs   =   pram_for_rom_Code->sport_device_info[nDeviceNum][eChannel].pSportRegs;
#endif
    /* Store the direction */
    pDevice->eDirection   =   eDirection;
    /* By default, Load the function table for interrupt mode.
       function adi_sport_EnalbeDMAMode should be called  to operated
       SPORT in DMA mode. */
    pDevice->pFunTable    =   (SPORT_FUNCTION_TABLE *)&gSportIntFuncTable;
    /* Initialize other parameters */
    /* set the mode in which opened device operates */
    pDevice->eMode                  =   eMode;
    /* set all required SPORT registers to reset value */
    /*Initialize the  Handle for DMA to NULL */
    pDevice->hDMAChnl =  NULL;

    pDevice->pDMAChnlMem = NULL;
    /* DMA transfer size. set for 16 bit transfer since it is most widely use case*/
    pDevice->eDmaWordSize = ADI_DMA_MSIZE_4BYTES;
    /* clear the Hw event status */
    pDevice->nHwError  = (uint32_t) ADI_SPORT_HW_ERR_NONE;
    /* Initialize  the buffer management pointers */
    InitDevice(pDevice);
    pDevice->pSPORTRegs->SPORT_MCTL   = 0X00U;
    pDevice->pSPORTRegs->SPORT_CS0    = 0X00U;
    pDevice->pSPORTRegs->SPORT_CS1    = 0X00U;
    pDevice->pSPORTRegs->SPORT_CS2    = 0X00U;
    pDevice->pSPORTRegs->SPORT_CS3    = 0X00U;
    pDevice->pSPORTRegs->SPORT_ERRCTL = 0X00U;

    /* Create the semaphore  */
    /* create a semaphore for buffer management */
    if(adi_osal_SemCreateStatic(&pDevice->SemMemory[0],ADI_OSAL_MAX_SEM_SIZE_CHAR,&pDevice->hSemaphore,0U) != ADI_OSAL_SUCCESS)
    {
       SPORT_eSTATE(nDeviceNum) =   ADI_SPORT_STATE_UNINITIALIZED;
       return ADI_SPORT_FAILED;
    }
    /* Is the supplied memory is enough for operating in DMA mode */
    if(nMemSize >= ADI_SPORT_DMA_MEMORY_SIZE)
    {
         pDevice->pDMAChnlMem =  (uint8_t *)  pDevMemory;
    }
    /* Register the SPORT driver status interrupt handler with SEC */
    if(adi_int_InstallHandler(pDevice->pSportInfo->nStatusIntID,
                              SPORTStatusHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
    	SPORT_eSTATE(nDeviceNum) =   ADI_SPORT_STATE_UNINITIALIZED;
        return ADI_SPORT_INT_REGISTRATION_FAILED;
    }

    /* Register the SPORT driver Data interrupt handler with SEC */
    if(adi_int_InstallHandler(pDevice->pSportInfo->nDMAIntID,
                              sport_DataHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
        SPORT_eSTATE(nDeviceNum) =   ADI_SPORT_STATE_UNINITIALIZED;
        return ADI_SPORT_INT_REGISTRATION_FAILED;
    }

    /*Enable the interrupt for error in primary,secondary,and frame sync */
    pDevice->pSPORTRegs->SPORT_ERRCTL   |= (uint32_t) (BITM_SPORT_ERR_DERRPMSK|BITM_SPORT_ERR_DERRSMSK |BITM_SPORT_ERR_FSERRMSK);

    /* Set the default clock divide factor. Enable Divide By One (EDBO) is set by default.
     * It will be cleared when the mode is set to IrDA as it is not valid in that mode. */
    pDevice->pSPORTRegs->SPORT_DIV        =   DEFAULT_SPORT_CLOCK_VALUE;

    /* set  the SPORT control register to default value */
    pDevice->pSPORTRegs->SPORT_CTL        =   DEFAULT_SPORT_CONTROL_REG_VALUE;
    /* Set the direction of operation */
    if(eDirection == ADI_SPORT_DIR_TX)
    {
        pDevice->pSPORTRegs->SPORT_CTL   |=   (uint32_t)ENUM_SPORT_CTL_TX;
    }
    /*   set  the SPORT control register-2 to default value **/
    pDevice->pSPORTRegs->SPORT_CTL2       =   DEFAULT_SPORT_CONTROL_REG2_VALUE;
    /* Enable the Multi channel mode */
    if(eMode == ADI_SPORT_MC_MODE)
    {
        pDevice->pSPORTRegs->SPORT_MCTL |= (uint32_t)(ENUM_SPORT_MCTL_EN);
    }
    /* Enable I2S mode */
    if(eMode == ADI_SPORT_I2S_MODE)
    {
        pDevice->pSPORTRegs->SPORT_CTL  |= (uint32_t)(ENUM_SPORT_CTL_I2S_MODE);
    }

    /* Return the device handle to the application */
    *phDevice   =   pDevice;

    return ADI_SPORT_SUCCESS;
}

/**
 * @brief       Closes the operation of specified SPORT device. 
 *
 * @details     Closes the operation of specified SPORT device. Associated DMA will be closed if 
 *              the device is operating in DMA mode. Device need to be opened again for any further 
 *              use. 
 *
 * @param [in]  hDevice  SPORT device handle whose operation is to be closed. This handle was 
 *                       obtained when a SPORT device is opened successfully. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                      Successfully closed the specified SPORT device. 
 *
 *  - #ADI_SPORT_DMA_FAILED                   Failed to close the DMA associated with the specified 
 *                                            SPORT device. 
 *
 *  - #ADI_SPORT_INT_REGISTRATION_FAILED      Failed to unregister the specified sport device with 
 *                                            interrupt manager. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE          [D]  Invalid SPORT device handle. 
 *
 * @note        It is user's responsibility to free/reuse the memory supplied during the opening of 
 *              the device. 
 *
 * @sa          adi_sport_Open() 
 *
 */
ADI_SPORT_RESULT adi_sport_Close(
                                  ADI_SPORT_HANDLE  hDevice
                                )
  {
      /* Pointer to SPORT device instance */
      ADI_SPORT_DEVICE     *pDevice = (ADI_SPORT_DEVICE *)hDevice;
      /* Return code */
      ADI_SPORT_RESULT     eResult;

  #ifdef ADI_DEBUG
      /* Validate the given handle */
      if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
      {
          return eResult;
      }
  #endif /* ADI_DEBUG */

      /* Disable  the SPORT interrupts  */
      pDevice->pSPORTRegs->SPORT_ERRCTL    =  0X00U;

      /* Close the DMA Channel associated with the specified SPORT device */
      if(pDevice->hDMAChnl != NULL)
      {
          if(pDevice->pFunTable->pfDMAClose(pDevice->hDMAChnl,false) != ADI_DMA_SUCCESS)
          {
              return ADI_SPORT_DMA_FAILED;
          }
      }
      else
      {
          if(adi_int_UninstallHandler(pDevice->pSportInfo->nDMAIntID) != ADI_INT_SUCCESS)
          {
               return ADI_SPORT_INT_REGISTRATION_FAILED;
          }
      }
      /* Unregister the status interrupt handler */
      if(adi_int_UninstallHandler(pDevice->pSportInfo->nStatusIntID) != ADI_INT_SUCCESS)
      {
           return ADI_SPORT_INT_REGISTRATION_FAILED;
      }
      if(adi_osal_SemDestroyStatic(pDevice->hSemaphore) != ADI_OSAL_SUCCESS)
      {
          return(ADI_SPORT_FAILED);
      }
      /* Free up the device */
      adi_osal_EnterCriticalRegion();
      pDevice->pSportInfo->eState  = ADI_SPORT_STATE_UNINITIALIZED;
#if !defined(ADI_CODE_IN_ROM)
      pDevice->pSportInfo->hDevice = NULL;
#endif
      adi_osal_ExitCriticalRegion();
      return ADI_SPORT_SUCCESS;
  }

/**
 * @brief       This function Enable/Disable the specified SPORT device.Associated DMA with the 
 *              device will be enabled if the DMA mode is selected. 
 *
 * @param [in]  hDevice  Device handle to SPORT device obtained when a SPORT device opened 
 *                       successfully. 
 *
 * @param [in]  bEnable  Boolean flag to indicate whether to enable/disable the SPORT as below. 
 * \n                    
 * \n                    true : Enable the device for data transfer. 
 * \n                    
 * \n                    false : Disable the device for data transfer. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully enable/disable the specified SPORT and DMA 
 *                                          if the device is operating in DMA mode. 
 *
 *  - #ADI_SPORT_DMA_FAILED                 Failed to enable the DMA associated with the device. 
 *
 *  - #ADI_SPORT_BUFFERS_NOT_SUBMITTED      Buffers are not submitted for data transfer. 
 *
 *  - #ADI_SPORT_FAILED                     Failed to enable/disable the specified SPORT device and 
 *                                          DMA. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 * @sa          adi_sport_EnableDMAMode() 
 *
 */
ADI_SPORT_RESULT adi_sport_Enable(
                                   ADI_SPORT_HANDLE  hDevice,
                                   bool              bEnable
                                 )
{
    /* Pointer to SPORT device instance */
    ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    ADI_SPORT_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Check if the data flow is already enabled/disabled */
    if((ADI_SPORT_STATE_DATA_FLOW_ENABLED == pDevice->pSportInfo->eState) && (bEnable == true))
    {
        /* If it already in the required state do nothing */
        return ADI_SPORT_SUCCESS;
    }
    /*Enable the device ? */
    if(bEnable == true)
    {

        /* Check if buffers are submitted for both Ping and Pong descriptors */
        if(pDevice->pActiveDesc->pStartAddress == NULL)
        {
            return ADI_SPORT_BUFFERS_NOT_SUBMITTED;
        }

        /* When in DMA mode enable the DMA */
        if(pDevice->bDMAMode == true)
        {
            if(pDevice->pFunTable->pfEnableDMA(pDevice->hDMAChnl,pDevice->eDmaWordSize,pDevice->ePWordSize) != ADI_DMA_SUCCESS)
            {
                return ADI_SPORT_DMA_FAILED;
            }
        }

        pDevice->pSPORTRegs->SPORT_CTL |= (uint32_t)(ENUM_SPORT_CTL_EN);
        /* Enable the interrupt for the specified  SPORT */
        pDevice->pSportInfo->eState =   ADI_SPORT_STATE_DATA_FLOW_ENABLED;
    }
    /* Disable the device since bEnable is false */
    else
    {
        /* Disable the interrupt for the specified  SPORT */
        pDevice->pSPORTRegs->SPORT_CTL   &= (uint32_t)( ~(ENUM_SPORT_CTL_EN));
        if((pDevice->pSportInfo->eState == ADI_SPORT_STATE_PAUSED )&&(pDevice->bDMAMode == false ))
        {

            if(adi_sec_ClearPendStatus(pDevice->pSportInfo->nDMAIntID)!= ADI_SPORT_SUCCESS)
            {
                return(ADI_SPORT_INT_REGISTRATION_FAILED);
            }

            if(adi_int_EnableInt(pDevice->pSportInfo->nDMAIntID,true)!=ADI_INT_SUCCESS)
            {
                 return ADI_SPORT_INT_REGISTRATION_FAILED;
            }

        }
        /* When in DMA mode disable the DMA */
        if(pDevice->bDMAMode == true)
        {
            if((pDevice->pFunTable->pfDisableDMA(pDevice->hDMAChnl)) != ADI_DMA_SUCCESS)
            {
                return ADI_SPORT_DMA_FAILED;
            }
        }
        /* Reset the buffer pointer. Reset all registers. */
        InitDevice(pDevice);
        /* Set the state of the device */
        pDevice->pSportInfo->eState = ADI_SPORT_STATE_DATA_FLOW_DISABLED;
    }
    return ADI_SPORT_SUCCESS;
}

/**
 * @brief       Submit the buffer for transmitting/receiving the dataE. This function can 
 *              be used to submit the buffers for both transmitting and receiving. This function 
 *              will be returned after successfully submitting the buffer for transmitting the data. 
 *              User will be notified if an call back function is registered with an event code 
 *              #ADI_SPORT_EVENT_RX_BUFFER_PROCESSED or #ADI_SPORT_EVENT_RX_BUFFER_PROCESSED" 
 *              depending on the direction in which device is operating. 
 *
 * @param [in]  hDevice   Device handle to SPORT device is obtained when a SPORT device is opened 
 *                        successfully. 
 *
 * @param [in]  pBuffer   Pointer to buffer from where data need to be transmitted OR to which 
 *                        received data need to to be written. 
 *
 * @param [in]  nBufSize  Size of the data to be transmitted (In bytes)/ received. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS             Successfully submitted the buffer to the specified device. 
 *
 *  - #ADI_SPORT_FAILED              Failed to submit the buffer for data transmission. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE [D]  Invalid SPORT device handle. 
 *
 * @sa          adi_sport_GetBuffer() 
 *
 */
ADI_SPORT_RESULT adi_sport_SubmitBuffer(
                                         ADI_SPORT_HANDLE  hDevice,
                                         void             *pBuffer,
                                         uint32_t          nBufSize
                                       )
{
    /* Pointer to SPORT device instance */
    ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    ADI_SPORT_RESULT         eResult;


#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */
    /* call an appropriate function based on mode in which device is operating */
    eResult = pDevice->pFunTable->pfSubmitBuffer(hDevice,pBuffer,nBufSize);
    return(eResult);
}

/**
 * @brief       Gets an "empty" or "filled" buffer from SPORT device manager. This function returns 
 *              the address of a processed buffer. This is a blocking function and waits till an 
 *              buffer is avilable. This function returns error if an callback function is 
 *              registered. #adi_sport_IsBufferAvailable can be used as a peek function to know 
 *              whether an buffer is avilable. 
 *
 * @param [in]  hDevice   Device handle to SPORT device, obtained when a SPORT device is opened 
 *                        successfully. 
 *
 * @param [out] ppBuffer  Pointer to a location where the the address of the buffer is to be 
 *                        written. Contains the address of an "empty" buffer (i.e the content of the 
 *                        buffer is transmitted) OR "filled" buffer which contains the received 
 *                        data. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS               Successfully returned. ppBuffer points to the address of the 
 *                                     buffer. 
 *
 *  - #ADI_SPORT_FAILED                Failed to get the buffer since device is operating in call 
 *                                     back mode. ppBuffer points NULL. 
 *
 *  - #ADI_SPORT_PERIPHERAL_ERROR      Device encountered an peripheral error. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE   [D]  Invalid SPORT device handle. 
 *
 * @sa          adi_sport_SubmitBuffer() 
 * @sa          adi_sport_IsBufferAvailable() 
 *
 */
ADI_SPORT_RESULT adi_sport_GetBuffer(
                                      ADI_SPORT_HANDLE  const hDevice,
                                      void            **ppBuffer
                                    )
{
    /* Pointer to SPORT device instance */
    ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    ADI_SPORT_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    *ppBuffer = NULL;

    if(pDevice->pfCallback != NULL)
    {
       return(ADI_SPORT_FAILED);
    }
    if(adi_osal_SemPend(pDevice->hSemaphore,ADI_OSAL_TIMEOUT_FOREVER) == ADI_OSAL_SUCCESS)
    {

        if(pDevice->nHwError != (uint32_t)ADI_SPORT_HW_ERR_NONE)
        {
            return(ADI_SPORT_PERIPHERAL_ERROR);
        }
        adi_osal_EnterCriticalRegion();
        *ppBuffer  = pDevice->pProcDesc->pStartAddress;
        /* set the address as NULL */
        pDevice->pProcDesc->pStartAddress = NULL;
        pDevice->pProcDesc = pDevice->pProcDesc->pNext;
        adi_osal_ExitCriticalRegion();
        return(ADI_SPORT_SUCCESS);
    }
    else
    {
        return(ADI_SPORT_FAILED);
    }
}

/**
 * @brief       Peek function to know whether an empty/filled buffer is avilable. Call to this 
 *              function is valid only if the call back function is not registered. Call to this 
 *              function results in error if an call back function is registered. 
 *
 * @param [in]  hDevice      Device handle to SPORT device obtained when a SPORT device is opened 
 *                           successfully. 
 *
 * @param [out] pbAvailable  Pointer to a boolean variable. Contains "True" if there is an 
 *                           empty/filled buffer and a call to #adi_sport_GetBuffer is ensured to be 
 *                           successful. Contains "false" if there is no empty buffer. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS             Successfully retrieved the status of availability of the 
 *                                   buffer. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE [D]  Failed to retrieve the status of the buffer availability. 
 *
 * @sa          adi_sport_GetBuffer() 
 * @sa          adi_sport_GetBuffer() 
 *
 */
ADI_SPORT_RESULT    adi_sport_IsBufferAvailable(
                                                 ADI_SPORT_HANDLE const hDevice,
                                                 bool            *const pbAvailable
                                               )
{
    /* Pointer to SPORT device instance */
    ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    *pbAvailable = false;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if(ValidateHandle(pDevice) != ADI_SPORT_SUCCESS)
    {
        return ADI_SPORT_INVALID_HANDLE;
    }

#endif /* ADI_DEBUG */
    /* An empty/filled buffer is avilable since "bInuse" flag is false */
    if((pDevice->pProcDesc->pStartAddress != NULL) && (pDevice->pProcDesc->bInUse == false))
    {
        *pbAvailable =true;
    }
    return(ADI_SPORT_SUCCESS);
}

/**
 * @brief       Registers a Callback function with the SPORT device driver. The registered call 
 *              back function will be called when an buffer is processed OR hardware error(s) 
 *              encountered. 
 *
 * @param [in]  hDevice     Device handle to SPORT device is obtained when a SPORT device is opened 
 *                          successfully. 
 *
 * @param [in]  pfCallback  Function pointer to Callback function. Passing a NULL pointer will 
 *                          unregister the call back function. 
 *
 * @param [in]  pCBparam    Call back function parameter. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully registered specified callback function. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */
ADI_SPORT_RESULT adi_sport_RegisterCallback(
                                              ADI_SPORT_HANDLE  hDevice,
                                              ADI_CALLBACK      pfCallback,
                                              void             *pCBparam
                                           )
{
    /* Pointer to SPORT device instance */
    ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
#ifdef ADI_DEBUG
    ADI_SPORT_RESULT eResult;
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    /* Check if the data flow is already enabled */
    if(ADI_SPORT_STATE_DATA_FLOW_ENABLED == pDevice->pSportInfo->eState )
    {
        /* Not allowed to register a callback if the data flow is enabled. */
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }

#endif /* ADI_DEBUG */
    /* Store the address of the callback function */
    pDevice->pfCallback =  pfCallback;
    /* Store the call back parameter */
    pDevice->pCBParam    =  pCBparam;
    /* Return success */
    return(ADI_SPORT_SUCCESS);
}

/**
 * @brief       Enable the DMA mode associated with specified SPORT device. 
 *
 * @param [in]  hDevice         Device handle to SPORT device is obtained when a SPORT device is 
 *                              opened successfully. 
 *
 * @param [in]  bEnableDMAmode  Boolean flag which indicates whether to enable/disable the DMA mode 
 *                              for the device. 
 * \n                           
 * \n                           true : Enable the DMA mode. 
 * \n                           
 * \n                           false : Disable the DMA mode. This makes it to work with interrupt 
 *                              mode. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                      Successfully enable/disabled the DMA mode of 
 *                                            operation for the specified SPORT. 
 *
 *  - #ADI_SPORT_DMA_FAILED                   Failed to enable/disable the DMA associated with 
 *                                            specified SPORT. 
 *
 *  - #ADI_SPORT_INT_REGISTRATION_FAILED      Failed to register/unregister device with interrupt 
 *                                            manager. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE          [D]  Invalid SPORT device handle. 
 *
 * @sa          adi_sport_Enable() 
 *
 */
ADI_SPORT_RESULT adi_sport_EnableDMAMode(
                                          ADI_SPORT_HANDLE  hDevice,
                                          bool              bEnableDMAmode
                                        )
{
    /* Pointer to SPORT device instance */
    ADI_SPORT_DEVICE     *pDevice = (ADI_SPORT_DEVICE *)hDevice;
#ifdef ADI_DEBUG
    if(pDevice->pDMAChnlMem == NULL)
    {
        return(ADI_SPORT_FAILED);
    }
#endif
    /* check whether DMA is already opened for the specified device */
    if(true == bEnableDMAmode)
    {
         /* Uninstall the current interrupt  handler */
         if(adi_int_UninstallHandler( pDevice->pSportInfo->nDMAIntID) != ADI_INT_SUCCESS)
         {
             return(ADI_SPORT_INT_REGISTRATION_FAILED);
         }

         /*  open the DMA channel*/
         if(adi_pdma_Open(pDevice->pSportInfo->eDMAChnlID,
                          pDevice->pDMAChnlMem ,
                         &pDevice->hDMAChnl,
                          SPORTDMAHandler,
                         (void *)pDevice) != ADI_DMA_SUCCESS)

        {
             return ADI_SPORT_DMA_FAILED;
        }
        /* Set the function table corresponding to DMA mode of operation  */
        if(adi_pdma_SetSync(pDevice->hDMAChnl,true) != ADI_DMA_SUCCESS)
         {
             return(ADI_SPORT_DMA_FAILED);
         }
     
#if !defined(ADI_CODE_IN_ROM)
        pDevice->pFunTable    =  (SPORT_FUNCTION_TABLE *)&gSportDMAFuncTable;
#else
        pram_for_rom_Code->gSportDMAFuncTable.pfDMAClose  =  adi_pdma_Close;
        pram_for_rom_Code->gSportDMAFuncTable.pfSubmitBuffer  =  &SubmitBufferDmaMode;
        pram_for_rom_Code->gSportDMAFuncTable.pfEnableDMA  =  &EnableDma;
        pram_for_rom_Code->gSportDMAFuncTable.pfDisableDMA  =  adi_pdma_DisableXfer;

        pDevice->pFunTable    =  &pram_for_rom_Code->gSportDMAFuncTable;
#endif

    }
    else
    {
         /* Uninstall the current interrupt handler */
         if(adi_int_UninstallHandler( pDevice->pSportInfo->nDMAIntID) != ADI_INT_SUCCESS)
         {
             return(ADI_SPORT_INT_REGISTRATION_FAILED);
         }
         /* Install the Data interrupt handler for operating in interrupt mode */
        if(adi_int_InstallHandler( pDevice->pSportInfo->nDMAIntID,
                                  sport_DataHandler,
                                  pDevice,
                                  true) != ADI_INT_SUCCESS)
        {
             /* return the error */
             return ADI_SPORT_INT_REGISTRATION_FAILED;
        }
        /* Install interrupt mode function table */
        pDevice->pFunTable    =  (SPORT_FUNCTION_TABLE *)&gSportIntFuncTable;
    }
    pDevice->bDMAMode  = bEnableDMAmode;
    return(ADI_SPORT_SUCCESS);
}

/**
 * @brief       Sets data format for the specified SPORT device. 
 *
 * @details     Sets data type,Big endian (MSB first) OR Little endian (LSB first) and word 
 *              length(in bits) for the specified SPORT device.This function return error if the 
 *              device is already enabled. 
 *
 * @param [in]  hDevice              Device handle to SPORT device. 
 *
 * @param [in]  eDataType            Specify the data type to be used. 
 *
 * @param [in]  nWordLength          Specify the word size of the data. Valid range is from 
 *                                   4(nWordLength = 3) to 32(nWordLength =31). For Stereo and 
 *                                   Multichannel mode the range is between 5 and 32. 
 *
 * @param [in]  bLSBFirst            Configure the specified SPORT device to operate either LSB 
 *                                   first or MSB first. 
 * \n                                
 * \n                                true : LSB first (Little endian) . 
 * \n                                
 * \n                                false : MSB first (Big endian) 
 *
 * @param [in]  bEnablePack          Boolean flag to indicate whether to Enable/Disable the packing 
 *                                   from 16bit 32 bit. By default it is enabled. 
 * \n                                
 * \n                                true : Enable DMA packing (from 16bit To 32bit) . 
 * \n                                
 * \n                                false : Disable DMA packing 
 *
 * @param [in]  bRightJustifiedMode  Enable/disable the Right Justified mode. Valid only in I2S 
 *                                   mode and ignored in other modes. 
 * \n                                
 * \n                                true : Enable Right Justified mode. 
 * \n                                
 * \n                                false : Disble Right Justified mode. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully configured the device to operate in 
 *                                          specified data format. 
 *
 *  - #ADI_SPORT_FAILED                     Failed set the specified data format. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_INVALID_WORD_LENGTH   [D]  Invalid word size. 
 *
 *  - #ADI_SPORT_INVALID_DATA_TYPE     [D]  Invalid word size. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */

ADI_SPORT_RESULT adi_sport_ConfigData(
                                                ADI_SPORT_HANDLE   hDevice,
                                             ADI_SPORT_DATATYPE eDataType,
                                       uint8_t            nWordLength,
                                       bool               bLSBFirst,
                                       bool               bEnablePack,
                                       bool               bRightJustifiedMode
                                     )
{
     /* Return code */
     ADI_SPORT_RESULT        eResult;
     uint32_t nDataCfg=0U;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    /* Not allowed to change when data flow is enabled */
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }
    /* Check for valid data type  */
    if(eDataType > ADI_SPORT_DTYPE_COMPAND_A_LAW)
    {
        return ADI_SPORT_INVALID_DATA_TYPE;
    }
#endif /* ADI_DEBUG */
    /* clear all the fields(i.e Set to "0" ) */
    pDevice->pSPORTRegs->SPORT_CTL &=(uint32_t) (~(BITM_SPORT_DATA_CONFIG));
    if(nWordLength <= SPORT_BYTE_TRANSFER_LENGTH )
    {
        pDevice->eDmaWordSize = ADI_DMA_MSIZE_1BYTE;
        pDevice->ePWordSize   = ADI_DMA_PSIZE_1BYTE;
        pDevice->nXmodify =(int32_t) sizeof(uint8_t);
        pDevice->nShiftValue =0U;
    }
    else if(nWordLength <= SPORT_HALFWORD_TRANSFER_LENGTH )
    {
        pDevice->eDmaWordSize = ADI_DMA_MSIZE_2BYTES;
        pDevice->ePWordSize   = ADI_DMA_PSIZE_2BYTES;
        pDevice->nXmodify = (int32_t)sizeof(uint16_t);
        pDevice->nShiftValue =1U;
    }
    else  if (nWordLength <= SPORT_WORD_TRANSFER_LENGTH)
    {
        pDevice->eDmaWordSize = ADI_DMA_MSIZE_4BYTES;
        pDevice->ePWordSize   = ADI_DMA_PSIZE_4BYTES;
        pDevice->nXmodify = (int32_t)sizeof(uint32_t);
        pDevice->nShiftValue =2U;
    }
    else
    {
        return ADI_SPORT_INVALID_WORD_LENGTH;
    }

    if(bLSBFirst == true)
    {
        /* set the the LSB first field */
        nDataCfg |= (uint32_t)ENUM_SPORT_CTL_LSB_FIRST;
    }
    if(bEnablePack == true)
    {
        /* Enable the packing */        
        nDataCfg |=(uint32_t) ENUM_SPORT_CTL_PACK_EN;
    }
    if(bRightJustifiedMode  == true)
    {
        /* Enable the right justified mode. */                
        nDataCfg |=(uint32_t) ENUM_SPORT_CTL_RJUST_EN;
    }
    /* Form the configuration word by appending the Data-type and wordlength */
    nDataCfg |=((uint32_t)((uint32_t)eDataType << BITP_SPORT_CTL_DTYPE) | ((uint32_t)nWordLength << BITP_SPORT_CTL_SLEN));

    pDevice->pSPORTRegs->SPORT_CTL |= nDataCfg;
    return(ADI_SPORT_SUCCESS);
    
}

/**
 * @brief       Configure the clock for the specified SPORT device. 
 *
 * @details     Configure the SPORT device to use the "internal/external " rising/falling clock 
 *              edge,clock edge and for enabling the gated Clock Mode. 
 *
 * @details     fspclk = fsclk/( nClockRatio + 1) 
 *
 * @details     fspclk: frequency of sport clock 
 *
 * @param [in]  hDevice        Device handle to SPORT device. 
 *
 * @param [in]  nClockRatio    The value which determines the ratio between System clock and sport 
 *                             clock as explained below. 
 *
 *
 * @param [in]  bUseIntlClock  Boolean flag to indicate whether to use internal clock or external 
 *                             clock for data transmission. By default, device is configured to use 
 *                             the external clock. 
 * \n                          
 * \n                          true : Device configured to use Internal clock. 
 * \n                          
 * \n                          false : Device configured to use external clock.. 
 *
 * @param [in]  bFallingEdge   Boolean flag to indicate whether to drive data and internal frame 
 *                             sync with rising edge OR falling edge of SP clock. 
 * \n                          
 * \n                          true : Use falling edge of the clock. 
 * \n                          
 * \n                          false : Use rising edge of the clock. 
 *
 * @param [in]  bGatedClk      Boolean flag to indicate whether to enable/disable gated clock for 
 *                             the specified SPORT channel.Ignored in Multi channel mode. Clock will 
 *                             be active only when active data is getting transmitted or received 
 *                             when this mode is enabled. 
 * \n                          true : Enable gated clock mode. 
 * \n                          
 * \n                          false : Disable gated clock mode. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully configured clock for the specified device. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */
ADI_SPORT_RESULT adi_sport_ConfigClock(
                                        ADI_SPORT_HANDLE  hDevice,
                                        uint16_t          nClockRatio,
                                        bool              bUseIntlClock,
                                        bool              bFallingEdge,
                                        bool              bGatedClk
                                      )

{

    /* Return code */
     ADI_SPORT_RESULT        eResult;
     uint32_t nClkCfg=0U;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }

#endif /* ADI_DEBUG */

    pDevice->pSPORTRegs->SPORT_DIV &= (uint32_t)(~(uint32_t)(BITM_SPORT_DIV_CLKDIV));
    /* Set the clock divisior */
    pDevice->pSPORTRegs->SPORT_DIV |= (uint32_t)((uint32_t) nClockRatio & BITM_SPORT_DIV_CLKDIV);

    /* clear all clock configuration  fields */
    pDevice->pSPORTRegs->SPORT_CTL &= (uint32_t)(~(uint32_t)(BITM_SPORT_CLOCK_CONFIG));
    if(bUseIntlClock == true)
    {
        /* select the internal clock */
        nClkCfg  |= (uint32_t)ENUM_SPORT_CTL_INTERNAL_CLK;
    }
    if(bFallingEdge == true)
    {
        /* select the rising edge of the  clock */
        nClkCfg |= (uint32_t)ENUM_SPORT_CTL_CLK_RISE_EDGE;
    }
    if((bGatedClk  == true )&& (pDevice->eMode != ADI_SPORT_MC_MODE))
    {
        /* Enable the Gated clock */        
        nClkCfg |= (uint32_t)ENUM_SPORT_CTL_GCLK_EN;
    }
    /* set the configuration word */
    pDevice->pSPORTRegs->SPORT_CTL |= nClkCfg;
    /* Return from the function */
    return (ADI_SPORT_SUCCESS);
}

/**
 * @brief       Frame Sync(FS) configuration for the specified sport. . 
 *
 * @details     Configure the SPORT to use internal/external frame sync,level/edge sensitive 
 *              early/late frame sync etc. 
 *
 * @param [in]  hDevice           Device handle to SPORT device. 
 *
 * @param [in]  nFsDivisor        The value which decides the number of sport clock cycles between
 *                                each frame count.
 *
 * @param [in]  bFSRequired       Boolean flag to indicate whether frame sync required OR not to 
 *                                the frame sync for the data transfer. 
 * \n                             
 * \n                             true : Device requires a frame sync for its operation. 
 * \n                             
 * \n                             false : Device does not requires a frame sync for its operation 
 * \n                             
 * \n                             
 *
 * @param [in]  bInternalFS       Boolean flag to indicate whether to configure the specified SPORT 
 *                                device to use the internal frame sync OR external frame sync as 
 *                                below. 
 * \n                             
 * \n                             true : Use internal frame sync. 
 * \n                             
 * \n                             false : Use external frame sync 
 * \n                             
 * \n                             
 *
 * @param [in]  bDataFS           Boolean flag to indicate whether to configure the specified SPORT 
 *                                device to use the data-independent frame sync OR Serial port uses 
 *                                a data-dependent frame sync. Valid only if the specified device is 
 *                                in "transmit"(TX)mode . Ignored if the device is opened in 
 *                                "receive"(RX) mode. 
 * \n                             
 * \n                             true : Use data-independent frame sync. 
 * \n                             
 * \n                             false : Use data-dependent frame sync. 
 * \n                             
 * \n                             
 *
 * @param [in]  bActiveHighFS     Boolean flag to indicate whether to configure the specified SPORT 
 *                                device for active low frame sync OR active high frame sync. Call 
 *                                to this function will return error if SPORT is configured in I2S 
 *                                mode. 
 * \n                             
 * \n                             true : Use active low frame sync. 
 * \n                             
 * \n                             false : Use active high frame sync. 
 * \n                             
 * \n                             
 *
 * @param [in]  bLateFS           Boolean flag to indicate whether to use the late frame sync OR 
 *                                Early frame sync. 
 * \n                             
 * \n                             true : Use late frame sync. 
 * \n                             
 * \n                             false : Use Early frame sync. 
 * \n                             
 * \n                             
 *
 * @param [in]  bEdgeSensitiveFS  Boolean flag to indicate whether frame sync is edge sensitive OR 
 *                                level sensitive. 
 * \n                             
 * \n                             true : Framesync is edge sensitive. 
 * \n                             
 * \n                             false : Framesync is level sensitive. 
 * \n                             
 * \n                             
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully configured the frame sync requirement. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */
ADI_SPORT_RESULT adi_sport_ConfigFrameSync(
                                             ADI_SPORT_HANDLE  hDevice,
                                             uint16_t          nFsDivisor,
                                             bool              bFSRequired,
                                             bool              bInternalFS,
                                             bool              bDataFS,
                                             bool              bActiveHighFS,
                                             bool              bLateFS,
                                             bool              bEdgeSensitiveFS
                                           )
{
    /* Return code */
     ADI_SPORT_RESULT        eResult;
     uint32_t nFsCfg=0U;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }

#endif /* ADI_DEBUG */
    pDevice->pSPORTRegs->SPORT_DIV &= (uint32_t)(~(uint32_t)(BITM_SPORT_DIV_FSDIV));

    pDevice->pSPORTRegs->SPORT_DIV |= (uint32_t)(((uint32_t)nFsDivisor << BITP_SPORT_DIV_FSDIV));

    /* clear all the fields of frame sync */
    pDevice->pSPORTRegs->SPORT_CTL &=(uint32_t)( ~(BITM_SPORT_FS_CONFIG));
    /* Select the data independent frame sync. This bit should be set when device is opened in 
       RX mode */
    if((pDevice->eDirection == ADI_SPORT_DIR_RX )||( bDataFS == true) )
    {
         nFsCfg = (uint32_t)ENUM_SPORT_CTL_DATA_INDP_FS ;
    }
    /* "Frame sync required" is reserved when device is operating in I2S and MC mode */
    if((bFSRequired == true) && (pDevice->eMode == ADI_SPORT_SERIAL_MODE))
    {
        /* Frame Sync(FS) is required */
        nFsCfg  |= (uint32_t)ENUM_SPORT_CTL_FS_REQ;
    }
    if(bInternalFS == true)
    {
        /* Select the internal Frame Sync(FS)*/        
        nFsCfg |= (uint32_t)ENUM_SPORT_CTL_INTERNAL_FS;
    }
    if(bActiveHighFS == true)
    {
        /* Select the Active High Frame Sync(FS)*/                 
        nFsCfg  |=(uint32_t) ENUM_SPORT_CTL_FS_HI;
    }
    if(bLateFS == true)
    {
        /* Select the Late  Frame Sync(FS)*/                         
        nFsCfg |= (uint32_t)ENUM_SPORT_CTL_LATE_FS;
    }
    if(bEdgeSensitiveFS == true)
    {
        /* Select the edge sensitive   Frame Sync(FS)*/                                 
        nFsCfg  |= (uint32_t)ENUM_SPORT_CTL_EDGE_FS;
    }
    /* write the Frame sync configuration to control register */
    pDevice->pSPORTRegs->SPORT_CTL |= nFsCfg;
    /* return success */
    return ADI_SPORT_SUCCESS;        
}
/**
 * @brief       Configure the SPORT for multi channel operation. 
 *
 * @param [in]  hDevice         Device handle to SPORT device. 
 *
 * @param [in]  nFrameDelay     Specify the interval, in number of serial clock cycles between the 
 *                              multichannel frame sync pulse and the first data bit. Valid values 
 *                              range from 0 to 15. 
 *
 * @param [in]  nNumSlots       Specify the number of multi channel solts.Valid range is between 1 
 *                              - 128. 
 *
 * @param [in]  nWindowSize     Specify the window offset The nWindowSize specifies where in the 
 *                              1024-channel range to place the start of the active window. Valid 
 *                              range is between 0-896 
 *
 * @param [in]  bEnableDMAPack  Boolean flag to indicate whether to enable(true)/disable(false) 
 *                              multi channel DMA data packing. 
 * \n                           
 * \n                           true : Enable DMA data packing in Multichannel mode. 
 * \n                           
 * \n                           false : Disable DMA data packing in Multichannel mode. 
 * \n                           
 * \n                           
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully set the multichannel configuration 
 *                                          parameters. 
 *
 *  - #ADI_SPORT_FAILED                     Failed configure the SPORT in multi channel mode. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_INVALID_FRAME_DELAY   [D]  Invalid frame delay. 
 *
 *  - #ADI_SPORT_INVALID_SLOT_NUM      [D]  Invalid slot number. 
 *
 *  - #ADI_SPORT_INVALID_WINDOW_OFFSET [D]  Invalid window offset. 
 *
 *  - #ADI_SPORT_MODE_NOT_PERMITTED    [D]  Device is not opened in multi channel mode. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */

ADI_SPORT_RESULT adi_sport_ConfigMC(
                                     ADI_SPORT_HANDLE  hDevice,
                                     uint8_t           nFrameDelay,
                                     uint8_t           nNumSlots,
                                     uint16_t          nWindowSize,
                                     bool              bEnableDMAPack
                                   )
{
     /* Return code */
     ADI_SPORT_RESULT        eResult;
     uint32_t nMCcfg = 0U;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->eMode != ADI_SPORT_MC_MODE)
    {
        return ADI_SPORT_MODE_NOT_PERMITTED;
    }
    if(nWindowSize > SPORT_MAX_WINDOW_SIZE)
    {
        return ADI_SPORT_INVALID_WINDOW_OFFSET;
    }
    if((nNumSlots > SPORT_MAX_NUM_MC_SLOTS)||(nNumSlots == 0U))
    {
        return ADI_SPORT_INVALID_SLOT_NUM;
    }
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }

    if(nFrameDelay > SPORT_MAX_FRM_DELAY)
    {
        return ADI_SPORT_INVALID_FRAME_DELAY;
    }
#endif /* ADI_DEBUG */
    pDevice->pSPORTRegs->SPORT_MCTL &= (uint32_t)(~(BITM_SPORT_MC_CONFIG));
    if(bEnableDMAPack  == true)
    {
        /* Enable the DMA packing in MC mode */
        nMCcfg   = (uint32_t)(BITM_SPORT_MCTL_MCPDE);
    }
    nMCcfg |=  ((uint32_t) nFrameDelay <<BITP_SPORT_MCTL_MFD)       |
               ( (uint32_t)nNumSlots <<BITP_SPORT_MCTL_WSIZE)       |
               ( (uint32_t)nWindowSize <<BITP_SPORT_MCTL_WOFFSET )  |
                 ( (uint32_t)ENUM_SPORT_MCTL_EN );
    pDevice->pSPORTRegs->SPORT_MCTL = nMCcfg;
    /* return success */
    return ADI_SPORT_SUCCESS;

}

/**
 * @brief       Selects specified channel(s) for MC operation. 
 *
 * @details     Sets the channel number for MC operation. This function call is valid only if the 
 *              SPORT is operating in Multi channel mode. 
 *
 * @param [in]  hDevice     Device handle to SPORT device is obtained when a SPORT device is opened 
 *                          successfully. 
 *
 * @param [in]  nStChnlNo   Starting channel number. 
 *
 * @param [in]  nEndChnlNo  End channel number. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Enabled the specified channel(s) for multi channel 
 *                                          operations. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_MODE_NOT_PERMITTED    [D]  Device is not opened in multi channel mode. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 * @sa          adi_sport_ConfigMC() 
 *
 */
ADI_SPORT_RESULT adi_sport_SelectChannel(
                                          ADI_SPORT_HANDLE  hDevice,
                                          uint32_t          nStChnlNo,
                                          uint32_t          nEndChnlNo
                                        )
{
     /* Return code */
     ADI_SPORT_RESULT        eResult;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;
     uint32_t i,nIndex,nChPosition,nChEnable=0U;
     volatile uint32_t *pCSReg;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    /* validate the channels specified */
    if((nStChnlNo >MAX_MULTI_CHANNEL_NUM )|| (nEndChnlNo >MAX_MULTI_CHANNEL_NUM))
    {
        return ADI_SPORT_INVALID_CHANNEL_NUM;
    }
    /* Can not select the channels when data flow is enabled */
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }
    /* Channel selection should  be done only in multi-channel mode. */
    if(pDevice->eMode != ADI_SPORT_MC_MODE)
    {
        return ADI_SPORT_MODE_NOT_PERMITTED;
    }
#endif /* ADI_DEBUG */
    pCSReg  = &pDevice->pSPORTRegs->SPORT_CS0;
    /* Index =nStChnlNo / 32 since each CSx register can enable/disable 32 channels */
    /* calculate the index to select the first CSx register */
    nIndex =nStChnlNo >>5;
    /* Adde the index address of CS0  register */
    pCSReg += nIndex;
    /* select the statrting channel with the CSx register */
    nChPosition = nStChnlNo - (nIndex<<5);
    /* Loop over number of channels */
    for(i=nStChnlNo;i<=nEndChnlNo;i++)
    {
        /* Enable the specified channel */
        nChEnable |= ((uint32_t)1U << nChPosition);
        nChPosition++;
        /* check whether last bit in the  CSx register reached */
        if(((( nChEnable)&0X80000000U)==0X80000000U )|| (i == nEndChnlNo) )
        {
          /* Increment the register pointer and reset the position */
         *pCSReg |=nChEnable;
          pCSReg++;
          nChEnable = nChPosition = 0U;
        }
    }
    return ADI_SPORT_SUCCESS;
}
/**
 * @brief       Select Right channel/ Left channel first in I2S mode. 
 *
 * @param [in]  hDevice        Device handle to SPORT device. 
 *
 * @param [in]  bRightChFirst  Boolean flag to indicate whether configure left channel first or 
 *                             right channel first.Valid only when device is operating in I2S mode. 
 * \n                          true : Right channel first. 
 * \n                          
 * \n                          false : Left channel first. 
 * \n                          
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully configured which channel is first in I2S 
 *                                          mode. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_MODE_NOT_PERMITTED    [D]  Device is not opened in I2S mode. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */
ADI_SPORT_RESULT adi_sport_SelectChannelOrder(
                                               ADI_SPORT_HANDLE  hDevice,
                                               bool              bRightChFirst
                                             )
{
    /* Return code */
     ADI_SPORT_RESULT        eResult;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }
    if(pDevice->eMode != ADI_SPORT_I2S_MODE)
    {
       return ADI_SPORT_MODE_NOT_PERMITTED;
    }
#endif /* ADI_DEBUG */
    if(bRightChFirst == true)
    {
        /* Select the right channel first */
        pDevice->pSPORTRegs->SPORT_CTL |= (uint32_t) (ENUM_SPORT_CTL_FS_HI);
    }
    else
    {
        /* Select the Left channel first */
        pDevice->pSPORTRegs->SPORT_CTL &=(uint32_t)( ~(ENUM_SPORT_CTL_FS_HI));
    }
    /* return success */
    return ADI_SPORT_SUCCESS;
}

/**
 * @brief       This function helps to find the Hardware error(s) encountered by the specified 
 *              sport device. 
 *
 * @param [in]  hDevice   Device handle to SPORT device, obtained when a SPORT device opened 
 *                        successfully. 
 *
 * @param [out] pHwError  Pointer to a variable where the error to be written. 
 *
 * @param [out] pArg      Additional parameter passed to the user. For SPORT device it is NULL 
 *                        since there is nothing to be sent to user. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully returned hardware error(s) occurred by the 
 *                                          specified device. 
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid sport device handle.
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Not allowed to be called when the call back function is registered.
 *
 */
ADI_SPORT_RESULT adi_sport_GetHWErrorStatus (
                                               ADI_SPORT_HANDLE  const  hDevice,
                                               uint32_t                *pHwError,
                                               void *                  *pArg
                                            )
{
    /* Pointer to SPORT device instance */
    register ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    ADI_SPORT_RESULT         eResult;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->pfCallback != NULL)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }    
    
#endif /* ADI_DEBUG */
     /* Return the encountered Hardware error(s) */
    *pHwError = pDevice->nHwError;
    *pArg     = NULL;
    /* Clear */
    pDevice->nHwError  = (uint32_t) ADI_SPORT_HW_ERR_NONE;
    /* Return success */
    return(ADI_SPORT_SUCCESS);
}
/**
 * @brief       Enable the secondary channel (channel B) of the SPORT. 
 *
 * @param [in]  hDevice           Device handle to SPORT device. 
 *
 * @param [in]  bEnableSecondary  Boolean flag to indicate whether to enable(true)/disable(false) 
 *                                the channel B(secondary channel) of the SPORT. By default 
 *                                Secondary channel is enabled. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully enabled/disabled channel B. 
 *
 *  - #ADI_SPORT_FAILED                     Failed enable/disable channel B of the specified sport. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */
ADI_SPORT_RESULT adi_sport_EnableSecondary(
                                             ADI_SPORT_HANDLE  hDevice,
                                             bool              bEnable
                                          )
{

    /* Return code */
     ADI_SPORT_RESULT        eResult;
     uint32_t nFsCfg;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }

#endif /* ADI_DEBUG */
    if(bEnable == true)
    {
        /* Enable the secondary channel */
        pDevice->pSPORTRegs->SPORT_CTL |=  (uint32_t)(ENUM_SPORT_CTL_SECONDARY_EN);
    }
    else
    {
        /* Disable  the secondary channel */
        pDevice->pSPORTRegs->SPORT_CTL &=(uint32_t)( ~(ENUM_SPORT_CTL_SECONDARY_EN));
    }
    /* return success */
    return ADI_SPORT_SUCCESS;

}
/**
 * @brief       Configure the SPORT use the Clocks and Frame Sync of other Half-Sport 
 *
 * @param [in]  hDevice       Device handle to SPORT device. 
 *
 * @param [in]  bUseOtherFS   Boolean flag to indicate whether to use own Frame Sync(false) OR to 
 *                            use frame sync of other half sport (true). 
 * \n                         
 * \n                         true : Use frame sync of other half sport device. 
 * \n                         
 * \n                         false : Use own frame sync. 
 *
 * @param [in]  bUseOtherClk  Boolean flag to indicate whether to use own clock clock(false) OR to 
 *                            use clock of other half sport(true). 
 * \n                         
 * \n                         true : Use clock of other half sport device. 
 * \n                         
 * \n                         false : Use own clock. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully enabled the specified SPORT to use the clk 
 *                                          and FS of other half sport. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */
ADI_SPORT_RESULT adi_sport_MuxHalfSport(
                                          ADI_SPORT_HANDLE  hDevice,
                                          bool              bUseOtherFS,
                                          bool              bUseOtherClk
                                        )
{
     /* Return code */
     ADI_SPORT_RESULT        eResult=ADI_SPORT_FAILED;
     uint32_t nCfg=0U;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }

    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }

#endif /* ADI_DEBUG */
     /* clear the muxing fields of the control register 2 */
    pDevice->pSPORTRegs->SPORT_CTL2 &=(uint32_t)(~(BITM_SPORT_MUX_CONFIG));
    if(bUseOtherFS  == true)
    {
        /* Use the  the frame sync of  other half sport*/
        nCfg    |= (uint32_t) (BITM_SPORT_CTL2_FSMUXSEL);
    }
    if(bUseOtherClk  == true)
    {
        /* Use the  the clock of  other half sport*/
        nCfg    |=  (uint32_t)(ENUM_SPORT_CTL2_CLK_MUX_EN);
    }

    pDevice->pSPORTRegs->SPORT_CTL2 =nCfg;

    return(ADI_SPORT_SUCCESS);
}

/**
 * @brief       Configure the SPORT loopback mode. i.e To use the data,frame sync and clock of 
 *              neighbouring half sport.If low, own data, frame sync and clock is used(Default 
 *              condition). 
 *
 * @param [in]  hDevice  Device handle to SPORT device. 
 *
 * @param [in]  bEnable  Boolean flag to indicate whether to enable(true) or disable(false) the 
 *                       loopback mode for the specified SPORT device.Valid only when device is 
 *                       operating in receive( #ADI_SPORT_DIR_RX) mode. 
 * \n                    
 * \n                    true : Enable loop back . 
 * \n                    
 * \n                    false : Disable loop back. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully enabled/disabled loopback to the specified 
 *                                          SPORT 
 *
 *  - #ADI_SPORT_FAILED                     Failed enable/disable loopback 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Enabling loopback allowed only if the device is 
 *                                          operating in receive mode. 
 *
 */
ADI_SPORT_RESULT adi_sport_EnableLoopBack(
                                           ADI_SPORT_HANDLE  hDevice,
                                           bool              bEnable
                                         )
{
     /* Return code */
     ADI_SPORT_RESULT        eResult=ADI_SPORT_FAILED;
     /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->eDirection != ADI_SPORT_DIR_RX)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }

#endif /* ADI_DEBUG */
    if(bEnable  == true)
    {
        /* enable the loop back */
        pDevice->pSPORTRegs->SPORT_CTL2    |=  (uint32_t)(ENUM_SPORT_CTL2_LOOPBACK_EN);
    }
    else
    {
        /* disable  the loop back */
        pDevice->pSPORTRegs->SPORT_CTL2    &= (uint32_t)(~(ENUM_SPORT_CTL2_LOOPBACK_EN));
    }
     return (ADI_SPORT_SUCCESS);
}

/**
 * @brief       Sets the control word to specified SPORT device. 
 *
 * @details     Sets the control word to specified SPORT device. It can be done either writing a 
 *              control word OR by calling other functional APIs. If only few fields to be 
 *              modified,call adi_sport_GetControlReg function to get the current register value and 
 *              modify the required fields. 
 *
 * @param [in]  hDevice       Device handle to which control word to be set. 
 *
 * @param [in]  eRegister     Indicate which control register to be set. 
 *
 * @param [in]  nControlWord  Sets the Control word for the specified SPORT device. There are 
 *                            functional APIs are are provided to set the some of the most commonly 
 *                            used fields. This function will overwrite the previous settings of the 
 *                            entire control word. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully set the control word to the specified 
 *                                          device. 
 *
 *  - #ADI_SPORT_INVALID_REGISTER           Specified register is invalid. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed when data flow is enabled. 
 *
 */
ADI_SPORT_RESULT adi_sport_SetControlReg(
                                            ADI_SPORT_HANDLE        hDevice,
                                            ADI_SPORT_CTL_REGISTER  eRegister,
                                            uint32_t                nControlWord
                                        )
{
    /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;
     /* Return code */
     ADI_SPORT_RESULT        eResult = ADI_SPORT_SUCCESS;
     /* Wordlength */
     uint32_t nWordLength;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    /* Don't allow to change the control register when the Data flow is enabled.
     */
    if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_DATA_FLOW_ENABLED)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */
    /* set the control register. */
    switch(eRegister)
    {
    case ADI_SPORT_CONTROL_REGISTER:
         /* Set the main control register */
        pDevice->pSPORTRegs->SPORT_CTL   =  nControlWord;
         nWordLength = ((nControlWord &(uint32_t) BITM_SPORT_CTL_SLEN))>>BITP_SPORT_CTL_SLEN;

         if(nWordLength <= SPORT_BYTE_TRANSFER_LENGTH )
         {
             pDevice->eDmaWordSize = ADI_DMA_MSIZE_1BYTE;
             pDevice->ePWordSize   = ADI_DMA_PSIZE_1BYTE;
             pDevice->nXmodify =(int32_t) sizeof(uint8_t);
             pDevice->nShiftValue =0U;
         }
         else if(nWordLength <= SPORT_HALFWORD_TRANSFER_LENGTH )
         {
             pDevice->eDmaWordSize = ADI_DMA_MSIZE_2BYTES;
             pDevice->ePWordSize   = ADI_DMA_PSIZE_2BYTES;
             pDevice->nXmodify =(int32_t) sizeof(uint16_t);
             pDevice->nShiftValue =1U;
         }
         else /*(nWordLength <= SPORT_WORD_TRANSFER_LENGTH )*/
         {
             pDevice->eDmaWordSize = ADI_DMA_MSIZE_4BYTES;
             pDevice->ePWordSize   = ADI_DMA_PSIZE_4BYTES;
             pDevice->nXmodify = (int32_t)sizeof(uint32_t);
             pDevice->nShiftValue =2U;
         }
         break;
    case ADI_SPORT_CONTROL_REGISTER2:
         /* Set the control register 2 which mux the clock and FS with other Half Sport */
         pDevice->pSPORTRegs->SPORT_CTL2  =  nControlWord;
         break;
    case ADI_SPORT_CONTROL_MC_REGISTER:
         /* Set the multi channel control register */
         pDevice->pSPORTRegs->SPORT_MCTL  =  nControlWord;
         break;
    case ADI_SPORT_ERRCTL_REGISTER:
         /* Set the error control register. Can be used to disable the status interrupt. */
        pDevice->pSPORTRegs->SPORT_ERRCTL = nControlWord;
         break;
   default:
         /* Specified register is invalid */
       eResult=ADI_SPORT_INVALID_REGISTER;
       break;
    }
    /* Return the success code */
    return eResult;
}


/**
 * @brief       Gets the the control word of the specified SPORT device. 
 *
 * @details     Gets the content of the control register (control word) of the specified SPORT 
 *              device if it is opened. This function can be used to know the current content of 
 *              control register. This function can be helpful to modify few fields in the control 
 *              word. 
 *
 * @param [in]  hDevice       Device handle to SPORT device. 
 *
 * @param [in]  eRegister     Indicate from which control register content to be retrieved. 
 *
 * @param [out] pControlWord  Pointer to the location where the content of the control register to 
 *                            be written. 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS               Successfully retrieved the control register of the specified 
 *                                     device. 
 *
 *  - #ADI_SPORT_INVALID_REGISTER      Specified register is invalid. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE   [D]  Invalid SPORT device handle. 
 *
 */
ADI_SPORT_RESULT adi_sport_GetControlReg(
                                          ADI_SPORT_HANDLE          hDevice,
                                          ADI_SPORT_CTL_REGISTER    eRegister,
                                          uint32_t                 *pControlWord
                                        )
{
    /* Pointer to SPORT device instance */
     ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;
     ADI_SPORT_RESULT        eResult = ADI_SPORT_SUCCESS;
#ifdef ADI_DEBUG
     /* Return code */
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
    /* set the control register. */
    switch(eRegister)
    {
        case ADI_SPORT_CONTROL_REGISTER:
             /* Read from control register */
             *pControlWord = pDevice->pSPORTRegs->SPORT_CTL;
             break;
        case ADI_SPORT_CONTROL_REGISTER2:
              /* Read from control register-2 */
             *pControlWord = pDevice->pSPORTRegs->SPORT_CTL2;
             break;
        case ADI_SPORT_CONTROL_MC_REGISTER:
              /* Read from multi channel control register */
             *pControlWord = pDevice->pSPORTRegs->SPORT_MCTL;
             break;
        default:
             /* Invalid register specified*/
            eResult= ADI_SPORT_INVALID_REGISTER;
            break;
    }
    return eResult;
}
/**
 * @brief       The adi_sport_StreamingEnable API enables / disables buffer streaming. It sets up 
 *              the DMA for list descriptor mode as opposed to the default DMA stop mode. 
 *
 * @param [in]  hDevice     Device handle to SPORT device. 
 *
 * @param [in]  bStreaming  Boolean flag to indicate whether to enable/disable the streaming. 
 * \n                       
 * \n                       true : To eanble the Streaming by DMA.. 
 * \n                       
 * \n                       false : To disable the Streaming by DMA. 
 * \n                       
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS             Successfully enabled/disabled the streaming mode. 
 *
 *  - #ADI_SPORT_DMA_FAILED          Failed to enable/disable the streaming mode. 
 *
 *  - #ADI_SPORT_INVALID_HANDLE [D]  Invalid SPORT device handle. 
 *
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Operation is not allowed if device is not operating in DMA mode. 
 *
 */
ADI_SPORT_RESULT adi_sport_StreamingEnable(ADI_SPORT_HANDLE const hDevice, bool bStreaming)
{

   /* Pointer to device instance */
    ADI_SPORT_DEVICE        *pDevice = (ADI_SPORT_DEVICE *)hDevice;

#ifdef ADI_DEBUG
    /* assume a return code */
    ADI_SPORT_RESULT eResult = ADI_SPORT_SUCCESS;
    
    /* Validate the given handle */
    if((eResult =ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return ADI_SPORT_INVALID_HANDLE;
    }
    if(pDevice->bDMAMode == false)
    {
        return(ADI_SPORT_OPERATION_NOT_ALLOWED); 
    }
#endif /* ADI_DEBUG */
   /* Set the function table corresponding to DMA mode of operation  */
   if(adi_pdma_SetSync(pDevice->hDMAChnl,!bStreaming) != ADI_DMA_SUCCESS)
   {
         return(ADI_SPORT_DMA_FAILED);
   }
   /* enable buffer streaming */
   if (adi_pdma_EnableStreaming(pDevice->hDMAChnl, bStreaming) != ADI_DMA_SUCCESS)
   {
       return ADI_SPORT_DMA_FAILED;
   }

   /* Return */
   return (ADI_SPORT_SUCCESS);
}
/**
 * @brief       This function helps to set the transfer size for the data transfer. This function call is valid only in DMA mode.
 *
 * @param [in]  hDevice   Device handle to SPORT device, obtained when a SPORT device opened
 *                        successfully.
 *
 * @param [out] eTransferSize     Size of the data transfer. It is user responsiablity to align the buffers based on the transfer size.
 *                                for example, if eSize is #ADI_SPORT_DMA_TRANSFER_16BYTES, buffers submitted for data transfer should be aligned
 *                                to 16 byte address boundary and  size of the data transfer should be integral multiple of 16. By default,
 *                                transfer width is set based on the word-lenght.
 *
 * @return      Status
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully returned hardware error(s) occurred by the
 *                                          specified device.
  *  - #ADI_SPORT_INVALID_TRANSFER_SIZE      Invalid Transfer size..
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid sport device handle.
 *  - #ADI_SPORT_OPERATION_NOT_ALLOWED [D]  Not allowed in interrupt mode. Valid only in DMA mode.
 *
 *  @note        By default, transfer size is set   based on the word-length.
 *
 *
 */
ADI_SPORT_RESULT adi_sport_SetDmaTransferSize(
                                                   ADI_SPORT_HANDLE    const   hDevice,
                                                   ADI_SPORT_DMA_TRANSFER_SIZE eTransferSize
                                                 )
{
    /* Pointer to SPORT device instance */
    register ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    ADI_SPORT_RESULT         eResult;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->bDMAMode == false)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    eResult = ADI_SPORT_SUCCESS;

    switch(eTransferSize)
    {
        case ADI_SPORT_DMA_TRANSFER_1BYTE:
            pDevice->nXmodify = 1;
            pDevice->eDmaWordSize = ADI_DMA_MSIZE_1BYTE;
            pDevice->nShiftValue =0U;
            break;

        case ADI_SPORT_DMA_TRANSFER_2BYTES:
            pDevice->nXmodify = 2;
            pDevice->eDmaWordSize = ADI_DMA_MSIZE_2BYTES;
            pDevice->nShiftValue = 1U;
            break;

        case ADI_SPORT_DMA_TRANSFER_4BYTES:
            pDevice->nXmodify = 4;
            pDevice->nShiftValue = 2U;
            pDevice->eDmaWordSize = ADI_DMA_MSIZE_4BYTES;
            break;

        case ADI_SPORT_DMA_TRANSFER_8BYTES:
            pDevice->nXmodify = 8;
            pDevice->eDmaWordSize = ADI_DMA_MSIZE_8BYTES;
            pDevice->nShiftValue = 3U;
            break;
        case ADI_SPORT_DMA_TRANSFER_16BYTES:
            pDevice->nXmodify = 16;
            pDevice->eDmaWordSize = ADI_DMA_MSIZE_16BYTES;
            pDevice->nShiftValue = 4U;
            break;
        case ADI_SPORT_DMA_TRANSFER_32BYTES:
            pDevice->nXmodify = 32;
            pDevice->eDmaWordSize = ADI_DMA_MSIZE_32BYTES;
            pDevice->nShiftValue = 5U;
            break;

        default:
        	eResult = ADI_SPORT_INVALID_TRANSFER_SIZE;
        	break;
    }
    return(eResult);
}
/**
 * @brief       Configures the trigger input setting for a specified SPORT device.
 *
 * @details     The DMA associated with SPORT  can be configured such the start of the DMA to wait for
 *              an external trigger to occur. The routing of the external trigger to the DMA associated with the 
 *              device should be done by the application by calling the API adi_tru_TriggerRoute.
 * 
 * @param [in]  hDevice   Device handle to SPORT device, obtained when a SPORT device opened 
 *                        successfully. 
 *
 * @param [out] bTriggerWait   
 * \n                           
 * \n                           true :To wait for an incoming trigger before starting the next DMA work unit.
 * \n                           
 * \n                           false :  To ignore incoming trigger and begin work unit automatically.
 *
 * @param [out] bTriggerOvError
 * \n                           
 * \n                           true : To enable and report trigger overrun error.
 * \n                           
 * \n                           false : To disable trigger overrun error.
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully configured Trigger Input for the specified device.
 *  - #ADI_SPORT_DMA_FAILED                 If failed to configure the DMA for trigger wait.
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid sport device handle. 
 *
 */
ADI_SPORT_RESULT adi_sport_ConfigTriggerIn ( ADI_SPORT_HANDLE  const   hDevice,
                                             bool              bTriggerWait,
                                             bool              bTriggerOvError
                                           )
{
    /* Pointer to SPORT device instance */
    register ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    ADI_SPORT_RESULT         eResult;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */
   if(adi_dma_ConfigTriggerIn (pDevice->hDMAChnl,
                               bTriggerWait,
                               bTriggerOvError
                  ) != ADI_DMA_SUCCESS)
   {
        return(ADI_SPORT_DMA_FAILED);
   }
    return(ADI_SPORT_SUCCESS);
}
/**
 * @brief                 Configures Trigger Output settings for the  DMA associated with  SPORT device.
 *
 * @details               The DMA associated with the device can be configured generate a trigger output which can
 *                        be used to synchronize other DMAs in the system.
 *
 * @param [in]  hDevice   Device handle to SPORT device, obtained when a SPORT device opened 
 *                        successfully. 
 *
 * @param [out] eTrigger   The condition to generate the trigger output.
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                    Successfully configured Trigger Input setting for the specified device.
 *  - #ADI_SPORT_DMA_FAILED                 Failed to configures Trigger Input setting by DMA service.
 *  - #ADI_SPORT_INVALID_HANDLE        [D]  Invalid sport device handle. 
 *
 */

ADI_SPORT_RESULT adi_sport_ConfigTriggerOut( ADI_SPORT_HANDLE    const   hDevice,
                                              ADI_SPORT_TRIGGER_OUT eTrigger 
                                           )
{
    /* Pointer to SPORT device instance */
    register ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    /* Return code */
    ADI_SPORT_RESULT         eResult;
#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_SPORT_SUCCESS)
    {
        return eResult;
    }
    if(pDevice->pfCallback != NULL)
    {
        return ADI_SPORT_OPERATION_NOT_ALLOWED;
    }    
#endif /* ADI_DEBUG */
   if(adi_dma_ConfigTriggerOut(pDevice->hDMAChnl,
                               (ADI_DMA_TRIGGER_OUT)eTrigger
                             ) != ADI_DMA_SUCCESS)
   {
        return(ADI_SPORT_DMA_FAILED);
   }
    return(ADI_SPORT_SUCCESS);
}

/*
 * @brief       Submit the buffer for Data transfer in DMA Mode. 
 *
 * @param [in]  hDevice  Device handle to SPORT device. 
 *
 * @param [in]  pBuffer  Pointer to buffer from where DMA reads or writes. 
 *
 * @param [in]  nSize    Size of the data to be transferred(In bytes). 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                 Successfully submitted the buffer to DMA associated with 
 *                                       SPORT. 
 *
 *  - #ADI_SPORT_BUFFERS_NOT_SUBMITTED   Failed to submit the buffer to DMA associated with SPORT. 
 *
 */
static ADI_SPORT_RESULT SubmitBufferDmaMode(
                                           ADI_SPORT_HANDLE  hDevice,
                                           void             *pBuffer,
                                           uint32_t          nSize
                                          )
{
       /* Pointer to SPORT device instance */
        ADI_SPORT_DEVICE         *pDevice = (ADI_SPORT_DEVICE *)hDevice;
    
       /* Pointer to the descriptor that caused the interrupt */
        ADI_SPORT_BUFF_LIST  *pDesc;

        pDesc    =   pDevice->pCurrDesc;
        /* Initialize common parameters */
        if(pDesc->pStartAddress == NULL)
        {

            /*  Submit the buffer to DMA manager for receiving the data  */
            if(pDevice->eDirection == ADI_SPORT_DIR_RX)
            {
                if(adi_pdma_PeripheralRead(pDevice->hDMAChnl,pBuffer,(nSize)>>pDevice->nShiftValue,pDevice->nXmodify,0U,0) != ADI_DMA_SUCCESS)
                {
                    return ADI_SPORT_BUFFERS_NOT_SUBMITTED;
                }
            }
            /*  Submit the buffer to DMA manager for receiving the data  */
            else
            {

                if(adi_pdma_PeripheralWrite(pDevice->hDMAChnl,pBuffer,(nSize)>>pDevice->nShiftValue,pDevice->nXmodify,0U,0) != ADI_DMA_SUCCESS)
                {
                    return ADI_SPORT_BUFFERS_NOT_SUBMITTED;
                }
            }
            /* Update the information in the list */
            pDesc->pStartAddress    =   pBuffer;
            pDesc->bInUse           =   true;
            /* Advance the current pointer */
            pDevice->pCurrDesc = pDevice->pCurrDesc->pNext;
            if(pDevice->pSportInfo->eState ==ADI_SPORT_STATE_PAUSED)
            {
                adi_osal_EnterCriticalRegion();
                   pDevice->pSportInfo->eState  = ADI_SPORT_STATE_DATA_FLOW_ENABLED;
                   adi_osal_ExitCriticalRegion();
            }

        }
        else
        {
             return (ADI_SPORT_BUFFERS_NOT_SUBMITTED);
        }
        return(ADI_SPORT_SUCCESS);        
    
}

/*
 * @brief       Submits the buffers for data transfer in interrupt mode. 
 *
 * @param [in]  hDevice  Device handle to SPORT device. 
 *
 * @param [in]  pBuffer  Pointer to buffer from where DMA reads or writes. 
 *
 * @param [in]  nSize    Size of the data to be transferred(In bytes). 
 *
 * @return      Status 
 *
 *  - #ADI_SPORT_SUCCESS                   Successfully submitted the buffer for data transfer. 
 *
 *  - #ADI_SPORT_BUFFERS_NOT_SUBMITTED     No free descriptor for data transfer. 
 *
 *  - #ADI_SPORT_INT_REGISTRATION_FAILED   Failed to Reenabling the Data interrupt. 
 *
 */
static ADI_SPORT_RESULT SubmitBufferIntMode(
                                              ADI_SPORT_HANDLE hDevice,
                                              void            *pBuffer,
                                              uint32_t         nSize
                                           )
{
        ADI_SPORT_DEVICE       *pDevice = (ADI_SPORT_DEVICE  *)hDevice;
        ADI_SPORT_BUFF_LIST    *pDesc;
        ADI_SPORT_BUFF_LIST    *pCurDesc;
       if(pDevice->pCurrDesc->pStartAddress != NULL)
       {
           return(ADI_SPORT_BUFFERS_NOT_SUBMITTED);
       }
       else
       {
           adi_osal_EnterCriticalRegion();
           pCurDesc = pDevice->pCurrDesc;
           /* set the Descriptor parameters for interrupt mode */
           pCurDesc->pStartAddress =   pBuffer;
           /* Always writes 32 bit word */
           pCurDesc->nCount        =   nSize >> 2;
           pCurDesc->nIndex        =   0U;
           pCurDesc->bInUse        =   true;
           pDevice->pCurrDesc = pDevice->pCurrDesc->pNext;
           adi_osal_ExitCriticalRegion();
           if(pDevice->pSportInfo->eState == ADI_SPORT_STATE_PAUSED)
           {
#if defined(__ADSPBF609_FAMILY__) || defined(__ADSPBF707_FAMILY__)
                 if(adi_sec_ClearPendStatus(pDevice->pSportInfo->nDMAIntID)!= ADI_SEC_SUCCESS)
                {
                    return(ADI_SPORT_INT_REGISTRATION_FAILED);
                 }
#endif
                if(adi_int_EnableInt(pDevice->pSportInfo->nDMAIntID,true)!=ADI_INT_SUCCESS)
                {
                    return ADI_SPORT_INT_REGISTRATION_FAILED;
                }
                adi_osal_EnterCriticalRegion();
                pDevice->pSportInfo->eState = ADI_SPORT_STATE_DATA_FLOW_ENABLED;
                adi_osal_ExitCriticalRegion();
            }
       }
       return(ADI_SPORT_SUCCESS);
}

/*
 * @brief       Sport interrupt handler for data transfer. 
 *
 * @param [in]  IID       Interrupt ID. 
 *
 * @param [in]  pCBParam  Callback parameter from interrupt manager. 
 *
 */
static void sport_DataHandler(uint32_t IID, void *pCBParam)
{
    ADI_SPORT_DEVICE     *pDevice = (ADI_SPORT_DEVICE  *)pCBParam;
    ADI_SPORT_BUFF_LIST  *pProcessingDesc = pDevice->pActiveDesc;
    ADI_SPORT_EVENT eEvent;
    volatile uint32_t *pNextWord = (uint32_t *)pProcessingDesc->pStartAddress;
#ifdef ADI_DEBUG
    /* This should not happen but a safety check */
    if(pDevice == NULL)
    {
        return;
    }
#endif /* ADI_DEBUG */
    /* Is processing buffer is NULL */
    if(pDevice->pActiveDesc->pStartAddress!= NULL)
    {
        if(ADI_SPORT_DIR_RX == pDevice->eDirection)
        {
            /* Read the data from RX register if the direction is ADI_SPORT_DIR_RX */
            pNextWord[pProcessingDesc->nIndex] =(uint32_t)ADI_ANOM_16000030_READ32((volatile uint32_t *)&pDevice->pSPORTRegs->SPORT_RXA);
            pProcessingDesc->nIndex++;
            eEvent = ADI_SPORT_EVENT_RX_BUFFER_PROCESSED;
        }
        else
        {
            /* Read the data into  TX register if the direction is ADI_SPORT_DIR_TX */
             pDevice->pSPORTRegs->SPORT_TXA = pNextWord[pProcessingDesc->nIndex];
             pProcessingDesc->nIndex++;
             eEvent = ADI_SPORT_EVENT_TX_BUFFER_PROCESSED;
        }

        if(pProcessingDesc->nIndex >= pProcessingDesc->nCount)
        {
            adi_osal_EnterCriticalRegion();
            /* Advance the processed buffer pointer in the list */
            pDevice->pActiveDesc = pDevice->pActiveDesc->pNext;
            /* Restore the count */
            pProcessingDesc->nCount = pProcessingDesc->nCount << 2;
            /* Set the bInUse flag as false since we processed the buffer  */
            pProcessingDesc->bInUse = false;
            adi_osal_ExitCriticalRegion();
            /* Call the callback function if the callback function is registered. */
            if(pDevice->pfCallback != NULL)
            {
                 pDevice->pfCallback(pDevice->pCBParam,(uint32_t)eEvent,pProcessingDesc->pStartAddress);
                 /* No need to keep the processed buffer address */
                 pProcessingDesc->pStartAddress = NULL;
            }
            else
            {
                /* No callback function is registerd. So post the semaphore */
                if(adi_osal_SemPost(pDevice->hSemaphore) != ADI_OSAL_SUCCESS)
                {

                }
            }
        }
    }
    else
    {
        adi_int_EnableInt(pDevice->pSportInfo->nDMAIntID,false);
        adi_osal_EnterCriticalRegion();
        pDevice->pSportInfo->eState = ADI_SPORT_STATE_PAUSED;
        adi_osal_ExitCriticalRegion();

    }
    return;
}
/*
 * @brief       Sport interrupt handler. 
 *
 * @param [in]  IID       Interrupt ID. 
 *
 * @param [in]  pCBParam  Callback parameter from interrupt manager. 
 *
 */
static void SPORTStatusHandler(uint32_t IID, void *pCBParam)
{
    ADI_SPORT_DEVICE     *pDevice = (ADI_SPORT_DEVICE  *)pCBParam;
    uint32_t            Mask;
    uint32_t            StatRegValue;
    uint32_t            TmpStatRegValue;
    uint32_t            eEvent = (uint32_t)ADI_SPORT_HW_ERR_NONE;
    void                *pArg;

#ifdef ADI_DEBUG
    /* This should not happen but a safety check */
    if(pDevice == NULL)
    {
        return;
    }
#endif /* ADI_DEBUG */

    /* Get the contents of status register */
    StatRegValue    =   pDevice->pSPORTRegs->SPORT_ERRCTL;
    TmpStatRegValue =   ((StatRegValue)&(uint32_t)(BITM_SPORT_ERR_FSERRSTAT | BITM_SPORT_ERR_DERRSSTAT | BITM_SPORT_ERR_DERRPSTAT ));

    /* For all the bits in the status register */
    for(Mask = (uint32_t)BITM_SPORT_ERR_DERRPSTAT; Mask <=(uint32_t)BITM_SPORT_ERR_FSERRSTAT;Mask <<=1U)
    {

        /* Take action only when the bit is set */
        if((TmpStatRegValue & Mask) != 0u)
        {
            pArg   =   NULL;
            switch(Mask)
            {
                /* Frame sync error*/
                case (uint32_t) BITM_SPORT_ERR_FSERRSTAT:
                    eEvent |=(uint32_t) ADI_SPORT_HW_ERR_FS;
                    break;

                /* Error in secondary channel error */
                case (uint32_t) BITM_SPORT_ERR_DERRSSTAT:
                   if(pDevice->eDirection == ADI_SPORT_DIR_RX)
                   {
                         eEvent |=(uint32_t) ADI_SPORT_HW_ERR_SECONDARY_CHNL_OVERFLOW;
                   } 
                   else
                   {
                         eEvent |=(uint32_t) ADI_SPORT_HW_ERR_SECONDARY_CHNL_UNDERFLOW;

                   }
                   break;

                /* Detected a primary channel  error */
                case (uint32_t) BITM_SPORT_ERR_DERRPSTAT:
                   if(pDevice->eDirection == ADI_SPORT_DIR_RX)
                   {
                         eEvent |=(uint32_t) ADI_SPORT_HW_ERR_PRIMARY_CHNL_OVERFLOW;
                   } 
                   else
                   {
                         eEvent |= (uint32_t)ADI_SPORT_HW_ERR_PRIMARY_CHNL_UNDERFLOW;

                   }
                   break;
                default:
                   break;  
            }
            /* Clear bit in local copy */
            TmpStatRegValue &= (~Mask);
        }

        /* If there are no more status bits set stop the loop */
        if(TmpStatRegValue == 0u)
        {
            break;
        }
    }

    /* Clear the status register bits (W1C) */
    if(pDevice->pfCallback != NULL)
    {
        pDevice->pfCallback( pDevice->pCBParam,
                             (uint32_t) eEvent,
                             NULL
                           );
    }
    else
    {
        pDevice->nHwError = eEvent;
        if(adi_osal_SemPost(pDevice->hSemaphore) != ADI_OSAL_SUCCESS)
        {
        /* TBD: To indicate the application that error has happened */
        }
    }
    pDevice->pSPORTRegs->SPORT_ERRCTL   =  (StatRegValue & (uint32_t)(BITM_SPORT_ERR_DERRPSTAT | BITM_SPORT_ERR_DERRSSTAT | BITM_SPORT_ERR_DERRPSTAT ));
    return;
}

/*
 * @brief       Callback function for DMA manager. 
 *
 * @param [in]  pCBParam  Callback parameter from DMA manager. 
 *
 * @param [in]  Event     Event ID. 
 *
 * @param [in]  pArg      Callback argument. 
 *
 */
static void SPORTDMAHandler( 
                             void *pCBParam, 
                             uint32_t Event, 
                             void *pArg
                            )
{
    /* Pointer to the SPORT device instance data */
    ADI_SPORT_DEVICE     *pDevice        = (ADI_SPORT_DEVICE  *)pCBParam;
    ADI_SPORT_BUFF_LIST  *pProcessedDesc = pDevice->pActiveDesc;
    ADI_SPORT_EVENT       eSportEvent    = ADI_SPORT_EVENT_TX_BUFFER_PROCESSED;
    /* Pointer to the buffer address */
    void *pProcessedBuff;
    bool bValidEvent= true;

#ifdef ADI_DEBUG
    /* This should not happen but a safety check */
    if(pDevice == NULL)
    {
        return;
    }
#endif /* ADI_DEBUG */
    pProcessedBuff    =   (void *)pArg;

    switch(Event)
    {
        case (uint32_t)ADI_DMA_EVENT_SUB_BUFFER_PROCESSED:
             break;
        /* The given buffer processed */
        case (uint32_t)ADI_DMA_EVENT_BUFFER_PROCESSED:
        case (uint32_t)ADI_DMA_EVENT_DESC_LIST_PROCESSED:
            adi_osal_EnterCriticalRegion();
            pDevice->pActiveDesc->bInUse = false;
            pDevice->pActiveDesc = pDevice->pActiveDesc->pNext;
            adi_osal_ExitCriticalRegion();
            break;
        default:
            /* We don't care about other events */
            bValidEvent = false;
            break;

    }
    if(bValidEvent == true)
    {
        if(pDevice->pfCallback != NULL)
        {
            if(pDevice->eDirection == ADI_SPORT_DIR_RX)
            {
                eSportEvent = ADI_SPORT_EVENT_RX_BUFFER_PROCESSED;
            }
            pDevice->pfCallback(pDevice->pCBParam,(uint32_t)eSportEvent,pProcessedBuff);
            pProcessedDesc->pStartAddress = NULL;
        }
        else
        {
             adi_osal_SemPost(pDevice->hSemaphore);
        }
    }
}
/*
 * @brief       Create a circular linked list for buffer management. 
 *
 * @param [in]  hDevice  Device handle to SPORT device. 
 *
 * @param [in]  NumDesc  Number of descriptorS. 
 *
 */
static void InitDevice (ADI_SPORT_DEVICE *pDevice)
{
    int32_t                i;
    ADI_SPORT_BUFF_LIST  *pDesc = &pDevice->BufInfo[0];
    /* Initialize the all descriptors. Make it circular. */
    for(i = 0; i < NUM_SPORT_BUFFER; i++)
    {
        pDesc[i].pStartAddress      =   NULL;
        pDesc[i].nCount             =   0u;
        pDesc[i].nIndex             =   0u;
        pDesc[i].pNext              =   &pDesc[i+1];
    }
    /* create a circular list */
    pDesc[i-1].pNext                =   &pDesc[0];

    pDevice->pActiveDesc =  &pDevice->BufInfo[0];
    pDevice->pProcDesc   =  &pDevice->BufInfo[0];
    pDevice->pCurrDesc   =  &pDevice->BufInfo[0];

    return;
}
/*
 * @brief       Enable the DMA and set PSIZE and MSIZE. 
 *
 * @param [in]  hDevice  DMA handle. 
 *
 * @param [in]  eMsize   MSize of for DMA transfer. 
 *
 * @param [in]  ePsize   PSize for DMA transfer. 
 *
 */
static ADI_DMA_RESULT EnableDma(ADI_DMA_CHANNEL_HANDLE hChannel, ADI_DMA_MSIZE eMsize, ADI_DMA_PSIZE ePsize)
{
    ADI_DMA_RESULT eResult;
    if((eResult = adi_pdma_SetPSize(hChannel,ePsize)) != ADI_DMA_SUCCESS)
    {
        return(eResult);
    }

    if((eResult = adi_pdma_SetMSize(hChannel,eMsize)) != ADI_DMA_SUCCESS)
    {
        return(eResult);
    }

    if((eResult = adi_pdma_EnableXfer (hChannel)) != ADI_DMA_SUCCESS)
    {
        return(eResult);
    }
    return(eResult);
}

#if  defined(ADI_ADSP_CM40Z)
/**
 * @brief       Clears pending interrupt status.
 *
 * @details     This API is usually not required to called. This can be useful
 *              in a scenario when a device is disabled and enabled back. To
 *              make sure that there no pending interrupts when the device was
 *              previously enabled, the the pending interrupt status can be
 *              cleared.
 *
 * @param[in]   SID             The system interrupt ID.
 *
 * @return      Status
 *              - #ADI_SPORT_SUCCESS if successfully cleared the pending interrupt
 *                 status.
 *              - #ADI_SPORT_FAILED [D] if the SID is not valid.
 */

static ADI_SPORT_RESULT adi_sec_ClearPendStatus(uint32_t SID)
{
    /* Pointer to the SSI status register */
     volatile uint32_t *pSSTAT;

 #ifdef ADI_DEBUG

     /* Check if the given SID is valid */
     if( SID > (uint32_t)127)
     {
         return ADI_SPORT_FAILED;
     }
 #endif /* ADI_DEBUG */
     NVIC_ClearPendingIRQ((IRQn_Type)SID);
     return ADI_SPORT_SUCCESS;
}
#endif
#ifdef ADI_DEBUG
static ADI_SPORT_RESULT ValidateHandle( ADI_SPORT_DEVICE *pInDevice)
{                                 
    /* Return code */             
    ADI_SPORT_RESULT    nResult = ADI_SPORT_INVALID_HANDLE;
    /* Pointer to SPORT device instance */
    ADI_SPORT_DEVICE_INFO *poDeviceInfo = &sport_device_info[0][0];
    uint32_t i;
    for(i=0U; i< (ADI_SPORT_NUM_INSTANCES *2U); i++)
    {
        if(pInDevice == poDeviceInfo->hDevice)
        {
            return(ADI_SPORT_SUCCESS);
        }
        poDeviceInfo++;
    }
    return (nResult);
}

#endif /* ADI_DEBUG */

/*@}*/



#ifdef _MISRA_RULES
#pragma diag(pop)
#endif
