/******************************************************************************

Copyright (c) 2011 Analog Devices.  All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
 *******************************************************************************
 *
 * @file:    adi_twi.c
 * @brief:   TWI device driver implementation
 * @version: $Revision: 21988 $
 * @date:    $Date: 2013-06-21 17:01:32 -0400 (Fri, 21 Jun 2013) $
 *
 ******************************************************************************/
/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"(advisory) Sections of code should not be commented out.")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be longer than 31 characters.")
#pragma diag(suppress:misra_rule_11_4:"(advisory) A cast should not be performed between a pointer to object type and a different pointer to object type.")
#pragma diag(suppress:misra_rule_13_7:"Boolean operations whose results are invariant shall not be permitted.")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#pragma diag(suppress:misra_rule_16_2:"Functions shall not call themselves, either directly or indirectly.")
#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic.")
#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser.")
#endif /* _MISRA_RULES */

/** @addtogroup TWI_Driver TWI Device Driver
 *  @{
 *
The TWI controller allows a device to interface to an inter IC bus as specified
by the Philips I2C Bus Specification version 2.1 dated January 2000.

The TWI is fully compatible with the widely used I2C bus standard. It
was designed with a high level of functionality and is compatible with
multi-master, multi-slave bus configurations. To preserve processor bandwidth
the TWI controller can be set up with transfer initiated interrupts
only to service FIFO buffer data reads and writes. Protocol related interrupts
are optional.

The TWI externally moves 8-bit data while maintaining compliance with
the I2C bus protocol. The TWI controller includes these features:

- Simultaneous master and slave operation on multiple device systems
- Support for multi-master bus arbitration
- 7-bit addressing
- 100K bits/second and 400K bits/second data rates
- General call address support
- Master clock synchronization and support for clock low extension
- Separate multiple-byte receive and transmit FIFOs
- Low interrupt rate
- Individual override control of data and clock lines in the event of bus lock-up
- Input filter for spike suppression
- Serial camera control bus support as specified in the OmniVision Serial Camera Control Bus (SCCB) Functional Specification version 2.1.
 */

#include <drivers/twi/adi_twi.h>
#include <services/int/adi_int.h>
#include <stddef.h>
#include <sys/exception.h>
#include <sys/16000030.h>  /* For anomaly workaround */

#include <adi_osal.h>
#include <assert.h>

#if defined(ADI_CODE_IN_ROM)
#include "../rom/adi_rom_def.h"
#endif


#if defined(__ADSPBF609_FAMILY__)
#include <services/int/adi_sec.h>
#define NUM_TWI_DEVICES (2u)
#define ADI_SID_TWI0 (INTR_TWI0_DATA)
#define ADI_SID_TWI1 (INTR_TWI1_DATA)
#elif defined(ADI_ADSP_CM40Z)   || defined (__ADSPBF707_FAMILY__)
#define NUM_TWI_DEVICES (1u)
#define ADI_SID_TWI ((uint32_t) INTR_TWI0_DATA)
#elif defined (__ADSPBLACKFIN__) && !defined(__ADSPBF60x__)
#include <services/int/adi_sid.h>
#else
#error "Unknown processor family"
#endif

#include "adi_twi_def.h"

#if !defined(ADI_CODE_IN_ROM)
static ADI_TWI_DRIVER adi_twi_Device[] =
{
    /* TWI 0 */
    {
        (ADI_TWI_REGISTER1_STRUCT *) pREG_TWI0_CLKDIV,  /* Base TWI register address */
        (ADI_TWI_REGISTER2_STRUCT *) pREG_TWI0_TXDATA8, /* TWI register address      */
        NULL                                            /* Data storage pointer      */

        }
#if defined(__ADSPBF609_FAMILY__) || defined(__ADSPBF538_FAMILY__) \
	|| ((defined(__ADSPBF548_FAMILY__) || defined(__ADSPBF548M_FAMILY__)) && !defined(__ADSPBF542__))
    /* TWI 1 */
    ,
    {
        (ADI_TWI_REGISTER1_STRUCT *) pREG_TWI1_CLKDIV,  /* Base TWI register address */
        (ADI_TWI_REGISTER2_STRUCT *) pREG_TWI1_TXDATA8, /* TWI register address      */
        NULL                                            /* Data storage pointer      */
        }
#endif
};
#endif



/******************************************************
 Internal function prototypes:
 *******************************************************/

static void Initialize(uint32_t const DeviceNum);
static void StopDevice(ADI_TWI_DRIVER *drv);
static ADI_TWI_RESULT StartDevice(ADI_TWI_DRIVER *drv);
static void FlushDevice(ADI_TWI_DRIVER const *drv);
static void BufferTransmit(ADI_TWI_DRIVER const *drv);
static bool BufferReceive(ADI_TWI_DRIVER const *drv);
static void MasterService(ADI_TWI_DRIVER *drv, uint16_t stat);
static void SlaveService(ADI_TWI_DRIVER *drv, uint16_t stat);
static void MasterProcessed(ADI_TWI_DRIVER *drv, ADI_TWI_EVENT event);
static void SlaveTxProcessed(ADI_TWI_DRIVER *drv, ADI_TWI_EVENT event);
static void SlaveRxProcessed(ADI_TWI_DRIVER *drv,  ADI_TWI_EVENT event);
static ADI_TWI_RESULT InitiateProcessing(ADI_TWI_DRIVER *drv);

static ADI_TWI_RESULT processInterrupt(ADI_TWI_DRIVER *drv);
static void TWIHandler (uint32_t SID, void *pParam);

#if defined(ADI_DEBUG)
static bool IsDeviceHandle(ADI_TWI_DRIVER const *pDevice);
#endif

/******************************************************
 Macros:
 *******************************************************/

#define MINFREQ       (20u)
#define MAXFREQ       (400u)
#define MINDUTYCYCLE  (1u)
#define MAXDUTYCYCLE  (99u)
#define MINPRESCALE   (1u)
#define MAXPRESCALE   (0x7fu)
#define MAXTRANSFER   (254u)
#define DEFFIFO       (0u)
#define DEFFREQ       (100u)
#define DEFPRESCALE   (12u)  /* assumes a 120 MHz SCLK */
#define DEFDUTY       (50u)
#define MAXCLKDIV     (255u)
#define MINCLKDIV     (0u)
#define FASTFREQ      (100u)
#define MAXDCNT       (255u)
#define MINADDR       (0x7u)
#define MAXADDR       (0x7Cu)

#define DEFRETRY1     (1u)
#define DEFRETRY3     (3u)

#define CLEARREG16    (0u)

#define OUTBOUND      (0u)
#define INBOUND       (1u)

#define TWIFIFOEMPTY  (0u)
#define TWIFIFOSINGLE (1u)
#define TWIFIFOFULL   (3u)

/******************************************************
 Internal functions:
 *******************************************************/

#if defined(ADI_DEBUG)
/*
 * Verifies a pointer to a driver points to one of the driver
 * struct's internal to this file.
 */
static bool IsDeviceHandle(ADI_TWI_DRIVER const *pDevice)
{
	uint32_t i = 0u;
	bool bFound = false;
	for (i=0u; i<NUM_TWI_DEVICES; i++)
	{
#if !defined(ADI_CODE_IN_ROM)
		if (pDevice == &adi_twi_Device[i])
#else
		if( pDevice == &pram_for_rom_Code->adi_twi_Device[i])
#endif
		{
			bFound = true;
			break;
		}
	}
	return bFound;
}
#endif
/*
 * Initialize the driver.
 */
static void Initialize(uint32_t const DeviceNum)
{
	ADI_TWI_DRIVER *drv;

#if !defined(ADI_CODE_IN_ROM)
	drv = &adi_twi_Device[DeviceNum];
#else
	drv = &pram_for_rom_Code->adi_twi_Device[DeviceNum];
#endif

#if defined(ADI_SID_TWI)
	drv->pData->sid = ADI_SID_TWI;
#else
	if (DeviceNum == 0u)
	{
		drv->pData->sid = (uint8_t)ADI_SID_TWI0;
	} else {
		drv->pData->sid = (uint8_t)ADI_SID_TWI1;
	}
#endif

	drv->pData->eBlockMode    = BLOCK_MODE_NONE;
	drv->pData->activeBuffer  = (ADI_TWI_BUFFER_STRUCT*)0u;
	drv->pData->hSemHandle    = (ADI_OSAL_SEM_HANDLE)0u;

	/* setup the default driver configuration */
	drv->pData->config.frequency    = DEFFREQ;       /* clock frequency */
	drv->pData->config.prescale     = DEFPRESCALE;   /* prescale value */
	drv->pData->config.duty_cycle   = DEFDUTY;       /* clock duty cycle */
	drv->pData->config.fifo         = DEFFIFO;       /* fifo word transfer */
	drv->pData->config.lostarbLimit = DEFRETRY3;     /* lostarb retry count */
	drv->pData->config.anakLimit    = DEFRETRY1;     /* anak retry count */
	drv->pData->config.dnakLimit    = DEFRETRY1;     /* dnak retry count */

	/* buffer 1 */
	drv->pData->buffer1.Data       = (void *)0;
	drv->pData->buffer1.lostarb    = 0u;
	drv->pData->buffer1.anak       = 0u;
	drv->pData->buffer1.dnak       = 0u;
	drv->pData->buffer1.pendRx     = 0u;
	drv->pData->buffer1.pendTx     = 0u;
	drv->pData->buffer1.sdir       = 0u;
	drv->pData->buffer1.gcall      = 0u;
	drv->pData->buffer1.Restart    = false;
	drv->pData->buffer1.Complete   = true;

	/* buffer 2 */
	drv->pData->buffer2.Data       = (void *)0;
	drv->pData->buffer2.lostarb    = 0u;
	drv->pData->buffer2.anak       = 0u;
	drv->pData->buffer2.dnak       = 0u;
	drv->pData->buffer2.pendRx     = 0u;
	drv->pData->buffer2.pendTx     = 0u;
	drv->pData->buffer2.sdir       = 0u;
	drv->pData->buffer2.gcall      = 0u;
	drv->pData->buffer2.Restart    = false;
	drv->pData->buffer2.Complete   = true;

	drv->pData->Callback          = (void*)0;
	drv->pData->pCallParam        = (void*)0;
	drv->pData->bInUse            = false;
}

/*
 * Set TWI to a neutral state, return TRUE if the TWI driver was running,
 *   otherwise return FALSE
 *      -   disable I2C services (both Master and Slave)
 *      -   flush FIFOs
 *      -   clear interrupts
 */
static void StopDevice(ADI_TWI_DRIVER *drv)
{
	drv->pReg1->imsk = CLEARREG16;           /* mask all interrupts */
	drv->pReg1->slvctl = CLEARREG16;         /* kill SEN */
	drv->pReg1->mstrctl = CLEARREG16;        /* kill MEN */
	drv->pReg1->mstrstat = (uint16_t)(BITM_TWI_MSTRSTAT_BUFWRERR|
			BITM_TWI_MSTRSTAT_BUFRDERR|BITM_TWI_MSTRSTAT_DNAK|
			BITM_TWI_MSTRSTAT_ANAK|BITM_TWI_MSTRSTAT_LOSTARB);               /* clear all 5 W1C error bits */
	drv->pReg1->istat = (uint16_t)(BITM_TWI_ISTAT_RXSERV|BITM_TWI_ISTAT_TXSERV|
			BITM_TWI_ISTAT_MERR|BITM_TWI_ISTAT_MCOMP|BITM_TWI_ISTAT_SOVF|
			BITM_TWI_ISTAT_SERR|BITM_TWI_ISTAT_SCOMP|BITM_TWI_ISTAT_SINIT);  /* clear all interrupts */

	FlushDevice(drv);

    adi_osal_EnterCriticalRegion();

	/* the driver has now been stopped so clear TWI_ENA and SCCB flags */
	drv->pReg1->ctl &= (uint16_t)~(BITM_TWI_CTL_EN|BITM_TWI_CTL_SCCB);

    adi_osal_ExitCriticalRegion();

    adi_int_UninstallHandler(drv->pData->sid);
}

/*
 * Starts the TWI device
 *      -   uses the config part of the data struct to setup the TWI
 *      -   set FAST mode if SCL is > 100 kHz
 */
static ADI_TWI_RESULT StartDevice(ADI_TWI_DRIVER *drv)
{
	ADI_TWI_RESULT result;
	uint16_t temp, hi, lo;

	/* check to see if its already started */
	if ((drv->pReg1->ctl & (uint16_t)BITM_TWI_CTL_EN) != 0u)
	{
		return ADI_TWI_SUCCESS;
	}

	FlushDevice(drv);

	if ( (drv->pData->config.prescale < MINPRESCALE) ||
			(drv->pData->config.prescale > MAXPRESCALE) )
	{
		return ADI_TWI_BAD_PRESCALE_VALUE;
	}

	/* set the control prescale */
	drv->pReg1->ctl = (drv->pReg1->ctl & (uint16_t)(~BITM_TWI_CTL_PRESCALE)) | drv->pData->config.prescale;

	if ((drv->pData->config.mode == ADI_TWI_MASTER) ||
			(drv->pData->config.mode == ADI_TWI_MASTER_SCCB) )
	{
		/* set divisors in CLKDIV
		 * total cycle divisor; round UP; 10000 = the 10 MHz from pre-scaler
		 */
		temp = 10000u/(drv->pData->config.frequency);
		hi = ((temp * drv->pData->config.duty_cycle) + 50u)/100u; /* round up before dividing */
		lo = temp - hi;

		/* check for over or under flows and bad combination of kHz and duty cycle */
		if ((hi > MAXCLKDIV) || (lo > MAXCLKDIV) || (hi == MINCLKDIV) ||
				(lo == MINCLKDIV))
		{
			return ADI_TWI_BAD_DUTYCYCLE;
		}
		drv->pReg1->clkdiv = (uint16_t)(hi << BITP_TWI_CLKDIV_CLKHI) | lo;

		/* set FAST mode bit if over 100 kHz */
		if (drv->pData->config.frequency > FASTFREQ)
		{
		    adi_osal_EnterCriticalRegion();

			drv->pReg1->mstrctl |= (uint16_t)BITM_TWI_MSTRCTL_FAST;

		    adi_osal_ExitCriticalRegion();

		}
	}

	if (adi_int_InstallHandler(drv->pData->sid, TWIHandler, drv, true) != ADI_INT_SUCCESS) {
    	return ADI_TWI_FAILURE;
    }

    adi_osal_EnterCriticalRegion();

	/* the driver has now started so set TWI_ENA flag */
	drv->pReg1->ctl |= (uint16_t)BITM_TWI_CTL_EN;

	switch (drv->pData->config.mode) {
	case ADI_TWI_MASTER:
		drv->pReg1->imsk = (uint16_t)(BITM_TWI_IMSK_RXSERV|BITM_TWI_IMSK_TXSERV|
				BITM_TWI_IMSK_MERR|BITM_TWI_IMSK_MCOMP);       /* setup master interrupts only */
		result = ADI_TWI_SUCCESS;
		break;
	case ADI_TWI_MASTER_SCCB:
		drv->pReg1->ctl |= (uint16_t)BITM_TWI_CTL_SCCB;
		drv->pReg1->imsk = (uint16_t)(BITM_TWI_IMSK_RXSERV|BITM_TWI_IMSK_TXSERV|
				BITM_TWI_IMSK_MERR|BITM_TWI_IMSK_MCOMP);       /* setup master interrupts only */
		result = ADI_TWI_SUCCESS;
		break;
	case ADI_TWI_SLAVE:
		drv->pReg1->fifoctl = (uint16_t)(BITM_TWI_FIFOCTL_RXILEN|BITM_TWI_FIFOCTL_TXILEN);
		drv->pReg1->imsk = (uint16_t)(BITM_TWI_IMSK_RXSERV|BITM_TWI_IMSK_TXSERV|
				BITM_TWI_IMSK_SOVF|BITM_TWI_IMSK_SERR|BITM_TWI_IMSK_SCOMP|
				BITM_TWI_IMSK_SINIT);       /* pass slave interrupts only */
		drv->pReg1->slvctl |= (uint16_t)(BITM_TWI_SLVCTL_TDVAL|BITM_TWI_SLVCTL_EN);    /* set STDVAL and SEN */
		result = ADI_TWI_SUCCESS;
		break;
	default:
		/* otherwise stop the driver and return */
		StopDevice(drv);
		result = ADI_TWI_FAILURE; /* return failure */
		break;
	}

    adi_osal_ExitCriticalRegion();

	return result;
}

/*
 * Flush the fifo buffers and clears the current data buffer.
 */
static void FlushDevice(ADI_TWI_DRIVER const *drv)
{
    adi_osal_EnterCriticalRegion();

	/* flush both rx and tx fifo's */
	drv->pReg1->fifoctl |= (uint16_t)(BITM_TWI_FIFOCTL_RXFLUSH|BITM_TWI_FIFOCTL_TXFLUSH);
	drv->pReg1->fifoctl &= (uint16_t)~(BITM_TWI_FIFOCTL_RXFLUSH|BITM_TWI_FIFOCTL_TXFLUSH);

    adi_osal_ExitCriticalRegion();

}

/*
 * Loads the next byte in the data buffer into the TWI fifo register.
 */
static void BufferTransmit(ADI_TWI_DRIVER const *drv)
{
	uint16_t TxStat, Data0, Data1;
	uint8_t (*data)[] = (uint8_t(*)[])drv->pData->activeBuffer->Data; /* avoid Misra rule 17.4 */

	/* if nothing left in buffer return */
	if (drv->pData->activeBuffer->Index == drv->pData->activeBuffer->Count)
	{
		return;
	}

	/* load data into the fifo */
	TxStat = drv->pReg1->fifostat & (uint16_t)BITM_TWI_FIFOSTAT_TXSTAT;
	switch (TxStat)
	{
	case TWIFIFOEMPTY:
		/* check if word transfers are enabled and that there is more than
		 * one byte to send, if so then load the xmt_data16 register */
		if (((drv->pReg1->fifoctl & (uint16_t) BITM_TWI_FIFOCTL_TXILEN) != 0u) &&
				(drv->pData->activeBuffer->Count >= 2u)) {
			/* load the twi xmt_data16 register */
			Data0 = (*data)[drv->pData->activeBuffer->Index];
			drv->pData->activeBuffer->Index++;
			Data1 = (*data)[drv->pData->activeBuffer->Index];
			drv->pData->activeBuffer->Index++;
			drv->pReg2->txdata16 = Data0 + (uint16_t)(Data1 << (uint16_t) 8);
		} else {
			/* load the twi xmt_data8 register */
			Data0 = (*data)[drv->pData->activeBuffer->Index];
			drv->pData->activeBuffer->Index++;
			drv->pReg2->txdata8 = Data0;
		}
		break;
	case TWIFIFOSINGLE:
		/* load the twi xmt_data8 register */
		Data0 = (*data)[drv->pData->activeBuffer->Index];
		drv->pData->activeBuffer->Index++;
		drv->pReg2->txdata8 = Data0;
		break;
	default:
		break;
	}

	/* after the first byte is sent then set the fifo control register to the default */
	drv->pReg1->fifoctl = drv->pData->config.fifo;

	/* if count is greater than the max transfer size */
	if (((drv->pReg1->mstrctl & (uint16_t)BITM_TWI_MSTRCTL_EN) != 0u) &&
			(drv->pData->activeBuffer->Count > MAXTRANSFER))
	{
		if (((drv->pReg1->mstrctl & (uint16_t)BITM_TWI_MSTRCTL_DCNT) >> (uint16_t)BITP_TWI_MSTRCTL_DCNT) == MAXDCNT)
		{
		    if ((drv->pData->activeBuffer->Count - drv->pData->activeBuffer->Index) < MAXDCNT)
		    {
		    	drv->pReg1->mstrctl &= (uint16_t)~BITM_TWI_MSTRCTL_DCNT;
		    	drv->pReg1->mstrctl |= (uint16_t)(drv->pData->activeBuffer->Count - drv->pData->activeBuffer->Index) << BITP_TWI_MSTRCTL_DCNT;
		    }

		}
	}
}

/*
 * Adds a byte into the buffer and returns.
 */
static bool BufferReceive(ADI_TWI_DRIVER const *drv)
{
	uint16_t len, rxdata;
	uint8_t (*data)[] = (uint8_t(*)[])drv->pData->activeBuffer->Data; /* avoid Misra rule 17.4 */

	/* get the number of bytes in the receive fifo */
	len = (drv->pReg1->fifostat & (uint16_t)BITM_TWI_FIFOSTAT_RXSTAT) >>
			(uint16_t)BITP_TWI_FIFOSTAT_RXSTAT;

	if (len == 0u)
	{
		return true;
	}

	if (len == 1u)
	{
		rxdata = ADI_ANOM_16000030_READ16(&(drv->pReg2->rxdata8));
	}
	else
	{
		rxdata = ADI_ANOM_16000030_READ16(&(drv->pReg2->rxdata16));
	}

	/* after the first byte is received then set the fifo control register to the default */
	drv->pReg1->fifoctl = drv->pData->config.fifo;

	/* if buffer is full then return failure */
	if (drv->pData->activeBuffer->Index == drv->pData->activeBuffer->Count)
	{
		return false;
	}

	/* otherwise write data to buffer */
	switch (len)
	{
	case TWIFIFOFULL:
		if ((drv->pData->activeBuffer->Count - drv->pData->activeBuffer->Index) >= 2u)
		{
			(*data)[drv->pData->activeBuffer->Index] =
					(uint8_t)(rxdata & (uint16_t)BITM_TWI_RXDATA8_VALUE);
			drv->pData->activeBuffer->Index++;
			(*data)[drv->pData->activeBuffer->Index] =
					(uint8_t)((rxdata >> 8u) & (uint16_t)BITM_TWI_RXDATA8_VALUE);
			drv->pData->activeBuffer->Index++;
		}
		break;
	case TWIFIFOSINGLE:
		(*data)[drv->pData->activeBuffer->Index] =
				(uint8_t)(rxdata & (uint16_t)BITM_TWI_RXDATA8_VALUE);
		drv->pData->activeBuffer->Index++;
		break;
	default:
		break;
	}

	/* if count is greater than the max transfer size */
	if (((drv->pReg1->mstrctl & (uint16_t)BITM_TWI_MSTRCTL_EN) != 0u) &&
			(drv->pData->activeBuffer->Count > MAXTRANSFER))
	{
		if (((drv->pReg1->mstrctl & (uint16_t)BITM_TWI_MSTRCTL_DCNT) >> (uint16_t)BITP_TWI_MSTRCTL_DCNT) == MAXDCNT)
		{
		    if ((drv->pData->activeBuffer->Count - drv->pData->activeBuffer->Index) < MAXDCNT)
		    {
		    	drv->pReg1->mstrctl &= (uint16_t)~BITM_TWI_MSTRCTL_DCNT;
		    	drv->pReg1->mstrctl |= (uint16_t)(drv->pData->activeBuffer->Count - drv->pData->activeBuffer->Index) << BITP_TWI_MSTRCTL_DCNT;
		    }
		}
	}

	return true;
}

/*
 * MasterService
 */
static void MasterService(ADI_TWI_DRIVER *drv, uint16_t stat)
{
	bool bResult;
	bool bTest;
	/* local copies of stat's to reduce peripheral bus access */
	uint16_t master_ctl;
	uint16_t master_stat = drv->pReg1->mstrstat;

	if (drv->pData->activeBuffer == (ADI_TWI_BUFFER_STRUCT*)0u)
	{
		/* no active buffer */
		return;
	}

	/* MERR - can be set along w/ MCOMP */
	if ((stat & (uint16_t)BITM_TWI_ISTAT_MERR) != 0u)
	{
		/* set STOP: work around early silicon RSTART on MERR bug */
		if ((drv->pReg1->mstrctl & (uint16_t)BITM_TWI_MSTRCTL_RSTART) != 0u)
		{
		    adi_osal_EnterCriticalRegion();

			drv->pReg1->mstrctl |= (uint16_t)BITM_TWI_MSTRCTL_STOP;

		    adi_osal_ExitCriticalRegion();
		}

		if ((master_stat & (uint16_t)BITM_TWI_MSTRSTAT_BUFWRERR) != 0u)
		{
			MasterProcessed(drv, ADI_TWI_EVENT_BUFWRERR);
		}

		if ((master_stat & (uint16_t)BITM_TWI_MSTRSTAT_BUFRDERR) != 0u)
		{
			MasterProcessed(drv, ADI_TWI_EVENT_BUFRDERR);
		}
		if ((master_stat & (uint16_t)BITM_TWI_MSTRSTAT_DNAK) != 0u)
		{
			if (drv->pData->activeBuffer->dnak < drv->pData->config.dnakLimit)
			{
				drv->pData->activeBuffer->dnak++;
			}
			if (drv->pData->activeBuffer->dnak >= drv->pData->config.dnakLimit)
			{
				MasterProcessed(drv, ADI_TWI_EVENT_DNAK);
			}
		}
		if ((master_stat & (uint16_t)BITM_TWI_MSTRSTAT_ANAK) != 0u)
		{
			if (drv->pData->activeBuffer->anak < drv->pData->config.anakLimit)
			{
				drv->pData->activeBuffer->anak++;
			}
			if (drv->pData->activeBuffer->anak >= drv->pData->config.anakLimit)
			{
				MasterProcessed(drv, ADI_TWI_EVENT_ANAK);
			}
		}
		if ((master_stat & (uint16_t)BITM_TWI_MSTRSTAT_LOSTARB) != 0u)
		{
			if (drv->pData->activeBuffer->lostarb < drv->pData->config.lostarbLimit)
			{
				drv->pData->activeBuffer->lostarb++;
			}
			if (drv->pData->activeBuffer->lostarb >= drv->pData->config.lostarbLimit)
			{
				MasterProcessed(drv, ADI_TWI_EVENT_LOSTARB);
			}
		}
		drv->pReg1->mstrstat = master_stat;

	}
	/* MCOMP (branch taken only if no MERR) */
	else if ((stat & (uint16_t)BITM_TWI_ISTAT_MCOMP) != 0u)
	{
		/* make sure the last byte is received */
		bResult = BufferReceive(drv);

		/* dispatch the buffer */
		MasterProcessed(drv, ADI_TWI_EVENT_BUFFER_PROCESSED);
	}
	else if ((stat & (uint16_t)BITM_TWI_ISTAT_RXSERV) != 0u)
	{
		bResult = BufferReceive(drv);
	}
	else if ((stat & (uint16_t)BITM_TWI_ISTAT_TXSERV) != 0u)
	{
		BufferTransmit(drv);
	}
	else
	{
		bResult = false; /* Misra else rule */
	}

	/* if int_stat is 0 or MCOMP or MERR then prepare a new buffer, otherwise we're done */
	bTest = (stat &	(uint16_t)(BITM_TWI_ISTAT_MERR|BITM_TWI_ISTAT_MCOMP)) != 0u;
	if (!((stat == 0u) || bTest))
	{
		return;
	}

	/* if buffer is complete then return */
	if (drv->pData->activeBuffer->Complete)
	{
		return;
	}

	/* if not an RSTART, then flush the twi buffers for peace of mind */
	if (!drv->pData->activeBuffer->Restart)
	{
		FlushDevice(drv);
	}

	/* decide if it's a long or short transfer and mark DCNT appropriately */
	if (drv->pData->activeBuffer->Count > MAXTRANSFER)
	{
		master_ctl = ((uint16_t)MAXDCNT << (uint16_t)BITP_TWI_MSTRCTL_DCNT);
	}
	else
	{
		master_ctl = ((uint16_t)drv->pData->activeBuffer->Count << (uint16_t)BITP_TWI_MSTRCTL_DCNT);
	}

    adi_osal_EnterCriticalRegion();

	/* set direction of transfer */
	if (drv->pData->activeBuffer->Direction == INBOUND)
	{
		master_ctl |= (uint16_t)BITM_TWI_MSTRCTL_DIR;
	}
	else
	{
		master_ctl &= (uint16_t)~BITM_TWI_MSTRCTL_DIR;
	}

	/* set repeat start, if restart flag is set and the current buffer is
	 * less than 255 bytes long and SCCB bit is not set
	 */
	bTest = (drv->pReg1->ctl & (uint16_t)BITM_TWI_CTL_SCCB) != 0u;
	if ((drv->pData->activeBuffer->Restart) &&
		(drv->pData->activeBuffer->Count <= MAXTRANSFER) &&
		(!bTest))
	{
		master_ctl |= (uint16_t)BITM_TWI_MSTRCTL_RSTART;
	}
	else
	{
		master_ctl &= (uint16_t)~BITM_TWI_MSTRCTL_RSTART;
	}

    adi_osal_ExitCriticalRegion();

	/* if the buffer is odd then send/receive the first byte,
	 * otherwise set the transfer to the user defined fifo rate
	 */
	if ((drv->pData->activeBuffer->Count & 0x1u) != 0u)
	{
		drv->pReg1->fifoctl = 0x0u;
	}
	else
	{
		drv->pReg1->fifoctl = drv->pData->config.fifo;
	}

	if (drv->pData->activeBuffer->Direction == OUTBOUND)
	{
		BufferTransmit(drv);
	}

	/* set the master control register, and enable the transfer */
	drv->pReg1->mstrctl = (master_ctl|(uint16_t)BITM_TWI_MSTRCTL_EN);

}

/*
 * MasterProcessed
 */
static void MasterProcessed(ADI_TWI_DRIVER *drv, ADI_TWI_EVENT event)
{
	ADI_OSAL_STATUS status;
	ADI_TWI_RESULT result;

	/* tell client if he has requested so */
	if (drv->pData->Callback != (void*)0)
	{
		(drv->pData->Callback)(drv, (uint32_t)event, drv->pData->pCallParam);
	}

	/* reset all the counters */
	drv->pData->activeBuffer->lostarb = 0u;
	drv->pData->activeBuffer->anak = 0u;
	drv->pData->activeBuffer->dnak = 0u;
	drv->pData->activeBuffer->Complete = true;

	if (event != ADI_TWI_EVENT_BUFFER_PROCESSED) {
		drv->pData->activeBuffer->bError = true;
	}

	if (drv->pData->eBlockMode == BLOCK_MODE_BLOCKING) {
		/* release blocking */
		status = adi_osal_SemPost(drv->pData->hSemHandle);
	}

	if (drv->pData->eBlockMode == BLOCK_MODE_NON_BLOCKING) {
		/* if not using callbacks Post Semaphore */
		if (drv->pData->Callback == (void*)0) {
			status = adi_osal_SemPost(drv->pData->hSemHandle);
		}

		/* continue processing if another buffer is ready */
		result = InitiateProcessing(drv);
	}
}

/*
 * SlaveService
 *
 * Process interrupts received in Slave mode.
 */
static void SlaveService(ADI_TWI_DRIVER *drv, uint16_t stat)
{
	bool bResult;
	ADI_TWI_EVENT event;
	uint16_t slave_stat = drv->pReg1->slvstat;

	if (drv->pData->activeBuffer == (ADI_TWI_BUFFER_STRUCT*)0u)
	{
		/* no active buffer */
		return;
	}

	/* SCOMP or SERR or SOVF will end a transfer */
	if ((stat & (uint16_t)(BITM_TWI_ISTAT_SCOMP|BITM_TWI_ISTAT_SERR|BITM_TWI_ISTAT_SOVF)) != 0u)
	{
		if ((stat & (uint16_t)BITM_TWI_ISTAT_SERR) != 0u)
		{
			/* "A restart or stop condition has occurred during the data receive
			 * phase of a transfer." -- this probably implies SCOMP as well
			 * this branch was NOT tested: how to force this event?
			 */
			event = ADI_TWI_EVENT_SERR;
		}
		else if ((stat & (uint16_t)BITM_TWI_ISTAT_SOVF) != 0u)
		{
			/* "The Slave Transfer Complete (SCOMP) bit was set at the time a
			 * subsequent transfer has acknowledged an address phase. The transfer
			 * continues; however, it may be difficult to delineate data of one
			 * transfer from another."
			 * this branch was NOT tested: how to force this event?
			 */
			event = ADI_TWI_EVENT_SOVF;
		}
		else
		{
			/* "The transfer is complete and either a stop, or a restart was detected." */
			event = ADI_TWI_EVENT_SCOMP;
		}

		if (drv->pData->activeBuffer->Direction == OUTBOUND)
		{
			/* Flush the fifo's and dispatch the transmit buffer */
			FlushDevice(drv);
			SlaveTxProcessed(drv, event);
		}
		else
		{
			/* if there is any data in the RCV FIFO, store it */
			if ((drv->pReg1->fifostat & (uint16_t)BITM_TWI_FIFOSTAT_RXSTAT) != 0u)
			{
				bResult = BufferReceive(drv);
			}
			SlaveRxProcessed(drv, event); /* send buffer back to the client */
		}
	}

	if ((stat & (uint16_t)BITM_TWI_IMSK_SINIT) != 0u)
	{
		/* "The slave has detected an address match and a transfer has been initiated." */
		drv->pData->activeBuffer->gcall = 0u;
		if ((slave_stat & (uint16_t)BITM_TWI_SLVSTAT_GCALL) != 0u)
		{
			if ((slave_stat & (uint16_t)BITM_TWI_SLVSTAT_DIR) != 0u) /* do not support GCALL XMT */
			{
			    adi_osal_EnterCriticalRegion();

				drv->pReg1->slvctl &= (uint16_t)~BITM_TWI_SLVCTL_EN; /* so kill transfer by */
				drv->pReg1->slvctl |= (uint16_t)BITM_TWI_SLVCTL_EN;  /* toggling SEN */

				adi_osal_ExitCriticalRegion();
			}
			else
			{
				drv->pData->activeBuffer->gcall = 1u;    /* process as normal receive until dispatch */
			}
		}
	}

	/* SINIT was found to occur with either the XMTSERV or RCVSERV that caused it */
	if ((stat & (uint16_t)BITM_TWI_ISTAT_RXSERV) != 0u)
	{
		bResult = BufferReceive(drv);
	}
	if ((stat & (uint16_t)BITM_TWI_ISTAT_TXSERV) != 0u)
	{
		BufferTransmit(drv);
	}
}

/*
 * Make a write buffer callback
 */
static void SlaveTxProcessed(ADI_TWI_DRIVER *drv,  ADI_TWI_EVENT event)
{
	ADI_OSAL_STATUS status;
	ADI_TWI_RESULT result;

	if (event == ADI_TWI_EVENT_SCOMP)
	{
		event = ADI_TWI_EVENT_XMT_PROCESSED;
	}

	if (drv->pData->Callback != (void*)0)
	{
		(drv->pData->Callback)(drv, (uint32_t)event, drv->pData->pCallParam);
	}

	/* reset all the counters */
	drv->pData->activeBuffer->lostarb = 0u;
	drv->pData->activeBuffer->anak = 0u;
	drv->pData->activeBuffer->dnak = 0u;
	drv->pData->activeBuffer->Complete = true;

	if (drv->pData->eBlockMode == BLOCK_MODE_BLOCKING) {
		/* release blocking */
		status = adi_osal_SemPost(drv->pData->hSemHandle);
	}

	if (drv->pData->eBlockMode == BLOCK_MODE_NON_BLOCKING) {
		/* if not using callbacks Post Semaphore */
		if (drv->pData->Callback == (void*)0) {
			status = adi_osal_SemPost(drv->pData->hSemHandle);
		}

		/* continue processing if another buffer is ready */
		result = InitiateProcessing(drv);
	}
}

/*
 * Make a read buffer callback
 */
static void SlaveRxProcessed(ADI_TWI_DRIVER *drv, ADI_TWI_EVENT event)
{
	ADI_OSAL_STATUS status;
	ADI_TWI_RESULT result;

	if (event == ADI_TWI_EVENT_SCOMP)
	{
		event = ADI_TWI_EVENT_RCV_PROCESSED;
	}

	/* slave gcall is cleared on SINIT */
	if ((drv->pData->activeBuffer->gcall != 0u) && (event == ADI_TWI_EVENT_RCV_PROCESSED))
	{
		event = ADI_TWI_EVENT_GCALL;
	}

	if (drv->pData->Callback != (void*)0)
	{
		(drv->pData->Callback)(drv, (uint32_t)event, drv->pData->pCallParam);
	}

	/* reset all the counters */
	drv->pData->activeBuffer->lostarb = 0u;
	drv->pData->activeBuffer->anak = 0u;
	drv->pData->activeBuffer->dnak = 0u;
	drv->pData->activeBuffer->Complete = true;

	if (drv->pData->eBlockMode == BLOCK_MODE_BLOCKING) {
		/* release blocking */
		status = adi_osal_SemPost(drv->pData->hSemHandle);
	}

	if (drv->pData->eBlockMode == BLOCK_MODE_NON_BLOCKING) {
		/* if not using callbacks Post Semaphore */
		if (drv->pData->Callback == (void*)0) {
			status = adi_osal_SemPost(drv->pData->hSemHandle);
		}

		/* continue processing if another buffer is ready */
		result = InitiateProcessing(drv);
	}
}

/*
 * Start or continue processing.  Processing stops when all buffers are complete.
 */
static 	ADI_TWI_RESULT InitiateProcessing(ADI_TWI_DRIVER *drv)
{
	ADI_TWI_RESULT result;

	/* process buffers */
	if (drv->pData->buffer1.Complete == false) {
		/* process buffer1 */
		drv->pData->activeBuffer = &(drv->pData->buffer1);
	} else if (drv->pData->buffer2.Complete == false) {
		/* process buffer1 */
		drv->pData->activeBuffer = &(drv->pData->buffer2);
	} else {
		/* all buffers complete */
		return ADI_TWI_SUCCESS;
	}

	/* make sure the device is running */
	result = StartDevice(drv);
	if (result == ADI_TWI_SUCCESS)
	{
		switch (drv->pData->config.mode) {
		case ADI_TWI_MASTER:
		case ADI_TWI_MASTER_SCCB:
			MasterService(drv, 0u);
			break;
		case ADI_TWI_SLAVE:
			/* preload the tx fifo */
			if (drv->pData->buffer1.Direction == OUTBOUND)
			{
				BufferTransmit(drv);
			}
			break;
		default:
			break;
		}
	}

	return result;
}

/*
 * TWI Interrupt handler
 */
static ADI_TWI_RESULT processInterrupt(ADI_TWI_DRIVER *drv)
{
	uint16_t stat;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if ((drv->pReg1->istat & drv->pReg1->imsk) == 0u)
	{
		return ADI_TWI_SUCCESS;
	}

	/* process all relevant interrupts */
	stat = (drv->pReg1->istat & drv->pReg1->imsk);
	while (stat != 0u)
	{
		/* clear interrupts about to be serviced and execute the twi handler */
		drv->pReg1->istat = stat;

		switch (drv->pData->config.mode) {
		case ADI_TWI_MASTER:
		case ADI_TWI_MASTER_SCCB:
			MasterService(drv, stat);
			break;
		case ADI_TWI_SLAVE:
			SlaveService(drv, stat);
			break;
		default:
			break;
		}

		stat = (drv->pReg1->istat & drv->pReg1->imsk);
	}
	return ADI_TWI_SUCCESS;
}

/*
 * TWI hardware interrupt handler
 */
static void TWIHandler (uint32_t SID, void *pParam)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER*) pParam;

#if defined(ADI_DEBUG)
	/* confirm this is a valid driver */
	uint32_t i = 0u;
	bool bFound = false;
	for (i=0u; i<NUM_TWI_DEVICES; i++)
	{
#if !defined(ADI_CODE_IN_ROM)
		if (drv == &adi_twi_Device[i])
#else
	    if( drv == &pram_for_rom_Code->adi_twi_Device[i])
#endif
		{
			bFound = true;
			break;
		}
	}
	if (!bFound) {
		return;
	}
#endif

	processInterrupt(drv);
}

/*
 * API implementation:
 */

/**
 * @brief       Open a TWI device instance.
 *
 *  If successful the returned ADI_TWI_HANDLE parameter will hold a device handle
 *  to be used as an input to other TWI API functions.  The TWI device must be
 *  opened before using any other TWI API functions.
 *
 *  The amount of memory required by the driver is specified by the ADI_TWI_MEMORY_SIZE
 *  definition.
 *
 * @param [in]  DeviceNum           The TWI device instance number to be opened.
 * @param [in]  eMode               The TWI operation mode.
 * @param [in]  pMemory             Application supplied memory space for use by the driver.
 * @param [in]  MemorySize          Application supplied memory size (bytes).
 * @param [out] phDevice            A pointer to a location where the handle to the
 *                                  opened device is written.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                 Successfully opened the given instance.
 *              - #ADI_TWI_BAD_DEVICE_NUMBER   [D] The device number is invalid.
 *              - #ADI_TWI_MODE_NOT_SUPPORTED  [D] The TWI mode is invalid mode.
 *              - #ADI_TWI_INSUFFICIENT_MEMORY [D] The application supplied memory size is insufficient.
 *              - #ADI_TWI_DEVICE_IN_USE           The TWI device is already open.
 *              - #ADI_TWI_SEMAPHORE_FAILED        A semaphore related error occurred.
 *
 *
 * @sa          adi_twi_Close()
 */
ADI_TWI_RESULT adi_twi_Open(
		uint32_t               const  DeviceNum,
		ADI_TWI_MODE           const  eMode,
	    void                 * const  pMemory,
	    uint32_t               const  MemorySize,
		ADI_TWI_HANDLE       * const  phDevice)
{
	ADI_OSAL_STATUS status;
#if !defined(ADI_CODE_IN_ROM)
	ADI_TWI_DRIVER *drv = &adi_twi_Device[DeviceNum];
#else
	ADI_TWI_DRIVER *drv = &pram_for_rom_Code->adi_twi_Device[DeviceNum];
#endif

	uint8_t *pDeviceMemory = (uint8_t*)pMemory;

#if defined(ADI_DEBUG)
	if (DeviceNum >= (uint32_t)NUM_TWI_DEVICES)
	{
		return ADI_TWI_BAD_DEVICE_NUMBER;
	}
	if (eMode > ADI_TWI_SLAVE)
	{
		return ADI_TWI_MODE_NOT_SUPPORTED;
	}

	/* Asserts to validate the memory size macros */
	assert(ADI_TWI_MEMORY_SIZE >= DATA_MEM_SIZE + ADI_OSAL_MAX_SEM_SIZE_CHAR);

	if (MemorySize < (uint32_t)ADI_TWI_MEMORY_SIZE)
	{
		return ADI_TWI_INSUFFICIENT_MEMORY;
	}
#endif

    /* check if device is not in use) */
    if (drv->pData == NULL)
    {
        /* initialize the memory for TWI parameters */
        drv->pData = (ADI_TWI_DATA_STRUCT *) pDeviceMemory;
    }
    else
    {
        return ADI_TWI_DEVICE_IN_USE;
    }

	/* initialize driver data */
	Initialize(DeviceNum);

    /* avoid Misra pointer arithmetic rule */
	pDeviceMemory += (uint32_t)DATA_MEM_SIZE;

    /* Create the semaphore */
    status = adi_osal_SemCreateStatic((void *)pDeviceMemory,
    		ADI_OSAL_MAX_SEM_SIZE_CHAR, &drv->pData->hSemHandle, 0u);
	if (status != E_ADI_OSAL_SUCCESS)
    {
        return ADI_TWI_SEMAPHORE_FAILED;
    }

	StopDevice(drv);

	drv->pData->config.mode = eMode;
	drv->pData->bInUse = true;

	*phDevice = (ADI_TWI_HANDLE)drv;

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Close the given TWI device.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully closed the given instance.
 *              - #ADI_TWI_FAILURE               Failed to close the given instance.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 *              - #ADI_TWI_SEMAPHORE_FAILED      A semaphore related error occurred.
 *
 * @sa          adi_twi_Open()
 */
ADI_TWI_RESULT adi_twi_Close(
		ADI_TWI_HANDLE         const hDevice)
{
	ADI_OSAL_STATUS status;
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	StopDevice(drv);

	drv->pData->bInUse = false;
	drv->pData->Callback = (void*)0;
	drv->pData->pCallParam = (void*)0;

	/* destroy semaphore only if it is there is one already created */
	if (drv->pData->hSemHandle != NULL)
	{
		/* destroy semaphore */
		status = adi_osal_SemDestroyStatic(drv->pData->hSemHandle);
		if (status != E_ADI_OSAL_SUCCESS)
		{
			drv->pData = NULL;
			return ADI_TWI_SEMAPHORE_FAILED;
		}
	}

	/* remove driver data */
	drv->pData = NULL;

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Register an application defined callback function.
 *
 * An application defined callback function is used to notify the application of
 * device related events.
 *
 * @param [in]  hDevice             Handle to the TWI device.
 * @param [in]  pfCallback          Application supplied callback function which is
 *                                  called to notify device related events.  A value of
 *                                  NULL disables application callbacks.
 * @param [in]  pCBParam            Application supplied callback parameter which
 *                                  will be passed back in the callback function.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully set the application defined callback function.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 *
 * @sa  #ADI_TWI_EVENT
 */
ADI_TWI_RESULT adi_twi_RegisterCallback(
    ADI_TWI_HANDLE         const hDevice,
    ADI_CALLBACK           pfCallback,
    void                  *pCBParam)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	drv->pData->Callback = pfCallback;
	drv->pData->pCallParam = pCBParam;

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Write data to a TWI device (non-blocking).
 *
 * This function returns immediately (does not wait for the write to complete).
 * An interrupt is generated when the transfer is complete or an error occurs.
 * The supplied buffer is owned by the driver during the data transfer process.
 * This buffer should not be modified or deleted by the application until the
 * data transfer is complete.  It's recommended that the buffer not be a
 * local variable.
 *
 * @param [in]  hDevice             Handle to the TWI device.
 * @param [in]  *pBuffer            The buffer that contains the data to transmit.
 * @param [in]  nBufSize            The number of bytes to write.
 * @param [in]  bRestart            Issue a repeat start condition at the conclusion of the current transfer.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                Successfully submitted the buffer for writing.
 *              - #ADI_TWI_FAILURE                Failed to submit the buffer for writing.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D]  The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D]  The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID  The blocking mode is invalid for this API.
 *
 *
 * @sa          adi_twi_SubmitRxBuffer()
 */
ADI_TWI_RESULT adi_twi_SubmitTxBuffer(
		ADI_TWI_HANDLE         const hDevice,
		void                        *pBuffer,
		uint32_t               const nBufSize,
		bool                   const bRestart)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode == BLOCK_MODE_BLOCKING) {
		return ADI_TWI_BLOCKING_MODE_INVALID;
	}

	if (drv->pData->buffer1.Complete) {
		drv->pData->buffer1.Data = pBuffer;
		drv->pData->buffer1.Count = nBufSize;
		drv->pData->buffer1.Index = 0u;
		drv->pData->buffer1.Direction = OUTBOUND;
		drv->pData->buffer1.Restart = bRestart;
		drv->pData->buffer1.Complete = false;
		drv->pData->buffer1.bError = false;
	} else if (drv->pData->buffer2.Complete) {
		drv->pData->buffer2.Data = pBuffer;
		drv->pData->buffer2.Count = nBufSize;
		drv->pData->buffer2.Index = 0u;
		drv->pData->buffer2.Direction = OUTBOUND;
		drv->pData->buffer2.Restart = bRestart;
		drv->pData->buffer2.Complete = false;
		drv->pData->buffer2.bError = false;
	} else {
		return ADI_TWI_BUFFER_NOT_AVAILABLE;
	}

	if (drv->pData->eBlockMode == BLOCK_MODE_NONE) {
		drv->pData->eBlockMode = BLOCK_MODE_NON_BLOCKING;
	}

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Read data from a TWI device (non-blocking).
 *
 * This function returns immediately (does not wait for the read to complete).
 * An interrupt is generated when the transfer is complete or an error occurs.
 * The supplied buffer is owned by the driver during the data transfer process.
 * This buffer should not be modified or deleted by the application until the
 * data transfer is complete.  It's recommended that the buffer not be a
 * local variable.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [out] pBuffer             The data buffer that will contain the received data.
 * @param [in]  nBufSize            The size (in bytes) of the data buffer.
 * @param [in]  bRestart            Issue a repeat start condition at the conclusion of the current transfer.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                Successfully submitted the buffer for reading.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE  [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN    [D] The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID  The blocking mode is invalid for this API.
 *
 * @sa          adi_twi_SubmitTxBuffer()
 */
ADI_TWI_RESULT adi_twi_SubmitRxBuffer(
		ADI_TWI_HANDLE         const hDevice,
		void                        *pBuffer,
		uint32_t               const nBufSize,
		bool                   const bRestart)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode == BLOCK_MODE_BLOCKING) {
		return ADI_TWI_BLOCKING_MODE_INVALID;
	}

	if (drv->pData->buffer1.Complete) {
		drv->pData->buffer1.Data = pBuffer;
		drv->pData->buffer1.Count = nBufSize;
		drv->pData->buffer1.Index = 0u;
		drv->pData->buffer1.Direction = INBOUND;
		drv->pData->buffer1.Restart = bRestart;
		drv->pData->buffer1.Complete = false;
		drv->pData->buffer1.bError = false;
	} else if (drv->pData->buffer2.Complete) {
		drv->pData->buffer2.Data = pBuffer;
		drv->pData->buffer2.Count = nBufSize;
		drv->pData->buffer2.Index = 0u;
		drv->pData->buffer2.Direction = INBOUND;
		drv->pData->buffer2.Restart = bRestart;
		drv->pData->buffer2.Complete = false;
		drv->pData->buffer2.bError = false;
	} else {
		return ADI_TWI_BUFFER_NOT_AVAILABLE;
	}

	if (drv->pData->eBlockMode == BLOCK_MODE_NONE) {
		drv->pData->eBlockMode = BLOCK_MODE_NON_BLOCKING;
	}

	return ADI_TWI_SUCCESS;
}

/**
 * @brief Checks if a Rx Buffer is available for processing.
 *
 * The Rx buffer is available if no Rx buffer transfer is in progress.
 *
 * @param [in]  hDevice             Handle to the TWI device.
 * @param [out] pbAvailable         True if a Rx buffer transfer is not in progress, false otherwise.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                Successfully checked if the Rx Buffer is available for processing.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE  [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN    [D] The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID  The blocking mode is invalid for this API.
 */
ADI_TWI_RESULT adi_twi_IsRxBufferAvailable(
	    ADI_TWI_HANDLE         const hDevice,
	    bool                   *pbAvailable)
{
	ADI_TWI_RESULT result;
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode == BLOCK_MODE_NON_BLOCKING) {
		if (drv->pData->buffer1.Complete) {
			*pbAvailable = true;
		} else if (drv->pData->buffer2.Complete) {
			*pbAvailable = true;
		} else {
			*pbAvailable = false;
		}

		result = ADI_TWI_SUCCESS;
	} else {
		result = ADI_TWI_BLOCKING_MODE_INVALID;
	}

	return result;
}

/**
 * @brief Checks if the Tx Buffer is available for processing.
 *
 * The Tx buffer is available if no Tx buffer transfer is in progress.
 *
 * @param [in]  hDevice             Handle to the TWI device.
 * @param [out] pbAvailable         True if a Tx buffer transfer is not in progress, false otherwise.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                Successfully checked if the Tx Buffer is available for processing.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE  [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN    [D] The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID  The blocking mode is invalid for this API.
 */
ADI_TWI_RESULT adi_twi_IsTxBufferAvailable(
	    ADI_TWI_HANDLE         const hDevice,
	    bool                   *pbAvailable)
{
	ADI_TWI_RESULT result;
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode == BLOCK_MODE_NON_BLOCKING) {
		if (drv->pData->buffer1.Complete) {
			*pbAvailable = true;
		} else if (drv->pData->buffer2.Complete) {
			*pbAvailable = true;
		} else {
			*pbAvailable = false;
		}

		result = ADI_TWI_SUCCESS;
	} else {
		result = ADI_TWI_BLOCKING_MODE_INVALID;
	}

	return result;
}

/**
 * @brief Return the Rx buffer if a filled buffer is available, otherwise
 *        wait until a buffer is filled.
 *
 * @note This API is used for non-blocking mode only.
 *
 * @param [in]  hDevice       Handle to the TWI device.
 * @param [out] ppBuffer      The available Rx buffer.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                 Successfully returned a Rx buffer.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE   [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN     [D] The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID   The blocking mode is invalid for this API.
 *              - #ADI_TWI_SEMAPHORE_FAILED        The semaphore related error occurred.
 *              - #ADI_TWI_BUFFER_NOT_AVAILABLE    A descriptor is not available to store the given buffer.
 *
 * @sa adi_twi_SubmitRxBuffer()
 */
ADI_TWI_RESULT adi_twi_GetRxBuffer (
                    ADI_TWI_HANDLE   const   hDevice,
                    void                   **ppBuffer)
{
	ADI_TWI_RESULT result = ADI_TWI_SUCCESS;
	ADI_OSAL_STATUS status;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode == BLOCK_MODE_BLOCKING) {
		return ADI_TWI_BLOCKING_MODE_INVALID;
	}

	/* pend until buffer is available */
	status = adi_osal_SemPend(drv->pData->hSemHandle, ADI_OSAL_TIMEOUT_FOREVER);
	if (status != E_ADI_OSAL_SUCCESS)
	{
		result = ADI_TWI_SEMAPHORE_FAILED;
	}

	if ((drv->pData->buffer1.Direction == INBOUND) && (drv->pData->buffer1.Complete)) {
		*ppBuffer = drv->pData->buffer1.Data;
	} else if ((drv->pData->buffer2.Direction == INBOUND) && (drv->pData->buffer2.Complete)) {
		*ppBuffer = drv->pData->buffer2.Data;
	} else {
		result = ADI_TWI_BUFFER_NOT_AVAILABLE;
	}

	if ((drv->pData->buffer1.Complete) && (drv->pData->buffer2.Complete)) {
		drv->pData->eBlockMode = BLOCK_MODE_NONE;
	}

	return result;
}

/**
 * @brief Return the Tx buffer if a filled buffer is available, otherwise
 *        wait until a buffer is filled.
 *
 * @note This API is used for non-blocking mode only.
 *
 * @param [in]  hDevice       Handle to the TWI device.
 * @param [out] ppBuffer      The available Tx buffer.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                 Successfully returned a Tx buffer.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE   [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN     [D] The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID   The blocking mode is invalid for this API.
 *              - #ADI_TWI_SEMAPHORE_FAILED        The semaphore related error occurred.
 *              - #ADI_TWI_BUFFER_NOT_AVAILABLE    A descriptor is not available to store the given buffer.
 *
 * @sa adi_twi_SubmitTxBuffer()
 */
ADI_TWI_RESULT adi_twi_GetTxBuffer (
                    ADI_TWI_HANDLE   const hDevice,
                    void                 **ppBuffer)
{
	ADI_TWI_RESULT result;
	ADI_OSAL_STATUS status;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode == BLOCK_MODE_BLOCKING) {
		return ADI_TWI_BLOCKING_MODE_INVALID;
	}

	/* pend until buffer is available */
	status = adi_osal_SemPend(drv->pData->hSemHandle, ADI_OSAL_TIMEOUT_FOREVER);
	if (status != E_ADI_OSAL_SUCCESS)
	{
		result = ADI_TWI_SEMAPHORE_FAILED;
	}

	if ((drv->pData->buffer1.Direction == OUTBOUND) && (drv->pData->buffer1.Complete)) {
		*ppBuffer = drv->pData->buffer1.Data;
	} else if ((drv->pData->buffer2.Direction == OUTBOUND) && (drv->pData->buffer2.Complete)) {
		*ppBuffer = drv->pData->buffer2.Data;
	} else {
		result = ADI_TWI_BUFFER_NOT_AVAILABLE;
	}

	if ((drv->pData->buffer1.Complete) && (drv->pData->buffer2.Complete)) {
		drv->pData->eBlockMode = BLOCK_MODE_NONE;
	}

	return result;
}

/*
 * @brief This function is used to initiate TWI data transfers in non-blocking mode.
 *
 * This function is called after submitting one or two buffers.
 *
 * @note This API is used for non-blocking mode only.
 *
 * @param [in] hDevice        Handle to the TWI device.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                 Successfully enabled data flow.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE   [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN     [D] The device is not open.
 *
 * @sa adi_twi_SubmitTxBuffer()
 * @sa adi_twi_SubmitRxBuffer()
 */
ADI_TWI_RESULT adi_twi_Enable (
                    ADI_TWI_HANDLE   const   hDevice)
{
	ADI_TWI_RESULT result;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	result = InitiateProcessing(drv);

	return result;
}

/**
 * @brief       Write data to a TWI device (blocking).
 *
 * This function blocks (does not return) until the write is complete.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  *pBuffer            The buffer that contains the data to transmit.
 * @param [in]  nBufSize            The number of bytes to write.
 * @param [in]  bRestart            Issue a repeat start condition at the conclusion of the current transfer.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                 Successfully submitted the buffer for writing.
 *              - #ADI_TWI_FAILURE                 Failed to submit the buffer for writing.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D]   The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D]   The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID   The blocking mode is invalid for this API.
 *              - #ADI_TWI_SEMAPHORE_FAILED        The semaphore related error occurred.
 *              - #ADI_TWI_BUFFER_ERROR            An error occurred while processing the buffer.
 *
 * @sa          adi_twi_Read()
 */
ADI_TWI_RESULT adi_twi_Write(
    ADI_TWI_HANDLE         const hDevice,
    void                        *pBuffer,
    uint32_t               const nBufSize,
    bool                   const bRestart)
{
	ADI_TWI_RESULT result;
	ADI_OSAL_STATUS status;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode != BLOCK_MODE_NONE) {
		return ADI_TWI_BLOCKING_MODE_INVALID;
	}

	drv->pData->eBlockMode = BLOCK_MODE_BLOCKING;

	drv->pData->buffer1.Data = pBuffer;
	drv->pData->buffer1.Count = nBufSize;
	drv->pData->buffer1.Index = 0u;
	drv->pData->buffer1.Direction = OUTBOUND;
	drv->pData->buffer1.Restart = bRestart;
	drv->pData->buffer1.Complete = false;
	drv->pData->buffer1.bError = false;

	result = adi_twi_Enable(hDevice);
	if (result == ADI_TWI_SUCCESS)
	{
		status = adi_osal_SemPend(drv->pData->hSemHandle, ADI_OSAL_TIMEOUT_FOREVER);
		if (status != E_ADI_OSAL_SUCCESS)
		{
			result = ADI_TWI_SEMAPHORE_FAILED;
		}
	}

	drv->pData->eBlockMode = BLOCK_MODE_NONE;

	if (drv->pData->buffer1.bError)
	{
		result = ADI_TWI_BUFFER_ERROR;
	}

	return result;
}

/**
 * @brief       Read data from a TWI device (blocking).
 *
 * This function blocks (does not return) until the read is complete.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [out] pBuffer             The data buffer that will contain the received data.
 * @param [in]  nBufSize            The size (in bytes) of the data buffer.
 * @param [in]  bRestart            Issue a repeat start condition at the conclusion of the current transfer.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                 Successfully submitted the buffer for reading.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D]   The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D]   The device is not open.
 *              - #ADI_TWI_BLOCKING_MODE_INVALID   The blocking mode is invalid for this API.
 *              - #ADI_TWI_SEMAPHORE_FAILED        The semaphore related error occurred.
 *              - #ADI_TWI_BUFFER_ERROR            An error occurred while processing the buffer.
 *
 * @sa          adi_twi_Write()
 */
ADI_TWI_RESULT adi_twi_Read(
    ADI_TWI_HANDLE         const hDevice,
    void                        *pBuffer,
    uint32_t               const nBufSize,
    bool                   const bRestart)
{
	ADI_TWI_RESULT result;
	ADI_OSAL_STATUS status;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	if (drv->pData->eBlockMode != BLOCK_MODE_NONE) {
		return ADI_TWI_BLOCKING_MODE_INVALID;
	}

	drv->pData->eBlockMode = BLOCK_MODE_BLOCKING;

	drv->pData->buffer1.Data = pBuffer;
	drv->pData->buffer1.Count = nBufSize;
	drv->pData->buffer1.Index = 0u;
	drv->pData->buffer1.Direction = INBOUND;
	drv->pData->buffer1.Restart = bRestart;
	drv->pData->buffer1.Complete = false;
	drv->pData->buffer1.bError = false;

	result = adi_twi_Enable(hDevice);
	if (result == ADI_TWI_SUCCESS)
	{
		status = adi_osal_SemPend(drv->pData->hSemHandle, ADI_OSAL_TIMEOUT_FOREVER);
		if (status != E_ADI_OSAL_SUCCESS)
		{
			result = ADI_TWI_SEMAPHORE_FAILED;
		}
	}

	drv->pData->eBlockMode = BLOCK_MODE_NONE;

	if (drv->pData->buffer1.bError)
	{
		result = ADI_TWI_BUFFER_ERROR;
	}

	return result;
}

/**
 * @brief       Set the TWI device address.
 *
 * In master mode this is the address transmitted to the slave device.
 * In slave mode this is the address that the slave responds to.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  Address             In master mode this is the address transmitted to the slave device.
 *                                  In slave mode this is the address that the slave responds to.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully set the address.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 *              - #ADI_TWI_BAD_ADDRESS       [D] The given address is invalid.
 */
ADI_TWI_RESULT adi_twi_SetHardwareAddress(
		ADI_TWI_HANDLE         const hDevice,
		uint16_t               const Address)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if ((Address < MINADDR) || (Address > MAXADDR)) {
		return ADI_TWI_BAD_ADDRESS;
	}
#endif

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	switch (drv->pData->config.mode){
	case ADI_TWI_MASTER:
	case ADI_TWI_MASTER_SCCB:
		drv->pReg1->mstraddr = Address;
		break;
	case ADI_TWI_SLAVE:
		drv->pReg1->slvaddr = Address;
		break;
	default:
		break;
	}

	return ADI_TWI_SUCCESS;
}

/**
 * @brief      Set the TWI prescale value.
 *
 * The prescale value is used to establish a relationship between
 * the system clock (SCLK) and the TWI controllers internally timed
 * events.  The internal time reference is derived from SCLK using a
 * prescaled value.  PRESCALE = fSCLK/10MHz
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  Prescale            The prescale value.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS                Successfully set the prescale value.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE  [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN    [D] The device is not open.
 *              - #ADI_TWI_BAD_PRESCALE_VALUE [D] The given prescale value is invalid.
 */
ADI_TWI_RESULT adi_twi_SetPrescale(
		ADI_TWI_HANDLE         const hDevice,
		uint16_t               const Prescale)
{
	ADI_TWI_RESULT result = ADI_TWI_SUCCESS;
	bool bRestart = false;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	/* if state is running then set restart flag */
	if ((drv->pReg1->ctl & (uint16_t)BITM_TWI_CTL_EN) != 0u)
	{
		bRestart = true;
	}

	/* stop the driver */
	StopDevice(drv);

#if defined(ADI_DEBUG)
	/* check the requested TWI prescale value */
	if ( (Prescale < MINPRESCALE) || (Prescale > MAXPRESCALE) ) {
		return ADI_TWI_BAD_PRESCALE_VALUE;
	}
#endif

	/* save the requested values */
	drv->pData->config.prescale = Prescale;

	/* restart the driver if it was stopped, otherwise success */
	if (bRestart)
	{
		result = StartDevice(drv);
	}

	return result;
}

/**
 * @brief       Set the TWI serial clock bit rate.
 *
 * Set the TWIx_CLKDIV register values used during master mode operation to
 * create the serial clock frequency (SCLx).
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  BitRate             The serial clock bit rate in kHz.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully set the bit rate.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 *              - #ADI_TWI_BAD_BITRATE       [D] The given bit rate is invalid.
 */
ADI_TWI_RESULT adi_twi_SetBitRate(
		ADI_TWI_HANDLE         const hDevice,
		uint16_t               const BitRate)
{
	ADI_TWI_RESULT result = ADI_TWI_SUCCESS;
	bool bRestart = false;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	/* if state is running then set restart flag */
	if ((drv->pReg1->ctl & (uint16_t)BITM_TWI_CTL_EN) != 0u)
	{
		bRestart = true;
	}

	/* stop the driver */
	StopDevice(drv);

#if defined(ADI_DEBUG)
	/* check the requested TWI bit rate value */
	if ( (BitRate < MINFREQ) || (BitRate > MAXFREQ) ) {
		return ADI_TWI_BAD_BITRATE;
	}
#endif

	/* save the requested values */
	drv->pData->config.frequency = BitRate;

	/* restart the driver if it was stopped, otherwise success */
	if (bRestart)
	{
		result = StartDevice(drv);
	}

	return result;
}

/**
 * @brief       Set the TWI duty cycle.
 *
 * Set the TWIx_CLKDIV register values used during master mode operation to
 * create the high and low durations of the serial clock (SCLx).
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  DutyCycle           The serial clock duty cycle (valid range 1-99).
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully set the duty cycle.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 *              - #ADI_TWI_BAD_DUTYCYCLE     [D] The given duty cycle is invalid.
 */
ADI_TWI_RESULT adi_twi_SetDutyCycle(
		ADI_TWI_HANDLE         const hDevice,
		uint16_t               const DutyCycle)
{
	ADI_TWI_RESULT result = ADI_TWI_SUCCESS;
	bool bRestart = false;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	/* if state is running then set restart flag */
	if ((drv->pReg1->ctl & (uint16_t)BITM_TWI_CTL_EN) != 0u)
	{
		bRestart = true;
	}

	/* stop the driver */
	StopDevice(drv);

#if defined(ADI_DEBUG)
	/* check the requested TWI duty cycle values */
	if ( (DutyCycle < MINDUTYCYCLE) || (DutyCycle > MAXDUTYCYCLE) )
	{
		return ADI_TWI_BAD_DUTYCYCLE;
	}
#endif

	/* save the requested values */
	drv->pData->config.duty_cycle = DutyCycle;

	/* restart the driver if it was stopped, otherwise success */
	if (bRestart)
	{
		result = StartDevice(drv);
	}

	return result;
}

/**
 * @brief       Get the value of the interrupt status register.
 *
 * The returned value can be compared using the ADI_TWI_INTERRUPT_STATUS enumeration masks.
 * e.g. if (Value & ADI_TWI_ISTAT_MERR)

 * @param [in]  hDevice             The handle to the TWI device.
 * @param [out] *Value              The value of the interrupt status register SERR and MERR bits.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully read the error status.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 */
ADI_TWI_RESULT adi_twi_GetInterruptStatus(
		ADI_TWI_HANDLE         const hDevice,
		uint16_t                     *Value)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	*Value = drv->pReg1->istat;

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Get the value of the master mode status register.
 *
 * The returned value can be compared using the ADI_TWI_MASTER_STATUS enumeration masks.
 * e.g. if (Value & ADI_TWI_BUFWRERR)
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [out] *Value              The value of the master mode status register.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully read the status register.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 */
ADI_TWI_RESULT adi_twi_GetMasterStatus(
		ADI_TWI_HANDLE               const hDevice,
		uint16_t                     *Value)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	*Value = drv->pReg1->mstrstat;

	return ADI_TWI_SUCCESS;
}

/*
 * @brief       Clear the master mode status register error bits
 *
 * The value is one or more ADI_TWI_MASTER_STATUS error enumeration mask.
 * e.g. ADI_TWI_BUFWRERR|ADI_TWI_BUFRDERR|ADI_TWI_DNAK|ADI_TWI_ANAK|ADI_TWI_LOSTARB
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  Value               The error bits to clear in the master mode status register.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully cleared the status register.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 */
ADI_TWI_RESULT adi_twi_ClearMasterStatus(
    ADI_TWI_HANDLE               const hDevice,
	uint16_t                     const Value)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

	/* write one to clear */
	drv->pReg1->mstrstat = Value;

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Enable master serial clock override.
 *
 * This bit can be used when direct control of the serial clock line is
 * required. Normal master and slave mode operation should not
 * require override operation.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  bEnable             True to enable serial clock override, false to disable.
 *
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully enabled/disabled serial clock override.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 */
ADI_TWI_RESULT adi_twi_EnableClockOverride(
		ADI_TWI_HANDLE        const hDevice,
		bool                  const bEnable)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

    adi_osal_EnterCriticalRegion();

	if (bEnable)
	{
		drv->pReg1->mstrctl |= (uint16_t)BITM_TWI_MSTRCTL_SCLOVR;
	}
	else
	{
		drv->pReg1->mstrctl &= (uint16_t)~BITM_TWI_MSTRCTL_SCLOVR;
	}

    adi_osal_ExitCriticalRegion();

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Enable master serial data override.
 *
 * This bit can be used when direct control of the serial data line is
 * required. Normal master and slave mode operation should not
 * require override operation.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  bEnable             True to enable serial data override, false to disable.
 *
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully enabled/disabled serial data override.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 */
ADI_TWI_RESULT adi_twi_EnableDataOverride(
		ADI_TWI_HANDLE        const hDevice,
		bool                  const bEnable)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

    adi_osal_EnterCriticalRegion();

	if (bEnable)
	{
		drv->pReg1->mstrctl |= (uint16_t)BITM_TWI_MSTRCTL_SDAOVR;
	}
	else
	{
		drv->pReg1->mstrctl &= (uint16_t)~BITM_TWI_MSTRCTL_SDAOVR;
	}

    adi_osal_ExitCriticalRegion();

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Enable slave general call address matching.
 *
 * General call address detection is available only when slave mode is enabled.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  bEnable             True to enable general call address matching, false to disable.
 *
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully enabled/disabled general call.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 */
ADI_TWI_RESULT adi_twi_EnableGeneralCall(
		ADI_TWI_HANDLE        const hDevice,
		bool                  const bEnable)
{
	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

    adi_osal_EnterCriticalRegion();

	if (bEnable)
	{
		drv->pReg1->slvctl |= (uint16_t)BITM_TWI_SLVCTL_GEN;
	}
	else
	{
		drv->pReg1->slvctl &= (uint16_t)~BITM_TWI_SLVCTL_GEN;
	}

    adi_osal_ExitCriticalRegion();

	return ADI_TWI_SUCCESS;
}

/**
 * @brief       Determines the rate at which receive buffer interrupts are to be generated.
 *
 * Interrupts may be generated with each byte received or after two bytes are received.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  eWidth              The data transfer width.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully set the data width.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 *              - #ADI_TWI_BAD_DATAWIDTH         The data width is invalid.
 */
ADI_TWI_RESULT adi_twi_SetReceiveDataWidth(
		ADI_TWI_HANDLE        const hDevice,
		ADI_TWI_DATA_WIDTH    const eWidth)
{
	ADI_TWI_RESULT result = ADI_TWI_SUCCESS;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

    adi_osal_EnterCriticalRegion();

	if (eWidth == ADI_TWI_TWO_BYTE_WIDTH)
	{
		drv->pReg1->fifoctl |= (uint16_t)BITM_TWI_FIFOCTL_RXILEN;
	}
	else if (eWidth == ADI_TWI_ONE_BYTE_WIDTH)
	{
		drv->pReg1->fifoctl &= (uint16_t)~BITM_TWI_FIFOCTL_RXILEN;
	}
	else
	{
		result = ADI_TWI_BAD_DATAWIDTH;
	}

    adi_osal_ExitCriticalRegion();

	return result;
}

/**
 * @brief       Determines the rate at which transmit buffer interrupts are to be generated.
 *
 * Interrupts may be generated with each byte transmitted or after two bytes are transmitted.
 *
 * @param [in]  hDevice             The handle to the TWI device.
 * @param [in]  eWidth              Data transfer width.
 *
 * @return      Status
 *              - #ADI_TWI_SUCCESS               Successfully set the data width.
 *              - #ADI_TWI_BAD_DEVICE_HANDLE [D] The device handle is invalid.
 *              - #ADI_TWI_DEVICE_NOT_OPEN   [D] The device is not open.
 *              - #ADI_TWI_BAD_DATAWIDTH         The data width is invalid.
 */
ADI_TWI_RESULT adi_twi_SetTransmitDataWidth(
		ADI_TWI_HANDLE        const hDevice,
		ADI_TWI_DATA_WIDTH    const eWidth)
{
	ADI_TWI_RESULT result = ADI_TWI_SUCCESS;

	ADI_TWI_DRIVER *drv = (ADI_TWI_DRIVER *)hDevice;

#if defined(ADI_DEBUG)
	if (!IsDeviceHandle(drv))
	{
		return ADI_TWI_BAD_DEVICE_HANDLE;
	}
	if (!drv->pData->bInUse)
	{
		return ADI_TWI_DEVICE_NOT_OPEN;
	}
#endif

    adi_osal_EnterCriticalRegion();

	if (eWidth == ADI_TWI_TWO_BYTE_WIDTH)
	{
		drv->pReg1->fifoctl |= (uint16_t)BITM_TWI_FIFOCTL_TXILEN;
	}
	else if (eWidth == ADI_TWI_ONE_BYTE_WIDTH)
	{
		drv->pReg1->fifoctl &= (uint16_t)~BITM_TWI_FIFOCTL_TXILEN;
	}
	else
	{
		result = ADI_TWI_BAD_DATAWIDTH;
	}

    adi_osal_ExitCriticalRegion();

	return result;
}

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

