/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_uart.c
*
* @brief     This is the primary source file for the UART Driver.
*
*            Contains the implementation of the APIs for the UART peripheral
*            driver. This implementation is specific to UART 4.0 specification
*            and may not function on the older revision of the UART.
*
*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

/** @addtogroup UART_Driver_BF6xx UART Device Driver (ADSP-BF6xx)
 *  @{
 */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


/*********************************************************************

Include files

*********************************************************************/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif

#include <stdint.h>
#include <stdbool.h>
#include <adi_osal.h>
#include <stddef.h>
#include <sys/platform.h>
#include <services/tmr/adi_tmr.h>
#include <services/dma/adi_dma.h>
#include <services/pwr/adi_pwr.h>
#include <assert.h>

/* For anamoly workaround */
#include <sys/exception.h>
#include <sys/16000030.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#include <services/int/adi_int.h>

#include <string.h>

#include <services/dma/adi_dma.h>
#include <drivers/uart/adi_uart.h>

#if defined(ADI_CODE_IN_ROM)
#include "../rom/adi_rom_def.h"
#else
#define CONST const
#endif

#include "adi_uart_def_bf60x.h"

#if defined(__ADSPBF609_FAMILY__)
#include "adi_uart_data_bf60x.c"
#elif defined(ADI_ADSP_CM40Z)
#include "adi_uart_data_cm40z.c"
#elif defined(__ADSPBF707_FAMILY__)
#include "adi_uart_data_bf70x.c"
#else
#error "This processor is not supported"
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_6_3:"Allow to use basic number data types. Required for float which is not defined in stdint")
#pragma diag(suppress:misra_rule_8_5:"Allow defenition of objects in the source file. MISRA treats this as header file.")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined outside the block eventhough they are used in one function, for ease maintainence.")
#pragma diag(suppress:misra_rule_10_4:"Allow typecasting between float and unsigned int. Required for computation of clock divide.")
#pragma diag(suppress:misra_rule_11_4:"Allow cast between different object types.")
#pragma diag(suppress:misra_rule_11_5:"Allow cast which removes const or volatile qualification.")
#pragma diag(suppress:misra_rule_12_13:"Allow increment(++) to be mixed with other operators in an expression for code efficiency.")
#pragma diag(suppress:misra_rule_13_1:"Allow assignment in if statements. Required for efficient code")
#pragma diag(suppress:misra_rule_13_5:"Allow expressions of for statement that are not concerned with loop control for code efficiency.")
#pragma diag(suppress:misra_rule_13_7:"Allow to add asserts for memory size macros")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_15_2:"Allow to return from the switch case statement for efficiency")
#pragma diag(suppress:misra_rule_16_7:"Allow non modified pointer parameters to be declared without const")
#pragma diag(suppress:misra_rule_17_4:"Allow pointer arithmetic for MMR address generation")
#pragma diag(suppress:misra_rule_19_7:"Allow macros to be used for readability")

#endif /* _MISRA_RULES */



/*
 * Function pointer tables for interrupt mode and DMA mode abstraction.
 *
 * By default interrupt mode table will be used and if UART is operating
 * only in interrupt mode the DMA mode table and its associated functions
 * will be eleminated by linker.
 */

/* Function pointer table for interrupt mode */
static const ADI_UART_FN_TABLE   gIntFnTable =
{
    &uart_Int_Close,
    &uart_Int_SubmitBuffer,
    &uart_Int_Enable
};

/* Function pointer table for DMA mode */
static const ADI_UART_FN_TABLE   gDMAFnTable =
{
    &uart_DMA_Close,
    &uart_DMA_SubmitBuffer,
    &uart_DMA_Enable
};

/* Timer callback function */
static void TimerCallback(void *pCBParam, uint32_t Event, void *pArg);

#ifdef ADI_DEBUG

/* Validates the given UART device */
static ADI_UART_RESULT  ValidateHandle(
    ADI_UART_DEVICE     *pDevice
);

#endif /* ADI_DEBUG */


/**
 * @brief       Open the given UART device instance
 *
 * @param [in]  nDeviceNum          UART device instance number to be opened.
 * @param [in]  eDirection          Data flow direction.
 * @param [in]  pMemory             Memory required for driver operation. The
 *                                  memory passed (in bytes) should be equal to one of these
 *                                  values base on the mode of operation.
 *
 *                                  ADI_UART_INT_MEMORY_SIZE        : For interrupt mode only.
 *                                  ADI_UART_UNIDIR_DMA_MEMORY_SIZE : Unidirectional mode with DMA.
 *                                  ADI_UART_UNIDIR_DMA_MEMORY_SIZE : Bidirectional mode with DMA.
 *
 *
 * @param [in]  nMemSize            Size of the given memory in bytes.
 * @param [out] phDevice            Pointer to a location where the handle to the
 *                                  opened device is written.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully opened the given instance.
 *
 *  - #ADI_UART_INVALID_DEVICE_NUM  [D]     if the given device number is greater than the
 *                                          number of UARTs present in the processor.
 *
 *  - #ADI_UART_NULL_POINTER        [D]     if the given pointer to the UART device
 *                                          handle is NULL or given pointer to the device
 *                                          memory is NULL.
 *
 *  - #ADI_UART_ALREADY_OPENED      [D]     if the given UART device instance is already opened
 *                                          or in use.
 *
 *  - #ADI_UART_INSUFFICIENT_MEMORY [D]     If the given amount of memory is not sufficient for
 *                                          given mode of operation.
 *
 *  - #ADI_UART_INT_FAILED                  if status interrupt registration failed.
 *
 * @sa          adi_uart_Close()
 */

ADI_UART_RESULT adi_uart_Open(
    uint32_t                      nDeviceNum,
    ADI_UART_DIRECTION            eDirection,
    void                   *const pMemory,
    uint32_t                      nMemSize,
    ADI_UART_HANDLE        *const phDevice
)
{
    /* Pointer to the UART Device Instance Data */
    ADI_UART_DEVICE         *pDevice;

    /* Pointer to the UART Device information */
    ADI_UART_INFO           *pDevInfo;

    /* Pointer to track the memory available for allocation */
    uint8_t                 *pAvailableMem = pMemory;

    /* Flag to store if application intends to operate in DMA mode (based on given memory size)*/
    bool                    bDMAMode = false;

    /* UART result code */
    ADI_UART_RESULT          eResult;

#ifdef ADI_DEBUG

    /* Check if the given device number is within the range of number UART
     * devices present in the processor
     */
    if(nDeviceNum > ADI_UART_NUM_INSTANCES)
    {
        return ADI_UART_INVALID_DEVICE_NUM;
    }

    /* Check if the pointer to device handle is valid */
    if(phDevice == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }

    /* Check if the given memory pointer is valid */
    if(pMemory == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }

    /* Asserts to validate the memory size macros */
    assert(ADI_UART_UNIDIR_INT_MEMORY_SIZE  ==  sizeof(ADI_UART_DEVICE) + sizeof(ADI_UART_CHANNEL));
    assert(ADI_UART_BIDIR_INT_MEMORY_SIZE   ==  sizeof(ADI_UART_DEVICE) + sizeof(ADI_UART_CHANNEL) * 2u);
    assert(ADI_UART_UNIDIR_DMA_MEMORY_SIZE  ==  (ADI_UART_UNIDIR_INT_MEMORY_SIZE + ADI_DMA_CHANNEL_REQ_MEMORY));
    assert(ADI_UART_BIDIR_DMA_MEMORY_SIZE   ==  (ADI_UART_BIDIR_INT_MEMORY_SIZE + (ADI_DMA_CHANNEL_REQ_MEMORY * 2u)));

    /*
     * Check if the given memory is sufficient for the operation
     */

    if(eDirection == ADI_UART_DIR_BIDIRECTION)
    {
        /* Check if the give memory is sufficient to operate in bidirectional interrupt mode.
         * If the given memory size is bigger than the interrupt mode memory size,
         * check if it is sufficient to operate in DMA mode (assuming that the application
         * is trying to open UART to operate in DMA mode)
         */
        if((nMemSize < ADI_UART_BIDIR_INT_MEMORY_SIZE) ||
           ((nMemSize > ADI_UART_BIDIR_INT_MEMORY_SIZE) && (nMemSize < ADI_UART_BIDIR_DMA_MEMORY_SIZE)))
        {
            return ADI_UART_INSUFFICIENT_MEMORY;
        }
    }
    else
    {
        /* Check if the give memory is sufficient to operate in unidirectional interrupt mode.
         * If the given memory size is bigger than the unidirectional interrupt mode memory size,
         * check if it is sufficient to operate in DMA mode (assuming that the application is trying
         * to open UART to operate in DMA mode)
         */
        if((nMemSize < ADI_UART_UNIDIR_INT_MEMORY_SIZE) ||
           ((nMemSize > ADI_UART_UNIDIR_INT_MEMORY_SIZE) && (nMemSize < ADI_UART_UNIDIR_DMA_MEMORY_SIZE)))
        {
            return ADI_UART_INSUFFICIENT_MEMORY;
        }
    }
#endif /* ADI_DEBUG */

    /* Get pointer to the device information */
#if !defined(ADI_CODE_IN_ROM)
    pDevInfo = &gUARTDevInfo[nDeviceNum];
#else
    pDevInfo = &pram_for_rom_Code->gUARTDevInfo[nDeviceNum];
#endif

    /* Check if the device instance is in use */
    if(pDevInfo->pDevice != NULL)
    {
        return ADI_UART_ALREADY_OPENED;
    }

    /* Grab the device before initializing it so that it cannot be taken
     * while being initialized
     */

    pDevice                         =   (ADI_UART_DEVICE *)pAvailableMem;

    adi_osal_EnterCriticalRegion();
    pDevInfo->pDevice               =   pDevice;
    adi_osal_ExitCriticalRegion();

    /* Clear the given memory */
    memset(pMemory, 0, nMemSize);

    /*
     * Fragment the given memory based on direction
     */

    /* Increment the memory tracking pointer by used amount */
    pAvailableMem                   +=  sizeof(ADI_UART_DEVICE);

    /* Based on the given memory size and direction find if application intends to operate in DMA mode */
    if( ((nMemSize >= ADI_UART_BIDIR_DMA_MEMORY_SIZE) && (eDirection == ADI_UART_DIR_BIDIRECTION)) ||
         ((nMemSize >= ADI_UART_UNIDIR_DMA_MEMORY_SIZE ) && (eDirection != ADI_UART_DIR_BIDIRECTION)) )
    {
        bDMAMode = true;
    }

    /* If the given direction is birection or only Tx direction */
    if((eDirection == ADI_UART_DIR_BIDIRECTION) || (eDirection == ADI_UART_DIR_TRANSMIT))
    {
        pDevice->pTxChannel =   (void *)pAvailableMem;
        pAvailableMem       +=   sizeof(ADI_UART_CHANNEL);

        /* If application intends to operate in DMA mode */
        if(bDMAMode == true)
        {
            pDevice->pTxChannel->pDMAChnlMemory = pAvailableMem;
            pAvailableMem   +=  ADI_DMA_CHANNEL_REQ_MEMORY;
        }

        /* Save the event to be notified to application when buffer is processed */
        pDevice->pTxChannel->eBufProcessedEvent = ADI_UART_EVENT_TX_BUFFER_PROCESSED;

        /* Initalize the Tx channel */
        eResult = InitializeChannel(pDevice->pTxChannel);

        if(eResult != ADI_UART_SUCCESS)
        {
            /* Failed to initialize the channel; free up device instance and return */
            pDevInfo->pDevice  = NULL;
            return eResult;
        }
    }

    /* If the given direction is birection or only Rx direction */
    if((eDirection == ADI_UART_DIR_BIDIRECTION) || (eDirection == ADI_UART_DIR_RECEIVE))
    {
        pDevice->pRxChannel =   (void *)pAvailableMem;
        pAvailableMem       +=   sizeof(ADI_UART_CHANNEL);

        /* If application intends to operate in DMA mode */
        if(bDMAMode == true)
        {
            pDevice->pRxChannel->pDMAChnlMemory    =   (void *)pAvailableMem;
            pAvailableMem   +=  ADI_DMA_CHANNEL_REQ_MEMORY;
        }

        /* Save the event to be notified to application when buffer is processed */
        pDevice->pRxChannel->eBufProcessedEvent = ADI_UART_EVENT_RX_BUFFER_PROCESSED;

        /* Initialize the Rx channel */
        eResult = InitializeChannel(pDevice->pRxChannel);

        if(eResult != ADI_UART_SUCCESS)
        {
            /* If operating in bidirection and a Tx semaphore was created destroy it */
            if((pDevice->pTxChannel != NULL) && (pDevice->pTxChannel->hSemaphore != NULL))
            {
                adi_osal_SemDestroyStatic(pDevice->pTxChannel->hSemaphore);
            }

            /* Failed to initialize the channel; free up device instance and return */
            pDevInfo->pDevice  = NULL;

            return eResult;
        }

    }

    /* Save the given direction */
    pDevice->eDirection             =   eDirection;

    /* Register the UART driver status interrupt handler with interrupt dispatcher */
    if(adi_int_InstallHandler(pDevInfo->nStatusIntID,
                              UARTStatusHandler,
                              pDevice,
                              true) != ADI_INT_SUCCESS)
    {
        /* Could not register the UART status interrupt handler, free up
         * UART device and return  */
         pDevInfo->pDevice  = NULL;

         return ADI_UART_INT_FAILED;
    }

    /* Assume to be operating in interrupt mode, so load the interrupt mode function table */
    pDevice->pFnTable                   =   &gIntFnTable;

    /* Get the pointer to UART registers and save it into the instance data */
    pDevice->pUARTRegs                  =   pDevInfo->pUARTRegs;

    /* Clear the status register */
    pDevice->pUARTRegs->UART_STAT       =   (uint32_t)DEFAULT_STATUS_VALUE;

    /* Clear the mask registers */
    pDevice->pUARTRegs->UART_IMSK_CLR   =   0xFFFFFFFFu;

    /* Set the default mask */
    pDevice->pUARTRegs->UART_IMSK_SET   =   (uint32_t)DEFAULT_MASK_VALUE;

    /* Set the default clock divide factor. Enable Divide By One (EDBO) is set by default.
     * It will be cleared when the mode is set to IrDA as it is not valid in that mode. */
    pDevice->pUARTRegs->UART_CLK        =   (uint32_t)DEFAULT_CLOCK_VALUE;

    /* Enable the UART clocks */
    pDevice->pUARTRegs->UART_CTL        =   (uint32_t)DEFAULT_CONTROL_VALUE;

    /* Change the state of the device to initialized */
    pDevice->eState                     =   ADI_UART_STATE_INITIALIZED;

    /* Store the pointer to the device information into the device */
    pDevice->pDevInfo                   =   pDevInfo;

    /* Set the timer number to be used for Autobaud. UARTx Rx is connected to TACIx of the timer.
     * So it is safe to assume that UART device number can be used as timer number for Autobaud.*/
    pDevice->nTmrDevNum                 =   nDeviceNum;

    /* Return the device handle to the application */
    *phDevice   =   pDevice;

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Closes the given UART
 *
 * Stops any transfer that is pending closes the DMA channels. It is application
 * responsibility to make sure that the data transfer is complete
 * before closing the device. Once the device is closed it looses all its
 * configuration, it should be reconfigured when opened again.
 *
 * Follow these steps to gracefully close the device. If application does not
 * require to wait until the data is transferred completely. It can close the
 * device immediately.
 *
 *  - Disable data flow by calling function adi_uart_EnableTx.
 *  - Wait for the data to be completely sent out by polling the Transmit
 *    Empty Status bit. Call function adi_uart_GetTxEmpty().
 *  - When adi_uart_GetTxEmpty() returns true, it is safe to close the device.
 *    call this adi_uart_Close() to close the device.
 *
 * @param [in]  hDevice             Handle to the UART device to be closed.
 *
 * @return      Status
 *  - #ADI_UART_SUCCESS                     if successfully closed the given instance.
 *  - #ADI_UART_INVALID_HANDLE      [D]     if the given UART handle is invalid.
 *  - #ADI_UART_INT_FAILED                  If failed to unregister the status interrupt handler.
 *  - #ADI_UART_SEM_FAILED                  If failed to destroy the seamphores.
 *  - #ADI_UART_DMA_FAILED                  if failed to close the Rx or Tx DMA Channels.
 *
 * @sa          adi_uart_Open()
 */
ADI_UART_RESULT adi_uart_Close(
    ADI_UART_HANDLE         const hDevice
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE     *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT     eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Stop the UART clocks */
    pDevice->pUARTRegs->UART_CTL        &= ~((uint32_t)BITM_UART_CTL_EN);

    /* Clear the mask registers */
    pDevice->pUARTRegs->UART_IMSK_CLR   =   (uint32_t)DEFAULT_MASK_VALUE;

    /* Destroy the semaphores if they were created */
    if(pDevice->pTxChannel->hSemaphore != NULL)
    {
        if(adi_osal_SemDestroyStatic(pDevice->pTxChannel->hSemaphore) != ADI_OSAL_SUCCESS)
        {
            return ADI_UART_SEM_FAILED;
        }
    }

    if(pDevice->pRxChannel->hSemaphore != NULL)
    {
        if(adi_osal_SemDestroyStatic(pDevice->pRxChannel->hSemaphore) != ADI_OSAL_SUCCESS)
        {
            return ADI_UART_SEM_FAILED;
        }
    }

    /* Close the UART Tx DMA Channel if opened */
    if((eResult = pDevice->pFnTable->pfClose(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Unregister the status interrupt handler */
    if(adi_int_UninstallHandler(pDevice->pDevInfo->nStatusIntID) != ADI_INT_SUCCESS)
    {
        return ADI_UART_INT_FAILED;
    }

    /* Free up the device */
    adi_osal_EnterCriticalRegion();
    pDevice->pDevInfo->pDevice   = NULL;
    adi_osal_ExitCriticalRegion();

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Sets the UART baud rate
 *
 * Configures UART divide registers according to the required Baud Rate and
 * System Clock frequency. If the system clock frequency changes, it is
 * application responsibility to call this function to change the divide
 * registers according to the new System Clock frequency.
 *
 * @param [in]  hDevice             Handle to the UART whose baud rate to be set.
 * @param [in]  BaudRate            Baud rate to be set in Hz.
 *
 * @return      Status
 *  - #ADI_UART_SUCCESS                     If successfully set the baud rate.
 *
 *  - #ADI_UART_GET_SYSCLK_FAILED           If failed to get system clock frequency.
 *
 *  - #ADI_UART_INVALID_PARAMETER       [D] If the given Baud Rate is not
 *                                          valid.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] If the given handle is not valid
 *                                          or NULL.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If trying to set the baud rate when
 *                                          auto baud is in progress or data flow is
 *                                          enabled.
 *
 * @sa          adi_uart_StartAutoBaud()
 */
ADI_UART_RESULT adi_uart_SetBaudRate(
    ADI_UART_HANDLE         const hDevice,
    uint32_t                      BaudRate
)
{
     /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;
     /* Bit Clock Prescaler value.  Based on UART_CLOCK.EDBO bit it will be either 1 or 16 */
     float                  nPreScale = 16.0;

     /* System Clock Frequencies*/
     uint32_t   fsysclk = 0u;   /* System Clock */
     uint32_t   fsclk0  = 0u;   /* System Clock0 */
     uint32_t   fsclk1  = 0u;   /* System Clock1 */

     /* UART Clock Divide Factor */
     uint32_t               ClkDivide;
     /* Contents of UART_CLOCK register */
     uint32_t               UARTClkValue;

#ifdef ADI_DEBUG
    /* Check if the given baud rate is valid */
    if(BaudRate == 0u)
    {
        return ADI_UART_INVALID_PARAMETER;
    }

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to change the baud rate when auto baud is in progress
     * or when data flow is enabled.
     */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* 2DO: Add code to protect critical region here */

    /* Get the current contents of UART_CLOCK register */
    UARTClkValue    =   pDevice->pUARTRegs->UART_CLK;

    /* If Enable Divide By One is set then prescale value is 1 otherwise 16 */
    if((UARTClkValue & BITM_UART_CLK_EDBO) == ENUM_UART_CLK_EN_DIV_BY_ONE)
    {
        nPreScale   =   1.0;
    }

    /* Get system clock frequencies */
    if(adi_pwr_GetSystemFreq(&fsysclk, &fsclk0, &fsclk1) != ADI_PWR_SUCCESS)
    {
        return ADI_UART_GET_SYSCLK_FAILED;
    }
    
    /* Using floating point divide for accuracy. Round up the result */
#if defined(ADI_ADSP_CM40Z) 
    /* In case of CM40Z processor there is only main system clock. And UART is
       connected to main system clock. So use fsysclk for computation. */
    ClkDivide = (uint32_t) ( ((float)fsysclk / (nPreScale * (float)BaudRate)) + 0.5 );
#else    
    /* UART is connected to SCLK0 on other processors */
    ClkDivide = (uint32_t) ( ((float)fsclk0 / (nPreScale * (float)BaudRate)) + 0.5 );
#endif    

    /* Clear the existing divide factor */
    UARTClkValue    &=  (uint32_t)(~BITM_UART_CLK_DIV);

    /* Update with the new divide factor */
    UARTClkValue    |=  (ClkDivide & (uint32_t) BITM_UART_CLK_DIV);

    /* 2DO: Add code to protect critical region here */

    /* Write back the register contents */
    pDevice->pUARTRegs->UART_CLK   =   UARTClkValue;

    /* Save the given Baud rate */
    pDevice->nBaudRate  =   BaudRate;
    
    return ADI_UART_SUCCESS;
}


/**
 * @brief       Configure UART device for autobaud detection.
 *
 * @details     This function helps to specify the timer to be used and the character to
 *              be used for the autobaud detection.
 *
 * @param [in]  hDevice         Handle to the UART device to be configured for Autobaud.
 * @param [in]  AutoBaudChar    Character for Autobaud.
 *
 * @return      Status
 *    - #ADI_UART_SUCCESS                   Successfully configured for Autobaud detection.
 *    - #ADI_UART_INVALID_HANDLE        [D] Invalid  UART device handle.
 *    - ADI_UART_OPERATION_NOT_ALLOWED  [D] When trying to configure Autobaud when
 *                                          Autobaud is in progress or dataflow is enabled.
 *
 * @sa  adi_uart_EnableAutobaud()
 *
 * @note:  By default, timer-0 and NULL character are used for the autobaud detection.
*/
ADI_UART_RESULT adi_uart_ConfigAutobaud(
    ADI_UART_HANDLE         const hDevice,
    uint8_t                       AutoBaudChar
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;
     /* Mask value for detecting number of bits in charecter */
     uint16_t               nMask;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to change the configuration when auto baud is in progress
     * or when data flow is enabled.
     */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }

#endif /* ADI_DEBUG */

    /* calculate divisor bits from the character value */
    nMask = 0x1u;

    pDevice->nDivisorBits = 0x1u;

    /* check each bit of the character for first occurance of 1 */
    while ((AutoBaudChar & nMask) == 0u)
    {
        nMask <<= 1u;
        pDevice->nDivisorBits += 1u;

        /* character set is a NULL character */
        if (pDevice->nDivisorBits == 9u)
        {
            break;
        }
    }

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Enables/Disables UART Auto Baud detection.
 *
 * @details     Default character for autobaud is space(' ')character and GP Timer 0
 *              is used for auto baud detection. These parameter can be configured using
 *              API #adi_uart_ConfigAutobaud().
 *
 *              Autobaud detection requires UART word size to be 8bit. Application should call
 *              the API #adi_uart_SetWordLen() to set the word size to 8bit. The default word
 *              length is 5bits.
 *
 *              The baud rate is detected using the hardware support (GP Timer).
 *              After the baud rate is detected an event (ADI_UART_EVENT_AUTOBAUD_COMPLETE)
 *              is notified to the application.  When a callback is not registered with UART driver,
 *              the API adi_uart_GetBaudRate can be used to know if the autobaud is complete.
 *              If the API adi_uart_GetBaudRate returns baudrate other than 0, it indicates that
 *              the autobaud complete, otherwise autobaud is still in progress.
 *
 * @param [in]  hDevice    Handle to UART device whose autobaud detection to be enabled/disabled.
 * @param [in]  bEnable    Boolean flag to indicate whether to enable or disable the autobaud.
 *                         - true to Enable the Autobaud
 *                         - false to Disable the Autobaud
 *
 * @return      Status
 *    - #ADI_UART_SUCCESS                       Successfully enabled/disabled Autobaud detection.
 *    - #ADI_UART_OPERATION_NOT_ALLOWED     [D] If trying to enable/disable Autobaud when
 *                                              dataflow is enabled.
 *    - #ADI_UART_TMR_FAILED                    Failed to configure timer for Autobaud.
 *    - #ADI_UART_INVALID_HANDLE            [D] Invalid UART device handle.
 *
 * @sa  adi_uart_ConfigAutobaud()
 * @sa  adi_uart_GetBaudRate()
 * @sa  adi_uart_SetWordLen()
 *
 */
ADI_UART_RESULT adi_uart_EnableAutobaud(
   ADI_UART_HANDLE          const hDevice,
   bool                           bEnable
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to enable/disable autobaud when data flow is enabled.
     */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    if(bEnable == true)
    {

        /* If number of divisor bits is not set, set it to default value */
        if(pDevice->nDivisorBits == 0u)
        {
            pDevice->nDivisorBits   =   ADI_UART_DEFAULT_DIVISOR_BITS;
        }

        /* Open the timer for autobaud detection */
        if(adi_tmr_Open(pDevice->nTmrDevNum,
                        pDevice->TimerMemory,
                        ADI_TMR_MEMORY,
                        TimerCallback,
                        pDevice,
                        &pDevice->hTimer
                        ) != ADI_TMR_SUCCESS)
        {
            return ADI_UART_TMR_FAILED;
        }
        /* Set the timer mode  */
        if(adi_tmr_SetMode(pDevice->hTimer,
                           ADI_TMR_MODE_CAPTURE_DEASSERT)!= ADI_TMR_SUCCESS)
        {
            return ADI_UART_TMR_FAILED;
        }

        /* Set the clock source */
        if(adi_tmr_SetClkInSource(pDevice->hTimer,
                                  ADI_TMR_CLKIN_SYSCLK) != ADI_TMR_SUCCESS)
        {
            return ADI_UART_TMR_FAILED;
        }

        /* Set timer intput (signal to be captured) to auxilary input */
        if(adi_tmr_SetInputSource(pDevice->hTimer, true) != ADI_TMR_SUCCESS)
        {
           return ADI_UART_TMR_FAILED;
        }


        /* Set the timer IRQ mode to generate interrupt after capturing delay + width */
        if(adi_tmr_SetIRQMode(pDevice->hTimer,
                        ADI_TMR_IRQMODE_WIDTH_DELAY) != ADI_TMR_SUCCESS)
        {
            return ADI_UART_TMR_FAILED;
        }

        /* Disable pad output */
        if(adi_tmr_DisableOutput(pDevice->hTimer, true) != ADI_TMR_SUCCESS)
        {
           return ADI_UART_TMR_FAILED;
        }

        /* Enable the timer */
        if(adi_tmr_Enable(pDevice->hTimer, true) != ADI_TMR_SUCCESS)
        {
           return ADI_UART_TMR_FAILED;
        }

        /* Change the state to indicate autobaud in progress */
        pDevice->eState = ADI_UART_STATE_AUTOBAUD_IN_PROGRESS;
    }
    else
    {
        /* Close the timer device */
        if(adi_tmr_Close(pDevice->hTimer) != ADI_TMR_SUCCESS)
        {
           return ADI_UART_TMR_FAILED;
        }

        /* Change the state back */
        pDevice->eState = ADI_UART_STATE_INITIALIZED;
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Get baud rate.
 *
 * @details     Get Baudrate of  the given UART device. This function will be helpful to know
 *              the baudrate when the device is set in autobaud mode.
 *
 * @param [in]  hDevice     Handle to UART device whose baudrate to be queried.
 * @param [in]  pnBaudRate  Pointer to a location where baudrate is to be written.
 *
 * @return      Status
 *    - #ADI_UART_SUCCESS              Successfully queried the baudrate.
 *    - #ADI_UART_INVALID_HANDLE [D]   Invalid UART device handle.
 *
 * @sa  adi_uart_EnableAutobaud()
 * @sa  adi_uart_ConfigAutobaud()
 *
 */
ADI_UART_RESULT adi_uart_GetBaudRate(
    ADI_UART_HANDLE         const hDevice,
    uint32_t *              const pnBaudRate
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Validate the given pointer */
    if(pnBaudRate == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Return the baudrate */
    *pnBaudRate     =   pDevice->nBaudRate;

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Set the UART operation mode
 *
 * @param [in]  hDevice             Handle to the UART whose operation mode to be set.
 * @param [in]  eMode               Operation mode.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully set the operation mode.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given handle is not valid
 *                                          or NULL.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to set the mode when auto baud
 *                                          is in progress or data flow is enabled.
 *
 * @note        In MDB mode the driver only supports the protocol by providing
 *              APIs required. The protocol itself is assumed to be implemented
 *              by the application.
 *
 * @sa          adi_uart_MDBTxAddress
 * @sa          adi_uart_MDBDiscardRxData
 *
 */
ADI_UART_RESULT adi_uart_SetMode(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_MODE                 eMode
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;
     /* UART Control Register Value */
     uint32_t               CtlRegValue;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to change the UART mode when auto baud is in progress
     * or when data flow is enabled.
     */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* 2DO Add code to protect critical region here */

    /* Get the current contents of UART Control register */
    CtlRegValue =   pDevice->pUARTRegs->UART_CTL;

    /* Clear the existing mode */
    CtlRegValue &=  (uint32_t) (~BITM_UART_CTL_MOD);

    /* Update with new mode */
    CtlRegValue |=  (uint32_t) eMode;

    /* Write back the contents of the Control register */
    pDevice->pUARTRegs->UART_CTL    =   CtlRegValue;

    /* Save the given mode */
    pDevice->eMode  =   eMode;

    /* If the given mode is IrDA mode clear the UART_CLK.EDBO bit and update
     * the divide factor accordingly.
     */
    if(eMode == ADI_UART_MODE_IRDA)
    {
        pDevice->pUARTRegs->UART_CTL &= (~ENUM_UART_CLK_EN_DIV_BY_ONE);
    }
    else
    {
        pDevice->pUARTRegs->UART_CTL |= ENUM_UART_CLK_EN_DIV_BY_ONE;
    }

    /* If EDBO bit is changed then the baud rate should be recomputed */
    if(pDevice->nBaudRate != 0u)
    {
        adi_uart_SetBaudRate(hDevice, pDevice->nBaudRate);
    }

    /* If it is MDB Mode enable address word interrupt otherwise disable it */
    if(eMode == ADI_UART_MODE_MDB)
    {
        pDevice->pUARTRegs->UART_IMSK_SET = (uint32_t)ENUM_UART_EAWI_HI;
    }
    else
    {
        pDevice->pUARTRegs->UART_IMSK_CLR = (uint32_t)ENUM_UART_EAWI_HI;
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Submits a buffer for transmission.
 *
 * @details     This function should be called atleast once before enabling the data flow.
 *              At most two buffers can be submitted before getting a processed buffer.
 *              The processed buffer can be obtained using adi_uart_GetTxBuffer (in case of
 *              no callback is registered) or via the callback as callback argument to
 *              the ADI_UART_EVENT_TX_BUFFER_PROCESSED event. Once a processed buffer
 *              is recieved a subsequent buffer can be submitted.
 *
 * @param [in]  hDevice             Handle to the UART to which the buffer is submitted.
 * @param [in]  pBuffer             Pointer to the buffer.
 * @param [in]  nBufferSize         Size of the given buffer in bytes.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully submitted the buffer.
 *
 * - #ADI_UART_INVALID_HANDLE           [D]     if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR   [D]     if trying to submit a buffer when
 *                                              device is not opened for this direction.
 *
 * - #ADI_UART_NULL_POINTER             [D]     if the given pointer to buffer is NULL.
 *
 * - #ADI_UART_INVALID_PARAMETER        [D]     If the given buffer size is zero or buffer
 *                                              size is not aligned to the DMA transfer size
 *                                              (in DMA mode) set by the API
 *                                              adi_uart_SetTxDmaTransferSize.
 *
 * @sa adi_uart_GetTxBuffer
 * @sa adi_uart_SetTxDmaTransferSize
 */
ADI_UART_RESULT adi_uart_SubmitTxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Rx Direction or bi-direction, for submitting Tx Buffers */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
        (pDevice->eDirection != ADI_UART_DIR_TRANSMIT))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }

    /* Submit buffer can be only used in Callback or Non-blocking mode */
    if(pDevice->pTxChannel->eBufTxnMode == ADI_UART_BUF_TXN_MODE_BLOCKING)
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }
#endif /* ADI_DEBUG */

    /* Check if there is room available to hold another buffer */
    return (SubmitBuffer(pDevice,
                         pDevice->pTxChannel,
                         pBuffer,
                         nBufferSize,
                         false));
}


/**
 * @brief       Submits a buffer for receive.
 *
 * @details     This
 *
 * @param [in]  hDevice             Handle to the UART to which the buffer is submitted.
 * @param [in]  pBuffer             Pointer to the buffer.
 * @param [in]  nBufferSize         Size of the given buffer in bytes.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully submitted the buffer.
 *
 * - #ADI_UART_INVALID_HANDLE           [D]     if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR   [D]     if trying to submit a buffer when
 *                                              device is not opened for this direction.
 *
 * - #ADI_UART_NULL_POINTER             [D]     if the given pointer to buffer is NULL.
 *
 * - #ADI_UART_INVALID_PARAMETER        [D]      If the given buffer size is zero or buffer
 *                                              size is not aligned to the DMA transfer size
 *                                              (in DMA mode) set by the API
 *                                              adi_uart_SetRxDmaTransferSize.
 *
 * @sa adi_uart_GetRxBuffer
 * @sa adi_uart_SetRxDmaTransferSize.
 */
ADI_UART_RESULT adi_uart_SubmitRxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Rx Direction or bi-direction, for submitting Rx Buffers */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
        (pDevice->eDirection != ADI_UART_DIR_RECEIVE))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }

    /* Submit buffer can be only used in Callback or Non-blocking mode */
    if(pDevice->pRxChannel->eBufTxnMode == ADI_UART_BUF_TXN_MODE_BLOCKING)
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }
#endif /* ADI_DEBUG */

    return (SubmitBuffer(pDevice,
                         pDevice->pRxChannel,
                         pBuffer,
                         nBufferSize,
                         true));
}


/**
 * @brief       Gets back a processed Rx buffer.
 *
 * @details     This is a blocking API which waits until a processed buffer is avaialable
 *              and returns the pointer to the processed buffer. This API also returns back
 *              immediately with an error if there was an hardware error detected.
 *
 *              To avoid the blocking a non-blocking adi_uart_IsRxBufferAvailable can be used.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the UART from which the buffer to be obtained.
 * @param [out] ppBuffer            Pointer to a location where the processed buffer pointer will
 *                                  be written.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully obtained a processed buffer.
 *
 * - #ADI_UART_INVALID_HANDLE               [D] if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR       [D] if trying to get a buffer when
 *                                              device is not opened for Rx direction.
 *
 * - #ADI_UART_INVALID_DATA_TRANSFER_MODE   [D] Trying to use this API when a callback is registered
 *                                              or when UART is operated in blocking mode (Before a
 *                                              adi_uart_Read returns).
 *
 * - #ADI_UART_NULL_POINTER                 [D] if the given pointer to buffer pointer is NULL.
 *
 * @sa          adi_uart_IsRxBufferAvailable
 * @sa          adi_uart_GetTxBuffer
 */
ADI_UART_RESULT adi_uart_GetRxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                  **const ppBuffer
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Rx Direction or bi-direction, for getting Rx Buffers */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
       (pDevice->eDirection != ADI_UART_DIR_TRANSMIT))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }

    /* Submit buffer can be only used in Callback or Non-blocking mode */
    if(pDevice->pRxChannel->eBufTxnMode != ADI_UART_BUF_TXN_MODE_NONBLOCKING)
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }

    /* Check if the given pointer to buffer pointer is valid */
    if(ppBuffer == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    return (GetBuffer(pDevice, pDevice->pRxChannel, ppBuffer));
}


/**
 * @brief       Gets back a processed Tx buffer.
 *
 * @details     This is a blocking API which waits until a processed buffer is avaialable
 *              and returns the pointer to the processed buffer. This API also returns back
 *              immediately with an error if there was an hardware error detected.
 *
 *              To avoid the blocking a non-blocking adi_uart_IsTxBufferAvailable can be used.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the UART from which the buffer to be obtained.
 * @param [out] ppBuffer            Pointer to a location where the processed buffer pointer will
 *                                  be written.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully obtained a processed buffer.
 *
 * - #ADI_UART_INVALID_HANDLE               [D] if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR       [D] if trying to get a buffer when
 *                                              device is not opened for Tx direction.
 *
 * - #ADI_UART_INVALID_DATA_TRANSFER_MODE   [D] Trying to use this API when a callback is registered
 *                                              or when UART is operated in blocking mode (Before a
 *                                              adi_uart_Write returns).
 *
 * - #ADI_UART_NULL_POINTER                 [D] if the given pointer to buffer pointer is NULL.
 *
 * @sa          adi_uart_IsTxBufferAvailable
 * @sa          adi_uart_GetRxBuffer
 */
ADI_UART_RESULT adi_uart_GetTxBuffer(
    ADI_UART_HANDLE         const hDevice,
    void                  **const ppBuffer
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Rx Direction or bi-direction, for getting Tx Buffers */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
       (pDevice->eDirection != ADI_UART_DIR_TRANSMIT))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }

    /* Submit buffer can be only used in Callback or Non-blocking mode */
    if(pDevice->pTxChannel->eBufTxnMode != ADI_UART_BUF_TXN_MODE_NONBLOCKING)
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }

    /* Check if the given pointer to buffer pointer is valid */
    if(ppBuffer == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    return (GetBuffer(pDevice, pDevice->pTxChannel, ppBuffer));
}


/**
 * @brief       Submits a buffer for transmit and blocks until completion.
 *
 * @details     This API can be used to short amount of writes. It will block until the
 *              given buffer is transmitted. If a hardware error is detected
 *              the API will return immediately without completing the transfer.
 *
 *              When using this API it is not required to explicity enable the data flow,
 *              it is enabled automatically and gets disabled once the write is done.
 *
 * @param [in]  hDevice             Handle to the UART from which data to be written.
 * @param [in]  pBuffer             Pointer to the buffer.
 * @param [in]  nBufferSize         Size of the given buffer in bytes.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully transferred the buffer.
 *
 * - #ADI_UART_INVALID_HANDLE           [D]     if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR   [D]     if trying to write the buffer when
 *                                              device is not opened transmit or bidrectional
 *                                              mode.
 *
 * - #ADI_UART_HW_ERROR                         A hardware error is detected while
 *                                              transmitting the buffer.
 *
 * - #ADI_UART_NULL_POINTER             [D]     if the given pointer to buffer is NULL.
 *
 * - #ADI_UART_INVALID_PARAMETER        [D]     If the given buffer size is zero.
 *
 * @sa          adi_uart_Read
 */
ADI_UART_RESULT adi_uart_Write(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Rx Direction or bi-direction, for submitting Rx Buffers */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
        (pDevice->eDirection != ADI_UART_DIR_RECEIVE))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }

    /* Blocking Write can only be used when in non-blocking mode and data flow is disabled */
    if((pDevice->pTxChannel->eBufTxnMode    != ADI_UART_BUF_TXN_MODE_NONBLOCKING) ||
       (pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED ))
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }
#endif /* ADI_DEBUG */

    /* Submit the buffer for write */
    if((eResult =   SubmitBuffer(pDevice,
                                 pDevice->pTxChannel,
                                 pBuffer,
                                 nBufferSize,
                                 false)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Wait for blocking mode buffer transaction completion */
    return(CompleteBlockingBuffferTxn(pDevice, pDevice->pTxChannel, pBuffer));

}

/**
 * @brief       Submits a buffer for receive and blocks until completion.
 *
 * @details     This API can be used to short amount of reads. It will block until the
 *              given buffer is filled with the data. If a hardware error is detected
 *              the API will return immediately with buffer partially filled.
 *
 *              When using this API it is not required to explicity enable the data flow,
 *              it is enabled automatically and gets disabled once the read is done.
 *
 * @param [in]  hDevice             Handle to the UART from which data to be read.
 * @param [in]  pBuffer             Pointer to the buffer.
 * @param [in]  nBufferSize         Size of the given buffer in bytes.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully read the buffer.
 *
 * - #ADI_UART_INVALID_HANDLE           [D]     if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR   [D]     if trying to read a buffer when
 *                                              device is not receive or bidirectional
 *                                              mode.
 *
 * - #ADI_UART_HW_ERROR                         A hardware error is detected while
 *                                              reading the buffer.
 *
 * - #ADI_UART_NULL_POINTER             [D]     If the given pointer to buffer is invalid.
 *
 * - #ADI_UART_INVALID_PARAMETER        [D]     If the given buffer size if zero.
 *
 * - #ADI_UART_NULL_POINTER             [D]     if the given pointer to buffer is NULL.
 *
 * - #ADI_UART_INVALID_PARAMETER        [D]     If the given buffer size is zero.*
 *
 * @sa          adi_uart_Write
 */
ADI_UART_RESULT adi_uart_Read(
    ADI_UART_HANDLE         const hDevice,
    void                   *const pBuffer,
    uint32_t                      nBufferSize
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Rx Direction or bi-direction, for submitting Rx Buffers */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
        (pDevice->eDirection != ADI_UART_DIR_RECEIVE))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }

    /* Blocking Read can only be used when in non-blocking mode and data flow is disabled */
    if((pDevice->pRxChannel->eBufTxnMode      != ADI_UART_BUF_TXN_MODE_NONBLOCKING) ||
       (pDevice->pRxChannel->eDataFlowState   != ADI_UART_DATAFLOW_DISABLED ))
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }
#endif /* ADI_DEBUG */

    /* Submit the buffer for read */
    if((eResult =   SubmitBuffer(pDevice,
                                 pDevice->pRxChannel,
                                 pBuffer,
                                 nBufferSize,
                                 true)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Wait for blocking mode buffer transaction completion */
    return(CompleteBlockingBuffferTxn(pDevice, pDevice->pRxChannel, pBuffer));
}


/**
 * @brief       Queries to find if a processed Tx buffer is available.
 *
 * @details     This API can be used to avoid the blocking when the
 *              adi_uart_GetTxBuffer is called. The adi_uart_GetTxBufffer can be
 *              called after finding a processed buffer is available.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the UART to be queried.
 *
 * @param [out] pbAvailable         Pointer to a boolean variable where the status of buffer
 *                                  availibility is written.
 *                                  'true'  - is written if buffer is available.
 *                                  'false' - is written if buffer is not available.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully queried the buffer availibility.
 *
 * - #ADI_UART_INVALID_HANDLE               [D] if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR       [D] if trying to query the buffer when
 *                                              device is not opened transmit or bidrectional
 *                                              mode.
 *
 * - #ADI_UART_INVALID_DATA_TRANSFER_MODE   [D] Trying to use this API when a callback is registered
 *                                              or when UART is operated in blocking mode (Before a
 *                                              adi_uart_Write returns).
 *
 * - #ADI_UART_NULL_POINTER                 [D] If the given pointer to availability status is invalid.
 *
 *
 * @sa          adi_uart_IsRxBufferAvailable
 */
ADI_UART_RESULT adi_uart_IsTxBufferAvailable (
    ADI_UART_HANDLE         const hDevice,
    bool                   *const pbAvailable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Tx Direction or bi-direction */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
       (pDevice->eDirection != ADI_UART_DIR_TRANSMIT))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }
    /* This API will only work in non-blocking buffer transaction mode */
    if(pDevice->pTxChannel->eBufTxnMode != ADI_UART_BUF_TXN_MODE_NONBLOCKING)
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }

    /* Check if the given pointer is invalid */
    if(pbAvailable == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }

#endif /* ADI_DEBUG */

    /* Return the status of buffer availability */
    *pbAvailable = ((pDevice->pTxChannel->pProcessedBuf->bInUse) ? false : true);

    return ADI_UART_SUCCESS;
}



/**
 * @brief       Queries to find if a processed Rx buffer is available.
 *
 * @details     This API can be used to avoid the blocking when the
 *              adi_uart_GetRxBuffer is called. The adi_uart_GetRxBufffer can be
 *              called after finding a processed buffer is available.
 *
 * @note        This API cannot be used when a callback is registered.
 *
 * @param [in]  hDevice             Handle to the UART to be queried.
 *
 * @param [out] pbAvailable         Pointer to a boolean variable where the status of buffer
 *                                  availibility is written.
 *                                  'true'  - is written if buffer is available.
 *                                  'false' - is written if buffer is not available.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS                          if successfully queried the buffer availibility.
 *
 * - #ADI_UART_INVALID_HANDLE               [D] if the given UART handle is invalid.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR       [D] if trying to query the buffer when
 *                                              device is not opened recieve or bidrectional
 *                                              mode.
 *
 * - #ADI_UART_INVALID_DATA_TRANSFER_MODE   [D] Trying to use this API when a callback is registered
 *                                              or when UART is operated in blocking mode (Before a
 *                                              adi_uart_Read returns)
 *
 * - #ADI_UART_NULL_POINTER                 [D] If the given pointer to availability status is invalid.
 *
 * @sa          adi_uart_IsTxBufferAvailable
 */
ADI_UART_RESULT adi_uart_IsRxBufferAvailable (
    ADI_UART_HANDLE         const hDevice,
    bool                   *const pbAvailable

)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened for Rx Direction or bi-direction */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
       (pDevice->eDirection != ADI_UART_DIR_RECEIVE))
    {
         return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }
    /* This API will only work in non-blocking buffer transaction mode */
    if(pDevice->pRxChannel->eBufTxnMode != ADI_UART_BUF_TXN_MODE_NONBLOCKING)
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }

    /* Check if the given pointer is invalid */
    if(pbAvailable == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Return the status of buffer availability */
    *pbAvailable = ((pDevice->pRxChannel->pProcessedBuf->bInUse) ? false : true);

    return ADI_UART_SUCCESS;
}



/**
 * @brief       Enables or Disables the Transmit data flow
 *
 * @details     Atleast one buffer should be submitted before enabling the
 *              data flow. Enabling data flow starts transmitting the given buffer
 *              over UART.
 *
 * @param [in]  hDevice             Handle to the UART to which the buffer is submitted.
 * @param [in]  bEnable             flag which indicates whether to enable or disable.
 *                                  'true' - to enable, 'false' to disable.
 *
 * @return      Status
 * - #ADI_UART_SUCCESS                          if successfully enabled/disabled the Tx data flow.
 *
 * - #ADI_UART_INVALID_HANDLE           [D]     if the given UART handle is invalid.
 *
 * - #ADI_UART_BUFFERS_NOT_SUBMITTED    [D]     if the initial two buffers are not submitted.
 *
 * - #ADI_UART_DMA_FAILED                       if failed to enable/disable the DMA when
 *                                              DMA mode is chosen.
 *
 * - #ADI_UART_BAUD_RATE_NOT_SET        [D]     if trying to enable data flow
 *                                              without setting the baud rate.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR   [D]     if trying to enable the data flow when device is
 *                                              not opened for this direction.
 *
 * @note        When the data flow is disabled the given buffers will be
 *              discarded and data flow will be stopped immediately. To gracefully
 *              disable the data flow without discarding the data, it is
 *              advisable to call this function after receiving processed buffers back.
 *
 * @note        Even when UART is opened in Bidirectional data flow mode, to
 *              disable the data flow in both the directions, it is required
 *              to call both the adi_uart_EnableTx and adi_uart_EnableRx APIs.
 *
 * @sa          adi_uart_EnableRx
 */
ADI_UART_RESULT adi_uart_EnableTx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened in Tx or Bidirectional mode before enabling
     * data flow for Tx direction.
     */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
       (pDevice->eDirection != ADI_UART_DIR_TRANSMIT))
    {
        return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }

#endif /* ADI_DEBUG */

    return (EnableDataFlow(pDevice, pDevice->pTxChannel, bEnable));
}


/**
 * @brief       Enables or Disables the recieve data flow.
 *
 * @details     Atleast one buffer should be submitted before enabling the
 *              data flow. Enabling data flow starts filling thethe given buffer
 *              with the data recieved over UART.
 *
 * @param [in]  hDevice             Handle to the UART to which the buffer is submitted.
 * @param [in]  bEnable             flag which indicates whether to enable or disable.
 *                                  'true' - to enable, 'false' to disable.
 *
 * @return      Status
 * - #ADI_UART_SUCCESS                          if successfully enabled/disabled the Rx data flow.
 *
 * - #ADI_UART_INVALID_HANDLE           [D]     if the given UART handle is invalid.
 *
 * - #ADI_UART_BUFFERS_NOT_SUBMITTED    [D]     if the initial two buffers are not submitted.
 *
 * - #ADI_UART_DMA_FAILED                       if failed to enable/disable the DMA when
 *                                              DMA mode is chosen.
 *
 * - #ADI_UART_BAUD_RATE_NOT_SET        [D]     if trying to enable data flow
 *                                              without setting the baud rate.
 *
 * - #ADI_UART_NOT_OPENED_IN_THIS_DIR   [D]     if trying to enable the data flow when device is
 *                                              not opened for this direction.
 *
 * @note        When the data flow is disabled the given buffers will be
 *              discarded and data flow will be stopped immediately. To gracefully
 *              disable the data flow without discarding the data, it is
 *              advisable to call this function after receiving processed buffers back.
 *
 * @note        Even when UART is opened in Bidirectional data flow mode, to
 *              disable the data flow in both the directions, it is required
 *              to call both the adi_uart_EnableTx and adi_uart_EnableRx APIs.
 *
 * @sa          adi_uart_EnableTx
 */
ADI_UART_RESULT adi_uart_EnableRx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if UART is opened in Rx or Bidirectional mode before enabling
     * data flow for Rx direction.
     */
    if((pDevice->eDirection != ADI_UART_DIR_BIDIRECTION) &&
       (pDevice->eDirection != ADI_UART_DIR_RECEIVE))
    {
        return ADI_UART_NOT_OPENED_IN_THIS_DIR;
    }
#endif /* ADI_DEBUG */

    return (EnableDataFlow(pDevice, pDevice->pRxChannel, bEnable));
}


/**
 * @brief       Gets the hardware error(s) the occurred.
 *
 * @details     When the adi_uart_GetRx/Tx buffer or adi_uart_Read/Write APIs return a
 *              ADI_UART_HW_ERROR result, this API can used to find which hardware
 *              error(s) occured. If this API is not called immediately after a hardware
 *              error is notified, the errors can be overwritten by new errors.
 *
 * @note        This API cannot be used when a callback is registered. When a callback is
 *              registered all the hardware errors are notfied as events via the callback.
 *
 * @param [in]  hDevice             Handle to the UART to which the buffer is submitted.
 *
 * @param [out] pHWError            Pointer to a location where hardware error(s) is written.
 *                                  There can be muliple errors that might have ocurred.
 *                                  All the errors are logically orred. Application can
 *                                  make use of the values defined by enumeration
 *                                  #ADI_UART_HW_ERR as a mask to find out which error has
 *                                  occurred.
 *
 * @return      Status
 * - #ADI_UART_SUCCESS                          if successfully queried the hardware error status.
 *
 * - #ADI_UART_INVALID_HANDLE               [D] if the given UART handle is invalid.
 *
 * - #ADI_UART_INVALID_DATA_TRANSFER_MODE   [D] Trying to use this API when a callback is registered.
 *
 * - #ADI_UART_NULL_POINTER                 [D] If the given pointer to hardware error status is NULL.
 *
 *
 * @note        When the data flow is disabled the given buffers will be
 *              discarded and data flow will be stopped immediately. To gracefully
 *              disable the data flow without discarding the data, it is
 *              advisable to call this function after receiving processed buffers back.
 *
 * @note        Even when UART is opened in Bidirectional data flow mode, to
 *              disable the data flow in both the directions, it is required
 *              to call both the adi_uart_EnableTx and adi_uart_EnableRx APIs.
 *
 * @sa          adi_uart_EnableTx
 */
ADI_UART_RESULT adi_uart_GetHWErrorStatus(
    ADI_UART_HANDLE         const hDevice,
    uint32_t               *const pHWError
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Check if the given pointer is valid */
    if(pHWError == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }

    /* This API will not work when a callback is registered */
    if(pDevice->pRxChannel->eBufTxnMode != ADI_UART_BUF_TXN_MODE_NONBLOCKING)
    {
        return  ADI_UART_INVALID_DATA_TRANSFER_MODE;
    }
#endif /* ADI_DEBUG */

    /* Return the current HW Error status */
    *pHWError   =   pDevice->nHWError;

    adi_osal_EnterCriticalRegion();
    /* Clear the HW Error status */
    pDevice->nHWError = (uint32_t)ADI_UART_HW_ERR_NONE;
    adi_osal_ExitCriticalRegion();

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Enables or Disables the line status events
 *
 * @details     The following line status events are enabled and notified by default
 *              but they can be disabled/enabled back if required.
 *              - Parity Error.
 *              - Framing Error.
 *              - Break Interrupt.
 *
 * @param [in]  hDevice             Handle to the UART.
 * @param [in]  bEnable             flag which indicates whether to enable or disable.
 *                                  'true' - to enable, 'false' to disable.
 *
 * @return      Status
 *   - #ADI_UART_SUCCESS                    if successfully enabled/disabled line status events.
 *   - #ADI_UART_INVALID_HANDLE     [D]     if the given UART handle is invalid.
 */
ADI_UART_RESULT adi_uart_EnableLineStatusEvents(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* If line status events to be enabled */
    if(bEnable == true)
    {
        /* Set the IMASK Set register to enable the line status interrupt */
        pDevice->pUARTRegs->UART_IMSK_SET   =   (uint32_t)ENUM_UART_ELSI_HI;
    }
    else
    {
        /* Set the IMASK Clear register to disable the line status interrupt */
        pDevice->pUARTRegs->UART_IMSK_CLR   =   (uint32_t)ENUM_UART_ELSI_HI;
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Transmits the given data as address in MDB mode
 *
 * @details     This API is valid only in MDB mode. The device has a separate
 *              data path to send the address to the slave devices in MDB mode.
 *              So to send the address followed by data. This API should be
 *              called first and the data buffer should be submitted using
 *              adi_uart_SubmitTxBuffer.
 *
 * @param [in]  hDevice             Handle to the UART through which address
 *                                  should be transmitted.
 * @param [in]  Address             Address to be transmitted.
 *
 * @return      Status
 *  - #ADI_UART_SUCCESS                     if successfully transmitted given address.
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to transmit Address when not in
 *                                          MDB Mode.
 *
 * @note        This API can be called before enabling the data flow. The given
 *              address is transmitted once the data flow is enabled. When the
 *              data flow is enabled the address is transmitted first followed by
 *              the given data buffers.
 */
ADI_UART_RESULT adi_uart_MDBTxAddress(
   ADI_UART_HANDLE         const hDevice,
   uint8_t                       Address
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Transmitting MDB address when not in MDB mode is not allowed */
    if(pDevice->eMode   !=  ADI_UART_MODE_MDB)
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Write the given address to TAIP register for transmitting it*/
    pDevice->pUARTRegs->UART_TAIP   =   Address;

    return ADI_UART_SUCCESS;
}



/**
 * @brief       Discards the received data in MDB mode.
 *
 * @details     In MDB mode all the slaves should be waiting for the address.
 *              Once the address is received the slaves will compare the received
 *              address to see if it is addressed. If the address does not match
 *              its address the data received will be discarded.
 *
 *              The hardware supports the mechanism to discard the data at the
 *              hardware level itself. This API sets the hardware to discard the
 *              received data and wait until the next address is received.
 *
 *              This API is advisable to be called after recieving the address.
 *              Application should check the received address and determine if
 *              rest of the data to be discarded.
 *
 * @param [in]  hDevice             Handle to the UART which should be configured
 *                                  to discard the data until the next address.
 *
 * @return      Status
 *
 *   - #ADI_UART_SUCCESS                    if successfully configured the device
 *                                          to discard the data.
 *
 *   - #ADI_UART_INVALID_HANDLE         [D] if the given UART handle is invalid.
 *
 *   - #ADI_UART_OPERATION_NOT_ALLOWED  [D] if trying to discard data when not in
 *                                          MDB Mode.
 *
 * @sa          adi_uart_MDBTxAddress
 */
ADI_UART_RESULT adi_uart_MDBDiscardRxData(
   ADI_UART_HANDLE         const hDevice
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Discarding data when not in MDB mode is not allowed */
    if(pDevice->eMode   !=  ADI_UART_MODE_MDB)
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Clear (W1C) the ASTKY bit in the status register to drop all the data
     * sent from the master until the next address is sent
     *
     * It is ok to write other bits with 0 as they are all W1C/W1S or RO
     */
    pDevice->pUARTRegs->UART_STAT   =   (uint32_t)ENUM_UART_STAT_ADDR_HI_STKY;

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Enables or disables loop back mode.
 *
 * @details     The loopback mode is helpful for debugging. When loopback mode
 *              is enabled the transmitted data is internally looped back to the
 *              receive buffer.
 *
 * @param [in]  hDevice             Handle to the UART in which loopback to be
 *                                  enabled.
 * @param [in]  bEnable             Flag which indicates whether to enable or
 *                                  disable loopback.
 *                                  'true' - to enable, 'false' - to disable.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                         if successfully enabled or disabled loopback.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D]     if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D]     if trying to set the loopback mode when the
 *                                              device data flow is enabled.
 *
 */
ADI_UART_RESULT adi_uart_EnableLoopBack(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the loop back mode when data flow is enabled */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    if(bEnable == true)
    {
        /* Set the loop back enable bit in the control register */
        pDevice->pUARTRegs->UART_CTL |= (uint32_t) ENUM_UART_CTL_LOOP_EN;
    }
    else
    {
        /* Clear the loop back enable bit in the control register */
        pDevice->pUARTRegs->UART_CTL &=  ~((uint32_t)ENUM_UART_CTL_LOOP_EN);
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Set the number of stop bits to be transmitted.
 *
 * @param [in]  hDevice             Handle to the UART in which stop bits to be
 *                                  configured.
 *
 * @param [in]  eStopBits           Number of stop bits. It can be one of the
 *                                  values from the enumeration ADI_UART_STOPBITS.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully set the number of stop bits.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to set the number of stop bits when the
 *                                          device data flow is enabled.
 */
ADI_UART_RESULT adi_uart_SetNumStopBits(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_STOPBITS             eStopBits
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;
     /* Contents of control register */
     uint32_t               CtlRegValue;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the number of stop bits when data flow is enabled */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Get current contents of UART control register */
    CtlRegValue = pDevice->pUARTRegs->UART_CTL;

    /* Clear the exiting number of stop bits */
    CtlRegValue     &=  (~( (uint32_t)BITM_UART_CTL_STB | (uint32_t)BITM_UART_CTL_STBH));

    switch(eStopBits)
    {
        /* One Stop bit */
        case ADI_UART_ONE_STOPBIT:
            /* Nothing needs to be set */
            break;

        /* 0ne and half Stop bits */
        case ADI_UART_ONE_AND_HALF_STOPBITS:
            CtlRegValue     |=  (uint32_t)ENUM_UART_CTL_1_EXTRA_STBH;
            break;

        /* Two Stop bits */
        case ADI_UART_TWO_STOPBITS:
            CtlRegValue     |=  (uint32_t)ENUM_UART_CTL_1_EXTRA_STB;
            break;

        /* Two and Half Stop bits */
        case ADI_UART_TWO_AND_HALF_STOPBITS:
            CtlRegValue     |=  ((uint32_t)ENUM_UART_CTL_1_EXTRA_STB | (uint32_t)ENUM_UART_CTL_1_EXTRA_STBH);
            break;

        default:
            break;
    }

    /* Write back the control register value */
    pDevice->pUARTRegs->UART_CTL    =   CtlRegValue;

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Set the UART word length.
 *
 * @details     Sets the Rx and Tx word lengths.
 *
 * @param [in]  hDevice             Handle to the UART in which stop bits to be
 *                                  configured.
 * @param [in]  eWordLen            Word length to be set. It can be one of the
 *                                  values from the enumeration ADI_UART_WORDLEN.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully set the word length.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to set the word length when the
 *                                          device data flow is enabled.
 */
ADI_UART_RESULT adi_uart_SetWordLen(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_WORDLEN              eWordLen
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;
     /* Contents of control register */
     uint32_t               CtlRegValue;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to change the word length when data flow is enabled */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Get current contents of UART control register */
    CtlRegValue = pDevice->pUARTRegs->UART_CTL;

    /* Clear the current word length */
    CtlRegValue     &=  ~((uint32_t)BITM_UART_CTL_WLS);

    /* Set the new word length */
    CtlRegValue     |=  (uint32_t)eWordLen;

    /* Write back the control register value */
     pDevice->pUARTRegs->UART_CTL    =   CtlRegValue;

     return ADI_UART_SUCCESS;
}

/**
 * @brief       Set the UART parity.
 *
 * @details     Sets the parity type to be transmitted and checked.
 *
 * @param [in]  hDevice             Handle to the UART in which stop bits to be
 *                                  configured.
 * @param [in]  eParity             Parity type to be set. It can be one of the
 *                                  values from the enumeration ADI_UART_PARITY.
 *
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully set the parity.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to set the word length when the
 *                                          device data flow is enabled.
 */
ADI_UART_RESULT adi_uart_SetParity(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_PARITY               eParity
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT        eResult;
    /* Contents of control register */
    uint32_t               CtlRegValue;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the parity when data flow is enabled */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Get current contents of UART control register */
    CtlRegValue = pDevice->pUARTRegs->UART_CTL;

    /* Clear the current parity settings */
    CtlRegValue &= (~((uint32_t)BITM_UART_CTL_STP | (uint32_t)BITM_UART_CTL_EPS | (uint32_t)BITM_UART_CTL_PEN));

    switch(eParity)
    {
        case ADI_UART_NO_PARITY:
            /* No parity required; do nothing */
            break;

        case ADI_UART_ODD_PARITY:
            /* Enable parity and set odd parity */
            CtlRegValue |=  (((uint32_t)ENUM_UART_CTL_PARITY_EN | (uint32_t)ENUM_UART_CTL_ODD_PARITY));
            break;

        case ADI_UART_EVEN_PARITY:
            /* Enable parity and set even parity */
            CtlRegValue |=  (((uint32_t)ENUM_UART_CTL_PARITY_EN | (uint32_t)ENUM_UART_CTL_EVEN_PARITY));
            break;

        case ADI_UART_EVEN_PARITY_STICKY:
            /* Enable parity, sticky parity and set even parity */
            CtlRegValue |=  (((uint32_t)ENUM_UART_CTL_PARITY_EN | (uint32_t)ENUM_UART_CTL_EVEN_PARITY | (uint32_t)ENUM_UART_CTL_STICKY_PARITY));
            break;

        case ADI_UART_ODD_PARITY_STICKY:
            /* Enable parity, sticky parity and set even parity */
            CtlRegValue |=  (((uint32_t)ENUM_UART_CTL_PARITY_EN | (uint32_t)ENUM_UART_CTL_ODD_PARITY | (uint32_t)ENUM_UART_CTL_STICKY_PARITY));
            break;

        default:
            break;
    }

    /* Write back the control register value */
    pDevice->pUARTRegs->UART_CTL    =   CtlRegValue;

    return ADI_UART_SUCCESS;
}



/**
 * @brief       Forces the UART to send out a break.
 *
 * @details     Sets the UART Tx pin to a logic-low/high (depending upon the
 *              Tx polarity) asynchronously. The UART keeps transmitting break
 *              until it is disabled to send the break.
 *
 * @param [in]  hDevice             Handle to the UART whose Tx is forced to
 *                                  send a break.
 * @param [in]  bEnable             Flag which indicates whether to enable or
 *                                  disable transmitting the break.
 *                                  'true' - to enable, 'false' - to disable.
 *
 * @return      Status
 *
 * - #ADI_UART_SUCCESS              if successfully enabled or disabled sending break.
 *
 * - #ADI_UART_INVALID_HANDLE [D]   if the given UART handle is invalid.
 */
ADI_UART_RESULT adi_uart_ForceTxBreak(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    if(bEnable == true)
    {
        /* Set the force break bit */
        pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_FORCE_BREAK;
    }
    else
    {
        /* Clear the force break bit */
        pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_FORCE_BREAK);
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Set the flow control polarity.
 *
 * @details     Sets the pins CTS (Clear To Send) and RTS (Request To Send) to
 *              be negative assertive or positive assertive.
 *
 * @param [in]  hDevice             Handle to the UART whose Flow control signals
 *                                  polarity to be set.
 *
 * @param [in]  bActiveHigh         Flag which indicates the polarity.
 *                                  'true' - CTS and RTS are positive assertive.
 *                                  'false'  - CTS and RTS are negative assertive.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully set the flow control polarity.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to set flow control parity when data
 *                                          flow is enabled or auto baud is in
 *  progress.
 */
ADI_UART_RESULT adi_uart_SetFCPolarity(
    ADI_UART_HANDLE         const hDevice,
    bool                          bActiveHigh
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the flow control polarity when data flow is enabled
     * or auto baud is in progress */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    if(bActiveHigh == true)
    {
        /* Set the flow control polarity bit */
        pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_FCPOL_HI;
    }
    else
    {
        /* Clear the flow control polarity bit */
        pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_FCPOL_HI);
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Set the Rx Pin polarity.
 *
 * @param [in]  hDevice             Handle to the UART whose Rx signal polarity
 *                                  to be set.
 * @param [in]  bActiveHigh         Flag which indicates the polarity.
 *                                  'true' -  In IrDA mode Rx line idles high
 *                                            In UART/MDB mode received data is
 *                                            interpreted as Inverted NRZ
 *                                  'false'-  In IrDA mode Rx line idles low.
 *                                            In UART/MDB mode data received data
 *                                            is interpreted as NRZ (Non-Return to Zero)
 * @return      Status
 *
 *   - #ADI_UART_SUCCESS                    if successfully set the Rx Polarity.
 *
 *   - #ADI_UART_INVALID_HANDLE         [D] if the given UART handle is invalid.
 *
 *   - #ADI_UART_OPERATION_NOT_ALLOWED  [D] if trying to set Rx polarity when data flow is
 *                                          enabled or auto baud is in progress.
 */
ADI_UART_RESULT adi_uart_SetRxPolarity(
    ADI_UART_HANDLE         const hDevice,
    bool                          bActiveHigh
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the Rx parity when data flow is enabled
     * or auto baud is in progress */
    if((pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    if(bActiveHigh == true)
    {
        /* Set the Rx polarity bit */
        pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_RPOLC_HI;
    }
    else
    {
        /* Clear the Rx polarity bit */
        pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_RPOLC_HI);
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Set the Rx Pin polarity.
 *
 * @param [in]  hDevice             Handle to the UART whose Tx signal polarity
 *                                  to be set.
 *
 * @param [in]  bActiveHigh         Flag which indicates the polarity.
 *                                  'true'  - In IrDA mode Tx line idles high.
 *                                            In UART/MDB mode data is transmitted
 *                                            as Inverted NRZ (Non-Return to Zero)
 *                                  'false' - In IrDA mode Tx line idles low.
 *                                            In UART/MDB mode data is transmitted
 *                                            as NRZ (Non-Return to Zero)
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully set the Tx polarity.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to set Tx polarity when data flow
 *                                          is enabled or auto baud is in progress.
 */
ADI_UART_RESULT adi_uart_SetTxPolarity(
    ADI_UART_HANDLE         const hDevice,
    bool                          bActiveHigh
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the Tx parity when data flow is enabled
     * or auto baud is in progress */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    if(bActiveHigh == true)
    {
        /* Set the Tx polarity bit */
        pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_TPOLC_HI;
    }
    else
    {
        /* Clear the Tx polarity bit */
        pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_TPOLC_HI);
    }

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Pauses or resumes the Rx
 *
 * @details     In manual flow control the transmitter can be signaled to stop
 *              sending the data by de-asserting RTS line. This API configures
 *              the hardware to assert/deassert RTS line to pause or resume
 *              receiving data.
 *
 *              This is not valid with hardware flow control and will do nothing
 *              if called.
 *
 * @param [in]  hDevice             Handle to the UART Rx to be paused or resumed.
 *
 * @param [in]  bPause              Flag to indicate pause or resume.
 *                                  'true' - to pause, 'false' - to resume.
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully paused or resumed Rx.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to pause Rx auto baud is in progress
 *                                          or when hardware flow control is enabled for Rx.
 *
 * @sa          adi_uart_PauseTx
 */
ADI_UART_RESULT adi_uart_PauseRx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bPause
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG
     /* Validate the given handle */
     if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
     {
         return eResult;
     }

     /* Don't allow to set the Tx parity when data flow is enabled
      * or auto baud is in progress */
     if(pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS)
     {
         return ADI_UART_OPERATION_NOT_ALLOWED;
     }

     /* If Hardware Flow control is enabled for Rx, it is not possible to
      * pause Rx.
      */
     if((pDevice->pUARTRegs->UART_CTL  & (uint32_t)BITM_UART_CTL_ARTS) != 0u)
     {
         return ADI_UART_OPERATION_NOT_ALLOWED;
     }
#endif /* ADI_DEBUG */


     if(bPause == true)
     {
         /* Set the MRTS bit to assert RTS for stopping the transmitter from sending more data */
         pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_RTS_ASSERT;
     }
     else
     {
         /* Clear the MRTS bit to de-assert RTS for allowing transmitter to send more data */
         pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_RTS_ASSERT);
     }

     return ADI_UART_SUCCESS;
}

/**
 * @brief       Pauses or resumes the Tx
 *
 * @details     This is not valid with hardware flow control and will do nothing
 *              if called.
 *
 * @param [in]  hDevice             Handle to the UART whose Tx to be paused or resumed.
 *
 * @param [in]  bPause              Flag to indicate pause or resume.
 *                                  'true' - to pause, 'false' - to resume.
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully paused or resumed Tx.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to pause Tx when hardware flow
 *                                          control is enabled for Tx.
 *
 * @sa          adi_uart_PauseRx
 */
ADI_UART_RESULT adi_uart_PauseTx(
    ADI_UART_HANDLE         const hDevice,
    bool                          bPause
)
{
    /* Pointer to UART device instance */
     ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
     /* Return code */
     ADI_UART_RESULT        eResult;

 #ifdef ADI_DEBUG

     /* Validate the given handle */
     if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
     {
         return eResult;
     }

     /* If Hardware Flow control is enabled for Tx, it is not possible to
      * pause Tx.
      */
     if((pDevice->pUARTRegs->UART_CTL  & (uint32_t)BITM_UART_CTL_ACTS) != 0u)
     {
         return ADI_UART_OPERATION_NOT_ALLOWED;
     }
 #endif /* ADI_DEBUG */


     if(bPause == true)
     {
         /* Set the XOFF bit to prevent content of THR to be continued to TSR if ACTS=0*/
         pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_TX_OFF;
     }
     else
     {
         /* Clear the XOFF bit to allow content of THR to be continued to TSR */
         pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_RTS_ASSERT);
     }

     return ADI_UART_SUCCESS;
}


/**
 * @brief       Enables or disables Tx hardware flow control.
 *
 * @details     When hardware flow control is enabled the CTS signal is automatically
 *              driven by the hardware.
 *
 * @param [in]  hDevice             Handle to the UART whose Tx flow control to be
 *                                  enabled or disabled.
 *
 * @param [in]  bEnable             Flag which indicates whether to enable or
 *                                  disable hardware flow control for Tx.
 *                                  'true' - to enable, 'false' - to disable.
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS             if successfully enabled the flow control for Tx.
 *
 *  - #ADI_UART_INVALID_HANDLE  [D] if the given UART handle is invalid.
 *
 * @sa          adi_uart_EnableRxHWFlowCntl
 */
ADI_UART_RESULT adi_uart_EnableTxHWFlowCntl(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    if(bEnable == true)
    {
        /* Set the ACTS bit to enable Tx hardware flow control */
        pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_CTS_AUTO;
    }
    else
    {
        /* Clear the ACTS bit for manual Tx flow control */
        pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_CTS_AUTO);
    }

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Enables or disables Rx hardware flow control.
 *
 * @details     When hardware flow control is enabled the RTS signal is automatically
 *              driven by the hardware.
 *
 * @param [in]  hDevice             Handle to the UART whose Rx flow control to be
 *                                  enabled or disabled.
 *
 * @param [in]  bEnable             Flag which indicates whether to enable or
 *                                  disable hardware flow control for Rx.
 *                                  'true' - to enable, 'false' - to disable.
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     if successfully enabled the flow control for Rx.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to set the  Rx hardware flow control when
 *                                          autobaud is in progress.
 *
 * @sa          adi_uart_EnableTxHWFlowCntl
 */
ADI_UART_RESULT adi_uart_EnableRxHWFlowCntl(
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE        *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT        eResult;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the Rx flow control when autobaud is in progress
     * as autobaud needs automatic Rx flow control. */
    if(pDevice->eState == ADI_UART_STATE_AUTOBAUD_IN_PROGRESS)
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    if(bEnable == true)
    {
        /* Set the ARTS bit to enable Rx hardware flow control */
        pDevice->pUARTRegs->UART_CTL    |=  (uint32_t)ENUM_UART_CTL_RTS_AUTO;
    }
    else
    {
        /* Clear the ARTS bit for manual Rx flow control */
        pDevice->pUARTRegs->UART_CTL    &=  ~((uint32_t)ENUM_UART_CTL_RTS_AUTO);
    }

    return ADI_UART_SUCCESS;
}

/**
 * @brief       Sets the Rx FIFO watermark levels.
 *
 * @details     When watermark levels are set, and when callback is registered
 *              an event (ADI_UART_EVENT_RX_FIFO_WM) will be notified once the number
 *              of words in the Rx FIFO reaches the given watermark level.
 *
 * @param [in]  hDevice             Handle to the UART whose water mark to be set.
 *
 * @param [in]  eFifoWM             Watermark level to be set in terms of number
 *                                  of words. It can be one of the enumerated
 *                                  values from ADI_UART_RX_FIFO_WM
 * @return      Status
 *  - #ADI_UART_SUCCESS             if successfully set the watermark levels.
 *  - #ADI_UART_INVALID_HANDLE  [D] if the given UART handle is invalid.
 */
ADI_UART_RESULT adi_uart_SetRxFifoWM(
    ADI_UART_HANDLE         const hDevice,
    ADI_UART_RX_FIFO_WM           eFifoWM
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;
    /* Return code */
    ADI_UART_RESULT         eResult;
    /* Pointer to the UART registers */
    volatile ADI_UART_REGS  *pUARTRegs;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* Get the pointer to the UART registers */
    pUARTRegs = pDevice->pUARTRegs;

    /* Set the IRQ generation threshold and RTS threshold according to the
     * given watermark levels.
     */
    if(eFifoWM == ADI_UART_RX_FIFO_WM_4WORDS)
    {
        /* Clear the RFRT and RFIT bits to set the IRQ generation and RTS control
         * threshold to 4 words */
        pUARTRegs->UART_CTL    &=  (~((uint32_t)ENUM_UART_CTL_RX_IRQ_TH7 | (uint32_t)ENUM_UART_CTL_RX_RTS_TH7));
    }
    else
    {
        /* Set the RFRT and RFIT bits to set the IRQ generation and RTS control
         * threshold to 7 words */
        pUARTRegs->UART_CTL    |=  ((uint32_t)ENUM_UART_CTL_RX_IRQ_TH7 | (uint32_t)ENUM_UART_CTL_RX_RTS_TH7);
    }

    /* Enable the Rx threshold interrupts */
    pUARTRegs->UART_IMSK_SET   =   (uint32_t)ENUM_UART_ERFCI_HI;

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Enables or disables DMA mode.
 *
 * @details     By default DMA mode is disabled. It can be enabled to operate
 *              in dma mode.
 *
 * @param [in]  hDevice             Handle to the UART device in which DMA mode
 *                                  to be set.
 *
 * @param [in]  bEnable             Flag which indicates whether to use DMA or
 *                                  interrupt mode.
 *                                  'true' - for DMA mode, 'false' - for Interrupt
 *                                  mode.
 *
 * @return      Status
 *  - #ADI_UART_SUCCESS                     if successfully set the DMA mode.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] if trying to enable/disable DMA mode when
 *                                          data flow is enabled.
 */
ADI_UART_RESULT adi_uart_EnableDMAMode (
    ADI_UART_HANDLE         const hDevice,
    bool                          bEnable
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT         eResult;

    /* Pointer to the UART registers */
    volatile ADI_UART_REGS  *pUARTRegs;

#ifdef ADI_DEBUG
    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the DMA mode when data flow is enabled */
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Get the pointer to the UART registers */
    pUARTRegs = pDevice->pUARTRegs;

    /* IF (we're already in DMA mode) */
    if (pDevice->bDMAMode == bEnable)
    {
    	return ADI_UART_SUCCESS;
    }

    /* IF DMA is required */
    if(bEnable == true)
    {
        /* Clear the ERXS and ETXS to route the interrupts to Rx and Tx interrupt lines
         * to send request to DMA control unit
         */
        pUARTRegs->UART_IMSK_CLR    =   ((uint32_t)ENUM_UART_ERXS_HI | (uint32_t)ENUM_UART_ETXS_HI);

        /*
         * Open the DMA channels based on the direction.
         */

        /* Open Recieve DMA channel if UART is opened in Rx direction or bidirection */
        if((pDevice->eDirection == ADI_UART_DIR_RECEIVE) ||
           (pDevice->eDirection == ADI_UART_DIR_BIDIRECTION))
        {
            if(adi_pdma_Open(pDevice->pDevInfo->eRxDMAChnlID,
                             pDevice->pRxChannel->pDMAChnlMemory,
                             &pDevice->pRxChannel->hDMAChnl,
                             UARTRxDMAHandler,
                             pDevice
                             ) != ADI_DMA_SUCCESS)
            {
                return ADI_UART_DMA_FAILED;
            }

            /* DMA is configured for 1 byte transfer by default */
            /* Enable DMA SYNC configuration */
            if (adi_pdma_SetSync (pDevice->pRxChannel->hDMAChnl, true) != ADI_DMA_SUCCESS)
            {
            	return ADI_UART_DMA_FAILED;
            }
        }

        /* Open Transmit DMA channel if UART is opened in Tx direction or bidirection */
        if((pDevice->eDirection == ADI_UART_DIR_TRANSMIT) ||
           (pDevice->eDirection == ADI_UART_DIR_BIDIRECTION))
        {
            if(adi_pdma_Open(pDevice->pDevInfo->eTxDMAChnlID,
                             pDevice->pTxChannel->pDMAChnlMemory,
                             &pDevice->pTxChannel->hDMAChnl,
                             UARTTxDMAHandler,
                             pDevice
                             ) != ADI_DMA_SUCCESS)
            {
                return ADI_UART_DMA_FAILED;
            }

            /* DMA is configured for 1 byte transfer by default */
            /* Enable DMA SYNC configuration */
            if (adi_pdma_SetSync (pDevice->pTxChannel->hDMAChnl, true) != ADI_DMA_SUCCESS)
            {
            	return ADI_UART_DMA_FAILED;
            }
        }

        /* Set the function pointer table to DMA mode functions */
        pDevice->pFnTable   =   &gDMAFnTable;

    }
    /* Interrupt Mode required */
    else
    {
        /* Set the ERXS and ETXS to route the interrupts to status interrupt */
        pUARTRegs->UART_IMSK_SET    =   ((uint32_t)ENUM_UART_ERXS_HI | (uint32_t)ENUM_UART_ETXS_HI);


        /*
         * If DMA channels were opened before, close them
         */

        /* Close Tx channel */
        if(pDevice->pTxChannel->hDMAChnl != NULL)
        {
            if(adi_pdma_Close(pDevice->pTxChannel->hDMAChnl, false) != ADI_DMA_SUCCESS)
            {
                return ADI_UART_DMA_FAILED;
            }
        }

        /* Close Rx Channel */
        if(pDevice->pRxChannel->hDMAChnl != NULL)
        {
            if(adi_pdma_Close(pDevice->pRxChannel->hDMAChnl, false) != ADI_DMA_SUCCESS)
            {
                return ADI_UART_DMA_FAILED;
            }
        }

        /* Set the function pointer table to Interrupt mode functions */
        pDevice->pFnTable   =   &gIntFnTable;
    }

    /* Save the chosen mode */
    pDevice->bDMAMode   =   bEnable;

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Registers or unregisters a callback with the driver.
 *
 * @details     It is not required to register a callback for the operation of the
 *              driver. Any hardware error will be notified via the
 *              adi_uart_GetRx/TxBuffer API. But if application requires the
 *              errors/events to be notified immediately it can register a callback
 *              with the driver which will be called to notify errors/events.
 *
 *              Whe a callback is registered the APIs adi_uart_Read/Write and
 *              adi_uart_GetRx/TxBuffer will not work and return an error.
 *
 * @param [in]  hDevice             Handle to the UART device with which the callback
 *                                  to be registered.
 *
 * @param [in]  pfCallback          Pointer to the callback function. The callback function
 *                                  has the prototype
 *                                  void callback(void *pCBParam, uint32_t nEvent, void *pArg)
 *
 *                                  To unregister a callback pass the the pointer to the callback
 *                                  function as NULL.
 *
 * @param [in]  pCBParam            Callback parameter which will be returned back to the
 *                                  application when the callback function is called.
 *
 * @return      Status
 *  - #ADI_UART_SUCCESS                     if successfully registered a callback.
 *
 *  - #ADI_UART_INVALID_HANDLE          [D] if the given UART handle is invalid.
 *
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If trying to register/unregister a callback after
 *                                          the data flow is enabled.
 *
 */
ADI_UART_RESULT adi_uart_RegisterCallback(
    ADI_UART_HANDLE         const hDevice,
    ADI_CALLBACK            const pfCallback,
    void                   *const pCBParam
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to register/unregister the callback after dataflow  is enabled.*/
    if((pDevice->pTxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    adi_osal_EnterCriticalRegion();

    pDevice->pfCallback     =   pfCallback;
    pDevice->pCBParam       =   pCBParam;

    adi_osal_ExitCriticalRegion();


    /* When a callback is being registered */
    if(pfCallback != NULL)
    {
        /* Set the buffer transaction mode to be callback mode */
        pDevice->pTxChannel->eBufTxnMode  =   ADI_UART_BUF_TXN_MODE_CALLBACK;
        pDevice->pRxChannel->eBufTxnMode  =   ADI_UART_BUF_TXN_MODE_CALLBACK;
    }
    else
    {
        /* Set the buffer transaction mode to default non-blocking mode */
         pDevice->pTxChannel->eBufTxnMode  =   ADI_UART_BUF_TXN_MODE_NONBLOCKING;
         pDevice->pRxChannel->eBufTxnMode  =   ADI_UART_BUF_TXN_MODE_NONBLOCKING;
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief Configures Trigger Input settings for an UART Rx channel.
 *
 * This API is applicable only in DMA mode. Calling this API in interrupt mode will result in an error.
 *
 * The UART buffer processing can be configured to wait for an external trigger. The trigger to be used
 * for starting the buffer processing should be configured by application using the Trigger Routing
 * Unit API adi_tru_TriggerRoute.
 *
 * @param [in]  hDevice          UART device handle to configure.
 *
 * @param [in]  bTriggerWait     Flag which indicates whether to wait for an incomming trigger to process the given buffer.
 *
 *                               'true' - Wait for an incoming trigger.
 *                               'false'- Ignore incoming trigger and process the buffer automatically.
 *
 * @param [in]  bTriggerOvError  Flag which indicates whether to generate an error when a trigger overrun occurs.
 *                               A trigger overrun occurs when a trigger is recieved before processing the buffer
 *                               which was initiated by a previous trigger.
 *
 *                               'true' to enable and report trigger overrun error.
 *                               'false' to disable trigger overrun error.
 *
 * @return  Status
 *  - #ADI_UART_SUCCESS                     Successfully configured Trigger input.
 *  - #ADI_UART_DMA_FAILED                  If failed to configure the DMA service for trigger input.
 *  - #ADI_UART_INVALID_HANDLE          [D] If the given UART handle is invalid.
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If configuring the trigger input in interrupt mode.
 *
 */
ADI_UART_RESULT  adi_uart_ConfigRxTriggerIn (
    ADI_UART_HANDLE const       hDevice,
    bool                        bTriggerWait,
    bool                        bTriggerOvError)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to configure the trigger in interrupt mode.*/
    if(pDevice->bDMAMode == false)
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Call the DMA service to configure the trigger input */
    if(adi_dma_ConfigTriggerIn (pDevice->pRxChannel,
                                bTriggerWait,
                                bTriggerOvError) != ADI_DMA_SUCCESS)
    {
        return ADI_UART_DMA_FAILED;
    }

    return ADI_UART_SUCCESS;
}



/**
 * @brief Configures Trigger Input settings for an UART Tx channel.
 *
 * This API is applicable only in DMA mode. Calling this API in interrupt mode will result in an error.
 *
 * The UART buffer processing can be configured to wait for an external trigger. The trigger to be used
 * for starting the buffer processing should be configured by application using the Trigger Routing
 * Unit API adi_tru_TriggerRoute.
 *
 * @param [in]  hDevice          UART device handle to configure.
 *
 * @param [in]  bTriggerWait     Flag which indicates whether to wait for an incomming trigger to process the given buffer.
 *
 *                               'true' - Wait for an incoming trigger.
 *                               'false'- Ignore incoming trigger and process the buffer automatically.
 *
 * @param [in]  bTriggerOvError  Flag which indicates whether to generate an error when a trigger overrun occurs.
 *                               A trigger overrun occurs when a trigger is recieved before processing the buffer
 *                               which was initiated by a previous trigger.
 *
 *                               'true' to enable and report trigger overrun error.
 *                               'false' to disable trigger overrun error.
 *
 * @return  Status
 *  - #ADI_UART_SUCCESS                     Successfully configured Trigger input.
 *  - #ADI_UART_INVALID_HANDLE          [D] If the given UART handle is invalid.
 *  - #ADI_UART_DMA_FAILED                  If failed to configure the DMA service for trigger input.
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If configuring the trigger input in interrupt mode.
 */
ADI_UART_RESULT  adi_uart_ConfigTxTriggerIn (
    ADI_UART_HANDLE const       hDevice,
    bool                        bTriggerWait,
    bool                        bTriggerOvError)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to configure the trigger in interrupt mode.*/
    if(pDevice->bDMAMode == false)
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Call the DMA service to configure the trigger input */
    if(adi_dma_ConfigTriggerIn (pDevice->pTxChannel,
                                bTriggerWait,
                                bTriggerOvError) != ADI_DMA_SUCCESS)
    {
        return ADI_UART_DMA_FAILED;
    }

    return ADI_UART_SUCCESS;
}

/**
 * @brief Configures Trigger Output settings for an UART Rx channel.
 *
 * A trigger can be generated after the completion of the UART buffer
 * processing. The trigger can be used to syncronize other processing
 * in the system.
 *
 * The Trigger Routing Unit should be seperately configured by the
 * application to route the outgoing UART trigger by calling the API
 * adi_tru_TriggerRoute.
 *
 * This API is only valid in DMA mode. Calling this API in interrupt
 * mode will result in an error.
 *
 * @param [in]  hChannel        UART device handle to configure.
 * @param [in]  eUartTrigger    Condition to generate an outgoing trigger.
 *
 * @return  Status
 *  - #ADI_UART_INVALID_HANDLE          [D] If the given UART handle is invalid.
 *  - #ADI_UART_DMA_FAILED                  If failed to configure the DMA service for trigger output.
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If configuring the trigger input in interrupt mode.
 */
ADI_UART_RESULT  adi_uart_ConfigRxTriggerOut (
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_TRIGGER_OUT        eUartTrigger)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to configure the trigger in interrupt mode.*/
    if(pDevice->bDMAMode == false)
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Call the DMA service to configure the trigger output */
    if(adi_dma_ConfigTriggerOut (pDevice->pRxChannel,
                                 (ADI_DMA_TRIGGER_OUT) eUartTrigger) != ADI_UART_SUCCESS)
    {
        return ADI_UART_DMA_FAILED;
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief Configures Trigger Output settings for an UART Tx channel.
 *
 * A trigger can be generated after the completion of the UART buffer
 * processing. The trigger can be used to syncronize other processing
 * in the system.
 *
 * The Trigger Routing Unit should be seperately configured by the
 * application to route the outgoing UART trigger by calling the API
 * adi_tru_TriggerRoute.
 *
 * This API is only valid in DMA mode. Calling this API in interrupt
 * mode will result in an error.
 *
 * @param [in]  hChannel        UART device handle to configure.
 * @param [in]  eUartTrigger    Condition to generate an outgoing trigger.
 *
 * @return  Status
 *  - #ADI_UART_INVALID_HANDLE          [D] If the given UART handle is invalid.
 *  - #ADI_UART_DMA_FAILED                  If failed to configure the DMA service for trigger output.
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If configuring the trigger input in interrupt mode.
 */
ADI_UART_RESULT  adi_uart_ConfigTxTriggerOut (
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_TRIGGER_OUT        eUartTrigger)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT         eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to configure the trigger in interrupt mode.*/
    if(pDevice->bDMAMode == false)
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    /* Call the DMA service to configure the trigger output */
    if(adi_dma_ConfigTriggerOut (pDevice->pTxChannel,
                                 (ADI_DMA_TRIGGER_OUT) eUartTrigger) != ADI_UART_SUCCESS)
    {
        return ADI_UART_DMA_FAILED;
    }

    return ADI_UART_SUCCESS;
}


/**
 * @brief       Sets DMA transfer width for Rx channel.
 *
 * @details     The DMA transfer width can changed to get better performance. When there are
 *              multiple peripherals in the system which require high data rate, the
 *              peripherals can starve for data. To avoid starvation the DMA transfer size
 *              can be increased. When transfer size is increased more data will be read
 *              at a stretch and avoid starvation due to external bus contention.The default
 *              DMA transfer width is 8bit.
 *
 * @note        Application should make sure that the buffer address and size is aligned to the 
 *              given DMA transfer width.
 *
 * @note        DMA transfer size should be set before submitting any buffers. DMA transfer
 *              size cannot be changed after data flow is enabled.
 *
 * @note        This API is valid only in DMA mode.
 *
 * @param [in]  hDevice                 Handle to the driver instance whose DMA transfer width
 *                                      to be set.
 *
 * @param [in]  eTransferSize           DMA transfer size to be set. It can be one of the
 *                                      values from #ADI_UART_DMA_TRANSFER_SIZE.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     If successfully configured the DMA transfer width.
 *  - #ADI_UART_INVALID_HANDLE          [D] If the given driver handle is not valid.
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If trying to set the DMA transfer size after
 *                                          data flow is enabled or when not in DMA mode.
 *  - #ADI_UART_INVALID_PARAMETER           If the given DMA transfer size is invalid.
 *  - #ADI_UART_DMA_FAILED              [D] If failed to set the DMA transfer size.
 *
 *  @sa adi_uart_SubmitRxBuffer()
 *  @sa adi_uart_SubmitTxBuffer()
 */
ADI_UART_RESULT  adi_uart_SetRxDmaTransferSize(
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_DMA_TRANSFER_SIZE  eTransferSize
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT          eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the DMA transfer size after dataflow  is enabled.*/
    if((pDevice->bDMAMode == false) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    return SetDMATransferSize(pDevice->pRxChannel, eTransferSize);
}

/**
 * @brief       Sets DMA transfer width for Tx channel.
 *
 * @details     The DMA transfer width can changed to get better performance. When there are
 *              multiple peripherals in the system which require high data rate, the
 *              peripherals can starve for data. To avoid starvation the DMA transfer size
 *              can be increased. When transfer size is increased more data will be read
 *              at a stretch and avoid starvation due to external bus contention.The default
 *              DMA transfer width is 32bit.
 *
 * @note        Application should make sure that the buffer address and size is aligned to the 
 *              given DMA transfer width.
 *
 * @note        DMA transfer size should be set before submitting any buffers. DMA transfer
 *              size cannot be changed after data flow is enabled.
 *
 * @note        This API is valid only in DMA mode.
 *
 * @param [in]  hDevice                 Handle to the driver instance whose DMA transfer width
 *                                      to be set.
 *
 * @param [in]  eTransferSize           DMA transfer size to be set. It can be one of the
 *                                      values from #ADI_UART_DMA_TRANSFER_SIZE.
 *
 * @return      Status
 *
 *  - #ADI_UART_SUCCESS                     If successfully configured the DMA transfer width.
 *  - #ADI_UART_INVALID_HANDLE          [D] If the given driver handle is not valid.
 *  - #ADI_UART_OPERATION_NOT_ALLOWED   [D] If trying to set the DMA transfer size after
 *                                          data flow is enabled or when not in DMA mode.
 *  - #ADI_UART_INVALID_PARAMETER           If the given DMA transfer size is invalid.
 *  - #ADI_UART_DMA_FAILED              [D] If failed to set the DMA transfer size.
 *
 *  @sa adi_uart_SubmitRxBuffer()
 *  @sa adi_uart_SubmitTxBuffer()
 */
ADI_UART_RESULT  adi_uart_SetTxDmaTransferSize(
    ADI_UART_HANDLE const       hDevice,
    ADI_UART_DMA_TRANSFER_SIZE  eTransferSize
)
{
    /* Pointer to UART device instance */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE *)hDevice;

    /* Return code */
    ADI_UART_RESULT          eResult;

#ifdef ADI_DEBUG

    /* Validate the given handle */
    if((eResult = ValidateHandle(pDevice)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Don't allow to set the DMA transfer size after dataflow  is enabled.*/
    if((pDevice->bDMAMode == false) ||
       (pDevice->pRxChannel->eDataFlowState != ADI_UART_DATAFLOW_DISABLED))
    {
        return ADI_UART_OPERATION_NOT_ALLOWED;
    }
#endif /* ADI_DEBUG */

    return SetDMATransferSize(pDevice->pTxChannel, eTransferSize);
}


/* Sets the DMA PSize and MSize based on the DMA transfer size */
static ADI_UART_RESULT SetDMATransferSize(ADI_UART_CHANNEL  *pChannel,
        ADI_UART_DMA_TRANSFER_SIZE  eTransferSize)
{
    /* M-Size */
    ADI_DMA_MSIZE            eMSize;

    /* Result Code */
    ADI_UART_RESULT         eResult;


    /* determine the number of bytes in the word transfer size */
    switch (eTransferSize)
    {
        case ADI_UART_DMA_TRANSFER_1BYTE:
            pChannel->nBytesPerTransfer = 1;
            eMSize = ADI_DMA_MSIZE_1BYTE;
            break;

        case ADI_UART_DMA_TRANSFER_2BYTES:
            pChannel->nBytesPerTransfer  = 2;
            eMSize = ADI_DMA_MSIZE_2BYTES;
            break;

        case ADI_UART_DMA_TRANSFER_4BYTES:
            pChannel->nBytesPerTransfer  = 4;
            eMSize = ADI_DMA_MSIZE_4BYTES;
            break;

        case ADI_UART_DMA_TRANSFER_8BYTES:
            pChannel->nBytesPerTransfer  = 8;
            eMSize = ADI_DMA_MSIZE_8BYTES;
            break;

        case ADI_UART_DMA_TRANSFER_16BYTES:
            pChannel->nBytesPerTransfer  = 16;
            eMSize = ADI_DMA_MSIZE_16BYTES;
            break;

        case ADI_UART_DMA_TRANSFER_32BYTES:
            pChannel->nBytesPerTransfer  = 32;
            eMSize = ADI_DMA_MSIZE_32BYTES;
            break;

        default:
            return ADI_UART_INVALID_PARAMETER;

    }

    /* Set the PSize */
    if(adi_pdma_SetPSize(pChannel->hDMAChnl, ADI_DMA_PSIZE_1BYTE) != ADI_DMA_SUCCESS)
    {
        return ADI_UART_DMA_FAILED;
    }
    /* Set the MSize */
    if(adi_pdma_SetMSize(pChannel->hDMAChnl, eMSize) != ADI_DMA_SUCCESS)
    {
        return ADI_UART_DMA_FAILED;
    }

    return ADI_UART_SUCCESS;
}


/* Collects the the received data into application buffer and notifies
 * application when the given buffer is filled
 */
static void CollectRxData(ADI_UART_DEVICE *pDevice)
{
    ADI_UART_BUF_INFO   *pBufInfo = pDevice->pRxChannel->pActiveBuf;

    /* When there is room to fill */
    if(pBufInfo->nIndex < pBufInfo->nBufSize)
    {
        /* Read the data from the UART Receive Buffer Register */
        pBufInfo->pBuffer[pBufInfo->nIndex++] = (uint8_t)ADI_ANOM_16000030_READ((uint8_t *)&pDevice->pUARTRegs->UART_RBR);

        /* Check if buffer is filled out */
        if(pBufInfo->nIndex == pBufInfo->nBufSize)
        {
            /* Take action on Rx Buffer completion */
            OnBufferCompletion(pDevice, pDevice->pRxChannel, pBufInfo);
        }
    }
   /* Not more Rx buffers so disable Rx UART interrupts */
    else
    {
        /* Disable the Rx UART */
        pDevice->pUARTRegs->UART_IMSK_CLR    =  (uint32_t)ENUM_UART_ERBFI_HI;
    }

    return;
}

/* Transmits the given data from the buffer and notifies
 * application when the given buffer is emptied
 */
static void TransmitData(ADI_UART_DEVICE *pDevice)
{
    ADI_UART_BUF_INFO   *pBufInfo = pDevice->pTxChannel->pActiveBuf;

    /* If there is data to transmit */
    if(pBufInfo->nIndex < pBufInfo->nBufSize)
    {
        /* Write the next word into THR register */
        pDevice->pUARTRegs->UART_THR = pBufInfo->pBuffer[pBufInfo->nIndex++];

        /* Check if the buffer is sent out */
        if(pBufInfo->nIndex == pBufInfo->nBufSize)
        {
            /* Take action on buffer completion */
            OnBufferCompletion(pDevice, pDevice->pTxChannel, pBufInfo);
        }
    }
    /* Not more Tx buffers so disable Rx UART interrupts */
    else
    {
          /* Disable the Tx UART */
          pDevice->pUARTRegs->UART_IMSK_CLR    =  (uint32_t)ENUM_UART_ETBEI_HI;
    }
    return;
}


/* Takes appropriate action on Buffer completion */
static void OnBufferCompletion(
    ADI_UART_DEVICE         const *pDevice,
    ADI_UART_CHANNEL              *pChannel,
    ADI_UART_BUF_INFO             *pBufInfo
)
{
    /* To make sure that the the buffer pointer is not overwitten by
     * Submit buffer as we are freeing the buffer here.
     */
    uint8_t *pBuffer = pBufInfo->pBuffer;

    /* Mark the buffer as free */
    pBufInfo->bInUse = false;

    /* If a callback is registered post the callback */
    if(pDevice->pfCallback != NULL)
    {
        /* Notify the application with buffer processed event */
        pDevice->pfCallback( pDevice->pCBParam,
                             (uint32_t)pChannel->eBufProcessedEvent,
                             pBuffer);

    }
    else
    {
        /* Post the semaphore when a callback is not registered */
        adi_osal_SemPost(pChannel->hSemaphore);
    }

    /* Get the next buffer to fill */
    pChannel->pActiveBuf    =   pBufInfo->pNext;

    return;
}


/*
 * Does the required buffer management and calls the abstracted
 * (DMA and Interrupt mode) functions
 */
static ADI_UART_RESULT SubmitBuffer(
    ADI_UART_DEVICE           *pDevice,
    ADI_UART_CHANNEL          *pChannel,
    void                      *pBuffer,
    uint32_t                   nBufferSize,
    bool                       bRead
)
{
    ADI_UART_BUF_INFO   *pBufInfo = pChannel->pFreeBuf;
    ADI_UART_RESULT      eResult;

#ifdef ADI_DEBUG
    /* Check if the given buffer pointer is valid */
    if(pBuffer == NULL)
    {
        return ADI_UART_NULL_POINTER;
    }

    /* Check the given buffer size */
    if(nBufferSize == 0u)
    {
        return ADI_UART_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */

    /* Check if there is room to submit new buffer */
    if(pBufInfo->bInUse == true)
    {
        return ADI_UART_SUBMIT_FAILED;
    }

    /* Mark the buffer as in use */
    pBufInfo->bInUse = true;

    /* Save the given buffer information */
    pBufInfo->pBuffer   = pBuffer;
    pBufInfo->nBufSize  = nBufferSize;
    pBufInfo->nIndex    = 0u;

    /* Submit the buffer */
    if((eResult = pDevice->pFnTable->pfSubmitBuffer(pDevice,
                                                    pChannel,
                                                    pBuffer,
                                                    nBufferSize,
                                                    bRead)) != ADI_UART_SUCCESS)
    {
        /* If there was a failure mark the buffer as free */
        pBufInfo->bInUse = false;

        return eResult;
    }
    
    /* Update the free buffer only on success */
    pChannel->pFreeBuf = pBufInfo->pNext;

    return ADI_UART_SUCCESS;
}

/*
 * Waits until the buffer is processed and returns the pointer
 * to the processed buffer
 */
static ADI_UART_RESULT GetBuffer(
    ADI_UART_DEVICE     const  *pDevice,
    ADI_UART_CHANNEL           *pChannel,
    void                      **ppBuffer
)
{
    ADI_UART_RESULT     eResult = ADI_UART_SUCCESS;
    ADI_UART_BUF_INFO  *pBufInfo;

    /* Wait until a processed buffer is available using
     * Semaphore pend. The Semaphore pend will return immediately
     * is there is buffer available. */
    if(adi_osal_SemPend(pChannel->hSemaphore,
                        ADI_OSAL_TIMEOUT_FOREVER) != ADI_OSAL_SUCCESS)
    {
        return ADI_UART_SEM_FAILED;
    }

    pBufInfo = pChannel->pProcessedBuf;

    /* Return the pointer to the processed buffer */
    *ppBuffer = pBufInfo->pBuffer;

    /* A processed buffer should be available as the semaphore is released.
     * If there is no processed buffer available it should be a hardware
     * error that caused the semaphore to be posted. */
    if(pBufInfo->nIndex < pBufInfo->nBufSize)
    {
        /* Return error */
        eResult     = ADI_UART_HW_ERROR;

        /* Return NULL pointer */
        *ppBuffer   = NULL;
    }

    /* Update the processed buffer queue */
    pChannel->pProcessedBuf = pChannel->pProcessedBuf->pNext;

    return eResult;
}


/* Handles blocking mode fuffer transaction  */
static ADI_UART_RESULT CompleteBlockingBuffferTxn(
    ADI_UART_DEVICE           *pDevice,
    ADI_UART_CHANNEL          *pChannel,
    void                      *pBuffer
)
{
    /* UART driver return code */
    ADI_UART_RESULT     eResult;

    /* Set the buffer transaction mode as blocking */
    pChannel->eBufTxnMode   =   ADI_UART_BUF_TXN_MODE_BLOCKING;

    /* Enable the data flow */
    if((eResult = EnableDataFlow(pDevice, pChannel, true)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Wait until completion */
    if((eResult = GetBuffer(pDevice, pChannel, &pBuffer)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Disalbe the data flow */
    if((eResult = EnableDataFlow(pDevice, pChannel, false)) != ADI_UART_SUCCESS)
    {
        return eResult;
    }

    /* Set back the buffer transaction mode to non-blocking */
    pChannel->eBufTxnMode   =   ADI_UART_BUF_TXN_MODE_NONBLOCKING;

    return ADI_UART_SUCCESS;
}


/*
 * UART Status interrupt handler
 */
static void UARTStatusHandler(uint32_t SID, void *pCBParam)
{
    ADI_UART_DEVICE     *pDevice = (ADI_UART_DEVICE  *)pCBParam;
    uint32_t            Mask;
    uint32_t            StatRegValue;
    uint32_t            TmpStatRegValue;
    uint32_t            Event;
    uint32_t            ErrorEvent;

#ifdef ADI_DEBUG
    /* This should not happen but a safety check */
    if(pDevice == NULL)
    {
        return;
    }
#endif /* ADI_DEBUG */

    /* Get the contents of status register */
    StatRegValue    =   ADI_ANOM_16000030_READ32(&pDevice->pUARTRegs->UART_STAT);
    TmpStatRegValue =   StatRegValue;

    /* For all the bits in the status register */
    for(Mask = 1u; Mask > 0u; Mask <<= 1u)
    {
        /* Take action only when the bit is set */
        if((TmpStatRegValue & Mask) != 0u)
        {
            /* Assume no event to be notified to application */
            Event       =   ADI_UART_NO_EVENT;
            ErrorEvent  =   ADI_UART_NO_EVENT;

            switch(Mask)
            {
                /* New data is available in the Rx FIFO */
                case (uint32_t)ENUM_UART_STAT_NEW_DATA:
                    CollectRxData(pDevice);
                    break;

                /* Rx FIFO overrun */
                case (uint32_t)ENUM_UART_STAT_OVR_ERR:
                    ErrorEvent   =   (uint32_t)ADI_UART_EVENT_OVERRUN_ERROR;
                    break;

                /* Detected a parity error */
                case (uint32_t)ENUM_UART_STAT_PARITY_ERR:
                    ErrorEvent   =   (uint32_t)ADI_UART_EVENT_PARITY_ERROR;
                    break;

                /* Detected a framing error */
                case (uint32_t)ENUM_UART_STAT_FRAMING_ERR:
                    ErrorEvent   =   (uint32_t)ADI_UART_EVENT_PARITY_ERROR;
                    break;

                /* A break is detected */
                case (uint32_t)ENUM_UART_STAT_BREAK_INT:
                    ErrorEvent   =   (uint32_t)ADI_UART_EVENT_BREAK_INTERRUPT;
                    break;

                /* Tx hold register is empty */
                case (uint32_t)ENUM_UART_STAT_THR_EMPTY:
                    /* 2DO: This check is not required. But as this interrupt
                     * is generated in DMA mode. This is a workaround.
                     */
                    if(pDevice->bDMAMode != true)
                    {
                        TransmitData(pDevice);
                    }
                    break;

                /* Transmission finished (Tx Hold Reg and Tx Shift Reg empty) */
                case (uint32_t)ENUM_UART_STAT_TX_DONE:
                    Event   =   (uint32_t)ADI_UART_EVENT_TX_COMPLETE;
                    break;

                /* Address is received from Master in MDB Mode */
                case (uint32_t)ENUM_UART_STAT_ADDR_HI_STKY:
                    /* Read the received address from the FIFO */
                    CollectRxData(pDevice);
                    break;

                /* Rx FIFO Water mark level reached */
                case (uint32_t)ENUM_UART_STAT_RFCS_HI:
                    Event   = (uint32_t)ADI_UART_EVENT_RX_FIFO_WM;
                    break;

                default:
                    /* We don't care about the rest of the events */
                    break;
            }

            /* Clear bit in local copy */
            TmpStatRegValue &= (~Mask);

            /* If there is an Error or normal event to be notified */
            if((Event | ErrorEvent) != ADI_UART_NO_EVENT)
            {
                /* If there was a callback registered notify them via callback */
                if(pDevice->pfCallback != NULL)
                {
                    /* Notify the event to application */
                    pDevice->pfCallback(pDevice->pCBParam,
                                        (Event | ErrorEvent),
                                        NULL);
                }
                else
                {
                    /* Unblock the blocking calls only if it is a hardware error */
                    if(ErrorEvent != ADI_UART_NO_EVENT)
                    {
                        /* Save the error and post the semaphore to release the
                         * blocking calls immediately.
                         */
                         pDevice->nHWError    |=  ErrorEvent;

                         /* Unblock Rx GetBuffer/Read calls */
                         if(pDevice->pRxChannel->hSemaphore != NULL)
                         {
                             adi_osal_SemPost(pDevice->pRxChannel->hSemaphore);
                         }

                         /* Unblock Tx GetBuffer/Read calls */
                         if(pDevice->pTxChannel->hSemaphore != NULL)
                         {
                             adi_osal_SemPost(pDevice->pTxChannel->hSemaphore);
                         }
                    }
                }
            }
        }

        /* If there are no more status bits set, stop the loop */
        if(TmpStatRegValue == 0u)
        {
            break;
        }
    }

    /* Clear the status register bits (W1C) */
    pDevice->pUARTRegs->UART_STAT   =   StatRegValue;

    return;
}

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_16_7:"Allow pointers parameter to be declared without const")
#endif
/*
 * UART Tx DMA interrupt handler
 */
static void UARTTxDMAHandler(void *pCBParam, uint32_t Event, void *pArg)
{
    /* Pointer to the UART device instance data */
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE  *)pCBParam;

    /* Pointer to the buffer that caused the interrupt */
    ADI_UART_BUF_INFO       *pActiveBuf = pDevice->pTxChannel->pActiveBuf;

    switch(Event)
    {
        /* The given buffer processed */
        case (uint32_t)ADI_DMA_EVENT_BUFFER_PROCESSED:

            /* Inidicate that the buffer is successfully processed */
            pActiveBuf->nIndex = pActiveBuf->nBufSize;

            OnBufferCompletion(pDevice, pDevice->pTxChannel, pDevice->pTxChannel->pActiveBuf);
            break;

        default:
            /* We don't care about other events */
            break;
    }

    return;
}

/*
 * UART Rx DMA interrupt handler
 */
static void UARTRxDMAHandler(void *pCBParam, uint32_t Event, void *pArg)
{
    /* Pointer to the UART device instance data */
    ADI_UART_DEVICE     *pDevice = (ADI_UART_DEVICE  *)pCBParam;

    /* Pointer to the buffer that caused the interrupt */
    ADI_UART_BUF_INFO   *pActiveBuf = pDevice->pRxChannel->pActiveBuf;

    switch(Event)
    {
        /* The given buffer processed */
        case (uint32_t)ADI_DMA_EVENT_BUFFER_PROCESSED:

            /* Inidicate that the buffer is successfully processed */
            pActiveBuf->nIndex = pActiveBuf->nBufSize;

            /* Call the function to take action on buffer completion event */
            OnBufferCompletion(pDevice, pDevice->pRxChannel, pActiveBuf);
            break;

        default:
            /* We don't care about other events */
            break;
    }

    return;
}
#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

/* Initializes the UART channel */
static ADI_UART_RESULT InitializeChannel(ADI_UART_CHANNEL *pChannel)
{
    /* Create the semaphore for Tx direction */
     if(adi_osal_SemCreateStatic(pChannel->SemMemory,
                                 ADI_OSAL_MAX_SEM_SIZE_CHAR,
                                 &pChannel->hSemaphore, 0u) != ADI_OSAL_SUCCESS)
     {
         return ADI_UART_SEM_FAILED;
     }

     /* Initialize DMA related parameters, by default DMA transfer size is set to 8-bits */
     pChannel->nBytesPerTransfer = 1;

     /* Initialize Tx buffer managment */
     InitializeBuffers(pChannel);

     return ADI_UART_SUCCESS;
}

/* Initaializes buffers and the queues */
static void InitializeBuffers(ADI_UART_CHANNEL *pChannel)
{
    uint32_t            nBuf;
    ADI_UART_BUF_INFO  *pBufInfo;

    /*
     * Initialize Tx buffer managment
     *
     * Buffers are circularly linked to make the management simpler.
     */
    for(nBuf = 0u; nBuf < ADI_UART_NUM_BUF; nBuf++)
    {
        /* Get the pointer to the buffer information */
        pBufInfo               =   &pChannel->BufInfo[nBuf];

        pBufInfo->bInUse       =   false;
        pBufInfo->nBufSize     =   0u;
        pBufInfo->nIndex       =   0u;
        pBufInfo->pBuffer      =   NULL;
        pBufInfo->pNext        =   (pBufInfo + 1u);
    }
    pChannel->BufInfo[nBuf - 1u].pNext  =   &pChannel->BufInfo[0];

    /* Initialize the Processed, Active and Free circular queues */
    pChannel->pProcessedBuf    =   &pChannel->BufInfo[0];
    pChannel->pActiveBuf       =   &pChannel->BufInfo[0];
    pChannel->pFreeBuf         =   &pChannel->BufInfo[0];

    return;
}



/* Enables/Disables the data flow */
static ADI_UART_RESULT EnableDataFlow(
    ADI_UART_DEVICE           *pDevice,
    ADI_UART_CHANNEL          *pChannel,
    bool                       bEnable
)
{
    /* UART return code */
    ADI_UART_RESULT     eResult;

#ifdef ADI_DEBUG
    /* Check if the data flow is already enabled/disabled */
    if( (( bEnable == true ) && (pChannel->eDataFlowState == ADI_UART_DATAFLOW_ENABLED)) ||
         ((bEnable == false) && (pChannel->eDataFlowState == ADI_UART_DATAFLOW_DISABLED)))
    {
        /* If it already in the required state do nothing */
        return ADI_UART_SUCCESS;
    }

    if(bEnable == true)
    {
        /* Check if buffers are submitted for either Ping and Pong descriptors */
        if((pChannel->BufInfo[ADI_UART_PING_BUF].pBuffer == NULL) &&
           (pChannel->BufInfo[ADI_UART_PONG_BUF].pBuffer == NULL))
        {
            return ADI_UART_BUFFERS_NOT_SUBMITTED;
        }

        if(pDevice->nBaudRate == 0u)
        {
            return ADI_UART_BAUD_RATE_NOT_SET;
        }
    }
#endif /* ADI_DEBUG */

    /* If enabling the data flow */
    if(bEnable == true)
    {
        /* Enable the data flow */
        if((eResult = pDevice->pFnTable->pfEnable(pDevice, pChannel, bEnable)) != ADI_UART_SUCCESS)
        {
            return eResult;
        }
        if(pChannel == pDevice->pTxChannel)
        {
            /* Enable the Tx UART */
            pDevice->pUARTRegs->UART_IMSK_SET    =  (uint32_t)ENUM_UART_ETBEI_HI;
        }
        else
        {
            /* Enable the Rx UART */
            pDevice->pUARTRegs->UART_IMSK_SET    =  (uint32_t)ENUM_UART_ERBFI_HI;
        }

        /* Update the dataflow state */
        pChannel->eDataFlowState    =   ADI_UART_DATAFLOW_ENABLED;
    }
    else
    {
        if(pChannel == pDevice->pTxChannel)
        {
            /* Disable the Tx UART */
            pDevice->pUARTRegs->UART_IMSK_CLR    =  (uint32_t)ENUM_UART_ETBEI_HI;
        }
        else
        {
            /* Disable the Rx UART */
             pDevice->pUARTRegs->UART_IMSK_CLR   =  (uint32_t)ENUM_UART_ERBFI_HI;
        }

        /* Enable the data flow */
        if((eResult = pDevice->pFnTable->pfEnable(pDevice, pChannel, bEnable)) != ADI_UART_SUCCESS)
        {
            return eResult;
        }

        /* Clear the buffers and retinialize queues */
        InitializeBuffers(pChannel);

        /* Update the dataflow sate */
        pChannel->eDataFlowState    =   ADI_UART_DATAFLOW_DISABLED;
    }

    return ADI_UART_SUCCESS;
}


/* ===================  Interrupt mode functions ======================== */

/* Takes care of cleanup required in interrupt mode */
static ADI_UART_RESULT uart_Int_Close(
   ADI_UART_DEVICE      *pDevice
)
{
    return ADI_UART_SUCCESS;
}

/* Submits a buffer for Rx/Tx in interrupt mode */
static ADI_UART_RESULT uart_Int_SubmitBuffer (
   ADI_UART_DEVICE      *pDevice,
   ADI_UART_CHANNEL     *pChannel,
   void                 *pBuffer,
   uint32_t              nBufferSize,
   bool                  bRead
)
{
    /* Re-enable the interrupts if data flow is enabled 
     * The interrupts will be disabled if there is not data 
     * available. As there is new data available enable
     * them back.
     */
    if(pChannel->eDataFlowState == ADI_UART_DATAFLOW_ENABLED)
    {
      if(bRead == true)
      {
        /* Enable Rx interrupts */
        pDevice->pUARTRegs->UART_IMSK_SET = ENUM_UART_ERBFI_HI;
      }
      else
      {
          /* Enable Tx interrupts */
          pDevice->pUARTRegs->UART_IMSK_SET = ENUM_UART_ETBEI_HI;
      }
    }
    return ADI_UART_SUCCESS;
}


/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_UART_RESULT uart_Int_Enable  (
   ADI_UART_DEVICE      *pDevice,
   ADI_UART_CHANNEL     *pChannel,
   bool                  bEnable
)
{
    return ADI_UART_SUCCESS;
}


/* ==================   DMA mode functions ============================== */

/* Takes care of cleanup required when closing UART which was in DMA mode */
static ADI_UART_RESULT uart_DMA_Close(
    ADI_UART_DEVICE      *pDevice
)
{
    /* If a Tx DMA channel was opened */
    if(pDevice->pTxChannel->hDMAChnl != NULL)
    {
        if(adi_pdma_Close(pDevice->pTxChannel->hDMAChnl, false) != ADI_DMA_SUCCESS)
        {
            return ADI_UART_DMA_FAILED;
        }
    }

    /* If a Rx DMA channel was opened */
    if(pDevice->pRxChannel->hDMAChnl != NULL)
    {
        if(adi_pdma_Close(pDevice->pRxChannel->hDMAChnl, false) != ADI_DMA_SUCCESS)
        {
            return ADI_UART_DMA_FAILED;
        }
    }

    return ADI_UART_SUCCESS;
}


/* Submits a buffer in DMA mode */
static ADI_UART_RESULT uart_DMA_SubmitBuffer(
    ADI_UART_DEVICE      *pDevice,                                             
    ADI_UART_CHANNEL     *pChannel,
    void                 *pBuffer,
    uint32_t              nBufferSize,
    bool                  bRead
)
{
    int32_t nBytesPerTransfer = pChannel->nBytesPerTransfer;
#ifdef ADI_DEBUG
    if((uint32_t)(nBufferSize & ((uint32_t)nBytesPerTransfer - 1ul)) != 0ul)
    {
        return ADI_UART_INVALID_PARAMETER;
    }
#endif /* ADI_DEBUG */
    if(bRead == true)
    {
        if(adi_pdma_PeripheralRead(pChannel->hDMAChnl,
                                   pBuffer,                                   /* Start Address */
                                   nBufferSize/(uint32_t)nBytesPerTransfer,   /* X Count */
                                   nBytesPerTransfer,                         /* X Modify */
                                   0u,                                        /* Y Count */
                                   0) != ADI_DMA_SUCCESS)                     /* Y Modify */
        {
            return ADI_UART_DMA_FAILED;
        }
    }
    else
    {
        if(adi_pdma_PeripheralWrite(pChannel->hDMAChnl,
                                    pBuffer,                                    /* Start Address */
                                    nBufferSize/(uint32_t)nBytesPerTransfer,    /* X Count */
                                    nBytesPerTransfer,                          /* X Modify */
                                    0u,                                         /* Y Count */
                                    0) != ADI_DMA_SUCCESS)                      /* Y Modify */
        {
            return ADI_UART_DMA_FAILED;
        }
    }

    return ADI_UART_SUCCESS;
}


/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_UART_RESULT uart_DMA_Enable(
    ADI_UART_DEVICE      *pDevice,
    ADI_UART_CHANNEL     *pChannel,
    bool                  bEnable
)
{
    /* If DMA channel to be enabled*/
    if(bEnable == true)
    {
        if(adi_pdma_EnableXfer(pChannel->hDMAChnl) != ADI_DMA_SUCCESS)
        {
            return ADI_UART_DMA_FAILED;
        }
    }
    /* DMA channel to be disabled */
    else
    {
        if(adi_pdma_DisableXfer(pChannel->hDMAChnl) != ADI_DMA_SUCCESS)
        {
            return ADI_UART_DMA_FAILED;
        }
    }

    return ADI_UART_SUCCESS;
}

/*
 * @brief       Callback function for Timer. Used only in Autobaud.
 *
 * @param [in]  pCBParam    Callback parameter from timer manager.
 * @param [in]  Event       Event ID.
 * @param [in]  pArg        Callback argument.
 *
 */
static void TimerCallback(void *pCBParam, uint32_t Event, void *pArg)
{
    ADI_UART_DEVICE         *pDevice = (ADI_UART_DEVICE  *)pCBParam;

    /* System Clock Frequency in Hz*/
    uint32_t                SysClk = 0u;
    /* UART Clock Divide Factor */
    uint32_t                ClkDivide;
    /* Width of the input signal */
    uint32_t                nWidth;
    /* Contents of UART_CLOCK register */
    uint32_t                UARTClkValue;
    /* Prescale value based on UART_CLOCK.EDBO (Will be 16 when set, otherwise 1) */
    float                   nPreScale = 16.0;
    /* System Clock Frequencies*/
    uint32_t   fsysclk = 0u;   /* System Clock */
    uint32_t   fsclk0  = 0u;   /* System Clock0 */
    uint32_t   fsclk1  = 0u;   /* System Clock1 */

    /* Get width of the pulse */
    adi_tmr_GetWidth(pDevice->hTimer,&nWidth);

    /* Get system clock frequencies */
    adi_pwr_GetSystemFreq(&fsysclk, &fsclk0, &fsclk1);

    /* Disable the timer */
    adi_tmr_Enable(pDevice->hTimer, false);

    /* Get the current contents of UART_CLOCK register */
    UARTClkValue    =   pDevice->pUARTRegs->UART_CLK;

    /* If Enable Divide By One is set then prescale value is 1 otherwise 16 */
    if((UARTClkValue & BITM_UART_CLK_EDBO) == ENUM_UART_CLK_EN_DIV_BY_ONE)
    {
       nPreScale   =   1.0;
    }

    /* calculate the baud rate */
#if defined(ADI_ADSP_CM40Z) 
    /* In case of CM40Z processor there is only main system clock. And UART is
       connected to main system clock. So use fsysclk for computation. */
    pDevice->nBaudRate = (fsysclk * pDevice->nDivisorBits)/nWidth;
#else    
    /* UART is connected to SCLK0 on other processors */
    pDevice->nBaudRate = (fsclk0 * pDevice->nDivisorBits)/nWidth;
#endif    

    /* Find clock divide value based on the found width */
    ClkDivide =  (uint32_t)((float)nWidth/((float)pDevice->nDivisorBits * nPreScale));

    /* Clear the existing divide factor and store the new value */
    UARTClkValue    =  ( (UARTClkValue & (uint32_t)(~BITM_UART_CLK_DIV)) | ClkDivide );

    /* Write back the register contents */
    pDevice->pUARTRegs->UART_CLK   =   UARTClkValue;

    /* Change the status of the device */
    pDevice->eState = ADI_UART_STATE_INITIALIZED;

    return;
}

/* ==================  Debug mode functions =============================== */
#ifdef ADI_DEBUG
static ADI_UART_RESULT  ValidateHandle(ADI_UART_DEVICE *pDevice)
{
    uint32_t i;

    for(i = 0u; i < ADI_UART_NUM_INSTANCES; i++)
    {
        if(pDevice ==  gUARTDevInfo[i].pDevice)
        {
            return ADI_UART_SUCCESS;
        }
    }

    return ADI_UART_INVALID_HANDLE;
}

#endif /* ADI_DEBUG */


/*@}*/



#ifdef _MISRA_RULES
#pragma diag(pop)
#endif
