/*!
 *****************************************************************************
 * @file:    adi_uart_def_bf60x.h
 * @brief:   UART Device Driver definition for blackfin 60x processor family
 * @version: $Revision: 22007 $
 * @date:    $Date: 2013-06-25 15:06:10 -0400 (Tue, 25 Jun 2013) $
 *****************************************************************************
Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************

File         :   adi_uart_def_bf60x.h

Description  :   Static function and structure definition needed by the UART 
                 device driver.

================================================================================ */


#ifndef _ADI_UART_DEF_BF60X_H_
#define _ADI_UART_DEF_BF60X_H_

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_17_4:"Allow pointer arithmetic for MMR address generation")
#pragma diag(suppress:misra_rule_19_7:"Allow macros to be used for readability")
#endif /* _MISRA_RULES */

/* Number of buffers supported */
#define ADI_UART_NUM_BUF            2u

/* Definition for Ping and Pong buffers */
#define ADI_UART_PING_BUF           0u
#define ADI_UART_PONG_BUF           1u

/* Event not required to be notified to application */
#define ADI_UART_NO_EVENT           0xFFFFFFFFu

/* Default Status Value to clear the status register */
#define DEFAULT_STATUS_VALUE    (                                     \
                                    ENUM_UART_STAT_OVR_ERR          | \
                                    ENUM_UART_STAT_PARITY_ERR       | \
                                    ENUM_UART_STAT_FRAMING_ERR      | \
                                    ENUM_UART_STAT_BREAK_INT        | \
                                    ENUM_UART_STAT_TX_DONE          | \
                                    ENUM_UART_STAT_ADDR_HI_STKY     | \
                                    ENUM_UART_STAT_ADDR_HI          | \
                                    ENUM_UART_STAT_CTS_HI_STKY        \
                                )


/* Default Mask value for the UART interrupt mask register */
#define DEFAULT_MASK_VALUE      (                                     \
                                    ENUM_UART_EDTPTI_HI             | \
                                    ENUM_UART_ELSI_HI               | \
                                    ENUM_UART_ETFI_HI               | \
                                    ENUM_UART_ERXS_HI               | \
                                    ENUM_UART_ETXS_HI                 \
                                )

/* Mask enable or disable line status interrupts */
#define LINE_STATUS_MASK        (                                     \
                                    ENUM_UART_ELSI_HI               | \
                                )

/* Default control register value */
#define DEFAULT_CONTROL_VALUE  (                                      \
                                    ENUM_UART_CTL_CLK_EN              \
                               )

#define DEFAULT_CLOCK_VALUE    (                                      \
                                    ENUM_UART_CLK_EN_DIV_BY_ONE     | \
                                    0xFFu                             \
                               )

/* Default number of divisor bits for default autobaud character space (' ') */
#define ADI_UART_DEFAULT_DIVISOR_BITS   6u

/* Forward declaration for pointer to UART device */
typedef struct _ADI_UART_DEVICE_*        ADI_UART_PDEVICE;

/* Enumeration of different UART states */
typedef enum
{
    ADI_UART_STATE_UNKNOWN       =   0,             /* UART is not yet initialized */
    ADI_UART_STATE_INITIALIZED,                     /* UART is initialized */
    ADI_UART_STATE_AUTOBAUD_IN_PROGRESS,            /* UART Auto baud operation is in progress */
    ADI_UART_STATE_AUTOBAUD_COMPLETED               /* UART Auto baud operation is completed */

} ADI_UART_STATE;

/* Enumeration of different data flow states */
typedef enum
{
    ADI_UART_DATAFLOW_DISABLED,
    ADI_UART_DATAFLOW_ENABLED,
    ADI_UART_DATAFLOW_PAUSED

} ADI_UART_DATAFLOW_STATE;


/* UART registers */
typedef struct _ADI_UART_REGS_
{
    volatile uint32_t  UART_CTL;                        /* UART Control Register */
    volatile uint32_t  UART_STAT;                       /* UART Status Register */
    volatile uint32_t  UART_SCR;                        /* UART Scratch Register */
    volatile uint32_t  UART_CLK;                        /* UART Clock Rate Register */
    volatile uint32_t  UART_IMSK;                       /* UART Interrupt Mask Register */
    volatile uint32_t  UART_IMSK_SET;                   /* UART Interrupt Mask Set Register */
    volatile uint32_t  UART_IMSK_CLR;                   /* UART Interrupt Mask Clear Register */
    volatile uint32_t  UART_RBR;                        /* UART Receive Buffer Register */
    volatile uint32_t  UART_THR;                        /* UART Transmit Hold Register */
    volatile uint32_t  UART_TAIP;                       /* UART Transmit Address/Insert Pulse Register */
    volatile uint32_t  UART_TSR;                        /* UART Transmit Shift Register */
    volatile uint32_t  UART_RSR;                        /* UART Receive Shift Register */
    volatile uint32_t  UART_TXCNT;                      /* UART Transmit Counter Register */
    volatile uint32_t  UART_RXCNT;                      /* UART Receive Counter Register */

} ADI_UART_REGS;

/* Enumeration of different buffer transaction modes */
typedef enum
{
    ADI_UART_BUF_TXN_MODE_NONBLOCKING,                  /* Non blocking mode (Submit/Get) */
    ADI_UART_BUF_TXN_MODE_BLOCKING,                     /* Blocking mode (Read/Write) */
    ADI_UART_BUF_TXN_MODE_CALLBACK                      /* Callback mode (always non blocking) */

}ADI_UART_BUF_TXN_MODE;

/* Structure which holds the buffer related information */
typedef struct _ADI_UART_BUF_INFO_
{
    bool                            bInUse;             /* Flag which indicates whether the buffer is in use by driver */
    uint8_t                        *pBuffer;            /* Pointer to the buffer supplied by application */
    uint32_t                        nIndex;             /* Index to the element in the buffer to be Rx/Tx */
    uint32_t                        nBufSize;           /* Size of the buffer */
    struct _ADI_UART_BUF_INFO_     *pNext;              /* Pointer to the next buffer info */
} ADI_UART_BUF_INFO;

/* Structure to hold the Rx/Tx channel related information */
typedef struct _ADI_UART_CHANNEL_
{
    /* DMA Channel Handle */
    ADI_DMA_CHANNEL_HANDLE          hDMAChnl;

    /* Semaphore handle */
    ADI_OSAL_SEM_HANDLE             hSemaphore;

    /* Data flow state */
    ADI_UART_DATAFLOW_STATE         eDataFlowState;

    /* Buffer transaction mode */
    ADI_UART_BUF_TXN_MODE           eBufTxnMode;

    /* Buffer management related information */
    ADI_UART_BUF_INFO               BufInfo[ADI_UART_NUM_BUF];

    /* Pointer to the buffer information that is being transmitted */
    ADI_UART_BUF_INFO              *pActiveBuf;

    /* Pointer to the buffer information which is processed and ready to be
     * returned to application.
     */
    ADI_UART_BUF_INFO              *pProcessedBuf;

    /* Pointer to the buffer information which can be reused for holding the
     * buffer supplied by the application.
     */
    ADI_UART_BUF_INFO              *pFreeBuf;

    /* Pointer to the memory required for opening DMA channel */
    void *                          pDMAChnlMemory;

    /* Event to indicate that the buffer is processed; As the buffer
     * processed events are different for Rx and Tx, to save code space
     * a common code handles both Rx and Tx buffer completion events.
     *
     * So it is required to save the event to be passed to application.
     */
    ADI_UART_EVENT                  eBufProcessedEvent;

    /* Memory required for creating Semaphore */
    uint8_t                         SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR];

    /* Number of bytes transferred in a DMA transfer */
    int32_t                         nBytesPerTransfer;

} ADI_UART_CHANNEL;


/* UART device related information */
typedef struct _ADI_UART_INFO_
{
    volatile ADI_UART_REGS * CONST   pUARTRegs;          /* Pointer to UART Memory Mapped Registers */
    ADI_DMA_CHANNEL_ID  CONST       eTxDMAChnlID;       /* Tx DMA Channel ID */
    ADI_DMA_CHANNEL_ID  CONST       eRxDMAChnlID;       /* Rx DMA Channel ID */
    uint32_t                        nStatusIntID;       /* Status Interrupt ID */
    ADI_UART_PDEVICE                pDevice;            /* Pointer to UART device instance data */

} ADI_UART_INFO;

/* Table to hold the function pointers related interrupt mode and DMA mode */
typedef struct _ADI_UART_FN_TABLE_
{
    ADI_UART_RESULT (* pfClose)(ADI_UART_PDEVICE  pDevice);
    ADI_UART_RESULT (* pfSubmitBuffer)(ADI_UART_PDEVICE pDevice, ADI_UART_CHANNEL *pChannel, void *pBuffer, uint32_t nBufSize, bool bRead);
    ADI_UART_RESULT (* pfEnable)(ADI_UART_PDEVICE pDevice, ADI_UART_CHANNEL *pChannel, bool bEnable);

}ADI_UART_FN_TABLE;


/* UART driver instance data */
typedef struct _ADI_UART_DEVICE_
{
    ADI_UART_INFO                  *pDevInfo;           /* UART device related information */
    ADI_UART_STATE                  eState;             /* UART driver state */
    ADI_CALLBACK                    pfCallback;         /* Application supplied callback */
    void                           *pCBParam;           /* Application supplied callback parameter */
    ADI_UART_DIRECTION              eDirection;         /* Direction in which the UART is opened */
    ADI_UART_MODE                   eMode;              /* Mode in which UART device operates */
    bool                            bDMAMode;           /* Flag which indicates whether DMA Mode is in use */
    uint32_t                        nTmrDevNum;         /* Timer to be used for Autobaud detection */
    ADI_TMR_HANDLE                  hTimer;             /* Handle to the timer used for Autobaud detection */
    uint32_t                        nDivisorBits;       /* Number of divisorBits */
    uint32_t                        nBaudRate;          /* Baud rate in Hz */
    volatile uint32_t               nHWError;           /* Holds the un-reported hardware errors */
    ADI_UART_FN_TABLE  const       * pFnTable;          /* Pointer to function pointer table */
    volatile ADI_UART_REGS         *pUARTRegs;          /* Pointer to UART registers for this instance */
    ADI_UART_CHANNEL               *pTxChannel;         /* Tx Channel related data */
    ADI_UART_CHANNEL               *pRxChannel;         /* Rx Channel related data */

    /* Memory required for autobaud timer */
    uint8_t                         TimerMemory[ADI_TMR_MEMORY];

} ADI_UART_DEVICE;

/* Number of UART instances in the processor */
#define ADI_UART_NUM_INSTANCES      (sizeof(gUARTDevInfo)/sizeof(ADI_UART_INFO))


/* ======================= Local functions ================================= */

/* Initializes an UART channel */
static ADI_UART_RESULT InitializeChannel(
    ADI_UART_CHANNEL    *pChannel
);

/* Initializes buffers and the queues */
static void InitializeBuffers(
    ADI_UART_CHANNEL *pChannel
);

/* Enables/Disables the data flow */
static ADI_UART_RESULT EnableDataFlow(
    ADI_UART_DEVICE           *pDevice,
    ADI_UART_CHANNEL          *pChannel,
    bool                       bEnable
);

/*
 * Waits until the buffer is processed and returns the pointer
 * to the processed buffer
 */
static ADI_UART_RESULT GetBuffer(
    ADI_UART_DEVICE     const  *pDevice,
    ADI_UART_CHANNEL           *pChannel,
    void                      **ppBuffer
);

/*
 * Does the required buffer management and calls the abstracted
 * (DMA and Interrupt mode) functions
 */
static ADI_UART_RESULT SubmitBuffer(
    ADI_UART_DEVICE           *pDevice,
    ADI_UART_CHANNEL          *pChannel,
    void                      *pBuffer,
    uint32_t                   nBufferSize,
    bool                       bRead
);

/* Handles blocking mode buffer transaction  */
static ADI_UART_RESULT CompleteBlockingBuffferTxn(
    ADI_UART_DEVICE           *pDevice,
    ADI_UART_CHANNEL          *pChannel,
    void                      *pBuffer
);

/* Takes appropriate action on Buffer completion */
static void OnBufferCompletion(
    ADI_UART_DEVICE         const *pDevice,
    ADI_UART_CHANNEL              *pChannel,
    ADI_UART_BUF_INFO             *pBufInfo
);

/* Sets the UART DMA transfer width */
static ADI_UART_RESULT SetDMATransferSize(ADI_UART_CHANNEL  *pChannel,
        ADI_UART_DMA_TRANSFER_SIZE  eTransferSize);

/*
 * UART Status interrupt handler
 */
static void UARTStatusHandler(
   uint32_t SID,
   void *pCBParam
);

/*
 * UART DMA interrupt handler
 */
static void UARTTxDMAHandler(void *pCBParam, uint32_t Event, void *pArg);
static void UARTRxDMAHandler(void *pCBParam, uint32_t Event, void *pArg);


/* Transmits the given data from the buffer and notifies
 * application when the given buffer is emptied
 */
static void TransmitData(
    ADI_UART_DEVICE *pDevice
);

/* Collects the the received data into application buffer and notifies
 * application when the given buffer is filled
 */
static void CollectRxData(
    ADI_UART_DEVICE *pDevice
);

/*
 * Abstraction functions for Interrupt mode and DMA mode.
 */

/*
 * Interrupt mode functions
 */

/* Takes care of cleanup required in interrupt mode */
static ADI_UART_RESULT uart_Int_Close(
   ADI_UART_DEVICE       *pDevice
);

/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_UART_RESULT uart_Int_Enable  (
   ADI_UART_DEVICE      *pDevice,
   ADI_UART_CHANNEL     *pChannel,
   bool                  bEnable
);

/* Submits a buffer in Interrupt mode */
static ADI_UART_RESULT uart_Int_SubmitBuffer(
   ADI_UART_DEVICE      *pDevice,                                             
   ADI_UART_CHANNEL     *pChannel,
   void                 *pBuffer,
   uint32_t              nBufferSize,
   bool                  bRead
);

/*
 * DMA mode functions
 */

/* Takes care of cleanup required when closing UART which was in DMA mode */
static ADI_UART_RESULT uart_DMA_Close(
    ADI_UART_DEVICE      *pDevice
);


/* Submits a buffer in DMA mode */
static ADI_UART_RESULT uart_DMA_SubmitBuffer(
    ADI_UART_DEVICE      *pDevice,                                             
    ADI_UART_CHANNEL     *pChannel,
    void                 *pBuffer,
    uint32_t              nBufferSize,
    bool                  bRead
);

/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_UART_RESULT uart_DMA_Enable(
    ADI_UART_DEVICE      *pDevice,
    ADI_UART_CHANNEL     *pChannel,
    bool                  bEnable
);



#ifdef _MISRA_RULES

#pragma diag(pop)

#endif /* _MISRA_RULES */

#endif /* _ADI_UART_DEF_BF60X_H_ */
