/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_usbd_dev_common.c ($Rev: 14975 $)
*
* @brief     USB Device Driver common functions.
*
* @details
* 			 The following functions are common to all controller driver implementations.
*   		 adi_usbd_dev_EpAddEntry         Adds Endpoint Entry to table
*   		 adi_usbd_dev_EpRemoveEntry      Removes Endpoint Entry from table
*   		 adi_usbd_dev_EpGetEntry         Retrieves Endpoint Entry from table
*   		 adi_usbd_dev_IsDataCacheEnabled Returns true if data cache enabled
*   		 adi_usbd_dev_FlushCachedBuffer	 Flushes DMA data buffer, if required
*/
/*
*********************************************************************************************************
*
* ADI USB (DEVICE MODE) Controller Driver common functions
*
*
*********************************************************************************************************
*/

#include <string.h>
#include <drivers/usb/controller/device/adi_usbd_dev.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_1:"Assembly language shall be encapsulated and isolated.")

#pragma diag(suppress:misra_rule_5_1:"")
#pragma diag(suppress:misra_rule_5_6:"")
#pragma diag(suppress:misra_rule_5_7:"")

#pragma diag(suppress:misra_rule_10_1_a:"The value of an expression of integer type shall not be implicitly converted to a different underlying type if it is not a conversion to a wider integer type of the same signedness.")
#pragma diag(suppress:misra_rule_10_3:"The value of a complex expression of integer type shall only be cast to a type of the same signedness that is no wider than the underlying type of the expression.")

#pragma diag(suppress:misra_rule_6_3:"")

#pragma diag(suppress:misra_rule_11_3:"")
#pragma diag(suppress:misra_rule_11_4:"")

#pragma diag(suppress:misra_rule_12_1:"")
#pragma diag(suppress:misra_rule_12_6:"")
#pragma diag(suppress:misra_rule_12_7:"Bitwise operators shall not be applied to operands whose underlying type is signed")

#pragma diag(suppress:misra_rule_13_2:"")

#pragma diag(suppress:misra_rule_16_7:"")

#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic.")

#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct.")
#pragma diag(suppress:misra_rule_19_7:"")

#endif /* _MISRA_RULES */

#if defined(__ADSPBLACKFIN__) && (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
/*
 * if CPLB data cache is enabled then we provide functions to maintain cache-dma coherency
 */
#include <cplb.h>
extern  int     __cplb_ctrl;
/*
 * Local functions' prototypes
 */
static void FlushArea(void *StartAddress, void *EndAddress);
static void FlushInvArea(void *StartAddress, void *EndAddress);

/*
 * Assembly code to Flush (& invalidate) a cache line
 */
#define FLUSH(P)           {asm volatile("FLUSH[%0++];":"+p"(P));}
#define FLUSHINV(P)        {asm volatile("FLUSHINV[%0++];":"+p"(P));}


/*
 * @brief       The adi_usbd_dev_FlushCachedBuffer API Flushes (and invalidates) the given
 * 				DMA buffer.
 *
 * @param[in]   Direction	Direction of transfer:
 * 								ADI_USBD_DEV_EP_DIR_RX - Host to Device
 * 								ADI_USBD_DEV_EP_DIR_TX - Device to Host
 * @param[in]   pData		Pointer to DMA buffer.
 * @param[in]   DataLen		Length in bytes of DMA buffer.
 *
 * @return		void
 */
void adi_usbd_dev_FlushCachedBuffer(
                ADI_USBD_DEV_EP_DIR      Direction,
                void                     *pData,
                uint32_t                 DataLen
)
{

    if( (__cplb_ctrl & CPLB_ENABLE_DCACHE ) || (__cplb_ctrl & CPLB_ENABLE_DCACHE2))
    {
        if (Direction == ADI_USBD_DEV_EP_DIR_RX)
        {
            FlushInvArea((void*)pData,(void*)(((uint8_t*)pData)+DataLen));
        }
        else {
            FlushArea((void*)pData,(void*)(((uint8_t*)pData)+DataLen));
        }
    }
}

/*
 * @brief       The adi_usbd_dev_IsDataCacheEnabled API is called to determine whether cache is enabled.
 *
 * @param[in]   void
 *
 * @return		bool: true if cache is enabled, false if not.
 */
bool adi_usbd_dev_IsDataCacheEnabled(void)
{
    bool Result = false;
    if( (__cplb_ctrl & CPLB_ENABLE_DCACHE ) || (__cplb_ctrl & CPLB_ENABLE_DCACHE2))
    {
        Result = true;
    }

    return Result;
}

/*
 * Flushes the contents of the cache from specified start address to the end address
 */
static void FlushArea(void *StartAddress, void *EndAddress)
{
    StartAddress = (void *)(((uint32_t)StartAddress)&(~31));
    while (StartAddress < EndAddress) {
        FLUSH(StartAddress);
    }
}

/*
 * Flushes and invalidates the contents of cache from the  specified start address to
 * the end address
 */
static void FlushInvArea(void *StartAddress, void *EndAddress)
{
    StartAddress = (void *)(((uint32_t)StartAddress)&(~31));
    while (StartAddress < EndAddress) {
        FLUSHINV(StartAddress);
    }
}


#else
bool adi_usbd_dev_IsDataCacheEnabled(void)
{
    return false;
}
#endif

/*
 * @brief       The adi_usbd_dev_EpCreateTable API creates the Endpoint entry table.
 *
 * @param[in]   NumberEntries	Number of endpoints for which table entries are required.
 * @param[in]   pDataMemory		Memory to use for table.
 * @param[out]  ppEpTableDef	Location to hold table pointer on return.
 * @param[in]   EntrySize		Size of each entry.
 *
 * @return		void
 *
 */
void adi_usbd_dev_EpCreateTable (
        uint8_t 					NumberEntries,
        void 						*pDataMemory,
        ADI_USBD_DEV_EP_TABLE_DEF 	**ppEpTableDef,
        uint32_t 					EntrySize
)
{
    uint8_t i;
    ADI_USBD_DEV_EP_DEF **pTable;
    uint32_t TableSize = sizeof(ADI_USBD_DEV_EP_TABLE_DEF) + NumberEntries*sizeof(ADI_USBD_DEV_EP_DEF*);
    uint8_t *pData = (uint8_t*)pDataMemory + TableSize;

    memset(pDataMemory, 0, TableSize);

    *ppEpTableDef = (ADI_USBD_DEV_EP_TABLE_DEF *)pDataMemory;
    (*ppEpTableDef)->pTable = (ADI_USBD_DEV_EP_DEF **)&(((uint8_t*)pDataMemory)[sizeof(ADI_USBD_DEV_EP_TABLE_DEF)]);
    (*ppEpTableDef)->NumberEntries = NumberEntries;

    pTable = (*ppEpTableDef)->pTable;
    for (i=0; i<NumberEntries; i++)
    {
        pTable[i] = (ADI_USBD_DEV_EP_DEF*)(pData + i*EntrySize);
    }

}

/*
 * @brief       The adi_usbd_dev_EpAddEntry API adds an entry to the Endpoint table.
 *
 * @param[in]   pEpTable Pointer to Endpoint table.
 *
 * @return		Pointer to entry added
 *
 */
ADI_USBD_DEV_EP_DEF *adi_usbd_dev_EpAddEntry(
        ADI_USBD_DEV_EP_TABLE_DEF *pEpTable
)
{
    uint8_t i;
    ADI_USBD_DEV_EP_DEF *pEntry;

    for (i=0;i<(pEpTable->NumberEntries);i++)
    {
        if (pEpTable->pTable[i]->bInUse == 0)
        {
            pEntry = pEpTable->pTable[i];
            pEntry->bInUse = 1;
            break;
        }
    }
    return pEntry;
}

/*
 * @brief       The adi_usbd_dev_EpRemoveEntry API removes an entry from the Endpoint table for the given
 * 				endpoint address.
 *
 * @param[in]   pEpTable Pointer to Endpoint table.
 * @param[in]   EpAddr   Endpoint address.
 *
 * @return		void
 *
 */
void adi_usbd_dev_EpRemoveEntry(
			ADI_USBD_DEV_EP_TABLE_DEF 	*pEpTable,
			uint8_t              		EpAddr
)

{
    uint8_t i;

    /* Find Entry */
    for (i=0;i<(pEpTable->NumberEntries);i++)
    {
        if ((pEpTable->pTable[i]) && (pEpTable->pTable[i]->EpAddr == EpAddr))
        {
            pEpTable->pTable[i]->bInUse = 0;
            break;
        }
    }

}

/*
 * @brief       The adi_usbd_dev_EpGetEntry API retrieves an entry from the Endpoint table for the given
 * 				endpoint address.
 *
 * @param[in]   pEpTable Pointer to Endpoint table.
 * @param[in]   EpAddr   Endpoint address.
 *
 * @return		Located entry; NULL if not found.
 *
 */
ADI_USBD_DEV_EP_DEF *adi_usbd_dev_EpGetEntry(
            ADI_USBD_DEV_EP_TABLE_DEF 	*pEpTable,
            uint8_t              		EpAddr
)
{
    uint8_t i;
    ADI_USBD_DEV_EP_DEF *pEntry = 0;

    /* Find Entry */
    for (i=0;i<(pEpTable->NumberEntries);i++)
    {
        if ((pEpTable->pTable[i]->bInUse) && (pEpTable->pTable[i]->EpAddr == EpAddr))
        {
            pEntry = pEpTable->pTable[i];
            break;
        }
    }

    return pEntry;
}


/*
 * @brief       The adi_usbd_dev_EpCreateBufferList API creates a table of buffer entries.
 *
 * @param[in]   NumberEntries	Number of buffer entries for which table entries are required.
 * @param[in]   pDataMemory		Memory to use for table.
 * @param[out]  ppEpBufferDef	Location to hold table pointer on return.
 *
 * @return		void.
 *
 */
void adi_usbd_dev_EpCreateBufferList (
        uint8_t 						NumberEntries,
        void 							*pDataMemory,
        ADI_USBD_DEV_EP_BUFFER_LIST 	**ppEpBufferDef
)
{
    uint8_t i;
    uint32_t ListSize = sizeof(ADI_USBD_DEV_EP_BUFFER_LIST) + NumberEntries*sizeof(ADI_USBD_DEV_EP_BUFFER*);
    uint8_t *pData = (uint8_t*)pDataMemory + ListSize;
    ADI_USBD_DEV_EP_BUFFER **pList;

    memset(pDataMemory, 0, ListSize);

    *ppEpBufferDef = (ADI_USBD_DEV_EP_BUFFER_LIST *)pDataMemory;
    (*ppEpBufferDef)->ppList = (ADI_USBD_DEV_EP_BUFFER **)&(((uint8_t*)pDataMemory)[sizeof(ADI_USBD_DEV_EP_BUFFER_LIST)]);
    (*ppEpBufferDef)->nMaxBuffers = NumberEntries;

    pList = (*ppEpBufferDef)->ppList;
    for (i=0; i<NumberEntries; i++)
    {
        pList[i] = (ADI_USBD_DEV_EP_BUFFER*)(pData + i*sizeof(ADI_USBD_DEV_EP_BUFFER));
    }

    adi_usbd_dev_EpResetBufferList(*ppEpBufferDef);
}

/*
 * @brief       The adi_usbd_dev_EpResetBufferList API resets the buffer table.
 *
 * @param[in]   pBufferList		Location of buffer table.
 *
 * @return		void.
 *
 */
void adi_usbd_dev_EpResetBufferList(
                    ADI_USBD_DEV_EP_BUFFER_LIST *pBufferList
)
{
    uint8_t i;
    for (i=0;i<(pBufferList->nMaxBuffers); i++)
    {
        pBufferList->ppList[i]->bInUse = false;
        pBufferList->ppList[i]->nBytesProcessed = 0;
        pBufferList->ppList[i]->nBytesThisPass = 0;
        pBufferList->ppList[i]->nBytesRemaining = 0;
        pBufferList->ppList[i]->pData = 0;
        pBufferList->ppList[i]->pNext = pBufferList->ppList[i+1];
    }
    pBufferList->ppList[(pBufferList->nMaxBuffers)-1]->pNext = pBufferList->ppList[0];

    pBufferList->pNextFreeBuffer = pBufferList->ppList[0];
    pBufferList->pNextQueued = 0;
}


/*
 * @brief       The adi_usbd_dev_EpQueueBuffer API places a buffer in the queue
 *
 * @param[in]   pBufferList		Location of buffer table.
 * @param[in]   pData.			Pointer to data buffer.
 * @param[in]   nTransferSize	length in bytes of data buffer.
 *
 * @return		The location of the buffer entry to which buffer has been assigned; NULL if no available entry.
 *
 */
ADI_USBD_DEV_EP_BUFFER* adi_usbd_dev_EpQueueBuffer(
            ADI_USBD_DEV_EP_BUFFER_LIST *pBufferList,
            void                        *pData,
            uint32_t                    nTransferSize
)
{
    ADI_USBD_DEV_EP_BUFFER* pBuffer;
    uint8_t i;
    pBuffer = pBufferList->pNextFreeBuffer;
    if (pBuffer == 0)
    {
        /* iterate list */
        for (i=0; i< pBufferList->nMaxBuffers; i++)
        {
            if (pBufferList->ppList[i]->pData == 0)
            {
                pBuffer = pBufferList->ppList[i];
                break;
            }
        }
    }

    if (pBuffer)
    {
        pBuffer->pData              = pData;
        pBuffer->nTransferSize      = nTransferSize;
        pBuffer->nBytesThisPass     = 0;
        pBuffer->nBytesProcessed    = 0;
        pBuffer->nBytesRemaining    = nTransferSize;

        /* Move the Next free pointer to the next buffer in chain if possible */
        pBufferList->pNextFreeBuffer = pBuffer->pNext;
        if ((pBufferList->pNextFreeBuffer) && (pBufferList->pNextFreeBuffer->pData))
        {
            pBufferList->pNextFreeBuffer = 0;
        }

        /* If no buffer already queued, then make this the Next Queued */
        if (!pBufferList->pNextQueued) {
            pBufferList->pNextQueued = pBuffer;
        }
    }

    return pBuffer;
}

/*
 * @brief       The adi_usbd_dev_EpFreeBuffer API frees a buffer entry for subsequent use
 *
 * @param[in]   pBufferList		Location of buffer table.
 * @param[in]   pBuffer.		Pointer to buffer entry to be freed.
 *
 * @return		void.
 *
 */
void adi_usbd_dev_EpFreeBuffer(
			 ADI_USBD_DEV_EP_BUFFER_LIST *pBufferList,
			 ADI_USBD_DEV_EP_BUFFER      *pBuffer
)
{
    /* set all to zero except pNext field Use flag to indicate free status
    */
    pBuffer->pData = 0;
    pBuffer->bInUse = false;
    pBuffer->nTransferSize = 0;
    pBuffer->nBytesProcessed = 0;
    pBuffer->nBytesRemaining = 0;

    /*
     * If next free pointer is unset, set to the newly freed buffer
     */
    if (!pBufferList->pNextFreeBuffer)
    {
        pBufferList->pNextFreeBuffer = pBuffer;
    }

    if (pBuffer == pBufferList->pNextQueued)
    {
        pBufferList->pNextQueued = 0;
    }

}

/*
 * @brief       The adi_usbd_dev_EpGetQueuedBuffer API retrieved the next ready buffer in the queue
 *
 * @param[in]   pBufferList		Location of buffer table.
 *
 * @return		The location of the buffer entry next in the queue; NULL if none queued
 *
 */
ADI_USBD_DEV_EP_BUFFER *adi_usbd_dev_EpGetQueuedBuffer(
                                 ADI_USBD_DEV_EP_BUFFER_LIST      *pBufferList
)
{
    uint8_t i;

    ADI_USBD_DEV_EP_BUFFER  *pBuffer = pBufferList->pNextQueued;
    if (!pBuffer)
    {
        /* iterate list */
        for (i=0; i< pBufferList->nMaxBuffers; i++)
        {
            if ((pBufferList->ppList[i]->pData != 0) && (!pBufferList->ppList[i]->bInUse))
            {
                pBuffer = pBufferList->ppList[i];
                pBuffer->bInUse = true;
                break;
            }
        }
    }


    return pBuffer;
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */
