/*********************************************************************************

Copyright(c) 2012-2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_usbd_dev_musbmhdrc_intrpt.c ($Rev: 21428 $)
*
* @brief     USB Device Driver (MUSBMHDRC) Interrupt handling.
*
* @details
*            This is a source file which contains the interrupt handlers for the
*            device driver for Mentor Graphics MUSBMHDRC Controller.
*
*/

#include "adi_usbd_dev_musbmhdrc_local.h"
#include <drivers/usb/controller/device/adi_usbd_dev_os.h>
#include <drivers/usb/controller/device/config/adi_usbd_dev_cfg.h>

#if defined(__ADSPBLACKFIN__) && !defined(WA_16000030)
#warning "WA_16000030 is not defined"
#endif


#ifdef _MISRA_RULES
#pragma diag(push)

#pragma diag(suppress:misra_rule_2_2:"")

#pragma diag(suppress:misra_rule_5_1:"")
#pragma diag(suppress:misra_rule_5_6:"")

#pragma diag(suppress:misra_rule_10_1_a:"The value of an expression of integer type shall not be implicitly converted to a different underlying type if it is not a conversion to a wider integer type of the same signedness.")
#pragma diag(suppress:misra_rule_10_1_c:"")
#pragma diag(suppress:misra_rule_10_5:"If the bitwise operators ~ and << are applied to an operand of underlying type unsigned char or unsigned short, the result shall be immediately cast to the underlying type of the operand.")

#pragma diag(suppress:misra_rule_11_3:"")
#pragma diag(suppress:misra_rule_11_4:"")
#pragma diag(suppress:misra_rule_11_5:"A cast should not be performed that removes any const or volatile qualification from the type addressed by a pointer.")

#pragma diag(suppress:misra_rule_12_5:"")
#pragma diag(suppress:misra_rule_12_6:"")
#pragma diag(suppress:misra_rule_12_7:"Bitwise operators shall not be applied to operands whose underlying type is signed")

#pragma diag(suppress:misra_rule_13_2:"")


#pragma diag(suppress:misra_rule_14_3:"Before preprocessing, a null statement shall only occur on a line by itself; it may be followed by a comment provided that the first character following the null statement is a white-space character.")
#pragma diag(suppress:misra_rule_14_10:"All if...else if constructs shall be terminated with an else clause.")

#pragma diag(suppress:misra_rule_16_7:"")
#pragma diag(suppress:misra_rule_16_10:"If a function returns error information, then that error information shall be tested.")

#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic.")
#pragma diag(suppress:misra_rule_19_14:"The defined preprocessor operator shall only be used in one of the two standard forms.")
#endif /* _MISRA_RULES */

static void ResumeHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);
static void SuspendHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);
static void DisconHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);
static void ResetHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);
static void SOFHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);

static void EP0InterruptHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t IntrTx, uint16_t *pIntrToDo);
static void DataEpRxHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t IntrRX, uint16_t IntrRXE, uint16_t *pIntrToDo);
static void DataEpTxHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t IntrTX, uint16_t IntrTXE, uint16_t *pIntrToDo);
static void DispatchIntrUSB(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_8BIT_REG_TYPE IntrUSB, ADI_USBD_8BIT_REG_TYPE IntrENA, ADI_USBD_8BIT_REG_TYPE *pIntrToDo);

static void DMAHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t IntrDMA, uint8_t *pIntrToDo);
#if (ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_NO)
static bool RxDmaHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef, bool bZLP);
static bool TxDmaHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef );
static void RestartDmaWithMode0(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);
static uint32_t ActualBytesProcessed(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);
#endif
static void ReadFifo(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef, uint32_t nBufSize);

void _adi_usbd_dev_USBinterruptHandler(
        uint32_t            iid,
        void*               handlerArg
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)handlerArg;

    ADI_USBD_8BIT_REG_TYPE IntrUSB, IntrENA, IntrToDo;
    uint16_t IntrRX, IntrRXE, IntrRxToDo;
    uint16_t IntrTX, IntrTXE, IntrTxToDo;
    volatile ADI_USBD_8BIT_REG_TYPE PrevIntrUSBToDo;
    volatile uint16_t PrevIntrRxToDo, PrevIntrTxToDo;
#if WA_16000030
    uint32_t intStatus;
#endif

    ADI_USBD_ENTER_CRITICAL();

#if WA_16000030
    intStatus = cli();
#endif

    READ_INT_REG(IntrUSB,pDevice,pIRQ,0xFF)
    IntrENA = GET_DEVICE_REG_VALUE(pDevice,pIEN,0xFF);

    READ_INT_REG(IntrTX,pDevice,pINTRTX,0xFFFF);
    IntrTXE = GET_DEVICE_REG_VALUE(pDevice,pINTRTXE,0xFFFF);

    READ_INT_REG(IntrRX,pDevice,pINTRRX,0xFFFF);
    IntrRXE = GET_DEVICE_REG_VALUE(pDevice,pINTRRXE,0xFFFF);

#if WA_16000030
    sti(intStatus);
#endif

    ADI_USBD_EXIT_CRITICAL();

    /* **************************************************************************
     * Bus Interrupts
    */
    if ( IntrUSB )
    {
        ADI_USBD_DBG("MUSBMHDRC IntrUSB");
        IntrToDo = IntrUSB&0x2F;
        while (IntrToDo)
        {
            PrevIntrUSBToDo = IntrToDo;
            DispatchIntrUSB(pDevice, PrevIntrUSBToDo, IntrENA, &IntrToDo);
        }
    }

    if ( IntrTX )
    {
        /* **************************************************************************
         * EP0 Interrupts
        */
        if ( IS_INTRPT(BITM_USB_INTRTX_EP0,IntrTX) )
        {
            PrevIntrTxToDo = IntrTX;
            EP0InterruptHandler(pDevice, PrevIntrTxToDo, &IntrTX);
        }
        /* **************************************************************************
         * TX Interrupts
        */
        if ( IS_INTRPT(0xFE,IntrTX) ) {
            IntrTxToDo = IntrTX;
            while (IntrTxToDo)
            {
                PrevIntrTxToDo = IntrTxToDo;
                DataEpTxHandler(pDevice, PrevIntrTxToDo, IntrTXE, &IntrTxToDo);
            }
        }
    }

    /* **************************************************************************
     * RX Interrupts
    */
    if (IntrRX && ((IntrRXE&IntrRX) != 0) )
    {
        IntrRxToDo = IntrRX;
        while (IntrRxToDo)
        {
            PrevIntrRxToDo = IntrRxToDo;
            DataEpRxHandler(pDevice, PrevIntrRxToDo, IntrRXE, &IntrRxToDo);
        }

    }

}

static void EP0InterruptHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t IntrTx, uint16_t *pIntrToDo)
{
    bool bMoreToDo;
    uint16_t RegCSR0;
    uint16_t RxCount;
    ADI_USBD_DEV_EP_DEF_EX *pEpDef = pDevice->pEp0Def;

    if (pEpDef) {

		ADI_USBD_DBG_EP("MUSBMHDRC EP 0 Interrupt",0);

		/* Disable EP0 interrupts
		*/
		ADI_USBD_ENTER_CRITICAL();

		*pDEVICE_REG(pDevice,pINDEX) = 0;
		RegCSR0 = GET_DEVICE_REG_VALUE(pDevice,pEP0_CSR0,0x01FF);
		bMoreToDo = 1;

		/* Page 118 of the MUSBMHDRC programmers guide dictates that if the current control
		 * transaction is aborted, either by a stall or by a premature end of the transfer,
		 * then we must abort the current transfer and return the state to IDLE.
		*/

		/* IF (StallSent set) */
		if ( (RegCSR0 & BITM_USB_EP0_CSR_P_SENTSTALL) == BITM_USB_EP0_CSR_P_SENTSTALL )
		{
			/* Clear stallsent bit */
			CLR_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_SENTSTALL );
			pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
			bMoreToDo = 0;
		}

		/* IF (SetupEnd set) */
		if ( (RegCSR0 & BITM_USB_EP0_CSR_P_SETUPEND) == BITM_USB_EP0_CSR_P_SETUPEND )
		{
			/* Set ServicedSetupEnd to clear SetupEnd bit */
			SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_SSETUPEND, ENUM_USB_EP0_CSR_P_SSETUPEND );
			pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
			bMoreToDo = 0;
		}

		if (bMoreToDo && pEpDef)
		{
			/* If an RX or TX transfer then adjust buffer counters */
			if ( (pEpDef->EpState == ADI_USBD_EP_STATE_RX) || (pEpDef->EpState == ADI_USBD_EP_STATE_TX))
			{
				USB_ASSERT((pEpDef->pCurrentBuffer==0), "pEpDef->pCurrentBuffer is NULL");

				if (pDevice->pEp0Def->EpState == ADI_USBD_EP_STATE_RX)
				{
					/*
					 * We need to unload the FIFO
					 */
					RxCount = adi_usbd_dev_musbhdrc_GetRxCount(pDevice, pEpDef);
					ADI_USBD_DBG_EP_ARG("MUSBMHDRC ADI_USBD_EP_STATE_RX",0,RxCount);

					ReadFifo(pDevice, pEpDef, RxCount);

				}
				else {
					pEpDef->pCurrentBuffer->nBytesRemaining -= pEpDef->pCurrentBuffer->nBytesThisPass;
					pEpDef->pCurrentBuffer->nBytesProcessed += pEpDef->pCurrentBuffer->nBytesThisPass;
					/* Transfer control to the EP0 state handler */
					adi_usbd_dev_musbmhdrc_Ep0StateHandler(pDevice, pEpDef);
				}
			}
			else {
				/* Transfer control to the EP0 state handler */
				adi_usbd_dev_musbmhdrc_Ep0StateHandler(pDevice, pEpDef);
			}
		}

		/* Enable EP0 interrupts
		*/
		ADI_USBD_EXIT_CRITICAL();
		*pIntrToDo &= ~BITM_USB_INTRTX_EP0;
    }
}


void _adi_usbd_dev_DMAinterruptHandler(
        uint32_t            iid,
        void*               handlerArg
)
{
#if (ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_NO)

    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)handlerArg;

    uint8_t IntrToDo, IntrDMA;
    volatile uint8_t PrevIntrToDo;
#if WA_16000030
    uint32_t intStatus;
#endif

    /*
    DMA Interrupts complicates things, as we need to identify the EP
    */
    ADI_USBD_ENTER_CRITICAL();

#if WA_16000030
    intStatus = cli();
#endif

    /* read the DMA_IRQ register */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    READ_INT_REG(IntrDMA,pDevice,pDMA_IRQ,0xFFFF);
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    IntrDMA = *pUSB_DMA_INTERRUPT;
#endif

#if WA_16000030
    sti(intStatus);
#endif

    /* For each channel */
    IntrToDo = IntrDMA;
    while (IntrToDo)
    {
        PrevIntrToDo = IntrToDo;
        DMAHandler(pDevice, PrevIntrToDo, (uint8_t*)&IntrToDo);
    }

    /* acknowledge all interrupts */
#if  (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    *pUSB_DMA_INTERRUPT = IntrDMA;
#endif
    ADI_USBD_EXIT_CRITICAL();
#endif
}

static void DispatchIntrUSB(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_8BIT_REG_TYPE IntrUSB, ADI_USBD_8BIT_REG_TYPE IntrENA, ADI_USBD_8BIT_REG_TYPE *pIntrToDo)
{
    if( IS_INTRPT(BITM_USB_IRQ_RESUME,IntrUSB) /*&& IS_INTRPT(BITM_USB_IRQ_RESUME,IntrENA)*/ )
    {
        /* Clear bit in bitmask */
        *pIntrToDo &= ~BITM_USB_IRQ_RESUME;
        if (IS_INTRPT(BITM_USB_IRQ_RESUME,IntrENA))
        {
            ResumeHandler(pDevice);
        }
    }
    else if( IS_INTRPT(BITM_USB_IRQ_SUSPEND,IntrUSB) /*&& IS_INTRPT(BITM_USB_IRQ_SUSPEND,IntrENA)*/ )
    {
        /* Clear bit in bitmask */
        *pIntrToDo &= ~BITM_USB_IRQ_SUSPEND;
        if (IS_INTRPT(BITM_USB_IRQ_SUSPEND,IntrENA))
        {
            SuspendHandler(pDevice);
        }
    }
    else if( IS_INTRPT(BITM_USB_IRQ_DISCON,IntrUSB) /*&& IS_INTRPT(BITM_USB_IRQ_DISCON,IntrENA)*/ )
    {
        /* Clear bit in bitmask */
        *pIntrToDo &= ~BITM_USB_IRQ_DISCON;
        if (IS_INTRPT(BITM_USB_IRQ_DISCON,IntrENA))
        {
            DisconHandler(pDevice);
        }
    }
    else if( IS_INTRPT(BITM_USB_IRQ_RSTBABBLE,IntrUSB) /*&& IS_INTRPT(BITM_USB_IRQ_RSTBABBLE,IntrENA)*/ )
    {
        /* Clear bit in bitmask */
        *pIntrToDo &= ~BITM_USB_IRQ_RSTBABBLE;
        if (IS_INTRPT(BITM_USB_IRQ_RSTBABBLE,IntrENA))
        {
            ResetHandler(pDevice);
        }
    }
    else if( IS_INTRPT(BITM_USB_IRQ_SOF,IntrUSB))
    {
        /* Clear bit in bitmask */
        *pIntrToDo &= ~BITM_USB_IRQ_SOF;
        if (IS_INTRPT(BITM_USB_IRQ_SOF,IntrENA))
        {
            SOFHandler(pDevice);
        }
    }
}



static void ResumeHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice)
{
    /* Call back */
    pDevice->pfBusCallback(pDevice->CallbackHandle, ADI_USBD_DEV_EVENT_BUS_RESUME, 0);
}

static void SuspendHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice)
{
    /* Call back */
    pDevice->pfBusCallback(pDevice->CallbackHandle, ADI_USBD_DEV_EVENT_BUS_SUSPEND, 0);
}

static void DisconHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice)
{
    /* Call back */
    pDevice->pfBusCallback(pDevice->CallbackHandle, ADI_USBD_DEV_EVENT_BUS_DISCONN, 0);
    adi_usbd_dev_musbmhdrc_DeviceModeResetToDefault(pDevice);
}

static void ResetHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice)
{
    /* Call back */
    /*
     * FIXIT: decide whether disabling USB interrupt over reset is required
     */
    ADI_USBD_DBG("MUSBMHDRC BUS RESET");
    pDevice->pfBusCallback(pDevice->CallbackHandle, ADI_USBD_DEV_EVENT_BUS_RESET, 0);
    adi_usbd_dev_musbmhdrc_DeviceModeResetToDefault(pDevice);

#if (ADI_USBD_DEV_HS_ENABLE == ADI_USBD_DEV_YES)
    /*
     * Enable the SOF interrupt. It will be used to determine when the controller
     * has been configured for HS mode.
     */
    SET_DEVICE_REG_VALUE(pDevice, pIEN, BITM_USB_IEN_SOF, ENUM_USB_IEN_SOFEN);
#endif
}


static void SOFHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice)
{
    ADI_USBD_DBG("MUSBMHDRC BUS SOF");
#if (ADI_USBD_DEV_HS_ENABLE == ADI_USBD_DEV_YES)

    /*
     * Check the power register for HS mode and send event accordingly
     */
    if (ENUM_USB_POWER_HSMODE == GET_DEVICE_REG_VALUE(pDevice, pPOWER, BITM_USB_POWER_HSMODE))
    {
        /* Pass the HS detected event to the USB stack */
        pDevice->pfBusCallback(pDevice->CallbackHandle, ADI_USBD_DEV_EVENT_BUS_HS_DETECTED, 0);

        /* Disable SOF interrupt */
        CLR_DEVICE_REG_VALUE(pDevice, pIEN, BITM_USB_IEN_SOF);
    }
#endif
    }

static void DataEpRxHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t IntrRX, uint16_t IntrRXE, uint16_t *pIntrToDo)
{
    uint8_t EpNum;
    ADI_USBD_DEV_EP_DEF_EX *pEpDef;
    uint16_t RxCount;

    /*
     * Determine EP number
    */
    for (EpNum=1;EpNum<ADI_USBD_DEV_MAX_EP_COUNT;EpNum++)
    {
         if ( (IntrRX &(1<<EpNum)) == (1<<EpNum))
         {
             *pIntrToDo &= ~(1<<EpNum);
             /*
              * Disable Interrupt
              */
             CLR_DEVICE_REG_VALUE(pDevice, pINTRRXE, (1<<EpNum) );

             /*
              * look up Ep definition
              */
             pEpDef = (ADI_USBD_DEV_EP_DEF_EX *)adi_usbd_dev_EpGetEntry(pDevice->pEpTable,EpNum);
             if (pEpDef)
             {
                 ADI_USBD_DBG_EP("MUSBMHDRC Rx Interrupt",pEpDef->basic.EpAddr);

                 /* Set INDEX register to this Ep Number */
                 *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
                 /*
                  * Check for stall sent bit
                 */
                 if ( GET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_SENTSTALL))
                 {
                     /* For BF54x/BF52x we need to write 1 to clear, otherwise the above read will
                      * have cleared it
                     */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
                     SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_SENTSTALL, ENUM_USB_EPI_RXCSR_P_STALSNT);
#endif
                 }

                 /*
                  * Read RXCOUNT register
                 */
                 RxCount = *pDEVICE_REG(pDevice,pEPI_RXCNT);

                 /* IF (RXCOUNT > 0) */
                 if (RxCount)
                 {
                     /* Interrupt recieved but no buffer supplied */
                     if (pEpDef->pCurrentBuffer == 0)
                     {
                         /* Check for a queued buffer */
                         pEpDef->pCurrentBuffer = adi_usbd_dev_EpGetQueuedBuffer(pEpDef->pBufferList);

                         /* Still no buffer so assert failure */
                         USB_ASSERT((pEpDef->pCurrentBuffer==0), "pEpDef->pCurrentBuffer is NULL");

                     }
                     if (pEpDef->pCurrentBuffer)
                     {
                         pEpDef->pCurrentBuffer->nBytesThisPass = RxCount;
                         ReadFifo(pDevice, pEpDef, RxCount);
                     }
                 }
                 /* ELSE (RXCOUNT == 0) */
                 else {
                    /* Zero Length Packet received  */

                    /* Clear RxPktRdy */
#if (ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_NO) && (ADI_USBD_DEV_RX_DMA_MODE0 == ADI_USBD_DEV_NO)
                    if (pEpDef->TransferMode == ADI_USBD_DEV_EP_MODE_DMA_1)
                    {
                        /* free DMA Channel */
                        adi_usbd_dev_musbmhdrc_FreeDMAChannel(pDevice, pEpDef->DMAChannel);
                        /* Process as DMA interrupt */
                        RxDmaHandler(pDevice, pEpDef, true);
                    }
                    else
#endif
                    {
                        if (pEpDef->pCurrentBuffer)
                        {
                            /* ZLP indicates end of transfer */
                            pEpDef->pCurrentBuffer->nBytesThisPass = 0;
                            pEpDef->pCurrentBuffer->nBytesRemaining = 0;

                            /* Process State */
                            pEpDef->pfEpStateHandler(pDevice, pEpDef);
                        }
                        adi_usbd_dev_musbmhdrc_ClearRxPktRdy(pDevice, pEpDef);
                    }
                 }
             /* ENDIF (RXCOUNT) */
             }
             /* ELSE (EP Info not located) */
             else {
                 /* FIXIT: This should never happen - add assert()? */
             /* ENDIF (EP Info located) */
             }
         /* END IF (interrupt found) */
         }
    /* END FOR */
    }
}

static void DataEpTxHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t IntrTX, uint16_t IntrTXE, uint16_t *pIntrToDo)
{
    uint8_t EpNum, LogEpNum;
    ADI_USBD_DEV_EP_DEF_EX *pEpDef;
    /*
    * Determine EP number
    */
    for (EpNum=1;EpNum<ADI_USBD_DEV_MAX_EP_COUNT;EpNum++)
    {
         if ( (IntrTX &(1<<EpNum)) == (1<<EpNum))
         {
             *pIntrToDo &= ~(1<<EpNum);

             /*
              * look up Ep definition
              */
             LogEpNum = EpNum | 0x80;
             pEpDef = (ADI_USBD_DEV_EP_DEF_EX *)adi_usbd_dev_EpGetEntry(pDevice->pEpTable,LogEpNum);
             if (pEpDef)
             {
                if (pEpDef->pCurrentBuffer)
                {
                     ADI_USBD_DBG_EP("MUSBMHDRC Tx Interrupt",pEpDef->basic.EpAddr);

                     pEpDef->pCurrentBuffer->nBytesRemaining -= pEpDef->pCurrentBuffer->nBytesThisPass;
                     pEpDef->pCurrentBuffer->nBytesProcessed += pEpDef->pCurrentBuffer->nBytesThisPass;

                     /* Set INDEX register to this Ep Number */
                     *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
                     /*
                      * Check for stall sent bit
                     */
                     if ( GET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_SENTSTALL))
                     {
                         /* For BF54x/BF52x we need to write 1 to clear, otherwise the above read will
                          * have cleared it
                         */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
                         SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_TXCSR_P_SENTSTALL, ENUM_USB_EPI_TXCSR_P_STALSNT);
#endif
                     }
                    pEpDef->pfEpStateHandler(pDevice, pEpDef);
                }
                else {
                    /* ZLP - simply issue TX complete callback event */
                    pEpDef->pfCallback(pEpDef->CallbackHandle, ADI_USBD_DEV_EVENT_TX, 0);
                }
             }
         }
    }
}


static void DMAHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t IntrDMA, uint8_t *pIntrToDo)
{
#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
    uint8_t DMAChannel, EpNum, EpDir, LogEpNum;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;
    bool bReleaseDMA;

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    /* Determine Endpoint number, direction and DMA mode */
    DMAChannel  = adi_usbd_dev_musbmhdrc_DetermineDMAChannel(pDevice, IntrDMA);
    EpNum       = (GET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_EP))>>BITP_USB_DMA_CTL_EP;
    EpDir       = (GET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_DIR));

    /* remove from mask of interrupts to process */
    *pIntrToDo &= ~pDevice->pDMAChannels[DMAChannel].IRQBitmask;

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    /* Determine Endpoint number, direction and DMA mode */
    unsigned short DMAControl=0;
    DMAChannel  = adi_usbd_dev_musbmhdrc_DetermineDMAChannel(pDevice, IntrDMA);
    DMAControl = adi_usbd_dev_get_dma_ctrl(DMAChannel);
    EpNum = (DMAControl & 0x00F0) >> 4;
    EpDir = DMAControl & 0x0002;
    /* remove from mask of interrupts to process */
    *pIntrToDo &= ~(1ul << DMAChannel);
#endif

    /* Assign Logical EP address */
    if (EpDir == ENUM_USB_DMA_CTL_DMADIR_TX)
    {
        LogEpNum = 0x80 | EpNum;
    }
    else {
        LogEpNum = EpNum;
    }
    /* Assign EP definition structure
    */
    if (EpNum == 0) {
        pEpDef = pDevice->pEp0Def;
    } else {
        pEpDef = (ADI_USBD_DEV_EP_DEF_EX *)adi_usbd_dev_EpGetEntry(pDevice->pEpTable,LogEpNum);
    }

    if (EpDir == ENUM_USB_DMA_CTL_DMADIR_TX)
    {
        bReleaseDMA = TxDmaHandler(pDevice, pEpDef);
    }
    else {
        bReleaseDMA = RxDmaHandler(pDevice, pEpDef, false);
    }

    /* Free DMA channel */
    if (bReleaseDMA) {
        adi_usbd_dev_musbmhdrc_FreeDMAChannel(pDevice, DMAChannel);
    }

#endif
}

#if (ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_NO)

static bool TxDmaHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef )
{
    bool bReleaseDMA = true;
    bool bTxPktRdySet = true;
    uint32_t nBytesProcessed;

    ADI_USBD_DBG_EP_ARG("MUSBMHDRC DMA Tx Complete",pEpDef->basic.EpAddr, pEpDef->pCurrentBuffer->nTransferSize);

    /* IF (DMA Mode 0) */
    if (pEpDef->TransferMode == ADI_USBD_DEV_EP_MODE_DMA_0)
    {
        /* IF (Control Endpoint) */
        if (pEpDef->EpNum == 0)
        {
            /*
             * For EP0 & DMA Mode 0, this will either be a full packet as part of multi-packet
             * control transfer, or the final short packet. In the case of short packet,
             * setting of TxPktRdy will be deferred to the status phase.
             */
            bTxPktRdySet = adi_usbd_dev_musbmhdrc_SetTxPktRdy(pDevice, pEpDef);
            if (!bTxPktRdySet) {
                /*
                 * With EP 0 last TX packet, we cannot set TxPktRdy until the status phase
                 */
                pEpDef->pfEpStateHandler(pDevice, pEpDef);
            }
        }
        /* ELSE (Data Endpoint) */
        else {
            /* Set TxPktRdy */
            adi_usbd_dev_musbmhdrc_SetTxPktRdy(pDevice, pEpDef);
            /* Since the state handler will submit another DMA transfer, we need to free
             * the channel now rather than later.
            */
            if (pEpDef->pCurrentBuffer->nBytesRemaining > pEpDef->pCurrentBuffer->nBytesThisPass )
            {
                adi_usbd_dev_musbmhdrc_FreeDMAChannel(pDevice, pEpDef->DMAChannel);
                bReleaseDMA = false;
            }
            adi_usbd_dev_musbhdrc_EnableTxInterrupt(pDevice, pEpDef);

            /* Command flow transfered to TX Interrupt Handler */
        }
        /* ENDIF */
    }
    /* ELSE (DMA Mode 1) */
    else {
        /*
         * check for short packet. If so, then FIFO loaded using DMA Mode 0
         */
        nBytesProcessed = ActualBytesProcessed(pDevice,pEpDef);
        pEpDef->pCurrentBuffer->nBytesRemaining -= nBytesProcessed;
        pEpDef->pCurrentBuffer->nBytesProcessed += nBytesProcessed;

        /* IF (short packet remaining) */
        if (pEpDef->pCurrentBuffer->nBytesRemaining)
        {
            /*
             * For short packet at end we need to process as if single packet
             */
            pEpDef->pCurrentBuffer->nBytesThisPass = pEpDef->pCurrentBuffer->nBytesRemaining;

            /* Transfer remaining packet with DMA Mode 0 */
            RestartDmaWithMode0(pDevice,pEpDef);
            bReleaseDMA = false;

            /* Command flow transferred to DMA Interrupt handler */
        }
        /* ELSE (Transfer Complete) */
        else {
            /* Command flow transferred to State handler */
            pEpDef->pfEpStateHandler(pDevice, pEpDef);
        }
        /* ENDIF */
    }
    /* ENDIF */
    return bReleaseDMA;
}

static bool RxDmaHandler(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef, bool bZLP)
{
    bool bReleaseDMA = true;
    bool bShortPacket = false;

    uint32_t nBytesProcessed = ActualBytesProcessed(pDevice,pEpDef);

    bShortPacket = (nBytesProcessed % pEpDef->nMaxPktSize > 0);

    if (bZLP || bShortPacket) {
        pEpDef->pCurrentBuffer->nBytesRemaining = 0;
    }
    else {
        pEpDef->pCurrentBuffer->nBytesRemaining -= nBytesProcessed;
    }
    pEpDef->pCurrentBuffer->nBytesProcessed += nBytesProcessed;

    ADI_USBD_DBG_EP_ARG("MUSBMHDRC DMA Rx Complete",pEpDef->basic.EpAddr, pEpDef->pCurrentBuffer->nBytesThisPass);

    if (pEpDef->pCurrentBuffer->nBytesRemaining )
    {
        adi_usbd_dev_musbmhdrc_FreeDMAChannel(pDevice, pEpDef->DMAChannel);
        bReleaseDMA = false;
    }
    /* Process State */
    pEpDef->pfEpStateHandler(pDevice, pEpDef);

    /*clear RxPktRdy */
    adi_usbd_dev_musbmhdrc_ClearRxPktRdy(pDevice, pEpDef);

    return bReleaseDMA;
}

static void RestartDmaWithMode0(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    /* Set up endpoint */
    pEpDef->bConfigured = false;
    pEpDef->TransferMode = ADI_USBD_DEV_EP_MODE_DMA_0;
    adi_usbd_dev_musbmhdrc_SetupEndPoint(pDevice,pEpDef);

    /* Disable DMA */
    CLR_DEVICE_REG_VALUE(pDevice, pDMA_CTL[pEpDef->DMAChannel], BITM_USB_DMA_CTL_EN);

    /* Change mode to DMA Mode 0 */
    CLR_DEVICE_REG_VALUE(pDevice, pDMA_CTL[pEpDef->DMAChannel], BITM_USB_DMA_CTL_MODE );

    /* Reset DMA Count to bytes this pass */
    *pDevice->pRegs[pDevice->DeviceNumber].pDMA_CNT[pEpDef->DMAChannel] = pEpDef->pCurrentBuffer->nBytesThisPass;

    /* Enable DMA */
    SET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[pEpDef->DMAChannel], BITM_USB_DMA_CTL_EN, ENUM_USB_DMA_CTL_DMAEN );

    /* Command flow transferred to DMA Interrupt handler */
#endif
}

static uint32_t ActualBytesProcessed(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
    uint32_t StartAddress = (uint32_t)(&((uint8_t*)pEpDef->pCurrentBuffer->pData)[pEpDef->pCurrentBuffer->nBytesProcessed]);
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    uint32_t EndAddress   = (uint32_t)*pDevice->pRegs[pDevice->DeviceNumber].pDMA_ADDR[pEpDef->DMAChannel];
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    uint32_t EndAddress;
    adi_usbd_dev_get_dma_addr(pEpDef->EpNum,&EndAddress);
#endif
    return (EndAddress - StartAddress);
}
#endif

/*
 * ReadFifo()
 *
 * Unloads the FIFO
 */
static void ReadFifo(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef, uint32_t nBufSize)
{
    //uint32_t nBytesProcessed;

    /* If we receive a shorter packet than queued, adjust bytes remaining */
    if (   (pEpDef->pCurrentBuffer->nBytesRemaining <= pEpDef->nMaxPktSize)
        && (pEpDef->pCurrentBuffer->nBytesThisPass  < pEpDef->pCurrentBuffer->nBytesRemaining))
    {
        pEpDef->pCurrentBuffer->nBytesRemaining = (uint32_t)pEpDef->pCurrentBuffer->nBytesThisPass;
    }

    adi_usbd_dev_musbmhdrc_SetTransferMode(pDevice, pEpDef, ADI_USBD_DEV_EP_DIR_RX);
    pEpDef->pCurrentBuffer->nBytesThisPass = nBufSize;

    switch(pEpDef->TransferMode)
    {
        case ADI_USBD_DEV_EP_MODE_PIO:
        {
            /* Unload FIFO */
            adi_usbd_dev_musbmhdrc_UnloadFifoPIO(pDevice, pEpDef);

            if (pEpDef->pCurrentBuffer->nBytesThisPass < pEpDef->nMaxPktSize)
            {
                pEpDef->pCurrentBuffer->nBytesRemaining = 0;
            }
            else {
                pEpDef->pCurrentBuffer->nBytesRemaining -= pEpDef->pCurrentBuffer->nBytesThisPass;
            }
            pEpDef->pCurrentBuffer->nBytesProcessed += pEpDef->pCurrentBuffer->nBytesThisPass;

            /* Process State */
            pEpDef->pfEpStateHandler(pDevice, pEpDef);

            /* Clear RxPktRdy */
            adi_usbd_dev_musbmhdrc_ClearRxPktRdy(pDevice, pEpDef);
        }
        break;

#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
    case ADI_USBD_DEV_EP_MODE_DMA_0:
        /* Unload the FIFO */
        adi_usbd_dev_musbmhdrc_ScheduleDMA(pDevice, pEpDef, ADI_USBD_DMA_MODE_0);
        break;

    case ADI_USBD_DEV_EP_MODE_DMA_1:
        /* This should never happen according to the Mentor PG, but does happen with
        * BF527/BF548, and now with BF609
        */

        /* For DMA Mode 1 we are here because the received data is
         * (a) less than that queued, and
         * (b) comprises a short packet to terminate
         * We need to updated the Processed byte count to what has already been transfered
        */
        //nBytesProcessed = ActualBytesProcessed(pDevice,pEpDef);
        //pEpDef->pCurrentBuffer->nBytesProcessed += nBytesProcessed;
        //pEpDef->pCurrentBuffer->nBytesRemaining -= nBytesProcessed;

        /*
        * DMA Count register holds size of remaining short packet and
        * DMA Address register holds target address for short packet.
        * Simply change to DMA Mode 0 and restart
        * This should be the same as RXCOUNT, so in case of host sending less than queued, change
        * the DMA count to RXCOUNT
        */

        /* Transfer remaining packet with DMA Mode 0 */
        RestartDmaWithMode0(pDevice,pEpDef);

        /* command flow is transfered to DMA Interrupt Handler */
        break;
#endif
    default:
     break;
    /* END SWITCH */
    }
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */
