/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_usbd_dev_.c ($Rev: 18528 $)
*
* @brief     USB Device Driver (MUSBMHDRC) global API implementation.
*
* @details
*            This is a source file which contains the implementation of the global
*            API functions for the device driver for Mentor Graphics MUSBMHDRC
*            Controller.
*/

#include "adi_usbd_dev_musbmhdrc_local.h"
#include <drivers/usb/controller/device/adi_usbd_dev_os.h>

/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"")
#pragma diag(suppress:misra_rule_5_6:"")

#pragma diag(suppress:misra_rule_10_1_a:"The value of an expression of integer type shall not be implicitly converted to a different underlying type if it is not a conversion to a wider integer type of the same signedness.")
#pragma diag(suppress:misra_rule_10_1_b:"")
#pragma diag(suppress:misra_rule_10_5:"If the bitwise operators ~ and << are applied to an operand of underlying type unsigned char or unsigned short, the result shall be immediately cast to the underlying type of the operand.")

#pragma diag(suppress:misra_rule_11_4:"")

#pragma diag(suppress:misra_rule_12_4:"The right-hand operand of a logical && or || operator shall not contain side effects. (Functions with no side effects should be declared with #pragma pure or #pragma const")
#pragma diag(suppress:misra_rule_12_6:"")
#pragma diag(suppress:misra_rule_12_7:"Bitwise operators shall not be applied to operands whose underlying type is signed")

#pragma diag(suppress:misra_rule_13_2:"")

#pragma diag(suppress:misra_rule_14_3:"Before preprocessing, a null statement shall only occur on a line by itself; it may be followed by a comment provided that the first character following the null statement is a white-space character.")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#pragma diag(suppress:misra_rule_14_9:"")
#pragma diag(suppress:misra_rule_14_10:"All if...else if constructs shall be terminated with an else clause.")

#pragma diag(suppress:misra_rule_16_2:"Functions shall not call themselves, either directly or indirectly.")
#pragma diag(suppress:misra_rule_16_7:"")

#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic.")

#pragma diag(suppress:misra_rule_6_1:"The plain char type shall be only used for storage and use of character values.")


#endif /* _MISRA_RULES */


#define _QUICK_FIX_FOR_TX_STATE_MODEL_


#if defined(_ADI_USBD_DEV_ENABLE_TEST_MODE_)
static void adi_usbd_dev_testmode(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,void *pArg);
static void CheckForTestMode(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t *pSetupPacket);
#endif

#ifdef ADI_DEBUG
#include "stdio.h"
void __adi_usbd_dev_assert(char *msg, int line, char *file)
{
    printf("%s at line %d in file %s\n", msg, line, file);
}
#endif

void adi_usbd_dev_musbmhdrc_DeviceModeResetToDefault(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice)
{
    uint8_t IntrUSB;
    uint16_t IntrRX, IntrTX;

    /* Set Index to 0 */
    SET_EP_INDEX(pDevice, 0);

    /* Reset function address */
    *pDEVICE_REG(pDevice,pFADDR) = 0;

    /* Flush FIFOs (twice in case of double-buffering) */
    SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0I_CSR_P_FLUSHFIFO, ENUM_USB_EP0I_CSR_P_FLUSH );
    SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0I_CSR_P_FLUSHFIFO, ENUM_USB_EP0I_CSR_P_FLUSH );

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
    /*
     * Reset FIFO start address - this function gets called during connect and disconnet phases so
     * we always start with base start address
     */
    pDevice->FifoRamAdress    = USB_DYNFIFO_START_ADDR;
#endif

    /* Reset EP0 CSR to zero */
    CLR_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, 0xFFFF );

    /* Acknowledge any previously pending interrupts */

    READ_INT_REG(IntrUSB,pDevice,pIRQ,0xFF);
    (void)IntrUSB;
    READ_INT_REG(IntrTX,pDevice,pINTRTX,0xFFFF);
    (void)IntrTX;
    if (IntrTX && IS_INTRPT(BITM_USB_INTRTX_EP0,IntrTX))
    {
        ADI_USBD_DBG_EP("MUSBMHDRC EP 0 Interrupt detected in reset action",0);
    }
    READ_INT_REG(IntrRX,pDevice,pINTRRX,0xFFFF);
    (void)IntrRX;

#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
    /* Initialize DMA channels */
    adi_usbd_dev_musbmhdrc_InitDMAChannels(pDevice);
#endif

    /* Enable Reset and other connection interrupts */
    SET_DEVICE_REG_VALUE(pDevice, pIEN,
                         (BITM_USB_IEN_RSTBABBLE|BITM_USB_IEN_DISCON|BITM_USB_IEN_SUSPEND),
                         (ENUM_USB_IEN_RSTBABBLEEN|ENUM_USB_IEN_DISCONEN|ENUM_USB_IEN_SUSPENDEN)
                         );

    /* Disable all interrupts except EP0 */
    CLR_DEVICE_REG_VALUE(pDevice, pINTRTXE, 0xFFFF );
    SET_DEVICE_REG_VALUE(pDevice, pINTRTXE, BITM_USB_INTRTXE_EP0, BITM_USB_INTRTXE_EP0);

    /* Disable all Rx Interrupts */
    CLR_DEVICE_REG_VALUE(pDevice, pINTRRXE, 0xFFFF );

    /* Set device state */
    pDevice->DevState = ADI_USBD_DEV_STATE_AWAIT_ENUM;
}

void adi_usbd_dev_musbmhdrc_FlushEndpoint (
                    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                    ADI_USBD_DEV_EP_DEF_EX     *pEpDef
)
{
    /* Set Index Register with required EP Number */
    SET_EP_INDEX(pDevice, pEpDef->EpNum);

    if (pEpDef->EpNum == 0) {
        SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_FLUSHFIFO, ENUM_USB_EP0_CSR_P_FLUSH);
        SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_FLUSHFIFO, ENUM_USB_EP0_CSR_P_FLUSH);
    }
    else if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_TX)
    {
        /* Flush FIFO (twice in case of double buffering */
        SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_FLUSHFIFO, ENUM_USB_EPI_TXCSR_P_FLUSH);
        SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_FLUSHFIFO, ENUM_USB_EPI_TXCSR_P_FLUSH);
        SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_CLRDATATGL, ENUM_USB_EPI_TXCSR_P_CLRTGL);
    }
    else {
        /* Flush FIFO (twice in case of double buffering */
        SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_FLUSHFIFO, ENUM_USB_EPI_RXCSR_P_FLUSH);
        SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_FLUSHFIFO, ENUM_USB_EPI_RXCSR_P_FLUSH);
        SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_CLRDATATGL, ENUM_USB_EPI_RXCSR_P_CLRTGL);
    }
}

void adi_usbd_dev_musbmhdrc_ResetEndpoint (
                    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                    ADI_USBD_DEV_EP_DEF_EX     *pEpDef
)
{
    ADI_USBD_ENTER_CRITICAL();
    adi_usbd_dev_musbmhdrc_FlushEndpoint(pDevice,pEpDef);

    pEpDef->basic.EpAddr = 0;
    pEpDef->basic.bInUse = 0;
    pEpDef->basic.pClient = 0;

    pEpDef->pCurrentBuffer = 0;
    pEpDef->EpType = ADI_USBD_DEV_EP_TYPE_INVALID;
    pEpDef->nMaxPktSize = 0;
    pEpDef->nHighCapacityMultiplier = 0;
    pEpDef->DMAChannel = 0;
    pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
    pEpDef->NextEpState = ADI_USBD_EP_STATE_IDLE;
    pEpDef->pfEpStateHandler = 0;
    pEpDef->pfCallback = 0;
    pEpDef->CallbackHandle = 0;
    pEpDef->bConfigured = false;

    adi_usbd_dev_EpResetBufferList(pEpDef->pBufferList);
    ADI_USBD_EXIT_CRITICAL();
}

bool adi_usbd_dev_musbmhdrc_SetTxPktRdy(
                    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                    ADI_USBD_DEV_EP_DEF_EX      *pEpDef
)
{
    static uint8_t ShortPktEntryCount=0;
    volatile uint16_t TxCSR;

    bool bTxPktRdySet = true;
    /*
     * IF Control TX Packet in FIFO
     */
    *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;

    if (pEpDef->EpNum == 0)
    {
        /*
         * IF Full packet so just set TxPktRdy
         *     (USB stack must make call to EpWriteZLP() after last full packet).
         */
        if (pEpDef->pCurrentBuffer->nBytesThisPass == pEpDef->nMaxPktSize)
        {
            ADI_USBD_DBG_EP("MUSBMHDRC Set TxPktRdy",pEpDef->basic.EpAddr);

            SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_TXPKTRDY, ENUM_USB_EP0_CSR_P_TXPKTRDY );
        }
        /*
         * ELSE IF Short packet, then this is the last packet in the transfer, so set both
         *    TxPktRdy & DataEnd Bits, but defer to status stage (EntryCount==1)
         */
        else if (ShortPktEntryCount==1)
        {
            ShortPktEntryCount--;
            ADI_USBD_DBG_EP("MUSBMHDRC Set TxPktRdy & DataEnd",pEpDef->basic.EpAddr);
            SET_DEVICE_REG_VALUE(   pDevice,
                                    pEP0_CSR0,
                                    ( BITM_USB_EP0_CSR_P_TXPKTRDY | BITM_USB_EP0_CSR_P_DATAEND),
                                    ( ENUM_USB_EP0_CSR_P_TXPKTRDY | ENUM_USB_EP0_CSR_P_DATAEND)
                                );
        }
        else {
            ShortPktEntryCount++;
            bTxPktRdySet = false;
        }
    }
    /*
     * ELSE Data endpoint
     */
    else {
        TxCSR = GET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_URUNERR);

        SET_DEVICE_REG_VALUE(pDevice, pINTRTXE, (1<<(pEpDef->EpNum)), (1<<(pEpDef->EpNum)) );
        SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_TXPKTRDY, ENUM_USB_EPI_TXCSR_P_PKTRDY );
    }

    return bTxPktRdySet;
}


void adi_usbd_dev_musbmhdrc_ClearRxPktRdy(
                    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                    ADI_USBD_DEV_EP_DEF_EX      *pEpDef
)
{
    *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;

    /*
     * For EP 0 we need to set serviced packet ready
     */
    if (pEpDef->EpNum == 0) {
        SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_SPKTRDY, ENUM_USB_EP0_CSR_P_SPKTRDY );
    }
    /*
     * For other EPs we need to clear RxPktRdy
     */
    else {
        if (GET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_RXPKTRDY))
        {
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_RXPKTRDY);
        }
    }
}


void adi_usbd_dev_musbmhdrc_DataEpStateHandler( void *hDevice, void *hEndpoint)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice         = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef          = (ADI_USBD_DEV_EP_DEF_EX *)hEndpoint;
    ADI_USBD_DEV_CB_FN         EpCallback       = pEpDef->pfCallback;
    void                       *EpCallbackParam = pEpDef->CallbackHandle;
    uint32_t                   nBytesThisPass   = 0;

    switch (pEpDef->EpState)
    {
        case ADI_USBD_EP_STATE_IDLE:
            break;

        case ADI_USBD_EP_STATE_RX:
            if ((pEpDef->pCurrentBuffer))
            {
                nBytesThisPass = adi_usbd_dev_musbmhdrc_ReceiveDataBuffer(pDevice, pEpDef);

                /* IF (no more data to send) */
                if (nBytesThisPass == 0)
                {
                    /* Set EP values that persist after current buffer is changed */
                    pEpDef->nBytesProcessed      = pEpDef->pCurrentBuffer->nBytesProcessed;
                    pEpDef->pProcessedDataBuffer = pEpDef->pCurrentBuffer->pData;

                    /* Execute the Endpoint Callback */
                    EpCallback(EpCallbackParam, ADI_USBD_DEV_EVENT_RX, (void*)pEpDef->pCurrentBuffer);
                    pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;

                    /* Free current buffer and schedule the next if available */
                    adi_usbd_dev_EpFreeBuffer(pEpDef->pBufferList, pEpDef->pCurrentBuffer);
                    pEpDef->pCurrentBuffer = adi_usbd_dev_EpGetQueuedBuffer(pEpDef->pBufferList);
                    if (pEpDef->pCurrentBuffer) {
                        adi_usbd_dev_musbmhdrc_ReceiveDataBuffer(pDevice, pEpDef);
                    }
                }
            }
            break;

        case ADI_USBD_EP_STATE_RX_STATUS:
            EpCallback(EpCallbackParam, ADI_USBD_DEV_EVENT_RX, (void*)0);
            pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
            break;

        case ADI_USBD_EP_STATE_TX:
            if ((pEpDef->pCurrentBuffer))
            {
                nBytesThisPass = adi_usbd_dev_musbmhdrc_TransmitDataBuffer(pDevice, pEpDef);

                /* IF (no more data to send) */
                if (nBytesThisPass == 0)
                {
                    /* Set EP values that persist after current buffer is changed */
                    pEpDef->nBytesProcessed      = pEpDef->pCurrentBuffer->nBytesProcessed;
                    pEpDef->pProcessedDataBuffer = pEpDef->pCurrentBuffer->pData;

                    /* Execute the Endpoint Callback */
                    EpCallback(EpCallbackParam, ADI_USBD_DEV_EVENT_TX, (void*)pEpDef->pCurrentBuffer);
                    pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
                     if(pEpDef->EpNum != 0)
                           adi_usbd_dev_musbhdrc_DisableTxInterrupt(pDevice, pEpDef);

                    /* Free current buffer and schedule the next if available */
                    adi_usbd_dev_EpFreeBuffer(pEpDef->pBufferList, pEpDef->pCurrentBuffer);
                    pEpDef->pCurrentBuffer = adi_usbd_dev_EpGetQueuedBuffer(pEpDef->pBufferList);
                    if (pEpDef->pCurrentBuffer) {
                        adi_usbd_dev_musbmhdrc_TransmitDataBuffer(pDevice, pEpDef);
                    }
                }
            }
            break;

        default:
            break;
    }

}


void adi_usbd_dev_musbmhdrc_Ep0StateHandler( void *hDevice, void *hEndpoint)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef  = (ADI_USBD_DEV_EP_DEF_EX *)hEndpoint;

    ADI_USBD_DEV_CB_FN  Ep0Callback       = pEpDef->pfCallback;
    void                *Ep0CallbackParam = pEpDef->CallbackHandle;
    uint16_t            RxCount;

    switch (pEpDef->EpState)
    {
        case ADI_USBD_EP_STATE_IDLE:
        {
            *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
            /* IF (RxPktRdy set ) */
            if ( GET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_RXPKTRDY ))
            {
                RxCount = *pDevice->pRegs[pDevice->DeviceNumber].pEP0_CNT0;
                if (RxCount == 0 )
                {
                     SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_SPKTRDY, ENUM_USB_EP0_CSR_P_SPKTRDY );
                     break;
                }
                /* Read Setup packet from FIFO */
                adi_usbd_dev_musbmhdrc_ReadFifo(
                                                    (ADI_USBD_DEV_FIFO_ENTRY *)pDevice->pRegs[pDevice->DeviceNumber].pEP_FIFO[0],
                                                    (ADI_USBD_DEV_FIFO_ENTRY *)pDevice->SetupPacket,
                                                    8
                                   );
                /*
                 * Tell controller that we have read the packet
                 */

                SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_SPKTRDY, ENUM_USB_EP0_CSR_P_SPKTRDY );
                ADI_USBD_DBG_EP_ARG("MUSBMHDRC SETUP INT ",0,pDevice->SetupPacket[1]);
                /*
                 * IF ( data phase ) then move to RX or TX state
                 */
                if (GET_SETUP_RQ_LEN(&pDevice->SetupPacket[0]))
                {
                    /* IF (Device to Host) then set state to TX */
                    if (GET_SETUP_RQ_DIR(&pDevice->SetupPacket[0])==0x80)
                    {
#if !defined(_QUICK_FIX_FOR_TX_STATE_MODEL_)
/* FIXIT need to review EP0 state model handling */
                        pEpDef->EpState = ADI_USBD_EP_STATE_TX;
#endif
                        pEpDef->EpDir   = ADI_USBD_DEV_EP_DIR_TX;
                    }
                    /* ELSE (Host to Device) then set state to RX */
                    else {
                        pEpDef->EpState = ADI_USBD_EP_STATE_RX;
                        pEpDef->EpDir   = ADI_USBD_DEV_EP_DIR_RX;
                        adi_usbd_dev_musbhdrc_DisableTxInterrupt(pDevice,pEpDef);
                        /* disable EP0 Interrupt */
                    }
                    /*
                     * we loop round to issue the EP0 callback in TX or RX state
                     */
                }
                else {
                        pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
                }

                /*
                 * Pass packet to stack for processing
                 */
#if defined(_ADI_USBD_DEV_ENABLE_TEST_MODE_)
                CheckForTestMode(pDevice,&pDevice->SetupPacket[0]);
#endif
                Ep0Callback(Ep0CallbackParam, ADI_USBD_DEV_EVENT_SETUP, &pDevice->SetupPacket[0]);

            }
        }
            break;

        /*
         * CASE (TX state) - we arrive here because of the TX_INT after TxPktRdy has been cleared, therefore we should
         * send the ADI_USBD_DEV_EVENT_TX event to the stack.
         */
        case ADI_USBD_EP_STATE_TX:

                ADI_USBD_DBG("MUSBMHDRC ADI_USBD_EP_STATE_TX");

                Ep0Callback(Ep0CallbackParam,ADI_USBD_DEV_EVENT_TX,(void*)pEpDef->pCurrentBuffer);

                if (pEpDef->pCurrentBuffer->nBytesRemaining == 0)
                {
                    if (pEpDef->pCurrentBuffer) {
                        ADI_USBD_ENTER_CRITICAL();
                        adi_usbd_dev_EpFreeBuffer(pEpDef->pBufferList, pEpDef->pCurrentBuffer);
                        pEpDef->pCurrentBuffer = 0;
#if defined(_QUICK_FIX_FOR_TX_STATE_MODEL_)
                        pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
#endif
                        ADI_USBD_EXIT_CRITICAL();
                    }
                }
                break;

        case ADI_USBD_EP_STATE_TX_PKT_SENT:
            pEpDef->EpState = pEpDef->NextEpState;
            adi_usbd_dev_musbmhdrc_Ep0StateHandler(pDevice,pEpDef);
            break;

        case ADI_USBD_EP_STATE_TX_STATUS:

            ADI_USBD_DBG("MUSBMHDRC ADI_USBD_EP_STATE_TX_STATUS");
            Ep0Callback(Ep0CallbackParam,ADI_USBD_DEV_EVENT_TX,(void*)pEpDef->pCurrentBuffer);
            /*
             * Release Buffer
             */
            ADI_USBD_ENTER_CRITICAL();
            if (pEpDef->pCurrentBuffer) {
                adi_usbd_dev_EpFreeBuffer(pEpDef->pBufferList, pEpDef->pCurrentBuffer);
                pEpDef->pCurrentBuffer = 0;
            }
            pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
            ADI_USBD_EXIT_CRITICAL();
            break;

        case ADI_USBD_EP_STATE_RX:
            ADI_USBD_DBG("MUSBMHDRC ADI_USBD_EP_STATE_RX");

            /* Pass control to the stack to process RX state */
            Ep0Callback(Ep0CallbackParam,ADI_USBD_DEV_EVENT_RX,(void*)pEpDef->pCurrentBuffer);
            break;

        case ADI_USBD_EP_STATE_RX_STATUS:
            /* call EP0 Callback to process completion event */
            Ep0Callback(Ep0CallbackParam,ADI_USBD_DEV_EVENT_RX,(void*)0);
            /*
             * Release Buffer
             */
            if (pEpDef->pCurrentBuffer) {
                adi_usbd_dev_EpFreeBuffer(pEpDef->pBufferList, pEpDef->pCurrentBuffer);
                pEpDef->pCurrentBuffer = 0;
            }
            /* Reset EP State to await next setup packet */
            pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;
            break;

        default:
            break;
    }
}

void adi_usbd_dev_musbhdrc_EnableRxInterrupt(
                            ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                            ADI_USBD_DEV_EP_DEF_EX      *pEpDef
)
{
    if (pEpDef->EpNum == 0)
    {
        /*
         * nothing to do as EP 0 interrupt is always enabled
         */
    }
    else {
        /* Set Indexed RXCSR register */
        *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
        SET_DEVICE_REG_VALUE(pDevice, pINTRRXE, (1<<(pEpDef->EpNum)), (1<<(pEpDef->EpNum)) );
    }
}
void adi_usbd_dev_musbhdrc_EnableTxInterrupt(
                            ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                            ADI_USBD_DEV_EP_DEF_EX      *pEpDef
)
{
    if (pEpDef->EpNum == 0)
    {
        /*
         * nothing to do as EP 0 interrupt is always enabled
         */
        *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
        SET_DEVICE_REG_VALUE(pDevice, pINTRTXE, BITM_USB_INTRTX_EP0, BITM_USB_INTRTX_EP0 );
    }
    else {
        /* Set Indexed RXCSR register */
        *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
        SET_DEVICE_REG_VALUE(pDevice, pINTRTXE, (1<<(pEpDef->EpNum)), (1<<(pEpDef->EpNum)) );
    }
}

void adi_usbd_dev_musbhdrc_DisableRxInterrupt(
                        ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                        ADI_USBD_DEV_EP_DEF_EX      *pEpDef
)
{
    if ((pEpDef->EpNum) == 0)
    {
        /*
         * nothing to do as EP 0 interrupt is always enabled
         */
    }
    else {
        /* Set Indexed RXCSR register */
        *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
        CLR_DEVICE_REG_VALUE(pDevice, pINTRRXE, (1<<(pEpDef->EpNum)) );
    }
}
void adi_usbd_dev_musbhdrc_DisableTxInterrupt(
                        ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                        ADI_USBD_DEV_EP_DEF_EX      *pEpDef
)
{
    if ((pEpDef->EpNum) == 0)
    {
        /*
         * nothing to do as EP 0 interrupt is always enabled
         */
        *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
        CLR_DEVICE_REG_VALUE(pDevice, pINTRTXE, BITM_USB_INTRTX_EP0 );
    }
    else {
        /* Set Indexed TXCSR register */
        *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
        CLR_DEVICE_REG_VALUE(pDevice, pINTRTXE, (1<<(pEpDef->EpNum)) );
    }
}

uint16_t adi_usbd_dev_musbhdrc_GetRxCount(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                ADI_USBD_DEV_EP_DEF_EX      *pEpDef
)
{
    *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
    return *pDEVICE_REG(pDevice,pEPI_RXCNT);
}

void adi_usbd_dev_musbhdrc_setEpIndex(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                uint8_t                     EpNum
)
{
    *pDEVICE_REG(pDevice,pINDEX) = EpNum;
}


#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)

/* ********************************************************************************************************
 *                               DMA Channel management routines                                          *
 * ********************************************************************************************************
*/
void adi_usbd_dev_musbmhdrc_InitDMAChannels(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice)
{
    uint8_t i;

    for(i=0;i<ADI_USBD_DEV_MAX_DMA_CHANNELS;i++)
    {
        adi_usbd_dev_musbmhdrc_FreeDMAChannel(pDevice,i);
    }
}

uint8_t adi_usbd_dev_musbmhdrc_DetermineDMAChannel(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t IntrDMA)
{
    uint8_t DMAChannel;
    uint8_t i;

    for(i=0;i<ADI_USBD_DEV_MAX_DMA_CHANNELS;i++)
    {
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
        if (IS_INTRPT(pDevice->pDMAChannels[i].IRQBitmask,IntrDMA))
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
        unsigned short DmaInterrupt = *pUSB_DMA_INTERRUPT;
        if ((DmaInterrupt >> i) & 0x1)
#endif
        {
            DMAChannel = i;
            break;
        }
    }

    return DMAChannel;
}

bool adi_usbd_dev_musbmhdrc_GetDMAChannel(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t *pDMAChannel)
{
    uint8_t i;

    for(i=0;i<ADI_USBD_DEV_MAX_DMA_CHANNELS;i++)
    {
        if (pDevice->pDMAChannels[i].bAvailable)
        {
            pDevice->pDMAChannels[i].bAvailable = false;
            *pDMAChannel = i;
            return true;
        }
    }
    return false;
}

void adi_usbd_dev_musbmhdrc_FreeDMAChannel(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t DMAChannel)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    CLR_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], 0xFFFF );
    pDevice->pDMAChannels[DMAChannel].bAvailable = true;
#endif
}

#endif

#if defined(_ADI_USBD_DEV_ENABLE_TEST_MODE_)
static void CheckForTestMode(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t *pSetupPacket)
{
    uint8_t  bmRequestType = (uint8_t)pSetupPacket[0];
    uint8_t  bRequest      = (uint8_t)pSetupPacket[1];
    uint16_t *pWvalue      = (uint16_t*)&pSetupPacket[2];
    uint16_t *pWindex      = (uint16_t*)&pSetupPacket[4];

    /* host to -device standard recipient device &&
     * set_feature request  &&
     * standard feature test mode
   */
    if( (bmRequestType == 0) &&  ( bRequest == 0x3) && (*pWvalue == 0x2))
    {
        adi_usbd_dev_testmode(pDevice,(void*)*pWindex);
    }
}

/* Usb Test mode selectors, pg 259 of USB 2.0 Specification */
#define USB_TMODE_TEST_J                        0x01   /* Test_J */
#define USB_TMODE_TEST_K                        0x02   /* Test_K */
#define USB_TMODE_TEST_SE0_NAK                  0x03   /* Test_SE0_NAK */
#define USB_TMODE_TEST_PACKET                   0x04   /* Test_PACKET */
#define USB_TMODE_TEST_FORCE_ENABLE             0x05   /* Test_Force_Enable */

static void adi_usbd_dev_testmode(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, void *pArg)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
/* For BF52x/BF54x controller, the buffer needs to be 2-byte aligned for PIO mode */
#pragma align(2)
#endif
   static char TestPacket[] =
   {
    /* 53 byte USB test packet (see USB 2.0 spec 7.1.20) */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,                   /* JKJKJKJK * 9         */
    0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA,                         /* JJKKJJKK * 8         */
    0xEE, 0xEE, 0xEE, 0xEE, 0xEE, 0xEE, 0xEE, 0xEE,                         /* JJJJKKKK * 8         */
    0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, /* JJJJJJJKKKKKKK * 8   */
    0x7F, 0xBF, 0xDF, 0xEF, 0xF7, 0xFB, 0xFD,                               /* JJJJJJJK * 8         */
    0xFC, 0x7E, 0xBF, 0xDF, 0xEF, 0xF7, 0xFB, 0xFD, 0x7E                    /* {JKKKKKKK * 10}, JK  */
    };
    uint32_t Index;
    int16_t wTest;
    int j;

    /* determine which test it is */
    Index = (uint32_t)pArg >> 8;

    /* set the EP number */
    SET_EP_INDEX(pDevice, 0);

    /*************************************************************************************
     * jm -- insert a delay before executing the test commands:
            The transition to test mode of an upstream facing port must not happen until after the status stage
            of the request. The power to the device must be cycled to exit test mode of an upstream facing port of a
            device. See Section 7.1.20 for definitions of each test mode. A device must support the TEST_MODE
            feature when in the Default, Address or Configured high-speed device states.
     ******************************************************************************************************/
    for (j = 0x1000; j>0; j--)
    {
    }

    switch(Index)
    {
        /* TEST_J */
        case USB_TMODE_TEST_J:
            wTest = 0x2;
        break;

        /* TEST_K */
        case USB_TMODE_TEST_K:
             wTest = 0x4;
        break;

        /* TEST_SE0_NAK */
        case USB_TMODE_TEST_SE0_NAK:
             wTest = 0x1;
        break;

        /* TEST_PACKET */
        case USB_TMODE_TEST_PACKET:
        {
            wTest = 0x8;

            /* fill EP0 with test packet */
            adi_usbd_dev_musbmhdrc_WriteFifo((ADI_USBD_DEV_FIFO_ENTRY *)pDevice->pRegs[pDevice->DeviceNumber].pEP_FIFO[0],
            								(ADI_USBD_DEV_FIFO_ENTRY *)TestPacket,
            								sizeof(TestPacket));
            /* set TXPKTRDY */
            SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_TXPKTRDY, ENUM_USB_EP0_CSR_P_TXPKTRDY );
        }
        break;

        /* TEST_FORCE_ENABLE */
        case USB_TMODE_TEST_FORCE_ENABLE:
             wTest = 0x10;
        break;

        default:
        break;
    }

    /* write to the TESTMODE register to enter the test mode */
    SET_DEVICE_REG_VALUE(pDevice, pTESTMODE, 0xFF, wTest );

    /* we do not want to return, we don't want to leave the test mode */

    while (1)
    {
    }
}
#endif /* _ADI_USBD_DEV_ENABLE_TEST_MODE_ */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */
