/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_dma_bf60x.c
 * @brief     DMA Manager Service for ADSP-BF60x/ADSP-CM40z processor family.
 * @version:  $Revision: 22948 $
 * @date:     $Date: 2013-09-03 20:32:22 -0400 (Tue, 03 Sep 2013) $
 *
 * @details
 *            This is the the primary source file for DMA Manager Service
 *            for ADSP-BF60x/ADSP-CM40z processor. This file includes the
 *            Peripheral and Memory DMA Manager implementation source files for
 *            ADSP-BF60x/ADSP-CM40z processor, and contains the implementation
 *            for APIs common to Peripheral and Memory DMA Manager.
 *
 */

/** @addtogroup DMA_Manager_bf60x
 *  @{

    DMA Manager for ADSP-BF60x family of processors

DMA manager APIs are grouped in to three categories

- Memory DMA APIs
- Peripheral DMA APIs
- APIs common to memory and peripheral DMAs

The common APIs can be used to query the current configuration of
a DMA channel register, configure the callback function of a DMA channel,
configure trigger settings, and to access the DMA traffic control registers.
The common APIs are
		- adi_dma_UpdateCallback()
		- adi_dma_GetCurrentDesc()
		- adi_dma_GetCurrentAddr()
		- adi_dma_GetCurrentXCount()
		- adi_dma_GetCurrentYCount()
		- adi_dma_ConfigTriggerIn()
		- adi_dma_ConfigTriggerOut()

<b>Trigger configuration</b>

Trigger configuration can be set on a per-buffer basis for all Peripheral DMA data transfer modes
and One-Shot/Circular Memory DMA transfer modes.

For DMA driven peripherals, application should use trigger configuration APIs provided by the
corresponding peripheral driver. Application can directly call adi_dma_ConfigTriggerIn() and
adi_dma_ConfigTriggerOut() APIs only for One-Shot/Circular Memory DMA transfer modes.

The trigger configuration APIs are <b>NOT</b> supported for Memory DMA Descriptor adi_mdma_CopyQueueList()
mode and Memory DMA Array data copy adi_mdma_Copy1DArray() and adi_mdma_Copy2DArray() modes.
Application can apply trigger on per-buffer basis for these modes by configuring the 'Config' field
in the descriptor list/array data structure before submitting it to the DMA Manager.

<em><b>Trigger Input configuration</b></em>

Trigger Input settings of a DMA channel can be configured using the adi_dma_ConfigTriggerIn() API.
The API takes two boolean parameters, one to enable/disable waiting for an incoming trigger
before starting the next DMA work unit. The second parameter is to enable/disable trigger overrun error.

<em><b>Trigger Output configuration</b></em>

Trigger Output settings of a DMA channel can be configured using the adi_dma_ConfigTriggerOut() API.
The API takes one parameter that specifies the condition on which a DMA channel can issue an outgoing Trigger.

<em><b>How to use Trigger APIs</b></em>

Trigger configuration can be set on a per-buffer basis. To configure a buffer with specific
trigger settings, the Trigger API should be called before and after submitting the buffer.

The following snippet shows how to configure trigger input wait on per buffer basis.

@code

Enable trigger input and/or overrun error
....
Submit data buffer
....
Disable trigger input and overrun error

@endcode

The following snippet shows how to configure a buffer with a condition to generate an outgoing trigger.

@code

Enable trigger output
....
Submit data buffer
....
Disable trigger output

@endcode

 */

/*=============  I N C L U D E S   =============*/

/* Include header file with definitions specific to
   ADSP-BF60x processor DMA manager implementation */
#include "adi_dma_def_bf60x.h"

/*============== D E B U G    M O D E    D E F I N E S  ===============*/

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/

/* Registers DMA channel with Interrupt dispatcher */
static ADI_DMA_RESULT RegisterWithIntDispatcher (ADI_DMA_INFO    *pDmaInfo);

/* Resets DMA Registers */
static void ResetDmaRegs (volatile ADI_DMA_REGS *pReg);

/* Uninstalls DMA Error handler */
static ADI_DMA_RESULT UninstallDmaErrorHandler (ADI_DMA_INFO    *pDmaInfo);

/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)

/* Gets the bitwise registry index and bitwise channel number */
static void GetBitWiseInfo(
    uint8_t             ChannelNum,
    uint8_t             *pBitWiseRegistryIndex,
    uint8_t             *pBitWiseChannelNum);

#endif

/*
 * DMA Interrupt Handlers
 */

/* DMA Interrupt handler for data transfer */
static void DmaDataIntHandler(uint32_t SID, void *pCBParam);
/* DMA Error interrupt handler */
static void DmaErrorIntHandler(uint32_t SID, void *pCBParam);

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validate DMA Channel Handle */
static ADI_DMA_RESULT ValidateChannelHandle (ADI_DMA_CHANNEL_HANDLE     hChannel);

/* Validates buffer address and its corresponding DMA register values */
static ADI_DMA_RESULT   ValidateBuffer (
    uint32_t            StartAddress,
    uint32_t            Config,
    uint32_t            Status,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify);

/* End of debug functions */
#endif /* ADI_DEBUG */

/*=============  D M A    I M P L E M E N T A T I O N    S O U R C E    F I L E S  =============*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_19_1:"Allow including source files")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined in a common file")
#endif /* _MISRA_RULES */

/* Include DMA data declaration file */
/* IF (ADSP-BF60x Family) */
#if defined( __ADSPBF609_FAMILY__)
/* ADSP-BF60x Processor specific data for DMA manager */
#include "adi_dma_data_bf60x.c"
/* ELSE (ADSP-CM40z processors) */
#elif defined (ADI_ADSP_CM40Z)
/* ADSP-CM40z Processor specific data for DMA manager */
#include "adi_dma_data_cm40z.c"
#elif defined (__ADSPBF707_FAMILY__)
/* ADSP-BF70X Processor specific data for DMA manager */
#include "adi_dma_data_bf70x.c"
#else
#error "This processor is not supported"
#endif

/* ADSP-BF60x/ADSP-CM40z Peripheral DMA manager implementation */
#include "adi_pdma_bf60x.c"
/* ADSP-BF60x/ADSP-CM40z Memory DMA manager implementation */
#include "adi_mdma_bf60x.c"

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_1:" ")
#pragma diag(suppress:misra_rule_8_5:" ")
#pragma diag(suppress:misra_rule_10_3:" ")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#pragma diag(suppress:misra_rule_11_1:"Allow Type-casting")
#pragma diag(suppress:misra_rule_11_3:"Allow Type-casting")
#pragma diag(suppress:misra_rule_12_7:" ")
#pragma diag(suppress:misra_rule_13_6:" ")
#pragma diag(suppress:misra_rule_14_7:" ")
#pragma diag(suppress:misra_rule_16_7:" ")
#pragma diag(suppress:misra_rule_17_4:" ")
#pragma diag(suppress:misra_rule_19_15:" ")
#endif /* _MISRA_RULES */

/*=============  C O D E  =============*/

/*=============  L O C A L    F U N C T I O N S  =============*/

/*
 * Resets DMA channel registers.
 *
 * Parameters
 *  - [in]  pReg   Pointer to DMA Channel registers to reset.
 *
 * Returns
 *  - None
 *
 */
static void ResetDmaRegs (volatile ADI_DMA_REGS *pReg)
{
    /* Reset DMA channel registers */
    pReg->Config        = 0u;
    pReg->NextDescPtr   = 0u;
    pReg->StartAddr     = 0u;
    pReg->XCount        = 0u;
    pReg->XModify       = 0;
    pReg->YCount        = 0u;
    pReg->YModify       = 0;
    pReg->Status        = ADI_DMA_CLEAR_INTERRUPTS; /* Clear any pending interrupts */
/* IF (ADSP-BF60x or ADSP-BF70x family)*/
#if defined( __ADSPBF609_FAMILY__) || \
    defined(__ADSPBF707_FAMILY__)
    asm("ssync;");
#endif
}

/*
 * Registers DMA channel with Interrupt dispatcher.
 *
 * Parameters
 *  - [in]  pDmaInfo    DMA Channel info to work on
 *
 * Returns
 *  - #ADI_DMA_SUCCESS: Successfully registered DMA Channel with interrupt dispatcher.
 *  - #ADI_DMA_INT_REGISTER_FAILED:  Failed to register DMA interrupt handler with the interrupt dispatcher.
 *
 */
static ADI_DMA_RESULT RegisterWithIntDispatcher (ADI_DMA_INFO   *pDmaInfo)
{
/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined(__ADSPBF707_FAMILY__)

    /* IF (DMA Error interrupt handler is not yet installed) */
    if (AdiDmaNumChannelsOpen == 0)
    {
        /* Install DMA Error interrupt handler */
        if(adi_int_InstallHandler(ADI_DMA_ERROR_SID, DmaErrorIntHandler, NULL, true) != ADI_INT_SUCCESS)
        {
            /* Failed to register DMA interrupt handler */
            return (ADI_DMA_INT_REGISTER_FAILED);
        }
    }

/* ELSE (ADSP-CM402/CM403/CM407/CM408 processors) */
#elif defined (ADI_ADSP_CM40Z)

    /* Install DMA Error interrupt handler */
    if(adi_int_InstallHandler((uint32_t)pDmaInfo->ErrorIntSID, DmaErrorIntHandler, pDmaInfo, true) != ADI_INT_SUCCESS)
    {
        /* Failed to register DMA interrupt handler */
        return (ADI_DMA_INT_REGISTER_FAILED);
    }

#endif

    /* Install DMA Data Interrupt handler */
    if (adi_int_InstallHandler ((uint32_t)pDmaInfo->DataIntSID, DmaDataIntHandler, pDmaInfo, true) != ADI_INT_SUCCESS)
    {

        /* Failed to install DMA data interrupt handler */

/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)

        /* IF (None of the DMA channels in this DMA Controller is open) */
        if (AdiDmaNumChannelsOpen == 0)
        {
            /* Un-register DMA Error interrupt handler */
            adi_int_UninstallHandler (ADI_DMA_ERROR_SID);
        }

/* ELSE (ADSP-CM402/CM403/CM407/CM408 processors) */
#elif defined (ADI_ADSP_CM40Z)
        /* Un-register DMA Error interrupt handler */
        adi_int_UninstallHandler ((uint32_t)pDmaInfo->ErrorIntSID);
#endif

        /* Failed to register DMA Interrupt handler */
        return (ADI_DMA_INT_REGISTER_FAILED);
    }

    /* Report success */
    return (ADI_DMA_SUCCESS);
}

/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)
/*
 * Gets the bitwise registry index and bitwise channel number for the supplied DMA channel number.
 *
 * Parameters
 *  - [in]  ChannelNum              DMA Channel number to work on.
 *  - [out] pBitWiseRegistryIndex   Location to store the Index to bitwise registry
 *                                  used to keep in track of open DMA channels.
 *  - [out] pBitWiseChannelNum      Location to store the DMA channel number relative to Bitwise registry index.
 *
 * Returns
 *  - None
 *
 */
static void GetBitWiseInfo(
    uint8_t             ChannelNum,
    uint8_t             *pBitWiseRegistryIndex,
    uint8_t             *pBitWiseChannelNum)
{
    /* Variable to calculate the registry index */
    uint8_t RegistryIndex = 0u;

    /* Continue until the channel number is less than number of
       channels that can be handled by a bitwise registry entry */
    while (ChannelNum > ADI_DMA_NUM_CHANNELS_PER_REGISTRY)
    {
        /* Increment the registry index */
        RegistryIndex++;
        /* update the channel number */
        ChannelNum -= ADI_DMA_NUM_CHANNELS_PER_REGISTRY;
    }

    /* Initialise the registry index */
    *pBitWiseRegistryIndex = RegistryIndex;
    /* Save the bitwise channel number */
    *pBitWiseChannelNum = ChannelNum;
}
#endif

/*
 * Uninstalls DMA Error handler when no DMA channels are left open
 *
 * Parameters
 *  - pDmaInfo  DMA Channel info to work on
 *
 * Returns
 *  - None
 */
static ADI_DMA_RESULT UninstallDmaErrorHandler (
    ADI_DMA_INFO            *pDmaInfo)
{

/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)

    /* Loop variable */
    uint8_t i;

    /* FOR (All Bitwise registry entries) */
    for (i = 0; i < ADI_DMA_NUM_REGISTRY; i++)
    {
        /* IF (Bitwise registry is non-zero) */
        if (AdiDmaBitWiseRegistry[i] != 0u)
        {
            /* we've a DMA channel open */
            return (ADI_DMA_SUCCESS);
        }
    }

    /* We've reached this point - means all DMA channels are closed */

#endif

    /* Un-register DMA Error interrupt handler */
/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)
    if (adi_int_UninstallHandler (ADI_DMA_ERROR_SID) != ADI_INT_SUCCESS)
/* ELSE (ADSP-CM402/CM403/CM407/CM408 processors) */
#elif defined (ADI_ADSP_CM40Z)
    if (adi_int_UninstallHandler ((uint32_t)pDmaInfo->ErrorIntSID) != ADI_INT_SUCCESS)
#endif
    {
        /* Return failure as we failed to un-register the interrupt */
        return (ADI_DMA_INT_UNREGISTER_FAILED);
    }
    /* ELSE (Successfuly un-installed DMA error interrupt handler) */
    else
    {
        return (ADI_DMA_SUCCESS);
    }
}

/*
 * DMA Data Interrupt handler.
 *
 * Parameters
 *  - [in]   SID         System Interrupt ID.
 *  - [in]   pCBParam    Callback parameter provided by DMA manager
 *                       while installing interrupt handler.
 *
 * Returns
 *  None
 *
 */
static void DmaDataIntHandler(uint32_t SID, void *pCBParam)
{
    /* Get the DMA channel info to which the interrupt belongs */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *) pCBParam;

    /* Clear the DMA DONE interrupt sticky bit */
    pDmaInfo->pReg->Status = ENUM_DMA_STAT_IRQDONE;

    /* Process this interrupt */
    pDmaInfo->pChannel->PDMA.pfDataIntHandle (pDmaInfo);
}

/*
 * DMA Error Interrupt handler.
 *
 * Parameters
 *  - [in]   SID         System Interrupt ID.
 *  - [in]   pCBParam    Callback parameter provided by DMA manager
 *                       while installing interrupt handler.
 *
 * Returns
 *  None
 *
 */
static void DmaErrorIntHandler(uint32_t SID, void *pCBParam)
{

/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)

    /* Flag to indicate if DMA channel number that generated the error has been located or not */
    bool        bChannelFound = false;
    /* Loop variable */
    uint8_t     i;
    /* DMA Channel number to which the interrupt belongs */
    uint8_t     ChannelNumber = 0;
    /* Bitwise registry value to work on */
    uint32_t    OpenChannels;

    /* FOR (All Bitwise registy entries) */
    for (i = 0; i < ADI_DMA_NUM_REGISTRY; i++)
    {
        /* FOR (All DMA channels marked as open in this bitwise registry) */
        for (OpenChannels = AdiDmaBitWiseRegistry[i]; OpenChannels > 0; )
        {
            /* IF (This DMA Channel is open) */
            if ((OpenChannels & 1) == 1u)
            {
                /* IF (DMA error is for this channel) */
                if ((ChannelInfo[ChannelNumber].pReg->Status & BITM_DMA_STAT_IRQERR) == ENUM_DMA_STAT_IRQERR)
                {
                    /* IF (We've a valid callback function) */
                    if ((ChannelInfo[ChannelNumber].pChannel)->PDMA.pfCallback != NULL)
                    {
                        /* Post callback to report DMA error, pass error cause field value as event specific parameter */
                        ((ChannelInfo[ChannelNumber].pChannel)->PDMA.pfCallback) ((ChannelInfo[ChannelNumber].pChannel)->PDMA.pCBParam,
                                                                                  ADI_DMA_EVENT_ERROR_INT,
                                                                                  (void *)(ChannelInfo[ChannelNumber].pReg->Status & BITM_DMA_STAT_ERRC));
                    }

                    /* Clear the DMA ERROR Interrupt sticky bit */
                    ChannelInfo[ChannelNumber].pReg->Status = ENUM_DMA_STAT_IRQERR;

                    /* Found DMA channel that generated the error interrupt */
                    bChannelFound = true;

                    /* quit this loop */
                    break;

                } /* End of IF (DMA error is for this channel) */

            } /* End of IF (This DMA Channel is open) */

            /* Move to next DMA channel */
            OpenChannels >>= 1;
            /* Increment channel number */
            ChannelNumber++;

        } /* End of FOR (All DMA channels marked as open in this bitwise registry) */

        /* IF (Found DMA channel that generated the error interrupt) */
        if (bChannelFound)
        {
            /* quit this loop */
            break;
        }

        /* Failed to find a channel match, move to the next DMA bitwise registry */
        ChannelNumber += 32;

    } /* End of FOR (DMA channels that are open) */

/* ELSE (ADSP-CM402/CM403/CM407/CM408 processors) */
#elif defined (ADI_ADSP_CM40Z)

    /* Get the DMA channel info to which the interrupt belongs */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *) pCBParam;

    /* IF (We've a valid callback function) */
    if (pDmaInfo->pChannel->PDMA.pfCallback != NULL)
    {
        /* Post callback to report DMA error, pass error cause field value as event specific parameter */
        (pDmaInfo->pChannel->PDMA.pfCallback ) (pDmaInfo->pChannel->PDMA.pCBParam,
                                                ADI_DMA_EVENT_ERROR_INT,
                                                (void *)(pDmaInfo->pReg->Status & BITM_DMA_STAT_ERRC));
    }

    /* Clear the DMA ERROR Interrupt sticky bit */
    pDmaInfo->pReg->Status = ENUM_DMA_STAT_IRQERR;

#endif

}

/*=============  D E B U G   F U N C T I O N S =============*/

/* IF (Debug mode enabled) */
#if defined (ADI_DEBUG)

/**
 * @brief Validates the supplied DMA channel handle.
 *
 * @param [in]  hChannel    DMA Channel handle to validate.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: DMA Channel handle is valid.
 *          - #ADI_DMA_FAILURE:  DMA Channel handle is invalid.
 *
 */
static ADI_DMA_RESULT ValidateChannelHandle (ADI_DMA_CHANNEL_HANDLE    hChannel)
{
    /* Loop variable */
    uint32_t    i;

    /* FOR (All DMA channels) */
    for (i = 0; i < (sizeof (ChannelInfo) / sizeof (ChannelInfo[0])); i++)
    {
        /* IF (Channel handle is valid) */
        if (hChannel == &ChannelInfo[i])
        {
            /* Channel valid, return success */
            return (ADI_DMA_SUCCESS);
        }
    }

    /* Channel invalid, return failure */
    return (ADI_DMA_FAILURE);
}

/*
 * Validates buffer address and its corresponding DMA register values.
 *
 * Parameters
 *  - [in]  StartAddress    Start address of the data buffer to be processed.
 *  - [in]  Config          DMA Config register value to validate.
 *  - [in]  Status          DMA Status register value to be used for validation.
 *  - [in]  XCount          Inner loop count start value in number of
 *                          DMA data bus width (MSIZE) transfers.
 *  - [in]  XModify         Inner loop address increment in bytes.
 *  - [in]  YCount          Outer loop count start value in number of rows.
 *  - [in]  YModify         Outer loop address increment in bytes.
 *
 * Returns  Status
 *  - ADI_DMA_SUCCESS: Supplied List buffer is valid.
 *  - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *  - ADI_DMA_BAD_XCOUNT: Supplied XCount is invalid.
 *  - ADI_DMA_BAD_YCOUNT: Supplied YCount is invalid.
 *  - ADI_DMA_BAD_START_ADDR: Supplied start address is invalid.
 *  - ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                  buffer address and will cause alignment error.
 *
 *
 * Address Allignment to MSIZE
 * +--------+---------------------------+
 * | MSIZE  |   Start Address (ADDR)    |
 * +--------+---------------------------+
 * |1 Byte  |   No restriction on ADDR  |
 * +--------+---------------------------+
 * |2 Bytes |   ADDR[0] = 0             |
 * +--------+---------------------------+
 * |4 Bytes |   ADDR[1:0] = 0           |
 * +--------+---------------------------+
 * |8 Bytes |   ADDR[2:0] = 0           |
 * +--------+---------------------------+
 * |16 Bytes|   ADDR[3:0] = 0           |
 * +--------+---------------------------+
 * |32 Bytes|   ADDR[4:0] = 0           |
 * +--------+---------------------------+
 *
 * The above table also true for XModify and YModify (in 2D mode)
 *
 */
static ADI_DMA_RESULT   ValidateBuffer (
    uint32_t            StartAddress,
    uint32_t            Config,
    uint32_t            Status,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify)
{
    /* Variables to validate buffer address and X/Y modify */
    uint32_t    MSize, Mask;

    /* IF (Start address is invalid) */
    if (StartAddress == 0)
    {
    	/* Report failure as supplied start address is invalid */
    	return (ADI_DMA_BAD_START_ADDR);
    }

    /* IF (XCount is invalid) */
    if (XCount == 0)
    {
    	/* Report failure as supplied XCount is invalid */
    	return (ADI_DMA_BAD_XCOUNT);
    }

    /* IF (YCount is invalid) */
    if ((YCount == 0) && (YModify != 0))
    {
    	/* Report failure as supplied YCount is invalid */
        return (ADI_DMA_BAD_YCOUNT);
    }

    /* IF (PSize is more than the allowed value) */
    if (((Config & BITM_DMA_CFG_PSIZE) >> BITP_DMA_CFG_PSIZE) > \
        ((Status & BITM_DMA_STAT_PBWID) >> BITP_DMA_STAT_PBWID))
    {
        /* Return failure as the PSize is invalid */
        return (ADI_DMA_BAD_PSIZE);
    }

    /* MSize must be more than 1-byte, Extract MSize */
    MSize = ((Config & BITM_DMA_CFG_MSIZE) >> BITP_DMA_CFG_MSIZE);
    /* Initialize Mask */
    Mask = ((uint32_t)ADI_DMA_ADDR_VALID_MASK >> (ADI_DMA_MAX_MSIZE_VAL - MSize));

    /* IF (Buffer alignment doesn't work with the MSize) */
    if ((StartAddress & MSize) != 0)
    {
        /* Return failure - address alignment error */
        return (ADI_DMA_ADDR_ALIGNEMNT_ERROR);
    }

    /* IF (XCount is valid) */
    if (XCount > 0)
    {
        /* IF (Buffer alignment doesn't work with the XModify) */
        if ((XModify & MSize) != 0)
        {
            /* Return failure - address alignment error */
            return (ADI_DMA_ADDR_ALIGNEMNT_ERROR);
        }
    }

    /* IF (YCount is valid) */
    if (YCount > 0)
    {
        /* IF (Buffer alignment doesn't work with the YModify) */
        if ((YModify & MSize) != 0)
        {
            /* Return failure - address alignment error */
            return (ADI_DMA_ADDR_ALIGNEMNT_ERROR);
        }
    }

    /* Return success as the supplied address is valid */
    return (ADI_DMA_SUCCESS);
}

/* End of debug functions */
#endif /* ADI_DEBUG */

/*=============  P U B L I C   F U N C T I O N S =============*/

/**
 * @brief Configures Trigger Input settings for a DMA channel.
 *
 * @param [in]  hChannel         DMA channel handle to configure.
 * @param [in]  bTriggerWait     'true' to wait for an incoming trigger before starting the next DMA work unit.
 *                               'false' to ignore incoming trigger and begin work unit automatically.
 * @param [in]  bTriggerOvError  'true' to enable and report trigger overrun error.
 *                               'false' to disable trigger overrun error.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully configured Trigger Output.
 *          - #ADI_DMA_NOT_OPEN: DMA channel is not open.
 *          - #ADI_DMA_BAD_HANDLE: Supplied DMA handle is invalid.
 *
 */
ADI_DMA_RESULT  adi_dma_ConfigTriggerIn (
    ADI_DMA_CHANNEL_HANDLE      hChannel,
    bool                        bTriggerWait,
    bool                        bTriggerOvError)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;
    /* Address where the DMA common config information is stored */
    uint32_t        *pCommonConfig;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Get the DMA Common Config element address for this channel */

    /* IF (Configure MDMA) */
    if (pDmaInfo->pChannel->PDMA.bIsMemDma)
    {
        /* IF (Source channel) */
        if (pDmaInfo->DataIntSID == pDmaInfo->pChannel->MDMA.Src.DataSID)
        {
            pCommonConfig = &(pDmaInfo->pChannel->MDMA.Src.CommonConfig);
        }
        /* ELSE (Destination channel) */
        else
        {
            pCommonConfig = &(pDmaInfo->pChannel->MDMA.Dest.CommonConfig);
        }
    }
    /* ELSE (Configure PDMA) */
    else
    {
        pCommonConfig = &(pDmaInfo->pChannel->PDMA.CommonConfig);
    }

    /* IF (Wait for Trigger input) */
    if (bTriggerWait)
    {
        /* Update DMA configuration register */
        *pCommonConfig |= ENUM_DMA_CFG_TRGWAIT;
    }
    /* ELSE (Ignore Trigger input) */
    else
    {
        /* Update DMA configuration register */
        *pCommonConfig &= ~ENUM_DMA_CFG_TRGWAIT;
    }

    /* IF (Enable Trigger overrun error) */
    if (bTriggerOvError)
    {
        /* Update DMA configuration register */
        *pCommonConfig |= ENUM_DMA_CFG_TOV_EN;
    }
    /* ELSE (Ignore Trigger overrun error) */
    else
    {
        /* Update DMA configuration register */
        *pCommonConfig &= ~ENUM_DMA_CFG_TOV_EN;
    }

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures Trigger Output settings for a DMA channel.
 *
 * @param [in]  hChannel         DMA channel handle to configure.
 * @param [in]  eDmaTrigger      Condition to generate an outgoing trigger.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully configured Trigger Output.
 *          - #ADI_DMA_NOT_OPEN: DMA channel is not open.
 *          - #ADI_DMA_BAD_HANDLE: Supplied DMA handle is invalid.
 *
 */
ADI_DMA_RESULT  adi_dma_ConfigTriggerOut (
    ADI_DMA_CHANNEL_HANDLE      hChannel,
    ADI_DMA_TRIGGER_OUT    		eDmaTrigger)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;
    /* Address where the DMA common config information is stored */
    uint32_t        *pCommonConfig;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Get the DMA Common Config element address for this channel */

    /* IF (Configure MDMA) */
    if (pDmaInfo->pChannel->PDMA.bIsMemDma)
    {
        /* IF (Source channel) */
        if (pDmaInfo->DataIntSID == pDmaInfo->pChannel->MDMA.Src.DataSID)
        {
            pCommonConfig = &(pDmaInfo->pChannel->MDMA.Src.CommonConfig);
        }
        /* ELSE (Destination channel) */
        else
        {
            pCommonConfig = &(pDmaInfo->pChannel->MDMA.Dest.CommonConfig);
        }
    }
    /* ELSE (Configure PDMA) */
    else
    {
        pCommonConfig = &(pDmaInfo->pChannel->PDMA.CommonConfig);
    }

    /* Clear outgoing trigger configuration field */
    *pCommonConfig &= ~BITM_DMA_CFG_TRIG;
    /* Update DMA configuration register */
    *pCommonConfig |= eDmaTrigger;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Update DMA Channel callback function.
 *
 * @param [in]  hChannel    DMA channel handle to work on.
 * @param [in]  pfCallback  Address of application callback function.
 * @param [in]  pCBParam    Parameter passed back to application callback.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated DMA channel callback.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't update callback function when the DMA data transfer is enabled.
 *
 */
ADI_DMA_RESULT adi_dma_UpdateCallback(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    ADI_CALLBACK                    pfCallback,
    void *const                     pCBParam)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pDmaInfo->pChannel->PDMA.bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* Save the supplied callback information */
    pDmaInfo->pChannel->PDMA.pfCallback = pfCallback;
    pDmaInfo->pChannel->PDMA.pCBParam   = pCBParam;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Current Descriptor pointer register value.
 *
 * @param [in]  hChannel        DMA Channel handle to work on.
 * @param [out] pCurrentDesc    Address of the location to store current descriptor address.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentDesc(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    void                            **pCurrentDesc)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    *pCurrentDesc = (void *) pDmaInfo->pReg->CurrDescPtr;

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Address Current Register value.
 *
 * @details Gets present memory transfer address of the active work unit.
 *
 * @param [in]  hChannel        DMA Channel handle to work on.
 * @param [out] pCurrentAddr    Address of the location to store current data address.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentAddr(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    void                            **pCurrentAddr)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    *pCurrentAddr = (void *) pDmaInfo->pReg->CurrAddr;

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Current Inner loop count (current X count) Register value.
 *
 * @param [in]  hChannel            DMA Channel handle to work on.
 * @param [out] *pCurrentXCount     Location to store Current X Count Register value.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentXCount(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    uint32_t                        *pCurrentXCount)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    *pCurrentXCount = pDmaInfo->pReg->CurrXCount;

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Current Outer loop count (current Y count) Register value.
 *
 *
 * @param [in]  hChannel            DMA Channel handle to work on.
 * @param [out] *pCurrentYCount     Location to store Current Y Count Register value.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentYCount(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    uint32_t                        *pCurrentYCount)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    *pCurrentYCount = pDmaInfo->pReg->CurrYCount;

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*@}*/

