/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_dma_data_bf60x.c
 * @brief     ADSP-BF60x processor specific data for DMA Manager.
 * @version:  $Revision: 18384 $
 * @date:     $Date: 2013-04-04 17:17:34 -0400 (Thu, 04 Apr 2013) $
 *
 * @details
 *            This is an additional source file for DMA Manager Service
 *            for ADSP-BF60x processor. This file contains processor specific
 *            data for ADSP-BF60x processor family.
 *
 */

/** @addtogroup DMA_Manager ADSP-BF60x processor specific data.
 *  @{
 */

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_8_5:"Objects defined in this source files are common to MDMA and PDMA source files")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined in a common file")
#pragma diag(suppress:misra_rule_9_2:" ")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#endif /* _MISRA_RULES */

/*=============  P R O C E S S O R    S P E C I F I C     D A T A  =============*/

/* Settings for ADSP-BF608/609 DMA channels and Streams */
#if defined(__ADSPBF609_FAMILY__)

/* Interrupt ID for Reserved DMA channel */
#define ADI_DMA_RSV_INTR_ID                     (255u)
/* Total number of DMA Channels available in this processor family */
#define ADI_DMA_TOTAL_NUM_CHANNELS              (47u)
/* Number of DMA Channels per bitwise registry */
#define ADI_DMA_NUM_CHANNELS_PER_REGISTRY       (32u)
/* Number of bitwise registry entries requied to handle all the DMA channels */
#define ADI_DMA_NUM_REGISTRY                    (2u)
/* DMA Error System Interrupt ID */
#define ADI_DMA_ERROR_SID                       (INTR_DMAC_ERR)

/* Bitwise registry to keep in track of open DMA channels */
static uint32_t AdiDmaBitWiseRegistry[ADI_DMA_NUM_REGISTRY] = { 0 };
/* Count to keep in track of number of open DMA channels */
static uint32_t AdiDmaNumChannelsOpen = (0u);

/* DMA Channel specific information */
static ADI_DMA_INFO     ChannelInfo[] =
{
   { INTR_SPORT0_A_DMA,    (volatile ADI_DMA_REGS *) REG_DMA0_DSCPTR_NXT,  NULL }, /* DMA  0 - SPORT0 Channel A */
   { INTR_SPORT0_B_DMA,    (volatile ADI_DMA_REGS *) REG_DMA1_DSCPTR_NXT,  NULL }, /* DMA  1 - SPORT0 Channel B */
   { INTR_SPORT1_A_DMA,    (volatile ADI_DMA_REGS *) REG_DMA2_DSCPTR_NXT,  NULL }, /* DMA  2 - SPORT1 Channel A */
   { INTR_SPORT1_B_DMA,    (volatile ADI_DMA_REGS *) REG_DMA3_DSCPTR_NXT,  NULL }, /* DMA  3 - SPORT1 Channel B */
   { INTR_SPORT2_A_DMA,    (volatile ADI_DMA_REGS *) REG_DMA4_DSCPTR_NXT,  NULL }, /* DMA  4 - SPORT2 Channel A */
   { INTR_SPORT2_B_DMA,    (volatile ADI_DMA_REGS *) REG_DMA5_DSCPTR_NXT,  NULL }, /* DMA  5 - SPORT2 Channel B */
   { INTR_SPI0_TXDMA,      (volatile ADI_DMA_REGS *) REG_DMA6_DSCPTR_NXT,  NULL }, /* DMA  6 - SPI0 TX */
   { INTR_SPI0_RXDMA,      (volatile ADI_DMA_REGS *) REG_DMA7_DSCPTR_NXT,  NULL }, /* DMA  7 - SPI0 RX */
   { INTR_SPI1_TXDMA,      (volatile ADI_DMA_REGS *) REG_DMA8_DSCPTR_NXT,  NULL }, /* DMA  8 - SPI1 TX */
   { INTR_SPI1_RXDMA,      (volatile ADI_DMA_REGS *) REG_DMA9_DSCPTR_NXT,  NULL }, /* DMA  9 - SPI1 RX */
   { INTR_RSI0_DMA,        (volatile ADI_DMA_REGS *) REG_DMA10_DSCPTR_NXT, NULL }, /* DMA 10 - RSI0 */
   { INTR_SDU0_DMA,        (volatile ADI_DMA_REGS *) REG_DMA11_DSCPTR_NXT, NULL }, /* DMA 11 - System Debug Unit 0 (SDU0) */
   { ADI_DMA_RSV_INTR_ID,  (volatile ADI_DMA_REGS *) REG_DMA12_DSCPTR_NXT, NULL }, /* DMA 12 - Reserved */
   { INTR_LP0_DMA,         (volatile ADI_DMA_REGS *) REG_DMA13_DSCPTR_NXT, NULL }, /* DMA 13 - Link Port 0 (LP0) */
   { INTR_LP1_DMA,         (volatile ADI_DMA_REGS *) REG_DMA14_DSCPTR_NXT, NULL }, /* DMA 14 - Link Port 1 (LP1) */
   { INTR_LP2_DMA,         (volatile ADI_DMA_REGS *) REG_DMA15_DSCPTR_NXT, NULL }, /* DMA 15 - Link Port 2 (LP2) */
   { INTR_LP3_DMA,         (volatile ADI_DMA_REGS *) REG_DMA16_DSCPTR_NXT, NULL }, /* DMA 16 - Link Port 3 (LP3) */
   { INTR_UART0_TXDMA,     (volatile ADI_DMA_REGS *) REG_DMA17_DSCPTR_NXT, NULL }, /* DMA 17 - UART0 TX */
   { INTR_UART0_RXDMA,     (volatile ADI_DMA_REGS *) REG_DMA18_DSCPTR_NXT, NULL }, /* DMA 18 - UART0 RX */
   { INTR_UART1_TXDMA,     (volatile ADI_DMA_REGS *) REG_DMA19_DSCPTR_NXT, NULL }, /* DMA 19 - UART1 TX */
   { INTR_UART1_RXDMA,     (volatile ADI_DMA_REGS *) REG_DMA20_DSCPTR_NXT, NULL }, /* DMA 20 - UART1 RX */
   { INTR_MDMA0_SRC,       (volatile ADI_DMA_REGS *) REG_DMA21_DSCPTR_NXT, NULL }, /* DMA 21 - MDMA0 Source */
   { INTR_MDMA0_DST,       (volatile ADI_DMA_REGS *) REG_DMA22_DSCPTR_NXT, NULL }, /* DMA 22 - MDMA0 Destination */
   { INTR_MDMA1_SRC,       (volatile ADI_DMA_REGS *) REG_DMA23_DSCPTR_NXT, NULL }, /* DMA 23 - MDMA1 Source */
   { INTR_MDMA1_DST,       (volatile ADI_DMA_REGS *) REG_DMA24_DSCPTR_NXT, NULL }, /* DMA 24 - MDMA1 Destination */
   { INTR_MDMA2_SRC,       (volatile ADI_DMA_REGS *) REG_DMA25_DSCPTR_NXT, NULL }, /* DMA 25 - MDMA2 Source */
   { INTR_MDMA2_DST,       (volatile ADI_DMA_REGS *) REG_DMA26_DSCPTR_NXT, NULL }, /* DMA 26 - MDMA2 Destination */
   { INTR_MDMA3_SRC,       (volatile ADI_DMA_REGS *) REG_DMA27_DSCPTR_NXT, NULL }, /* DMA 27 - MDMA3 Source */
   { INTR_MDMA3_DST,       (volatile ADI_DMA_REGS *) REG_DMA28_DSCPTR_NXT, NULL }, /* DMA 28 - MDMA3 Destination */
   { INTR_EPPI0_CH0_DMA,   (volatile ADI_DMA_REGS *) REG_DMA29_DSCPTR_NXT, NULL }, /* DMA 29 - EPPI0 Channel 0 */
   { INTR_EPPI0_CH1_DMA,   (volatile ADI_DMA_REGS *) REG_DMA30_DSCPTR_NXT, NULL }, /* DMA 30 - EPPI0 Channel 1 */
   { INTR_EPPI2_CH0_DMA,   (volatile ADI_DMA_REGS *) REG_DMA31_DSCPTR_NXT, NULL }, /* DMA 31 - EPPI2 Channel 0 */
   { INTR_EPPI2_CH1_DMA,   (volatile ADI_DMA_REGS *) REG_DMA32_DSCPTR_NXT, NULL }, /* DMA 32 - EPPI2 Channel 1 */
   { INTR_EPPI1_CH0_DMA,   (volatile ADI_DMA_REGS *) REG_DMA33_DSCPTR_NXT, NULL }, /* DMA 33 - EPPI1 Channel 0 */
   { INTR_EPPI1_CH1_DMA,   (volatile ADI_DMA_REGS *) REG_DMA34_DSCPTR_NXT, NULL }, /* DMA 34 - EPPI1 Channel 1 */

/* IF (This processor has a PIXC) */
#if defined (__ADI_HAS_PIXC__)
   { INTR_PIXC0_CH0_DMA,   (volatile ADI_DMA_REGS *) REG_DMA35_DSCPTR_NXT, NULL }, /* DMA 35 - PIXC0 Channel 0 */
   { INTR_PIXC0_CH1_DMA,   (volatile ADI_DMA_REGS *) REG_DMA36_DSCPTR_NXT, NULL }, /* DMA 36 - PIXC0 Channel 1 */
   { INTR_PIXC0_CH2_DMA,   (volatile ADI_DMA_REGS *) REG_DMA37_DSCPTR_NXT, NULL }, /* DMA 37 - PIXC0 Channel 2 */
#endif /* __ADI_HAS_PIXC__ */

/* IF (This processor has a PVP) */
#if defined (__ADI_HAS_PVP__)
   { INTR_PVP0_CPDOB_DMA,  (volatile ADI_DMA_REGS *) REG_DMA38_DSCPTR_NXT, NULL }, /* DMA 38 - PVP0 Camera Pipe Data Out B */
   { INTR_PVP0_CPDOC_DMA,  (volatile ADI_DMA_REGS *) REG_DMA39_DSCPTR_NXT, NULL }, /* DMA 39 - PVP0 Camera Pipe Data Out C */
   { INTR_PVP0_CPSTAT_DMA, (volatile ADI_DMA_REGS *) REG_DMA40_DSCPTR_NXT, NULL }, /* DMA 40 - PVP0 Camera Pipe Status Out */
   { INTR_PVP0_CPCI_DMA,   (volatile ADI_DMA_REGS *) REG_DMA41_DSCPTR_NXT, NULL }, /* DMA 41 - PVP0 Camera Pipe Control In */
   { INTR_PVP0_MPDO_DMA,   (volatile ADI_DMA_REGS *) REG_DMA42_DSCPTR_NXT, NULL }, /* DMA 42 - PVP0 Memory Pipe Data Out */
   { INTR_PVP0_MPDI_DMA,   (volatile ADI_DMA_REGS *) REG_DMA43_DSCPTR_NXT, NULL }, /* DMA 43 - PVP0 Memory Pipe Data In */
   { INTR_PVP0_MPSTAT_DMA, (volatile ADI_DMA_REGS *) REG_DMA44_DSCPTR_NXT, NULL }, /* DMA 44 - PVP0 Memory Pipe Status Out */
   { INTR_PVP0_MPCI_DMA,   (volatile ADI_DMA_REGS *) REG_DMA45_DSCPTR_NXT, NULL }, /* DMA 45 - PVP0 Memory Pipe Control In */
   { INTR_PVP0_CPDOA_DMA,  (volatile ADI_DMA_REGS *) REG_DMA46_DSCPTR_NXT, NULL }  /* DMA 46 - PVP0 Camera Pipe Data Out A */
#endif /*  __ADI_HAS_PVP__ */
};

/* Memory DMA Stream specific information */
static const ADI_DMA_MEM_STREAM_INFO    StreamInfo[] =
{
    { ADI_DMA_CHNL_MEMDMA_D0, ADI_DMA_CHNL_MEMDMA_S0 },  /* MDMA Stream 0 Source & Destination channel IDs */
    { ADI_DMA_CHNL_MEMDMA_D1, ADI_DMA_CHNL_MEMDMA_S1 },  /* MDMA Stream 1 Source & Destination channel IDs */
    { ADI_DMA_CHNL_MEMDMA_D2, ADI_DMA_CHNL_MEMDMA_S2 },  /* MDMA Stream 2 Source & Destination channel IDs */
    { ADI_DMA_CHNL_MEMDMA_D3, ADI_DMA_CHNL_MEMDMA_S3 }   /* MDMA Stream 3 Source & Destination channel IDs */
};

#endif /* __ADSPBF609_FAMILY__ */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*
** EOF
*/

/*@}*/

