/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_dma_def_bf60x.h
 * @brief:   Private header file for for ADSP-BF60x/ADSP-CM40z processor DMA Manager.
 * @version: $Revision: 18384 $
 * @date:    $Date: 2013-04-04 17:17:34 -0400 (Thu, 04 Apr 2013) $
 *
 * @details
 *           This is a private header file for the DMA Manager service
 *           for ADSP-BF60x/ADSP-CM40z processor family, which contains the API declarations,
 *           data and constant definitions used in DMA Manager implementation
 */

/*! \addtogroup DMA_Manager ADSP-BF60x DMA Manager private definitions
 *  Private definitions for ADSP-BF60x DMA Manager
 *  @{
 */

#ifndef __ADI_DMA_DEF_BF60X_H__
#define __ADI_DMA_DEF_BF60X_H__

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_3:" ")
#pragma diag(suppress:misra_rule_5_4:" ")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_4:" ")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/

/* OSAL includes */
#include <adi_osal.h>
/* Interrupt Manager includes */
#include <services/int/adi_int.h>
/* IF (ADSP-BF608/609) */
#if defined( __ADSPBF609_FAMILY__)
#include <services/int/adi_sec.h>
#endif
/* DMA Manager includes */
#include <services/dma/adi_dma.h>
/* memcpy includes */
#include <string.h>
/* Memory size check */
#include <assert.h>

/*==============  D E F I N E S  ===============*/

/* Clear DMA interrupts */
#define ADI_DMA_CLEAR_INTERRUPTS            (ENUM_DMA_STAT_PIRQ | ENUM_DMA_STAT_IRQERR | ENUM_DMA_STAT_IRQDONE)
/* Maximum MSize */
#define ADI_DMA_MAX_MSIZE_VAL               (ENUM_DMA_CFG_MSIZE32 >> BITP_DMA_CFG_MSIZE)
/* Mask to validate buffer address and X/Y modify.
   For maximum Msize of 32 bytes, ADDR[4:0] must be zero 0 and the mask should be 11111b */
#define ADI_DMA_ADDR_VALID_MASK             (0x1Fu)

/* Structure to hold Memory DMA channel info */
typedef struct
{
    ADI_DMA_CHANNEL_ID     DestChannel; /* Destination Channel ID */
    ADI_DMA_CHANNEL_ID     SrcChannel;  /* Source Channel ID */
} ADI_DMA_MEM_STREAM_INFO;

/* Memory DMA data transfer mode */
typedef enum
{
    ADI_MDMA_XFER_NONE		 = (0u),                                  /* MDMA is idle */
    ADI_MDMA_XFER_CIRCULAR	 = (1u),                                  /* Transfer to/from a memory mapped location or a single buffer */
    ADI_MDMA_XFER_ONE_SHOT	 = (2u),                                  /* One shot 1D/2D transfer */
    ADI_MDMA_XFER_LARGE_LIST = ADI_DMA_EVENT_DESC_LIST_PROCESSED,     /* Large model Descriptor type transfer */
    ADI_MDMA_XFER_ARRAY_1D   = ADI_DMA_EVENT_DESC_1D_ARRAY_PROCESSED, /* 1D Array Descriptor type transfer */
    ADI_MDMA_XFER_ARRAY_2D   = ADI_DMA_EVENT_DESC_2D_ARRAY_PROCESSED  /* 2D Array Descriptor type transfer */
} ADI_MDMA_DATA_XFER_MODE;

/* DMA Info structure definition for prototypes */
struct ADI_DMA_INFO;

/* MDMA instance structure definition for prototypes */
struct ADI_MDMA_CHANNEL;

/* Prototype for Memory DMA Data transfer enable functions */
typedef void (*ADI_MDMA_ENABLE_FN) (struct ADI_MDMA_CHANNEL *, uint32_t *, bool);

/* Prototype for Peripheral DMA Data transfer enable functions */
typedef ADI_DMA_RESULT (*ADI_PDMA_ENABLE_FN) (struct ADI_DMA_INFO *);

/* Prototype for DMA Interrupt handler functions */
typedef void (*ADI_DMA_DATA_INT_FN) (struct ADI_DMA_INFO *);

#pragma pack(4)
/* DMA register structure common to PDMA and MDMA */
typedef struct
{
    volatile uint32_t   NextDescPtr;    /* Pointer to next Initial Descriptor */
    volatile uint32_t   StartAddr;      /* Start address of current buffer */
    volatile uint32_t   Config;         /* Configuration register */
    volatile uint32_t   XCount;         /* XCOUNT - Inner loop count start value in number of MSIZE data transfers*/
    volatile int32_t    XModify;        /* XMODIFY - Inner loop address increment, in bytes */
    volatile uint32_t   YCount;         /* YCOUNT - Outer loop count start value (2D only), in number of rows*/
    volatile int32_t    YModify;        /* YMODIFY - Outer loop address increment (2D only), in bytes*/
    uint32_t            Padding0;
    uint32_t            Padding1;
    volatile uint32_t   CurrDescPtr;    /* Current Descriptor Pointer */
    volatile uint32_t   PrevDescPtr;    /* Previous Initial Descriptor Pointer */
    volatile uint32_t   CurrAddr;       /* Current Address */
    volatile uint32_t   Status;         /* Status Register */
    volatile uint32_t   CurrXCount;     /* Current XCOUNT */
    volatile uint32_t   CurrYCount;     /* Current YCOUNT */
    uint32_t            Padding3;
    volatile uint32_t   BwLimit;        /* Bandwidth Limit Count */
    volatile uint32_t   CurrBwLimit;    /* Bandwidth Limit Count Current */
    volatile uint32_t   BwMonitor;      /* Bandwidth Monitor Count */
    volatile uint32_t   CurrBwMonitor;  /* Bandwidth Monitor Count Current */
} ADI_DMA_REGS;
#pragma pack()

/* DMA descriptor work unit for PDMA */
typedef struct
{
    volatile void       *pNextDesc;         /* Pointer to next Descriptor */
    volatile void       *pStartAddr;        /* Start address of current buffer */
    volatile uint32_t   Config;             /* Configuration register */
    volatile uint32_t   XCount;             /* XCOUNT - Inner loop count start value in number of MSIZE data transfers*/
    volatile int32_t    XModify;            /* XMODIFY - Inner loop address increment, in bytes */
    volatile uint32_t   YCount;             /* YCOUNT - Outer loop count start value (2D only), in number of rows*/
    volatile int32_t    YModify;            /* YMODIFY - Outer loop address increment (2D only), in bytes*/
    bool                bIsFree_BufDone;    /* For List mode - TRUE when this work unit free and ready to accept new values
                                               For Circular Mode - used to store the 'bBufferDoneCallback' configuration */
} ADI_PDMA_DESC_WORK_UNIT;

/* DMA descriptor registers for MDMA */
typedef struct
{
    volatile void       *pStartAddr;    /* Start address of current buffer */
    volatile uint32_t   Config;         /* Configuration register */
    volatile uint32_t   XCount;         /* XCOUNT - Inner loop count start value in number of MSIZE data transfers*/
    volatile int32_t    XModify;        /* XMODIFY - Inner loop address increment, in bytes */
    volatile uint32_t   YCount;         /* YCOUNT - Outer loop count start value (2D only), in number of rows*/
    volatile int32_t    YModify;        /* YMODIFY - Outer loop address increment (2D only), in bytes*/
} ADI_MDMA_DESC_REGS;

/* Structure to handle a MDMA Channel */
typedef struct ADI_MDMA_CHANNEL
{
    uint8_t                 BitWiseRegistryIndex;   /* Index to bitwise registry used to keep in track of open DMA channels */
    uint8_t                 BitWiseChannelNum;      /* Channel number for the DMA channel relative to Bitwise registry index
                                                       This value is used to locate the channel within the bitwise registry
                                                       used to keep in track of open DMA channels by DMA error info and
                                                       DMA error interrupt */
    uint8_t					DataSID;			    /* Data interrupt SID for this channel to identify the exact channel to configure */
    uint32_t                CommonConfig;           /* DMA Configuration values common to all transfer modes */
    ADI_MDMA_ENABLE_FN      pfEnableXfer;           /* Function to enable MDMA dataflow/data transfer */
    ADI_DMA_DATA_INT_FN     pfDataIntHandle;        /* Function to handle DMA data transfer interrupt for this channel
                                                       provided that interrupts are allowed for this channel */
    ADI_MDMA_DATA_XFER_MODE eXferMode;              /* MDMA channel Data transfer mode */
    ADI_MDMA_DESC_REGS      DescRegCache;           /* Local copy of DMA descriptor values for one shot and circular mode */
    struct ADI_DMA_INFO     *pDmaInfo;              /* Pointer to DMA info of this channel */
    void                    *pActiveHead;           /* Head of live queue */
    void                    *pActiveTail;           /* Tail of live queue */
    void                    *pStandbyHead;          /* Head of standby queue */
    void                    *pStandbyTail;          /* Tail of standby queue */
} ADI_MDMA_CHANNEL;

/* Structure to handle MDMA Stream */
typedef struct ADI_MDMA_STREAM
{
    bool                    bIsMemDma;              /* Always FALSE as this is not a Memory DMA channel */
    bool                    bIsXferInProgress;      /* TRUE when a DMA data transfer is currently in progress */
    ADI_CALLBACK            pfCallback;             /* Client supplied callback function */
    void                    *pCBParam;              /* Client supplied callback parameter */
    ADI_DMA_DATA_INT_FN     pfDataIntHandle;        /* Function to handle DMA data transfer interrupts for channel that allow interrupts */
    bool                    bIsCrcMode;             /* TRUE when the DMA stream is in CRC mode - Field set inside the CRC driver */
    ADI_MDMA_CHANNEL        Src;                    /* MDMA Channel instance memory to handle source channel */
    ADI_MDMA_CHANNEL        Dest;                   /* MDMA Channel instance memory to handle destination channel */
    struct ADI_MDMA_CHANNEL *pMDmaChannel;          /* Pointer to instance memory of the MDMA channel that allows interrupt */
    struct ADI_DMA_INFO     *pPairedDmaInfo;        /* Pointer to DMA info of the channel paired with MDMA channel that allows interrupt */
} ADI_MDMA_STREAM;

/* Structure to handle Peripheral DMA channel */
typedef struct ADI_PDMA_CHANNEL
{
    bool                    bIsMemDma;              /* Always FALSE as this is not a Memory DMA channel */
    bool                    bIsXferInProgress;      /* TRUE when a DMA data transfer is enabled and/or in progress */
    ADI_CALLBACK            pfCallback;             /* Client supplied callback function */
    void                    *pCBParam;              /* Client supplied callback parameter */
    ADI_DMA_DATA_INT_FN     pfDataIntHandle;        /* Function to handle DMA data transfer interrupts */
    bool                    bIsPaused;              /* Not used */
    bool                    bBufRepeat;             /* TRUE when buffer repeat (Loopback) is enabled */
    bool                    bIsStreaming;           /* TRUE when DMA Streaming is enabled */
    bool                    bCircularXfer;          /* TRUE when DMA is set for circular data transfer */
    bool                    bOneShotListXfer;       /* TRUE when DMA is set for One-shot or List data transfer */
    bool					bIsIntDisabled;			/* TRUE when the DMA interrupt is disabled */
    uint8_t                 BitWiseRegistryIndex;   /* Index to bitwise registry used to keep in track of open DMA channels */
    uint8_t                 BitWiseChannelNum;      /* Channel number for the DMA channel relative to Bitwise registry index
                                                       This value is used to locate the channel within the bitwise registry
                                                       used to keep in track of open DMA channels by DMA error info and
                                                       DMA error interrupt */
    uint32_t                CommonConfig;           /* DMA Configuration values common to all transfer modes */
    ADI_PDMA_ENABLE_FN      pfEnableXfer;           /* Function to enable DMA dataflow/data transfer */
    uint32_t                RemainSubBufs;          /* Only for circular mode: Number of Sub-buffers that are yet to be processed */
    void                    *pBufferAddr;           /* Only for Circular mode: Address of the most recent Sub-buffer that was processed */
    uint32_t                BytesPerSubBuf;         /* Only for circular mode: Number of bytes per sub-buffer */
    ADI_PDMA_DESC_WORK_UNIT WorkUnit0;              /* Local cache of DMA descriptor work unit */
    ADI_PDMA_DESC_WORK_UNIT WorkUnit1;              /* Local cache of DMA descriptor work unit */
    ADI_PDMA_DESC_WORK_UNIT *pActiveWorkUnit;       /* Pointer to Work Unit that is active and recently processed by DMA */
} ADI_PDMA_CHANNEL;

/* Union of data structures to handle DMA channel and stream */
typedef union ADI_DMA_CHANNEL
{
    ADI_PDMA_CHANNEL        PDMA;   /* Peripheral DMA instance */
    ADI_MDMA_STREAM         MDMA;   /* Memory DMA instance */
} ADI_DMA_CHANNEL;

/* Structure to hold DMA channel specific information */
typedef struct ADI_DMA_INFO
{
    const uint8_t           DataIntSID;     /* Data Interrupt source ID of this DMA Channel */
/* ADSP-CM402/CM403/CM407/CM408 Specific */
#if defined(ADI_ADSP_CM40Z)
    const uint8_t           ErrorIntSID;    /* Error Interrupt source ID of this DMA Channel */
#endif
    volatile ADI_DMA_REGS 	*pReg;          /* DMA channel Registers */
    ADI_DMA_CHANNEL         *pChannel;      /* Address of the memory allocated to handle the DMA channel */
} ADI_DMA_INFO;

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif  /* __ADI_DMA_DEF_BF60X_H__ */

/*****/

/*@}*/

