/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_mdma_bf60x.c
 * @brief     Memory DMA Manager for ADSP-BF60x processor family.
 * @version:  $Revision: 22947 $
 * @date:     $Date: 2013-09-03 16:42:55 -0400 (Tue, 03 Sep 2013) $
 *
 * @details
 *            This is the the primary source file for Memory DMA Manager Service
 *            for ADSP-BF60x processor.
 *
 */

/** @addtogroup MDMA_Manager_bf60x
 *  @{

    Memory DMA Manager for ADSP-BF60x family of processors

- Memory DMA open/close/query APIs
	- adi_mdma_Open()
	- adi_mdma_Close()
	- adi_mdma_IsCopyInProgress()

- Memory copy APIs
	- adi_mdma_CopyCircular1D()
	- adi_mdma_Copy1D()
	- adi_mdma_Copy2D()
	- adi_mdma_CopyQueueList()
	- adi_mdma_Copy1DArray()
	- adi_mdma_Copy2DArray()

Memory DMA copy starts soon after the DMA manager is provided with a valid
source and destination buffer/descriptors. The source and destination buffer
can be submitted using a single Memory copy API or with any two APIs. When
two APIs are used to submit the buffers, the buffer information specific to
the channel that is not used in that API should be passed as NULL, and the
source/destination buffers can be submitted in any order.

<b>Example Scenario 1:</b>

Consider a scenario where the source is a big linear buffer that needs to be
copied to randomly located multiple buffers whose collective size equals the
source buffer. In such case, the source buffer can be submitted using
adi_mdma_Copy1D() function and the destination can submitted as a chain of
list descriptors that can be submitted using adi_mdma_CopyQueueList() or
array descriptor submitted using adi_mdma_Copy1DArray() / adi_mdma_Copy2DArray().

<em>Code snippet for scenario 1:</em>

Submit source buffer
@code
adi_mdma_Copy1D (<MDMA Stream Handle>,
                 <Destination address as NULL>,
                 <A valid source buffer address>,
                 <Memory copy size in ADI_DMA_MSIZE>,
                 <Number of elements in source buffer>);
@endcode

Submit destination buffer
@code
adi_mdma_CopyQueueList (<MDMA Stream Handle>,
                        <A valid destination descriptor chain>,
                        <Source descriptor address as NULL>);
@endcode

<b>Example Scenario 2:</b>

A memory mapped location as source and a linear buffer as destination

<em>Code snippet for scenario 2:</em>
Submit source buffer
@code
adi_mdma_CopyCircular1D (<MDMA Stream Handle>,
                         <Address of the memory mapped location which is the source>,
                         <Element size to be read, which is 1 for memory mapped location>,
                         <Element width in ADI_DMA_MSIZE>,
                         <'true' to indicate this configuration is for source channel>);
@endcode

Submit destination buffer
@code
adi_mdma_Copy1D (<MDMA Stream Handle>,
                 <A valid destination buffer address>,
                 <Source buffer address as NULL>,
                 <Memory copy size in ADI_DMA_MSIZE>,
                 <Number of elements in source buffer>);
@endcode

 */

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_1:" ")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_8_5:" ")
#pragma diag(suppress:misra_rule_8_7:"Objects shall be defined at block scope if they are only accessed from within a single function")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#pragma diag(suppress:misra_rule_10_3:" ")
#pragma diag(suppress:misra_rule_11_3:"Allow Type-casting")
#pragma diag(suppress:misra_rule_11_4:"Allow Type-casting")
#pragma diag(suppress:misra_rule_11_5:" ")
#pragma diag(suppress:misra_rule_12_7:" ")
#pragma diag(suppress:misra_rule_12_8:" ")
#pragma diag(suppress:misra_rule_13_7:"Allow 'assert' function to validate memory size")
#pragma diag(suppress:misra_rule_14_7:" ")
#pragma diag(suppress:misra_rule_16_7:" ")
#pragma diag(suppress:misra_rule_17_4:" ")
#endif /* _MISRA_RULES */

/*==============  D E F I N E S  ===============*/

/* DMA Config register value for 1D list descriptor for Source channel */
#define ADI_MDMA_DMA_CFG_SRC_LLIST_1D       (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_FETCH05 | ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D list descriptor for Source channel */
#define ADI_MDMA_DMA_CFG_SRC_LLIST_2D       (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_FETCH07 | ENUM_DMA_CFG_EN)
/* DMA Config register value for 1D list descriptor for Destination channel */
#define ADI_MDMA_DMA_CFG_DEST_LLIST_1D      (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_FETCH05 | ENUM_DMA_CFG_WRITE | ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D list descriptor for Destination channel */
#define ADI_MDMA_DMA_CFG_DEST_LLIST_2D      (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_FETCH07 | ENUM_DMA_CFG_WRITE | ENUM_DMA_CFG_EN)
/* DMA Config register Mask for stop flow mode */
#define ADI_MDMA_DMA_CFG_STOP_MODE_MASK     (BITM_DMA_CFG_TWOD | BITM_DMA_CFG_INT | BITM_DMA_CFG_MSIZE | BITM_DMA_CFG_PSIZE | BITM_DMA_CFG_WNR | BITM_DMA_CFG_EN)
/* DMA Config register Mask for Array and queue mode */
#define ADI_MDMA_DMA_CFG_ARRAY_Q_MODE_MASK  (BITM_DMA_CFG_MSIZE | BITM_DMA_CFG_PSIZE | BITM_DMA_CFG_TWAIT | BITM_DMA_CFG_TRIG)
/* DMA Config register value for 1D Array descriptor */
#define ADI_MDMA_DMA_CFG_1D_ARRAY           (ENUM_DMA_CFG_DSCARRAY | ENUM_DMA_CFG_FETCH04 | ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D Array descriptor */
#define ADI_MDMA_DMA_CFG_2D_ARRAY           (ENUM_DMA_CFG_DSCARRAY | ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_FETCH06 | ENUM_DMA_CFG_EN)
/* DMA Config register value for 1D stop flow mode */
#define ADI_MDMA_DMA_CFG_1D_STOP_FLOW      	(ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D stop flow mode */
#define ADI_MDMA_DMA_CFG_2D_STOP_FLOW      	(ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_EN)

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/

/* Resets MDMA Stream instance */
static void ResetMDmaStream (ADI_MDMA_STREAM    *pStream);
/* Resets a MDMA channel instance */
static void ResetMDmaChannel (ADI_MDMA_CHANNEL  *pMDmaChannel);

/* Gets maximum PSize configuration value */
static uint32_t GetMaxPSize(
    uint32_t        Status,
    uint32_t        MSize);

/* Gets maximum MSize and PSize configuration for CRC operation */
static void GetMaxMSizePSize(
    void                *pMemDest,
    void                *pMemSrc,
    uint32_t            Status,
    uint32_t            *pMSize,
    uint32_t            *pElementCount,
    uint32_t            *pMSizePSizeCfg);

/* Prepares a MDMA Channel for one-shot transfer */
static ADI_DMA_RESULT MDmaPrepareOneShot(
    ADI_DMA_INFO const              *pDmaInfo,
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    void                            *pStartAddress,
    uint32_t                        XCount,
    int32_t                         XModify,
    uint32_t                        YCount,
    int32_t                         YModify,
    uint32_t                        DmaConfig);

/* Loads DMA registers form values in descriptor cache for One shot/Circular data transfer */
static void MDmaLoadRegsOneShotCircXfer (ADI_MDMA_CHANNEL const *pMDmaChannel);

/* Prepares 1D Array for MDMA transfer */
static ADI_DMA_RESULT MDmaPrepareArray1D(
    ADI_DMA_INFO                    *pDmaInfo,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pDescArray,
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint16_t                        NumEntries,
    uint32_t                        DmaDirection);

/* Prepares 2D Array for MDMA transfer */
static ADI_DMA_RESULT MDmaPrepareArray2D(
    ADI_DMA_INFO                    *pDmaInfo,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pDescArray,
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint16_t                        NumEntries,
    uint32_t                        DmaDirection);

/* Enables One shot/Circular data transfer of a MDMA channel */
static void MDmaEnableOneShotCircXfer (
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint32_t                        *pConfig,
    bool                            bEnableInt);

/* Enables One shot data transfer for CRC operation */
static ADI_DMA_RESULT MDmaCrcModeOneShotXfer (ADI_DMA_INFO   *pDmaInfo);

/* Enables List Descriptor data transfer of a MDMA channel */
static void MDmaEnableListDescXfer (
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint32_t                        *pConfig,
    bool                            bEnableInt);

/* Enables 1D/2D Array data transfer of a MDMA channel */
static void MDmaEnableArrayXfer (
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint32_t                        *pConfig,
    bool                            bEnableInt);

/* DMA Interrupt handler for One-shot data transfer */
static void MDmaOneShotXferIntHandler (ADI_DMA_INFO *pDmaInfo);
/* DMA Interrupt handler for List descriptor data transfer */
static void MDmaListDescXferIntHandler (ADI_DMA_INFO *pDmaInfo);
/* DMA Interrupt handler for Array descriptor data transfer */
static void MDmaArrayXferIntHandler (ADI_DMA_INFO *pDmaInfo);

/* Updates transfer status of a MDMA stream */
static ADI_DMA_RESULT MDmaUpdateXferStatus (ADI_DMA_INFO    *pDmaInfo);

/* Finds the next List descriptor that would generate an interrupt */
static void MDmaGetNextListDescToInterrupt (
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    ADI_DMA_DESC_LIST_MODE          *pProcessedDesc);

/* Moves descriptors from standby to active queue */
static void MoveStandbyToActive (ADI_MDMA_CHANNEL    *pMDmaChannel);

/* Load DMA Registers with next descriptor in chain */
static void MDmaListXferLoadDmaRegs (
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint32_t                        *pConfig);

/* Initialises a MDMA channel instance for List Mode */
static void InitForListMode (ADI_MDMA_CHANNEL   *pMDmaChannel);

/* Disable MDMA transfer */
static void DisableXfer (ADI_DMA_INFO *pDmaInfo);

/* Prepares List mode descriptor for source channel queuing */
static void PrepareSrcListDesc (
    ADI_DMA_DESC_LIST_MODE          *pWorkingDesc,
    ADI_DMA_DESC_LIST_MODE          *pWorkingDestDesc,
    ADI_MDMA_DATA_XFER_MODE         eDestXferMode);

/* Prepares List mode descriptor for destination channel queuing */
static void PrepareDestListDesc (ADI_DMA_DESC_LIST_MODE		*pWorkingDesc);

/* Checks if the supplied descriptor is last in chain */
static void IsLastInList (
    ADI_DMA_DESC_LIST_MODE          *pWorkingDesc,
    ADI_DMA_DESC_LIST_MODE          **pLastDesc);

/* Add the descriptor list to chain */
static void AddToChain (
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    void                            *pFirstDesc,
    void                            *pLastDesc);

/* Disable interrupt for source channel list descriptors */
static void DisableSrcListInt (ADI_DMA_DESC_LIST_MODE   *pWorkingDesc);

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validate MDMA Stream handle */
static ADI_DMA_RESULT ValidateStreamHandle (ADI_DMA_STREAM_HANDLE   hStream);

/* Validate Descriptor List */
static ADI_DMA_RESULT ValidateDescList (
    ADI_DMA_INFO                	*pDmaInfo,
    ADI_DMA_DESC_LIST_MODE          *pDestDesc,
    ADI_DMA_DESC_LIST_MODE          *pSrcDesc);

/* Validate Source and Destination DMA descriptor configuration */
static ADI_DMA_RESULT ValidateStreamDesc (ADI_DMA_INFO  *pDmaInfo);

/* Validate Descriptor 1D Array */
static ADI_DMA_RESULT ValidateDescArray1D (
    ADI_DMA_INFO                    *pDmaInfo,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pDescArray,
    uint16_t                        NumEntries,
    uint32_t                        DmaDirection);

/* Validate Descriptor 2D Array */
static ADI_DMA_RESULT ValidateDescArray2D (
    ADI_DMA_INFO                    *pDmaInfo,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pDescArray,
    uint16_t                        NumEntries,
    uint32_t                        DmaDirection);

#endif /* ADI_DEBUG */

/*=============  C O D E  =============*/

/*=============  L O C A L   F U N C T I O N S =============*/

/*
 * Resets MDMA Stream instance.
 *
 * Parameters
 *  - [in]  pStream   Pointer to MDMA stream instance to work on.
 *
 * Returns
 *  - None
 *
 */
static void ResetMDmaStream (ADI_MDMA_STREAM    *pStream)
{
    /* Data transfer is not in progress */
    pStream->bIsXferInProgress  = false;
    /* Function to handle DMA Data Interrupt is yet to be allocated */
    pStream->pfDataIntHandle    = NULL;
    /* Reset Destination DMA channel registers */
    ResetDmaRegs ((pStream->Dest.pDmaInfo)->pReg);
    /* Reset Source DMA channel registers */
    ResetDmaRegs ((pStream->Src.pDmaInfo)->pReg);
    /* Reset the Destination channel instance */
    ResetMDmaChannel (&pStream->Dest);
    /* Reset the Source channel instance */
    ResetMDmaChannel (&pStream->Src);
}

/*
 * Resets a MDMA DMA channel instance.
 *
 * Parameters
 *  - [in]  pChannel   Pointer to MDMA DMA channel instance to work on.
 *
 * Returns
 *  - None
 *
 */
static void ResetMDmaChannel (ADI_MDMA_CHANNEL    *pMDmaChannel)
{
    /* Channel transfer mode */
    pMDmaChannel->eXferMode = ADI_MDMA_XFER_NONE;
    /* Reset Descriptor register cache */
    pMDmaChannel->DescRegCache.pStartAddr = NULL;
    pMDmaChannel->DescRegCache.Config     = 0;
    pMDmaChannel->DescRegCache.XCount     = 0;
    pMDmaChannel->DescRegCache.XModify    = 0;
    pMDmaChannel->DescRegCache.YCount     = 0;
    pMDmaChannel->DescRegCache.YModify    = 0;
    /* Flush Descriptor queue */
    pMDmaChannel->pActiveHead   = NULL;
    pMDmaChannel->pActiveTail   = NULL;
    pMDmaChannel->pStandbyHead  = NULL;
    pMDmaChannel->pStandbyTail  = NULL;
}

/*
 * Gets maximum MSize and PSize configuration for CRC operation.
 *
 * Parameters
 *  - [in]      pMemDest        Destination buffer address.
 *  - [in]      pMemSrc         Source buffer address.
 *  - [in]      Status          DMA channel status register value.
 *  - [in/out]  pMSize          Location to hold MSize value
 *  - [in/out]  pElementCount   Location to hold number of elements in a buffer
 *  - [out]     pMSizePSizeCfg  MSize/PSize configuration value
 *
 * Returns
 *  - None
 *
 */
static void GetMaxMSizePSize(
    void                *pMemDest,
    void                *pMemSrc,
    uint32_t            Status,
    uint32_t            *pMSize,
    uint32_t            *pElementCount,
    uint32_t            *pMSizePSizeCfg)
{
    /* Variables to calculate MSize */
    uint32_t    ElementCountInBytes, MSize, Mask, Value;

    /* Initialize Mask */
    Mask = ADI_DMA_ADDR_VALID_MASK;

    /* Get byte equivalent of MSize and calculate Element count in bytes */
    ElementCountInBytes = (*pElementCount * (1 << *pMSize));

    /* Calculate the value that will be used to get Max MSize */
    Value = (ElementCountInBytes | (uint32_t)pMemDest | (uint32_t)pMemSrc);

    /* FOR (Valid MSize is reached or MSize is zero) */
    for (MSize = ADI_DMA_MAX_MSIZE_VAL; MSize > 0 ; MSize--)
    {
        /* Validate the OR'd result of element count and buffer address */
        if ((Value & Mask) == 0u)
        {
            break;
        }

        /* Update Mask */
        Mask >>= 1;
    }

    /* Get the PSize Configuration */
    *pMSizePSizeCfg = GetMaxPSize (Status, MSize);
    /* Append the MSize Configuration */
    *pMSizePSizeCfg |= (MSize << BITP_DMA_CFG_MSIZE);

    /* Update MSize to be used */
    *pMSize = MSize;
    /* Update Element count to be used */
    *pElementCount = (ElementCountInBytes >> MSize);
}

/*
 * Gets maximum PSize configuration value.
 *
 * Parameters
 *  - [in]  Status          DMA channel status register value.
 *  - [in]  MSize           MSize value to be used for the DMA transfer.
 *
 * Returns
 *  - PSize configuration value.
 *
 */
static uint32_t GetMaxPSize(
    uint32_t        Status,
    uint32_t        MSize)
{
    /* Location to save maximum PSize and PSize to be used */
    uint32_t    MaxPSize, PSize;

    /* Calculate Maximum PSize */
    MaxPSize = ((Status & BITM_DMA_STAT_PBWID) >> BITP_DMA_STAT_PBWID);

    /* IF (MSize less than Max PSize) */
    if (MSize < MaxPSize)
    {
        /* Limit PSize value to MSize */
        PSize = MSize;
    }
    /* ELSE (MSize greater than PSize) */
    else
    {
        PSize = MaxPSize;
    }

    /* Return PSize configuration */
    return (PSize << BITP_DMA_CFG_PSIZE);
}

/*
 * Prepares a MDMA Channel for one-shot transfer.
 *
 * Parameters
 *  - [in]  pDmaInfo        DMA Info to work on.
 *  - [in]  pMDmaChannel    MDMA Channel instance to work on.
 *  - [in]  pStartAddress   Start address of the data buffer to be processed.
 *  - [in]  XCount          Inner loop count start value in number of
 *                          DMA data bus width (MSIZE) transfers.
 *  - [in]  XModify         Inner loop address increment in bytes.
 *  - [in]  YCount          Outer loop count start value in number of rows.
 *  - [in]  YModify         Outer loop address increment in bytes.
 *  - [in]  DmaConfig       Location to calculate DMA Config.
 *
 * Returns
 *  - ADI_DMA_SUCCESS: Successfully submitted buffer.
 *  - ADI_DMA_ALREADY_RUNNING: MDMA Stream is already in the middle of a data copy
 *  - ADI_DMA_XFER_TYPE_ALREADY_SET: Data transfer type is already set
 *
 */
static ADI_DMA_RESULT MDmaPrepareOneShot(
    ADI_DMA_INFO const          *pDmaInfo,
    ADI_MDMA_CHANNEL            *pMDmaChannel,
    void                        *pStartAddress,
    uint32_t                    XCount,
    int32_t                     XModify,
    uint32_t                    YCount,
    int32_t                     YModify,
    uint32_t                    DmaConfig)
{
    /* Maximum Peripheral bus width supported */
    uint32_t    MaxPBWidth;

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pDmaInfo->pChannel->MDMA.bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* IF (Transfer mode of the DMA channel is other than idle/none) */
    if (pMDmaChannel->eXferMode != ADI_MDMA_XFER_NONE)
    {
        /* Report failure as transfer mode is already set for this channel */
        return (ADI_DMA_XFER_TYPE_ALREADY_SET);
    }

    /* Reset Memory DMA Channel instance memory */
    ResetMDmaChannel (pMDmaChannel);
    /* Set DMA Transfer mode as One-shot */
    pMDmaChannel->eXferMode = ADI_MDMA_XFER_ONE_SHOT;
    /* Update Enable data transfer function pointer */
    pMDmaChannel->pfEnableXfer = &MDmaEnableOneShotCircXfer;
    /* Update function pointer to handle DMA data interrupt */
    pMDmaChannel->pfDataIntHandle = &MDmaOneShotXferIntHandler;

    /* Save the supplied values to Descriptor cache */
    pMDmaChannel->DescRegCache.pStartAddr = pStartAddress;
    pMDmaChannel->DescRegCache.XCount     = XCount;
    pMDmaChannel->DescRegCache.XModify    = XModify;
    pMDmaChannel->DescRegCache.YCount     = YCount;
    pMDmaChannel->DescRegCache.YModify    = YModify;

    /* IF (2D transfer) */
    if (YCount > 0)
    {
        /* Enable 2D transfer */
        DmaConfig |= ENUM_DMA_CFG_ADDR2D;
    }

    /* Save the config register calculated for this transfer */
    pMDmaChannel->DescRegCache.Config = DmaConfig;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/*
 * Loads DMA registers form values in descriptor cache for One shot/Circular data transfer.
 *
 * Parameters
 *  - [in]  pMDmaChannel        Pointer to DMA Channel instance memory to work on
 *
 * Returns
 *  - None
 *
 */
static void MDmaLoadRegsOneShotCircXfer (ADI_MDMA_CHANNEL  const  *pMDmaChannel)
{
    /* Loads DMA Registers from the descriptor cache */
    pMDmaChannel->pDmaInfo->pReg->StartAddr = (uint32_t) pMDmaChannel->DescRegCache.pStartAddr;
    pMDmaChannel->pDmaInfo->pReg->XCount    = pMDmaChannel->DescRegCache.XCount;
    pMDmaChannel->pDmaInfo->pReg->XModify   = pMDmaChannel->DescRegCache.XModify;
    pMDmaChannel->pDmaInfo->pReg->YCount    = pMDmaChannel->DescRegCache.YCount;
    pMDmaChannel->pDmaInfo->pReg->YModify   = pMDmaChannel->DescRegCache.YModify;
}

/*
 * Enables One shot/Circular data transfer of a MDMA channel.
 *
 * Parameters
 *  - [in]  pMDmaChannel        Pointer to DMA Channel instance memory to work on.
 *  - [out] pConfig             Pointer to location to store the config register value of the
 *                              DMA channel whose data transfer is to be enabled.
 *  - [in]  bEnableInt          TRUE to enable interrupt on transfer completion for this channel.
 *
 * Returns
 *  - None
 *
 */
static void MDmaEnableOneShotCircXfer (
    ADI_MDMA_CHANNEL    *pMDmaChannel,
    uint32_t            *pConfig,
    bool                bEnableInt)
{
    /* Load DMA Registers from the descriptor cache */
    MDmaLoadRegsOneShotCircXfer (pMDmaChannel);

    /* IF (Enable interrupt for this channel) */
    if (bEnableInt)
    {
        /* IF (2D transfer - YCount is non-zero) */
        if (pMDmaChannel->DescRegCache.YCount > 0)
        {
            /* Update DMA Config register in Descriptor cache to interrupt when YCOUNT is zero */
            pMDmaChannel->DescRegCache.Config |= ENUM_DMA_CFG_YCNT_INT;
        }
        /* ELSE (1D transfer) */
        else
        {
            /* Update DMA Config register in Descriptor cache to interrupt when XCOUNT is zero */
            pMDmaChannel->DescRegCache.Config |= ENUM_DMA_CFG_XCNT_INT;
        }
    }

    /* Pass the DMA Config register value */
    *pConfig = pMDmaChannel->DescRegCache.Config;
}

/*
 * Enables One-shot data transfer for CRC operation.
 *
 * Parameters
 *  - [in]  pDmaInfo    DMA Channel info to work on.
 *
 * Returns
 *  - None
 *
 */
static ADI_DMA_RESULT MDmaCrcModeOneShotXfer (ADI_DMA_INFO   *pDmaInfo)
{

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (Source or Destination channel not set for one-shot mode) */
    if ((pDmaInfo->pChannel->MDMA.Src.eXferMode != ADI_MDMA_XFER_ONE_SHOT) &&\
    	(pDmaInfo->pChannel->MDMA.Dest.eXferMode != ADI_MDMA_XFER_ONE_SHOT))
    {
        /* Return failure as function is not supported */
        return (ADI_DMA_FN_NOT_SUPPORTED);
    }

#endif /* ADI_DEBUG */

/* ADSP-CM402/CM403/CM407/CM408 processors */
#if defined(ADI_ADSP_CM40Z)

    /*
     * In CRC mode, make sure that interrupt is enabled only for the
     * DMA channels that are to used for the CRC operation.
     */

    /* Pointer to Destination DMA Channel Info */
    ADI_DMA_INFO    *pDestDmaInfo = pDmaInfo;
    /* Pointer to Source DMA Channel Info */
    ADI_DMA_INFO    *pSrcDmaInfo = pDmaInfo->pChannel->MDMA.pPairedDmaInfo;

    /* Uninstall Source channel interrupts */
    adi_int_UninstallHandler ((uint32_t)pSrcDmaInfo->DataIntSID);
    adi_int_UninstallHandler ((uint32_t)pSrcDmaInfo->ErrorIntSID);

    /* Uninstall Destination channel interrupts */
    adi_int_UninstallHandler ((uint32_t)pDestDmaInfo->DataIntSID);
    adi_int_UninstallHandler ((uint32_t)pDestDmaInfo->ErrorIntSID);

    /* IF (Destination DMA is set for one-shot transfer) */
    if (pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_ONE_SHOT)
    {
        /* Install DMA interrupt handler for destination channel */
        RegisterWithIntDispatcher (pDestDmaInfo);
    }
    /* ELSE (Source DMA must be set for one-shot transfer) */
    else
    {
        /* Install DMA interrupt handler for source channel */
        RegisterWithIntDispatcher (pSrcDmaInfo);
    }

#endif  /* ADI_ADSP_CM40Z  */

    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

    /*** Write to DMA Config registers to kick-off DMA data transfer ***/

    /* IF (Source DMA is set for one-shot transfer) */
    if (pDmaInfo->pChannel->MDMA.Src.eXferMode == ADI_MDMA_XFER_ONE_SHOT)
    {
        /* Load Source DMA Registers from the descriptor cache */
        MDmaLoadRegsOneShotCircXfer (&(pDmaInfo->pChannel->MDMA.Src));

        /* IF (Destination DMA not used) */
        if (pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_NONE)
        {
        	/* Enable DMA Done Interrupt */
        	pDmaInfo->pChannel->MDMA.Src.DescRegCache.Config |= ENUM_DMA_CFG_XCNT_INT;
        	/* Use the Source DMA interrupt handler function */
        	pDmaInfo->pChannel->MDMA.pfDataIntHandle = pDmaInfo->pChannel->MDMA.Src.pfDataIntHandle;
        	/* Source channel allows interrupt */
        	pDmaInfo->pChannel->MDMA.pMDmaChannel = &(pDmaInfo->pChannel->MDMA.Src);
        }

        /* Write to Source DMA Config register */
        (pDmaInfo->pChannel->MDMA.pPairedDmaInfo)->pReg->Config = pDmaInfo->pChannel->MDMA.Src.DescRegCache.Config;
    }

    /* IF (Destination DMA is set for one-shot transfer) */
    if (pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_ONE_SHOT)
    {
        /* Load Destination DMA Registers from the descriptor cache */
        MDmaLoadRegsOneShotCircXfer (&(pDmaInfo->pChannel->MDMA.Dest));
        /* Use the Destination DMA interrupt handler function */
        pDmaInfo->pChannel->MDMA.pfDataIntHandle = pDmaInfo->pChannel->MDMA.Dest.pfDataIntHandle;
        /* Destination channel allows interrupt */
        pDmaInfo->pChannel->MDMA.pMDmaChannel = &(pDmaInfo->pChannel->MDMA.Dest);

        /* Write to Destination DMA Config register with interrupt enabled */
        pDmaInfo->pReg->Config = (pDmaInfo->pChannel->MDMA.Dest.DescRegCache.Config | ENUM_DMA_CFG_XCNT_INT);
    }

    /* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

	/* return */
	return (ADI_DMA_SUCCESS);
}

/*
 * Initialises a MDMA channel instance for List Mode.
 *
 * Parameters
 *  - [in]  pMDmaChannel     MDMA channel instance to work on.
 *
 * Returns
 *  - None
 *
 */
static void InitForListMode (ADI_MDMA_CHANNEL    *pMDmaChannel)
{
    /* Reset Memory DMA Channel instance memory */
    ResetMDmaChannel (pMDmaChannel);
    /* Set DMA transfer type as large descriptor list mode */
    pMDmaChannel->eXferMode       = ADI_MDMA_XFER_LARGE_LIST;
    /* Update Enable data transfer function pointer */
    pMDmaChannel->pfEnableXfer    = &MDmaEnableListDescXfer;
    /* Update function pointer to handle DMA data interrupt */
    pMDmaChannel->pfDataIntHandle = &MDmaListDescXferIntHandler;
}

/*
 * Prepares List mode descriptor for source channel queuing.
 *
 * Parameters
 *  - [in] pWorkingDesc        Descriptor work unit to prepare.
 *  - [in] pWorkingDestDesc    Corresponding destination descriptor work unit.
 *  - [in] eDestXferMode       Destination channel transfer mode.
 *
 * Returns
 *  None
 *
 */
static void PrepareSrcListDesc (
    ADI_DMA_DESC_LIST_MODE      *pWorkingDesc,
    ADI_DMA_DESC_LIST_MODE      *pWorkingDestDesc,
    ADI_MDMA_DATA_XFER_MODE     eDestXferMode)
{
    /* Clear the config register fields other than
       word transfer size and trigger configuration */
    pWorkingDesc->Config &= ADI_MDMA_DMA_CFG_ARRAY_Q_MODE_MASK;

    /* IF (2D mode) */
    if (pWorkingDesc->YCount > 0)
    {
        /* 2D transfer */
        pWorkingDesc->Config |= ADI_MDMA_DMA_CFG_SRC_LLIST_2D;
    }
    /* ELSE (1D mode) */
    else
    {
        /* 1D transfer */
        pWorkingDesc->Config |= ADI_MDMA_DMA_CFG_SRC_LLIST_1D;
    }

    /* IF (Destination channel is operating in list mode) */
    if (pWorkingDestDesc != NULL)
    {
        /* IF (Callback enabled for the corresponding destination descriptor) */
        if (pWorkingDestDesc->bCallbackWhenDone)
        {
            /* Put this descriptor in stop mode */
            pWorkingDesc->Config &= ADI_MDMA_DMA_CFG_STOP_MODE_MASK;
        }
    }
    /* ELSE (Destination transfer mode other than list transfer) */
    else
    {
        /* IF (Destination transfer mode doesn't allow interrupt on buffer completion) */
        if ((eDestXferMode == ADI_MDMA_XFER_NONE) || (eDestXferMode == ADI_MDMA_XFER_CIRCULAR))
        {
			/* IF (Callback required for this descriptor work unit) */
			if (pWorkingDesc->bCallbackWhenDone)
			{
				/* Allow completion of work unit to generate a data interrupt */
				/* IF (2D transfer - YCount is non-zero) */
				if (pWorkingDesc->YCount > 0)
				{
					/* Callback after completing whole buffer */
					pWorkingDesc->Config |= ENUM_DMA_CFG_YCNT_INT;
				}
				/* ELSE (1D transfer) */
				else
				{
					/* Callback after completing whole buffer */
					pWorkingDesc->Config |= ENUM_DMA_CFG_XCNT_INT;
				}

				/* Put this descriptor in stop mode */
				pWorkingDesc->Config &= ~BITM_DMA_CFG_FLOW;
            }
        }
    }
}

/*
 * Prepares List mode descriptor for destination channel queuing.
 *
 * Parameters
 *  - [in] pWorkingDesc        Descriptor work unit to prepare.
 *
 * Returns
 *  - None
 *
 */
static void PrepareDestListDesc (ADI_DMA_DESC_LIST_MODE		*pWorkingDesc)
{
    /* Clear the config register fields other than
       word transfer size and trigger configuration */
    pWorkingDesc->Config &= ADI_MDMA_DMA_CFG_ARRAY_Q_MODE_MASK;

    /* IF (2D mode) */
    if (pWorkingDesc->YCount > 0)
    {
        /* 2D transfer */
        pWorkingDesc->Config |= ADI_MDMA_DMA_CFG_DEST_LLIST_2D;
    }
    /* ELSE (1D mode) */
    else
    {
        /* 1D transfer */
        pWorkingDesc->Config |= ADI_MDMA_DMA_CFG_DEST_LLIST_1D;
    }

    /* IF (Callback required for this descriptor work unit) */
    if (pWorkingDesc->bCallbackWhenDone)
    {
        /* Allow completion of work unit to generate a data interrupt */
		/* IF (2D transfer - YCount is non-zero) */
		if (pWorkingDesc->YCount > 0)
		{
			/* Callback after completing whole buffer */
			pWorkingDesc->Config |= ENUM_DMA_CFG_YCNT_INT;
		}
		/* ELSE (1D transfer) */
		else
		{
			/* Callback after completing whole buffer */
			pWorkingDesc->Config |= ENUM_DMA_CFG_XCNT_INT;
		}

		/* Put this descriptor in stop mode */
		pWorkingDesc->Config &= ~BITM_DMA_CFG_FLOW;
    }
}

/*
 * Checks if the supplied descriptor is last in chain.
 *
 * Parameters
 *  - [in] pWorkingDesc    Descriptor work unit to check.
 *  - [in] pLastDesc       Location to store address of the last work unit in chain
 *
 * Returns
 *  - None
 *
 */
static void IsLastInList (
    ADI_DMA_DESC_LIST_MODE      *pWorkingDesc,
    ADI_DMA_DESC_LIST_MODE      **pLastDesc)
{
    /* IF (This is the last descriptor work unit in chain) */
    if (pWorkingDesc->pNext == NULL)
    {
        /* Save the config value of this descriptor to reserved field
           in case we need it later when more descriptor work units are queued */
        pWorkingDesc->Reserved = (uint32_t) pWorkingDesc->Config;
        /* Put this descriptor in stop mode */
        pWorkingDesc->Config &= ADI_MDMA_DMA_CFG_STOP_MODE_MASK;
        /* Save the last descriptor address */
        *pLastDesc = pWorkingDesc;
    }
    /* ELSE (we've more descriptor work units in chain) */
    else
    {
        /* Update the reserved field with next descriptor address in chain */
        pWorkingDesc->Reserved = (uint32_t) pWorkingDesc->pNext;
    }
}

/*
 * Add the descriptor list to chain.
 *
 * Parameters
 *  - [in]  pMDmaChannel    MDMA channel instance to work on.
 *  - [in]  pFirstDesc      First descriptor in the supplied list.
 *  - [in]  pLastDesc       Last descriptor in the supplied list.
 *
 * Returns
 *  - None
 *
 */
static void AddToChain (
    ADI_MDMA_CHANNEL        *pMDmaChannel,
    void                    *pFirstDesc,
    void                    *pLastDesc)
{
    /* List Descriptor Work unit to work on */
    ADI_DMA_DESC_LIST_MODE  *pListDesc;

    /* IF (This is the first descriptor to be queued) */
    if (pMDmaChannel->pStandbyHead == NULL)
    {
        /* Add the supplied chain to queue */
        pMDmaChannel->pStandbyHead = pFirstDesc;
        pMDmaChannel->pStandbyTail = pLastDesc;
    }
    /* ELSE (Add this to last descriptor in queue) */
    else
    {
        /* type-cast to list descriptor */
        pListDesc = (ADI_DMA_DESC_LIST_MODE *) pMDmaChannel->pStandbyTail;

        /* Update the config register value of the last descriptor in active chain
           to Large descriptor transfer mode. This config value was stored in the
           Reserved field of the descriptor work unit */
        pListDesc->Config = pListDesc->Reserved;

        /* Add the supplied descriptor chain to queue */
        pListDesc->Reserved = (uint32_t) pFirstDesc;
        pMDmaChannel->pStandbyTail = pLastDesc;
    }
}

/*
 * Enables List Descriptor data transfer of a MDMA channel.
 *
 * Parameters
 *  - [in]  pMDmaChannel        Pointer to DMA Channel instance memory to work on.
 *  - [out] pConfig             Pointer to location to store the config register value of the
 *                              DMA channel whose data transfer is to be enabled.
 *  - [in]  bEnableInt          TRUE to enable interrupt on transfer completion for this channel.
 *
 * Returns
 *  - None
 *
 */
static void MDmaEnableListDescXfer (
    ADI_MDMA_CHANNEL    *pMDmaChannel,
    uint32_t            *pConfig,
    bool                bEnableInt)
{
    /* Moves descriptors from standby to active queue */
    MoveStandbyToActive (pMDmaChannel);

    /* IF (We're working on source channel) */
    if (pMDmaChannel == &(pMDmaChannel->pDmaInfo->pChannel->MDMA.Src))
    {
        /* IF (Do not interrupt on source channel) */
        if (!bEnableInt)
        {
            /* Disable interrupt for source channel list descriptors */
            DisableSrcListInt ((ADI_DMA_DESC_LIST_MODE *) pMDmaChannel->pActiveHead);
        }
    }

    /* Load the DMA Registers */
    MDmaListXferLoadDmaRegs (pMDmaChannel, pConfig);
}

/*
 * Disable interrupt for source channel list descriptors.
 *
 * Parameters:
 *  - [in] pWorkingDesc    Start of descriptor work unit.
 *
 * Returns
 *  - [in] None
 *
 */
static void DisableSrcListInt (
    ADI_DMA_DESC_LIST_MODE      *pWorkingDesc)
{
    /* DO (for all descriptors in source channel list) */
    while (pWorkingDesc != NULL)
    {
        /* Disable interrupt for this list descriptor */
        pWorkingDesc->Config &= ~BITM_DMA_CFG_INT;
        /* Move to next descriptor in chain */
        pWorkingDesc = (ADI_DMA_DESC_LIST_MODE *) pWorkingDesc->Reserved;
    }
}

/*
 * Prepares 1D Array for MDMA transfer.
 *
 * Parameters
 *  - [in]  pDmaInfo        DMA Channel info to work on.
 *  - [in]  pDescArray      Address of the first descriptor work unit in array.
 *  - [in]  pMDmaChannel    Pointer to MDMA Channel instance to work on.
 *  - [in]  NumEntries      Number of array entries.
 *  - [in]  DmaDirection    DMA Direction.
 *
 * Returns
 *  - ADI_DMA_SUCCESS: Success.
 *  - ADI_DMA_FN_NOT_PERMITTED: Function not permitted when the dataflow is enabled.
 *  - ADI_DMA_XFER_TYPE_ALREADY_SET: DMA Data transfer type is already set.
 *
 */
static ADI_DMA_RESULT MDmaPrepareArray1D(
    ADI_DMA_INFO                    *pDmaInfo,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pDescArray,
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint16_t                        NumEntries,
    uint32_t                        DmaDirection)
{
    /* 1D descriptor Array to work on */
    ADI_DMA_DESC_ARRAY_1D_MODE   *pWorkUnit1D = pDescArray;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Return code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* Validate 1D Descriptor Array */
    eResult = ValidateDescArray1D (pDmaInfo,
                                   pDescArray,
                                   NumEntries,
                                   DmaDirection);

    /* IF (Descriptor array is invalid) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    /* IF (MDMA is already running) */
    if (pDmaInfo->pChannel->MDMA.bIsXferInProgress)
    {
        return (ADI_DMA_FN_NOT_PERMITTED);
    }

    /* IF (DMA transfer mode is already set) */
    if (pMDmaChannel->eXferMode != ADI_MDMA_XFER_NONE)
    {
        return (ADI_DMA_XFER_TYPE_ALREADY_SET);
    }

    /* Reset Memory DMA Channel instance memory */
    ResetMDmaChannel (pMDmaChannel);
    /* Set DMA transfer type as 1D Array mode */
    pMDmaChannel->eXferMode       = ADI_MDMA_XFER_ARRAY_1D;
    /* Update Enable data transfer function pointer */
    pMDmaChannel->pfEnableXfer    = &MDmaEnableArrayXfer;
    /* Update function pointer to handle DMA data interrupt */
    pMDmaChannel->pfDataIntHandle = &MDmaArrayXferIntHandler;

    /* DO (for all descriptors in Array) */
    do
    {
        /* Clear the config register fields other than
           word transfer size and trigger configuration */
        pWorkUnit1D->Config &= ADI_MDMA_DMA_CFG_ARRAY_Q_MODE_MASK;
        /* Append DMA Direction */
        pWorkUnit1D->Config |= DmaDirection;

        /* IF (This is the last descriptor work unit in array) */
        if (NumEntries <= 1)
        {
            /* Update DMA Config for STOP flow mode */
            pWorkUnit1D->Config |= ADI_MDMA_DMA_CFG_1D_STOP_FLOW;
        }
        /* ELSE (we've more entries in the array) */
        else
        {
            /* Update DMA Config for 1D Array flow mode */
            pWorkUnit1D->Config |= ADI_MDMA_DMA_CFG_1D_ARRAY;
            /* Move to next work unit in array */
            pWorkUnit1D++;
        }

        /* Decement number of array entries to work on */
        NumEntries--;

    } while (NumEntries != 0u);

    /* Entering critical region, disable interrupts */
	adi_osal_EnterCriticalRegion();

	/* Save the supplied array address */
	pMDmaChannel->pStandbyHead = (void *) pDescArray;
	pMDmaChannel->pStandbyTail = (void *) pWorkUnit1D;

    /* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

	/* Return */
	return (ADI_DMA_SUCCESS);
}

/*
 * Prepares 2D Array for MDMA transfer.
 *
 * Parameters
 *  - [in]  pDmaInfo        DMA Channel info to work on.
 *  - [in]  pDescArray      Address of the first descriptor work unit in array.
 *  - [in]  pMDmaChannel    Pointer to MDMA Channel instance to work on.
 *  - [in]  NumEntries      Number of array entries.
 *  - [in]  DmaDirection    DMA Direction.
 *
 * Returns
 *  - ADI_DMA_SUCCESS: Success.
 *  - ADI_DMA_FN_NOT_PERMITTED: Function not permitted when the dataflow is enabled.
 *  - ADI_DMA_XFER_TYPE_ALREADY_SET: DMA Data transfer type is already set.
 *
 */
static ADI_DMA_RESULT MDmaPrepareArray2D(
    ADI_DMA_INFO                    *pDmaInfo,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pDescArray,
    ADI_MDMA_CHANNEL                *pMDmaChannel,
    uint16_t                        NumEntries,
    uint32_t                        DmaDirection)
{
    /* 2D descriptor Array to work on */
    ADI_DMA_DESC_ARRAY_2D_MODE   *pWorkUnit2D = pDescArray;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Return code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* Validate 1D Descriptor Array */
    eResult = ValidateDescArray2D (pDmaInfo,
                                   pDescArray,
                                   NumEntries,
                                   DmaDirection);

    /* IF (Descriptor array is invalid) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    /* IF (MDMA is already running) */
    if (pDmaInfo->pChannel->MDMA.bIsXferInProgress)
    {
        return (ADI_DMA_FN_NOT_PERMITTED);
    }

    /* IF (DMA transfer mode is already set) */
    if (pMDmaChannel->eXferMode != ADI_MDMA_XFER_NONE)
    {
        return (ADI_DMA_XFER_TYPE_ALREADY_SET);
    }

    /* Reset Memory DMA Channel instance memory */
    ResetMDmaChannel (pMDmaChannel);
    /* Set DMA transfer type as 2D Array mode */
    pMDmaChannel->eXferMode       = ADI_MDMA_XFER_ARRAY_2D;
    /* Update Enable data transfer function pointer */
    pMDmaChannel->pfEnableXfer    = &MDmaEnableArrayXfer;
    /* Update function pointer to handle DMA data interrupt */
    pMDmaChannel->pfDataIntHandle = &MDmaArrayXferIntHandler;

    /* DO (for all descriptors in Array) */
    do
    {
        /* Clear the config register fields other than
           word transfer size and trigger configuration */
        pWorkUnit2D->Config &= ADI_MDMA_DMA_CFG_ARRAY_Q_MODE_MASK;
        /* Append DMA Direction */
        pWorkUnit2D->Config |= DmaDirection;

        /* IF (This is the last descriptor work unit in array) */
        if (NumEntries <= 1)
        {
            /* Update DMA Config for STOP flow mode */
            pWorkUnit2D->Config |= ADI_MDMA_DMA_CFG_2D_STOP_FLOW;
        }
        /* ELSE (we've more entries in the array) */
        else
        {
            /* Update DMA Config for 2D Array flow mode */
            pWorkUnit2D->Config |= ADI_MDMA_DMA_CFG_2D_ARRAY;
            /* Move to next work unit in array */
            pWorkUnit2D++;
        }

        /* Decement number of array entries to work on */
        NumEntries--;

    } while (NumEntries != 0u);

    /* Entering critical region, disable interrupts */
	adi_osal_EnterCriticalRegion();

	/* Save the supplied array address */
	pMDmaChannel->pStandbyHead = (void *) pDescArray;
	pMDmaChannel->pStandbyTail = (void *) pWorkUnit2D;

    /* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

	/* Return */
	return (ADI_DMA_SUCCESS);
}

/*
 * Enables 1D/2D Array data transfer of a MDMA channel.
 *
 * Parameters
 *  - [in]  pMDmaChannel        Pointer to DMA Channel instance memory to work on.
 *  - [out] pConfig             Pointer to location to store the config register value of the
 *                              DMA channel whose data transfer is to be enabled.
 *  - [in]  bEnableInt          TRUE to enable interrupt on transfer completion for this channel.
 *
 * Returns
 *  - None
 *
 */
static void MDmaEnableArrayXfer (
    ADI_MDMA_CHANNEL    *pMDmaChannel,
    uint32_t            *pConfig,
    bool                bEnableInt)
{
    /* IF (Enable interrupt for this channel) */
    if (bEnableInt)
    {
    	/* Enable interrupt for the last work unit in array which is already configured in STOP flow mode */
        /* IF (2D transfer) */
        if (((((ADI_DMA_DESC_ARRAY_1D_MODE *) pMDmaChannel->pStandbyTail)->Config) & BITM_DMA_CFG_TWOD) == ENUM_DMA_CFG_ADDR2D)
        {
            /* Enable interrupt when YCOUNT reaches zero for the last work unit in chain */
    	    ((ADI_DMA_DESC_ARRAY_1D_MODE *) pMDmaChannel->pStandbyTail)->Config |= ENUM_DMA_CFG_YCNT_INT;
        }
        /* ELSE (1D transfer) */
        else
        {
            /* Enable interrupt when XCOUNT reaches zero for the last work unit in chain */
    	    ((ADI_DMA_DESC_ARRAY_1D_MODE *) pMDmaChannel->pStandbyTail)->Config |= ENUM_DMA_CFG_XCNT_INT;
        }
    }

    /* Load the First descriptor work unit address */
    pMDmaChannel->pDmaInfo->pReg->CurrDescPtr = (uint32_t) pMDmaChannel->pStandbyHead;
    /* Save the Config value */
    *pConfig = ((ADI_DMA_DESC_ARRAY_1D_MODE *) pMDmaChannel->pStandbyHead)->Config;
}

/*
 * DMA Interrupt handler for One-shot data transfer.
 *
 * Parameters
 *  - [in]  pDmaInfo  DMA Channel Info to which the interrupt belongs.
 *
 * Returns
 *  - None
 *
 */
static void MDmaOneShotXferIntHandler (ADI_DMA_INFO *pDmaInfo)
{
    /* Callback Argument */
	/* Get the callback argument (start address of the processed data buffer) */
    void    *pArg = (void *) (pDmaInfo->pChannel->MDMA.pMDmaChannel)->DescRegCache.pStartAddr;

    /* IF (Not in CRC mode) */
    if (pDmaInfo->pChannel->MDMA.bIsCrcMode == false)
    {
        /* Disable MDMA transfer */
        DisableXfer (pDmaInfo);
    }

    /* IF (We've a valid callback function) */
    if (pDmaInfo->pChannel->MDMA.pfCallback != NULL)
    {
    	/* Post callback */
    	/* Callback Argument as Address of data buffer processed */
    	(pDmaInfo->pChannel->MDMA.pfCallback) (pDmaInfo->pChannel->MDMA.pCBParam, ADI_DMA_EVENT_BUFFER_PROCESSED, pArg);
    }
}

/*
 * DMA Interrupt handler for List descriptor data transfer.
 *
 * Parameters
 *  - [in]  pDmaInfo  DMA Channel Info to which the interrupt belongs.
 *
 * Returns
 *  - None
 *
 */
static void MDmaListDescXferIntHandler (ADI_DMA_INFO *pDmaInfo)
{
    /* Config register values */
    uint32_t            SrcConfig, DestConfig;
    /* Callback Argument */
	/* Start address of the list descriptor that was processed */
    void                *pArg = (void *) (pDmaInfo->pChannel->MDMA.pMDmaChannel)->DescRegCache.pStartAddr;
    /* Pointer to Source MDMA channel instance memory */
    ADI_MDMA_CHANNEL    *pSrcChannel = &(pDmaInfo->pChannel->MDMA.Src);
    /* Pointer to Destination MDMA channel instance memory */
    ADI_MDMA_CHANNEL    *pDestChannel = &(pDmaInfo->pChannel->MDMA.Dest);

    /* Clear Paired channel interrupt sticky bits */
    (pDmaInfo->pChannel->MDMA.pPairedDmaInfo)->pReg->Status = ADI_DMA_CLEAR_INTERRUPTS;

    /* IF (This is the last descriptor in active chain) */
    if (((ADI_DMA_DESC_LIST_MODE *) pArg)->Reserved == 0)
    {
        /* Move Source channel descriptors from standby to active queue */
        MoveStandbyToActive (pSrcChannel);
        /* Move Destination channel descriptors from standby to active queue */
        MoveStandbyToActive (pDestChannel);
    }
     /* ELSE (we've more descriptor work units in chain) */
    else
    {
        /* Update Source channel active head */
        /* IF (Source MDMA channel is in list transfer mode) */
        if (pSrcChannel->eXferMode == ADI_MDMA_XFER_LARGE_LIST)
        {
            /* Purge the processed work units */
            pSrcChannel->pActiveHead = (void *) (((ADI_DMA_DESC_LIST_MODE *) pSrcChannel->DescRegCache.pStartAddr)->Reserved);
        }

        /* Update Destination channel active head */
        /* IF (Destination MDMA channel is in list transfer mode) */
        if (pDestChannel->eXferMode == ADI_MDMA_XFER_LARGE_LIST)
        {
            /* Purge the processed work units */
            pDestChannel->pActiveHead = (void *) (((ADI_DMA_DESC_LIST_MODE *) pDestChannel->DescRegCache.pStartAddr)->Reserved);
        }
    }

    /* IF (we've valid descriptor work unit to process) */
    if ((pDmaInfo->pChannel->MDMA.pMDmaChannel)->pActiveHead != NULL)
    {
        /* IF (Source channel is set for List transfer) */
        if (pSrcChannel->eXferMode == ADI_MDMA_XFER_LARGE_LIST)
        {
            /* Load Source DMA Registers */
            MDmaListXferLoadDmaRegs(pSrcChannel, &SrcConfig);
            /* Load Source config register */
            pSrcChannel->pDmaInfo->pReg->Config = SrcConfig;
        }

        /* IF (Destination channel is set for List transfer) */
        if (pDestChannel->eXferMode == ADI_MDMA_XFER_LARGE_LIST)
        {
            /* Load Destination DMA Registers */
            MDmaListXferLoadDmaRegs(pDestChannel, &DestConfig);
            /* Load Destination config register */
            pDestChannel->pDmaInfo->pReg->Config = DestConfig;
        }
    }
    /* ELSE (End of descriptor work units) */
    else
    {
        /* Disable MDMA transfer */
        DisableXfer (pDmaInfo);
    }

	/* IF (Callback enabled for the processed descriptor and we've a valid callback function) */
    if ((((ADI_DMA_DESC_LIST_MODE *) pArg)->bCallbackWhenDone) && (pDmaInfo->pChannel->MDMA.pfCallback != NULL))
    {
    	/* Post callback */
        /* Callback Argument as Address of processed List descriptor */
        (pDmaInfo->pChannel->MDMA.pfCallback) (pDmaInfo->pChannel->MDMA.pCBParam,
                                               ADI_DMA_EVENT_DESC_LIST_PROCESSED,
                                               pArg);
    }
}

/*
 * DMA Interrupt handler for Array descriptor data transfer.
 *
 * Parameters
 *  - [in]  pDmaInfo  DMA Channel Info to which the interrupt belongs.
 *
 * Returns
 *  - None
 *
 */
static void MDmaArrayXferIntHandler (ADI_DMA_INFO *pDmaInfo)
{
    /* Callback Argument */
	/* Get the callback argument (start address of the processed array) */
    void        *pArg = (void *) (pDmaInfo->pChannel->MDMA.pMDmaChannel)->pStandbyHead;
    /* Callback Event */
    uint32_t    Event = (uint32_t) (pDmaInfo->pChannel->MDMA.pMDmaChannel)->eXferMode;

    /* Disable MDMA transfer */
    DisableXfer (pDmaInfo);

    /* IF (We've a valid callback function) */
    if (pDmaInfo->pChannel->MDMA.pfCallback != NULL)
    {
    	/* Post callback */
    	/* Callback Argument as Address processed array */
        (pDmaInfo->pChannel->MDMA.pfCallback) (pDmaInfo->pChannel->MDMA.pCBParam, Event, pArg);
    }
}

/*
 * Updates transfer status of a DMA stream.
 *
 * Parameters
 *  - [in]  pDmaInfo    DMA Channel info to work on.
 *
 * Returns
 *  - ADI_DMA_SUCCESS: Successfully updated transfer status.
 *
 */
static ADI_DMA_RESULT MDmaUpdateXferStatus (ADI_DMA_INFO    *pDmaInfo)
{
    /* Flag to enable interrupt for source or destination channel */
    bool        bIntOnSrc;
    /* Location to store Destination channel config value to use */
    uint32_t    DestConfig;
    /* Location to store Source channel config value to use */
    uint32_t    SrcConfig;

    /* IF (Data transfer already in progress) */
    if (pDmaInfo->pChannel->MDMA.bIsXferInProgress)
    {
        /* Do nothing, just return */
        return (ADI_DMA_SUCCESS);
    }

    /* IF (Destination and Source DMA's have a valid transfer type) */
    if ((pDmaInfo->pChannel->MDMA.Src.eXferMode != ADI_MDMA_XFER_NONE) &&\
        (pDmaInfo->pChannel->MDMA.Dest.eXferMode != ADI_MDMA_XFER_NONE))
    {

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

        /* Return code */
        ADI_DMA_RESULT  eResult;

        /* Validate Source and Destination DMA descriptor configuration */
        eResult = ValidateStreamDesc (pDmaInfo);

        /* IF (Failed to validate Source and Destination descriptor configuration) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

#endif /* ADI_DEBUG */

        /* IF (Destination is in circular mode) */
        if (pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_CIRCULAR)
        {
            /* Interrupt on source channel */
            bIntOnSrc = true;

            /* Use the Source DMA interrupt handler function */
            pDmaInfo->pChannel->MDMA.pfDataIntHandle = pDmaInfo->pChannel->MDMA.Src.pfDataIntHandle;
            /* Source channel allows interrupt */
            pDmaInfo->pChannel->MDMA.pMDmaChannel = &(pDmaInfo->pChannel->MDMA.Src);
        }
        /* ELSE (Interrupts should be generated only by the Destination channel) */
        else
        {
            /* Interrupt on destination channel */
            bIntOnSrc = false;

            /* Use the Destination DMA interrupt handler function */
            pDmaInfo->pChannel->MDMA.pfDataIntHandle = pDmaInfo->pChannel->MDMA.Dest.pfDataIntHandle;
            /* Destination channel allows interrupt */
            pDmaInfo->pChannel->MDMA.pMDmaChannel = &(pDmaInfo->pChannel->MDMA.Dest);
        }

        /* Enable Source DMA for data transfer */
        pDmaInfo->pChannel->MDMA.Src.pfEnableXfer (&(pDmaInfo->pChannel->MDMA.Src), &SrcConfig, bIntOnSrc);
        /* Enable Destination DMA for data transfer */
        pDmaInfo->pChannel->MDMA.Dest.pfEnableXfer (&(pDmaInfo->pChannel->MDMA.Dest), &DestConfig, !bIntOnSrc);

        /* Entering critical region, disable interrupts */
	    adi_osal_EnterCriticalRegion();

        /*** Write to DMA Config registers to kick-off DMA data transfer ***/

        /* Write to Source DMA Config register first */
        (pDmaInfo->pChannel->MDMA.pPairedDmaInfo)->pReg->Config = SrcConfig;
        /* Write to Destination DMA Config register to kick-off DMA data transfer */
        pDmaInfo->pReg->Config = DestConfig;

        /* Update DMA data transfer status flag */
        pDmaInfo->pChannel->MDMA.bIsXferInProgress = true;

        /* Re-enable interrupts */
	    adi_osal_ExitCriticalRegion();
    }

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/*
 * Moves descriptors from standby to active queue.
 *
 * Parameters
 *  - [in]  pChannel    MDMA Channel Instance to work on.
 *
 * Returns
 *  - None
 *
 */
static void MoveStandbyToActive (ADI_MDMA_CHANNEL    *pMDmaChannel)
{
    /* Move the standby queue to active queue */
    pMDmaChannel->pActiveHead  = pMDmaChannel->pStandbyHead;
    pMDmaChannel->pActiveTail  = pMDmaChannel->pStandbyTail;
    pMDmaChannel->pStandbyHead = NULL;
    pMDmaChannel->pStandbyTail = NULL;

    /* IF (Channel is in list transfer mode) */
    if ((pMDmaChannel->eXferMode == ADI_MDMA_XFER_LARGE_LIST) && (pMDmaChannel->pActiveTail != NULL))
    {
        /* Clear the reserved field to indicate end of descriptor chain */
    	(((ADI_DMA_DESC_LIST_MODE *) pMDmaChannel->pActiveTail)->Reserved) = 0;
    }
}

/*
 * Load DMA Registers with next descriptor in chain.
 *
 * Parameters
 *  - [in] pMDmaChannel     MDMA Channel instance to work on.
 *  - [in] pConfig          DMA Configuration register value.
 *
 * Returns
 *  - None
 *
 */
static void MDmaListXferLoadDmaRegs (
    ADI_MDMA_CHANNEL    *pMDmaChannel,
    uint32_t            *pConfig)
{
    /* Pointer to active list descriptor work unit */
    ADI_DMA_DESC_LIST_MODE  *pListDesc = (ADI_DMA_DESC_LIST_MODE *) pMDmaChannel->pActiveHead;

    /* 'DescRegCache.pStartAddr' field is used to hold the most recent descriptor chain that generated an interrupt */
    /* Clear this field to start processing a new descriptor chain */
    pMDmaChannel->DescRegCache.pStartAddr = NULL;

    /* FOR (all descriptors in list) */
    for (; pListDesc != NULL ; pListDesc = pListDesc->pNext)
    {
        /* IF (This descriptor configured in stop flow mode) */
        if ((pListDesc->Config & BITM_DMA_CFG_FLOW) == ENUM_DMA_CFG_STOP)
        {
            /* Save the address of next descriptor work unit configured in stop flow mode */
            pMDmaChannel->DescRegCache.pStartAddr = (void *) pListDesc;
            break;
        }
    }

    /* Get the descriptor address to load */
    pListDesc = (ADI_DMA_DESC_LIST_MODE *) pMDmaChannel->pActiveHead;

    /* IF (Descriptor to load is configured for list flow mode) */
    if ((pListDesc->Config & BITM_DMA_CFG_FLOW) == ENUM_DMA_CFG_DSCLIST)
    {
        /* Load Next Desc Ptr */
        pMDmaChannel->pDmaInfo->pReg->NextDescPtr = (uint32_t) pMDmaChannel->pActiveHead;
    }
    /* ELSE (Descriptor to load must be configured in stop flow mode) */
    else
    {
        pMDmaChannel->pDmaInfo->pReg->StartAddr = (uint32_t) pListDesc->pStartAddress;
        pMDmaChannel->pDmaInfo->pReg->XCount    = pListDesc->XCount;
        pMDmaChannel->pDmaInfo->pReg->XModify   = pListDesc->XModify;
        pMDmaChannel->pDmaInfo->pReg->YCount    = pListDesc->YCount;
        pMDmaChannel->pDmaInfo->pReg->YModify   = pListDesc->YModify;
    }

    /* Save the DMA configuration value */
    *pConfig = pListDesc->Config;
}

/*
 * Disable MDMA transfer.
 *
 * Parameters
 *  - [in]  pDmaInfo  DMA Channel Info to work on.
 *
 * Returns
 *  - None
 *
 */
static void DisableXfer (ADI_DMA_INFO *pDmaInfo)
{
    /* Disable the MDMA channel */
    pDmaInfo->pReg->Config = 0;
    /* Disable Paired channel DMA */
    (pDmaInfo->pChannel->MDMA.pPairedDmaInfo)->pReg->Config = 0;
    /* Clear Interrupt sticky bits */
    pDmaInfo->pReg->Status = ADI_DMA_CLEAR_INTERRUPTS;
    /* Clear Paired channel interrupt sticky bits */
    (pDmaInfo->pChannel->MDMA.pPairedDmaInfo)->pReg->Status = ADI_DMA_CLEAR_INTERRUPTS;

    /* Set DMA data transfer status as stopped */
    pDmaInfo->pChannel->MDMA.bIsXferInProgress = false;

    /* Clear DMA Channel data transfer type */
    pDmaInfo->pChannel->MDMA.Src.eXferMode  = ADI_MDMA_XFER_NONE;
    pDmaInfo->pChannel->MDMA.Dest.eXferMode = ADI_MDMA_XFER_NONE;
}

/*=============  D E B U G   F U N C T I O N S =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validates MDMA Stream handle */
static ADI_DMA_RESULT ValidateStreamHandle (ADI_DMA_STREAM_HANDLE   hStream)
{
    /* Loop variable */
    uint32_t    i;

    /* FOR (All MDMA Streams) */
    for (i = 0; i < (sizeof (StreamInfo) / sizeof (StreamInfo[0])); i++)
    {
        /* IF (Stream handle is valid) */
        if (hStream == &ChannelInfo[StreamInfo[i].DestChannel])
        {
            /* Channel valid, return success */
            return (ADI_DMA_SUCCESS);
        }
    }

    /* Channel invalid, return failure */
    return (ADI_DMA_FAILURE);
}

/* Validate Descriptor List */
static ADI_DMA_RESULT ValidateDescList (
    ADI_DMA_INFO                	*pDmaInfo,
    ADI_DMA_DESC_LIST_MODE          *pDestDesc,
    ADI_DMA_DESC_LIST_MODE          *pSrcDesc)
{
    /* Return Code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* DO (for all descriptors in list) */
    while (pDestDesc != NULL)
    {
        /* Validate buffer address */
        eResult = ValidateBuffer ((uint32_t) pDestDesc->pStartAddress,
                                  pDestDesc->Config,
                                  pDmaInfo->pReg->Status,
                                  pDestDesc->XCount,
                                  pDestDesc->XModify,
                                  pDestDesc->YCount,
                                  pDestDesc->YModify);

        /* IF (Error) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

        /* Move to next descriptor in chain */
        pDestDesc = pDestDesc->pNext;
    }

    /* DO (for all descriptors in list) */
    while (pSrcDesc != NULL)
    {
        /* Validate buffer address */
        eResult = ValidateBuffer ((uint32_t) pSrcDesc->pStartAddress,
                                  pSrcDesc->Config,
                                  (pDmaInfo->pChannel->MDMA.pPairedDmaInfo)->pReg->Status,
                                  pSrcDesc->XCount,
                                  pSrcDesc->XModify,
                                  pSrcDesc->YCount,
                                  pSrcDesc->YModify);

        /* IF (Error) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

        /* Move to next descriptor in chain */
        pSrcDesc = pSrcDesc->pNext;
    }

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/* Validate Source and Destination DMA descriptor configuration */
static ADI_DMA_RESULT ValidateStreamDesc (ADI_DMA_INFO  *pDmaInfo)
{
    /* Return Code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* IF (Destination is in one-shot or circular mode) */
    if ((pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_CIRCULAR) &&\
        (pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_ONE_SHOT))
    {
        /* Validate buffer address */
        eResult = ValidateBuffer ((uint32_t) pDmaInfo->pChannel->MDMA.Dest.DescRegCache.pStartAddr,
                                  pDmaInfo->pChannel->MDMA.Dest.DescRegCache.Config,
                                  pDmaInfo->pReg->Status,
                                  pDmaInfo->pChannel->MDMA.Dest.DescRegCache.XCount,
                                  pDmaInfo->pChannel->MDMA.Dest.DescRegCache.XModify,
                                  pDmaInfo->pChannel->MDMA.Dest.DescRegCache.YCount,
                                  pDmaInfo->pChannel->MDMA.Dest.DescRegCache.YModify);

        /* IF (Error) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }
    }

    /* IF (Destination is in one-shot or circular mode) */
    if ((pDmaInfo->pChannel->MDMA.Src.eXferMode == ADI_MDMA_XFER_CIRCULAR) &&\
        (pDmaInfo->pChannel->MDMA.Src.eXferMode == ADI_MDMA_XFER_ONE_SHOT))
    {
        /* Validate buffer address */
        eResult = ValidateBuffer ((uint32_t) pDmaInfo->pChannel->MDMA.Src.DescRegCache.pStartAddr,
                                  pDmaInfo->pChannel->MDMA.Src.DescRegCache.Config,
                                  (pDmaInfo->pChannel->MDMA.pPairedDmaInfo)->pReg->Status,
                                  pDmaInfo->pChannel->MDMA.Src.DescRegCache.XCount,
                                  pDmaInfo->pChannel->MDMA.Src.DescRegCache.XModify,
                                  pDmaInfo->pChannel->MDMA.Src.DescRegCache.YCount,
                                  pDmaInfo->pChannel->MDMA.Src.DescRegCache.YModify);
    }

    /* Return */
    return (eResult);
}

/* Validate Descriptor 1D Array */
static ADI_DMA_RESULT ValidateDescArray1D (
    ADI_DMA_INFO                *pDmaInfo,
    ADI_DMA_DESC_ARRAY_1D_MODE  *pDescArray,
    uint16_t                    NumEntries,
    uint32_t                    DmaDirection)
{
    /* Return Code */
    ADI_DMA_RESULT              eResult = ADI_DMA_SUCCESS;
    /* 1D descriptor Array to work on */
    ADI_DMA_DESC_ARRAY_1D_MODE  *pWorkUnit1D = pDescArray;

    /* DO (for all descriptors in Array) */
    while (NumEntries != 0u)
    {
        /* Validate buffer address */
        eResult = ValidateBuffer ((uint32_t) pWorkUnit1D->pStartAddress,
                                  pWorkUnit1D->Config,
                                  pDmaInfo->pReg->Status,
                                  pWorkUnit1D->XCount, pWorkUnit1D->XModify, 0, 0);

        /* IF (Error) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

        /* Decement number of array entries to work on */
        NumEntries--;

        /* IF (More descriptor work unit in array) */
        if (NumEntries > 1)
        {
            /* Move to next work unit in array */
            pWorkUnit1D++;
        }
    }

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/* Validate Descriptor 2D Array */
static ADI_DMA_RESULT ValidateDescArray2D (
    ADI_DMA_INFO                *pDmaInfo,
    ADI_DMA_DESC_ARRAY_2D_MODE  *pDescArray,
    uint16_t                    NumEntries,
    uint32_t                    DmaDirection)
{
    /* Return Code */
    ADI_DMA_RESULT              eResult = ADI_DMA_SUCCESS;
    /* 2D descriptor Array to work on */
    ADI_DMA_DESC_ARRAY_2D_MODE  *pWorkUnit2D = pDescArray;

    /* DO (for all descriptors in Array) */
    while (NumEntries != 0u)
    {
        /* Validate buffer address */
        eResult = ValidateBuffer ((uint32_t) pWorkUnit2D->pStartAddress,
                                  pWorkUnit2D->Config,
                                  pDmaInfo->pReg->Status,
                                  pWorkUnit2D->XCount, pWorkUnit2D->XModify,
                                  pWorkUnit2D->YCount, pWorkUnit2D->YModify);

        /* IF (Error) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

        /* Decement number of array entries to work on */
        NumEntries--;

        /* IF (More descriptor work unit in array) */
        if (NumEntries > 1)
        {
            /* Move to next work unit in array */
            pWorkUnit2D++;
        }
    }

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

#endif /* ADI_DEBUG */

/*=============  P U B L I C   F U N C T I O N S =============*/

/**
 * @brief Opens a Memory DMA stream for use.
 *
 * @param [in]  eStreamID       MDMA Stream ID to open.
 * @param [in]  pStreamMemory   Pointer to ADI_DMA_STREAM_REQ_MEMORY sized memory
 *                              location to handle the DMA stream.
 * @param [out] phStream        Address where the MDMA stream handle will be stored.
 * @param [out] phSrcChannel    Address where the Source DMA channel handle will be stored.
 * @param [out] phDestChannel   Address where the Destination DMA channel handle will be stored.
 * @param [in]  pfCallback      Address of application callback function.
 * @param [in]  pCBParam        Parameter passed back to application callback.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully opened a MDMA Stream.
 *          - #ADI_DMA_FAILURE: Supplied pointer parameters are invalid.
 *          - #ADI_DMA_IN_USE: Supplied MDMA Stream ID is already in use.
 *          - #ADI_DMA_BAD_STREAM_ID: Supplied MDMA Stream ID is invalid.
 *          - #ADI_DMA_INT_REGISTER_FAILED: Failed to register DMA interrupt handler with the interrupt dispatcher.
 *
 * @note
 *      Source and Destination channel handles are made available to the
 *      programmer so that the handles can be used to query channel specific
 *      information such as current XCount etc., using the APIs common to
 *      Memory and Peripheral DMA channels.
 *
 */
ADI_DMA_RESULT adi_mdma_Open(
    ADI_DMA_STREAM_ID               eStreamID,
    void                            *pStreamMemory,
    ADI_DMA_STREAM_HANDLE           *phStream,
    ADI_DMA_CHANNEL_HANDLE          *phSrcChannel,
    ADI_DMA_CHANNEL_HANDLE          *phDestChannel,
    ADI_CALLBACK                    pfCallback,
    void                            *pCBParam)
{
    /* Return Code */
    ADI_DMA_RESULT      eResult = ADI_DMA_SUCCESS;
    /* Source and Destination DMA Channel number */
    uint8_t             SrcChannel, DestChannel;
    /* Pointer to memory allocated to handle DMA Stream */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) pStreamMemory;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (This is not a valid DMA stream) */
    if (eStreamID > ADI_DMA_MEMDMA_END_COUNT)
    {
        /* Report failure as bad stream id */
        return (ADI_DMA_BAD_STREAM_ID);
    }

    /* Check if the given pointer parameters are valid */
    if((pStreamMemory == NULL) || (phStream == NULL) || (phSrcChannel == NULL)  || (phDestChannel == NULL))
    {
        return (ADI_DMA_FAILURE);
    }

    /* Asserts to validate memory size required to manage a DMA stream */
    assert(ADI_DMA_STREAM_REQ_MEMORY >= sizeof(ADI_MDMA_STREAM));

#endif  /* ADI_DEBUG */

    /* Get DMA Channel number */
    SrcChannel  = (uint8_t) StreamInfo[(uint8_t)eStreamID].SrcChannel;
    DestChannel = (uint8_t) StreamInfo[(uint8_t)eStreamID].DestChannel;

    /* Initialise MDMA stream instance Memory */
    /* This is a Memory DMA */
    pStream->bIsMemDma  = true;
    /* Save callback parameters */
    pStream->pfCallback = pfCallback;
    pStream->pCBParam   = pCBParam;

    /*** Initialise Destination channel ***/
    /* Pointer to DMA info of the Destination channel */
    pStream->Dest.pDmaInfo = &ChannelInfo[DestChannel];

/* IF (ADSP-BF60x or ADSP-BF70x family */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)
    /* Get the Bitwise channel number and registry index */
    GetBitWiseInfo (DestChannel, &pStream->Dest.BitWiseRegistryIndex, &pStream->Dest.BitWiseChannelNum);
#elif defined(ADI_ADSP_CM40Z)
    /* acknowledge the pending interrupt at the NVIC level */
    ADI_ACK_PENDINT((IRQn_Type)pStream->Dest.pDmaInfo->DataIntSID);
#endif

    /*** Initialise Source channel ***/
    /* Pointer to DMA info of the Source channel */
    pStream->Src.pDmaInfo = &ChannelInfo[SrcChannel];

/* IF (ADSP-BF60x or ADSP-BF70x family */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)
    /* Get the Bitwise channel number and registry index */
    GetBitWiseInfo (SrcChannel, &pStream->Src.BitWiseRegistryIndex, &pStream->Src.BitWiseChannelNum);
#elif defined(ADI_ADSP_CM40Z)
    /* acknowledge the pending interrupt at the NVIC level */
    ADI_ACK_PENDINT((IRQn_Type)pStream->Src.pDmaInfo->DataIntSID);
#endif

	/* Entering critical region, disable interrupts */
	adi_osal_EnterCriticalRegion();

	/* IF (DMA stream already in use) */
    if (ChannelInfo[DestChannel].pChannel != NULL)
    {
        /* Report failure as the stream is already in use */
        eResult = ADI_DMA_IN_USE;
    }
	/* ELSE (DMA Stream is available for use) */
	else
	{
		/* Save the Memory address allocated to handle this MDMA stream */
		ChannelInfo[SrcChannel].pChannel  = (ADI_DMA_CHANNEL *)pStream;
		ChannelInfo[DestChannel].pChannel = (ADI_DMA_CHANNEL *)pStream;
		/* Save the Dma info address of the Source channel paired with the Destination */
		pStream->pPairedDmaInfo = &ChannelInfo[SrcChannel];
	}

	/* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

	/* IF (DMA Stream is available for use) */
    if (eResult == ADI_DMA_SUCCESS)
    {
		/* Register MDMA stream with Interrupt dispatcher */

        /* Register Destination channel interrupts */
        eResult = RegisterWithIntDispatcher (&ChannelInfo[DestChannel]);

/* IF (ADSP-CM402/CM403/CM407/CM408 processors) */
#if defined (ADI_ADSP_CM40Z)

        /* IF (Success) */
        if (eResult == ADI_DMA_SUCCESS)
        {
            /* Install Source channel Error interrupt handler */
            if(adi_int_InstallHandler((uint32_t)ChannelInfo[SrcChannel].ErrorIntSID,
                                      DmaErrorIntHandler,
                                      &ChannelInfo[SrcChannel],
                                      true) != ADI_INT_SUCCESS)
            {
                /* Failed to register DMA interrupt handler */
                return (ADI_DMA_INT_REGISTER_FAILED);
            }
        }

#endif

	}

    /* IF (Successfully registered MDMA stream with Interrupt dispatcher) */
    if (eResult == ADI_DMA_SUCCESS)
    {
        /* Reset the MDMA Stream instance memory with defaults */
        ResetMDmaStream (pStream);

/* IF (ADSP-BF608/609) or ADSP-BF70x family */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)

        /* Entering critical region, disable interrupts */
	    adi_osal_EnterCriticalRegion();

        /* Mark Source and Destination DMA channel as open to handle DMA error */
        AdiDmaBitWiseRegistry[pStream->Src.BitWiseRegistryIndex]  |= (1 << pStream->Src.BitWiseChannelNum);
        AdiDmaBitWiseRegistry[pStream->Dest.BitWiseRegistryIndex] |= (1 << pStream->Dest.BitWiseChannelNum);
        /* Update the DMA channel open count */
        AdiDmaNumChannelsOpen += 2;

        /* Re-enable interrupts */
        adi_osal_ExitCriticalRegion();

#endif

        /* Save the SID */
        pStream->Dest.DataSID = ChannelInfo[DestChannel].DataIntSID;
        pStream->Src.DataSID  = ChannelInfo[SrcChannel].DataIntSID;

        /* Pass a valid handle to this MDMA Stream
           Destination channel info address passed as stream handle */
        *phStream = (ADI_DMA_STREAM_HANDLE) &ChannelInfo[DestChannel];
        /* Pass Channel Handle for source and destination channels */
        *phSrcChannel  = (ADI_DMA_CHANNEL_HANDLE) &ChannelInfo[SrcChannel];
        *phDestChannel = (ADI_DMA_CHANNEL_HANDLE) &ChannelInfo[DestChannel];
    }
	/* ELSE (Failed to register MDMA stream with interrupt dispatcher) */
	else
	{
		/* Clear the memory address allocated to handle the MDMA stream */
		ChannelInfo[SrcChannel].pChannel = NULL;
		ChannelInfo[DestChannel].pChannel = NULL;
	}

    /* Return */
    return (eResult);
}

/**
 * @brief Closes a Memory DMA stream.
 *
 * @param [in]  hStream     MDMA Stream handle to be closed.
 * @param [in]  bWaitFlag   Wait for transfers to complete flag
 *                          (TRUE to wait, FALSE to force close).
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully closed a MDMA Stream.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_INT_UNREGISTER_FAILED: Failed to unregister interrupt.
 *
 */
ADI_DMA_RESULT adi_mdma_Close(
    ADI_DMA_STREAM_HANDLE           hStream,
    bool                            bWaitFlag)
{
    /* Return Code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;
    /* Pointer to Destination DMA Channel Info */
    ADI_DMA_INFO    *pDestDmaInfo = (ADI_DMA_INFO *)hStream;
    /* Pointer to Source DMA Channel Info */
    ADI_DMA_INFO    *pSrcDmaInfo;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDestDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Get the address of Source DMA channel info */
    pSrcDmaInfo = pDestDmaInfo->pChannel->MDMA.pPairedDmaInfo;

    /* IF (wait for DMA to complete transfer) */
    if (bWaitFlag)
    {
        /* wait until DMA is idle */
        while ((pDestDmaInfo->pReg->Status & BITM_DMA_STAT_RUN) != ENUM_DMA_STAT_STOPPED)
        {
          __NOP();  // KEIL: asm ("NOP;");
        }
    }

    /* Reset Destination DMA registers */
    ResetDmaRegs (pDestDmaInfo->pReg);
    /* Reset Source DMA registers */
    ResetDmaRegs (pSrcDmaInfo->pReg);

    /* Un-register DMA data interrupt handler for destination channel */
    if (adi_int_UninstallHandler ((uint32_t)pDestDmaInfo->DataIntSID) != ADI_INT_SUCCESS)
    {
    	/* Return failure as we failed to un-register the interrupt */
    	return (ADI_DMA_INT_UNREGISTER_FAILED);
    }

/* IF (ADSP-CM402/CM403/CM407/CM408 processors) */
#if defined (ADI_ADSP_CM40Z)

    /* Un-register DMA data interrupt handler for source channel */
    if (adi_int_UninstallHandler ((uint32_t)pSrcDmaInfo->DataIntSID) != ADI_INT_SUCCESS)
    {
    	/* Return failure as we failed to un-register the interrupt */
    	return (ADI_DMA_INT_UNREGISTER_FAILED);
    }

#endif

    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

/* IF (ADSP-BF608/609) or ADSP-BF707 family */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)

    /* Remove the Destination DMA channel from handling DMA errors */
    AdiDmaBitWiseRegistry[pDestDmaInfo->pChannel->MDMA.Dest.BitWiseRegistryIndex] &= ~(1 << pSrcDmaInfo->pChannel->MDMA.Src.BitWiseChannelNum);
    /* Remove the Source DMA channel from handling DMA errors */
    AdiDmaBitWiseRegistry[pDestDmaInfo->pChannel->MDMA.Src.BitWiseRegistryIndex]  &= ~(1 << pDestDmaInfo->pChannel->MDMA.Dest.BitWiseChannelNum);
    /* Decrement the DMA channel open count */
    AdiDmaNumChannelsOpen -= 2;

#endif

    /* Clear the memory address allocated to handle the MDMA Stream */
    pDestDmaInfo->pChannel = NULL;
    pSrcDmaInfo->pChannel = NULL;

    /* Re-enable interrupts */
    adi_osal_ExitCriticalRegion();

    /* Un-install DMA Error handler (destination channel used) */
    eResult = UninstallDmaErrorHandler(pDestDmaInfo);

/* IF (ADSP-CM402/CM403/CM407/CM408 processors) */
#if defined (ADI_ADSP_CM40Z)

    /* IF (Success) */
    if (eResult == ADI_DMA_SUCCESS)
    {
        /* Un-install DMA Error handler for source channel */
        eResult = UninstallDmaErrorHandler(pSrcDmaInfo);
    }

#endif

    return (eResult);
}

/**
 * @brief Gets the current data copy status of a Memory DMA stream.
 *
 * @param [in]  hStream             MDMA stream handle to query.
 * @param [out] pbCopyInProgress    Location to store current data copy status.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried MDMA dataflow status.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *
 */
ADI_DMA_RESULT adi_mdma_IsCopyInProgress(
    ADI_DMA_STREAM_HANDLE           hStream,
    bool                            *pbCopyInProgress)
{
    /* MDMA Stream to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hStream;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Get the current data transfer status of the DMA channel */
    *pbCopyInProgress = pDmaInfo->pChannel->MDMA.bIsXferInProgress;

    /* Report success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Disables/terminates Memory DMA copy that is currently in progress.
 *
 * @param [in]  hStream     MDMA stream handle to work on.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully disabled MDMA dataflow.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *
 */

ADI_DMA_RESULT adi_mdma_Disable (
    ADI_DMA_STREAM_HANDLE           hStream)
{
    /* MDMA Stream to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hStream;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (MDMA Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Disable MDMA transfer */
    DisableXfer (pDmaInfo);

    /* Report success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief   Submits Memory DMA configuration for Circular One-Dimensional copy.
 *
 * @details Sets the Source or Destination channel of the MDMA stream for
 *          continuous data transfer from a Memory Mapped location or a
 *          single data buffer.
 *
 * @param [in] hStream         MDMA Stream handle to work on.
 * @param [in] pAddress        Memory Mapped address to read/write
 * @param [in] ElementCount    Number of data elements to copy
 * @param [in] eElementWidth   Data element width in terms of ADI_DMA_MSIZE
 * @param [in] bUseSrcChannel  'true' to use the data transfer settings for Source MDMA channel,
 *                             'false' to use it for Destination MDMA channel.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted circular 1D buffer.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't submit new buffer when the DMA dataflow is already enabled.
 *          - #ADI_DMA_XFER_TYPE_INVALID: Transfer type is invalid since both Source and Destination
 *                                        can't be configured in circular/memory mapped transfer mode.
 *          - #ADI_DMA_XFER_TYPE_ALREADY_SET: Data transfer type is already set for one or both DMA channel
 *                                            in this Memory stream.
 *
 * @note
 *      The DMA Channel set for circular data transfer will not
 *      generate a data buffer processed interrupt. The DMA channel will
 *      be configured in Auto-buffer (Circular buffer) mode thereby configuring
 *      it for non-stop transfer from the specified location.
 *
 */
ADI_DMA_RESULT adi_mdma_CopyCircular1D(
    ADI_DMA_STREAM_HANDLE           hStream,
    void                            *pAddress,
    uint32_t                        ElementCount,
    ADI_DMA_MSIZE                   eElementWidth,
    bool                            bUseSrcChannel)
{
    /* Default DMA Config - Auto Buffer mode, No Interrupt on completion */
    uint32_t            Config = (ENUM_DMA_CFG_AUTO | eElementWidth | ENUM_DMA_CFG_EN);
    /* Pointer to DMA Channel info to work on */
    ADI_DMA_INFO        *pDmaInfo = (ADI_DMA_INFO *)hStream;
    /* Pointer to MDMA Channel instance to work on */
    ADI_MDMA_CHANNEL    *pMDmaChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pDmaInfo->pChannel->MDMA.bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* IF (Read from a Memory Mapped location) */
    if (bUseSrcChannel)
    {
        /* IF (Source channel is already set for circular transfer) */
        if (pDmaInfo->pChannel->MDMA.Src.eXferMode == ADI_MDMA_XFER_CIRCULAR)
        {
            /* Return failure as only Source or Destination can be memory mapped */
            return (ADI_DMA_XFER_TYPE_INVALID);
        }
        /* Configure Source channel for Memory mapped read */
        pMDmaChannel = &(pDmaInfo->pChannel->MDMA.Src);
    }
    /* ELSE (Write to a Memory Mapped location) */
    else
    {
        /* IF (Destination channel is already set for circular transfer) */
        if (pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_CIRCULAR)
        {
            /* Return failure as only Source or Destination can be memory mapped */
            return (ADI_DMA_XFER_TYPE_INVALID);
        }
        /* Configure Destination channel for Memory mapped write */
        pMDmaChannel = &(pDmaInfo->pChannel->MDMA.Dest);
        /* Set config register for DMA Write */
        Config |= ENUM_DMA_CFG_WRITE;
    }

    /* IF (Transfer mode of the DMA channel is other than idle/none) */
    if (pMDmaChannel->eXferMode != ADI_MDMA_XFER_NONE)
    {
        /* Report failure as transfer mode is already set for this channel */
        return (ADI_DMA_XFER_TYPE_ALREADY_SET);
    }

    /* Reset Memory DMA Channel instance memory */
    ResetMDmaChannel (pMDmaChannel);
    /* Set channel transfer mode as circular transfer */
    pMDmaChannel->eXferMode = ADI_MDMA_XFER_CIRCULAR;
    /* Update Enable data transfer function pointer */
    pMDmaChannel->pfEnableXfer = &MDmaEnableOneShotCircXfer;
    /* Update function pointer to handle DMA data interrupt */
    pMDmaChannel->pfDataIntHandle = NULL;

    /* Save the supplied values to Descriptor cache */
    pMDmaChannel->DescRegCache.pStartAddr = pAddress;
    pMDmaChannel->DescRegCache.XCount     = ElementCount;
    /* Calculate element width in bytes */
    pMDmaChannel->DescRegCache.XModify    = (int32_t) (1 << ((uint32_t)eElementWidth >> BITP_DMA_CFG_MSIZE));
    /* Append trigger configuration */
    pMDmaChannel->DescRegCache.Config	  = (Config | pMDmaChannel->CommonConfig);

    /* Update data transfer status of this DMA stream */
    return (MDmaUpdateXferStatus (pDmaInfo));
}

/**
 * @brief Copies data between memory in One-Dimensional mode.
 *
 * @param [in]  hStream         MDMA Stream handle to work on.
 * @param [in]  pMemDest        Memory destination address.
 * @param [in]  pMemSrc         Memory source address.
 * @param [in]  eElementWidth   Data element width in terms of ADI_DMA_MSIZE.
 * @param [in]  ElementCount    Number of data elements to copy.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 1D Memory transfer info.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't submit new buffer when the DMA dataflow is already enabled.
 *          - #ADI_DMA_XFER_TYPE_ALREADY_SET: Data transfer type is already set for the DMA channel/MDMA Stream.
 *
 */
ADI_DMA_RESULT adi_mdma_Copy1D(
    ADI_DMA_STREAM_HANDLE           hStream,
    void                            *pMemDest,
    void                            *pMemSrc,
    ADI_DMA_MSIZE                   eElementWidth,
    uint32_t                        ElementCount)
{
    /* MSize/PSize Configuration */
    uint32_t        MSize, MSizePSizeCfg;
    /* Return code */
    ADI_DMA_RESULT  eResult;
    /* DMA Channel info to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hStream;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Get MSize value */
    MSize = ((uint32_t)eElementWidth >> BITP_DMA_CFG_MSIZE);

    /* IF (MDMA stream opened for CRC operation) */
    if (pDmaInfo->pChannel->MDMA.bIsCrcMode)
    {
        /* Get PSize/MSize configuration */
        GetMaxMSizePSize (pMemDest, pMemSrc, pDmaInfo->pReg->Status, &MSize, &ElementCount, &MSizePSizeCfg);
    }
    /* ELSE (Normal MDMA operation) */
    else
    {
        /* Append MSize, keep PSize to zero (1 byte) */
        MSizePSizeCfg = (uint32_t) eElementWidth;
    }

    /* IF (we've a valid Source data buffer address) */
    if (pMemSrc != NULL)
    {
        /* Prepare DMA Configuration register for Source DMA (READS from Memory, Writes to DMA FIFO) */
        eResult = MDmaPrepareOneShot (pDmaInfo,
                                      &(pDmaInfo->pChannel->MDMA.Src),
                                      pMemSrc,
                                      ElementCount,
                                      (1u << (int32_t)MSize),
                                      0,
                                      0,
                                      (MSizePSizeCfg | pDmaInfo->pChannel->MDMA.Src.CommonConfig | ENUM_DMA_CFG_EN));

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* IF (we've a valid Destination data buffer address) */
    if (pMemDest != NULL)
    {
        /* Prepare DMA Configuration register for Destination DMA (WRITES to Memory, Reads from DMA FIFO) */
        eResult = MDmaPrepareOneShot (pDmaInfo,
                                      &(pDmaInfo->pChannel->MDMA.Dest),
                                      pMemDest,
                                      ElementCount,
                                      (1u << (int32_t)MSize),
                                      0,
                                      0,
                                      (ENUM_DMA_CFG_WRITE | MSizePSizeCfg | pDmaInfo->pChannel->MDMA.Dest.CommonConfig | ENUM_DMA_CFG_EN));

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* IF (MDMA stream opened for CRC operation) */
    if (pDmaInfo->pChannel->MDMA.bIsCrcMode)
    {
        /* Load DMA registers for CRC operation */
        return (MDmaCrcModeOneShotXfer (pDmaInfo));
    }
    /* ELSE (MDMA stream opened for data copying) */
    else
    {
        /* Update data transfer status of this DMA stream */
        return (MDmaUpdateXferStatus (pDmaInfo));
    }
}

/**
 * @brief Copies data between memory in Two-Dimensional mode.
 *
 * @param [in]  hStream         MDMA Stream handle to work on.
 * @param [in]  eElementWidth   Data element width in terms of ADI_DMA_MSIZE.
 * @param [in]  pDestDesc       Address of an instance that holds Destination DMA channel
 *                              specific information to perform a Memory DMA copy.
 * @param [in]  pSrcDesc        Address of an instance that holds Source DMA channel
 *                              specific information to perform a Memory DMA copy.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 2D Memory transfer info.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't submit new buffer when the DMA dataflow is already enabled.
 *          - #ADI_DMA_XFER_TYPE_ALREADY_SET: Data transfer type is already set for the DMA channel/MDMA Stream.
 *
 */
ADI_DMA_RESULT adi_mdma_Copy2D(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_MSIZE                   eElementWidth,
    ADI_DMA_2D_MEM_TRANSFER         *pDestDesc,
    ADI_DMA_2D_MEM_TRANSFER         *pSrcDesc)
{
    /* Return code */
    ADI_DMA_RESULT      eResult;
    /* DMA Channel info to work on */
    ADI_DMA_INFO        *pDmaInfo = (ADI_DMA_INFO *)hStream;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (we've a valid transfer info for Source DMA channel) */
    if (pSrcDesc != NULL)
    {
        /* Prepare DMA Configuration register for Source DMA (READS from Memory, Writes to DMA FIFO) */
        eResult = MDmaPrepareOneShot (pDmaInfo,
                                      &(pDmaInfo->pChannel->MDMA.Src),
                                      pSrcDesc->pStartAddress,
                                      pSrcDesc->XCount,
                                      pSrcDesc->XModify,
                                      pSrcDesc->YCount,
                                      pSrcDesc->YModify,
                                      (eElementWidth | pDmaInfo->pChannel->MDMA.Src.CommonConfig | ENUM_DMA_CFG_EN));

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* IF (we've a valid transfer info for Destination DMA channel) */
    if (pDestDesc != NULL)
    {
        /* Prepare DMA Configuration register for Destination DMA (WRITES to Memory, Reads from DMA FIFO) */
        eResult = MDmaPrepareOneShot (pDmaInfo,
                                      &(pDmaInfo->pChannel->MDMA.Dest),
                                      pDestDesc->pStartAddress,
                                      pDestDesc->XCount,
                                      pDestDesc->XModify,
                                      pDestDesc->YCount,
                                      pDestDesc->YModify,
                                      (ENUM_DMA_CFG_WRITE | eElementWidth | pDmaInfo->pChannel->MDMA.Dest.CommonConfig | ENUM_DMA_CFG_EN));

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* Update data transfer status of this DMA stream */
    return (MDmaUpdateXferStatus (pDmaInfo));
}

/**
 * @brief Copies data between memory using List descriptor(s).
 *
 * @param [in]  hStream     MDMA Stream handle to work on.
 * @param [in]  pDestDesc   Address of the first list descriptor in chain that is to be
 *                          submitted to the Destination DMA channel.
 * @param [in]  pSrcDesc    Address of the first list descriptor in chain that is to be
 *                          submitted to the Source DMA channel.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted Memory transfer descriptor chain.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_XFER_TYPE_ALREADY_SET: Data transfer type already set.
 *          - #ADI_DMA_NO_BUFFER_TO_PROCESS: There is no valid data buffer to process.
 *
 */
ADI_DMA_RESULT adi_mdma_CopyQueueList(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_LIST_MODE          *pDestDesc,
    ADI_DMA_DESC_LIST_MODE          *pSrcDesc)
{
    /* DMA Channel info to work on */
    ADI_DMA_INFO            *pDmaInfo = (ADI_DMA_INFO *)hStream;
    /* Pointer to descriptor list instance to work on */
    ADI_DMA_DESC_LIST_MODE  *pWorkingDestDesc, *pWorkingSrcDesc;
    /* Pointer to last descriptor in chain */
    ADI_DMA_DESC_LIST_MODE  *pLastDestDesc, *pLastSrcDesc;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Return code */
    ADI_DMA_RESULT      eResult;

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

    /* IF (Both Source and Destination descriptor pointers are invalid) */
    if ((pDestDesc == NULL) && (pSrcDesc == NULL))
    {
        /* Return failure as we've no valid buffers to process */
        return (ADI_DMA_NO_BUFFER_TO_PROCESS);
    }

    /* IF (Destination descriptor pointer is valid and
           its transfer mode is other than none or List mode) */
    if ((pDestDesc != NULL) &&\
        ((pDmaInfo->pChannel->MDMA.Dest.eXferMode != ADI_MDMA_XFER_NONE) &&\
         (pDmaInfo->pChannel->MDMA.Dest.eXferMode != ADI_MDMA_XFER_LARGE_LIST)))
    {
        /* Return failure as the transfer type is already set */
        return (ADI_DMA_XFER_TYPE_ALREADY_SET);
    }

    /* IF (Source descriptor pointer is valid and
           its transfer mode is other than none or List mode) */
    if ((pSrcDesc != NULL) &&\
        ((pDmaInfo->pChannel->MDMA.Src.eXferMode != ADI_MDMA_XFER_NONE) &&\
         (pDmaInfo->pChannel->MDMA.Src.eXferMode != ADI_MDMA_XFER_LARGE_LIST)))
    {
        /* Return failure as the transfer type is already set */
        return (ADI_DMA_XFER_TYPE_ALREADY_SET);
    }

    /* Validate Descriptor List */
    eResult = ValidateDescList (pDmaInfo, pSrcDesc, pDestDesc);

    /* IF (Descriptor List is invalid) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    /* Initialise the working descriptor pointers */
    pWorkingDestDesc = pDestDesc;
    pWorkingSrcDesc  = pSrcDesc;

    /* DO (for all descriptors in list) */
    while ((pWorkingDestDesc != NULL) || (pWorkingSrcDesc != NULL))
    {
        /* Configure Source Descriptor */
        if (pWorkingSrcDesc != NULL)
        {
            /* Prepare list descriptor for source channel queuing */
            PrepareSrcListDesc (pWorkingSrcDesc, pWorkingDestDesc, pDmaInfo->pChannel->MDMA.Dest.eXferMode);
            /* Checks if this is the last descriptor in list */
            IsLastInList(pWorkingSrcDesc, &pLastSrcDesc);
            /* Move to next descriptor in chain */
            pWorkingSrcDesc = pWorkingSrcDesc->pNext;
        }

        /* Configure Destination Descriptor */
        if (pWorkingDestDesc != NULL)
        {
            /* Prepare list descriptor for destination channel queuing */
            PrepareDestListDesc (pWorkingDestDesc);
             /* Checks if this is the last descriptor in list */
            IsLastInList(pWorkingDestDesc, &pLastDestDesc);
            /* Move to next descriptor in chain */
            pWorkingDestDesc = pWorkingDestDesc->pNext;
        }
    }

    /*** Add the list descriptor chain to standby queue ***/
    /* Entering critical region, disable interrupts */
	adi_osal_EnterCriticalRegion();

    /* IF (The Destination channel transfer mode is not yet set) */
    if (pDmaInfo->pChannel->MDMA.Dest.eXferMode == ADI_MDMA_XFER_NONE)
    {
    	/* IF (Destination descriptor is valid) */
    	if (pDestDesc != NULL)
    	{
    		/* Initialise the Destination channel for List transfer */
    		InitForListMode (&pDmaInfo->pChannel->MDMA.Dest);
    	}
    }

    /* IF (The Source channel transfer mode is not yet set) */
    if (pDmaInfo->pChannel->MDMA.Src.eXferMode == ADI_MDMA_XFER_NONE)
    {
    	/* IF (Source descriptor is valid) */
    	if (pSrcDesc != NULL)
    	{
    		/* Initialise the Source channel for List transfer */
    		InitForListMode (&pDmaInfo->pChannel->MDMA.Src);
    	}
    }

    /* IF (we've a valid destination descriptor chain) */
    if (pDestDesc != NULL)
    {
        /* Add the descriptor list to Chain */
        AddToChain (&pDmaInfo->pChannel->MDMA.Dest,
                    (void *) pDestDesc,
                    (void *) pLastDestDesc);
    }

    /* IF (we've a valid Source descriptor chain) */
    if (pSrcDesc != NULL)
    {
        /* Add the descriptor list to Chain */
        AddToChain (&pDmaInfo->pChannel->MDMA.Src,
                    (void *) pSrcDesc,
                    (void *) pLastSrcDesc);
    }

    /* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

    /* Update data transfer status of this DMA stream */
    return (MDmaUpdateXferStatus (pDmaInfo));
}

/**
 * @brief Copies data between memory using One-Dimensional Array descriptor(s).
 *
 * @param [in]  hStream     MDMA Stream handle to work on.
 * @param [in]  pDestDesc   Address of the first array descriptor in chain that is to be
 *                          submitted to the Destination DMA channel.
 * @param [in]  pSrcDesc    Address of the first array descriptor in chain that is to be
 *                          submitted to the Source DMA channel.
 * @param [in]  NumEntries  Number of descriptor entries in the array.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 1D descriptor array.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Bad DMA Handle.
 *
 */
ADI_DMA_RESULT adi_mdma_Copy1DArray(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pDestDesc,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pSrcDesc,
    uint16_t					    NumEntries)
{
    /* Return code */
    ADI_DMA_RESULT      eResult;
    /* DMA Channel info to work on */
    ADI_DMA_INFO        *pDmaInfo = (ADI_DMA_INFO *)hStream;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (we've a valid transfer info for Source DMA channel) */
    if (pSrcDesc != NULL)
    {
        /* Queue the descriptor list to Source DMA (READS from memory, Writes to DMA FIFO) */
        eResult = MDmaPrepareArray1D (pDmaInfo,
        							  pSrcDesc,
                                      &(pDmaInfo->pChannel->MDMA.Src),
                                      NumEntries,
                                      ENUM_DMA_CFG_READ);

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* IF (we've a valid transfer info for Destination DMA channel) */
    if (pDestDesc != NULL)
    {
        /* Queue the descriptor list to Source DMA (READS from memory, Writes to DMA FIFO) */
        eResult = MDmaPrepareArray1D (pDmaInfo,
        							  pDestDesc,
                                      &(pDmaInfo->pChannel->MDMA.Dest),
                                      NumEntries,
                                      ENUM_DMA_CFG_WRITE);

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* Update data transfer status of this DMA stream */
    return (MDmaUpdateXferStatus (pDmaInfo));
}

/**
 * @brief Copies data between memory using Two-Dimensional Array descriptor(s).
 *
 * @param [in]  hStream     MDMA Stream handle to work on.
 * @param [in]  pDestDesc   Address of the first array descriptor in chain that is to be
 *                          submitted to the Destination DMA channel.
 * @param [in]  pSrcDesc    Address of the first array descriptor in chain that is to be
 *                          submitted to the Source DMA channel.
 * @param [in]  NumEntries  Number of descriptor entries in the array.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 2D descriptor array.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Bad DMA Handle.
 *
 */
ADI_DMA_RESULT adi_mdma_Copy2DArray(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pDestDesc,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pSrcDesc,
    uint16_t					    NumEntries)
{
    /* Return code */
    ADI_DMA_RESULT      eResult;
    /* DMA Channel info to work on */
    ADI_DMA_INFO        *pDmaInfo = (ADI_DMA_INFO *)hStream;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Stream is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (we've a valid transfer info for Source DMA channel) */
    if (pSrcDesc != NULL)
    {
        /* Queue the descriptor list to Source DMA (READS from memory, Writes to DMA FIFO) */
        eResult = MDmaPrepareArray2D (pDmaInfo,
                                      pSrcDesc,
                                      &(pDmaInfo->pChannel->MDMA.Src),
                                      NumEntries,
                                      ENUM_DMA_CFG_READ);

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* IF (we've a valid transfer info for Destination DMA channel) */
    if (pDestDesc != NULL)
    {
        /* Queue the descriptor list to Source DMA (READS from memory, Writes to DMA FIFO) */
        eResult = MDmaPrepareArray2D (pDmaInfo,
                                      pDestDesc,
                                      &(pDmaInfo->pChannel->MDMA.Dest),
                                      NumEntries,
                                      ENUM_DMA_CFG_WRITE);

        /* IF (Failed to prepare DMA Configuration register) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            /* Return error */
            return (eResult);
        }
    }

    /* Update data transfer status of this DMA stream */
    return (MDmaUpdateXferStatus (pDmaInfo));
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*@}*/

