/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_pdma_bf60x.c
 * @brief     Peripheral DMA Manager for ADSP-BF60x processor family.
 * @version:  $Revision: 22948 $
 * @date:     $Date: 2013-09-03 20:32:22 -0400 (Tue, 03 Sep 2013) $
 *
 * @details
 *            This is the the primary source file for Peripheral DMA Manager Service
 *            for ADSP-BF60x processor.
 *
 */

/** @addtogroup PDMA_Manager_bf60x
 *  @{

<b>Peripheral DMA Manager for ADSP-BF60x family of processors</b>

All DMA driven peripherals should configure the MSIZE (of type #ADI_DMA_MSIZE) and
PSIZE (of type #ADI_DMA_PSIZE) of its corresponding DMA channel before enabling the
DMA channel dataflow. MSIZE can be configured using adi_pdma_SetMSize() API and
PSIZE using adi_pdma_SetPSize() API.

It is strongly recommended that all DMA driven drivers follow the DMA - Peripheral enable/disable
sequence specified in the Hardware Reference Manual. As per the HRM, while enabling dataflow,
the DMA channel should be enabled first before enabling the peripheral. While disabling dataflow,
the peripheral should be disabled first before disabling its corresponding DMA channel.

The current System Interrupt ID (SID) of a DMA channel can be obtained by calling adi_pdma_GetSID() API.

<b>Trigger Functionality</b>

The trigger can be used to synchronize other processing in the system. A peripheral DMA channel
can be configured to wait for a trigger input before processing a buffer and/or to generate a
trigger output after processing a data buffer. The trigger input to DMA can be routed from a
trigger master. Similarly, the trigger output generated by DMA channel can be routed to
one or more slave devices. Trigger routing can be done by calling the Trigger Routing Unit APIs.

Trigger configuration can be set on a per-buffer basis for all Peripheral DMA data transfer modes.
To configure a buffer with specific trigger settings, application should call trigger configuration
APIs provided by the corresponding peripheral driver.

Trigger settings can be applied to a buffer by calling the trigger in/out configuration APIs before
submitting the buffer to the peripheral and the trigger configuration can be cleared after the
buffer is submitted by calling the same trigger in/out APIs again.

 */

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_1:" ")
#pragma diag(suppress:misra_rule_8_5:" ")
#pragma diag(suppress:misra_rule_10_3:" ")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#pragma diag(suppress:misra_rule_11_1:"Allow Type-casting")
#pragma diag(suppress:misra_rule_11_3:"Allow Type-casting")
#pragma diag(suppress:misra_rule_11_4:"Allow Type-casting")
#pragma diag(suppress:misra_rule_11_5:" ")
#pragma diag(suppress:misra_rule_12_6:"Allow using logical operators")
#pragma diag(suppress:misra_rule_12_7:" ")
#pragma diag(suppress:misra_rule_13_7:"Allow 'assert' function to validate memory size")
#pragma diag(suppress:misra_rule_14_7:" ")
#pragma diag(suppress:misra_rule_16_7:" ")
#endif /* _MISRA_RULES */

/*==============  D E F I N E S  ===============*/

/* DMA Transfer modes */
/* 1D List transfer */
#define ADI_PDMA_DMA_CFG_LLIST_1D           (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_XCNT_INT | ENUM_DMA_CFG_FETCH05 | ENUM_DMA_CFG_EN)
/* 2D List transfer */
#define ADI_PDMA_DMA_CFG_LLIST_2D           (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_YCNT_INT | ENUM_DMA_CFG_FETCH07 | ENUM_DMA_CFG_EN)
/* Circular transfer */
#define ADI_PDMA_DMA_CFG_CIRCULAR           (ENUM_DMA_CFG_AUTO | ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_EN)
/* Common DMA Configuration Mask */
#define ADI_PDMA_MASK_COMMON_CFG            (BITM_DMA_CFG_PSIZE | BITM_DMA_CFG_MSIZE | BITM_DMA_CFG_TWAIT | BITM_DMA_CFG_TRIG | BITM_DMA_CFG_TOVEN)

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/

/* Resets Peripheral DMA work unit */
static void ResetPDmaWorkUnit (ADI_PDMA_DESC_WORK_UNIT    *pWorkUnit);
/* Resets a Peripheral DMA channel instance */
static void ResetPDmaChannel (ADI_PDMA_CHANNEL    *pChannel);

/* Loads a work unit for List descriptor transfer mode */
static ADI_DMA_RESULT LoadAsListDescWorkUnit(
    ADI_DMA_INFO                *pDmaInfo,
    void                        *pStartAddress,
    uint32_t                    XCount,
    int32_t                     XModify,
    uint32_t                    YCount,
    int32_t                     YModify,
    uint32_t                    Direction);

/* Prepares DMA descriptors for Circular transfer */
static ADI_DMA_RESULT LoadAsCircDescWorkUnit(
    ADI_PDMA_CHANNEL            *pChannel,
    void                        *pStartAddress,
    uint32_t                    NumSubBuffers,
    uint32_t                    ElementsPerSubBuffer,
    bool                        bBufferDoneCallback,
    bool                        bSubBufferDoneCallback,
    uint32_t                    Direction);

/* Loads Peripheral DMA registers with a work unit */
static ADI_DMA_RESULT LoadPDmaRegsWithWorkUnit (
    ADI_DMA_INFO const          *pDmaInfo,
    ADI_PDMA_DESC_WORK_UNIT     *pWorkUnitToLoad);

/*
 * Functions to enable DMA data transfer
 */
static  ADI_DMA_RESULT PDmaEnableListXfer (ADI_DMA_INFO     	*pDmaInfo);
static  ADI_DMA_RESULT PDmaEnableCircOneShotXfer (ADI_DMA_INFO	*pDmaInfo);

/*
 * Peripheral DMA Interrupt Handlers
 */

/* DMA Interrupt handler for One-Shot/Large List Descriptor data transfer */
static void PDmaOneShotListXferIntHandler(ADI_DMA_INFO const *pDmaInfo);
/* DMA Interrupt handler for Circular data transfer */
static void PDmaCircXferIntHandler(ADI_DMA_INFO const *pDmaInfo);

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Checks if the given channel ID belongs to peripheral DMA or not */
static ADI_DMA_RESULT ValidatePDmaChannel (ADI_DMA_CHANNEL_ID   eChannelID);

/* Validates list buffer address and its corresponding DMA register values */
static ADI_DMA_RESULT   ValidatePDmaListBuffer (
    ADI_PDMA_CHANNEL    *pChannel,
    uint32_t            StartAddress,
    uint32_t            Status,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify);

/* Validates circular buffer address and its corresponding DMA register values */
static ADI_DMA_RESULT   ValidatePDmaCircBuffer (
    ADI_DMA_INFO        *pDmaInfo,
    uint32_t            StartAddress,
    uint32_t            Status,
    uint32_t            NumSubBuffers,
    uint32_t            ElementsPerSubBuffer);

/* End of debug functions */

#endif /* ADI_DEBUG */

/*=============  C O D E  =============*/

/*=============  L O C A L   F U N C T I O N S =============*/

/*
 * Resets a Peripheral DMA channel instance.
 *
 * Parameters
 *  - [in]  pChannel   Pointer to DMA channel instance to work on.
 *
 * Returns
 *  - None
 *
 */
static void ResetPDmaChannel (ADI_PDMA_CHANNEL    *pChannel)
{
    /* Not a Memory DMA */
    pChannel->bIsMemDma             = false;
    /* DMA Transfer not enabled */
    pChannel->bIsXferInProgress     = false;
    /* Reset transfer mode */
    pChannel->bCircularXfer         = false;
    pChannel->bOneShotListXfer      = false;
    /* Clear circular mode related values */
    pChannel->RemainSubBufs         = 0;
    pChannel->pBufferAddr           = NULL;
    pChannel->BytesPerSubBuf        = 0;
    /* Reset function pointers */
    pChannel->pfDataIntHandle       = NULL;
    pChannel->pfEnableXfer          = NULL;
    /* No work unit is being processed */
    pChannel->pActiveWorkUnit       = NULL;
    /* Reset DMA work units */
    ResetPDmaWorkUnit (&(pChannel->WorkUnit0));
    ResetPDmaWorkUnit (&(pChannel->WorkUnit1));
    /* Link the work units */
    pChannel->WorkUnit0.pNextDesc = (void *) &pChannel->WorkUnit1;
    pChannel->WorkUnit1.pNextDesc = (void *) &pChannel->WorkUnit0;
}

/*
 * Resets DMA work unit.
 *
 * Parameters
 *  - [in]  pWorkUnit   Pointer to DMA Work unit to clear.
 *
 * Returns
 *  - None
 *
 */
static void ResetPDmaWorkUnit (ADI_PDMA_DESC_WORK_UNIT    *pWorkUnit)
{
    /* Reset DMA work unit */
    pWorkUnit->pNextDesc        = NULL;
    pWorkUnit->pStartAddr       = NULL;
    pWorkUnit->Config           = 0;
    pWorkUnit->XCount           = 0;
    pWorkUnit->XModify          = 0;
    pWorkUnit->YCount           = 0;
    pWorkUnit->YModify          = 0;
    pWorkUnit->bIsFree_BufDone  = true;
}

/*
 * Load a work unit for List descriptor transfer mode.
 *
 * Copies the supplied values to a free work unit.
 *
 * Parameter
 *  - [in]  pDmaInfo        DMA instance to work on.
 *  - [in]  pStartAddress   Start address of the data buffer to be processed.
 *  - [in]  XCount          Inner loop count start value in number of
 *                          DMA data bus width (MSIZE) transfers.
 *  - [in]  XModify         Inner loop address increment in bytes.
 *  - [in]  YCount          Outer loop count start value in number of rows.
 *  - [in]  YModify         Outer loop address increment in bytes.
 *  - [in]  Direction       DMA Direction
 *
 * Returns  Status
 *  - ADI_DMA_SUCCESS: Successfully submitted a list buffer.
 *  - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *  - ADI_DMA_BAD_XCOUNT: Supplied XCount is invalid.
 *  - ADI_DMA_BAD_YCOUNT: Supplied YCount is invalid.
 *  - ADI_DMA_BAD_START_ADDR: Supplied list buffer start address is invalid.
 *  - ADI_DMA_WORK_UNIT_FULL: Can't submit a list buffer as the DMA work units are full.
 *  - ADI_DMA_BUFFER_TYPE_MISMATCH: Can't submit a list buffer since the DMA is
 *                                  already configured for circular data transfer.
 *  - ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                  buffer address and will cause alignment error.
 *
 */
static ADI_DMA_RESULT LoadAsListDescWorkUnit(
    ADI_DMA_INFO        *pDmaInfo,
    void                *pStartAddress,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify,
    uint32_t            Direction)
{
    /* DMA Config */
    uint32_t                    Config;
    /* DMA Channel instance to work on */
    ADI_PDMA_CHANNEL            *pChannel = (ADI_PDMA_CHANNEL *) pDmaInfo->pChannel;
    /* Work unit that is available for use */
    ADI_PDMA_DESC_WORK_UNIT     *pWorkUnitToUse;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Return code */
    ADI_DMA_RESULT  eResult;

    /* Validate buffer address and its corresponding DMA register values */
    eResult = ValidatePDmaListBuffer (pChannel,
                                      (uint32_t) pStartAddress,
                                      pDmaInfo->pReg->Status,
                                      XCount,
                                      XModify,
                                      YCount,
                                      YModify);

    /* IF (Failed to validate buffer address and its corresponding DMA register values) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA is set for circular transfer) */
    if (pChannel->bCircularXfer)
    {
        /* Report failure since the DMA is set for circular data transfer */
        return (ADI_DMA_BUFFER_TYPE_MISMATCH);
    }

    /* Operate DMA channel in one-shot/List transfer mode */
    pChannel->bOneShotListXfer = true;

    /* IF (Pointer to DMA data interrupt handler is not yet set) */
    if (pChannel->pfDataIntHandle == NULL)
    {
        /* Update DMA data interrupt and enable dataflow function pointers */
        pChannel->pfDataIntHandle  = (ADI_DMA_DATA_INT_FN) &PDmaOneShotListXferIntHandler;
        pChannel->pfEnableXfer     = (ADI_PDMA_ENABLE_FN)  &PDmaEnableListXfer;
    }

    /* IF (we've valid YCount) */
    if (YModify > 0)
    {
        /* Prepare for 2D transfer, Interrupt when YCNTCUR reaches zero */
        Config = ADI_PDMA_DMA_CFG_LLIST_2D;
    }
    /* ELSE (1D transfer) */
    else
    {
        /* Prepare for 1D transfer, Interrupt when XCNTCUR reaches zero */
        Config = ADI_PDMA_DMA_CFG_LLIST_1D;
    }

    /* Append direction and configuration common to all transfer modes */
    Config |= (Direction | pChannel->CommonConfig);

    /* IF (One-shot transfer) */
    if ((pChannel->bIsStreaming == false) && (pChannel->bBufRepeat == false))
    {
        /* Put the descriptor in STOP Flow mode */
        Config &= ~(BITM_DMA_CFG_FLOW | BITM_DMA_CFG_NDSIZE);
    }

    /* IF (Data transfer in progress and buffer repetition is enabled) */
    if ((pChannel->bIsXferInProgress) && (pChannel->bBufRepeat))
    {
        /* Entering critical region, disable interrupts */
        adi_osal_EnterCriticalRegion();

        /* Get address of DMA work unit that is not active */
        pWorkUnitToUse = (ADI_PDMA_DESC_WORK_UNIT *) (pDmaInfo->pChannel->PDMA.pActiveWorkUnit->pNextDesc);

        /* Update work unit */
        pWorkUnitToUse->pStartAddr  = pStartAddress;
        pWorkUnitToUse->XCount      = XCount;
        pWorkUnitToUse->XModify     = XModify;
        pWorkUnitToUse->YCount      = YCount;
        pWorkUnitToUse->YModify     = YModify;
        pWorkUnitToUse->Config      = Config;

        /* Re-enable interrupts */
        adi_osal_ExitCriticalRegion();
    }
    /* ELSE (DMA Paused/Data transfer not enabled/Buffer repeat disabled) */
    else
    {
        /* Entering critical region, disable interrupts */
        adi_osal_EnterCriticalRegion();

        /* IF (Work Unit 0 is free) */
        if (pChannel->WorkUnit0.bIsFree_BufDone)
        {
            /* Use work unit 0 */
            pWorkUnitToUse = &pChannel->WorkUnit0;
        }
        /* ELSE IF (Work Unit 1 is free) */
        else if (pChannel->WorkUnit1.bIsFree_BufDone)
        {
            /* Use work unit 1 */
            pWorkUnitToUse = &pChannel->WorkUnit1;
        }
        /* ELSE (Both work units are in use) */
        else
        {
        	/* Re-enable interrupts */
        	adi_osal_ExitCriticalRegion();
        	/* Report failure since the DMA work units are full */
        	return (ADI_DMA_WORK_UNIT_FULL);
        }

        /* Load work unit */
        pWorkUnitToUse->pStartAddr  = pStartAddress;
        pWorkUnitToUse->XCount      = XCount;
        pWorkUnitToUse->XModify     = XModify;
        pWorkUnitToUse->YCount      = YCount;
        pWorkUnitToUse->YModify     = YModify;
        pWorkUnitToUse->Config      = Config;
        /* Mark this work unit as in use */
        pWorkUnitToUse->bIsFree_BufDone = false;

        /* Re-enable interrupts */
        adi_osal_ExitCriticalRegion();

        /* IF (DMA data transfer enabled and DMA is in idle state) */
        if ((pChannel->bIsXferInProgress) &&\
            ((pDmaInfo->pReg->Status & BITM_DMA_STAT_RUN) == ENUM_DMA_STAT_STOPPED))
        {
            /* Load DMA registers to start Peripheral DMA data transfer */
            return (LoadPDmaRegsWithWorkUnit (pDmaInfo, pWorkUnitToUse));
        }
    }

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/*
 * Load a work unit for Circular transfer mode. This function
 * copies the supplied values to a free work unit.
 *
 * Parameters
 *  - [in]  pChannel                Peripheral DMA Channel to work on.
 *  - [in]  pStartAddress           Start address of the data buffer to be processed.
 *  - [in]  NumSubBuffers           Number of sub-buffers in this circular buffer.
 *  - [in]  ElementsPerSubBuffer    Number of data elements per sub-buffer.
 *  - [in]  bBufferDoneCallback     TRUE to callback on buffer completion.
 *  - [in]  bSubBufferDoneCallback  TRUE to callback on each sub-buffer completion.
 *  - [in]  Direction               DMA Direction
 *
 * Return
 *  - ADI_DMA_SUCCESS: Successfully submitted a circular buffer.
 *  - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *  - ADI_DMA_BAD_START_ADDR: Supplied circular buffer start address is invalid.
 *  - ADI_DMA_BAD_CIRC_BUFFER: Circular Buffer configuration is invalid.
 *  - ADI_DMA_FN_NOT_PERMITTED:  Function not permitted when DMA data transfer is enabled.
 *  - ADI_DMA_BUFFER_TYPE_MISMATCH: Can't submit a circular buffer since the DMA is
 *                                  already configured for list buffer data transfer.
 *  - ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                  buffer address and will cause alignment error.
 *
 */
static ADI_DMA_RESULT LoadAsCircDescWorkUnit(
    ADI_PDMA_CHANNEL                *pChannel,
    void                            *pStartAddress,
    uint32_t                        NumSubBuffers,
    uint32_t                        ElementsPerSubBuffer,
    bool                            bBufferDoneCallback,
    bool                            bSubBufferDoneCallback,
    uint32_t                        Direction)
{

    /* Transfer size in bytes */
    int32_t     ElementWidthInBytes;
    /* DMA Config */
    uint32_t    Config;

    /* IF (DMA data transfer in progress) */
    if (pChannel->bIsXferInProgress)
    {
        /* Report error as this function is not allowed at this stage */
        return (ADI_DMA_FN_NOT_PERMITTED);
    }

    /* IF (DMA is set for List or one-shot transfer) */
    if (pChannel->bOneShotListXfer)
    {
        /* Report failure since the DMA is set for other data transfer mode */
        return (ADI_DMA_BUFFER_TYPE_MISMATCH);
    }

    /* Mark this DMA channel is set for circular mode */
    pChannel->bCircularXfer = true;

    /* Calculate element width in bytes */
    ElementWidthInBytes = (int32_t) (1 << ((pChannel->CommonConfig & BITM_DMA_CFG_MSIZE) >> BITP_DMA_CFG_MSIZE));

    /* Load work unit */
    pChannel->WorkUnit0.pNextDesc       = NULL;
    pChannel->WorkUnit0.pStartAddr      = pStartAddress;
    pChannel->WorkUnit0.XCount          = ElementsPerSubBuffer;
    pChannel->WorkUnit0.XModify         = ElementWidthInBytes;
    pChannel->WorkUnit0.YCount          = NumSubBuffers;
    pChannel->WorkUnit0.YModify         = ElementWidthInBytes;
    /* In circular mode, 'bIsFree_BufDone' flag is used to store the 'bBufferDoneCallback' settings */
    pChannel->WorkUnit0.bIsFree_BufDone = bBufferDoneCallback;

    /* Get DMA Config value for this work unit */
    Config = (ADI_PDMA_DMA_CFG_CIRCULAR | pChannel->CommonConfig | Direction);

    /* IF (Callback on sub-buffer completion) */
    if (bSubBufferDoneCallback)
    {
        /* Interrupt on inner loop completion (XCNTCUR reaches zero) */
        Config |= ENUM_DMA_CFG_XCNT_INT;
        /* Load variables to keep in track of the sub-buffer processed */
        pChannel->RemainSubBufs  = NumSubBuffers;
        pChannel->pBufferAddr   = (uint32_t *) pStartAddress;
    }
    /* ELSE (Callback on sub-buffer disabled) */
    else
    {
    	/* IF (Callback on buffer completion) */
    	if (bBufferDoneCallback)
    	{
    		/* Interrupt when YCNTCUR reaches zero */
    	    Config |= ENUM_DMA_CFG_YCNT_INT;
    	}
        pChannel->RemainSubBufs = 0;
        pChannel->pBufferAddr   = NULL;
    }

    /* Update DMA data interrupt and enable dataflow function pointers */
    pChannel->pfDataIntHandle = (ADI_DMA_DATA_INT_FN) &PDmaCircXferIntHandler;
    pChannel->pfEnableXfer    = (ADI_PDMA_ENABLE_FN)  &PDmaEnableCircOneShotXfer;

    /* Load the DMA config value to the work unit */
    pChannel->WorkUnit0.Config = Config;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/*
 * Enables List descriptor data transfer of a Peripheral DMA channel.
 *
 * Parameters
 *  - [in]  pDmaInfo    Pointer to Peripheral DMA Channel info whose
 *                      data transfer is to be enabled.
 *
 * Returns
 *  - ADI_DMA_SUCCESS: Successfully enabled List descriptor data transfer of a Peripheral DMA Channel.
 *  - ADI_DMA_NO_BUFFER_TO_PROCESS: Can't start DMA data transfer as there is no valid buffer to process.
 *  - ADI_DMA_NO_VALID_CALLBACK_FN: Can't start DMA data transfer as a valid callback function is not supplied.
 *
 */
static  ADI_DMA_RESULT PDmaEnableListXfer (ADI_DMA_INFO     *pDmaInfo)
{

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)
    /* Return code */
    ADI_DMA_RESULT  eResult;
#endif /* ADI_DEBUG */

    /* IF (Work unit 0 is in use) */
    if (pDmaInfo->pChannel->PDMA.WorkUnit0.bIsFree_BufDone == false)
    {
        /* Clear the common configuration data */
        pDmaInfo->pChannel->PDMA.WorkUnit0.Config &= ~ADI_PDMA_MASK_COMMON_CFG;
        /* Update with the current common DMA configuration data */
        pDmaInfo->pChannel->PDMA.WorkUnit0.Config |= pDmaInfo->pChannel->PDMA.CommonConfig;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Validate buffer address and its corresponding DMA register values */
    eResult = ValidatePDmaListBuffer ((ADI_PDMA_CHANNEL *) pDmaInfo->pChannel,
                                      (uint32_t) pDmaInfo->pChannel->PDMA.WorkUnit0.pStartAddr,
                                      pDmaInfo->pReg->Status,
                                      pDmaInfo->pChannel->PDMA.WorkUnit0.XCount,
                                      pDmaInfo->pChannel->PDMA.WorkUnit0.XModify,
                                      pDmaInfo->pChannel->PDMA.WorkUnit0.YCount,
                                      pDmaInfo->pChannel->PDMA.WorkUnit0.YModify);

    /* IF (Failed to validate buffer address and its corresponding DMA register values) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    }

    /* IF (Work unit 1 is in use) */
    if (pDmaInfo->pChannel->PDMA.WorkUnit1.bIsFree_BufDone == false)
    {
        /* Clear the common configuration data */
        pDmaInfo->pChannel->PDMA.WorkUnit1.Config &= ~ADI_PDMA_MASK_COMMON_CFG;
        /* Update with the current common DMA configuration data */
        pDmaInfo->pChannel->PDMA.WorkUnit1.Config |= pDmaInfo->pChannel->PDMA.CommonConfig;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Validate buffer address and its corresponding DMA register values */
    eResult = ValidatePDmaListBuffer ((ADI_PDMA_CHANNEL *) pDmaInfo->pChannel,
                                      (uint32_t) pDmaInfo->pChannel->PDMA.WorkUnit1.pStartAddr,
                                      pDmaInfo->pReg->Status,
                                      pDmaInfo->pChannel->PDMA.WorkUnit1.XCount,
                                      pDmaInfo->pChannel->PDMA.WorkUnit1.XModify,
                                      pDmaInfo->pChannel->PDMA.WorkUnit1.YCount,
                                      pDmaInfo->pChannel->PDMA.WorkUnit1.YModify);

    /* IF (Failed to validate buffer address and its corresponding DMA register values) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    }

    /* IF (Buffer repetition enabled and the second work unit is free) */
    if ((pDmaInfo->pChannel->PDMA.bBufRepeat) && (pDmaInfo->pChannel->PDMA.WorkUnit1.bIsFree_BufDone))
    {
        /* Copy Work Unit 0 to Work Unit 1 */
    	memcpy (&pDmaInfo->pChannel->PDMA.WorkUnit1, &pDmaInfo->pChannel->PDMA.WorkUnit0, sizeof (ADI_PDMA_DESC_WORK_UNIT));
    	/* Overwrite the next descriptor pointer on Work unit 1 to point to Work unit 0 */
    	pDmaInfo->pChannel->PDMA.WorkUnit1.pNextDesc = &pDmaInfo->pChannel->PDMA.WorkUnit0;
    }

    /* Load DMA registers to start Peripheral DMA data transfer */
    return (LoadPDmaRegsWithWorkUnit (pDmaInfo, &(pDmaInfo->pChannel->PDMA.WorkUnit0)));
}

/*
 * Enables circular/One-shot data transfer of a Peripheral DMA channel.
 *
 * Parameters
 *  [in]  pDmaInfo    Pointer to Peripheral DMA Channel info whose
 *                    data transfer is to be enabled.
 *
 * Returns
 *  - ADI_DMA_SUCCESS: Successfully enabled circular data transfer of a Peripheral DMA Channel.
 *  - ADI_DMA_NO_BUFFER_TO_PROCESS: Can't start DMA data transfer as there is no valid buffer to process.
 *  - ADI_DMA_NO_VALID_CALLBACK_FN: Can't start DMA data transfer as a valid callback function is not supplied.
 *
 */
static  ADI_DMA_RESULT PDmaEnableCircOneShotXfer (ADI_DMA_INFO     *pDmaInfo)
{
    /* Element width in bytes */
    int32_t     ElementWidthInBytes;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)
    /* Return code */
    ADI_DMA_RESULT  eResult;
#endif /* ADI_DEBUG */

    /* Clear the common configuration data */
    pDmaInfo->pChannel->PDMA.WorkUnit0.Config &= ~ADI_PDMA_MASK_COMMON_CFG;
    /* Update with the current common DMA configuration data */
    pDmaInfo->pChannel->PDMA.WorkUnit0.Config |= pDmaInfo->pChannel->PDMA.CommonConfig;

    /* IF (DMA is in circular mode) */
	if (pDmaInfo->pChannel->PDMA.bCircularXfer)
	{
/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

        /* Validate buffer address and its corresponding DMA register values */
        eResult = ValidatePDmaCircBuffer (pDmaInfo,
                                          (uint32_t) pDmaInfo->pChannel->PDMA.WorkUnit0.pStartAddr,
                                          pDmaInfo->pReg->Status,
                                          pDmaInfo->pChannel->PDMA.WorkUnit0.YCount,
                                          pDmaInfo->pChannel->PDMA.WorkUnit0.XCount);

        /* IF (Failed to validate buffer address and its corresponding DMA register values) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

#endif /* ADI_DEBUG */

        /* Calculate element width in bytes */
        ElementWidthInBytes = (int32_t) (1 << ((pDmaInfo->pChannel->PDMA.CommonConfig & BITM_DMA_CFG_MSIZE) >> BITP_DMA_CFG_MSIZE));

        /* Update work unit */
        pDmaInfo->pChannel->PDMA.WorkUnit0.XModify = ElementWidthInBytes;
        pDmaInfo->pChannel->PDMA.WorkUnit0.YModify = ElementWidthInBytes;

        /* IF (Callback on sub-buffer completion) */
        if ((pDmaInfo->pChannel->PDMA.WorkUnit0.Config & BITM_DMA_CFG_INT) == ENUM_DMA_CFG_XCNT_INT)
        {
            /* Update bytes per sub-buffer */
            pDmaInfo->pChannel->PDMA.BytesPerSubBuf = ((uint32_t)pDmaInfo->pChannel->PDMA.WorkUnit0.XCount * ElementWidthInBytes);
        }
    }
/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)
    /* ELSE (One-shot mode) */
    else
    {
        /* Validate buffer address and its corresponding DMA register values */
        eResult = ValidatePDmaListBuffer ((ADI_PDMA_CHANNEL *) pDmaInfo->pChannel,
                                          (uint32_t) pDmaInfo->pChannel->PDMA.WorkUnit0.pStartAddr,
                                          pDmaInfo->pReg->Status,
                                          pDmaInfo->pChannel->PDMA.WorkUnit0.XCount,
                                          pDmaInfo->pChannel->PDMA.WorkUnit0.XModify,
                                          pDmaInfo->pChannel->PDMA.WorkUnit0.YCount,
                                          pDmaInfo->pChannel->PDMA.WorkUnit0.YModify);

        /* IF (Failed to validate buffer address and its corresponding DMA register values) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }
    }
#endif /* ADI_DEBUG */

    /* Load DMA registers to start Peripheral DMA data transfer */
    return (LoadPDmaRegsWithWorkUnit (pDmaInfo, &(pDmaInfo->pChannel->PDMA.WorkUnit0)));
}

/*
 * Loads Peripheral DMA registers with a work unit.
 *
 * Parameters
 *  - [in]  pDmaInfo            DMA Channel Info to work on.
 *  - [in]  pWorkUnitToLoad     DMA work unit to load.
 *
 * Returns
 *  - ADI_DMA_SUCCESS: Successfully loaded Peripheral DMA registers with a work unit.
 *  - ADI_DMA_NO_BUFFER_TO_PROCESS: Can't start DMA data transfer as there is no valid buffer to process.
 *  - ADI_DMA_NO_VALID_CALLBACK_FN: Can't start DMA data transfer as a valid callback function is not supplied.
 *
 */
static ADI_DMA_RESULT LoadPDmaRegsWithWorkUnit (
    ADI_DMA_INFO const          *pDmaInfo,
    ADI_PDMA_DESC_WORK_UNIT     *pWorkUnitToLoad)
{

    /* DMA Channel instance to work on */
    ADI_PDMA_CHANNEL    *pChannel = (ADI_PDMA_CHANNEL *) pDmaInfo->pChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

	/* IF (DMA is in circular mode) */
	if (pChannel->bCircularXfer)
	{
		/* IF (DMA data interrupt enabled and we've no valid callback function) */
		if ((pWorkUnitToLoad->Config & BITM_DMA_CFG_INT) && (pChannel->pfCallback == NULL))
		{
			/* Return failure as we need a valid callback function */
			return (ADI_DMA_NO_VALID_CALLBACK_FN);
		}
	}
	/* ELSE (DMA is not in circular mode) */
	else
	{
		/* IF (we've no valid callback function) */
		if (pChannel->pfCallback == NULL)
		{
			/* Return failure as we need a valid callback function */
			return (ADI_DMA_NO_VALID_CALLBACK_FN);
		}
	}

	/* IF (Work unit does not have valid descriptors to start data transfer) */
	if ((pWorkUnitToLoad->Config & BITM_DMA_CFG_EN) == ENUM_DMA_CFG_DIS)
	{
		/* DMA EN bit is not set for this work unit */
		/* Report error since we've no valid buffer to start data transfer */
		return (ADI_DMA_NO_BUFFER_TO_PROCESS);
	}

#endif /* ADI_DEBUG */

	/* Entering critical region, disable interrupts */
	adi_osal_EnterCriticalRegion();

	/* Data transfer in progress */
	pChannel->bIsXferInProgress = true;
	/* Update pointer to work unit being processed */
	pChannel->pActiveWorkUnit   = pWorkUnitToLoad;

    /* IF (Large List transfer) */
    if ((pChannel->bOneShotListXfer) && ((pChannel->bIsStreaming) || (pChannel->bBufRepeat)))
    {
        /* Load the Next descriptor address */
	    pDmaInfo->pReg->NextDescPtr = (uint32_t) pWorkUnitToLoad;
    }
    /* ELSE (One-shot or circular transfer) */
    else
    {
	    /* Load the work unit */
	    pDmaInfo->pReg->StartAddr   = (uint32_t) pWorkUnitToLoad->pStartAddr;
	    pDmaInfo->pReg->XCount      = pWorkUnitToLoad->XCount;
	    pDmaInfo->pReg->XModify     = pWorkUnitToLoad->XModify;
	    pDmaInfo->pReg->YCount      = pWorkUnitToLoad->YCount;
	    pDmaInfo->pReg->YModify     = pWorkUnitToLoad->YModify;
    }

    /* Clear any pending DMA interrupts */
    pDmaInfo->pReg->Status = ADI_DMA_CLEAR_INTERRUPTS;
	/* Load DMA Config register */
	pDmaInfo->pReg->Config = pWorkUnitToLoad->Config;

	/* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

	/* IF (DMA interrupt disabled) */
	if (pChannel->bIsIntDisabled)
	{
		/* Update flag */
		pChannel->bIsIntDisabled = false;
		/* Enable DMA data interrupt */
		adi_int_EnableInt ((uint32_t)pDmaInfo->DataIntSID, true);
	}

	/* Return success */
	return (ADI_DMA_SUCCESS);
}

/*
 * DMA Interrupt handler for One-Shot/List descriptor data transfer
 *
 * Parameters
 *  - [in]  pDmaInfo  DMA Channel Info to which the interrupt belongs.
 *
 * Returns
 *  None
 *
 */
static void PDmaOneShotListXferIntHandler(ADI_DMA_INFO const *pDmaInfo)
{
    /* DMA FLOW Mode of the processed work unit */
    uint32_t                DmaFlowMode;
    /* Peripheral DMA channel to work on */
    ADI_PDMA_CHANNEL        *pChannel = (ADI_PDMA_CHANNEL *) pDmaInfo->pChannel;
    /* Next Work Unit in line to process */
    ADI_PDMA_DESC_WORK_UNIT *pNextWorkUnit = (ADI_PDMA_DESC_WORK_UNIT *) pChannel->pActiveWorkUnit->pNextDesc;
    /* Callback Argument */
    void                    *pArg = (void *) pChannel->pActiveWorkUnit->pStartAddr;

    /* IF (Buffer Repetition Enabled) */
    if (pChannel->bBufRepeat)
    {
        /* Update the active work unit pointer */
        pChannel->pActiveWorkUnit = pNextWorkUnit;
    }
    /* ELSE (Buffer repetition disabled) */
    else
    {
        /* Extract the DMA Flow mode of the processed work unit */
        DmaFlowMode = (pChannel->pActiveWorkUnit->Config & BITM_DMA_CFG_FLOW);
        /* Disable DMA for the Processed Work Unit */
        pChannel->pActiveWorkUnit->Config = 0;
        /* Set the processed work unit as free */
        pChannel->pActiveWorkUnit->bIsFree_BufDone = true;

        /* IF (we've no work unit to be process) */
        if (pNextWorkUnit->bIsFree_BufDone)
        {
            /* IF (The processed work unit was not configured in STOP Flow mode) */
            if (DmaFlowMode != ENUM_DMA_CFG_STOP)
            {
            	/* Update flag as interrupt disabled */
            	pChannel->bIsIntDisabled = true;
                /* Disable DMA interrupts since there are no buffers to process */
                /* This has to be done to avoid unnecessary interrupts that
                   occur when there is no valid work unit to process */
                adi_int_EnableInt ((uint32_t)pDmaInfo->DataIntSID, false);
            }
        }
        /* ELSE (we've next work unit ready to load) */
        else
        {
            /* IF (The processed work unit was configured in STOP Flow mode) */
            if (DmaFlowMode == ENUM_DMA_CFG_STOP)
            {
                /* Load the next work unit */
	            pDmaInfo->pReg->StartAddr = (uint32_t) pNextWorkUnit->pStartAddr;
	            pDmaInfo->pReg->XCount    = pNextWorkUnit->XCount;
	            pDmaInfo->pReg->XModify   = pNextWorkUnit->XModify;
	            pDmaInfo->pReg->YCount    = pNextWorkUnit->YCount;
	            pDmaInfo->pReg->YModify   = pNextWorkUnit->YModify;
	            pDmaInfo->pReg->Config    = pNextWorkUnit->Config;
	        }

	        /* Update the active work unit pointer */
            pChannel->pActiveWorkUnit = pNextWorkUnit;
        }
    }

    /* IF (We've a valid callback function) */
    if (pChannel->pfCallback != NULL)
    {
    	/* Post callback */
    	/* Callback Argument as Address of data buffer processed */
        (pChannel->pfCallback) (pChannel->pCBParam, ADI_DMA_EVENT_BUFFER_PROCESSED, pArg);
    }

}

/*
 * DMA Interrupt handler for Circular data transfer
 *
 * Parameters
 *  - [in]  pDmaInfo  DMA Channel Info to which the interrupt belongs.
 *
 * Returns
 *  - None
 *
 */
static void PDmaCircXferIntHandler(ADI_DMA_INFO const *pDmaInfo)
{
    /* Peripheral DMA channel to work on */
    ADI_PDMA_CHANNEL    *pChannel = (ADI_PDMA_CHANNEL *) pDmaInfo->pChannel;

    /* IF (Post callback when a sub-buffer is processed) */
    if (pChannel->pBufferAddr != NULL)
    {
    	/* IF (We've a valid callback function) */
    	if (pChannel->pfCallback != NULL)
    	{
    		/* Post callback */
    		/* Callback Argument as Address of sub-buffer processed */
    	    (pChannel->pfCallback) (pChannel->pCBParam, ADI_DMA_EVENT_SUB_BUFFER_PROCESSED, pChannel->pBufferAddr);
    	}

        /* Decrement the remain sub-buffers count */
        pChannel->RemainSubBufs--;

        /* IF (we've no more sub-buffers to process) */
        if (pChannel->RemainSubBufs == 0u)
        {
            /* Back to the start address of the circular buffer */
            pChannel->pBufferAddr = (void *) pChannel->WorkUnit0.pStartAddr;
        }
        /* ELSE (we've more sub-buffers to process) */
        else
        {
            /* Get the address of sub-buffer that is next in line for processing */
            pChannel->pBufferAddr = (void *) ((uint32_t)pChannel->pBufferAddr + pChannel->BytesPerSubBuf);
        }
    }

    /* IF (we've no sub-buffers to process) */
    if (pChannel->RemainSubBufs == 0u)
    {
        /* In circular mode, 'bIsFree_BufDone' flag is used to store the 'bBufferDoneCallback' settings */
    	/* IF (We've a valid callback function and Callback on buffer completion is enabled) */
    	if ((pChannel->pfCallback != NULL) && (pChannel->WorkUnit0.bIsFree_BufDone))
    	{
    		/* Post callback */
    	    /* Callback Argument as Address of data buffer processed */
    	    (pChannel->pfCallback) (pChannel->pCBParam, ADI_DMA_EVENT_BUFFER_PROCESSED, (void *) pChannel->WorkUnit0.pStartAddr);
    	}

        /* IF (Post callback when a sub-buffer is processed) */
        if (pChannel->pBufferAddr != NULL)
        {
            /* Reload sub-buffer count */
            pChannel->RemainSubBufs = pChannel->WorkUnit0.YCount;
        }
    }
}

/*=============  D E B U G   F U N C T I O N S =============*/

/* IF (Debug mode enabled) */
#if defined (ADI_DEBUG)

/*
 * Checks if the given channel ID belongs to peripheral DMA or not.
 *
 * Parameters
 *  - [in]  eChannelID    DMA Channel ID to validate.
 *
 * Returns  Status
 *  - ADI_DMA_SUCCESS: DMA Channel ID is valid.
 *  - ADI_DMA_BAD_CHANNEL_ID: DMA Channel ID is invalid.
 *
 */
static ADI_DMA_RESULT ValidatePDmaChannel (ADI_DMA_CHANNEL_ID    eChannelID)
{
    /* Loop variable */
    uint8_t    i;

    /* IF (DMA channel ID is invalid) */
    if (eChannelID > ADI_DMA_CHANNEL_END)
    {
        /* Report failure as bad channel id */
        return (ADI_DMA_BAD_CHANNEL_ID);
    }

    /* FOR (All Memory DMA channels in this processor) */
    for (i = 0; i < sizeof (StreamInfo)/sizeof (StreamInfo[0]); i++)
    {
        /* IF (Channel Number belongs to a MDMA stream) */
        if ((eChannelID == StreamInfo[i].DestChannel) ||\
            (eChannelID == StreamInfo[i].SrcChannel))
        {
            /* Report failure as bad channel id */
            return (ADI_DMA_BAD_CHANNEL_ID);
        }
    }

    /* Channel ID is valid and must be a peripheral DMA */
    return (ADI_DMA_SUCCESS);
}

/*
 * Validates list buffer address and its corresponding DMA register values.
 *
 * Parameters
 *  - [in]  pChannel        DMA Channel to which the buffer belongs.
 *  - [in]  StartAddress    Start address of the data buffer to be processed.
 *  - [in]  Status          DMA Status register value.
 *  - [in]  XCount          Inner loop count start value in number of
 *                          DMA data bus width (MSIZE) transfers.
 *  - [in]  XModify         Inner loop address increment in bytes.
 *  - [in]  YCount          Outer loop count start value in number of rows.
 *  - [in]  YModify         Outer loop address increment in bytes.
 *
 * Returns  Status
 *  - ADI_DMA_SUCCESS: Supplied List buffer is valid.
 *  - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *  - ADI_DMA_BAD_XCOUNT: Supplied XCount is invalid.
 *  - ADI_DMA_BAD_YCOUNT: Supplied YCount is invalid.
 *  - ADI_DMA_BAD_START_ADDR: Supplied start address is invalid.
 *  - ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                  buffer address and will cause alignment error.
 *
 */
static ADI_DMA_RESULT   ValidatePDmaListBuffer (
    ADI_PDMA_CHANNEL    *pChannel,
    uint32_t            StartAddress,
    uint32_t            Status,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify)
{
    /* Validate buffer address */
    return (ValidateBuffer (StartAddress,
                            pChannel->CommonConfig, Status,
                            XCount, XModify, YCount, YModify));
}

/*
 * Validates circular buffer address and its corresponding DMA register values.
 *
 * Parameters
 *  - [in]  pDmaInfo                DMA Channel to which the buffer belongs.
 *  - [in]  StartAddress            Start address of the data buffer to be processed.
 *  - [in]  Status                  DMA Status register value.
 *  - [in]  NumSubBuffers           Number of sub-buffers in this circular buffer.
 *  - [in]  ElementsPerSubBuffer    Number of data elements per sub-buffer.
 *
 * Returns  Status
 *  - ADI_DMA_SUCCESS: Supplied circular buffer is valid.
 *  - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *  - ADI_DMA_BAD_START_ADDR: Supplied start address is invalid.
 *  - ADI_DMA_BAD_CIRC_BUFFER: Circular Buffer configuration is invalid.
 *  - ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                  buffer address and will cause alignment error.
 *
 */
static ADI_DMA_RESULT   ValidatePDmaCircBuffer (
    ADI_DMA_INFO        *pDmaInfo,
    uint32_t            StartAddress,
    uint32_t            Status,
    uint32_t            NumSubBuffers,
    uint32_t            ElementsPerSubBuffer)
{
    /* Element width in bytes */
    uint32_t    ElementWidthInBytes;

    /* IF (Elements per sub buffer or number of sub buffer count is invalid) */
    if ((ElementsPerSubBuffer == 0) || (NumSubBuffers == 0))
    {
    	/* Report failure as circular buffer configuration is invalid */
        return (ADI_DMA_BAD_CIRC_BUFFER);
    }

    /* Calculate element width in bytes */
    ElementWidthInBytes = (int32_t) (1 << ((pDmaInfo->pChannel->PDMA.CommonConfig & BITM_DMA_CFG_MSIZE) >> BITP_DMA_CFG_MSIZE));

    /* Validate buffer address */
    return (ValidateBuffer (StartAddress,
                            pDmaInfo->pChannel->PDMA.CommonConfig,
                            Status,
                            ElementsPerSubBuffer,
                            ElementWidthInBytes,
                            NumSubBuffers,
                            ElementWidthInBytes));
}

/* End of debug functions */
#endif /* ADI_DEBUG */

/*=============  P U B L I C   F U N C T I O N S =============*/

/**
 * @brief Opens a peripheral DMA channel for use.
 *
 * @param [in]  eChannelID      DMA Channel ID to open.
 * @param [in]  pChannelMemory  Pointer to ADI_DMA_CHANNEL_REQUIRED_MEMORY sized memory
 *                              location to handle the DMA channel.
 * @param [out] phChannel       Address where the DMA channel handle will be stored.
 * @param [in]  pfCallback      Address of application callback function.
 * @param [in]  pCBParam        Parameter passed back to application callback.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully opened a DMA Channel.
 *          - #ADI_DMA_FAILURE: Supplied pointer parameters are invalid.
 *          - #ADI_DMA_IN_USE: DMA Channel can't be opened as it is already in use.
 *          - #ADI_DMA_BAD_CHANNEL_ID: Supplied Peripheral DMA channel ID is invalid.
 *          - #ADI_DMA_INT_REGISTER_FAILED: Failed to register DMA interrupt handler with the interrupt dispatcher.
 *
 */
ADI_DMA_RESULT adi_pdma_Open(
    ADI_DMA_CHANNEL_ID              eChannelID,
    void *const                     pChannelMemory,
    ADI_DMA_CHANNEL_HANDLE *const   phChannel,
    ADI_CALLBACK                    pfCallback,
    void *const                     pCBParam)
{
    /* Return Code */
    ADI_DMA_RESULT      eResult;
    /* Memory to handle DMA channel */
    ADI_PDMA_CHANNEL    *pChannel = (ADI_PDMA_CHANNEL *) pChannelMemory;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (This is not a peripheral DMA channel) */
    if (ValidatePDmaChannel(eChannelID) != ADI_DMA_SUCCESS)
    {
        /* Report failure as bad channel id */
        return (ADI_DMA_BAD_CHANNEL_ID);
    }

    /* Check if the given pointer parameters are valid */
    if((pChannelMemory == NULL) || (phChannel == NULL))
    {
        return (ADI_DMA_FAILURE);
    }

    /* Asserts to validate memory size required to manage a DMA channel */
    assert(ADI_DMA_CHANNEL_REQ_MEMORY >= sizeof(ADI_PDMA_CHANNEL));

#endif  /* ADI_DEBUG */

    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

    /* IF (DMA channel is already in use) */
    if (ChannelInfo[(uint8_t)eChannelID].pChannel != NULL)
    {
        /* Report failure as channel is already in use */
        eResult = ADI_DMA_IN_USE;
    }
    /* ELSE (DMA Stream is available for use) */
	else
	{
		/* Save the memory address allocated to handle this DMA channel */
        ChannelInfo[(uint8_t)eChannelID].pChannel = (ADI_DMA_CHANNEL *)pChannel;
        eResult = ADI_DMA_SUCCESS;
	}

	/* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();

    /* IF (DMA Channel is available for use) */
    if (eResult == ADI_DMA_SUCCESS)
    {
        /* Reset DMA channel registers */
        ResetDmaRegs (ChannelInfo[(uint8_t)eChannelID].pReg);

/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)
        /* Get the Bitwise channel number and registry index */
        GetBitWiseInfo ((uint8_t)eChannelID, &(pChannel->BitWiseRegistryIndex), &(pChannel->BitWiseChannelNum));
#endif

        /* Reset DMA Configuration common to all modes */
        pChannel->CommonConfig  = 0;
        /* Buffer repeat is disabled by default */
        pChannel->bBufRepeat    = false;
        /* DMA transfer mode set to one-shot transfer by default */
        pChannel->bIsStreaming  = false;
        pChannel->pfCallback    = pfCallback;
        pChannel->pCBParam      = pCBParam;
        /* Reset this channel instance */
        ResetPDmaChannel (pChannel);

        /* Register DMA channel with Interrupt dispatcher */
        eResult = RegisterWithIntDispatcher (&ChannelInfo[(uint8_t)eChannelID]);
    }

    /* IF (Successfully registered DMA channel with Interrupt dispatcher) */
    if (eResult == ADI_DMA_SUCCESS)
    {
    	/* Update flag as interrupt enabled */
    	pChannel->bIsIntDisabled = false;

        /* Entering critical region, disable interrupts */
	    adi_osal_EnterCriticalRegion();

/* IF (ADSP-BF60x) or ADSP-BF70x family */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)
        /* Mark this DMA channel as open to handle DMA error */
        AdiDmaBitWiseRegistry[pChannel->BitWiseRegistryIndex] |= (1 << pChannel->BitWiseChannelNum);
        /* Update the number of DMA channels open count */
        AdiDmaNumChannelsOpen ++;
#endif

        /* Re-enable interrupts */
	    adi_osal_ExitCriticalRegion();

        /* Pass a valid handle to this DMA Channel */
        *phChannel = (ADI_DMA_CHANNEL_HANDLE) &ChannelInfo[(uint8_t)eChannelID];
    }
    /* ELSE (Failed to register MDMA stream with interrupt dispatcher) */
	else
	{
		/* Clear the memory address allocated to handle the DMA Channel */
		ChannelInfo[(uint8_t)eChannelID].pChannel = NULL;
	}

    /* Return */
    return (eResult);
}

/**
 * @brief Closes a peripheral DMA channel.
 *
 * @param [in]  hChannel    DMA Channel handle to be closed.
 * @param [in]  bWaitFlag   Wait for transfers to complete flag
 *                          (TRUE to wait, FALSE to force close).
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully closed a DMA Channel.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_INT_UNREGISTER_FAILED: Failed to un-register DMA interrupt handler.
 *
 */
ADI_DMA_RESULT adi_pdma_Close(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bWaitFlag)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (wait for DMA to complete transfer) */
    if (bWaitFlag)
    {
        /* wait until DMA is idle */
        while ((pDmaInfo->pReg->Status & BITM_DMA_STAT_RUN) != ENUM_DMA_STAT_STOPPED)
        {
          __NOP();  // KEIL: asm ("NOP;");
        }
    }

    /* Disable DMA */
    pDmaInfo->pReg->Config = 0;
    /* Reset DMA registers */
    ResetDmaRegs (pDmaInfo->pReg);

    /* Un-register DMA data interrupt handler for this channel */
    if (adi_int_UninstallHandler ((uint32_t)pDmaInfo->DataIntSID) != ADI_INT_SUCCESS)
    {
    	/* Return failure as we failed to un-register the interrupt */
    	return (ADI_DMA_INT_UNREGISTER_FAILED);
    }

    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

/* IF (ADSP-BF60x or ADSP-BF70x family) */
#if defined( __ADSPBF609_FAMILY__) || \
    defined( __ADSPBF707_FAMILY__)
    /* Remove this DMA channel from handling DMA errors */
    AdiDmaBitWiseRegistry[pDmaInfo->pChannel->PDMA.BitWiseRegistryIndex] &= ~(1 << pDmaInfo->pChannel->PDMA.BitWiseChannelNum);
    /* Decrement the number of DMA channels open count */
    AdiDmaNumChannelsOpen --;
#endif

    /* Clear the memory address allocated to handle the channel */
    pDmaInfo->pChannel = NULL;

    /* Re-enable interrupts */
    adi_osal_ExitCriticalRegion();

    /* If required, un-install DMA Error handler */
    return (UninstallDmaErrorHandler(pDmaInfo));
}

/**
 * @brief Enables/Disables Peripheral data streaming.
 *
 * @details Enable/Disable Data streaming applies only when a driver
 *          uses buffer queuing APIs adi_pdma_PeripheralWrite and
 *          adi_pdma_PeripheralRead for buffer submission.
 *          Data streaming is always enabled for Circular transfer modes.
 *          When streaming is disabled, each buffer will be treated as
 *          a single entity for short burst of data transfers. This mode
 *          is suitable for devices such as SPI or UART.
 *          When streaming mode is enabled, each buffer will be treated as
 *          part of a buffer chain used for streaming data. This mode is
 *          suitable for high speed peripherals such as PPI or SPORT.
 *
 * @param [in]  hChannel    Handle to the DMA Channel to work on.
 * @param [in]  bEnable     'true' to enable streaming, 'false' to disable (default).
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated DMA Streaming settings.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *
 * @note
 *        Streaming mode is disabled by default when a DMA channel is opened.
 *        Streaming mode needs to be set only once after opening the DMA channel
 *        since the DMA Manager will retain the settings even when the DMA
 *        dataflow is disabled. DMA Streaming settings should be updated
 *        before submitting buffers. Any changes to DMA streaming configuration
 *        will not affect the buffers that are already submitted to the DMA channel.
 *
 * @sa  adi_pdma_PeripheralWrite, adi_pdma_PeripheralRead
 *
 */
ADI_DMA_RESULT adi_pdma_EnableStreaming(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Update DMA streaming mode */
    pDmaInfo->pChannel->PDMA.bIsStreaming = bEnable;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Enables/Disables buffer repetition.
 *
 * @details Enable/Disable buffer repetition applies only when a driver
 *          uses buffer queuing APIs adi_pdma_PeripheralWrite and
 *          adi_pdma_PeripheralRead for buffer submission.
 *          When buffer repetition is enabled, the DMA channel will keep
 *          on processing (repeating) the last two buffers queued.
 *          The buffer can be updated using the buffer queuing APIs
 *          or using adi_pdma_UpdateDescriptor API when only the buffer address
 *          and/or XCount parameter in a descriptor needs to be updated.
 *
 * @param [in]  hChannel    DMA Channel handle to work on.
 * @param [in]  bEnable     'true' to enable buffer repetition, 'false' to disable (default).
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated DMA buffer repeat settings.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't update buffer repeat settings when the
 *                                      DMA data transfer is enabled.
 *
 * @note: Buffer repetition mode is disabled by default when a DMA channel is opened.
 *        Buffer repetition mode needs to be set only once after opening the DMA channel
 *        since the DMA Manager will retain the settings even when the DMA
 *        dataflow is disabled.
 *        Buffer repetition is useful in cases such as video/audio playback
 *        where the dataflow should not stop.
 *        For video playback, when an application has to
 *          - Render a single image or
 *          - Repeat a frame for certain number of times or
 *          - Fails to submit new data buffer in time
 *        the buffer repetition mode will make sure that the display is
 *        kept alive by transmitting the last two buffers submitted.
 *        For audio playback, when there is no audio available for playback,
 *        the application can submit a buffer to continuously play silence or
 *        repeat the last two buffers of audio data.
 *        Buffer repetition settings can't be changed when the DMA dataflow is enabled.
 *
 * @sa  adi_pdma_UpdateDescriptor, adi_pdma_EnableStreaming
 *
 */
ADI_DMA_RESULT adi_pdma_EnableBufferRepeat(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA data transfer in progress) */
    if (pDmaInfo->pChannel->PDMA.bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* Update buffer repetition settings */
    pDmaInfo->pChannel->PDMA.bBufRepeat = bEnable;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configure DMA Sync field in DMA Configuration register
 *
 * @details Enables/Disables DMA Sync operation. By default, SYNC is disabled.
 *
 * @param [in]  hChannel    Handle to the DMA Channel to work on.
 * @param [in]  bEnable     'true' to enable SYNC, 'false' to disable (default).
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated SYNC settings.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't update DMA SYNC settings when the
 *                                      DMA data transfer is enabled.
 *
 */
ADI_DMA_RESULT adi_pdma_SetSync(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA data transfer in progress) */
    if (pDmaInfo->pChannel->PDMA.bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* IF (Enable SYNC) */
    if (bEnable)
    {
        /* Update SYNC config information */
        pDmaInfo->pChannel->PDMA.CommonConfig |= ENUM_DMA_CFG_SYNC;
    }
    /* ELSE (Disable SYNC) */
    else
    {
        pDmaInfo->pChannel->PDMA.CommonConfig &= ~BITM_DMA_CFG_SYNC;
    }

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures Peripheral interrupt forward settings
 *
 * @details Enables/Disables Peripheral interrupt forward.
 *          Peripheral interrupt forward is disabled by default.
 *
 * @param [in]  hChannel    Handle to the DMA Channel to work on.
 * @param [in]  bEnable     'true' to enable Peripheral interrupt forward, 'false' to disable
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated Peripheral interrupt forward settings.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *
 */
ADI_DMA_RESULT adi_pdma_SetPeripheralIntFwd(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (Enable Peripheral interrupt forward) */
    if (bEnable)
    {
       /* Future Release */
    }
    /* ELSE (Disable Peripheral interrupt forward) */
    else
    {
        /* Future Release */
    }

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures Peripheral data request forward settings
 *
 * @details Enables/Disables Peripheral data request forward.
 *          Peripheral data request forward is disabled by default.
 *
 * @param [in]  hChannel    Handle to the DMA Channel to work on.
 * @param [in]  bEnable     'true' to enable Peripheral data request forward, 'false' to disable
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated Peripheral data request forward settings.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *
 */
ADI_DMA_RESULT adi_pdma_SetPeripheralReqFwd(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    bool                            bEnable)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (Enable Peripheral data request forward) */
    if (bEnable)
    {
        /* Update Peripheral data request forward settings */
        pDmaInfo->pChannel->PDMA.CommonConfig |= ENUM_DMA_CFG_PDAT_FWD;
    }
    /* ELSE (Disable Peripheral data request forward) */
    else
    {
        pDmaInfo->pChannel->PDMA.CommonConfig &= ~BITM_DMA_CFG_PDRF;
    }

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures Peripheral DMA Memory Transfer Word Size.
 *
 * @param [in]  hChannel        DMA Channel handle to work on.
 * @param [in]  eMSize          Memory Transfer Word Size to be set.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated DMA Channel settings.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet.
 *          - #ADI_DMA_BAD_MSIZE: Supplied MSize exceeds the maximum value supported by this channel.
 *          - #ADI_DMA_BAD_HANDLE: Bad DMA Channel Handle.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't change DMA MSize when channel dataflow is enabled.
 *
 * @sa  adi_pdma_SetPSize
 *
 */
ADI_DMA_RESULT adi_pdma_SetMSize(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    ADI_DMA_MSIZE                   eMSize)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

    /* IF (The supplied MSize is more than the maximum MSize supported) */
    if ((uint32_t)eMSize > (uint32_t)ADI_DMA_MSIZE_32BYTES)
    {
        return (ADI_DMA_BAD_MSIZE);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pDmaInfo->pChannel->PDMA.bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* Clear the current MSize Configuration */
    pDmaInfo->pChannel->PDMA.CommonConfig &= ~BITM_DMA_CFG_MSIZE;
    /* Save the supplied MSize Configuration */
    pDmaInfo->pChannel->PDMA.CommonConfig |= eMSize;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures DMA Peripheral Transfer word size.
 *
 * @param [in]  hChannel        DMA Channel handle to work on.
 * @param [in]  ePSize          PSize to be set.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated DMA Channel settings.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet.
 *          - #ADI_DMA_BAD_PSIZE: Supplied PSize exceeds the maximum value supported by this channel.
 *          - #ADI_DMA_BAD_HANDLE: Bad DMA Channel Handle.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't change DMA PSize when channel dataflow is enabled.
 *
 * @sa  adi_pdma_SetMSize
 *
 */
ADI_DMA_RESULT adi_pdma_SetPSize(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    ADI_DMA_PSIZE                   ePSize)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Maximum Peripheral bus width supported */
    uint32_t    MaxPBWidth;

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

    /* Extract the Peripheral Bus Width information from Status register */
    MaxPBWidth = ((pDmaInfo->pReg->Status & BITM_DMA_STAT_PBWID) >> BITP_DMA_STAT_PBWID);
    /* Shift the value to MSize field location within the DMA Config register */
    MaxPBWidth <<= BITP_DMA_CFG_PSIZE;

    /* IF (The supplied PSize is more than the maximum PSize supported) */
    if ((uint32_t)ePSize > MaxPBWidth)
    {
        return (ADI_DMA_BAD_PSIZE);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pDmaInfo->pChannel->PDMA.bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* Clear the current PSize Configuration */
    pDmaInfo->pChannel->PDMA.CommonConfig &= ~BITM_DMA_CFG_PSIZE;
    /* Save the supplied PSize Configuration */
    pDmaInfo->pChannel->PDMA.CommonConfig |= ePSize;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Enable dataflow (data transfer) of a DMA channel.
 *
 * @param [in]  hChannel    DMA Channel handle whose dataflow is to be enabled.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully enabled DMA Channel dataflow.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_NO_BUFFER_TO_PROCESS:  Can't start DMA data transfer as there is no valid buffer to process.
 *          - #ADI_DMA_NO_VALID_CALLBACK_FN: Can't start DMA data transfer as a valid callback function is not supplied.
 *
 * @sa  adi_pdma_DisableXfer
 *
 */
ADI_DMA_RESULT adi_pdma_EnableXfer (ADI_DMA_CHANNEL_HANDLE const  hChannel)
{

    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA data transfer in progress) */
    if (pDmaInfo->pChannel->PDMA.bIsXferInProgress)
    {
    	/* Do nothing, just return success */
        return (ADI_DMA_SUCCESS);
    }
    /* ELSE IF (we've a valid data transfer enable function) */
    else if (pDmaInfo->pChannel->PDMA.pfEnableXfer != NULL)
    {
        /* Enable data transfer */
        return (pDmaInfo->pChannel->PDMA.pfEnableXfer (pDmaInfo));
    }
    /* ELSE (we've no buffer to process) */
    else
    {
        /* Return failure as we've no buffer to start data transfer */
        return (ADI_DMA_NO_BUFFER_TO_PROCESS);
    }
}

/**
 * @brief Disable dataflow (data transfer) of a DMA channel.
 *
 * @param [in]  hChannel    DMA Channel handle whose dataflow is to be disabled.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully disabled DMA Channel dataflow.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *
 * @sa  adi_pdma_EnableXfer
 *
 */
ADI_DMA_RESULT adi_pdma_DisableXfer (ADI_DMA_CHANNEL_HANDLE const    hChannel)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

    /* Disable DMA channel */
    pDmaInfo->pReg->Config = 0;
    /* Clear any pending DMA interrupts */
    pDmaInfo->pReg->Status = ADI_DMA_CLEAR_INTERRUPTS;
    /* Disable DMA interrupts */
	adi_int_EnableInt ((uint32_t)pDmaInfo->DataIntSID, false);
	/* Update flag as interrupt disabled */
	pDmaInfo->pChannel->PDMA.bIsIntDisabled = true;
    /* Reset Peripheral DMA Channel instance */
    ResetPDmaChannel ((ADI_PDMA_CHANNEL *) pDmaInfo->pChannel);

    /* Re-enable interrupts */
    adi_osal_ExitCriticalRegion();

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Submits a buffer to Peripheral DMA to write to Peripheral FIFO.
 *
 * @details Submits a buffer to read from memory and write to
 *          the peripheral linked to the DMA channel.
 *
 * @param [in]  hChannel        Handle to the DMA Channel to work on.
 * @param [in]  pStartAddress   Start address of the data buffer to be processed.
 * @param [in]  XCount          Inner loop count start value in number of
 *                              DMA data bus width (ADI_DMA_MSIZE) transfers.
 * @param [in]  XModify         Inner loop address increment in bytes.
 * @param [in]  YCount          Outer loop count start value in number of rows.
 * @param [in]  YModify         Outer loop address increment in bytes.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted a list buffer.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_BAD_XCOUNT: Supplied XCount is invalid.
 *          - #ADI_DMA_BAD_YCOUNT: Supplied YCount is invalid.
 *          - #ADI_DMA_BAD_START_ADDR: Supplied list buffer start address is invalid.
 *          - #ADI_DMA_WORK_UNIT_FULL: Can't submit a list buffer as the DMA work units are full.
 *          - #ADI_DMA_BUFFER_TYPE_MISMATCH: Can't submit a list buffer since the DMA is
 *                                           already configured for circular data transfer.
 *          - #ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                           buffer address and will cause alignment error.
 *
 * @note
 *      A Callback will be posted on completion of the buffer to notify write complete.
 *
 * @sa  adi_pdma_PeripheralRead
 *
 */
ADI_DMA_RESULT adi_pdma_PeripheralWrite(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        XCount,
    int32_t                         XModify,
    uint32_t                        YCount,
    int32_t                         YModify)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Load given values to Work unit */
    return (LoadAsListDescWorkUnit (pDmaInfo,
                                    pStartAddress,
                                    XCount,
                                    XModify,
                                    YCount,
                                    YModify,
                                    ENUM_DMA_CFG_READ));
}

/**
 * @brief Submits a buffer to Peripheral DMA to read from Peripheral FIFO.
 *
 * @details Submits a buffer to read data from a peripheral and write it to memory.
 *
 * @param [in]  hChannel        Handle to the DMA Channel to work on.
 * @param [in]  pStartAddress   Start address of the data buffer to be processed.
 * @param [in]  XCount          Inner loop count start value in number of
 *                              DMA data bus width (ADI_DMA_MSIZE) transfers.
 * @param [in]  XModify         Inner loop address increment in bytes.
 * @param [in]  YCount          Outer loop count start value in number of rows.
 * @param [in]  YModify         Outer loop address increment in bytes.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted a list buffer.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_BAD_XCOUNT: Supplied XCount is invalid.
 *          - #ADI_DMA_BAD_YCOUNT: Supplied YCount is invalid.
 *          - #ADI_DMA_BAD_START_ADDR: Supplied list buffer start address is invalid.
 *          - #ADI_DMA_WORK_UNIT_FULL: Can't submit a list buffer as the DMA work units are full.
 *          - #ADI_DMA_BUFFER_TYPE_MISMATCH: Can't submit a list buffer since the DMA is
 *                                           already configured for circular data transfer.
 *          - #ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                           buffer address and will cause alignment error.
 *
 * @note
 *      A Callback will be posted on completion of the buffer to notify read complete.
 *
 * @sa  adi_pdma_PeripheralWrite
 *
 */
ADI_DMA_RESULT adi_pdma_PeripheralRead(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        XCount,
    int32_t                         XModify,
    uint32_t                        YCount,
    int32_t                         YModify)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Load given values to Work unit */
    return (LoadAsListDescWorkUnit (pDmaInfo,
                                    pStartAddress,
                                    XCount,
                                    XModify,
                                    YCount,
                                    YModify,
                                    ENUM_DMA_CFG_WRITE));
}

/**
 * @brief Updates Start Address and XCount of a Descriptor work unit that is currently not active.
 *
 * @details Updates Start address and XCount of a descriptor work unit
 *          that is currently NOT being processed by the DMA.
 *          This Function is supported only when the DMA channel is operated in
 *          buffer repetition mode and when the DMA data transfer is enabled.
 *
 * @param [in]  hChannel        Handle to the DMA Channel to work on.
 * @param [in]  pNewStartAddr   Start address of the data buffer to update with.
 * @param [in]  XCount          Inner loop count start value in number of
 *                              DMA data bus width (ADI_DMA_MSIZE) transfers.
 *                              Pass it as zero to update only the start address field.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted buffer.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_BAD_START_ADDR: Supplied start address is invalid.
 *          - #ADI_DMA_FN_NOT_PERMITTED: Function not permitted when data transfer is disabled.
 *          - #ADI_DMA_FN_NOT_SUPPORTED: Function is supported only for Peripheral DMA channels operated in
 *                                       list transfer mode with buffer repetition mode enabled.
 *
 * @sa  adi_pdma_EnableBufferRepeat, adi_pdma_PeripheralWrite, adi_pdma_PeripheralRead
 *
 */
ADI_DMA_RESULT adi_pdma_UpdateDescriptor(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pNewStartAddr,
    uint32_t                        XCount)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO        *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

    /* IF (Start address is invalid) */
    if (pNewStartAddr == NULL)
    {
    	/* Report failure as supplied start address is invalid */
    	return (ADI_DMA_BAD_START_ADDR);
    }

    /* IF (DMA not set for Large List descriptor transfer) */
    if (pDmaInfo->pChannel->PDMA.bOneShotListXfer == false)
    {
        /* Report failure as this function is not supported for current transfer mode */
        return (ADI_DMA_FN_NOT_SUPPORTED);
    }

	/* IF (Data transfer is disabled) */
    if (pDmaInfo->pChannel->PDMA.bIsXferInProgress == false)
    {
    	/* Report failure as this function is not permitted when data transfer is disabled */
        return (ADI_DMA_FN_NOT_PERMITTED);
    }

#endif /* ADI_DEBUG */

    /* IF (Buffer repetition enabled) */
    if (pDmaInfo->pChannel->PDMA.bBufRepeat)
    {
        /* Entering critical region, disable interrupts */
        adi_osal_EnterCriticalRegion();

        /* Update the start address of the work unit that is currently not active
           ie., start address of the next work unit in chain for DMA processing */
        ((ADI_PDMA_DESC_WORK_UNIT *) pDmaInfo->pChannel->PDMA.pActiveWorkUnit->pNextDesc)->pStartAddr = pNewStartAddr;

        /* IF (XCount is valid) */
        if (XCount != 0u)
        {
            /* Update XCount of the work unit that is currently not active*/
            ((ADI_PDMA_DESC_WORK_UNIT *) pDmaInfo->pChannel->PDMA.pActiveWorkUnit->pNextDesc)->XCount = XCount;
        }

        /* Re-enable interrupts */
        adi_osal_ExitCriticalRegion();

        /* Report failure as DMA channel already running */
        return (ADI_DMA_SUCCESS);
    }
    /* ELSE (Buffer repetition disabled) */
    else
    {
        /* Report failure as this function supported only in Buffer repetition mode */
        return (ADI_DMA_FN_NOT_SUPPORTED);
    }
}

/**
 * @brief Submits a circular buffer to Peripheral DMA to write to Peripheral FIFO.
 *
 * @details Submits a circular buffer to read from memory and write to
 *          the peripheral linked to the DMA channel.
 *
 * @param [in]  hChannel                Handle to the DMA Channel to work on.
 * @param [in]  pStartAddress           Start address of the data buffer to be processed.
 * @param [in]  NumSubBuffers           Number of sub-buffers in this circular buffer
 * @param [in]  ElementsPerSubBuffer    Number of data elements per sub-buffer.
 * @param [in]  bBufferDoneCallback     'true' to post a callback on buffer completion.
 * @param [in]  bSubBufferDoneCallback  'true' to post a callback on each sub-buffer completion.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted a circular buffer.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_BAD_START_ADDR: Supplied circular buffer start address is invalid.
 *          - #ADI_DMA_BAD_CIRC_BUFFER: Circular Buffer configuration is invalid.
 *          - #ADI_DMA_FN_NOT_PERMITTED: Function not permitted when DMA data transfer is enabled.
 *          - #ADI_DMA_BUFFER_TYPE_MISMATCH: Can't submit a circular buffer since the DMA is
 *                                           already configured for list buffer data transfer.
 *          - #ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                           buffer address and will cause alignment error.
 *
 * @sa  adi_pdma_PeripheralCircRead
 *
 */
ADI_DMA_RESULT adi_pdma_PeripheralCircWrite(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        NumSubBuffers,
    uint32_t                        ElementsPerSubBuffer,
    bool                            bBufferDoneCallback,
    bool                            bSubBufferDoneCallback)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Return code */
    ADI_DMA_RESULT  eResult;

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

    /* Validate buffer address and its corresponding DMA register values */
    eResult = ValidatePDmaCircBuffer (pDmaInfo,
                                      (uint32_t) pStartAddress,
                                      pDmaInfo->pReg->Status,
                                      NumSubBuffers,
                                      ElementsPerSubBuffer);

    /* IF (Failed to validate buffer address and its corresponding DMA register values) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    /* Load given values to Work unit */
    return (LoadAsCircDescWorkUnit ((ADI_PDMA_CHANNEL *) pDmaInfo->pChannel,
                                    pStartAddress,
                                    NumSubBuffers,
                                    ElementsPerSubBuffer,
                                    bBufferDoneCallback,
                                    bSubBufferDoneCallback,
                                    ENUM_DMA_CFG_READ));
}

/**
 * @brief Submits a circular buffer to Peripheral DMA to read from Peripheral FIFO.
 *
 * @details Submits a circular buffer to read data from a peripheral and
 *          write it to memory.
 *
 * @param [in]  hChannel                Handle to the DMA Channel to work on.
 * @param [in]  pStartAddress           Start address of the data buffer to be processed.
 * @param [in]  NumSubBuffers           Number of sub-buffers in this circular buffer.
 * @param [in]  ElementsPerSubBuffer    Number of data elements per sub-buffer.
 * @param [in]  bBufferDoneCallback     'true' to post a callback on buffer completion.
 * @param [in]  bSubBufferDoneCallback  'true' to post a callback on each sub-buffer completion.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted a circular buffer.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_BAD_START_ADDR: Supplied circular buffer start address is invalid.
 *          - #ADI_DMA_BAD_CIRC_BUFFER: Circular Buffer configuration is invalid.
 *          - #ADI_DMA_FN_NOT_PERMITTED: Function not permitted when DMA data transfer is enabled.
 *          - #ADI_DMA_BUFFER_TYPE_MISMATCH: Can't submit a circular buffer since the DMA is
 *                                           already configured for list buffer data transfer.
 *          - #ADI_DMA_ADDR_ALIGNEMNT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                           buffer address and will cause alignment error.
 *
 * @sa  adi_pdma_PeripheralCircWrite
 *
 */
ADI_DMA_RESULT adi_pdma_PeripheralCircRead(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    void *const                     pStartAddress,
    uint32_t                        NumSubBuffers,
    uint32_t                        ElementsPerSubBuffer,
    bool                            bBufferDoneCallback,
    bool                            bSubBufferDoneCallback)
{
    /* DMA Channel to work on */
    ADI_DMA_INFO    *pDmaInfo = (ADI_DMA_INFO *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* Return code */
    ADI_DMA_RESULT  eResult;

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaInfo->pChannel == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

    /* Validate buffer address and its corresponding DMA register values */
    eResult = ValidatePDmaCircBuffer (pDmaInfo,
                                      (uint32_t) pStartAddress,
                                      pDmaInfo->pReg->Status,
                                      NumSubBuffers,
                                      ElementsPerSubBuffer);

    /* IF (Failed to validate buffer address and its corresponding DMA register values) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        return (eResult);
    }

#endif /* ADI_DEBUG */

    /* Load given values to Work unit */
    return (LoadAsCircDescWorkUnit ((ADI_PDMA_CHANNEL *) pDmaInfo->pChannel,
                                    pStartAddress,
                                    NumSubBuffers,
                                    ElementsPerSubBuffer,
                                    bBufferDoneCallback,
                                    bSubBufferDoneCallback,
                                    ENUM_DMA_CFG_WRITE));
}

/**
 * @brief Gets the System Interrupt ID (SID) of a Peripheral DMA channel.
 *
 *
 * @param [in]  eChannelID      DMA Channel ID to query
 * @param [out] pSID            Location to store the SID of the supplied DMA Channel ID.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA channel id.
 *          - #ADI_DMA_BAD_CHANNEL_ID: No such DMA channel ID.
 *
 */
ADI_DMA_RESULT adi_pdma_GetSID(
    ADI_DMA_CHANNEL_ID          eChannelID,
    uint32_t *const             pSID)
{

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (This is not a peripheral DMA channel) */
    if (ValidatePDmaChannel(eChannelID) != ADI_DMA_SUCCESS)
    {
        /* Report failure as bad channel id */
        return (ADI_DMA_BAD_CHANNEL_ID);
    }

#endif /* ADI_DEBUG */

    /* Pass the DMA Channel SID to client */
    *pSID = (uint32_t) ChannelInfo[(uint8_t)eChannelID].DataIntSID;

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*@}*/

