/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_gpio_pint.c
* @brief     This is the no pin interrupt source file for the GPIO Driver.
* @version:  $Revision: 11521 $
* @date:     $Date: 2011-10-03 15:30:49 -0400 (Mon, 03 Oct 2011) $
*
*            Data definitions and interrupt handler for processors
*            without a pin interrupt module.
*
*/
#include <services/gpio/adi_gpio.h>
#include <sys/exception.h>
#include <services/int/adi_int.h>
#include <adi_osal.h>
#include <stddef.h>
#include <services/int/adi_sid.h>

/*
 * Internal functions
 */

static bool Validate(ADI_GPIO_PORT port);

static bool Validate(ADI_GPIO_PORT port)
{
	bool bResult = true;

	if((int8_t)(port) >= NUM_GPIO_PORTS)
	{
		bResult = false;
	}

	return bResult;
}

typedef uint16_t REGSIZE;

/*
 * ADI_GPIO_PORT_INFO
 *
 * Internal device register data
 */

typedef struct {
	ADI_GPIO_PORT portId;                    /* port ID */
#if !(defined(__ADSPBF561_FAMILY__) || defined(__ADSPBF533_FAMILY__))
	volatile uint16_t *Fer;                  /* gpio pin control */
#endif
	volatile uint16_t *Data;                 /* gpio data register */
	volatile uint16_t *Clear;                /* clear gpio register (W1C) */
	volatile uint16_t *Set;                  /* set gpio register (W1C) */
	volatile uint16_t *Toggle;               /* toggle gpio register (W1C) */
	volatile uint16_t *MaskAData;            /* interrupt A mask data register */
	volatile uint16_t *MaskAClear;           /* interrupt A mask clear register (W1C) */
	volatile uint16_t *MaskASet;             /* interrupt A mask set register (W1C) */
	volatile uint16_t *MaskBData;            /* interrupt B mask data register */
	volatile uint16_t *MaskBClear;           /* interrupt B mask clear register (W1C) */
	volatile uint16_t *MaskBSet;             /* interrupt B mask set register (W1C) */
	volatile uint16_t *Direction;            /* gpio direction register */
	volatile uint16_t *Polar;                /* gpio polar register */
	volatile uint16_t *Edge;                 /* gpio edge sensitive register */
	volatile uint16_t *Both;                 /* gpio both edge register */
	volatile uint16_t *InputEnable;          /* gpio input enable register */
    ADI_GPIO_CALLBACK  pfCallback;           /* user installed callback function */
    ADI_GPIO_INTERRUPT_CHANNEL Channel;
    uint8_t pidA;                            /* Peripheral Interrupt A ID */
    uint8_t pidB;                            /* Peripheral Interrupt B ID */

} ADI_GPIO_PORT_INFO;

#if defined(__ADSPBF561_FAMILY__)
static ADI_GPIO_PORT_INFO PortInfo[] = {
		{ /* port f0 */
			ADI_GPIO_PORT_F0,
    		pFIO0_FLAG_D,
    		pFIO0_FLAG_C,
    		pFIO0_FLAG_S,
    		pFIO0_FLAG_T,
    		pFIO0_MASKA_D,
    		pFIO0_MASKA_C,
    		pFIO0_MASKA_S,
    		pFIO0_MASKB_D,
    		pFIO0_MASKB_C,
    		pFIO0_MASKB_S,
    		pFIO0_DIR,
    		pFIO0_POLAR,
    		pFIO0_EDGE,
    		pFIO0_BOTH,
    		pFIO0_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PF0_15_A,
    		ADI_SID_PF0_15_B
		},
		{ /* port f1 */
			ADI_GPIO_PORT_F1,
    		pFIO1_FLAG_D,
    		pFIO1_FLAG_C,
    		pFIO1_FLAG_S,
    		pFIO1_FLAG_T,
    		pFIO1_MASKA_D,
    		pFIO1_MASKA_C,
    		pFIO1_MASKA_S,
    		pFIO1_MASKB_D,
    		pFIO1_MASKB_C,
    		pFIO1_MASKB_S,
    		pFIO1_DIR,
    		pFIO1_POLAR,
    		pFIO1_EDGE,
    		pFIO1_BOTH,
    		pFIO1_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PF16_31_A,
    		ADI_SID_PF16_31_B
		},
		{ /* port f2 */
			ADI_GPIO_PORT_F2,
    		pFIO2_FLAG_D,
    		pFIO2_FLAG_C,
    		pFIO2_FLAG_S,
    		pFIO2_FLAG_T,
    		pFIO2_MASKA_D,
    		pFIO2_MASKA_C,
    		pFIO2_MASKA_S,
    		pFIO2_MASKB_D,
    		pFIO2_MASKB_C,
    		pFIO2_MASKB_S,
    		pFIO2_DIR,
    		pFIO2_POLAR,
    		pFIO2_EDGE,
    		pFIO2_BOTH,
    		pFIO2_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PF32_47_A,
    		ADI_SID_PF32_47_B
		}
};
#elif defined(__ADSPBF538_FAMILY__)
static ADI_GPIO_PORT_INFO PortInfo[] = {
		{ /* port c */
			ADI_GPIO_PORT_C,
			pPORTCIO_FER,
    		pPORTCIO,
    		pPORTCIO_CLEAR,
    		pPORTCIO_SET,
    		pPORTCIO_TOGGLE,
    		0,
    		0,
    		0,
    		0,
    		0,
    		0,
    		pPORTCIO_DIR,
    		0,
    		0,
    		0,
    		pPORTCIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		0,
    		0
		},
		{ /* port d */
			ADI_GPIO_PORT_D,
			pPORTDIO_FER,
    		pPORTDIO,
    		pPORTDIO_CLEAR,
    		pPORTDIO_SET,
    		pPORTDIO_TOGGLE,
    		0,
    		0,
    		0,
    		0,
    		0,
    		0,
    		pPORTDIO_DIR,
    		0,
    		0,
    		0,
    		pPORTDIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		0,
    		0
		},
		{ /* port e */
			ADI_GPIO_PORT_E,
			pPORTEIO_FER,
    		pPORTEIO,
    		pPORTEIO_CLEAR,
    		pPORTEIO_SET,
    		pPORTEIO_TOGGLE,
    		0,
    		0,
    		0,
    		0,
    		0,
    		0,
    		pPORTEIO_DIR,
    		0,
    		0,
    		0,
    		pPORTEIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		0,
    		0
		},
		{ /* port f */
			ADI_GPIO_PORT_F,
			0,
    		pPORTFIO,
    		pPORTFIO_CLEAR,
    		pPORTFIO_SET,
    		pPORTFIO_TOGGLE,
    		pPORTFIO_MASKA,
    		pPORTFIO_MASKA_CLEAR,
    		pPORTFIO_MASKA_SET,
    		pPORTFIO_MASKB,
    		pPORTFIO_MASKB_CLEAR,
    		pPORTFIO_MASKB_SET,
    		pPORTFIO_DIR,
    		pPORTFIO_POLAR,
    		pPORTFIO_EDGE,
    		pPORTFIO_BOTH,
    		pPORTFIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PFA,
    		ADI_SID_PFB
		}
};
#else
static ADI_GPIO_PORT_INFO PortInfo[] = {
#if defined(__ADSPBF533_FAMILY__)
		{ /* port f */
			ADI_GPIO_PORT_F,
    		pFIO_FLAG_D,
    		pFIO_FLAG_C,
    		pFIO_FLAG_S,
    		pFIO_FLAG_T,
    		pFIO_MASKA_D,
    		pFIO_MASKA_C,
    		pFIO_MASKA_S,
    		pFIO_MASKB_D,
    		pFIO_MASKB_C,
    		pFIO_MASKB_S,
    		pFIO_DIR,
    		pFIO_POLAR,
    		pFIO_EDGE,
    		pFIO_BOTH,
    		pFIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PFA,
    		ADI_SID_PFB
		},
#else
		{ /* port f */
			ADI_GPIO_PORT_F,
    		pPORTF_FER,
    		pPORTFIO,
    		pPORTFIO_CLEAR,
    		pPORTFIO_SET,
    		pPORTFIO_TOGGLE,
    		pPORTFIO_MASKA,
    		pPORTFIO_MASKA_CLEAR,
    		pPORTFIO_MASKA_SET,
    		pPORTFIO_MASKB,
    		pPORTFIO_MASKB_CLEAR,
    		pPORTFIO_MASKB_SET,
    		pPORTFIO_DIR,
    		pPORTFIO_POLAR,
    		pPORTFIO_EDGE,
    		pPORTFIO_BOTH,
    		pPORTFIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PORTF_INTA,
    		ADI_SID_PORTF_INTB
		},
#endif
#if !defined(__ADSPBF533_FAMILY__)
		{ /* port g */
			ADI_GPIO_PORT_G,
			pPORTG_FER,
			pPORTGIO,
			pPORTGIO_CLEAR,
			pPORTGIO_SET,
			pPORTGIO_TOGGLE,
			pPORTGIO_MASKA,
			pPORTGIO_MASKA_CLEAR,
			pPORTGIO_MASKA_SET,
			pPORTGIO_MASKB,
			pPORTGIO_MASKB_CLEAR,
			pPORTGIO_MASKB_SET,
			pPORTGIO_DIR,
			pPORTGIO_POLAR,
			pPORTGIO_EDGE,
			pPORTGIO_BOTH,
			pPORTGIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PORTG_INTA,
    		ADI_SID_PORTG_INTB
		},
#endif
#if !(defined(__ADSPBF533_FAMILY__) || defined(__ADSPBF592_FAMILY__))
		{ /* port h */
			ADI_GPIO_PORT_H,
			pPORTH_FER,
			pPORTHIO,
			pPORTHIO_CLEAR,
			pPORTHIO_SET,
			pPORTHIO_TOGGLE,
			pPORTHIO_MASKA,
			pPORTHIO_MASKA_CLEAR,
			pPORTHIO_MASKA_SET,
			pPORTHIO_MASKB,
			pPORTHIO_MASKB_CLEAR,
			pPORTHIO_MASKB_SET,
			pPORTHIO_DIR,
			pPORTHIO_POLAR,
			pPORTHIO_EDGE,
			pPORTHIO_BOTH,
			pPORTHIO_INEN,
    		(void*)0,
    		(ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED,
    		ADI_SID_PORTH_INTA,
    		ADI_SID_PORTH_INTB
		}
#endif
};
#endif

/* callback entry structure */
typedef struct {
	ADI_GPIO_PORT            ePort;
	uint32_t                 Pins;
	ADI_GPIO_CALLBACK        pfCallback;
	void                     *pCBParam;
} ADI_GPIO_CALLBACK_ENTRY;

/* callback memory */
typedef struct {
	ADI_GPIO_CALLBACK_ENTRY *pCallbackData;
	uint32_t MaxCallbacks;
} ADI_GPIO_CALLBACK_INFO;

static ADI_GPIO_CALLBACK_INFO callbackInfo = { (ADI_GPIO_CALLBACK_ENTRY*)0, 0u };


/******************************************************
 Internal function prototypes:
 *******************************************************/

static void GPIOHandler (uint32_t SID, void *pCBParam);

static void GPIOHandler (uint32_t SID, void *pCBParam)
{
	REGSIZE data;
	uint32_t index;
	ADI_GPIO_INTERRUPT_CHANNEL channel;
	ADI_GPIO_CALLBACK_ENTRY *pEntry;
	ADI_GPIO_PORT_INFO *port = (ADI_GPIO_PORT_INFO*) pCBParam;
	ADI_GPIO_PORT_INFO *portEntry;

	channel = port->Channel;
	data = *(port->Data);

	if (channel == ADI_GPIO_MASK_A) {
		data &= (REGSIZE) *(port->MaskAData);
	} else if (channel == ADI_GPIO_MASK_B){
		data &= (REGSIZE) *(port->MaskBData);
	} else {
		data = 0u;
	}

	/* iterate through the registered callbacks for a matching port */
	pEntry = callbackInfo.pCallbackData;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if (pEntry->ePort != (ADI_GPIO_PORT)ADI_GPIO_PORT_UNDEFINED) {
			portEntry = &PortInfo[pEntry->ePort];
			if ((portEntry->portId == port->portId) && (pEntry->pfCallback != (ADI_GPIO_CALLBACK)0)) {
				/* port ID matches and callback entry found */
				if ((data & pEntry->Pins) != 0u) {
					/* if user callback installed for this port */
					(pEntry->pfCallback)(port->portId, (uint32_t)data, pEntry->pCBParam);
				}
			}
		}
		pEntry++;
    }

	/* clear the interrupt */
	*(port->Clear) = (REGSIZE)data;
}

/**
 * @brief Enable or disable the interrupt mask for the given GPIO pin(s)
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * Each of the GPIO modules provides two independent interrupt channels.
 * Identical in functionality, these are called interrupt A and interrupt
 * B. Both interrupt channels have their own mask register which lets
 * you assign the individual GPIOs to none, either, or both interrupt
 * channels.
 *
 * @param [in] ePort       The GPIO port.
 * @param [in] Pins        The pin(s) to assign to the interrupt channel.
 * @param [in] Channel     The interrupt channel.
 * @param [in] bEnable     True to enable the pin(s) in the interrupt channel,
 *                         false to disable the pin(s) in the interrupt channel.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS           Successfully assigned pins to the interrupt mask.
 *             - #ADI_GPIO_INVALID_PORT [D]  The port parameter is invalid.
 *             - #ADI_GPIO_INTERRUPT_CHANNEL The interrupt channel parameter is invalid.
 */
ADI_GPIO_RESULT adi_gpio_EnableInterruptMask(
    ADI_GPIO_PORT              const ePort,
    uint32_t                   const Pins,
    ADI_GPIO_INTERRUPT_CHANNEL const Channel,
    bool                       const bEnable)
{
	ADI_GPIO_RESULT result = ADI_GPIO_SUCCESS;
	ADI_GPIO_PORT_INFO *port = &PortInfo[ePort];

#ifdef ADI_DEBUG
    if (!Validate(ePort))
    {
    	return ADI_GPIO_INVALID_PORT;
    }
#endif

	PortInfo[ePort].Channel = (ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED;

#if defined(ADI_DEBUG) && defined(__ADSPBF538_FAMILY__)
	if (ePort != ADI_GPIO_PORT_F) {
		return ADI_GPIO_NOT_SUPPORTED;
	}
#endif

	adi_osal_EnterCriticalRegion();

	if (Channel == ADI_GPIO_MASK_A)
	{
		PortInfo[ePort].Channel = ADI_GPIO_MASK_A;
		if (bEnable) {
			*(port->MaskASet) |= (REGSIZE)Pins;
		} else {
			*(port->MaskAClear) |= (REGSIZE)Pins;
		}
	} else if (Channel == ADI_GPIO_MASK_B) {
		PortInfo[ePort].Channel = ADI_GPIO_MASK_B;
		if (bEnable) {
			*(port->MaskBSet) |= (REGSIZE)Pins;
		} else {
			*(port->MaskBClear) |= (REGSIZE)Pins;
		}
	} else {
		result = ADI_GPIO_INVALID_MASK;
	}

	adi_osal_ExitCriticalRegion();

	return result;
}

/**
 * @brief Enable or disable input hysteresis for the given GPIO pin(s).
 * @note Supported on ADSP-BF50x, ADSP-BF51x, ADSP-BF52x and ADSP-BF59x only.
 *
 * Control the hysteresis (via Schmitt triggering) for the PORTx inputs.
 * The Schmitt trigger can be set only for pin groups.
 *
 * - ADSP-BF50x groups:
 *  - GROUP_0 : PF0, PG0 or PH0
 *  - GROUP_1 : PF1, PG1 or PH1
 *  - GROUP_2 : PF5-PF2, PG2 or PH2
 *  - GROUP_3 : PF7-PF6, PG5-PG3
 *  - GROUP_4 : PF9-P8, PG8-PG6
 *  - GROUP_5 : PF12-PF10, PG11-PG9
 *  - GROUP_6 : PF13, PG13-PG12
 *  - GROUP_7 : PF15-PG14, PG15-PG14
 *
 * - ADSP-BF51x groups:
 *  - GROUP_0 : PF0, PG2-PG0, PH3-PH0
 *  - GROUP_1 : PF6-PF1, PG3, PH5-PH4
 *  - GROUP_2 : PF7, PG4, PH6
 *  - GROUP_3 : PF10-PF8, PG7-PG5, PH7
 *  - GROUP_4 : PF14-PF11, PG8
 *  - GROUP_5 : PF15, PG10-PG9
 *  - GROUP_6 : PG11
 *  - GROUP_7 : PG15-PG12
 *
 * - ADSP-BF52x groups:
 *   - GROUP_0 : PF7-PF0, PG0 or PH7_PH0
 *   - GROUP_1 : PF9-PF8, PG4-PG1 or PH8
 *   - GROUP_2 : PF10, PG6-PG5 or PH15-PH9
 *   - GROUP_3 : PF11 or PG8-PG7
 *   - GROUP_4 : PF13-PF12 or PG9
 *   - GROUP_5 : PF15-PF14 or PG10
 *   - GROUP_6 : PG13-PG11
 *   - GROUP_7 : PG15-PG14
 *
 * - ADSP-BF59x groups:
 *   - GROUP_0 : PF0-PF1, PG0-PG1
 *   - GROUP_1 : PF2-PF3, PG2-PG3
 *   - GROUP_2 : PF4-PF5, PG4-PG5
 *   - GROUP_3 : PF6-PF7, PG6-PG7
 *   - GROUP_4 : PF8-PF10, PG8
 *   - GROUP_5 : PF11-PF12, PG9-PG10
 *   - GROUP_6 : PF13-PF14, PG11-PG12
 *   - GROUP_7 : PF15, PG13-PG15
 *
 * @param [in] ePort       The GPIO port.
 * @param [in] ePinGroup   The pin group(s) to configure input hysteresis.
 * @param [in] bEnable     True to enable input hysteresis, false to disable.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS          Successfully set input hysteresis.
 *             - #ADI_GPIO_INVALID_PORT [D] The port parameter is invalid.
 *             - #ADI_GPIO_NOT_SUPPORTED    The API is not supported for this processor type.
 *
 */
ADI_GPIO_RESULT adi_gpio_EnableInputHysteresis(
    ADI_GPIO_PORT               const ePort,
    ADI_GPIO_HYSTERESIS_GROUP   const ePinGroup,
    bool                        const bEnable)
{
#if defined(__ADSPBF506F_FAMILY__) || defined(__ADSPBF518_FAMILY__) || defined(__ADSPBF526_FAMILY__) || \
	defined(__ADSPBF527_FAMILY__) || defined(__ADSPBF592_FAMILY__)

#if defined(__ADSPBF592_FAMILY__)
	static volatile uint16_t *PortHyst[] = {
			(pPORTF_PADCTL),
			(pPORTG_PADCTL)
	};
	uint16_t value = 1u;
	uint16_t position = (uint16_t)ePinGroup;
#else
	static volatile uint16_t *PortHyst[] = {
			(pPORTF_HYSTERESIS),
			(pPORTG_HYSTERESIS),
			(pPORTH_HYSTERESIS)
	};
	uint16_t value = 3u;
	uint16_t position = (uint16_t)ePinGroup * 2u;
#endif

	volatile uint16_t *hyst = PortHyst[ePort];
    uint16_t mask;

#ifdef ADI_DEBUG
    if (!Validate(ePort))
    {
    	return ADI_GPIO_INVALID_PORT;
    }
#endif

	adi_osal_EnterCriticalRegion();

    if (bEnable)
    {
        mask = (uint16_t)1u << position;
    	*(hyst) |= mask;
    } else {
        mask = value << position;
		*(hyst) &= (uint16_t)~mask;
    }

    adi_osal_ExitCriticalRegion();

    return ADI_GPIO_SUCCESS;
#else
    return ADI_GPIO_NOT_SUPPORTED;
#endif
}

/**
 * @brief Set the given GPIO pin(s) input edge sensitivity.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * The GPIO interrupt sensitivity registers are used to configure each of the
 * inputs as either a level-sensitive or an edge-sensitive source. When using
 * an edge-sensitive mode, an edge detection circuit is used to prevent a situation
 * where a short event is missed because of the system clock rate. The
 * GPIO interrupt sensitivity register has no effect on GPIOs that are
 * defined as outputs. The contents of the GPIO interrupt sensitivity registers
 * are cleared at reset, defaulting to level sensitivity.
 *
 * @param [in] ePort       The GPIO port.
 * @param [in] Pins        The pin(s) to set the edge sensitivity.
 * @param [in] eSense      The edge sensitivity type.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS            Successfully set the edge sensitivity type.
 *             - #ADI_GPIO_INVALID_PORT [D]   The port parameter is invalid.
 *             - #ADI_GPIO_INVALID_EDGE_SENSE The edge sensitivity type parameter is invalid.
 */
ADI_GPIO_RESULT adi_gpio_SetInputEdgeSense(
    ADI_GPIO_PORT         const ePort,
    uint32_t              const Pins,
    ADI_GPIO_SENSE        const eSense)
{
	ADI_GPIO_PORT_INFO *port = &PortInfo[ePort];
	ADI_GPIO_RESULT result = ADI_GPIO_SUCCESS;

#ifdef ADI_DEBUG
    /* validate sensitivity */
    if(((int32_t)eSense > ADI_GPIO_SENSE_BOTH_EDGES))
    {
        return ADI_GPIO_INVALID_EDGE_SENSE;
    }

#if defined(__ADSPBF538_FAMILY__)
	if (ePort != ADI_GPIO_PORT_F) {
		return ADI_GPIO_NOT_SUPPORTED;
	}
#endif

#endif /* ADI_DEBUG */

#ifdef ADI_DEBUG
    if (!Validate(ePort))
    {
    	return ADI_GPIO_INVALID_PORT;
    }
#endif

	adi_osal_EnterCriticalRegion();

    switch(eSense)
    {
    case ADI_GPIO_SENSE_LEVEL_HIGH:
    	/* write PORTxIO_EDGE to clear appropriate bits for level sensitivity */
		*(port->Edge) &= (REGSIZE)~(REGSIZE)Pins;
		/* write PORTxIO_POLAR to clear appropriate bits for high level sensitivity */
		*(port->Polar) &= (REGSIZE)~(REGSIZE)Pins;
		break;
    case ADI_GPIO_SENSE_LEVEL_LOW:
    	/* write PORTxIO_EDGE to clear appropriate bits for level sensitivity */
		*(port->Edge) &= (REGSIZE)~(REGSIZE)Pins;
		/* write PORTxIO_POLAR to set appropriate bits for low level sensitivity */
		*(port->Polar) |= (REGSIZE)Pins;
    	break;
    case ADI_GPIO_SENSE_RISING_EDGE:
    	/* write PORTxIO_EDGE to set appropriate bits for edge sensitivity */
		*(port->Edge) |= (REGSIZE)Pins;
		/* write PORTxIO_BOTH to clear appropriate bits for edge sensitivity */
		*(port->Both) &= (REGSIZE)~(REGSIZE)Pins;
		/* write PORTxIO_POLAR to clear appropriate bits for rising edge sensitivity */
		*(port->Polar) &= (REGSIZE)~(REGSIZE)Pins;
    	break;
    case ADI_GPIO_SENSE_FALLING_EDGE:
    	/* write PORTxIO_EDGE to set appropriate bits for edge sensitivity */
		*(port->Edge) |= (REGSIZE)Pins;
		/* write PORTxIO_BOTH to clear appropriate bits for edge sensitivity */
		*(port->Both) &= (REGSIZE)~(REGSIZE)Pins;
		/* write PORTxIO_POLAR to set appropriate bits for falling edge sensitivity */
		*(port->Polar) |= (REGSIZE)Pins;
    	break;
    case ADI_GPIO_SENSE_BOTH_EDGES:
    	/* write PORTxIO_EDGE to set appropriate bits for edge sensitivity */
		*(port->Edge) |= (REGSIZE)Pins;
		/* write PORTxIO_BOTH to set appropriate bits for both edge sensitivity */
		*(port->Both) |= (REGSIZE)Pins;
    	break;
    default:
    	result = ADI_GPIO_INVALID_EDGE_SENSE;
    	break;
    }

    adi_osal_ExitCriticalRegion();

    return result;
}

/**
 * @brief Register a user callback function for the given GPIO port.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * The user callback function is executed when a GPIO interrupt on the specified
 * port and pins occurs.  Callback memory space must be initialized using the
 * adi_gpio_Init() API before registering a callback.  The adi_gpio_EnableInterruptMask()
 * must be called before this API.
 *
 *
 * @param [in] ePort       The GPIO port.
 * @param [in] Pins        The GPIO pin(s) to used to trigger the callback.
 * @param [in] pfCallback  A user defined callback function.
 * @param [in] pCBParam    A user defined callback parameter.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS              Successfully installed a user callback function.
 *             - #ADI_GPIO_CALLBACK_MEMORY_FULL There is no callback memory space available.
 *             - #ADI_GPIO_INVALID_PORT [D]     The port parameter is invalid.
 *             - #ADI_GPIO_CALLBACK_INVALID [D] The callback function is invalid.
 *             - #ADI_GPIO_INVALID_MASK [D]     The interrupt mask has not been defined.
 *
 * @sa adi_gpio_Init
 * @sa adi_gpio_EnableInterruptMask
 * @sa adi_gpio_UnRegisterCallback
 */
ADI_GPIO_RESULT adi_gpio_RegisterCallback(
    ADI_GPIO_PORT            const ePort,
    uint32_t                 const Pins,
    ADI_GPIO_CALLBACK        const pfCallback,
    void *                   const pCBParam)
{
    ADI_GPIO_CALLBACK_ENTRY *pEntry;
    uint32_t index;
    uint32_t sid;
	ADI_GPIO_PORT_INFO *port = &PortInfo[ePort];
	ADI_GPIO_RESULT result = ADI_GPIO_CALLBACK_MEMORY_FULL;

#if defined(ADI_DEBUG) && defined(__ADSPBF538_FAMILY__)
	if (ePort != ADI_GPIO_PORT_F) {
		return ADI_GPIO_INVALID_PORT;
	}
#endif

#if defined(ADI_DEBUG)
	if (PortInfo[ePort].Channel == (ADI_GPIO_INTERRUPT_CHANNEL)ADI_GPIO_CHANNEL_UNDEFINED) {
		return ADI_GPIO_INVALID_MASK;
	}

	if (pfCallback == (ADI_GPIO_CALLBACK)0) {
		return ADI_GPIO_CALLBACK_INVALID;
	}

    if (!Validate(ePort))
    {
    	return ADI_GPIO_INVALID_PORT;
    }
#endif

	pEntry = callbackInfo.pCallbackData;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if (pEntry->pfCallback == (ADI_GPIO_CALLBACK)0) {
    		/* empty entry found */
    		pEntry->ePort = ePort;
    		pEntry->Pins = Pins;
    		pEntry->pfCallback = pfCallback;
    		pEntry->pCBParam = pCBParam;

    		/* clear interrupts for this port */
   			*(port->Clear) = 0xffffu;

   			if (port->Channel == ADI_GPIO_MASK_A) {
   				sid = port->pidA;
   			} else {
   				sid = port->pidB;
   			}

   			if (adi_int_InstallHandler(sid, GPIOHandler, &PortInfo[ePort], true) != ADI_INT_SUCCESS) {
   		    	result = ADI_GPIO_FAILURE;
   		    } else {
   		    	result = ADI_GPIO_SUCCESS;
   		    }
    		break;
    	}
    	pEntry++;
    }

    return result;
}

/**
 * @brief Removes the user callback function for the given port and pins.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * When the callback function is removed the user application code is not notified when
 * a GPIO interrupt occurs.
 *
 * @param [in]  ePort       The GPIO port.
 * @param [in]  Pins        The pin(s) to used to trigger the callback.
 * @param [in]  pfCallback  A user defined callback function.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS              Successfully removed the callback.
 *             - #ADI_GPIO_CALLBACK_NOT_FOUND   The given callback was not found.
 *             - #ADI_GPIO_CALLBACK_INVALID [D] The given callback is invalid.
 *             - #ADI_GPIO_INVALID_PORT [D]     The port parameter is invalid.
 *
 * @sa adi_gpio_RegisterCallback
 */
ADI_GPIO_RESULT adi_gpio_UnRegisterCallback(
    ADI_GPIO_PORT            const ePort,
    uint32_t                 const Pins,
    ADI_GPIO_CALLBACK        const pfCallback)
{
    ADI_GPIO_CALLBACK_ENTRY *pEntry;
	ADI_GPIO_PORT_INFO *port = &PortInfo[ePort];
    uint32_t sid;
    uint32_t index;
    bool bFound;
	ADI_GPIO_RESULT result = ADI_GPIO_CALLBACK_NOT_FOUND;

#if defined(ADI_DEBUG) && defined(__ADSPBF538_FAMILY__)
	if (ePort != ADI_GPIO_PORT_F) {
		return ADI_GPIO_INVALID_PORT;
	}
#endif

#if defined(ADI_DEBUG)
	if (pfCallback == (ADI_GPIO_CALLBACK)0) {
		return ADI_GPIO_CALLBACK_INVALID;
	}

	if (!Validate(ePort))
    {
    	return ADI_GPIO_INVALID_PORT;
    }
#endif

	pEntry = callbackInfo.pCallbackData;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if ((pEntry->ePort == ePort) && (pEntry->pfCallback == pfCallback) &&
    	    (pEntry->Pins == Pins)) {
    		/* entry found */
    		pEntry->ePort = (ADI_GPIO_PORT)ADI_GPIO_PORT_UNDEFINED;
    		pEntry->Pins = 0u;
    		pEntry->pfCallback = (ADI_GPIO_CALLBACK)0;
    		pEntry->pCBParam = (void*)0;

    		result = ADI_GPIO_SUCCESS;
    		break;
    	}
    	pEntry++;
    }

    /* are there any more callbacks for this port */
	pEntry = callbackInfo.pCallbackData;
	bFound = false;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if ((pEntry->ePort == ePort) && (pEntry->pfCallback != (ADI_GPIO_CALLBACK)0)) {
    		/* entry found */
    		bFound = true;
    		break;
    	}
    	pEntry++;
    }

    if (!bFound) {
		if (port->Channel == ADI_GPIO_MASK_A) {
			sid = port->pidA;
		} else {
			sid = port->pidB;
		}

        /* no more callbacks for this port - remove interrupt handler */
    	adi_int_UninstallHandler(sid);
    }

    return result;
}
