/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_gpio_pint.c
* @brief     This is the pin interrupt source file for the GPIO Driver.
* @version:  $Revision: 18484 $
* @date:     $Date: 2013-04-26 10:04:51 -0400 (Fri, 26 Apr 2013) $
*
*            Data definitions and interrupt handler for processors
*            with a pin interrupt module.
*
*/
#include <services/gpio/adi_gpio.h>
#if defined(__ADSPBLACKFIN__)
#include <sys/exception.h>
#endif
#include <services/int/adi_int.h>
#include <adi_osal.h>
#include <stddef.h>

#if defined(__ADSPBF609_FAMILY__)
#include <services/int/adi_sec.h>
#include "adi_gpio_data_bf60x.c"
#elif defined(ADI_ADSP_CM40Z)
#include "adi_gpio_data_cm40z.c"
#elif defined(__ADSPBF707_FAMILY__)
#include "adi_gpio_data_bf70x.c"
#elif defined(__ADSPBF548_FAMILY__) || defined(__ADSPBF548M_FAMILY__)
#include <services/int/adi_sid.h>
#include "adi_gpio_data_bf54x.c"
#else
#error "Unknown processor family"
#endif 

/* callback entry structure */
typedef struct {
	ADI_GPIO_PIN_INTERRUPT   ePinInt;
	uint32_t                 Pins;
	ADI_GPIO_CALLBACK        pfCallback;
	void                     *pCBParam;
} ADI_GPIO_CALLBACK_ENTRY;

/* callback memory */
typedef struct {
	ADI_GPIO_CALLBACK_ENTRY *pCallbackData;
	uint32_t MaxCallbacks;
} ADI_GPIO_CALLBACK_INFO;

static ADI_GPIO_CALLBACK_INFO callbackInfo = { (ADI_GPIO_CALLBACK_ENTRY*)0, 0u };

/******************************************************
 Internal function prototypes:
 *******************************************************/

static void GPIOHandler (uint32_t SID, void *pCBParam);
static bool ValidatePinInt(ADI_GPIO_PIN_INTERRUPT pinint);

static bool ValidatePinInt(ADI_GPIO_PIN_INTERRUPT pinint)
{
	bool bResult = true;

	if((int8_t)(pinint) >= NUM_GPIO_PIN_INTS)
	{
		bResult = false;
	}

	return bResult;
}

static void GPIOHandler (uint32_t SID, void *pCBParam)
{
	uint32_t data;
	uint32_t index;

	ADI_GPIO_PIN_INT_INFO *pinint = (ADI_GPIO_PIN_INT_INFO*) pCBParam;
	ADI_GPIO_CALLBACK_ENTRY *pEntry;
	ADI_GPIO_PIN_INT_INFO *pinIntEntry;

#if defined(ADI_DEBUG)
	/* confirm this is a valid port */
	int8_t i;
	bool bFound = false;
    for (i=0; i < NUM_GPIO_PIN_INTS; i++) {
    	/* find port number */
    	if (pinint->Irq == PinIntInfo[i].Irq) {
    		bFound = true;
    		break;
    	}
    }

    if (!bFound) {
    	/* invalid port */
    	return;
    }
#endif

	data = *(pinint->Irq);

	/* iterate through the registered callbacks for a matching pin int */
	pEntry = callbackInfo.pCallbackData;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if (pEntry->ePinInt != (ADI_GPIO_PIN_INTERRUPT)ADI_GPIO_PIN_INT_UNDEFINED) {
    		pinIntEntry = &PinIntInfo[pEntry->ePinInt];
			if ((pinIntEntry->pinintId == pinint->pinintId) && (pEntry->pfCallback != (void*)0u)) {
				/* port ID matches and callback entry found */
				if ((data & pEntry->Pins) != 0u) {
					/* if user callback installed for this port */
					(pEntry->pfCallback)(pinint->pinintId, data, pEntry->pCBParam);
				}
			}
		}
		pEntry++;
    }

	/* clear the interrupt */
	*(pinint->Latch) = data;
}

/* Pin Interrupt Registers */

/**
 * @brief Assign port pin(s) to a pin interrupt channel.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * ADSP-BF54x processor Blackfin processors have four SIC interrupt channels
 * dedicated to pin interrupt purposes. These channels are managed by
 * four hardware blocks, called PINT0, PINT1, PINT2, and PINT3. Every PINTx
 * block can sense to up to 32 pins. While PINT0 and PINT1 can sense the
 * pins of port A and port B, PINT2 and PINT3 manage all the pins from port
 * C to port J.
 *
 * @param [in] ePinInt       The pin interrupt channel to assign.
 * @param [in] eByte         The pin interrupt channel byte to assign.
 * @param [in] ePort         The 8-bit half port register to assign.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS                   Successfully assigned the pin interrupt channel.
 *             - #ADI_GPIO_INVALID_PIN_INTERRUPT [D] The pin interrupt channel is invalid.
 *             - #ADI_GPIO_INVALID_PIN_BYTE          The pin interrupt channel byte is invalid.
 *             - #ADI_GPIO_INVALID_PIN_PORT          The half port register is invalid.
 *
 * @sa adi_gpio_EnablePinInterruptMask
 */
ADI_GPIO_RESULT adi_gpio_PinInterruptAssignment(
	ADI_GPIO_PIN_INTERRUPT   const ePinInt,
    ADI_GPIO_PIN_ASSIGN_BYTE const eByte,
    ADI_GPIO_PIN_ASSIGN_PORT const ePort)
{
	ADI_GPIO_PIN_INT_INFO *pinint = &PinIntInfo[ePinInt];

#ifdef ADI_DEBUG
	if (!ValidatePinInt(ePinInt))
	{
		return ADI_GPIO_INVALID_PIN_INTERRUPT;
	}
#endif

	adi_osal_EnterCriticalRegion();

	*(pinint->Assign) &= ~((uint32_t)0xFFu << (uint8_t)eByte);
	*(pinint->Assign) |= ((uint32_t)ePort << (uint8_t)eByte);

	adi_osal_ExitCriticalRegion();

	return ADI_GPIO_SUCCESS;
}

/**
 * @brief Enable or disable a pin interrupt.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * @param [in] ePinInt       The pin interrupt channel.
 * @param [in] Pins          The pin(s) to enable/disable interrupt.
 * @param [in] bEnable       True to enable the interrupt, false to disable.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS                   Successfully assigned the pin interrupt channel.
 *             - #ADI_GPIO_INVALID_PIN_INTERRUPT [D] The pin interrupt channel is invalid.
 *
 * @sa adi_gpio_PinInterruptAssignment
 */
ADI_GPIO_RESULT adi_gpio_EnablePinInterruptMask(
	ADI_GPIO_PIN_INTERRUPT   const ePinInt,
    uint32_t                 const Pins,
    bool                     const bEnable)
{
	ADI_GPIO_PIN_INT_INFO *pinint = &PinIntInfo[ePinInt];

#ifdef ADI_DEBUG
	if (!ValidatePinInt(ePinInt))
	{
		return ADI_GPIO_INVALID_PIN_INTERRUPT;
	}
#endif

	adi_osal_EnterCriticalRegion();

	if (bEnable) {
		*(pinint->MaskSet) |= Pins;
	} else {
		*(pinint->MaskClear) |= Pins;
	}

	/* clear interrupts */
	*(pinint->Latch) |= Pins;

	adi_osal_ExitCriticalRegion();

	return ADI_GPIO_SUCCESS;
}

/**
 * @brief Set the given interrupt pin(s) input edge sensitivity.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * The interrupt can be generated on an active high level of the signal or a
 * raising edge of the signal. The default behavior is level sensitivity.
 * PINTx_EDGE_SET register can be used to change the behavior to edge sensitivity.
 * By enabling the inverter using the PINTx_INVERT_SET register, the interrupt
 * behavior can be altered to trigger on active-low signals or falling edges.
 *
 * @param [in] ePinInt     The pin interrupt channel.
 * @param [in] Pins        The pin(s) to set the edge sensitivity.
 * @param [in] eSense      The edge sensitivity type.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS                   Successfully set the edge sensitivity type.
 *             - #ADI_GPIO_INVALID_PIN_INTERRUPT [D] The pin interrupt channel is invalid.
 *             - #ADI_GPIO_INVALID_EDGE_SENSE        The edge sensitivity type is invalid.
 */
ADI_GPIO_RESULT adi_gpio_SetPinIntEdgeSense(
	ADI_GPIO_PIN_INTERRUPT   const ePinInt,
    uint32_t                 const Pins,
    ADI_GPIO_SENSE           const eSense)
{
	ADI_GPIO_PIN_INT_INFO *pinint = &PinIntInfo[ePinInt];
	ADI_GPIO_RESULT result = ADI_GPIO_SUCCESS;

#ifdef ADI_DEBUG
    /* validate sensitivity */
    if(((int32_t)eSense > ADI_GPIO_SENSE_BOTH_EDGES))
    {
        return ADI_GPIO_INVALID_EDGE_SENSE;
    }

	if (!ValidatePinInt(ePinInt))
	{
		return ADI_GPIO_INVALID_PIN_INTERRUPT;
	}
#endif /* ADI_DEBUG */

	adi_osal_EnterCriticalRegion();

    switch(eSense)
    {
    case ADI_GPIO_SENSE_LEVEL_HIGH:
		/* write PINT_INVERT_CLEAR to clear appropriate bits for high level sensitivity */
		*(pinint->InvertClear) |= Pins;
    	/* write PINT_EDGE_CLEAR to clear appropriate bits for level sensitivity */
		*(pinint->EdgeClear) |= Pins;
		break;
    case ADI_GPIO_SENSE_LEVEL_LOW:
		/* write PINT_INVERT_SET to set appropriate bits for low level sensitivity */
		*(pinint->InvertSet) |= Pins;
    	/* write PINT_EDGE_CLEAR to clear appropriate bits for level sensitivity */
		*(pinint->EdgeClear) |= Pins;
    	break;
    case ADI_GPIO_SENSE_RISING_EDGE:
		/* write PINT_INVERT_CLEAR to clear appropriate bits for rising edge sensitivity */
		*(pinint->InvertClear) |= Pins;
    	/* write PINT_EDGE_SET to set appropriate bits for edge sensitivity */
		*(pinint->EdgeSet) |= Pins;
    	break;
    case ADI_GPIO_SENSE_FALLING_EDGE:
		/* write PINT_INVERT_SET to clear appropriate bits for rising edge sensitivity */
		*(pinint->InvertSet) |= Pins;
    	/* write PINT_EDGE_SET to set appropriate bits for edge sensitivity */
		*(pinint->EdgeSet) |= Pins;
    	break;
    case ADI_GPIO_SENSE_BOTH_EDGES:
    	result = ADI_GPIO_INVALID_EDGE_SENSE;
    	break;
    default:
    	result = ADI_GPIO_INVALID_EDGE_SENSE;
    	break;
    }

    adi_osal_ExitCriticalRegion();

    return result;
}

/**
 * @brief Get the logic level of the given pins(s) on the given interrupt channel
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * The return value is a packed value containing the status of all the pins
 * on the given interrupt channel. To test the level of a specific pin, simply perform a
 * logical AND of the return value with the pin interrupt definition. For example
 * to see if pin 4 on interrupt channel 1 is set, the following is used:
 *
 * adi_gpio_GetPinInterruptState(ADI_GPIO_PIN_INTERRUPT_1, &data)
 * if (data & ADI_GPIO_INT_PIN_4)
 *    pin 4 on interrupt channel 1 is set
 * else
 *    pin 4 on interrupt channel 1 is clear
 *
 * @param [in]  ePinInt     The pin interrupt channel.
 * @param [out] pValue      The value of the interrupt channel pins.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS                   Successfully got the GPIO pin data.
 *             - #ADI_GPIO_INVALID_PIN_INTERRUPT [D] The pin interrupt channel is invalid.
 *
 */
ADI_GPIO_RESULT adi_gpio_GetPinInterruptState(
	ADI_GPIO_PIN_INTERRUPT   const ePinInt,
    uint32_t                       *pValue)
{
	ADI_GPIO_PIN_INT_INFO *pinint = &PinIntInfo[ePinInt];

#ifdef ADI_DEBUG
	if (!ValidatePinInt(ePinInt))
	{
		return ADI_GPIO_INVALID_PIN_INTERRUPT;
	}
#endif

	*pValue = *(pinint->PinState);

	return ADI_GPIO_SUCCESS;
}

/**
 * @brief Register a user callback function for the given GPIO port.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * The user callback function is executed when a GPIO interrupt on the specified
 * port and pins occurs.  Callback memory space must be initialized using the
 * adi_gpio_Init() API before registering a callback.
 *
 *
 * @param [in] ePinInt     The pin interrupt channel.
 * @param [in] Pins        The GPIO pin(s) to used to trigger the callback.
 * @param [in] pfCallback  A user defined callback function.
 * @param [in] pCBParam    A user defined callback parameter.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS              Successfully installed a user callback function.
 *             - #ADI_GPIO_CALLBACK_MEMORY_FULL There is no callback memory space available.
 *             - #ADI_GPIO_CALLBACK_INVALID [D] The callback function is invalid.
 *
 * @sa adi_gpio_Init
 * @sa adi_gpio_EnableInterruptMask
 * @sa adi_gpio_UnRegisterCallback
 */
ADI_GPIO_RESULT adi_gpio_RegisterCallback(
    ADI_GPIO_PIN_INTERRUPT     const ePinInt,
    uint32_t                   const Pins,
    ADI_GPIO_CALLBACK          const pfCallback,
    void *                     const pCBParam)
{
    ADI_GPIO_CALLBACK_ENTRY *pEntry;
    uint32_t index;
    ADI_GPIO_PIN_INT_INFO *pinint = &PinIntInfo[ePinInt];
	ADI_GPIO_RESULT result = ADI_GPIO_CALLBACK_MEMORY_FULL;

#if defined(ADI_DEBUG)
	if (pfCallback == (void*)0u) {
		return ADI_GPIO_CALLBACK_INVALID;
	}
#endif

#ifdef ADI_DEBUG
	if (!ValidatePinInt(ePinInt))
	{
		return ADI_GPIO_INVALID_PIN_INTERRUPT;
	}
#endif

	pEntry = callbackInfo.pCallbackData;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if (pEntry->pfCallback == (void*)0u) {
    		/* empty entry found */
    		pEntry->ePinInt = ePinInt;
    		pEntry->Pins = Pins;
    		pEntry->pfCallback = pfCallback;
    		pEntry->pCBParam = pCBParam;

    		/* clear the interrupt */
   			*(pinint->Latch) = 0xffffffffu;

   			if (adi_int_InstallHandler((uint32_t)pinint->pid, GPIOHandler, &PinIntInfo[ePinInt], true) != ADI_INT_SUCCESS) {
   		    	result = ADI_GPIO_FAILURE;
   		    } else {
   		    	result = ADI_GPIO_SUCCESS;
   		    }
    		break;
    	}
    	pEntry++;
    }

    return result;
}

/**
 * @brief Removes the user callback function for the given port and pins.
 * @note Supported on processors with Pin Interrupt Registers.
 *
 * When the callback function is removed the user application code is not notified when
 * a GPIO interrupt occurs.
 *
 * @param [in]  ePinInt     The pin interrupt channel.
 * @param [in]  Pins        The pin(s) to used to trigger the callback.
 * @param [in]  pfCallback  A user defined callback function.
 *
 * @return
 *             - #ADI_GPIO_SUCCESS              Successfully removed the callback.
 *             - #ADI_GPIO_CALLBACK_NOT_FOUND   The given callback was not found.
 *             - #ADI_GPIO_CALLBACK_INVALID [D] The given callback is invalid.
 *
 * @sa adi_gpio_RegisterCallback
 */
ADI_GPIO_RESULT adi_gpio_UnRegisterCallback(
	ADI_GPIO_PIN_INTERRUPT     const ePinInt,
	uint32_t                   const Pins,
	ADI_GPIO_CALLBACK          const pfCallback)
{
    ADI_GPIO_CALLBACK_ENTRY *pEntry;
    uint32_t index;
    bool bFound;
    ADI_GPIO_PIN_INT_INFO *pinint = &PinIntInfo[ePinInt];
	ADI_GPIO_RESULT result = ADI_GPIO_CALLBACK_NOT_FOUND;

#if defined(ADI_DEBUG)
	if (pfCallback == (void*)0u) {
		return ADI_GPIO_CALLBACK_INVALID;
	}
#endif

#ifdef ADI_DEBUG
	if (!ValidatePinInt(ePinInt))
	{
		return ADI_GPIO_INVALID_PIN_INTERRUPT;
	}
#endif

	pEntry = callbackInfo.pCallbackData;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if ((pEntry->ePinInt == ePinInt) && (pEntry->pfCallback == pfCallback) &&
    	    (pEntry->Pins == Pins)) {
    		/* entry found */
    		pEntry->ePinInt = (ADI_GPIO_PIN_INTERRUPT)ADI_GPIO_PIN_INT_UNDEFINED;
    		pEntry->Pins = 0u;
    		pEntry->pfCallback = (void*)0u;
    		pEntry->pCBParam = (void*)0u;

    		result = ADI_GPIO_SUCCESS;
    		break;
    	}
    	pEntry++;
    }

    /* are there any more callbacks for this port */
	pEntry = callbackInfo.pCallbackData;
	bFound = false;
    for (index = 0u; index < callbackInfo.MaxCallbacks; index++) {
    	if ((pEntry->ePinInt == ePinInt) && (pEntry->pfCallback != (ADI_GPIO_CALLBACK)0)) {
    		/* entry found */
    		bFound = true;
    		break;
    	}
    	pEntry++;
    }

    if (!bFound) {
        /* no more callbacks for this port - remove interrupt handler */
    	adi_int_UninstallHandler((uint32_t)pinint->pid);
    }

    return result;
}
