/*********************************************************************************

Copyright(c) 2004-2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

$Revision: 22918 $
$Date: 2013-09-03 13:26:01 -0400 (Tue, 03 Sep 2013) $


*********************************************************************************/

/*!
* @file      adi_sec.c
*
* @brief     SEC driver API implementation
*
*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

/** @addtogroup SEC_Driver Driver for SEC
 *  @{
 */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


/* 2DO - Multi-core locking to be added */

/*********************************************************************

Include files

*********************************************************************/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <services/int/adi_int.h>
#include <services/int/adi_sec.h>
#include <sys/platform.h>
#include <interrupt.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_6_3:"Allow basic numerical types. RTL macros uses basic types")
#pragma diag(suppress:misra_rule_11_3:"Allow integral types to be typecasted to pointer. Required for passing enumerated values as callback arguments")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_16_7:"Allow pointer parameter to be declared without const")
#pragma diag(suppress:misra_rule_17_1:"Allow pointer arithmetic on pointers, required to get MMR address based on the interrupt ID and core ID")
#pragma diag(suppress:misra_rule_17_4:"Allow pointer arithmetic for MMR address generation")
#pragma diag(suppress:misra_rule_19_7:"Allow macros to be used for readability")

#endif /* _MISRA_RULES */


/*********************************************************************

Local Definitions

*********************************************************************/

/* Stride to get the offset of particular source interface registers based on the given SID */
#define ADI_SEC_SSI_STRIDE          1u

/* Stride to get the offset of particular core interface registers based on the given core ID */
#define ADI_SEC_SCI_STRIDE          4u

/* All the interrupt groups */
#define ADI_SEC_INT_ALL_GROUPS      0xFu

/* SEC error interrupt ID */
#define ADI_SEC_ERR_INTR            (INTR_SEC0_ERR)

/* Macros to lock and unlock the register */
#ifdef ADI_SEC_ENABLE_LOCKING

#define ADI_SEC_LOCK_REG(p)         (*p |= (1 << 31))
#define ADI_SEC_UNLOCK_REG(p)       (*p &=  ~(1 << 31)

#else

#define ADI_SEC_LOCK_REG(p)
#define ADI_SEC_UNLOCK_REG(p)

#endif /* ADI_SEC_ENABLE_LOCKING */



/*********************************************************************

Declarations

*********************************************************************/

#define    ADI_SEC_NOT_INITIALIZED      0ul        /* SEC is not initialized */
#define    ADI_SEC_INITIALIZED          1ul        /* SEC is initialized */


/* Structure which holds the data related to SEC driver */
typedef struct _ADI_SEC_DATA_
{
    /* SEC initialization state
     *
     * It is a bitmask of initialization state of all the cores.
     *
     * When a bit corresponding to a core is set, it indicates that that
     * particular core is set.
     * */
    uint32_t            nInitState;

    /* Application supplied event handler */
    ADI_CALLBACK        pfHandler;

    /* Application supplied callback paramter */
    void *              pCBParam;

} ADI_SEC_DATA;


static ADI_SEC_DATA gSECData     =   {

    0u,                             /* SEC is not yet initialized */
    NULL,                           /* Event handler is not registered */
    NULL                            /* No Callback parameter supplied */
};

/* SEC Error handler which handles the errors generated by SEC */
static void adi_SEC_ErrorHandler(uint32_t SID, void *pCBParam);


/* Default error handler which does nothing */
static void DefaultErrorHandler (
        void    * pCBParam,       /*!< Client supplied callback param */
        uint32_t  Event,          /*!< Event ID specific to the Driver/Service */
        void    * pArg);          /*!< Pointer to the event specific argument */

/*********************************************************************

API Definitions

*********************************************************************/

/* ADSP-CM40x only implements the SEC fault interface */
#if defined(ADI_ADSP_CM40Z)

/**
 * @brief       Initializes the SEC driver.
 *
 * @details     SEC driver should be initialized before calling any of
 *              its APIs.
 *
 *              This API will reset all the SEC fault registers when called 
 *              for the first time.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS  if successfully initialized SEC.
 */
ADI_SEC_RESULT adi_sec_Init(void)
{
    /* Do this only when it is not initialized */
    if(gSECData.nInitState == ADI_SEC_NOT_INITIALIZED)
    {
        /* Reset fault interface */
        adi_sec_ResetSFI();
        
        /* By default enable SEC */
        adi_sec_Enable(true);
    }

    /* Mark as the SEC is initialized */
    gSECData.nInitState |= ADI_SEC_INITIALIZED;

    return ADI_SEC_SUCCESS;
}

#else /* else not ADI_ADSP_CM40Z */

/**
 * @brief       Initializes the SEC driver.
 *
 * @details     SEC driver should be initialized before calling any of
 *              its APIs.
 *
 *              This API will reset all the SEC registers when called from
 *              Core0 for the first time. It is application responsibility
 *              to make sure that adi_sec_Init is called from Core0 first
 *              and then from other cores.
 *
 *              This API can be called any number of times from any cores.
 *              But the order of cores from which it is called should be
 *              maintained (First Core0 and then other cores).
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS  if successfully initialized SEC.
 *              - #ADI_SEC_REGISTRATION_FAILED if failed to register the SEC
 *                error interrupt handler with the dispatcher.
 */
ADI_SEC_RESULT adi_sec_Init(void)
{
    /* Get the core from which init is called from */
    uint32_t nCoreID = (*pDSPID & (uint32_t)BITM_DSPID_COREID);

    /* Check if SEC is initialized for the core from which it is called */
    if( (gSECData.nInitState & (1ul << nCoreID)) == 0ul)
    {
        /* Reset core interface*/
        adi_sec_ResetSCI((ADI_SEC_CORE_ID) nCoreID);

        /* Clear NMI status before enable */
        adi_sec_ClearNMIStatus((ADI_SEC_CORE_ID)nCoreID);

        /* Do this only when called from Core 0 and when it is not initialized */
        if((nCoreID == 0u) && (gSECData.nInitState == ADI_SEC_NOT_INITIALIZED))
        {
            /* Reset SEC */
            adi_sec_Reset();

            /* Initialize SEC data structure */
            gSECData.pfHandler   =   &DefaultErrorHandler;
            gSECData.pCBParam    =   NULL;

            /* Register the SEC error handler */
            if(adi_int_InstallHandler((uint32_t)ADI_SEC_ERR_INTR,
                                          adi_SEC_ErrorHandler,
                                          NULL, true) != ADI_INT_SUCCESS)
            {
                return ADI_SEC_REGISTRATION_FAILED;
            }

            /* Reset fault interface */
            adi_sec_ResetSFI();

            /* By default enable SEC */
            adi_sec_Enable(true);
        }

        /* Enable NMI by default */
        adi_sec_EnableNMI((ADI_SEC_CORE_ID)nCoreID, true);

        /* Enable the SEC Core Interface */
        adi_sec_EnableSCI((ADI_SEC_CORE_ID)nCoreID, true);

        /* Mark as the SEC is initialized for the core from which it is called */
        gSECData.nInitState |= (ADI_SEC_INITIALIZED << nCoreID);
    }

    return ADI_SEC_SUCCESS;
}
#endif /* not ADI_ADSP_CM40Z */

/**
 * @brief       Enables or disables SEC.
 *
 * @details     When SEC is disabled it halts the operation of SEC without
 *              clearing any register settings. When enabled it resumes the
 *              operation.
 *
 * @param [in]  bEnable      'true' to enable, 'false' to disable
 *
 * @return      None
 *
 */
void adi_sec_Enable(bool bEnable)
{
    uint32_t secGCTL = *pREG_SEC0_GCTL;

    if( bEnable == true)
    {
        /* Set the enable bit */
        secGCTL |= (uint32_t)ENUM_SEC_GCTL_EN;
    }
    else
    {
        /* Clear the enable bit */
        secGCTL = secGCTL & ((uint32_t)~BITM_SEC_GCTL_EN);
    }

    ADI_SEC_UNLOCK_REG(pREG_SEC0_GCTL);

    *pREG_SEC0_GCTL = secGCTL;

    ADI_SEC_LOCK_REG(pREG_SEC0_GCTL);

    return;
}

/* ADSP-CM40x only implements the SEC fault interface */
#if !defined(ADI_ADSP_CM40Z)

/**
 * @brief       Resets SEC.
 *
 * @details     When SEC is reset all its register settings will be cleared
 *              and reset to default values. SEC will not be operational anymore.
 *              Application should reconfigure all registers.
 *
 * @return      none
 *
 */
void adi_sec_Reset(void)
{
    ADI_SEC_UNLOCK_REG(pREG_SEC0_GCTL);

    /* Set the reset bit */
    *pREG_SEC0_GCTL = (uint32_t)ENUM_SEC_GCTL_RESET;

    ADI_SEC_LOCK_REG(pREG_SEC0_GCTL);

    return;
}


/**
 * @brief       Registers the application supplied error handler.
 *
 * @details     Application can register an event handler which will be called
 *              back when an error is triggered by SEC. In addition to the
 *              error handler function pointer application can register callback
 *              parameter which will be passed back to the application when
 *              the given handler is called.
 *
 *              The errors enumerated by ADI_SEC_ERRORS will be notified to the
 *              application. Some of the errors will have additional information
 *              to be notified to the application. This information is passed
 *              back to the application via pArg argument of the error handler.
 *
 *              The details regarding the argument type that is passed back for
 *              each error is documented in the the ADI_SEC_ERRORS enumeration.
 *
 * @param [in]  pfHandler       Application supplied error handler which will
 *                              be called to notify errors.
 *
 * @param [in]  pCBParam        Callback parameter which will be passed back
 *                              to the application when the given error handler
 *                              is called. This parameter can be passed as NULL
 *                              if application does not require any callback
 *                              parameter.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS  if successfully registered the error handler.
 *              - #ADI_SEC_INVALID_POINTER [D] if the given pointer to error handler
 *                is NULL.
 *
 * @sa adi_sec_EnableError()
 */
ADI_SEC_RESULT adi_sec_RegisterErrorHandler(ADI_CALLBACK pfHandler, void *pCBParam)
{
#ifdef ADI_DEBUG

    if(pfHandler == NULL)
    {
        return ADI_SEC_INVALID_POINTER;
    }

#endif /* ADI_DEBUG */

    /* Disable the error interrupt */
    adi_sec_EnableInterrupt((uint32_t)ADI_SEC_ERR_INTR, false);

    /* Store the given handler and callback parameter */
    gSECData.pfHandler  =   pfHandler;
    gSECData.pCBParam   =   pCBParam;

    /* Enable back the error interrupt */
    adi_sec_EnableInterrupt((uint32_t)ADI_SEC_ERR_INTR, true);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief  Handles the errors generated by SEC.
 *
 * If a error handler is registered with SEC, it reports the errors to the
 * application otherwise clears the error cause and continues.
 *
 * @param [in]  SID         SID related to error interrupt.
 * @param [in]  Index       Index to the vector table maintained in RTL.
 * @param [in]  pCBParam    Callback parameter
 *
 * @return      None
 */
static void adi_SEC_ErrorHandler(uint32_t SID, void *pCBParam)
{
    /* Global Status register value */
    uint32_t GStat = *pREG_SEC0_GSTAT;

    /* Call back function */
    ADI_CALLBACK  pfHandler = gSECData.pfHandler;

    /*
     * Assuming that there will be only one error reported at a time
     *
     */

    /* Check if there are errors related to SSI, SCI or SFI */
    if((GStat & (uint32_t)BITM_SEC_GSTAT_ERR) != 0u)
    {
        /* Check which interface has caused the error */
        switch(GStat & (uint32_t)BITM_SEC_GSTAT_ERRC)
        {
            /* Error in fault interface */
            case ((uint32_t)ENUM_SEC_GSTAT_SFIERR):
                {
                    uint32_t FStat = *(pREG_SEC0_FSTAT);

                    /* Find if there are any errors */
                    if((FStat & (uint32_t)BITM_SEC_FSTAT_ERR) != 0u)
                    {
                        /* Find the error cause and report to application */
                        if((FStat & (uint32_t)BITM_SEC_FSTAT_ERRC) == (uint32_t)ENUM_SEC_FSTAT_ENDERR)
                        {
                            pfHandler(gSECData.pCBParam, (uint32_t)ADI_SEC_SFI_ERROR, (void *)ADI_SEC_SFI_END_ERR);
                        }

                        /* Clear the errors */
                        *pREG_SEC0_FSTAT    =   FStat;
                    }
                }
                break;

            /* Error in Core Interface */
            case ((uint32_t)ENUM_SEC_GSTAT_SCIERR):
                {
                    /* Core interface that caused the error */
                    uint32_t CoreID     =   (GStat & (uint32_t)BITM_SEC_GSTAT_SCI) >> (uint32_t)BITP_SEC_GSTAT_SCI;
                    /*
                     * Based on the Core ID offset to the appropriate
                     * Core Interface status register
                     */
                    volatile uint32_t *pCStat     =   pREG_SEC0_CSTAT0 + (CoreID << ADI_SEC_SCI_STRIDE);

                    /* Core Interface Status */
                    uint32_t CStat  =   *pCStat;

                    /* SCI error information to passed to application */
                    ADI_SEC_SCI_ERROR_INFO  SCIErrInfo;

                    /* Check if there was an error in Core Interface */
                    if((CStat & (uint32_t)BITM_SEC_CSTAT_ERR) != 0u)
                    {
                        /* Extract the error and report it to application */
                        if((CStat & (uint32_t)BITM_SEC_CSTAT_ERRC) == (uint32_t)ENUM_SEC_CSTAT_ACKERR)
                        {
                            SCIErrInfo.CoreID       =   CoreID;
                            SCIErrInfo.eErrCause    =   ADI_SEC_SCI_ACK_ERR;
                            pfHandler(gSECData.pCBParam, (uint32_t)ADI_SEC_SCI_ERROR, (void *)&SCIErrInfo);
                        }

                        /* Clear the errors */
                        *pCStat =   CStat;
                    }
                }
                break;

            /* Error in System Interface */
            case ((uint32_t)ENUM_SEC_GSTAT_SSIERR):
                {
                    /* System interface that caused the error */
                    uint32_t nErrSID     =   (GStat & (uint32_t)BITM_SEC_GSTAT_SID) >> (uint32_t)BITP_SEC_GSTAT_SID;
                    /*
                     * Based on the SID offset to the appropriate
                     * System Interface status register
                     */
                    volatile uint32_t *pSStat     =   pREG_SEC0_SSTAT0 + (nErrSID << ADI_SEC_SSI_STRIDE);

                    /* Core Interface Status */
                    uint32_t SStat  =   *pSStat;

                    /* SCI error information to passed to application */
                    ADI_SEC_SSI_ERROR_INFO  SSIErrInfo;

                    /* Check if there was an error in System interface */
                    if((SStat & (uint32_t)BITM_SEC_SSTAT_ERR) != 0u)
                    {
                        SSIErrInfo.SID  =   nErrSID;

                        /* Extract the error and report it */
                        switch(SStat & (uint32_t)BITM_SEC_SSTAT_ERRC)
                        {
                            /* SSI Overflow error */
                            case ((uint32_t)ENUM_SEC_SSTAT_SOVFERR):
                                SSIErrInfo.eErrCause    =   ADI_SEC_SSI_SRC_OVF_ERR;
                                break;

                            /* SSI End error */
                            case ((uint32_t)ENUM_SEC_SSTAT_ENDERR):
                                SSIErrInfo.eErrCause    =   ADI_SEC_SSI_END_ERR;
                                break;

                            default:
                                break;
                        }

                        pfHandler(gSECData.pCBParam, (uint32_t)ADI_SEC_SSI_ERROR, (void *)&SSIErrInfo);

                        /* Clear the errors */
                        *pSStat =   SStat;
                    }
                }
                break;

            /* Reserved error cause */
            default:
                break;
        }
    }

    /* Write back the register to clear the errors */
    *pREG_SEC0_GSTAT   =  GStat;

    return;
}

/**
 * @brief       Get the status of Lock Write Error.
 *
 * @details     This error is generated when attempted to write a register
 *              that was locked and global lock was set. When this error happens
 *              a hardware error interrupt is triggered. It is appropriate
 *              to call this function from the Hardware Error interrupt handler
 *              to know if Lock Write Error triggered the Hardware Error.
 *
 *              Upon returning the status the error is automatically cleared
 *              by this API.
 *
 * @param[out]   pError         Location where the Lock Write Error status is
 *                              written.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS  if successfully queried the Lock Write Error
 *                status.
 *              - #ADI_SEC_INVALID_POINTER if the given pointer is invalid.
 */
ADI_SEC_RESULT adi_sec_GetLockWriteError(bool *pError)
{
    /* Global Status register value */
    uint32_t GStat = *pREG_SEC0_GSTAT;

#ifdef ADI_DEBUG

    /* Check if the given pointer is valid */
    if(pError == NULL)
    {
        return ADI_SEC_INVALID_POINTER;
    }

#endif /* ADI_DEBUG */

    /* Check if there is Lock Write Error */
    if((GStat & BITM_SEC_GSTAT_LWERR) != 0u)
    {
        *pError = true;

        /* Clear the Lock Write Error (Write 1 to clear) */
        *pREG_SEC0_GSTAT = ENUM_SEC_GSTAT_LWERR;
    }
    else
    {
        *pError = false;
    }

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Get the status of Address Error.
 *
 * @details     This error is generated when attempted to write a read-only
 *              register or an access with invalid address. When this error happens
 *              a hardware error interrupt is triggered. It is appropriate
 *              to call this function from the Hardware Error interrupt handler
 *              to know if Address Error triggered the Hardware Error.
 *
 *              Upon returning the status the error is automatically cleared
 *              by this API.
 *
 * @param[out]   pError         Location where the Address Error status is
 *                              written.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS  if successfully queried the Address Error
 *                status.
 *              - #ADI_SEC_INVALID_POINTER if the given pointer is invalid.
 */
ADI_SEC_RESULT adi_sec_GetAddrError(bool *pError)
{
    /* Global Status register value */
    uint32_t GStat = *pREG_SEC0_GSTAT;

#ifdef ADI_DEBUG

    /* Check if the given pointer is valid */
    if(pError == NULL)
    {
        return ADI_SEC_INVALID_POINTER;
    }

#endif /* ADI_DEBUG */

    /* Check if thre is a Address Error */
    if((GStat & (uint32_t)BITM_SEC_GSTAT_ADRERR) != 0u)
    {
        *pError = true;

        /* Clear the Clear Address Error (Write 1 to clear) */
        *pREG_SEC0_GSTAT = (uint32_t)ENUM_SEC_GSTAT_ADRERR;
    }
    else
    {
        *pError = false;
    }

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Raise a System Interrupt.
 *
 * @details     Set pending status of the source specified by the SID
 *
 * @param[in]   SID         SID number for which the interrupt to be raised
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS  if successfully raised the interrupt.
 *              - #ADI_SEC_INVALID_SID_ID [D] if the given SID is out of range of
 *                number of SIDs supported by the processor.
 */
ADI_SEC_RESULT adi_sec_Raise(uint32_t SID)
{
#ifdef ADI_DEBUG
    /* Check if the given SID is valid */
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }
#endif /* ADI_DEBUG */

    /* Write the SID to assert */
    *pREG_SEC0_RAISE = SID;

    return ADI_SEC_SUCCESS;
}



/******************************************************************************
 *
 * SEC_SCTLn
 *
 *
 *****************************************************************************/


/**
 * @brief  Enable or disable the interrupt.
 *
 * This controls only whether the interrupt requested is forwarded to the core
 * interface. To get the interrupts, the interrupt source also should be enabled.
 * This can be done by calling the function adi_sec_EnableSource.
 *
 * @param [in] SID          The System interrupt ID that is to be enabled or disabled.
 * @param [in] bEnable      Flag which indicates whether to enable or disable interrupt.
 *                          'true' to enable, 'false' to disable.
 * @return  Status
 *          - #ADI_SEC_SUCCESS if successfully enabled or disabled interrupt.
 *          - #ADI_SEC_INVALID_SID_ID [D] if the given SID is not valid.
 *
 * @sa  adi_sec_EnableSource()
 */
#pragma optimize_for_space /* wa for 16-00-0032 */
ADI_SEC_RESULT adi_sec_EnableInterrupt(uint32_t SID, bool bEnable)
{
    volatile uint32_t *pSCTL;
    uint32_t secSCTL;

#ifdef ADI_DEBUG
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }
#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Get the contents of source control register */
    secSCTL = *pSCTL;

    if(bEnable == true)
    {
        /* Set the interrupt enable bit */
        secSCTL |= (uint32_t)ENUM_SEC_SCTL_INT_EN;
    }
    else
    {
        /* Clear the interrupt enable bit */
        secSCTL &= ((uint32_t)(~BITM_SEC_SCTL_IEN));
    }

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief  Enable or disable the interrupt source.
 *
 * When interrupt source is enabled the interrupt source can influence the
 * pending status. But to get the interrupt to be forwarded to the core, the
 * interrupt should be enabled. Call adi_sec_EnableInterrupt to enable the
 * interrupt.
 *
 * @param [in] SID          The System interrupt ID that is to be enabled or disabled.
 * @param [in] bEnable      Flag which indicates whether to enable or disable source.
 *                          'true' to enable, 'false' to disable.
 * @return  Status
 *          - #ADI_SEC_SUCCESS if successfully enabled or disabled interrupt source.
 *          - #ADI_SEC_INVALID_SID_ID [D] if the given SID is not valid.
 *
 * @sa  adi_sec_EnableInterrupt()
 */
ADI_SEC_RESULT adi_sec_EnableSource(uint32_t SID, bool bEnable)
{
    volatile uint32_t *pSCTL;
    uint32_t secSCTL;

#ifdef ADI_DEBUG
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT )
    {
        return ADI_SEC_INVALID_SID_ID;
    }
#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Get the contents of source control register */
    secSCTL = *pSCTL;

    if(bEnable == true)
    {
        /* Set the source enable bit */
        secSCTL |= (uint32_t)ENUM_SEC_SCTL_SRC_EN;
    }
    else
    {
        /* Clear the source enable bit */
        secSCTL &= ((uint32_t)(~BITM_SEC_SCTL_SEN));
    }

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);

    return ADI_SEC_SUCCESS;
}



/**
 * @brief  Enable or disable the fault.
 *
 * This controls whether an interrupt request is forwarded to the Fault Interface.
 * Fault enable does not affect the ability for an interrupt source to set an
 * interrupt as pending, it only affects whether the pending request will be
 * forwarded to the SFI.
 *
 * @param [in] SID          The System interrupt ID that need to be forward or
 *                          not forwarded to Fault Interface.
 * @param [in] bEnable      Flag which indicates whether to forward the interrupt
 *                          to Fault Interface.
 *                          'true' to enable, 'false' to disable.
 * @return  Status
 *          - #ADI_SEC_SUCCESS if successfully enabled or disabled forwarding
 *            the given interrupt to Fault Interface.
 *          - #ADI_SEC_INVALID_SID_ID [D] if the given SID is not valid.
 *
 * @sa adi_sec_EnableInterrupt()
 * @sa adi_sec_EnableSource()
 */
ADI_SEC_RESULT adi_sec_EnableFault(uint32_t SID, bool bEnable)
{
    volatile uint32_t *pSCTL;
    uint32_t secSCTL;

#ifdef ADI_DEBUG
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }
#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Get the contents of source control register */
    secSCTL = *pSCTL;

    if(bEnable == true)
    {
        /* Set the fault enable bit */
        secSCTL |= (uint32_t)ENUM_SEC_SCTL_FAULT_EN;
    }
    else
    {
        /* Clear the fault enable bit */
        secSCTL &= ((uint32_t)(~BITM_SEC_SCTL_FEN));
    }

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief   Enable or disable edge sensitivity.
 *
 * @details A system interrupt can be configured to be edge or level
 *          sensitive.
 *
 * @param [in] SID          The System interrupt ID that need to be configured.
 * @param [in] bEnable      Flag which indicates whether to configure the interrupt
 *                          to be edge or level sensitive.
 *
 *                          'true' for Edge sensitive, 'false' for Level sensitive.
 * @return  Status
 * - #ADI_SEC_SUCCESS               If successfully set the sensitivity.
 * - #ADI_SEC_INVALID_SID_ID [D]    if the given SID is not valid.
 *
 * @sa adi_sec_EnableInterrupt()
 * @sa adi_sec_EnableSource()
 */
ADI_SEC_RESULT adi_sec_EnableEdgeSense(uint32_t SID, bool bEnable)
{

    volatile uint32_t *pSCTL;
    uint32_t secSCTL;

#ifdef ADI_DEBUG
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }
#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Get the contents of source control register */
    secSCTL = *pSCTL;

    if(bEnable == true)
    {
        /* Set the edge sensitive */
        secSCTL |= (uint32_t)ENUM_SEC_SCTL_EDGE;
    }
    else
    {
        /* Set level sensitive */
        secSCTL &= ((uint32_t)(~BITM_SEC_SCTL_ES));
    }

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief  Enables or disables errors detection for the SID.
 *
 * Enables or disables the error detection for the given Interrupt Source.
 * If the application registers an error handler (using adi_sec_RegisterErrorHandler)
 * errors are reported through the error handler.
 *
 * @param [in]  SID          The System interrupt ID for which error detection
 *                           to be enabled or disabled.
 * @param [in]  bEnable      Flag which indicates whether to enable or disable
 *                           error detection.
 *                           'true' to enable, false to disable.
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully enabled or disabled errors.
 *              - #ADI_SEC_INVALID_SID_ID [D] if the SID is not valid
 *
 * @sa adi_sec_RegisterErrorHandler()
 *
 */
ADI_SEC_RESULT adi_sec_EnableError(uint32_t SID, bool bEnable)
{
    volatile uint32_t *pSCTL;
    uint32_t secSCTL;

#ifdef ADI_DEBUG
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }
#endif /* ADI_DEBUG */

    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);                  /* SID used as offset to point at correct SCTL */

    /* Get the contents of source control register */
    secSCTL = *pSCTL;

    if( bEnable == true )
    {
        /* Set the error enable bit */
        secSCTL |= (uint32_t) ENUM_SEC_SCTL_ERR_EN;
    }
    else
    {
        /* Clear the error enable bit */
        secSCTL &= ((uint32_t)(~ENUM_SEC_SCTL_ERR_EN));
    }

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);


    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Set Priority of the SID.
 *
 * @details     Sets the SID relative priority with respect to other SIDs.
 *              The number of available priority levels depends upon the number
 *              of bits assigned for the priority levels. The total number of
 *              priority levels can be divided into levels and sub-levels.
 *              please refer adi_sec_SetPriorityLevels for more explanation on
 *              levels and sub-levels.
 *
 * @param[in]   SID        The system interrupt ID.
 * @param[in]   Priority   SID Priority level. The lower the number the higher
 *                         the priority. It can be any value between 0 and
 *                         ADI_SEC_MAX_INT_PRIO
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully set the SID priority.
 *              - #ADI_SEC_INVALID_SID_ID [D] if the SID is not valid.
 *              - #ADI_SEC_INVALID_PRIORITY [D] if the given priority is
 *                greater than the number of priority levels.
 *
 * @sa adi_sec_SetPriorityLevels()
 * @sa adi_sec_SetPriorityMask()
 */
ADI_SEC_RESULT adi_sec_SetPriority(uint32_t SID, uint32_t Priority)
{
    /* Pointer to the source control register for the given SID */
    volatile uint32_t *pSCTL;

    /* Contents of the source control register */
    uint32_t secSCTL;

#ifdef ADI_DEBUG

    /* Check if the given SID is valid */
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }

    /* 2DO: Add code to check the priority level based on the grouping
     */
    if(Priority > ADI_SEC_MAX_INT_PRIO)
    {
        return ADI_SEC_INVALID_PRIORITY;
    }

#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Read the current contents of Source control MMR */
    secSCTL = *pSCTL;

    /* Clear the previous priority setting */
    secSCTL = secSCTL &  ((uint32_t)~(BITM_SEC_SCTL_PRIO));

    /* Write the new priority settings */
    secSCTL = secSCTL | ((Priority << (uint32_t)BITP_SEC_SCTL_PRIO) & (uint32_t)BITM_SEC_SCTL_PRIO);

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Assign SID to Group(s).
 *
 * @details     The System Interrupt can be assigned to a group or multiple
 *              groups.
 *
 *              An example use case for grouping is to have a group
 *              of all interrupts that can wakeup the processor. Before going
 *              to sleep/idle all the non-wakeup interrupts can be masked and
 *              only wakeup interrups can be enabled to wakeup the processor.
 *
 * @param[in]   SID         The system interrupt ID
 * @param[in]   grouping    A combination of groups to which the SID is assigned.
 *                          To add SID into multiple groups the Group ID's should
 *                          be logically orred.
 *
 *                          For example to Add a SID into group 0 and 3
 *
 *                          grouping = ADI_SEC_INT_GRP0 | ADI_SEC_INT_GRP3
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully assigned SID to Group(s).
 *              - #ADI_SEC_INVALID_SID_ID [D] if the SID is not valid.
 *              - #ADI_SEC_INVALID_GROUPING [D] if the given grouping is invalid.
 *
 * @sa adi_sec_SetGroupMask()
 */
ADI_SEC_RESULT adi_sec_SetSIDGrouping(uint32_t SID, uint32_t grouping)
{
    /* Pointer to the source control register for the given SID */
    volatile uint32_t *pSCTL;

    /* Contents of the source control register */
    uint32_t secSCTL;

#ifdef ADI_DEBUG

    /* Check if the given SID is valid */
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }

    /* Check if the given grouping is valid */
    if(grouping > ADI_SEC_INT_ALL_GROUPS)
    {
        return ADI_SEC_INVALID_GROUPING;
    }

#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Read the current contents of Source control MMR */
    secSCTL = *pSCTL;

    /* Clear the previous group setting */
    secSCTL = secSCTL & ((uint32_t)~BITM_SEC_SCTL_GRP);

    /* Write the new group settings */
    secSCTL = secSCTL | ((grouping << (uint32_t)BITP_SEC_SCTL_GRP) & (uint32_t)BITM_SEC_SCTL_GRP);

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Assign system interrupt to a core.
 *
 * @details     Any system interrupt source can be assigned to any of the
 *              available cores on the processor.
 *
 * @param[in]   SID         The system interrupt ID.
 * @param[in]   eCoreID     Core ID to which the interrupt to be assigned.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully assigned SID to the given Core.
 *              - #ADI_SEC_INVALID_SID_ID [D] if the SID is not valid.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid
 */
ADI_SEC_RESULT adi_sec_SetCoreID(uint32_t SID, ADI_SEC_CORE_ID eCoreID)
{
    /* Pointer to the source control register for the given SID */
    volatile uint32_t *pSCTL;

    /* Contents of the source control register */
    uint32_t secSCTL;

#ifdef ADI_DEBUG

    /* Check if the given SID is valid */
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT )
    {
        return ADI_SEC_INVALID_SID_ID;
    }

    /* Check if the given core ID is valid */
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }

#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSCTL = pREG_SEC0_SCTL0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Read the current contents of Source control MMR */
    secSCTL = *pSCTL;

    /* Clear the previous core assignment */
    secSCTL = secSCTL &  ((uint32_t)(~BITM_SEC_SCTL_CTG));

    /* Write the given core assignment */
    secSCTL = secSCTL | (((uint32_t)eCoreID << (uint32_t)BITP_SEC_SCTL_CTG) & (uint32_t)BITM_SEC_SCTL_CTG);

    ADI_SEC_UNLOCK_REG(pSCTL);

    /* Write back the source control register contents */
    *pSCTL = secSCTL;

    ADI_SEC_LOCK_REG(pSCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Get SID Channel ID.
 *
 * @details     The Channel ID field indicates the ID of the specific source
 *              (from a set of sources sharing one SSI input) that asserted the
 *              SSI input. An SSI input may support multiple system sources in
 *              which case the assertion must be qualified by an identifier to
 *              determine the channel that generated the assertion. The CHID
 *              field provides this value in the form of a numeric reference that
 *              matches the ordering of the integration. A table for each shared
 *              SSI input will be provided to indicate the CHID to source
 *              mapping. The prioritization for simultaneously asserted sources
 *              is according to ID with 0 being the highest priority. CHID is
 *              captured when the SSI input is acknowledged.
 *              (see SCI for more detail on acknowledge).
 *
 * @param[in]   SID             The system interrupt ID.
 * @param[out]  pChannelID      Pointer to a location where the channel id is
 *                              written.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully obtained the Channel ID.
 *              - #ADI_SEC_INVALID_SID_ID [D] if the SID is not valid.
 *              - #ADI_SEC_INVALID_POINTER [D] if the given pointer is invalid.
 */
ADI_SEC_RESULT adi_sec_GetSIDChannelID(uint32_t SID, uint32_t *pChannelID)
{
    /* Pointer to the SSI status register */
    volatile uint32_t *pSSTAT;

#ifdef ADI_DEBUG

    /* Check if the given SID is valid */
    if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
    {
        return ADI_SEC_INVALID_SID_ID;
    }

    /* Check if the given pointer is valid */
    if( pChannelID == NULL)
    {
        return ADI_SEC_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    /* SID used as offset to point at correct SCTL */
    pSSTAT = pREG_SEC0_SSTAT0 + (SID << ADI_SEC_SSI_STRIDE);

    /* Get the Channel ID from the status register */
    *pChannelID = (*pSSTAT & (uint32_t)BITM_SEC_SSTAT_CHID) >> (uint32_t)BITP_SEC_SSTAT_CHID;

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Clears pending interrupt status.
 *
 * @details     This API is usually not required to called. This can be useful
 *              in a scenario when a device is disabled and enabled back. To
 *              make sure that there no pending interrupts when the device was
 *              previously enabled, the the pending interrupt status can be
 *              cleared.
 *
 * @param[in]   SID             The system interrupt ID.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully cleared the pending interrupt
 *                 status.
 *              - #ADI_SEC_INVALID_SID_ID [D] if the SID is not valid.
 */
ADI_SEC_RESULT adi_sec_ClearPendStatus(uint32_t SID)
{
    /* Pointer to the SSI status register */
     volatile uint32_t *pSSTAT;

 #ifdef ADI_DEBUG

     /* Check if the given SID is valid */
     if( SID > (uint32_t)PARAM_SEC0_SCOUNT)
     {
         return ADI_SEC_INVALID_SID_ID;
     }
 #endif /* ADI_DEBUG */

     /* SID used as offset to point at correct SCTL */
     pSSTAT = pREG_SEC0_SSTAT0 + (SID << ADI_SEC_SSI_STRIDE);

     /* clear the pending status (W1C) */
     *pSSTAT = (uint32_t)BITM_SEC_SSTAT_PND;

     return ADI_SEC_SUCCESS;
}

/******************************************************************************
 *
 * SEC_CCTLm
 *  The SEC_CCTLm register controls the operational state of the SCI
 *
 * SEC_CSTATm
 *  The SEC_CSTATm register indicates operational state of the SCI.
 *****************************************************************************/

/**
 * @brief  Enables or disables the operational state of the SCI.
 *
 * @details    When SCI is disabled, it only halts the execution of given
 *             Core Interface but register values are not affected. Also
 *             INT signal to the core is not affected.
 *
 *             When enabled begins or resumes the operation with the
 *             current configuration and status.
 *
 * @param [in]  eCoreID     Core ID whose interface to the System to be enabled.
 * @param [in]  bEnable     Flag which indicates whether to enable or disable.
 *                          'true' to enable, 'false' to disable.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully enabled or disabled core
 *                interface.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid
 *
 * @sa adi_sec_ResetSCI()
 * @sa adi_sec_Init()
 */
ADI_SEC_RESULT adi_sec_EnableSCI(ADI_SEC_CORE_ID eCoreID, bool bEnable )
{
    /* Pointer to the SCI control register */
    volatile uint32_t *pCCTL;
    /* Contents of SCI control register */
    uint32_t secCCTL;

#ifdef ADI_DEBUG
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* CORE ID used as offset to point at correct SCTL */
    pCCTL = pREG_SEC0_CCTL0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the contents of Core Interface Control register */
    secCCTL = *pCCTL;

    if(bEnable == true)
    {
        /* Set the SCI enable bit */
        secCCTL |=  (uint32_t)ENUM_SEC_CCTL_EN;
    }
    else
    {
        /* Clear the SCI enable bit */
        secCCTL &=  (uint32_t)(~BITM_SEC_CCTL_EN);
    }

    ADI_SEC_UNLOCK_REG(pCCTL);

    /* Write back the control register contents */
    *pCCTL = secCCTL;

    ADI_SEC_LOCK_REG(pCCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Resets core interface.
 *
 * @details     Resets all core interface registers to their default values
 *
 * @param [in]  eCoreID      Core ID which need to be reset.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully reset the core interface.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 *
 * @sa adi_sec_EnableSCI()
 */
ADI_SEC_RESULT adi_sec_ResetSCI(ADI_SEC_CORE_ID eCoreID)
{
    /* Pointer to the SCI control register */
    volatile uint32_t *pCCTL;
    /* Contents of SCI control register */
    uint32_t secCCTL;

#ifdef ADI_DEBUG
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* CORE ID used as offset to point at correct CCTL */
    pCCTL = pREG_SEC0_CCTL0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    ADI_SEC_UNLOCK_REG(pCCTL);

    /* Set the reset bit */
    *pCCTL |= (uint32_t)ENUM_SEC_CCTL_RESET;

    ADI_SEC_LOCK_REG(pCCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Wait for Idle indication from core.
 *
 * @details     Force SCI to wait for idle indication from core to resume its
 *              activity.
 *
 * @param [in]  eCoreID      Core ID whose interface to be set for Wait For Idle.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully set the Wait For Idle.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 *
 * @sa adi_sec_GetWFIStatus()
 * @sa adi_sec_ClearWFIStatus()
 *
 */
/* 2DO: Should we a bool flag as parameter to be able to disable WFI */
ADI_SEC_RESULT adi_sec_WaitForIdle(ADI_SEC_CORE_ID eCoreID)
{
    /* Pointer to the SCI control register */
    volatile uint32_t *pCCTL;
    /* Contents of SCI control register */
    uint32_t secCCTL;

#ifdef ADI_DEBUG
    if( (uint32_t) eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* CORE ID used as offset to point at correct SCTL */
    pCCTL = pREG_SEC0_CCTL0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the current contents of SCI control register */
    secCCTL = *pCCTL;

    /* Set Wait for Idle */
    secCCTL = secCCTL | ((uint32_t)ENUM_SEC_CCTL_WAITIDLE);

    ADI_SEC_UNLOCK_REG(pCCTL);

    /* Write back the  contents of SCI control register */
    *pCCTL = secCCTL;

    ADI_SEC_LOCK_REG(pCCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Enables or disables Non Maskable Interrupt.
 *
 * @details     When enabled allows Non Maskable interrupt to be propagated to
 *              the core for servicing.
 *
 *              By default NMI is enabled when the SEC is initialized. It can
 *              be disabled by using this API.
 *
 * @param [in]  eCoreID     Core ID for which the the NMI to be enabled.
 * @param [in]  bEnable     Flag which indicates whether to enable or disable NMI.
 *                          'true' to enable, 'false' to disable.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully enabled or disabled NMI.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 *
 * @sa adi_sec_Init()
 */
ADI_SEC_RESULT adi_sec_EnableNMI(ADI_SEC_CORE_ID eCoreID, bool bEnable )
{
    /* Pointer to the SCI control register */
    volatile uint32_t *pCCTL;
    /* Contents of SCI control register */
    uint32_t secCCTL;

#ifdef ADI_DEBUG
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* CORE ID used as offset to point at correct SCTL */
    pCCTL = pREG_SEC0_CCTL0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the current contents of SCI control register */
    secCCTL = *pCCTL;

    if(bEnable == true)
    {
        /* Set the enable bit */
        secCCTL = secCCTL | ((uint32_t)ENUM_SEC_CCTL_NMI_EN);
    }
    else
    {
        /* Clear the enable bit */
        secCCTL = secCCTL &  ((uint32_t)(~BITM_SEC_CCTL_NMIEN));
    }

    ADI_SEC_UNLOCK_REG(pCCTL);

    /* Write back the  contents of SCI control register */
    *pCCTL = secCCTL;

    ADI_SEC_LOCK_REG(pCCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Get WFI Status.
 *
 * @details     Returns the SCI's WFI status.
 *              WFI indicates whether the SCI is currently waiting for core
 *              idle indication.
 *
 * @param [in]  eCoreID     Core ID for WFI status to be queried.
 * @param [in]  pWFI        Pointer to a location where the WFI status is written.
 *                          'true' when WFI status is set.
 *                          'false' when WFI status is cleared.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully queried WFI status.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 */
ADI_SEC_RESULT adi_sec_GetWFIStatus(ADI_SEC_CORE_ID eCoreID, bool *pWFI)
{
    /* SCI Status register */
    volatile uint32_t *pCSTAT;

#ifdef ADI_DEBUG
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* core ID used as offset to point at correct SCTL */
    pCSTAT = pREG_SEC0_CSTAT0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the WFI status */
    *pWFI = (*pCSTAT & (uint32_t)BITM_SEC_CSTAT_WFI) ? true : false;

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Clears Wait For Idle Status.
 *
 * @param [in]  eCoreID      Core ID whose WFI status to be cleared.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully cleared WFI status.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 */
ADI_SEC_RESULT adi_sec_ClearWFIStatus(ADI_SEC_CORE_ID eCoreID)
{
    /* SCI Status register */
    volatile uint32_t *pCSTAT;

    /* Contents of SCI Status register */
    uint32_t secCSTAT;

#ifdef ADI_DEBUG
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* core ID used as offset to point at correct SCTL */
    pCSTAT = pREG_SEC0_CSTAT0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the contents of SCI status register */
    secCSTAT = *pCSTAT;

    /* Set the WFI bit to clear it (W1C) */
    secCSTAT |= ((uint32_t)ENUM_SEC_CSTAT_WAITING);

    /* Write back the status register contents */
    *pCSTAT = secCSTAT;

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Clears NMI Status.
 *
 * @param [in]  eCoreID      Core ID whose NMI status to be cleared.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully cleared NMI status.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 */
ADI_SEC_RESULT adi_sec_ClearNMIStatus(ADI_SEC_CORE_ID eCoreID)
{
    /* SCI Status register */
    volatile uint32_t *pCSTAT;

    /* Contents of SCI Status register */
    uint32_t secCSTAT;

#ifdef ADI_DEBUG
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* core ID used as offset to point at correct SCTL */
    pCSTAT = pREG_SEC0_CSTAT0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the contents of SCI status register */
    secCSTAT = *pCSTAT;

    /* Set the NMI bit to clear it (W1C) */
    secCSTAT |= ((uint32_t)ENUM_SEC_CSTAT_NMI);

    /* Write back the status register contents */
    *pCSTAT = secCSTAT;

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Set the priority mask.
 *
 * @details     Interrupts below the set priority level will be masked.
 *
 * @param [in]  eCoreID     Core ID for which the mask to be set.
 * @param [in]  Priority    Priority level below which the interrupts
 *                          to be masked.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully set the priority mask.
 *              - #ADI_SEC_INVALID_CORE_ID if the given Core ID is invalid.
 *              - #ADI_SEC_INVALID_PRIORITY if the given priority level is
 *                is invalid.
 *
 * @sa adi_sec_SetPriority()
 */
ADI_SEC_RESULT adi_sec_SetPriorityMask(ADI_SEC_CORE_ID eCoreID, uint32_t Priority)
{
    /* Priority Mask register */
    volatile uint32_t *pCPMSK;

    /* Priority mask value */
    uint32_t secCPMSK;

#ifdef ADI_DEBUG

    /* Check if the given Core ID is valid */
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }

    /* Check if the given priority level is valid */
    if(Priority > ADI_SEC_MAX_INT_PRIO)
    {
        return ADI_SEC_INVALID_PRIORITY;
    }
#endif /* ADI_DEBUG */

    /* core ID used as offset to point at correct SCTL */
    pCPMSK = pREG_SEC0_CPMSK0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the current contents of priority mask register */
    secCPMSK = *pCPMSK;

    /* Clear the existing priority mask */
    secCPMSK = secCPMSK &  ((uint32_t)(~BITM_SEC_CPMSK_PRIO));

    /* Set the given priority mask */
    secCPMSK = secCPMSK | ((Priority << (uint32_t)BITP_SEC_CPMSK_PRIO) & (uint32_t)BITM_SEC_CPMSK_PRIO);

    ADI_SEC_UNLOCK_REG(pCPMSK);

    /* Write back the contents of the priority mask register */
    *pCPMSK = secCPMSK;

    ADI_SEC_LOCK_REG(pCPMSK);

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Set the Group Mask.
 *
 * @details     The interrupts from the selected groups will masked.
 *
 * @param [in]  eCoreID     Core ID for which the mask to be set.
 * @param [in]  Groups      Groups to be masked.
 *                          If more than one group to be masked, the groups
 *                          should be logically orred.
 *
 *                          For example to mask Group 0 and Group 3
 *
 *                          Groups = ADI_SEC_INT_GRP0 | ADI_SEC_INT_GRP3
 *
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully set the group mask.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 *
 * @sa adi_sec_SetGroup()
 * @sa adi_sec_SetUnGroupedMask()
 */
ADI_SEC_RESULT adi_sec_SetGroupMask(ADI_SEC_CORE_ID eCoreID, uint32_t Groups)
{
    /* Group Mask register */
    volatile uint32_t *pCGMSK;
    /* Mask register value */
    uint32_t secCGMSK;

#ifdef ADI_DEBUG

    /* Check if the given Core ID is valid */
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* core ID used as offset to point at correct SCTL */
    pCGMSK = pREG_SEC0_CGMSK0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the current contents of the Group Mask register */
    secCGMSK = *pCGMSK;

    /* Clear the existing group mask */
    secCGMSK = secCGMSK &  ((uint32_t)(~BITM_SEC_CGMSK_GRP));

    /* Set the new group mask */
    secCGMSK = secCGMSK | ((Groups << (uint32_t)BITP_SEC_CGMSK_GRP) & (uint32_t)BITM_SEC_CGMSK_GRP);

    ADI_SEC_UNLOCK_REG(pCGMSK);

    /* Write back the contents of the Group Mask register */
    *pCGMSK = secCGMSK;

    ADI_SEC_LOCK_REG(pCGMSK);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Mask or Unmask the ungrouped interrupts.
 *
 * @details     Interrupts that are not grouped are masked or un masked.
 *
 * @param [in]  eCoreID     Core ID for which the mask to be set.
 * @param [in]  bMask       Flag to indicate whether to mask or unmask
 *                          'true' to Mask (Disable) the ungrouped interrupts.
 *                          'false' to Unmask (Enable) the ungrouped interrupts.
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully masked or unmasked ungrouped
 *                interrupts.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 *
 * @sa adi_sec_SetGroupMask()
 * @sa adi_sec_SetGroup()
 */
ADI_SEC_RESULT adi_sec_SetUnGroupedMask(ADI_SEC_CORE_ID eCoreID, bool bMask)
{
    /* Group Mask register */
    volatile uint32_t *pCGMSK;
    /* Mask register value */
    uint32_t secCGMSK;

#ifdef ADI_DEBUG
    /* Check if the given Core ID is valid */
    if((uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* core ID used as offset to point at correct SCTL */
    pCGMSK = pREG_SEC0_CGMSK0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the current contents of the Group Mask register */
    secCGMSK = *pCGMSK;

    if(bMask == true)
    {
        /* Set the UGRP bit */
        secCGMSK = secCGMSK | ((uint32_t)ENUM_SEC_CGMSK_MASK);
    }
    else
    {
        /* Clear the UGRP bit */
        secCGMSK = secCGMSK &  ((uint32_t)(~ENUM_SEC_CGMSK_MASK));
    }

    ADI_SEC_UNLOCK_REG(pCGMSK);

    /* Write back the contents of the Group Mask register */
    *pCGMSK = secCGMSK;

    ADI_SEC_LOCK_REG(pCGMSK);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Set the number of Priority Levels.
 *
 * @details     The total number of available priority levels can be divided into
 *              levels and sub levels. priority resolution provides the tie
 *              breaker for simultaneously pending interrupts assigned to the
 *              same level.
 *
 * @param [in]  eCoreID     Core ID for which priority levels to be set.
 * @param [in]  eLevels     Priority levels to be set. One of the values from
 *                          the enumeration ADI_SEC_PRIORITY_LEVELS
 *
 * @return      Status
 *              - #ADI_SEC_SUCCESS if successfully set the priority levels.
 *              - #ADI_SEC_INVALID_CORE_ID [D] if the Core ID is not valid.
 *
 * @sa adi_sec_SetPriority()
 * @sa adi_sec_SetPriorityMask()
 */
ADI_SEC_RESULT adi_sec_SetPriorityLevels(ADI_SEC_CORE_ID eCoreID, ADI_SEC_PRIORITY_LEVELS eLevels)
{
    /* Priority levels register */
    volatile uint32_t *pCPLVL;
    /* Priority level register value */
    uint32_t secCPLVL;

#ifdef ADI_DEBUG
    /* Check if the given Core ID is valid */
    if( (uint32_t)eCoreID > (uint32_t)PARAM_SEC0_CCOUNT)
    {
        return ADI_SEC_INVALID_CORE_ID;
    }
#endif /* ADI_DEBUG */

    /* core ID used as offset to point at correct SCTL */
    pCPLVL = pREG_SEC0_CPLVL0 + ((uint32_t)eCoreID << ADI_SEC_SCI_STRIDE);

    /* Get the current contents of Priority Level register */
    secCPLVL = *pCPLVL;

    /* Clear the current priority levels */
    secCPLVL = secCPLVL &  ((uint32_t)(~BITM_SEC_CPLVL_PLVL));

    /* Set the given number of priority levels */
    secCPLVL = secCPLVL | (((uint32_t)eLevels << (uint32_t)BITP_SEC_CPLVL_PLVL) & (uint32_t)BITM_SEC_CPLVL_PLVL);

    ADI_SEC_UNLOCK_REG(pCPLVL);

    /* Write back the Priority level register contents */
    *pCPLVL = secCPLVL;

    ADI_SEC_LOCK_REG(pCPLVL);

    return ADI_SEC_SUCCESS;
}

#endif /* !defined (ADI_ADSP_CM40Z) */

/*==================================
   SEC Fault interface (SFI) APIs
 *==================================*/

/**
 * @brief       Reset SEC Fault Interface (SFI).
 *
 * @details     When SFI is reset all the registers will be reset to default values.
 *              The FAULT signal will be deasserted.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully reset the SFI.
 */
ADI_SEC_RESULT adi_sec_ResetSFI(void)
{
    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = (uint32_t)ENUM_SEC_FCTL_RESET;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Enable/disable SEC Fault Interface (SFI).
 *
 * @details     By default fault interface is disabled. When SFI is disabled
 *              the current configurations are retained.
 *
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          the Fault Interface.
 *                          true    -   To eanable.
 *                          false   -   To disable.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully enabled/disabled SFI.
 */
ADI_SEC_RESULT adi_sec_EnableSFI(bool bEnable)
{
    uint32_t    secFCTL;

    /* Get the value of Fault control register */
    secFCTL =   *pREG_SEC0_FCTL;

    if(bEnable == true)
    {
        /* Enable the fault interface */
        secFCTL |=  (uint32_t)ENUM_SEC_FCTL_EN;
    }
    else
    {
        /* Disable the fault interface */
        secFCTL &=  (uint32_t)~BITM_SEC_FCTL_EN;
    }

    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = secFCTL;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Enable/Disable fault output generation.
 *
 * @details     The same signal is used for COP (Computer Operating Properly)
 *              and Fault. When COP mode is enabled (using API #adi_sec_EnableCOPMode)
 *              even though fault is enabled the COP signal is sent out.
 *
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          the Fault output.
 *                          true    -   To enable.
 *                          false   -   To disable.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully enabled/disabled Fault output.
 *
 * @sa adi_sec_EnableCOPMode()
 */
ADI_SEC_RESULT adi_sec_EnableFaultOutput(bool bEnable)
{
    uint32_t    secFCTL;

    /* Get the value of Fault control register */
    secFCTL =   *pREG_SEC0_FCTL;

    if(bEnable == true)
    {
        /* Enable the fault output */
        secFCTL |=  (uint32_t)ENUM_SEC_FCTL_FLTOUT_EN;
    }
    else
    {
        /* Disable the fault output */
        secFCTL &=  (uint32_t)~BITM_SEC_FCTL_FOEN;
    }

    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = secFCTL;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Enable/Disable trigger output.
 *
 * @details     The SEC fault interface can generate a trigger output for a
 *              fault. The trigger can be configured
 *              (using API #adi_sec_SelectTriggerEvent) to assert when the fault
 *              is pending or when the fault changes to active mode (fault not
 *              handled before the fault delay expires).
 *
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          the Fault output.
 *                          true    -   To enable.
 *                          false   -   To disable.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully enabled/disabled Trigger output.
 *
 * @sa adi_sec_SelectTriggerEvent()
 * @sa adi_sec_FaultDelay()
 */
ADI_SEC_RESULT adi_sec_EnableTriggerOutput(bool bEnable)
{
    uint32_t    secFCTL;

    /* Get the value of Fault control register */
    secFCTL =   *pREG_SEC0_FCTL;

    if(bEnable == true)
    {
        /* Enable the trigger output */
        secFCTL |=  (uint32_t)ENUM_SEC_FCTL_TRGOUT_EN;
    }
    else
    {
        /* Disable the trigger output */
        secFCTL &=  (uint32_t)~BITM_SEC_FCTL_TOEN;
    }

    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = secFCTL;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}



/**
 * @brief       Enable/Disable system reset.
 *
 * @details     The SEC fault interface can generate a System Reset signal when
 *              a fault is not handled before the fault delay expires. The fault
 *              delay can be set using API #adi_sec_SetFaultDelay.
 *
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          Reset output.
 *                          true    -   To enable.
 *                          false   -   To disable.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully enabled/disabled Reset output.
 *
 * @sa adi_sec_FaultDelay()
 */
ADI_SEC_RESULT adi_sec_EnableSystemReset(bool bEnable)
{
    uint32_t    secFCTL;

    /* Get the value of Fault control register */
    secFCTL =   *pREG_SEC0_FCTL;

    if(bEnable == true)
    {
        /* Enable the Reset output */
        secFCTL |=  (uint32_t)ENUM_SEC_FCTL_RESET;
    }
    else
    {
        /* Disable the Reset output */
        secFCTL &=  (uint32_t)~BITM_SEC_FCTL_RESET;
    }

    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = secFCTL;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Enable/Disable fault input.
 *
 * @details     The SEC fault interface can accept a fault input. when enabled
 *              the Fault and Fault inverse signals are sampled by fault interface.
 *
 *              The Fault and Fault inverse signals are bi-directional. When in
 *              fault mode (COP mode is disabled) and when Fault input is enabled
 *              the Fault and Fault inverse signals are sampled when the Fault
 *              output is not asserted. A fault is generated when any one of the
 *              Fault or Fault inverse signals are asserted from an external source.
 *
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          Fault input.
 *                          true    -   To enable.
 *                          false   -   To disable.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully enabled/disabled Fault input.
 *
 * @sa adi_sec_EnableFaultOutput()
 * @sa adi_sec_EnableCOPMode()
 */
ADI_SEC_RESULT adi_sec_EnableFaultInput(bool bEnable)
{
    uint32_t    secFCTL;

    /* Get the value of Fault control register */
    secFCTL =   *pREG_SEC0_FCTL;

    if(bEnable == true)
    {
        /* Enable the Fault input */
        secFCTL |=  (uint32_t)ENUM_SEC_FCTL_FLTIN_EN;
    }
    else
    {
        /* Disable the Fault input */
        secFCTL &=  (uint32_t)~BITM_SEC_FCTL_FIEN;
    }

    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = secFCTL;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Enable/Disable COP Mode.
 *
 * @details     The Fault interface can be operated in COP (Computer Operating
 *              Properly mode) to generate a COP signal (through Fault and
 *              Fault inverse) when there is no fault. When there is no fault
 *              the COP signal is generated periodically based on the configured
 *              COP period (Period can be configured using API #adi_sec_SetCOPPeriod).
 *              The COP signal is stopped when a fault is detected.
 *
 *              When COP mode is disabled, the fault interface operates in fault mode.
 *
 * @note        When in COP mode the Fault and Fault inverse signals are not sampled
 *              even when the fault input is enabled.
 *
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          COP Mode.
 *                          true    -   To enable (Operate in COP Mode).
 *                          false   -   To disable (Operate in Fault Mode).
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully enabled/disabled COP mode.
 *
 * @sa adi_sec_SetCOPPeriod()
 * @sa adi_sec_EnableFaultInput()
 */
ADI_SEC_RESULT adi_sec_EnableCOPMode(bool bEnable)
{
    uint32_t    secFCTL;

    /* Get the value of Fault control register */
    secFCTL =   *pREG_SEC0_FCTL;

    if(bEnable == true)
    {
        /* Enable the COP mode */
        secFCTL |=  (uint32_t)ENUM_SEC_FCTL_COP_MODE;
    }
    else
    {
        /* Disable the COP mode (operate in Fault mode) */
        secFCTL &=  (uint32_t)~BITM_SEC_FCTL_CMS;
    }

    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = secFCTL;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Get the Fault ID that caused the fault.
 *
 * @details     A fault can be caused by different sources. If the fault is caused
 *              by an external source (fault input line), then the fault ID will
 *              be equal to ADI_SEC_EXTERNAL_FAULT.
 *
 * @param [out] pFID    Pointer to a location where the Fault ID is written.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully retrieved the Fault ID.
 * - #ADI_SEC_INVALID_POINTER       If the given pointer is invalid.
 *
 */
ADI_SEC_RESULT adi_sec_GetFaultID(uint32_t *pFID)
{

#ifdef ADI_DEBUG
    /* Validate the given pointer */
    if(pFID == NULL)
    {
        return ADI_SEC_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Return the Fault ID */
    *pFID    =   *pREG_SEC0_FSID;

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Clear a pending fault.
 *
 * @details     When a fault goes to pending it can be cleared after taking
 *              handling the fault so that the fault does not change its state
 *              to active. When the fault changes its state to active, the
 *              configured fault actions (System Reset/Trigger/Fault output) are
 *              executed.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully cleared the pending fault.
 */
ADI_SEC_RESULT adi_sec_ClearFault(void)
{

    /* Read the Fault SID (FSID) register value and write it to Fault End (FEND)
     * register to clear the pend (SEC_FSTAT.PND) and active (SEC_FSTAT.ACT)
     * fault status.
     */
    *pREG_SEC0_FEND = *pREG_SEC0_FSID;

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Select trigger event mode.
 *
 * @details     When a trigger output is enabled (using API #adi_sec_EnableTriggerOutput),
 *              it can be configured to generate trigger as soon as the fault is in
 *              pending state or when it goes to an active state. When the fault is
 *              active the configured action will be performed and cannot be stopped.
 *              It will be too late to stop the actions by the time the fault is active.
 *              To avoid that, this API can be used to generate the trigger as soon as
 *              the fault goes to pending state and clear the fault
 *              (using API #adi_sec_ClearFault)before the fault delay
 *              (set using API #adi_sec_SetFaultDelay) expires.
 *
 * @param [in]  eTrigEvent  Trigger event type. One of the values from the enumeration
 *                          #ADI_SEC_TRIG_EVENT.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully set the trigger event type.
 *
 * @sa adi_sec_EnableTriggerOutput()
 * @sa adi_sec_SetFaultDelay()
 * @sa adi_sec_ClearFault()
 */
ADI_SEC_RESULT adi_sec_SelectTriggerEvent(ADI_SEC_TRIG_EVENT eTrigEvent)
{
    uint32_t    secFCTL;

    /* Get the value of Fault control register */
    secFCTL =   *pREG_SEC0_FCTL;

    if(eTrigEvent == ADI_SEC_TRIG_FAULT_PENDING)
    {
        /* Set the trigger output to be generated as soon as the fault is pending */
        secFCTL |=  (uint32_t)ENUM_SEC_FCTL_FLTPND_MODE;
    }
    else
    {
        /* Set the trigger output to be generated after the fault is active */
        secFCTL &=  (uint32_t)~BITM_SEC_FCTL_TES;
    }

    /* Write back the register contents */
    ADI_SEC_UNLOCK_REG(pREG_SEC0_FCTL);
    *pREG_SEC0_FCTL = secFCTL;
    ADI_SEC_LOCK_REG(pREG_SEC0_FCTL);

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Set the fault delay.
 *
 * @details     A fault delay (in terms of System Clock Cycles) is the System
 *              delay between the fault changing its state from pending to active.
 *              When a fault is detected it will be in pending state and after the fault
 *              delay is expired, it becomes active and the configured actions
 *              System Reset/Trigger/Fault output) are executed.
 *
 * @note        The system reset assertion can be delayed further from fault active
 *              by setting the System Reset Delay (using the API #adi_sec_SetSysResetDelay).
 *
 * @param [in]  nDelay      Delay to be set in terms of system clock cycles.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS               If successfully set the fault delay.
 *
 * @sa adi_sec_SetSysResetDelay()
 * @sa adi_sec_EnableFaultOutput()
 * @sa adi_sec_EnableSystemReset()
 * @sa adi_sec_EnableTriggerOutput()
 */
ADI_SEC_RESULT adi_sec_SetFaultDelay(uint32_t nDelay)
{
    /* Set the given delay */
    *pREG_SEC0_FDLY =   nDelay;

    return ADI_SEC_SUCCESS;
}

/**
 * @brief       Set the System Reset delay.
 *
 * @details     The system reset assertion (when enabled using
 *              API #adi_sec_EnableSystemReset) can be delayed further from fault
 *              becoming active by setting the System Reset Delay.
 *
 * @param [in]  nDelay      Delay to be set in terms of system clock cycles.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS       If successfully set the System Reset delay.
 *
 * @sa adi_sec_EnableSystemReset()
 */
ADI_SEC_RESULT adi_sec_SetSysResetDelay(uint32_t nDelay)
{
    /* Set the given delay */
    *pREG_SEC0_FSRDLY =   nDelay;

    return ADI_SEC_SUCCESS;
}


/**
 * @brief       Set the COP Period.
 *
 * @details     The fault interface can be configured to generate a COP
 *              signal (Computer Operating Properly) periodically. This
 *              API can be used to set the width (in terms of System
 *              clock cycles) of each phase (high/low) of the COP signal.
 *              The actual width is +1 the value that is set.
 *
 * @note        The COP mode should be enabled
 *              (using API #adi_sec_EnableCOPMode) to generate the
 *              COP signal.
 *
 * @param [in]  nPeriod     Period to be set in terms of system clock cycles.
 *
 * @return      Status
 * - #ADI_SEC_SUCCESS       If successfully set the System Reset delay.
 *
 * @sa adi_sec_EnableCOPMode()
 */
ADI_SEC_RESULT adi_sec_SetCOPPeriod(uint32_t nPeriod)
{
    /* Set the given period */
    *pREG_SEC0_FCOPP =   nPeriod;

    return ADI_SEC_SUCCESS;
}

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_16_7:"Allow non modified pointers to be declared without const.\
                                      Default handler does not use any parameters")
#endif

/* Default error handler which does nothing */
static void DefaultErrorHandler (
        void    * pCBParam,       /*!< Client supplied callback param */
        uint32_t  Event,          /*!< Event ID specific to the Driver/Service */
        void    * pArg)           /*!< Pointer to the event specific argument */
{
    return;
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


/******************************************************************************
 *
 * SEC Fault Control
 *
 *
 *****************************************************************************/


/*@}*/


#ifdef _MISRA_RULES
#pragma diag(pop)
#endif
