/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_pwm_bf6xx.c
*
* @brief     PWM service global API implementation
*
* @details
*            This is a source file which contains the implementation of global
*            API functions of the PWM service.
*/

/** @addtogroup PWM_Service_BF6xx PWM Service
*  @{
*/
/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"There shall be no definitions of objects or functions in a header file.")
#pragma diag(suppress:misra_rule_13_7:"Allow to add asserts for memory size macros")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#endif /* _MISRA_RULES */
#include "adi_pwm_data_bf6xx.c"
#include <assert.h>


/* Function declarations */
static void Pwm_Sync_Interrupt_Handler(uint32_t SID, void *pCBParam);
static void Pwm_Trip_Interrupt_Handler(uint32_t SID, void *pCBParam);

static ADI_PWM_RESULT ValidatePwmHandle(ADI_PWM_HANDLE const hDevice);


/*!
 * @brief       The adi_pwm_Open API opens a specified SPI device.
 *
 * @param[in]   DeviceNumber    The device instance number to be opened.
 * @param[in]   pDeviceMemory   The pointer to the memory passed by application.
 * @param[in]   MemorySize      The memory size passed by application.
 * @param[out]  phDevice        The pointer to a location where the handle to the opened device is written. 
 * @param[in]   pfCallback      The application supplied callback which will be called to notify device
 *                              related events.
 * @param[in]   pCBParam        The application supplied callback parameter which can be passed back in
 *                              the callback function. 
 *
 * @return      Status
 *              - #ADI_PWM_SUCCESS                The device is successfully opened for the given instance.
 *              - #ADI_PWM_FAILURE                A generalised API failure.
 *              - #ADI_PWM_IN_USE                 The specified device has already been initialised.
 *              - #ADI_PWM_INSUFFICIENT_MEMORY    Not enough memory for DMA mode
 *              - #ADI_PWM_INVALID_DEVICE_NUM [D] The given device number is greater than the number of devices.
 *              - #ADI_PWM_INVALID_HANDLE [D]     The pointer to the SPI device handle is NULL.
 *              - #ADI_PWM_INTERRUPT_FAILED       The driver's interrupt failed.
 *
 * No other PWM APIs may be called until the device open function is called.
 *
 * Opens a specified PWM device and passed back to the application a handle.  It initialises the service's
 * default data parameters.  
 *
 * @sa          adi_pwm_Close().
 */
ADI_PWM_RESULT adi_pwm_Open(uint32_t        DeviceNumber,
                            void            *pDeviceMemory,
                            uint32_t        MemorySize,
                            ADI_PWM_HANDLE  *phDevice,                            
                            ADI_CALLBACK    const pfCallback,
                            void            *const pCBParam)                            
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)
    /* check if device ID is invalid */
    if (nDeviceNumber >= ADI_PWM_NUM_DEVICES)
    {
        /* Bad Device Number */
        return(ADI_PWM_BAD_RESOURCE_NUMBER);
    }
    /* Check if the given pointer to device handle is valid */
    if(phDevice == NULL)
    {
        return (ADI_PWM_INVALID_POINTER);
    }
        
#endif
    /* get PWM device instance */
    pDevice = &aPwmDevice[DeviceNumber];

    /* check if device is not in use) */
    if (pDevice->pDevData != NULL)
    {
        return ADI_PWM_IN_USE;
    }
    else
    {
        /* initialise the memory */
        pDevice->pDevData = (ADI_PWM_DEV_DATA *) pDeviceMemory;
    }

    /* asserts to validate the memory size macros */
    assert(ADI_PWM_MEMORY_SIZE == sizeof(ADI_PWM_DEV_DATA));

    /* Clear the given memory */
    memset(pDeviceMemory, 0, MemorySize);

    /* save parameters for device instance    */
    pDevice->pDevData->pfCallback = pfCallback;
    pDevice->pDevData->pCBParam   = pCBParam;

    /* deafult PWM MMR */
    pDevice->pRegs->PWM_CONTROL  = 0x00020000u;
    pDevice->pRegs->PWM_CHANCFG  = 0x0u;
    pDevice->pRegs->PWM_TRIPCFG  = 0x0u;
    pDevice->pRegs->PWM_IMASK    = 0x0u;
    pDevice->pRegs->PWM_ILAT     = 0x0u;
    pDevice->pRegs->PWM_CHOPCFG  = 0x0u;
    pDevice->pRegs->PWM_DT       = 0x0u;
    pDevice->pRegs->PWM_SYNCWT   = 0x000003FFu;
    pDevice->pRegs->PWM_TM0      = 0x0u;
    pDevice->pRegs->PWM_TM1      = 0x0u;
    pDevice->pRegs->PWM_TM2      = 0x0u;
    pDevice->pRegs->PWM_TM3      = 0x0u;
    pDevice->pRegs->PWM_TM4      = 0x0u;
    pDevice->pRegs->PWM_DELAYA   = 0x0u;
    pDevice->pRegs->PWM_DELAYB   = 0x0u;
    pDevice->pRegs->PWM_DELAYC   = 0x0u;
    pDevice->pRegs->PWM_DELAYD   = 0x0u;
    pDevice->pRegs->PWM_CTRLCHA  = 0x0u;
    pDevice->pRegs->PWM_AH0      = 0x0u;
    pDevice->pRegs->PWM_AH1      = 0x0u;
    pDevice->pRegs->PWM_HPAH0    = 0x0u;
    pDevice->pRegs->PWM_HPAH1    = 0x0u;
    pDevice->pRegs->PWM_AL0      = 0x0u;
    pDevice->pRegs->PWM_AL1      = 0x0u;
#if defined(ADI_ADSP_CM40Z)
    pDevice->pRegs->PWM_HPAL0    = 0x0u;
    pDevice->pRegs->PWM_HPAL1    = 0x0u;
#endif    
    pDevice->pRegs->PWM_CTRLCHB  = 0x0u;
    pDevice->pRegs->PWM_BH0      = 0x0u;
    pDevice->pRegs->PWM_BH1      = 0x0u;
    pDevice->pRegs->PWM_HPBH0    = 0x0u;
    pDevice->pRegs->PWM_HPBH1    = 0x0u;
    pDevice->pRegs->PWM_BL0      = 0x0u;
    pDevice->pRegs->PWM_BL1      = 0x0u;
#if defined(ADI_ADSP_CM40Z)
    pDevice->pRegs->PWM_HPBL0    = 0x0u;
    pDevice->pRegs->PWM_HPBL1    = 0x0u;
#endif    
    pDevice->pRegs->PWM_CTRLCHC  = 0x0u;
    pDevice->pRegs->PWM_CH0      = 0x0u;
    pDevice->pRegs->PWM_CH1      = 0x0u;
    pDevice->pRegs->PWM_HPCH0    = 0x0u;
    pDevice->pRegs->PWM_HPCH1    = 0x0u;
    pDevice->pRegs->PWM_CL0      = 0x0u;
    pDevice->pRegs->PWM_CL1      = 0x0u;
#if defined(ADI_ADSP_CM40Z)
    pDevice->pRegs->PWM_HPCL0    = 0x0u;
    pDevice->pRegs->PWM_HPCL1    = 0x0u;
#endif    
    pDevice->pRegs->PWM_CTRLCHD  = 0x0u;
    pDevice->pRegs->PWM_DH0      = 0x0u;
    pDevice->pRegs->PWM_DH1      = 0x0u;
    pDevice->pRegs->PWM_HPDH0    = 0x0u;
    pDevice->pRegs->PWM_HPDH1    = 0x0u;
    pDevice->pRegs->PWM_DL0      = 0x0u;
    pDevice->pRegs->PWM_DL1      = 0x0u;
#if defined(ADI_ADSP_CM40Z)
    pDevice->pRegs->PWM_HPDL0    = 0x0u;
    pDevice->pRegs->PWM_HPDL1    = 0x0u;
#endif    

    /* register the spi tx data interrupt handler with SEC */
    if(adi_int_InstallHandler(pDevice->PwmSyncSID,
    						  Pwm_Sync_Interrupt_Handler,
    		                  pDevice,
                              true)
                              != ADI_INT_SUCCESS)
    {
    	return ADI_PWM_INTERRUPT_FAILED;
    }

    /* register the spi rx data interrupt handler with SEC */
    if(adi_int_InstallHandler(pDevice->PwmTripSID,
    						  Pwm_Trip_Interrupt_Handler,
    		                  pDevice,
                              true)
                              != ADI_INT_SUCCESS)
    {
    	return ADI_PWM_INTERRUPT_FAILED;
    }

    /* Pass PWM Device handle to client */
    *phDevice = (ADI_PWM_HANDLE)pDevice;

    /* return */
    return (Result);
}

/**
 * @brief   adi_pwm_Close
 *          This function closes the selected PWM device
 *
 * @param   [in]  hDevice               Handle of the PWM device
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */
ADI_PWM_RESULT adi_pwm_Close(ADI_PWM_HANDLE const hDevice)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;

    /* PWM Device */
    ADI_PWM *pDevice;

    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* disable PWM trip interrupt */
    if(adi_int_EnableInt(pDevice->PwmTripSID, false) != ADI_INT_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }

    /* disable PWM sync interrupt */
    if(adi_int_EnableInt(pDevice->PwmSyncSID, false) != ADI_INT_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }

    /* uninstall PWM trip interrupt handler */
    if(adi_int_UninstallHandler(pDevice->PwmTripSID) != ADI_INT_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
    /* uninstall PWM sync interrupt handler */
    if(adi_int_UninstallHandler(pDevice->PwmSyncSID) != ADI_INT_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }

    /* free device memory */
    pDevice->pDevData = NULL;

    /* return */
    return (Result);
}


/**
 * @brief   adi_pwm_InterruptEnable
 *          This function enables / disables the interrupts of the PWM.
 *
 * @param   [in] hDevice                Handle of the PWM device
 * @param   [in] eInterrupt             PWM interrupt ID
 * @param   [in] bEnable                Flag to manage the enabling/disabling of an interrupt.
 *               						- true       Enable interrupt.
 *               						- false      Disable interrupt.
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_InterruptEnable(ADI_PWM_HANDLE const hDevice, ADI_PWM_INTERRUPT_ID eInterrupt, bool bEnable)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;
    
    if(bEnable)
    {
    	/* enable PWM interrupt */
        pDevice->pRegs->PWM_IMASK |= (uint32_t)eInterrupt;
    }     
    else
    {
        /* disable PWM interrupt */
        pDevice->pRegs->PWM_IMASK &= ~(uint32_t)eInterrupt;
    }

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetReferencePeriod
 *          This function sets the reference period of the main timer (TIMER0)
 *          It is possible to slave one or more PWM channels to this common timer or
 *          alternatively have independent timers for each PWM channel.
 *
 * @param   [in]  hDevice				Handle of the PWM device
 * @param   [in]  Period                16 bit period of the main timer
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetReferencePeriod(ADI_PWM_HANDLE const hDevice, uint32_t Period)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;
    /* set the period of the reference timer, TIMER0 */
    pDevice->pRegs->PWM_TM0 = Period;

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetPeriod
 *          This function sets the period of TIMER_x for the PWM_x channel
 *
 * @param   [in]  hDevice              	Handle of the PWM device
 * @param   [in]  eChannel             	PWM Channel pair identifier
 * @param   [in]  Period               	32 bit period of the PWM channel
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetPeriod(ADI_PWM_HANDLE const hDevice, ADI_PWM_CHANNEL eChannel, uint32_t Period)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel set the PWM period */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set period of timer 1 for channel A */
        pDevice->pRegs->PWM_TM1 = Period;
    }
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set period of timer 2 for channel B */
        pDevice->pRegs->PWM_TM2 = Period;
    }
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set period of timer 3 for channel C */
        pDevice->pRegs->PWM_TM3 = Period;
    }
    else if(eChannel == ADI_PWM_CHANNEL_D)
    {
        /* set period of timer 4 for channel D */
        pDevice->pRegs->PWM_TM4 = Period;
    }
    else
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetDelay
 *          This function sets the delay time between main timer (TIMER0) and
 *          the start of TIMER_x of the PWM_x channel.
 *
 * @param   [in]  hDevice              	Handle of the PWM device
 * @param   [in]  eChannel             	PWM Channel pair identifier
 * @param   [in]  Delay                	16 bit delay of the PWM channel
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetDelay(ADI_PWM_HANDLE const hDevice, ADI_PWM_CHANNEL eChannel, uint32_t Delay)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel to set the PWM delay */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set delay for channel A */
        pDevice->pRegs->PWM_DELAYA = Delay;
    }
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set delay for channel B */
        pDevice->pRegs->PWM_DELAYB = Delay;
    }
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set delay for channel C */
        pDevice->pRegs->PWM_DELAYC = Delay;
    }
    else if(eChannel == ADI_PWM_CHANNEL_D)
    {
        /* set delay for channel D */
        pDevice->pRegs->PWM_DELAYD = Delay;
    }
    else
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }

    /* Return */
    return (Result);
}
/**
 * @brief   adi_pwm_SetDeadtime
 *          This function sets the deadtime between the high side and low side outputs
 *          of a PWM channel.
 *
 * @param   [in]  hDevice              	Handle of the PWM device
 * @param   [in]  Deadtime             	10 bit deadtime between the high and low side outputs of a PWM channel
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetDeadtime(ADI_PWM_HANDLE const hDevice, uint32_t Deadtime)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* set the PWM deadtime bewtween all high and low side outputs*/
    pDevice->pRegs->PWM_DT = Deadtime;
    
    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetChopFrequency
 *          This function sets the period of the high-frequency chopping signal.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  Chop                  8 bit chopping frequency on the chop enabled outputs
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetChopFrequency(ADI_PWM_HANDLE const hDevice, uint32_t Chop)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
   /* debug build only */
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* set the PWM gated chop frequency */
    pDevice->pRegs->PWM_CHOPCFG = Chop;

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetSyncWidth
 *          This function sets the width of the internal SYNC pulse.
 *
 * *param   [in]  hDevice              	Handle of the PWM device
 * *param   [in]  SyncWidth            	10 bit internal SYNC pulse width output
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetSyncWidth(ADI_PWM_HANDLE const hDevice, uint32_t SyncWidth)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* set the PWM sync pulse width */
    pDevice->pRegs->PWM_SYNCWT = SyncWidth;

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetOutputDutyCycle
 *          This function sets the duty cycle / phase of either the high side or low side output,
 *          of a PWM channel.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM channel pair identifier
 * @param   [in]  eChannelOutput        PWM channel pair output identifier, either high or low side
 * @param   [in]  Duty                  16 bit duty  cycle of the PWM channel output
 * @param   [in]  Duty1                 16 bit duty1 cycle of the PWM channel output
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetOutputDutyCycle(ADI_PWM_HANDLE 	     const hDevice,
		                                  ADI_PWM_CHANNEL 	     eChannel,
		                                  ADI_PWM_CHANNEL_OUTPUT eChannelOutput,
		                                  uint32_t 			     Duty,
		                                  uint32_t 			     Duty1)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* pointer to PWM register */    
    volatile uint32_t *pPWMReg;
    volatile uint32_t *pPWMReg1;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel and output channel to set duty cycle0 parameter */
    if((eChannel == ADI_PWM_CHANNEL_A) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
        /* set pointer to duty cycle0 register for channel A high side output */
        pPWMReg  =  &pDevice->pRegs->PWM_AH0;
        pPWMReg1 =  &pDevice->pRegs->PWM_AH1;
    }
    else if((eChannel == ADI_PWM_CHANNEL_A) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
       /* set pointer to duty cycle0 register for channel A low side output */
       pPWMReg  =  &pDevice->pRegs->PWM_AL0;
       pPWMReg1 =  &pDevice->pRegs->PWM_AL1;
    }
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
       /* set pointer to duty cycle0 register for channel B high side output */
        pPWMReg  =  &pDevice->pRegs->PWM_BH0;
        pPWMReg1 =  &pDevice->pRegs->PWM_BH1;
    }
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
       /* set pointer to duty cycle0 register for channel B low side output */
       pPWMReg  =  &pDevice->pRegs->PWM_BL0;
       pPWMReg1 =  &pDevice->pRegs->PWM_BL1;
    }
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
       /* set pointer to duty cycle0 register for channel C high side output */
       pPWMReg  =  &pDevice->pRegs->PWM_CH0;
       pPWMReg1 =  &pDevice->pRegs->PWM_CH1;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
       /* set pointer to duty cycle0 register for channel C low side output */
       pPWMReg  =  &pDevice->pRegs->PWM_CL0;
       pPWMReg1 =  &pDevice->pRegs->PWM_CL1;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
       /* set pointer to duty cycle0 register for channel D high side output */
       pPWMReg  =  &pDevice->pRegs->PWM_DH0;
       pPWMReg1 =  &pDevice->pRegs->PWM_DH1;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
       /* set pointer to duty cycle0 register for channel D low side output */
       pPWMReg  =  &pDevice->pRegs->PWM_DL0;
       pPWMReg1 =  &pDevice->pRegs->PWM_DL1;
    }
    else
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }
         
    /* set duty cycle */
    *pPWMReg  = Duty;
    *pPWMReg1 = Duty1;

    /* Return */
    return (Result);
}


/*****************************************************************************/
/* CONTROL REGISTER
*/
/*****************************************************************************/

/**
 * @brief   adi_pwm_Enable
 *          This function is the global PWM enable
 *          It enables/disables all PWM outputs
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  bEnable               Flag to manage the enabling/disabling of the PWM.
 *               						- true       Enable PWM.
 *               						- false      Disable PWM.
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_Enable(ADI_PWM_HANDLE const hDevice, bool bEnable)
{

    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;

    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;
           
    if(bEnable)
    {
      /* global enable PWM */
      pDevice->pRegs->PWM_CONTROL |= (uint32_t)BITM_PWM_CTL_GLOBEN;
  }
  else
   {
	  /* global disable PWM */
      pDevice->pRegs->PWM_CONTROL &= (uint32_t)~BITM_PWM_CTL_GLOBEN;
   }

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_DelayEnable
 *
 * This function enables / disables the timer delay of a PWM channel.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM channel pair identifier
 * @param   [in]  bEnable           	Flag to manage the enabling/disabling of delay.
 *               						- true       Enable delay.
 *               						- false      Disable delay.
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_DelayEnable(ADI_PWM_HANDLE const hDevice, ADI_PWM_CHANNEL eChannel, bool bEnable)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits */
    uint32_t Mask;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel to enable / disable delay */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set mask for channel A delay enable */
    	Mask = (uint32_t)BITM_PWM_CTL_DLYAEN;
    }
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set mask for channel B delay enable */
    	Mask = (uint32_t)BITM_PWM_CTL_DLYBEN;
    }
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set mask for channel C delay enable */
        Mask = (uint32_t)BITM_PWM_CTL_DLYCEN;
     }
     else if(eChannel == ADI_PWM_CHANNEL_D)
     {
        /* set mask for channel D delay enable */
        Mask = (uint32_t)BITM_PWM_CTL_DLYDEN;
     }
     else
     {
        /* parameter out of range */
     return ADI_PWM_BAD_PARAMETER;
     }
      
     if(bEnable)
     {
        /* enable PWM channel delay */
        pDevice->pRegs->PWM_CONTROL |= Mask;
     }
     else
     {
        /* disable PWM channel delay */
        pDevice->pRegs->PWM_CONTROL &= ~Mask;
     }

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_ExtSyncEnable
 *          This function enables / disables the input of an external SYNC pulse
 *          The SYNC pulse can be either a synchronous or asynchronous input
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  bSync                 Flag to manage the synchronous/asynchronous mode of the sync pulse.
 *               						- true       Sync pulse is synchronised to the timer.
 *               						- false      Sync pulse is asynchronous
 * @param   [in]  bEnable               Flag to manage the enabling/disabling of external synce.
 *               						- true       Enable external sync.
 *               						- false      Disable external sync.
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */
ADI_PWM_RESULT adi_pwm_ExtSyncEnable(ADI_PWM_HANDLE const hDevice, bool bSync, bool bEnable)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    
    /* register value */
    uint32_t RegValue; 

    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;
    
    /* read register */
    RegValue = pDevice->pRegs->PWM_CONTROL;

    /* determine sync pulse to synchronous or asynchronous with respect to the timer */
    if(bSync)
    {
        /* sync pulse is synchronised to timer */
        RegValue |= (uint32_t)BITM_PWM_CTL_EXTSYNCSEL;
    }
    else
    {
      /* sync pulse is asynchronous with respect to the timer */
        RegValue &= (uint32_t)~(BITM_PWM_CTL_EXTSYNCSEL);
    }


    if(bEnable)
    {
        /* sync pulse generated externally */
        RegValue |= (uint32_t)BITM_PWM_CTL_EXTSYNC;
    }
    else
    {
      /* sync pulse generated internally */
        RegValue &= (uint32_t)~(BITM_PWM_CTL_EXTSYNC);
    }
    
    /* updtae register */
    pDevice->pRegs->PWM_CONTROL |= RegValue;

    /* Return */
    return (Result);

}

/**
 * @brief   adi_pwm_SetIntSyncTimerMode
 *          This function sets the reference timer for the generation of the internal
 *          SYNC pulse output
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eTimer                PWM timer identifier
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetIntSyncTimerMode(ADI_PWM_HANDLE const hDevice, ADI_PWM_TIMER eTimer)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* register value */
    uint32_t RegValue;     
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }

    if(eTimer > ADI_PWM_TIMER4)
    {
        /* parameter out of range */
        return (ADI_PWM_BAD_PARAMETER);

    }
#endif

    pDevice = (ADI_PWM *)hDevice;
    
    /* read register */
    RegValue = pDevice->pRegs->PWM_CONTROL;    
           
    /* clear internal timer refernce bits*/
    RegValue &= (uint32_t)~BITM_PWM_CTL_INTSYNCREF;

    /* shift pulse mode data to the correct field position  */
    /* and update pulse mode bits                           */
     pDevice->pRegs->PWM_CONTROL |= (((uint32_t)eTimer << (uint32_t)BITP_PWM_CTL_INTSYNCREF) | RegValue);

    /* Return */
    return (Result);

}

/**
 * @brief   adi_pwm_OutputEnable
 *          This function disables the output of a PWM channel
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM Channel pair identifier
 * @param   [in]  eChannelOutput        PWM channel pair output identifier, either high or low side
 * @param   [in]  bEnable               Flag to manage the enabling/disabling of the high /low side output.
 *               						- true       Enable output.
 *               						- false      Disable output.
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */
ADI_PWM_RESULT adi_pwm_OutputEnable(ADI_PWM_HANDLE 		const hDevice,
		                            ADI_PWM_CHANNEL 		eChannel,
		                            ADI_PWM_CHANNEL_OUTPUT eChannelOutput,
		                            bool 					bEnable)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits */
    uint32_t Mask;
    /* pointer to PWM register */
    volatile uint32_t *pPWMReg;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;
    
    /* check to see if output is high or low side   */
    /* and set the output side mask accordingly     */
    if(eChannelOutput == ADI_PWM_HIGH_SIDE)
    {
        /* set the high side mask field bit */
        Mask = (uint32_t)BITM_PWM_ACTL_DISHI;
    }    
    else
    {
        /* set the low side mask field bit */
        Mask = (uint32_t)BITM_PWM_ACTL_DISLO;
    }  
      
    /* determine which channel to enable / disable */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set register address for channel A */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHA;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set register address for channel B */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHB;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set register address for channel C */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHC;
    }
    else if(eChannel == ADI_PWM_CHANNEL_D)
    {
        /* set register address for channel D */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHD;
    }
    else
    {
        return (ADI_PWM_BAD_PARAMETER);
    }
    
    /* to enable output write a '0' and to disable a '1'    */
    /* this register has enable bits inverted               */
    if(bEnable)
    {
        /*enable channel output by writing a '1' */
    	*pPWMReg &= ~Mask;
    } 
    else
    {
        /* disable channel output by writing a '0' */
    	*pPWMReg |= Mask;
    }
    
    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_HiLoCrossoverEnable
 *          This function enables / disables the crossover or switching of the outputs from
 *          the high side to the low side and vice versa.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM Channel pair identifier 
 * @param   [in]  bEnable               Flag to manage the enabling/disabling of the crossover.
 *               						- true       The timer assigned to the selected channel is the reference.
 *               						- false      Timer0 is the reference timer.
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE            	PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_HiLoCrossoverEnable(ADI_PWM_HANDLE const  hDevice, ADI_PWM_CHANNEL eChannel, bool bEnable)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits */
    uint32_t Mask;
    /* pointer to PWM register */
    volatile uint32_t *pPWMReg;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel to enable / disable delay */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set register address for channel A */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHA;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set register address for channel B */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHB;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set register address for channel C */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHC;
    }
    else if(eChannel == ADI_PWM_CHANNEL_D)
    {
        /* set register address for channel D */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHD;
    }
    else
    {
        return (ADI_PWM_BAD_PARAMETER);
    }
    
    if(bEnable)
    {
       /* enable PWM channel crossover */
        *pPWMReg |= (uint32_t)BITM_PWM_ACTL_XOVR;
    }     
    else
    {
        /* disable PWM channel crossover */
        *pPWMReg &= (uint32_t)~BITM_PWM_ACTL_XOVR;
    }

    /* Return */
    return (Result);
}


/**
 * @brief   adi_pwm_SetPulseMode
 *          This function sets the type of pulses generated on the outputs of a PWM channel
 *          The pulses can be symmetric or asymmetric about the centre point of the PWM period
 *          Alternatively they can start in the left or right half on the PWM period
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM hannel pair identifier
 * @param   [in]  eChannelOutput        PWM channel pair output identifier the high or low side
 * @param   [in]  ePulse                PWM pulse type
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */
ADI_PWM_RESULT adi_pwm_SetPulseMode(ADI_PWM_HANDLE         const hDevice,
		                            ADI_PWM_CHANNEL        eChannel,
		                            ADI_PWM_CHANNEL_OUTPUT eChannelOutput,
		                            ADI_PWM_PULSE_TYPE     ePulse)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits */
    uint32_t Mask;
    /* field mask position */
    uint32_t MaskPosition;
    /* pointer to PWM register */
    volatile uint32_t *pPWMReg;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;
    
    /* check to see if output is high or low side   */
    /* and set the output side mask accordingly     */
    if(eChannelOutput == ADI_PWM_HIGH_SIDE)
    {
        /* set the high side mask field bit */
        Mask = (uint32_t)BITM_PWM_ACTL_PULSEMODEHI;
        MaskPosition = (uint32_t)BITP_PWM_ACTL_PULSEMODEHI;
    }    
    else
    {
        /* set the low side mask field bit */
        Mask = (uint32_t)BITM_PWM_ACTL_PULSEMODELO;
        MaskPosition = (uint32_t)BITP_PWM_ACTL_PULSEMODELO;
    }     
      
    /* determine which channel to set the pulse mode */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set register address for channel A */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHA;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set register address for channel B */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHB;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set register address for channel C */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHC;
    }
    else if(eChannel == ADI_PWM_CHANNEL_D)
    {
        /* set register address for channel D */
        pPWMReg =  &pDevice->pRegs->PWM_CTRLCHD;
    }
    else
    {
        return (ADI_PWM_BAD_PARAMETER);
    }           
    /* clear pulse mode bits*/
     *pPWMReg &= ~Mask;

    /* shift pulse mode data to the correct field position  */
    /* and update pulse mode bits                           */
     *pPWMReg |= ((uint32_t)ePulse << MaskPosition);
    
    /* Return */
    return (Result);
}


/**
 * @brief   adi_pwm_SetTimerMode
 *          This function sets the reference timer for each PWM channel pair.
 *          The reference timer selection can be either the main reference timer
 *          (ADI_PWM_TIMER0) or the timer assigned to a specific channel pair,
 *          (ADI_PWM_TIMER1, ADI_PWM_TIMER2, ADI_PWM_TIMER3, or ADI_PWM_TIMER 4)
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM Channel pair identifier
 * @param   [in]  bTimer                Flag to manage the selection of Timer0 or Timerx.
 *               						- true       The timer assigned to the selected channel is the reference.
 *               						- false      Timer0 is the reference timer.
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetTimerMode(ADI_PWM_HANDLE const hDevice, ADI_PWM_CHANNEL eChannel, bool bTimer)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM  *pDevice;
    /* mask bits */
    uint32_t Mask;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel to set the reference timer */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set mask for channel A reference timer */
        Mask = (uint32_t)BITM_PWM_CHANCFG_REFTMRA;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set mask for channel B reference timer */
        Mask = (uint32_t)BITM_PWM_CHANCFG_REFTMRB;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set mask for channel C reference timer */
        Mask = (uint32_t)BITM_PWM_CHANCFG_REFTMRC;
    }
    else if(eChannel == ADI_PWM_CHANNEL_D)
    {
        /* set mask for channel D reference timer */
        Mask = (uint32_t)BITM_PWM_CHANCFG_REFTMRD;
    }
    else
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }
    
    if(bTimer)
    {
        /* low side and high side operation independently */
        pDevice->pRegs->PWM_CHANCFG |= Mask;
    }     
    else
    {
        /* low side is the inverse of the high side */
        pDevice->pRegs->PWM_CHANCFG &= ~Mask;
    }
    
    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetLowSideMode
 *          This function sets the mode of operation of the PWM channel low side output
 *          The low side output can be the inversion of the high side or be independent.
 *
 * *param   [in]  hDevice               Handle of the PWM device
 * *param   [in]  eChannel              PWM Channel pair identifier
 * *param   [in]  bMode                 'true' independent low side output
 *                                      'false' low side inverse of the high side
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetLowSideMode(ADI_PWM_HANDLE const hDevice, ADI_PWM_CHANNEL eChannel, bool bMode)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits */
    uint32_t Mask;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel to set the low side mode of operation */
    if(eChannel == ADI_PWM_CHANNEL_A)
    {
        /* set mask for channel A low side operation mode */
        Mask = (uint32_t)BITM_PWM_CHANCFG_MODELSA;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_B)
    {
        /* set mask for channel B low side operation mode */
        Mask = (uint32_t)BITM_PWM_CHANCFG_MODELSB;
    }   
    else if(eChannel == ADI_PWM_CHANNEL_C)
    {
        /* set mask for channel C low side operation mode */
        Mask = (uint32_t)BITM_PWM_CHANCFG_MODELSC;
    }
    else if(eChannel == ADI_PWM_CHANNEL_D)
    {
        /* set mask for channel D low side operation mode */
        Mask = (uint32_t)BITM_PWM_CHANCFG_MODELSD;
    }
    else  
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }
    
    if(bMode)
    {
    	/* low side and high side operation independently */
    	pDevice->pRegs->PWM_CHANCFG |= Mask;
    }     
    else
    {
        /* low side is the inverse of the high side */
        pDevice->pRegs->PWM_CHANCFG &= ~Mask;
    }

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetOutputPolarity
 *          This function sets the polarity of either the high side or low side
 *          output of a PWM channel.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM channel pair identifier
 * @param   [in]  eChannelOutput        PWM channel pair output identifier high or low side
 * @param   [in]  bPolarity             'true' active high, 'false' active low
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetOutputPolarity(ADI_PWM_HANDLE 		const hDevice,
		                                 ADI_PWM_CHANNEL 		eChannel,
		                                 ADI_PWM_CHANNEL_OUTPUT eChannelOutput,
		                                 bool 					bPolarity)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* polarity mask */    
    uint32_t MaskPolarity;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel and output channel to set the polarity parameter */
    if((eChannel == ADI_PWM_CHANNEL_A) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
        /* set polarity mask for channel A high side output */
        MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLAH;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_A) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
       /* set polarity mask for channel A low side output */
        MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLAL;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
        /* set polarity mask for channel B high side output */
        MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLBH;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
    	/* set polarity mask for channel B low side output */
    	MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLBL;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
        /* set polarity mask for channel C high side output */
        MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLCH;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
    	/* set polarity mask for channel C low side output */
        MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLCL;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
        /* set polarity mask for channel D high side output */
        MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLDH;
    }
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
        /* set polarity mask for channel D low side output */
        MaskPolarity = (uint32_t)BITM_PWM_CHANCFG_POLDL;
    }       
    else  
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }   
    if(bPolarity)
    {
        /* active high polarity */
        pDevice->pRegs->PWM_CHANCFG |= MaskPolarity;
    }     
    else
    {
    	/* active low polarity */
    	pDevice->pRegs->PWM_CHANCFG &= ~MaskPolarity;
    }
    /* Return */
    return (Result);
}
/**
  * @brief  adi_pwm_OutputChoppingEnable
 *          This function enables / disables gated chopping of either the high side or low side output of
 *          a PWM channel.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM channel pair identifier
 * @param   [in]  eChannelOutput        PWM channel pair output identifier, either the high or low side PWM output
 * @param   [in]  bEnable               'true' to enable, 'false' to disable
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_OutputChoppingEnable(ADI_PWM_HANDLE 			const hDevice,
											ADI_PWM_CHANNEL 		eChannel,
											ADI_PWM_CHANNEL_OUTPUT  eChannelOutput,
											bool 					bEnable)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits */
    uint32_t Mask;
    
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel and output channel to enable / disable gated chopping */
    if((eChannel == ADI_PWM_CHANNEL_A) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
        /* set mask for channel A high side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPAH;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_A) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
    	/* set mask for channel A low side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPAL;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
    	/* set mask for channel B high side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPBH;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
    	/* set mask for channel B low side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPBL;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
    	/* set mask for channel C high side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPCH;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
    	/* set mask for channel C low side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPCL;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eChannelOutput == ADI_PWM_HIGH_SIDE))
    {
    	/* set mask for channel D high side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPDH;
    }
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eChannelOutput == ADI_PWM_LOW_SIDE))
    {
    	/* set mask for channel D low side output */
        Mask = (uint32_t)BITM_PWM_CHANCFG_ENCHOPDL;
    }   
    else  
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }   
         
    if(bEnable)
    {
      /* enable PWM channel output gated chopping */
        pDevice->pRegs->PWM_CHANCFG |= Mask;
    }     
    else
    {
        /* disable PWM channel output gated chopping */
        pDevice->pRegs->PWM_CHANCFG &= ~Mask;
    }
    
    /* Return */
    return (Result);
}


/**
 * @brief   adi_pwm_TripEnable
 *          This function enables / disables the PWM TRIP source for each channel.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM Channel pair identifier
 * @param   [in]  eTripSource           Identifies the source of the TRIP input
 * @param   [in]  bEnable               Enable TRIP
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */
ADI_PWM_RESULT adi_pwm_SetTripEnable(ADI_PWM_HANDLE 		const hDevice,
									 ADI_PWM_CHANNEL 		eChannel,
									 ADI_PWM_TRIP_SOURCE 	eTripSource,
									 bool 					bEnable)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits for trip source */
    uint32_t MaskTripSource;

        
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel to set the reference timer and which trip source */
    if((eChannel == ADI_PWM_CHANNEL_A) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 source mask for channel A */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN0A;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_A) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 source mask for channel A */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN1A;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 source mask for channel B */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN0B;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 source mask for channel B */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN1B;
    }
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 source mask for channel B */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN0C;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 source mask for channel C */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN1C;
    }
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 source mask for channel D */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN0D;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 source mask for channel D */
        MaskTripSource = (uint32_t)BITM_PWM_TRIPCFG_EN1D;
    }
    else
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }

    if(bEnable)
    {
      /* enable PWM TRIP */
        pDevice->pRegs->PWM_TRIPCFG |= MaskTripSource;
    }     
    else
    {
      /* disable PWM TRIP */
        pDevice->pRegs->PWM_TRIPCFG &= ~MaskTripSource;
    }

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_SetTripMode
 *          This function configures the operation of the PWM TRIP.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 * @param   [in]  eChannel              PWM Channel pair identifier
 * @param   [in]  eTripSource           Identifies the source of the TRIP input
 * @param   [in]  bMode                 Determines how the TRIP is handled when triggered,
 *                                      reports a fault or self-restarts PWM
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_SetTripMode(ADI_PWM_HANDLE 		const hDevice,
		                           ADI_PWM_CHANNEL 		eChannel,
		                           ADI_PWM_TRIP_SOURCE 	eTripSource,
		                           bool 				bMode)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;
    /* mask bits for trip mode */
    uint32_t MaskTripMode;

        
    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* determine which channel to set the reference timer and which trip source */
    if((eChannel == ADI_PWM_CHANNEL_A) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 mode mask for channel A */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE0A;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_A) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 mode mask for channel A */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE1A;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 mode mask for channel B */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE0B;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_B) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 mode mask for channel B */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE1B;
    }
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 mode mask for channel B */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE0C;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_C) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 mode mask for channel C */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE1C;
    }
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eTripSource == ADI_PWM_TRIP0_SRC))
    {
        /* set trip0 mode mask for channel D */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE0D;
    }   
    else if((eChannel == ADI_PWM_CHANNEL_D) && (eTripSource == ADI_PWM_TRIP1_SRC))
    {
        /* set trip1 mode mask for channel D */
        MaskTripMode = (uint32_t)BITM_PWM_TRIPCFG_MODE1D;
    }
    else
    {
        /* field value is out of range */
        return(ADI_PWM_BAD_PARAMETER);
    }

    /* set the trip mode */
    if(bMode)
    {
        /* set trip self restart mode */
        pDevice->pRegs->PWM_TRIPCFG |= MaskTripMode;
    }     
    else
    {
    	/* set trip fault mode */
    	pDevice->pRegs->PWM_TRIPCFG &= ~MaskTripMode;
    }

    /* Return */
    return (Result);
}


/**
 * @brief   adi_pwm_ClearTrip
 *          This function enables the application to recover from a trip condition
 *          by clear the trip status register and re-enabling trip interrupts.
 *
 * @param   [in]  hDevice               Handle of the PWM device
 *
 * @return  ADI_PWM_SUCCESS             Successful function call
 * @return  ADI_PWM_FAILURE              PWM device failure
 *
 */

ADI_PWM_RESULT adi_pwm_ClearTrip(ADI_PWM_HANDLE const hDevice)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;
    /* PWM Device */
    ADI_PWM     *pDevice;

    /* debug build only */
#if defined (ADI_DEV_DEBUG)

    /* validate PWM device handle */
    Result = ValidatePwmHandle(hDevice);

    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;

    /* clear the status register of all trip fault events */
    pDevice->pRegs->PWM_STATUS = (uint32_t)ADI_PWM_TRIP_ALL;

    /* enable PWM TRIP0 and TRIP1 interrupts */
    pDevice->pRegs->PWM_IMASK |= (uint32_t)(ENUM_PWM_STAT_TRIP0 | ENUM_PWM_STAT_TRIP0);

    /* Return */
    return (Result);
}

/**
 * @brief   adi_pwm_GetStatusRegister
 *          This function gets the contents of the PWM status register.
 *
 * @param   [in]  hDevice           Handle of the PWM device
 * @param   [out] pStatus           Pointer to store the value of the status register
 *
 * @return  ADI_PWM_SUCCESS        Successful function call
 * @return  ADI_PWM_FAILURE        PWM device failure
 *
 */
ADI_PWM_RESULT adi_pwm_GetStatusRegister(ADI_PWM_HANDLE hDevice,
                                         uint32_t       *pStatus)
{
    /* assume a return code */
    ADI_PWM_RESULT Result = ADI_PWM_SUCCESS;

    /* Pointer to device instance */
    ADI_PWM *pDevice;

/* debug build only */
#ifdef ADI_DEBUG
    /* validate device handle */
    Result = ValidatePwmHandle(hDevice);
    /* if valid handle */
    if (Result != ADI_PWM_SUCCESS)
    {
        return(ADI_PWM_FAILURE);
    }
#endif

    pDevice = (ADI_PWM *)hDevice;
    /* get the status register. */
    *pStatus = pDevice->pRegs->PWM_STATUS;
    /* Return result */
    return Result;
}

/*********************************************************************

    Function: ValidatePwmHandle

        Validates PWM device handle

    Parameters:
        hDevice - Device handle to validate

    Return:
        TBD

*********************************************************************/
static ADI_PWM_RESULT ValidatePwmHandle (ADI_PWM_HANDLE const hDevice)
{
    /* index counter */
    uint8_t  i;
    /* return code */
    ADI_PWM_RESULT Result = ADI_PWM_INVALID_HANDLE;

    /* all pwm device instances) */
    for (i = 0u; i < (uint8_t)ADI_PWM_NUM_DEVICES; i++)
    {
        if (hDevice == (ADI_PWM_HANDLE)&aPwmDevice[i])
        {
            Result = ADI_PWM_SUCCESS; /* Found a valid device */
            break;
        }
    }

    /* Return */
    return (Result);
}

/********************** PWM Interrupt handlers **********************/


/*********************************************************************

    Function:       adi_pwm_TripHandler

    Description:    Processes PWM Trip Event interrupts

*********************************************************************/
static void Pwm_Trip_Interrupt_Handler(uint32_t SID, void *pCBParam)
{
    /* status register value */
    uint32_t     Status;
    uint32_t i;

    /* Pointer to device instance */
    ADI_PWM *pDevice = (ADI_PWM *)pCBParam;

#ifdef ADI_DEBUG
    /*  check device */
    if(pDevice == NULL)
    {
        return;
    }
#endif /* ADI_DEBUG */

    /* Get the status register value */
    Status = pDevice->pRegs->PWM_STATUS;

     /* check each status field bit */
    for (i = 0u; i < (uint32_t)ADI_PWM_NUMTRIPEVENTS; i++)
    {
    	/* mask the staus value with each status error field */
        if((Status & (uint32_t)aStatusTripEventFields[i]) != 0u)
        {
            if(pDevice->pDevData->pfCallback != NULL)
            {
        	    (pDevice->pDevData->pfCallback)(pDevice->pDevData->pCBParam, (uint32_t)aStatusTripEventFields[i], NULL);
            }
        }
    }

    /* if the interrupt is an external trip, disable them, because application needs handle PWM recovery */
    if((Status & ENUM_PWM_STAT_TRIP0) || (Status & ENUM_PWM_STAT_TRIP1))
    {
        /* disable PWM TRIP0 and TRIP1 interrupts and do not clear them, application to handle this */
        pDevice->pRegs->PWM_IMASK &= ~(uint32_t)(ENUM_PWM_STAT_TRIP0 | ENUM_PWM_STAT_TRIP0);
    }
    else
    {
        /* clear the status register of all trip fault events */
        pDevice->pRegs->PWM_STATUS = (uint32_t)ADI_PWM_TRIP_ALL;
    }

    return;
}


/*********************************************************************

    Function:       adi_pwm_SyncHandler

    Description:    Processes PWM Sync Event interrupts

*********************************************************************/
static void Pwm_Sync_Interrupt_Handler(uint32_t SID, void *pCBParam)
{

    /* status register value */
    uint32_t     Status;
    uint32_t i;

    /* Pointer to device instance */
    ADI_PWM *pDevice = (ADI_PWM *)pCBParam;

#ifdef ADI_DEBUG
    /*  check device */
    if(pDevice == NULL)
    {
        return;
    }
#endif /* ADI_DEBUG */

    /* Get the status register value */
    Status = pDevice->pRegs->PWM_STATUS;

     /* check each status field bit */
    for (i = 0u; i < (uint32_t)ADI_PWM_NUMPERIODEVENTS; i++)
    {
    	/* mask the staus value with each status error field */
        if((Status & (uint32_t)aStatusPeriodEventFields[i]) != 0u)
        {
            if(pDevice->pDevData->pfCallback != NULL)
            {            
        	    (pDevice->pDevData->pfCallback)(pDevice->pDevData->pCBParam, (uint32_t)aStatusPeriodEventFields[i], NULL);
        	}    
        }
    }
    /* clear the status register of all trip field events */
    pDevice->pRegs->PWM_STATUS = (uint32_t)ADI_PWM_PERIOD_ALL;

    return;
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */
