/* $Id: adi_pwm_data_bf6xx.c 18384 2013-04-04 21:17:34Z mteris $ */
/*********************************************************************************

 Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

 This software is proprietary and confidential.  By using this software you agree
 to the terms of the associated Analog Devices License Agreement.

 *********************************************************************************/
/*!
 * @file      adi_pwm_data_bf6x.c
 *
 * @brief     PWM service global API implementation
 *
 * @details
 *            This is a source file which contains the non-exposed definitions of
 *            the adi_pwm_bf6xx.c service.
 */

/** @addtogroup PWM_Service_BF6xx PWM Service
 *  @{
 */
 
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif

#include <adi_types.h>
#include <stddef.h>
#include <string.h>
#include <sys/platform.h> 
#include <sys/exception.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#if defined(__ADSPBF609_FAMILY__)
#include <services/int/adi_sec.h>
#endif

#include <adi_osal.h>
#include <services\int\adi_int.h>
#include <services\pwm\adi_pwm.h> 

/* PWM driver memory parameters */
typedef struct 
{
	ADI_CALLBACK pfCallback; 							        /* Application supplied callback function               */
	void* pCBParam; 									        /* Callback parm passed back in callbacks               */
} ADI_PWM_DEV_DATA;

/* PWM Register structure */
typedef volatile struct
{
    volatile uint32_t    PWM_CONTROL;                           /* PWM control register                                 */
    volatile uint32_t    PWM_CHANCFG;                           /* PWM control register                                 */
    volatile uint32_t    PWM_TRIPCFG;                           /* PWM trip interrupt configuration                     */
    volatile uint32_t    PWM_STATUS;                            /* PWM status                                           */
    volatile uint32_t    PWM_IMASK;                             /* PWM mask                                             */
    volatile uint32_t    PWM_ILAT;                              /* PWM latch                                            */
    volatile uint32_t    PWM_CHOPCFG;                           /* PWM chop frequency configuration                     */
    volatile uint32_t    PWM_DT;                                /* PWM deadtime                                         */
    volatile uint32_t    PWM_SYNCWT;                            /* PWM sync pulse width                                 */
    volatile uint32_t    PWM_TM0;                               /* PWM timer0 period                                    */
    volatile uint32_t    PWM_TM1;                               /* PWM timer1 period                                    */
    volatile uint32_t    PWM_TM2;                               /* PWM timer2 period                                    */
    volatile uint32_t    PWM_TM3;                               /* PWM timer3 period                                    */
    volatile uint32_t    PWM_TM4;                               /* PWM timer4 period                                    */
    volatile uint32_t    PWM_DELAYA;                            /* PWM channel A delay                                  */
    volatile uint32_t    PWM_DELAYB;                            /* PWM channel B delay                                  */
    volatile uint32_t    PWM_DELAYC;                            /* PWM channel C delay                                  */
    volatile uint32_t    PWM_DELAYD;                            /* PWM channel D delay                                  */
    volatile uint32_t    PWM_CTRLCHA;                           /* PWM channel Control                                  */
    volatile uint32_t    PWM_AH0;                               /* PWM channel A high side duty0 cycle                  */
    volatile uint32_t    PWM_AH1;                               /* PWM channel A high side duty1  cycle                 */
    volatile uint32_t    PWM_HPAH0;                             /* PWM channel A high precision edge0                   */
    volatile uint32_t    PWM_HPAH1;                             /* PWM channel A high precision edge1                   */
    volatile uint32_t    PWM_AL0;                               /* PWM channel A low side duty0 cycle                   */
    volatile uint32_t    PWM_AL1;                               /* PWM channel A low side duty1 cycle                   */
#if defined(ADI_ADSP_CM40Z)
    volatile uint32_t    PWM_HPAL0;                             /* PWM channel A low side high precision edge0                   */
    volatile uint32_t    PWM_HPAL1;                             /* PWM channel A low side high precision edge1                   */
#endif
    volatile uint32_t    PWM_CTRLCHB;                           /* PWM channel B control                                */
    volatile uint32_t    PWM_BH0;                               /* PWM channel B high side duty0 cycle                  */
    volatile uint32_t    PWM_BH1;                               /* PWM channel B high side duty1 cycle                  */
    volatile uint32_t    PWM_HPBH0;                             /* PWM channel B high precision edge0                   */
    volatile uint32_t    PWM_HPBH1;                             /* PWM channel B high precision edge1                   */
    volatile uint32_t    PWM_BL0;                               /* PWM channel B low side duty0 cycle                   */
    volatile uint32_t    PWM_BL1;                               /* PWM channel B low side duty1 cycle                   */
#if defined(ADI_ADSP_CM40Z)
    volatile uint32_t    PWM_HPBL0;                             /* PWM channel B low side high precision edge0                   */
    volatile uint32_t    PWM_HPBL1;                             /* PWM channel B low side high precision edge1                   */
#endif
    volatile uint32_t    PWM_CTRLCHC;                           /* PWM channel C control                                */
    volatile uint32_t    PWM_CH0;                               /* PWM channel C high side duty0 cycle                  */
    volatile uint32_t    PWM_CH1;                               /* PWM channel C high side duty1 cycle                  */
    volatile uint32_t    PWM_HPCH0;                             /* PWM channel C high precision edge0                   */
    volatile uint32_t    PWM_HPCH1;                             /* PWM channel C high precision edge1                   */
    volatile uint32_t    PWM_CL0;                               /* PWM channel C low side duty0 cycle                   */
    volatile uint32_t    PWM_CL1;                               /* PWM channel C low side duty1 cycle                   */
#if defined(ADI_ADSP_CM40Z)
    volatile uint32_t    PWM_HPCL0;                             /* PWM channel C low side high precision edge0                   */
    volatile uint32_t    PWM_HPCL1;                             /* PWM channel C low side high precision edge1                   */
#endif
    volatile uint32_t    PWM_CTRLCHD;                           /* PWM channel D control                                */
    volatile uint32_t    PWM_DH0;                               /* PWM channel D high side duty0 cycle                  */
    volatile uint32_t    PWM_DH1;                               /* PWM channel D high side duty1 cycle                  */
    volatile uint32_t    PWM_HPDH0;                             /* PWM channel D high precision edge0                   */
    volatile uint32_t    PWM_HPDH1;                             /* PWM channel D high precision edge1                   */
    volatile uint32_t    PWM_DL0;                               /* PWM channel D low side duty0 cycle                   */
    volatile uint32_t    PWM_DL1;                               /* PWM channel D low side duty1 cycle                   */
#if defined(ADI_ADSP_CM40Z)
    volatile uint32_t    PWM_HPDL0;                             /* PWM channel D low side high precision edge0                   */
    volatile uint32_t    PWM_HPDL1;                             /* PWM channel D low side high precision edge1                   */
#endif
} ADI_PWM_REGS;

/* PWM Device Structure */
typedef struct _ADI_PWM_
{
	const uint32_t                  PwmSyncSID;                 /* PWM status interrupt ID                              */
	const uint32_t                  PwmTripSID;                 /* PWM status interrupt ID                              */	
	ADI_PWM_REGS *const             pRegs;                      /* Base Address of Registers                            */
	ADI_PWM_DEV_DATA                *pDevData;                  /* PWM driver data                                      */
} ADI_PWM;

/* ADSP-BF542/544/547/548/549 instances */
#if defined(__ADSPBF609_FAMILY__)
static ADI_PWM aPwmDevice[] =
{
    /* PWM 0 */
    {
    	(uint32_t)INTR_PWM0_SYNC,                               /* interrupt ID for PWM0 trip                          */
    	(uint32_t)INTR_PWM0_TRIP,                               /* interrupt ID for PWM0 sync                          */
        (ADI_PWM_REGS *)(pREG_PWM0_CTL),                        /* base Address of PWM0 Registers                      */
        NULL                                                    /* null pointer                                        */
    },
    /* PWM 1 */
    {
    	(uint32_t)INTR_PWM1_SYNC,                               /* interrupt ID for PWM1 trip                          */
    	(uint32_t)INTR_PWM1_TRIP,                               /* interrupt ID for PWM1 sync                          */
        (ADI_PWM_REGS *)(pREG_PWM1_CTL),                        /* base Address of PWM1 Registers                      */
        NULL                                                    /* null pointer                                        */
    },
};
#elif defined(ADI_ADSP_CM40Z)
static ADI_PWM aPwmDevice[] =
{
    /* PWM 0 */
    {
    	(uint32_t)INTR_PWM0_SYNC,                               /* interrupt ID for PWM0 trip                          */
    	(uint32_t)INTR_PWM0_TRIP,                               /* interrupt ID for PWM0 sync                          */
        (ADI_PWM_REGS *)(pREG_PWM0_CTL),                        /* base Address of PWM0 Registers                      */
        NULL                                                    /* null pointer                                        */
    },
    /* PWM 1 */
    {
    	(uint32_t)INTR_PWM1_SYNC,                               /* interrupt ID for PWM1 trip                          */
    	(uint32_t)INTR_PWM1_TRIP,                               /* interrupt ID for PWM1 sync                          */
        (ADI_PWM_REGS *)(pREG_PWM1_CTL),                        /* base Address of PWM1 Registers                      */
        NULL                                                    /* null pointer                                        */
    },
    /* PWM 2 */
    {
    	(uint32_t)INTR_PWM2_SYNC,                               /* interrupt ID for PWM1 trip                          */
    	(uint32_t)INTR_PWM2_TRIP,                               /* interrupt ID for PWM1 sync                          */
        (ADI_PWM_REGS *)(pREG_PWM2_CTL),                        /* base Address of PWM1 Registers                      */
        NULL                                                    /* null pointer                                        */
    },
};
#else  /* device is not supported */
#error "PWM is not supported "
#endif

/* status register */
ADI_PWM_EVENT aStatusTripEventFields[] =
{
	ADI_PWM_TRIP0,
	ADI_PWM_TRIP1,
	ADI_PWM_HW_ERR_TRIP_FAULTA,
	ADI_PWM_HW_ERR_TRIP_FAULTB,
	ADI_PWM_HW_ERR_TRIP_FAULTC,
	ADI_PWM_HW_ERR_TRIP_FAULTD
};

#define ADI_PWM_NUMTRIPEVENTS (sizeof(aStatusTripEventFields)/sizeof(uint32_t))
/* status register */
ADI_PWM_EVENT aStatusPeriodEventFields[] =
{
	ADI_PWM_TMR0_BOUNDARY,
	ADI_PWM_TMR1_BOUNDARY,
	ADI_PWM_TMR2_BOUNDARY,
	ADI_PWM_TMR3_BOUNDARY,
	ADI_PWM_TMR4_BOUNDARY
};

#define ADI_PWM_NUMPERIODEVENTS (sizeof(aStatusPeriodEventFields)/sizeof(uint32_t))

/* Number of PWM devices */
#define ADI_PWM_NUM_DEVICES (sizeof(aPwmDevice)/sizeof(ADI_PWM))
