/******************************************************************************

Copyright (c) 2011 Analog Devices.  All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
 *******************************************************************************
 *
 * @file:    adi_pwr_bf60x.c
 * @brief:   Dynamic Power Management service implementation
 * @version: $Revision: 22793 $
 * @date:    $Date: 2013-08-26 12:17:34 -0400 (Mon, 26 Aug 2013) $
 *
 ******************************************************************************/

/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"(advisory) Sections of code should not be commented out.")
#pragma diag(suppress:misra_rule_8_5:"There shall be no definitions of objects or functions in a header file")
#pragma diag(suppress:misra_rule_11_3:"(advisory) A cast should not be performed between a pointer type and an integral type.")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#endif /* _MISRA_RULES */

/** @addtogroup Power_Service_BF60x Dynamic Power Management Service
 *  @{
 *
 */

#if defined(__ADSPBLACKFIN__)
#pragma file_attr("prefersMem=internal")
#pragma file_attr("prefersMemNum=30")
#include <ccblkfn.h>
#include <services/int/adi_sec.h>
#endif

#include <services/pwr/adi_pwr.h>
#include <adi_osal.h>
#include <math.h>

#if defined(USE_SYSCTRL)
#define CGU0_DIV_REGISTER    (BITM_ROM_SYSCTRL_CGU_DIV)
#define CGU0_CTL_REGISTER    (BITM_ROM_SYSCTRL_CGU_CTL)
#define CGU0_STAT_REGISTER   (BITM_ROM_SYSCTRL_CGU_STAT)
#else
#define CGU0_DIV_REGISTER    (0x01u)
#define CGU0_CTL_REGISTER    (0x02u)
#define CGU0_STAT_REGISTER   (0x04u)
#endif

/* maximum number of listeners that get notified about power service events */
#define ADI_PWR_MAX_LISTENERS      (16u)

/******************************************************
 Internal data structures:
 *******************************************************/

/*
 * ADI_PWR_PLL_REGISTER_STRUCT
 *
 * Internal device register data
 */
typedef struct {
	uint32_t cguctl;
	uint32_t cgudiv;
	uint32_t cgustat;
	uint32_t clkoutsel;
} ADI_PWR_PLL_REGISTER_STRUCT;

/*
 * ADI_PWR_SERVICE_STRUCT
 *
 * Internal device data
 */
typedef struct {
	bool  bInitialized;
	uint32_t clkin;
	uint32_t cclkMax;
	uint32_t sclkMax;
	uint32_t vcoMin;
	ADI_PWR_PLL_REGISTER_STRUCT reg;
	ADI_CALLBACK  pwrCallback[ADI_PWR_MAX_LISTENERS];
} ADI_PWR_SERVICE_STRUCT;

/* alias for the actual device structure */
typedef ADI_PWR_SERVICE_STRUCT ADI_PWR_SERVICE;

static ADI_PWR_SERVICE Power = {0};

/*
 * ADI_PWR_CLK_STRUCT
 *
 * Internal device data
 */
typedef struct {
	uint32_t cclk0;
	uint32_t cclk1;
	uint32_t syscclk0;
	uint32_t sclk0;
	uint32_t sclk1;
	uint32_t dlk;
	uint32_t outclk;
} ADI_PWR_CLK_STRUCT;

#define SET (1u)
#define CLEAR (0u)

/******************************************************
 Internal functions:
 *******************************************************/
static void ReadRegister(uint32_t regflags);
static bool WriteRegister(uint32_t regflags);
static uint32_t ConvertFromMSEL(uint32_t msel);
static uint32_t GetFieldPosition(uint32_t mask);
static uint32_t ConvertSEL(const uint32_t sel, const uint32_t max);
static void GetFreq(ADI_PWR_CLK_STRUCT *clk);
static void NotifyListeners(int32_t event, uint32_t value);
static void WaitForLock(void);

#if defined(USE_SYSCTRL)
static void ReadRegister(uint32_t regflags)
{
	uint32_t flags = (uint32_t)regflags;
	STRUCT_ROM_SYSCTRL read;

	flags |= (uint32_t)BITM_ROM_SYSCTRL_CGU_READ;

	rom_SysControl(flags, &read, (void*)0);

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_CTL) != 0u) {
		Power.reg.cguctl = read.ulCGU_CTL;
	}

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_DIV) != 0u) {
		Power.reg.cgudiv = read.ulCGU_DIV;
	}

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_STAT) != 0u) {
		Power.reg.cgustat = read.ulCGU_STAT;
	}

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_CLKOUTSEL) != 0u) {
		Power.reg.clkoutsel = read.ulCGU_CLKOUTSEL;
	}
}

static bool WriteRegister(uint32_t regflags)
{
	STRUCT_ROM_SYSCTRL write;
	uint32_t value;
	uint32_t flags = (uint32_t)regflags;
	bool bError = false;

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_CTL) != 0u) {
		write.ulCGU_CTL = Power.reg.cguctl;
	}

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_DIV) != 0u) {
		write.ulCGU_DIV = Power.reg.cgudiv;
	}

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_STAT) != 0u) {
		write.ulCGU_STAT = Power.reg.cgustat;
	}

	if ((flags & (uint32_t)BITM_ROM_SYSCTRL_CGU_CLKOUTSEL) != 0u) {
		write.ulCGU_CLKOUTSEL = Power.reg.clkoutsel;
	}

	flags |= (uint32_t)BITM_ROM_SYSCTRL_CGU_WRITE;

	rom_SysControl(flags, &write, (void*)0);

	/* check for errors in CGU_STAT */
	value = *pREG_CGU0_STAT & (uint32_t)(ADI_PWR_STAT_ADDR_ERR|ADI_PWR_STAT_LOCK_WR_ERR|
		ADI_PWR_STAT_READ_BUSY_ERR|ADI_PWR_STAT_PLL_CONFIG_ERR);
	if (value != 0u)
	{
		bError = true;
	}

	return bError;
}
#else /* no SYSCTRL */
static void ReadRegister(uint32_t regflags)
{
    if ((regflags & CGU0_DIV_REGISTER) != 0u) {
        Power.reg.cgudiv = *pREG_CGU0_DIV;
    }

    if ((regflags & CGU0_CTL_REGISTER) != 0u) {
        Power.reg.cguctl = *pREG_CGU0_CTL;
    }

    if ((regflags & CGU0_STAT_REGISTER) != 0u) {
        Power.reg.cgustat = *pREG_CGU0_STAT;
    }    
}

static bool WriteRegister(uint32_t regflags)
{
    if ((regflags & CGU0_DIV_REGISTER) != 0u) {
    	*pREG_CGU0_DIV = Power.reg.cgudiv;
    }

    if ((regflags & CGU0_CTL_REGISTER) != 0u) {
        *pREG_CGU0_CTL = Power.reg.cguctl;
    }

    return false;
}
#endif /* USE_SYSCTRL */


static uint32_t GetFieldPosition(uint32_t mask)
{
	/* field position is first non-zero bit */
	uint32_t pos;

	for (pos = 0u; pos < 32u; pos++) {
		if ((mask & (uint32_t)((uint32_t)1 << pos)) != 0u) {
			return pos;
		}
	}
	return 0u;
}

/* Return the divider based on the SEL value */
static uint32_t ConvertSEL(const uint32_t sel, const uint32_t max)
{
	uint32_t divider;

	if ((sel == 0u) || (sel >=max))
	{
		divider = max;
	}
	else
	{
		divider = sel;
	}

	return divider;
}

/* Return the multiplier based on the MSEL value */
static uint32_t ConvertFromMSEL(uint32_t msel)
{
	uint32_t mult;

	if ((msel == 0u) || (msel >= MSELMAX))
	{
		mult = MSELMAX;
	}
	else
	{
		mult = msel;
	}

	return mult;
}

/* Calculate the CCLK and SCLK frequencies. */
static void GetFreq(ADI_PWR_CLK_STRUCT *clk)
{
	ADI_PWR_RESULT result;
	uint32_t pllen;
	uint32_t bypass;
	uint32_t msel;
	uint32_t df;
	uint32_t syssel;
	uint32_t csel;
	uint32_t s0sel;
	uint32_t s1sel;
	uint32_t dsel;
	uint32_t osel;
	uint32_t cselFactor;
	uint32_t mselFactor;
	uint32_t s0selFactor;
	uint32_t s1selFactor;
	uint32_t dselFactor;
	uint32_t oselFactor;
	uint32_t sysselFactor;
	uint32_t fvco;

    /* If PLL not used CCLK and SCLK are CLKIN */
	result = adi_pwr_GetClkStatusRegister(ADI_PWR_CLK_STAT_PLL_EN, &pllen);
	result = adi_pwr_GetClkStatusRegister(ADI_PWR_CLK_STAT_PLL_BYPASS, &bypass);
	if ((pllen == 0u) || (bypass == 1u)) {
		clk->cclk0 = Power.clkin;
		clk->cclk1 = Power.clkin;
		clk->syscclk0 = Power.clkin;
		clk->sclk0 = Power.clkin;
		clk->sclk1 = Power.clkin;
		clk->dlk = Power.clkin;
		clk->outclk = Power.clkin;
	}
    else
    {
        /* If PLL is used, calculate  CCLK and SCLK using and PLL_CTL and PLL_DIV settings */
    	result = adi_pwr_GetClkControlRegister(ADI_PWR_CLK_CTL_DF, &df);

    	result = adi_pwr_GetClkControlRegister(ADI_PWR_CLK_CTL_MSEL, &msel);
    	mselFactor = ConvertFromMSEL(msel);

    	result = adi_pwr_GetClkDivideRegister(ADI_PWR_CLK_DIV_SYSSEL, &syssel);
    	sysselFactor = ConvertSEL(syssel, SYSSELMAX);

    	result = adi_pwr_GetClkDivideRegister(ADI_PWR_CLK_DIV_CSEL, &csel);
    	cselFactor = ConvertSEL(csel, CSELMAX);

    	result = adi_pwr_GetClkDivideRegister(ADI_PWR_CLK_DIV_S0SEL, &s0sel);
   		s0selFactor = ConvertSEL(s0sel, SSELMAX);

    	result = adi_pwr_GetClkDivideRegister(ADI_PWR_CLK_DIV_S1SEL, &s1sel);
   		s1selFactor = ConvertSEL(s1sel, SSELMAX);

    	result = adi_pwr_GetClkDivideRegister(ADI_PWR_CLK_DIV_DSEL, &dsel);
   		dselFactor = ConvertSEL(dsel, DSELMAX);

    	result = adi_pwr_GetClkDivideRegister(ADI_PWR_CLK_DIV_OSEL, &osel);
   		oselFactor = ConvertSEL(osel, OSELMAX);

		fvco = (Power.clkin/(df+1u))*mselFactor;

		clk->cclk0 = fvco/cselFactor;
		clk->cclk1 = fvco/cselFactor;
		clk->syscclk0 = fvco/syssel;
		clk->sclk0 = clk->syscclk0/s0selFactor;
		clk->sclk1 = clk->syscclk0/s1selFactor;
		clk->dlk = fvco/dsel;
		clk->outclk = fvco/osel;

    }
}

static void NotifyListeners(int32_t event, uint32_t value)
{
	ADI_CALLBACK  *pCallbackEntry;    /* callback entry */
	uint32_t i;

	for (i=0u; i<ADI_PWR_MAX_LISTENERS; i++) {
		pCallbackEntry = &Power.pwrCallback[i];
		if ((*pCallbackEntry) != (void*)0u) {
            /* execute the callback */
            (*pCallbackEntry)((void*)0, (uint32_t)event, (void*)&value);
		}
	}
}

/*
 * Poll clock status for PLL lock and clock alignment 
 */
static void WaitForLock(void)
{
#if !defined(USE_SYSCTRL)
  ADI_PWR_RESULT result;
  uint32_t pllLock = 0u;
  uint32_t clkAlign = 1u;
  uint32_t pllErr = 0u;
  
#if defined (ADI_ADSP_CM40Z) || defined (__ADSPBF707_FAMILY__)
  /* polling loop */
  while ((pllLock == 0u) || (clkAlign == 1u))
  {
    result = adi_pwr_GetClkStatusRegister(ADI_PWR_CLK_STAT_PLL_LOCKED, &pllLock);
    result = adi_pwr_GetClkStatusRegister(ADI_PWR_CLK_STAT_CLK_ALIGN, &clkAlign);
  }
#else
  /* polling loop */
  while (((pllLock == 0u) || (clkAlign == 1u)) && (pllErr == 0u))
  {
    result = adi_pwr_GetClkStatusRegister(ADI_PWR_CLK_STAT_PLL_LOCKED, &pllLock);
    result = adi_pwr_GetClkStatusRegister(ADI_PWR_CLK_STAT_CLK_ALIGN, &clkAlign);
    result = adi_pwr_GetClkStatusRegister(ADI_PWR_CLK_STAT_PLL_LOCK_ERR, &pllErr);
  }  
#endif
  
#endif  /* USE_SYSCTRL */
}

/******************************************************
 API implementation:
 *******************************************************/

/**
 * @brief Initialize the dynamic power management service.
 *
 * The external clock (CLKIN) is provided by hardware external to the
 * processor.  You must obtain the CLKIN value from the hardware documentation.
 * The core clock, system clock and VCO limit values are obtained from the
 * processor data sheet.  You must use the limits for the processor type
 * and grade that you are using.
 *
 * This API must be called before using other power service API's.
 *
 * @param [in]  fclkin     The external input clock frequency (Hz)
 * @param [in]  fcclkMax   The maximum core clock frequency (Hz)
 * @param [in]  fsclkMax   The maximum system clock frequency (Hz)
 * @param [in]  fvcoMin    The minimum VCO clock frequency (Hz)
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS               Successfully initialized the power service.
 *              - #ADI_PWR_FAILURE               Failed to initialize the power service.
 *
 * @sa adi_pwr_SetFreq()
 */
ADI_PWR_RESULT adi_pwr_Init(
	uint32_t   const fclkin,
	uint32_t   const fcclkMax,
	uint32_t   const fsclkMax,
	uint32_t   const fvcoMin)
{
	Power.clkin = fclkin;
	Power.cclkMax = fcclkMax;
	Power.sclkMax = fsclkMax;
	Power.vcoMin = fvcoMin;

	Power.bInitialized = true;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief  Set the core clock (CCLK) and system clock (SYSCLK) frequencies.
 *
 * The processor VCO (DF and MSEL scaling) and clock dividers (SYSSEL and CSEL)
 * will be adjusted to obtain the desired frequency.  Only valid frequency combinations
 * will be set.  A valid combination must have a common multiple between the core,
 * system and clkin frequencies.  If no common multiple is found or if the PLL limits
 * are exceeded this function will return a failure result.
 *
 * The core clock value must always be larger than the system clock value.
 *
 * @param [in]  cclk        The Core Clock frequency (Hz)
 * @param [in]  sclk        The System Clock frequency (Hz)
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS               Successfully set the frequencies.
 *              - #ADI_PWR_NOT_INITIALIZED       The power service is not initialized.
 *              - #ADI_PWR_INVALID_FREQUENCY     The desired frequency values could not be set.
 *
 * @sa adi_pwr_GetCoreFreq()
 * @sa adi_pwr_GetSystemFreq()
 */
ADI_PWR_RESULT adi_pwr_SetFreq(
	uint32_t   const cclk,
	uint32_t   const sclk)
{
	uint32_t df;
	uint32_t msel;
	uint32_t testcsel;
	uint32_t csel;
	uint32_t ssel;

	bool bCSELset;
    bool bError;

	uint32_t factor;
	uint32_t regValue;
    
   	ADI_PWR_RESULT result;
    
	extern volatile int _Processor_cycles_per_sec;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}

	if (cclk > Power.cclkMax) {
		return ADI_PWR_INVALID_FREQUENCY;
	}

	if (sclk > Power.sclkMax) {
		return ADI_PWR_INVALID_FREQUENCY;
	}

	if (cclk < sclk) {
		/* cclk must be >= sclk */
		return ADI_PWR_INVALID_FREQUENCY;
    }
#endif

    df = 0u;
    factor = 0u;
    csel = 0u;
    bCSELset = false;
    
    /* find the least common multiple */
	for (testcsel=CSELMIN; testcsel<CSELMAX; testcsel++)
    {
        factor = cclk * testcsel;
        if (((factor % sclk) == 0u) && ((factor % Power.clkin) == 0u))
        {
            bCSELset = true;
            csel = testcsel;
            df = 0u;
        }
        else if (((factor % sclk) == 0u) && ((factor % (Power.clkin / 2u)) == 0u))
        {
            bCSELset = true;
            csel = testcsel;
            df = 1u;
        }
        else
        {
            bCSELset = false;
        }

        if (bCSELset == true)
        {
        	break;
        }
    }
    
    if ((bCSELset == false) || (csel < CSELMIN) || (csel > CSELMAX))
    {
		return ADI_PWR_INVALID_FREQUENCY;
    }
    
    /* check limits */
    if ((factor < Power.vcoMin) || (factor > (ADI_PWR_PLLMAX * 1000000u)))
    {
		return ADI_PWR_INVALID_FREQUENCY;        
    }

    /* we have PLL, determine SSEL*/
    ssel = factor / sclk;

    if ((ssel < SYSSELMIN) || (ssel > SYSSELMAX))
    {
		return ADI_PWR_INVALID_FREQUENCY;
    }
       
    /* determine MSEL */
    msel = factor / (Power.clkin / (df+1u));

    if ((msel < MSELMIN) || (msel > MSELMAX))
    {
		return ADI_PWR_INVALID_FREQUENCY;
    }
    
    NotifyListeners(ADI_PWR_EVENT_FREQ_PRE_CHANGE, sclk);

	/* program the settings */
  
    WaitForLock();
    
    adi_osal_EnterCriticalRegion();

	ReadRegister(CGU0_DIV_REGISTER);

	regValue = Power.reg.cgudiv;
    regValue &= (uint32_t)~ADI_PWR_CLK_DIV_CSEL;
	regValue |= (uint32_t)(csel << BITP_CGU_DIV_CSEL) & (uint32_t)ADI_PWR_CLK_DIV_CSEL;

    regValue &= (uint32_t)~ADI_PWR_CLK_DIV_SYSSEL;
	regValue |= (uint32_t)(ssel << BITP_CGU_DIV_SYSSEL) & (uint32_t)ADI_PWR_CLK_DIV_SYSSEL;

    regValue &= (uint32_t)~ADI_PWR_CLK_DIV_UPDT;
    
	Power.reg.cgudiv = regValue;

	bError = WriteRegister(CGU0_DIV_REGISTER);

    /* control reg */
   	ReadRegister(CGU0_CTL_REGISTER);

	regValue = Power.reg.cguctl;
    regValue &= (uint32_t)~ADI_PWR_CLK_CTL_DF;
	regValue |= (uint32_t)(df << BITP_CGU_CTL_DF) & (uint32_t)ADI_PWR_CLK_CTL_DF;

    regValue &= (uint32_t)~ADI_PWR_CLK_CTL_MSEL;
	regValue |= (uint32_t)(msel << BITP_CGU_CTL_MSEL) & (uint32_t)ADI_PWR_CLK_CTL_MSEL;

    regValue &= (uint32_t)~ADI_PWR_CLK_CTL_WFI;
    
	Power.reg.cguctl = regValue;

	bError = WriteRegister(CGU0_CTL_REGISTER);

    adi_osal_ExitCriticalRegion();

    WaitForLock();
        
	_Processor_cycles_per_sec = (int)cclk;

	NotifyListeners(ADI_PWR_EVENT_FREQ_POST_CHANGE, sclk);

   	return ADI_PWR_SUCCESS;
}

/**
 * @brief  Calculate the core clock frequency.
 *
 * The frequency calculation is based on CLKIN, VCO (DF and MSEL scaling)
 * and clock divider (CSEL).
 *
 * @param [out]  fcclk       The Core Clock frequency (Hz)
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS               Successfully calculated the frequency.
 *              - #ADI_PWR_NOT_INITIALIZED       The power service is not initialized.
 *
 * @sa adi_pwr_SetFreq()
 */
ADI_PWR_RESULT adi_pwr_GetCoreFreq(
	uint32_t   *fcclk)
{
	ADI_PWR_CLK_STRUCT clk;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	GetFreq(&clk);

	*fcclk = clk.cclk0;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief  Calculate the system clock frequency.
 *
 * The frequency calculation is based on CLKIN, VCO (DF and MSEL scaling)
 * and clock divider (SYSSEL, S0SEL and S1SEL).
 *
 * @param [out]  fsysclk      The System clock frequency (Hz)
 * @param [out]  fsclk0       The Core0 system clock frequency (Hz)
 * @param [out]  fsclk1       The Core1 system clock frequency (Hz)
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS               Successfully calculated the frequency.
 *              - #ADI_PWR_NOT_INITIALIZED       The power service is not initialized.
 *
 * @sa adi_pwr_SetFreq()
 */
ADI_PWR_RESULT adi_pwr_GetSystemFreq(
	uint32_t   *fsysclk,
	uint32_t   *fsclk0,
	uint32_t   *fsclk1)
{
	ADI_PWR_CLK_STRUCT clk;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	GetFreq(&clk);

	*fsysclk = clk.syscclk0;
	*fsclk0 = clk.sclk0;
	*fsclk1 = clk.sclk1;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief  Set the dynamic power management operating mode.
 *
 * - Full-on mode is the maximum performance mode. In this mode, the PLL is enabled
 *   and not bypassed. Full-on mode is the normal execution state of the processor,
 *   with the processor and all enabled peripherals running at full speed.
 *
 * - In active mode, the PLL is enabled but bypassed. Because the PLL is bypassed,
 *   the processor core clock (CCLK) and system clock (SCLK) run at the input
 *   clock (CLKIN) frequency. DMA access is available to appropriately configured
 *   L1 and external memories.
 *
 * - Deep sleep mode maximizes power savings by disabling the PLL, CCLK, and SCLK.
 *   In this mode, the processor core and all peripherals (except those enabled as
 *   wakeup sources) are disabled. DMA is not supported in this mode.
 *
 * - Hibernate State - For lowest possible power dissipation, this state allows
 *   the internal supply (VDDINT) to be powered down by the external regulator,
 *   while keeping the I/O supply (VDDEXT) running.
 *
 * @param [in]  eMode  The dynamic power management operating mode.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully set the power mode.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *              - #ADI_PWR_INVALID_MODE             The power mode is invalid.
 *              - #ADI_PWR_INVALID_MODE_TRANSITION  The power mode transition is invalid.
 *
 * @sa adi_pwr_GetPowerMode()
 *
 */
ADI_PWR_RESULT adi_pwr_SetPowerMode(
	ADI_PWR_MODE  const eMode)
{
	ADI_PWR_RESULT result;
	ADI_PWR_MODE  currentMode;
	uint32_t pllStat, clkStat, lockStat;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	/* get the current mode */
	result = adi_pwr_GetPowerMode(&currentMode);
	if (result != ADI_PWR_SUCCESS) {
		return ADI_PWR_INVALID_MODE;
	}

	if (currentMode == eMode) {
		/* no mode change */
		result = ADI_PWR_SUCCESS;
	} else if (currentMode == ADI_PWR_MODE_FULL_ON) {
		switch(eMode) {
#if !defined (__ADSPBF707_FAMILY__)
			case ADI_PWR_MODE_ACTIVE:
				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_ENTER_ACTIVE, SET);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE);
				break;
			case ADI_PWR_MODE_ACTIVE_PLLDISABLED:
				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE_PLLDISABLED);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_ENTER_ACTIVE, SET);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_PLL_DISABLE, SET);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE_PLLDISABLED);
				break;
#endif
			case ADI_PWR_MODE_DEEP_SLEEP:
				/* pre-sleep status */
				result = adi_pwr_GetClkStatusRegister(
						ADI_PWR_CLK_STAT_PLL_BYPASS, &pllStat);

				result = adi_pwr_GetClkStatusRegister(
						ADI_PWR_CLK_STAT_CLK_ALIGN, &clkStat);

				result = adi_pwr_GetClkStatusRegister(
						ADI_PWR_CLK_STAT_PLL_LOCKED, &lockStat);

				if ((pllStat != 0u) || (clkStat != 0u) || (lockStat != 1u))
				{
					return ADI_PWR_FAILURE;
				}

				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_DEEP_SLEEP);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_DEEP_SLEEP, SET);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_DEEP_SLEEP);
				break;
#if defined(USE_HIBERNATEB)                               
			case ADI_PWR_MODE_HIBERNATE:
				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_HIBERNATE);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_HIBERNATE, SET);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_HIBERNATE);
				break;
#endif                                
			default:
				result = ADI_PWR_INVALID_MODE_TRANSITION;
				break;
		}
	} else if ((currentMode == ADI_PWR_MODE_ACTIVE) ||
	           (currentMode == ADI_PWR_MODE_ACTIVE_PLLDISABLED)){
		switch(eMode) {
			case ADI_PWR_MODE_FULL_ON:
				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_FULL_ON);
#if !defined (__ADSPBF707_FAMILY__)
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_PLL_DISABLE, CLEAR);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_EXIT_ACTIVE, SET);
#endif
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_FULL_ON);
				break;
#if !defined (__ADSPBF707_FAMILY__)
			case ADI_PWR_MODE_ACTIVE:
				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_PLL_DISABLE, CLEAR);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE);
				break;
			case ADI_PWR_MODE_ACTIVE_PLLDISABLED:
				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE_PLLDISABLED);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_PLL_DISABLE, SET);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_ACTIVE_PLLDISABLED);
				break;
#endif
			case ADI_PWR_MODE_DEEP_SLEEP:
				/* pre-sleep status */
				result = adi_pwr_GetClkStatusRegister(
						ADI_PWR_CLK_STAT_PLL_BYPASS, &pllStat);

				result = adi_pwr_GetClkStatusRegister(
						ADI_PWR_CLK_STAT_CLK_ALIGN, &clkStat);

				result = adi_pwr_GetClkStatusRegister(
						ADI_PWR_CLK_STAT_PLL_LOCKED, &lockStat);

				if ((pllStat != 0u) || (clkStat != 0u) || (lockStat != 1u))
				{
					return ADI_PWR_FAILURE;
				}

				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_DEEP_SLEEP);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_DEEP_SLEEP, SET);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_DEEP_SLEEP);
				break;
#if defined(USE_HIBERNATEB)                                
			case ADI_PWR_MODE_HIBERNATE:
				NotifyListeners(ADI_PWR_EVENT_MODE_PRE_CHANGE,
						(uint32_t)ADI_PWR_MODE_HIBERNATE);
				result = adi_pwr_SetControlRegister(ADI_PWR_CTL_HIBERNATE, SET);
				NotifyListeners(ADI_PWR_EVENT_MODE_POST_CHANGE,
						(uint32_t)ADI_PWR_MODE_HIBERNATE);
				break;
#endif                                
			default:
				result = ADI_PWR_INVALID_MODE_TRANSITION;
				break;
		}
	} else {
		result = ADI_PWR_INVALID_MODE;
	}

        WaitForLock();
        
	return result;
}

/**
 * @brief  Returns the dynamic power management operating mode.
 *
 * This function is used to obtain the current dynamic power
 * management operating mode.  Since the deep sleep and hibernate modes
 * disable the core, this function can only determine full-on and active modes.
 *
 * @param [out]  eMode  The dynamic power management operating mode.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully determined the power mode.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *              - #ADI_PWR_MODE_UNKNOWN             The power mode can't be determined.
 *
 * @sa adi_pwr_SetPowerMode()
 *
 */
ADI_PWR_RESULT adi_pwr_GetPowerMode(
	ADI_PWR_MODE  *eMode)
{
	uint32_t status;
	uint32_t pos;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition((uint32_t)ADI_PWR_STAT_CURR_MODE);

	status = (*pREG_DPM0_STAT & (uint32_t)ADI_PWR_STAT_CURR_MODE) >> pos;

	if(status == (uint32_t)ADI_PWR_MODE_FULL_ON) {
		*eMode = ADI_PWR_MODE_FULL_ON;
	} else if (status == (uint32_t)ADI_PWR_MODE_ACTIVE) {
		*eMode = ADI_PWR_MODE_ACTIVE;
	} else if(status == (uint32_t)ADI_PWR_MODE_ACTIVE_PLLDISABLED) {
		*eMode = ADI_PWR_MODE_ACTIVE_PLLDISABLED;
	} else {
		return ADI_PWR_MODE_UNKNOWN;
	}

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Set the signals used to wake-up the processor.
 *
 * When the operating mode is in deep sleep or hibernate a special
 * wake-up signal is needed to transition the processor back to an active
 * or full-on state.
 *
 * @param [in]  eWakeup       The wake-up signal.
 * @param [in]  ePol          The polarity of the wake-up signal.
 * @param [in]  bEnable       True to enable the wake-up signal, false to disable.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully set the voltage level.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 */
ADI_PWR_RESULT adi_pwr_EnableWakeupSignal(
	ADI_PWR_WAKE_SIGNAL const eWakeup,
	ADI_PWR_WAKE_POL    const ePol,
	bool                const bEnable)
{
	uint32_t regValue;
	uint32_t wake = (uint32_t)eWakeup;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

    adi_osal_EnterCriticalRegion();

	regValue = *pREG_DPM0_WAKE_POL;
	if (ePol == ADI_PWR_WAKE_POL_HIGH)
	{
		regValue |= wake;
	}
	else
	{
		regValue &= (uint32_t)~wake;
	}
	*pREG_DPM0_WAKE_POL = regValue;

	regValue = *pREG_DPM0_WAKE_EN;
	if (bEnable)
	{
		regValue |= wake;
	}
	else
	{
		regValue &= (uint32_t)~wake;
	}
	*pREG_DPM0_WAKE_EN = regValue;

    adi_osal_ExitCriticalRegion();

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Get Clock Generation Unit (CGU) control register values.
 *
 * The CGU control register is used to set the PLL and power mode
 * functionality.  Use the adi_pwr_SetFreq() and adi_pwr_SetPowerMode()
 * functions to automatically set the CGU control register.
 *
 * @param [in]  eCtl    The register field to get.
 * @param [out] value  The register field value.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully got the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *
 * @sa adi_pwr_SetClkControlRegister()
 *
 */
ADI_PWR_RESULT adi_pwr_GetClkControlRegister(
    ADI_PWR_CLK_CTL        const eCtl,
    uint32_t               *value)
{
	uint32_t pos;
	uint32_t ctl = (uint32_t)eCtl;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	ReadRegister(CGU0_CTL_REGISTER);

	pos = GetFieldPosition(ctl);

	*value = (Power.reg.cguctl & ctl) >> pos;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Set Clock Generation Unit (CGU) control register values.
 *
 * The CGU control register is used to set the PLL and power mode
 * functionality.  Use the adi_pwr_SetFreq() and adi_pwr_SetPowerMode()
 * functions to automatically set the PLL control register.
 *
 * @param [in]  eCtl   The register field to set.
 * @param [in]  value  The register value to set.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully set the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *              - #ADI_PWR_INVALID_VALUE            The register value is invalid.
 *
 * @sa adi_pwr_SetFreq()
 * @sa adi_pwr_SetPowerMode()
 * @sa adi_pwr_GetClkControlRegister()
 *
 */
ADI_PWR_RESULT adi_pwr_SetClkControlRegister(
    ADI_PWR_CLK_CTL        const eCtl,
    uint32_t               const value)
{
	uint32_t pos;
	uint32_t regValue;
	bool bError;
	uint32_t ctl = (uint16_t)eCtl;
	ADI_PWR_RESULT result = ADI_PWR_SUCCESS;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition(ctl);

#if defined(ADI_DEBUG)
	if (value > (ctl >> pos)) {
		return ADI_PWR_INVALID_VALUE;
	}
#endif

    adi_osal_EnterCriticalRegion();

	ReadRegister(CGU0_CTL_REGISTER);

	regValue = Power.reg.cguctl & (uint32_t)~ctl;
	regValue |= (uint32_t)(value << pos) & ctl;
	Power.reg.cguctl = regValue;

	bError = WriteRegister(CGU0_CTL_REGISTER);

    adi_osal_ExitCriticalRegion();

    if (bError)
    {
    	result = ADI_PWR_RESULT_PLL_ERROR;
    }

	return result;
}

/**
 * @brief Get Clock Generation Unit (CGU) divide register values.
 *
 * The CGU divide register is used to set the core select (CSEL), system
 * select (SYSSEL, S0SEL, S1SEL), DDR (DSEL) and OUTCLK (OSEL) values.
 * Use the adi_pwr_SetFreq() function to automatically set the CGU divide register.
 *
 * @param [in]  eDiv   The register field to get.
 * @param [out] value  The register field value.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully got the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *
 * @sa adi_pwr_SetClkDivideRegister()
 *
 */
ADI_PWR_RESULT adi_pwr_GetClkDivideRegister(
	ADI_PWR_CLK_DIV        const eDiv,
    uint32_t               *value)
{
	uint32_t pos;
	uint32_t plldiv = (uint32_t)eDiv;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	ReadRegister(CGU0_DIV_REGISTER);

	pos = GetFieldPosition(plldiv);

	*value = (Power.reg.cgudiv & plldiv) >> pos;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Set Clock Generation Unit (CGU) divide register values.
 *
 * The CGU divide register is used to set the core select (CSEL), system
 * select (SYSSEL, S0SEL, S1SEL), DDR (DSEL) and OUTCLK (OSEL) values.
 * Use the adi_pwr_SetFreq() function to automatically set the CGU divide register.
 *
 * @param [in]  eDiv   The register field to set.
 * @param [in]  value  The register value to set.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully set the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *              - #ADI_PWR_INVALID_VALUE            The register value is invalid.
 *
 * @sa adi_pwr_SetFreq()
 * @sa adi_pwr_GetClkDivideRegister()
 *
 */
ADI_PWR_RESULT adi_pwr_SetClkDivideRegister(
	ADI_PWR_CLK_DIV        const eDiv,
    uint32_t               const value)
{
	uint32_t pos;
	uint32_t regValue;
	bool bError;
	uint32_t plldiv = (uint32_t)eDiv;
	ADI_PWR_RESULT result = ADI_PWR_SUCCESS;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition(plldiv);

#if defined(ADI_DEBUG)
	if (value > (plldiv >> pos)) {
		return ADI_PWR_INVALID_VALUE;
	}
#endif

    WaitForLock();

    adi_osal_EnterCriticalRegion();

	ReadRegister(CGU0_DIV_REGISTER);

	regValue = Power.reg.cgudiv & (uint32_t)~plldiv;
	regValue |= (uint32_t)(value << pos) & plldiv;
    regValue |= (uint32_t)ADI_PWR_CLK_DIV_UPDT;

	Power.reg.cgudiv = regValue;

	bError = WriteRegister(CGU0_DIV_REGISTER);

    adi_osal_ExitCriticalRegion();

    WaitForLock();

    if (bError)
    {
    	result = ADI_PWR_RESULT_PLL_ERROR;
    }

	return result;
}

/**
 * @brief  Calculate the DDR clock frequency.
 *
 * The frequency calculation is based on CLKIN, VCO (DF and MSEL scaling)
 * and clock divider (DSEL).
 *
 * @param [out]  fdclk       The DDR clock frequency (Hz)
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS               Successfully calculated the frequency.
 *              - #ADI_PWR_NOT_INITIALIZED       The power service is not initialized.
 *
 * @sa adi_pwr_SetClkDivideRegister()
 */
ADI_PWR_RESULT adi_pwr_GetDDRClkFreq(
	uint32_t   *fdclk)
{
	ADI_PWR_CLK_STRUCT clk;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	GetFreq(&clk);

	*fdclk = clk.dlk;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief  Calculate the OUTCLK clock frequency.
 *
 * The frequency calculation is based on CLKIN, VCO (DF and MSEL scaling)
 * and clock divider (OSEL).
 *
 * @param [out]  foclk       The OUTCLK clock frequency (Hz)
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS               Successfully calculated the frequency.
 *              - #ADI_PWR_NOT_INITIALIZED       The power service is not initialized.
 *
 * @sa adi_pwr_SetClkDivideRegister()
 */
ADI_PWR_RESULT adi_pwr_GetOutClkFreq(
	uint32_t   *foclk)
{
	ADI_PWR_CLK_STRUCT clk;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	GetFreq(&clk);

	*foclk = clk.outclk;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Get the Clock Generation Unit (CGU) CLKOUT register values.
 *
 * The CGU_CLKOUTSEL selects the signal that the CGU drives through
 * the CLKOUT multiplexer.  Also, this register selects the divisor
 * for the USBCLK output.
 *
 * @param [in]  eSel   The register field to get.
 * @param [out] value  The register field value.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully got the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *
 * @sa adi_pwr_SetClkOutSelectRegister()
 */
ADI_PWR_RESULT adi_pwr_GetClkOutSelectRegister(
	ADI_PWR_CLK_SELECT     const eSel,
    uint32_t               *value)
{
	uint32_t pos;
	uint32_t clksel = (uint32_t)eSel;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition(clksel);

	*value = (*pREG_CGU0_CLKOUTSEL & clksel) >> pos;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Set the Clock Generation Unit (CGU) CLKOUT register values.
 *
 * The CGU_CLKOUTSEL selects the signal that the CGU drives through
 * the CLKOUT multiplexer.  Also, this register selects the divisor
 * for the USBCLK output.
 *
 * @param [in]  eSel   The register field to set.
 * @param [in]  value  The register field value.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully got the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *
 * @sa adi_pwr_GetClkOutSelectRegister()
 */
ADI_PWR_RESULT adi_pwr_SetClkOutSelectRegister(
	ADI_PWR_CLK_SELECT     const eSel,
    uint32_t               const value)
{
	uint32_t pos;
	uint32_t regValue;
	uint32_t clksel = (uint32_t)eSel;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition(clksel);

#if defined(ADI_DEBUG)
	if (value > (clksel >> pos)) {
		return ADI_PWR_INVALID_VALUE;
	}
#endif

    adi_osal_EnterCriticalRegion();

	regValue = *pREG_CGU0_CLKOUTSEL & (uint32_t)~clksel;
	regValue |= (uint32_t)(value << pos) & clksel;
	*pREG_CGU0_CLKOUTSEL = regValue;

    adi_osal_ExitCriticalRegion();

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Get the Clock Generation Unit (CGU) status register values.
 *
 * The CGU_STAT register reflects the PLL status and errors detected
 * during the PLL configuration.
 *
 * @param [in]  eStat  The register field to get.
 * @param [out] value  The register field value.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully got the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *
 * @sa adi_pwr_SetFreq()
 * @sa adi_pwr_SekClkDivideRegister()
 * @sa adi_pwr_SekClkControlRegister()
 *
 */
ADI_PWR_RESULT adi_pwr_GetClkStatusRegister(
	ADI_PWR_CLK_STAT       const eStat,
    uint32_t               *value)
{
	uint32_t pos;
	uint32_t stat = (uint32_t)eStat;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition(stat);

	*value = (*pREG_CGU0_STAT & stat) >> pos;

	return ADI_PWR_SUCCESS;
}

/*
 * Dynamic Power Management unit (DPM) Functions
 */

/**
 * @brief Get Dynamic Power Management unit (DPM) control register values.
 *
 * The DPM_CTL register determine the modes in which the part operates.
 *
 * @param [in]  eCtl   The register field to get.
 * @param [out] value  The register field value.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully got the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *
 * @sa adi_pwr_SetControlRegister()
 *
 */
ADI_PWR_RESULT adi_pwr_GetControlRegister(
	ADI_PWR_CTL            const eCtl,
    uint32_t               *value)
{
	uint32_t pos;
	uint32_t ctl = (uint32_t)eCtl;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition(ctl);

	*value = (*pREG_DPM0_CTL & ctl) >> pos;

	return ADI_PWR_SUCCESS;
}

/**
 * @brief Set Dynamic Power Management unit (DPM) control register values.
 *
 * The DPM_CTL register determine the modes in which the part operates.  Use the
 * adi_pwr_SetPowerMode() API to automatically set this register.
 *
 * @param [in]  eCtl   The register field to set.
 * @param [in]  value  The register field value.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                  Successfully got the register value.
 *              - #ADI_PWR_NOT_INITIALIZED          The power service is not initialized.
 *
 * @sa adi_pwr_SetPowerMode()
 * @sa adi_pwr_GetControlRegister()
 *
 */
ADI_PWR_RESULT adi_pwr_SetControlRegister(
    ADI_PWR_CTL            const eCtl,
    uint32_t               const value)
{
	uint32_t pos;
	uint32_t regValue;
	uint32_t ctl = (uint16_t)eCtl;

#if defined(ADI_DEBUG)
	if (!Power.bInitialized) {
		return ADI_PWR_NOT_INITIALIZED;
	}
#endif

	pos = GetFieldPosition(ctl);

#if defined(ADI_DEBUG)
	if (value > (ctl >> pos)) {
		return ADI_PWR_INVALID_VALUE;
	}
#endif

    adi_osal_EnterCriticalRegion();

	regValue = *pREG_DPM0_CTL & (uint32_t)~ctl;
	regValue |= (uint32_t)(value << pos) & ctl;
	*pREG_DPM0_CTL = regValue;

    adi_osal_ExitCriticalRegion();

#if defined(USE_HIBERNATEB)
    if ((eCtl == ADI_PWR_CTL_DEEP_SLEEP) || (eCtl == ADI_PWR_CTL_HIBERNATE))
    {
    	/* TO-DO must idle both cores */
    	idle();
    }
#else
    if (eCtl == ADI_PWR_CTL_DEEP_SLEEP)
    {
          	/* TO-DO */
    }
#endif      
	return ADI_PWR_SUCCESS;
}

/**
 * @brief Install a user defined callback function to receive power service notifications.
 *
 * The power service invokes the callback functions before the frequency or mode
 * changes (pre change) and after the frequency or mode changes (post change).
 * The installed callback functions should handle frequency or mode sensitive changes
 * such as shutting down services before the core sleeps or changing frequency dependent
 * drivers.
 *
 * The system clock or power mode is passed as an argument to the callback function.
 *
 * @code
 *     //Power Service Callback function
 *     static void PowerCallback(void *pCBParam, uint32_t Event, void *pArg)
 *     {
 *       uint32_t sclk;
 *       ADI_PWR_MODE  mode;
 *
 *       switch ( (ADI_PWR_EVENT)Event )
 *       {
 *         case ADI_PWR_EVENT_FREQ_PRE_CHANGE:
 *           sclk = *(uint32_t*)(pArg);
 *           // code to execute before the core or system clock changes
 *           break;
 *         case ADI_PWR_EVENT_FREQ_POST_CHANGE:
 *           sclk = *(uint32_t*)(pArg);
 *           // code to execute after the core or system clock changes
 *           break;
 *         case ADI_PWR_EVENT_MODE_PRE_CHANGE:
 *           mode = (ADI_PWR_MODE)pArg;
 *           // code to execute before the power mode changes
 *            break;
 *         case ADI_PWR_EVENT_MODE_POST_CHANGE:
 *           mode = (ADI_PWR_MODE)pArg;
 *           // code to execute after the power mode changes
 *           break;
 *         }
 *      }
 * @endcode
 *
 * @param [in]  pfCallback   The user defined callback function.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                     Successfully installed the callback.
 *              - #ADI_PWR_CALLBACK_INSTALLED          The callback function is already installed.
 *              - #ADI_PWR_CALLBACK_LIMIT_EXCEEDED     The number of installed callbacks is exceeded.
 *
 * @sa adi_pwr_RemoveCallback()
 *
 */
ADI_PWR_RESULT adi_pwr_InstallCallback(
    ADI_CALLBACK        pfCallback)
{
	ADI_CALLBACK  *pCallbackEntry;    /* callback entry */
	uint32_t i;

    /* see if this callback is already installed */
    for (i=0u; i<ADI_PWR_MAX_LISTENERS; i++) {
    	pCallbackEntry = &Power.pwrCallback[i];
    	if (pfCallback == *pCallbackEntry) {
    		/* already installed */
    		return ADI_PWR_CALLBACK_INSTALLED;
    	}
    }

    /* find an empty slot */
    for (i=0u; i<ADI_PWR_MAX_LISTENERS; i++) {
    	pCallbackEntry = &Power.pwrCallback[i];
    	if ((*pCallbackEntry) == (void*)0u) {
    		/* empty slot found */
            *pCallbackEntry = pfCallback;
            return ADI_PWR_SUCCESS;
    	}
    }

	return ADI_PWR_CALLBACK_LIMIT_EXCEEDED;
}

/**
 * @brief Remove a user defined callback function that receives power service notifications.
 *
 * The callback function will no longer receive power service notifications.
 *
 * @param [in]  pfCallback   The user defined callback function.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS                     Successfully installed the callback.
 *              - #ADI_PWR_CALLBACK_NOT_INSTALLED      The callback function is not installed.
 *
 * @sa adi_pwr_InstallCallback()
 *
 */
ADI_PWR_RESULT adi_pwr_RemoveCallback(
    ADI_CALLBACK        pfCallback)
{
	ADI_CALLBACK  *pCallbackEntry;    /* callback entry */
	uint32_t i;

    /* find this callback entry */
    for (i=0u; i<ADI_PWR_MAX_LISTENERS; i++) {
    	pCallbackEntry = &Power.pwrCallback[i];
    	if (pfCallback == *pCallbackEntry) {
    		/* remove */
    		*pCallbackEntry = (ADI_CALLBACK)0u;
    		return ADI_PWR_SUCCESS;
    	}
    }

	return ADI_PWR_CALLBACK_NOT_INSTALLED;
}

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */


