/******************************************************************************

Copyright (c) 2012 Analog Devices.  All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
 *******************************************************************************
 *
 * @file:    adi_pwr_coreb.c
 * @brief:   Dynamic Power Management service implementation
 * @version: $Revision: 15380 $
 * @date:    $Date: 2012-07-12 08:53:34 -0400 (Thu, 12 Jul 2012) $
 *
 ******************************************************************************/

/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#endif /* _MISRA_RULES */

#pragma file_attr("prefersMem=internal")
#pragma file_attr("prefersMemNum=30")

#if defined(__ADSPBF561_FAMILY__)
#pragma file_attr("DualCoreMem=CoreB")
#endif

#include <ccblkfn.h>
#include <services\pwr\adi_pwr.h>
#include <services\int\adi_sic.h>
#include <services\int\adi_sid.h>
#include <adi_osal.h>
#include <sys/exception.h>

#define SUP0_IMASK    (0x40000000)
#define SUP0_CLEAR    (0x0800)

#define IWR0_REGISTER (pSICB_IWR0)

void Sup0Handler (uint32_t SID, void *pCBParam);

/*
 * This handler is used for Supplemental interrupt 0.  Core B must
 * enter idle to program the power setting.
 */
void Sup0Handler (uint32_t SID, void *pCBParam)
{
    uint32_t iMask;
    uint32_t sicIwr;
    uint32_t cclk;
    ADI_PWR_RESULT result;

    extern volatile int _Processor_cycles_per_sec;

	/* is this a Sup 0 interrupt */
    if ((*pSICB_SYSCR & SUP0_SET) == 0u) {
    	return;
    }

	/* save the IWR register and set pll wakeup */
	sicIwr = *IWR0_REGISTER;
	*IWR0_REGISTER |= (uint32_t)(1 << ADI_SID_PLL_WAKEUP);

	/* Acknowledge Supplemental Interrupt */
	*pSICB_SYSCR |= (uint16_t)SUP0_CLEAR;

	/* idle needed when adjusting power freq or mode */
	iMask = cli();
	idle();
	sti(iMask);

	/* restore the IWR register */
	*IWR0_REGISTER = sicIwr;

	if (adi_pwr_GetCoreFreq(&cclk) == ADI_PWR_SUCCESS)
	{
		_Processor_cycles_per_sec = (int32_t)cclk;
	}
}

/******************************************************
 API implementation:
 *******************************************************/

/**
 * @brief Initialize the dynamic power management service for Core B.
 *
 * For the dual-core ADSP-BF561 processor this function must be
 * called from Core B.  The Core A power service needs to synchronize Core B
 * when changing the power settings.
 *
 *
 * This API must be called before using other power service API's.
 *
 * @return      Result
 *              - #ADI_PWR_SUCCESS               Successfully initialized the power service.
 *              - #ADI_PWR_FAILURE               Failed to initialize the power service.
 *
 */
ADI_PWR_RESULT adi_pwr_InitCoreB()
{
	ADI_INT_STATUS status;
	ADI_SIC_RESULT result;

	result = adi_sic_SetCoreID(ADI_SID_SI0, ADI_SIC_CORE1);
	status = adi_int_InstallHandler(ADI_SID_SI0, Sup0Handler, NULL, true);

	if ((result != ADI_SIC_SUCCESS) || (status != ADI_INT_SUCCESS))
	{
		return ADI_PWR_FAILURE;
	}

	return ADI_PWR_SUCCESS;
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */


