/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_tmr_ctmr.c
*
* @brief     Core Timer service API implementation.
*
* @details
*            This is a source file which contains the implementation of core
*            timer functions.
*/

/** @addtogroup Timer_Service Core Timer Service
*  @{
*/

/*********************************************************************

Include files

*********************************************************************/
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be longer than 31 characters")
#endif

#include <adi_types.h>
#include <services/tmr/adi_ctmr.h>
#include <blackfin.h>
#include <stddef.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


#ifndef BITP_TCNTL_INT

#define BITP_TCNTL_INT                        3ul    /* Timer generated interrupt (sticky) */
#define BITP_TCNTL_AUTORLD                    2ul    /* Timer auto reload */
#define BITP_TCNTL_EN                         1ul    /* Timer enable */
#define BITP_TCNTL_PWR                        0ul    /* Timer Low Power Control */

#define BITM_TCNTL_INT                        8ul    /* Timer generated interrupt (sticky) */
#define BITM_TCNTL_AUTORLD                    4ul    /* Timer auto reload */
#define BITM_TCNTL_EN                         2ul    /* Timer enable */
#define BITM_TCNTL_PWR                        1ul    /* Timer Low Power Control */

#endif  /* BITP_TCNTL_INT */


#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_8_7:"Objects shall be defined at block scope if they are only accessed from within a single function.")
#pragma diag(suppress:misra_rule_10_1_a:"Allow typecasting between variables of same width")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_16_7:"Allow pointer parameter to be declared without const")
#endif /* _MISRA_RULES */

/*********************************************************************

Declarations

*********************************************************************/
typedef struct _ADI_CTMR_REGS
{
	volatile uint32_t  CTMR_TCNTL;
	volatile uint32_t  CTMR_TPERIOD;
	volatile uint32_t  CTMR_TSCALE;
	volatile uint32_t  CTMR_TCOUNT;
} ADI_CTMR_REGS;

typedef struct _ADI_CTMR_DATA
{
	volatile ADI_CTMR_REGS   	*pCTmrRegs;
	ADI_CALLBACK 	             pfCallback;
	void            	        *pCBParam;
	bool                         bInUse;
	/* Add other fields required */

} ADI_CTMR_DATA;

/* Table of available timers in the processor */
static ADI_CTMR_DATA	gCTimerData[] = {
	{
			(volatile ADI_CTMR_REGS  *)TCNTL,
			NULL,
			NULL,
			false
	},
};

static void CoreTimerHandler(uint32_t iid, void *pCBParam);

/* Validates the device handle */
static ADI_CTMR_RESULT ValidateDeviceHandle (
		ADI_CTMR_DATA     *const pDevice
);



/* Core Timer Interrupt handler */
static void CoreTimerHandler(uint32_t iid, void *pCBParam)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)pCBParam;
	if((pDevice != NULL) && (pDevice->pfCallback != NULL))
	{
		pDevice->pfCallback( pDevice->pCBParam, (uint32_t) ADI_CTIMER_EXPIRED, NULL);
	}

    return;
}

/**
 * @brief       Opens a given timer.
 *
 * @details     Opens a given timer device and returns the handle to the timer
 *              which should be used other APIs to configure and use the timer.
 *
 * @param [in]  eDevice     Timer number to open.
 * @param [in]  pfCallback  Pointer to callback function to notify events.
 * @param [in]  pCBParam    Pointer to callback parameter which will be passed
 *                          back to the application in the callback.
 * @param [out] phCTimer    Pointer to a location where the handle to the opened
 *                          timer is written.
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS      if successfully opened the given timer.
 *              - #ADI_CTMR_INT_FAILURE  if unable to establish the core timer interrupt
 *              - #ADI_CTMR_IN_USE [D]   if the given timer is in use.
 *              - #ADI_CTMR_INVALID_POINTER [D] if any of the given pointer are invalid
 *              - #ADI_CTMR_INVALID_TMR_ID  [D] if the given timer number is invalid
 *
 * @sa          adi_ctmr_Close()
 *
 * @note        pfCallback is optional and may be NULL if no callback is needed
 * @note        pCBParam is also optional
 * @note        phCTimer will be NULL upon failure
 */

ADI_CTMR_RESULT  adi_ctmr_Open (
    ADI_CTMR_DEVICE             eDevice,
    ADI_CALLBACK                pfCallback,
    void               *const   pCBParam,
    ADI_CTMR_HANDLE            *phCTimer
)
{
	ADI_CTMR_DATA          *pDevice;
	volatile ADI_CTMR_REGS *pCTMRRegs;
	ADI_INT_STATUS          intStatus;


#ifdef ADI_DEBUG
	/* Check if the given pointer to device handle is valid */
	if(phCTimer == NULL)
	{
		return ADI_CTMR_INVALID_POINTER;
	}

	/* check to see if the timer ID is valid */
	if( eDevice >= ADI_MAX_CTMRS )
	{
	    /* Return NULL handle upon failure */
	    *phCTimer = (ADI_CTMR_HANDLE)0;
		return ADI_CTMR_INVALID_TMR_ID;
	}

#endif

	pDevice = &gCTimerData[eDevice];

#ifdef ADI_DEBUG
	if( pDevice->bInUse == true )
	{
	    *phCTimer = (ADI_CTMR_HANDLE)0;
		return ADI_CTMR_IN_USE;
	}
#endif

    /* Register NMI Handler */
	intStatus = adi_int_InstallHandler(
			ADI_CID_IVTMR,
			CoreTimerHandler,
			pDevice,
			true);
    if( intStatus != ADI_INT_SUCCESS) {
    	return  ADI_CTMR_INT_FAILURE;
    }

	pDevice->pfCallback       =   pfCallback;
    pDevice->pCBParam         =   pCBParam;
	pDevice->bInUse           =   true;

	pCTMRRegs = pDevice->pCTmrRegs;

	pCTMRRegs->CTMR_TCOUNT 		= 0;					/* initialize TCOUNT to 0 */
	pCTMRRegs->CTMR_TPERIOD 	= 0;					/* initialize TPERIOD to 0 */
	pCTMRRegs->CTMR_TSCALE 		= 0;					/* initialize TSCALE to 0 */
	pCTMRRegs->CTMR_TCNTL 		|= BITM_TCNTL_PWR;		/* initialize to Active State */

	*phCTimer = (ADI_CTMR_HANDLE) pDevice;

	return ADI_CTMR_SUCCESS;
}


/**
 * @brief       Closes the given timer.
 *
 * @details     Upon closing the timer will be disabled and all of
 *              its registers will be reset to default values.
 *
 * @param [in]  hTimer      Handle to the timer
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS      if successfully closed the given timer.
 *              - #ADI_CTMR_INT_FAILURE  if unable to uninstall the core timer interrupt
 *              - #ADI_CTMR_INVALID_HANDLE [D] if the given timer handle is invalid.
 *
 * @sa          adi_ctmr_Open()
 */
ADI_CTMR_RESULT  adi_ctmr_Close (
    ADI_CTMR_HANDLE      const hTimer
)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)hTimer;
	volatile ADI_CTMR_REGS *pCTMRRegs;
	ADI_INT_STATUS          intStatus;


#ifdef ADI_DEBUG
	/* Validate the given handle */
    if(ValidateDeviceHandle(pDevice) != ADI_CTMR_SUCCESS)
    {
        return ADI_CTMR_INVALID_HANDLE;
    }
#endif

	pCTMRRegs = pDevice->pCTmrRegs;

	/* clear registers to avoid any interrupts related to the core timer */
	pCTMRRegs->CTMR_TCOUNT = 0;		/* initialize TCOUNT to 0 */
	pCTMRRegs->CTMR_TPERIOD = 0;	/* initialize TPERIOD to 0 */
	pCTMRRegs->CTMR_TSCALE = 0;		/* initialize TSCALE to 0 */
	pCTMRRegs->CTMR_TCNTL  = 0;		/* initialize to Active State */

	intStatus = adi_int_UninstallHandler( ADI_CID_IVTMR );
	if( intStatus != ADI_INT_SUCCESS ) {
		return ADI_CTMR_INT_FAILURE;
	}

	return ADI_CTMR_SUCCESS;
}

/**
 * @brief       Enables/disables the auto reload of the count.
 *
 * @details     When auto reload is enabled if the TCOUNT reaches 0 the TCOUNT
 *              register is reloaded with the contents of the PERIOD register and the
 *              timer continues to count.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bEnable     'true'  -   Enable auto reload
 *                          'false' -   Disable auto reload
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS  if successfully enabled/disabled auto reload.
 *              - #ADI_CTMR_INVALID_HANDLE [D] if the given timer handle is invalid.
 *
 * @sa          adi_ctmr_SetPeriod()
 */
ADI_CTMR_RESULT  adi_ctmr_EnableAutoReload (
    ADI_CTMR_HANDLE      const  hTimer,
    bool                        bEnable
)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)hTimer;
	volatile ADI_CTMR_REGS *pCTMRRegs;

#ifdef ADI_DEBUG
	/* Validate the given handle */
    if(ValidateDeviceHandle(pDevice) != ADI_CTMR_SUCCESS)
    {
        return ADI_CTMR_INVALID_HANDLE;
    }
#endif

	pCTMRRegs = pDevice->pCTmrRegs;

	if(bEnable)
	{
		pCTMRRegs->CTMR_TCNTL  |= BITM_TCNTL_AUTORLD;		/* enable auto reload bit */
	}
	else
	{
		pCTMRRegs->CTMR_TCNTL  &= ~BITM_TCNTL_AUTORLD;		/* disable auto reload bit */
	}

	return ADI_CTMR_SUCCESS;
}

/**
 * @brief       Enables the timer.
 *
 * @details     Enables/Disables the timer
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bEnable     'true'  -   Enable the timer
 *                          'false' -   Disable the timer
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS  if successfully enabled/disabled the timer.
 *              - #ADI_CTMR_INVALID_HANDLE [D] if the given timer handle is invalid.
 *
 * @note        if the timer is in low power mode enabling the timer will not
 *              start the timer. By default timer will be configured in active
 *              mode when it is opened. But the application can change it to low
 *              power mode using adi_ctmr_Activate API.
 *
 * @sa          adi_ctmr_Activate()
 */
ADI_CTMR_RESULT  adi_ctmr_Enable (
    ADI_CTMR_HANDLE      const hTimer,
    bool                       bEnable
)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)hTimer;
	volatile ADI_CTMR_REGS *pCTMRRegs;

#ifdef ADI_DEBUG
	/* Validate the given handle */
    if(ValidateDeviceHandle(pDevice) != ADI_CTMR_SUCCESS)
    {
        return ADI_CTMR_INVALID_HANDLE;
    }
#endif

	pCTMRRegs = pDevice->pCTmrRegs;

	if(bEnable)
	{
	    pCTMRRegs->CTMR_TCNTL  |= BITM_TCNTL_EN;		/* enable the timer */
	}
	else
	{
	    pCTMRRegs->CTMR_TCNTL  &= ~BITM_TCNTL_EN;		/* disable the timer */
	}

	return ADI_CTMR_SUCCESS;
}


/**
 * @brief       Sets the timer period value
 *
 * @details     When auto reload is enabled, the TCOUNT register is reloaded
 *              with the TPERIOD whenever TCOUNT reaches 0
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  nPeriod     Period value to be set
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS  if successfully set the period value.
 *              - #ADI_CTMR_INVALID_HANDLE [D] if the given timer handle is invalid.
 *
 * @sa          adi_ctmr_EnableAutoReload()
 * @sa          adi_ctmr_SetScale()
 */
ADI_CTMR_RESULT  adi_ctmr_SetPeriod (
    ADI_CTMR_HANDLE      const hTimer,
    uint32_t                   nPeriod
)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)hTimer;
	volatile ADI_CTMR_REGS *pCTMRRegs;

#ifdef ADI_DEBUG
	/* Validate the given handle */
    if(ValidateDeviceHandle(pDevice) != ADI_CTMR_SUCCESS)
    {
        return ADI_CTMR_INVALID_HANDLE;
    }
#endif

	pCTMRRegs = pDevice->pCTmrRegs;

	pCTMRRegs->CTMR_TPERIOD  = nPeriod;		/* set the period */

	return ADI_CTMR_SUCCESS;
}

/**
 * @brief       Sets the timer scale value
 *
 * @details     The scale value indicates the frequency by which the count value
 *              is decremented. In general, the count will be decremented every
 *              (scale+1) core clock cycles.
 *
 *              If the scale value is set to 0 then the count
 *              value is decremented every core clock cycle, when set to 1 it
 *              will be decremented by 2 and so on.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  nScale      Scale value to be set
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS  if successfully set the scale value.
 *              - #ADI_CTMR_INVALID_HANDLE [D] if the given timer handle is invalid.
 *
 * @sa          adi_ctmr_SetPeriod()
 */
ADI_CTMR_RESULT  adi_ctmr_SetScale (
    ADI_CTMR_HANDLE      const hTimer,
    uint32_t                   nScale
)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)hTimer;
	volatile ADI_CTMR_REGS *pCTMRRegs;

#ifdef ADI_DEBUG
	/* Validate the given handle */
    if(ValidateDeviceHandle(pDevice) != ADI_CTMR_SUCCESS)
    {
        return ADI_CTMR_INVALID_HANDLE;
    }
#endif

	pCTMRRegs = pDevice->pCTmrRegs;

	pCTMRRegs->CTMR_TSCALE  = nScale;		/* set the scale */

	return ADI_CTMR_SUCCESS;
}


/**
 * @brief       Get the current count value
 *
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  pCount      Pointer to a location where count value is written.
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS  if successfully read the count value.
 *              - #ADI_CTMR_INVALID_HANDLE [D] if the given timer handle is invalid.
 *              - #ADI_CTMR_INVALID_POINTER [D] if the value of pCount is NULL
 *
 * @sa          adi_ctmr_Enable()
 */
ADI_CTMR_RESULT  adi_ctmr_GetCount (
    ADI_CTMR_HANDLE      const hTimer,
    uint32_t                  *pCount
)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)hTimer;
	volatile ADI_CTMR_REGS *pCTMRRegs;

#ifdef ADI_DEBUG
	/* Validate the given handle */
    if(ValidateDeviceHandle(pDevice) != ADI_CTMR_SUCCESS)
    {
        return ADI_CTMR_INVALID_HANDLE;
    }

	if( pCount == 0 )
	{
		return ADI_CTMR_INVALID_POINTER;
	}
#endif

	pCTMRRegs = pDevice->pCTmrRegs;

	*pCount = pCTMRRegs->CTMR_TCOUNT;		/* get the count */

	return ADI_CTMR_SUCCESS;
}



/**
 * @brief       Puts the timer in low power mode or active mode.
 *
 * @details     When the timer is in low power mode, enabling the timer will not
 *              start the timer.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bActive     'true'  -   Put the timer in active state
 *                          'false' -   Put the timer in low power mode.
 *
 * @return      Status
 *              - #ADI_CTMR_SUCCESS  if successfully changed the timer state.
 *              - #ADI_CTMR_INVALID_HANDLE [D] if the given timer handle is invalid.
 *
 * @note        if the timer is in low power mode enabling the timer will not
 *              start the timer. By default timer will be configured in active
 *              mode when it is opened. But the application can change it to low
 *              power mode using this API. Alternatively, if the timer is in low
 *              power the application can use this API to switch the timer into
 *              active mode.
 *
 * @sa          adi_ctmr_Enable()
 */
ADI_CTMR_RESULT  adi_ctmr_Activate (
    ADI_CTMR_HANDLE      const hTimer,
    bool                       bActive
)
{
	ADI_CTMR_DATA *pDevice = (ADI_CTMR_DATA *)hTimer;
	volatile ADI_CTMR_REGS *pCTMRRegs;

#ifdef ADI_DEBUG
	/* Validate the given handle */
    if(ValidateDeviceHandle(pDevice) != ADI_CTMR_SUCCESS)
    {
        return ADI_CTMR_INVALID_HANDLE;
    }
#endif

	pCTMRRegs = pDevice->pCTmrRegs;

	if(bActive)
	{
		pCTMRRegs->CTMR_TCNTL  |= BITM_TCNTL_PWR;		/* enable auto reload bit */
	}
	else
	{
		pCTMRRegs->CTMR_TCNTL  &= ~BITM_TCNTL_PWR;		/* disable auto reload bit */
	}

	return ADI_CTMR_SUCCESS;
}


#ifdef ADI_DEBUG
/*
 * Verify that the handle is valid
 */
static ADI_CTMR_RESULT ValidateDeviceHandle (
		ADI_CTMR_DATA     *const pDevice
)
{
    int32_t i;

    /* Check if the given pointer is valid */
    if(pDevice == 0)
    {
    	return ADI_CTMR_INVALID_POINTER;
    }

	for( i=0; i<ADI_MAX_CTMRS; i++)
	{
		if( pDevice == &gCTimerData[i])
		{
			return ADI_CTMR_SUCCESS;
		}
	}
    return ADI_CTMR_INVALID_POINTER;
}
#endif

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif
