/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_tmr_bf6xx.c
*
* @brief     Timer service global API implementation.
*
* @details
*            This is a source file which contains the implementation of global
*            API functions of the timer device.
*/


/*********************************************************************

Include files

*********************************************************************/
#ifndef _ADI_TMR_BF6XX_C_
#define _ADI_TMR_BF6XX_C_

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be longer than 31 characters")
#endif

#include <sys/platform.h> 
#include <stdint.h>
#include <stdbool.h>
#include <services/tmr/adi_tmr.h>
#include <services/int/adi_int.h>
#include <adi_osal.h>
#include <stddef.h>
#include <string.h>
#include <assert.h>

#include "adi_tmr_data_bf6xx.c"
#include "adi_tmr_def_bf6xx.h"

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif


#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#pragma diag(suppress:misra_rule_8_5:"Allow the definition of objects in the source file. MISRA treats this as include file as it is included by other sources")
#pragma diag(suppress:misra_rule_10_3:"Allow typecasting to a value wider than underlying expression. For code readability")
#pragma diag(suppress:misra_rule_10_5:"Allow bitwise operations without requiring to typecast. This is for improving code readability")
#pragma diag(suppress:misra_rule_13_1:"Allow assignment operators in if statment to improve code efficiency")
#pragma diag(suppress:misra_rule_13_7:"Allow to add the assert check for the size constant.")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_16_7:"Allow pointer parameter to be declared without const")
#pragma diag(suppress:misra_rule_17_1:"Allow pointer arithmetic on pointers, required to get MMR address based on the interrupt ID and core ID")
#endif /* _MISRA_RULES */


/** @addtogroup Timer_Service_BF6xx Timer Service (ADSP-BF6xx)
*  @{
*/

/*
 * @brief       Initializes the timer service.
 *
 * @details     Initializes the timer service. This API should be called
 *              before calling any other Timer service APIs. It is acceptable
 *              to call this API multiple times. If the timer service is
 *              already initialized it will do nothing.
 *
 * @return      Status
 *              - #ADI_TMR_SUCCESS  if successfully initialized the timer service.
 *
 */
static ADI_TMR_RESULT  adi_tmr_Init(
    void
)
{
    /* Timer service state */
    static ADI_TMR_STATE  geState = ADI_TMR_STATE_UNKNOWN;
    uint32_t    i;

    /* Check if the service is already initialized */
    if(geState == ADI_TMR_STATE_UNKNOWN)
    {
      
        /* Stop all the timers */
        *pREG_TIMER0_STOP_CFG_SET    =  (uint16_t) (0xFFFFu);
        *pREG_TIMER0_RUN_CLR         =  (uint16_t) (0xFFFFu);
        *pREG_TIMER0_DATA_ILAT       =  (uint16_t) (0xFFFFu);
        *pREG_TIMER0_STAT_ILAT       =  (uint16_t) (0xFFFFu);
        *pREG_TIMER0_DATA_IMSK       =  (uint16_t) (0xFFFFu); 
        *pREG_TIMER0_DATA_IMSK       =  (uint16_t) (0xFFFFu); 

        /* Clear the trigger input and output masks */
        *pREG_TIMER0_TRG_MSK         =  0u;
        *pREG_TIMER0_TRG_IE          =  0u;

        for(i = 0u; i < ADI_NUM_TIMERS; i++)
        {
            volatile ADI_TMR_REGS *pTmrRegs = gaTimers[i].pTmrRegs;
            
            pTmrRegs->CFG = 0u;
            pTmrRegs->CNT = 0u;
            pTmrRegs->PER = 0u;
            pTmrRegs->WID = 0u;
            pTmrRegs->DLY = 0u;            
        }
        
        /* Install the status interrupt handler which is common to all the timers */
        if(adi_int_InstallHandler((uint32_t)INTR_TIMER0_STAT,
                                  StatusIntHandler,
                                  gaTimers,
                                  true) != ADI_INT_SUCCESS)
        {
            return ADI_TMR_FAILURE;
        }

        /* Mark the timer service state as initialized */
        geState = ADI_TMR_STATE_INITIALIZED;
    }

	return ADI_TMR_SUCCESS;
}

/**
 * @brief       Opens a given timer.
 *
 * @details     Opens a given timer device and returns the handle to the timer
 *              which should be used in other APIs to configure the timer.
 *
 * @param [in]  nTimerNum   Timer number to open.
 * @param [in]  pMemory     Pointer to the memory which is required to operate the timer instance.
 *                          The size of the memory should be equal to #ADI_TMR_MEMORY bytes.
 * @param [in]  nMemSize    Size of the given memory in bytes.
 * @param [in]  pfCallback  Pointer to application callback function to notify events.
 * @param [in]  pCBParam    Pointer to callback parameter which will be passed
 *                          back to the application when callback function is called.
 * @param [out] phTimer     Pointer to a location where the handle to the opened
 *                          timer is written.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS                   If successfully opened the given timer.
 * - #ADI_TMR_IN_USE                [D] If the given timer is in use.
 * - #ADI_TMR_INVALID_POINTER       [D] If any of the given pointer are invalid.
 * - #ADI_TMR_INSUFFICIENT_MEMORY   [D] If given memory is not sufficient to operate the timer
 *                                      instance.
 * - #ADI_TMR_INIT_FAILED               If timer service initialization failed.
 * - #ADI_TMR_INT_FAILED                If failed to register interrupt handler with the interrupt dispatcher.
 *
 * @sa          adi_tmr_Close()
 */
ADI_TMR_RESULT  adi_tmr_Open (
    uint32_t                nTimerNum,
    void *            const pMemory,
    uint32_t                nMemSize,
    ADI_CALLBACK            pfCallback,
    void *            const pCBParam,
    ADI_TMR_HANDLE   *const phTimer
)
{
    ADI_TMR_DATA   *pTmrData = (ADI_TMR_DATA *)pMemory;
    ADI_TMR_DEV    *pDevice;

#ifdef ADI_DEBUG

    /* assert to validate the memory size constant */
    assert(ADI_TMR_MEMORY == sizeof(ADI_TMR_DATA));

    /* Check if the timer number is valid */
    if(nTimerNum >= ADI_NUM_TIMERS)
    {
        return ADI_TMR_INVALID_TIMERID;
    }

    /* Check if the pointer parameters are valid */
    if((pMemory == NULL) || (phTimer == NULL) || (pfCallback == NULL))
    {
        return ADI_TMR_INVALID_POINTER;
    }

    /* Check if the given memory is sufficient for the device operation */
    if(nMemSize < ADI_TMR_MEMORY)
    {
        return ADI_TMR_INSUFFICIENT_MEMORY;
    }
#endif /* ADI_DEBUG */

    /* Initialize the timer service initialization */
    if(adi_tmr_Init() != ADI_TMR_SUCCESS)
    {
        return ADI_TMR_INIT_FAILED;
    }

    /* Get the pointer to timer device */
    pDevice = &gaTimers[nTimerNum];

    /* Check if the timer is available */
    if(pDevice->pDevData != NULL)
    {
        return ADI_TMR_IN_USE;
    }

    /* Clear the given memory */
    memset(pMemory, 0, nMemSize);

    /* Store the given callback and callback parameter */
    pTmrData->pfCallback    =   pfCallback;
    pTmrData->pCBParam      =   pCBParam;

    /*======== Initialize timer registers ======= */

    /* Stop timer and clear the graceful stop configuration */
    *pREG_TIMER0_STOP_CFG_CLR    =  (TMR_REG_TYPE) (1u << nTimerNum);
    *pREG_TIMER0_RUN_CLR         =  (TMR_REG_TYPE) (1u << nTimerNum);

    /* Clear the trigger input and output masks */
    *pREG_TIMER0_TRG_MSK         &=  (TMR_REG_TYPE) (~(1u << nTimerNum));
    *pREG_TIMER0_TRG_IE          &=  (TMR_REG_TYPE) (~(1u << nTimerNum));

    /* Clear any previous errors */
    *pREG_TIMER0_ERR_TYPE        &=  (~((uint32_t)BITM_TIMER_ERR_TYPE_TERR0 << nTimerNum));

    /* Save the pointer to the device data, to mark the timer as in use */
    adi_osal_EnterCriticalRegion();
    pDevice->pDevData   =   pTmrData;
    adi_osal_ExitCriticalRegion();   

    /* Register data interrupt handler for the timer */
    if(adi_int_InstallHandler(pDevice->nDataIntSID,
                              DataIntHandler,
                              pDevice,
                              true
                              ) != ADI_INT_SUCCESS)
    {
        adi_osal_EnterCriticalRegion();
        pDevice->pDevData   =   NULL;
        adi_osal_ExitCriticalRegion();   
        
        return ADI_TMR_INT_FAILED;
    }


    /* By default enable status and data interrupts (0 for enable, 1 for disable) */
    *pREG_TIMER0_DATA_IMSK   &=  (TMR_REG_TYPE) ~(1u << nTimerNum);
    *pREG_TIMER0_STAT_IMSK   &=  (TMR_REG_TYPE) ~(1u << nTimerNum);

    /* Return the timer handle */
    *phTimer    =   (ADI_TMR_HANDLE) pDevice;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Closes the given timer.
 *
 * @details     Once the timer is closed the handle to the timer will be
 *              invalid. It cannot be used to access any of the timer APIs.
 *
 *              The API stops the timer immediately before closing it. To
 *              gracefully stop the timer enaable the timer to stop gracefully
 *              using API #adi_tmr_EnableGracefulStop and disable the timer by
 *              using API #adi_tmr_Enable. Once the timer is stopped (The last
 *              timer interrupt is received), this API can be called to
 *              close the timer.
 *
 * @param [in]  hTimer      Handle to the timer.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully closed the given timer.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_TMR_INT_FAILED            Failed to uninstall the data interrupt handler.
 *
 * @sa adi_tmr_Open()
 * @sa adi_tmr_Enable()
 * @sa adi_tmr_EnableGracefulStop()
 */
ADI_TMR_RESULT  adi_tmr_Close (
    ADI_TMR_HANDLE    const hTimer
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;
    uint32_t         nTimerID;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    nTimerID = pDevice->nTimerID;

    /* Disable the timer before closing it */
    *pREG_TIMER0_RUN_CLR =  (TMR_REG_TYPE)(1u << nTimerID);

    /* Clear the interrupts mask and latch registers before uninstalling the
     * interrupt handler.
     */
    *pREG_TIMER0_DATA_IMSK   |= (TMR_REG_TYPE)(1u << nTimerID);
    *pREG_TIMER0_STAT_IMSK   |= (TMR_REG_TYPE)(1u << nTimerID);
    *pREG_TIMER0_DATA_ILAT   |= (TMR_REG_TYPE) nTimerID;
    *pREG_TIMER0_STAT_ILAT   |= (TMR_REG_TYPE) nTimerID;

    /* Uninstall the data interrupt handler for this timer */
    if(adi_int_UninstallHandler(pDevice->nDataIntSID) != ADI_INT_SUCCESS)
    {
        return ADI_TMR_INT_FAILED;
    }

    /* Free the timer by clearing the device instance data pointer */
    adi_osal_EnterCriticalRegion();
    pDevice->pDevData   =   NULL;
    adi_osal_ExitCriticalRegion();

    return ADI_TMR_SUCCESS;
}

/**
 * @brief       Sets the timer operation mode.
 *
 * @details     Sets the timer operation mode. Can be set only when timer is
 *              disabled.
 *
 * @param [in]  hTimer      Handle to the timer.
 * @param [in]  eTimerMode  Mode of operation. It can be one of the modes
 *                          listed by the enumeration #ADI_TMR_MODE.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully set the operation mode.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_OPERATION [D] If trying to set the mode when the timer is running.
 */
ADI_TMR_RESULT  adi_tmr_SetMode(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_MODE            eTimerMode
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & ((TMR_REG_TYPE)(1u << pDevice->nTimerID)) ) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Clear the previous mode and set the new mode */
    pDevice->pTmrRegs->CFG  =  ( (pDevice->pTmrRegs->CFG & (TMR_REG_TYPE)(~BITM_TIMER_TMR_CFG_TMODE)) | (TMR_REG_TYPE) eTimerMode);

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets timer width window.
 *
 * @details     Sets the width window for width watchdog mode
 *              (#ADI_TMR_MODE_WIDTH_WD). The timer should be setup in width window watchdog
 *              mode before calling this API.
 *
 *              The timer can be setup to monitor the timer input signal and generate an
 *              event (#ADI_TMR_IRQMODE_INSIDE_WINDOW or #ADI_TMR_IRQMODE_OUTSIDE_WINDOW)
 *              when the width of the timer input signal falls within the configured window
 *              or outside the configured window.
 *
 *              Application can choose to get the the event when the timer input signal falls 
 *              outside the window or inside the window by setting the IRQ mode to
 *              ADI_TMR_IRQMODE_INSIDE_WINDOW or ADI_TMR_IRQMODE_OUTSIDE_WINDOW
 *              using the API adi_tmr_SetIRQMode.
 *
 *              The timer input signal can be selected using the API #adi_tmr_SetInputSource.
 *
 *
 * @param [in]  hTimer      Handle to the timer.
 *
 * @param [in]  nMinWidth   Minimum width in terms of SCLK or ALTCLK cycles. If it required
 *                          check only the upper limit on the pulse width, then Minimum width
 *                          should be set to 0 or 1 and IRQ mode should be set to
 *                          #ADI_TMR_IRQMODE_OUTSIDE_WINDOW (using API adi_tmr_SetIRQMode).
 *
 * @param [in]  nMaxWidth   Maximum width in terms of SCLK or ALTCLK cycles. Maximum width
 *                          should be greater than the minimum width and should not be less
 *                          than 1.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully set window width.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the timer handle is invalid.
 * - #ADI_TMR_INVALID_PARAMETER [D] If maximum width less than minimum width or less than 1.
 * - #ADI_TMR_INVALID_OPERATION [D] If trying to set width parameters when not in width window
 *                                  watchdog mode or when the timer is running.
 *
 * @sa adi_tmr_SetInputSource()
 * @sa adi_tmr_SetIRQMode()
 * @sa adi_tmr_SetMode()
 * @sa adi_tmr_SetClkInSource()
 */
ADI_TMR_RESULT  adi_tmr_SetWidthWindow(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nMinWidth,
    uint32_t                nMaxWidth
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Validate the width parameters */
    if((nMaxWidth < 1u) || (nMinWidth < nMaxWidth))
    {
        return ADI_TMR_INVALID_PARAMETER;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (uint32_t)(1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }

    /* Check if the mode is set to width window watchdog mode */
    if( ((ADI_TMR_MODE)(pDevice->pTmrRegs->CFG & (TMR_REG_TYPE)BITM_TIMER_TMR_CFG_TMODE) ) != ADI_TMR_MODE_WIDTH_WD)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Minimum value is stored in delay register */
    pDevice->pTmrRegs->DLY  =   nMinWidth;

    /* Maximum value is stored in the period register */
    pDevice->pTmrRegs->PER  =   nMaxWidth;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets timer period window.
 *
 * @details     Sets the period window for period watchdog mode
 *              (#ADI_TMR_MODE_PERIOD_WD). The timer should be setup in period window
 *              watchdog mode before calling this API.
 *
 *              The timer can be setup to monitor the timer input signal and generate an
 *              event (ADI_TMR_EVENT_EDGE_INSIDE_WINDOW or ADI_TMR_EVENT_EDGE_OUTSIDE_WINDOW)
 *              when the period of the timer input signal falls within the configured window
 *              or outside the configured window. 
 * 
 *              Application can choose to get the the event when the timer input signal falls 
 *              outside the window or inside the window by setting the IRQ mode to
 *              ADI_TMR_IRQMODE_INSIDE_WINDOW or ADI_TMR_IRQMODE_OUTSIDE_WINDOW
 *              using the API adi_tmr_SetIRQMode.
 *
 *              The timer input signal can be selected using the API #adi_tmr_SetInputSource.
 *
 * @param [in]  hTimer      Handle to the timer.
 *
 * @param [in]  nMinPeriod  Minimum period in terms of SCLK or ALTCLK cycles. If it is required
 *                          to check only the upper limit on the pulse period, then Minimum period
 *                          should be set to 0 or 1 and IRQ mode should be set to
 *                          #ADI_TMR_IRQMODE_OUTSIDE_WINDOW (using API adi_tmr_SetIRQMode).
 *
 * @param [in]  nMaxPeriod  Maximum period in terms of SCLK or ALTCLK cycles. Maximum period
 *                          should be greater than the minimum period and should not be
 *                          less than 1.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully set window width.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the timer handle is invalid.
 * - #ADI_TMR_INVALID_PARAMETER [D] If maximum period less than minimum period or less than 1.
 * - #ADI_TMR_INVALID_OPERATION [D] If trying to set period parameters when not in period window
 *                                  watchdog mode or when the timer is running.
 */
ADI_TMR_RESULT  adi_tmr_SetPeriodWindow(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nMinPeriod,
    uint32_t                nMaxPeriod
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Validate the period parameters */
    if((nMaxPeriod < 1u) || (nMinPeriod < nMaxPeriod))
    {
        return ADI_TMR_INVALID_PARAMETER;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE)(1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }

    /* Check if the mode is set to period window watchdog mode */
    if( ((ADI_TMR_MODE)(pDevice->pTmrRegs->CFG & (TMR_REG_TYPE) BITM_TIMER_TMR_CFG_TMODE)) != ADI_TMR_MODE_PERIOD_WD)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Minimum value is stored in delay register */
    pDevice->pTmrRegs->DLY  =   nMinPeriod;

    /* Maximum value is stored in the period register */
    pDevice->pTmrRegs->PER  =   nMaxPeriod;

    return ADI_TMR_SUCCESS;
}

/**
 * @brief       Sets the timer IRQ mode.
 *
 * @details     Sets the condition for when the timer interrupt is generated.
 *              The possible IRQ modes are listed by the enumeration #ADI_TMR_IRQ_MODE.
 *
 *              IRQ Mode cannot be set when the timer is running.
 *
 * @param [in]  hTimer      Handle to the timer.
 * @param [in]  eIRQMode    IRQ mode to be set.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully set the IRQ mode.
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *  - #ADI_TMR_INVALID_OPERATION    [D] If trying to set the IRQ mode when the timer is
 *                                      running.
 */
ADI_TMR_RESULT  adi_tmr_SetIRQMode(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_IRQ_MODE        eIRQMode
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE)(1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Clear the previous IRQ mode and set the new mode */
    pDevice->pTmrRegs->CFG  =  ( (pDevice->pTmrRegs->CFG & (TMR_REG_TYPE) (~BITM_TIMER_TMR_CFG_IRQMODE)) | (TMR_REG_TYPE) eIRQMode);

    return ADI_TMR_SUCCESS;
}

/**
 * @brief       Sets the pulse polarity.
 *
 * @details     Sets the polarity of the pulse. Indicates how positive edge
 *              and negative edges to be treated.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bActiveHigh 'true'  - Positive edge is treated as active.
 *                          'false' - Negative edge is treated as active.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully set the polarity of the pulse.
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *  - #ADI_TMR_INVALID_OPERATION    [D] If trying to set the pulse polarity when the
 *                                      timer is running.
 *
 */
ADI_TMR_RESULT  adi_tmr_SetPulsePolarity(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bActiveHigh
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE)(1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* If positive edge should be treated as active high */
    if(bActiveHigh == true)
    {
        /* Set the PULSEHI bit */
        pDevice->pTmrRegs->CFG  |=  (TMR_REG_TYPE)ENUM_TIMER_TMR_CFG_POS_EDGE;
    }
    else
    {
        /* Clear the PULSEHI bit */
        pDevice->pTmrRegs->CFG  &=  (TMR_REG_TYPE)(~BITM_TIMER_TMR_CFG_PULSEHI);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets the clock input source.
 *
 * @details     Set the clock input source for the timer. The timer is clocked
 *              based on configured clock input source.
 *
 *              Clock input source can be set only when the timer is stopped.
 *
 * @param [in]  hTimer      Handle to the timer.
 * @param [in]  eClkIn      Clock input source.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully set clock input source
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *  - #ADI_TMR_INVALID_OPERATION    [D] If trying to set the clock input source when
 *                                      the timer is running.
 */
ADI_TMR_RESULT  adi_tmr_SetClkInSource(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_CLKIN           eClkIn
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE)(1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* Clear the previous clock input source and set the new source */
    pDevice->pTmrRegs->CFG  =  ( (pDevice->pTmrRegs->CFG & (TMR_REG_TYPE)(~BITM_TIMER_TMR_CFG_CLKSEL)) | (TMR_REG_TYPE) eClkIn);

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets the timer input source.
 *
 * @details     Selects the timer input which will be used in Width capture,
 *              watchdog window mode and pulse interrupt mode.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bAuxIn      'true'  - Use TMR_AUX_IN input.
 *                          'false' - Use TMR_IN input.
 * @return      Status
 *
 * - #ADI_TMR_SUCCESS                   If successfully set timer input source
 * - #ADI_TMR_INVALID_HANDLE        [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_OPERATION     [D] If trying to set the input source when
 *                                      the timer is running.
 */
ADI_TMR_RESULT  adi_tmr_SetInputSource(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bAuxIn
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE)(1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* If auxiliary input to be used as input */
    if(bAuxIn == true)
    {
        /* Set the TINSEL bit */
        pDevice->pTmrRegs->CFG  |=  (TMR_REG_TYPE) ENUM_TIMER_TMR_CFG_TINSEL_AUX;
    }
    else
    {
        /* Clear the TINSEL bit */
        pDevice->pTmrRegs->CFG  &=  (TMR_REG_TYPE) (~BITM_TIMER_TMR_CFG_TINSEL);
    }

    return ADI_TMR_SUCCESS;
}

/**
 * @brief       Disables/enables the timer pad output.
 *
 * @details     The timer output can be disabled while the timer is still running.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bDisable    'true'  - Disable timer pad output
 *                          'false' - Enable timer pad output
  * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully disabled/enabled timer pad output
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *
 */
ADI_TMR_RESULT  adi_tmr_DisableOutput(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bDisable
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* If timer pad output to be disabled */
    if(bDisable == true)
    {
        /* Set the OUTDIS bit */
        pDevice->pTmrRegs->CFG  |=  (TMR_REG_TYPE)ENUM_TIMER_TMR_CFG_PADOUT_DIS;
    }
    else
    {
        /* Clear the OUTDIS bit */
        pDevice->pTmrRegs->CFG  &=  (TMR_REG_TYPE)(~BITM_TIMER_TMR_CFG_OUTDIS);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Enables/Disables broadcast setting option.
 *
 * @details     Enables/Disables the broadcast setting option for
 *              DELAY, WIDTH or PERIOD. The DELAY, WIDTH and PERIOD parameters
 *              of the timers can be set at once for all the timers whose
 *              broadcast enable bits are set.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  eParam      Parameter (DELAY, WIDTH or PERIOD) for which the
 *                          broadcast option to be enabled/disabled.
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          broadcast setting for the given timer.
 *                          'true' - To enable broadcast.
 *                          'false' - To disable broadcast.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS                   If successfully enabled/disabled
 *                                      broadcast option for the given parameter.
 * - #ADI_TMR_INVALID_HANDLE        [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_OPERATION     [D] If trying to enable/disable broadcast option
 *                                      when the timer is running.
 */
ADI_TMR_RESULT  adi_tmr_EnableBroadcast(
    ADI_TMR_HANDLE    const hTimer,
    ADI_TMR_BCAST_PARAM     eParam,
    bool                    bEnable
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE)(1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* If broad cast to be enabled for the given parameter type */
    if(bEnable == true)
    {
        /* Set the bit to enable the broadcast option for the given parameter type */
        pDevice->pTmrRegs->CFG  |=  (TMR_REG_TYPE) eParam;
    }
    else
    {
        /* Clear the bit to disable the broadcast option for the given parameter type */
        pDevice->pTmrRegs->CFG  &=   ~((TMR_REG_TYPE)eParam);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets the emulation run option.
 *
 * @details     By enabling this option, timer counter can be left counting
 *              during emulation.
 *
 *              This option can be set only when the timer is stopped.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bRun        'true'  -   Timer counter runs during emulation.
 *                          'false' -   Timer counter stops during emulation.
 * @return      Status
 * - #ADI_TMR_SUCCESS                   If successfully set the emulation run option
 * - #ADI_TMR_INVALID_HANDLE        [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_OPERATION     [D] If trying to set the emulation run option
 *                                      when the timer is running.
 */
ADI_TMR_RESULT  adi_tmr_SetEmuRun(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bRun
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE) (1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */

    /* If the timer counter should be kept counting during emulation */
    if(bRun == true)
    {
        /* Set the EMURUN bit */
        pDevice->pTmrRegs->CFG  |=  (TMR_REG_TYPE) ENUM_TIMER_TMR_CFG_EMU_CNT;
    }
    else
    {
        /* Clear the EMURUN bit */
        pDevice->pTmrRegs->CFG  &=  (TMR_REG_TYPE) (~BITM_TIMER_TMR_CFG_EMURUN);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets the timer period.
 *
 * @details     The timer period cannot be set in following timer modes:
 *              - Width Capture mode (#ADI_TMR_MODE_CAPTURE_ASSERT or
 *                ADI_TMR_MODE_CAPTURE_DEASSERT)
 *              - Pin interrupt mode (#ADI_TMR_MODE_PINT)
 *
 *              For watchdog modes (ADI_TMR_MODE_PERIOD_WD and
 *              ADI_TMR_MODE_PERIOD_WD) use the APIs adi_tmr_SetPeriodWindow
 *              and adi_tmr_SetWidthWindow.
 *
 * @param [in]  hTimer      Handle to the timer.
 * @param [in]  nPeriod     Timer period to be set.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully set the timer period
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *  - #ADI_TMR_INVALID_OPERATION    [D] If trying to set the period when in the
 *                                      timer modes in which it cannot not be
 *                                      set.
 *
 * @sa adi_tmr_SetPeriodWindow()
 * @sa adi_tmr_SetWidthWindow()
 * @sa adi_tmr_SetMode()
 * @sa adi_tmr_GetPeriod()
 */
ADI_TMR_RESULT  adi_tmr_SetPeriod(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nPeriod
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    ADI_TMR_MODE    eTimerMode;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Get the current mode */
    eTimerMode = (ADI_TMR_MODE) (pDevice->pTmrRegs->CFG & (uint16_t) BITM_TIMER_TMR_CFG_TMODE);

    /* It is not allowed to set the period in these modes */
    if((eTimerMode == ADI_TMR_MODE_CAPTURE_ASSERT)  ||
       (eTimerMode == ADI_TMR_MODE_CAPTURE_DEASSERT)||
       (eTimerMode == ADI_TMR_MODE_PINT))
    {
        return ADI_TMR_INVALID_OPERATION;
    }

#endif /* ADI_DEBUG */

    /* Set the given period */
    pDevice->pTmrRegs->PER  =   nPeriod;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets the timer width.
 *
 * @details     The timer width can be only set PWMOUT modes
 *              (#ADI_TMR_MODE_SINGLE_PWMOUT and #ADI_TMR_MODE_CONTINUOUS_PWMOUT)
 *
 * @param [in]  hTimer     Handle to the timer.
 * @param [in]  nWidth     Timer width to be set.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully set the timer width.
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *  - #ADI_TMR_INVALID_OPERATION    [D] If trying to set the width when when in the
 *                                      timer modes in which it cannot not be
 *                                      set.
 *
 * @sa adi_tmr_SetPeriodWindow()
 * @sa adi_tmr_SetWidthWindow()
 * @sa adi_tmr_SetMode()
 * @sa adi_tmr_GetWidth()
 */
ADI_TMR_RESULT  adi_tmr_SetWidth(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nWidth
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    ADI_TMR_MODE    eTimerMode;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Get the current mode */
    eTimerMode = (ADI_TMR_MODE) (pDevice->pTmrRegs->CFG & (uint16_t) BITM_TIMER_TMR_CFG_TMODE);

    /* It is allowed to set the width only in these modes */
    if((eTimerMode != ADI_TMR_MODE_SINGLE_PWMOUT) &&
       (eTimerMode != ADI_TMR_MODE_CONTINUOUS_PWMOUT))
    {
        return ADI_TMR_INVALID_OPERATION;
    }

#endif /* ADI_DEBUG */

    /* Set the given width */
    pDevice->pTmrRegs->WID  =   nWidth;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets the timer delay.

 * @details     The timer delay can be only set PWMOUT modes
 *              (#ADI_TMR_MODE_SINGLE_PWMOUT and #ADI_TMR_MODE_CONTINUOUS_PWMOUT)
 *
 *              For watchdog modes (ADI_TMR_MODE_PERIOD_WD and
 *              ADI_TMR_MODE_PERIOD_WD) use the APIs adi_tmr_SetPeriodWindow
 *              and adi_tmr_SetWidthWindow.
 *
 *
 * @param [in]  hTimer     Handle to the timer
 * @param [in]  nDelay     Timer delay to be set
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully set the timer delay
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *  - #ADI_TMR_INVALID_OPERATION    [D] If trying to set the delay when when in the
 *                                      timer modes in which it cannot not be
 *                                      set.
 *
 * @sa adi_tmr_GetDelay()
 */
ADI_TMR_RESULT  adi_tmr_SetDelay(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t                nDelay
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    ADI_TMR_MODE    eTimerMode;

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Get the current mode */
    eTimerMode = (ADI_TMR_MODE) (pDevice->pTmrRegs->CFG & (uint16_t)BITM_TIMER_TMR_CFG_TMODE);

    /* It is allowed to set the width only in these modes */
    if((eTimerMode != ADI_TMR_MODE_SINGLE_PWMOUT) &&
       (eTimerMode != ADI_TMR_MODE_CONTINUOUS_PWMOUT)&&
       (eTimerMode != ADI_TMR_MODE_PERIOD_WD) &&
       (eTimerMode != ADI_TMR_MODE_WIDTH_WD))
    {
        return ADI_TMR_INVALID_OPERATION;
    }

#endif /* ADI_DEBUG */

    /* Set the given delay */
    pDevice->pTmrRegs->DLY  =   nDelay;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Broadcast timer period.
 *
 * @details     This can be used to set the period for all the timers which are
 *              enabled to accept period value that is broadcasted.
 *
 * @param [in]  nPeriod     Timer period to be broadcasted.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS      if successfully broadcasted the timer period.
 *
 * @sa          adi_tmr_EnableBroadcast()
 */
ADI_TMR_RESULT  adi_tmr_BCastPeriod(
    uint32_t                nPeriod
)
{
    /* Set the given period in the period broadcast register */
    *pREG_TIMER0_BCAST_PER  =   nPeriod;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Broadcast timer width.
 *
 * @details     This can be used to set the width for all the timers which are
 *              enabled to accept width value that is broadcasted.

 * @param [in]  nWidth     Timer width to be set
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS          If successfully broadcasted the timer width.
 *
 * @sa          adi_tmr_EnableBroadcast()
 */
ADI_TMR_RESULT  adi_tmr_BCastWidth(
    uint32_t                nWidth
)
{
    /* Set the given width in the width broadcast register */
    *pREG_TIMER0_BCAST_WID  =   nWidth;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Broadcast timer delay.
 *
 * @details     This can be used to set the delay for all the timers which are
 *              enabled to accept delay value that is broadcasted.
 *
 * @param [in]  nDelay     Timer delay to be set
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS          If successfully broadcasted the timer delay.
 *
 * @sa          adi_tmr_EnableBroadcast()
 *
 */
ADI_TMR_RESULT  adi_tmr_BCastDelay(
    uint32_t                nDelay
)
{
    /* Set the given delay in the delay broadcast register */
     *pREG_TIMER0_BCAST_DLY  =   nDelay;

     return ADI_TMR_SUCCESS;
}

/**
 * @brief       Enable/Disable the timer to stop gracefully.
 *
 * @details     When the timer is used in PWM mode and not enabled to stop gracefully,
 *              it will be stopped immediately after calling this API. This may corrupt
 *              PWM signal pattern.
 *
 * @param [in]  hTimer      Handle to the timer.
 *  @param [in]  bGraceful  Flag which indicates whether to gracefully disable the
 *                          timer or immediately stop it. This parameter will be used
 *                          only when the timer is being disabled.
 *
 *                          'true'   - Gracefully disable the timer (useful in
 *                                     PWM_OUT mode where the timer is stopped after
 *                                     current waveform period)
 *                          'false'  - Disable immediately.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully enabled/disabled the timer
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 *
 */
ADI_TMR_RESULT  adi_tmr_EnableGracefulStop(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bGraceful
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG
    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    /* If the timer to be stopped Gracefully */
    if(bGraceful == true)
    {
        *pREG_TIMER0_STOP_CFG_SET   =   (TMR_REG_TYPE) (1u << pDevice->nTimerID);
    }
    else
    {
        *pREG_TIMER0_STOP_CFG_CLR   =   (TMR_REG_TYPE) (1u << pDevice->nTimerID);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Enable/Disable the timer.
 *
 * @details     When the timer is used in PWM mode and not enabled to stop gracefully,
 *              it will be stopped immediately after calling this API. This may corrupt
 *              PWM signal pattern. To avoid this the timer can be enabled to stop
 *              gracefully using the API #adi_tmr_EnableGracefulStop.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bEnable     Flag which indicates whether to enable or disable the timer.
 *                          'true'   - enable the timer.
 *                          'false'  - disable the timer.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully enabled/disabled the timer
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 *
 * @sa adi_tmr_EnableGracefulStop()
 */
ADI_TMR_RESULT  adi_tmr_Enable(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;
    uint16_t        nTmrMask;

#ifdef ADI_DEBUG
    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }
#endif /* ADI_DEBUG */

    nTmrMask = (uint16_t) (1u << pDevice->nTimerID);

    /* Enable/disable the timer */
    EnableTimers(nTmrMask, bEnable);

    return ADI_TMR_SUCCESS;
}



/**
 * @brief       Enable/Disable a group of timers.
 *
 * @details     Enable or disable group timers at once. This allows the timers
 *              to run synchronously.
 *
 *              When the timer is used in PWM mode and not enabled to stop gracefully,
 *              it will be stopped immediately after calling this API. This may corrupt
 *              PWM signal pattern. To avoid this the timer can be enabled to stop
 *              gracefully using the API #adi_tmr_EnableGracefulStop.
 *
 * @param [in]  TmrGroupMsk Bitmask of timers to be enabled/disabled. Bit 0
 *                          corresponds to timer0, Bit 2 for timer2 and so on.
 *
 * @param [in]  bEnable     Flag which indicates whether to enable or disable the timers.
 *                          'true'   - enable the group.
 *                          'false'  - disable the group.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS      If successfully enabled/disabled the timers.
 *
 * @sa adi_tmr_EnableGracefulStop()
 */
ADI_TMR_RESULT  adi_tmr_GroupEnable(
    uint32_t                TmrGroupMsk,
    bool                    bEnable
)
{
    /* Enable/disable the timers */
    EnableTimers ((uint16_t)TmrGroupMsk, bEnable);

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Sets the trigger slave mode.
 *
 * @details     Sets the trigger slave mode. Timer can be started or stopped
 *              upon receiving a trigger input. This API cannot be called
 *              when the timer is running.
 *
 * @param [in]  hTimer      Handle to the timer
 *
 * @param [in]  bRun        Flag to indicate whether to stop or start the timer
 *                          upon receiving a trigger pulse.
 *
 *                          'true'  - start the timer if is stopped.
 *                          'false' - stop the timer if it running. When bGraceful
 *                                    is true, timer will be stopped gracefully.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS                  If successfully set the trigger slave mode.
 *  - #ADI_TMR_INVALID_HANDLE       [D] If the given timer handle is invalid.
 *  - #ADI_TMR_INVALID_OPERATION    [D] If trying to set the trigger slave mode when the
 *                                      timer is running.
 */
ADI_TMR_RESULT  adi_tmr_SetTrigSlaveMode(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bRun
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Check if the timer is running */
    if((*pREG_TIMER0_RUN & (TMR_REG_TYPE) (1u << pDevice->nTimerID)) != 0u)
    {
        return ADI_TMR_INVALID_OPERATION;
    }
#endif /* ADI_DEBUG */


    if(bRun == true)
    {
        /* Set the slave trigger mode bit to start the timer upon
           receiving trigger pulse */
        pDevice->pTmrRegs->CFG  |=  (TMR_REG_TYPE) ENUM_TIMER_TMR_CFG_TRIGSTART;
    }
    else
    {
        /* Clear the slave trigger mode bit to stop the timer upon
           receiving trigger pulse */
        pDevice->pTmrRegs->CFG  &=  (uint16_t) (~BITM_TIMER_TMR_CFG_SLAVETRIG);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Enable/Disable the trigger input.
 *
 * @details     This will enable/disable the timer to act as a trigger slave.
 *              When the timer is enabled to be a trigger slave, it can be
 *              started or stopped upon receiving the trigger. The timer can
 *              be configured to start/stop when trigger is received using the
 *              API #adi_tmr_SetTrigSlaveMode.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bEnable     Flag to indicate whether to enable/disable the
 *                          trigger input.
 *                          'true'   - enable the trigger input.
 *                          'false'  - disable the trigger input.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS              If successfully enabled/disabled the trigger input.
 *  - #ADI_TMR_INVALID_HANDLE   [D] If the given timer handle is invalid.
 *
 * @sa adi_tmr_SetTrigSlaveMode()
 */
ADI_TMR_RESULT  adi_tmr_EnableTriggerIn(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */

    /* If trigger input to be enabled */
    if(bEnable == true)
    {
        *pREG_TIMER0_TRG_IE      |=  (TMR_REG_TYPE) (1u << pDevice->nTimerID);
    }
    /* Trigger input to be disabled */
    else
    {
        *pREG_TIMER0_TRG_IE      &=  (TMR_REG_TYPE) ~(1u << pDevice->nTimerID);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Enable/Disable the trigger output.
 *
 * @details     A timer can be used as a trigger master. This will enable/disable
 *              a timer to be a trigger master.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bEnable     Flag which indicates whether the timer to be
 *                          enabled/disabled as trigger master.
 *                          'true'   - enable the trigger output
 *                          'false'  - disable the trigger output
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS              If successfully enabled/disabled the timer as
 *                                  trigger master.
 *  - #ADI_TMR_INVALID_HANDLE   [D] If the given timer handle is invalid.
 *
 */
ADI_TMR_RESULT  adi_tmr_EnableTriggerOut(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */

    /* If trigger output to be enabled */
    if(bEnable == true)
    {
        *pREG_TIMER0_TRG_MSK      |=  (TMR_REG_TYPE) (1u << pDevice->nTimerID);
    }
    /* Trigger output to be disabled */
    else
    {
        *pREG_TIMER0_TRG_MSK      &=  (TMR_REG_TYPE) ~(1u << pDevice->nTimerID);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Enable/Disable the data interrupt.
 *
 * @details     This will enable/disable the data interrupt generated by a timer.
 *              By default data interrupts are enabled when the timer is opened.
 *
 * @param [in]  hTimer      Handle to the timer
 * @param [in]  bEnable     Flag which indicates whether to enable/disable the
 *                          data interrupts.
 *                          'true'   - enable the data interrupt.
 *                          'false'  - disable the data interrupt.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS              If successfully enabled/disabled the data
 *                                  interrupt.
 *  - #ADI_TMR_INVALID_HANDLE   [D] If the given timer handle is invalid.
 *
 */
ADI_TMR_RESULT  adi_tmr_EnableDataInt(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */

    /* If data interrupt to be enabled */
    if(bEnable == true)
    {
        *pREG_TIMER0_DATA_IMSK   &=  (TMR_REG_TYPE) ~(1u << pDevice->nTimerID);
    }
    /* data interrupt to be disabled */
    else
    {
        *pREG_TIMER0_DATA_IMSK   |=  (TMR_REG_TYPE) (1u << pDevice->nTimerID);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Enable/Disable the status interrupt.
 *
 * @details     A status interrupt is generated to notify status events such
 *              as counter overflow, programming error of delay, width and period
 *              registers. This will enable/disable the notification of status
 *              events.
 *
 *              By default status interrupts are enabled when the timer is opened.
 *
 * @param [in]  hTimer      Handle to the timer.
 * @param [in]  bEnable     Flag which indicates whether to enable or disable
 *                          status interrupts.
 *                          'true'   - enable the status interrupt.
 *                          'false'  - disable the status interrupt.
 *
 * @return      Status
 *  - #ADI_TMR_SUCCESS              If successfully enabled/disabled the status
 *                                  interrupt.
 *  - #ADI_TMR_INVALID_HANDLE   [D] If the given timer handle is invalid.
 */
ADI_TMR_RESULT  adi_tmr_EnableStatusInt(
    ADI_TMR_HANDLE    const hTimer,
    bool                    bEnable
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

#endif /* ADI_DEBUG */

    /* If status interrupt to be enabled */
    if(bEnable == true)
    {
        *pREG_TIMER0_STAT_IMSK   &=  (TMR_REG_TYPE) ~(1u << pDevice->nTimerID);
    }
    /* status interrupt to be disabled */
    else
    {
        *pREG_TIMER0_STAT_IMSK   |=  (TMR_REG_TYPE) (1u << pDevice->nTimerID);
    }

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Gets the current timer counter value.
 *
 * @param [in]  hTimer     Handle to the timer.
 * @param [out] pCount     Pointer to a location where current count value is written.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully queried the timer counter.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_POINTER   [D] If given pointer to count value is invalid.
 */
ADI_TMR_RESULT  adi_tmr_GetCount(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pCount
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Validate the given pointer */
    if(pCount == NULL)
    {
        return ADI_TMR_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Get the current count value */
    *pCount = pDevice->pTmrRegs->CNT;

    return ADI_TMR_SUCCESS;
}

/**
 * @brief       Gets the timer delay value.
 *
 * @param [in]  hTimer     Handle to the timer.
 * @param [out] pDelay     Pointer to a location where delay value is written.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully queried the timer delay value.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_POINTER   [D] If given pointer to delay value is invalid.
 *
 * @sa adi_tmr_SetDelay()
 */
ADI_TMR_RESULT  adi_tmr_GetDelay(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pDelay
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Validate the given pointer */
    if(pDelay == NULL)
    {
        return ADI_TMR_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Get the delay value */
    *pDelay = pDevice->pTmrRegs->DLY;

    return ADI_TMR_SUCCESS;
}


/**
 * @brief       Gets the timer width value.
 *
 * @details     This API is useful to get the width of the input pulse when the
 *              timer is configured to be in capture mode.
 *
 * @param [in]  hTimer     Handle to the timer.
 * @param [out] pWidth     Pointer to a location where width value is written.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully queried the timer width value.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_POINTER   [D] If given pointer to width is invalid.
 *
 * @sa adi_tmr_SetWidth()
 * @sa adi_tmr_SetWidthWindow()
 * @sa adi_tmr_SetMode()
 */
ADI_TMR_RESULT  adi_tmr_GetWidth(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pWidth
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Validate the given pointer */
    if(pWidth == NULL)
    {
        return ADI_TMR_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Get the width value */
    *pWidth = pDevice->pTmrRegs->WID;

    return ADI_TMR_SUCCESS;
}

/**
 * @brief       Gets the timer period value.
 *
 * @details     This API is useful to get the period of the input pulse when the
 *              timer is configured to be in capture mode.
 *
 * @param [in]  hTimer     Handle to the timer.
 * @param [out] pPeriod    Pointer to a location where period value is written.
 *
 * @return      Status
 * - #ADI_TMR_SUCCESS               If successfully queried the timer period value.
 * - #ADI_TMR_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_TMR_INVALID_POINTER   [D] If given pointer to period is invalid.
 *
 * @sa adi_tmr_SetPeriod()
 * @sa adi_tmr_SetPeriodWindow()
 * @sa adi_tmr_SetMode()
 *
 */
ADI_TMR_RESULT  adi_tmr_GetPeriod(
    ADI_TMR_HANDLE    const hTimer,
    uint32_t *        const pPeriod
)
{
    ADI_TMR_RESULT  eResult;
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) hTimer;

#ifdef ADI_DEBUG

    /* Validate the given timer handle */
    if((eResult = ValidateDeviceHandle(pDevice)) != ADI_TMR_SUCCESS)
    {
        return eResult;
    }

    /* Validate the given pointer */
    if(pPeriod == NULL)
    {
        return ADI_TMR_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Get the period value */
    *pPeriod = pDevice->pTmrRegs->PER;

    return ADI_TMR_SUCCESS;
}


/* Status interrupt handler */
static void StatusIntHandler(
    uint32_t            IID,
    void               *pCBParam
)
{
    TMR_REG_TYPE        StatusILATCH;
    uint16_t        nTimerNum;
    uint16_t        nMask;

    /* Read the Status ILAT register */
    StatusILATCH = *pREG_TIMER0_STAT_ILAT;

    /* Handles the first latched interrupt that is found assuming
     * that the status interrupt is regenerated if the latch is
     * not cleared.
     */
    for(nTimerNum = 0u; nTimerNum < ADI_NUM_TIMERS; nTimerNum++)
    {
        nMask = (uint16_t)(1u << nTimerNum);

        /* Check if there is a pending status interrupt to be handled
         * this timer.
         */
        if((StatusILATCH & nMask) != 0u)
        {
            ADI_TMR_DATA *pDevData = gaTimers[nTimerNum].pDevData;

            /* Clear the handled status interrupt latch bit */
            *pREG_TIMER0_STAT_ILAT = nMask;

            /* FIX-ME: Use the error mask from CODA header file once it is fixed */

            /* Call the application handler */
            pDevData->pfCallback(pDevData->pCBParam,
                                (*pREG_TIMER0_ERR_TYPE & (0x3ul << nTimerNum)) >> nTimerNum,
                                 NULL);


            /* Handled the first status interrupt found, stop the loop */
            break;
        }
    }

    return;
}


/* Data interrupt handler */
static void DataIntHandler(
    uint32_t            IID,
    void               *pCBParam
)
{
    ADI_TMR_DEV     *pDevice = (ADI_TMR_DEV *) pCBParam;

    /* Clear the data interrupt latch */
    *pREG_TIMER0_DATA_ILAT = (TMR_REG_TYPE)(1u << pDevice->nTimerID);

    /* Call the application handler */
    pDevice->pDevData->pfCallback(pDevice->pDevData->pCBParam,
                                  (uint32_t)ADI_TMR_EVENT_DATA_INT,
                                  NULL);
    return;
}


/* Enables/disables group of timers */
static void EnableTimers(
    uint16_t nTmrMask,
    bool     bEnable
)
{
    /* If enabling the timers */
    if(bEnable == true)
    {
        *pREG_TIMER0_RUN_SET    =   nTmrMask;
    }
    else
    {
        *pREG_TIMER0_RUN_CLR    =   nTmrMask;
    }

    return;
}


#ifdef ADI_DEBUG

/*
 * Verify that the handle is valid
 */
static ADI_TMR_RESULT ValidateDeviceHandle (
    ADI_TMR_DEV     *const pDevice
)
{
    uint32_t nTimerNum;

    /* Check if the given pointer is valid */
    if(pDevice == NULL)
    {
    	return ADI_TMR_INVALID_POINTER;
    }

    /* For all the timers */
	for( nTimerNum = 0u; nTimerNum < ADI_NUM_TIMERS; nTimerNum++)
	{
	    /* Check if the given handle matches with this timer */
		if( pDevice == &gaTimers[nTimerNum])
		{
			return ADI_TMR_SUCCESS;
		}
	}

    return ADI_TMR_INVALID_HANDLE;
}
#endif

/*@}*/


#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif /* _ADI_TMR_BF6XX_C_ */
