/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_tru.c
 * @brief     Trigger Routing Unit Manager.
 * @version:  $Revision: 18484 $
 * @date:     $Date: 2013-04-26 10:04:51 -0400 (Fri, 26 Apr 2013) $
 *
 * @details
 *            This is the the primary source file for Trigger Routing Unit Manager
 *
 */

/** @addtogroup TRU_Manager Trigger Routing Unit Manager
 *  @{

<b>Trigger Routing Unit (TRU) Manager</b>

The TRU provides system-level sequence control without core intervention. The TRU
maps trigger masters (generators of triggers) to trigger slaves (receivers of triggers).
Slave endpoints can be configured to respond to triggers in various ways.

Common applications enabled by the TRU include:

    - Automatically triggering the start of a DMA sequence after a
      sequence from another DMA channel completes
    - Software triggering
    - Synchronization of concurrent activities

The TRU Manager supports the following features

    - Trigger routing of any trigger master to any trigger slave.
    - Software generation of any trigger master ID.

Trigger configuration can be set on a per-buffer basis for all Peripheral DMA data transfer modes
and One-Shot/Circular Memory DMA transfer modes. For DMA driven peripherals, application should
use trigger configuration APIs provided by the corresponding peripheral driver.

Sequence of function calls to route trigger master and slave

    - adi_tru_Open() opens the TRU device.
    - adi_tru_TriggerRoute() to route a trigger slave to a trigger master.
    - adi_tru_Enable() to enable trigger routing.
    - Trigger configuration APIs specific to master/slave device(s).
    - Configure master/slave device(s) to act on its trigger settings.
    - adi_tru_ConfigTriggerMaster() to generate up to four outgoing software-driven trigger
      per function call.
    - adi_tru_Enable() to disable trigger routing.
    - adi_tru_TriggerRoute () with master id as zero to clear the trigger routing that
      was established earlier
    - adi_tru_Close() closes the TRU device

 */

/*=============  I N C L U D E S   =============*/
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be longer than 31 characters")
#endif

/* TRU Manager includes */
#include <services/tru/adi_tru.h>
/* Include header file with data structure definitions used in TRU manager implementation */
#include "adi_tru_def.h"

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug mode enabled) */
#if defined (ADI_DEBUG)

#endif /* ADI_DEBUG */

/*=============  T R U    I M P L E M E N T A T I O N    S O U R C E    F I L E S  =============*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_6:"")
#pragma diag(suppress:misra_rule_5_7:"")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined in a common file")
#pragma diag(suppress:misra_rule_11_4:"")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exit for a function")
#pragma diag(suppress:misra_rule_16_7:"")
#endif /* _MISRA_RULES */

/* Processor specific data for TRU Service */
/* IF (ADSP-BF606/607/608/609) */
#if defined( __ADSPBF609_FAMILY__)
/* ADSP-BF60x Processor specific data for TRU manager */
#include "adi_tru_data_bf60x.c"
/* ELSE (ADSP-CM402/CM403/CM407/CM408 processors) */
#elif defined (ADI_ADSP_CM40Z)
/* ADSP-CM40z Processor specific data for TRU manager */
#include "adi_tru_data_cm40z.c"
#elif defined (__ADSPBF707_FAMILY__)
/* ADSP-BF70x Processor specific data for TRU manager */
#include "adi_tru_data_bf70x.c"
/* ELSE */
#else
#error "TRU Manager not available for this processor"
#endif

/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validates TRU device handle */
static ADI_TRU_RESULT ValidateTruHandle (ADI_TRU_HANDLE const  hDevice);

#endif /* ADI_DEBUG */

/*=============  C O D E  =============*/

/*=============  L O C A L    F U N C T I O N S  =============*/

/*=============  D E B U G   F U N C T I O N S =============*/

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

/* Validates TRU device handle */
static ADI_TRU_RESULT ValidateTruHandle (ADI_TRU_HANDLE const  hDevice)
{
    /* Loop variable */
    uint32_t    i;

    /* FOR (All CRC Device instances) */
    for (i = 0u; i < (sizeof (TruInfo) / sizeof (TruInfo[0])); i++)
    {
        /* IF (TRU handle is valid) */
        if (hDevice == &TruInfo[i])
        {
            /* Channel valid, return success */
            return (ADI_TRU_SUCCESS);
        }
    }

    /* Device ID invalid, return failure */
    return (ADI_TRU_FAILURE);
}

#endif /* ADI_DEBUG */

/*=============  P U B L I C   F U N C T I O N S =============*/

/**
 * @brief Opens a Trigger Routing Unit.
 *
 * @param [in]  DeviceNumber    Trigger Routing Unit Device number to open.
 * @param [in]  pMemory         Pointer to ADI_TRU_REQ_MEMORY sized memory
 *                              location to handle the TRU device.
 * @param [in]  MemorySize      Size of the buffer to which "pMemory" points.
 * @param [in]  phDevice        Address where the TRU device handle will be stored.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully opened trigger routing unit.
 *          - #ADI_TRU_IN_USE: TRU device is already in use.
 *
 */
ADI_TRU_RESULT  adi_tru_Open(
    uint32_t            DeviceNumber,
    void *const         pMemory,
    uint32_t            MemorySize,
    ADI_TRU_HANDLE      *phDevice)
{
    /* Return Code */
    ADI_TRU_RESULT      eResult = ADI_TRU_SUCCESS;
    /* Memory to handle TRU */
    ADI_TRU_DEV         *pDevice = (ADI_TRU_DEV *) pMemory;

    /* IF (TRU device is already in use) */
    if (TruInfo[DeviceNumber].pDevice != NULL)
    {
        /* Report failure as TRU is already in use */
        eResult = ADI_TRU_IN_USE;
    }

    /* IF (TRU device is available for use) */
    if (eResult == ADI_TRU_SUCCESS)
    {
        /* Entering critical region, disable interrupts */
        adi_osal_EnterCriticalRegion();

        /* Save the memory address allocated to handle this device channel */
        TruInfo[DeviceNumber].pDevice = pDevice;
        /* Save the device number */
        pDevice->DeviceNumber = (uint8_t) DeviceNumber;
        /* Soft-Reset TRU */
        TruInfo[DeviceNumber].pReg->GlobalControl |= BITM_TRU_GCTL_RESET;

        /* Re-enable interrupts */
	    adi_osal_ExitCriticalRegion();

	    /* Pass a valid handle to this TRU device */
        *phDevice = (ADI_TRU_HANDLE) &TruInfo[DeviceNumber];
	}

    return (eResult);
}

/**
 * @brief Closes a Trigger Routing Unit.
 *
 * @param [in]  hDevice     Handle to the TRU device to be closed.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully closed trigger routing unit.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 */
ADI_TRU_RESULT  adi_tru_Close(
    ADI_TRU_HANDLE      hDevice)
{
    /* Memory to handle TRU */
    ADI_TRU_INFO    *pDevInfo = (ADI_TRU_INFO *) hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Disable TRU */
    adi_tru_Enable (hDevice, false);
    /* Clear the device handle */
    pDevInfo->pDevice = NULL;

    return (ADI_TRU_SUCCESS);
}

/**
 * @brief Routes/Maps a trigger master to a slave.
 *
 * @details This API will make the given Slave to wait (listen) for the trigger from the
 *          given master. Multiple slaves can wait for the trigger from the single master.
 *          To achieve that this API should be called multiple times with different
 *          Slave ID but with same Master ID.
 *
 * @param [in]  hDevice             Handle to the TRU device to work on.
 * @param [in]  TriggerSlave        Trigger Slave ID.
 * @param [in]  TriggerMaster       Trigger Master ID.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully mapped a trigger master to slave.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 * @note
 *      The Trigger Master and Slave IDs are defined in processor definition file (defxxxx.h)
 *      defBF609.h - Definition file for ADSP-BF60x Processor family
 *      defCM40z.h - Definition file for ADSP-CM40z Processor family
 *
 */
ADI_TRU_RESULT  adi_tru_TriggerRoute(
    ADI_TRU_HANDLE      hDevice,
    uint32_t            TriggerSlave,
    uint32_t            TriggerMaster)
{
    /* Return Code */
    ADI_TRU_RESULT      eResult;
    /* Memory to handle TRU */
    ADI_TRU_INFO        *pDevInfo = (ADI_TRU_INFO *)hDevice;
    /* Pointer to Slave Select registers */
    volatile uint32_t   (*TriggerSlaveReg)[] = (volatile uint32_t(*)[])pDevInfo->pSSReg;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Map trigger master to slave */
    (*TriggerSlaveReg)[TriggerSlave] = TriggerMaster;

    return (ADI_TRU_SUCCESS);
}

/**
 * @brief Enables/Disables Trigger Routing Unit.
 *
 * Parameters:
 *
 * @param [in]  bEnable     'true' to enable Trigger routing, 'false' to disable.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully configured TRU.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 */
ADI_TRU_RESULT  adi_tru_Enable(
    ADI_TRU_HANDLE      hDevice,
    bool                bEnable)
{
    /* Return Code */
    ADI_TRU_RESULT      eResult;
    /* Memory to handle TRU */
    ADI_TRU_INFO        *pDevInfo = (ADI_TRU_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* IF (Enable TRU) */
    if (bEnable)
    {
        pDevInfo->pReg->GlobalControl |= BITM_TRU_GCTL_EN;
    }
    /* ELSE (Disable TRU) */
    else
    {
        pDevInfo->pReg->GlobalControl &= ~BITM_TRU_GCTL_EN;
    }

    return (ADI_TRU_SUCCESS);
}

/**
 * @brief Configure Trigger Master register to generate outgoing Trigger(s).
 *
 * @details This API configures the Trigger Master register. Any trigger including the
 *          software based triggers can be generated by calling this API
 *          The trigger will be generated only if the TRU is enabled by calling adi_tru_Enable()
 *
 * @param [in]  MasterTriggerVal    Trigger Master Register value to configure.
 *
 * @return  Status
 *          - #ADI_TRU_SUCCESS: Successfully configured TRU.
 *          - #ADI_TRU_BAD_HANDLE: TRU device handle is invalid.
 *
 */
ADI_TRU_RESULT adi_tru_ConfigTriggerMaster(
    ADI_TRU_HANDLE      hDevice,
    uint32_t            MasterTriggerVal)
{
    /* Return Code */
    ADI_TRU_RESULT      eResult;
    /* Memory to handle TRU */
    ADI_TRU_INFO        *pDevInfo = (ADI_TRU_INFO *)hDevice;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (TRU device handle is invalid) */
    if (ValidateTruHandle (hDevice) != ADI_TRU_SUCCESS)
    {
        return (ADI_TRU_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    /* Configure the Trigger Master register */
    pDevInfo->pReg->MasterTrigger = MasterTriggerVal;

    return (ADI_TRU_SUCCESS);
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*@}*/

