/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_wd_bf6xx.c
*
* @brief     Watch dog timer service global API implementation.
*
* @details
*            This is a source file which contains the implementation of global
*            API functions of the watch dog timer device.
*/


/*********************************************************************

Include files

*********************************************************************/
#ifndef _ADI_WD_BF6XX_C_
#define _ADI_WD_BF6XX_C_

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be longer than 31 characters")
#endif

#include <sys/platform.h> 
#include <stddef.h>
#include <services/wd/adi_wd.h>

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"Allow the definition of objects in the source file. MISRA treats this as include file as it is included by other sources")
#pragma diag(suppress:misra_rule_8_7:" MISRA Rule 8.7(Req): Objects shall be defined at block scope if they are only accessed from within a single function")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#endif /* _MISRA_RULES */

/* macro that acknowledges the pending interrupt at NVIC level */
#if defined(ADI_ADSP_CM40Z)
#define ADI_WD_ACK_PENDINT(irq)  \
    do {                           \
    if (NVIC_GetPendingIRQ(irq))   \
    {                              \
        NVIC_ClearPendingIRQ(irq); \
    }                              \
    }while(0)
#else
#define ADI_WD_ACK_PENDINT(irq)
#endif

/** @addtogroup Watchdog_Service_BF6xx Watchdog Timer Service
#define ADI_WD_ENABLE   0x0FF0
#define ADI_WD_DISABLE  0x0AD0

#define BITM_WDOG_CTL_WDRO      0x8000
#define BITM_WDOG_CTL_WDEN      0x0FF0
*  @{
*/

/*! Watchdog timer register structure */
typedef volatile struct
{
    uint32_t    WD_CONTROL;             /* Watch dog timer control register  */
    uint32_t    WD_COUNT;               /* Watch dog timer counter register  */
    uint32_t    WD_STATUS;              /* Watch dog timer status register   */
} ADI_WD_REGS;


/*! Watch dog Device Structure */
typedef struct AdiWD
{
    ADI_WD_REGS 		*const pRegs;   	/*!< Base address of registers 				*/
    const uint32_t  	WatchDogSID;   		/*!< Watchdog interrupt ID                  */
    ADI_CALLBACK		pfCallback;     	/*!< Application supplied callback function */
    void*           	pCBParam;       	/*!< Callback parm passed back in callbacks */
} ADI_WD;


/*! ADSP-BF60x instances */
#if defined(__ADSPBF609_FAMILY__)
static ADI_WD  gaWDTimers[] =
{
    /* Watch dog timer A */
    {
        (ADI_WD_REGS *)(pREG_WDOG0_CTL),    /* Base Address of Registers    */        
        (uint32_t)INTR_WDOG0_EXP,	    /* Interrupt ID					*/
        NULL,							/* No callback					*/
        NULL							/* No callback					*/
    },
        /* Watch dog timer B */
    {
        (ADI_WD_REGS *)(pREG_WDOG1_CTL),    /* Base Address of Registers    */        
        (uint32_t)INTR_WDOG1_EXP,		/* Interrupt ID					*/
        NULL,							/* No callback					*/
        NULL							/* No callback					*/
    }
};
/*! ADSP-CM40z instances */
#elif defined(ADI_ADSP_CM40Z)
static ADI_WD  gaWDTimers[] =
{
    /* Watch dog timer A */
    {
        (ADI_WD_REGS *)(pREG_WDOG0_CTL),    /* Base Address of Registers    */        
        (uint32_t)INTR_WDOG0_EXP,	    /* Interrupt ID					*/
        NULL,							/* No callback					*/
        NULL							/* No callback					*/
    }
};
#elif defined(__ADSPBF707_FAMILY__)
static ADI_WD  gaWDTimers[] =
{
    /* Watch dog timer A */
    {
        (ADI_WD_REGS *)(pREG_WDOG0_CTL),    /* Base Address of Registers    */
        (uint32_t)INTR_WDOG0_EXP,       /* Interrupt ID                 */
        NULL,                           /* No callback                  */
        NULL                            /* No callback                  */
    }
};
#else
#error "This processor is not supported"
#endif

static ADI_WD_RESULT ValidateDeviceHandle (ADI_WD *const pDevice);
static void WatchDog_Interrupt(uint32_t SID, void *pCBParam);

/**
 * @brief       Opens a given watch dog timer.
 *
 * @details     Opens a given watch dog timer device and returns the handle to the watch dog timer
 *              which should be used in other APIs to configure and control the watch dog timer.
 *
 * @param [in]  Device      Watch dog timer number to open.
 * @param [out] phWDTimer   Pointer to a location where the handle to the opened
 *                          watch dog timer is written.
 * @param[in]   pfCallback  The application supplied callback which will be called to notify device
 *                          related events.
 * @param[in]   pCBParam    The application supplied callback parameter which can be passed back in
 *                          the callback function. 
 * @return      Status
 * - #ADI_WD_SUCCESS                   If successfully opened the given watch dog timer.
 * - #ADI_WD_IN_USE                [D] If the given watch dog timer is in use.
 * - #ADI_WD_INVALID_TIMERID       [D] Invalid device number.
 * - #ADI_WD_INVALID_POINTER       [D] If any of the given pointer are invalid.
 *
 * @sa          adi_wd_Close()
 */
ADI_WD_RESULT  adi_wd_Open (
    uint32_t        Device,
    ADI_WD_HANDLE   *const phWDTimer,
    ADI_CALLBACK    const pfCallback,
    void            *const pCBParam
)
{
    ADI_WD    *pDevice;

#ifdef ADI_DEBUG

    /* Check if the watch dog timer number is valid */
    if(Device >= ADI_WD_NUM_DEVICES)
    {
        return ADI_WD_INVALID_TIMERID;
    }

    /* Check if the pointer parameters are valid */
    if(phWDTimer == NULL)
    {
        return ADI_WD_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    /* Get the pointer to watch dog timer device */
    pDevice = &gaWDTimers[Device];

    /* save parameters for device instance    */
    pDevice->pfCallback = pfCallback;
    pDevice->pCBParam   = pCBParam;
    
    /* set registers to default condition */
    pDevice->pRegs->WD_CONTROL = (uint32_t)ADI_WD_DISABLE;
    pDevice->pRegs->WD_COUNT = 0x80000u;
    pDevice->pRegs->WD_CONTROL = (uint32_t)BITM_WDOG_CTL_WDRO;
    pDevice->pRegs->WD_STATUS = 0x0u;

    /* The below code segment is required to disable Watchdog interrupt at the NVIC level.
     * If the program is already running from Flash Watchdog interrupt gets invoked as soon
     * as we install the handler. Below code acknowledges any pending interrupts at the
     * NVIC level
     */
    ADI_WD_ACK_PENDINT(ADI_INT_WDOG0_EXP);

    /* Register the watchdog interrupt handler */
    if (adi_int_InstallHandler(pDevice->WatchDogSID, WatchDog_Interrupt, pDevice, true) != ADI_INT_SUCCESS)
    {
        return ADI_WD_FAILURE;
    }

    /* Return the watch dog timer handle */
    *phWDTimer    =   (ADI_WD_HANDLE) pDevice;

    return ADI_WD_SUCCESS;
}


/**
 * @brief       Closes the given watch dog timer.
 *
 * @details     Closes the watch dog timer and frees it handle.
 *
 *              The API disables the watch dog timer immediately before closing it.
 *
 * @param [in]  hWDTimer      Handle to the watch dog timer.
 *
 * @return      Status
 * - #ADI_WD_SUCCESS               If successfully closed the given watch dog timer.
 * - #ADI_WD_INVALID_HANDLE    [D] If the given watchdog timer handle is invalid.
 *
 */
ADI_WD_RESULT  adi_wd_Close (ADI_WD_HANDLE    const hWDTimer)
{
    /* assume a return code */
    ADI_WD_RESULT Result = ADI_WD_SUCCESS;
    /* Pointer to device instance */
    ADI_WD *pDevice;

#ifdef ADI_DEBUG

    /* Validate the given watch dog timer handle */
    if(ValidateDeviceHandle(hWDTimer) != ADI_WD_SUCCESS)
    {
        return Result;
    }
#endif /* ADI_DEBUG */
    
    pDevice = (ADI_WD *)hWDTimer;

    /* disable watch dog timer */
    pDevice->pRegs->WD_CONTROL = (uint32_t)ADI_WD_DISABLE;

    /* Unregister the watchdog status interrupt handler */
    if(adi_int_UninstallHandler(pDevice->WatchDogSID) != ADI_INT_SUCCESS)
    {
        return ADI_WD_FAILURE;
    }
    
    /* free handle */
    pDevice = NULL;

    return ADI_WD_SUCCESS;
}

/**
 * @brief      Resets the watch dog timer counter and prevents it from expiring.
 *
 * @details    Reloads and resets the watch dog timer counter value to prevent it from expiring
 *
 * @param [in]  hWDTimer    Handle to the watch dog timer.
 * @param [in]  WDCounter   Watch dog timer count value.
 *
 * @return      Status
 * - #ADI_WD_SUCCESS               If the API is success.
 * - #ADI_WD_INVALID_HANDLE    [D] If the given watch dog timer handle is invalid.
 */
ADI_WD_RESULT  adi_wd_Reset(
    ADI_WD_HANDLE    const hWDTimer,
    uint32_t         WDCounter
)
{
    /* assume a return code */
    ADI_WD_RESULT Result = ADI_WD_SUCCESS;
    /* Pointer to device instance */
    ADI_WD *pDevice;

#ifdef ADI_DEBUG

    /* Validate the given watch dog timer handle */
    if(ValidateDeviceHandle(hWDTimer) != ADI_WD_SUCCESS)
    {
        return ADI_WD_INVALID_HANDLE;
    }

#endif /* ADI_DEBUG */

    pDevice = (ADI_WD *)hWDTimer;
    
    /* resets the watch dog timer counter */
    pDevice->pRegs->WD_STATUS = WDCounter;

    return ADI_WD_SUCCESS;
}

/**
 * @brief      Sets the watch dog timer counter value.
 *
 * @details    Sets the watch dog timer counter value.
 *
 * @param [in]  hWDTimer    Handle to the watch dog timer.
 * @param [in]  WDCounter   Watch dog timer count value.
 *
 * @return      Status
 * - #ADI_WD_SUCCESS               If the API is success.
 * - #ADI_WD_INVALID_HANDLE    [D] If the given watch dog timer handle is invalid.
 */
ADI_WD_RESULT  adi_wd_SetCounter(
    ADI_WD_HANDLE    const hWDTimer,
    uint32_t         WDCounter
)
{
    /* assume a return code */
    ADI_WD_RESULT Result = ADI_WD_SUCCESS;
    /* Pointer to device instance */
    ADI_WD *pDevice;

#ifdef ADI_DEBUG

    /* Validate the given watch dog timer handle */
    if(ValidateDeviceHandle(hWDTimer) != ADI_WD_SUCCESS)
    {
        return ADI_WD_INVALID_HANDLE;
    }

#endif /* ADI_DEBUG */

    pDevice = (ADI_WD *)hWDTimer;

    /* set the watch dog timer counter */
    pDevice->pRegs->WD_COUNT = WDCounter;

    return ADI_WD_SUCCESS;
}

/**
 * @brief       Gets the current watch dog timer counter value.
 *
 * @param [in]  hWDTimer    Handle to the watch dog timer.
 * @param [out] pCounter    Pointer to a location where current count value is written.
 *
 * @return      Status
 * - #ADI_WD_SUCCESS               If successfully queried the timer counter.
 * - #ADI_WD_INVALID_HANDLE    [D] If the given watch dog timer handle is invalid.
 * - #ADI_WD_INVALID_POINTER   [D] If given pointer to watch dog counter value is invalid.
 */
ADI_WD_RESULT  adi_wd_GetCounter(
    ADI_WD_HANDLE    const hWDTimer,
    uint32_t         *pCounter
)
{
    /* assume a return code */
    ADI_WD_RESULT Result = ADI_WD_SUCCESS;
    /* Pointer to device instance */
    ADI_WD *pDevice;

#ifdef ADI_DEBUG

    /* Validate the given watch dog timer handle */
    if(ValidateDeviceHandle(hWDTimer) != ADI_WD_SUCCESS)
    {
        return ADI_WD_INVALID_HANDLE;
    }

    /* Validate the given pointer */
    if(pCounter == NULL)
    {
        return ADI_WD_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    pDevice = (ADI_WD *)hWDTimer;

    /* Get the current count value */
    *pCounter =  pDevice->pRegs->WD_STATUS;

    return ADI_WD_SUCCESS;
}

/**
 * @brief       Enable/Disable the watch dog timer.
 *
 * @details     Enables/disbales the watch dog timer.
 *
 * @param [in]  hWDTimer    Handle to the watch dog timer
 * @param [in]  bEnable     Flag which indicates whether to enable or disable the watch dog timer.
 *                          'true'   - enable the watch dog timer.
 *                          'false'  - disable the watch dog timer.
 *
 * @return      Status
 * - #ADI_WD_SUCCESS               If successfully enabled/disabled the watch dog timer
 * - #ADI_WD_INVALID_HANDLE    [D] If the given watch dog timer handle is invalid.
 *
 */
ADI_WD_RESULT  adi_wd_Enable(
    ADI_WD_HANDLE    const hWDTimer,
    bool             bEnable
)
{
    /* assume a return code */
    ADI_WD_RESULT Result = ADI_WD_SUCCESS;
    /* Pointer to device instance */
    ADI_WD *pDevice;

    /* register value */
    uint32_t RegValue;

#ifdef ADI_DEBUG
    /* Validate the given watch dog timer handle */
    if(ValidateDeviceHandle(hWDTimer) != ADI_WD_SUCCESS)
    {
        return ADI_WD_INVALID_HANDLE;
    }
#endif /* ADI_DEBUG */

    pDevice = (ADI_WD *)hWDTimer;

    RegValue = pDevice->pRegs->WD_CONTROL & (uint32_t)~(BITM_WDOG_CTL_WDEN);

    if(bEnable)
    {
        /* enable watch dog timer */
        pDevice->pRegs->WD_CONTROL = (uint32_t)ADI_WD_ENABLE | RegValue;
    }
    else
    {
        /* disable watch dog timer */
        pDevice->pRegs->WD_CONTROL = (uint32_t)ADI_WD_DISABLE | RegValue;
    }
    return ADI_WD_SUCCESS;
}


/**
 * @brief       Gets the rollover status of the watch dog timer counter.
 *
 * @param [in]  hWDTimer        Handle to the watch dog timer.
 * @param [out] pRolloverStatus Pointer to the watch dog timer rollover state.
 *              - #true      	Watch dog timer has rolled over and expired.
 *              - #false        Watch dog timer has not rolled over and expired.
 *
 * @return      Status
 * - #ADI_WD_SUCCESS               If successfully queried the timer counter.
 * - #ADI_WD_INVALID_HANDLE    [D] If the given timer handle is invalid.
 * - #ADI_WD_INVALID_POINTER   [D] If given pointer to status value is invalid.
 */
ADI_WD_RESULT  adi_wd_GetRollOverStatus(
    ADI_WD_HANDLE    const hWDTimer,
    bool             *pRolloverStatus
)
{
    /* assume a return code */
    ADI_WD_RESULT Result = ADI_WD_SUCCESS;
    /* Pointer to device instance */
    ADI_WD *pDevice;
    /* register value */
    uint32_t RegValue;
    
#ifdef ADI_DEBUG

    /* Validate the given watchdog timer handle */
    if(ValidateDeviceHandle(hWDTimer) != ADI_WD_SUCCESS)
    {
        return ADI_WD_INVALID_HANDLE;
    }

    /* Validate the given pointer */
    if(pRolloverStatus == NULL)
    {
        return ADI_WD_INVALID_POINTER;
    }
#endif /* ADI_DEBUG */

    pDevice = (ADI_WD *)hWDTimer;

    /* Get the rollover field count value */
    RegValue = pDevice->pRegs->WD_CONTROL;

    if((RegValue & (uint32_t)BITM_WDOG_CTL_WDRO) == BITM_WDOG_CTL_WDRO)
    {
        *pRolloverStatus = true;
    }    
    else
    {
        *pRolloverStatus = false;
    }    
    return ADI_WD_SUCCESS;
}

#ifdef ADI_DEBUG
/*
 * Verify that the handle is valid
 */
static ADI_WD_RESULT ValidateDeviceHandle (ADI_WD *const pDevice)
{
    uint32_t WDTimerNum;

    /* Check if the given pointer is valid */
    if(pDevice == NULL)
    {
    	return ADI_WD_INVALID_POINTER;
    }

    /* For all the timers */
	for( WDTimerNum = 0u; WDTimerNum < ADI_WD_NUM_DEVICES; WDTimerNum++)
	{
	    /* Check if the given handle matches with this watch dog timer */
		if( pDevice == &gaWDTimers[WDTimerNum])
		{
			return ADI_WD_SUCCESS;
		}
	}
    return ADI_WD_INVALID_HANDLE;
}
#endif

static void WatchDog_Interrupt(uint32_t SID, void *pCBParam)
{
    /* Pointer to device instance */
    ADI_WD *pDevice = (ADI_WD *)pCBParam;

#ifdef ADI_DEBUG
    /*  check device */
    if(pDevice == NULL)
    {
        return;
    }
#endif /* ADI_DEBUG */

    /* post callback */
    if(pDevice->pfCallback != NULL)
    {
    	(pDevice->pfCallback)(pDevice->pCBParam, (uint32_t)ADI_WD_EVENT_GP_INT, NULL);
    }

    return;
}


/*@}*/


#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif /* _ADI_WD_BF6XX_C_ */
