/*
*********************************************************************************************************
*                                    uC/USB-Device Controller Driver
*                                      ADI Driver Interface Layer
*
*                    Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.
*
*               This software is proprietary and confidential. By using this software you agree
*               to the terms of the associated Analog Devices License Agreement.
*
*                          (c) Copyright 2004-2011; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/USB is provided in source form to registered licensees ONLY.  It is
*               illegal to distribute this source code to any third party unless you receive
*               written permission by an authorized Micrium representative.  Knowledge of
*               the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                    USB DEVICE CONTROLLER DRIVER
*                                  ADI device driver implementation
*
*********************************************************************************************************
* Note(s)       : (1) With an appropriate BSP, this device driver will support the OTG device module on
*                     the Blackfin and the NET2272 peripheral controller attached to the Aysnchronous
*                     bus on Blackfin and SHARC.
*
*
*                (2) This driver is an interface layer to the ADI SSLDD 2.0 interface driver.
*
*********************************************************************************************************
*/



/*
*********************************************************************************************************
*                                              INCLUDE FILES
*********************************************************************************************************
*/

#include <usbd_core.h>
#include <usbd_drv_adi.h>
#include <adi_usbd_dev.h>

/*
*********************************************************************************************************
*                                              LOCAL DEFINES
*********************************************************************************************************
*/
// this may need replaced at some point
#define ADI_USBD_DEV_NUMBER 0
#define EP0_ADDR 0
#define EP0_MAX_PKT_SIZE 64
#define EP0_LOG_ADDR 0

#define TRUE  1
#define FALSE 0

/*
*********************************************************************************************************
*                                            LOCAL DATA TYPES
*********************************************************************************************************
*/

/*
 * DRIVER DATA TYPE
*/
typedef struct usbd_drv_data {
    const ADI_USBD_DEV_API    *pDrvAPI;
    CPU_INT08U                pDriverData[ADI_USBD_DEV_DATA_SIZE];
    ADI_USBD_DEV_HANDLE       hDevice;
    CPU_INT08U                DeviceAddress;
    CPU_BOOLEAN               bSettingAddress;
} USBD_DRV_DATA;

static USBD_DRV_DATA f_DriverData = { 0 };

/*
*********************************************************************************************************
*                                       LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/

#ifndef ADI_USBD_DEV_DRVR
#error "ADI_USBD_DEV_DRVR       not #define'd in 'adi_usbd_dev.h"
#error "                        [Must be ADI_USBD_DEV_BF_MUSBHDRC, ADI_USBD_DEV_BF_MUSBHDRC, ADI_USBD_DEV_BF_NET2272 or ADI_USBD_DEV_SH_NET2272]"
#else
#if (ADI_USBD_DEV_DRVR == 0) || (ADI_USBD_DEV_DRVR > 5)
#error "ADI_USBD_DEV_DRVR       illegally #define'd in 'adi_usbd_dev.h"
#error "                        [Must be ADI_USBD_DEV_BF_MUSBHDRC, ADI_USBD_DEV_BF_MUSBHDRC, ADI_USBD_DEV_BF_NET2272 or ADI_USBD_DEV_SH_NET2272]"
#endif
#endif

#if (USBD_CFG_DBG_TRACE_EN == DEF_ENABLED)
extern void  USBD_DbgArg (const  CPU_CHAR    *p_msg,
                          CPU_INT08U   ep_addr,
                          CPU_INT08U   if_nbr,
                          CPU_INT32U   arg,
                          USBD_ERR     err);
#endif


/*
*********************************************************************************************************
*                                        LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static void EndpointZeroCallback( void *hClient, uint32_t Event, void *pBuffer );
static void EndpointCallback( void *hClient, uint32_t Event, void *pBuffer );
static void BusEventCallback( void *hClient, uint32_t Event, void *pBuffer );
void adi_usbd_Debug(const char *pStr, uint8_t EpAddr, uint8_t Iface, uint32_t value)
{
#if (USBD_CFG_DBG_TRACE_EN == DEF_ENABLED)
    USBD_ERR     err=USBD_ERR_NONE;
    USBD_DbgArg ( (CPU_CHAR*)pStr,
                  (CPU_INT08U)EpAddr,
                  (CPU_INT08U)Iface,
                  (CPU_INT32U)value,
                  err);
#endif

}

/* -------------- ADI SSLDD 2.0 DRIVER API  --------------- */
static  void         USBD_DrvInit          (USBD_DRV     *p_drv,
                                            USBD_ERR     *p_err);

static  void         USBD_DrvStop          (USBD_DRV     *p_drv);

static  void         USBD_DrvEP_Open       (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            CPU_INT08U    ep_attrib,
                                            CPU_INT16U    max_pkt_size,
                                            CPU_INT08U    transaction_frame,
                                            USBD_ERR     *p_err);

static  void         USBD_DrvEP_Close      (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr);

static  void         USBD_DrvEP_RxStart    (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            CPU_INT08U   *p_buf,
                                            CPU_INT32U    buf_len,
                                            USBD_ERR     *p_err);

static  CPU_INT32U   USBD_DrvEP_Tx         (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            CPU_INT08U   *p_buf,
                                            CPU_INT32U    buf_len,
                                            USBD_ERR     *p_err);

static  void         USBD_DrvEP_TxStart    (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            CPU_INT08U   *p_buf,
                                            CPU_INT32U    buf_len,
                                            USBD_ERR     *p_err);

static  CPU_INT32U   USBD_DrvEP_Rx         (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            CPU_INT08U   *p_buf,
                                            CPU_INT32U    buf_len,
                                            USBD_ERR     *p_err);

static  void         USBD_DrvStart         (USBD_DRV     *p_drv,
                                            USBD_ERR     *p_err);

static  CPU_BOOLEAN  USBD_DrvAddrSet       (USBD_DRV     *p_drv,
                                            CPU_INT08U    dev_addr);

static  void         USBD_DrvAddrEn        (USBD_DRV     *p_drv,
                                            CPU_INT08U    dev_addr);

static  CPU_BOOLEAN  USBD_DrvCfgSet        (USBD_DRV     *p_drv,
                                            CPU_INT08U    cfg_val);

static  void         USBD_DrvCfgClr        (USBD_DRV     *p_drv,
                                            CPU_INT08U    cfg_val);

static  CPU_INT16U   USBD_DrvGetFrameNbr   (USBD_DRV     *p_drv);


static  void         USBD_DrvEP_RxZLP      (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            USBD_ERR     *p_err);


static  void         USBD_DrvEP_TxZLP      (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            USBD_ERR     *p_err);

static  CPU_BOOLEAN  USBD_DrvEP_Abort      (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr);

static  CPU_BOOLEAN  USBD_DrvEP_Stall      (USBD_DRV     *p_drv,
                                            CPU_INT08U    ep_addr,
                                            CPU_BOOLEAN   stall);

static  void         USBD_DrvISR_Handler   (USBD_DRV     *p_drv);


/*
*********************************************************************************************************
*                                      MICRIUM USB DEVICE CONTROLLER DRIVER API
*********************************************************************************************************
*/

const  USBD_DRV_API  USBD_DrvAPI_ADI          = { USBD_DrvInit,
                                                  USBD_DrvStart,
                                                  USBD_DrvStop,
                                                  USBD_DrvAddrSet,
                                                  USBD_DrvAddrEn,
                                                  USBD_DrvCfgSet,
                                                  USBD_DrvCfgClr,
                                                  USBD_DrvGetFrameNbr,
                                                  USBD_DrvEP_Open,
                                                  USBD_DrvEP_Close,
                                                  USBD_DrvEP_RxStart,
                                                  USBD_DrvEP_Rx,
                                                  USBD_DrvEP_RxZLP,
                                                  USBD_DrvEP_Tx,
                                                  USBD_DrvEP_TxStart,
                                                  USBD_DrvEP_TxZLP,
                                                  USBD_DrvEP_Abort,
                                                  USBD_DrvEP_Stall,
                                                  USBD_DrvISR_Handler,
                                                };


/*
*********************************************************************************************************
*********************************************************************************************************
*                                     SSL/DD 2.0 DRIVER INTERFACE FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/

#undef CPU_CRITICAL_ENTER
#undef CPU_CRITICAL_EXIT
#undef CPU_SR_ALLOC
#define CPU_CRITICAL_ENTER()
#define CPU_CRITICAL_EXIT()
#define CPU_SR_ALLOC()

/*
*********************************************************************************************************
*                                        USBD_DrvInit()
*
* Description : Initialize the device for FIFO mode operation.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE       Device successfully initialized.
*                               USBD_ERR_ALLOC      Memory allocation failed.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_DevInit() via 'p_drv_api->Init()'.
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  USBD_DrvInit     (USBD_DRV  *p_drv,
                                USBD_ERR  *p_err)
{
    USBD_DRV_DATA       *p_drv_data;
    uint32_t             Result;
    CPU_SR_ALLOC();


                                                                /* Allocate driver internal data.                       */
#if 1
    /* for now we will use static data */
    p_drv_data = &f_DriverData;
#else
    p_drv_data = Mem_HeapAlloc((CPU_SIZE_T  ) sizeof(USBD_DRV_DATA),
                               (CPU_SIZE_T  ) 1u,
                               (CPU_SIZE_T *)&reqd_octets,
                               (LIB_ERR    *)&lib_err);
#endif

    if (p_drv_data == (void *)0) {
       *p_err = USBD_ERR_ALLOC;
        return;
    }

    CPU_CRITICAL_ENTER();
    p_drv->DataPtr = p_drv_data;                                /* Store drv internal data ptr.                         */
    p_drv_data->pDrvAPI = pADI_USBD_Drv;

    p_drv_data->bSettingAddress = DEF_FALSE;

    Result = (uint32_t)pADI_USBD_Drv->Open(
                                 ADI_USBD_DEV_NUMBER,
                                 &p_drv_data->pDriverData[0],
                                 sizeof(p_drv_data->pDriverData),
                                 USBD_CFG_MAX_NBR_EP_OPEN,
                                 BusEventCallback,
                                 (void*)p_drv,
                                 &p_drv_data->hDevice
                                 );

    CPU_CRITICAL_EXIT();

    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        *p_err = USBD_ERR_NONE;
    }
    else
    {
        *p_err = USBD_ERR_FAIL;
    }


}


/*
*********************************************************************************************************
*                                        USBD_DrvStart()
*
* Description : Start device operation :
*
*                   (1) Enable device controller bus state interrupts.
*                   (2) Call board/chip specific connect function.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                           USBD_ERR_NONE    Device successfully connected.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_DevStart() via 'p_drv_api->Start()'.
*
* Note(s)     : Typically, the start function activates the pull-down on the D- pin to simulate
*               attachment to host. Some MCUs/MPUs have an internal pull-down that is activated by a
*               device controller register; for others, this may be a GPIO pin. Additionally, interrupts
*               for reset and suspend are activated.
*********************************************************************************************************
*/

static  void  USBD_DrvStart (USBD_DRV  *p_drv,
                             USBD_ERR  *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    CPU_SR_ALLOC();

    /* Enable USB */
    CPU_CRITICAL_ENTER();
    p_drv_data->pDrvAPI->SetCallbackHandle(p_drv_data->hDevice, (void*)p_drv);
    p_drv_data->pDrvAPI->EnableUSB(p_drv_data->hDevice, TRUE);
    CPU_CRITICAL_EXIT();

    *p_err = USBD_ERR_NONE;
}

/*
*********************************************************************************************************
*                                      USBD_DrvStop()
*
* Description : Disconnect device from the host.
*
* Argument(s) : p_drv   Pointer to driver structure.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_DevStop() via 'p_drv_api->Start()'.
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  USBD_DrvStop (USBD_DRV  *p_drv)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;

    /* Disable USB */
    p_drv_data->pDrvAPI->EnableUSB(p_drv_data->hDevice, FALSE);

}

/*
*********************************************************************************************************
*                                          USBD_DrvCfgSet()
*
* Description : Bring device into configured state.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               cfg_val     Configuration value.
*
* Return(s)   : DEF_OK,   if NO error(s).
*
*               DEF_FAIL, otherwise.
*
* Caller(s)   : USBD_CfgOpen() via 'p_drv_api->CfgSet()'.
*
* Note(s)     : Typically, the set configuration function sets the device as configured. For some
*               controllers, this may not be necessary.
*********************************************************************************************************
*/


static  CPU_BOOLEAN  USBD_DrvAddrSet (USBD_DRV     *p_drv,
                                      CPU_INT08U    dev_addr)

{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;

    p_drv_data->DeviceAddress = dev_addr;
    p_drv_data->bSettingAddress = DEF_TRUE;

    /* Pass request to device layer */
    //p_drv_data->pDrvAPI->SetAddress(p_drv_data->hDevice, (uint8_t)dev_addr );

    return (DEF_OK);
}


/*
*********************************************************************************************************
*                                          USBD_DrvAddrEn()
*
* Description : Enable address on device.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               dev_addr    Device address assigned by the host.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_StdReqHandler() via 'p_drv_api->AddrEn()'.
*
* Note(s)     : (1) For device controllers that have hardware assistance to enable the device address after
*                   the status stage has completed, no operation needs to be performed.
*
*               (2) For device controllers that change the device address immediately, without waiting the
*                   status phase to complete, the device address must be set and enabled.
*********************************************************************************************************
*/

static  void  USBD_DrvAddrEn (USBD_DRV    *p_drv,
                              CPU_INT08U   dev_addr)
{
    /*
     * This is not required for the MUSBMHDRC controller
     */
}


/*
*********************************************************************************************************
*                                          USBD_DrvCfgSet()
*
* Description : Bring device into configured state.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               cfg_val     Configuration value.
*
* Return(s)   : DEF_OK,   if NO error(s).
*
*               DEF_FAIL, otherwise.
*
* Caller(s)   : USBD_CfgOpen() via 'p_drv_api->CfgSet()'.
*
* Note(s)     : Typically, the set configuration function sets the device as configured. For some
*               controllers, this may not be necessary.
*********************************************************************************************************
*/

static  CPU_BOOLEAN  USBD_DrvCfgSet (USBD_DRV    *p_drv,
                                     CPU_INT08U   cfg_val)
{
	USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;

	/* Pass request to device layer */
	p_drv_data->pDrvAPI->SetConfig(p_drv_data->hDevice, (uint8_t)cfg_val );

    return (DEF_OK);
}


/*
*********************************************************************************************************
*                                          USBD_DrvCfgClr()
*
* Description : Bring device into de-configured state.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               cfg_val     Configuration value.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_CfgClose() via 'p_drv_api->CfgClr()'.
*
* Note(s)     : (1) Typically, the clear configuration function sets the device as not being configured.
*                   For some controllers, this may not be necessary.
*
*               (2) This functions in invoked after a bus reset or before the status stage of some
*                   SET_CONFIGURATION requests.
*********************************************************************************************************
*/

static  void  USBD_DrvCfgClr (USBD_DRV    *p_drv,
                              CPU_INT08U   cfg_val)
{
	USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;

	/* Pass request to device layer */
	p_drv_data->pDrvAPI->ClearConfig(p_drv_data->hDevice, (uint8_t)cfg_val );
}


/*
*********************************************************************************************************
*                                        USBD_DrvGetFrameNbr()
*
* Description : Retrieve current frame number.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
* Return(s)   : Frame number.
*
* Caller(s)   : USBD_EP_Open() via 'p_drv_api->GetFrameNbr()'.
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  CPU_INT16U  USBD_DrvGetFrameNbr (USBD_DRV  *p_drv)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    CPU_INT16U          FrameNbr;

    /* pass request to device layer */
    p_drv_data->pDrvAPI->GetFrameNumber(p_drv_data->hDevice, (uint16_t*)&FrameNbr);

    return (FrameNbr);
}



/*
*********************************************************************************************************
*                                          USBD_DrvEP_Open()
*
* Description : Open and configure a device endpoint, given its characteristics (e.g., endpoint type,
*               endpoint address, maximum packet size, etc).
*
* Argument(s) : p_drv               Pointer to device driver structure.
*
*               ep_addr             Endpoint address.
*
*               ep_type             Endpoint type :
*
*                                       USB_EP_TYPE_CTRL,
*                                       USB_EP_TYPE_ISOC,
*                                       USB_EP_TYPE_BULK,
*                                       USB_EP_TYPE_INTR.
*
*               max_pkt_size        Maximum packet size.
*
*               transaction_frame   Endpoint transactions per frame.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE               Endpoint successfully opened.
*                               USBD_ERR_EP_INVALID_ADDR    Invalid endpoint address.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_EP_Open() via 'p_drv_api->EP_Open()',
*               USBD_CtrlOpen().
*
* Note(s)     : (1) Typically, the endpoint open function performs the following operations:
*
*                   (a) Validate endpoint address, type and maximum packet size.
*                   (b) Configure endpoint information in the device controller. This may include not
*                       only assigning the type and maximum packet size, but also making certain that
*                       the endpoint is successfully configured (or realized or mapped). For some
*                       device controllers, this may not be necessary.
*
*               (2) If the endpoint address is valid, then the endpoint open function should validate
*                   the attributes allowed by the hardware endpoint :
*
*                   (a) The maximum packet size 'max_pkt_size' should be validated to match hardware
*                       capabilities.
*********************************************************************************************************
*/


static  void  USBD_DrvEP_Open    (USBD_DRV    *p_drv,
                                  CPU_INT08U   ep_addr,
                                  CPU_INT08U   ep_type,
                                  CPU_INT16U   max_pkt_size,
                                  CPU_INT08U   transaction_frame,
                                  USBD_ERR    *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;
    ADI_USBD_DEV_EP_DEF		*pEpDef;
    CPU_SR_ALLOC();

    /* Assume failure - success has to change this value */
    *p_err = USBD_ERR_FAIL;

    CPU_CRITICAL_ENTER();

    /* Bind the endpoint */
    Result = p_drv_data->pDrvAPI->BindEndpoint(p_drv_data->hDevice, ep_addr, (uint8_t)ep_type, (uint16_t)max_pkt_size, (uint8_t)transaction_frame, &pEpDef );

    /* IF (Successfully assigned a HW EP */
    if (Result == ADI_USBD_DEV_RESULT_SUCCESS && !pEpDef->pClient)
    {
        /* Assign driver pointer */
        pEpDef->pClient = p_drv;

        //EpAddEntry(p_drv, hEndpoint);

        /* register callback function with endpoint
         * callback pointer is set to EP entry address for easy lookup on callback
        */
        if ((ep_addr&0x7F)==0) {
            Result = p_drv_data->pDrvAPI->EpRegisterCallback(p_drv_data->hDevice, ep_addr, EndpointZeroCallback, (void*)pEpDef);
        } else {
            Result = p_drv_data->pDrvAPI->EpRegisterCallback(p_drv_data->hDevice, ep_addr, EndpointCallback, (void*)pEpDef);
        }

        if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
        {
            *p_err = USBD_ERR_NONE;
        }
        else {
            /* Release endpoint on failure */
            p_drv_data->pDrvAPI->ReleaseEndpoint(p_drv_data->hDevice, ep_addr);
            *p_err = USBD_ERR_EP_NONE_AVAIL;
        }
    }
    /* ELSE (The endpoint is an EP0 duplicate) */
    else if (Result == ADI_USBD_DEV_RESULT_SUCCESS && pEpDef->pClient)
    {
        *p_err = USBD_ERR_NONE;
    }
    /* ELSE (No EP available) */
    else {
    	*p_err = USBD_ERR_EP_NONE_AVAIL;
    }

    CPU_CRITICAL_EXIT();

}


/*
*********************************************************************************************************
*                                         USBD_DrvEP_Close()
*
* Description : Close a device endpoint, and uninitialize/clear endpoint configuration in hardware.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_EP_Close() via 'p_drv_api->EP_Close()',
*               USBD_CtrlOpen().
*
* Note(s)     : Typically, the endpoint close function clears the endpoint information in the device
*               controller. For some controllers, this may not be necessary.
*********************************************************************************************************
*/

static  void  USBD_DrvEP_Close (USBD_DRV   *p_drv,
                                CPU_INT08U ep_addr)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;

    CPU_SR_ALLOC();

    CPU_CRITICAL_ENTER();

    /* Release endpoint */
    p_drv_data->pDrvAPI->ReleaseEndpoint(p_drv_data->hDevice, ep_addr);

    CPU_CRITICAL_EXIT();

}


/*
*********************************************************************************************************
*                                        USBD_DrvEP_RxStart()
*
* Description : Configure endpoint with buffer to receive data.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
*               p_buf       Pointer to data buffer.
*
*               buf_len     Length of the buffer.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE   Receive successfully configured.
*                               USBD_ERR_RX     Generic Rx error.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_EP_Rx() via 'p_drv_api->EP_Rx()',
*               USBD_EP_Process().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  USBD_DrvEP_RxStart     (USBD_DRV    *p_drv,
                                      CPU_INT08U   ep_addr,
                                      CPU_INT08U  *p_buf,
                                      CPU_INT32U   buf_len,
                                      USBD_ERR    *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;

    if (buf_len == 0) {
    	Result = p_drv_data->pDrvAPI->EpReadZLP(p_drv_data->hDevice, ep_addr);
    } else {
    	Result = p_drv_data->pDrvAPI->EpSubmitRxBuffer(p_drv_data->hDevice, ep_addr, (void*)p_buf, (uint32_t)buf_len);
    }

    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        *p_err = USBD_ERR_NONE;
    } else {
        *p_err = USBD_ERR_RX;
    }
}


/*
*********************************************************************************************************
*                                           USBD_DrvEP_Rx()
*
* Description : Receive the specified amount of data from device endpoint.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
*               p_buf       Pointer to data buffer.
*
*               buf_len     Length of the buffer.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE   Data successfully received.
*                               USBD_ERR_RX     Generic Rx error.
*
* Return(s)   : Number of octets received, if NO error(s).
*
*               0,                         otherwise.
*
* Caller(s)   : USBD_EP_Rx() via 'p_drv_api->EP_Rx()',
*               USBD_EP_Process().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  CPU_INT32U  USBD_DrvEP_Rx     (USBD_DRV    *p_drv,
                                       CPU_INT08U   ep_addr,
                                       CPU_INT08U  *p_buf,
                                       CPU_INT32U   buf_len,
                                       USBD_ERR    *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;
    uint32_t nBytes = 0;
    void *pBuffer;

    /* this call will block until the tranfser is complete - it should not block as we have already
     * received a call back indicating transfer completion
    */
    Result = p_drv_data->pDrvAPI->EpGetRxBuffer(p_drv_data->hDevice, ep_addr, &pBuffer, &nBytes);

    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        *p_err = USBD_ERR_NONE;
    } else {
        *p_err = USBD_ERR_RX;
    }


    return (nBytes);
}


/*
*********************************************************************************************************
*                                         USBD_DrvEP_RxZLP()
*
* Description : Receive zero-length packet from endpoint.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE   Zero-length packet successfully received.
*                               USBD_ERR_RX     Generic Rx error.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_EP_RxZLP() via 'p_drv_api->EP_RxZLP()'.
*
* Note(s)     : (1) $$$$ Implement 2 different functions for FIFO and DMA APIs.
* 			    (2) Dec 06, 2011: use of this function may be deprecated
*
*********************************************************************************************************
*/

static  void  USBD_DrvEP_RxZLP (USBD_DRV    *p_drv,
                                CPU_INT08U   ep_addr,
                                USBD_ERR    *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;
#if 1
    Result = ADI_USBD_DEV_RESULT_SUCCESS;
#else
    Result = p_drv_data->pDrvAPI->EpReadZLP(p_drv_data->hDevice, (uint8_t)ep_addr);
#endif

    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        *p_err = USBD_ERR_NONE;
    } else {
        *p_err = USBD_ERR_RX;
    }
}


/*
*********************************************************************************************************
*                                           USBD_DrvEP_Tx()
*
* Description : Configure endpoint with buffer to transmit data.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
*               p_buf       Pointer to buffer of data that will be transmitted.
*
*               buf_len     Number of octets to transmit.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE   Transmit successfully configured.
*                               USBD_ERR_TX     Generic Tx error.
*
* Return(s)   : Number of octets transmitted, if NO error(s).
*
*               0,                            otherwise.
*
* Caller(s)   : USBD_EP_Tx() via 'p_drv_api->EP_Tx()',
*               USBD_EP_Process().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  CPU_INT32U  USBD_DrvEP_Tx     (USBD_DRV    *p_drv,
                                       CPU_INT08U   ep_addr,
                                       CPU_INT08U  *p_buf,
                                       CPU_INT32U   buf_len,
                                       USBD_ERR    *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;
    uint32_t nBytes = 0;

    /*
     * the Micrium documentation states (p19) that this function should load the FIFO and return the
     * amount loaded.
     *
     */
    Result = p_drv_data->pDrvAPI->EpGetMaxTxTransferSize(p_drv_data->hDevice, (uint8_t)ep_addr,(uint8_t*)p_buf,&nBytes);

    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        /*
         * transfer size is minimum of bytes requested and the max transfer size
         */
        if (buf_len == 0 || nBytes > buf_len)
        {
            nBytes = buf_len;
        }
        *p_err = USBD_ERR_NONE;
    }
    else {
        *p_err = USBD_ERR_TX;
    }

    return nBytes;
}


/*
*********************************************************************************************************
*                                        USBD_DrvEP_TxStart()
*
* Description : Transmit the specified amount of data to device endpoint.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
*               p_buf       Pointer to buffer of data that will be transmitted.
*
*               buf_len     Number of octets to transmit.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE   Data successfully transmitted.
*                               USBD_ERR_TX     Generic Tx error.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_EP_Tx() via 'p_drv_api->EP_TxStart()',
*               USBD_EP_Process().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  USBD_DrvEP_TxStart     (USBD_DRV    *p_drv,
                                      CPU_INT08U   ep_addr,
                                      CPU_INT08U  *p_buf,
                                      CPU_INT32U   buf_len,
                                      USBD_ERR    *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;

    if (buf_len == 0) {
    	Result = p_drv_data->pDrvAPI->EpWriteZLP(p_drv_data->hDevice, ep_addr);
    } else {
    	Result = p_drv_data->pDrvAPI->EpSubmitTxBuffer(p_drv_data->hDevice, ep_addr, (void*)p_buf, (uint32_t)buf_len);
    }
    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        *p_err = USBD_ERR_NONE;
    } else {
        *p_err = USBD_ERR_TX;
    }
}


/*
*********************************************************************************************************
*                                         USBD_DrvEP_TxZLP()
*
* Description : Transmit zero-length packet from endpoint.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
*               p_err       Pointer to variable that will receive the return error code from this function :
*
*                               USBD_ERR_NONE   Zero-length packet successfully transmitted.
*                               USBD_ERR_TX     Generic Tx error.
*
* Return(s)   : none.
*
* Caller(s)   : USBD_EP_Tx() via 'p_drv_api->EP_TxZLP()',
*               USBD_EP_TxZLP(),
*               USBD_EP_Process().
*
* Note(s)     : (1) $$$$ Implement 2 different functions for FIFO and DMA APIs.
* 			    (2) Dec 06, 2011: this may well be deprecated in future.
*
*********************************************************************************************************
*/

static  void  USBD_DrvEP_TxZLP (USBD_DRV    *p_drv,
                                CPU_INT08U   ep_addr,
                                USBD_ERR    *p_err)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;

    Result = p_drv_data->pDrvAPI->EpWriteZLP(p_drv_data->hDevice, (uint8_t)ep_addr);

    if (((ep_addr&0x7F) == 0) && p_drv_data->bSettingAddress) {
        p_drv_data->pDrvAPI->SetAddress(p_drv_data->hDevice, (uint8_t)p_drv_data->DeviceAddress );
        p_drv_data->bSettingAddress = DEF_FALSE;
    }

    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        *p_err = USBD_ERR_NONE;
    } else {
        *p_err = USBD_ERR_TX;
    }
}


/*
*********************************************************************************************************
*                                         USBD_DrvEP_Abort()
*
* Description : Abort any pending transfer on endpoint.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint Address.
*
* Return(s)   : DEF_OK,   if NO error(s).
*
*               DEF_FAIL, otherwise.
*
* Caller(s)   : USBD_EP_Abort().
*
* Note(s)     : (1) $$$$ Implement 2 different functions for FIFO and DMA APIs.
*********************************************************************************************************
*/

static  CPU_BOOLEAN  USBD_DrvEP_Abort (USBD_DRV    *p_drv,
                                       CPU_INT08U   ep_addr)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    ADI_USBD_DEV_RESULT Result;
    bool                bValid = DEF_OK;

    Result = p_drv_data->pDrvAPI->EpAbortTransfer(p_drv_data->hDevice, ep_addr);

    if (Result != ADI_USBD_DEV_RESULT_SUCCESS)
    {
        bValid = DEF_FAIL;
    }

    return bValid;
}


/*
*********************************************************************************************************
*                                         USBD_DrvEP_Stall()
*
* Description : Set or clear stall condition on endpoint.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
*               ep_addr     Endpoint address.
*
*               state       Endpoint stall state.
*
* Return(s)   : DEF_OK,   if NO error(s).
*
*               DEF_FAIL, otherwise.
*
* Caller(s)   : USBD_EP_Stall() via 'p_drv_api->EP_Stall()',
*               USBD_CtrlStall().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  CPU_BOOLEAN  USBD_DrvEP_Stall (USBD_DRV     *p_drv,
                                       CPU_INT08U    ep_addr,
                                       CPU_BOOLEAN   state)
{
    USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;
    CPU_BOOLEAN         bValid = false;
    ADI_USBD_DEV_RESULT Result;

    if (state == DEF_SET) {
        Result = (CPU_BOOLEAN)p_drv_data->pDrvAPI->EpSetStall(p_drv_data->hDevice, ep_addr);
    }
    else {
        Result = (CPU_BOOLEAN)p_drv_data->pDrvAPI->EpClearStall(p_drv_data->hDevice, ep_addr);
    }

    if (Result == ADI_USBD_DEV_RESULT_SUCCESS)
    {
        bValid = true;
    }
    return (bValid);
}


/*
*********************************************************************************************************
*                                         EndpointZeroCallback()
*
* Description : Entry point for Endpoint Zero events from the device driver.
*
* Argument(s) : hClient     handle specified on registering the callback.
*
*               Event       Event code; can be control or bus events
*
*               pBuffer     Data buffer that has just completed
*
* Return(s)   : none
*
* Caller(s)   : ADI SSL/DD 2.0 USB controller driver
*
* Note(s)     : none.
*********************************************************************************************************
*/
static void EndpointZeroCallback( void *hClient, uint32_t Event, void *pBuffer )
{
    ADI_USBD_DEV_EP_DEF *pEpCB = (ADI_USBD_DEV_EP_DEF *)hClient;
    USBD_DRV            *p_drv = (USBD_DRV *)pEpCB->pClient;

    switch (Event) {
        case ADI_USBD_DEV_EVENT_SETUP:
            /* Pass SETUP packet to Core */
            USBD_EventSetup(p_drv, pBuffer);
            break;

        case ADI_USBD_DEV_EVENT_RX:
            /* Pass RX completion event to Core */
            USBD_EP_RxCmpl(p_drv, EP0_LOG_ADDR);
            break;

        case ADI_USBD_DEV_EVENT_TX:
            /* Pass TX completion event to Core */
        	USBD_EP_TxCmpl(p_drv, EP0_LOG_ADDR);
            break;
    }
}


/*
*********************************************************************************************************
*                                         EndpointCallback()
*
* Description : Entry point for Endpoint Zero events from the device driver.
*
* Argument(s) : hClient     handle specified on registering the callback.
*
*               Event       Event code; can be control or bus events
*
*               pBuffer     Data buffer that has just completed
*
* Return(s)   : none
*
* Caller(s)   : ADI SSL/DD 2.0 USB controller driver
*
* Note(s)     : none.
*********************************************************************************************************
*/
static void EndpointCallback( void *hClient, uint32_t Event, void *pBuffer )
{
    ADI_USBD_DEV_EP_DEF *pEpDef = (ADI_USBD_DEV_EP_DEF *)hClient;
    USBD_DRV            *p_drv = (USBD_DRV *)pEpDef->pClient;
    //USBD_DRV_DATA       *p_drv_data = p_drv->DataPtr;

    switch (Event)
    {
        case ADI_USBD_DEV_EVENT_RX:
            USBD_EP_RxCmpl(p_drv,USBD_EP_ADDR_TO_LOG(pEpDef->EpAddr));
            break;

        case ADI_USBD_DEV_EVENT_TX:
            USBD_EP_TxCmpl(p_drv,USBD_EP_ADDR_TO_LOG(pEpDef->EpAddr));
            break;

        case ADI_USB_EVENT_PKT_RCVD_NO_BUFFER:
            {
            /* (1) issue read request
             * (2) call back core
            */
            }
            break;

    }
}

/*
*********************************************************************************************************
*                                         BusEventCallback()
*
* Description : Dispatches bus events.
*
* Argument(s) : p_drv       pointer to driver instance data.
*
*               Event       The bus event code.
*
*               state       Endpoint stall state.
*
* Return(s)   : none
*
* Caller(s)   : EndpointZeroCallback()
*
* Note(s)     : none.
*********************************************************************************************************
*/
static void BusEventCallback( void *hClient, uint32_t Event, void *pBuffer )
{
    USBD_DRV  *p_drv = (USBD_DRV *)hClient;

    switch (Event)
    {
        case ADI_USBD_DEV_EVENT_BUS_CONN:
            USBD_EventConn(p_drv);
            break;

        case ADI_USBD_DEV_EVENT_BUS_DISCONN:
            USBD_EventDisconn(p_drv);
            break;

        case ADI_USBD_DEV_EVENT_BUS_SUSPEND:
            USBD_EventSuspend(p_drv);
            break;

        case ADI_USBD_DEV_EVENT_BUS_RESUME:
            USBD_EventResume(p_drv);
            break;

        case ADI_USBD_DEV_EVENT_BUS_RESET:
            USBD_EventReset(p_drv);
            break;

        case ADI_USBD_DEV_EVENT_BUS_HS_DETECTED:
            USBD_EventHS(p_drv);
            break;

    }
}

/*
*********************************************************************************************************
*                                        USBD_DrvISR_Handler()
*
* Description : USB device Interrupt Service Routine (ISR) handler.
*
* Argument(s) : p_drv       Pointer to device driver structure.
*
* Return(s)   : none.
*
* Caller(s)   : This is an ISR.
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  USBD_DrvISR_Handler (USBD_DRV  *p_drv)
{
    /* ADI SSL/DD 2.0 controller driver implementation handles ISR */
}
