/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 *
 * $Date:        15. April 2014
 * $Revision:    V1.00
 *  
 * Project:      Flash Programming Functions for AnalogDevices CM40x Flashes
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */ 

#include <stdint.h>
#include "..\FlashOS.H"                          /* FlashOS Structures */

typedef volatile uint8_t    vu8_t;
typedef volatile uint16_t   vu16_t;
typedef volatile uint32_t   vu32_t;

#define M8(adr)  (*((vu8_t  *) (adr)))
#define M16(adr) (*((vu16_t *) (adr)))
#define M32(adr) (*((vu32_t *) (adr)))


/* SPI Controller register */
typedef struct {
  vu32_t RESERVED0[1];
  vu32_t CTL;                           /* Control Register */
  vu32_t RXCTL;                         /* Receive Control Register */
  vu32_t TXCTL;                         /* Transmit Control Register */
  vu32_t CLK;                           /* Clock Rate Register */
  vu32_t DLY;                           /* Delay Register */
  vu32_t SLVSEL;                        /* Slave Select Register */
  vu32_t RWC;                           /* Received Word Count Register */
  vu32_t RWCR;                          /* Received Word Count Reload Register */
  vu32_t TWC;                           /* Transmitted Word Count Register */
  vu32_t TWCR;                          /* Transmitted Word Count Reload Register */
  vu32_t RESERVED1[1];
  vu32_t IMSK;                          /* Interrupt Mask Register */
  vu32_t IMSK_CLR;                      /* Interrupt Mask Clear Register */
  vu32_t IMSK_SET;                      /* Interrupt Mask Set Register */
  vu32_t  RESERVED2[1];
  vu32_t STAT;                          /* Status Register */
  vu32_t ILAT;                          /* Masked Interrupt Condition Register */
  vu32_t ILAT_CLR;                      /* Masked Interrupt Clear Register */
  vu32_t RESERVED3[1];
  vu32_t RFIFO;                         /* Receive FIFO Data Register */
  vu32_t RESERVED4[1];
  vu32_t TFIFO;                         /* Transmit FIFO Data Register */
  vu32_t RESERVED5[1];
  vu32_t MMRDH;                         /* Memory Mapped Read Header */
  vu32_t MMTOP;                         /* SPI Memory Top Address */
} SPI_TypeDef;

/* SPI Bit Definitions */
#define BITM_SPI_CTL_MSTR           0x00000002ul  /* Master/Slave */
#define BITM_SPI_CTL_EN             0x00000001ul  /* Enable */
#define BITM_SPI_CTL_MMSE           0x80000000ul  /* Memory-Mapped SPI Enable */
#define BITM_SPI_MMRDH_CMDSKIP      0x10000000ul  /* Command Skip Enable */
#define BITM_SPI_MMRDH_MODE         0x00FF0000ul  /* Mode Field */
#define BITM_SPI_SLVSEL_SSE1        0x00000002ul  /* Slave Select 1 Enable */
#define BITM_SPI_SLVSEL_SSEL1       0x00000200ul  /* Slave Select 1 Input */
#define BITM_SPI_TXCTL_TEN          0x00000001ul  /* Transmit Enable */
#define BITM_SPI_TXCTL_TTI          0x00000004ul  /* Transmit Transfer Initiate */
#define BITM_SPI_TXCTL_TWCEN        0x00000008ul  /* Transmit Word Counter Enable */
#define BITM_SPI_STAT_TFS           0x00070000ul  /* SPI_TFIFO Status */
#define BITM_SPI_STAT_TF            0x00000800ul  /* Transmit Finish Indication */
#define BITM_SPI_STAT_RF            0x00000400ul  /* Receive Finish Indication */
#define BITM_SPI_STAT_RFE           0x00400000ul  /* SPI_RFIFO Empty */
#define BITM_SPI_RXCTL_REN          0x00000001ul  /* Receive Enable */
#define BITM_SPI_RXCTL_RWCEN        0x00000008ul  /* Receive Word Counter Enable */
#define BITM_SPI_RXCTL_RTI          0x00000004ul  /* Receive Transfer Initiate */


#define SPI2_BASE          0x4002C000   /* Base address of SPI2 */
#define SPI2              (( SPI_TypeDef *)   SPI2_BASE)


 /* M4P System Controller register */
typedef struct {
  vu32_t CACHE_CFG;                     /* Code Cache Configuration and Status Register */
  vu32_t CACHE_PEADDR;                  /* Code Cache Parity Error Address Register */
  vu32_t CACHE_MEMX;                    /* MEMX Space Configuration Register */
  vu32_t CACHE_MEMY;                    /* MEMY Space Configuration Register */
  vu32_t RESERVED0[4];
  vu32_t SRAM_CFG;                      /* SRAM Configuration Register */
  vu32_t SRAM_PEADDR_CORE;              /* SRAM Parity Error Address (Core) Register */
  vu32_t SRAM_PEADDR_DMA;               /* SRAM Parity Error Address (DMA) Register */
  vu32_t BUSFLT;                        /* Bus Fault Error Information Register */
  vu32_t STCALIB;                       /* SysTick Calibration Register */
  vu32_t RESERVED1[3];
  vu32_t CACHE_CNTCTL;                  /* Cache Counter Control Register */
  vu32_t CACHE_IREF;                    /* Cache ICODE Reference Counter Register */
  vu32_t CACHE_DREF;                    /* Cache DCODE Reference Counter Register */
  vu32_t CACHE_IMISS;                   /* Cache ICODE Miss Counter Register */
  vu32_t CACHE_DMISS;                   /* Cache DCODE Miss Counter Register */
  vu32_t CACHE_IFILL;                   /* Cache ICODE Line Fill Counter Register */
  vu32_t CACHE_DFILL;                   /* Cache DCODE Line Fill Counter Register */
} M4P_TypeDef;


#define M4P0_BASE          0xf0000000    /*Base address of M4P0 */
#define M4P0              (( M4P_TypeDef *)   M4P0_BASE)



#define XIP_CONFIRMATION_BIT         0ul
#define SPI2_SSEL_ID                 0ul
#define SLOW_SPI_CLOCK               3ul
#define SUBSECTOR_SIZE            4096ul
#define PAGE_SIZE                  256ul

#define READ_DATA                 0x03    /* READ opcode */
#define FAST_READ                 0x0B    /* FAST_READ opcode */
#define SUBSECTOR_ERASE           0x20    /* SSE opcode */
#define BULK_ERASE                0xC7    /* BE opcode */
#define PAGE_PROGRAM              0x02    /* PP opcode */
#define READ_STATUS_REGISTER      0x05    /* RDSR opcode */
#define WRITE_ENABLE              0x06    /* WREN opcode */
#define QUAD_IO_FAST_READ         0xEB    /* QIOFR opcode */
#define READ_CONFIG_REGISTER      0x85    /* RDVCR opcode */
#define WRITE_CONFIG_REGISTER     0x81    /* WRVCR opcode */

/* STATUS bits */
#define STATUS_BUSY               0x01
#define STATUS_WEL                0x02


unsigned long base_adr;        /* Base Address  */


/* SPI register save data */
uint32_t spiSavCTL;
uint32_t spiSavMMRDH;
uint32_t spiSavCLK;
uint32_t spiSavRXCTL;
uint32_t spiSavTXCTL;

/*buffer for SPI data */
uint8_t  spiData[8];
uint8_t  tbuf[4 + PAGE_SIZE];



/*----------------------------------------------------------------------------
   Local Functions  (see adi_flash.c)
  ----------------------------------------------------------------------------*/
static __inline void    sync_cache       (void);
static          void    exit_mm_xip_mode (void);
static          void    enter_mm_xip_mode(void);
static          uint8_t read_reg         (uint8_t reg);
static          void    write_reg        (uint8_t reg, uint8_t regVal);
static          void    enable_writes    (void);
static          void    send_bytes       (uint8_t *pData, uint32_t nNumBytes);
static          void    recv_bytes       (uint8_t *pData, uint32_t count);
static __inline void    selectSlave      (void);
static __inline void    deselectSlave    (void);
static          void    wait_while_busy  (void);

/* Clear the CPU cache */
static __inline
void sync_cache(void) {

  // Clear the CPU's cache, so that it sees the new contents of the flash memory.
  M4P0->CACHE_CFG |= 0x10;           /* set the cache clear bit */
  while (M4P0->CACHE_CFG & 0x10);    /* Wait for cache hardware to clear the bit */
}

/* Activate the chip select for the specified SPI slave */
static __inline
void selectSlave(void) {

  SPI2->SLVSEL = (           (BITM_SPI_SLVSEL_SSE1  << SPI2_SSEL_ID) |
                  (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << SPI2_SSEL_ID)) ); /* set slave select low */
}

/* Deactivate the chip select for the specified SPI slave */
static __inline
void deselectSlave(void) {

  SPI2->SLVSEL = ((BITM_SPI_SLVSEL_SSE1 << SPI2_SSEL_ID) | (0xFF00)); /* set slave select high */
//  SPI2->SLVSEL = ((BITM_SPI_SLVSEL_SSE1 << SPI2_SSEL_ID) | (0xFF00)); /* set slave select high */
}

/* Send the specified bytes to the SPI device */
static
void send_bytes(uint8_t * pData, uint32_t nNumBytes) {

  uint32_t i;
  uint32_t txctl = SPI2->TXCTL;

  SPI2->RXCTL = 0;
  SPI2->TWC = nNumBytes;
  SPI2->TXCTL = BITM_SPI_TXCTL_TEN | BITM_SPI_TXCTL_TTI | BITM_SPI_TXCTL_TWCEN;

  for (i = 0; i < nNumBytes; i++)
  {
    SPI2->TFIFO = pData[i];
    while((SPI2->STAT & BITM_SPI_STAT_TFS) == 0);  /* Wait until there is space in the FIFO */
  }

  while(0 == (SPI2->STAT & BITM_SPI_STAT_TF));     /* Wait until finished */

  SPI2->STAT = BITM_SPI_STAT_TF;
  SPI2->TXCTL = txctl & ~BITM_SPI_TXCTL_TEN;
}

/* Receive the specified number of bytes from the SPI device */
static
void recv_bytes(uint8_t *pData, uint32_t count) {
  uint32_t i;

  SPI2->TXCTL = 0;
  SPI2->STAT = BITM_SPI_STAT_RF;
  SPI2->RWC = count;
  SPI2->RXCTL = BITM_SPI_RXCTL_REN | BITM_SPI_RXCTL_RWCEN | BITM_SPI_RXCTL_RTI;

  for (i = 0; i < count; i++)
  {
    while(SPI2->STAT & BITM_SPI_STAT_RFE);  /* Wait until data is received in the receive FIFO */
    pData[i] = SPI2->RFIFO & 0xFF;
  }
  while((SPI2->RWC) != 0);                  /* Also make sure the read word counter is now zero */

  SPI2->STAT = BITM_SPI_STAT_RF;
  SPI2->RXCTL = 0;

  SPI2->CTL &= ~0x00000001;
  SPI2->CTL |=  0x00000001;
}

/* Send a WRITE_ENABLE command to the flash memory to set the WE flag */
static
void enable_writes(void) {
  uint8_t flashStatus;

  do {
    spiData[0] = WRITE_ENABLE;

    selectSlave();
    send_bytes(spiData, 1);
    deselectSlave();

    flashStatus = read_reg(READ_STATUS_REGISTER);
  } while (0u == (flashStatus & STATUS_WEL));
}

/* Read the contents of a specified flash device register, via the SPI bus */
static
uint8_t read_reg(uint8_t reg) {
  uint8_t regVal = 0;

  spiData[0] = reg;

  selectSlave();
  send_bytes(spiData, 1);
  recv_bytes(&regVal, 1);
  deselectSlave();

  return regVal;
}

/* Write a value to a specified flash device register, via the SPI bus */
static
void write_reg(uint8_t reg, uint8_t regVal) {

  enable_writes();

  spiData[0] = reg;
  spiData[0] = regVal;

  selectSlave();
  send_bytes(spiData, 2);
  deselectSlave();
}

/* Switch the SPI device to peripheral mode, saving its previous state first */
void exit_mm_xip_mode(void) {

  /* Save copies of the SPI registers that we're going to change */
  spiSavCTL   = SPI2->CTL;
  spiSavCLK   = SPI2->CLK;
  spiSavRXCTL = SPI2->RXCTL;
  spiSavTXCTL = SPI2->TXCTL;
  spiSavMMRDH = SPI2->MMRDH;

  /* need to ensure SPI is configured for 1 bit mode of operation */
  SPI2->CTL   = 0u;             /* Disable the SPI */
  SPI2->CLK   = SLOW_SPI_CLOCK; /* Slow down the clock for the program operations */
  SPI2->RXCTL = 0u;
  SPI2->TXCTL = 0u;
  SPI2->MMRDH = 0u;
  SPI2->CTL   = BITM_SPI_CTL_MSTR | BITM_SPI_CTL_EN;

  /* If the CTL register value of the saved SPI state has the MMSE bit set
   * *and* the MMRDH register value of the saved SPI state has the CMDSKIP bit
   * set then we are currently in XIP mode, so we need to return the flash
   * to normal mode.
   */
  if ((spiSavCTL   & BITM_SPI_CTL_MMSE)     &&
      (spiSavMMRDH & BITM_SPI_MMRDH_CMDSKIP)  )
  {
    spiData[0] = 0xEB;
    spiData[1] = 0x00;
    spiData[2] = 0x00;
    spiData[3] = 0xFF;
    spiData[4] = 0xFF;

    selectSlave();
    send_bytes(spiData, 5);
    deselectSlave();
  }

}

/* Restore the previous mode of the SPI device, which is assumed to be a
 * memory-mapped mode, either normal or execute-in-place (XIP).
 */
void enter_mm_xip_mode(void) {
  uint8_t flashVCR = 0;

  /* If the CTL register value of the saved SPI state has the MMSE bit set
   * *and* the MMRDH register value of the saved SPI state has the CMDSKIP bit
   * set then we were previously in XIP mode, so we need to return the flash
   * to XIP mode prior to restoring the SPI state.
   */
  if ((spiSavCTL   & BITM_SPI_CTL_MMSE)     &&
      (spiSavMMRDH & BITM_SPI_MMRDH_CMDSKIP)  )
  {
    /* Clear the XIP disable bit to signal "Ready to enter XIP mode".
     * The function of this bit, when zero, is "To make the data on DQ0 during
     * the first dummy clock NOT Dont Care. I.e. if the XIP bit is zero then
     * the flash memory will test the value of DQ0 during the first dummy clock
     * to determine if it is an XIP (no opcode) operation.
     */
    flashVCR = read_reg(READ_CONFIG_REGISTER);           /* Read flash config register */
    flashVCR &= ~0x08;                                   /* Enable XIP functionality */
    write_reg(WRITE_CONFIG_REGISTER, flashVCR);          /* Write config reg back */

    /* Send a read operation to the flash. This is a dummy operation which
     * just provides the opcode that will be used for subsequent XIP reads.
     * The address and the returned data are don't-cares.
     *
     * "The SPI Flash device must be primed with an initial read command,
     * before the CMDSKIP bit is set." - HRM 0.1 p22-23
     */
    spiData[0] = QUAD_IO_FAST_READ;
    spiData[1] = 0x00;
    spiData[2] = 0x00;
    spiData[3] = 0x00;
    spiData[4] = 0x00;

    selectSlave();
    send_bytes(spiData, 5);
    recv_bytes(spiData, 4);    // may not be required
    deselectSlave();

    /* In order to maintain XIP mode we must set the mode bits to the required
     * value(LSB == 1), so that a 1 value is driven onto DQ0 during the first
     * dummy clock. Clearing this will cause XIP mode to be exited.
     * We know that the CMDSKIP bit is already set in the saved MMRDH value
     * because we checked it above.
     */
    SPI2->CTL = 0u;       /* Disable the SPI before reconfiguring */
    SPI2->MMRDH = (spiSavMMRDH & (~BITM_SPI_MMRDH_MODE)) | XIP_CONFIRMATION_BIT | BITM_SPI_MMRDH_CMDSKIP;
  }
  else
  {
    SPI2->CTL = 0u;       /* Disable the SPI before reconfiguring */
    SPI2->MMRDH = spiSavMMRDH;
  }

  SPI2->CLK   = spiSavCLK;
  SPI2->RXCTL = spiSavRXCTL;
  SPI2->TXCTL = spiSavTXCTL;
  SPI2->CTL   = spiSavCTL;    // restore CTL last, enabling SPI
}

/* Loop reading the flash memory's status register until the BUSY flag is clear */
void wait_while_busy(void) {
  uint8_t flashStatus;

  do {
    flashStatus = read_reg(READ_STATUS_REGISTER);
  } while (flashStatus & STATUS_BUSY);

}








/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  base_adr = adr;             // store Flash Start address
  exit_mm_xip_mode();

  return (0);
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  enter_mm_xip_mode();
  sync_cache();

  return (0);
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {

  enable_writes();

  spiData[0] = BULK_ERASE;

  selectSlave();
  send_bytes(spiData, 1);
  deselectSlave();

  wait_while_busy();          // Wait for erase completion

  return (0);
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {

  enable_writes();

  spiData[0] = SUBSECTOR_ERASE;
  spiData[1] = (uint8_t)(adr >> 16);
  spiData[2] = (uint8_t)(adr >>  8);
  spiData[3] = (uint8_t)(adr >>  0);

  selectSlave();
  send_bytes(spiData, 4);
  deselectSlave();

  wait_while_busy();          // Wait for erase completion

  return (0);
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  int i;

  enable_writes();

  tbuf[0] = PAGE_PROGRAM;
  tbuf[1] = (uint8_t)(adr >> 16);
  tbuf[2] = (uint8_t)(adr >>  8);
  tbuf[3] = (uint8_t)(adr >>  0);

  for (i = 0 ; i < sz; i++)
  {
     tbuf[i + 4] = *buf++;
  }

  /* Send the PAGE_PROGRAM operation to the flash memory. */
  selectSlave();
  send_bytes(tbuf, sz + 4);
  deselectSlave();

  wait_while_busy();

  return (0);
}


/*
 *  Blank Check Checks if Memory is Blank
 *    Parameter:      adr:  Block Start Address
 *                    sz:   Block Size (in bytes)
 *                    pat:  Block Pattern
 *    Return Value:   0 - OK,  1 - Failed
 */

int BlankCheck (unsigned long adr, unsigned long sz, unsigned char pat) {
  unsigned long i, j, k;

  for (i = 0; i < sz; i += PAGE_SIZE) {
                              // Read PAGE_SIZE bytes
    spiData[0] = READ_DATA;
    spiData[1] = (uint8_t)((adr+i) >> 16);
    spiData[2] = (uint8_t)((adr+i) >>  8);
    spiData[3] = (uint8_t)((adr+i) >>  0);

    selectSlave();
    send_bytes(spiData, 4);
    recv_bytes(tbuf, PAGE_SIZE);
    deselectSlave();

    // Determine size to compare
    if ((sz - i) >= PAGE_SIZE) k = PAGE_SIZE;
    else k = (sz - i);

    // Check up to PAGE_SIZE bytes if equal to pattern "pat"
    for (j = 0; j < k; j++)
      if (tbuf[j] != pat)
        return (1);           // Memory is not blank
    }

    return (0);               // Memory is blank

}


/*
 *  Verify Flash Contents
 *    Parameter:      adr:  Start Address
 *                    sz:   Size (in bytes)
 *                    buf:  Data
 *    Return Value:   (adr+sz) - OK, Failed Address
 */

unsigned long Verify (unsigned long adr, unsigned long sz, unsigned char *buf) {
  unsigned long i, j, k;

  for (i = 0; i < sz; i += PAGE_SIZE) {
                              // Read PAGE_SIZE bytes
    spiData[0] = READ_DATA;
    spiData[1] = (uint8_t)((adr+i) >> 16);
    spiData[2] = (uint8_t)((adr+i) >>  8);
    spiData[3] = (uint8_t)((adr+i) >>  0);

    selectSlave();
    send_bytes(spiData, 4);
    recv_bytes(tbuf, PAGE_SIZE);
    deselectSlave();

    // Determine size to compare
    if ((sz - i) >= PAGE_SIZE) k = PAGE_SIZE;
    else k = (sz - i);

    // Check up to PAGE_SIZE bytes if equal to pattern "pat"
    for (j = 0; j < k; j++)
      if (tbuf[j] != *buf++)
        return (adr + i);     // Memory is not blank
  }

  return (adr + sz);          // Memory is blank

}
#if 0
unsigned long Verify (unsigned long adr, unsigned long sz, unsigned char *buf) {
  uint8_t *src = ((uint8_t *)tbuf);

  sz = (sz + 3) & ~3;         // Adjust size for Words

  spiData[0] = READ_DATA;
  spiData[1] = (uint8_t)(adr >> 16);
  spiData[2] = (uint8_t)(adr >>  8);
  spiData[3] = (uint8_t)(adr >>  0);

  selectSlave();                    // read Page size data bytes
  send_bytes(spiData, 4);
  recv_bytes(tbuf, sz);
  deselectSlave();

  while (sz) {
    if (*((uint32_t *)src)	!= *((uint32_t *)buf)) {                 // Compare Data
      return (adr);
    }

    adr += 4;                                           // Go to next Word
    buf += 4;
    src += 4;
    sz  -= 4;
  }

  return (adr);

}
#endif
